package network

import (
    "fmt"
	"strings"
    "math/big"
)

var manuf = map[string]string {
    "20.255861678145536": "Tibit Communications",
    "24.24664": "Copper Mountain Communications, Inc.",
    "24.7364181": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "24.2789": "ScottCare Corporation",
    "24.10544663": "Matis",
    "12.346853203968": "Energy ICT",
    "24.2640103": "Huawei Device Co., Ltd.",
    "12.346852909056": "ApSecure Technologies (Canada), Inc.",
    "24.1347526": "Apple, Inc.",
    "12.123917685493760": "FeldTech GmbH",
    "24.2785": "EG Technology",
    "24.11839823": "Mercury Corporation",
    "24.4458936": "Salcomp (Shenzhen) CO., LTD.",
    "24.3159974": "Cisco Systems, Inc",
    "12.71297883848704": "Resch Electronic Innovation GmbH",
    "24.2781": "Allworx Corp.",
    "24.529": "Nature Worldwide Technology Corp.",
    "12.123917689090048": "X-SPEX GmbH",
    "24.528": "Fenecom",
    "24.11838645": "Zen Eye Co.,Ltd",
    "12.123917685649408": "HDL da Amazônia Industria Eletrônica Ltda",
    "24.2159888": "Codex Digital Limited",
    "12.123917690417152": "KWS-Electronic GmbH",
    "24.9236754": "Motorola Mobility LLC, a Lenovo Company",
    "24.3946258": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "20.53091445506048": "Anacove LLC",
    "24.1108569": "Cisco SPVTG",
    "24.846537": "ARRIS Group, Inc.",
    "20.216145524162560": "Sercomm Corporation.",
    "20.141720406720512": "Krizer international Co,. Ltd.",
    "24.520": "Unify Networks, Inc.",
    "24.11562175": "3ality Digital Systems GmbH",
    "24.523": "Native Networks, Inc.",
    "24.7370495": "Kerlink",
    "24.522": "Gefran Spa",
    "24.4219574": "Tele system communication",
    "24.525": "Micronpc.com",
    "24.44387": "Dedicated Micros Malta LTD",
    "24.11111": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.14711827": "zte corporation",
    "24.524": "Metro-Optix",
    "24.527": "Aatr",
    "12.123917689634816": "Heim- & Bürokommunikation Ilmert e.K.",
    "24.526": "ECI Telecom Ltd.",
    "12.346861973504": "AS Solar GmbH",
    "24.2892247": "IMAX Corporation",
    "12.123917679271936": "Tattile Srl",
    "20.264625142628352": "Groupeer Technologies",
    "12.123917685997568": "Transelektronik Messgeräte GmbH",
    "24.12085911": "Edgecore Networks Corporation",
    "24.4467455": "E3 Technology, Inc.",
    "24.7084863": "MiraeSignal Co., Ltd",
    "24.14725728": "Teno Network Technologies Company Limited",
    "24.4467459": "Cisco Systems, Inc",
    "24.9462190": "Intel Corporate",
    "24.9473999": "UNO System Co., Ltd",
    "12.346859757568": "Quantec Networks GmbH",
    "24.41609": "Cisco Systems, Inc",
    "20.158667120836608": "Nuwa Robotics (HK) Limited Taiwan Branch",
    "12.346868219904": "Moritex Corporation",
    "24.14303586": "Xperi Corporation",
    "20.172584087846912": "Capital Engineering & Research Incorporation Ltd.",
    "24.9715145": "Asahi Net, Inc.",
    "24.4241032": "LEGIC Identsystems AG",
    "24.2655482": "SHARP Corporation",
    "24.16264572": "Apple, Inc.",
    "20.167137981235200": "Stoerk-Tronic, Stoerk GmbH & Co.KG",
    "24.6968": "Compal Information (Kunshan) Co., Ltd.",
    "24.6969": "Proxicast",
    "24.7642088": "Hitron Technologies. Inc",
    "24.6962": "QLogic Corporation",
    "24.6963": "Nokia Danmark A/S",
    "24.6960": "Solitech Inc.",
    "24.6961": "Neural Image. Co. Ltd.",
    "24.6966": "Tsubata Engineering Co.,Ltd. (Head Office)",
    "24.6967": "Computec Oy",
    "24.6964": "Focus System Inc.",
    "24.6965": "ChongQing JINOU Science & Technology Development CO.,Ltd",
    "24.5439": "Alcatel Alenia Space Italia",
    "24.5438": "Q-Matic Sweden AB",
    "24.1998": "Cabletime Limited",
    "24.1999": "Anoto AB",
    "24.9969585": "Shenzhen Lemon Network Technology Co.,Ltd",
    "24.1992": "Brain21, Inc.",
    "24.1993": "Technol Seven Co., Ltd.",
    "24.1990": "VDS Vosskuhler GmbH",
    "24.1991": "Synectics Systems Limited",
    "24.1996": "Kaba Benzing GmbH",
    "24.1997": "Kumoh Electronic Co, Ltd",
    "24.1994": "Creatix Polymedia Ges Fur Kommunikaitonssysteme",
    "24.5436": "Kprotech Co., Ltd.",
    "24.3993484": "Zhejiang Dahua Technology Co., Ltd.",
    "24.13654268": "BSkyB Ltd",
    "24.11305459": "Nokia Corporation",
    "24.8951538": "Valeo Schalter und Sensoren GmbH",
    "24.16554746": "Motus Global Inc.",
    "12.71297883475968": "Thermo Fisher Scientific",
    "24.10526234": "Polar Electro Oy",
    "24.16301581": "A2UICT Co.,Ltd.",
    "12.346853658624": "Postec Data Systems Ltd.",
    "12.123917683404800": "Grossenbacher Systeme AG",
    "24.13908581": "IGRS Engineering Lab Ltd.",
    "24.10486492": "Amazon Technologies Inc.",
    "20.132777845456896": "ShenZhen TuLing Robot CO.,LTD",
    "24.10762289": "RF elements s.r.o.",
    "24.7662658": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.8948761": "Brady Corp Asia Pacific Ltd",
    "24.16311759": "Cgi It Uk Limited",
    "24.6303956": "Apple, Inc.",
    "12.123917678161920": "Fuchu Giken, Inc.",
    "12.123917687341056": "Digital Media Professionals",
    "24.297794": "Skspruce Technologies",
    "12.123917691158528": "Al Zajel Modern Telecomm",
    "12.123917681664000": "Aplex Technology Inc.",
    "24.23444": "Apple, Inc.",
    "12.346859573248": "Linkit S.R.L.",
    "24.8684401": "Avaya Inc",
    "24.3996508": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.9476152": "Huawei Device Co., Ltd.",
    "24.13136165": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "24.7108151": "Huawei Device Co., Ltd.",
    "24.6870155": "Hailo Technologies Ltd.",
    "24.13431825": "Google, Inc.",
    "20.31477064531968": "Npo Telecom Jsc",
    "24.12383660": "LG Electronics (Mobile Communications)",
    "20.154680185323520": "LLC Taipit - Measuring Equipment",
    "24.7872633": "ID Tech",
    "20.93065819717632": "genua GmbH",
    "12.346867376128": "Xplore Technologies Corp",
    "20.171812088446976": "Wunda Group plc",
    "24.2871": "Manufacture Des Montres Rolex Sa",
    "24.16003645": "Sichuan tianyi kanghe communications co., LTD",
    "24.14692766": "Valve Corporation",
    "24.11838044": "Cambium Networks Limited",
    "12.123917686263808": "wtec GmbH",
    "24.3873": "Scientific Atlanta, Inc",
    "24.3872": "Hewlett Packard",
    "24.3871": "Dell Inc.",
    "24.3870": "Chengdu KT Electric Co.of High & New Technology",
    "24.3877": "AimValley B.V.",
    "24.3876": "Cisco Systems, Inc",
    "24.3875": "Cisco Systems, Inc",
    "24.3874": "Helius, Inc.",
    "24.3879": "TEAL Electronics, Inc.",
    "24.3878": "WorldAccxx LLC",
    "12.346854621184": "Spacesaver Corporation",
    "12.123917690175488": "Camwell India LLP",
    "24.13390473": "Shenzhen Optfocus Technology.,Ltd",
    "24.4240072": "Nortel Networks",
    "12.123917675474944": "Liteon Technology Corporation",
    "24.2877": "CONTAL OK Ltd.",
    "12.346854977536": "CSA Computer & Antriebstechnik GmbH",
    "24.4472": "Chiron Technology Ltd",
    "24.4473": "Singular Technology Co. Ltd.",
    "24.4470": "Intellambda Systems, Inc.",
    "24.4471": "Coaxial Networks, Inc.",
    "24.4476": "e-zy.net",
    "24.4477": "ZMD America, Inc.",
    "24.4474": "Singim International Corp.",
    "24.4475": "Büchi Labortechnik AG",
    "24.838533": "Motorola Mobility LLC, a Lenovo Company",
    "24.4478": "Midmark Corp",
    "24.4479": "Neotune Information Technology Corporation,.LTD",
    "12.346868719616": "Telemisis Ltd",
    "24.6837128": "P&S Technology",
    "24.12856132": "Huawei Device Co., Ltd.",
    "12.123917682315264": "ENVItech s.r.o.",
    "24.29122": "Pegatron Corporation",
    "24.4225664": "Nokia Corporation",
    "20.62663551877120": "New Garden Co., Ltd.",
    "24.49298": "Mennen Medical Inc.",
    "24.49299": "Alta Research Corp.",
    "20.88010612801536": "jiangsu zhongling high-tech CO.,LTD.",
    "20.97755435368448": "Zhuhai Raysharp Technology Co.,Ltd",
    "12.346868408320": "TruTeq Wireless (Pty) Ltd",
    "20.84372887699456": "Charge-Amps AB",
    "12.123917679554560": "Vivaldi Clima Srl",
    "20.167152462069760": "Summa nv",
    "24.3297": "The Open Group",
    "24.3296": "Trek Diagnostics Inc.",
    "24.3295": "JAI Manufacturing",
    "24.3294": "ABB STOTZ-KONTAKT GmbH",
    "24.3293": "AOS technologies AG",
    "24.3292": "BECS Technology, Inc",
    "24.3291": "Brocade Communications Systems LLC",
    "24.3290": "FreeHand Systems, Inc.",
    "12.346855522304": "Picture Elements, Inc.",
    "24.49290": "Lauterbach GmbH",
    "24.3299": "Option International N.V.",
    "24.3298": "Rolls-Royce",
    "24.49291": "Risq Modular Systems, Inc.",
    "20.137149112385536": "3S Technology Co., Ltd.",
    "20.268735384387584": "Pulsar Engineering srl",
    "24.49296": "Praim S.R.L.",
    "12.346865459200": "Campbell Scientific Canada Corp.",
    "12.123917679792128": "Vonsch",
    "20.194284239192064": "Sprocomm Technologies CO.,LTD.",
    "24.49297": "Jabil Circuit, Inc.",
    "24.49294": "Network Information Technology",
    "12.123917690519552": "Digifocus Technology Inc.",
    "24.49295": "Panasonic Electric Works Co., Ltd.",
    "24.9961665": "GuangZhou CREATOR Technology Co.,Ltd.(CHINA)",
    "24.8960": "Cayee Computer Ltd.",
    "24.8961": "Witron Technology Limited",
    "24.8962": "Cobalt Digital, Inc.",
    "24.8963": "LITE-ON IT Corporation",
    "24.8964": "Cisco Systems, Inc",
    "24.8965": "Cisco Systems, Inc",
    "24.8966": "Alpsalpine Co,.Ltd",
    "24.8967": "Future Innovation Tech Co.,Ltd",
    "24.8968": "Arcadyan Technology Corporation",
    "24.8969": "Janam Technologies LLC",
    "24.3121": "Cisco Systems, Inc",
    "24.3120": "Cisco Systems, Inc",
    "24.3123": "Compucase Enterprise Co. Ltd.",
    "24.3122": "Avionic Design GmbH",
    "24.3125": "KaVo Dental GmbH & Co. KG",
    "24.3124": "Vixen Co., Ltd.",
    "24.3127": "Geomation, Inc.",
    "24.3126": "Sharp Takaya Electronics Industry Co.,Ltd.",
    "24.3129": "Sentinel Wireless Inc.",
    "24.3128": "TelcoBridges Inc.",
    "24.14985821": "Cisco Systems, Inc",
    "24.2636070": "CyberTAN Technology Inc.",
    "24.2907064": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346866061312": "Area Energy, Inc.",
    "12.346861674496": "ASSET InterTech, Inc.",
    "24.10243638": "Intel Corporate",
    "24.3954621": "Kessler Crane Inc.",
    "24.28658": "Mitsumi Electric Co.,Ltd.",
    "20.180698124124160": "Air Liquide Medical Systems",
    "24.7876182": "China Mobile Group Device Co.,Ltd.",
    "24.2682218": "Apple, Inc.",
    "24.4747003": "Davit System Technology Co., Ltd.",
    "12.123917685133312": "PABLO AIR Co., LTD",
    "24.6513": "Guangzhou Unicomp Technology Co.,Ltd",
    "24.6512": "Z-Com, Inc.",
    "24.6511": "SensoPart GmbH",
    "24.6510": "Metacom (Pty) Ltd.",
    "24.6517": "Beijing Huisen networks technology Inc",
    "24.6516": "16063",
    "24.6515": "Zeugma Systems",
    "24.6514": "Plexus (Xiamen) Co.,ltd.",
    "24.6519": "Extreme Networks, Inc.",
    "24.6518": "Xipher Technologies, LLC",
    "24.13122693": "Apple, Inc.",
    "24.4742635": "Just Add Power",
    "24.4775180": "eero inc.",
    "12.71297883836416": "ADVALY SYSTEM Inc.",
    "12.346867699712": "Arcontia Technology AB",
    "24.7077995": "Samsung Electronics Co.,Ltd",
    "20.44169481420800": "Efento T P Szydłowski K Zaręba Spółka Jawna",
    "24.14704038": "Detlef Fink Elektronik & Softwareentwicklung",
    "20.57176919900160": "Angee Technologies Ltd.",
    "24.3727982": "ShenZhen Sweet Rain Electronics Co.,Ltd.",
    "12.346853634048": "Precision Filters, Inc.",
    "20.49277258694656": "Minno LLC",
    "24.13433072": "Xi'an HISU Multimedia Technology Co.,Ltd.",
    "24.6042939": "Compal Broadband Networks, Inc.",
    "24.6356510": "China Palms Telecom.Ltd",
    "24.6356512": "2Wire Inc",
    "24.789": "Cidco Incorporated",
    "24.788": "Teleware Network Systems",
    "24.15519808": "GEA Farm Technologies GmbH",
    "24.781": "Uniwill Computer Corp.",
    "24.780": "Telesoft Technologies Ltd.",
    "24.783": "Digital China (Shanghai) Networks Ltd.",
    "24.782": "Core Communications Co., Ltd.",
    "24.785": "Micro Technology Co., Ltd.",
    "24.784": "E-Globaledge Corporation",
    "24.787": "Access Media SPA",
    "24.786": "TRsystems GmbH",
    "24.12619552": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.12106344": "Xi'an Jizhong Digital Communication Co.,Ltd",
    "12.346857934848": "Devitech ApS",
    "24.6877293": "Along Industrial Co., Limited",
    "12.71297883508736": "CS Instruments Asia",
    "12.346856284160": "Commoca, Inc",
    "20.124214333931520": "CLIP Inc.",
    "24.433": "General Bandwidth",
    "24.432": "Fulltek Technology Co., Ltd.",
    "24.431": "Artesyn Embedded Technologies",
    "24.430": "Trex Enterprises",
    "24.437": "Turin Networks, Inc.",
    "24.436": "Wayport, Inc.",
    "24.435": "Precision Electronic Manufacturing",
    "24.434": "Digital Processing Systems, Inc.",
    "24.439": "Centos, Inc.",
    "24.438": "SAEJIN T&M Co., Ltd.",
    "24.9778": "Instrumentation Technologies d.d.",
    "24.9779": "MIR - Medical International Research",
    "24.1563": "Notebook Development Lab. Lenovo Japan Ltd.",
    "24.1562": "Zetari Inc.",
    "24.1561": "Connection Technology Systems",
    "24.1560": "DigiPower Manufacturing Inc.",
    "24.1567": "Vision Components GmbH",
    "24.1566": "Maxan Systems",
    "24.1565": "MIP Telecom, Inc.",
    "24.1564": "Hoshino Metal Industries, Ltd.",
    "24.5270019": "Netgear",
    "24.1569": "Hinox, Co., Ltd.",
    "24.1568": "Serial System Ltd.",
    "24.8134150": "New H3C Technologies Co., Ltd",
    "12.346853736448": "Zultys Technologies",
    "24.4743587": "Concern \"Axion\" JSC",
    "24.5531385": "ConMet",
    "12.346860277760": "Promate Electronic Co.Ltd",
    "24.541334": "Mobile Technology Solutions LLC",
    "24.6070752": "EmbedWay Technologies",
    "20.31372477464576": "Eolos IT Corp",
    "12.346861191168": "Mediana",
    "24.12641614": "ARIES Embedded GmbH",
    "24.9483910": "Murata Manufacturing Co., Ltd.",
    "24.1590807": "Samsung Electronics Co.,Ltd",
    "24.9231572": "Hewlett Packard",
    "12.71297885028352": "QED Advanced Systems Limited",
    "24.12863931": "Mitsumi Electric Co.,Ltd.",
    "24.14173483": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.1387": "C.P. Technology Co., Ltd.",
    "24.1386": "Heuft Systemtechnik GmbH",
    "24.1385": "VMware, Inc.",
    "24.1384": "Piltofish Networks AB",
    "24.1383": "Etymonic Design, Inc.",
    "24.1382": "Secui.com Corporation",
    "24.1381": "Tailyn Communication Company Ltd.",
    "24.1380": "Tsinghua Bitway Co., Ltd.",
    "24.13955111": "Trust Power Ltd.",
    "24.1389": "Pacific Corporation",
    "24.1388": "Hung Chang Co., Ltd.",
    "24.4766146": "Teletics Inc.",
    "12.123917679874048": "Intermind Inc.",
    "24.1091395": "SK Mtek Limited",
    "24.12114679": "Advanced Photonics, Inc.",
    "12.123917682835456": "Lumacron Technology Ltd.",
    "24.10753841": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917680324608": "MB connect line GmbH Fernwartungssysteme",
    "24.271840": "Taicang T&W Electronics",
    "12.346866204672": "TV1 GmbH",
    "24.323880": "HMD Global Oy",
    "12.346858061824": "Rudolph Technologies, Inc.",
    "24.7651255": "Nusoft Corporation",
    "24.12063051": "Cisco Systems, Inc",
    "24.9443502": "Shanghai Meridian Technologies, Co. Ltd.",
    "24.6328156": "Samsung Electronics Co.,Ltd",
    "24.7114677": "Microsoft Mobile Oy",
    "20.52819391414272": "Shenzhen Sunvell Electronics Co., Ltd.",
    "24.14432633": "Cisco Systems, Inc",
    "24.5826234": "Apple, Inc.",
    "24.13637482": "Samsung Electronics Co.,Ltd",
    "24.4809": "ARRIS Group, Inc.",
    "24.4808": "Perfect tech",
    "12.346862108672": "Creacon Technologies B.V.",
    "24.4803": "Wit S.A.",
    "24.4802": "Apex Electronics Factory",
    "24.4801": "Check Point Software Technologies",
    "24.4800": "HotLava Systems, Inc.",
    "24.4807": "SECURAY Technologies Ltd.Co.",
    "24.4806": "TGC America, Inc",
    "24.4805": "V-Show Technology (China) Co.,Ltd",
    "24.4804": "Viseon, Inc.",
    "12.123917690064896": "Boffins Technologies AB",
    "24.15383556": "G3-PLC Alliance",
    "24.319944": "IMS Soluções em Energia Ltda.",
    "24.16289263": "Tenebraex",
    "24.8187001": "Texas Instruments",
    "24.14168527": "Samsung Electronics Co.,Ltd",
    "24.8962926": "Beijing Ereneben lnformation Technology Limited",
    "12.123917683113984": "FIRST RF Corporation",
    "24.3412632": "Apple, Inc.",
    "24.2675127": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.6346581": "Huawei Technologies Co.,Ltd",
    "12.123917683699712": "QuestHouse, Inc.",
    "24.7385405": "Samsung Electronics Co.,Ltd",
    "24.14430673": "vivo Mobile Communication Co., Ltd.",
    "24.7667104": "Compupal (Group) Corporation",
    "12.123917686128640": "Veo Technologies",
    "20.211776053444608": "Big Dutchman International GmbH",
    "12.123917687324672": "Senso2Me",
    "12.346867724288": "FBT Elettronica spa",
    "20.137141731459072": "Izkare",
    "24.6613138": "Morio Denki Co., Ltd.",
    "20.145727916343296": "EPSa Elektronik & Präzisionsbau Saalfeld GmbH",
    "24.14739030": "Nethesis srl",
    "24.2898326": "Sagemcom Broadband SAS",
    "20.216602658209792": "ShenZhen ZYT Technology co., Ltd",
    "24.10757656": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.79215073427456": "Solaredge LTD.",
    "20.101728417480704": "Cable Matters Inc.",
    "24.3149123": "Microsoft Mobile Oy",
    "12.123917686923264": "Dynics",
    "24.5565859": "Navdy, Inc.",
    "24.3198375": "Samsung Electronics Co.,Ltd",
    "20.150262234742784": "Honeywell spol. s.r.o. HTS CZ o.z.",
    "12.123917679493120": "B/E Aerospace, Inc.",
    "12.346864365568": "Dsp Design Ltd",
    "20.277981933076480": "Cirque Audio Technology Co.,Ltd",
    "12.346868805632": "ACD Elektronik GmbH",
    "20.13692003418112": "Gemini Data Loggers (UK) Limited",
    "24.12096566": "Huawei Technologies Co.,Ltd",
    "12.123917678751744": "Monnit Corporation",
    "24.1845423": "Aruba, a Hewlett Packard Enterprise Company",
    "24.1082322": "Microseven Systems, LLC",
    "12.346868162560": "DSPCon",
    "12.123917687681024": "Potter Electric Signal Co. LLC",
    "24.8155324": "IEEE Registration Authority",
    "24.13675477": "Intel Corporate",
    "20.57176933531648": "ChengDu HuiZhong Cloud Information Technology Co., Ltd.",
    "24.11829086": "Foresight Manufacture (S) Pte Ltd",
    "24.6050127": "Intel Corporate",
    "20.141720401477632": "Beijing Gaokezhongtian Technology Co Ltd",
    "20.250817668972544": "Connected Cars A/S",
    "12.123917691621376": "eSight",
    "12.346863423488": "Monitor Business Machines Ltd.",
    "24.15250606": "Shenzhen C&D Electronics Co.,Ltd",
    "12.123917690310656": "Advitronics telecom bv",
    "24.8964560": "LG Electronics (Mobile Communications)",
    "12.123917680930816": "ABB S.p.A.",
    "12.123917682683904": "HCH. Kündig & CIE. AG",
    "24.9748372": "Juniper Networks",
    "24.8147065": "Intel Corporate",
    "12.123917679194112": "Hydra Controls",
    "24.1637835": "IEEE Registration Authority",
    "24.13405809": "Cisco Systems, Inc",
    "12.346858582016": "Imagine Communications",
    "24.8200": "Cable & Computer Technology",
    "20.4471490871296": "CyOne Security AG",
    "12.71297884504064": "Venco",
    "24.4752114": "Appel Elektronik GmbH",
    "24.2679926": "Pi-Coral",
    "24.820444": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.14210851": "IDS, Inc",
    "12.119269949440": "J-D.Com",
    "24.2112957": "LG Innotek",
    "12.346860900352": "Rohde&Schwarz Topex SA",
    "20.71414658891776": "Lennox International Incorporated",
    "12.123917675556864": "Tempus Fugit Consoles bvba",
    "24.8209": "Canopus Co., Ltd.",
    "24.49015": "Cisco Systems, Inc",
    "12.346856316928": "dAFTdATA Limited",
    "24.4951": "Soyal Technology Co., Ltd.",
    "12.346868256768": "G3 Technologies",
    "24.832676": "Xintai Automobile Intelligent Network Technology",
    "24.1048829": "LaonPeople",
    "24.2418362": "JSC Zavod im. Kozitsky",
    "24.16050869": "Extreme Networks, Inc.",
    "24.4997708": "Information Modes software modified addresses (not registered?)",
    "12.346865135616": "Icon Time Systems",
    "24.32819": "EMS Aviation, Inc.",
    "24.32816": "Nexus Electronics",
    "24.32817": "Basys, Corp.",
    "24.32814": "Castle Rock Computing",
    "24.32815": "National Instruments Corp.",
    "24.32812": "The Sage Group Plc",
    "24.32813": "Xylogics, Inc.",
    "24.32810": "Test Systems & Simulations Inc",
    "24.32811": "Integrated Marketing Co",
    "24.3717948": "Avaya Inc",
    "20.62663548731392": "OOO Group of Industrial Technologies",
    "24.3156414": "Shanghai MRDcom Co.,Ltd",
    "20.246679821352960": "Chengdu Song Yuan Electronic Technology Co.,Ltd",
    "12.346859880448": "STÖRK-TRONIC Störk GmbH&Co. KG",
    "20.70559717130240": "LS Energy Solutions",
    "24.16545046": "D-Link International",
    "12.123917675114496": "Audi Ag",
    "24.4771": "Trust International B.V.",
    "24.4773": "Dolphin Interconnect Solutions AS",
    "20.272805458345984": "Wuhan Guide Infrared Co.,Ltd",
    "24.8673075": "Microsoft Corporation",
    "12.123917676224512": "Elvys s.r.o",
    "24.16552507": "Nokia Corporation",
    "24.13685308": "Apple, Inc.",
    "20.159182269448192": "Beijing Acorn Networks Corporation",
    "12.123917691400192": "Elva-1 Microwave Handelsbolag",
    "12.71297884491776": "Key Systems, Inc.",
    "24.13400527": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "12.346853572608": "Communication Solutions, Inc.",
    "12.346855981056": "Silvertree Engineering Ltd",
    "12.123917690875904": "Lyse AS",
    "24.1316070": "Ningbo Sanhe Digital Co.,Ltd",
    "24.4754186": "Cisco Systems, Inc",
    "24.11821567": "Lumigon A/S",
    "24.4240624": "Juniper Networks",
    "20.194600860909568": "DMAC Security LLC",
    "12.346864029696": "Aeroflex GmbH",
    "24.14165424": "Rockeetech Systems Co.,Ltd.",
    "20.106196536131584": "GovComm",
    "12.346854572032": "Supertel",
    "20.272805455200256": "Digital Imaging Technology",
    "24.5563450": "Apple, Inc.",
    "24.3413267": "Texas Instruments",
    "24.7394791": "Wellcore Corporation",
    "20.277981930979328": "Shen Zhen Xin Hao Yuan Precision Technology Co.,L Td",
    "24.11018159": "Krypton Polska",
    "12.346854735872": "Fundación TECNALIA Research & Innovation",
    "20.233630193418240": "OPTiM Corporation",
    "12.346866946048": "Zoe Medical",
    "12.346852667392": "Trikon Technologies Ltd.",
    "12.123917676781568": "Cavagna Group Spa",
    "12.346852683776": "Dunti Corporation",
    "12.346858016768": "Callpod, Inc.",
    "24.10263581": "Starkey Labs Inc.",
    "24.10263583": "Espressif Inc.",
    "24.9720255": "Hyundai ESG",
    "12.123917689446400": "Idem Inc.",
    "24.4240277": "Apple, Inc.",
    "12.71297884327936": "OOO Group of Industrial Technologies",
    "24.15775159": "Disruptive Technologies Research AS",
    "24.9752113": "CTS Limited",
    "24.6570822": "GuangDong Quick Network Computer CO.,LTD",
    "24.7625803": "GN Audio A/S",
    "24.8397479": "NEC Platforms, Ltd.",
    "24.12326127": "ITON Technology Limited",
    "24.3716307": "BSH Hausgeraete GmbH",
    "24.9753241": "Techmation Co.,Ltd.",
    "20.268341261369344": "Sterna Security",
    "12.346861514752": "C-COM Satellite Systems Inc.",
    "20.180680393752576": "Viper Design, LLC",
    "20.48688646848512": "DPS Electronics",
    "12.123917677469696": "DEUTA-WERKE GmbH",
    "24.12889177": "Murata Manufacturing Co., Ltd.",
    "24.7613282": "Fujitsu Limited",
    "24.1588975": "Apple, Inc.",
    "24.3186205": "Ohsung",
    "12.123917678104576": "eze System, Inc.",
    "20.167152463118336": "Hangzhou Sanxin Network Technology Co.,Ltd",
    "24.5775498": "IEEE Registration Authority",
    "24.10507954": "Harman/Becker Automotive Systems GmbH",
    "24.2108920": "Digital Media Cartridge Ltd.",
    "12.346863030272": "Mecc Co., Ltd.",
    "12.346865946624": "VAF Instruments BV",
    "24.2630214": "Beijing Sinoix Communication Co., LTD",
    "24.11562976": "Samsung Electronics Co.,Ltd",
    "24.6075038": "ASCO Power Technologies",
    "24.5430": "Powertech co.,Ltd",
    "24.16012339": "Rockwell Automation",
    "24.5433": "Technodrive srl",
    "24.5432": "RFID, Inc.",
    "24.5435": "EMH metering GmbH & Co. KG",
    "24.5775144": "Huawei Technologies Co.,Ltd",
    "12.123917685809152": "Array Technologies Inc.",
    "24.5434": "Shenzhen Syscan Technology Co.,Ltd.",
    "20.96896791085056": "Sambo Hitech",
    "24.5437": "Elim Product Co.",
    "12.346861596672": "Luceat",
    "24.2103814": "Compal Information (Kunshan) Co., Ltd.",
    "24.1995": "Freebox Sas",
    "24.2674756": "Shenzhen Xinyin technology company",
    "12.346857369600": "WaveIP",
    "24.14702324": "AxesNetwork Solutions inc.",
    "24.13170317": "S.E.Technologies Limited",
    "12.123917682671616": "Neurio Technology Inc.",
    "24.825400": "Xiaomi Communications Co Ltd",
    "24.2140035": "miControl GmbH",
    "12.346852548608": "DIVA Systems",
    "20.48835645669376": "Shen Zhen Sis Science & Technology Ltd.",
    "24.7092280": "Marking System Technology Co., Ltd.",
    "24.17890": "CyberTAN Technology Inc.",
    "24.41069": "Mannesmann Tally Corporation",
    "24.41068": "Shindengen Electric Mfg. Co., Ltd.",
    "12.123917691043840": "PolyTech A/S",
    "12.346867109888": "Higeco S.r.l.",
    "24.41060": "Kvb/Analect",
    "12.346867200000": "Eolane Montceau",
    "24.41062": "Isa Co., Ltd.",
    "24.41065": "Symmetricom, Inc.",
    "24.41064": "Bhp Limited",
    "24.41067": "Dms Dorsch Mikrosystem Gmbh",
    "24.41066": "Verilink Corporation",
    "24.2412863": "Mexus Co.,Ltd",
    "24.11314224": "Peerless AV",
    "12.123917689532416": "Intra Corporation",
    "24.10012741": "PacketAccess",
    "24.5305818": "Amazon Technologies Inc.",
    "24.14179114": "Samsung Electronics Co.,Ltd",
    "20.14223584264192": "Hubei Century Network Technology Co., Ltd",
    "24.3703706": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.6580818": "Avaya Inc",
    "12.123917687599104": "XAVi Technologies Corp.",
    "12.123917689901056": "Nanospeed Technologies Limited",
    "24.9980319": "Domo Tactical Communications",
    "24.10022236": "Tesla,Inc.",
    "24.1578092": "Zhejiang Tmall Technology Co., Ltd.",
    "24.13418617": "Murata Manufacturing Co., Ltd.",
    "20.180659900383232": "Comsel System Ltd",
    "12.123917680459776": "Weihai Weigao Medical Imaging Technology Co., Ltd",
    "24.3441374": "Kiio Inc",
    "12.346865631232": "IT-IS International Ltd.",
    "12.123917687750656": "Mitech Integrated Systems Inc.",
    "12.346863001600": "ReaMetrix, Inc.",
    "24.11796636": "CableWorld Ltd.",
    "12.346854268928": "Audio Riders Oy",
    "12.123917690159104": "Exfrontier Co., Ltd.",
    "24.5297879": "Takahata Precision",
    "24.15231588": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.13920359": "Neoventus Design Group",
    "12.346857656320": "Digital communications Technologies",
    "12.346854088704": "shanghai trend intelligent systems CO.,LTD",
    "24.1879755": "Huawei Technologies Co.,Ltd",
    "24.7354925": "Shenzhen V-Link Technology CO., LTD.",
    "24.9465093": "Bxb Electronics Co., Ltd",
    "12.346854146048": "Soronti, Inc.",
    "24.13168430": "Beijing Gefei Tech. Co., Ltd",
    "12.123917680070656": "Moog Rekofa GmbH",
    "24.6581612": "Huawei Technologies Co.,Ltd",
    "24.5031230": "Zyxel Communications Corporation",
    "12.346861936640": "Tech Fass s.r.o.",
    "24.14447784": "Samsung Electronics Co.,Ltd",
    "24.11302675": "Honeywell Safety Products (Shanghai) Co.,Ltd",
    "12.123917685362688": "Bolide Technology Group, Inc.",
    "24.46751": "Latch",
    "24.7346925": "Advas Co., Ltd.",
    "24.45228": "SIAE-Microelettronica S.p.A.",
    "24.1580693": "AuVerte",
    "24.9456": "Seanodes",
    "24.12108327": "PSTec",
    "12.346858213376": "Radiant Imaging, Inc.",
    "12.346853011456": "Pcsc",
    "24.5779730": "Drust",
    "24.12909482": "Huawei Technologies Co.,Ltd",
    "24.12883277": "IEEE Registration Authority",
    "24.6609236": "Taicang T&W Electronics",
    "24.16037167": "Juniper Networks",
    "24.4485131": "Iffu",
    "12.346860879872": "K.K. Rocky",
    "24.1327868": "Texas Instruments",
    "24.7910": "Shenzhen Advanced Video Info-Tech Co., Ltd.",
    "24.7911": "Epic Systems Inc",
    "24.7912": "Mytek",
    "24.7913": "Stoneridge Electronics AB",
    "24.7914": "Sensor Switch, Inc.",
    "24.7915": "Talk-A-Phone Co.",
    "24.7916": "Compal Information (Kunshan) Co., Ltd.",
    "24.7917": "Adventiq Ltd.",
    "24.7918": "ETL Systems Ltd",
    "24.7919": "Cantronic International Limited",
    "24.4774032": "Fn-Link Technology Limited",
    "24.2149930": "IEEE Registration Authority",
    "24.2652145": "Apple, Inc.",
    "24.13386723": "Trump s.a.",
    "24.13695517": "Qihoo 360 Technology Co.,Ltd",
    "24.8930820": "Xiaomi Communications Co Ltd",
    "24.2914775": "Sagemcom Broadband SAS",
    "20.255861682339840": "Digifocus Technology Inc.",
    "24.13420844": "Schindler - PORT Technology",
    "24.3984787": "Lg Electronics Inc",
    "24.2363856": "Chongqing Ehs Science and Technology Development Co.,Ltd.",
    "24.3720970": "WiFiSong",
    "12.71297885155328": "Mega Electronics Ltd",
    "24.2105765": "LG Electronics (Mobile Communications)",
    "12.346862739456": "Baudisch Electronic GmbH",
    "24.13688775": "Samsung Electronics Co.,Ltd",
    "20.17622850600960": "Shenzhen Xinfa Electronic Co.,ltd",
    "24.12873739": "Guangzhou Chip Technologies Co.,Ltd",
    "20.39919489646592": "Tekelek Europe Ltd",
    "24.13126773": "Cisco Systems, Inc",
    "24.1852029": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.3442052": "Extreme Networks, Inc.",
    "24.4469448": "Technicolor CH USA Inc.",
    "12.71297884000256": "Quantaflow",
    "24.10493069": "Huawei Technologies Co.,Ltd",
    "12.123917676769280": "Teenage Engineering AB",
    "24.14202561": "NetworkAccountant, Inc.",
    "24.15760744": "D-Link Corporation",
    "20.149935441838080": "iRoom GmbH",
    "24.10766004": "Huawei Device Co., Ltd.",
    "24.3412904": "Mediplan Limited",
    "24.2894841": "LG Innotek",
    "24.13380897": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.13380891": "Sfr",
    "24.7120768": "Nokia Corporation",
    "24.53325": "Div Of Research & Statistics",
    "24.53324": "Eurotel Telecom Ltd.",
    "24.53327": "Bitronics, Inc.",
    "24.53326": "Logibag",
    "24.53321": "Impresstek Co., Ltd.",
    "24.53320": "Ecton, Inc.",
    "24.53323": "La Cie Group S.A.",
    "24.53322": "Presence Technology Gmbh",
    "20.233258804576256": "EISST Ltd",
    "24.53329": "O2 Micro, Inc.",
    "24.53328": "Iskratel d.o.o.",
    "12.123917691719680": "Data Strategy Limited",
    "12.123917679435776": "ARKS Enterprises, Inc.",
    "12.346859565056": "Zumbach Electronic AG",
    "24.16568131": "Huizhou City Huiyang District Meisiqi Industry Development Co,.Ltd",
    "24.1819": "CDVI Americas Ltd",
    "24.1818": "Finedigital Inc.",
    "24.1817": "Mobiis Co., Ltd.",
    "24.1816": "iCanTek Co., Ltd.",
    "24.1815": "Wieland Electric GmbH",
    "24.1814": "J & S Marine Ltd.",
    "24.1813": "General Research of Electronics, Inc.",
    "24.1812": "Brightcom",
    "24.1811": "IP One, Inc.",
    "24.1810": "JAL Information Technology",
    "24.4989324": "Panasonic India Private limited",
    "24.1096855": "vivo Mobile Communication Co., Ltd.",
    "24.8951478": "Global Fire Equipment S.A.",
    "24.13654184": "zte corporation",
    "12.123917687848960": "HongSeok Ltd.",
    "24.9457297": "IEEE Registration Authority",
    "12.123917680111616": "Nidec Avtron Automation Corp",
    "24.6339340": "Beijing HuaqinWorld Technology Co,Ltd",
    "24.14458015": "Shenzhen YOUHUA Technology Co., Ltd",
    "12.346858070016": "iNET Systems Inc.",
    "24.57473": "Tyan Computer Corp.",
    "24.13171494": "Cisco SPVTG",
    "12.123917682950144": "Shadin Avionics",
    "24.5668": "Teneros, Inc.",
    "24.5669": "Impinj, Inc.",
    "24.16019093": "Extreme Networks, Inc.",
    "12.346864308224": "Rx Monitoring Services",
    "12.123917689602048": "Condev-Automation GmbH",
    "24.5660": "e:cue",
    "24.5661": "Innovative Wireless Technologies, Inc.",
    "24.5662": "Woojinnet",
    "24.5663": "SUNWAVETEC Co., Ltd.",
    "24.5664": "Sony Mobile Communications Inc",
    "24.5665": "Colorado Vnet",
    "24.5666": "Bbh Systems Gmbh",
    "24.5667": "Interval Media",
    "12.346853298176": "Vidco, Inc.",
    "24.4392": "Streamit",
    "24.4391": "TASI, Inc",
    "24.4390": "Venstar Inc.",
    "24.4397": "iPulse Systems",
    "24.4396": "IZT GmbH",
    "24.4395": "NetModule AG",
    "24.4394": "Niko NV",
    "24.4399": "ASUSTek COMPUTER INC.",
    "24.4398": "Ceicom",
    "12.346860081152": "Telairity",
    "24.10492338": "IEEE Registration Authority",
    "24.7127924": "Texas Instruments",
    "24.13930720": "Topcon Electronics GmbH & Co. KG",
    "12.346867036160": "Beam Ltd",
    "24.15761839": "Irz Automation Technologies Ltd",
    "12.346857668608": "Mesure Controle Commande",
    "24.15249916": "Intel Corporate",
    "12.123917681643520": "AOT System GmbH",
    "12.346862837760": "YUYAMA MFG Co.,Ltd",
    "24.7880027": "TELNET Redes Inteligentes S.A.",
    "24.9496516": "HTC Corporation",
    "12.346862297088": "TEKO TELECOM SpA",
    "12.346865745920": "Baudisch Electronic GmbH",
    "12.123917685833728": "provedo",
    "24.2793785": "Pointguard, Llc",
    "12.123917687144448": "Cygnetic Technologies (Pty) Ltd",
    "24.9502585": "Metro Ethernet Forum",
    "24.14690413": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.9481724": "Telechips, Inc.",
    "12.71297883623424": "BroadSoft, INC",
    "20.128612183310336": "Songam Syscom Co. LTD.",
    "24.12847232": "Shenzhen UTEPO Tech Co., Ltd.",
    "12.123917683261440": "Sanko-sha,inc.",
    "24.9956": "Canal +",
    "24.9957": "AEG Power Solutions",
    "24.9954": "LG Electronics (Mobile Communications)",
    "24.9955": "Dti",
    "24.9952": "Asiteq",
    "24.9953": "Stanford University, OpenFlow Group",
    "24.9950": "Fdi Matelec",
    "24.9951": "TaiDoc Technology Corp.",
    "20.75247166423040": "ZHEJIANG HIKAILINK TECHNOLOGY Co., Ltd",
    "24.9958": "Visionhitech Co., Ltd.",
    "24.9959": "Shanghai ONLAN Communication Tech. Co., Ltd.",
    "24.4537": "Inner Range Pty. Ltd.",
    "24.4536": "Liebherr - Elektronik GmbH",
    "24.4535": "Octalix B.V.",
    "24.4534": "Open Systems International",
    "24.4533": "Shenzhen Powercom Co.,Ltd",
    "24.4532": "Westermo Network Technologies AB",
    "24.4531": "Yoshimiya Co.,Ltd.",
    "24.4530": "2001 Technology Inc.",
    "12.123917675040768": "HolidayCoro",
    "24.4539": "Cisco Systems, Inc",
    "24.4538": "Elexol Pty Ltd",
    "12.123917687812096": "Keycom Corp.",
    "12.346862600192": "Fsr, Inc.",
    "12.346859327488": "Becker Electronics GmbH",
    "24.4497279": "DataCore Software Corporation",
    "24.3622": "Gincom Technology Corp.",
    "24.3623": "Crere Networks, Inc.",
    "24.3620": "Huwell Technology Inc.",
    "24.3621": "Hannae Technology Co., Ltd",
    "24.3626": "Private",
    "24.3627": "Safari Technologies",
    "24.3624": "Dynamic Ratings P/L",
    "24.3625": "Shester Communications Inc",
    "24.3628": "Netcodec co.",
    "24.3629": "Hyundai Digital Technology Co.,Ltd.",
    "24.11592245": "Xiaomi Communications Co Ltd",
    "24.9480": "Maquet Cardiopulmonary AG",
    "24.9481": "SHARETRONIC Group LTD",
    "24.9482": "Security Expert Co. Ltd",
    "24.9483": "Centrofactor Inc",
    "24.9484": "Senet Inc",
    "24.9485": "GZT Telkom-Telmor sp. z o.o.",
    "24.9486": "gt german telematics gmbh",
    "24.9487": "On-Ramp Wireless, Inc.",
    "24.9488": "Pico-Tesla Magnetic Therapies",
    "24.9489": "Elitegroup Computer Systems Co.,Ltd.",
    "24.2148816": "Apple, Inc.",
    "24.5543032": "Silvershore Technology Partners",
    "24.14689407": "Huawei Technologies Co.,Ltd",
    "24.9760415": "HMD Global Oy",
    "20.194284249677824": "Zoox",
    "24.10791951": "Huawei Device Co., Ltd.",
    "12.71297884786688": "Yoozma Corporation",
    "24.1326688": "Nokia",
    "12.123917682749440": "A Trap, USA",
    "24.10772497": "Verifone",
    "24.9218434": "Intel Corporate",
    "12.346865037312": "Weltek Technologies Co. Ltd.",
    "20.93020698443776": "Elite Silicon Technology, Inc.",
    "12.123917676007424": "Season Electronics Ltd",
    "12.123917688184832": "Unitro Fleischmann",
    "12.346853539840": "Torse",
    "24.20617": "Safety Management Systems",
    "24.20616": "Amano Corporation",
    "24.20615": "Terasaki Electric Co., Ltd.",
    "24.20614": "Telkom Sa, Ltd.",
    "24.49252": "General Datacomm Ind Inc",
    "24.20612": "Quantum Corp.",
    "24.20611": "Gilbarco, Inc.",
    "24.20610": "Foresson Corporation",
    "12.346863329280": "Bassett Electronic Systems ltd",
    "12.123917680762880": "Ascon Tecnologic S.r.l.",
    "24.20619": "Hewlett Packard",
    "12.346863300608": "YellowSoft Co., Ltd.",
    "24.10247442": "Extreme Networks, Inc.",
    "24.16566423": "zte corporation",
    "24.6581984": "Intel Corporate",
    "24.793104": "Acoustic Stream",
    "12.71297884217344": "Cytech Technology Pte Ltd",
    "24.4905488": "Emotiva Audio Corp",
    "24.3199339": "Liteon Technology Corporation",
    "12.123917677428736": "Datum Electronics Limited",
    "24.1354192": "F5 Networks, Inc.",
    "12.123917686505472": "boekel",
    "20.27056770383872": "Phinetworks",
    "20.52819402948608": "Honeywell",
    "24.10287323": "shenzhen eyunmei technology co,.ltd",
    "12.346852343808": "Narrowband Telecommunications",
    "24.14814223": "Kingston Technologies",
    "20.167860143915008": "Rpe \"Radico\"",
    "12.123917687189504": "Yuksek Kapasite Radyolink Sistemleri San. ve Tic. A.S.",
    "24.7876667": "Stephen Technologies Co.,Limited",
    "24.2417864": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.119269847040": "Plexstar Inc.",
    "24.1324859": "PROCOM Systems",
    "12.123917676453888": "Espai De Producció I Electróni",
    "24.2179": "Hewlett Packard",
    "24.2178": "Sigma Corporation",
    "24.7919344": "Samsung Electro Mechanics Co., Ltd.",
    "24.2171": "RTX Telecom A/S",
    "24.2170": "Wipotec GmbH",
    "24.2173": "Cisco Systems, Inc",
    "24.2172": "Cisco Systems, Inc",
    "24.2175": "SPAUN electronic GmbH & Co. KG",
    "24.2174": "Bon Electro-Telecom Inc.",
    "24.2177": "Digital Hands Co.,Ltd.",
    "24.2176": "BroadTel Canada Communications inc.",
    "24.1488": "Solinet Systems",
    "24.1489": "Metavector Technologies",
    "24.13636382": "Sunrex Technology Corp",
    "24.13388513": "fourtec -Fourier Technologies",
    "24.1480": "Verytech",
    "24.1481": "LG Innotek",
    "24.1482": "Hitron Technology, Inc.",
    "24.1483": "ROIS Technologies, Inc.",
    "24.1484": "Sumtel Communications, Inc.",
    "24.1485": "D&M Holdings Inc.",
    "24.1486": "Prolink Microsystems Corporation",
    "24.1487": "Thunder River Technologies, Inc.",
    "12.346856964096": "Ultimate Technology, Inc.",
    "12.123917689171968": "Akasakatec Inc.",
    "12.123917678133248": "Loanguard T/A SE Controls",
    "20.14223576924160": "Sentry360",
    "12.346856939520": "Teledyne Tekmar",
    "24.12113259": "Mirka Ltd.",
    "24.4907071": "Chengdu Superxon communication Technology Co. Ltd.",
    "24.1884442": "Wavetronix",
    "24.2283": "ROMWin Co.,Ltd.",
    "24.2282": "Motion Control Engineering, Inc",
    "24.2281": "NextGig",
    "24.2280": "Excel Master Ltd.",
    "24.2287": "Dibal,S.A.",
    "24.2286": "Logic Product Development",
    "24.2285": "ST&T Instrument Corp.",
    "24.2284": "Optical Zonu Corporation",
    "24.12598361": "Intel Corporate",
    "24.2812437": "Tibit Communications",
    "24.2289": "Voltaire",
    "24.2288": "Next Generation Systems, Inc.",
    "24.3208239": "Esp",
    "12.123917682839552": "Dr.Bridge Aquatech",
    "12.346865819648": "nCk Research LLC",
    "24.11829511": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "12.346860339200": "Sweco Japs Ab",
    "24.14436530": "Baraja Pty. Ltd.",
    "24.14191964": "Elentec",
    "24.2922417": "Huawei Technologies Co.,Ltd",
    "12.123917690339328": "Park24",
    "20.31368184594432": "Zhejiang Itenal Technology Co.,Ltd",
    "24.2418771": "LG Innotek",
    "24.10781671": "Apple, Inc.",
    "12.346866274304": "Collectric AB",
    "24.11047468": "LG Electronics (Mobile Communications)",
    "24.739": "LITE-ON Communications, Inc.",
    "24.576": "Seedek Co., Ltd.",
    "24.577": "Amer.com",
    "24.574": "Selta Telematica S.p.a",
    "24.575": "Compal Electronics INC.",
    "24.572": "Creative Technology, Ltd.",
    "24.573": "Cisco Systems, Inc",
    "24.570": "ZSK Stickmaschinen GmbH",
    "24.571": "Ericsson",
    "24.24619": "Peak Audio",
    "24.24618": "Symicron Computer Communications, Ltd.",
    "24.578": "Videoframe Systems",
    "24.579": "Raysis Co., Ltd.",
    "12.123917689843712": "Woodside Electronics",
    "20.45069245612032": "Vigil Monitoring",
    "24.7662833": "Acumen",
    "12.123917687066624": "Instro Precision Limited",
    "24.5566354": "Shenzhen Elink Technology Co., LTD",
    "20.246679828692992": "Gemalto \"Document Readers\"",
    "24.5267903": "Cisco Systems, Inc",
    "24.2827": "Corrent Corporation",
    "24.2826": "dBm Optics",
    "24.2825": "Ifoundry Systems Singapore",
    "24.2824": "Pillar Data Systems",
    "24.2823": "Voxpath Networks",
    "24.2822": "ARRIS Group, Inc.",
    "24.2821": "Pacific Broadband Networks",
    "24.2820": "Volktek Corporation",
    "24.4465393": "S.Fac, Inc",
    "12.123917683625984": "Rockwell Collins Canada",
    "24.2829": "Air2U, Inc.",
    "24.2828": "Agile Systems Inc.",
    "24.9105": "Maxian",
    "24.9104": "Algolware Corporation",
    "24.9107": "Ajinextek",
    "24.9106": "Proteus Industries Inc.",
    "24.8": "Xerox Corporation",
    "24.9": "powerpipes?",
    "24.9103": "Nidec Copal Corporation",
    "24.9102": "ADB Broadband Italia",
    "24.4": "Xerox Corporation",
    "24.5": "Xerox Corporation",
    "24.6": "Xerox Corporation",
    "24.7": "Xerox Corporation",
    "24.0": "Officially Xerox, but 0:0:0:0:0:0 is more common",
    "24.1": "Xerox Corporation",
    "24.2": "Xerox Corporation",
    "24.3": "Xerox Corporation",
    "24.1732": "Piolink Inc.",
    "24.1733": "INNOVI Technologies Limited",
    "24.1730": "Smartmatic Corporation",
    "24.1731": "Schindler Elevator Ltd.",
    "24.1736": "Sumitomo Metal Micro Devices, Inc.",
    "24.1737": "Technical Marketing Research, Inc.",
    "24.1734": "lesswire AG",
    "24.1735": "RFNET Technologies Pte Ltd (S)",
    "12.123917682266112": "Wiingtech International Co. LTD.",
    "24.1738": "American Computer & Digital Components, Inc. (ACDC)",
    "24.1739": "Jotron Electronics A/S",
    "24.3435334": "Sagemcom Broadband SAS",
    "24.2100656": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "12.123917688459264": "Apantac LLC",
    "24.837226": "Nokia Corporation",
    "20.233630195515392": "Huaqin Telecom Technology Co.,Ltd.",
    "12.346866208768": "TX Technology Corp",
    "24.2885965": "Raptor Engineering, LLC",
    "12.346861490176": "Berthel GmbH",
    "12.123917689257984": "ACD Elekronik GmbH",
    "24.6313853": "eero inc.",
    "12.346852847616": "WaterCove Networks",
    "24.9190407": "Skiva Technologies, Inc.",
    "24.8442931": "LzLabs GmbH",
    "20.83999508660224": "High Infinity Germany",
    "24.8701930": "Sony Mobile Communications Inc",
    "12.123917683802112": "BORMANN EDV und Zubehoer",
    "12.346853912576": "Streaming Networks",
    "24.6880149": "Generalplus Technology Inc.",
    "24.13403884": "Silicon Laboratories",
    "24.3417437": "Shenzhen Loadcom Technology Co.,Ltd",
    "24.323965": "Tarana Wireless",
    "24.3672892": "Texas Instruments",
    "12.346854383616": "Applied Technologies Associates",
    "24.69327": "PROFINET Multicast",
    "12.123917679411200": "Wit.com Inc",
    "20.66219389485056": "ELSA Japan Inc.",
    "24.13895784": "Beijing Lanxum Computer Technology CO.,LTD.",
    "24.6574462": "Long Ben (Dongguan) Electronic Technology Co.,Ltd.",
    "24.12860094": "Sony Mobile Communications Inc",
    "24.11824648": "Huawei Technologies Co.,Ltd",
    "24.8274": "Ragula Systems",
    "24.8275": "Huntsville Microsystems, Inc.",
    "24.8276": "Sycamore Networks",
    "24.8277": "Altech Co., Ltd.",
    "24.8270": "Network Security Systems, Inc.",
    "12.346854105088": "GFI Chrono Time",
    "24.8272": "Korea Computer Inc.",
    "24.8273": "Verilink Corporation",
    "12.123917683855360": "Oso Technologies",
    "24.6093309": "Taicang T&W Electronics",
    "24.8278": "Neoproducts",
    "24.8279": "TITZE DATENTECHNIK GmbH",
    "12.123917675077632": "KST technology",
    "24.2899774": "Alge-Timing GmbH",
    "24.7373291": "Apple, Inc.",
    "24.5813608": "Securitas Direct España, Sau",
    "12.123917680599040": "Ohashi Engineering Co.,Ltd.",
    "20.277958201704448": "Eltek brojila d.o.o.",
    "24.36971": "Applied Resources, Inc.",
    "24.36970": "Turnstone Systems, Inc.",
    "24.36973": "Cisco Systems, Inc",
    "24.36972": "Sartorius Hamburg GmbH",
    "12.346853064704": "Elbit Systems Ltd.",
    "24.36974": "Praxon, Inc.",
    "24.36977": "Applied Innovation Inc.",
    "24.36976": "Neo Networks, Inc.",
    "12.346857271296": "Thermo Electron",
    "24.36978": "Simrad As",
    "24.3450947": "KYOCERA Display Corporation",
    "24.310640": "Huawei Technologies Co.,Ltd",
    "12.123917691682816": "PCME Ltd.",
    "20.66129354555392": "Comatis",
    "24.5517194": "Tellescom Industria E Comercio Em Telecomunicacao",
    "24.5517193": "Euclid Laboratories, Inc.",
    "24.8914763": "LG Electronics (Mobile Communications)",
    "24.9198273": "Huawei Device Co., Ltd.",
    "24.1085438": "Huawei Technologies Co.,Ltd",
    "24.12381819": "ASUSTek COMPUTER INC.",
    "20.66256285728768": "Edit Srl",
    "24.16577183": "China Mobile Iot Limited company",
    "24.15255375": "Liteon Technology Corporation",
    "12.123917686087680": "Grossenbacher Systeme AG",
    "24.1068407": "Innovative Computer Engineering",
    "24.6048937": "Huawei Technologies Co.,Ltd",
    "12.346864652288": "Zucchetti Spa",
    "12.346859393024": "Boeckeler Instruments, Inc.",
    "20.211670374809600": "ista International GmbH",
    "24.14196436": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3939518": "Jadak Llc",
    "12.346856218624": "ABB Power Technologies S.p.A. Unità Operativa SACE (PTMV)",
    "24.3168695": "Microsoft",
    "24.16042532": "FreeBit Co., Ltd.",
    "24.14170869": "Wideband Labs LLC",
    "12.346864205824": "Bettini srl",
    "20.207147368448000": "Scientific Games",
    "24.13906333": "Sinwatec",
    "24.2684215": "IEEE Registration Authority",
    "24.7907041": "Brocade Communications Systems LLC",
    "24.4509309": "Sfr",
    "12.123917681561600": "Burk Technology",
    "20.194363556626432": "XMetrics",
    "24.6857143": "Honghao Mingchuan Technology (Beijing) CO.,Ltd.",
    "12.346862661632": "EEG Enterprises Inc",
    "24.14419437": "Nabtesco Corporation",
    "24.1607340": "Nokia Danmark A/S",
    "24.1078383": "Motorola Solutions Malaysia Sdn. Bhd.",
    "24.13920231": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.137149119725568": "Hitiq Limited",
    "24.10234480": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.2658768": "Stage Tec Entwicklungsgesellschaft für professionelle Audiotechnik mbH",
    "24.9964674": "Samsung Electro Mechanics Co., Ltd.",
    "24.15004870": "Netgear",
    "24.2646973": "Aptiv Services Us, Llc",
    "12.123917675458560": "Beijing Daswell Science and Technology Co.LTD",
    "24.7094634": "Cisco Systems, Inc",
    "12.346854252544": "Safetran Traffic Systems Inc.",
    "20.175931701329920": "Kontakt Micro-Location Sp z o.o.",
    "24.12368609": "AREC Inc.",
    "24.8419758": "ShanDong Tecsunrise Co.,Ltd",
    "12.346865614848": "SCHRAML GmbH",
    "24.14454816": "Ruru Tek Private Limited",
    "24.6568625": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.123917687390208": "Shs Srl",
    "12.346867224576": "T8 Ltd",
    "24.11594428": "Askey Computer Corp",
    "12.123917686341632": "Reo Ag",
    "24.8936413": "Extreme Networks, Inc.",
    "20.18540141740032": "Fuzhou Rockchip Electronics Co.,Ltd",
    "24.13384076": "Shiningtek",
    "24.1605877": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "12.123917681647616": "SonoSound ApS",
    "24.7809": "CNB Technology Inc.",
    "12.346867064832": "Saia-Burgess Controls AG",
    "24.10762492": "Plastic Logic",
    "12.123917676568576": "Oculi Vision",
    "24.10488": "Intel Corporate",
    "24.2150943": "Wincal Technology Corp.",
    "24.4752308": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1375925": "Dell Inc.",
    "24.3202694": "Cedac Software S.r.l.",
    "12.346859356160": "L-3 Communications Mobile-Vision, Inc.",
    "20.368733847552": "Datapath Limited",
    "24.14162298": "Nuheara Ltd",
    "24.8407205": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.71297884155904": "GeneSys Elektronik GmbH",
    "24.15771570": "Futaba Corporation",
    "24.15740788": "ASUSTek COMPUTER INC.",
    "24.14978444": "Routerboard.com",
    "24.9478874": "Gheo Sa",
    "12.123917684031488": "IHI Rotating Machinery Engineering Co.,Ltd.",
    "24.3459485": "Eidolon Communications Technology Co. Ltd.",
    "24.15247240": "Xiamen Leelen Technology Co., Ltd",
    "24.9206293": "Ruckus Wireless",
    "12.346857975808": "Innovative Dynamics GmbH",
    "12.123917678825472": "KMtronic ltd",
    "12.346868523008": "Telvent",
    "24.7083534": "ShotTracker",
    "12.123917679562752": "Protium Technologies, Inc.",
    "24.7368780": "Purple Communications, Inc",
    "24.11827016": "Shenzhen Neoway Technology Co.,Ltd.",
    "24.6331336": "Hipad Intelligent Technology Co., Ltd.",
    "24.7389074": "GN Audio A/S",
    "20.180659909820416": "Shenzhen Huadoo Bright Group Limitied",
    "24.9976598": "AMPAK Technology, Inc.",
    "24.7896673": "Apple, Inc.",
    "24.14724939": "Pluribus Networks, Inc.",
    "12.123917677977600": "Kvazar LLC",
    "12.346858283008": "BioAccess Tecnologia em Biometria Ltda.",
    "24.11068398": "Sony Interactive Entertainment Inc.",
    "24.10016390": "Chyi Lee industry Co., ltd.",
    "12.346858799104": "PaloDEx Group Oy",
    "24.12618135": "Samsung Electronics Co.,Ltd",
    "12.123917686390784": "Sodaq",
    "24.3703007": "Motorola Mobility LLC, a Lenovo Company",
    "24.842956": "Inala Technologies",
    "24.9461645": "modas GmbH",
    "24.1905": "Embedded System Corporation",
    "24.5485": "Ubiquiti Networks Inc.",
    "24.9969146": "Samsung Electronics Co.,Ltd",
    "24.1907": "Ascom Powerline Communications Ltd.",
    "24.1906": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.40989": "Red Lion Controls, LP",
    "24.40988": "Nascent Networks Corporation",
    "24.40985": "Nebula Consultants, Inc.",
    "24.5480": "Dilithium Networks",
    "24.40987": "Premisys Communications, Inc.",
    "24.40986": "Binar Elektronik Ab",
    "24.40981": "Wyle",
    "24.40980": "Csir",
    "24.40983": "J B M Corporation",
    "24.5481": "PECO II, Inc.",
    "12.123917682565120": "Jabil",
    "24.1903": "Synoptics Limited",
    "20.189549208666112": "8Cups",
    "12.119270027264": "inomatic GmbH",
    "24.5483": "Perfisans Networks Corp.",
    "20.57353104785408": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "12.346860724224": "VDC Display Systems",
    "24.13925237": "HARTING Electronics GmbH",
    "24.2625563": "MagnaCom",
    "24.5488": "Zebra Technologies Inc",
    "24.1908": "GuangZhou Thinker Technology Co. Ltd.",
    "24.3420": "Robert Bosch GmbH, VT-ATMO",
    "24.3421": "Raritan Computer, Inc",
    "24.3422": "NEC Personal Products",
    "24.3423": "Minds Inc",
    "24.3424": "IBM Corp",
    "24.3425": "Giga-Byte Technology Co., Ltd.",
    "24.3426": "Funkwerk Dabendorf GmbH",
    "24.3427": "DENT Instruments, Inc.",
    "24.3428": "COMAG Handels AG",
    "24.3429": "Cisco Systems, Inc",
    "24.3832010": "SoundTalks NV",
    "12.346853072896": "Frederick Engineering",
    "24.3698312": "zte corporation",
    "20.264165320032256": "DongGuan Siyoto Electronics Co., Ltd",
    "24.1093216": "Private",
    "24.14946845": "Samsung Electronics Co.,Ltd",
    "24.14732797": "Beijing E3Control Technology Co, LTD",
    "24.12249": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.14946840": "ShenZhen Rapoo Technology Co., Ltd.",
    "12.123917687853056": "Merlin CSI",
    "12.346865520640": "Hi Tech Electronics Ltd",
    "12.123917682069504": "RCH Vietnam Limited Liability Company",
    "24.1600921": "Adanis Inc.",
    "24.1341451": "Varex Imaging Deutschland AG",
    "20.180659903528960": "Selektro Power Inc",
    "12.123917677559808": "Amazon Technologies Inc.",
    "24.4747545": "Sgb Group Ltd.",
    "24.3398": "Parker SSD Drives",
    "24.3399": "Collex",
    "24.10499824": "Comprehensive Power",
    "24.1113270": "mirusystems CO.,LTD",
    "24.3392": "Verint Loronix Video Solutions",
    "24.3393": "Siemens AG ICM MP UC RD IT KLF1",
    "24.3390": "APLUX Communications Ltd.",
    "24.3391": "VTI Instruments Corporation",
    "24.3396": "Audio BU - Logitech",
    "24.3397": "Tottori SANYO Electric Co., Ltd.",
    "24.3394": "Newbest Development Limited",
    "24.3395": "DRS Tactical Systems Inc.",
    "24.29": "Cabletron Systems, Inc.",
    "24.28": "Bell Technologies",
    "24.7912144": "Shinybow Technology Co. Ltd.",
    "24.23": "Oracle",
    "24.22": "Du Pont Pixel Systems.",
    "24.21": "Datapoint Corporation",
    "24.20": "Netronix",
    "24.27": "Novell (now Eagle Technology)",
    "24.26": "Advanced Micro Devices",
    "24.25": "Applied Dynamics International",
    "24.24": "Webster Computer Corporation",
    "24.8929788": "AirTies Wireless Networks",
    "24.13119124": "Halfa Enterprise Co., Ltd.",
    "12.346868776960": "Ease Inc.",
    "12.346859802624": "Affolter Technologies SA",
    "24.4228243": "Clavister AB",
    "12.123917685088256": "Trinity College Dublin",
    "24.1065343": "Apple, Inc.",
    "12.346860744704": "Momentum Data Systems",
    "12.123917677043712": "YUYAMA MFG Co.,Ltd",
    "20.255191453532160": "Private",
    "24.12877260": "Zebra Technologies Inc",
    "12.123917681311744": "Foxtrot Research Corp",
    "24.8134323": "2N TELEKOMUNIKACE a.s.",
    "24.41212": "Image Sciences, Inc.",
    "24.41213": "Scitex Digital Printing, Inc.",
    "24.41210": "Marconi Communication GmbH",
    "24.41211": "Toray Engineering D Solutions Co., Ltd.",
    "12.123917677862912": "Active Brains",
    "24.41214": "Boston Technology, Inc.",
    "24.41215": "Tellabs Operations, Inc.",
    "20.52821129953280": "Giax GmbH",
    "24.7927742": "Samsung Electronics Co.,Ltd",
    "12.346864910336": "ELREHA GmbH",
    "24.8672924": "Coho Data, Inc.,",
    "12.123917681684480": "Axnes AS",
    "12.123917675925504": "MiWave Consulting, LLC",
    "12.123917689798656": "RoomMate AS",
    "24.16022094": "Woojeon&Handan",
    "24.2141340": "China Telecom Corporation Limited",
    "12.123917685751808": "STRACK LIFT AUTOMATION GmbH",
    "24.13651536": "Mobiwire Mobiles (NingBo) Co., LTD",
    "24.1619870": "Invoxia",
    "24.7895": "H-Stream Wireless, Inc.",
    "20.31873099104256": "Shanghai YottaTech Co Ltd (上海尧它科技有限公司）",
    "24.7897": "Mitsubishi Precision Co.,LTd.",
    "24.7896": "Digital United Inc.",
    "24.7891": "Dot Technology Int'l Co., Ltd.",
    "24.7890": "Ray Shine Video Technology Inc",
    "24.7893": "Tekon-Automatics",
    "24.7892": "Doble Engineering",
    "24.7379467": "Italian Institute of Technology",
    "24.12854358": "Palo Alto Networks",
    "24.7899": "Giken Trastem Co., Ltd.",
    "24.7898": "Wesemann Elektrotechniek B.V.",
    "24.10504151": "Best IT World (India) Pvt Ltd",
    "24.9500463": "Phillips Machine & Welding Co., Inc.",
    "24.4728879": "zte corporation",
    "12.346866794496": "Potter Electric Signal Co. LLC",
    "20.62366880366592": "Ajax Systems Inc",
    "12.123917678309376": "Seneco A/S",
    "20.66256280485888": "Dongguan Taide Industrial Co.,Ltd.",
    "24.7642869": "Samsung Electronics Co.,Ltd",
    "24.7917922": "Apple, Inc.",
    "12.346864939008": "Reflex Ces",
    "24.11564501": "zte corporation",
    "24.11837056": "Sichuan Changhong Electric Ltd.",
    "20.52912299442176": "OLIMEX Ltd",
    "24.3721946": "Apple, Inc.",
    "12.71297884061696": "NEXT! s.c. S.Piela B.Dryja",
    "24.2669": "EKS Elektronikservice GmbH",
    "24.14186379": "Intelbras",
    "24.1097461": "D-Link International",
    "12.346861240320": "Altek Electronics",
    "24.2668": "Walchem Corporation",
    "12.346854096896": "Guardian Controls International Ltd",
    "12.123917676466176": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "12.346861314048": "Fr. Sauter AG",
    "12.123917688942592": "Hermann Lümmen GmbH",
    "24.842177": "Visteon Corporation",
    "24.5245273": "Technicolor CH USA Inc.",
    "24.2665": "SUNNY bell Technology Co., Ltd.",
    "20.247076403281920": "Agora",
    "24.2664": "Solarflare Communications Inc.",
    "24.11807934": "Direct Payment Solutions Limited",
    "24.8957447": "Apple, Inc.",
    "24.13395634": "Nokia",
    "24.2667": "Tadiran Telecom Business Systems LTD",
    "24.3180011": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6278": "El-Tech, Inc.",
    "24.6279": "Metasystem SpA",
    "24.13242972": "Microsoft Corporation",
    "24.6272": "Maxim Integrated Products",
    "24.6273": "Buyang Electronics Industrial Co., Ltd",
    "24.6270": "RGB Spectrum",
    "24.6271": "Zodianet",
    "24.6276": "Fon Technology S.L.",
    "24.6277": "Motorola Solutions Inc.",
    "24.6274": "Huawei Technologies Co.,Ltd",
    "24.6275": "Formosa21 Inc.",
    "24.5296756": "Steffes Corporation",
    "24.8136244": "ARRIS Group, Inc.",
    "20.22739801866240": "iSolution Technologies Co.,Ltd.",
    "24.2663": "OngCorp",
    "24.2662": "Mitsubishi Electric System & Service Co.,Ltd.",
    "24.3209074": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9443089": "Hagenuk Marinekommunikation GmbH",
    "24.1627676": "Shenzhen TINNO Mobile Technology Corp.",
    "24.12647812": "zte corporation",
    "24.14687297": "Extreme Networks, Inc.",
    "12.346854981632": "Telda Electronics",
    "12.123917687336960": "eWireless",
    "12.346866876416": "KST Technology",
    "20.102196706279424": "VPInstruments",
    "24.3982049": "Xinhua Control Engineering Co.,Ltd",
    "12.346855428096": "Imarda New Zealand Limited",
    "20.167860153352192": "MCS Micronic Computer Systeme GmbH",
    "12.346861301760": "Calnex Solutions Limited",
    "24.5805854": "Cisco Systems, Inc",
    "12.71297883942912": "Whiptail",
    "12.346862563328": "AFORE Solutions Inc.",
    "24.7101460": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.8128035": "Ultra Electronics Sonar System Division",
    "24.1130": "Navini Networks",
    "24.1131": "Palm Wireless, Inc.",
    "24.1132": "Cyber Technology Co., Ltd.",
    "24.1133": "Cisco Systems, Inc",
    "24.1134": "Cisco Systems, Inc",
    "24.1135": "Digitel S/A Industria Eletronica",
    "24.1136": "ipUnplugged AB",
    "24.1137": "IPrad",
    "24.1138": "Telelynx, Inc.",
    "24.1139": "Photonex Corporation",
    "24.52287": "Universal Electronics, Inc.",
    "24.4488822": "Trek Technology (S) Pte Ltd",
    "24.14475693": "Shenzhen Wintop Photoelectric Technology Co., Ltd",
    "12.123917685624832": "PPI Inc.",
    "24.1311": "Taijin Media Co., Ltd.",
    "24.10525274": "Time Domain",
    "24.2411211": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.5508601": "Huawei Device Co., Ltd.",
    "24.6858": "Tilera Corporation",
    "24.6859": "Autocom Products Ltd",
    "24.2400372": "Apple, Inc.",
    "24.6857": "Suzuken Co.,Ltd",
    "24.6854": "Micro Control Designs",
    "24.3678946": "HP Inc.",
    "24.6852": "2Wire Inc",
    "24.6853": "Keysight Technologies, Inc.",
    "24.6850": "YEC Co.,Ltd.",
    "24.6851": "Scientific-Atlanta, Inc",
    "24.6558635": "Huawei Technologies Co.,Ltd",
    "24.14159055": "C.G.V. S.A.S.",
    "24.2928863": "Soliton Technologies Pvt Ltd",
    "24.55393": "Micro-Star INTL CO., LTD.",
    "24.3686923": "Sonos, Inc.",
    "24.161": "Marquette Electric Co.",
    "24.160": "SANYO Electric Co., Ltd.",
    "24.163": "Network Application Technology",
    "24.162": "Bay Networks",
    "24.165": "Tattile SRL",
    "24.164": "Acorn Computers Limited",
    "24.167": "Network Computing Devices Inc.",
    "24.166": "Network General (internal assignment, not for products)",
    "24.169": "Network Systems Corp.",
    "24.168": "Stratus Technologies",
    "24.8174284": "IEEE Registration Authority",
    "12.346864680960": "Aim",
    "12.123917687283712": "Franke Aquarotter GmbH",
    "24.15475896": "Texas Instruments",
    "24.12892621": "Cisco Systems, Inc",
    "24.14432210": "Hunan HKT Electronic Technology Co., Ltd",
    "20.159182267351040": "Teletek Electronics",
    "24.2396603": "Huawei Technologies Co.,Ltd",
    "12.123917687754752": "Softland India Ltd",
    "12.346864513024": "PTS GmbH",
    "24.7920904": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5301577": "Giga-Byte Technology Co.,Ltd.",
    "24.16314910": "Intel Corporate",
    "12.346855940096": "Optical Air Data Systems",
    "12.346854518784": "Halliburton - NUMAR",
    "12.346862477312": "Innovation, Institute, Inc",
    "24.11062928": "Cvc",
    "24.3987": "Landis+Gyr Ltd.",
    "24.3986": "Microhard Systems Inc.",
    "24.3985": "Aerotelecom Co.,Ltd.",
    "24.3984": "Cisco Systems, Inc",
    "24.3983": "Cisco Systems, Inc",
    "24.3982": "Dongyang Telecom Co.,Ltd.",
    "24.3981": "FAST TV-Server AG",
    "24.3980": "Gigawavetech Pte Ltd",
    "12.123917679800320": "Impulse Networks Pte Ltd",
    "24.11555004": "Shenzhen Basicom Electronic Co.,Ltd.",
    "24.3989": "ELECOM Co.,LTD Laneed Division",
    "24.3988": "Genexis BV",
    "24.5004749": "Oy Finnish Electric Vehicle Technologies Ltd",
    "12.123917688549376": "Xirgo Technologies LLC",
    "12.123917678673920": "SeaTech Intelligent Technology (Shanghai) Co., LTD",
    "12.123917686472704": "Season Design Technology",
    "12.123917678829568": "myenergi Ltd",
    "12.71297883267072": "Par-Tech, Inc.",
    "24.9734112": "Xiaomi Communications Co Ltd",
    "20.224447336284160": "Hebei ZHSF Technology Co.,Ltd.",
    "24.12371482": "Spectra Embedded Systems",
    "24.10790927": "Shenzhen Coship Electronics Co., Ltd.",
    "24.16302368": "Xi'an Link-Science Technology Co.,Ltd",
    "12.346858274816": "Buyang Electronics Industrial Co., Ltd.",
    "12.123917683851264": "bentrup Industriesteuerungen",
    "12.346861522944": "AT-Automation Technology GmbH",
    "24.3770": "Hanmi Semiconductor Co., Ltd.",
    "12.346859286528": "Stream Processors",
    "24.57382": "Redlake MASD LLC",
    "12.123917688602624": "Idneo Technologies S.A.U.",
    "12.123917690077184": "Davitor AB",
    "24.3772": "Paragon Fidelity GmbH",
    "24.15205387": "Apple, Inc.",
    "24.57380": "Gadzoox Networks",
    "24.3735237": "Ellips B.V.",
    "24.839": "Intel Corporation",
    "24.838": "Hitachi Kokusai Electric, Inc.",
    "24.837": "Routrek Networks Corporation",
    "24.836": "Tietech.Co., Ltd.",
    "24.835": "Martin Professional A/S",
    "24.834": "Nortel Networks",
    "24.833": "Axon Digital Design",
    "24.832": "Floware Wireless Systems, Ltd.",
    "24.831": "BigBand Networks, Ltd.",
    "24.830": "Tateyama System Laboratory Co., Ltd.",
    "24.3776": "Nortel Networks",
    "24.2391373": "Texas Instruments",
    "24.3777": "MYNAH Technologies",
    "24.11826446": "Belkin International Inc.",
    "24.4224224": "Intel Corporate",
    "24.2624348": "Apple, Inc.",
    "24.9981591": "Starlight Marketing (H. K.) Ltd.",
    "12.346855452672": "Red Rock Networks",
    "12.123917676986368": "Tecnologix s.r.l.",
    "12.71297884667904": "Private",
    "12.346859741184": "Naturela Ltd.",
    "24.8159789": "Nortec",
    "24.3677657": "Fn-Link Technology Limited",
    "20.189244296396800": "HellaStorm, Inc.",
    "24.9461550": "TCT mobile ltd",
    "12.346867798016": "QUANTA S.r.l.",
    "24.7667272": "Advantech Co., Ltd.",
    "20.211525437489152": "Wingtech Mobile Communications Co.,Ltd.",
    "24.7635354": "PT. Hartono Istana Teknologi",
    "12.123917681414144": "Telonic Berkeley Inc",
    "24.8929098": "Texas Instruments",
    "24.184095": "CMC",
    "24.11810513": "Renesas Electronics (Penang) Sdn. Bhd.",
    "24.5045418": "Tesla,Inc.",
    "24.184092": "Communication Machinery Corporation",
    "24.4517940": "Kaonmedia CO., LTD.",
    "12.346861735936": "iQue RFID Technologies BV",
    "24.292963": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "12.123917676761088": "Techworld Industries Ltd",
    "24.15210595": "IEEE Registration Authority",
    "24.3150729": "Siemens AG, Automations & Drives,",
    "12.346858893312": "flsystem",
    "20.83999506563072": "Fuse",
    "20.105645770539008": "insensiv GmbH",
    "20.75685687197696": "CETC Avionics.L td",
    "24.9708851": "Huawei Technologies Co.,Ltd",
    "12.123917690834944": "Vitro Technology Corporation",
    "20.176032227262464": "GfG mbH",
    "12.123917690482688": "Virtual Control Systems Ltd",
    "24.11332956": "Intel Corporate",
    "12.123917688352768": "Postmark Incorporated",
    "24.2328": "Samsung Techwin Co.,Ltd",
    "24.2329": "MDS Gateways",
    "24.11075514": "Huawei Technologies Co.,Ltd",
    "24.2320": "Simple Access Inc.",
    "24.2321": "Cisco Systems, Inc",
    "24.2322": "Cisco Systems, Inc",
    "24.2323": "SystemK Corporation",
    "24.2324": "Computrols Inc.",
    "24.2325": "CAS Corp.",
    "24.2326": "Listman Home Technologies, Inc.",
    "24.2327": "WEM Technology Inc",
    "20.207147369496576": "Intuitive Surgical, Inc",
    "20.124214340222976": "TiVACI CORPORATION PTE LTD",
    "24.2897861": "Qingdao Haier Intelligent Home Appliance Technology Co.,Ltd",
    "24.3436189": "Ericsson AB",
    "24.12330695": "Cisco Systems, Inc",
    "24.7662": "Nextvision Sistemas Digitais De Televisão Ltda.",
    "24.7663": "Trimm, Inc.",
    "24.7660": "Marusys",
    "24.7661": "Grid Net, Inc.",
    "24.7666": "Netflix, Inc.",
    "24.7667": "SBS Science & Technology Co., Ltd",
    "24.7664": "Vidient Systems, Inc.",
    "24.7665": "Intego Systems, Inc.",
    "24.7668": "Magellan Technology Pty Limited",
    "24.7669": "Sunshine Co,LTD",
    "24.8898": "Proview Eletrônica do Brasil LTDA",
    "24.8899": "Zeeport Technology Inc.",
    "24.13416584": "AnB Securite s.a.",
    "24.6856950": "Huawei Technologies Co.,Ltd",
    "24.13403500": "Shenzhen Mdk Digital Technology Co.,Ltd",
    "20.17622842212352": "Audio Engineering Ltd.",
    "12.123917687267328": "Mettler Toledo",
    "12.123917685137408": "Tecsys do Brasil Industrial Ltda",
    "12.346856497152": "ads-tec GmbH",
    "12.346855493632": "Consens Zeiterfassung GMBH",
    "24.1370888": "Huawei Technologies Co.,Ltd",
    "12.123917686779904": "Insitu, Inc",
    "24.3210168": "Huawei Technologies Co.,Ltd",
    "12.346854567936": "Azimuth Systems, Inc.",
    "24.9443919": "EM Microelectronic",
    "24.1875790": "Coosea Group (Hk) Company Limited",
    "24.4504379": "Google, Inc.",
    "24.4249148": "Equitech Industrial(DongGuan)Co.,Ltd",
    "24.3801746": "Video Electronics Standards Association",
    "20.149935434498048": "Shenzhen JingHanDa Electronics Co.Ltd",
    "20.101728418529280": "Shenzhen IP3 Century Intelligent Technology CO.,Ltd",
    "24.7102236": "Lenovo Mobile Communication Technology Ltd.",
    "24.1677": "SEPATON, Inc.",
    "24.1676": "3Com",
    "24.1675": "AirRunner Technologies, Inc.",
    "24.1674": "NeuronNet Co. Ltd. R&D Center",
    "24.1673": "yLez Technologies Pte Ltd",
    "24.3462930": "Smartisan Digital Co., Ltd",
    "24.1671": "Omnitron Systems Technology, Inc.",
    "24.1670": "ZARDCOM Co., Ltd.",
    "24.1679": "Telemonitor, Inc.",
    "24.1678": "HID Corporation",
    "12.123917681037312": "Shanghai JaWay Information Technology Co., Ltd.",
    "24.317315": "Y&H E&C Co.,LTD.",
    "20.163808567885824": "Silver-I Co.,LTD.",
    "20.176069207392256": "Delta Tau Data Systems, Inc.",
    "20.277783365287936": "Shenzhen Kuku Technology Co.,Ltd",
    "24.7347195": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.83999500271616": "Nuviz Oy",
    "24.48798": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.154055558037504": "Riegl Laser Measurement Systems GmbH",
    "12.123917678522368": "Nuance Hearing Ltd.",
    "12.346864164864": "Advanced Desktop Systems Ltd",
    "24.8959142": "Motorola Mobility LLC, a Lenovo Company",
    "24.16561546": "Wapice Ltd.",
    "24.6131": "Harris Corporation",
    "24.6130": "Apple, Inc.",
    "24.6133": "Lig Neoptek",
    "24.6132": "Zeron Alliance",
    "24.6135": "CEM Solutions Pvt Ltd",
    "24.6134": "Pyramid Meriden Inc.",
    "24.6137": "Forcom Sp. z o.o.",
    "24.6136": "Motech Industries Inc.",
    "24.6139": "Fa",
    "24.6138": "Microsoft Corporation",
    "24.24794": "Red Lion Controls, LP",
    "24.24795": "Ntp Elektronik A/S",
    "24.24792": "Elmic Systems, Inc.",
    "24.24793": "Transys Networks Inc.",
    "24.24790": "NovAtel Inc.",
    "24.24791": "Ecole Polytechnique Federale De Lausanne (Epfl)",
    "24.16534924": "Shenzhen Pante Electronics Technology Co., Ltd",
    "24.2925980": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.9068": "Apple, Inc.",
    "24.16571466": "G-Wearables Corp.",
    "24.6605870": "Xiaomi Communications Co Ltd",
    "20.39986092048384": "Hunan Honestone lntelligence Technology Co.,Ltd",
    "12.346858115072": "Emcom Systems",
    "24.2418983": "TomTom International BV",
    "12.346862063616": "Ipitek",
    "24.48423": "Exar Corp.",
    "12.346853474304": "ITS, Inc.",
    "24.6566875": "Texas Instruments",
    "24.1899784": "IEEE Registration Authority",
    "20.233630186078208": "Sammi Onformation Systems",
    "24.320593": "Texas Instruments",
    "24.15245517": "Nintendo Co.,Ltd",
    "24.16519558": "Logic3 plc",
    "12.346868817920": "Pulsotronic Anlagentechnik GmbH",
    "24.13925444": "IEEE Registration Authority",
    "24.11333232": "Fontem Ventures BV",
    "24.6563166": "Observint Technologies, Inc.",
    "20.9814040117248": "Sango Electronics Co",
    "24.11061883": "Huawei Technologies Co.,Ltd",
    "24.5262171": "CONTROLtronic GmbH",
    "12.123917687631872": "Fstudio Co Ltd",
    "24.9314": "Rayzone Corporation",
    "24.9315": "Phybridge Inc",
    "24.1345766": "Apple, Inc.",
    "24.9317": "Elentec",
    "24.9310": "Hivision Co.,ltd",
    "24.9311": "Vine Telecom CO.,Ltd.",
    "24.9312": "Giaval Science Development Co. Ltd.",
    "24.9313": "Shin Wang Tech.",
    "24.9318": "Unitron nv",
    "24.9319": "AOC International (Europe) GmbH",
    "20.229206566895616": "CentrAlert, Inc.",
    "24.284389": "Intel Corporate",
    "24.320041": "Qingdao Haier Technology Co.,Ltd",
    "24.10749953": "Amazon Technologies Inc.",
    "24.10800914": "Espressif Inc.",
    "24.7906976": "Hangzhou Ezviz Software Co.,Ltd.",
    "12.123917680713728": "Crde",
    "24.825037": "Mercury Corporation",
    "20.132869112463360": "CS Instruments GmbH",
    "24.14728259": "C2 Microsystems, Inc.",
    "12.346861170688": "ODF Optronics, Inc.",
    "24.1873245": "I-Lax Electronics Inc.",
    "24.7125445": "Delta Networks, Inc.",
    "12.123917678514176": "Boutronic",
    "12.123917677121536": "Esource Srl",
    "24.11336048": "Huawei Technologies Co.,Ltd",
    "24.3934065": "Sony Corporation",
    "24.5506523": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.8943998": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917683081216": "Zumbach Electronic AG",
    "12.346866438144": "Mistral Solutions Pvt. Ltd",
    "24.14214062": "Cirtec Medical Systems",
    "24.11072989": "ARRIS Group, Inc.",
    "20.229108130775040": "Nanjing Huamai Technology Co.,Ltd",
    "20.225209249431552": "Shenzhen Royole Technologies Co., Ltd.",
    "24.10538027": "Murata Manufacturing Co., Ltd.",
    "24.3167876": "Shenzhen Yunji Intelligent Technology Co.,Ltd",
    "12.123917683134464": "Data Sciences International",
    "24.16458": "West Australian Department",
    "24.16459": "Maple Computer Systems",
    "24.16454": "Udc Research Limited",
    "24.16455": "Wind River Systems",
    "24.16456": "Smd Informatica S.A.",
    "24.16457": "Roche Diagnostics International Ltd.",
    "24.16450": "N.A.T. Gmbh",
    "24.16451": "Nokia Data Communications",
    "24.16452": "Qnix Computer Co., Ltd.",
    "24.16453": "Twinhead Corporation",
    "24.9776": "Acorel S.A.S",
    "12.346857766912": "Bedo Elektronik GmbH",
    "24.9777": "Commtact Ltd",
    "24.11278374": "Seiko Epson Corporation",
    "24.45079": "InfoGear Technology Corp.",
    "24.9774": "Chengdu Jiuzhou Electronic Technology Inc",
    "20.251018413604864": "Hangzhou Ole-Systems Co., Ltd",
    "24.9775": "Hamamatsu Toa Electronics",
    "24.15732907": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "24.9772": "IKT Advanced Technologies s.r.o.",
    "12.123917675823104": "Fin Robotics Inc",
    "24.9212959": "Cisco Systems, Inc",
    "12.123917689671680": "Contiweb",
    "24.9773": "Wistron Corporation",
    "12.123917684867072": "LGM Ingénierie",
    "24.9224093": "Shanghai Xinyou Information Technology Ltd. Co.",
    "24.9770": "Proxense, LLC",
    "24.1315635": "AzureWave Technology Inc.",
    "24.9771": "Wongs Electronics Co. Ltd.",
    "24.9594": "CAMCO Produktions- und Vertriebs-GmbH für Beschallungs- und Beleuchtungsanlagen",
    "24.9965716": "Shenzhen Vsun communication technology Co.,ltd",
    "24.9595": "Stj Electronics Pvt Ltd",
    "24.10024833": "ARRIS Group, Inc.",
    "24.2942668": "Ruckus Wireless",
    "24.6306479": "Samsung Electronics Co.,Ltd",
    "20.272805451005952": "Dongguan Shun Hing Plastics Limited",
    "12.346852704256": "DIALOG4 System Engineering GmbH",
    "24.14471415": "Bentek Systems Ltd.",
    "24.8696147": "Apple, Inc.",
    "20.255861681291264": "plc2 Design GmbH",
    "12.346866626560": "CT Company",
    "12.123917687394304": "CableLabs",
    "12.346861871104": "Keith & Koep GmbH",
    "12.71297884360704": "shanghai Anjian Information technology co. , ltd.",
    "24.1361559": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346862112768": "Soehnle Professional GmbH & Co.KG",
    "20.146101708521472": "Dantherm",
    "24.3424680": "ER-Telecom",
    "24.10260188": "Teracom Limited",
    "24.14176055": "Proxis, spol. s r.o.",
    "12.346867675136": "FAL Corp",
    "24.14743998": "Cloudena Corp.",
    "24.12425": "Spectrapoint Wireless, LLC",
    "24.12424": "Ericsson",
    "24.12427": "Brix Networks",
    "24.12426": "Nicotra Sistemi S.P.A",
    "24.12421": "Cisco Systems, Inc",
    "24.12420": "Allied Telesyn Internaional",
    "24.12423": "Vega Grieshaber Kg",
    "24.4235": "Laseranimation Sollinger Gmbh",
    "24.270668": "Insight Energy Ventures LLC",
    "24.3427133": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12429": "Pinnacle Systems, Inc.",
    "24.12428": "Quantum Corporation",
    "24.3451115": "Apple, Inc.",
    "24.16013378": "Boxx TV Ltd",
    "12.71297883631616": "Dorlet Sau",
    "12.123917676126208": "DiTEST Fahrzeugdiagnose GmbH",
    "24.11553727": "Pt Han Sung Electoronics Indonesia",
    "24.15529041": "Arcadyan Corporation",
    "24.3701308": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.145727909003264": "Vayyar Imaging Ltd.",
    "24.7882163": "Huawei Device Co., Ltd.",
    "24.585643": "Sayme Wireless Sensor Network",
    "24.1873311": "Veethree Electronics And Marine LLC",
    "24.12626160": "Adamson Systems Engineering",
    "24.9751737": "ARRIS Group, Inc.",
    "20.39986085756928": "Lty Llc",
    "24.55893": "Cisco Systems, Inc",
    "24.10265975": "Osorno Enterprises Inc.",
    "24.15733697": "Hannto Technology Co., Ltd",
    "12.123917688119296": "Sicon srl",
    "24.5249023": "Cisco Systems, Inc",
    "24.2400608": "Espressif Inc.",
    "24.12386125": "Espressif Inc.",
    "24.6051192": "iryx corp",
    "12.123917683609600": "CommBox P/L",
    "24.10520756": "Intel Corporate",
    "12.119269302272": "Tokyo Cosmos Electric, Inc.",
    "24.3170578": "PAV GmbH",
    "12.71297883611136": "elgris UG",
    "24.553029": "Apple, Inc.",
    "24.5805528": "IEEE Registration Authority",
    "24.3946947": "JW Electronics Co., Ltd.",
    "24.10519239": "P.T.I Co.,LTD",
    "24.9962395": "Mellanox Technologies, Inc.",
    "20.79215068184576": "Ridango AS",
    "24.11297815": "Siemens AG",
    "20.83889190076416": "Shandong Linkotech Electronic Co., Ltd.",
    "24.8421797": "TONGQING COMMUNICATION EQUIPMENT (SHENZHEN) Co.,Ltd",
    "24.7638811": "MasterImage 3D",
    "24.1328728": "Huawei Technologies Co.,Ltd",
    "12.123917675909120": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "24.7614171": "Perinet GmbH",
    "24.558624": "Tecno Mobile Limited",
    "24.3948814": "Cisco Systems, Inc",
    "20.176189943578624": "s&t embedded GmbH",
    "24.5022997": "Baikal Electronics JSC",
    "24.10265875": "Samsung Electronics Co.,Ltd",
    "24.3412624": "Treeview Co.,Ltd.",
    "24.10768469": "Busware.De",
    "24.11581543": "Hewlett Packard Enterprise",
    "20.97400597250048": "Shenzhen DOOGEE Hengtong Technology CO.,LTD",
    "24.45890": "MacroSAN Technologies Co., Ltd.",
    "12.346855632896": "North Pole Engineering, Inc.",
    "20.66256283631616": "Teknoware Oy",
    "12.346866278400": "Telvent",
    "12.346868498432": "Digital Barriers",
    "24.4740": "Lab33 Srl",
    "24.4741": "Gizmondo Europe Ltd",
    "24.4742": "Endevco Corp",
    "24.4743": "Digital Everywhere Unterhaltungselektronik GmbH",
    "24.4744": "2Wire Inc",
    "24.4745": "Advance Sterilization Products",
    "24.4746": "ARRIS Group, Inc.",
    "24.4747": "Sensory Networks Inc",
    "24.4748": "Woodward Governor",
    "24.4749": "STB Datenservice GmbH",
    "24.15266531": "Starcor Beijing Co.,Limited",
    "24.15266530": "LG Innotek",
    "24.14463281": "Shenzhen Huaruian Technology Co.,Ltd",
    "24.3219": "Xeline Co., Ltd.",
    "24.3218": "WolfVision Gmbh",
    "24.3217": "Riverhead Networks Inc.",
    "24.3216": "Octasic Inc.",
    "24.3215": "Nergal s.r.l.",
    "24.3214": "Mentor Engineering Inc",
    "24.3213": "MATRIX VISION GmbH",
    "24.3212": "Kodicom Co.,Ltd.",
    "24.3211": "Connect Tech Inc",
    "24.3210": "Bose Corporation",
    "20.216039231062016": "Silvus technologies inc",
    "24.10802354": "Shenzhen MeiG Smart Technology Co.,Ltd",
    "24.8670197": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917685465088": "Ideas srl",
    "12.123917685604352": "Meca System",
    "24.14446998": "Intel Corporate",
    "24.4746028": "Bbk Educational Electronics Corp.,Ltd.",
    "24.1064772": "Xiaomi Communications Co Ltd",
    "12.123917684903936": "Rch Italia Spa",
    "12.123917678182400": "Dsd Microtechnology,Inc.",
    "12.123917676806144": "OES Inc.",
    "24.8414573": "Samsung Electronics Co.,Ltd",
    "12.346854039552": "EVER Sp. z o.o.",
    "24.1849279": "Cloudium Systems Ltd.",
    "24.10288168": "Apple, Inc.",
    "12.123917681565696": "MB connect line GmbH Fernwartungssysteme",
    "24.1051684": "pomdevices, LLC",
    "24.49340": "Telecom Australia/Cssc",
    "24.3438649": "zte corporation",
    "24.13423233": "Apple, Inc.",
    "12.123917686972416": "YUYAMA MFG Co.,Ltd",
    "20.229206581575680": "Revox Inc.",
    "24.11585599": "Jiangsu Cynray IOT Co., Ltd.",
    "24.8686221": "Cisco Systems, Inc",
    "24.15221886": "Bitdefender SRL",
    "12.346861297664": "Shadrinskiy Telefonny Zavod",
    "20.123598316503040": "Rivian Automotive LLC",
    "12.123917675982848": "Dextera Labs",
    "24.15475287": "JiangSu NanJing University Electronic Information Technology Co.,Ltd",
    "12.346864635904": "RF Code, Inc",
    "24.2626673": "Lantis co., LTD.",
    "24.13406507": "TE Connectivity Touch Solutions",
    "24.2160734": "Delta Electronics",
    "12.119269457920": "SEED International Ltd.",
    "24.13920821": "Cisco Systems, Inc",
    "12.123917676941312": "MondeF",
    "24.16041748": "Huawei Technologies Co.,Ltd",
    "12.346866397184": "Axlon AB",
    "24.13372908": "Cisco SPVTG",
    "24.3440118": "Intel Corporate",
    "24.7111166": "Mitsuba Corporation",
    "24.3939844": "D-Link International",
    "12.346855280640": "Digilent Inc.",
    "24.701": "Bionet Co., Ltd.",
    "24.700": "LVL 7 Systems, Inc.",
    "24.703": "dotRocket, Inc.",
    "24.702": "Totsu Engineering, Inc.",
    "24.705": "Innovative Electronic Designs, Inc.",
    "24.704": "Bencent Tzeng Industry Co., Ltd.",
    "24.707": "Arelnet Ltd.",
    "24.706": "Net Vision Telecom",
    "24.709": "Evertz Microsystems Ltd.",
    "24.708": "OPT Machine Vision Tech Co., Ltd",
    "24.16508": "Qume Corporation",
    "24.12383327": "zte corporation",
    "24.7460": "Aclara Power-Line Systems Inc.",
    "24.7461": "Samsung Electronics Co.,Ltd",
    "24.7462": "Rockridgesound Technology Co.",
    "24.7463": "Nac-Intercom",
    "24.7464": "Sony Mobile Communications Inc",
    "24.7465": "Doro AB",
    "24.7466": "Shenzhen Bul-Tech Co.,Ltd.",
    "24.7467": "Wuhan Pont Technology CO. , LTD",
    "24.7468": "Wavetrend Technologies (Pty) Limited",
    "24.7469": "Pylone, Inc.",
    "24.6839094": "Powertech Industrial Co., Ltd.",
    "24.6839093": "Apple, Inc.",
    "12.346854678528": "ATOM GIKEN Co.,Ltd.",
    "24.1885163": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346858885120": "Dsp Design",
    "24.14953730": "TCT mobile ltd",
    "20.181338943520768": "Beijing ICPC CO.,Ltd.",
    "24.5828": "SiRF Technology, Inc.",
    "24.5829": "Shenzhen Xing Feng Industry Co.,Ltd",
    "24.5822": "INFRANET, Inc.",
    "24.5823": "PaloDEx Group Oy",
    "24.5820": "Nokia Danmark A/S",
    "24.5821": "ATI Industrial Automation",
    "24.5826": "Avtec Systems Inc",
    "24.5827": "BA Systems Inc",
    "24.5824": "Semtech Corporation",
    "24.5825": "Eleksen Ltd",
    "12.123917681766400": "ACD Elekronik GmbH",
    "24.1628489": "Qvis Labs, LLC",
    "12.123917680836608": "812th AITS",
    "24.16559375": "Qts Networks",
    "24.1108088": "Nokia",
    "12.346866110464": "Yardney Technical Products Inc.",
    "12.346864074752": "ADI Video Technologies",
    "24.1636975": "ISC applied systems corp",
    "24.9208102": "Arcom",
    "12.123917681610752": "Sanmina Israel Medical Systems Ltd",
    "12.123917691453440": "Rohde&Schwarz Topex SA",
    "12.346868490240": "CONET Solutions GmbH",
    "12.346857209856": "RCS Energy Management Ltd",
    "24.4773196": "Jeda Networks",
    "24.6080979": "Palladium Energy Eletronica Da Amazonia Ltda",
    "24.9473309": "GH Technologies",
    "24.8951775": "Entrypass Corporation Sdn. Bhd.",
    "24.5549776": "DASAN Networks, Inc.",
    "20.31346168692736": "winsun AG",
    "24.5549778": "CSL Dualcom Ltd",
    "24.1587481": "Unicoi Systems",
    "24.13386851": "Apple, Inc.",
    "24.2650": "GreenNET Technologies Co.,Ltd.",
    "24.2651": "Power-One as",
    "24.2652": "Carel s.p.a.",
    "24.2653": "FingerTec Worldwide Sdn Bhd",
    "24.2654": "3Com",
    "24.2655": "almedio inc.",
    "24.6339": "CS Corporation",
    "24.6338": "Firetide, Inc",
    "24.6337": "Almitec Informática e Comércio",
    "24.6336": "ARRIS Group, Inc.",
    "24.6335": "Essence Technology Solution, Inc.",
    "24.6334": "ANSA Corporation",
    "24.6333": "Shenzhen Dvbworld Technology Co., Ltd.",
    "12.346855219200": "QUBIsoft S.r.l.",
    "24.6331": "Eliwell Controls srl",
    "24.6330": "Cisco Systems, Inc",
    "24.3825155": "Cirrus Data Solutions, Inc",
    "24.1309": "Airocon, Inc.",
    "24.1308": "Xnet Technology Corp.",
    "24.1307": "Magic Control Technology Corporation",
    "24.1306": "3Com Europe Ltd",
    "24.1305": "Siemens Building Technologies AG,",
    "24.1304": "Jupiters Technology",
    "24.1303": "Shellcomm, Inc.",
    "24.1302": "SMART Modular Technologies",
    "24.1301": "Nuark Co., Ltd.",
    "24.1300": "KDT Systems Co., Ltd.",
    "12.346861993984": "kuroneko dennnou kenkyuushitsu",
    "12.346860331008": "CEDAR Audio Limited",
    "24.8429": "Giga-Byte Technology Co., Ltd.",
    "24.8428": "Techware Systems Corp.",
    "24.8425": "Dantel",
    "24.8424": "Datatrek Corporation",
    "24.8427": "Cincinnati Microwave, Inc.",
    "24.8426": "Efficient Networks, Inc.",
    "24.8421": "Apex Data",
    "24.8420": "Hsing Tech Enterprise Co., Ltd",
    "24.8423": "B&W Nuclear Service Company",
    "24.8422": "Lidkoping Machine Tools Ab",
    "12.346858868736": "NetworkSound, Inc",
    "12.123917688504320": "M.S. Control",
    "24.5537229": "Texas Instruments",
    "24.9457983": "Elettronica Santerno SpA",
    "12.346864033792": "SMH Technologies",
    "24.8694948": "Beijing Novel Super Digital TV Technology Co., Ltd",
    "24.12069596": "EFR Europäische Funk-Rundsteuerung GmbH",
    "24.9232663": "zte corporation",
    "24.5178": "Raytalk International Srl",
    "24.5179": "Sensovation AG",
    "20.123598309163008": "Ibyte",
    "24.5170": "Tarallax Wireless, Inc.",
    "24.5171": "Empower Technologies(Canada) Inc.",
    "24.5172": "Keri Systems, Inc",
    "24.5173": "CityCom Corp.",
    "24.5174": "Qwerty Elektronik AB",
    "24.5175": "GSTeletech Co.,Ltd.",
    "24.5176": "Hewlett Packard Enterprise",
    "24.5177": "Blonder Tongue Laboratories, Inc",
    "24.4883": "GuangZhou Post & Telecom Equipment ltd",
    "24.4882": "Amedia Networks Inc.",
    "24.4881": "ARRIS Group, Inc.",
    "24.4880": "Cisco-Linksys, LLC",
    "24.4887": "GN Netcom A/S",
    "24.4886": "L-S-B Broadcast Technologies GmbH",
    "24.4885": "Sony Interactive Entertainment Inc.",
    "24.4884": "Asiamajor Inc.",
    "12.346856398848": "Contec Steuerungstechnik & Automation GmbH",
    "24.4889": "Cisco Systems, Inc",
    "24.4888": "DGSTATION Co., Ltd.",
    "24.16569398": "Motorola Mobility LLC, a Lenovo Company",
    "24.6855117": "Cisco Systems, Inc",
    "24.16255947": "Cisco Systems, Inc",
    "20.268735386484736": "Stype CS d.o.o.",
    "24.13107855": "Nova Electronics (Shanghai) Co., Ltd.",
    "24.9992048": "Pep Digital Technology (Guangzhou) Co., Ltd",
    "12.346860912640": "Wireless Acquisition LLC",
    "24.12375478": "d2d technologies",
    "20.176772042719232": "UnaliWear, Inc.",
    "24.9993798": "Emizon Networks Limited",
    "24.6350730": "Unisem",
    "24.16540345": "Alpsalpine Co,.Ltd",
    "24.12365096": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.14219024": "Libre Wireless Technologies Inc.",
    "12.346855985152": "LabJack Corporation",
    "12.123917685288960": "Iotize",
    "24.264613": "HFR, Inc.",
    "24.3168400": "Frontier Silicon Ltd",
    "12.346855223296": "Lingg & Janke OHG",
    "24.284": "Universal Talkware Corporation",
    "24.56587": "Ungermann-Bass Inc.",
    "24.56584": "Ungermann-Bass Inc.",
    "24.287": "RC Networks, Inc.",
    "24.56582": "Ungermann-Bass Inc.",
    "24.56583": "Ungermann-Bass Inc.",
    "24.56580": "Ungermann-Bass Inc.",
    "24.283": "Unizone Technologies, Inc.",
    "24.288": "Oscilloquartz S.A.",
    "24.289": "WatchGuard Technologies, Inc.",
    "24.56588": "Ungermann-Bass Inc.",
    "24.56589": "Ungermann-Bass Inc.",
    "24.2914318": "Huawei Device Co., Ltd.",
    "24.11569602": "Zyptonite",
    "12.346868224000": "NDC Infrared Engineering, Inc.",
    "24.4227": "HP-UX E 9000/889",
    "24.4226": "Jna Telecommunications Limited",
    "24.4225": "Dps, Inc.",
    "24.4224": "Metawave Communications",
    "24.4223": "Crestron Electronics, Inc.",
    "24.4222": "BACHMANN ELECTRONIC GmbH",
    "24.4221": "Aurora Communications, Ltd.",
    "24.4220": "P-Com, Inc.",
    "24.6577854": "Yahoo! JAPAN",
    "24.12064281": "ARRIS Group, Inc.",
    "24.4229": "Polaris Communications, Inc.",
    "24.4228": "K-Bot Communications",
    "24.12908601": "Actia Automotive",
    "24.6836950": "UGame Technology Co.,Ltd",
    "24.4209916": "Apple, Inc.",
    "24.1841856": "Apple, Inc.",
    "24.9987818": "Industrial Control Communications, Inc.",
    "24.53291": "Jetcell, Inc.",
    "24.53290": "Voxent Systems Ltd.",
    "24.53293": "Resideo",
    "24.53292": "Campbell Scientific, Inc.",
    "24.53295": "Vlsi Technology Inc.",
    "24.53294": "Communication Automation Corp.",
    "24.53297": "Industrial Logic Corporation",
    "24.53296": "Safetran Systems Corp",
    "24.53299": "Dalian Daxian Network",
    "24.53298": "Yano Electric Co., Ltd.",
    "12.346855829504": "Foresearch",
    "24.8653613": "Samsung Electro Mechanics Co., Ltd.",
    "20.224447324749824": "Terma Sp. z o.o.",
    "20.71070624251904": "National Research Council of Canada",
    "24.5539516": "Cisco Systems, Inc",
    "12.346857226240": "Herholdt Controls srl",
    "24.14208905": "Beijing DoSee Science and Technology Co., Ltd.",
    "12.123917680410624": "Aplex Technology Inc.",
    "24.5281976": "New H3C Technologies Co., Ltd",
    "24.4479017": "Cisco SPVTG",
    "24.5279087": "CHAHOO Limited",
    "12.123917678678016": "Crde",
    "20.39919493840896": "Leshi Internet Information & Technology (Beijing) Corp.",
    "12.123917683478528": "Waco Giken Co., Ltd.",
    "12.123917686083584": "IDEAL INDUSTRIES Ltd t/a Casella",
    "24.4985445": "ARRIS Group, Inc.",
    "24.5566069": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "24.8436715": "Satmap Systems Ltd",
    "12.71297883668480": "Nachi-Fujikoshi Corp",
    "12.346857472000": "Summit Developmen",
    "24.7617056": "New H3C Technologies Co., Ltd",
    "12.123917682589696": "Centro de Ingenieria y Desarrollo industrial",
    "24.11565861": "Dell Inc.",
    "24.12060502": "Cisco Meraki",
    "24.9699504": "Huawei Technologies Co.,Ltd",
    "20.70443721555968": "Lexi Devices, Inc.",
    "12.119269801984": "Triax A/S",
    "12.346862280704": "Metodo2",
    "12.123917689241600": "Dewetron GmbH",
    "12.123917683576832": "Elektronik Art S.C.",
    "24.3461744": "Amazon Technologies Inc.",
    "24.6859550": "Apple, Inc.",
    "12.346868305920": "Telebyte Inc.",
    "24.10523463": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346864533504": "Vvdn Technologies Pvt. Ltd.",
    "20.171977816932352": "Toughdog Security Systems",
    "12.346854498304": "Chantry Networks",
    "24.28516": "Samsung Electronics Co.,Ltd",
    "12.123917679620096": "GreenWake Technologies",
    "12.346863079424": "Advanced Integrated Systems",
    "12.346864611328": "Euro-Konsult Sp. z o.o.",
    "24.2109931": "Hdsn",
    "12.123917690044416": "Thurlby Thandar Instruments LTD",
    "24.12589859": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "20.71414657843200": "Kato Engineering Inc.",
    "12.123917678305280": "Nemeus-Sas",
    "24.3936367": "Albahith Technologies",
    "24.8127448": "Sagemcom Broadband SAS",
    "24.71": "Nicolet Instruments Corp.",
    "24.7612503": "Mayfield Robotics",
    "24.5039734": "Buffalo.Inc",
    "24.3443218": "Aruba, a Hewlett Packard Enterprise Company",
    "24.9534": "Sensus Metering Systems",
    "24.9536": "Quasar Technologies, Inc.",
    "24.9537": "Maquet Critical Care AB",
    "24.9530": "CEVA, Ltd.",
    "24.9531": "din Dietmar Nocker Facilitymanagement GmbH",
    "24.9532": "2Wire Inc",
    "24.9533": "DRS Consolidated Controls",
    "24.9538": "Pittasoft",
    "24.9539": "Moneytech",
    "12.346858196992": "R. L. Drake Company",
    "20.264036224598016": "Xiamen Jinhaode Electronic Co.,Ltd",
    "12.346857566208": "IRTrans GmbH",
    "12.346866155520": "NIK-ELEKTRONIKA Ltd",
    "24.14993938": "zte corporation",
    "24.7105143": "Intel Corporate",
    "12.346855329792": "Symbium Corporation",
    "24.12122195": "Arcadyan Corporation",
    "12.123917678153728": "Wayne Analytics Llc",
    "24.538854": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.3968248": "TRENDnet, Inc.",
    "24.2911221": "Juniper Networks",
    "12.123917676486656": "4Jtech s.r.o.",
    "12.123917683929088": "Contec Americas Inc.",
    "24.32896": "Datamedia Corporation",
    "24.7146": "Nintendo Co., Ltd.",
    "24.32894": "Southern Pacific Ltd.",
    "24.7144": "Ultratronik GmbH",
    "24.7143": "Postek Electronics Co., Ltd.",
    "24.7142": "Vr Ag",
    "24.32890": "Aitech Systems Ltd.",
    "24.7140": "Townet Srl",
    "24.10543279": "Cisco Systems, Inc",
    "12.123917690826752": "Vitec",
    "24.7149": "Brocade Communications Systems LLC",
    "24.7148": "Netio Technologies Co., Ltd",
    "24.2517": "Signal Communication, Inc.",
    "24.2516": "Transtech Networks",
    "24.2515": "Western DataCom Co., Inc.",
    "24.2514": "Mai Logic Inc.",
    "24.2513": "Seranoa Networks Inc",
    "24.2512": "Solacom Technologies Inc.",
    "24.2511": "iAd GmbH",
    "24.2510": "SpaceBridge Semiconductor Corp.",
    "20.228844751552512": "RHENAC Systems GmbH",
    "24.2519": "DC Security Products",
    "24.2518": "KNC One GmbH",
    "12.123917676638208": "Newtec A/S",
    "24.12096029": "Bg T&A",
    "12.123917687517184": "Sensortech Systems Inc.",
    "24.2115546": "Xiaomi Communications Co Ltd",
    "24.7142996": "Intel Corporate",
    "12.346867167232": "Wine Technology Marlborough",
    "20.57176930385920": "Hibertek International Limited",
    "24.8921571": "Nebusens, S.L.",
    "24.1842744": "PCCW Global, Inc.",
    "12.346857459712": "Clearsonics Pty. Ltd.",
    "12.123917681106944": "Contec Americas Inc.",
    "12.123917684158464": "KST technology",
    "24.13956950": "zte corporation",
    "24.14974898": "Fujitsu Limited",
    "24.3686323": "Technicolor CH USA Inc.",
    "20.368730701824": "KoolPOS Inc.",
    "20.212014906474496": "Ixon B.V.",
    "20.211525432246272": "Stello",
    "24.5300554": "Private",
    "20.229631758172160": "Hyundai Autohow",
    "24.4520770": "Apple, Inc.",
    "24.5294496": "Skipper As",
    "24.15261398": "Intrising Networks, Inc.",
    "24.5523935": "Peloton Interactive, Inc",
    "24.12860871": "Netgear",
    "12.346867171328": "Tattile srl",
    "24.823886": "Rice Lake Weighing Systems",
    "24.8402007": "OT Systems Limited",
    "24.7380781": "zte corporation",
    "24.10283671": "Honor Device Co., Ltd.",
    "12.71297884622848": "Solidscape Inc",
    "12.123917684809728": "Sigmann Elektronik GmbH",
    "24.3987418": "SK Mtek microelectronics(shenzhen)limited",
    "24.8941807": "SC Professional Trading Co., Ltd.",
    "20.238070232252416": "GLO Science",
    "24.16280947": "Zengge Co., Limited",
    "20.88914841829376": "AEG Identifikationssysteme GmbH",
    "20.132869113511936": "Björkviks Consulting AB",
    "12.123917675499520": "CheckBill Co,Ltd.",
    "20.84531183878144": "outpaceIO",
    "24.822680": "Top Eight Ind Corp",
    "20.211670380052480": "Gardner Denver Thomas GmbH",
    "24.11298692": "Google, Inc.",
    "24.8145784": "Red Sun Synthesis Pte Ltd",
    "24.15730897": "Espressif Inc.",
    "24.6334289": "Huawei Device Co., Ltd.",
    "12.346865258496": "TAE Antriebstechnik GmbH",
    "24.10502304": "Netgear",
    "12.346858471424": "HCJB Global",
    "12.123917681455104": "EarTex",
    "24.11596220": "Dhemax Ingenieros Ltda",
    "12.123917681963008": "OutdoorLink",
    "12.346855112704": "Worth Data, Inc.",
    "24.5255597": "ABB Global Industries and Services Private Limited",
    "20.71389203660800": "A-Iox Inc.",
    "24.10258605": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16271987": "Eumtech Co., Ltd",
    "12.346852233216": "Bihl + Wiedemann GmbH",
    "24.2387606": "Dell Inc.",
    "24.15848": "LG Electronics (Mobile Communications)",
    "24.7891996": "Apple, Inc.",
    "20.207357804019712": "Wind Mobility Technology (Beijing) Co., Ltd",
    "24.3203216": "Linctronix Ltd,",
    "12.123917683306496": "Xerox International Partners",
    "24.12851595": "Dominion Voting Systems Corporation",
    "24.14200635": "Hangzhou Bigbright Integrated communications system Co.,Ltd",
    "24.2384555": "Espressif Inc.",
    "24.15773778": "Ruckus Wireless",
    "24.8683325": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917685583872": "Tunstall A/S",
    "24.2150084": "Peraso Technologies",
    "24.8167854": "Apple, Inc.",
    "12.346856501248": "Power-One Inc.",
    "24.12536": "Dynapro Systems, Inc.",
    "12.346863869952": "Trench Austria GmbH",
    "24.806377": "Apple, Inc.",
    "24.13107728": "LG Innotek",
    "24.40334": "Cardiac Recorders, Inc.",
    "24.7384869": "Nishiyama Industry Co.,LTD.",
    "12.71297885110272": "Owl Cyber Defense Solutions, LLC",
    "24.1110371": "Tnk Co.,Ltd",
    "24.12616649": "Huawei Device Co., Ltd.",
    "12.346853822464": "Momentum Data Systems",
    "24.12539": "AZS Technology AG",
    "24.10810305": "Open Source Robotics Foundation, Inc.",
    "24.43580": "Olivetti Telecom Spa (Olteco)",
    "24.15215725": "SmartRG, Inc.",
    "12.346855718912": "Tattile srl",
    "24.5297564": "Thai Habel Industrial Co., Ltd.",
    "24.12882188": "Apple, Inc.",
    "24.2136701": "Sagemcom Broadband SAS",
    "24.7642489": "Sercomm Corporation.",
    "20.189549205520384": "JSC InfoTeCS",
    "24.13685728": "Aruba, a Hewlett Packard Enterprise Company",
    "20.52819401900032": "Sichuan Nebula Networks Co.,LTD.",
    "24.13167352": "zte corporation",
    "24.7359867": "ASUSTek COMPUTER INC.",
    "24.1880447": "NEC Platforms, Ltd.",
    "24.283731": "Apple, Inc.",
    "12.123917691416576": "Nordson Corporation",
    "12.346862809088": "meconet e. K.",
    "12.346867474432": "Solace Systems",
    "24.10768": "Cisco Systems, Inc",
    "24.16397": "LANNET Data Communications",
    "24.16396": "General Micro Systems, Inc.",
    "24.16395": "Cisco Systems, Inc",
    "24.16394": "Pivotal Technologies, Inc.",
    "24.16393": "Tachibana Tectron Co., Ltd.",
    "24.16392": "A Plus Info Corporation",
    "24.16391": "Telmat Informatique",
    "24.16390": "Sampo Technology Corporation",
    "24.16399": "Datacom Technologies",
    "24.16398": "Memotec, Inc.",
    "24.3203962": "Intel Corporate",
    "24.14711906": "Whistle Labs, Inc.",
    "12.346867793920": "Kamacho Scale Co., Ltd.",
    "12.346864435200": "JSC Electrical Equipment Factory",
    "12.346856394752": "Ktec Ltd",
    "24.11297258": "Apple, Inc.",
    "12.123917684625408": "Suzhou Wansong Electric Co.,Ltd",
    "24.14173165": "Suzuken",
    "20.198611828736000": "Shenzhen Zhuifengma Technology Co., Ltd",
    "20.189244310028288": "Duravit AG",
    "20.175931700281344": "Beijing Lyratone Technology Co., Ltd",
    "12.123917690122240": "Shenzhen Shi Fang Communication Technology Co., Ltd",
    "24.15784930": "Apple, Inc.",
    "24.11584852": "D-Link International",
    "24.6347207": "Apple, Inc.",
    "24.15791534": "Adaptrum Inc.",
    "24.9982538": "Technicolor CH USA Inc.",
    "24.15243373": "Palo Alto Networks",
    "24.12360753": "Huawei Technologies Co.,Ltd",
    "12.123917689745408": "Vtron Pty Ltd",
    "24.13398662": "Xi'an Fengyu Information Technology Co., Ltd.",
    "24.7387887": "Hangzhou H3C Technologies Co., Limited",
    "24.12910054": "Drtech",
    "20.159304571158528": "Sindoh Techno Co., Ltd.",
    "24.3446235": "Logitec Corporation",
    "20.13592324734976": "Adi",
    "12.346857783296": "ads-tec GmbH",
    "24.2915204": "OOO Petr Telegin",
    "24.16558556": "Renesas Electronics (Penang) Sdn. Bhd.",
    "24.7612076": "Cisco Systems, Inc",
    "24.13632658": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12620438": "zte corporation",
    "20.163603968688128": "Gowing Business And Contracting Wenzhou Co., LTD",
    "12.346856013824": "InAccess Networks",
    "12.346853457920": "NBG Industrial Automation B.V.",
    "12.123917677809664": "Opgal Optronic Industries",
    "24.15257753": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917690261504": "Natav Services Ltd.",
    "20.264165328420864": "Shenzhen Nufilo Electronic Technology Co., Ltd.",
    "12.346856603648": "Ducommun LaBarge Technologies, Inc",
    "24.12375827": "Owl Labs",
    "24.7391283": "Wirepas Oy",
    "24.6034487": "Thyssenkrupp Aufzugswerke GmbH",
    "24.13687119": "IEEE Registration Authority",
    "24.807957": "Intel Corporate",
    "24.303894": "Texas Instruments",
    "12.346866196480": "EMAC, Inc.",
    "24.12337408": "CODACO ELECTRONIC s.r.o.",
    "20.176189949870080": "Wuhan Rui Ying Tong Network Technology Co., Ltd(China)",
    "24.3418352": "ATN International Limited",
    "24.3989417": "ARRIS Group, Inc.",
    "24.16279130": "Apple, Inc.",
    "20.272689499471872": "Technology Research, LLC",
    "12.346868785152": "Elan Systems",
    "24.5027868": "SAM Electronics GmbH",
    "12.123917687472128": "Kunshan excellent Intelligent Technology Co., Ltd.",
    "24.6057807": "S.A. Sistel",
    "20.163603972882432": "Shandong free optical technology co., ltd.",
    "12.123917678145536": "Sunstone Engineering",
    "12.123917683822592": "chargeBIG",
    "12.123917678575616": "White Matter LLC",
    "24.15239276": "Shenzhen SC Technologies Co.,LTD",
    "24.8681779": "profichip GmbH",
    "24.5376": "Intel Corporate",
    "24.5377": "LexBox",
    "24.5374": "Artech Electronics",
    "24.5375": "Precise Automation, Inc.",
    "24.5372": "Extandon, Inc.",
    "24.5373": "Thecus Technology Corp.",
    "24.5370": "AsGa S.A.",
    "24.5371": "Technical Solutions Inc.",
    "24.2416932": "Xiamen Milesight IoT Co., Ltd.",
    "24.5378": "BETA tech",
    "24.5379": "PROFIcomms s.r.o.",
    "12.123917685346304": "HumanEyes Technologies Ltd.",
    "24.14428441": "AltoBeam (Xiamen) Technology Ltd, Co.",
    "24.15004116": "vivo Mobile Communication Co., Ltd.",
    "12.346854535168": "RGB Spectrum",
    "24.1112782": "Reacheng Communication Technology Co.,Ltd",
    "24.13659978": "Topwell International Holdings Limited",
    "20.255861684436992": "Medica Corporation",
    "20.159182263156736": "Neone, Inc.",
    "24.12081168": "Numera, Inc.",
    "24.155744": "3com",
    "24.3951250": "Hewlett Packard",
    "24.1311847": "SNK Technologies Co.,Ltd.",
    "24.1067363": "Harting K.K.",
    "20.66426710786048": "TP Radio",
    "12.123917676261376": "Vidisys GmbH",
    "24.14462366": "GreenPriz",
    "20.88383264129024": "Hagiwara Solutions Co., Ltd",
    "24.16297511": "John Deere Electronic Solutions",
    "24.8155128": "ShenZhen ACTO Digital Video Technology Co.,Ltd.",
    "20.277981928882176": "Soma GmbH",
    "24.14218172": "Liteon Technology Corporation",
    "12.123917680988160": "Lehigh Electric Products Co",
    "12.123917679280128": "Smith Meter, Inc.",
    "24.57479": "LeCroy - Networking Productions Division",
    "24.57478": "Emerson Network Power, Avocent Division",
    "24.3188873": "Decimator Design",
    "12.346856370176": "Rohde&Schwarz Topex SA",
    "24.57470": "Walt Disney Imagineering",
    "12.346867556352": "Alyrica Networks",
    "24.57472": "Control Resources Corporation",
    "24.57475": "Jato Technologies, Inc.",
    "24.57474": "Anerma",
    "24.57477": "Global Maintech, Inc.",
    "24.57476": "Compulite R&D",
    "12.346856378368": "Guralp Systems Limited",
    "24.15253121": "Sagemcom Broadband SAS",
    "24.16277052": "Shenzhen Zhibotong Electronics Co.,Ltd",
    "20.212188458385408": "Shenzhen Comix Hst Cloud Computing Co., Ltd.",
    "12.346865524736": "Code Blue Corporation",
    "24.13161513": "Muehlbauer AG",
    "24.5014494": "Cisco SPVTG",
    "24.12359984": "Huawei Technologies Co.,Ltd",
    "24.11846774": "Wuhan Maritime Communication Research Institute",
    "20.198611820347392": "QKM Technology(Dongguan)Co.,Ltd",
    "12.346868142080": "Project service S.a.s",
    "24.5016405": "Grupo Digicon",
    "24.2650864": "Athena",
    "20.57194448945152": "uikismart",
    "24.10770447": "IEEE Registration Authority",
    "24.3973914": "Oilfind International LLC",
    "12.346860666880": "Telcom",
    "24.8921996": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917687951360": "R Cubed Engineering, LLC",
    "24.9961588": "Raisecom Technology CO., LTD",
    "24.13384487": "Tecno Mobile Limited",
    "24.5041840": "SHC Netzwerktechnik GmbH",
    "24.6338071": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.9975224": "Assembled Products Corporation",
    "12.123917676494848": "Ocean",
    "24.2901849": "Beijing MEGA preponderance Science & Technology Co. Ltd",
    "12.119269654528": "HCS KABLOLAMA SISTEMLERI SAN. ve TIC.A.S.",
    "24.9210755": "Texas Instruments",
    "12.123917678379008": "Opti-Sciences, Inc.",
    "24.6617415": "Senscomm Semiconductor Co., Ltd.",
    "24.16043093": "Juniper Networks",
    "12.346857390080": "Hwayoung RF Solution Inc",
    "24.3008": "China IWNComm Co., Ltd.",
    "24.3009": "Bay Microsystems, Inc.",
    "24.3006": "Cisco Systems, Inc",
    "24.3007": "Cisco Systems, Inc",
    "24.3004": "En Garde Systems, Inc.",
    "24.3005": "Connexionz Limited",
    "24.3002": "Harmonic, Inc",
    "24.3003": "Etin Systems Co., Ltd",
    "24.3000": "Kihoku Electronic Co.",
    "24.3001": "Imsys AB",
    "12.346866491392": "M & Paul, Inc",
    "24.20569": "iBAHN",
    "24.20568": "Sangoma Technologies",
    "12.123917682229248": "Institute of Power Engineering, Gdansk Division",
    "24.20561": "Iwatsu Electric Co., Ltd.",
    "24.20560": "Cisco Systems, Inc",
    "24.20563": "Cisco Systems, Inc",
    "24.20562": "Tiara Networks, Inc.",
    "24.20565": "Doms A/S",
    "24.20564": "Cisco Systems, Inc",
    "24.20567": "Broadband Access Systems",
    "24.20566": "VMware, Inc.",
    "24.13654952": "zte corporation",
    "12.346866372608": "CS-Instruments",
    "20.62335520604160": "Doepke Schaltgeräte GmbH",
    "24.1884110": "Shenzhen Century Xinyang Technology Co., Ltd",
    "24.10513673": "Intelcan Technosystems Inc.",
    "24.6292148": "Wistron Neweb Corporation",
    "12.123917675728896": "AirBie AG",
    "20.146101706424320": "Hangzhou Nationalchip Science & Technology Co.,Ltd.",
    "24.8135128": "Shenzhen Think Will Communication Technology co., LTD.",
    "24.13387321": "WAAV, Inc.",
    "24.8965189": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346860605440": "Acube Systems s.r.l.",
    "12.123917685428224": "RFEL Ltd",
    "24.8655262": "Samsung Electronics Co.,Ltd",
    "24.15243610": "PiiGAB, Processinformation i Goteborg AB",
    "12.346867486720": "nicai-systems",
    "24.15747627": "PYRAMID Computer GmbH",
    "24.20699": "Contemporary Control",
    "24.20698": "3Com",
    "24.20697": "ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA",
    "24.20696": "Unicorn Computer Corp.",
    "24.20695": "Telstrat",
    "24.20694": "Atlas Copco Tools Ab",
    "24.20693": "Ad Systems Corp.",
    "24.20692": "Joohong Information &",
    "24.20691": "Digital Audio Processing Pty. Ltd.",
    "24.20690": "CMC Electronics Inc",
    "24.13924438": "Avaya Inc",
    "24.53881": "Vingroup Joint Stock Company",
    "12.123917678698496": "GPSat Systems",
    "24.6987172": "Global Reach Technology",
    "12.123917678350336": "Firecom, Inc.",
    "12.346863812608": "Phytec Messtechnik GmbH",
    "24.19969": "Dell Inc.",
    "24.6081433": "Samsung Electronics Co.,Ltd",
    "24.6788": "V One Multimedia Pte Ltd",
    "24.6789": "NV Michel Van de Wiele",
    "12.123917681483776": "Lineage Power Pvt Ltd.,",
    "24.6782": "LN Srithai Comm Ltd.",
    "24.6783": "GCI Science & Technology Co.,LTD",
    "24.6780": "Hirschmann Multimedia B.V.",
    "24.6781": "cyber-blue(HK)Ltd",
    "24.6786": "PROBA Building Automation Co.,LTD",
    "24.6787": "Pegasus Technologies Inc.",
    "24.6784": "Sony Corporation",
    "24.6785": "Zelax",
    "12.346859520000": "The Dini Group, La Jolla inc.",
    "20.260167874641920": "Sarcos Corp",
    "24.810165": "IEEE Registration Authority",
    "12.71297884045312": "Mitsubishi Heavy Industries, Ltd.",
    "12.123917680689152": "Telensa Ltd",
    "24.4254968": "Siemens AG",
    "24.12859098": "Rusteletech Ltd.",
    "24.11282746": "Apple, Inc.",
    "12.346858762240": "MangoDSP",
    "24.3433096": "Cisco Systems, Inc",
    "24.5027050": "Hrd (S) Pte., Ltd.",
    "24.7785": "Thomson Inc.",
    "24.7784": "Quanta Computer Inc.",
    "24.7787": "Cisco SPVTG",
    "24.7786": "Beijing Bluexon Technology Co.,Ltd",
    "24.7781": "Intel Corporate",
    "24.7780": "Intel Corporate",
    "24.7783": "Intel Corporate",
    "24.7782": "RESOL Elektronische Regelungen GmbH",
    "24.7789": "IT R&D Center",
    "24.7788": "Opaque Systems",
    "12.346855272448": "Gillam-Fei S.A.",
    "24.3824167": "Shenzhen Longsys Electronics Co.,Ltd.",
    "24.48699": "Huawei Technologies Co.,Ltd",
    "20.93065822863360": "Ausounds Intelligence, Llc",
    "24.9633": "Enalasys",
    "24.9632": "Nintendo Co., Ltd.",
    "24.9631": "TechnoDigital Technologies GmbH",
    "24.9630": "Huawei Technologies Co.,Ltd",
    "24.9637": "Walnut Media Network",
    "24.9636": "EuroDesign embedded technologies GmbH",
    "24.9635": "Trimax Wireless, Inc.",
    "24.9634": "Alta Definicion LINCEO S.L.",
    "24.9639": "itron",
    "24.9638": "Central Network Solution Co., Ltd.",
    "24.12358069": "Infinix mobility limited",
    "20.97755440611328": "Broad Air Technology Co., LTD.",
    "24.1400": "Private",
    "24.1401": "Universal Control Solution Corp.",
    "24.1402": "Overture Networks",
    "24.1403": "Chung Nam Electronic Co., Ltd.",
    "24.1404": "RCO Security AB",
    "24.1405": "Sun Communications, Inc.",
    "24.1406": "Eckelmann Steuerungstechnik GmbH",
    "24.1407": "Acqis Technology",
    "24.1408": "FibroLAN Ltd.",
    "24.1409": "Snell",
    "24.10014124": "Espressif Inc.",
    "12.71297884577792": "Emphysys, Inc.",
    "24.8953850": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8961400": "Flaircomm Microelectronics,Inc.",
    "24.15229374": "Melec Inc.",
    "24.8967762": "AMERGINT Technologies",
    "24.1090745": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "24.15211944": "Ruckus Wireless",
    "12.123917675794432": "Nexus Technologies Pty Ltd",
    "20.229558872702976": "Dali A/S",
    "24.13377817": "Samsung Electronics Co.,Ltd",
    "24.9189": "IPaXiom Networks",
    "24.9188": "IPnect co. ltd.",
    "24.8163618": "Samsung Electronics Co.,Ltd",
    "24.9185": "Cavena Image Products AB",
    "24.9184": "INO Therapeutics LLC",
    "24.9187": "Microtronic AG",
    "24.9186": "SEA Signalisation",
    "24.9181": "Elgin S.A.",
    "24.9180": "Benein, Inc",
    "24.9183": "Apple, Inc.",
    "20.13692011806720": "Beijing Miiiw Technology Co., Ltd",
    "24.15747898": "Juniper Networks",
    "24.3160562": "ADB Broadband Italia",
    "24.15766469": "Garland Technology",
    "24.14978868": "Vodafone Italia S.p.A.",
    "24.6595321": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.2674116": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.346868350976": "Noralta Technologies Inc",
    "24.14718870": "Huawei Technologies Co.,Ltd",
    "24.10796641": "EutroVision System, Inc.",
    "24.4516317": "Sagemcom Broadband SAS",
    "24.1608517": "Cisco Systems, Inc",
    "20.145211815624704": "Guangdong Creator&Flyaudio Electronic Technology Co.,LTD",
    "24.7636905": "OCT Technology Co., Ltd.",
    "24.8582": "Universal Global Scientific Industrial Co., Ltd.",
    "24.8583": "Imacs GmbH",
    "24.8580": "Powersoft Srl",
    "24.8581": "Micro-Star Int'L Co.,Ltd.",
    "24.8586": "Electronic Design and Manufacturing Company",
    "24.8587": "Wescon Technology, Inc.",
    "24.8584": "EMC Corporation",
    "24.8585": "AppTech, Inc.",
    "24.5819218": "Google, Inc.",
    "24.8588": "TopControl GMBH",
    "24.8589": "AP Router Ind. Eletronica LTDA",
    "20.171812084252672": "Wireless Environment, LLC",
    "24.10017476": "Tp-Link Technologies Co.,Ltd.",
    "24.10490587": "Tabuchi Electric Co.,Ltd",
    "20.145211817721856": "Goldmund Switzerland",
    "12.123917680033792": "Fresenius Medical Care",
    "24.11572622": "Tp-Link Technologies Co.,Ltd.",
    "24.4928": "AD.EL s.r.l.",
    "24.4929": "Shandong New Beiyang Information Technology Co.,Ltd",
    "24.9748373": "Shenzhen Coship Electronics Co., Ltd",
    "24.57558": "Computer & Communication Research Lab.",
    "24.4920": "Fresenius-Vial",
    "24.4921": "CCV Deutschland GmbH",
    "24.4922": "VadaTech Inc.",
    "24.4923": "Speed Dragon Multimedia Limited",
    "24.4924": "Quintron Systems Inc.",
    "24.4925": "Micro Memory Curtiss Wright Co",
    "24.4926": "MetaSwitch",
    "24.4927": "Eppendorf Instrumente GmbH",
    "24.1101381": "Miromico AG",
    "12.346868379648": "SYSTEMTECHNIK GmbH",
    "20.128612188553216": "Simon Electric (China) Co.,ltd",
    "24.190163": "Nixdorf Computer Corp.",
    "24.15508958": "ONYX WIFI Inc",
    "12.346853646336": "Time Domain",
    "24.12632000": "Huawei Technologies Co.,Ltd",
    "24.7626268": "Pioneer Corporation",
    "12.119269568512": "Metrycom Communications Ltd",
    "24.16024525": "Tp-Link Technologies Co.,Ltd.",
    "12.71297884303360": "Testbook Ltd",
    "12.346859470848": "RV Technology Limited",
    "24.3678877": "ADB Broadband Italia",
    "12.346861154304": "EIZO Technologies GmbH",
    "24.5512699": "Freestyle Energy Ltd",
    "24.13910373": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.119269752832": "LUMINO GmbH",
    "24.3935451": "Unionman Technology Co.,Ltd",
    "24.12335850": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.10753079": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917690798080": "Riegl Laser Measurement Systems GmbH",
    "12.123917682212864": "Abb Spa - Dmpc",
    "24.4753128": "Microsoft Corporation",
    "24.10285959": "Apple, Inc.",
    "24.6563667": "Huawei Device Co., Ltd.",
    "24.3421755": "Apple, Inc.",
    "12.346854830080": "Orion Technologies,LLC",
    "24.9706582": "Actiontec Electronics, Inc",
    "12.123917684600832": "LG Electronics",
    "12.71297885126656": "Aplex Technology Inc.",
    "24.11534919": "AMPAK Technology, Inc.",
    "24.4995956": "VOGTEC(H.K.) Co., Ltd",
    "24.12903686": "Piper Networks, Inc.",
    "24.1058681": "Sadel S.p.A.",
    "24.3410687": "Qingdao Hisense Communications Co.,Ltd.",
    "24.4515033": "Cisco Systems, Inc",
    "24.1628655": "Shuttle Inc.",
    "24.14157750": "Tp-Link Technologies Co.,Ltd.",
    "24.2905820": "ASUSTek COMPUTER INC.",
    "24.11583784": "Adler ELREHA GmbH",
    "12.123917675323392": "HUGEL GmbH",
    "24.16308748": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8713195": "Espressif Inc.",
    "24.1848141": "ITS Telecom",
    "12.346867363840": "LS Control A/S",
    "12.346860290048": "European Industrial Electronics B.V.",
    "24.3430524": "Mirae Information Technology Co., Ltd.",
    "24.5791734": "zte corporation",
    "12.346855239680": "Profline B.V.",
    "24.15522128": "Irt Sa",
    "24.16010026": "Cisco SPVTG",
    "24.6834862": "Huawei Technologies Co.,Ltd",
    "24.11272400": "zte corporation",
    "24.11335142": "Cisco Systems, Inc",
    "24.1880132": "Askey Computer Corp",
    "24.15524605": "SKG Electric Group(Thailand) Co., Ltd.",
    "24.11277212": "Adtran Inc",
    "12.123917679132672": "Intervala",
    "24.14424577": "Ecoliv Technology ( Shenzhen ) Ltd.",
    "12.346857373696": "Prodco International Inc.",
    "24.12861508": "RackTop Systems Inc.",
    "24.829607": "Samsung Electronics Co.,Ltd",
    "24.6312236": "TCT mobile ltd",
    "20.149935438692352": "Gigatech R&D Corp.",
    "24.562006": "Huawei Technologies Co.,Ltd",
    "20.176069199003648": "rNET Controls",
    "24.7628044": "Apple, Inc.",
    "24.12123290": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "24.7385879": "Cisco Systems, Inc",
    "24.10243726": "ALT Systems Ltd",
    "12.119269683200": "LLC Emzior",
    "24.5252395": "Nethra Imaging Incorporated",
    "24.2146267": "Dvl",
    "12.123917676359680": "C21 Systems Ltd",
    "12.123917680898048": "Megger Germany GmbH",
    "12.123917681389568": "Cominfo, Inc.",
    "12.346854494208": "Stins Coman",
    "24.9487690": "vivo Mobile Communication Co., Ltd.",
    "12.346858020864": "Spansion",
    "12.123917682593792": "NETWAYS GmbH",
    "12.346865188864": "Dorlet Sau",
    "24.12333181": "Wistron Neweb Corporation",
    "24.12333182": "Wistron Neweb Corporation",
    "12.123917681074176": "ACD Elekronik GmbH",
    "24.14951252": "Shenzhen Fuzhi Software Technology Co.,Ltd",
    "12.123917683642368": "Gentec",
    "12.123917691637760": "Laser Imagineering Vertriebs GmbH",
    "12.346865385472": "Saia-Burgess Controls AG",
    "24.9438760": "Samsung Electronics Co.,Ltd",
    "12.346856026112": "Norbit ODM AS",
    "24.560719": "SF Software Solutions",
    "20.154019633823744": "Anyware Solutions ApS",
    "24.3965035": "Hunan Voc Acoustics Technology Co., Ltd.",
    "12.346860535808": "TIS Dialog LLC",
    "24.8935942": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.15748988": "Amazon Technologies Inc.",
    "24.63008": "Google, Inc.",
    "24.58852": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "20.67047989968896": "UltiMachine",
    "12.346860658688": "InfoGLOBAL",
    "20.242213040685056": "Adial",
    "24.262687": "Huawei Technologies Co.,Ltd",
    "12.346861195264": "Systemtechnik GmbH",
    "20.150399872925696": "Adopt Nettech Pvt Ltd",
    "12.346856562688": "Topway Industries Ltd.",
    "12.346867785728": "B:TECH, a. s.",
    "24.8168989": "Mhl, Llc",
    "24.9759282": "Apple, Inc.",
    "24.810200": "DOLI Elektronik GmbH",
    "12.123917690089472": "Potomac Electric Corporation",
    "12.123917683212288": "Computechnic AG",
    "20.31372471173120": "Newps co.,ltd",
    "24.5037069": "Coral Telecom Limited",
    "24.5520996": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.10810562": "Vnpt Technology",
    "12.346861838336": "ALPHATRONICS nv",
    "12.346860920832": "Art Spa",
    "12.123917675737088": "Reonix Automation",
    "24.14178690": "Huawei Technologies Co.,Ltd",
    "24.4204299": "Mobileeco Co., Ltd",
    "20.211525438537728": "Gronn Kontakt AS",
    "12.123917690253312": "Dream D&S Co.,Ltd",
    "20.57176926191616": "Sithon Technologies SAS",
    "24.6617536": "Analog Devices",
    "20.216602649821184": "Danego BV",
    "24.10281059": "Samsung Electronics Co.,Ltd",
    "24.4240333": "Chiyoda Electronics Co.,Ltd.",
    "12.346861428736": "UltraVision Security Systems, Inc.",
    "12.123917686996992": "Torion Plasma Corporation",
    "24.6860799": "Shanghai Cambricon Information Technology Co., Ltd.",
    "24.2123644": "Hewlett Packard Enterprise",
    "24.6611314": "Shenzhen Qihoo Intelligent Technology Co.,Ltd",
    "24.16053720": "Google, Inc.",
    "12.123917675872256": "Guangzhou Male Industrial Animation Technology Co.,Ltd.",
    "24.10800419": "Shenzhenshi Xinzhongxin Co., Ltd",
    "24.307559": "Xiaomi Communications Co Ltd",
    "24.9997898": "Noxus(Beijing) Technology Co.,Ltd",
    "12.346859016192": "RF Code, Inc.",
    "24.1324427": "Mediabridge Products, LLC.",
    "12.346859294720": "Rgm Spa",
    "12.123917676331008": "GS Elektromedizinsiche Geräte G. Stemple GmbH",
    "12.346868371456": "PumpWell Solutions Ltd.",
    "20.184991243829248": "Beglec",
    "12.71297883205632": "Digital Audio SA",
    "20.172584095186944": "Guangzhou LANGO Electronics Technology Co., Ltd.",
    "24.9494144": "Shenzhen Century Xinyang Technology Co., Ltd",
    "24.13413164": "HUMAX Co., Ltd.",
    "24.12616826": "Apple, Inc.",
    "24.822492": "Suunto Oy",
    "12.346867240960": "Detech Electronics ApS",
    "24.12636198": "Huawei Device Co., Ltd.",
    "12.346863706112": "Master Co Ltd",
    "20.57353105833984": "Nanjing Sandemarine Electric Co.,Ltd",
    "24.2674650": "Apple, Inc.",
    "24.11842402": "zte corporation",
    "24.2111238": "Cisco Systems, Inc",
    "24.9717398": "BaudTec Corporation",
    "24.9459330": "Lenbrook Industries Limited",
    "24.6295714": "Crompton Instruments",
    "20.180394589683712": "Shenzhen Yiwanjia Information Technology Co.,Ltd",
    "20.171977815883776": "Beijing Picohood Technology Co.,Ltd",
    "24.12633014": "Automatic Systems",
    "24.8651560": "Juniper Networks",
    "12.346863894528": "Applied Micro Electronics AME bv",
    "24.17995": "Huawei Technologies Co.,Ltd",
    "20.14286623604736": "Dspread International Co.,Limited",
    "24.15768145": "Shanghai Viroyal Electronic Technology Company Limited",
    "24.9761400": "Elma Electronic",
    "12.346866425856": "Saia-Burgess Controls AG",
    "12.123917683785728": "KBS Industrieelektronik GmbH",
    "24.15500591": "Apple, Inc.",
    "24.5519720": "Edgewater Networks Inc",
    "24.32818": "Access Co., Ltd.",
    "24.7643235": "Huawei Technologies Co.,Ltd",
    "24.10539306": "AXPRO Technology Inc.",
    "24.14435437": "Allwinner Technology Co., Ltd",
    "24.3703621": "Tp-Link Technologies Co.,Ltd.",
    "12.346863792128": "Elcoteq Design Center Oy",
    "20.101728414334976": "28 Gorilla",
    "12.346852167680": "JMBS Developpements",
    "12.123917675933696": "SinTau SrL",
    "24.12856323": "Apple, Inc.",
    "24.12881551": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9750882": "Teseq AG",
    "12.346859589632": "RIEXINGER Elektronik",
    "20.14286622556160": "Fender Musical Instrument",
    "24.1089449": "Actiontec Electronics, Inc",
    "24.12850236": "Cisco Systems, Inc",
    "20.67047979483136": "Home Control AS",
    "24.6876822": "Cisco SPVTG",
    "24.15762017": "Sagemcom Broadband SAS",
    "24.1608469": "ShenZhen ZhongRuiJing Technology co.,LTD",
    "24.15218997": "Hewlett Packard",
    "24.13680540": "Hewlett Packard",
    "24.4986692": "Amazon Technologies Inc.",
    "24.11040226": "Aventura Technologies, Inc.",
    "24.15732213": "ACOMA Medical Industry Co,. Ltd.",
    "12.123917683777536": "XLOGIC srl",
    "20.212014909620224": "Ernitec",
    "24.11840931": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "24.12636228": "Sagemcom Broadband SAS",
    "12.123917681594368": "swissled technologies AG",
    "20.171977812738048": "PCI Limited",
    "12.123917688922112": "Contineo s.r.o.",
    "12.123917676089344": "DiTEST Fahrzeugdiagnose GmbH",
    "24.7919616": "IEEE Registration Authority",
    "24.3202957": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.11336357": "digitron",
    "12.123917685891072": "Environexus",
    "12.123917679980544": "Exsom Computers LLC",
    "24.6842741": "Angler Labs Inc",
    "24.14479453": "Letta Teknoloji",
    "24.8704346": "TCT mobile ltd",
    "20.83889183784960": "Multitek Elektronik Sanayi ve Ticaret A.S.",
    "24.809630": "Wi-SUN Alliance",
    "24.12823": "Cisco Systems, Inc",
    "24.14200464": "Itibia Technologies",
    "24.6052168": "Apple, Inc.",
    "12.346864918528": "RealTime Systems Ltd",
    "24.8873": "LG Electronics (Mobile Communications)",
    "24.2361623": "Devlin Electronics Limited",
    "12.123917680054272": "Contraves Advanced Devices Sdn. Bhd.",
    "20.264036231938048": "Shenyang Ali Technology Company Limited",
    "12.346855477248": "Imax",
    "24.3719318": "Alpsalpine Co,.Ltd",
    "24.8877": "Telesis Technologies, Inc.",
    "12.346858692608": "3DSP Corporation",
    "20.58144492355584": "Hi-Tech.Org",
    "12.346857144320": "Bio-logic Systems Corp",
    "12.123917677633536": "Sambo Hitech",
    "24.12379922": "Cisco Systems, Inc",
    "12.346857295872": "Sedo Systems Ltd",
    "20.242213046976512": "LongSung Technology (Shanghai) Co.,Ltd.",
    "20.110162573524992": "Livongo Health",
    "24.4730090": "Motorola Inc Business Light Radios",
    "20.176032222019584": "El Sewedy Electrometer Egypt S.A.E.",
    "24.13925090": "Samsung Electronics Co.,Ltd",
    "12.71297883922432": "Telvent",
    "24.1517": "Technikum Joanneum GmbH",
    "24.7916036": "Nokia Corporation",
    "24.7819": "Infra Access Korea Co., Ltd.",
    "24.7818": "eCopy, Inc",
    "24.12855341": "Embedded Intellect Pty Ltd",
    "24.7815": "Realease Limited",
    "24.7814": "MEL Co.,Ltd.",
    "24.7817": "CRFS Limited",
    "24.7816": "Andor System Support Co., Ltd.",
    "24.7811": "LAN/MAN Standards Association (LMSC)",
    "24.7810": "SanDisk Corporation",
    "24.7813": "Lagotek Corporation",
    "24.7812": "Pika Technologies Inc.",
    "12.346866974720": "Divelbiss Corporation",
    "12.346854440960": "Intrinsity",
    "12.346868748288": "Enyx SA",
    "12.123917675433984": "ONDEMAND LABORATORY Co., Ltd.",
    "12.346853900288": "Anova Broadband",
    "24.1621082": "Zhejiang Tmall Technology Co., Ltd.",
    "12.346855211008": "S.M.V. Systemelektronik GmbH",
    "24.3473093": "Shenzhen Sunwoda intelligent hardware Co.,Ltd",
    "24.5523051": "Shenzhen CZTIC Electronic Technology Co., Ltd",
    "24.16568": "Idea Associates",
    "24.16569": "Macq Electronique Sa",
    "20.167137987526656": "Fritz Kuebler GmbH",
    "24.16560": "Bytex Corporation, Engineering",
    "24.16561": "Codonics Inc.",
    "24.16562": "Systemforschung",
    "24.16563": "ParTech Inc.",
    "24.16564": "Nextcom K.K.",
    "24.16565": "Video Technology Computers Ltd",
    "24.16566": "Computerm Corporation",
    "24.16567": "Stealth Computer Systems",
    "20.93020694249472": "Niveo International BV",
    "12.123917678231552": "Grau Elektronik GmbH",
    "24.16034199": "Canon Inc.",
    "24.15519251": "Cisco Systems, Inc",
    "12.123917682356224": "StromIdee GmbH",
    "20.154238420254720": "Shenzhen Juding Electronics Co., Ltd.",
    "12.123917687562240": "Nesa Srl",
    "12.346868854784": "Axible Technologies",
    "24.3696368": "Apple, Inc.",
    "24.11311650": "NXP Semiconductors",
    "24.2655246": "silicon valley immigration service",
    "24.12351088": "Huawei Technologies Co.,Ltd",
    "24.14994265": "Beijing Loveair Science and Technology Co. Ltd.",
    "24.14985120": "Intel Corporate",
    "24.14948801": "Huawei Technologies Co.,Ltd",
    "24.1354736": "ARRIS Group, Inc.",
    "24.7910866": "Ever Treasure Industrial Limited",
    "24.13138779": "zte corporation",
    "12.346858987520": "Commet AB",
    "24.2102845": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.1100827": "Liverock technologies,INC",
    "12.346865319936": "Lanmark Controls Inc.",
    "24.7369139": "Brain Corporation",
    "12.123917685592064": "Qualitronix Madrass Pvt Ltd",
    "24.5505": "MAKUS Inc.",
    "24.5504": "U-Way Corporation",
    "24.5507": "IVT corporation",
    "24.5506": "Pulse Eight Limited",
    "24.5501": "Posdata",
    "24.12298": "Aztech Electronics Pte Ltd",
    "24.5503": "ChuanG International Holding CO.,LTD.",
    "24.5502": "Weidmüller Interface GmbH & Co. KG",
    "24.6331073": "Apple, Inc.",
    "24.5509": "Aonvision Technolopy Corp.",
    "24.5508": "Schenck Process GmbH",
    "24.13420075": "Tri-Systems Corporation",
    "24.11548085": "Apple, Inc.",
    "20.368740139008": "Speechlab",
    "24.8951225": "Unified Packet Systems Crop",
    "12.346867945472": "Bluetest AB",
    "12.346863345664": "Flexible Picture Systems",
    "24.11807289": "ORBIT MERRET, spol. s r. o.",
    "24.6035143": "Big Switch Networks",
    "20.220515722592256": "Fibergate.Inc",
    "12.123917683224576": "Gedomo GmbH",
    "24.9755610": "Huawei Technologies Co.,Ltd",
    "24.815004": "Shaw Communications Inc.",
    "24.13897872": "iNFORM Systems AG",
    "24.2148175": "SensorPush",
    "24.13664160": "Samsung Electronics Co.,Ltd",
    "24.10013773": "Shenzhen mantunsci co., LTD",
    "12.346853740544": "Task 84 Spa",
    "24.14704777": "Cloudleaf, Inc.",
    "24.2396686": "Samsung Electronics Co.,Ltd",
    "24.10253838": "Apple, Inc.",
    "24.1864968": "Compal Information (Kunshan) Co., Ltd.",
    "24.11301513": "Intel Corporate",
    "12.123917685350400": "System 11 Sp. z o.o.",
    "20.268735391727616": "Terminus (Shanghai) Technology Co.,Ltd.",
    "20.255553896972288": "Koal Software Co., Ltd",
    "24.4674": "Millennial Net",
    "24.4675": "Cisco Systems, Inc",
    "24.4676": "Cisco Systems, Inc",
    "24.4677": "Zellweger Analytics, Inc.",
    "24.4670": "ERUNE technology Co., Ltd.",
    "24.4671": "Dell Inc.",
    "24.4672": "Amoi Electronics Co.,Ltd",
    "24.4673": "a2i marketing center",
    "24.14429130": "Zera GmbH",
    "24.4678": "T.O.M Technology Inc..",
    "24.4679": "Samsung Electronics Co.,Ltd",
    "24.9758698": "Huawei Technologies Co.,Ltd",
    "12.123917690773504": "Neuronal Innovation Control S.L.",
    "24.12861686": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.2652564": "Realplay Digital Technology(Shenzhen) Co.,Ltd",
    "24.15224452": "Samsung Electronics Co.,Ltd",
    "24.8675458": "Private",
    "24.1518": "Vanderbilt International (SWE) AB",
    "24.3909": "Stretch, Inc.",
    "24.3908": "Tivella Inc.",
    "24.3907": "Wasabi Systems Inc.",
    "24.3906": "Xalyo Systems",
    "24.3905": "Zipher Ltd",
    "24.3904": "Optical Internetworking Forum",
    "24.3903": "Big Bear Networks",
    "24.3902": "CardioNet, Inc",
    "24.3901": "D-Link Corporation",
    "24.3900": "Endeleo Limited",
    "24.2132791": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346854264832": "OlympusNDT",
    "24.11813645": "Broadlink Pty Ltd",
    "12.123917678903296": "nVideon, Inc.",
    "24.6566224": "Hewlett Packard",
    "24.1519": "ADOIR Digital Technology",
    "24.10247062": "NMR Corporation",
    "24.16521512": "Actions Microelectronics Co., Ltd",
    "24.12081112": "Samsung Electronics Co.,Ltd",
    "24.9980932": "Zhangzhou Keneng Electrical Equipment Co Ltd",
    "20.145727915294720": "Newell Brands",
    "12.123917676363776": "Crde",
    "24.8420158": "Samsung Electronics Co.,Ltd",
    "24.6297749": "Dell Inc.",
    "24.12089465": "Roche Diagnostics GmbH",
    "24.7627002": "Huawei Technologies Co.,Ltd",
    "20.83889184833536": "Jiangsu acrel Co., Ltd.",
    "12.123917688090624": "EM Clarity Pty Ltd",
    "24.6442": "Antiope Associates",
    "24.3176555": "TIANJIN Golden Pentagon Electronics Co., Ltd.",
    "24.20718": "Tek Digitel Corporation",
    "24.20719": "SPE Systemhaus GmbH",
    "24.20712": "Nomadix Inc.",
    "24.2118602": "Risk Technology Ltd",
    "24.20711": "Paradise Innovations (Asia)",
    "24.20716": "Olicom A/S",
    "24.20717": "Anda Networks",
    "12.346858184704": "Card Access Services Pty Ltd",
    "24.20715": "Alpha-Top Corporation",
    "24.7370": "Shanghai Gaozhi Science & Technology Development Co.",
    "24.7371": "Forth Corporation Public Company Limited",
    "24.7372": "BlackBerry RTS",
    "24.7373": "Alektrona Corporation",
    "24.7374": "By Techdesign",
    "24.7375": "Limetek",
    "24.7376": "Circleone Co.,Ltd.",
    "24.7377": "Waves Audio LTD",
    "24.7378": "King Champion (Hong Kong) Limited",
    "24.7379": "ZP Engineering SEL",
    "24.4760294": "Resideo",
    "20.31477067677696": "Private",
    "24.14694167": "Technicolor CH USA Inc.",
    "24.9233114": "Circle Media Inc",
    "12.123917684330496": "Shanghai Armour Technology Co., Ltd.",
    "12.123917691244544": "2M Technology",
    "12.346856755200": "Tattile Srl",
    "12.123917682634752": "Eva Automation",
    "12.123917681397760": "Zao Zeo",
    "24.1627847": "Tp-Link Technologies Co.,Ltd.",
    "20.225209254674432": "Lubelskie Fabryki Wag FAWAG S.A.",
    "24.6083083": "Yytek Co., Ltd.",
    "24.53439": "Pivotal Technologies",
    "24.53438": "Emutec Inc.",
    "24.8818": "American Micro-Fuel Device Corp.",
    "24.8819": "Techway",
    "24.8814": "Gowell Electronic Limited",
    "24.8815": "3onedata Technology Co. Ltd.",
    "24.53433": "Microtek International, Inc.",
    "24.53432": "Iomega Corporation",
    "24.8810": "Honeywell",
    "24.53434": "Cisco Systems, Inc",
    "24.8812": "LinkSprite Technologies, Inc.",
    "24.53436": "Cisco Systems, Inc",
    "24.4996118": "Samsung Electronics Co.,Ltd",
    "20.5239429136384": "Invasys",
    "24.8444721": "Power Quotient International Co., Ltd.",
    "24.14950377": "Dunkermotoren GmbH",
    "24.7915000": "Huawei Device Co., Ltd.",
    "24.12598799": "BSkyB Ltd",
    "24.7594": "DrayTek Corp.",
    "24.7595": "SwissQual License AG",
    "24.7596": "Gigamon Systems LLC",
    "24.7597": "Sinotech Engineering Consultants, Inc. Geotechnical Enginee",
    "24.7590": "Media Numerics Limited",
    "24.7591": "Seamless Internet",
    "24.7592": "Takahata Electronics Co.,Ltd",
    "24.7593": "Castles Technology, Co., LTD",
    "24.7598": "Chang Tseng Technology Co., Ltd",
    "24.7599": "Nortel Networks",
    "24.1331488": "Huawei Device Co., Ltd.",
    "24.2686686": "COMESTA, Inc.",
    "12.123917691121664": "RFRain LLC",
    "12.346864001024": "Saia-Burgess Controls AG",
    "12.346859241472": "Gloster Sante Europe",
    "12.119269289984": "Simavita Pty Ltd",
    "24.6429": "Nintendo Co., Ltd.",
    "24.6428": "Sensicast Systems",
    "24.6427": "Sputnik Engineering AG",
    "24.6426": "Irlink",
    "24.6425": "ASTEL Inc.",
    "24.6424": "Interactive Wear AG",
    "24.6423": "Posiflex Inc.",
    "24.3678294": "Cisco Systems, Inc",
    "24.6421": "TECOM Co., Ltd.",
    "24.6420": "Winix Co., Ltd",
    "12.346862944256": "Thales UK Limited",
    "12.346859474944": "Revox GmbH",
    "20.71389202612224": "GuangZhou FiiO Electronics Technology Co.,Ltd",
    "20.67047985774592": "Nox Medical",
    "24.15497545": "Fujitsu Limited",
    "24.7132959": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917681913856": "Alazar Technologies Inc.",
    "12.123917682941952": "Immolas",
    "12.123917686374400": "Euklis by GSG International",
    "20.251018414653440": "Channel Enterprises (HK) Ltd.",
    "24.12643904": "Laird Technologies",
    "24.7114177": "Apple, Inc.",
    "24.3967949": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.7088843": "Paxton Access Ltd",
    "20.136823648026624": "Private",
    "24.13921616": "Cisco Systems, Inc",
    "24.24716": "3Com (1990 onwards)",
    "24.24717": "Unipulse Corp.",
    "24.24714": "Citadel Computer",
    "24.24715": "ConferTech International",
    "24.24712": "Analog Devices, Inc.",
    "24.24713": "Xata",
    "24.24710": "Logic Replacement Tech. Ltd.",
    "24.24711": "Kansai Electric Co., Ltd.",
    "24.24718": "HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH",
    "24.24719": "Tekram Technology Co., Ltd.",
    "12.346856271872": "ISS International",
    "24.11285707": "Shanhai GBCOM Communication Technology Co. Ltd",
    "24.2127705": "Optical Network Video Technologies (Shenzhen) Co., Ltd.",
    "24.6335215": "Huawei Device Co., Ltd.",
    "20.224641223229440": "Kymati GmbH",
    "20.168198560284672": "ARIMA Communications Corp.",
    "24.2944": "Lycium Networks",
    "24.2945": "Kaparel Corporation",
    "24.2946": "Grandstream Networks, Inc.",
    "24.2947": "Datawatt B.V.",
    "24.2940": "Telex Communications",
    "24.2941": "Solomon Extreme International Ltd.",
    "24.2942": "SAGINOMIYA Seisakusho Inc.",
    "24.2943": "Align Engineering LLC",
    "12.346863816704": "Telegenix",
    "24.7665180": "HTC Corporation",
    "24.2948": "Bodet",
    "24.3164496": "IEEE Registration Authority",
    "20.30931577470976": "Global Design Solutions Ltd",
    "24.37019": "Markem-Imaje",
    "24.37018": "One World Systems, Inc.",
    "24.329": "Tdt Ag",
    "24.328": "X-traWeb Inc.",
    "24.327": "Zhone Technologies",
    "24.326": "Tesco Controls, Inc.",
    "24.325": "Winsystems, Inc.",
    "24.324": "Dell EMC",
    "12.346865180672": "Electronica Keld",
    "24.37010": "Cisco Systems, Inc",
    "24.37013": "Universal Avionics",
    "24.320": "Sendtek Corporation",
    "12.123917680799744": "INCAA Computers",
    "24.1331159": "IEEE Registration Authority",
    "24.11046978": "Beijing Wanwei Intelligent Technology Co., Ltd.",
    "24.5998": "Ducati Sistemi",
    "24.5999": "PAX Computer Technology(Shenzhen) Ltd.",
    "24.5996": "Pivot3, Inc.",
    "24.5997": "Core Corporation",
    "24.5994": "Avago Technologies",
    "24.5995": "Kiyon, Inc.",
    "24.5992": "Zinwave Ltd",
    "12.346868080640": "H&L Instruments, LLC",
    "24.5990": "Accense Technology, Inc.",
    "24.5991": "Earforce AS",
    "24.16272602": "Huawei Technologies Co.,Ltd",
    "24.10274990": "Becton, Dickinson and Company",
    "24.16276380": "SB SYSTEMS Co.,Ltd",
    "12.123917683195904": "Cubitech",
    "24.13673894": "Cisco Systems, Inc",
    "24.16564787": "ASUSTek COMPUTER INC.",
    "24.9398": "Seagate Technology",
    "24.9399": "GridPoint, Inc.",
    "12.346868932608": "Sensata Technologies",
    "24.9394": "Netgear",
    "24.9395": "Graf-Syteco GmbH & Co. KG",
    "24.9396": "ESCATRONIC GmbH",
    "24.197": "ARRIS Group, Inc.",
    "24.9390": "Idemia",
    "24.9391": "Dish Technologies Corp",
    "24.9392": "Esab Ab",
    "24.9393": "Coulomb Technologies",
    "24.572739": "Netgear",
    "12.123917689008128": "WyreStorm Technologies Ltd",
    "24.8191842": "Huizhou Super Electron Technology Co.,Ltd.",
    "24.10748208": "ABIsystems Co., LTD",
    "20.39919492792320": "Shenzhen Ruixunyun Technology Co.,Ltd.",
    "12.346854522880": "Geoquip Ltd",
    "24.13919174": "Motorola Mobility LLC, a Lenovo Company",
    "24.11036082": "Dongguan Yishang Electronic Technology Co., Limited",
    "24.15529147": "Dell Inc.",
    "24.12366323": "Samsung Electronics Co.,Ltd",
    "20.216039228964864": "GETEMED Medizin- und Informationstechnik AG",
    "12.123917682196480": "DEUTA-WERKE GmbH",
    "20.277783357947904": "Token",
    "12.123917675405312": "RM Michaelides Software & Elektronik GmbH",
    "24.5552130": "Samsung Electronics Co.,Ltd",
    "24.524318": "Apollo Computer Inc.",
    "24.524319": "Sharp Corporation",
    "24.3988668": "Samsung Electronics Co.,Ltd",
    "12.346867425280": "GD Mission Systems",
    "24.524314": "Tiara/ 10Net",
    "24.524315": "Dell EMC",
    "24.524316": "Kdd-Kokusai Debnsin Denwa Co.",
    "24.524317": "Able Communications Inc.",
    "24.524310": "Barrister Info Sys Corp",
    "24.524311": "National Semiconductor Corp. (used to have Network System Corp., wrong NSC)",
    "24.524312": "Pirelli Focom Networks",
    "24.524313": "General Electric Corporation",
    "24.8661757": "HP Inc.",
    "12.346854453248": "HTEC Limited",
    "24.16534300": "Intersensor S.R.L.",
    "12.346853306368": "Wintegra Ltd.",
    "24.3675176": "Dell Inc.",
    "24.3685158": "Jiangsu Qinheng Co., Ltd.",
    "12.346864594944": "GasTOPS Ltd.",
    "24.8049": "xG Technology, Inc.",
    "24.8048": "Botik Technologies LTD",
    "24.9502111": "Silicon Laboratories",
    "24.8043": "LG Electronics (Mobile Communications)",
    "24.8042": "PacketFlux Technologies, Inc.",
    "24.8041": "Pingood Technology Co., Ltd.",
    "24.8040": "Martinsson Elektronik AB",
    "24.8047": "Fujian Sunnada Communication Co.,Ltd.",
    "24.8046": "Vtech Engineering Corporation",
    "24.8045": "Cisco Systems, Inc",
    "24.8044": "Cisco Systems, Inc",
    "24.3732475": "Asperiq",
    "24.4767891": "Xirrus, Inc.",
    "12.123917690535936": "shanghai qiaoqi zhinengkeji",
    "12.123917689782272": "Plasmapp Co.,Ltd.",
    "12.123917679898624": "TruTeq Wireless (Pty) Ltd",
    "12.123917688446976": "HBH Microwave GmbH",
    "20.145383536721920": "Guangzhou Heygears Technology Ltd",
    "24.6857592": "Freebox Sas",
    "12.123917676421120": "Enabler Ltd.",
    "24.2887229": "Samsung Electro-Mechanics(Thailand)",
    "24.11073504": "PLANET Technology Corporation",
    "24.8014756": "Innostor Technology Corporation",
    "24.7396847": "Huawei Device Co., Ltd.",
    "24.15229215": "Dawning Information Industry Co.,Ltd",
    "24.6333309": "Apple, Inc.",
    "12.123917686988800": "Edgeware AB",
    "12.346860294144": "Primary Integration Encorp LLC",
    "24.6066519": "Huawei Technologies Co.,Ltd",
    "12.346863296512": "Procon Engineering Limited",
    "12.123917678112768": "Military Research Institute",
    "12.346858500096": "Shanghai QianJin Electronic Equipment Co. Ltd.",
    "24.14736544": "Siemens Industrial Automation Products Ltd Chengdu",
    "12.346861981696": "full electronic system",
    "12.123917684580352": "Adcole Space",
    "12.123917681205248": "Kys,Inc",
    "24.3424605": "Intel Corporate",
    "20.225209256771584": "Hangzhou Scooper Technology Co.,Ltd.",
    "24.889": "Proscend Communications, Inc.",
    "24.9971878": "Compal Information (Kunshan) Co., Ltd.",
    "24.9804": "Shanghai DigiVision Technology Co., Ltd.",
    "24.9805": "Arcadyan Technology Corporation",
    "24.9806": "r2p GmbH",
    "24.9807": "Krüger &Gothe GmbH",
    "24.9800": "Emitech Corp.",
    "24.9802": "Apple, Inc.",
    "20.114975478644736": "Quantex",
    "24.15261839": "Gravotech Marking Sas",
    "24.9808": "2Wire Inc",
    "24.9809": "Cisco Systems, Inc",
    "24.8394820": "Dell Inc.",
    "24.1607574": "HTC Corporation",
    "12.71297884495872": "Tetracore, Inc.",
    "24.882": "Ulan",
    "12.123917682933760": "AVI Pty Ltd",
    "12.123917687255040": "SYS TEC electronic GmbH",
    "24.10263901": "Raden Inc",
    "24.9043": "F E T Elettronica snc",
    "24.389": "Hitachi Aloka Medical, Ltd.",
    "24.12324775": "Ouster",
    "24.881": "Acomz Networks Corp.",
    "24.8926142": "Ivenix, Inc.",
    "24.388": "Sieb & Meyer Ag",
    "20.83889196367872": "Connected IO",
    "24.8933597": "Infiniband Trade Association",
    "12.346852401152": "PLLB Elettronica SPA",
    "24.11537617": "Hewlett Packard",
    "12.123917690011648": "Advanced Vision Technology Ltd",
    "12.346857541632": "Aris Technologies",
    "24.7642339": "KodaCloud Canada, Inc",
    "12.123917688025088": "Herholdt Controls srl",
    "24.385": "Nortel Networks",
    "20.5239424942080": "Guangdong New Pulse Electric Co., Ltd.",
    "24.1900377": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.384": "AOpen, Inc.",
    "24.10493256": "Hewlett Packard",
    "20.97843033407488": "Inforce Computing Inc.",
    "24.1107834": "STMicrolectronics International NV",
    "12.346854092800": "Pleora Technologies Inc.",
    "20.88668794519552": "Traxens",
    "12.123917679054848": "Kniggendorf + Kögler Security GmbH",
    "24.14470721": "Fe Global Hong Kong Limited",
    "24.2098437": "Radmax Communication Private Limited",
    "24.11011447": "Netlist, Inc.",
    "12.123917687980032": "Gogo BA",
    "12.123917690335232": "Gigaband IP LLC",
    "24.11840300": "MtM Technology Corporation",
    "24.1699": "Bitran Corporation",
    "20.52821132050432": "Sixth Energy Technologies Private Limited",
    "24.1698": "Microtune, Inc.",
    "24.10232": "Brocade Communications Systems LLC",
    "12.123917681950720": "Active Research Limited",
    "12.123917689520128": "EMSCAN Corp.",
    "24.1048951": "Huawei Technologies Co.,Ltd",
    "12.123917676785664": "Graphcore Ltd",
    "20.264036232986624": "Domotz Ltd",
    "24.1695": "Kuokoa Networks",
    "24.1604002": "Gmj Electric Limited",
    "12.123917680369664": "Fundación Cardiovascular de Colombia",
    "24.1694": "UNIQA, Inc.",
    "24.15491684": "Global411 Internet Services, LLC",
    "24.310265": "Pavilion Data Systems Inc",
    "24.1697": "Celsian Technologies, Inc.",
    "12.346853945344": "Dsp Design",
    "12.123917688037376": "Orion Technologies, LLC",
    "24.1696": "Mx Imaging",
    "24.10812622": "Security Expert Ltd.",
    "12.123917677047808": "Smart Energy Code Company Limited",
    "24.7925915": "Huawei Device Co., Ltd.",
    "24.1691": "AVT Audio Video Technologies GmbH",
    "24.8395060": "Intel Corporate",
    "20.145211814576128": "igus GmbH",
    "24.1690": "e & Tel",
    "20.211776056590336": "Inveo",
    "24.9711068": "Samsung Electronics Co.,Ltd",
    "24.16307113": "AXAN Networks",
    "24.1693": "Petards Ltd",
    "20.84531187023872": "Daikoku Denki Co.,Ltd.",
    "12.119269253120": "TRIAX-HIRSCHMANN Multi-Media GmbH",
    "20.189244302688256": "Green IT Korea Co., Ltd.",
    "24.13915582": "Shenzhen Fast Technologies Co.,Ltd",
    "12.346853949440": "DSP Group Inc.",
    "24.9755059": "Tp-Link Technologies Co.,Ltd.",
    "12.346867499008": "Robert Bosch Healthcare Systems, Inc.",
    "12.123917681782784": "Fraunhofer-Institut IIS",
    "12.123917681627136": "Roxford",
    "20.189549214957568": "Bluewave Global Manufacturing Limited",
    "24.14457756": "Apple, Inc.",
    "12.123917685579776": "devAIs s.r.l.",
    "24.7368577": "Private",
    "24.5773068": "Sony Mobile Communications Inc",
    "24.4735965": "Amazon Technologies Inc.",
    "24.7308": "Dial Technology Ltd.",
    "24.16552212": "Technicolor CH USA Inc.",
    "24.13289598": "Allen Organ Company",
    "24.8157362": "Wyze Labs Inc",
    "12.123917685366784": "Ofil USA",
    "24.5568630": "Abb Ag",
    "12.346852982784": "The Trane Company",
    "24.5801620": "EFM Networks",
    "20.128612180164608": "Provision-ISR",
    "20.225207121870848": "inoage GmbH",
    "20.119709244588032": "Shenzhen HDCVT Technology",
    "20.167152466264064": "Boeing Ssg",
    "24.16256879": "Cisco Systems, Inc",
    "20.18540136497152": "Annapurna labs",
    "12.123917680164864": "Hangzhou Youshi Industry Co., Ltd.",
    "20.264036227743744": "Emu Technology",
    "24.15996856": "AirTies Wireless Networks",
    "12.123917683482624": "Smith Meter, Inc.",
    "24.6871356": "Apple, Inc.",
    "24.7134652": "Samsung Electronics Co.,Ltd",
    "24.14446916": "Samsung Electro Mechanics Co., Ltd.",
    "24.9747": "Engel Axil S.L.",
    "12.123917679538176": "Foerster-Technik GmbH",
    "24.11015638": "Shenzhen Meione Technology CO., LTD",
    "24.10766074": "AmTRAN Video Corporation",
    "24.1860280": "Samsung Electronics Co.,Ltd",
    "20.53091446554624": "Xio Research, Inc",
    "12.123917682552832": "SA Instrumentation Limited",
    "24.9746": "Space Exploration Technologies",
    "24.4483084": "Wacom Co.,Ltd.",
    "24.5253758": "Smart electronic GmbH",
    "20.13692018098176": "Taiwan Pulse Motion Co., Ltd.",
    "24.15772836": "HBC-radiomatic",
    "24.59091": "Nixdorf Computer Corp.",
    "20.224420096376832": "i-Trinetech Co.,Ltd.",
    "12.346865659904": "CoreEL TEchnologies (I) Pvt Ltd",
    "12.346862899200": "Owandy",
    "24.6616334": "Kinion Technology Company Limited",
    "24.2140039": "Bointec Taiwan Corporation Limited",
    "24.15382683": "Thread Group, Inc.",
    "12.346854297600": "LK Ltd",
    "24.1353835": "ShenZhen Telacom Science&Technology Co., Ltd",
    "24.7111478": "Samsung Electronics Co.,Ltd",
    "24.1873042": "RUAG Schweiz AG",
    "24.11052949": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1837249": "LG Innotek",
    "12.123917684260864": "Guangdong Transtek Medical Electronics Co., Ltd.",
    "12.119269990400": "Nt Microsystems",
    "24.1844459": "Burlywood",
    "24.3176395": "Maike Industry(Shenzhen)CO.,LTD",
    "12.123917686067200": "Code Blue Corporation",
    "24.3449749": "Apple, Inc.",
    "24.9964846": "HTC Corporation",
    "12.123917688983552": "WM Design s.r.o",
    "24.585145": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917679075328": "AVI Pty Ltd",
    "24.12068564": "Furukawa Industrial S.A. Produtos Elétricos",
    "12.123917687660544": "Beijing Wisetone Information Technology Co.,Ltd.",
    "20.154039768580096": "Abside Networks, Inc.",
    "12.346860261376": "Technovare Systems, Inc.",
    "24.578379": "Techman Electronics (Changshu) Co., Ltd.",
    "24.2144192": "OMICRON electronics GmbH",
    "12.346867863552": "Horiba Abx Sas",
    "24.5010981": "Dell Inc.",
    "20.48835650912256": "Wellntel, Inc.",
    "24.14700683": "Zhejiang Dahua Technology Co., Ltd.",
    "24.14975897": "ARRIS Group, Inc.",
    "24.2389239": "GoPro",
    "24.46592": "VOIM Co., Ltd.",
    "24.6553645": "Powerlinq Co., LTD",
    "12.346860228608": "Ateme",
    "24.14429030": "InfoBLOCK S.A. de C.V.",
    "24.10538843": "Cisco Systems, Inc",
    "12.346856583168": "PowerWAN, Inc",
    "24.1321942": "Guangdong Appscomm Co.,Ltd",
    "24.7385422": "ARRIS Group, Inc.",
    "20.145735989329920": "Emotiv Inc",
    "12.123917684584448": "eumig industrie-tv GmbH",
    "24.16047322": "Zhuhai Jiahe Communication Technology Co., limited",
    "12.346856914944": "Da Sistemi Spa",
    "24.11580047": "Sagemcom Broadband SAS",
    "24.188475": "Prominet Corporation",
    "24.6613900": "Seiko Epson Corporation",
    "24.9733498": "Evantage Industries Corp",
    "12.123917675134976": "Micro Debug, Y.K.",
    "24.1863714": "Murata Manufacturing Co., Ltd.",
    "24.1864560": "Neotech",
    "12.123917676122112": "Leonardo Sistemi Integrati S.r.l.",
    "12.346859921408": "Gutermann Technology GmbH",
    "20.176772033282048": "AiCare Corp.",
    "24.10548313": "Fn-Link Technology Limited",
    "24.14418096": "Freebox Sas",
    "24.11837158": "Shenzhen Technology Co Ltd",
    "24.11584743": "Samsung Electronics Co.,Ltd",
    "24.14433526": "Atomic Rules LLC",
    "24.7868113": "TRUMPF Werkzeugmaschinen GmbH+Co.KG",
    "20.110373785042944": "Redstone Systems, Inc.",
    "24.8423154": "Intel Corporate",
    "12.346866466816": "DEUTA-WERKE GmbH",
    "24.41061": "Symantec Corporation",
    "12.346864996352": "Pla Electro Appliances Pvt. Ltd.",
    "24.796452": "Cisco Systems, Inc",
    "24.41063": "Network Services Group",
    "24.10019173": "Accutome",
    "24.15754233": "Huawei Technologies Co.,Ltd",
    "24.2361749": "Huawei Technologies Co.,Ltd",
    "24.15758564": "Thundercomm Technology Co., Ltd",
    "24.807427": "Agm Group Limited",
    "12.119269453824": "ITW Reyflex North America",
    "24.14715518": "Exponent",
    "24.8691044": "SMC Corporation",
    "24.16535204": "Apple, Inc.",
    "12.346852196352": "Portable Add-Ons",
    "24.2893726": "IEEE Registration Authority",
    "24.3692556": "Panaccess Systems GmbH",
    "24.5260217": "OktoInform RUS",
    "24.2104968": "Intel Corporate",
    "24.6617116": "Huawei Technologies Co., Ltd.",
    "12.119269773312": "Oberon microsystems, Inc.",
    "12.71297884393472": "Dsp Design",
    "12.346863972352": "J-Systems Inc.",
    "24.13689152": "Apple, Inc.",
    "24.5828456": "Belkin International Inc.",
    "24.6560944": "Samsung Electronics Co.,Ltd",
    "24.13946873": "Shenzhen SEI Robotics Co.,Ltd",
    "24.9497680": "C.O.B.O. Spa",
    "24.1573347": "Bittium Wireless Ltd",
    "24.8164530": "Philips Healthcare PCCI",
    "24.9234248": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.8172082": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.123917675028480": "Piranha EMS Inc.",
    "12.123917677518848": "Spectra Displays Ltd",
    "24.10515452": "Rancore Technologies Private Limited",
    "24.6560942": "Samsung Electronics Co.,Ltd",
    "12.123917691179008": "Qowisio",
    "24.15493249": "Palo Alto Networks",
    "12.123917681762304": "KLEIBER Infrared GmbH",
    "12.346860175360": "Calibre UK LTD",
    "24.9473388": "Apple, Inc.",
    "24.7397167": "Cisco Systems, Inc",
    "24.15486850": "Calix Inc.",
    "12.346859577344": "Pasan SA",
    "12.346865278976": "Kyosha Industries",
    "24.15993747": "BlackBerry RTS",
    "24.4256775": "Nintendo Co., Ltd.",
    "12.346868527104": "Newell Technologies Limited",
    "12.123917688406016": "Profen Communications",
    "20.48688639508480": "Advanced Electronics Company Ltd",
    "24.7378753": "Arcadyan Corporation",
    "24.587560": "GLOBO Multimedia Sp. z o.o. Sp.k.",
    "24.10524677": "OpenVox Communication Co Ltd",
    "24.12849632": "Bull Group Co., Ltd",
    "24.12858965": "Shenzhen Fenglian Technology Co., Ltd.",
    "24.3197985": "zte corporation",
    "12.71297884798976": "EOS S.r.l.",
    "24.5011570": "Cav. Uff. Giacomo Cimberio S.p.A.",
    "12.346866540544": "Prosoft-Systems Ltd",
    "24.14696004": "Broadcom",
    "12.346865856512": "Saia-Burgess Controls AG",
    "24.12068805": "Huawei Device Co., Ltd.",
    "24.7133516": "vivo Mobile Communication Co., Ltd.",
    "20.127654890045440": "Baudisch Electronic GmbH",
    "24.6320988": "Cisco Systems, Inc",
    "24.208": "Develcon Electronics Ltd.",
    "24.209": "Adaptec, Inc.",
    "24.204": "Densan Co., Ltd.",
    "24.205": "Allied Telesis Labs Ltd",
    "24.206": "Megadata Corp.",
    "24.207": "Hayes Microcomputer Products",
    "24.200": "Altos Computer Systems",
    "24.201": "Emulex",
    "24.202": "ARRIS Group, Inc.",
    "24.203": "Compu-Shack Electronic Gmbh",
    "24.15486279": "Huawei Technologies Co.,Ltd",
    "24.570575": "Abicom International",
    "20.27056781918208": "Taiwan Name Plate Co.,LTD",
    "24.7663874": "Beijing HCHCOM Technology Co., Ltd",
    "24.15486270": "Beijing Xiaomi Mobile Software Co., Ltd",
    "20.268735385436160": "ITS Co., Ltd.",
    "12.346862731264": "Elbit Systems of America - Tallahassee Operations",
    "12.346860072960": "OMICRON electronics GmbH",
    "12.123917686542336": "G.S.D Group Inc.",
    "24.6351706": "Asterfusion Data Technologies Co.,Ltd",
    "20.31372478513152": "Sensys Networks, Inc.",
    "24.57900": "China Mobile Group Device Co.,Ltd.",
    "24.13155557": "Sense Things Japan INC.",
    "24.1894923": "Edgecore Networks Corporation",
    "24.1929": "Allradio Co., Ltd",
    "24.1928": "Clipcomm, Inc.",
    "24.5512029": "Techno-Innov",
    "24.6354547": "Terumo Corporation",
    "12.123917684527104": "DHK Storage, LLC",
    "24.1923": "SynCom Network, Inc.",
    "24.1922": "Oracle Corporation",
    "24.1921": "Itron Inc.",
    "24.1920": "Bluegiga Technologies OY",
    "24.1927": "Idea System Co., Ltd.",
    "24.1926": "Wireless Networks Inc.",
    "24.1925": "Cisco Systems, Inc",
    "24.1924": "Cisco Systems, Inc",
    "12.123917691670528": "Malin Space Science System",
    "24.6086853": "Intel Corporate",
    "24.5395532": "Novell 2000",
    "24.12347268": "Environics Oy",
    "24.6086858": "FeiTian United (Beijing) System Technology Co., Ltd.",
    "24.13654297": "zte corporation",
    "20.176032223068160": "Beijing Deephi Intelligent Technology Co., Ltd",
    "12.346868604928": "ATI Automação Telecomunicações e Informática Ltda",
    "24.564744": "Google, Inc.",
    "20.154055549648896": "Septentrio NV",
    "20.111031375364096": "Sichuan Haige Actec Communication Technology Co.,Ltd.",
    "24.5714": "Hoatech Technologies, Inc.",
    "24.5715": "LEGO System A/S IE Electronics Division",
    "24.5716": "Flex-P Industries Sdn. Bhd.",
    "24.5717": "FUHO TECHNOLOGY Co., LTD",
    "24.5710": "Nokia Danmark A/S",
    "24.5711": "World Ethnic Broadcastin Inc.",
    "24.5712": "Kratos EPD",
    "24.5713": "Exeo Systems",
    "24.2662987": "Juniper Networks",
    "24.5719": "Aegate Ltd",
    "12.123917687808000": "Knowledge Resources GmbH",
    "20.181256794931200": "Quuppa Oy",
    "24.5512790": "Juniper Networks",
    "24.5002279": "Linepro Sp. z o.o.",
    "24.8918947": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.106196527742976": "AdvanWISE Corporation",
    "12.346857537536": "McCain Traffic Supply",
    "24.16530835": "Longcheer Telecommunication Limited",
    "20.123598317551616": "Freund Elektronika D.O.O., Ip-Integra Technologies",
    "24.978": "Crossbeam Systems, Inc.",
    "24.979": "Internet Energy Systems, Inc.",
    "24.3718": "Alcatel North America",
    "24.3719": "adp Gauselmann GmbH",
    "24.3716": "Cisco Systems, Inc",
    "24.973": "Clovertech, Inc.",
    "24.970": "MTS Systems Corp.",
    "24.971": "SystemGear Co., Ltd.",
    "24.976": "KOANKEISO Co., Ltd.",
    "24.977": "Takaya Corporation",
    "24.974": "ETEN Technologies, Inc.",
    "24.975": "Muxcom, Inc.",
    "24.14698138": "Netgear",
    "24.3687820": "MyCloud Technology corporation",
    "24.7013": "China Gridcom Co., Ltd",
    "24.16019437": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6081916": "MEIZU Technology Co.,Ltd.",
    "24.1371557": "Wächter GmbH Sicherheitssysteme",
    "24.16561880": "Apple, Inc.",
    "12.123917675892736": "Brechbuehler AG",
    "12.123917681901568": "MoviTHERM",
    "20.57353103736832": "Bellman & Symfon",
    "24.1851736": "Gemalto M2M GmbH",
    "24.28848": "M/A-Com Inc. Companies",
    "24.9751980": "Creowave Oy",
    "12.346865364992": "Mstronic Co., Ltd.",
    "12.123917677899776": "Proterra",
    "24.1582694": "Samsung Electronics Co.,Ltd",
    "12.346852835328": "Bounty Systems Pty Ltd.",
    "24.1592328": "Rockwell Automation",
    "12.123917688721408": "3DGence sp. z o.o.",
    "12.123917686235136": "SmartD Technologies Inc",
    "24.21679": "Continental Automotive Systems Inc.",
    "24.3408035": "Huawei Technologies Co.,Ltd",
    "24.6315917": "eero inc.",
    "12.123917689556992": "Duplomatic MS spa",
    "20.132777841262592": "Xronos-Inc",
    "24.9979062": "Infranor Sas",
    "12.123917689860096": "Guan Show Technologe Co., Ltd.",
    "12.123917682622464": "PHYZHON Health Inc",
    "24.52624": "MAS Elektronik AG",
    "24.13155283": "Embrane",
    "24.10012580": "Shenzhen HS Fiber Communication Equipment CO., LTD",
    "20.4471488774144": "Shenzhen YIZHENG Technology Co.,Ltd",
    "24.15736372": "ORICO Technologies Co., Ltd",
    "24.6321293": "Atmaca Elektronik",
    "24.537883": "Shenzhen Jialihua Electronic Technology Co., Ltd",
    "24.8942468": "Toshiba",
    "24.318294": "Apple, Inc.",
    "24.4235064": "AzureWave Technology Inc.",
    "12.123917676023808": "OOO Research and Production Center \"Computer Technologies\"",
    "12.123917676904448": "Shenzhen INVT Electric Co.,Ltd",
    "24.8424075": "vivo Mobile Communication Co., Ltd.",
    "24.4519001": "Samsung Electronics Co.,Ltd",
    "12.346858258432": "Automata Spa",
    "24.5531102": "ARRIS Group, Inc.",
    "12.123917684764672": "Mart Network Solutions Ltd",
    "12.346866323456": "Tattile srl",
    "20.48835655106560": "EATON FHF Funke + Huster Fernsig GmbH",
    "12.123917688889344": "enders GmbH",
    "24.5831899": "IEEE Registration Authority",
    "12.123917683191808": "ORCA Technologies, LLC",
    "12.123917679116288": "Outsight SA",
    "20.84187708129280": "S.I.C.E.S. srl",
    "24.394": "Roi Computer Ag",
    "12.123917688389632": "Benchmark Electronics BV",
    "24.2123471": "Freebox Sas",
    "24.8991": "Guangda Electronic & Telecommunication Technology Development Co., Ltd.",
    "24.8990": "Cezzer Multimedia Technologies",
    "24.8993": "Avitech International Corp",
    "24.8992": "Nicira Networks",
    "24.8995": "Zylin AS",
    "24.8994": "KISS Teknical Solutions, Inc.",
    "24.8997": "IOLAN Holding",
    "24.8996": "G-Pro Computer",
    "24.8999": "Shouyo Electronics CO., LTD",
    "24.8998": "Fujitsu Limited",
    "24.3939859": "Hangzhou Sunrise Technology Co., Ltd",
    "24.2649495": "SAMJIN Co., Ltd.",
    "12.123917678919680": "SciTronix",
    "24.11305543": "Intel Corporate",
    "20.128612187504640": "Tbm Co., Ltd.",
    "12.123917687218176": "Gluon Solutions Inc.",
    "12.123917691322368": "Atman Tecnologia Ltda",
    "24.11547797": "Apple, Inc.",
    "24.8157672": "PayRange Inc.",
    "12.346867765248": "Procon Electronics",
    "24.13955395": "IPROAD.,Inc",
    "24.4233389": "Apple, Inc.",
    "24.4766886": "​ASUNG TECHNO CO.,Ltd",
    "12.123917678284800": "Yite technology",
    "24.11064697": "Beijing Chushang Science and Technology Co.,Ltd",
    "12.346860449792": "V2Green, Inc.",
    "24.8668": "TECNOALARM S.r.l.",
    "24.8663": "Cisco Systems, Inc",
    "24.8662": "LXI Consortium",
    "24.8661": "X2E GmbH",
    "24.8660": "Vollmer Werke GmbH",
    "24.8667": "Santachi Video Technology (Shenzhen) Co., Ltd.",
    "24.8666": "Automation Products Group Inc.",
    "24.8665": "Sekonic Corporation",
    "24.8664": "Cisco Systems, Inc",
    "12.346857242624": "Avantry Ltd.",
    "24.16303347": "Shanghai Infinity Wireless Technologies Co.,Ltd.",
    "12.123917684543488": "Thesycon Software Solutions GmbH & Co. KG",
    "12.346861510656": "Ateme",
    "12.123917680103424": "Velvac Incorporated",
    "24.4208557": "Macro Image Technology, Inc.",
    "24.1608027": "Samsung Electronics Co.,Ltd",
    "20.36069848383488": "Intelligraphics",
    "24.16273239": "Cisco Systems, Inc",
    "24.4499407": "Murata Manufacturing Co., Ltd.",
    "24.1875526": "Apple, Inc.",
    "24.13130614": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.1080602": "Dell Inc.",
    "12.346860363776": "Bittitalo Oy",
    "24.11534516": "Cisco Systems, Inc",
    "24.26466": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3963562": "China Dragon Technology Limited",
    "20.225207119773696": "BBPOS International Limited",
    "24.12069367": "Blaster Tech",
    "24.2599": "Apple, Inc.",
    "24.2598": "CEIA S.p.A.",
    "24.2597": "Ceragon Networks",
    "24.2596": "Octave Communications",
    "24.2595": "Parama Networks Inc",
    "24.2594": "Amperion Inc",
    "24.2593": "Integra Telecom Co. Ltd",
    "24.2592": "SVA Networks, Inc.",
    "24.2591": "ART WARE Telecommunication Co., Ltd.",
    "24.2590": "Red-M Products Limited",
    "24.1028": "Makino Milling Machine Co., Ltd.",
    "24.1029": "ACN Technologies",
    "24.1026": "Nexsan Technologies, Ltd.",
    "24.1027": "Nexsi Corporation",
    "24.1024": "Lexmark (Print Server)",
    "24.1025": "Osaki Electric Co., Ltd.",
    "24.1022": "Cisco Systems, Inc",
    "12.346857123840": "Protonic Holland",
    "24.1020": "Intertex Data AB",
    "24.1021": "Cisco Systems, Inc",
    "24.2116266": "Hanscan Spain S.A.",
    "20.132855986388992": "Blue Sparq, Inc.",
    "24.10258138": "Phoenix Geophysics Ltd.",
    "12.346859126784": "Teletask",
    "20.70443723653120": "Kanda Kogyo",
    "24.12356878": "Nokia",
    "24.15234278": "ADB Broadband Italia",
    "12.346861850624": "Lighting Science Group",
    "20.123598304968704": "Linksys Telecom Shenzhen CO., LTD",
    "12.123917689753600": "Engage Technologies",
    "24.12641332": "AzureWave Technology Inc.",
    "24.13149962": "Verizon Business",
    "12.123917681254400": "FreeFlight Systems",
    "12.346853502976": "UniSUR",
    "12.123917683593216": "Arnouse Digital Devices Corp",
    "24.8190368": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.9211": "IP Datatel, LLC.",
    "24.9210": "RG Nets, Inc.",
    "24.9213": "AFT Atlas Fahrzeugtechnik GmbH",
    "24.9212": "Ultra Stereo Labs, Inc",
    "24.9215": "Beijing HTTC Technology Ltd.",
    "24.9214": "Biodevices, SA",
    "24.9217": "D-Link Corporation",
    "24.9216": "Nortel Networks",
    "24.9219": "Nokia Danmark A/S",
    "24.9218": "Op-Tection GmbH",
    "20.198611835027456": "Dongguan Finslink Communication Technology Co.,Ltd.",
    "12.123917678075904": "Aplex Technology Inc.",
    "24.10785612": "Cisco Systems, Inc",
    "20.9814043262976": "Savox Communications",
    "24.12335526": "CSUN System Technology Co.,LTD",
    "24.8450545": "Tech4home, Lda",
    "20.110373788188672": "Shenzhen C & D Electronics Co., Ltd.",
    "24.15751689": "Samsung Electronics Co.,Ltd",
    "12.123917688135680": "Woodam Co., Ltd.",
    "12.346855915520": "Embedit A/S",
    "12.346854260736": "Whittier Mailing Products, Inc.",
    "24.562415": "Wistron Infocomm (Zhongshan) Corporation",
    "24.6064538": "Tp-Link Technologies Co.,Ltd.",
    "24.6868239": "U Tek Company Limited",
    "24.8449651": "Shuttle Inc.",
    "12.346856230912": "NeuLion Incorporated",
    "12.346861449216": "DCOM Network Technology (Pty) Ltd",
    "24.11797014": "Cisco Systems, Inc",
    "24.8163294": "DWnet Technologies(Suzhou) Corporation",
    "12.123917684711424": "TruTeq Devices (Pty) Ltd",
    "24.3693049": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.3693048": "MindMade Sp. z o.o.",
    "12.123917685747712": "PHPower Srl",
    "24.524299": "Unisys also Ascom-Timeplex (former Unisys subsidiary)",
    "24.524298": "Nestar Systems Incorporated",
    "12.123917675057152": "Scrona AG",
    "20.137149117628416": "Xuji Changnan Communication Equipment Co., Ltd.",
    "24.524291": "Advanced Computer Comm.",
    "24.524290": "Bridge Communications Inc.",
    "24.524293": "Symbolics",
    "24.524292": "Cromemco Incorporated",
    "24.524295": "Apple, Inc.",
    "24.524294": "Siemens Nixdorf",
    "24.524297": "Hewlett Packard",
    "24.524296": "Bolt Beranek And Newman Inc.",
    "24.528734": "Bitel Co., Ltd.",
    "12.346854391808": "Serveron Corporation",
    "24.3732887": "home2net GmbH",
    "12.123917690372096": "Dia-Stron Limited",
    "24.8160006": "Glen Dimplex Nordic as",
    "24.9183498": "Tp-Link Technologies Co.,Ltd.",
    "24.305889": "Beijing Microvision Technology Co.,Ltd",
    "24.16011796": "Mellanox Technologies, Inc.",
    "12.346866515968": "IMAGO Technologies GmbH",
    "24.2891498": "Aerodev",
    "20.238070236446720": "Shenzhen Yidong Technology Co.,Ltd",
    "12.123917686022144": "Telefrank GmbH",
    "24.9470296": "Apple, Inc.",
    "24.4216418": "GuoTengShengHua Electronics LTD.",
    "12.123917677957120": "Integer.pl S.A.",
    "24.4201817": "Biometric Associates, LP",
    "24.15773265": "Xi3 Corp",
    "24.1101019": "Aximum Produits Electroniques",
    "20.97843034456064": "Shenzhen Siecom Communication Technology Development Co.,Ltd.",
    "12.123917677940736": "Korea Airports Corporation",
    "12.123917681508352": "idaqs Co.,Ltd.",
    "12.123917691756544": "Barracuda Measurement Solutions",
    "20.4884642398208": "Armstrong Fluid Technology",
    "20.70682234847232": "Dorel Juvenile",
    "24.8935277": "XTA Electronics Ltd",
    "24.6863065": "Act KDE, Inc.",
    "12.346855145472": "Syn-Tech Systems Inc",
    "20.180394583392256": "INTER CONTROL Hermann Köhler Elektrik GmbH & Co.KG",
    "12.123917676482560": "Service Plus LLC",
    "24.16283914": "ARRIS Group, Inc.",
    "20.62636956844032": "RFbeam Microwave GmbH",
    "24.8145538": "Portsmith LLC",
    "12.346858573824": "Sutus Inc",
    "24.1872256": "Apple, Inc.",
    "24.5509423": "Sulzer Pump Solutions Finland Oy",
    "24.5296659": "CviLux Corporation",
    "24.3942243": "SLE quality engineering GmbH & Co. KG",
    "24.529061": "Amazon Technologies Inc.",
    "24.3455642": "Asiatelco Technologies Co.",
    "24.6821264": "Sagemcom Broadband SAS",
    "24.3971186": "Sercomm Corporation.",
    "12.71297884532736": "Vocality International Ltd",
    "24.16304824": "Dell Inc.",
    "12.346863677440": "Vocality International Ltd",
    "24.8415539": "Huawei Technologies Co.,Ltd",
    "12.346866409472": "CONTEC GmbH",
    "24.16049147": "Huawei Technologies Co.,Ltd",
    "24.14966098": "Avaya Inc",
    "24.13157646": "Shenzhen Primestone Network Technologies.Co., Ltd.",
    "24.2106724": "Pegatron Corporation",
    "12.346867417088": "Epec Oy",
    "12.123917686403072": "Seraphim Optronics Ltd",
    "24.14966097": "Sfr",
    "12.119269675008": "TES Electronic Solutions (I) Pvt. Ltd.",
    "24.10794286": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.9968181": "Huawei Technologies Co.,Ltd",
    "20.30931566985216": "Ognios GmbH",
    "12.123917684432896": "Selmatec AS",
    "24.5562465": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.346859032576": "CXR Larus Corporation",
    "24.7397367": "ARRIS Group, Inc.",
    "24.12342521": "Drogoo Technology Co., Ltd.",
    "24.13934920": "Huawei Technologies Co.,Ltd",
    "24.12099957": "Roku, Inc.",
    "12.123917676838912": "W. H. Leary Co., Inc.",
    "12.346862804992": "Preferred Oil, LLC",
    "24.2625957": "Bose Corporation",
    "12.123917686775808": "Adigitalmedia",
    "12.71297884147712": "IntelliDesign Pty Ltd",
    "20.128612181213184": "Shenzhen Melon Electronics Co.,Ltd",
    "24.7123729": "Shenzhen Lianrui Electronics Co.,Ltd",
    "12.346866192384": "system2 GmbH",
    "20.180659899334656": "Luceor",
    "24.6606809": "Texas Instruments",
    "24.5300281": "Zyxel Communications Corporation",
    "24.15478929": "Cisco Systems, Inc",
    "20.268902724534272": "Airbeam Wireless Technologies Inc.",
    "24.13392467": "Cisco Systems, Inc",
    "24.4500213": "ARRIS Group, Inc.",
    "12.346864926720": "bach-messtechnik gmbh",
    "12.123917677391872": "Satsky Communication Equipment Co.,Ltd.",
    "24.12592487": "Nexxt Solutions",
    "24.5249100": "Platina Systems Inc.",
    "12.123917675773952": "Practical Software Studio LLC",
    "24.3175248": "Inpeco SA",
    "12.71297883955200": "Manufacturing System Insights Inc",
    "24.843841": "iDruide",
    "12.123917687308288": "Boundary Technologies Ltd",
    "12.123917675175936": "BAE Systems Surface Ships Limited",
    "12.346867466240": "i3 International Inc.",
    "24.2107454": "Huawei Technologies Co.,Ltd",
    "12.123917690265600": "Macnica Technology",
    "24.16279249": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.4475916": "Apple, Inc.",
    "20.136646470139904": "Midwest Microwave Solutions",
    "12.123917675544576": "Rako Controls Ltd",
    "24.61521": "KWB Gmbh",
    "12.346857623552": "MMS Servis s.r.o.",
    "24.4754130": "Mercury Communication Technologies Co.,Ltd.",
    "24.6875371": "Linktel Technologies Co.,Ltd",
    "24.12588286": "Cisco Systems, Inc",
    "12.123917680078848": "Technological Ray GmbH",
    "24.11830014": "Askey Computer Corp",
    "12.346855206912": "SafeView, Inc.",
    "12.346864193536": "Mitsuya Laboratories Inc.",
    "12.123917678239744": "Austco Marketing & Service (USA) ltd.",
    "20.260167862059008": "Hesai Photonics Technology Co., Ltd",
    "24.15681": "Hatteland Computer AS",
    "24.7652144": "Shenzhen Fast Technologies Co.,Ltd",
    "24.6299138": "EltexAlatau",
    "12.123917675315200": "mapna group",
    "12.123917680484352": "Panoramic Power",
    "24.10779343": "Vibicom Communications Inc.",
    "12.346860945408": "Eutron S.p.A.",
    "20.136646479577088": "I-Convergence.com",
    "24.14338173": "SE Development AB",
    "20.22739795574784": "Shenzhen Honor Electronic Co.,Ltd",
    "24.7118057": "Nimble Storage",
    "12.71297883967488": "Xepto Computing Inc",
    "24.16293307": "Tintri",
    "24.16540270": "Beijing MDC Telecom",
    "24.3189610": "Samsung Electro-Mechanics(Thailand)",
    "12.123917687668736": "Grupo Epelsa S.L.",
    "24.4029": "MRV Communications (Networks) LTD",
    "24.4028": "Onkey Technologies, Inc.",
    "24.4025": "Adaptive Instruments",
    "24.4024": "CallURL Inc.",
    "24.4027": "Nokia Siemens Networks GmbH & Co. KG.",
    "24.4026": "Tevebox AB",
    "24.4021": "Netgear",
    "24.4020": "Timespace Technology",
    "24.4023": "Cavium",
    "24.4022": "Europlex Technologies",
    "12.123917687459840": "FeCon GmbH",
    "24.1113072": "KangSheng LTD.",
    "24.3558": "Youngbo Engineering Co.,Ltd",
    "24.3559": "Snap-on OEM Group",
    "24.3554": "CMZ Sistemi Elettronici",
    "24.3555": "AT Sweden AB",
    "24.3556": "DIGINICS, Inc.",
    "24.3557": "Samsung Thales",
    "24.3550": "Joyteck Co., Ltd.",
    "24.3551": "Japan Image & Network Inc.",
    "24.3552": "ICPDAS Co.,LTD",
    "24.3553": "Control Products, Inc.",
    "24.797289": "electric imp, incorporated",
    "24.15263538": "Alcatel-Lucent Enterprise",
    "12.123917684801536": "pureLiFi Ltd",
    "20.62636952649728": "Shanghai qinzhuo Electronic Co., Ltd.",
    "20.48688647897088": "SureFlap Ltd",
    "24.845871": "Bonfiglioli Vectron GmbH",
    "12.346867216384": "CAIPO Automazione Industriale s.r.l.",
    "12.123917677670400": "Arnouse Digital Devices, Corp.",
    "12.123917682630656": "\"Smart Systems\" LLC",
    "24.11819105": "CRemote, LLC",
    "24.7120969": "Avaya Inc",
    "12.123917676068864": "Plum sp. z o.o",
    "12.123917681692672": "DesignA Electronics Limited",
    "24.4470639": "Young Electric Sign Co",
    "24.9204590": "Fujitsu Limited",
    "24.2930323": "Radware",
    "12.346859495424": "Blue Origin",
    "12.123917690548224": "Beijing Lihong Create Co., Ltd.",
    "24.9188729": "QDIQO Sp. z o.o.",
    "12.123917678727168": "Imp-Telekomunikacije Doo",
    "24.9367": "Cisco Systems, Inc",
    "24.5303376": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.123917684072448": "Crde",
    "24.5516173": "Apple, Inc.",
    "24.7924852": "Kyland-USA",
    "20.194363551383552": "Shenzhen Ktc Technology Group",
    "24.10528611": "Polytron Vertrieb GmbH",
    "20.136823644880896": "Lukup Media",
    "24.11062383": "HMD Global Oy",
    "20.149945496633344": "Apoidea Technology Co., Ltd.",
    "24.9474108": "Trison Technology Corporation",
    "24.12359903": "Motorola Mobility LLC, a Lenovo Company",
    "24.8177958": "Wuhan GreeNet Information Service Co.,Ltd.",
    "24.12122420": "Sony Mobile Communications Inc",
    "24.12875938": "Huawei Device Co., Ltd.",
    "24.5039534": "Tianjin Beebox Intelligent Technology Co.,Ltd.",
    "12.71297884508160": "Weber Marking Systems GmbH",
    "24.15774700": "Apple, Inc.",
    "24.12339": "Orient Telecom Co., Ltd.",
    "24.12338": "MagicRam, Inc.",
    "12.119269421056": "Fibrain Co. Ltd.",
    "24.7355669": "Hangzhou H3C Technologies Co., Limited",
    "24.12331": "Inalp Networks, Inc.",
    "24.12330": "Southern Information",
    "24.12333": "Quantum Bridge Communications",
    "24.12332": "Sylantro Systems Corporation",
    "24.12335": "GE Aviation System",
    "24.12334": "Hoft & Wessel AG",
    "24.12337": "Lightwave Communications, Inc.",
    "24.12336": "Harmonix Corporation",
    "24.6083611": "UCZOON Technology Co., LTD",
    "24.14460404": "Cisco Systems, Inc",
    "20.79215070281728": "Thales CETCA Avionics CO., Ltd",
    "24.8411645": "Noviga",
    "24.10026356": "Act.Co.Ltd",
    "20.264809155133440": "Mstar Technologies,Inc",
    "24.3086": "XtremeSpectrum, Inc.",
    "24.3087": "Techno-One Co., Ltd",
    "24.3084": "Appro Technology Inc.",
    "24.3085": "Communications & Power Industries / Satcom Division",
    "24.3082": "Guangdong Province Electronic Technology Research Institute",
    "24.3083": "Broadbus Technologies",
    "24.3080": "HUMEX Technologies Corp.",
    "24.3081": "Hitachi IE Systems Co., Ltd",
    "24.3088": "PNI Corporation",
    "24.3089": "Nippon Dempa Co.,Ltd.",
    "24.1019": "ENEGATE Co.,Ltd.",
    "12.123917680328704": "Intelligent Security Systems (ISS)",
    "12.71297884397568": "Jadpod Communication Company Limited",
    "24.6088887": "Oraimo Technology Limited",
    "12.346854367232": "Mst Systems Limited",
    "12.123917681053696": "Haag-Streit AG",
    "24.7709": "East Coast Datacom, Inc.",
    "24.7708": "SWS Australia Pty Limited",
    "24.7705": "Gtri",
    "24.7704": "Radio Activity srl",
    "24.7707": "Digital Stream Technology, Inc.",
    "24.7706": "Best Source Taiwan Inc.",
    "24.7701": "Beech Hill Electronics",
    "24.7700": "Cisco Systems, Inc",
    "24.7703": "Stn Bv",
    "24.7702": "Keytronix",
    "24.10249808": "Glodio Technolies Co.,Ltd Tianjin Branch",
    "24.6829749": "DriveScale, Inc.",
    "12.346868174848": "HUSTY M.Styczen J.Hupert Sp.J.",
    "24.6308986": "Water-i.d. GmbH",
    "24.4770670": "Cello Electronics (UK) Ltd",
    "24.9235016": "Samsung Electronics Co.,Ltd",
    "24.7355083": "Google, Inc.",
    "24.8971851": "Juniper Networks",
    "24.2932232": "Apple, Inc.",
    "24.3718170": "MEIZU Technology Co., Ltd.",
    "24.5014223": "Dish Technologies Corp",
    "24.16410227": "Nells Electronic Systems (Nes)",
    "12.346855071744": "Initial Electronic Security Systems",
    "24.13652648": "Physical Graph Corporation",
    "12.346868072448": "Terratel Technology s.r.o.",
    "24.1864488": "Connected Home",
    "20.229206580527104": "tecnotron elekronik gmbh",
    "20.229558885285888": "Iflytek Co.,Ltd.",
    "20.31368130068480": "Rdp.Ru",
    "24.13149728": "Nebula, Inc",
    "24.5268545": "Greenlee",
    "24.16279921": "Seibu Electric Co.,",
    "12.346856144896": "Linkwell Telesystems Pvt Ltd",
    "24.12877649": "Inventek Systems",
    "12.119269244928": "Intellvisions Software Ltd",
    "24.10013052": "Shenzhen iComm Semiconductor CO.,LTD",
    "12.123917685846016": "Pcsc",
    "12.123917681029120": "Aton srl",
    "24.2143686": "Mimosa Networks",
    "24.4218593": "Shenzhen H&T Intelligent Control Co.,Ltd.",
    "12.123917678530560": "Fresh Idea Factory BV",
    "12.123917678366720": "Jonsa Australia Pty Ltd",
    "24.6078": "Tratec Telecom B.V.",
    "24.6079": "Coherent Research Limited",
    "24.6074": "Sedo Co., Ltd.",
    "24.6075": "Syrinx Industrial Electronics",
    "24.6076": "Touchtunes Music Corporation",
    "24.6077": "Tibetsystem",
    "24.2751": "Hirota Ss",
    "24.6071": "Tonze Technology Co.",
    "24.6072": "Novatron Co., Ltd.",
    "24.6073": "Gambro Lundia AB",
    "12.346860752896": "St Jude Medical, Inc.",
    "24.13420893": "Apple, Inc.",
    "24.1228": "Peek Traffic B.V.",
    "24.1229": "Extenway Solutions Inc",
    "24.1224": "LIBA Maschinenfabrik GmbH",
    "24.1225": "Micro Electron Co., Ltd.",
    "24.1226": "FreeMs Corp.",
    "24.1227": "Tdsoft Communication, Ltd.",
    "24.1220": "Audiotonix Group Limited",
    "24.1221": "ASE Technologies, USA",
    "24.1222": "Yamaha Motor Co.,Ltd",
    "24.1223": "NetMount",
    "24.564153": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.8508": "AliphCom",
    "24.8509": "Cermetek Microelectronics, Inc.",
    "12.123917676425216": "Armstrong International, Inc.",
    "24.2638316": "Huawei Technologies Co.,Ltd",
    "24.8502": "ARRIS Group, Inc.",
    "24.8503": "Bay Controls, LLC",
    "24.8500": "Brandywine Communications",
    "24.8501": "Alcatel-Lucent",
    "24.8506": "Winchester Systems Inc.",
    "24.8507": "Berkshire Products, Inc",
    "24.8504": "Cepheid",
    "24.8505": "Escherlogic Inc.",
    "12.123917686034432": "exceet electronics GesmbH",
    "24.11827953": "Hewlett Packard Enterprise",
    "24.13637030": "Aruba, a Hewlett Packard Enterprise Company",
    "24.12604214": "Beijing National Railway Research & Design Institute of Signal & Communication Group Co..Ltd.",
    "20.145211818770432": "Dangerous Music Group, LLC",
    "12.123917683896320": "Liquid Instruments Pty Ltd",
    "24.6327339": "Hanson Technology",
    "12.346863759360": "NARA Controls Inc.",
    "12.123917676642304": "Fantom Wireless, Inc.",
    "24.6558313": "ARRIS Group, Inc.",
    "24.52049": "Sagemcom Broadband SAS",
    "20.154794717085696": "Dark Horse Connect LLC",
    "12.123917679579136": "Sensapex Oy",
    "24.14692205": "Cisco Systems, Inc",
    "24.7914473": "Samsung Electronics Co.,Ltd",
    "24.6831229": "Magic Intelligence Technology Limited",
    "20.123731215122432": "CVnet",
    "12.123917687099392": "Ida",
    "12.123917681164288": "Hexagon Metrology SAS",
    "24.15996213": "SPON Communication Technology Co.,Ltd",
    "24.14953171": "Magneti Marelli S.p.A. Powertrain",
    "24.4255128": "IEEE Registration Authority",
    "24.4989392": "Sony Mobile Communications Inc",
    "20.48835651960832": "Schneider Electric Korea",
    "24.1596339": "Samsung Electronics Co.,Ltd",
    "24.13921884": "Huawei Technologies Co.,Ltd",
    "12.123917684539392": "Glova Rail A/S",
    "12.123917682561024": "Swiss Audio",
    "24.14731853": "Shenzhen Unistar Communication Co.,LTD",
    "20.149935436595200": "Schmidt & Co.,(H.K.)Ltd.",
    "24.6291860": "Espressif Inc.",
    "12.346862374912": "Sensitron Semiconductor",
    "12.346855387136": "SAIA Burgess Controls AG",
    "24.7346692": "Liteon Technology Corporation",
    "24.3472495": "Alcea",
    "20.180680387461120": "Foshan Yisihang Electrical Technology Co., Ltd.",
    "24.15745147": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3956419": "Cisco Systems, Inc",
    "24.5035": "Telemotive AG",
    "24.5034": "ALS & TEC Ltd.",
    "24.5037": "Sendo Ltd",
    "24.5036": "Sunmyung Electronics Co., LTD",
    "24.5031": "Battelle Memorial Institute",
    "24.5030": "Extricom Ltd",
    "24.5033": "Sony Corporation",
    "24.5032": "Tanisys Technology",
    "24.8404552": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917676728320": "Metronic Aparatura Kontrolno - Pomiarowa",
    "24.5039": "NUMA Technology,Inc.",
    "24.5038": "Radiance Technologies, Inc.",
    "24.4344": "Texio Technology Corporation",
    "24.4345": "Unique Systems, Inc.",
    "24.4346": "Apple, Inc.",
    "24.4347": "Zida Technologies Limited",
    "24.4340": "Vertical Communications",
    "24.4341": "Amherst Systems, Inc.",
    "24.4342": "Cisco Systems, Inc",
    "24.4343": "IRIICHI TECHNOLOGIES Inc.",
    "24.9745009": "Huawei Technologies Co.,Ltd",
    "24.4348": "Broadband Networks, Inc.",
    "12.346857672704": "Waves System",
    "24.3967664": "Juniper Networks",
    "20.150262233694208": "Solaredge LTD.",
    "24.9490301": "Qingdao Hisense Communications Co.,Ltd.",
    "24.53394": "Glenayre Western Multiplex",
    "24.53395": "Tq - Components Gmbh",
    "24.53396": "Seeion Control LLC",
    "24.53397": "Alcatel-Lucent Enterprise",
    "24.53390": "Grass Valley, A Belden Brand",
    "24.53391": "Ardent Technologies, Inc.",
    "24.53392": "Cisco Systems, Inc",
    "24.53393": "Smartsan Systems, Inc.",
    "24.11547915": "Mobiwire Mobiles (Ningbo) Co.,Ltd",
    "24.5281105": "ARRIS Group, Inc.",
    "24.6837100": "SPnS Co.,Ltd",
    "24.53398": "3Com Europe Ltd",
    "24.53399": "Cisco Systems, Inc",
    "12.123917689360384": "Biovigil Hygiene Technologies",
    "24.6073807": "Apple, Inc.",
    "20.255191439900672": "AVCON Information Technology Co.,Ltd",
    "12.123917676744704": "Oceanix Inc.",
    "24.12368578": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.8667495": "Apple, Inc.",
    "24.9702357": "Himax Technologies, Inc",
    "24.5303672": "Changzhou Yongse Infotech Co.,Ltd",
    "24.4492370": "KTIS CO., Ltd",
    "24.5506198": "Gigawave LTD",
    "24.53020": "Communication Machinery Corporation",
    "12.346856886272": "Computer Sciences Corp",
    "24.2367780": "Nokia",
    "12.123917687435264": "Power Electronics Espana, S.L.",
    "12.123917680234496": "Enless Wireless",
    "24.9183124": "RF Surgical System Inc.",
    "24.6561951": "Tilgin AB",
    "12.346859995136": "SunGard Vivista",
    "24.8439637": "Panasonic Appliances Company",
    "20.277783355850752": "Hooc Ag",
    "12.71297884422144": "GSP Sprachtechnologie GmbH",
    "24.8427210": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5019156": "Juniper Networks",
    "12.123917676855296": "Smeg S.p.A.",
    "12.123917686886400": "Ctag - Esg36871424",
    "24.7864975": "Adaptive Spectrum and Signal Alignment (ASSIA), Inc.",
    "24.3962854": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.110162574573568": "Smartplus Inc.",
    "24.36869": "Protech Systems Co., Ltd.",
    "24.5041085": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.15481840": "NovelSat",
    "24.36868": "3Com Europe Ltd",
    "12.123917679013888": "GS Elektromedizinsiche Geräte G. Stemple GmbH",
    "24.11803696": "zte corporation",
    "24.7929236": "Apple, Inc.",
    "12.123917684187136": "F-domain corporation",
    "20.114869211758592": "LEAPS s.r.o.",
    "12.123917680873472": "LECO Corporation",
    "24.16561816": "Cambridge Industries(Group) Co.,Ltd.",
    "20.154055555940352": "T+A elektroakustik GmbH & Co.KG",
    "12.346859798528": "Letechnic Ltd",
    "24.1849671": "Compal Information (Kunshan) Co., Ltd.",
    "24.7403348": "AMPAK Technology,Inc.",
    "24.266521": "IEEE Registration Authority",
    "12.346862895104": "Brähler ICS Konferenztechnik AG",
    "24.6052802": "YIK Corporation",
    "24.9238387": "Nokia",
    "12.123917678125056": "Toray Medical Co.,Ltd",
    "12.123917690060800": "Walton Hi-Tech Industries Ltd.",
    "24.36867": "Aplio",
    "12.123917686169600": "Proximus",
    "24.36866": "Allgon Ab",
    "24.3723573": "EasyIO Corporation Sdn. Bhd.",
    "12.346863407104": "Showtacle",
    "12.346861281280": "Servaind Sa.",
    "12.346866900992": "Pmac Japan",
    "12.346856660992": "Elettronica Santerno SpA",
    "12.123917684039680": "Innovative Control Systems, LP",
    "12.123917686378496": "aelettronica group srl",
    "24.8400404": "azeti Networks AG",
    "24.13415889": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.71297883783168": "Nomad Digital Limited",
    "24.1599203": "Veros Systems, Inc.",
    "12.123917689704448": "Lobaro GmbH",
    "24.10289090": "AVI Systems GmbH",
    "12.346859634688": "Colorado Engineering Inc.",
    "12.346859937792": "Digidrive Audio Limited",
    "24.8170674": "Bosch Software Innovations GmbH",
    "24.14212447": "BALMUDA Inc.",
    "24.9471": "QLogic Corporation",
    "24.9470": "AVM GmbH",
    "24.9473": "JSC \"Supertel\"",
    "24.9472": "Apple, Inc.",
    "24.9475": "IBM Corp",
    "24.9474": "NaturalPoint",
    "24.9477": "eks Engel GmbH & Co. KG",
    "24.9476": "Valiant Communications Limited",
    "24.9479": "ASTAK Inc.",
    "24.9478": "A.I. Antitaccheggio Italia Srl",
    "12.346863919104": "Crde",
    "24.16282516": "Netgear",
    "24.9176401": "Koubachi AG",
    "20.39986088902656": "Tata Sky Limited",
    "20.75247159083008": "Shenzhen Mestechs Technology CO., LTD",
    "12.123917684506624": "Eginity, Inc.",
    "24.15002640": "Nokia Corporation",
    "24.16048450": "Delta Elektronika BV",
    "20.229206577381376": "InventDesign",
    "24.840962": "Westell Technologies Inc.",
    "24.9445507": "General Engine Management Systems Ltd.",
    "12.346867318784": "Will corp.",
    "12.123917689487360": "Deep Secure Limited",
    "24.13112401": "Huawei Technologies Co.,Ltd",
    "24.8014": "ConMed Linvatec",
    "24.2634086": "vivo Mobile Communication Co., Ltd.",
    "24.7344220": "Cisco Systems, Inc",
    "12.123917682823168": "Sonel S.A.",
    "12.123917678067712": "Grupo Epelsa S.L.",
    "12.123917675229184": "Eiffage Energie Electronique",
    "12.346861432832": "Polygon Informatics Ltd.",
    "24.2898685": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.7064": "Samsung Electronics Co.,Ltd",
    "24.2883628": "Unowhy",
    "24.7066": "Apollo Fire Detectors Ltd",
    "20.39919504326656": "WithWin Technology ShenZhen CO.,LTD",
    "24.7060": "T.E.M.A. S.p.A.",
    "24.7061": "Video Systems Srl",
    "24.7062": "General Sensing",
    "24.7063": "Violin Technologies",
    "12.123917682155520": "Ahrens & Birner Company GmbH",
    "12.123917686927360": "Crdmdeveoppements",
    "24.7068": "SATEL sp. z o.o.",
    "24.7069": "Novus Security Sp. z o.o.",
    "12.346865553408": "Tecnovum AG",
    "24.2254": "IPMobileNet Inc.",
    "24.2255": "Nippon Koei Power Systems Co., Ltd.",
    "24.2256": "Musashi Engineering Co., LTD.",
    "24.2257": "Karel Inc.",
    "24.2250": "TwinHan Technology Co.,Ltd",
    "24.2251": "Zeta Broadband Inc.",
    "24.2252": "Remotec, Inc.",
    "24.2253": "With-Net Inc",
    "24.2258": "ZOOM Networks Inc.",
    "24.2259": "Hercules Technologies S.A.S.",
    "24.3437923": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.12122302": "Bluecom",
    "12.123917677113344": "WOOJIN Inc",
    "20.48835640426496": "Shanghai Shuncom Electronic Technology Co.,Ltd",
    "12.123917690290176": "Lumiplan Duhamel",
    "24.3937554": "Shenzhen Huanhu Technology Co.,Ltd",
    "20.176032235651072": "Ahgora Sistemas SA",
    "24.14193667": "Hewlett Packard Enterprise",
    "12.123917686292480": "Lexi Devices, Inc.",
    "12.123917691351040": "Typhon Treatment Systems Ltd",
    "24.2922263": "I.C.Y. B.V.",
    "24.13904957": "Sky Light Digital Limited",
    "20.110162580865024": "Shenzhen Huanyin Electronics Ltd.",
    "12.346856878080": "BRIT Inc.",
    "24.6309398": "Xiamen Vann Intelligent Co., Ltd",
    "24.12910367": "Huawei Technologies Co.,Ltd",
    "24.6344133": "Taiwan Carol Electronics., Ltd",
    "24.15477833": "University of Tokyo",
    "12.346857111552": "Shenzhen Hongdian Technologies.,Ltd",
    "12.123917690298368": "Cellier Domesticus inc",
    "20.158667123982336": "CUTTER Systems spol. s r.o.",
    "12.346862206976": "Xafax Nederland bv",
    "24.3431185": "Evi Heat Ab",
    "24.13144963": "Embedded Automation, Inc.",
    "24.3673556": "Primax Electronics Ltd.",
    "12.123917687644160": "SYSDEV Srl",
    "24.3987755": "Hewlett Packard",
    "24.7657077": "WYMA Tecnologia",
    "12.71297884831744": "RF Code",
    "24.30177": "Ampt, LLC",
    "12.346868363264": "Deltacast.Tv",
    "24.8142452": "Huawei Device Co., Ltd.",
    "24.3976078": "Apple, Inc.",
    "24.5822297": "Huawei Technologies Co.,Ltd",
    "12.123917690724352": "Meiden System Solutions",
    "12.123917682974720": "Innerspec Technologies Inc.",
    "12.346861400064": "GigaLinx Ltd.",
    "24.264835": "Alcatel-Lucent",
    "24.5797153": "CJSC RTSoft",
    "24.7610186": "Hangzhou H3C Technologies Co., Limited",
    "12.346857717760": "Sintecnos srl",
    "24.2631773": "Zyxel Communications Corporation",
    "12.346866393088": "Manz",
    "12.123917685985280": "NOREYA Technology e.U.",
    "12.346867662848": "Ibernex Ingenieria, S.L.",
    "24.1340275": "Tubitak Uekae",
    "24.138364": "Perq Systems Corporation",
    "24.13924507": "Cisco Systems, Inc",
    "20.70682236944384": "Shenzhen Yifang Digital Technology Co., LTD.",
    "12.346852777984": "Utronix Elektronikutreckling AB",
    "24.16549418": "Huawei Device Co., Ltd.",
    "12.123917688795136": "Light field Lab",
    "12.346866257920": "EMAC, Inc.",
    "24.11047762": "Shanghai Zhongmi Communication Technology Co.,Ltd",
    "12.346868584448": "Hijet Print d.o.o.",
    "24.2442": "EqualLogic Inc",
    "24.7349139": "fos4X GmbH",
    "12.346862071808": "Systèmes Pran",
    "24.822415": "Kamo Technology Limited",
    "20.89058726379520": "Goetting KG",
    "24.5950419": "Simaudio Ltd",
    "12.346861424640": "Peek Traffic Corp",
    "12.123917678891008": "Preston Industries dba PolyScience",
    "20.273458029133824": "Whizpace Pte. Ltd.",
    "12.346866311168": "LYNX Technik AG",
    "24.13135645": "Apple, Inc.",
    "20.27056775626752": "Shenzhen Tong Tai Yi information Technology Co.,Ltd",
    "24.833417": "Honor Device Co., Ltd.",
    "20.44213332869120": "Knowles Electronics LLC",
    "24.46325": "DongGuan Siyoto Electronics Co., Ltd",
    "24.6039050": "Dell Inc.",
    "24.10486846": "Parker Hannifin Manufacturing Germany GmbH & Co. KG",
    "24.10534173": "Zhejiang Dahua Technology Co., Ltd.",
    "24.10286813": "IEEE Registration Authority",
    "24.9447211": "Algorab S.r.l.",
    "12.123917680914432": "CT Company",
    "12.346857750528": "Weinzierl Engineering GmbH",
    "24.6554495": "Samsung Electronics Co.,Ltd",
    "24.10494747": "TeleComp R&D Corp.",
    "24.2401886": "zte corporation",
    "24.15466803": "Trinus Systems Inc.",
    "24.1102300": "INSTAR Deutschland GmbH",
    "12.346855591936": "Act",
    "24.1347728": "KongTop industrial(shen zhen)CO.,LTD",
    "24.4513934": "Cisco SPVTG",
    "24.7125688": "Sagemcom Broadband SAS",
    "20.44213336014848": "Sabinetek",
    "24.1345386": "Apple, Inc.",
    "24.4475568": "Zhejiang Moorgen Intelligence Technology Co., Ltd",
    "12.123917682556928": "Mansion Industry Co., Ltd.",
    "20.229631757123584": "Optigo Networks",
    "24.20519": "Genicom Corporation",
    "24.12624059": "D-Link International",
    "24.4393": "Paradise Datacom Ltd.",
    "12.123917688737792": "Senior Group LLC",
    "24.2145216": "Cisco Systems, Inc",
    "12.346857455616": "Airwise Technology Co., Ltd.",
    "12.123917678923776": "Dao Qin Technology Co.Ltd.",
    "24.11823393": "Intel Corporate",
    "24.6609170": "Solidica, Inc.",
    "24.1085114": "Molekule",
    "24.3944824": "Qolsys Inc.",
    "12.123917689606144": "Simpulse",
    "20.228844762038272": "Xperio Labs Ltd.",
    "24.1858155": "Philips Electronics Nederland BV",
    "24.5030994": "Shang Hai Tyd. Electon Technology Ltd.",
    "12.123917691748352": "Dutile, Glines and Higgins Corporation",
    "12.346865061888": "Industrial Automation Systems",
    "24.20512": "Mediastar Co., Ltd.",
    "20.175931697135616": "Donguan Amsamotion Automation Technology Co., Ltd",
    "24.15779711": "Texas Instruments",
    "12.123917680623616": "Saratoga Speed, Inc.",
    "24.10787663": "Huawei Technologies Co.,Ltd",
    "24.11287563": "Apple, Inc.",
    "20.71389212049408": "Integrated Design Ltd",
    "20.212188449996800": "Shenzhen Heqiang Electronics Limited",
    "12.346864898048": "Private",
    "24.10803746": "IEEE Registration Authority",
    "12.346864615424": "Phaedrus Limited",
    "12.123917676232704": "Cielle S.R.L.",
    "24.2667377": "Ruckus Wireless",
    "24.8963622": "Logitech, Inc",
    "24.5292962": "nFore Technology Co.,Ltd.",
    "20.22333152559104": "Unirobot Corporation",
    "20.35758082621440": "Shanghai Kenmyond Industrial Network Equipment Co.,Ltd",
    "12.346865971200": "Video Tech Laboratories, Inc.",
    "24.3975743": "iKey, Ltd.",
    "24.12616458": "2Wire Inc",
    "20.31477062434816": "NovTech, Inc.",
    "12.346855088128": "Putercom Co., Ltd",
    "12.123917683171328": "Hunan Shengyun Photoelectric Technology Co., Ltd.",
    "24.3968288": "Google, Inc.",
    "24.11839834": "Avaya Inc",
    "20.189549216006144": "Groupe Citypassenger Inc",
    "12.346861776896": "Dsp Design",
    "24.12589189": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917678772224": "SIPRO s.r.l.",
    "24.7109048": "Texas Instruments",
    "24.550552": "Apple, Inc.",
    "12.123917680193536": "Xps Eletronica Ltda",
    "24.10761234": "Thales Alenia Space",
    "12.346868396032": "MCM Electronics",
    "24.8224": "Megatron Computer Industries Pty, Ltd.",
    "24.2379387": "IEEE Registration Authority",
    "24.9215803": "Qingdao Hisense Communications Co.,Ltd.",
    "24.584685": "World Elite Technology Co.,LTD",
    "12.123917678383104": "Sierra Nevada Corporation",
    "12.123917690429440": "EnergizeEV",
    "24.9239226": "Apple, Inc.",
    "24.12092163": "Hewlett Packard Enterprise",
    "20.66129356652544": "Travis Holding B.V.",
    "24.8933754": "HMD Global Oy",
    "12.346862182400": "Link Technologies, Inc",
    "12.123917676937216": "Blue Skies Global LLC",
    "20.176069212635136": "Hunan Youmei Science&Technology Development Co.,Ltd.",
    "24.10256215": "UNIC Memory Technology Co Ltd",
    "24.4473391": "Testop Co.,Ltd.",
    "24.12519": "Cnf Mobile Solutions, Inc.",
    "24.12518": "Draeger Medical Systems, Inc.",
    "12.123917676531712": "Akribis Systems",
    "12.123917686685696": "Datasat Digital Entertainment",
    "24.7662015": "Huawei Technologies Co.,Ltd",
    "24.2404613": "Prama Hikvision India Private Limited",
    "20.264036221452288": "BSP RUS Ltd.",
    "24.12511": "Kb/Tel Telecomunicaciones",
    "24.12510": "WAGO Kontakttechnik GmbH",
    "24.12513": "Network Equipment Technologies, Inc.",
    "24.12512": "Oxford Semiconductor Ltd.",
    "24.12515": "Sedona Networks Corp.",
    "24.12514": "Garnet Systems Co., Ltd.",
    "24.12517": "Amper Datos S.A.",
    "24.12516": "Chiyoda System Riken",
    "24.7394067": "Huawei Technologies Co.,Ltd",
    "20.31614745706496": "Exigent Sensors",
    "24.804509": "MS Sedco",
    "24.1345817": "2bps",
    "24.5233": "Eastern Asia Technology Limited",
    "24.5232": "Prokom Software SA",
    "24.5231": "Kohler Co",
    "24.5230": "H. Stoll GmbH & Co. KG",
    "24.5237": "Wiline Networks, Inc.",
    "24.5236": "K40 Electronics",
    "24.5235": "Bookham Inc",
    "24.5234": "China Broadband Wireless IP Standard group(ChinaBWIPS)",
    "24.5239": "Trilliant",
    "24.5238": "MultiCom Industries Limited",
    "24.301299": "Alpsalpine Co,.Ltd",
    "24.4182": "Sodick Co., Ltd.",
    "24.4183": "Rebel.com, Inc.",
    "24.4180": "Cisco Systems, Inc",
    "24.4181": "Fujitsu Microelectronics, Inc.",
    "24.4186": "3Com",
    "24.4187": "Net Insight Ab",
    "24.4184": "ArrowPoint Communications",
    "24.4185": "Diablo Research Co. Llc",
    "20.176726068953088": "Sandal Plc",
    "24.4188": "Quantum Designs (H.K.) Ltd.",
    "24.4189": "Draeger Medical",
    "24.13925884": "Tecno Mobile Limited",
    "20.154314898145280": "Scharfe-Sicht GmbH",
    "24.9200802": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.5273233": "Tekpea, Inc.",
    "12.346861793280": "Phytec Messtechnik GmbH",
    "24.1854422": "AzureWave Technology Inc.",
    "24.5589": "Nicevt",
    "24.5588": "Emitor AB",
    "24.5547545": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.5585": "ARRIS Group, Inc.",
    "24.5584": "ARRIS Group, Inc.",
    "24.5587": "Pantech&Curitel Communications, Inc.",
    "24.5586": "Xantech Corporation",
    "24.5581": "Exartech International Corp.",
    "24.5580": "Uquest, Ltd.",
    "24.5583": "ARRIS Group, Inc.",
    "24.5582": "ARRIS Group, Inc.",
    "24.6330387": "Shanghai Visking Digital Technology Co. LTD",
    "24.13120050": "Hunting Innova",
    "24.12622042": "China Mobile IOT Company Limited",
    "24.32793": "Dayna Communications",
    "12.346861232128": "Selco",
    "24.15736109": "Cisco Systems, Inc",
    "12.346868760576": "Insitu, Inc",
    "24.12089390": "Apple, Inc.",
    "12.123917686820864": "Hon Hai Precision IND.CO.,LTD",
    "12.123917689303040": "Voleatech GmbH",
    "12.346868068352": "Bio Guard component & technologies",
    "24.57538": "NECSY S.p.A.",
    "24.57539": "Sakai System Development Corp.",
    "24.57534": "Genroco International, Inc.",
    "24.57535": "Torrent Networking Technologies Corp.",
    "24.57536": "Seiwa Electric Mfg. Co., Ltd.",
    "24.57537": "Memorex Telex Japan, Ltd.",
    "24.57530": "BERGHOF AUTOMATIONSTECHNIK GmbH",
    "12.346854666240": "Vansco Electronics Oy",
    "12.346868060160": "Radyne Corporation",
    "24.57533": "Interface Systems, Inc.",
    "24.6072426": "Huawei Technologies Co.,Ltd",
    "24.5012697": "Apple, Inc.",
    "24.2605": "Cabot Communications Limited",
    "24.5807814": "Gigaset Communications GmbH",
    "12.346856517632": "Scheidt & Bachmann GmbH",
    "24.14694871": "Plexxi, Inc.",
    "24.9763560": "Shenzhen Eycom Technology Co., Ltd",
    "24.280165": "Murata Manufacturing Co., Ltd.",
    "12.123917678329856": "ATION Corporation",
    "12.346853019648": "Renaissance Networking, Inc.",
    "24.6354973": "Apple, Inc.",
    "24.16003880": "CIMCON Lighting Inc.",
    "12.346867875840": "Private",
    "24.1088185": "Tosibox Oy",
    "20.66426704494592": "Bosch Automotive Products (Suzhou) Co., Ltd.",
    "24.2660666": "Arista Networks",
    "24.6353989": "Apple, Inc.",
    "24.11587405": "MI-Zone Technology Ireland",
    "24.3329": "P&E Microcomputer Systems, Inc.",
    "24.3328": "Seaway Networks Inc.",
    "24.16520176": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "24.1356887": "Silicon Laboratories",
    "24.3323": "Korea Network Systems",
    "24.3322": "Digital Systems Corp",
    "24.3321": "Xylem Water Solutions",
    "24.3320": "Nortel Networks",
    "24.3327": "MRO-TEK Realty Limited",
    "24.3326": "Grand Electronic Co., Ltd",
    "24.3325": "Hyundai ImageQuest Co.,Ltd.",
    "24.3324": "S2io Technologies Corp",
    "12.346864746496": "ACD Elektronik GmbH",
    "24.9761467": "Valeo Vision Systems",
    "12.71297883480064": "Gateway Technologies SA de CV",
    "24.9201864": "zte corporation",
    "24.9234971": "Edgecore Networks Corporation",
    "12.123917687332864": "DWEWOONG ELECTRIC Co., Ltd.",
    "24.11027945": "Firich Enterprises CO., LTD.",
    "24.12090479": "NXP (China) Management Ltd.",
    "24.6673": "Google, Inc.",
    "20.238070228058112": "Scanmatik",
    "24.14437341": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "20.71414654697472": "Beijing Zongheng Electro-Mechanical Technology Development Co.",
    "12.346858881024": "Keith & Koep GmbH",
    "24.12597398": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9180285": "IEEE Registration Authority",
    "20.52819395608576": "Velsitec-Clibase",
    "24.6072097": "New H3C Technologies Co., Ltd",
    "24.7659630": "Ergophone GmbH",
    "20.137141735653376": "Virgin Orbit",
    "12.346857099264": "Valley Tecnologia",
    "12.123917678952448": "Hagiwara Solutions Co., Ltd.",
    "24.3713375": "Actifio Inc",
    "24.7359204": "Rinstrum Pty Ltd",
    "24.5957216": "nyantec GmbH",
    "24.2625020": "Sony Interactive Entertainment Inc.",
    "24.5258436": "Lenovo Mobile Communication Technology Ltd.",
    "20.167860144963584": "SPM Instrument AB",
    "24.1605333": "shenzhen kaism technology Co. Ltd",
    "24.5544524": "Guangdong Homecare Technology Co.,Ltd.",
    "24.14985439": "Compal Information (Kunshan) Co., Ltd.",
    "12.123917681745920": "Siame",
    "12.346852966400": "InterEpoch Technology,INC.",
    "12.123917683662848": "PLUTO Solution co.,ltd.",
    "12.123917685555200": "w-tec AG",
    "12.346858139648": "HITECOM System",
    "24.5796038": "Shanghai Xiaoyi Technology Co., Ltd.",
    "24.9758792": "Fylde Micro Ltd",
    "24.11541372": "Ontrol A.S.",
    "12.123917677961216": "Shanghai Irisian Optronics Technology Co.,Ltd.",
    "24.15738226": "Cisco Systems, Inc",
    "24.9448244": "Giga-Byte Technology Co.,Ltd.",
    "24.2492": "Utility, Inc",
    "12.346863726592": "Fareco",
    "24.2490": "MAKU Informationstechik GmbH",
    "24.2491": "MathStar, Inc.",
    "24.2496": "6Wind",
    "24.2497": "Proces-Data A/S",
    "24.2494": "Mamiya-OP Co.,Ltd.",
    "24.2495": "Nintendo Co., Ltd.",
    "24.2498": "Onity, Inc.",
    "24.2499": "Netas",
    "12.123917685764096": "Oledcomm",
    "24.12102974": "Bluecom",
    "24.639": "ask-technologies.com",
    "24.638": "Cisco Systems, Inc",
    "24.15472486": "Research Centre Module",
    "20.57194444750848": "Church & Dwight Co., Inc.",
    "24.635": "Amplify Net, Inc.",
    "24.634": "IOI Technology Corporation",
    "24.637": "Cisco Systems, Inc",
    "24.636": "Trilithic, Inc.",
    "24.631": "Cash Systemes Industrie",
    "24.630": "Primax Electronics Ltd.",
    "24.633": "Control Applications, Ltd.",
    "24.632": "Samsung Electro Mechanics Co., Ltd.",
    "20.212014903328768": "Fuzhou Jinshi Technology Co.,Ltd.",
    "12.123917685063680": "Kita Kirmizi Takim Bilgi Guvenligi Danismanlik ve Egitim A.S.",
    "12.346866601984": "Saia-Burgess Controls AG",
    "24.11063862": "Lightware Visual Engineering",
    "24.7790": "Shenzhen First Mile Communications Ltd",
    "20.101728426917888": "Annapurna labs",
    "12.346865950720": "Rch Group",
    "24.7518": "Coming Media Corp.",
    "24.7519": "OverSpeed SARL",
    "20.44768533938176": "Anser Coding Inc.",
    "24.7514": "2Wire Inc",
    "24.7515": "Tecvan Informática Ltda",
    "24.7516": "Tom Communication Industrial Co.,Ltd.",
    "24.7517": "Control Dynamics Pty. Ltd.",
    "12.346854850560": "Ebneuro Spa",
    "24.7511": "CAETEC Messtechnik",
    "24.7512": "CQ Inc",
    "24.7513": "Mitra Energy & Infrastructure",
    "24.9720979": "Rigado, LLC",
    "24.9743562": "trivum technologies GmbH",
    "24.14949898": "IEEE Registration Authority",
    "20.31477535342592": "Nanjing Bilin Intelligent Identification Technology Co.,Ltd",
    "24.11573436": "Huawei Device Co., Ltd.",
    "24.37095": "Horsch Elektronik Ag",
    "24.5917": "Digit",
    "24.37097": "Janz Computer Ag",
    "24.5915": "Innovation Lab Corp.",
    "24.5912": "Vansco Electronics Oy",
    "24.5913": "Audiocodes USA, Inc",
    "24.5910": "Qno Technology Inc.",
    "24.5911": "Leica Geosystems AG",
    "24.9062": "Beijing Siasun Electronic System Co.,Ltd.",
    "24.9063": "UniControls a.s.",
    "24.9060": "Power Instruments Pte Ltd",
    "24.9061": "Insta Elektro GmbH",
    "24.9066": "SmartRG Inc",
    "24.9067": "Xembedded, Inc.",
    "24.5918": "Theo Benning GmbH & Co. KG",
    "24.5919": "IMV Corporation",
    "12.123917684723712": "Symboticware Incorporated",
    "24.12098090": "Intel Corporate",
    "12.123917675577344": "Airmar Technology Corp",
    "20.233630194466816": "Strive Orthopedics Inc",
    "12.123917686628352": "Xi'An Shenming Electron Technology Co.,Ltd",
    "20.18540140691456": "Sanofi (Beijing) Pharmaceutical Co., Ltd.",
    "12.346853748736": "Mixbaal S.A. de C.V.",
    "24.6042510": "Alpha Networks Inc.",
    "24.7638776": "Brocade Communications Systems LLC",
    "24.13415339": "shenzhen Biocare Bio-Medical Equipment Co.,Ltd.",
    "12.123917675905024": "Talleres de Escoriaza",
    "24.8146008": "Scale Computing, Inc.",
    "12.123917675896832": "Aquavision Distribution Ltd",
    "24.3460756": "Parsec (Pty) Ltd",
    "12.123917686841344": "Medex",
    "12.123917680537600": "Ce Link Limited",
    "24.8695788": "Buffalo.Inc",
    "20.194600859860992": "Vista Manufacturing",
    "24.2412321": "Shenzhen State Micro Technology Co., Ltd",
    "12.123917676580864": "Task Sistemas",
    "24.12113938": "IEEE Registration Authority",
    "24.16261205": "Green Information System",
    "12.346853998592": "WillMonius Inc.",
    "12.123917679775744": "Omega Bilance Srl Societa' Unipersonale",
    "12.123917682692096": "LandmarkTech Systems Technology Co.,Ltd.",
    "24.10524909": "Shandong Intelligent Optical Communication Development Co., Ltd.",
    "24.10795885": "Dell Inc.",
    "20.255191447240704": "BSM Wireless Inc.",
    "12.346863312896": "ELPA sas",
    "24.8339": "Landings Technology Corp.",
    "24.8338": "Chess Engineering B.V.",
    "24.6309844": "FORICS Electronic Technology Co., Ltd.",
    "12.123917686005760": "QUICCO SOUND Corporation",
    "24.8331": "Lapis Technologies, Inc.",
    "24.8330": "Sonix Communications, Ltd.",
    "24.8333": "Cmd Technology",
    "24.8332": "Galaxy Networks, Inc.",
    "24.8335": "ECI Telecom Ltd.",
    "24.8334": "Chevin Software Eng. Ltd.",
    "24.8337": "J125, National Security Agency",
    "24.8336": "Advanced Compression Technology, Inc.",
    "12.123917691224064": "Manvish eTech Pvt. Ltd.",
    "24.13679873": "DS International",
    "24.8952998": "BTB Korea INC",
    "12.123917688643584": "Arctos Showlasertechnik GmbH",
    "12.346860187648": "Guidance Navigation Limited",
    "24.9999433": "Skyworth Wireless Technology Ltd.",
    "12.123917682892800": "RCH Vietnam Limited Liability Company",
    "12.346863353856": "VisiCon GmbH",
    "24.8127308": "Sagemcom Broadband SAS",
    "24.16008979": "Leading Public Performance Co., Ltd.",
    "24.2414443": "Syntronic AB",
    "12.123917679284224": "Idea Spa",
    "24.11565372": "Revolv Inc",
    "12.123917681459200": "Mect Srl",
    "24.7865080": "Xiaomi Communications Co Ltd",
    "12.123917689737216": "Sigma Connectivity AB",
    "12.71297883672576": "DelfiSolutions A/S",
    "24.26137": "Huawei Device Co., Ltd.",
    "12.346862141440": "Atel Corporation",
    "24.316192": "Itel Mobile Limited",
    "24.2366330": "Apple, Inc.",
    "24.3733309": "Amazon Technologies Inc.",
    "24.4727051": "Pegatron Corporation",
    "24.12202068": "George Wilson Industries Ltd",
    "24.524428": "Network Research Corporation",
    "24.524429": "XyVision",
    "24.524426": "PerfTech, Inc.",
    "24.524427": "Pyramid Technology Corp.",
    "24.524424": "Brocade Communications Systems LLC",
    "24.524425": "Kinetics",
    "24.524422": "Imagen/QMS [Now Konica Minolta]",
    "24.524423": "Xyplex",
    "24.524420": "Tomen Electronics Corp.",
    "24.524421": "Elxsi",
    "12.346862612480": "Dave Jones Design",
    "24.15264539": "Microchip Technology Inc.",
    "24.13670398": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.15210027": "IDEO Security Co., Ltd.",
    "24.7923665": "Hewlett Packard",
    "20.114869204418560": "Cndi Co.,Ltd",
    "12.71297883602944": "Oki Seatec Co., Ltd.",
    "12.346857385984": "BES Technology Group",
    "12.346855231488": "4RF Communications Ltd",
    "20.255861680242688": "Miltek Industries Pte Ltd",
    "24.524394": "At&T",
    "24.524395": "Accel Technologies Inc.",
    "24.524396": "Suntek Technology Int'L",
    "24.524397": "Whitechapel Computer Works",
    "24.524390": "AGFA",
    "24.524391": "ComDesign",
    "24.524392": "Ridge Computers",
    "24.524393": "Silicon Graphics",
    "24.198": "HP Intelligent Networks Operation (formerly Eon Systems)",
    "24.199": "Arix Corporation",
    "24.13141049": "Sunrise Technologies",
    "24.524398": "Masscomp",
    "24.524399": "Philips Apeldoorn B.V.",
    "12.71297885143040": "Embedded Technology Corporation",
    "24.5562211": "Intrakey technologies GmbH",
    "24.11563053": "Apple, Inc.",
    "12.346868301824": "Chrisso Technologies LLC",
    "24.12120855": "iSun Smasher Communications Private Limited",
    "24.1432": "CRONOS S.r.l.",
    "24.13110933": "OmniLync Inc.",
    "24.13133405": "Extreme Networks, Inc.",
    "24.11541458": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "24.12906961": "Beijing Sogou Technology Development Co., Ltd.",
    "12.346863857664": "Ateme",
    "12.346867777536": "Rohde&Schwarz Topex SA",
    "24.14460466": "Raspberry Pi Trading Ltd",
    "12.346860335104": "ViewPlus Technologies, Inc.",
    "24.9888": "moblic",
    "24.9889": "Megger",
    "24.9884": "Itus Japan Co. Ltd",
    "24.9885": "M2Mnet Co., Ltd.",
    "24.9886": "Quanta Computer Inc.",
    "12.346861621248": "Link Communications, Inc",
    "24.9880": "Cisco Systems, Inc",
    "24.9881": "Cisco Systems, Inc",
    "24.9882": "Carina System Co., Ltd.",
    "24.9883": "SOKRAT Ltd.",
    "20.216039232110592": "Gc Automation Co,Ltd",
    "24.14736092": "JVC KENWOOD Corporation",
    "24.3207121": "Alstom Strongwish (Shenzhen) Co., Ltd.",
    "12.346865635328": "NDC Infrared Engineering, Inc.",
    "24.10520060": "Microsoft Corporation",
    "24.2682931": "Apple, Inc.",
    "12.123917681238016": "Invisible Systems Limited",
    "24.2921572": "Cincoze Co., Ltd.",
    "20.70682228555776": "MITHRAS Technology Co., LTD",
    "12.123917683544064": "Tattile Srl",
    "24.13398543": "Viscount Systems Inc.",
    "24.816922": "Apple, Inc.",
    "24.7621609": "Explorer Hypertech Co.,Ltd",
    "24.1637940": "Espressif Inc.",
    "20.168210322161664": "Guangdong DE at science and technology co., LTD",
    "12.346854416384": "Ross Video Limited",
    "24.1877298": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.12065028": "Nokia Shanghai Bell Co., Ltd.",
    "24.15006461": "Intel Corporate",
    "20.452713250816": "aversix",
    "24.12093214": "Good Mind Industries Co., Ltd.",
    "20.212014910668800": "ALNETz Co.,LTD",
    "24.5264679": "LG Electronics (Mobile Communications)",
    "24.7113683": "Cisco Systems, Inc",
    "24.10520681": "Intel Corporate",
    "24.11573544": "Fujitsu Limited",
    "12.346856300544": "STL GmbH",
    "20.70682232750080": "Swarco Technology ApS",
    "24.12859744": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "20.132811184930816": "Ihm",
    "12.123917691248640": "Alpha Elettronica s.r.l.",
    "20.273458021793792": "Combiwins Technology Co.,Limited",
    "12.346852773888": "Seatech",
    "24.11594583": "Huawei Technologies Co.,Ltd",
    "12.123917683621888": "Tofino",
    "24.7629455": "VS Vision Systems GmbH",
    "12.123917684760576": "BP Lubricants USA, Inc.",
    "24.9176972": "Flow Data Inc",
    "24.13407344": "Kinestral Technologies, Inc.",
    "24.13677720": "Robert Bosch LLC Automotive Electronics",
    "12.123917686894592": "Shanghai Qinyue Communication Technology Co., Ltd.",
    "12.346854072320": "Shenyang Internet Technology Inc",
    "24.13144569": "Sagemcom Broadband SAS",
    "12.123917683572736": "NELS Ltd.",
    "12.346860707840": "HEWI Heinrich Wilke GmbH",
    "12.123917684805632": "Zulex International Co.,Ltd.",
    "24.9193": "F5 Networks, Inc.",
    "24.13677885": "Sepro Robotique",
    "24.15505330": "SUMEC Machinery & Electric Co.,Ltd.",
    "24.10523080": "zte corporation",
    "12.346859831296": "Brightlights Intellectual Property Ltd",
    "12.123917683765248": "Tsubakimoto Chain Co.",
    "12.123917688958976": "C-COM Satellite Systems Inc.",
    "24.12646618": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9482564": "Samsung Electronics Co.,Ltd",
    "24.9195": "Cisco Systems, Inc",
    "24.9480067": "Jsw Pacific Corporation",
    "24.14200285": "Sonavation, Inc.",
    "12.346861010944": "Logel S.R.L.",
    "24.1843185": "Huawei Device Co., Ltd.",
    "24.10491993": "Shenzhen Yidashi Electronics Co Ltd",
    "24.8916918": "Jabil Circuits India Pvt Ltd,-EHTP unit",
    "24.4258431": "Preh Car Connect GmbH",
    "20.180659912966144": "Private",
    "12.346867081216": "Vocality International Ltd",
    "24.49256": "HME Clear-Com LTD.",
    "24.7605979": "Optowiz Co., Ltd",
    "24.49257": "Axxcelera Broadband Wireless",
    "20.184991249072128": "Exel s.r.l. unipersonale",
    "24.49254": "Docupoint, Inc.",
    "24.7616303": "Tongfang Shenzhen Cloudcomputing Technology Co.,Ltd",
    "24.32968": "D-Link",
    "24.32969": "Alberta Microelectronic Centre",
    "24.1876657": "ruwido austria gmbh",
    "24.32962": "IEEE",
    "24.32963": "Bicc Information Systems & Svc",
    "24.32960": "Penril Datability Networks",
    "24.32961": "Lanex Corporation",
    "24.32966": "National Datacomm Corporation",
    "24.32967": "Xircom, Inc.",
    "24.32964": "Document Technologies, Inc.",
    "24.49253": "Scope Communications, Inc.",
    "24.307617": "Samsung Electronics Co.,Ltd",
    "24.49250": "Impulse Technology",
    "24.7868892": "XIAOMI Electronics,CO.,LTD",
    "12.346860068864": "Level Telecom",
    "24.1609864": "eero inc.",
    "12.346858418176": "Metrologic Group",
    "24.8399528": "Ubiquiti Networks Inc.",
    "24.49251": "Morning Star Technologies Inc",
    "12.346866462720": "GD Mission Systems",
    "12.123917688254464": "Saankhya Labs Private Limited",
    "24.6031687": "Apple, Inc.",
    "24.1882564": "Ruckus Wireless",
    "24.289949": "Cisco Systems, Inc",
    "24.8447296": "ARRIS Group, Inc.",
    "24.6835368": "Shenzhen Herotel Tech. Co., Ltd.",
    "12.123917686583296": "Internet Protocolo Logica Sl",
    "24.6574000": "Hyunjin.com",
    "24.49258": "Zahner-Elektrik Gmbh & Co KG",
    "24.797415": "Beijing General Research Institute of Mining and Metallurgy",
    "20.31873097007104": "Banmak Technogies Co.,Ltd",
    "24.13114219": "Innova Security",
    "24.14429643": "Beijing Unis HengYue Technology Co., Ltd.",
    "12.123917684875264": "Anello Photonics",
    "24.4483243": "Juin Company, Limited",
    "12.346868195328": "Z-App Systems, Inc.",
    "24.13404669": "Nokia Corporation",
    "24.5300423": "TurControlSystme AG",
    "12.346860441600": "Nystrom Engineering",
    "12.123917685370880": "LG Electronics",
    "24.5263461": "TAKT Corporation",
    "24.10502494": "Opsens Solution Inc.",
    "20.52912292102144": "YiSheng technology co.,LTD",
    "24.5282380": "Dell Inc.",
    "20.203238163873792": "Entotem LTD",
    "24.3463092": "Tributary Systems, Inc.",
    "20.4884644495360": "Energport Inc",
    "12.123917680992256": "Ethical Lighting and Sensor Solutions Limited",
    "24.13951807": "Nokia",
    "24.11015858": "LG Electronics (Mobile Communications)",
    "24.10761517": "TRIZ Networks corp.",
    "12.123917680005120": "Moving iMage Technologies LLC",
    "24.1634702": "ChipER Technology co. ltd",
    "24.9447396": "Apple, Inc.",
    "12.123917687406592": "YUYAMA MFG Co.,Ltd",
    "12.346854531072": "Mds Sciex",
    "24.539183": "Guangzhou Juan Intelligent Tech Joint Stock Co.,Ltd",
    "24.11296905": "Huawei Technologies Co.,Ltd",
    "24.6099": "Etymotic Research, Inc.",
    "24.4765918": "Homewins Technology Co.,Ltd.",
    "24.7388176": "Microsoft Corporation",
    "24.12339798": "Cisco Systems, Inc",
    "24.41155": "Unicomputer Gmbh",
    "24.41154": "R.A. Systems Co., Ltd.",
    "24.41157": "Zyxel Communications Corporation",
    "24.41156": "Cristie Electronics Ltd.",
    "24.41151": "Wireless Data Group Motorola",
    "24.41150": "Integrated Circuit Systems, Inc. Communications Group",
    "24.41153": "Ortivus Medical Ab",
    "24.41152": "Digital Link Corp.",
    "20.70559703498752": "Shanghai Dahua Scale Factory",
    "24.41159": "Tadiran Telecommunications",
    "24.41158": "Qualcomm Inc.",
    "24.14464196": "Microsoft XCG",
    "24.12623621": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.229108123435008": "Atoll Solutions Private Limited",
    "20.84187710226432": "Hangzhou Hangtu Technology Co.,Ltd.",
    "20.22126501298176": "Byzero",
    "24.12095174": "Stateless Networks",
    "20.58144478724096": "Tianjin Sublue Ocean Science & Technology Co., Ltd",
    "24.1292": "Network Photonics, Inc.",
    "24.9742268": "BodyMedia, Inc.",
    "24.15488781": "AFE Inc.",
    "20.180394586537984": "AlterG, Inc.",
    "24.11322773": "Graphite Systems",
    "24.9742263": "zte corporation",
    "20.184991240683520": "Guangzhou Tupu Internet Technology Co., Ltd.",
    "12.346856480768": "Videotek Sistemas Eletronicos Ltda.",
    "24.5558509": "AXIM Communications",
    "20.27478313664512": "Cabtronix AG",
    "24.9237032": "Mercury Communication Technologies Co.,Ltd.",
    "24.14200705": "ZAO \"NPK Rotek\"",
    "12.346854141952": "Bluesocket, Inc.",
    "24.10767917": "Adaptive Wireless Solutions, LLC",
    "12.123917689143296": "D.T.S Illuminazione Srl",
    "20.62663546634240": "Rademacher Geraete-Elektronik GmbH",
    "24.299137": "Pensando Systems, Inc.",
    "12.123917681651712": "ID Quantique SA",
    "12.346864242688": "SilverNet",
    "24.14209660": "Jema Energy, Sa",
    "24.13409994": "HMD Global Oy",
    "12.123917678444544": "BÄR Bahnsicherung AG",
    "12.123917684617216": "ENTEC Electric & Electronic Co., LTD.",
    "24.9476682": "Tp-Link Technologies Co.,Ltd.",
    "24.5254429": "Nokia Corporation",
    "20.194363554529280": "TEM Mobile Limited",
    "12.123917685157888": "Shanghai Brellet Telecommunication Technology Co., Ltd.",
    "24.3184379": "Samsung Electronics Co.,Ltd",
    "24.13934634": "Cisco Systems, Inc",
    "24.1355210": "China Mobile Iot Limited company",
    "24.8951374": "ARRIS Group, Inc.",
    "24.8577": "Si2 Microsystems Limited",
    "12.123917677776896": "Taitotekniikka",
    "12.123917683515392": "Novathings",
    "12.346857041920": "iseg Spezialelektronik GmbH",
    "12.346857287680": "Romteck Pty Ltd",
    "12.346853453824": "Vaisala Oyj",
    "24.4210795": "Icomera",
    "12.123917678567424": "Solarlytics, Inc.",
    "24.6828844": "Kenstel Networks Limited",
    "12.123917678862336": "Exicom Technologies fze",
    "12.123917684568064": "JSC \"InformInvestGroup\"",
    "12.346857725952": "Convergent Design",
    "24.5285946": "S Mobile Devices Limited",
    "24.2131948": "Huawei Technologies Co.,Ltd",
    "24.10236622": "Canon Inc.",
    "12.346862637056": "Bitt technology-A Ltd.",
    "12.346862338048": "Eukrea Electromatique Sarl",
    "24.16489": "Lemcom Systems, Inc.",
    "24.16488": "Extended Systems",
    "12.346865983488": "RS Gesellschaft fur Informationstechnik mbH & Co KG",
    "12.346856349696": "HaiVision Systems Inc",
    "24.16483": "Via Technologies, Inc.",
    "24.16482": "E-Systems, Inc./Garland Div.",
    "24.16481": "Datatech Enterprises Co., Ltd.",
    "24.16480": "Comendec Ltd",
    "24.16487": "Omnibyte Corporation",
    "24.16486": "APRESIA Systems Ltd",
    "24.16485": "Gte Spacenet",
    "12.346858426368": "ROTHARY Solutions AG",
    "24.8685299": "Greenvity Communications",
    "24.5827821": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.8390111": "Montage Technology Group Limited",
    "12.123917690302464": "Toptech Systems, Inc.",
    "20.93065821814784": "Shenzhen C & D Electronics Co., Ltd.",
    "24.14956027": "Sabre Technology (Hull) Ltd",
    "24.8427185": "ARRIS Group, Inc.",
    "12.346855833600": "EKE-Electronics Ltd.",
    "12.123917679677440": "Ewattch",
    "24.14437022": "IEEE Registration Authority",
    "20.242876967550976": "WECAN Solution Inc.",
    "24.33003": "Compcontrol B.V.",
    "12.346865090560": "HPC Platform",
    "24.12854138": "WhizNets Inc.",
    "20.172584084701184": "Ithor IT Co.,Ltd.",
    "24.8682475": "Dell Inc.",
    "24.4484": "Humo Laboratory,Ltd.",
    "24.15519447": "Apple, Inc.",
    "24.14179810": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.583759": "Huawei Technologies Co.,Ltd",
    "12.346864062464": "SLICAN sp. z o.o.",
    "24.11556104": "Huawei Technologies Co.,Ltd",
    "24.4741540": "zte corporation",
    "24.16295738": "Cisco Systems, Inc",
    "12.123917683286016": "11811347 CANADA Inc.",
    "24.6953": "Avantis.Co.,Ltd",
    "24.6952": "Polygon, Jsc",
    "24.6951": "Merlin CSI",
    "24.6950": "RON-Telecom ZAO",
    "24.6957": "Med-Eng Systems Inc.",
    "24.6956": "ATRON electronic GmbH",
    "24.6955": "Cisco Systems, Inc",
    "24.6954": "Cisco Systems, Inc",
    "24.6959": "Netgear",
    "24.6958": "Sinkyo Electron Inc",
    "24.10234035": "Apple, Inc.",
    "20.14223579021312": "Firmware Design AS",
    "20.216602656112640": "Shenzhen C & D Electronics Co., Ltd.",
    "24.3410584": "Huawei Technologies Co.,Ltd",
    "24.1354582": "Wuxi Funide Digital Co.,Ltd",
    "12.123917681631232": "Robert Bosch, LLC",
    "12.123917677404160": "Unmukti Technology Pvt Ltd",
    "24.12105758": "Intel Corporate",
    "24.7659051": "Liteon Technology Corporation",
    "12.123917676335104": "Sicon srl",
    "12.123917684879360": "FOCAL-JMLab",
    "24.9228175": "ITC Systems",
    "20.264809148841984": "ADAM Audio GmbH",
    "24.12494": "Zaffire",
    "24.12495": "Two Technologies, Inc.",
    "24.12496": "Tellabs",
    "24.12497": "Inova Corporation",
    "24.12490": "Discovery Com",
    "24.12491": "Omni Flow Computers, Inc.",
    "24.12492": "Tenor Networks, Inc.",
    "24.12493": "Conexant Systems, Inc.",
    "12.123917685243904": "aelettronica group srl",
    "24.3454371": "Cig Shanghai Co Ltd",
    "24.12498": "Win Technologies, Co., Ltd.",
    "24.12499": "Agilent Technologies, Inc.",
    "24.8448111": "Cisco Systems, Inc",
    "24.11550384": "MICRODIA Ltd.",
    "24.1577490": "Cepton Technologies",
    "24.7638844": "Apple, Inc.",
    "12.123917687869440": "Stiebel Eltron GmbH",
    "12.123917687263232": "Lumiplan Duhamel",
    "24.2936877": "Cisco Systems, Inc",
    "12.123917676679168": "Smart Controls LLC",
    "24.15496685": "Citrix Systems, Inc.",
    "24.542764": "Raycore Taiwan Co., LTD.",
    "24.5798": "Dovado FZ-LLC",
    "24.5799": "Aweta G&P",
    "24.8659094": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.5794": "CentraLite Systems, Inc.",
    "24.5795": "Ingeteam Transmission&Distribution, S.A.",
    "24.5796": "Ezurio Ltd",
    "24.5797": "Tandberg Storage ASA",
    "24.5790": "TV One Ltd",
    "24.5791": "Vimtron Electronics Co., Ltd.",
    "24.5792": "Auto-Maskin",
    "24.5793": "3Leaf Networks",
    "24.3680490": "Fujian Netcom Technology Co., LTD",
    "12.123917681463296": "Ozray",
    "12.346858123264": "Dorlet Sau",
    "24.15526403": "Darfon Lighting Corp",
    "24.4470127": "Neterix Ltd",
    "24.281196": "Huawei Technologies Co.,Ltd",
    "24.15232357": "AUDIO MOBIL Elektronik GmbH",
    "24.9978043": "Dell Inc.",
    "12.346864484352": "Galaxia Electronics",
    "20.168198559236096": "GoodBox",
    "20.119143418298368": "Vertiv Industrial Systems",
    "12.346866237440": "ALPHA Corporation",
    "24.1048666": "IBM Corp",
    "24.6338273": "Texas Instruments",
    "24.8170336": "Apple, Inc.",
    "24.8657758": "Netgear",
    "24.5021439": "Zyxel Communications Corporation",
    "24.2927397": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.16557773": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917676515328": "Aetina Corporation",
    "24.5805597": "Intel Corporate",
    "12.346857582592": "Sicon srl",
    "24.7885810": "Roxton Systems Ltd.",
    "24.3828": "Kasda Networks Inc",
    "24.3829": "iPAC Technology Co., Ltd.",
    "24.532975": "Samsung Electronics Co.,Ltd",
    "24.14180051": "Giga-Byte Technology Co.,Ltd.",
    "24.3824": "Festo AG & Co. KG",
    "20.31368100708352": "Surtec Industries, Inc",
    "24.3826": "Infinico Corporation",
    "24.3827": "Smartlabs, Inc.",
    "24.3820": "Orban",
    "24.3821": "Nokia Danmark A/S",
    "24.3822": "Muco Industrie BV",
    "24.3823": "Private",
    "24.6087030": "Cisco Systems, Inc",
    "24.291477": "Apple, Inc.",
    "20.127658782359552": "Blocks Wearables Inc.",
    "24.10806961": "Shanghai Joindata Technology Co.,Ltd.",
    "24.14993675": "Radwin",
    "24.4449": "NetStreams, LLC",
    "24.4448": "ARTDIO Company Co., LTD",
    "12.346863022080": "Phytec Messtechnik GmbH",
    "24.4443": "Elitegroup Computer Systems Co.,Ltd.",
    "24.4442": "Ivoclar Vivadent AG",
    "12.346853040128": "Inco Startec GmbH",
    "24.4440": "Nortel Networks",
    "24.4447": "ITX Security Co., Ltd.",
    "24.4446": "ProMinent Dosiertechnik GmbH",
    "24.4445": "Cisco Systems, Inc",
    "24.4444": "Cisco Systems, Inc",
    "24.3796": "Cresitt Industrie",
    "24.3797": "COPAN Systems Inc.",
    "24.3794": "Filtronic plc",
    "24.3795": "Epicenter, Inc.",
    "24.3792": "Privaris, Inc.",
    "24.3793": "Osaka Micro Computer.",
    "24.3790": "S.I.T.T.I. S.p.A.",
    "24.3791": "PROFIBUS Nutzerorganisation e.V.",
    "12.71297883594752": "TeraTron GmbH",
    "24.3798": "Cisco Systems, Inc",
    "24.3799": "Cisco Systems, Inc",
    "12.346861998080": "Picsolve International Limited",
    "12.346864988160": "ELEKTRO-AUTOMATIK GmbH & Co. KG",
    "12.123917689356288": "Arnouse Digital Devices Corp",
    "24.2119457": "Salix Technology CO., Ltd.",
    "24.6702": "Ziova Coporation",
    "20.27056767238144": "Dectris Ltd.",
    "24.10775478": "Intel Corporate",
    "24.8154534": "Superwave Group LLC",
    "24.10797368": "Telink Semiconductor (Taipei) Co. Ltd.",
    "12.119269769216": "Applied Innovations Research LLC",
    "24.13667583": "Upwis Ab",
    "24.6701": "The Navvo Group",
    "24.7239": "Hangzhou Hollysys Automation Co., Ltd",
    "24.7238": "Qtum",
    "24.7235": "Samsung Electronics Co.,Ltd",
    "24.7234": "Parallels, Inc.",
    "24.7237": "Chenbro Micom Co., Ltd.",
    "24.7236": "Bosch Security Systems BV",
    "24.7231": "International Police Technologies, Inc.",
    "24.7230": "ECKey Corporation",
    "24.7233": "scemtec Transponder Technology GmbH",
    "24.7232": "VDG-Security bv",
    "24.6704": "Cisco Systems, Inc",
    "12.123917686673408": "Tattile Srl",
    "24.6705": "Scan Coin Ab",
    "24.7632920": "Jurumani Solutions",
    "24.9729835": "Huawei Technologies Co.,Ltd",
    "24.3688524": "Apple, Inc.",
    "24.12870675": "Cisco Systems, Inc",
    "12.346858455040": "Protec Fire Detection plc",
    "12.123917680431104": "Tanaka Electric Industry Co., Ltd.",
    "24.5518815": "Symeo GmbH",
    "24.6708": "Konka Group Co., Ltd.",
    "24.49322": "Silicon Valley Computer",
    "24.49323": "Telco Systems, Inc.",
    "24.49320": "Gvc Corporation",
    "24.49321": "Barron Mccann Ltd.",
    "24.49326": "Towercom Co. Inc. Dba Pc House",
    "24.49327": "Teklogix Inc.",
    "24.49324": "Gambit Computer Communications",
    "24.49325": "Marben Communication Systems",
    "24.49328": "Gcc Technologies,Inc.",
    "24.49329": "Genius Net Co.",
    "24.8933622": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "24.8919": "Nintendo Co., Ltd.",
    "24.8918": "Cypak AB",
    "24.1086359": "ARRIS Group, Inc.",
    "24.8910": "Cisco SPVTG",
    "24.8913": "Albrecht Jung GmbH & Co. KG",
    "24.8912": "Polar Electro Oy",
    "24.8915": "Hub-Tech",
    "24.8914": "All Earth Comércio de Eletrônicos LTDA.",
    "24.8917": "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi",
    "24.8916": "ComWorth Co., Ltd.",
    "12.123917676752896": "Access Control Systems JSC",
    "12.123917691420672": "Munters",
    "24.2139776": "Huawei Technologies Co.,Ltd",
    "24.7491": "Shenzhen G-link Digital Technology Co., Ltd.",
    "24.7490": "Nortel Networks",
    "24.7493": "Cisco Systems, Inc",
    "24.7492": "Krohne",
    "24.7495": "Covote GmbH & Co KG",
    "24.7494": "Cisco Systems, Inc",
    "24.7497": "Innovation Wireless Inc.",
    "24.7496": "Sensor-Technik Wiedemann GmbH",
    "24.7499": "Grid Connect Inc.",
    "24.7498": "Carestream Health, Inc.",
    "24.11536851": "Avizia",
    "20.272689500520448": "Dueton Systems s.r.o.",
    "24.1884604": "Yichip Microelectronics (Hangzhou) Co.,Ltd",
    "20.39672893931520": "Shenzhen Xtooltech Co., Ltd",
    "12.346856714240": "Danbridge",
    "24.2049": "HighSpeed Surfing Inc.",
    "24.2048": "Multitech Systems, Inc.",
    "24.11288484": "Taiyo Yuden Co.,Ltd",
    "24.2045": "LANergy Ltd.",
    "24.2044": "Adept Systems Inc.",
    "24.2047": "Gluon Networks",
    "24.2046": "Rigaku Corporation",
    "24.2041": "Sensaphone",
    "24.2040": "ITDevices, Inc.",
    "24.2043": "Giga Stream UMTS Technologies GmbH",
    "24.2042": "ITT Co., Ltd.",
    "24.37002": "Bayly Communications, Inc.",
    "12.346863587328": "ABS Gesellschaft f. Automatisierung, Bildverarbeitung und Software mbH",
    "24.9233903": "Huawei Technologies Co.,Ltd",
    "24.7646099": "Blink by Amazon",
    "12.123917679185920": "NaraControls Inc",
    "12.71297885057024": "MG S.r.l.",
    "24.16551534": "StreamCCTV, LLC",
    "24.11803947": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.3168172": "Irlab Ltd.",
    "12.123917688111104": "Link Care Services",
    "20.57353106882560": "Wiesheu GmbH",
    "24.24454": "Cisco Systems, Inc",
    "24.6081696": "Gridwiz Inc.",
    "24.14718055": "Intel Corporate",
    "12.123917682044928": "Grossenbacher Systeme AG",
    "24.7891412": "Shenyang Vibrotech Instruments Inc.",
    "12.123917675438080": "Guangdong Centnet Technology Co.,Ltd",
    "24.14992081": "Huawei Technologies Co.,Ltd",
    "12.123917680062464": "Hefei STAROT Technology Co.,Ltd",
    "24.12462": "Times N System, Inc.",
    "24.5556239": "Tp-Link Technologies Co.,Ltd.",
    "24.7618251": "Gentrice tech",
    "12.119269974016": "Veilux Inc.",
    "24.9991862": "Adafruit",
    "20.62663553974272": "iSmart electronic technology co.,LTD",
    "24.1369324": "mLogic LLC",
    "24.16569586": "The Coca Cola Company",
    "24.3408518": "Intel Corporate",
    "24.37110": "Escalate Networks, Inc.",
    "24.37111": "Nbase Communications Ltd.",
    "24.408": "Darim Vision",
    "24.37113": "Imagine Communications",
    "24.37114": "Emulex Corporation",
    "24.37115": "Portwell, Inc.",
    "24.37116": "Network Computing Devices",
    "24.9748": "Ktnf",
    "24.37118": "Elecom Co., Ltd. (Laneed Div.)",
    "24.37119": "Tellus Technology Inc.",
    "24.400": "Smk-M",
    "24.401": "SYRED Data Systems",
    "24.406": "Cisco Systems, Inc",
    "24.407": "Cisco Systems, Inc",
    "24.9741": "Mercury Systems, Inc.",
    "24.405": "Sena Technologies, Inc.",
    "24.8422569": "oshkosh Corporation",
    "24.12599253": "Elitegroup Computer Systems Co.,Ltd.",
    "24.5899": "Contela, Inc.",
    "24.5898": "Inew Digital Company",
    "24.1538": "Cirkitech Electronics Co.",
    "24.1539": "Baker Hughes Inc.",
    "24.12083522": "Beijing Tricolor Technology Co., Ltd",
    "24.1534": "Traficon N.V.",
    "24.5892": "Shinco Electronics Group Co.,Ltd",
    "24.1536": "Toshiba Teli Corporation",
    "24.5890": "Osung Midicom Co., Ltd",
    "24.5897": "Exalt Communications",
    "24.5896": "Hewlett Packard",
    "24.5895": "InGrid, Inc",
    "24.5894": "Techfaithwireless Communication Technology Limited.",
    "20.84187700789248": "Shenzhen Dangs Science & Technology CO.,LTD",
    "20.176032226213888": "Szbroad Technology (Hk) Co.,Ltmited",
    "24.8429557": "Samsung Electronics Co.,Ltd",
    "24.1874252": "Murata Manufacturing Co., Ltd.",
    "24.14194309": "GoPro",
    "24.2632706": "IEEE Registration Authority",
    "24.11277522": "wi-daq, inc.",
    "20.180698116784128": "Stone Lock Global, Inc.",
    "12.346860625920": "Femto SA",
    "24.3191714": "Shenzhen Heguang Measurement & Control Technology Co.,Ltd",
    "24.9298": "Silicon Software GmbH",
    "24.4237552": "Hewlett Packard",
    "24.9291": "Perceptron Inc",
    "24.9290": "Voyant International",
    "24.9293": "Hokkaido Electronics Corporation",
    "12.346852876288": "Zarak Systems Corp.",
    "24.9295": "Asantron Technologies Ltd.",
    "24.9294": "RadChips, Inc.",
    "24.9297": "Cisco Systems, Inc",
    "24.9296": "Cisco Systems, Inc",
    "12.346858053632": "Legerity",
    "24.13387342": "Samsung Electronics Co.,Ltd",
    "20.93065824960512": "RED Hydrogen LLC",
    "20.229558877945856": "Innovative Industrial(HK)Co., Limited",
    "24.24613": "Active Imaging Inc.",
    "24.4195892": "Huawei Device Co., Ltd.",
    "20.247076400136192": "Start Today Co.,Ltd.",
    "24.14429955": "Step forward Group Co., Ltd.",
    "12.346852200448": "Datakinetics Ltd.",
    "24.24612": "Gradient Technologies, Inc.",
    "24.528040": "Velex s.r.l.",
    "24.24611": "Pericom Semiconductor Corp.",
    "24.24610": "Vicom Systems, Inc.",
    "20.132811181785088": "Huatune Technology (Shanghai) Co., Ltd.",
    "24.24617": "Cary Peripherals Inc.",
    "24.24616": "Macrovision Corporation",
    "12.123917680750592": "ELAMAKATO GmbH",
    "20.52819405045760": "ZhongLi HengFeng (Shenzhen) Technology co.,Ltd.",
    "24.24615": "Superior Modular Products",
    "24.1608605": "Cisco Systems, Inc",
    "24.24614": "VIKING Modular Solutions",
    "24.12633449": "Shanghai Lynuc Cnc Technology Co.,Ltd",
    "24.5264995": "Texas Instruments",
    "20.268341254029312": "Axel srl",
    "24.5818096": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346862116864": "RF Industries",
    "24.12110633": "Dell Inc.",
    "24.13926871": "Dell Inc.",
    "12.123917690286080": "ECON Technology Co.Ltd",
    "12.346865741824": "Daiken Automacao Ltda",
    "24.1101545": "Apple, Inc.",
    "12.346852724736": "Fenwal Italia S.P.A.",
    "24.3437402": "Dell Inc.",
    "24.14985033": "Palo Alto Networks",
    "24.11562687": "ASUSTek COMPUTER INC.",
    "24.7101754": "Microsoft Corporation",
    "24.8660388": "Tariox Limited",
    "24.2642766": "Huawei Technologies Co.,Ltd",
    "24.6093277": "Dell Inc.",
    "12.346865201152": "MSB Elektronik und Geraetebau GmbH",
    "24.9176319": "Technicolor CH USA Inc.",
    "24.1583201": "AirTies Wireless Networks",
    "24.8168": "KURUSUGAWA Electronics Industry Inc,.",
    "24.8169": "Printrex, Inc.",
    "24.4856": "WNI Resources, LLC",
    "24.4857": "Uryu Seisaku, Ltd.",
    "24.4850": "Brocade Communications Systems LLC",
    "24.4851": "connectBlue AB",
    "24.4852": "Belco International Co.,Ltd.",
    "24.4853": "Imarda New Zealand Limited",
    "24.8160": "EdgeVelocity Corp",
    "24.8161": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8162": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8163": "LG Electronics (Mobile Communications)",
    "24.8164": "Sony Mobile Communications Inc",
    "24.8165": "In-Circuit GmbH",
    "24.8166": "Alphion Corporation",
    "24.8167": "Simet",
    "24.1049034": "Ashley Butterworth",
    "24.5279564": "Xiaomi Communications Co Ltd",
    "24.16282530": "Avaya Inc",
    "24.9463575": "Alphion India Private Limited",
    "24.1317680": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "24.9966802": "Avonic",
    "20.251330460385280": "Shanghai Tieda Telecommunications Equipment Co.,LTD.",
    "24.2154303": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "12.123917676216320": "Akse srl",
    "12.346865889280": "CUE, a.s.",
    "12.346855399424": "Dataline AB",
    "24.2635539": "Shenzhen 3Nod Digital Technology Co., Ltd.",
    "24.4763693": "NVIDIA Corporation",
    "24.12100576": "BenRui Technology Co.,Ltd",
    "24.13403978": "Nokia",
    "20.45069250854912": "Poket Hardware GmbH",
    "12.346854436864": "Riegl Laser Measurement Systems GmbH",
    "24.1625950": "Liteon Technology Corporation",
    "12.123917675286528": "Navaero Avionics AB",
    "12.123917689139200": "Peter Huber Kaeltemaschinenbau AG",
    "12.123917683617792": "HiDes, Inc.",
    "12.346868547584": "Amplus Communication Pte Ltd",
    "24.13150805": "Hunan Comtom Electronic Incorporated Co.,Ltd",
    "20.89058725330944": "Sichuan Dowlab Electronics Technology Co. Ltd",
    "24.2151871": "Samsung Electronics Co.,Ltd",
    "24.7397415": "Hongyu Communication Technology Limited",
    "24.8660523": "Nokia",
    "12.346861813760": "Xilar Corp.",
    "24.4729316": "ZAO \"NPK Rotek\"",
    "20.111031378509824": "ChengDu KeChuang LongXin Sci-tech Co.,Ltd",
    "12.123917676851200": "Shanghai Tiancheng Communication Technology Corporation",
    "24.10786747": "Hitachi Industrial Equipment Systems Co.,Ltd",
    "24.6838991": "Samsung Electronics Co.,Ltd",
    "12.346867335168": "Task Sistemas De Computacao Ltda",
    "24.5793562": "Barnes&Noble",
    "24.3980285": "Samsung Electronics Co.,Ltd",
    "24.14420054": "Alcatel-Lucent Enterprise",
    "20.136646473285632": "Shenzhen Eunicum Electric Co.,Ltd.",
    "20.194284238143488": "Blake UK",
    "24.12114673": "Strahlenschutz- Entwicklungs- und Ausruestungsgesellschaft mbH",
    "12.123917676093440": "Season Design Technology",
    "12.346858627072": "Prisma Engineering srl",
    "12.346853531648": "Corinex Global",
    "12.123917688221696": "Tieto Sweden AB",
    "24.308326": "BSP Co., Ltd.",
    "24.11813242": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.6055054": "Zyxel Communications Corporation",
    "24.3161773": "Sonavox Canada Inc",
    "24.14424773": "vivo Mobile Communication Co., Ltd.",
    "24.6066782": "Zioncom Electronics (Shenzhen) Ltd.",
    "24.1602606": "Amazon Technologies Inc.",
    "24.10533357": "Skytap",
    "12.346862886912": "Telecor Inc.",
    "24.8920572": "Cisco Systems, Inc",
    "20.242185279635456": "Neusoft Corporation",
    "24.8947394": "STABILO International GmbH",
    "24.3175574": "Huawei Technologies Co.,Ltd",
    "24.4517378": "MTI Ltd.",
    "12.71297884569600": "iart",
    "24.4232536": "Aisino Corporation",
    "24.60373": "Cisco Systems, Inc",
    "12.71297884262400": "Rosslare Enterprises Limited",
    "12.123917683331072": "AeroVision Avionics, Inc.",
    "12.123917681344512": "IST ElektronikgesmbH",
    "24.311387": "Tigo Energy",
    "12.346856050688": "NEC TOKIN Corporation",
    "24.7105747": "Cisco Systems, Inc",
    "24.7878048": "Zurn Industries LLC",
    "24.1879813": "Samsung Electronics Co.,Ltd",
    "24.7351069": "E-Domus International Limited",
    "24.3246": "Ailocom Oy",
    "12.346862972928": "Deuta GmbH",
    "24.3247": "Tri Term Co.,Ltd.",
    "24.14158069": "Arcadia Networks Co. Ltd.",
    "24.9101": "Techno Design Co., Ltd.",
    "24.9100": "Private",
    "12.346855538688": "Ernitec A/S",
    "24.9192501": "Shanghai BroadMobi Communication Technology Co., Ltd.",
    "12.71297884700672": "EcoGuard AB",
    "24.7369276": "Huawei Technologies Co.,Ltd",
    "24.13395373": "ARRIS Group, Inc.",
    "24.6337476": "Elber Srl",
    "24.62091": "Cisco Systems, Inc",
    "20.150399886557184": "Origins Technology Limited",
    "24.14696265": "ASUSTek COMPUTER INC.",
    "24.9109": "ARRIS Group, Inc.",
    "24.9721038": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.9108": "Samjeon",
    "24.12330563": "Elprotronic Inc.",
    "24.823295": "Huawei Technologies Co.,Ltd",
    "24.16308959": "EcoTech, Inc.",
    "24.12097783": "Gionee Communication Equipment Co,Ltd.ShenZhen",
    "24.4501163": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.14733101": "Gemintek Corporation",
    "12.123917682601984": "LLC Open Converged Networks",
    "24.6827484": "Wuhan Changjiang Electro-Communication Equipment CO.,LTD",
    "20.220515718397952": "German Pipe GmbH",
    "24.13908713": "DONGGUAN ipt INDUSTRIAL CO., LTD",
    "12.123917678403584": "Dynamic Connect (Suzhou) Hi-Tech Electronic Co.,Ltd.",
    "12.123917690277888": "NAS Australia P/L",
    "12.346857709568": "IPTC Tech. Comm. AB",
    "24.11320949": "Ufine Technologies Co.,Ltd.",
    "24.15267620": "Hewlett Packard Enterprise",
    "12.346853711872": "WOLF Advanced Technology.",
    "12.346853146624": "VHB Technologies, Inc.",
    "12.123917675356160": "Mirae Information Technology Co., Ltd.",
    "24.13645637": "Tp-Link Technologies Co.,Ltd.",
    "24.16282346": "Cisco Systems, Inc",
    "24.10502726": "IT Telecom Co., Ltd.",
    "24.5785231": "Koncar Electronics and Informatics",
    "24.11321014": "Visualedge Technology Co., Ltd.",
    "24.4766071": "PulseOn Oy",
    "24.7896563": "Sterlite Technologies Limited",
    "24.4460994": "Telegartner Karl Gartner GmbH",
    "24.9194222": "Giga Tms Inc",
    "20.264809149890560": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "24.6074719": "Oceanblue Cloud Technology Limited",
    "12.346858971136": "SAIA Burgess Controls AG",
    "24.1320053": "4DReplay, Inc",
    "12.346862407680": "Scansonic MI GmbH",
    "12.346865381376": "Sm electronic co.",
    "24.8674066": "Huawei Technologies Co.,Ltd",
    "24.5029263": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917685694464": "Toughdog Security Systems",
    "12.123917685264384": "PK Sound",
    "20.66256290971648": "Compal Electronics INC.",
    "24.1873498": "Weifang Goertek Electronics Co.,Ltd",
    "24.8397429": "Beijing Beny Wave Technology Co Ltd",
    "24.5564156": "Tp-Link Technologies Co.,Ltd.",
    "24.8453104": "Quectel Wireless Solutions Co., Ltd.",
    "20.127654883753984": "Marmitek",
    "24.3713105": "Moog, Ing",
    "24.3673930": "Apple, Inc.",
    "24.14461297": "Intel Corporate",
    "24.6823563": "InfoSight Corporation",
    "24.6035232": "Huawei Device Co., Ltd.",
    "24.1874113": "Cloud Network Technology Singapore Pte. Ltd.",
    "24.1873725": "PRICOM Design",
    "24.7911450": "Inter Sales A/S",
    "24.14196467": "Dell Inc.",
    "20.136823637540864": "Canary Connect, Inc.",
    "24.5516522": "Protectron",
    "12.346852311040": "Ipitek",
    "12.346858979328": "Honey Network Research Limited",
    "24.15261326": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.268341263466496": "Bradar Industria Sa",
    "12.71297883504640": "ATG UV Technology",
    "24.6820865": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346862034944": "Bittium Wireless Ltd",
    "24.10542607": "Huawei Device Co., Ltd.",
    "24.3946983": "IEEE Registration Authority",
    "24.15741956": "TCT mobile ltd",
    "12.71297883549696": "Data Flow Systems, Inc.",
    "24.15250174": "HUMAX Co., Ltd.",
    "24.10549728": "Vivatel Company Limited",
    "20.62366875123712": "Sirin Mobile Technologies",
    "24.10765931": "EOC Technology",
    "24.11322092": "Clt Int'L Ind. Corp.",
    "24.15003389": "MR&D Manufacturing",
    "12.123917678927872": "MyDefence Communication ApS",
    "12.119269781504": "Onnet Technologies and Innovations LLC",
    "24.12869080": "HDMI Forum",
    "24.15230660": "Diamanti",
    "24.8697244": "Juniper Networks",
    "24.5009308": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.12603774": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "20.58144489209856": "OrCam Technologies",
    "24.41050": "Kofax Image Products",
    "24.41051": "Marquip, Inc.",
    "24.41052": "Inventory Conversion, Inc./",
    "24.41053": "CS COMPUTER SYSTEME GmbH",
    "24.41054": "Myriad Logic Inc.",
    "24.41055": "BTG Electronics Design BV",
    "24.41056": "Acer Peripherals, Inc.",
    "24.41057": "Puritan Bennett",
    "12.346868486144": "RE2 Inc",
    "24.41059": "Jrl Systems, Inc.",
    "24.7365102": "Sunwoda Electronic Co.,Ltd",
    "24.12342326": "Apple, Inc.",
    "24.14206920": "Aruba, a Hewlett Packard Enterprise Company",
    "12.123917682638848": "Kdt",
    "24.11829883": "AKG Acoustics GmbH",
    "24.15475328": "D-Link International",
    "12.346863321088": "PSi Printer Systems international GmbH",
    "24.2912767": "Amazon Technologies Inc.",
    "24.8393439": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24.9499397": "HUMAX Co., Ltd.",
    "24.1344895": "Intel Corporate",
    "12.119269691392": "Center for E-Commerce Infrastructure Development, The University of Hong Kong",
    "24.46704": "Cisco Systems, Inc",
    "12.71297884995584": "KbDevice,Inc.",
    "24.7876375": "Xlab Co.,Ltd",
    "24.8683668": "Meter, Inc.",
    "20.158667114545152": "Spirtech",
    "24.839123": "Eastriver Technology Co., Ltd.",
    "24.16049879": "Solar Power Technologies, Inc.",
    "12.71297883996160": "Mettler Toledo",
    "12.123917684367360": "Network Additions",
    "24.5243323": "Samsung Electronics Co.,Ltd",
    "12.123917686480896": "True Networks Ltd.",
    "24.7359653": "Fortinet, Inc.",
    "24.8707511": "Cilag GmbH International",
    "24.7640556": "Alpsalpine Co,.Ltd",
    "24.7969": "Inner Mongolia Yin An Science & Technology Development Co.,L",
    "24.7968": "Logitech Europe SA",
    "24.1105078": "IEEE Registration Authority",
    "24.10019033": "Apple, Inc.",
    "20.105645771587584": "Annapurna labs",
    "24.7961": "Ben-Ri Electronica S.A.",
    "24.7960": "Hakusan.Mfg.Co,.Ltd",
    "24.7963": "RoyalTek Company Ltd.",
    "24.7962": "Prominvest",
    "24.7965": "Atlas Material Testing Technology LLC",
    "24.7964": "KOBISHI ELECTRIC Co.,Ltd.",
    "24.7967": "Edimax Technology Co. Ltd.",
    "24.7966": "Astec Technology Co., Ltd",
    "20.45069243514880": "Digital Signal Corp",
    "20.67047987871744": "Energous Corporation",
    "24.9493098": "FOCUS H&S Co., Ltd.",
    "24.5827324": "ELL-IoT Inc",
    "24.8677777": "Nokia",
    "24.1899112": "Tp-Link Technologies Co.,Ltd.",
    "12.123917678977024": "TTI Ltd",
    "12.71297883795456": "Burton Technical Services LLC",
    "20.220515730980864": "Phoenix Engineering Corp.",
    "24.12877310": "A.N. Solutions GmbH",
    "20.277958192267264": "Qctek Co.,Ltd.",
    "12.346868494336": "Baudisch Electronic GmbH",
    "24.1355397": "IEEE Registration Authority",
    "24.11027351": "Innogrit Corporation",
    "24.12639450": "Samsung Electronics Co.,Ltd",
    "24.15231207": "Espressif Inc.",
    "24.8651986": "Kirale Technologies SL",
    "24.6062503": "Network Devices Pty Ltd",
    "24.2413852": "Zhongtian broadband technology co., LTD",
    "24.4512913": "Planex Communications Inc.",
    "24.6873053": "Xiaomi Communications Co Ltd",
    "20.70443722604544": "Fujian Kuke3D Technology Co.,LTD",
    "24.9474404": "ChongQing Lavid Technology Co., Ltd.",
    "20.132777847554048": "Guangzhou Hongcai Stage Equipment co.,ltd",
    "20.128612179116032": "WiFi Hotspots, SL",
    "24.14421084": "Apple, Inc.",
    "12.123917687975936": "Clean-Lasersysteme GmbH",
    "20.57353107931136": "Virtual Trunk Pte Ltd",
    "24.12106605": "ENERES Co.,Ltd.",
    "24.12351391": "SBM Co., Ltd.",
    "24.12341940": "Nokia",
    "24.12341943": "Apple, Inc.",
    "12.119269539840": "Rhino Controls Ltd.",
    "24.2362154": "Viettel Group",
    "24.2941608": "DeviceDesign",
    "24.3729646": "Nanjing Youkuo Electric Technology Co., Ltd",
    "20.137215117099008": "DTECH Labs, Inc.",
    "12.346855563264": "CA Technology, Inc",
    "24.15269352": "CeLa Link Corporation",
    "12.123917689802752": "Galaxy Next Generation, Inc.",
    "24.5505591": "Teltronic AG",
    "24.3672896": "Samsung Electronics Co.,Ltd",
    "24.10240798": "IEEE Registration Authority",
    "24.10799776": "Huawei Technologies Co.,Ltd",
    "24.13123068": "AlphaTheta Corporation",
    "24.11542313": "Helvetia INC.",
    "24.9716818": "Belkin International Inc.",
    "12.123917676797952": "DACOM West GmbH",
    "24.13371364": "Lenovo Mobile Communication Technology Ltd.",
    "12.346856251392": "Crde",
    "24.10521853": "Hewlett Packard",
    "24.2103000": "Liteon Technology Corporation",
    "24.4222762": "BlackBerry RTS",
    "24.10249710": "DASAN Network Solutions",
    "24.7139401": "Giga-Byte Technology Co.,Ltd.",
    "20.74781685710848": "Omnisense Systems Private Limited Taiwan Branch",
    "24.9708728": "Gree Electric Appliances, Inc. Of Zhuhai",
    "24.6326711": "Kael Mühendi̇sli̇k Elektroni̇k Ti̇caret Sanayi̇ Li̇mi̇ted Şi̇rketi̇",
    "24.5777292": "Buffalo.Inc",
    "24.8174447": "Cosco Electronics Co., Ltd.",
    "24.16515486": "Vievu",
    "24.13414786": "Huawei Technologies Co.,Ltd",
    "24.9197253": "Nintendo Co., Ltd.",
    "24.10769007": "ADB Broadband Italia",
    "24.12088356": "Viessmann Elektronik GmbH",
    "24.12601661": "Maitrise Technologique",
    "24.53318": "Dolby Laboratories, Inc.",
    "20.31372480610304": "Xingtera China Ltd",
    "24.57373": "WebTV NETWORKS, INC.",
    "24.53314": "Mahlo Gmbh & Co. Ug",
    "24.53315": "Zonal Retail Data Systems",
    "24.53316": "Alidian Networks, Inc.",
    "24.53317": "Kvaser Ab",
    "24.53310": "Rocketchips, Inc.",
    "24.53311": "American Communication",
    "20.48835643572224": "Cloudproject Generation Srl",
    "24.53313": "Amigo Technology Co., Ltd.",
    "12.346863538176": "elettrondata srl",
    "20.44213345452032": "Swisson AG",
    "24.9235531": "Dell Inc.",
    "12.346863185920": "Techno Sobi Co. Ltd.",
    "24.7005770": "S&O Electronics (Malaysia) Sdn. Bhd.",
    "12.346861563904": "TGE Co., Ltd.",
    "24.13380739": "DarkMatter L.L.C",
    "24.14691506": "Lenovo Mobile Communication (Wuhan) Company Limited",
    "24.15492232": "Honor Device Co., Ltd.",
    "24.4749645": "Samsung Electronics Co.,Ltd",
    "24.7927992": "Rako Controls Ltd",
    "12.346860916736": "Nuvation",
    "12.346855264256": "ADChips",
    "12.123917689913344": "Cursor Systems NV",
    "24.5548858": "Quanta Computer Inc.",
    "24.5611": "zte corporation",
    "24.5610": "Tellumat (Pty) Ltd",
    "24.5613": "Fulcrum Microsystems, Inc.",
    "24.5612": "Boca Devices LLC",
    "24.5615": "NEC TOKIN Corporation",
    "24.5614": "Omnex Control Systems",
    "24.5617": "KYLINK Communications Corp.",
    "24.5616": "Ego Bv",
    "24.5619": "Peltor Ab",
    "24.5618": "ASUSTek COMPUTER INC.",
    "12.346868133888": "TAMS firmware co.",
    "24.15526155": "Sistema Solucoes Eletronicas Ltda - Easytech",
    "20.225207115579392": "Neptune Systems",
    "12.123917676994560": "Sanmina Israel",
    "12.346859069440": "Casabyte Inc.",
    "24.13693147": "Ericsson",
    "24.14472260": "Huawei Device Co., Ltd.",
    "20.31477060337664": "OOO Tekhnotronika",
    "24.2130008": "Ciena Corporation",
    "24.8271": "Deutsche Aerospace Ag",
    "24.1058865": "Morion Inc.",
    "24.3619": "Incipient, Inc.",
    "24.3618": "Private",
    "12.346864345088": "Saia-Burgess Controls AG",
    "12.123917675655168": "COMTECH Kft.",
    "24.3613": "ARION Technology Inc.",
    "24.3612": "Hach Company",
    "24.3611": "IAV GmbH",
    "24.3610": "JPS Communications",
    "24.3617": "MTU Friedrichshafen GmbH",
    "24.3616": "ACCESS Systems Americas, Inc.",
    "24.3615": "TCL Networks Equipment Co., Ltd.",
    "24.3614": "QLogic Corporation",
    "24.6050184": "Motorola Mobility LLC, a Lenovo Company",
    "24.9463795": "Alcatel Lucent",
    "24.7603243": "Lenovo Mobile Communication (Wuhan) Company Limited",
    "24.11847416": "Axilspot Communication",
    "24.3696531": "Asia Optical Co., Inc.",
    "12.123917677772800": "GSP Sprachtechnologie GmbH",
    "12.123917675118592": "Kumu Networks",
    "24.9481755": "Tp-Link Technologies Co.,Ltd.",
    "24.6559334": "Nest Labs Inc.",
    "24.6294275": "Hangzhou H3C Technologies Co., Limited",
    "12.346863419392": "Barrick",
    "24.14168038": "Zhejiang Tmall Technology Co., Ltd.",
    "24.6067818": "Rtnet",
    "24.12367971": "Apple, Inc.",
    "20.163603964493824": "Shenzhen Link technology Co.,Ltd",
    "24.9969": "ProCurve Networking by HP",
    "24.9968": "cTrixs International GmbH.",
    "24.9967": "Technology Advancement Group, Inc.",
    "24.9966": "TKM GmbH",
    "24.9965": "zte corporation",
    "24.9964": "Legrand Home Systems, Inc",
    "24.9963": "Advanced Spectrum Technology Co., Ltd.",
    "24.9962": "Cheerchip Electronic Technology (ShangHai) Co., Ltd.",
    "24.9961": "SP Corp",
    "24.9960": "Murata Manufacturing Co., Ltd.",
    "24.14687484": "D-Link International",
    "24.16551876": "Intelligent Technology Inc.",
    "12.346868027392": "SEITEC Co. Ltd",
    "24.6826230": "Vubiq Networks, Inc.",
    "12.123917688418304": "RealD",
    "12.123917677793280": "Power Electronics Espana, S.L.",
    "24.12870883": "Texas Instruments",
    "24.11565320": "Cummings Engineering",
    "24.6040303": "r2p Asia-Pacific Pty Ltd",
    "12.346857533440": "Traquair Data Systems, Inc.",
    "20.48548677681152": "Forties Inc.",
    "24.786907": "Infinix mobility limited",
    "24.11818073": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "24.4510803": "Sagemcom Broadband SAS",
    "24.4743065": "Cloud Network Technology (Samoa) Limited",
    "12.346858037248": "MultiTrode Pty Ltd",
    "12.123917677268992": "CONTROL SYSTEMS Srl",
    "20.66219396825088": "iiM AG",
    "12.123917678911488": "Silex Inside",
    "12.123917691109376": "Nable Communications, Inc.",
    "12.123917685608448": "Fluid Components International",
    "24.36975": "Cisco Systems, Inc",
    "12.71297883742208": "Leder Elektronik Design",
    "24.13159011": "Huawei Device Co., Ltd.",
    "24.4767604": "Baicells Technologies Co.,LTD",
    "24.2132335": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.290377": "TaiYear Electronic Technology (Suzhou) Co., Ltd",
    "24.6056919": "Foshan VIOMI Electric Appliance Technology Co. Ltd.",
    "12.346864267264": "Fbb",
    "24.36979": "Gaio Technology",
    "24.49289": "Telindus Distribution",
    "24.49288": "Ekf Elektronik Gmbh",
    "24.20668": "Hammer Storage Solutions",
    "24.20669": "Cisco Systems, Inc",
    "24.49285": "Canon",
    "24.49284": "Data Link Corp. Ltd.",
    "24.49287": "Uunet Technologies, Inc.",
    "24.49286": "The Lynk Corporation",
    "24.49281": "Metrodata Ltd.",
    "24.49280": "Netstar, Inc.",
    "24.49283": "Trace Mountain Products, Inc.",
    "24.49282": "Moore Products Co.",
    "24.5026156": "Huawei Technologies Co.,Ltd",
    "12.123917686882304": "Packet Power",
    "12.123917682397184": "Omnisensing Photonics Llc",
    "24.13924248": "Cisco Systems, Inc",
    "12.346853462016": "Formula One Management Ltd.",
    "20.71070627397632": "SFT Co., Ltd.",
    "24.62605": "Liteon Technology Corporation",
    "24.560172": "Samsung Electronics Co.,Ltd",
    "24.796501": "Valuable Techologies Limited",
    "24.10769947": "MCOT Corporation",
    "24.3964783": "Telechips, Inc.",
    "12.123917687296000": "KOSMEK.Ltd",
    "24.8708": "Koratek",
    "24.8709": "WeLink Solutions, Inc.",
    "24.6296124": "Huawei Technologies Co.,Ltd",
    "24.8700": "Nokia Danmark A/S",
    "24.8701": "Lacroix Traffic S.A.U",
    "24.8702": "Nokia Danmark A/S",
    "24.8703": "Cyfrowy Polsat SA",
    "24.8704": "IBM Corp",
    "24.8705": "Aksys Networks Inc",
    "24.1348422": "PNI sensor corporation",
    "24.8707": "Glensound Electronics Ltd",
    "12.346859651072": "HSC-Regelungstechnik GmbH",
    "24.4996063": "Microsoft Corporation",
    "12.123917681160192": "Semacon Business Machines",
    "24.2636694": "Cool Control LTD",
    "24.9219721": "Y-cam Solutions Ltd",
    "24.6562016": "T3 Technology Co., Ltd.",
    "24.7635951": "Arista Networks",
    "12.346868822016": "Private",
    "24.10233278": "Wildlife Acoustics, Inc.",
    "24.10510944": "Private",
    "24.2686198": "Shenzhen Xin KingBrand enterprises Co.,Ltd",
    "24.2124543": "Atlas Elektronik UK Limited",
    "24.10759176": "Masterwork Automodules",
    "24.6080921": "New H3C Technologies Co., Ltd",
    "24.12097817": "7signal Solutions, Inc",
    "24.6739": "Zylaya",
    "24.6738": "Meshlinx Wireless Inc.",
    "24.6733": "Giga-Byte Technology Co.,Ltd.",
    "24.6732": "Crossbow Technology, Inc",
    "24.6731": "Hewlett Packard",
    "24.6730": "Qumranet Inc.",
    "24.6737": "Alfred Mann Foundation",
    "24.6736": "PheeNet Technology Corp.",
    "24.6735": "AVM GmbH",
    "24.6734": "NTI AG / LinMot",
    "24.1709": "KB Electronics Ltd.",
    "24.1708": "Intersoft Co.",
    "24.13636605": "LG Electronics (Mobile Communications)",
    "24.1703": "Primarion",
    "24.1702": "Artistic Licence Engineering Ltd",
    "24.1701": "PINON Corp.",
    "24.1700": "INNOWELL Corp.",
    "24.1707": "W-Link Systems, Inc.",
    "24.1706": "VT Miltope",
    "24.1705": "Universal Instruments Corp.",
    "24.1704": "KC Technology, Inc.",
    "12.346867126272": "Stichting Sunrise",
    "12.123917681819648": "BASF Corporation",
    "24.9220329": "Samsung Electronics.,LTD",
    "12.123917681000448": "Hella Gutmann Solutions GmbH",
    "24.14726741": "Beijing Xiaomi Electronics Co., Ltd.",
    "24.297029": "Bay Networks",
    "24.5537620": "Ingenico",
    "24.9759214": "Huawei Device Co., Ltd.",
    "12.346863460352": "Marathon Products, Inc.",
    "12.123917687840768": "Dosadores Alltronic",
    "24.1370931": "BSMediasoft Co., Ltd.",
    "12.123917680025600": "PCB Piezotronics",
    "12.346868506624": "JLCooper Electronics",
    "12.123917691072512": "HighTechSystem Co.,Ltd.",
    "24.3465981": "Apple, Inc.",
    "24.8711151": "Cisco Systems, Inc",
    "24.3992815": "Super Micro Computer, Inc.",
    "24.3165": "Chic Technology (China) Corp.",
    "24.9471115": "HDR10+ Technologies, LLC",
    "20.180680382218240": "Mahle Electronics, Slu",
    "20.39672896028672": "Satellite Link Technology CO.,LTD",
    "12.123917687844864": "Code Blue Corporation",
    "24.2896164": "Cisco Systems, Inc",
    "12.123917680504832": "Abbott Diagnostics Technologies AS",
    "12.123917683843072": "Profcon AB",
    "12.123917691404288": "Energybox Limited",
    "24.9158": "SMC Corporation",
    "24.9159": "AVSystem",
    "24.9156": "Lux Lumen",
    "24.9157": "Radiation Safety and Control Services Inc",
    "24.9154": "SAMSUNG Electronics. Co. LTD",
    "24.9155": "LogMeIn, Inc.",
    "24.9152": "Broadway Networks",
    "24.9153": "Securitas Direct AB",
    "24.9150": "Cisco SPVTG",
    "24.9151": "Mainpine, Inc.",
    "24.586840": "Huawei Device Co., Ltd.",
    "24.6854753": "Beacon Inc",
    "24.9449933": "Onyx Healthcare Inc.",
    "24.10758464": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.5799767": "Apple, Inc.",
    "12.71297884180480": "CST Group",
    "20.70443726798848": "Mirc Electronics Ltd",
    "24.4779527": "Silver Palm Technologies LLC",
    "24.10022603": "Wistron Infocomm (Zhongshan) Corporation",
    "24.5006193": "Xiaomi Communications Co Ltd",
    "24.9188605": "Huawei Technologies Co.,Ltd",
    "24.15207480": "Daeyoung Information System Co., Ltd",
    "24.2371074": "Huawei Technologies Co.,Ltd",
    "24.13674039": "Apple, Inc.",
    "24.12903696": "Mechatro, Inc.",
    "24.10265513": "Guangzhou Juan Optical and Electronical Tech Joint Stock Co., Ltd",
    "24.13433344": "LG Electronics (Mobile Communications)",
    "20.14286636187648": "Newgreen Tech Co., Ltd.",
    "12.346860027904": "IQ Solutions GmbH & Co. KG",
    "20.141268011188224": "Interplan Co., Ltd.",
    "24.13134046": "Huawei Device Co., Ltd.",
    "12.346853691392": "Sei S.p.A.",
    "12.346858446848": "Novacomm Ltda",
    "12.123917676949504": "Hudson Robotics",
    "24.14979815": "Gridlink Tech. Co.,Ltd.",
    "24.3196970": "WI-BIZ srl",
    "12.346859765760": "McQuay China",
    "24.3178651": "New H3C Technologies Co., Ltd",
    "12.346868285440": "Deckma GmbH",
    "24.806746": "ASA-RT s.r.l.",
    "20.225207126065152": "NantEnergy",
    "24.2669080": "NextNav, LLC",
    "24.6346908": "HMD Global Oy",
    "24.4206470": "LG Innotek",
    "12.71297884553216": "Shikino High-Tech",
    "24.10767560": "Dell Inc.",
    "24.10023297": "New H3C Technologies Co., Ltd",
    "24.11312356": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "12.123917689651200": "RMA Mess- und Regeltechnik GmbH & Co.KG",
    "12.123917689765888": "Automata Spa",
    "12.346867433472": "GS Elektromedizinische Geraete G. Stemple GmbH",
    "24.36948": "Innovative Semiconductors, Inc",
    "24.36949": "Parker Hannifin Corporation Compumotor Division",
    "24.11074743": "Consolidated Resource Imaging",
    "12.71297883713536": "Delopt",
    "12.123917684781056": "Spectrum Techniques, LLC",
    "24.36940": "Epigram, Inc.",
    "24.36941": "Spec S.A.",
    "24.36942": "Delem Bv",
    "24.36943": "Abb Power T&D Company, Inc.",
    "24.36944": "Teleste Corporation",
    "24.36945": "Ultimate Technology Corp.",
    "24.36946": "Selcom Elettronica S.R.L.",
    "24.36947": "Daewoo Electronics Co., Ltd.",
    "12.71297883885568": "Space Micro",
    "24.8151191": "Huawei Technologies Co.,Ltd",
    "24.13902995": "GW Technologies Co.,Ltd",
    "20.180394594926592": "Viloc",
    "24.535728": "Network Instruments",
    "24.16574470": "Edifier International",
    "20.93020687958016": "SpearX Inc.",
    "12.346852958208": "Alchemy Semiconductor, Inc.",
    "12.123917690204160": "Sankyo Intec Co,ltd",
    "24.9178275": "Amper",
    "12.123917681426432": "Orlaco Products B.V.",
    "12.123917676027904": "InsideRF Co., Ltd.",
    "24.4467296": "Apple, Inc.",
    "12.346860863488": "Performance Motion Devices",
    "24.16522239": "V-Zug Ag",
    "24.35413": "Huawei Device Co., Ltd.",
    "20.246679823450112": "Hale Products",
    "12.123917675339776": "Sudo Premium Engineering",
    "24.8707068": "Nokia",
    "24.3177546": "Huawei Device Co., Ltd.",
    "24.11022364": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "12.346866970624": "Elvees",
    "24.2382759": "Apple, Inc.",
    "24.11831903": "Juniper Networks",
    "12.123917682688000": "Cosmoit.Co.Ltd",
    "24.1872062": "Ericsson AB",
    "20.255191444094976": "Artech Solution Co.,Ltd",
    "24.9438135": "Parrot Sa",
    "24.8153843": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.71297883275264": "Staneo Sas",
    "24.13632513": "Motorola Mobility LLC, a Lenovo Company",
    "24.8167799": "Huawei Technologies Co.,Ltd",
    "12.346861985792": "BEEcube Inc.",
    "12.123917683429376": "Tecnint HTE SRL",
    "24.1104451": "Netgear",
    "24.12634486": "Shenzhen TINNO Mobile Technology Corp.",
    "24.816419": "Beijing Gehua Catv Network Co.,Ltd",
    "24.8401491": "Intel Corporate",
    "24.6061437": "Technicolor CH USA Inc.",
    "24.12370754": "Panasonic Communications Co., Ltd.",
    "24.11017053": "Foxtel Management Pty Ltd",
    "24.11017050": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.48543316312064": "Jiangsu JARI Technology Group Co., LTD",
    "24.4999386": "Beijing Autelan Technology Co.,Ltd",
    "12.346859524096": "Hauch & Bach ApS",
    "24.11329641": "ISAAC Instruments",
    "24.5825654": "Centron Communications Technologies Fujian Co.,Ltd",
    "24.1072969": "Cisco SPVTG",
    "24.3716941": "Fijowave Limited",
    "24.16026418": "Xiaomi Communications Co Ltd",
    "20.31657717399552": "Private",
    "12.346868703232": "Critical Link LLC",
    "20.52819394560000": "Punkt Tronics AG",
    "24.4997075": "Texas Instruments",
    "24.6564333": "AO \"PKK Milandr\"",
    "12.123917688946688": "Elystec Technology Co., Ltd",
    "12.123917679624192": "Shanghai Junqian Sensing Technology Co., LTD",
    "24.16294614": "Liteon Technology Corporation",
    "12.123917675151360": "G+D Mobile Security",
    "24.15782028": "TCT mobile ltd",
    "24.11808086": "Extreme Networks, Inc.",
    "24.6618002": "PPC Broadband Inc.",
    "12.123917683032064": "dieEntwickler Elektronik GmbH",
    "12.346866470912": "megatec electronic GmbH",
    "24.14423714": "Medtronic Diabetes",
    "24.7897550": "China Mobile Iot Limited company",
    "12.123917689245696": "SANO SERVICE Co.,Ltd",
    "24.6068576": "Samsung Electronics Co.,Ltd",
    "24.13137765": "Tiesse SpA",
    "24.9740366": "Bostex Technology Co., LTD",
    "24.791307": "Uniqoteq Ltd.",
    "20.150262245228544": "AndroVideo Inc.",
    "12.71297884626944": "Luna-Nexus",
    "24.2400993": "Apple, Inc.",
    "24.3410466": "Top-Access Electronics Co Ltd",
    "24.13139617": "TCL MOKA International Limited",
    "24.5017509": "Jastec",
    "24.12372661": "ARRIS Group, Inc.",
    "12.346864742400": "I.S.A. S.r.l.",
    "12.346860376064": "DEUTA-WERKE GmbH",
    "20.14223582167040": "Rotel",
    "12.346863861760": "Micromax Pty. Ltd.",
    "24.10812535": "Mega Well Limited",
    "24.7891487": "ARRIS Group, Inc.",
    "24.11063523": "Systech Electronics Ltd",
    "12.346859810816": "STAR-Dundee Ltd",
    "12.123917690105856": "Applied Processing",
    "24.16547180": "HYESUNG TECHWIN Co., Ltd",
    "24.10527292": "Gpms",
    "12.123917690884096": "JSC “Scientific Industrial Enterprise \"Rubin\"",
    "12.123917686939648": "Sakura Seiki Co.,Ltd.",
    "24.4108": "Ito Co., Ltd.",
    "24.4109": "Cisco Systems",
    "24.4102": "Thales Contact Solutions Ltd.",
    "24.4793": "Fusion Digital Technology",
    "24.4100": "The Brantley Coile Company,Inc",
    "24.4101": "Uec Commercial",
    "24.4106": "Williams Communications Group",
    "20.31368125874176": "Dspread Technology (Beijing) Inc.",
    "24.4104": "Vienna Systems Corporation",
    "24.4105": "Horanet",
    "24.3459": "Sanmina-SCI Hungary Ltd.",
    "24.3458": "Phsnet",
    "12.346862657536": "Reference, LLC.",
    "24.3451": "Consensys Computers Inc.",
    "24.3450": "DiGATTO Asia Pacific Pte Ltd",
    "24.3453": "Afco Systems",
    "24.3452": "Codian Ltd",
    "24.3455": "MIDAS COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)",
    "24.3454": "Axiowave Networks, Inc.",
    "24.3457": "Pepperl+Fuchs GmbH",
    "24.3456": "Online Development Inc",
    "12.123917688512512": "Centero",
    "24.4248202": "Nintendo Co., Ltd.",
    "24.6085016": "D-Link Corporation",
    "24.11580867": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1346045": "Samsung Electronics Co.,Ltd",
    "24.5301862": "Shenzhen Techtion Electronics Co., Ltd.",
    "24.15216164": "Out of the Fog Research LLC",
    "24.16015479": "Texas Instruments",
    "24.3186683": "Ardomus Networks Corporation",
    "12.71297883246592": "Flexim Security Oy",
    "24.8696417": "Cisco Systems, Inc",
    "20.172584092041216": "Savari Inc",
    "24.10267108": "zte corporation",
    "24.13168136": "Tangtop Technology Co.,Ltd",
    "24.4229125": "Motorola Mobility LLC, a Lenovo Company",
    "24.15764478": "Costel.,Co.Ltd",
    "12.123917685125120": "Honeywell",
    "24.5785792": "Coflec",
    "24.13134909": "Amazon Technologies Inc.",
    "24.5037361": "Samsung Electronics Co.,Ltd",
    "12.119269576704": "Lumiplan Transport",
    "24.4242536": "Wuhan Funshion Online Technologies Co.,Ltd",
    "20.146101709570048": "RAY Co.,LTD",
    "12.346854395904": "Getinge IT Solutions ApS",
    "24.1106645": "Qi Hardware Inc.",
    "24.14986054": "UAB Selteka",
    "12.123917690490880": "Tattile Srl",
    "24.13170134": "IEEE Registration Authority",
    "20.242876969648128": "JB-Lighting Lichtanlagen GmbH",
    "24.8971179": "Huawei Technologies Co.,Ltd",
    "24.301980": "Eadingcore Intelligent Technology Co., Ltd.",
    "24.12851198": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7873454": "ARRIS Group, Inc.",
    "24.24767": "Macraigor Systems, Inc.",
    "24.6084766": "Huawei Device Co., Ltd.",
    "24.10549687": "Ademco Smart Homes Technology(Tianjin)Co.,Ltd.",
    "24.16267392": "Apple, Inc.",
    "24.13385090": "Intel Corporate",
    "24.139336": "# At least some 2810 send with locally assigned flag set",
    "24.3972054": "Vizio, Inc",
    "12.123917677879296": "Coherent Logix, Inc.",
    "20.176726060564480": "IVision Electronics Co.,Ltd",
    "12.346862137344": "Bogart Engineering",
    "24.2896935": "Erco & Gener",
    "12.346861277184": "RingCube Technologies, Inc.",
    "24.2142455": "Enclustra GmbH",
    "12.346856296448": "Salland Electronics Holding BV",
    "24.7402330": "Cisco Systems, Inc",
    "12.71297884561408": "Home Automation Europe",
    "24.14690074": "TTC Next-generation Home Network System WG",
    "12.123917676171264": "Dakton Microlabs LLC",
    "24.15531646": "BlueRadios, Inc.",
    "24.14442657": "Teledyne DALSA Professional Imaging",
    "12.123917682253824": "Edgeware AB",
    "24.13382373": "Tp-Link Technologies Co.,Ltd.",
    "12.123917681242112": "Rtone",
    "24.3447255": "Universal Flow Monitors, Inc.",
    "12.346861641728": "Board Level Limited",
    "24.8691822": "Sagemcom Broadband SAS",
    "20.88010620141568": "Diamond Traffic Products, Inc",
    "24.8706075": "Tp-Link Technologies Co.,Ltd.",
    "24.6832978": "Bhuu Limited",
    "24.15759367": "Apple, Inc.",
    "12.123917679439872": "Kalycito Infotech Private Limited",
    "12.71297885032448": "Private",
    "24.15503029": "Hewlett Packard",
    "20.48476171796480": "Beijing Jishi Huitong Technology Co., Ltd.",
    "24.15503021": "Dlx",
    "24.15503022": "Nagravision SA",
    "24.3209911": "Tunai Creative",
    "24.8679786": "Huawei Device Co., Ltd.",
    "24.1867564": "Huawei Technologies Co.,Ltd",
    "12.123917678649344": "APG Cash Drawer, LLC",
    "24.15992342": "Apple, Inc.",
    "24.14155853": "Apple, Inc.",
    "12.346855989248": "IntelliDesign Pty Ltd",
    "24.1353": "Salira Optical Network Systems",
    "12.346858160128": "WestfaliaSurge GmbH",
    "24.10763640": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.273339": "PicoCELA, Inc.",
    "24.6876580": "Shenzhen Seavo Technology Co.,Ltd",
    "24.14475222": "zte corporation",
    "24.3990964": "Kidasen Industria E Comercio De Antenas Ltda",
    "12.346868109312": "Grupo Epelsa S.L.",
    "24.572220": "Flextronics Tech.(Ind) Pvt Ltd",
    "24.1352127": "ASSA ABLOY Korea Co., Ltd Unilock",
    "24.2422178": "Belkin International Inc.",
    "20.181256792834048": "Eho.Link",
    "20.96896782696448": "Jia Huang Jhan Ye Co.,Ltd",
    "12.346868137984": "Vemco Sp. z o. o.",
    "20.71414663086080": "Creanord",
    "24.2418": "Securebase,Inc",
    "24.2419": "Lenten Technology Co., Ltd.",
    "24.13421646": "Sun Fountainhead USA. Corp",
    "24.2412": "Imedia Semiconductor Corp.",
    "24.2413": "Powernet Technologies Corp.",
    "24.2410": "Cloverleaf Communications Inc.",
    "24.2411": "IBM Corp",
    "24.2416": "Vibration Research Corporation",
    "24.2417": "Time Management, Inc.",
    "24.2414": "Giant Electronics Ltd.",
    "24.2415": "Beijing Zhongqing Elegant Tech. Corp.,Limited",
    "24.1149": "Motorola Solutions Inc.",
    "24.1148": "Skidata AG",
    "12.123917687029760": "HOERMANN GmbH",
    "24.1141": "3Com",
    "24.1140": "Legrand",
    "24.1143": "Scalant Systems, Inc.",
    "24.1142": "3Com",
    "24.1145": "Radius Co., Ltd.",
    "24.1144": "G. Star Technology Corporation",
    "24.1147": "Schlumberger",
    "24.1146": "Axxessit Asa",
    "24.3189214": "LG Innotek",
    "24.4255864": "Cisco Systems, Inc",
    "24.14453544": "Intel Corporate",
    "24.64837": "Hewlett Packard Enterprise",
    "24.6242": "Seagate Technology",
    "24.6241": "Ooma, Inc.",
    "24.23004": "Cisco Systems, Inc",
    "24.6247": "Datalogic ADC",
    "24.6246": "Leutron Vision",
    "12.346859175936": "Smart Key International Limited",
    "24.6244": "Eaton Corporation",
    "24.6249": "Kingjim",
    "24.6248": "Cisco SPVTG",
    "24.3470893": "SHARP Corporation",
    "24.8394249": "Sleep Number",
    "24.14183426": "Huaqin Telecom Technology Co.,Ltd",
    "24.2138852": "Apple, Inc.",
    "24.12641275": "Huawei Technologies Co.,Ltd",
    "12.346857000960": "Critical Link LLC",
    "12.346857127936": "Saia Burgess Controls AG",
    "12.123917688610816": "Scheiber",
    "12.123917689221120": "Molu Technology Inc., LTD.",
    "12.346867437568": "Dsp Design Ltd",
    "12.123917675802624": "Aug. Winkhaus GmbH & Co. KG",
    "20.176189947772928": "Shenzhen Nufilo Inc.",
    "24.7391686": "Cisco Systems, Inc",
    "24.7427": "Design Solutions Inc.",
    "20.136646466994176": "Powerland Limited",
    "24.292528": "Cisco Systems, Inc",
    "24.15248557": "zte corporation",
    "24.4204": "EDNT GmbH",
    "12.346863542272": "Essepie Srl",
    "24.11817465": "NB Software",
    "24.3993617": "Intel Corporate",
    "24.8967035": "FirmTek, LLC",
    "24.811686": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.7865009": "Cisco Systems, Inc",
    "24.6867": "Vamp Ltd.",
    "24.6866": "Eletronica Nitron Ltda",
    "24.6865": "Fargo Co., Ltd.",
    "24.6864": "Albis Technologies AG",
    "24.6863": "C.T. Elettronica",
    "24.6862": "Yupiteru Corporation",
    "24.6861": "Tidel Engineering LP",
    "24.6860": "Celestial Semiconductor, Ltd",
    "24.12108677": "Huawei Technologies Co.,Ltd",
    "24.6869": "Kmc Chain Industrial Co., Ltd.",
    "24.6868": "iPOX Technology Co., Ltd.",
    "24.3678934": "Hangzhou H3C Technologies Co., Limited",
    "24.3196167": "ECI Telecom Ltd.",
    "24.13139319": "Shenzhen Kingtech Communication Equipment Co.,Ltd",
    "24.13411188": "Guangdong Guanglian Electronic Technology Co.Ltd",
    "24.2626028": "Huawei Technologies Co.,Ltd",
    "24.16560324": "Shanghai DareGlobal Technologies Co.,Ltd",
    "24.536118": "Huawei Device Co., Ltd.",
    "24.11857914": "Lemobile Information Technology (Beijing) Co., Ltd.",
    "12.346861211648": "Intune Networks",
    "12.346852392960": "Etheira Technologies",
    "24.8403094": "SHARP Corporation",
    "20.250817682604032": "Shanghai LeXiang Technology Co., Ltd",
    "24.15492069": "ePBoard Design LLC",
    "12.346867650560": "RSUPPORT Co., Ltd.",
    "24.6081074": "Theben AG",
    "24.118": "Abekas Video System",
    "24.119": "Interphase",
    "24.12904751": "AB Sciex",
    "12.123917685194752": "Loma Systems s.r.o.",
    "24.3472623": "LG Electronics (Mobile Communications)",
    "24.110": "Artisoft Inc.",
    "24.111": "Madge Networks Ltd.",
    "24.112": "Hcl Limited",
    "24.113": "Adra Systems Inc.",
    "24.114": "Miniware Technology",
    "24.115": "Siecor Corporation",
    "24.116": "Ricoh Company Ltd.",
    "12.346859036672": "Selco",
    "12.123917683138560": "Morgan Schaffer Inc.",
    "24.9739": "Cisco Systems, Inc",
    "24.12366055": "Huawei Technologies Co.,Ltd",
    "24.13918514": "Pro Concept Manufacturer Co.,Ltd.",
    "24.14161110": "SURE SYSTEM Co Ltd",
    "24.13918510": "Huawei Technologies Co.,Ltd",
    "24.13678018": "Texas Instruments",
    "24.16298326": "Dell Inc.",
    "24.8163044": "Texas Instruments",
    "12.123917681111040": "Supracon AG",
    "24.9188353": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7899274": "CDR Sp. z o.o. Sp. k.",
    "24.6071275": "Lokel s.r.o.",
    "24.9224481": "Panasonic Corporation AVC Networks Company",
    "24.4234280": "Apple, Inc.",
    "24.11576190": "Qing Dao Haier Telecom Co.,Ltd.",
    "24.10771873": "ADB Broadband Italia",
    "12.346856726528": "Funkwerk plettac electronic GmbH",
    "12.346853470208": "Sentito Networks",
    "24.1354211": "Mst Corporation",
    "12.123917682241536": "Aml",
    "20.167137984380928": "Grammer EiA Electronics nv",
    "12.123917675167744": "AT-Automation Technology GmbH",
    "24.9734": "RAUMFELD GmbH",
    "24.7636193": "Dongguan Haoyuan Electronics Co.,Ltd",
    "20.79215076573184": "BravoCom（xiamen）TechCo.Ltd",
    "24.3428736": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "24.13898607": "Asia Pacific Brands",
    "12.123917682511872": "EMAC, Inc.",
    "24.4469580": "Shenzhen Bilian electronic CO.,LTD",
    "12.346863636480": "Green Goose",
    "24.534297": "APS systems/electronic AG",
    "12.346866098176": "ATH system",
    "12.346860978176": "Aethon, Inc.",
    "24.6560290": "Heliospectra AB",
    "12.123917691330560": "TechOne",
    "12.123917683904512": "Invixium Access Inc.",
    "24.3966673": "vivo Mobile Communication Co., Ltd.",
    "24.14467307": "Assa Abloy Côte Picarde",
    "24.9231362": "zte corporation",
    "24.15994402": "Samsung Electronics Co.,Ltd",
    "24.6323876": "Shanghai Imilab Technology Co.Ltd",
    "20.224420099522560": "Earphone Connection, Ubc.",
    "24.16567599": "IEEE Registration Authority",
    "12.123917686509568": "Critical Link LLC",
    "24.11037068": "Oracle Corporation",
    "24.6573424": "dSPACE GmbH",
    "24.806": "Iwasaki Information Systems Co., Ltd.",
    "24.807": "Act'L",
    "24.804": "SANYO Consumer Electronics Co., Ltd.",
    "24.805": "Arima Computer Corp.",
    "24.802": "IDIS Co., Ltd.",
    "24.803": "Cornet Technology, Inc.",
    "24.800": "Xpeed, Inc.",
    "24.801": "Reco Research Co., Ltd.",
    "24.11035517": "D-Link International",
    "24.4489451": "Motorola Mobility LLC, a Lenovo Company",
    "24.15224971": "Samsung Electro-Mechanics(Thailand)",
    "24.15744917": "Huawei Technologies Co.,Ltd",
    "24.808": "Mace Group, Inc.",
    "24.809": "F3, Inc.",
    "24.10242683": "Nokia Corporation",
    "24.7617125": "NEC Corporation",
    "20.145211812478976": "Leybold GmbH",
    "12.346858827776": "Silverbrook Research",
    "24.1849738": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346868502528": "TTi LTD (Thurlby Thandar Instruments LTD)",
    "12.123917691338752": "MB connect line GmbH Fernwartungssysteme",
    "24.14199179": "Texas Instruments",
    "12.123917691531264": "myUpTech AB",
    "20.176726072098816": "Ewig Industries Macao Commercial Offshore Ltd",
    "12.346867089408": "Grupo Epelsa S.L.",
    "24.1333765": "Apple, Inc.",
    "24.10779525": "ULTIMEDIA Co Ltd,",
    "24.14209925": "Hewlett Packard",
    "24.24590": "Wavenet International, Inc.",
    "24.24591": "Westell Technologies Inc.",
    "24.24592": "Network Machines, Inc.",
    "24.24593": "Crystal Semiconductor Corp.",
    "24.24594": "Power Computing Corporation",
    "24.24595": "Netstal Maschinen Ag",
    "24.24596": "Edec Co., Ltd.",
    "24.24597": "Net2Net Corporation",
    "24.24598": "Clariion",
    "24.24599": "Tokimec Inc.",
    "24.7665654": "Shanghai Sunmi Technology Co.,Ltd.",
    "24.13663277": "Leie IOT technology Co., Ltd",
    "20.180394591780864": "accesso Technology Group",
    "12.71297884028928": "Icraft Oy",
    "12.123917684137984": "Eldes Ltd",
    "24.8414761": "Prescope Technologies CO.,LTD.",
    "12.346854305792": "Rafael",
    "24.9481279": "BrightSign LLC",
    "24.6817546": "TPVision Europe B.V",
    "12.123917686968320": "Daatrics LTD",
    "24.15498653": "Cpi",
    "24.10533560": "Pixon Imaging",
    "24.16040212": "Technicolor CH USA Inc.",
    "12.71297884213248": "MONOGRAM technologies ltd",
    "24.11591617": "Tenda Technology Co.,Ltd.Dongguan branch",
    "20.145383544061952": "Diamond Products LLC",
    "24.13646194": "Texas Instruments",
    "24.11593501": "Shanghai Maigantech Co.,Ltd",
    "12.346865008640": "BAE Systems Bofors AB",
    "24.7380": "Nokia Danmark A/S",
    "24.7383": "Harman/Becker Automotive Systems GmbH",
    "24.7382": "Nokia Danmark A/S",
    "24.7385": "GlobalTop Technology Inc.",
    "24.7384": "BlueAnt Wireless",
    "24.7387": "Carpoint Co.,Ltd",
    "24.7386": "Exegin Technologies Limited",
    "24.7389": "Cowbell Engineering Co., Ltd.",
    "24.7388": "Custom Computer Services, Inc.",
    "24.12385932": "Altronic, LLC",
    "24.8148455": "YSI, Inc.",
    "24.2351": "Akom Technology Corporation",
    "24.2350": "B&Tech System Inc.",
    "24.2353": "Future Internet, Inc.",
    "24.2352": "AeroConcierge Inc.",
    "24.2355": "Ophit Co.Ltd.",
    "24.2354": "Omnilux",
    "24.2357": "Sandvine Incorporated",
    "24.2356": "Dream-Multimedia-Tv GmbH",
    "24.2359": "Inventec Appliance Corp",
    "24.2358": "Ipetronik GmbH & Co. KG",
    "24.12858475": "Hewlett Packard",
    "24.2135690": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "24.8913749": "Arcadyan Technology Corporation",
    "24.16051732": "Mercury Communication Technologies Co.,Ltd.",
    "12.346857922560": "Digital Way",
    "24.6575138": "Equinox Payments",
    "24.14971064": "Intel Corporate",
    "24.13109481": "LG Electronics",
    "12.123917682696192": "TimeMachines Inc.",
    "24.1972": "Cisco Systems, Inc",
    "24.53488": "Convision Technology Gmbh",
    "24.53489": "Sega Enterprises, Ltd.",
    "24.7659": "DINEC International",
    "24.7658": "Commtest Instruments Ltd",
    "24.15515675": "Huawei Technologies Co.,Ltd",
    "24.7653": "Cisco Systems, Inc",
    "24.53481": "Advantage Century Telecommunication Corp.",
    "24.7651": "Intuicom",
    "24.7650": "Radionor Communications",
    "24.7657": "Nokia Danmark A/S",
    "24.53485": "Xiox",
    "24.7655": "Marine Sonic Technology, Ltd.",
    "24.7654": "Cisco Systems, Inc",
    "12.346853556224": "Jostra AB",
    "24.13155528": "Apple, Inc.",
    "24.4516568": "Texas Instruments",
    "12.123917691142144": "silicom",
    "12.346862665728": "TP Radio",
    "24.14739387": "Nureva, Inc.",
    "24.13915055": "VIGO System S.A.",
    "24.823648": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.119269310464": "Private",
    "20.175931696087040": "Argos Solutions AS",
    "12.123917677432832": "PixelApps s.r.o.",
    "12.346859646976": "Power Electronics Espana, S.L.",
    "24.14210278": "Hytec Inter Co., Ltd.",
    "12.346861084672": "Warecube,Inc.",
    "24.10236342": "Kulite Semiconductor Products Inc",
    "20.31477537439744": "tec5AG",
    "24.9453546": "Huawei Technologies Co.,Ltd",
    "24.1637243": "Dell Inc.",
    "12.346866270208": "Computech International",
    "24.8191566": "Shenzhen Safe vision Technology Co.,LTD",
    "12.123917681659904": "E2G srl",
    "12.346859249664": "HoseoTelnet Inc...",
    "20.176189957210112": "Jining SmartCity Infotech Co.Ltd.",
    "24.3967998": "Samsung Electronics Co.,Ltd",
    "24.1589412": "Shenzhen Trylong Smart Science and Technology Co., Ltd.",
    "24.30342": "Cisco Systems, Inc",
    "24.15214217": "Aruba, a Hewlett Packard Enterprise Company",
    "12.71297885048832": "Andy-L Ltd.",
    "24.11292972": "Infinix mobility limited",
    "24.15511777": "Ciena Corporation",
    "12.346859311104": "Schnick-Schnack-Systems GmbH",
    "20.145211809333248": "AIBIoT GmbH",
    "12.123917683752960": "Hubbell Power Systems",
    "24.15468651": "Tp-Link Technologies Co.,Ltd.",
    "24.24831": "QuVis, Inc.",
    "24.24830": "Lynx System Developers, Inc.",
    "24.3207989": "Huawei Technologies Co.,Ltd",
    "24.8924119": "Addénergie Technologies",
    "24.3210801": "Nokia",
    "12.123917687828480": "Vironova AB",
    "24.9994775": "Disruptive Ltd",
    "12.346867351552": "sp controls, inc",
    "24.7141842": "Chengdu Povodo Electronic Technology Co., Ltd",
    "24.3933744": "Apple, Inc.",
    "24.2896737": "Apple, Inc.",
    "24.5563080": "Dongguan Aoyuan Electronics Technology Co., Ltd",
    "24.6855303": "Amazon Technologies Inc.",
    "12.123917678915584": "A-M Systems",
    "12.346853818368": "Cominfo, Inc.",
    "24.11860963": "AltoBeam (China) Inc.",
    "24.7401957": "Xetawave LLC",
    "24.14964920": "Samsung Electronics Co.,Ltd",
    "24.8706974": "Pink Nectarine Health AB",
    "24.2948965": "Oki Electric Industry Co., Ltd.",
    "20.206931527467008": "Altronix Corporation",
    "24.6577395": "ParanTek Inc.",
    "24.2678518": "bioMerieux Inc.",
    "24.2407672": "Huawei Technologies Co.,Ltd",
    "24.10534349": "BSkyB Ltd",
    "20.114975470256128": "Multi Alarm Zrt.",
    "12.346863124480": "Dsp Design Ltd",
    "24.6868124": "Mine Site Technologies",
    "20.102196704182272": "CHIPSEN Co.,Ltd.",
    "24.15470294": "Itech Instruments Sas",
    "20.62636947406848": "Polska Fabryka Wodomierzy i Ciep?omierzy FILA",
    "24.10020762": "Foxconn(NanJing) Communication Co.,Ltd.",
    "24.4515438": "Apple, Inc.",
    "24.8428320": "Intel Corporate",
    "12.123917675364352": "Telink Semiconductor CO, Limtied, Taiwan",
    "24.7911148": "Scuf Gaming International LLC",
    "12.123917676118016": "Orion Power Systems, Inc.",
    "12.123917679083520": "Neptec Technologies Corp.",
    "12.123917680578560": "infrachip",
    "24.11839876": "Symantec Corporation",
    "20.220515723640832": "Shenyang Machine Tool(Group) Research & Design Institute Co., Ltd, Shanghai Branch",
    "24.13414264": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1058750": "Tvip",
    "12.346861539328": "Detection Technologies Ltd.",
    "24.13917540": "ASUSTek COMPUTER INC.",
    "12.123917691494400": "Apogee Applied Research, Inc.",
    "20.180680395849728": "Ssk Corporation",
    "12.123917677182976": "signalparser",
    "20.84531180732416": "Beijing Sutongwang E-Business Co., Ltd",
    "24.2945117": "Micro-Star INTL CO., LTD.",
    "24.1325802": "Dynapower Company LLC",
    "24.14439292": "Compal Broadband Networks, Inc.",
    "24.5035958": "Helmer Scientific",
    "24.2117247": "Netgear",
    "12.71297883303936": "Sitep Italia Spa",
    "20.225209247334400": "MagTarget LLC",
    "24.16403": "NTT Data Communication Systems Corp",
    "24.16402": "Windata, Inc.",
    "24.16401": "Andover Controls Corporation",
    "24.16400": "Sonic",
    "24.16407": "Silex Technology America",
    "24.16406": "ADC - Global Connectivity Solutions Division",
    "24.16405": "Ascom Infrasys Ag",
    "24.16404": "Comsoft Gmbh",
    "24.16409": "Aeon Systems, Inc.",
    "24.16408": "Adobe Systems, Inc.",
    "12.71297884332032": "Sicon srl",
    "12.123917683458048": "Advice",
    "24.544284": "H2A Systems, LLC",
    "12.346852179968": "SCI Technology Inc.",
    "12.346865156096": "Pal Software Service Co.,Ltd.",
    "24.6326588": "Thermo Fisher Scientific P.O.A.",
    "12.123917685309440": "Wartsila Voyage Limited",
    "12.346864295936": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.2647403": "Keystone Microtech Corporation",
    "24.4521531": "Arcadyan Corporation",
    "24.4212186": "Spreadtrum Communications (Shanghai) Co., Ltd.",
    "24.10018512": "Tp-Link Technologies Co.,Ltd.",
    "24.3721606": "Apple, Inc.",
    "12.123917687992320": "Xentech Solutions Limited",
    "12.123917680570368": "Assembly Contracts Limited",
    "12.123917680332800": "Pcsc",
    "24.12902622": "Zhengzhou XindaJiean Information Technology Co.,Ltd.",
    "12.346854117376": "peiker acustic GmbH",
    "24.2151943": "Nokia Corporation",
    "24.3727": "Sercomm Corporation.",
    "24.16318219": "Electronic Technology Inc.",
    "24.924": "OptiMight Communications, Inc.",
    "12.346853343232": "XStore, Inc.",
    "12.123917688864768": "Vista Research, Inc.",
    "12.346857926656": "Orderite, Inc.",
    "20.18540144885760": "BBPOS International Limited",
    "24.3446386": "Tp-Link Technologies Co.,Ltd.",
    "24.926": "Tera System Co., Ltd.",
    "24.13903308": "IEEE Registration Authority",
    "12.123917680500736": "Asiga Pty Ltd",
    "24.921": "Dongju Informations & Communications Co., Ltd.",
    "20.128510969511936": "Loctek Visual Technology Corp.",
    "24.920": "Wisi",
    "24.6044469": "Gehirn Inc.",
    "24.923": "NetChip Technology, Inc.",
    "24.10927": "LARsys-Automation GmbH",
    "24.922": "SiConnect",
    "12.346864312320": "Onix Electronic Systems Inc",
    "24.9212169": "Toyoshima Electric Technoeogy(Suzhou) Co.,Ltd.",
    "12.71297883500544": "Haein S&S Co., Ltd",
    "20.31368177254400": "A-Gear Company Limited",
    "12.123917686611968": "Vteq",
    "24.8154255": "Ams Neve Ltd",
    "12.123917675753472": "HAI ROBOTICS Co., Ltd.",
    "12.123917687926784": "S Labs sp. z o.o.",
    "24.12418": "Taihan Electric Wire Co., Ltd.",
    "24.12419": "Ivron Systems",
    "24.10250676": "IEEE Registration Authority",
    "24.15257660": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "24.12414": "Redflex Communication Systems",
    "24.12415": "Irlan Ltd.",
    "24.12416": "Cisco Systems, Inc",
    "24.12417": "Altos C&C",
    "24.12410": "Advanced Technology & Systems",
    "24.12411": "Cisco Systems, Inc",
    "24.1073227": "Hewlett Packard",
    "24.12413": "Gre America, Inc.",
    "24.5808331": "TrackNet, Inc",
    "20.224420104765440": "Matter Labs Pty Ltd",
    "24.10799417": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917676589056": "Evco S.p.a.",
    "12.346861121536": "Hollis Electronics Company LLC",
    "20.141720403574784": "Cavalry Storage Inc",
    "12.346853244928": "Balogh S.A.",
    "12.346863284224": "PDQ Manufacturing Inc.",
    "20.225135234646016": "Winterthur Gas & Diesel Ltd.",
    "24.12360552": "Technicolor CH USA Inc.",
    "12.346861768704": "Flight Deck Resources",
    "24.9975933": "Itronic Technology Co. , Ltd.",
    "24.12615232": "Shenzhen Xdk Communication Equipment Co.,Ltd",
    "20.198237813211136": "Taizhou convergence Information technology Co.,LTD",
    "24.13679488": "SHL Telemedicine International Ltd.",
    "24.8674775": "Shenzhen Netcom Electronics Co.,Ltd",
    "24.3965400": "Sagemcom Broadband SAS",
    "12.346862075904": "EMAC, Inc.",
    "20.203238158630912": "V5 Technology Corporation",
    "24.2359751": "Cisco Systems, Inc",
    "24.10785483": "Nokia",
    "24.13425749": "Dragonchip Limited",
    "24.15504167": "MEMMERT GmbH + Co. KG",
    "20.163808566837248": "Anvil Systems Group, Inc.",
    "24.15737774": "AMPAK Technology,Inc.",
    "12.123917689876480": "Flextronics International Kft.",
    "24.5468": "Dresser Wayne",
    "24.5469": "Microsoft Corporation",
    "24.7368301": "Cisco Systems, Inc",
    "24.16576973": "Optex-Fa Co.,Ltd.",
    "24.5462": "Sagemcom Broadband SAS",
    "24.5463": "Olivetti",
    "24.5460": "Atalum Wireless S.A.",
    "24.5461": "DFM GmbH",
    "24.3170584": "Paradom Inc.",
    "24.5467": "Sampo Corporation",
    "24.5464": "Foxconn",
    "24.5465": "Securaplane Technologies, Inc.",
    "24.4739": "Nortel Networks",
    "24.4738": "Qovia",
    "24.4731": "VIA Networking Technologies, Inc.",
    "24.4730": "Sanyu Industry Co.,Ltd.",
    "24.4733": "MobileAria",
    "24.4732": "Swegon Ab",
    "24.4735": "Cisco Systems, Inc",
    "24.4734": "Digital Lifestyles Group, Inc.",
    "24.4737": "March Networks S.p.A.",
    "24.4736": "Cisco Systems, Inc",
    "24.587753": "HRCP Research and Development Partnership",
    "24.3186723": "Rock Path S.R.L",
    "24.13946679": "Symbolic IO",
    "24.13655162": "Sartura d.o.o.",
    "24.6320320": "Apple, Inc.",
    "24.8134925": "Cisco Systems, Inc",
    "24.8134926": "Cisco Systems, Inc",
    "24.13943240": "zte corporation",
    "12.346861047808": "Milesys",
    "12.123917686190080": "QUISS GmbH",
    "20.36069839994880": "Annapurna labs",
    "24.13945163": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917687545856": "comtac AG",
    "24.15784752": "Yottabyte",
    "24.16528008": "CELOT Wireless Co., Ltd",
    "12.346861498368": "MKT Systemtechnik",
    "24.57365": "Heiwa Corporation",
    "24.57364": "Cisco Systems, Inc",
    "24.57367": "EXXACT GmbH",
    "24.57366": "rapid-city (now a part of bay networks)",
    "24.57361": "Uniden Corporation",
    "24.57360": "HESS SB-AUTOMATENBAU GmbH",
    "24.57363": "Eastern Electronic Co., Ltd.",
    "24.57362": "Pluto Technologies International Inc.",
    "24.4751940": "Life Fitness / Div. of Brunswick",
    "24.5526997": "Huawei Device Co., Ltd.",
    "24.6075684": "Sony Mobile Communications Inc",
    "24.57369": "Ing. Giordano Elettronica",
    "24.57368": "Asustek",
    "12.346853486592": "Aartesys Ag",
    "12.346865672192": "Sicon srl",
    "12.346865573888": "Sanmina Shenzhen",
    "24.13930767": "Amazon Technologies Inc.",
    "20.198611821395968": "DCI International, LLC.",
    "20.137215115001856": "Astrum Technologies CC",
    "12.123917691326464": "Solid State Disks Ltd",
    "12.123917679542272": "Golding Audio Ltd",
    "24.16053736": "Google, Inc.",
    "24.14460889": "Compressor Controls Corp",
    "24.16543671": "ARRIS Group, Inc.",
    "24.14479113": "Netgear",
    "24.6080233": "Edifier International",
    "12.346855075840": "Siliquent Technologies Ltd",
    "24.5775849": "Twpi",
    "24.13381760": "VAIO Corporation",
    "12.346860720128": "Intellectronika",
    "20.260167863107584": "Simula Technology Inc.",
    "24.9758481": "Xirka Dama Persada PT",
    "24.838541": "ASCO Numatics GmbH",
    "24.3172": "X2 MSA Group",
    "24.3173": "Sunin Telecom",
    "24.3170": "ABB AB, Cewe-Control",
    "24.3171": "Zenith Electronics Corporation",
    "24.1053256": "ITG, Inc.",
    "24.3177": "National Radio Astronomy Observatory",
    "24.3174": "Pronto Networks Inc",
    "24.3175": "Oyo Electric Co.,Ltd",
    "24.15502490": "Huawei Technologies Co.,Ltd",
    "24.839633": "SPRINGWAVE Co., Ltd",
    "24.3178": "Mbari",
    "24.3179": "Kurz Industrie-Elektronik GmbH",
    "24.3440128": "Huawei Device Co., Ltd.",
    "24.1066676": "FormericaOE",
    "12.123917678944256": "Cellular Specialties, Inc.",
    "24.8693635": "Alcatel Lucent",
    "12.346866307072": "Voltech Instruments",
    "24.1063246": "Hisense broadband multimedia technology Co.,Ltd",
    "20.228844759941120": "GNS-GmbH",
    "24.3990379": "Zhejiang Dahua Technology Co., Ltd.",
    "24.10515172": "Nj System Co.,Ltd",
    "24.3696861": "Inventec Corporation",
    "24.7131462": "FRAMOS GmbH",
    "24.14447259": "Huawei Technologies Co.,Ltd",
    "12.346854281216": "SystIng",
    "24.5779198": "Lighting Science Group",
    "24.40984": "Creative Controllers, Inc.",
    "20.110373780848640": "MiiVii Dynamics Technology CO.,LTD",
    "24.58290": "Samsung Electronics Co.,Ltd",
    "24.8659848": "Ruckus Wireless",
    "12.71297883332608": "Symetrics Industries",
    "24.9975153": "Sub10 Systems Ltd",
    "24.792435": "Huawei Device Co., Ltd.",
    "24.13432442": "Ayecka Communication Systems LTD",
    "12.123917676294144": "Sienna Corporation",
    "24.8671617": "Samsung Electronics Co.,Ltd",
    "24.14434040": "Nokia Corporation",
    "12.123917678616576": "Matelex",
    "24.40982": "Micropolis Corp.",
    "12.346867572736": "Otaki Electric Corporation",
    "24.13918718": "Hangzhou H3C Technologies Co., Limited",
    "12.123917679861760": "DEVAU Lemppenau GmbH",
    "12.119269371904": "MAGO di Della Mora Walter",
    "24.778": "Argus Technologies",
    "24.779": "Hunter Technology, Inc.",
    "24.54040": "SPG Controls",
    "12.346861903872": "TruMedia Measurement Ltd.",
    "24.2360821": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.771": "JAMA Electronics Co., Ltd.",
    "24.772": "Pacific Broadband Communications",
    "24.773": "MSC Vertriebs GmbH",
    "24.774": "Fusion In Tech Co., Ltd.",
    "24.775": "Secure Works, Inc.",
    "24.776": "AM Communications, Inc.",
    "24.777": "Texcel Technology PLC",
    "24.5781625": "Ricoh Company, Ltd.",
    "24.13657546": "Huawei Technologies Co.,Ltd",
    "12.123917684056064": "Winners Digital Corporation",
    "12.346860892160": "Ettus Research LLC",
    "24.7419": "ARRIS Group, Inc.",
    "24.7418": "Alarm.com",
    "24.6309214": "Liptel s.r.o.",
    "24.7411": "Evs Broadcast Equipment",
    "24.7410": "Tenlon Technology Co.,Ltd.",
    "24.7413": "Wiseblue Technology Limited",
    "24.7412": "Media Technology Systems Inc",
    "24.7415": "AudioScience",
    "24.7414": "Cisco Systems, Inc",
    "24.7417": "Cisco Systems, Inc",
    "24.7416": "Parade Technologies, Ltd.",
    "24.2629": "Audio-Technica Corp.",
    "24.2628": "Avery Dennison Deutschland GmbH",
    "24.480": "Fast Systems, Inc.",
    "24.481": "Kinpo Electronics, Inc.",
    "24.486": "Hewlett Packard",
    "24.487": "Hewlett Packard",
    "24.484": "Sitera, Inc.",
    "24.485": "Supernet, Inc.",
    "24.2621": "Elo Sistemas Eletronicos S.A.",
    "24.2620": "Enerpoint Ltd.",
    "24.488": "Force10 Networks, Inc.",
    "24.489": "Litton Marine Systems B.V.",
    "24.2625": "Cisco Systems, Inc",
    "24.2624": "Crown Audio -- Harmanm International",
    "24.2627": "Chunghwa Telecom Co., Ltd.",
    "24.2626": "Cisco Systems, Inc",
    "24.5813": "ARRIS Group, Inc.",
    "24.5812": "Private",
    "24.5811": "Photonicbridges (China) Co., Ltd.",
    "24.5810": "DriveCam Inc",
    "24.5817": "ProCurve Networking by HP",
    "24.5816": "Sony Mobile Communications Inc",
    "24.5815": "Seoul Commtech",
    "24.5814": "Cisco-Linksys, LLC",
    "24.5819": "Law-Chain Computer Technology Co Ltd",
    "24.5818": "Weathernews Inc.",
    "12.346855043072": "Perax",
    "24.11071174": "Muuselabs NV/SA",
    "24.1853231": "Huawei Device Co., Ltd.",
    "12.123917676720128": "Telairity Semiconductor",
    "24.2937571": "Guangzhou Aoshi Electronic Co.,Ltd",
    "24.15494298": "Arcadyan Corporation",
    "12.123917685972992": "Pietro Fiorentini",
    "24.2937575": "Nokia Corporation",
    "12.123917680230400": "Mettler Toledo",
    "24.5260893": "ASUSTek COMPUTER INC.",
    "12.123917685534720": "ambiHome GmbH",
    "24.6064028": "Intel Corporate",
    "24.12871888": "VTech Telecommunications Ltd.",
    "24.5254003": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.8688544": "Huawei Device Co., Ltd.",
    "24.14186623": "Ubee Interactive Co., Limited",
    "12.123917689716736": "Olssen",
    "24.8962020": "Huawei Technologies Co.,Ltd",
    "20.9814046408704": "Guard RFID Solutions",
    "12.346866499584": "RFL Electronics, Inc.",
    "24.3679949": "Andtek",
    "24.9492562": "Comtec Co., Ltd.",
    "12.123917691129856": "Power Diagnostic Service",
    "24.7878288": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "12.123917689430016": "SwineTech, Inc.",
    "12.119269269504": "Mercury HMI Ltd",
    "12.123917678120960": "Consospy",
    "12.123917681881088": "Peek Traffic",
    "24.7634458": "Red Embedded Design Limited",
    "12.123917685932032": "Controles S.A.",
    "24.6817226": "Intel Corporate",
    "12.123917675958272": "VNT electronics s.r.o.",
    "24.8476": "Cisco Systems, Inc",
    "24.8477": "Dataline AB",
    "24.8474": "LInTech Corporation",
    "24.8475": "Cisco Systems, Inc",
    "24.8472": "Athena Tech, Inc.",
    "24.8473": "Samsung Electro Mechanics Co., Ltd.",
    "24.8470": "Transcon Electronic Systems, spol. s r. o.",
    "24.8471": "Tellord",
    "12.346856873984": "Energie Umwelt Systemtechnik GmbH",
    "12.123917684887552": "MSB Elektronik und Gerätebau GmbH",
    "24.8478": "ARRIS Group, Inc.",
    "24.8479": "Shinsung Deltatech Co.,Ltd.",
    "24.11583632": "Chicony Electronics Co., Ltd.",
    "12.123917682962432": "Mitsubishi Electric India Pvt. Ltd.",
    "24.13118358": "Espressif Inc.",
    "20.97843029213184": "Spang Power Electronics",
    "24.15483160": "XIAOMI Electronics,CO.,LTD",
    "12.346862718976": "OJSC Rawenstvo",
    "24.3408539": "Plexonics Technologies LImited",
    "24.4898879": "Wuhan Hikstorage Technology Co.,Ltd",
    "24.5431": "Ventus Networks",
    "24.5101": "Psia",
    "24.5100": "Netsnapper Technologies SARL",
    "24.5103": "Kingjon Digital Technology Co.,Ltd",
    "24.5102": "JBX Designs Inc.",
    "24.5105": "AMOD Technology Co., Ltd.",
    "24.5104": "Wavefront Semiconductor",
    "24.5107": "Giga-byte Communications Inc.",
    "24.5106": "Klas Ltd",
    "24.5109": "Akimbi Systems",
    "24.5108": "Psitek (Pty) Ltd",
    "12.346854023168": "Remco Italia Spa",
    "12.346864693248": "Wanco Inc.",
    "12.71297885016064": "Triamec Motion AG",
    "12.123917684563968": "Creotech Instruments S.A.",
    "20.141268004896768": "Quantel USA, Inc.",
    "12.123917687418880": "TT Group SRL",
    "24.14180756": "Objetivos y Sevicios de Valor Añadido",
    "20.176726074195968": "Messtechnik Sachs GmbH",
    "24.8409191": "W & D Technology Corporation",
    "24.4256787": "Rubezh",
    "24.8948573": "Storage Appliance Corporation",
    "24.1059649": "Samsung Electronics Co.,Ltd",
    "24.13911625": "Huawei Technologies Co.,Ltd",
    "20.277783354802176": "Cliptech Industria e Comercio Ltda",
    "24.14699430": "Huawei Technologies Co.,Ltd",
    "20.39986080514048": "Blue Iris Labs",
    "20.62636959989760": "Shenzhen MADIGI Electronic Technology Co., Ltd",
    "24.529711": "Samsung Electronics Co., Ltd. ARTIK",
    "24.4463030": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "20.44169486663680": "Systec Intelligent Building Technology (Tianjin) Co.,Ltd.",
    "12.123917680660480": "Clecell",
    "24.14429802": "HCT. Co., Ltd.",
    "12.346868555776": "Power-One",
    "12.123917680001024": "OOO Research and Production Center \"Computer Technologies\"",
    "24.15483787": "Yaptv",
    "24.465": "CoNet Communications, Inc.",
    "12.123917681889280": "Peek Traffic",
    "24.4256257": "Sagemcom Broadband SAS",
    "24.316309": "Motorola Mobility LLC, a Lenovo Company",
    "12.119270043648": "Wizzilab Sas",
    "24.7885410": "Shenzhen Hojy Software Co., Ltd.",
    "12.123917686030336": "SOREL GmbH Mikroelektronik",
    "12.346854883328": "Harvad University",
    "24.10228835": "Zhejiang Dahua Technology Co., Ltd.",
    "20.127937504346112": "EDOMO Systems GmbH",
    "24.10228837": "Edata Elektronik San. ve Tic. A.Ş.",
    "24.4730064": "Huawei Technologies Co.,Ltd",
    "24.12100687": "Axis Communications AB",
    "12.346852921344": "Helmut Mauell GmbH Werk Weida",
    "24.6585729": "Yokota Industrial Co,.Ltd",
    "24.14475302": "Patrol Tag, Inc",
    "24.10487335": "NEXPA System",
    "12.346859229184": "Mitron Oy",
    "24.3967717": "Tensun Information Technology(Hangzhou) Co.,LTD",
    "24.10280049": "Hewlett Packard Enterprise",
    "12.123917677690880": "GETT Geraetetechnik GmbH",
    "24.1110514": "LG Electronics (Mobile Communications)",
    "20.154314899193856": "Dantherm Cooling Inc.",
    "24.9549": "Singapore Technologies Electronics Limited",
    "24.5506358": "Vivago Oy",
    "24.9469626": "Utek Technology (Shenzhen) Co.,Ltd",
    "24.7096769": "o2ones Co., Ltd.",
    "24.5001826": "Fujitsu Technology Solutions GmbH",
    "20.114869217001472": "Ametek Solidstate Controls",
    "20.132855973806080": "Lyngsoe Systems",
    "20.167152467312640": "Samwontech",
    "12.123917690736640": "Software Systems Plus",
    "12.346863992832": "MTD GmbH",
    "24.835391": "Shenzhen Lencotion Technology Co.,Ltd",
    "24.14468130": "Systembase Limited",
    "24.2667435": "Genmark Automation",
    "12.346867671040": "BitWise Controls",
    "24.16549699": "Huawei Technologies Co.,Ltd",
    "24.14197905": "Huawei Device Co., Ltd.",
    "24.319684": "Triumph-Adler Ag",
    "20.62335525847040": "ShenZhen ShuaiXian Electronic Equipment Co.Ltd",
    "24.8964531": "IEEE Registration Authority",
    "12.123917680680960": "Harvard Technology Ltd",
    "24.12870027": "Apple, Inc.",
    "24.14683876": "DWnet Technologies(Suzhou) Corporation",
    "24.1622593": "SonicWall",
    "24.12602871": "Serelec",
    "12.346863902720": "Byres Security Inc",
    "24.14683873": "We Corporation Inc.",
    "20.84187709177856": "Camsat Przemysław Gralak",
    "24.8399252": "Technicolor CH USA Inc.",
    "12.346857938944": "AllSun A/S",
    "24.8953395": "TCT mobile ltd",
    "24.15738404": "Wafa Technologies Co., Ltd.",
    "12.123917677998080": "ProtoPixel SL",
    "24.5279305": "Xiaomi Communications Co Ltd",
    "24.3166461": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.12889585": "GOPEACE Inc.",
    "24.15790902": "Tp-Link Technologies Co.,Ltd.",
    "24.16018692": "ASUSTek COMPUTER INC.",
    "12.346863497216": "SP Controls, Inc",
    "12.123917685886976": "Omsk Manufacturing Association named after A.S. Popov",
    "24.9761064": "Hewlett Packard Enterprise",
    "12.346865844224": "JVL Industri Elektronik",
    "20.251018416750592": "Ottomate International Pvt. Ltd.",
    "12.346852712448": "Validyne Engineering Corp.",
    "24.14424993": "Intel Corporate",
    "24.10229876": "Nokia Danmark A/S",
    "24.12109454": "Samsung Electronics Co.,Ltd",
    "20.141268009091072": "IDair, LLC",
    "24.4233997": "Shenzhen Yourf Kwan Industrial Co., Ltd",
    "24.9448647": "C-MAX Asia Limited",
    "12.346858332160": "PYRAMID Computer GmbH",
    "24.1083020": "ARRIS Group, Inc.",
    "12.346859028480": "Isochron Inc",
    "24.7902440": "Radio Bridge",
    "12.123917687635968": "Tech4Race",
    "24.1334225": "ARRIS Group, Inc.",
    "24.10009687": "Sva Technologies Co.Ltd",
    "12.346860560384": "Private",
    "24.7665757": "Berkeley Nucleonics Corp",
    "20.154019627532288": "Remotec Technology Limited",
    "24.5785915": "Palo Alto Networks",
    "20.97843030261760": "Certis Technology International",
    "12.123917689155584": "ConectaIP Tecnologia S.L.",
    "12.119269740544": "Greatcom AG",
    "24.270490": "Panasonic Corporation AVC Networks Company",
    "12.346854318080": "PYRAMID Computer GmbH",
    "24.5273254": "Ecil Informatica Ind. Com. Ltda",
    "20.5297541218304": "Launch Tech Co., Ltd.",
    "24.10755008": "Fastec Imaging Corporation",
    "24.13647786": "Apple, Inc.",
    "24.16558512": "Miartech (Shanghai),Inc.",
    "24.12338039": "Hang Zhou Huite Technology Co.,ltd.",
    "12.123917684920320": "Coester Automação Ltda",
    "24.2672393": "Apple, Inc.",
    "24.14216286": "Belkin International Inc.",
    "24.2677143": "Yuduan Mobile Co., Ltd.",
    "24.15501588": "Huawei Technologies Co.,Ltd",
    "24.14177699": "zte corporation",
    "12.71297883234304": "Brightwell Dispensers",
    "24.2092": "Homag AG",
    "12.123917682880512": "CHIeru., CO., Ltd.",
    "24.5255797": "Samsung Electronics Co.,Ltd",
    "12.123917682884608": "Shenzhen bayue software co. LTD",
    "24.15528770": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.11286473": "RAID Incorporated",
    "24.9445434": "BYD Precision Manufacture Co.,Ltd",
    "12.119269498880": "ChamSys Ltd",
    "24.5554020": "Dell Inc.",
    "24.14455231": "Amazon Technologies Inc.",
    "24.2659343": "Cisco Systems, Inc",
    "24.32848": "Ziatech Corporation",
    "24.32845": "Cyclone Microsystems, Inc.",
    "24.32844": "Contec Co., Ltd.",
    "24.32847": "Daikin Industries, Ltd.",
    "24.32846": "Apex Computer Company",
    "24.32841": "Nissin Electric Co., Ltd.",
    "24.32840": "Compex, used by Commodore and DEC at least",
    "24.32843": "Eagle Technologies Pty.Ltd.",
    "24.32842": "Pro-Log",
    "24.15515357": "Sagemcom Broadband SAS",
    "24.1609944": "Sagemcom Broadband SAS",
    "24.12618865": "Motorola Mobility LLC, a Lenovo Company",
    "24.826131": "Shanghai Magic Mobile Telecommunication Co.Ltd.",
    "24.9967752": "Cisco Meraki",
    "24.3708128": "Syrotech Networks. Ltd.",
    "12.123917683105792": "Decentlab GmbH",
    "24.14467049": "Motorola Mobility LLC, a Lenovo Company",
    "12.346856984576": "Trellia Networks",
    "24.5774737": "Advanced Mobile Telecom co.,ltd.",
    "20.66426713931776": "Phytium Technology Co., Ltd.",
    "24.2630515": "Digita",
    "24.10279883": "Wuhan Funshion Online Technologies Co.,Ltd",
    "24.14458788": "Nokia Corporation",
    "20.154680187420672": "Gigawave",
    "24.6835080": "Galtronics Telemetry Inc.",
    "12.71297883717632": "Papendorf Software Engineering GmbH",
    "12.123917678411776": "Renesas Electronics",
    "12.346856132608": "Dilax Intelcom AG",
    "20.228784606281728": "iLOQ Oy",
    "24.837700": "Cambridge Mobile Telematics, Inc.",
    "24.10750186": "Murata Manufacturing Co., Ltd.",
    "24.8947677": "DarbeeVision Inc.",
    "12.346860183552": "Asiana Idt",
    "24.11281174": "Mist Systems, Inc.",
    "20.137141730410496": "Maco Lighting Pty. Ltd.",
    "12.346868903936": "Human Intech",
    "12.346868568064": "Intuitive Surgical, Inc",
    "24.5821257": "Apple, Inc.",
    "20.145727910051840": "Zephyr Engineering, Inc.",
    "24.12063963": "Ohsung",
    "24.13931610": "Cosmo Co., Ltd",
    "24.5830335": "Kyoto University",
    "20.66129355603968": "Authentico Technologies",
    "12.346855317504": "EuroDesign embedded technologies GmbH",
    "24.13423586": "Jiangsu Yinhe Electronics Co.,Ltd.",
    "24.285702": "Zmodo Technology Corporation",
    "24.2939252": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "20.145735988281344": "Engisat Lda",
    "24.7118": "Measurement Devices Ltd",
    "20.31372482707456": "gekartel AG",
    "24.4206908": "XIAOMI Electronics,CO.,LTD",
    "20.75247164325888": "Global Telecom Engineering, Inc",
    "24.35002": "Nc&C",
    "24.13931424": "Fidelix Oy",
    "24.8970400": "Shenzhen VisionSTOR Technologies Co., Ltd",
    "12.346865065984": "Elaso AG",
    "24.1315835": "Huawei Technologies Co.,Ltd",
    "12.71297884745728": "Telvent",
    "24.7362177": "Apple, Inc.",
    "12.346865528832": "Megacon AB",
    "12.346855628800": "ABB Power Technologies S.p.A. Unità Operativa SACE (PTMV)",
    "24.7663741": "BnCOM Co.,Ltd",
    "20.141268012236800": "Oliotalo Oy",
    "12.346855284736": "Tornado Modular Systems, Ltd",
    "24.6334256": "Shenzhen Yipinfang Internet Technology Co.,Ltd",
    "24.11326820": "Chongqing Fugui Electronics Co.,Ltd.",
    "20.194600858812416": "IDspire Corporation Ltd.",
    "24.279906": "ANDRA Sp. z o. o.",
    "24.14700899": "Arcadyan Corporation",
    "12.123917690789888": "TextSpeak Corporation",
    "24.4986859": "Sagemcom Broadband SAS",
    "20.211525428051968": "Paragon Robotics LLC",
    "12.71297883348992": "Tinkerforge GmbH",
    "24.12095034": "Infinite Technologies JLT",
    "24.10546304": "zte corporation",
    "12.346860646400": "Odin TeleSystems Inc",
    "24.16015351": "DQ Technology Inc.",
    "24.3933464": "Samsung Electronics Co.,Ltd",
    "20.84197035212800": "Shandong Senter Electronic Co., Ltd",
    "24.576679": "Cisco Systems, Inc",
    "20.62070507700224": "Lightform, Inc.",
    "24.2549": "Emerson Network Power Co.,Ltd",
    "12.123917679636480": "Airborne Engineering Limited",
    "24.8441762": "Huawei Device Co., Ltd.",
    "24.11841135": "Circle Reliance, Inc DBA Cranberry Networks",
    "24.11835724": "Texas Instruments",
    "24.15220475": "Geodesic Ltd.",
    "12.123917684928512": "Engage Technologies",
    "24.1859371": "D-Link International",
    "24.1887129": "TCT mobile ltd",
    "12.346867642368": "Aplex Technology Inc.",
    "20.97843038650368": "Hi-Target Surveying Instrument Co., Ltd.",
    "24.10795013": "Apple, Inc.",
    "24.4197991": "Tremol Ltd.",
    "24.9213535": "Shandong Zhongan Technology Co., Ltd.",
    "12.123917686177792": "West-Com Nurse Call Systems, Inc.",
    "24.13134285": "Apple, Inc.",
    "12.346868051968": "Elan Systems",
    "24.4780155": "Alpsalpine Co,.Ltd",
    "24.10784266": "DeLaval International AB",
    "20.75685679857664": "Simplered Technology Ltd.",
    "20.193649354735616": "Dalian GigaTec Electronics Co.,Ltd",
    "12.346866221056": "HiSpeed Data, Inc.",
    "24.801110": "Observator instruments",
    "24.3176642": "Innowireless / QUCELL Networks",
    "12.71297883627520": "C-COM Satellite Systems Inc.",
    "24.5809721": "Samsung Electronics Co.,Ltd",
    "24.9440456": "Google, Inc.",
    "24.12624290": "MarqMetrix",
    "12.346864427008": "St Michael Strategies Inc",
    "24.1581132": "Kummler+Matter AG",
    "24.16271511": "Hitachi, Ltd.",
    "24.10505307": "Shenzhen TINNO Mobile Technology Corp.",
    "12.346859024384": "Simtek Inc.",
    "24.1313883": "PLNetworks",
    "24.13385936": "Netgear",
    "24.4216194": "Nokia",
    "24.12594298": "Shenzhen Horn Audio Co.,Ltd.",
    "20.168198558187520": "Tonycore Technology Co.,Ltd.",
    "24.12883200": "Amazon Technologies Inc.",
    "12.123917680795648": "Aliter Technologies",
    "12.123917683503104": "bobz GmbH",
    "24.14478163": "Wuhan QianXiao Elecronic Technology CO.,LTD",
    "24.8394423": "Brunel University",
    "12.346858844160": "Invocon, Inc.",
    "24.5815715": "Nintendo Co., Ltd.",
    "24.10544211": "Sony Mobile Communications Inc",
    "12.123917685657600": "QIAGEN Instruments AG",
    "12.123917679263744": "Avant Technologies, Inc",
    "20.93065826009088": "Chengdu EVECCA Technology Co.,Ltd.",
    "20.97400608784384": "SuZhou Ruishengwei Intelligent Technology Co.,Ltd",
    "24.8688780": "Hitron Technologies. Inc",
    "12.123917682724864": "Nanjing Magewell Electronics Co., Ltd.",
    "12.123917685022720": "Autocom Diagnostic Partner AB",
    "24.1580664": "Sagemcom Broadband SAS",
    "24.9974941": "Krauss Maffei Technologies GmbH",
    "24.5550595": "Pegatron Corporation",
    "24.6342061": "Samsung Electronics Co.,Ltd",
    "24.45250": "Cisco Systems, Inc",
    "24.45255": "Tellabs Operations, Inc.",
    "12.123917681737728": "KRISTECH Krzysztof Kajstura",
    "20.198237805871104": "G4S Monitoring Technologies Ltd",
    "24.12352191": "Samsung Electronics Co.,Ltd",
    "12.123917683769344": "Robot Pub Group",
    "24.14714266": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.50651": "Datatech Sistemas Digitales Avanzados SL",
    "24.11311766": "Maxlinear, Inc",
    "24.8666745": "Shenzhen Belon Technology CO.,LTD",
    "12.123917681209344": "Loma Systems s.r.o.",
    "24.14983654": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.11292726": "Pi-Coral Inc",
    "12.346867929088": "Wheatstone Corporation",
    "12.123917688598528": "Gentec Electro-Optics",
    "12.123917688016896": "Az-Technology Sdn Bhd",
    "24.1840": "Hutchison OPTEL Telecom Technology Co., Ltd.",
    "24.1841": "Ophir-Spiricon LLC",
    "24.1842": "AAEON Technology Inc.",
    "24.1843": "DANCONTROL Engineering",
    "24.1844": "ONStor, Inc.",
    "24.1845": "Flarion Technologies, Inc.",
    "24.5349": "Alticast",
    "24.5348": "infinias, LLC",
    "24.1848": "Young Technology Co., Ltd.",
    "24.5346": "datacom systems inc.",
    "24.5345": "Data Display AG",
    "24.5344": "LET'S Corporation",
    "24.5343": "HI-P Tech Corporation",
    "24.5342": "Sage Instruments Inc.",
    "24.5341": "Covergence Inc.",
    "24.5340": "Communication System Design & Manufacturing (CSDM)",
    "24.3187394": "Huawei Device Co., Ltd.",
    "12.123917687939072": "Teletask",
    "24.5532376": "Elster Water Metering",
    "24.8697958": "Beijing XiaoLu technology co. LTD",
    "24.14452190": "AzureWave Technology Inc.",
    "20.238070233300992": "Library Ideas",
    "24.3418386": "Huawei Technologies Co.,Ltd",
    "24.2936934": "Xiaomi Communications Co Ltd",
    "24.4754742": "Arcadyan Corporation",
    "24.5699": "Sunhillo Corporation",
    "24.5698": "Pangolin",
    "12.346868150272": "DENSEI COMMUNICATION Inc.",
    "12.123917686345728": "Sensile Technologies SA",
    "12.346855497728": "Manusa-Gest, S.L.",
    "24.5691": "Communications & Power Industries",
    "24.5690": "Yves Technology Co., Ltd.",
    "24.5693": "Tsinghua Tongfang Legend Silicon Tech. Co., Ltd.",
    "24.5692": "Rebox B.V.",
    "24.5695": "CReTE SYSTEMS Inc.",
    "24.5694": "Xensource, Inc.",
    "24.5697": "Universal Global Scientific Industrial Co., Ltd.",
    "24.5696": "Asmobile Communication Inc.",
    "20.66219402067968": "Private",
    "24.10241379": "DIMEP Sistemas",
    "24.15006557": "Huawei Technologies Co.,Ltd",
    "12.346862161920": "Psigenics Corporation",
    "24.1622150": "Broadcom",
    "12.123917679501312": "Canon Electron Tubes & Devices Co., Ltd.",
    "12.123917680123904": "Transit Solutions, LLC.",
    "20.211670367469568": "Gemvax Technology ,. Co.Ltd",
    "12.346854154240": "United Tri-Tech Corporation",
    "24.6079239": "Huawei Technologies Co.,Ltd",
    "24.9479390": "Wistron Neweb Corporation",
    "24.12346945": "IEEE Registration Authority",
    "24.1329632": "Samsung Electro-Mechanics(Thailand)",
    "12.346864226304": "GHL Advanced Technolgy GmbH & Co. KG",
    "24.16013118": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346853494784": "ELC Lighting",
    "24.14428474": "Shenzhen Nuoshi Technology Co., LTD.",
    "20.102196713619456": "BrightSky, LLC",
    "24.12607087": "V. Stonkaus firma \"Kodinis Raktas\"",
    "24.548974": "ASUSTek COMPUTER INC.",
    "12.123917691166720": "Guan Show Technologe Co., Ltd.",
    "24.12084114": "Guangdong Genius Technology Co., Ltd.",
    "24.14949677": "Mellanox Technologies, Inc.",
    "24.3181490": "Cisco Systems, Inc",
    "24.5013583": "Armstrong Monitoring Corp",
    "24.15482547": "Zyxel Communications Corporation",
    "24.16299368": "IEEE Registration Authority",
    "12.346856640512": "MICRONIC s.r.o.",
    "24.5017292": "Silkan Sa",
    "20.62070508748800": "Shenzhen Cse Technology Co., Ltd",
    "24.9752601": "Cydle Corp.",
    "24.7894": "Alentec & Orion AB",
    "20.216145522065408": "Teletronik AG",
    "12.123917683826688": "Stealth Communications",
    "24.2126238": "SF Technology Co.,Ltd",
    "12.346853330944": "Destiny Networks, Inc.",
    "24.4568": "ASUSTek COMPUTER INC.",
    "24.1853819": "Gemtek Technology Co., Ltd.",
    "20.48476178087936": "Cloudwalk Technology Co.,Ltd",
    "24.4743356": "Apple, Inc.",
    "24.11801830": "Huawei Technologies Co.,Ltd",
    "24.3470235": "WizLAN Ltd.",
    "24.3470234": "Intel Corporate",
    "24.4562": "Perception Digital Ltd",
    "24.4563": "NextGenTel Holding ASA",
    "24.4564": "NetEnrich, Inc",
    "24.4565": "Hangzhou Sunyard System Engineering Co.,Ltd.",
    "24.4566": "HandEra, Inc.",
    "24.4567": "eWerks Inc",
    "24.3693": "Murata Manufacturing Co., Ltd.",
    "24.3692": "Device Drivers Limited",
    "24.3691": "Janitza electronics GmbH",
    "24.3690": "3Com Ltd",
    "24.3697": "Gemstar Technology Development Ltd.",
    "24.3696": "in2 Networks",
    "24.3695": "IRIS Corporation Berhad",
    "24.3694": "MAT S.A. (Mircrelec Advanced Technology)",
    "24.1052801": "zte corporation",
    "24.3699": "Tpack A/S",
    "24.3698": "CTS electronics",
    "24.588778": "AMPAK Technology,Inc.",
    "24.13553": "Radicom Research, Inc.",
    "24.57440": "Sherwood",
    "24.57441": "EdgePoint Networks, Inc.",
    "24.57442": "Host Engineering",
    "24.57443": "Cabletron Systems, Inc.",
    "24.57444": "Samsung Electronics",
    "24.57445": "Optical Access International",
    "24.57446": "ProMax Systems, Inc.",
    "24.57447": "eac AUTOMATION-CONSULTING GmbH",
    "24.57448": "Merrimac Systems Inc.",
    "24.57449": "Jaycor",
    "24.2391570": "Cisco Systems, Inc",
    "24.16300774": "Nokia",
    "24.16037773": "Huawei Technologies Co.,Ltd",
    "24.20550": "Menicx International Co., Ltd.",
    "24.20551": "Private",
    "24.20552": "Infolibria",
    "24.20553": "Arbor Networks Inc",
    "24.20554": "Elteco A.S.",
    "24.20555": "Barconet N.V.",
    "24.20556": "Galil Motion Control",
    "24.20557": "Tokyo Electron Device Limited",
    "24.20558": "Sierra Monitor Corp.",
    "24.20559": "Olencom Electronics",
    "24.16521680": "Cloud Vision Networks Technology Co.,Ltd.",
    "24.9757100": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.6312727": "Sandstone Technologies",
    "24.11557735": "Huawei Technologies Co.,Ltd",
    "20.272805456248832": "Dacons",
    "24.13125516": "Beken Corporation",
    "24.8929383": "infomark",
    "24.2361873": "TCT mobile ltd",
    "24.15222968": "TP-Link Corporation Limited",
    "12.346853322752": "Digital Vision AB",
    "20.246679820304384": "Aitec System Co., Ltd.",
    "20.140782591803392": "Beijing Gooagoo Technical Service Co.,Ltd.",
    "12.123917684060160": "Loehnert Elektronik GmbH",
    "24.13381489": "Intel Corporate",
    "12.346862501888": "FTM Marketing Limited",
    "12.123917689032704": "Vitec",
    "24.5278411": "Shenzhen Maxmade Technology Co., Ltd.",
    "24.49205": "Quintar Company",
    "24.49204": "Transaction Network",
    "24.49207": "Dynatem",
    "24.49206": "Raytech Electronic Corp.",
    "24.49201": "Design Research Systems, Inc.",
    "24.49200": "Integrated Engineering B. V.",
    "24.49203": "Telebit Corporation",
    "24.49202": "I-Cubed Limited",
    "24.49209": "Teridian Semiconductor Corporation",
    "24.49208": "Raster Image Processing System",
    "12.346856046592": "Contrôle Analytique inc.",
    "24.3037": "TOHOKU RICOH Co., LTD.",
    "24.3036": "Akcp",
    "24.3035": "Dell Inc.",
    "24.3034": "EyeCross Co.,Inc.",
    "24.3033": "General Hydrogen",
    "24.3032": "Industrial Scientific Corp.",
    "24.3031": "DORMA Time + Access GmbH",
    "24.3030": "Paxton Access Ltd",
    "24.3039": "Shenzhen RouterD Networks Limited",
    "24.3038": "TELDIX GmbH",
    "24.9203024": "Shenzhen Mtc Co Ltd",
    "24.3963530": "ARRIS Group, Inc.",
    "24.6645": "Imagination Technologies Ltd",
    "24.9981139": "Mantis Deposition",
    "24.6644": "Convergens Oy Ltd",
    "24.10515946": "BoxLock, Inc.",
    "24.11591740": "TCT mobile ltd",
    "24.8780": "Nintendo Co., Ltd.",
    "24.8781": "Mitac International Corp.",
    "24.8782": "SEEnergy Corp.",
    "24.8783": "Byzoro Networks Ltd.",
    "24.8784": "Point Six Wireless, LLC",
    "24.8785": "Lumasense Technologies",
    "24.8786": "ZOLL Lifecor Corporation",
    "24.8787": "Entorian Technologies",
    "24.8788": "Bigelow Aerospace",
    "24.8789": "Cisco Systems, Inc",
    "24.2141526": "Continental Automotive Systems Inc.",
    "20.246679822401536": "Link of Things Co., Ltd.",
    "12.346854035456": "modas GmbH",
    "12.123917689126912": "Nxvi Microelectronics Technology (Jinan) Co., Ltd.",
    "24.2128": "Arizona Instrument Corp.",
    "24.2129": "Canadian Bank Note Company, Ltd.",
    "20.136823638589440": "Bidgely",
    "24.1362195": "LG Electronics",
    "24.2126": "DivergeNet, Inc.",
    "24.2127": "Qualstar Corporation",
    "24.3169702": "ADVALY SYSTEM Inc.",
    "12.346856177664": "Wisnu and Supak Co.,Ltd.",
    "24.12388": "Adlink Technology, Inc.",
    "24.12389": "Apple, Inc.",
    "12.346856632320": "Pickering Laboratories",
    "24.3954481": "Cisco Systems, Inc",
    "24.12380": "SMAR Laboratories Corp.",
    "24.12381": "Digitra Systems, Inc.",
    "24.12382": "Abelko Innovation",
    "24.12383": "Hasselblad",
    "24.12384": "Powerfile, Inc.",
    "24.12385": "MobyTEL",
    "24.12386": "IP Video Networks Inc",
    "24.12387": "Santera Systems, Inc.",
    "12.123917681233920": "Fater Rasa Noor",
    "24.3950903": "ASSMANN Electronic GmbH",
    "24.540711": "Gridstore Inc.",
    "12.123917690527744": "COMM-connect A/S",
    "24.16539803": "New H3C Technologies Co., Ltd",
    "24.5293741": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.5265064": "Jolla Ltd",
    "24.14482178": "Buffalo.Inc",
    "20.84197036261376": "5Voxel Co., Ltd.",
    "20.251330457239552": "Red Point Positioning, Corp.",
    "24.7089372": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.13388659": "AMPAK Technology, Inc.",
    "12.123917687529472": "Twin Development",
    "12.346856677376": "QQ Technology,Inc",
    "24.1373226": "Samsung Electronics Co.,Ltd",
    "24.13143846": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.24662": "Network Tools, Inc.",
    "24.24663": "Murata Manufacturing Co., Ltd.",
    "24.24660": "Controlware Gmbh",
    "24.24661": "Cornell University",
    "24.24666": "Celcore, Inc.",
    "24.24667": "IntraServer Technology, Inc.",
    "24.2788": "Wistron Corporation",
    "24.24665": "Technical Communications Corp.",
    "24.2786": "Binatone Electronics International, Ltd",
    "24.2787": "Yang Mei Technology Co., Ltd",
    "24.2784": "Fujitsu Softek",
    "24.24669": "Scanivalve Corp.",
    "24.2782": "Happy Communication Co., Ltd.",
    "24.2783": "Gennum Corporation",
    "24.2780": "RuggedCom Inc.",
    "20.368742236160": "Donguan WideLink Communication Technology Co.,Ltd.",
    "24.30605": "Cisco Systems, Inc",
    "24.1614474": "Essel-T Co., Ltd",
    "24.9668": "Ruckus Wireless",
    "24.9669": "Star Link Communication Pvt. Ltd.",
    "12.123917689507840": "Cooltera Limited",
    "24.9664": "ZillionTV Corporation",
    "24.9665": "Nawoo Korea Corp.",
    "24.9666": "RingBell Co.,Ltd.",
    "24.9667": "21168",
    "24.9660": "Apple, Inc.",
    "24.9661": "Italdata Ingegneria dell'Idea S.p.A.",
    "24.9662": "Tektrap Systems Inc.",
    "24.9663": "Wireless Cables Inc.",
    "24.1479": "I/F-Com A/S",
    "24.1478": "Triz Communications",
    "20.127658783408128": "BÄR Bahnsicherung AG",
    "24.7887986": "Hioso Technology Co., Ltd.",
    "24.1471": "JustEzy Technology, Inc.",
    "24.1470": "Kongsberg Seatex AS",
    "24.1473": "A-Kyung Motion, Inc.",
    "24.1472": "Digital Network Alacarte Co., Ltd.",
    "24.1475": "Pacific Instruments, Inc.",
    "24.1474": "Soronti, Inc.",
    "24.1477": "Flaga HF",
    "24.1476": "Telect, Inc.",
    "24.8172157": "Intel Corporate",
    "12.123917690032128": "BELT S.r.l.",
    "24.2919410": "Hitachi Metals America Ltd",
    "20.71389204709376": "Knox Company",
    "12.123917676339200": "3City Electronics",
    "24.2878": "BittWare, Inc",
    "24.2879": "Anthology Solutions Inc.",
    "24.14162964": "Compacta International, Ltd.",
    "24.5308313": "IEEE Registration Authority",
    "12.346856140800": "Analytica GmbH",
    "24.2870": "Productivity Systems, Inc.",
    "12.346857844736": "Dataton Utvecklings AB",
    "24.2872": "Knürr GmbH",
    "24.2873": "Keisoku Giken Co.,Ltd.",
    "24.2874": "QuStream Corporation",
    "24.2875": "devolo AG",
    "24.2876": "Cygnal Integrated Products, Inc.",
    "24.1055077": "New H3C Technologies Co., Ltd",
    "24.1783": "Alpsalpine Co,.Ltd",
    "24.1782": "Cisco Systems, Inc",
    "24.1781": "Alpsalpine Co,.Ltd",
    "24.1780": "Prime Electronics & Satellitics Inc.",
    "24.1787": "Hitachi Printing Solutions, Ltd.",
    "24.1786": "IP SQUARE Co, Ltd.",
    "24.1785": "Mitsui Zosen Systems Research Inc.",
    "24.1784": "The Boeing Company",
    "24.1789": "Comjet Information Systems Corp.",
    "24.1788": "Fnet Co., Ltd.",
    "24.3949454": "ARRIS Group, Inc.",
    "24.2672408": "Altierre",
    "24.11024217": "Huawei Device Co., Ltd.",
    "24.14960529": "Iris ID Systems, Inc.",
    "24.2632598": "Huawei Device Co., Ltd.",
    "24.4479391": "Criticare Systems, Inc",
    "20.216039220576256": "Ati",
    "24.8403545": "At&T",
    "12.346856095744": "EDA Industries Pte. Ltd",
    "24.9190474": "NAKAYO Inc",
    "12.123917679841280": "Shenzhen Chanslink Network Technology Co., Ltd",
    "20.167152465215488": "Gs Global Security Inc",
    "12.123917691392000": "BBI Engineering, Inc.",
    "24.524559": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.4460798": "Huizhou Foryou General Electronics Co., Ltd.",
    "24.3461110": "Intel Corporate",
    "12.346864979968": "Secad Sa",
    "20.159304576401408": "Shenzhen Dingsheng Intelligent Technology Co., Ltd",
    "12.346865881088": "MSR-Office GmbH",
    "24.12067407": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.8205": "Carl Zeiss",
    "24.8204": "Adastra Systems Corp.",
    "24.8207": "EBRAINS Inc",
    "24.8206": "NSSLGlobal Technologies AS",
    "24.8201": "Packard Bell Elec., Inc.",
    "24.178945": "Octothorpe Corp.",
    "24.8203": "Octagon Systems Corp.",
    "12.346862309376": "Chronos Technology Ltd.",
    "24.14730227": "zte corporation",
    "24.3428201": "Ekinops Sas",
    "24.8208": "Jeol System Technology Co. Ltd",
    "12.346862329856": "Mesa Labs, Inc.",
    "24.4950": "FLIR Radiation Inc",
    "24.4953": "ProTelevision Technologies A/S",
    "24.4952": "Realm Systems, Inc.",
    "24.4955": "PanelLink Cinema, LLC",
    "24.4954": "Project T&E Limited",
    "24.4957": "NTTPC Communications, Inc.",
    "24.4956": "OnSite Systems, Inc.",
    "24.4959": "Cisco Systems, Inc",
    "24.4958": "Eab/Rwi/K",
    "24.6864836": "Samsung Electronics Co.,Ltd",
    "24.16525908": "Connected Data, Inc.",
    "24.14203126": "Nantworks",
    "24.9198320": "Exeltech Solar Products",
    "24.1612839": "ARRIS Group, Inc.",
    "12.71297884024832": "Clark-MXR, Inc.",
    "12.346861535232": "Xentras Communications",
    "24.8151114": "Avelon",
    "20.168198564478976": "HIROIA Communications Pte. Ltd. Taiwan Branch",
    "24.5768187": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "20.225135232548864": "Byterg Llc",
    "20.66426702397440": "Rio Lago Technologies LLC",
    "20.167860154400768": "Comarch S.A.",
    "24.6618252": "Zonar Systems",
    "24.12899925": "Tercel technology co.,ltd",
    "12.346863648768": "Dixell S.p.A.",
    "12.123917683433472": "Sensing Labs",
    "24.1844996": "zte corporation",
    "24.2935829": "Nokia Corporation",
    "12.123917676621824": "Aplex Technology Inc.",
    "12.123917676228608": "RIC Electronics",
    "12.346854223872": "ARC'Créations",
    "12.346868617216": "Matelex",
    "20.39986082611200": "Suzhou Jiangzhi electronic technology co., Ltd",
    "24.2948531": "TCL Technoly Electronics(Huizhou).,Ltd",
    "24.2683750": "Shenzhen Bilian electronic CO.,LTD",
    "24.2118906": "Huawei Technologies Co.,Ltd",
    "24.6563639": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.3421780": "zte corporation",
    "24.14160485": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.346856919040": "EREE Electronique",
    "24.14732110": "ASUSTek COMPUTER INC.",
    "24.2899723": "Cisco Meraki",
    "20.49277255548928": "Zenic Inc.",
    "24.12094188": "Nintendo Co.,Ltd",
    "20.62070512943104": "Shenzhen Jixian Technology Co., Ltd.",
    "24.3467412": "Tp-Link Technologies Co.,Ltd.",
    "12.123917689323520": "Tagarno AS",
    "12.346863255552": "Par-Tech, Inc.",
    "24.12878414": "Changzhou Uchip Electronics Co., LTD.",
    "12.123917678968832": "ABC Electric Co.",
    "12.346865197056": "TBS Holding AG",
    "12.346866524160": "SoftwareCannery",
    "24.6843976": "Prophet Electronic Technology Corp.,Ltd",
    "24.10489985": "Samsung Electronics Co.,Ltd",
    "24.6603486": "LG Electronics (Mobile Communications)",
    "24.9993203": "Texas Instruments",
    "20.48688638459904": "Progress Rail Services, Inspection and Information Systems",
    "24.10275068": "ARRIS Group, Inc.",
    "24.816492": "Anaren Microwave, Inc.",
    "12.346854551552": "Gradual Tecnologia Ltda.",
    "24.16520009": "Global Apps Corp.",
    "24.9739261": "Trans New Technology, Inc.",
    "12.123917687873536": "eumig industrie-TV GmbH.",
    "24.8663786": "Anhui Wanzten P&T Co., Ltd",
    "24.12635202": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "20.84197031018496": "Diehl Controls Nanjing Co., Ltd.",
    "24.13113145": "Itel Mobile Limited",
    "20.39919502229504": "Chunghsin Technology Group Co.,Ltd",
    "24.7892459": "GE Lighting",
    "12.346858295296": "Crimson Microsystems, Inc.",
    "20.167295462670336": "Nanjing BianYu Future Home Technology Co.Ltd",
    "12.346855624704": "BMK professional electronics GmbH",
    "24.10271984": "Xiaomi Communications Co Ltd",
    "24.10530749": "Intel Corporate",
    "24.15259493": "Huawei Technologies Co.,Ltd",
    "24.15787459": "Drägerwerk AG & Co. KG aA",
    "24.11593874": "Seiko Epson Corporation",
    "24.820517": "Cisco Systems, Inc",
    "12.71297883676672": "4embedded",
    "24.12866628": "Quanta Computer Inc.",
    "12.123917689593856": "Advanced Ventilation Applications, Inc.",
    "24.3933863": "Tp-Link Technologies Co.,Ltd.",
    "24.318346": "Suntech International Ltd.",
    "12.123917690781696": "ModulaTeam GmbH",
    "12.123917689454592": "Enertex Bayern GmbH",
    "12.71297883840512": "Xemex NV",
    "12.346861809664": "Saia-Burgess Controls AG",
    "24.15773981": "Nokia",
    "24.9187810": "Dayouplus",
    "12.123917689823232": "Federated Wireless, Inc.",
    "12.346868322304": "Safe Place Solutions Ltd",
    "20.71070630543360": "Amobile Intelligent Corp.",
    "20.198611829784576": "Current Ways, Inc.",
    "12.346863603712": "ClearCorp Enterprises, Inc",
    "12.123917689585664": "Nation-E Ltd.",
    "24.4755183": "Huawei Technologies Co.,Ltd",
    "24.3452716": "Nintendo Co., Ltd.",
    "12.346856996864": "Envisacor Technologies Inc.",
    "12.123917684629504": "Medical Monitoring Center OOD",
    "24.10278467": "D-Link International",
    "24.3154994": "Intel Corporate",
    "20.5297548558336": "Chengdu Interlink Science And Technology Co.,Ltd",
    "12.346858614784": "JungleSystem Co., Ltd.",
    "20.212014915911680": "Zhinengguo technology company limited",
    "12.346854006784": "Weltronics Corp.",
    "12.123917691269120": "Tata Communications Ltd.",
    "12.123917686943744": "Ideas srl",
    "24.12360901": "Beijing Huafei Technology Co., Ltd.",
    "24.583305": "Apple, Inc.",
    "12.346856472576": "Infrasafe, Inc.",
    "20.141268015382528": "Mersen",
    "24.8943731": "Intel Corporate",
    "24.9740095": "Optek Digital Technology company limited",
    "24.9740094": "Sonos, Inc.",
    "12.123917686718464": "Elman srl",
    "24.3988010": "zte corporation",
    "24.16055595": "ZOYI Company",
    "24.9099814": "Eleven Inc.",
    "24.7369423": "EdgeCore Networks",
    "24.8692771": "Sagemcom Broadband SAS",
    "24.5005266": "Alcatel-Lucent",
    "12.346863153152": "Orange Tree Technologies",
    "24.14194556": "Grey Innovation",
    "12.123917680222208": "Thruvision Limited",
    "20.154680177983488": "H3 Platform",
    "24.8939065": "Huawei Technologies Co.,Ltd",
    "24.9437720": "BSkyB Ltd",
    "12.119269662720": "IST GmbH",
    "12.346867228672": "Orep",
    "24.4484132": "Reallin Electronic Co.,Ltd",
    "12.123917680349184": "MB connect line GmbH Fernwartungssysteme",
    "24.10236585": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.346865872896": "Gunma Electronics Co Ltd",
    "24.8403958": "Look Easy International Limited",
    "24.563032": "Shenzhen Strong Rising Electronics Co.,Ltd DongGuan Subsidiary",
    "20.48543319457792": "Motec GmbH",
    "12.346853994496": "International Test & Engineering Services Co.,Ltd.",
    "12.346864406528": "Prediktor AS",
    "12.119269548032": "Mitsubishi Electric India PVT. LTD",
    "24.12845518": "Presition (2000) Co., Ltd.",
    "12.71297884008448": "ReliOn Inc",
    "24.16571286": "Enervalley Co., Ltd",
    "12.123917685968896": "Taejin InfoTech",
    "12.123917688979456": "JL World Corporation Limited",
    "24.15768809": "Extreme Networks, Inc.",
    "24.11843725": "Thuh Company",
    "24.15783511": "Echosens",
    "12.123917685452800": "Foerster-Technik GmbH",
    "24.7665946": "Onface",
    "24.12083120": "Huawei Technologies Co.,Ltd",
    "24.15510010": "Guangdong Genius Technology Co., Ltd.",
    "24.12094961": "Nanjing BigFish Semiconductor Co., Ltd.",
    "24.7349765": "SONY Visual Products Inc.",
    "24.2159698": "Shanghai IUV Software Development Co. Ltd",
    "12.123917676433408": "Vtron Pty Ltd",
    "24.14442038": "Paterson Technology",
    "24.2159695": "Nokia",
    "20.452705910784": "Comcast-SRL",
    "12.123917678174208": "Lemz-T, Llc",
    "24.7371800": "ADATA Technology Co., Ltd",
    "24.10763022": "Huawei Device Co., Ltd.",
    "24.7080836": "Universal Global Scientific Industrial Co., Ltd.",
    "24.10259738": "INTEG process group inc",
    "24.3968064": "Hangzhou H3C Technologies Co., Limited",
    "24.13134983": "Zyxel Communications Corporation",
    "24.9736208": "Huawei Technologies Co.,Ltd",
    "24.2402268": "BSkyB Ltd",
    "24.8390914": "Keysight Technologies, Inc.",
    "24.9977696": "Huawei Technologies Co.,Ltd",
    "24.4456521": "Amazon Technologies Inc.",
    "12.346868367360": "Thor Technologies Pty Ltd",
    "24.4456525": "Huawei Technologies Co.,Ltd",
    "24.7093331": "SoundHawk Corp",
    "24.32857": "Stanley Electric Co., Ltd",
    "24.32854": "SPHINX Electronics GmbH & Co KG",
    "24.9210234": "Augtek",
    "12.123917676920832": "Brinkmann Audio GmbH",
    "24.14462824": "Weifang Goertek Electronics Co.,Ltd",
    "24.1347970": "Belkin International Inc.",
    "20.52821131001856": "Thermo Process Instruments, LP",
    "24.16541107": "Huawei Device Co., Ltd.",
    "20.137215126536192": "Aplex Technology Inc.",
    "12.123917685653504": "Peek Traffic",
    "24.10546637": "Apple, Inc.",
    "24.10505409": "Helixtech Corp.",
    "20.35227539865600": "halstrup-walcher GmbH",
    "24.13155484": "Huawei Device Co., Ltd.",
    "24.10807950": "Extreme Networks, Inc.",
    "20.66426705543168": "figur8, Inc.",
    "12.123917685919744": "Leviathan Solutions Ltd.",
    "24.16309119": "HTC Corporation",
    "24.14956503": "Henri Depaepe S.A.S.",
    "24.1073893": "Hewlett Packard",
    "24.9470560": "IEEE 1904.1 Working Group",
    "12.123917683601408": "Alpiq InTec Management AG",
    "24.1073899": "D-Link International",
    "24.1623121": "Tucson Embedded Systems",
    "24.6822316": "Guangzhou Xianyou Intelligent Technogoly CO., LTD",
    "12.123917676736512": "Elk Solutions, LLC",
    "24.13924000": "Fortinet, Inc.",
    "20.79215080767488": "Shenzhen Topwell Technology Co..Ltd",
    "12.123917689757696": "SNK, Inc.",
    "24.11580203": "Slican Sp. z o.o.",
    "20.61822327586816": "SDNware technology co.,LTD",
    "24.10228651": "Chanson Water Co., Ltd.",
    "20.211776052396032": "LTD Delovoy Office",
    "12.346853089280": "Visual Circuits Corp.",
    "12.123917686218752": "Power Electronics Espana, S.L.",
    "24.14720506": "Wanan Hongsheng Electronic Co.Ltd",
    "24.3980606": "Beijing Xiaomi Electronics Co., Ltd.",
    "12.123917689405440": "Gamber Johnson-LLC",
    "24.5287381": "Dynalec Corporation",
    "24.6856": "ISL (Instrumentation Scientifique de Laboratoire)",
    "12.119269531648": "Certis Technology International Pte Ltd",
    "24.7083028": "BUJEON ELECTRONICS Co,.Ltd",
    "24.15488547": "Huawei Technologies Co.,Ltd",
    "24.2122955": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917683453952": "Quickware Eng & Des LLC",
    "12.123917679091712": "machineQ",
    "24.612": "AudioRamp.com",
    "24.13935653": "SMAX Technology Co., Ltd.",
    "24.6855": "Unipoint",
    "12.346865397760": "xxter b.v.",
    "20.71389216243712": "Borda Technology",
    "24.12107455": "Cisco Systems, Inc",
    "24.78": "Ampex Corporation",
    "24.79": "Logicraft 386-Ware P.C. Emulator",
    "12.123917677744128": "XTech2 SIA",
    "24.10280881": "Shenzhen Crave Communication Co., LTD",
    "24.70": "Olivetti North America",
    "12.346868764672": "American Microsystems, Ltd.",
    "24.72": "Seiko Epson Corporation",
    "24.1329291": "Shenzhen Doov Technology Co.,Ltd",
    "24.74": "Adc Codenoll Technology Corp.",
    "24.75": "Icl Data Oy",
    "24.76": "NEC Corporation",
    "24.77": "Dci Corporation",
    "12.123917687046144": "Electronic Navigation Ltd",
    "12.346863640576": "ACD Elektronik GmbH",
    "24.9240048": "Qualcomm Inc.",
    "24.12629453": "Huawei Device Co., Ltd.",
    "24.619": "BCM Computers Co., Ltd.",
    "12.346852696064": "GFI Chrono Time",
    "24.618": "Cocess Telecom Co., Ltd.",
    "24.1887717": "Shanghai Wind Technologies Co.,Ltd",
    "12.123917677637632": "TextNinja Co.",
    "12.123917679050752": "Xia Llc",
    "24.10547076": "Seine Image Int'l Co., Ltd",
    "24.6293372": "Jala Group",
    "24.2932375": "Ingenieurbuero Bickele und Buehler GmbH",
    "24.1581074": "Aztech Associates Inc.",
    "24.12646801": "GME Standard Communications P/L",
    "20.137149116579840": "Société de Transport de Montréal",
    "24.15841": "Huawei Device Co., Ltd.",
    "12.346855059456": "Digital SP Ltd",
    "12.346868015104": "Dell'Orto S.P.A.",
    "20.264036223549440": "Audeara Pty. Ltd.",
    "24.5021312": "KYOKKO ELECTRIC Co., Ltd.",
    "24.5792666": "Tpl Systemes",
    "12.346859479040": "Dcns",
    "24.6062262": "Intel Corporate",
    "12.346855518208": "Radix Corporation",
    "24.7364044": "EFM Networks",
    "24.8136292": "Sagemcom Broadband SAS",
    "20.180577508524032": "Maxon Australia",
    "24.13956998": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.15991619": "Hewlett Packard Enterprise",
    "24.14419983": "Apple, Inc.",
    "24.7846": "Best IT World (India) Pvt. Ltd.",
    "24.7847": "Actiontec Electronics, Inc",
    "24.7844": "Nokia Danmark A/S",
    "24.7845": "ROBOTOUS, Inc.",
    "24.7842": "Symx Systems, Inc.",
    "24.7843": "Nokia Danmark A/S",
    "24.7840": "XLN-t",
    "24.7841": "Brunata a/s",
    "24.5547155": "IEEE Registration Authority",
    "24.15738444": "Sigrist-Photometer AG",
    "20.5297547509760": "Envision Energy",
    "24.7848": "Datang Mobile Communications Equipment CO.,LTD",
    "24.7849": "Nintendo Co., Ltd.",
    "24.8689592": "Memjet Inc.",
    "24.8967592": "Itel Mobile Limited",
    "12.346865438720": "Intelligent Devices",
    "24.9983003": "Intel Corporate",
    "12.123917687664640": "Guangzhou Xianhe Technology Engineering Co., Ltd",
    "12.123917691543552": "Tecnint HTE SRL",
    "24.1348680": "Blu Castle S.A.",
    "12.346859134976": "Victory Concept Industries Ltd.",
    "24.1878951": "Apple, Inc.",
    "20.168198561333248": "Renalsense",
    "24.12855180": "Huawei Device Co., Ltd.",
    "12.346859442176": "ITS Telecom",
    "12.123917691711488": "Hangzhou Hualan Microelectronique Co.,Ltd",
    "24.4205417": "Kumho Electric Inc.",
    "24.7916392": "Daehap Hyper-Tech",
    "24.8961349": "Samsung Electronics Co.,Ltd",
    "24.3950296": "Tp-Link Technologies Co.,Ltd.",
    "24.1856214": "Flat Display Technology Corporation",
    "12.123917676695552": "Automata GmbH & Co. KG",
    "24.10018139": "Takumi Japan Ltd",
    "12.123917678100480": "Radixon s.r.o.",
    "24.14982674": "Freebox Sas",
    "24.4507229": "Apple, Inc.",
    "24.12855189": "Technicolor CH USA Inc.",
    "24.11573291": "Sagemcom Broadband SAS",
    "12.346864107520": "FEW Bauer GmbH",
    "24.10016867": "Shanghai High-Flying Electronics Technology Co., Ltd",
    "12.71297883176960": "Xronos.Inc",
    "12.71297884274688": "Contrans TI sp. z o.o.",
    "20.168198555041792": "Pozyx NV",
    "20.79602634457088": "Dnv Gl",
    "24.10250341": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3209087": "S Mobile Devices Limited",
    "20.225135229403136": "Guardiar USA",
    "24.13682775": "IEEE Registration Authority",
    "24.13900828": "Rcf S.P.A.",
    "12.346863144960": "Saintronic",
    "12.346853879808": "Tidel Engineering, L.P.",
    "12.346865029120": "Galvamat & Unican Technologies SA",
    "24.6071197": "Samsung Electro Mechanics Co., Ltd.",
    "24.4733041": "Huawei Device Co., Ltd.",
    "24.1096984": "Cisco Systems, Inc",
    "24.13416689": "Eagle Kingdom Technologies Limited",
    "12.123917677072384": "Automata GmbH & Co. KG",
    "24.3955907": "Shenzhen First Blue Chip Technology Ltd",
    "20.154794714988544": "Guardtec,Inc",
    "12.346857754624": "Control Alternative Solutions, Inc.",
    "12.346868006912": "GD Mission Systems",
    "24.16029634": "Nebulon Inc",
    "24.53184": "China Mobile Group Device Co.,Ltd.",
    "24.5271415": "Huawei Technologies Co.,Ltd",
    "24.14457814": "TCT mobile ltd",
    "24.14482133": "STRONG Ges.m.b.H.",
    "24.6307110": "Technicolor CH USA Inc.",
    "24.5795215": "Cisco-Linksys, LLC",
    "12.123917691273216": "DEUTA-WERKE GmbH",
    "24.5558": "ShinMaywa Industries, Ltd.",
    "24.5559": "Toshiba",
    "24.5556": "Polymap Wireless LLC",
    "24.5557": "CI Network Corp.",
    "24.5554": "Advanced Industrial Computer, Inc.",
    "24.5555": "Caretech AB",
    "24.5552": "Autotelenet Co.,Ltd",
    "24.5553": "Ambient Corporation",
    "24.5550": "kyung il",
    "24.5551": "AzureWave Technology Inc.",
    "12.346858336256": "Computechnic AG",
    "24.12380922": "Hewlett Packard",
    "24.25664": "Cisco Systems, Inc",
    "24.1352628": "Huawei Device Co., Ltd.",
    "20.154314886610944": "Fujian Chaozhi Group Co., Ltd.",
    "12.123917678755840": "Romteck Australia",
    "12.346857181184": "Scirocco AB",
    "20.70443732041728": "BWT Tianjin Ltd.",
    "12.346868310016": "Gwt Llc",
    "12.123917687476224": "Pantec Engineering AG",
    "12.71297884315648": "Uniscan LLC",
    "24.12598598": "Shanghai Sango Network Technology Co.,Ltd",
    "24.12381344": "Compal Information (Kunshan) Co., Ltd.",
    "24.10521060": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.8926738": "SRC Computers, LLC",
    "24.5286718": "Qibixx AG",
    "24.16441744": "In Win Development Inc.",
    "12.71297883529216": "Group 57",
    "24.800662": "Bizlink Technology, Inc.",
    "24.8159860": "Shenzhen Hikeen Technology CO.,LTD",
    "24.13678090": "Xingluo Technology Company Limited",
    "24.572937": "Astrol Electronic AG",
    "12.346857353216": "Autronica Fire And Securirty",
    "24.1577038": "Cedint-Upm",
    "24.1594851": "Apple, Inc.",
    "12.123917690269696": "Dismuntel, S.A.",
    "24.2360591": "MTN Electronic Co. Ltd",
    "12.123917681152000": "Theatrixx Technologies, Inc.",
    "12.119269523456": "XPossible Technologies Pte Ltd",
    "12.123917691428864": "LogiM GmbH Software und Entwicklung",
    "24.7660277": "Apple, Inc.",
    "20.53091441311744": "Shenzhen Ldrobot Co., Ltd.",
    "12.123917682618368": "EA Elektroautomatik GmbH & Co. KG",
    "12.346867380224": "Witree Co.,Ltd",
    "12.123917678010368": "YUYAMA MFG Co.,Ltd",
    "12.346863976448": "Ampcontrol Pty Ltd",
    "20.193649363124224": "Technion Oy",
    "12.346855411712": "Invatron Systems Corp.",
    "12.346858074112": "Lumel S.A.",
    "24.9240180": "Ruckus Wireless",
    "12.346853122048": "iReady Corporation",
    "24.820752": "Juniper Networks",
    "24.888": "HUMAX Co., Ltd.",
    "24.1065984": "Huawei Technologies Co.,Ltd",
    "24.886": "Graphtec Technology, Inc.",
    "24.887": "Gigabit Wireless",
    "24.884": "Control Microsystems",
    "24.885": "NetMedia, Inc.",
    "12.346863206400": "S-Sys",
    "24.883": "Aselsan A.S",
    "24.880": "NXTV, Inc.",
    "12.346858770432": "Tera Information System Labs",
    "12.346856235008": "SysNova",
    "12.123917684076544": "Tucsen Photonics Co., Ltd.",
    "20.53091443408896": "Ledworks SRL",
    "24.7345": "Cisco Systems, Inc",
    "24.27534": "Shanghai Feixun Communication Co.,Ltd.",
    "24.6838469": "ZF TRW Automotive",
    "24.7309": "Mesa Imaging",
    "12.346865991680": "ACTIV Financial Systems, Inc.",
    "24.7301": "Eunicorn",
    "24.7300": "STL Solution Co.,Ltd.",
    "24.7303": "Uriver Inc.",
    "24.7302": "Cranite Systems, Inc.",
    "24.7305": "Force Communications, Inc.",
    "24.7304": "Transystem Inc.",
    "24.7307": "MJ Innovations Ltd.",
    "24.7306": "Cirrascale Corporation",
    "12.123917679972352": "Boulder Amplifiers, Inc.",
    "24.7358479": "Apple, Inc.",
    "12.123917684334592": "Mothonic AB",
    "24.8132109": "Saia-Burgess Controls AG",
    "20.137215119196160": "Hangzhou Kaicom Communication Co.,Ltd",
    "24.7608421": "Shanghai DareGlobal Technologies Co.,Ltd",
    "20.206931525369856": "IPLINK Technology Corp",
    "24.13920936": "Huawei Technologies Co.,Ltd",
    "24.14457108": "Huawei Technologies Co.,Ltd",
    "24.53400": "Photon Dynamics Canada Inc.",
    "24.53401": "Elcard Wireless Systems Oy",
    "24.53402": "Filanet Corporation",
    "24.53403": "Spectel Ltd.",
    "24.53404": "Kapadia Communications",
    "24.53405": "Veris Industries",
    "24.53406": "2Wire Inc",
    "24.53407": "Novtek Test Systems",
    "24.53408": "Mips Denmark",
    "24.53409": "Oskar Vierling Gmbh + Co. Kg",
    "12.123917689085952": "ard sa",
    "24.11847214": "Molex CMS",
    "24.13159352": "Hewlett Packard",
    "12.123917677621248": "ST Aerospace Systems",
    "24.689974": "IEEE 1901 Working Group",
    "24.6083300": "Intel Corporate",
    "24.2669592": "SourcingOverseas Co. Ltd",
    "24.2415532": "Huawei Technologies Co.,Ltd",
    "24.2415533": "ShopperTrak RCT Corporation",
    "24.326207": "Opticore Inc.",
    "12.346867957760": "Wincor Nixdorf Sp. z o.o.",
    "12.123917678485504": "Globalcom Engineering SPA",
    "24.2124426": "Wistron Infocomm (Zhongshan) Corporation",
    "12.346858033152": "Fen Technology Limited",
    "24.5243012": "Siemens Canada",
    "20.167152464166912": "Chengdu Shuwei Communication Technology Co.,Ltd",
    "24.11331451": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.1628": "Malachite Technologies, Inc.",
    "24.1629": "Heidelberg Web Systems",
    "24.2127883": "Delta Faucet Company",
    "24.1620": "Winpresa Building Automation Technologies GmbH",
    "24.1621": "Yipee, Inc.",
    "24.1622": "Tactel AB",
    "24.1623": "Market Central, Inc.",
    "24.1624": "Helmut Fischer GmbH Institut für Elektronik und Messtechnik",
    "24.1625": "EAL (Apeldoorn) B.V.",
    "24.1626": "Strix Systems",
    "24.1627": "Dell Inc.",
    "24.314900": "Wilocity LTD.",
    "12.119269756928": "WonATech Co., Ltd.",
    "24.15782887": "Sensometrix SA",
    "24.2390515": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346862694400": "IDS GeoRadar s.r.l.",
    "24.6162": "Beijing Xinwei Telecom Technology Co., Ltd.",
    "24.6163": "Sony Mobile Communications Inc",
    "24.6160": "IPTrade S.A.",
    "24.6161": "Neuros Technology International, LLC.",
    "24.6166": "Ubixon Co., Ltd.",
    "24.6167": "D. E. Shaw Research, LLC",
    "24.6164": "Mitutoyo Corporation",
    "24.6165": "GZ Technologies, Inc.",
    "24.15514542": "Digivoice Tecnologia em Eletronica Ltda",
    "24.6168": "Cisco Systems, Inc",
    "24.6169": "Cisco Systems, Inc",
    "24.9185162": "Vocollect Inc",
    "24.1060916": "Cara Systems",
    "24.16273436": "Tianjin Geneuo Technology Co.,Ltd",
    "24.8694428": "MitraStar Technology Corp.",
    "24.11061945": "Samsung Electro Mechanics Co., Ltd.",
    "12.346859212800": "unipo GmbH",
    "24.9188183": "HiteVision Digital Media Technology Co.,Ltd.",
    "24.2624696": "Mikrosay Yazilim ve Elektronik A.S.",
    "20.83889195319296": "Stored Energy Systems",
    "24.2927684": "Conextop",
    "24.8917398": "Huawei Technologies Co.,Ltd",
    "24.9471019": "LEGO System A/S",
    "12.71297884467200": "Davin Technologies Co.,Ltd",
    "12.346853552128": "Advanced Devices SpA",
    "12.346856062976": "Keith & Koep GmbH",
    "12.346852732928": "HYPERCHIP Inc.",
    "24.1334889": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.6318158": "Samsung Electronics Co.,Ltd",
    "12.123917683425280": "Becton Dickinson",
    "12.346859094016": "Bug Labs, Inc.",
    "24.16578734": "Intel Corporate",
    "24.316": "Tiw Systems",
    "24.317": "RiscStation Ltd.",
    "24.314": "Shelcad Communications, Ltd.",
    "24.315": "Bna Systems",
    "24.312": "XAVi Technologies Corp.",
    "24.313": "Point Multimedia Systems",
    "24.310": "CyberTAN Technology Inc.",
    "24.311": "IT Farm Corporation",
    "24.6081235": "Chipsea Technologies (Shenzhen) Corp.",
    "24.323075": "Texas Instruments",
    "24.318": "Ascom Tateco AB",
    "24.319": "Neighbor World Co., Ltd.",
    "24.37024": "8X8 Inc.",
    "24.37025": "Flying Pig Systems/High End Systems Inc.",
    "24.37026": "CyberTAN Technology Inc.",
    "24.37027": "Corecess Inc.",
    "24.37020": "ARRIS Group, Inc.",
    "24.37021": "NovaTech Process Solutions, LLC",
    "24.37022": "Critical IO, LLC",
    "24.37023": "Digi-Data Corporation",
    "24.13430049": "Nokia Shanghai Bell Co., Ltd.",
    "24.37028": "Altiga Networks",
    "24.37029": "Spectra Logic",
    "24.32768": "Multitech Systems, Inc.",
    "24.32769": "Periphonics Corporation",
    "24.6340799": "ON Semiconductor",
    "12.119269261312": "Dalaj Electro-Telecom",
    "12.123917676212224": "Vitec",
    "24.2100167": "Huawei Technologies Co.,Ltd",
    "24.2407361": "Absolute Analysis",
    "20.228784612573184": "Airthings",
    "24.1049857": "Pegatron Corporation",
    "24.10784059": "Vizio, Inc",
    "24.9365": "ARRIS Group, Inc.",
    "24.9364": "Shenzhen Baoxin Tech CO., Ltd.",
    "20.44768531841024": "Convertertec Deutschland GmbH",
    "24.9366": "Ginzinger electronic systems",
    "24.9361": "Samsung Electronics Co.,Ltd",
    "24.9360": "Samsung Electronics Co.,Ltd",
    "24.9363": "ARRIS Group, Inc.",
    "24.9362": "Motorola, Broadband Solutions Group",
    "24.9369": "Aquila Technologies",
    "24.9368": "Cisco Systems, Inc",
    "24.5569983": "Scheidt & Bachmann GmbH",
    "24.5299007": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.71297883774976": "Wasserbauer GmbH",
    "12.123917686824960": "FAS Electronics (Fujian) Co.,LTD.",
    "24.11044573": "Apple, Inc.",
    "24.11290403": "Genelec",
    "24.11577273": "TP-Link Corporation Limited",
    "12.346853359616": "Tokai Soft Corporation",
    "24.9477600": "Newland Design + Assoc. Inc.",
    "24.8821": "Belkin International Inc.",
    "20.132777852796928": "Shanghai Hanyi Technologies Co,.Ltd.",
    "12.346852417536": "MSU UK Ltd",
    "24.8820": "FamilyPhone AB",
    "20.154055545454592": "GESAS GmbH",
    "24.6846756": "ELS-GmbH & Co. KG",
    "24.8823": "NEC Australia Pty Ltd",
    "12.346867388416": "Cosmo Life Co.,Ltd",
    "24.8822": "Triple EYE B.V.",
    "24.8825": "Nippon Conlux Co., Ltd.",
    "24.825062": "Cloud Network Technology (Samoa) Limited",
    "24.6837501": "Amazon Technologies Inc.",
    "24.8824": "Shenzhen Tongfang Multimedia Technology Co.,Ltd.",
    "24.16522193": "Huawei Technologies Co.,Ltd",
    "24.8827": "Apogee Labs, Inc.",
    "20.194284251774976": "IPvideo Corporation",
    "24.8826": "Telecom Design",
    "12.71297883914240": "Mitsubishi Electric System & Service Co.,Ltd.",
    "24.5552101": "Rayson Technology Co., Ltd.",
    "24.524325": "Control Data",
    "24.524324": "10Net Communications/Dca",
    "24.524327": "PCS Computer Systems GmbH",
    "24.524326": "Norsk Data (Nord)",
    "24.524321": "3M Company",
    "24.524320": "Oracle Corporation",
    "24.524323": "Panasonic Communications Co., Ltd.",
    "24.524322": "NBI (Nothing But Initials)",
    "24.12631975": "Juniper Networks",
    "24.3461304": "IEEE Registration Authority",
    "24.524329": "Megatek Corporation",
    "24.524328": "TI",
    "24.36865": "Nishimu Electronics Industries Co., Ltd.",
    "24.36864": "Diamond Multimedia",
    "12.123917684502528": "DTRON Communications (Pty) Ltd",
    "20.123598303920128": "Full Solution Telecom",
    "24.7341740": "Eastcompeace Technology Co., Ltd",
    "24.16534359": "Peerless Instrument Division of Curtiss-Wright",
    "24.310719": "Samsung Electronics Co.,Ltd",
    "24.3192399": "Juniper Networks",
    "24.8015": "Thinkware Co. Ltd.",
    "24.8016": "Swissdis AG",
    "24.8017": "HD Communications Corp",
    "24.8010": "Albentia Systems S.A.",
    "24.8011": "Lineage Power",
    "24.8012": "Roseman Engineering Ltd",
    "24.8013": "Segnetics LLC",
    "24.8018": "UVT Unternehmensberatung fur Verkehr und Technik GmbH",
    "24.8019": "GEMAC Chemnitz GmbH",
    "12.346855301120": "Saab AB",
    "24.542540": "Nokia",
    "24.13653037": "Cambridge Industries(Group) Co.,Ltd.",
    "24.6342232": "PHYTRONIX Co.,Ltd.",
    "12.123917688553472": "Deltronic Security AB",
    "20.167295461621760": "Wuxi GuoYiHaiJu Technology Co.,Ltd.",
    "24.14462699": "Ruckus Wireless",
    "12.123917688365056": "Ensura Solutions BV",
    "24.1101166": "Fisys",
    "24.2406540": "Crenus Co.,Ltd.",
    "24.12093825": "Chengdu InnoThings Technology Co., Ltd.",
    "24.7651663": "Huawei Technologies Co.,Ltd",
    "12.71297888473088": "Shenzhen MaiWei Cable TV Equipment CO.,LTD.",
    "20.102196705230848": "EUROIMMUN Medizinische Labordiagnostika AG",
    "20.88010607558656": "Igor, Inc.",
    "24.4491867": "Micro-Star INT'L CO., LTD.",
    "12.346863849472": "Triax A/S",
    "24.13954182": "LinkedHope Intelligent Technologies Co., Ltd",
    "12.123917685100544": "ITS Industrial Turbine Services GmbH",
    "24.7910329": "ShangHai sunup lighting CO.,LTD",
    "24.1100417": "Precia",
    "24.15752059": "Samsung Electronics Co.,Ltd",
    "24.4741814": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.12866982": "Cadac Holdings Ltd",
    "24.9714149": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.787456": "Jantar d.o.o.",
    "20.154794719182848": "ITECH Electronic Co.,ltd.",
    "12.346853105664": "RouteFree, Inc.",
    "24.3455775": "BlackBerry RTS",
    "24.7344108": "Poindus Systems Corp.",
    "12.123917682393088": "Potter Electric Signal Co. LLC",
    "24.6033811": "Seal One AG",
    "24.14742594": "Huawei Device Co., Ltd.",
    "24.15007709": "Electron India",
    "12.346853384192": "Symbolic Sound Corp.",
    "24.8161622": "Samsung Electronics Co.,Ltd",
    "12.346855309312": "Soredex Instrumentarium Oyj",
    "24.10779108": "KLINFO Corp",
    "12.346857394176": "Network Automation mxc AB",
    "24.5522440": "Nokia Corporation",
    "20.44169479323648": "Shenzhen emb-star technology co. LTD",
    "24.5816698": "Cambium Networks Limited",
    "24.9738921": "Microsoft Corporation",
    "12.346863693824": "XoByte LLC",
    "12.123917679345664": "Underground Systems, Inc.",
    "24.30856": "Cisco Systems, Inc",
    "24.7080202": "Gematica Srl",
    "24.14217026": "Huawei Device Co., Ltd.",
    "12.123917685800960": "Alere Technologies AS",
    "12.123917686738944": "Qwave Inc",
    "24.311149": "Zyxel Communications Corporation",
    "24.4992128": "Beijing Skyway Technologies Co.,Ltd",
    "24.11030657": "Huawei Technologies Co.,Ltd",
    "12.123917683240960": "aquila biolabs GmbH",
    "24.15997857": "Apple, Inc.",
    "12.123917690986496": "Ооо \"Ронекс\"",
    "24.14951077": "Plantronics, Inc.",
    "12.123917683740672": "Crde",
    "24.3951721": "Infinity System S.L.",
    "24.7114929": "SanLogic Inc",
    "12.123917681840128": "LDA Audiotech",
    "24.2643902": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917680386048": "Syncopated Engineering Inc",
    "24.5767395": "Liteon Technology Corporation",
    "12.71297884004352": "Neutrik Ag",
    "12.346852622336": "Katchall Technologies Group",
    "20.106196524597248": "Q-SENTECH Co.,Ltd.",
    "24.5300117": "Lytx",
    "24.8707852": "Net2Grid Bv",
    "12.346853289984": "Ace Electronics, Inc.",
    "12.123917677821952": "Beijing Zhongyi Yue Tai Technology Co., Ltd",
    "12.123917677572096": "WeVo Tech",
    "24.2674891": "Cambridge Communication Systems Ltd",
    "24.6604391": "Barnes&Noble",
    "20.48688643702784": "Scout Security, Inc.",
    "24.13642821": "littleBits Electronics, Inc.",
    "24.4782259": "Cisco-Linksys, LLC",
    "24.14696061": "data-complex GmbH",
    "12.123917675102208": "Delitech Group",
    "24.10809438": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.6822578": "zte corporation",
    "24.12621274": "NXP India Private Limited",
    "24.12907941": "Kumalift Co., Ltd.",
    "12.346867453952": "Dlite Comercio, Importadora e Serviços de Automação Ltda",
    "24.15503923": "Eddyfi NDT Inc",
    "20.66219397873664": "chipsguide technology Co.,LTD",
    "24.1835309": "Ficer Technology",
    "24.13913959": "Huawei Technologies Co.,Ltd",
    "12.346865086464": "Eilersen Electric A/S",
    "20.167137989623808": "Simplo Technology Co.,LTD",
    "24.12867326": "Lava International Ltd.",
    "24.7627083": "Chongqing Huijiatong Information Technology Co., Ltd.",
    "12.346854432768": "Crescendo Networks",
    "24.2416551": "Hangzhou BroadLink Technology Co.,Ltd",
    "24.7122349": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.6598888": "Huawei Device Co., Ltd.",
    "24.309788": "Huawei Device Co., Ltd.",
    "24.2399": "Telebyte, Inc.",
    "24.15527640": "ZTLX Network Technology Co.,Ltd",
    "24.5523472": "Dell Inc.",
    "24.2398": "Masstech Group Inc.",
    "24.3717808": "Broadcom",
    "24.808121": "Nokia",
    "24.2146735": "i Wit Digital Co., Limited",
    "20.36069842092032": "Jabil",
    "24.11552025": "TCT mobile ltd",
    "20.228844749455360": "Spirit IT B.V.",
    "24.7895946": "Apple, Inc.",
    "24.11552021": "mira fitness,LLC.",
    "24.3710851": "OTN Systems N.V.",
    "24.2395": "Netgear",
    "24.33008": "Panasonic Communications Co., Ltd.",
    "24.33009": "Opus Systems",
    "24.33006": "Thomson Csf",
    "24.33007": "Rational",
    "24.33004": "Supercomputing Solutions, Inc.",
    "24.33005": "Iq Technologies, Inc.",
    "24.33002": "ADVA Optical Networking Ltd.",
    "24.2397": "Dialogue Technology Corp.",
    "24.33000": "Cumulus Corporatiion",
    "24.33001": "Madge Ltd.",
    "24.2396": "Philips Medical Systems - Cardiac and Monitoring Systems (CM",
    "24.2391": "Supercaller, Inc.",
    "24.13937538": "ARRIS Group, Inc.",
    "12.346855534592": "Eti",
    "12.123917677314048": "EDFelectronics JRMM Sp z o.o. sp.k.",
    "24.2390": "Network Systems Group, Ltd. (NSG)",
    "24.1107138": "Samsung Electronics Co.,Ltd",
    "24.2393": "Sitecsoft",
    "20.105645767393280": "Faster CZ spol. s r.o.",
    "24.2392": "Intelnet S.A.",
    "12.346861109248": "IAdea Corporation",
    "24.7350997": "Samsung Electronics Co.,Ltd",
    "12.346854289408": "Solectron Ind.Com.Servs.Exportadora do Brasil Ltda.",
    "24.5785348": "Shenzhen Webridge Technology Co.,Ltd",
    "24.8413367": "Cloud Network Technology Singapore Pte. Ltd.",
    "20.268341262417920": "Kodpro Ltd.",
    "12.346865238016": "9Solutions Oy",
    "12.346852823040": "Visable Genetics, Inc.",
    "12.123917679353856": "Zehnder Group Nederland",
    "12.123917680848896": "Asis Pro",
    "12.123917684117504": "M-Tech Innovations Limited",
    "24.10020936": "Axiim",
    "12.123917687140352": "Internet Protocolo Logica Sl",
    "24.2406931": "Riso Kagaku Corporation",
    "12.346853523456": "Optibase Ltd.",
    "24.13949961": "Mist Systems, Inc.",
    "12.123917687361536": "Yite technology",
    "24.553986": "Apple, Inc.",
    "24.15764731": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3456512": "Samsung Electronics Co.,Ltd",
    "24.16284514": "FASTWEL INTERNATIONAL CO., LTD. Taiwan Branch",
    "24.16572818": "Sichuan Jinwangtong Electronic Science&Technology Co,.Ltd",
    "24.2625477": "Beijing Leadsec Technology Co., Ltd.",
    "24.3959340": "IEEE Registration Authority",
    "24.847897": "Malgn Technology Co., Ltd.",
    "24.5525136": "Apple, Inc.",
    "12.346856210432": "3D perception",
    "12.123917682274304": "Beringar",
    "12.346858450944": "Gazelle Monitoring Systems",
    "12.71297883684864": "Agramkow Fluid Systems A/S",
    "24.14169326": "Stem Audio",
    "12.346866483200": "Sine Systems, Inc.",
    "24.16556419": "Amazon Technologies Inc.",
    "24.298296": "Netcore Technology Inc.",
    "12.346855260160": "Efficient Channel Coding",
    "12.123917683859456": "Nippon Safety co,ltd",
    "24.16053540": "Motorola Mobility LLC, a Lenovo Company",
    "24.7362040": "Cerebras Systems Inc",
    "24.8144517": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.5813811": "Ruckus Wireless",
    "12.123917677830144": "Sun Creative (ZheJiang) Technology INC.",
    "24.13939049": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "24.3240": "Garuda Networks Corporation",
    "24.3241": "Ebtron Inc.",
    "24.3242": "Cubic Transportation Systems Inc",
    "24.3243": "Commend International GmbH",
    "24.3244": "Citizen Watch Co., Ltd.",
    "24.3245": "BTU International",
    "12.346860101632": "SMARTRO Co.,Ltd.",
    "24.3440653": "Cavium Inc",
    "24.3248": "Star Semiconductor Corporation",
    "24.3249": "Salland Engineering (Europe) BV",
    "24.4211965": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.12609455": "Cisco Systems, Inc",
    "24.1867042": "Fuji Xerox Co., Ltd.",
    "12.346867281920": "Sika Technology AG",
    "24.13657012": "SolidRun Ltd.",
    "24.7006002": "UNIS-WDC Storage Co.,Ltd",
    "24.5969": "Online Corporation",
    "24.15729195": "Chrontel",
    "24.3184949": "zte corporation",
    "24.8667254": "Drivenets",
    "24.1050058": "Cisco Systems, Inc",
    "24.3459866": "Procter & Gamble Company",
    "12.346856189952": "TTC TELEKOMUNIKACE Ltd",
    "12.346854449152": "Lord Ingenierie",
    "12.123917687242752": "AdInte, inc.",
    "24.7661510": "ARRIS Group, Inc.",
    "24.7624210": "ARRIS Group, Inc.",
    "24.3442493": "Quectel Wireless Solutions Co., Ltd.",
    "24.16527274": "Nokia",
    "24.15759205": "Sourcefire, Inc",
    "12.123917690101760": "Eneon sp. z o.o.",
    "24.8968124": "DEP Company",
    "24.3190395": "Deqing Dusun Electron CO., LTD",
    "12.123917680320512": "Techno Broad,Inc",
    "20.31368137408512": "Hekatron Vertriebs GmbH",
    "24.5537704": "TELCO systems, s.r.o.",
    "24.6601405": "SDJ Technologies, Inc.",
    "24.4482431": "Calix Inc.",
    "24.524405": "DDE (Danish Data Elektronik A/S)",
    "20.194600871395328": "Habana Labs LTD",
    "24.14452440": "Apple, Inc.",
    "20.132855985340416": "Quidel Corporation",
    "12.123917677371392": "Comeo Technology Co.,Ltd",
    "12.346864734208": "Ernemann Cine Tec GmbH",
    "24.7117760": "Chemoptics Inc.",
    "24.7375222": "Tuya (HK) Limietd",
    "12.123917683183616": "Molekuler Goruntuleme A.S.",
    "24.524403": "Tecmar Inc.",
    "24.2936350": "Cloudtronics Pty Ltd",
    "24.5547739": "Huawei Device Co., Ltd.",
    "12.123917678108672": "Tattile Srl",
    "24.9201784": "Nortek-AS",
    "24.14684803": "Shenzhen Fanzhuo Communication Technology Co., Lt",
    "12.346856550400": "Win4NET",
    "12.346868695040": "Kyowadensi",
    "20.62070504554496": "Live Sentinel",
    "24.14206584": "MitraStar Technology Corp.",
    "24.2369090": "SHARP Corporation.",
    "24.686": "Scannex Electronics Ltd.",
    "24.9731006": "Ubicquia LLC",
    "24.524408": "Accell Corporation",
    "24.7665463": "Kce",
    "24.3418002": "Freebox Sas",
    "24.4993753": "M Rutty Holdings Pty. Ltd.",
    "12.71297884856320": "Rohde&Schwarz Topex SA",
    "24.16314373": "Newport Media Inc.",
    "12.346855108608": "Diseño de Sistemas en Silicio S.A.",
    "12.123917687005184": "UB330.net d.o.o.",
    "24.9225825": "Current, powered by GE",
    "24.2916696": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346859864064": "AixSolve GmbH",
    "12.123917683998720": "UnI Systech Co.,Ltd",
    "12.123917684948992": "Nu-Tek Power Controls and Automation",
    "24.6031737": "Huawei Technologies Co.,Ltd",
    "24.6360": "ARCH METER Corporation",
    "24.6361": "Santosha Internatonal, Inc",
    "24.6362": "Würth Elektronik eiSos GmbH & Co. KG",
    "24.6363": "EPL Technology Ltd",
    "24.6364": "Prostar Co., Ltd.",
    "24.6365": "Silicondust Engineering Ltd",
    "24.6366": "Intel Corporate",
    "24.6367": "The Morey Corporation",
    "24.6368": "Anaveo",
    "24.6369": "Verkerk Service Systemen",
    "24.8658032": "zte corporation",
    "20.273272606294016": "Protos GmbH",
    "24.11313683": "Shenzhen Bilian electronic CO.,LTD",
    "12.346852290560": "KBS Industrieelektronik GmbH",
    "24.8130254": "Cisco Systems, Inc",
    "20.220958190206976": "Oscars Pro",
    "20.180577510621184": "Recognition Systems LLC",
    "24.1587784": "VostroNet",
    "24.10772530": "MRV Communications (Networks) LTD",
    "12.346859200512": "CommoDaS GmbH",
    "12.346852499456": "Mayo Foundation",
    "24.8692661": "Propulsion systems",
    "20.277981936222208": "Winglet Systems Inc.",
    "12.123917688852480": "King-On Technology Ltd.",
    "24.8137971": "Secure Electrans Ltd",
    "20.127654895288320": "IDEXX Laboratories, Inc",
    "12.123917675212800": "Sailmon BV",
    "12.346868031488": "Harland Simon plc",
    "24.264563": "Hewlett Packard Enterprise",
    "24.8164785": "Extreme Networks, Inc.",
    "24.14418574": "zte corporation",
    "24.6984": "Shenzhen Lantech Electronics Co., Ltd.",
    "24.6985": "Roberts Radio limited",
    "24.6986": "W&W Communications, Inc.",
    "24.6987": "SANION Co., Ltd.",
    "24.6980": "SanDisk Corporation",
    "24.6981": "ABB AS, Division Automation Products",
    "24.6982": "Blueone Technology Co.,Ltd",
    "24.6983": "Futarque A/S",
    "12.346862559232": "SISMODULAR - Engenharia, Lda",
    "24.6988": "Signtech",
    "24.6989": "Areca Technology Corporation",
    "12.123917682446336": "2M Technology",
    "24.1350": "KDDI Network & Solultions Inc.",
    "24.1351": "Starent Networks",
    "24.1352": "Disco Corporation",
    "12.346861101056": "Wartsila Voyage Limited",
    "24.1354": "Ario Data Networks, Inc.",
    "24.1355": "Eaton Automation AG",
    "24.1356": "RF Innovations Pty Ltd",
    "24.1357": "Brans Technologies, Inc.",
    "24.1358": "Philips",
    "24.1359": "Garmin International",
    "12.123917679763456": "George Wilson Industries Ltd",
    "12.123917680488448": "Mr.Loop",
    "24.4500007": "udworks Co., Ltd.",
    "24.12347256": "Apple, Inc.",
    "20.368734896128": "Nanoleaf",
    "24.16048644": "Traqueur",
    "20.452711153664": "Datapan d.o.o.",
    "24.257": "Private",
    "24.256": "Equip'Trans",
    "24.255": "Camtec Electronics Ltd.",
    "24.254": "Annapolis Micro Systems, Inc.",
    "24.253": "High Level Hardware (Orion, UK)",
    "24.252": "Meiko",
    "24.251": "Rechner Zur Kommunikation",
    "24.250": "Microsage Computer Systems Inc",
    "12.119269502976": "Coincident, Inc.",
    "24.259": "3Com",
    "24.258": "3Com",
    "24.12336647": "Hitron Technologies. Inc",
    "24.2970": "Shanghai Ulink Telecom Equipment Co. Ltd.",
    "24.5189": "Telefon-Gradnja d.o.o.",
    "24.5188": "Grundfos Holding",
    "12.346867441664": "CT Company",
    "24.1854822": "Amazon Technologies Inc.",
    "24.5181": "Aevoe Inc.",
    "24.5180": "Rheinmetall Canada Inc.",
    "24.5183": "Hotway Technology Corporation",
    "24.5182": "AirLink Communications, Inc.",
    "24.5185": "Innovation Sound Technology Co., LTD.",
    "24.5184": "ATOMIC Corporation",
    "24.5187": "Consultronics Europe Ltd",
    "24.5186": "Atto Corporation",
    "24.4278": "Entrata Communications Corp.",
    "24.4279": "Coyote Technologies, Llc",
    "20.273458027036672": "Beijing Wanji Techonology Co., Ltd.",
    "24.10249648": "Samsung Electronics Co.,Ltd",
    "24.4270": "Shinko Electric Industries Co.",
    "24.4271": "Tac Systems, Inc.",
    "24.4272": "Meridian Technology Corp.",
    "24.4273": "For-A Co., Ltd.",
    "24.4274": "Coactive Aesthetics",
    "24.4275": "Nokia Multimedia Terminals",
    "24.4276": "Atmosphere Networks",
    "24.4277": "Accton Technology Corp",
    "24.45264": "Computer Products International",
    "24.5805016": "Texas Instruments",
    "24.9233294": "Kioxia Corporation",
    "12.123917686247424": "Dynamic Controls",
    "24.3954882": "Intel Corporate",
    "12.123917685161984": "Clearly IP Inc",
    "20.79602627117056": "Swistec Systems AG",
    "24.14966327": "Intel Corporate",
    "24.1974": "Telecom Technology Ltd.",
    "24.1975": "Samurai Ind. Prods Eletronicos Ltda",
    "24.1976": "Corvalent Corporation",
    "24.1977": "Ginganet Corporation",
    "24.1970": "Transaccess S.A.",
    "24.1971": "Cisco Systems, Inc",
    "12.346865893376": "Glyn GmbH & Co.KG",
    "24.1973": "Any One Wireless Ltd.",
    "12.346858463232": "Chauvin Arnoux",
    "12.346868928512": "Touchless Biometric Systems AG",
    "24.1978": "UTStarcom Inc",
    "24.1979": "Candera Inc.",
    "24.3197355": "Cloud Network Technology Singapore Pte. Ltd.",
    "24.12846203": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.6354068": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917684416512": "Dea System Spa",
    "24.13141063": "Beautiful Enterprise Co., Ltd",
    "20.203238157582336": "iModesty Technology Corp.",
    "12.123917680414720": "Inventeq B.V.",
    "24.5265408": "WyTec International, Inc.",
    "24.11300825": "Valueplus Inc.",
    "24.5545349": "EnerAccess inc",
    "12.71297884012544": "Ateme",
    "24.2202": "Alcatel Microelectronics",
    "12.71297884196864": "Invisua Lighting BV",
    "24.1850255": "Selve GmbH & Co. KG",
    "20.273458031230976": "Solarius",
    "24.5266370": "Liteon Technology Corporation",
    "24.2636263": "Preceno Technology Pte.Ltd.",
    "12.123917689311232": "Roboteq",
    "24.6818519": "Yancheng Kecheng Optoelectronic Technology Co., Ltd",
    "24.5278965": "China Mobile Group Device Co.,Ltd.",
    "12.123917683314688": "The Things Network Foundation",
    "12.346867912704": "ESI Ventures",
    "24.10530209": "JMR Electronics, Inc",
    "24.352": "ELMEX Co., LTD.",
    "24.12380715": "CityCom GmbH",
    "12.346858381312": "Condre Corporation",
    "24.353": "Meta Machine Technology",
    "24.3952062": "Apple, Inc.",
    "24.7362903": "Medallion Instrumentation Systems",
    "24.350": "Best Technology Co., Ltd.",
    "24.15753520": "Advantage Pharmacy Services, LLC",
    "24.925": "Qisda Corporation",
    "24.3726": "SparkLAN Communications, Inc.",
    "24.3725": "Systems in Progress Holding GmbH",
    "24.3724": "Siemens AG",
    "24.3723": "Astarte Technology Co, Ltd.",
    "24.3722": "Avara Technologies Pty. Ltd.",
    "24.3721": "Clematic",
    "24.3720": "Videotron Corp.",
    "24.9013": "Linkflex Co.,Ltd",
    "24.7922365": "Vodafone Automotive S.p.A.",
    "24.13116897": "Lemobile Information Technology (Beijing) Co., Ltd",
    "24.929": "HIPER Information & Communication, Inc.",
    "24.928": "Cisco Systems, Inc",
    "24.3729": "Navico Auckland Ltd",
    "24.3728": "Ponico Corp.",
    "24.16279282": "Cisco Systems, Inc",
    "20.180394592829440": "ISE GmbH",
    "12.346866909184": "Radinetworks Co., Ltd",
    "24.354": "Cygnet Technologies, Inc.",
    "24.355": "NDC (National Datacomm Corporation)",
    "12.346857340928": "MedAvant Healthcare",
    "24.2160508": "vivo Mobile Communication Co., Ltd.",
    "20.44169480372224": "Dexin Digital Technology Corp. Ltd.",
    "20.264036222500864": "Ezvis Limited",
    "24.2134127": "Shenzhen Tencent Computer System Co., Ltd.",
    "12.346858659840": "RoseTechnology A/S",
    "24.13693567": "SteadyServ Technoligies, LLC",
    "12.123917683056640": "Exi Flow Measurement Ltd",
    "20.260167867301888": "Paw-Taw-John Services, Inc.",
    "12.71297883561984": "inoage GmbH",
    "24.2921855": "zte corporation",
    "24.2098664": "OOO InProMedia",
    "24.16278712": "zte corporation",
    "24.2122356": "Microsoft Corporation",
    "24.6872053": "Amazon Technologies Inc.",
    "12.346852630528": "Neuberger Gebaeudeautomation GmbH & Co.",
    "20.159182261059584": "ekey biometric systems gmbh",
    "12.123917685837824": "Embest Technology Co., Ltd",
    "24.2391116": "Herman Miller",
    "24.9450748": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.66426703446016": "Olibra LLC",
    "12.346853220352": "Million Tech Development Ltd.",
    "24.3685532": "Fujian Newland Payment Technology Co.,Ltd.",
    "24.819816": "Tp-Link Technologies Co.,Ltd.",
    "24.819818": "Wuhan Huagong Genuine Optics Technology Co., Ltd",
    "20.132777840214016": "Shenzhen ELI Technology co.,ltd",
    "24.1599194": "Dell Inc.",
    "12.346857967616": "Horiba Abx Sas",
    "12.346868359168": "MaxDeTec AG",
    "24.7883037": "The Will-Burt Company",
    "24.9497184": "SPI Lasers Ltd",
    "24.9545": "Jeorich Tech. Co.,Ltd.",
    "24.9544": "Nokia Danmark A/S",
    "24.9547": "Apple, Inc.",
    "24.9546": "RingCube Technologies, Inc.",
    "24.9541": "Cisco Systems, Inc",
    "24.9540": "LoJack Corporation",
    "24.9543": "Nokia Danmark A/S",
    "24.9542": "Cisco Systems, Inc",
    "12.346855084032": "Roseman Engineering Ltd.",
    "24.9548": "Videon Central, Inc.",
    "24.4519031": "Juniper Networks",
    "24.8155510": "Wuxi iData Technology Company Ltd.",
    "20.242185285926912": "Private",
    "24.6052586": "Ford",
    "20.127654892142592": "Quest Payment Systems",
    "12.123917678583808": "Becton Dickinson",
    "24.11802496": "DTI Group Ltd",
    "24.12897605": "Beijing Boomsense Technology CO.,LTD.",
    "24.2889185": "Phicomm (Shanghai) Co., Ltd.",
    "24.2098": "Cisco Systems, Inc",
    "24.2096": "Cisco Systems, Inc",
    "24.2097": "Cisco Systems, Inc",
    "24.2094": "Multitone Electronics PLC",
    "24.2095": "Cisco Systems, Inc",
    "12.346862620672": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.2093": "Indus Teqsite Private Limited",
    "24.2090": "Powerwallz Network Security",
    "24.2091": "Wooksung Electronics, Inc.",
    "24.16287225": "Calix Inc.",
    "20.154794718134272": "Toho Denki Ind.Co.,Ltd",
    "12.123917691219968": "Satcube AB",
    "12.123917688803328": "Sportzcast",
    "24.2883755": "ARRIS Group, Inc.",
    "24.7119": "Dataupia Corporation",
    "20.180577512718336": "Alpwise",
    "12.123917688377344": "BEEcube Inc.",
    "24.7110": "Strato Rechenzentrum AG",
    "24.7111": "StarVedia Technology Inc.",
    "24.7112": "Miura Co.,Ltd",
    "24.7113": "Fsn Display Inc",
    "24.7114": "Beijing Run Technology LTD. Company",
    "24.7115": "Pempek Systems Pty Ltd",
    "24.7116": "Kingtek Cctv Alliance Co., Ltd.",
    "24.7117": "Daviscomms (S) Pte Ltd",
    "24.2548": "Alcon Laboratories, Inc.",
    "24.1104970": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.1330202": "Max Communication GmbH",
    "24.2540": "Daktronics, Inc.",
    "24.2541": "CipherOptics",
    "24.2542": "Meikyo Electric Co.,Ltd",
    "24.2543": "Vocera Communications",
    "24.2544": "Shimizu Technology Inc.",
    "24.2545": "Yamaki Electric Corporation",
    "24.2546": "Cohu, Inc., Electronics Division",
    "24.2547": "WELL Communication Corp.",
    "24.10786835": "ARRIS Group, Inc.",
    "24.2662443": "Apple, Inc.",
    "24.14683365": "Huawei Technologies Co.,Ltd",
    "24.8634": "Texas Instruments",
    "24.8635": "Riken Keiki Co., Ltd.",
    "24.8636": "Zala Computer",
    "24.8637": "Nintendo Co., Ltd.",
    "24.8630": "Triacta Power Technologies Inc.",
    "24.8631": "Lexmark International, Inc.",
    "24.8632": "Inphi Corporation",
    "24.8633": "Universal Devices Inc.",
    "24.8638": "Cisco SPVTG",
    "24.8639": "Hitachi High-Tech Control Systems Corporation",
    "20.154055544406016": "CEOS Pty Ltd",
    "24.4218818": "Texas Instruments",
    "24.15751241": "CareView Communications",
    "20.225207128162304": "ShenZhenBoryNet Co.,LTD.",
    "24.4218814": "Rim",
    "24.2140571": "Microsoft Corporation",
    "12.123917683236864": "Grupo Epelsa S.L.",
    "24.7089715": "Accelink Technologies Co.,Ltd.",
    "24.2642529": "Cisco Systems, Inc",
    "24.4510202": "Shenzhen Yunlink Technology Co., Ltd",
    "12.123917688336384": "HKC Limited",
    "20.176094525259776": "Medical Evolution Kft",
    "24.14681883": "Hewlett Packard Enterprise",
    "24.15732253": "Megafone Limited",
    "24.6647": "Onset Computer Corporation",
    "24.6646": "Acconet (PTE) Ltd",
    "12.346856173568": "American LED-gible Inc.",
    "12.346860736512": "Elmec, Inc.",
    "24.6643": "Cetis, Inc",
    "24.6642": "Teradyne K.K.",
    "24.6641": "Star Communication Network Technology Co.,Ltd",
    "24.6640": "Unionman Technology Co.,Ltd",
    "24.5289923": "Samsung Electronics Co.,Ltd",
    "24.3687397": "Grotech Inc",
    "24.6649": "TDK-Lambda",
    "24.6648": "Embedded Systems Design, Inc.",
    "24.1017": "Pleiades Communications, Inc.",
    "24.1016": "SanCastle Technologies, Inc.",
    "24.1015": "Plast-Control GmbH",
    "24.1014": "Allegro Networks, Inc.",
    "24.1013": "Chip2Chip",
    "24.1012": "NetBurner",
    "24.1011": "Dazzle Multimedia, Inc.",
    "24.1010": "Seneca Networks",
    "12.123917683646464": "The Dini Group, La Jolla inc.",
    "12.346861531136": "Technologies Sensio Inc",
    "24.30269": "Veea",
    "24.1018": "TiMetra Networks",
    "12.123917676032000": "Cygnus LLC",
    "12.346865410048": "The Logical Company",
    "24.14960528": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346855923712": "RapidWave Inc.",
    "12.346866434048": "Gamber Johnson LLC",
    "24.578202": "Proformatique",
    "20.181338950860800": "leakSMART",
    "24.4236983": "Intel Corporate",
    "24.8894": "Cisco Systems, Inc",
    "24.8895": "SieAmp Group of Companies",
    "20.84531193315328": "Remonde Network",
    "24.8896": "Shenzhen Forcelink Electronic Co, Ltd",
    "24.5307051": "L-tek d.o.o.",
    "24.8897": "Active Storage Inc.",
    "24.9471604": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.123917688291328": "Crde",
    "24.8890": "HUTH Elektronik Systeme GmbH",
    "24.13149993": "SYStronics Co., Ltd.",
    "24.5267844": "Avaya Inc",
    "24.284783": "Sichuan Whayer information industry Co.,LTD",
    "12.123917679276032": "Axess AG",
    "24.8892": "JDSU France SAS",
    "24.6836068": "Dell Inc.",
    "24.8893": "Cisco Systems, Inc",
    "20.368732798976": "Novexx Solutions GmbH",
    "12.123917678059520": "Schneider Electric Motion",
    "24.7346315": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.74781675225088": "Annapurna labs",
    "24.5545926": "Cisco Systems, Inc",
    "20.198237812162560": "Perspicace Intellegince Technology",
    "24.9997161": "Shenzhen hylitech Co.,LTD",
    "24.4215008": "Milton Security Group LLC",
    "24.34561": "Samsung Electronics Co.,Ltd",
    "12.346858156032": "AVerMedia Technologies, Inc.",
    "24.1335499": "Apple, Inc.",
    "24.16409569": "Samsung Electronics (UK) Ltd",
    "20.212014905425920": "Hofon Automation Co.,Ltd",
    "12.346864177152": "Blankom",
    "24.3177388": "New H3C Technologies Co., Ltd",
    "12.123917685743616": "Vigorcloud Co., Ltd.",
    "24.13688730": "Liteon Technology Corporation",
    "24.10262989": "Voippartners",
    "24.3166712": "BYK-Gardner GmbH",
    "20.225209244188672": "Inx Co.,Ltd.",
    "24.12353945": "BASIC Co.,Ltd.",
    "24.7919455": "Apple, Inc.",
    "12.346862698496": "Phytec Messtechnik GmbH",
    "24.13920874": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.220327870201856": "Shenzhen Wesion Technology Co., Ltd",
    "12.346863718400": "POLA s.r.l.",
    "12.123917688811520": "StreamPlay Oy Ltd",
    "24.6837338": "Intel Corporate",
    "24.15774437": "Cisco Systems, Inc",
    "12.123917678034944": "Schneider Electric Motion USA",
    "24.10803505": "Apple, Inc.",
    "12.123917687857152": "Crde",
    "24.5017978": "IEEE Registration Authority",
    "12.123917683830784": "AVL DiTEST",
    "12.123917681299456": "Tricom Research Inc.",
    "20.154314894999552": "Nanonord A/S",
    "12.346853507072": "Alltec GmbH",
    "24.14201130": "Panasonic Mobile Communications Co.,Ltd.",
    "24.14171071": "Intel Corporate",
    "12.71297883619328": "Rohde&Schwarz Topex SA",
    "20.137149115531264": "Nanning auto digital technology co.,LTD",
    "24.2897239": "ELLIY Power CO..Ltd",
    "12.346863493120": "ChronoLogic Pty. Ltd.",
    "24.16035009": "Cisco Systems, Inc",
    "20.97400603541504": "Norgren Manufacturing Co., Ltd.",
    "24.11540291": "Edison Global Circuits Llc",
    "24.11578578": "zte corporation",
    "24.1323206": "Motorola Mobility LLC, a Lenovo Company",
    "24.13938258": "Dell Inc.",
    "24.5023085": "Samsung Electronics Co.,Ltd",
    "24.7396248": "Cisco Systems, Inc",
    "24.3705132": "Apple, Inc.",
    "24.11051159": "ScioTeq bvba",
    "20.70559710838784": "Suteng Innovation Technology Co., Ltd.",
    "12.123917676191744": "Henri Systems Holland bv",
    "12.346863661056": "Midwest Microwave Solutions Inc.",
    "24.7387050": "vivo Mobile Communication Co., Ltd.",
    "24.15774761": "Xiaomi Communications Co Ltd",
    "24.7106302": "Core Logic Inc.",
    "24.6605727": "Tecno Mobile Limited",
    "24.13404198": "LG Innotek",
    "24.16302225": "Highgates Technology",
    "12.346861006848": "b-plus GmbH",
    "24.2106776": "Teleview",
    "20.141720408817664": "Neutronics",
    "12.123917683085312": "VOCAL Technologies Ltd.",
    "12.346861879296": "Wireonair A/S",
    "24.4509242": "Jiangsu Huacun Electronic Technology Co., Ltd.",
    "24.3203990": "Huawei Device Co., Ltd.",
    "24.3995656": "Huawei Technologies Co.,Ltd",
    "24.16037512": "Plantronics, Inc.",
    "24.4756743": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.8672677": "Samsung Electronics Co.,Ltd",
    "24.13912109": "Shenzhen Deejoy Lighting Technology Co.,Ltd.",
    "24.7882072": "Ubiquiti Networks Inc.",
    "24.526530": "Samsung Electronics Co.,Ltd",
    "24.14696455": "Huawei Device Co., Ltd.",
    "24.4507291": "Wuhan Feng Tian Information Network CO.,LTD",
    "20.123598318600192": "Bas-Ip Lp",
    "24.12348237": "Nokia",
    "12.123917677420544": "Chengdu Cove Technology CO.,LTD",
    "12.123917686984704": "Movis",
    "20.27056772481024": "Starfire Industries LLC",
    "12.123917686398976": "Isac Srl",
    "24.7873860": "Omnima Limited",
    "24.15767999": "Apple, Inc.",
    "12.123917685940224": "JSC Electrical Equipment Factory",
    "12.123917683109888": "Aloxy",
    "12.346855903232": "Esensors, Inc.",
    "12.346864906240": "Qtechnology A/S",
    "24.15767990": "Apple, Inc.",
    "24.7642511": "Huawei Technologies Co.,Ltd",
    "12.123917677187072": "Dataspeed Inc",
    "24.4483767": "ARRIS Group, Inc.",
    "24.586167": "Towerstream Corpration",
    "24.9482477": "Apple, Inc.",
    "20.207357806116864": "Shenzhen Colorwin Optical Technology Co.,Ltd",
    "24.544681": "Cisco Systems, Inc",
    "24.11300378": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917680775168": "Thermokon Sensortechnik GmbH",
    "24.7090910": "Indieon Technologies Pvt. Ltd.",
    "24.12076922": "Dasan Electron Co., Ltd.",
    "24.13425555": "Cisco Systems, Inc",
    "12.123917680730112": "Shandong Hospot IOT Technology Co.,Ltd.",
    "20.57353098493952": "Chengdu Meross Technology Co., Ltd.",
    "24.15265029": "ARRIS Group, Inc.",
    "24.8409115": "Nokia Corporation",
    "12.123917687734272": "Elbit Systems of America",
    "20.4884648689664": "KittyHawk Corporation",
    "24.2404180": "NEXGEN Mediatech Inc.",
    "24.9195781": "Shenzhen ireadygo Information Technology CO.,LTD.",
    "12.346862546944": "Dorlet Sau",
    "24.10018": "Ubiquiti Networks Inc.",
    "20.114975479693312": "WiseCube",
    "24.10016": "New-Sol Com",
    "24.10017": "Shenzhen Baoan Fenda Industrial Co., Ltd",
    "24.10014": "Xagyl Communications",
    "24.10015": "MIPRO Electronics Co., Ltd",
    "24.10012": "Mercury Corporation",
    "24.10013": "Comba Telecom Systems (China) Ltd.",
    "24.10010": "Geenovo Technology Ltd.",
    "24.10011": "Alec Sicherheitssysteme GmbH",
    "12.346860077056": "Giga-tronics, Inc.",
    "24.7001": "Sony Mobile Communications Inc",
    "24.4259341": "Maxio",
    "24.12623885": "ARRIS Group, Inc.",
    "20.145727904808960": "SmartGiant Technology",
    "24.1672": "Telways Communication Co., Ltd.",
    "24.2416326": "Aruba, a Hewlett Packard Enterprise Company",
    "12.346868391936": "C3 Llc",
    "24.9467037": "IRay Technology Co., Ltd.",
    "24.840373": "Binatone Telecommunication Pvt. Ltd",
    "24.4076": "ARKUS Inc.",
    "24.4077": "Anam Electronics Co., Ltd",
    "24.4074": "Giga-Byte Technology Co.,LTD.",
    "24.4075": "Cylon Controls",
    "24.4072": "Lobos, Inc.",
    "24.4073": "Gw Technologies Co.,Ltd.",
    "24.4070": "MBTech Systems, Inc.",
    "24.4071": "Lutron Electronics Co., Inc.",
    "24.527371": "SysMik GmbH Dresden",
    "24.4078": "XTec, Incorporated",
    "24.4079": "Thales e-Transactions GmbH",
    "24.9437262": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.193649365221376": "Steffens Systems GmbH",
    "24.7354548": "Low Tech Solutions",
    "24.9212753": "Jigowatts Inc.",
    "12.71297884565504": "BlinkPipe Ltd",
    "24.10797739": "Hangzhou LEAD-IT Information & Technology Co.,Ltd",
    "12.123917688422400": "AEC s.r.l.",
    "24.14463064": "Bürkert Werke GmbH",
    "12.346854526976": "Contronics Automacao Ltda",
    "20.167295454281728": "Anhui Shengren Electronic Technology Co., Ltd",
    "20.13592322637824": "Colordeve International",
    "24.10808275": "Samsung Electronics Co.,Ltd",
    "24.2144048": "TeconGroup, Inc",
    "20.176094522114048": "Kryptus Information Security S/A",
    "24.4496321": "Huawei Technologies Co.,Ltd",
    "24.49185": "Netexpress",
    "24.7362950": "Ooo Ttv",
    "24.5254342": "Gree Electric Appliances, Inc. Of Zhuhai",
    "24.578973": "Sagemcom Broadband SAS",
    "12.123917686063104": "Cyg Contron Co.Ltd",
    "24.2131884": "AES motomation",
    "20.44768527646720": "Beijing Kitten&Puppy Technology Co.,Ltd.",
    "12.123917686874112": "AEye, Inc.",
    "20.255553906409472": "z-max mediasolution",
    "24.15753050": "Getriebebau NORD GmbH and Co. KG",
    "24.2662787": "Alpsalpine Co,.Ltd",
    "24.13130399": "zte corporation",
    "24.4987541": "Novakon Co., Ltd.",
    "12.346859380736": "Phytec Messtechnik GmbH",
    "20.58071056384000": "Orosound Sas",
    "12.346860761088": "LaserLinc, Inc.",
    "24.14685246": "Modoosis Inc.",
    "24.15505601": "Beijing Risbo Network Technology Co.,Ltd",
    "24.6292114": "Pegatron Corporation",
    "20.154238418157568": "HuiZhou MIKI Communication Equipment Co.,LTD",
    "12.346860470272": "Lyngsoe Systems",
    "24.16018316": "Hon Hai Precision Ind. Co., Ltd.",
    "12.123917680455680": "Nippon Marine Enterprises, Ltd.",
    "24.5287757": "zte corporation",
    "12.123917679038464": "Aplex Technology Inc.",
    "24.3182991": "Technicolor",
    "24.12300": "Congruency, Ltd.",
    "24.12301": "MMC Technology, Inc.",
    "24.12302": "Klotz Digital AG",
    "24.12303": "IMT - Information Management T",
    "24.12304": "Visionetics International",
    "24.12305": "HMS Industrial Networks",
    "24.12306": "Digital Engineering Ltd.",
    "24.12307": "NEC Corporation",
    "24.12308": "Divio, Inc.",
    "24.12309": "Cp Clare Corp.",
    "24.7382732": "Intel Corporate",
    "24.6032398": "Guizhou Huaxintong Semiconductor Technology Co Ltd",
    "24.7386600": "Dell Inc.",
    "24.7119028": "BSkyB Ltd",
    "24.5540130": "Zelfy Inc",
    "24.2902545": "Ciena Corporation",
    "24.7117513": "Valentine Research, Inc.",
    "24.1087123": "Amazon Technologies Inc.",
    "24.9445638": "BSkyB Ltd",
    "24.6586521": "Intel Corporate",
    "24.287081": "Media Global Links Co., Ltd.",
    "12.123917687914496": "Combilent",
    "24.7756": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7757": "Welkin Sciences, LLC",
    "24.7754": "Cisco Systems, Inc",
    "24.7755": "City Theatrical",
    "12.346867761152": "osf Hansjuergen Meier GmbH & Co. KG",
    "24.7753": "Cisco Systems, Inc",
    "24.7750": "ARRIS Group, Inc.",
    "24.7751": "PT. Hariff Daya Tunggal Engineering",
    "24.2410913": "Ruckus Wireless",
    "20.162752342523904": "Zimmer GmbH",
    "24.7758": "DAKO EDV-Ingenieur- und Systemhaus GmbH",
    "24.7759": "Dell Inc.",
    "24.64059": "Cloos Electronic Gmbh",
    "20.154680183226368": "Chengdu BillDTE Technology Co., Ltd",
    "24.12867438": "Samsung Electronics Co.,Ltd",
    "24.1065203": "Apple, Inc.",
    "20.22333145219072": "Qingdao Wodatong Electronics Co., Ltd.",
    "24.24798": "Kayser-Threde GmbH",
    "24.24799": "Brocade Communications Systems LLC",
    "24.24796": "NEC Magnus Communications,Ltd.",
    "20.264809151987712": "Rievtech Electronic Co.,Ltd",
    "24.9194986": "Cerio Corporation",
    "24.8134226": "Microsoft",
    "24.24797": "Myricom, Inc.",
    "12.123917686562816": "Multivoice Llc",
    "24.2885510": "Huawei Device Co., Ltd.",
    "20.207147371593728": "Shenzhen Yaguang communication CO.,LTD",
    "12.346864148480": "Private",
    "20.220092031827968": "Grav I.T.",
    "24.5776425": "Google, Inc.",
    "24.4738089": "Huawei Technologies Co.,Ltd",
    "20.89058728476672": "Informa LLC",
    "24.1578070": "Nokia Corporation",
    "12.123917682290688": "iungo",
    "24.6841177": "Advanced Digital Broadcast SA",
    "12.123917686603776": "Zumbach Electronic AG",
    "24.11846734": "VXL eTech Pvt Ltd",
    "12.123917683101696": "QIAGEN Instruments AG",
    "24.16555950": "Fidus Systems Inc",
    "24.14479504": "Nubia Technology Co.,Ltd.",
    "24.12338841": "Tp-Link Technologies Co.,Ltd.",
    "24.13127886": "Huawei Technologies Co.,Ltd",
    "24.6015": "Worldsmart Retech",
    "12.71297884938240": "Autonomous Solutions, Inc",
    "24.12073162": "Kyokko Tsushin System CO.,LTD",
    "24.2706": "Presonus Corporation",
    "24.2707": "W2 Networks, Inc.",
    "24.2704": "Bayside Interactive, Inc.",
    "24.2705": "HemoCue AB",
    "24.2702": "Invacom Ltd",
    "24.2703": "Aska International Inc.",
    "24.2700": "Guardware Systems Ltd.",
    "24.2701": "Eurotherm Limited",
    "20.27056779821056": "Christ Electronic System GmbH",
    "24.280730": "Apple, Inc.",
    "24.6089": "Samsung Electronics Co.,Ltd",
    "24.6088": "KYOCERA Display Corporation",
    "24.2708": "ShangHai cellink CO., LTD",
    "24.2709": "Apple, Inc.",
    "24.13427157": "Buffalo.Inc",
    "12.123917677285376": "Leder Elektronik Design",
    "24.8533": "Cisco Systems, Inc",
    "24.8532": "D-TACQ Solutions Ltd",
    "24.8531": "SeaMicro Inc.",
    "24.8530": "General Satellite Research & Development Limited",
    "24.8537": "Juniper Networks",
    "24.8536": "Style Flying Technology Co.",
    "24.8535": "National Datacast, Inc.",
    "24.8534": "Cisco Systems, Inc",
    "24.8539": "SenseAnywhere",
    "24.8538": "Hewlett Packard",
    "20.13592326832128": "BSU Inc",
    "24.2148531": "Shenzhen Bul-Tech Co.,Ltd.",
    "12.123917683523584": "Becker Nachrichtentechnik GmbH",
    "24.5827348": "Proteus Digital Health",
    "24.6596965": "Linkflow Co., Ltd.",
    "12.123917684142080": "Contec Americas Inc.",
    "24.10017426": "Vuzix Corporation",
    "24.2924610": "Huawei Device Co., Ltd.",
    "20.150262239985664": "kimura giken corporation",
    "12.123917685673984": "Project Service S.r.l.",
    "20.167137986478080": "EBI Ltd.",
    "20.180659898286080": "Olssen B.V.",
    "24.5068": "Tall Maple Systems",
    "24.5069": "MTI co. LTD",
    "24.5066": "Pico Digital",
    "24.5067": "Zenitel Norway AS",
    "24.5064": "ADB Broadband Italia",
    "24.5065": "Beyond Achieve Enterprises Ltd.",
    "24.5062": "OpenGear, Inc",
    "24.5063": "IONOS Co.,Ltd.",
    "24.5060": "Cisco Systems, Inc",
    "24.5061": "Lightron Fiber-Optic Devices Inc.",
    "24.8289": "Dynatech Communications Inc",
    "24.8288": "ALCATEL ITALIA S.p.A.",
    "24.8285": "Nanomatic Oy",
    "24.8284": "InterNet Systems of Florida, Inc.",
    "24.8287": "Gammadata Computer Gmbh",
    "24.8286": "Castle Rock, Inc.",
    "24.8281": "Miro Computer Products Ag",
    "24.8280": "Allied Signal Inc.",
    "24.8283": "Kentrox, LLC",
    "24.8282": "Computer Identics",
    "24.10490683": "HiTi Digital, Inc.",
    "12.119270047744": "Dialine",
    "12.123917685039104": "Kaeonit",
    "24.13162628": "Universal Electronics, Inc.",
    "12.123917679865856": "Tualcom Elektronik A.S.",
    "12.123917681979392": "Partilink Inc.",
    "24.12871633": "Huawei Technologies Co.,Ltd",
    "20.159182274691072": "Private",
    "24.12644021": "Enice Network.",
    "24.14161601": "Dewav (Hk) Electronics Limited",
    "12.346853965824": "Electronic Systems Development",
    "24.15780305": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "20.114869209661440": "Babbit and Friends, SIA",
    "12.346861494272": "Sandel Avionics, Inc.",
    "24.13134878": "Display Systems Ltd",
    "24.15998315": "Huawei Technologies Co.,Ltd",
    "24.8650962": "Sony Mobile Communications Inc",
    "24.6855774": "AcSiP Technology Corp.",
    "24.13661070": "Dong Oh Precision Co., Ltd.",
    "24.571372": "Wireless Seismic",
    "12.123917677318144": "Cameon S.A.",
    "12.123917678260224": "SilTerra Malaysia Sdn. Bhd.",
    "24.3465431": "Dongguan Qisheng Electronics Industrial Co., Ltd",
    "24.16262023": "Shenzhen Horn Audio Co.,Ltd.",
    "12.346852507648": "Optionexist Limited",
    "24.6607188": "Routerboard.com",
    "12.123917677654016": "Hypex Electronics BV",
    "20.198237810065408": "Shenzhen Hi-Net Technology Co., Ltd.",
    "24.14686493": "Huawei Technologies Co.,Ltd",
    "20.114869203369984": "Annapurna labs",
    "20.368735944704": "Ooo \"Dekatron\"",
    "24.11577910": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.311737": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11854841": "Silicon Laboratories",
    "12.346861527040": "Pcsc",
    "12.346868875264": "GD Mission Systems",
    "24.8157310": "Samsung Electronics Co.,Ltd",
    "24.6846031": "Sony Mobile Communications Inc",
    "12.346867916800": "Micro-Key BV",
    "24.1850355": "Tp-Link Technologies Co.,Ltd.",
    "24.15469486": "Hangzhou BroadLink Technology Co.,Ltd",
    "24.6092774": "Apple, Inc.",
    "12.346854055936": "Datatek Applications, Inc.",
    "24.2391633": "zte corporation",
    "24.4464273": "ARVIDA Intelligent Electronics Technology Co.,Ltd.",
    "24.5832029": "Hangzhou Xinyun technology Co., Ltd.",
    "12.123917678702592": "GWF MessSysteme AG",
    "24.12845378": "MaxMedia Technology Limited",
    "24.11571511": "ISis ImageStream Internet Solutions, Inc",
    "24.12361125": "DASCOM Europe GmbH",
    "20.62366879318016": "ExaScaler Inc.",
    "12.123917680242688": "Shanghai AiGentoo Information Technology Co.,Ltd.",
    "12.346867048448": "Oshoksh Corporation",
    "24.57392": "Melita International Corp.",
    "12.123917688127488": "Linc Technology Corporation dba Data-Linc Group",
    "24.263126": "Nintendo Co.,Ltd",
    "24.3684897": "IEEE Registration Authority",
    "24.14737833": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "24.13660034": "Huawei Technologies Co.,Ltd",
    "24.6292874": "Hitachi Metals, Ltd.",
    "20.229631762366464": "Shenzhen FDC Electuonic Co.,Ltd.",
    "20.27056780869632": "legendsky tech",
    "12.346862272512": "Entwicklung Hard- & Software",
    "12.123917687697408": "Labinvent JSC",
    "24.4736083": "HDS ???",
    "24.9734238": "Surfilter Network Technology Co., Ltd.",
    "24.16266717": "Gemtek Technology Co., Ltd.",
    "24.6605124": "LARK Technologies, Inc",
    "12.123917676773376": "NAL Research Corporation",
    "24.2417265": "Qingdao Hisense Communications Co.,Ltd.",
    "24.1849641": "Ohsung",
    "12.123917683118080": "TableConnect GmbH",
    "20.101728416432128": "mobilogix HongKong",
    "12.346867507200": "ThinkRF Corp",
    "24.11818352": "Borea",
    "20.119709252976640": "Hardmeier",
    "24.12905914": "Texas Instruments",
    "24.2401588": "SynTrust Tech International Ltd.",
    "24.13906528": "GoPro",
    "24.5768488": "Labris Networks",
    "24.16525811": "McPay Co.,LTD.",
    "20.132811182833664": "Nomiku",
    "24.15746522": "Texas Instruments",
    "24.9187129": "IBA Dosimetry GmbH",
    "24.16532831": "Jiangxi Shanshui Optoelectronic Technology Co.,Ltd",
    "12.346862190592": "Keith & Koep GmbH",
    "24.9448402": "Huawei Technologies Co.,Ltd",
    "20.154314893950976": "OBO Pro.2 Inc.",
    "24.3451531": "Samsung Electronics Co.,Ltd",
    "24.10536290": "ARRIS Group, Inc.",
    "24.63075": "Cisco Systems, Inc",
    "12.123917690896384": "Yi An Electronics Co., Ltd",
    "20.228784611524608": "Video Security, Inc.",
    "12.123917676314624": "Virta Laboratories, Inc.",
    "24.1630132": "Bosung Powertec Co.,Ltd.",
    "12.119269380096": "andersen lighting GmbH",
    "24.10812022": "New H3C Technologies Co., Ltd",
    "24.2419146": "Huawei Device Co., Ltd.",
    "12.119269249024": "Private",
    "12.346865942528": "Expro North Sea",
    "24.14959435": "Dell Inc.",
    "12.123917685633024": "LG Electronics",
    "12.123917681332224": "PBSI Group Limited",
    "12.346868756480": "Aster Electric Co.,Ltd.",
    "24.12330480": "3D Display Technologies Co., Ltd.",
    "12.123917689610240": "Crde",
    "20.31368181448704": "Beidian Group",
    "20.102196703133696": "beijing your wonderful control system technology co.,ltd",
    "12.346865684480": "Maretron, LLP",
    "12.346859671552": "SP Controls, Inc",
    "24.11286640": "Lenovo Mobile Communication Technology Ltd.",
    "24.6350576": "Wistron Neweb Corporation",
    "24.9227682": "ACTP, Inc.",
    "12.123917688930304": "HEADROOM Broadcast GmbH",
    "24.9316": "Bridge Technologies Co AS",
    "12.346868043776": "Samway Electronic SRL",
    "24.2156433": "Siemens Healthcare Diagnostics, Inc",
    "24.5040435": "RailComm, LLC",
    "12.346866880512": "Ateme",
    "24.13904964": "Cisco Systems, Inc",
    "12.346858823680": "Hisstema AB",
    "12.123917683728384": "NetBoxSC, LLC",
    "24.837425": "Currant, Inc.",
    "20.31614743609344": "Ospicon Company Limited",
    "24.5295410": "NanJing Chaoran Science & Technology Co.,Ltd.",
    "12.346859950080": "Mobilisme",
    "24.14737602": "China Mobile Group Device Co.,Ltd.",
    "12.346856345600": "HARTMANN software GbR",
    "24.62305": "Amazon Technologies Inc.",
    "24.9985453": "Texas Instruments",
    "24.10795032": "PENTA Gesellschaft für elektronische Industriedatenverarbeitung mbH",
    "12.346857070592": "ads-tec GmbH",
    "24.16577988": "Espressif Inc.",
    "24.9752617": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.8156787": "ENMAS GmbH",
    "12.123917679546368": "Heartlandmicropayments",
    "24.16097": "Apple, Inc.",
    "24.10253168": "Huawei Technologies Co.,Ltd",
    "24.16273011": "Apple, Inc.",
    "24.10272480": "Biosoundlab Co., Ltd.",
    "24.15245893": "Juniper Networks",
    "12.123917687775232": "Triax A/S",
    "24.12884060": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.10026858": "OTEC(Shanghai)Technology Co.,Ltd.",
    "24.1842566": "Cisco Systems, Inc",
    "24.832281": "Samsung Electronics Co.,Ltd",
    "24.3435535": "Pramod Telecom Pvt. Ltd",
    "24.7904389": "August Home, Inc.",
    "12.346854760448": "Maestro Pty Ltd",
    "24.3699690": "IEEE Registration Authority",
    "12.346863796224": "Rosslare Enterprises Limited",
    "24.9718284": "Sercomm Corporation.",
    "24.7093916": "KE Knestel Elektronik GmbH",
    "20.145727918440448": "Price Industries Limited",
    "24.10235995": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1861021": "Liteon Technology Corporation",
    "20.150262238937088": "Sieper Lüdenscheid GmbH & Co. KG",
    "24.7752": "Wi-Links",
    "24.6611115": "Unionman Technology Co.,Ltd",
    "12.123917689954304": "Neusoft Reach Automotive Technology (Shenyang) Co.,Ltd",
    "20.128612185407488": "Ballard Technology, Inc,",
    "24.4243182": "Shenzhen Yunding Information Technology Co.,Ltd",
    "24.11280343": "Real Vision Technology Co.,Ltd.",
    "24.3438277": "I-O Data Device,Inc.",
    "24.1059243": "Samsung Electronics Co.,Ltd",
    "12.123917676711936": "HMicro Inc",
    "20.154019630678016": "Shenzhen Meidou Technology Co, Ltd.",
    "20.102196711522304": "Itron UK Limited",
    "24.4767595": "Apple, Inc.",
    "20.242185273344000": "General Microsystems Sdn Bhd",
    "24.5811880": "Huawei Technologies Co.,Ltd",
    "24.551895": "Silicon Laboratories",
    "24.10263934": "Beijing Xiaomi Mobile Software Co., Ltd",
    "20.145727914246144": "Shenzhen Xtooltech Co., Ltd",
    "12.123917686116352": "Bandelin electronic GmbH & Co. KG",
    "24.4482317": "Amazon Technologies Inc.",
    "24.10809832": "Apple, Inc.",
    "12.123917679423488": "Digital Way",
    "24.6033605": "Samsung Electronics Co.,Ltd",
    "24.10752578": "NEC Platforms, Ltd.",
    "24.7396769": "Tellabs",
    "24.1630914": "Samsung Electronics Co.,Ltd",
    "24.14470285": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.1859583": "Beijing Ereneben Information Technology Co.,Ltd Shenzhen Branch",
    "24.8165219": "Openmatics s.r.o.",
    "12.71297883230208": "HuNS",
    "12.346864992256": "ELSIST S.r.l.",
    "24.1598864": "Apple, Inc.",
    "24.11287694": "Flextronics Computing(Suzhou)Co.,Ltd.",
    "12.123917686525952": "dongsheng",
    "20.193649366269952": "Rademacher Geraete-Elektronik GmbH",
    "24.11329871": "Autonomic Controls, Inc.",
    "12.123917686026240": "SHENZHEN HUINENGYUAN Technology Co., Ltd",
    "12.123917678940160": "thingdust AG",
    "24.11057587": "Essys",
    "12.123917690654720": "COMM-connect A/S",
    "12.123917691412480": "RFL Electronics, Inc.",
    "12.346855727104": "Vanguard Technology Corp.",
    "24.5562064": "Sevio Srl",
    "24.9757437": "Boge Kompressoren OTTO Boge GmbH & Co. KG",
    "24.11333179": "6harmonics Inc",
    "12.71297884766208": "Satkirit Ltd",
    "12.123917677535232": "Talleres de Escoriaza SA",
    "12.346867736576": "Joint Stock Company \"Svyaz Inginiring M\"",
    "24.6081482": "Fujian Star-Net Communication Co.,Ltd",
    "20.48548669292544": "Private",
    "24.12620853": "ARRIS Group, Inc.",
    "12.71297884434432": "Tattile Srl",
    "24.1358184": "Huawei Technologies Co.,Ltd",
    "24.13956286": "Palo Alto Networks",
    "20.44768534986752": "Simaudio Ltd",
    "12.123917676056576": "Adolf Nissen Elektrobau GmbH + Co. KG",
    "24.4719458": "Desay Electronics(Huizhou)Co.,Ltd",
    "12.123917683630080": "Emdep Centro Tecnologico Mexico",
    "24.10803775": "Bionics Corp.",
    "24.12587586": "Alpha Security Products",
    "24.4246464": "Apple, Inc.",
    "24.14421296": "Shenzhen Feasycom Technology Co., Ltd.",
    "12.123917675487232": "Private Enterprise \"Scientific and Production Private Enterprise\"Sparing-Vist Center\"\"",
    "12.123917681688576": "Smart Service Technologies CO., LTD",
    "12.123917687353344": "Autonomic Controls, Inc.",
    "12.123917682171904": "KMtronic ltd",
    "12.123917683494912": "«Intellect module» LLC",
    "24.16537": "Newgen Systems Corp.",
    "24.16536": "Dressler Gmbh & Co.",
    "24.16535": "Datex Division Of",
    "12.346855956480": "A.R.G ElectroDesign Ltd",
    "24.16533": "R.P.T. Intergroups Int'L Ltd.",
    "24.3166069": "DASAN Network Solutions",
    "24.16531": "Paxdata Networks Ltd.",
    "24.16530": "Asp Computer Products, Inc.",
    "24.45115": "HiQ Networks",
    "24.7906781": "Shenzhen Smarteye Digital Electronics Co., Ltd",
    "24.16539": "Hal Computer Systems Inc.",
    "24.16538": "Network Express, Inc.",
    "24.3995025": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.272805460443136": "Ophrys Systèmes",
    "24.3456912": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917685448704": "Edgeware AB",
    "24.5248181": "Richfit Information Technology Co., Ltd",
    "24.7380905": "Tecno Mobile Limited",
    "24.11328474": "Shenzhen Geniatech Inc, Ltd",
    "24.12373964": "HTC Corporation",
    "20.93020693200896": "Orient Direct, Inc.",
    "20.31372481658880": "Itw-Feg",
    "12.123917683060736": "Hessware GmbH",
    "20.167594811195392": "DH Mechatronic AG",
    "12.346862411776": "Protronic GmbH",
    "24.2362052": "Espressif Inc.",
    "24.14724337": "Cisco Systems, Inc",
    "24.5268": "Esu Ag",
    "24.5269": "2Wire Inc",
    "24.9762733": "Juniper Networks",
    "24.5264": "ASP Corporation",
    "24.5265": "Daniels Electronics Ltd. dbo Codan Rado Communications",
    "24.5266": "Liteon, Mobile Media Solution SBU",
    "24.5267": "Systimax Solutions",
    "24.5260": "General Dynamics Mission Systems",
    "24.5261": "Cubic Defense Simulation Systems",
    "24.5262": "Tele Power Inc.",
    "24.5263": "Protronic (Far East) Ltd.",
    "12.123917684477952": "tetronik GmbH AEN",
    "24.544522": "Huawei Technologies Co.,Ltd",
    "12.346866253824": "C. Rob. Hammerstein GmbH & Co. KG",
    "24.13138723": "Bose Corporation",
    "24.7369148": "Pegatron Corporation",
    "20.158667121885184": "Ibm",
    "24.8925972": "Texas Instruments",
    "20.145735981989888": "Smart Code (Shenzhen) Technology Co.,Ltd",
    "12.123917686906880": "GTI Technologies Inc",
    "24.14207209": "Phicomm (Shanghai) Co., Ltd.",
    "24.5025443": "Hewlett Packard Enterprise",
    "24.16301070": "eero inc.",
    "24.13682267": "Huawei Technologies Co.,Ltd",
    "24.13172330": "Juniper Networks",
    "24.17665": "Midmark RTLS",
    "12.346856812544": "Neuroware",
    "12.71297885024256": "Cornfed Systems LLC",
    "24.8659441": "Huawei Technologies Co.,Ltd",
    "24.4475774": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.27478322053120": "Sercomm Corporation.",
    "24.12520": "Ensim Corp.",
    "24.12521": "Gma Communication Manufact'G",
    "24.12522": "TeraForce Technology Corporation",
    "12.346858713088": "Speech Technology Center Limited",
    "24.12524": "Borgardt",
    "24.12525": "Expert Magnetics Corp.",
    "24.12526": "DSG Technology, Inc.",
    "24.12527": "Neon Technology, Inc.",
    "24.12528": "Uniform Industrial Corp.",
    "24.12529": "Accton Technology Corp",
    "24.8148814": "COBI GmbH",
    "24.56613": "Shenzhen hechengdong Technology Co., Ltd",
    "20.44169485615104": "Shenzhen Neoway Technology Co.,Ltd.",
    "24.15504365": "DDoS-Guard LTD",
    "24.11058727": "Apple, Inc.",
    "12.346866675712": "Tornado Modular Systems",
    "12.123917685096448": "Eldes",
    "20.163603966590976": "Shenzhen Sanray Technology Co.,Ltd",
    "24.3458512": "Hagleitner Hygiene International GmbH",
    "24.11310956": "Greenliant",
    "24.5025192": "Panoptics Corp.",
    "24.10804816": "Total Walther GmbH",
    "24.573047": "Green Electronics",
    "20.27056768286720": "ChengDu Vantron Technology, Ltd.",
    "24.3959453": "Dexatek Technology LTD.",
    "24.6349740": "LG Electronics (Mobile Communications)",
    "24.9502678": "Honor Device Co., Ltd.",
    "12.346865852416": "Weinert Engineering GmbH",
    "12.123917687541760": "Acacia Research",
    "24.5769539": "Private",
    "24.15260192": "Nintendo Co.,Ltd",
    "12.346865905664": "TECHKON GmbH",
    "20.66426711834624": "WICKS Co., Ltd.",
    "12.346859601920": "illunis LLC",
    "24.53372": "Koyo Electronics Inc. Co.,Ltd.",
    "12.123917678481408": "KST technology",
    "24.6609688": "OpenPattern",
    "12.119269810176": "TrendPoint Systems",
    "24.4605": "Korg Inc.",
    "24.4604": "HARTING Electronics GmbH",
    "24.4607": "Digitro Tecnologia Ltda",
    "24.4606": "Keiyo System Research, Inc.",
    "24.4601": "Nortel Networks",
    "24.4600": "AIRAYA Corp",
    "24.4603": "Heidelberg Engineering GmbH",
    "12.346855964672": "StreetFire Sound Labs, LLC",
    "24.5013651": "Intel Corporate",
    "24.4609": "Cisco Systems, Inc",
    "24.4608": "Cisco Systems, Inc",
    "24.7366051": "Kandao lightforge Co., Ltd.",
    "24.16311374": "Mcot Inc.",
    "24.15474059": "Cisco Systems, Inc",
    "24.10247309": "Firemax Indústria E Comércio De Produtos Eletrônicos Ltda",
    "24.3950": "BBox",
    "24.3951": "FTA Communication Technologies",
    "24.3952": "Wintec Industries, inc.",
    "24.3953": "Sanmei Electronics Co.,Ltd",
    "24.3954": "Sandburst",
    "24.794835": "vivo Mobile Communication Co., Ltd.",
    "24.3956": "Qamcom Technology AB",
    "24.3957": "First Silicon Solutions",
    "24.3958": "Digital Keystone, Inc.",
    "24.3959": "Dentum Co.,Ltd",
    "12.123917685403648": "Preston Industries dba PolyScience",
    "24.57563": "ViaVideo Communications, Inc.",
    "24.57562": "Alcatel-Lucent Enterprise",
    "24.57561": "Tazmo Co., Ltd.",
    "24.57560": "LANBit Computer, Inc.",
    "24.57567": "DZS GmbH",
    "24.57566": "Datax Nv",
    "24.57565": "Zenith Electronics Corporation",
    "24.57564": "Nexware Corp.",
    "12.346865446912": "SAE IT-systems GmbH & Co. KG",
    "24.8700143": "Samsung Electronics Co.,Ltd",
    "24.57569": "G2 Networks, Inc.",
    "24.57568": "Si Electronics, Ltd.",
    "12.346866163712": "Embedded Monitoring Systems Ltd.",
    "24.16018108": "Adonit Corp. Ltd.",
    "20.13592323686400": "Zhengzhou coal machinery hydraulic electric control Co.,Ltd",
    "12.123917690068992": "Critical Link LLC",
    "24.4746129": "Fleetwood Group Inc.",
    "12.346865000448": "Sysacom",
    "24.11846000": "Yi chip Microelectronics (Hangzhou) Co., Ltd",
    "12.123917679161344": "Advansid",
    "24.6071853": "Cisco Systems, Inc",
    "24.29664": "Samsung Electronics Co.,Ltd",
    "12.346863484928": "ProPhotonix",
    "24.16008685": "Portable Innovation Technology Ltd.",
    "12.123917679681536": "FR-Team International SA",
    "20.277958191218688": "Genitek Engineering sprl",
    "20.30931565936640": "Toyo System CO.,LTD.",
    "24.16266033": "Texas Instruments",
    "24.4760248": "Chengdu Vision-Zenith Tech.Co,.Ltd",
    "12.123917681156096": "Hangzhou AwareTec Technology Co., Ltd",
    "24.6842826": "Hitachi, Ltd.",
    "12.123917681946624": "T+A elektroakustik GmbH & Co.KG",
    "24.4760247": "Kodofon JSC",
    "24.3374": "Matsushita Avionics Systems Corporation",
    "24.3375": "AIN Comm.Tech.Co., LTD",
    "24.3376": "IceFyre Semiconductor",
    "24.3377": "Compellent Technologies, Inc.",
    "24.3370": "Scanmatic AS",
    "24.3371": "Racal Instruments",
    "24.3372": "Net2Edge Limited",
    "24.3373": "NCT Deutschland GmbH",
    "24.5831968": "Systemhouse Solutions AB",
    "12.123917682102272": "CJSC «Russian telecom equipment company» (CJSC RTEC)",
    "24.13940991": "Home Control Singapore Pte Ltd",
    "24.3378": "DispenseSource, Inc.",
    "24.3379": "Prediwave Corp.",
    "24.7361781": "Huawei Technologies Co.,Ltd",
    "20.31368127971328": "Ibeo Automotive Systems GmbH",
    "24.16019856": "Samsung Electronics Co.,Ltd",
    "20.67047983677440": "GRG Banking Technology Co.,Ltd",
    "24.16053669": "Nokia Corporation",
    "12.346856361984": "Promess Incorporated",
    "24.6618890": "Wistron Neweb Corporation",
    "12.123917691363328": "Krabbenhøft og Ingolfsson",
    "24.5045246": "Sercomm Japan Corporation",
    "24.299362": "Microchip Technology Inc.",
    "24.5045061": "Huawei Technologies Co.,Ltd",
    "24.8869": "Texas Instruments",
    "24.8868": "2Wire Inc",
    "24.8865": "Huawei Symantec Technologies Co.,Ltd.",
    "24.8864": "Aptiv Services Us, Llc",
    "24.8867": "California Eastern Laboratories",
    "24.8866": "Xtramus Technologies",
    "24.1086441": "Apple, Inc.",
    "24.8860": "Verismo Networks Inc",
    "24.8863": "Sensys Traffic AB",
    "24.8862": "Social Aid Research Co., Ltd.",
    "24.14209168": "Dell Inc.",
    "24.11589580": "Tridonic GmbH & Co KG",
    "24.10769390": "IEEE Registration Authority",
    "24.25324": "Cisco Systems, Inc",
    "12.346863452160": "Orion Technologies,LLC",
    "24.8960208": "Dongguan Koppo Electronic Co.,Ltd",
    "20.145383545110528": "Guangdong SunMeng Information Technology Co. Ltd.",
    "24.1053678": "Justec International Technology INC.",
    "24.14974397": "Samsung Electronics Co.,Ltd",
    "24.15206539": "Huawei Technologies Co.,Ltd",
    "24.2682579": "Shenzhen Super D Technology Co., Ltd",
    "24.26667": "Huawei Device Co., Ltd.",
    "20.5239434379264": "Sealed Unit Parts Co., Inc.",
    "24.6470": "Cianet Industria e Comercio S/A",
    "24.6471": "Cisco SPVTG",
    "24.6472": "AireSpider Networks",
    "24.6473": "Tentel Comtech Co., Ltd.",
    "24.6474": "Testo Ag",
    "24.6475": "Sagemcom Broadband SAS",
    "24.6476": "Fujian Stelcom information & Technology CO.,Ltd",
    "24.6477": "Avago Technologies Sdn Bhd",
    "24.6478": "Ultra Electronics - TCS (Tactical Communication Systems)",
    "24.6479": "Nokia Danmark A/S",
    "24.16258644": "Intel Corporate",
    "24.6085": "SonicWALL",
    "24.13913014": "Zhejiang Tmall Technology Co., Ltd.",
    "24.6084": "Quanta Microsystems, INC.",
    "24.6087": "MARA Systems Consulting AB",
    "24.2679630": "Huawei Technologies Co.,Ltd",
    "24.6086": "Cross Match Technologies Inc",
    "24.11856304": "Cisco Systems, Inc",
    "24.6081": "CM Precision Technology LTD.",
    "24.15790244": "Amazon Technologies Inc.",
    "24.6080": "PureTech Systems, Inc.",
    "20.264036230889472": "Shenzhen Lachesis Mhealth Co., Ltd.",
    "24.6083": "KTF Technologies Inc.",
    "24.6082": "ADB Broadband Italia",
    "24.24745": "Gesytec Mbh",
    "24.24744": "Tidomat Ab",
    "24.24747": "Larscom Incorporated",
    "24.24746": "Intelligent Devices Inc. (Idi)",
    "24.3687293": "Compal Broadband Networks, Inc.",
    "24.24740": "GEW Technologies (PTY)Ltd",
    "24.24743": "MICROSENS GmbH & CO. KG",
    "24.24742": "Particle Measuring Systems",
    "12.346863779840": "Keith & Koep GmbH",
    "12.123917684518912": "Shenzhen Riitek Technology Co.,Ltd",
    "24.13433316": "Intel Corporate",
    "24.24749": "MegaChips Corporation",
    "24.24748": "Resilience Corporation",
    "24.2394260": "shenzhen lensun Communication Technology LTD",
    "12.123917677006848": "comtime GmbH",
    "12.346867204096": "Agri-hitech LLC",
    "24.16527343": "UTT Technologies Co., Ltd.",
    "24.13386126": "MSA Innovation",
    "24.14955211": "Samsung Electronics Co.,Ltd",
    "24.6309109": "Easy Digital Ltd.",
    "24.14459699": "Shenzhen YOUHUA Technology Co., Ltd",
    "20.154019629629440": "Unwired Networks",
    "24.15223107": "YUGE Information technology Co. Ltd",
    "24.11010858": "Espressif Inc.",
    "24.3960419": "Mitron OY",
    "24.6294": "Great Well Electronic LTD",
    "24.6295": "JESS-LINK PRODUCTS Co., LTD",
    "24.6296": "Kingstate Electronics Corporation",
    "24.6297": "ShenZhen jieshun Science&Technology Industry CO,LTD.",
    "24.6290": "ads-tec GmbH",
    "24.6291": "Shenzhen Photon Broadband Technology Co.,Ltd",
    "24.6292": "NPCore, Inc.",
    "24.6293": "Hansun Technologies Inc.",
    "24.10523916": "China Mobile IOT Company Limited",
    "24.6298": "HANA Micron Inc.",
    "24.6299": "Thomson Inc.",
    "24.9053": "Cisco Systems, Inc",
    "24.9052": "Aprius, Inc.",
    "24.9051": "Gulfstream",
    "24.9050": "Compal Information (Kunshan) Co., Ltd.",
    "24.9057": "Unigen Corporation",
    "24.9056": "Lookit Technology Co., Ltd",
    "24.9055": "Silicon Micro Sensors GmbH",
    "24.9054": "Cisco Systems, Inc",
    "24.396": "Mega Vision",
    "24.397": "AudeSi Technologies",
    "24.9059": "Zhuhai Raysharp Technology Co.,Ltd",
    "24.9058": "Goldline Controls",
    "24.392": "LXCO Technologies ag",
    "24.393": "Refraction Technology, Inc.",
    "24.390": "Uwe Disch",
    "24.391": "I2SE GmbH",
    "24.10795638": "Huawei Technologies Co.,Ltd",
    "24.5929": "Ubicod Co.LTD",
    "24.5928": "Selex Communications",
    "24.5927": "Thermo Ramsey Italia s.r.l.",
    "24.5926": "m2c Electronic Technology Ltd.",
    "24.5925": "Liquid Computing",
    "24.5924": "Studer Professional Audio GmbH",
    "24.5923": "Summit Data Communications",
    "24.5922": "Hanazeder Electronic GmbH",
    "24.5921": "FITRE S.p.A.",
    "24.5920": "Image Sensing Systems, Inc.",
    "24.4482812": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917686751232": "Medtronic",
    "24.12849909": "D-Link International",
    "12.123917689036800": "Andasis Elektronik San. ve Tic. A.Ş.",
    "12.123917683400704": "Qingdao CNR HITACH Railway Signal&communication co.,ltd",
    "12.346852225024": "XLN-t",
    "12.346862989312": "Quality & Design",
    "24.14999856": "TCT mobile ltd",
    "24.13665456": "Sagemcom Broadband SAS",
    "12.123917677666304": "Transit Solutions, LLC.",
    "24.8403466": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346862981120": "Y-products co.ltd.",
    "24.7645162": "Amazon Technologies Inc.",
    "12.123917676818432": "Efento T P Szydłowski K Zaręba Spółka Jawna",
    "24.532459": "BinCube",
    "12.346861223936": "BALOGH T.A.G Corporation",
    "24.7347431": "Intel Corporate",
    "24.9714321": "Amazon Technologies Inc.",
    "12.71297883353088": "Nomatronics",
    "20.88383276711936": "COTT Electronics",
    "24.579008": "Seers Technology Co., Ltd",
    "24.3163297": "Shanghai Nanchao Information Technology",
    "12.346856513536": "STOM System",
    "20.264809153036288": "Shen Zhen Michip Technologies Co.,Ltd.",
    "12.123917675048960": "Axing Ag",
    "24.16560397": "Shenzhen Tian Kun Technology Co.,LTD.",
    "24.16534460": "Sunplus Technology Co., Ltd.",
    "24.2897713": "Shenzhen Yifang Digital Technology Co.,Ltd.",
    "20.35227548254208": "bcheck NV",
    "12.346852618240": "Digital Services Group",
    "24.6354551": "Intel Corporate",
    "24.7097215": "Vossloh-Schwabe Deutschland GmbH",
    "24.11536575": "Vital Connect, Inc.",
    "12.123917684023296": "Algodue Elettronica Srl",
    "24.55183": "Cisco Systems, Inc",
    "20.145383539867648": "Shenzhen Ramos Digital Technology Co,.Ltd.",
    "12.123917677191168": "Flextronics",
    "24.10516503": "Apple, Inc.",
    "24.11294252": "Lava International(H.K) Limited",
    "12.346854875136": "OpVista",
    "12.346860097536": "M Squared Lasers Limited",
    "24.10752562": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1315632": "Anakreon UK LLP",
    "20.167152468361216": "Shenzhen Y&D Electronics Information Co., Ltd",
    "24.9439876": "Mellanox Technologies, Inc.",
    "20.211670377955328": "Suzhou Siheng Science and Technology Ltd.",
    "24.16256833": "Wistron Infocomm (Zhongshan) Corporation",
    "24.14993409": "BLU Products Inc",
    "24.8098": "Datron World Communications, Inc.",
    "24.8099": "T&W Electronics(Shenzhen)Co.,Ltd.",
    "24.8094": "Cisco Systems, Inc",
    "24.8095": "Thomson Telecom Belgium",
    "24.8096": "A10 Networks",
    "24.8097": "Gtran Inc",
    "12.346866380800": "Syes srl",
    "24.8091": "Posbro",
    "24.8092": "Ledco",
    "24.531908": "Thermo Fisher Scientific Messtechnik GmbH",
    "24.12895587": "Tecno Mobile Limited",
    "24.14741285": "Lintes Technology Co., Ltd.",
    "24.2424155": "SmartThings, Inc.",
    "12.123917681991680": "Intermobility",
    "24.9986003": "Microsoft Corporation",
    "12.346858291200": "Technifor Sas",
    "24.10277723": "Samsung Electronics Co.,Ltd",
    "24.14198614": "Sagemcom Broadband SAS",
    "12.346862903296": "Duevi Snc Di Mora E Santese",
    "20.251018407313408": "Alert Alarm AB",
    "12.123917682864128": "Microchip Technology Germany II GmbH&Co.KG",
    "12.123917679144960": "Spea Spa",
    "24.13112997": "Masimo Corporation",
    "12.123917676982272": "Hatel Elektronik LTD. STI.",
    "12.346865606656": "SIM2 Multimedia S.p.A.",
    "12.123917675761664": "Easy Digital Concept",
    "24.1609429": "TP Vision Belgium N.V. - innovation site Brugge",
    "12.346859778048": "Elektro-Top 3000 Ltd.",
    "12.123917677195264": "Tecnint HTE SRL",
    "24.7096160": "Sunitec Enterprise Co.,Ltd",
    "12.346856321024": "Advantage R&D",
    "24.9180615": "Huawei Technologies Co.,Ltd",
    "24.7346479": "Huawei Technologies Co.,Ltd",
    "20.159304573255680": "bitsensing Inc.",
    "12.346859331584": "ADL Electronics Ltd.",
    "12.346866020352": "LJT & Associates, Inc.",
    "24.6328819": "vivo Mobile Communication Co., Ltd.",
    "24.1612391": "CSE-Servelec Limited",
    "24.5278897": "Texas Instruments",
    "24.13646314": "NetApp",
    "12.123917682454528": "elsys",
    "20.194284243386368": "Zenlayer",
    "20.66129344069632": "Abrites Ltd.",
    "24.3708836": "Elecom Co.,Ltd.",
    "24.16302728": "Cisco Systems, Inc",
    "24.13141545": "Shenzhen Duubee Intelligent Technologies Co.,LTD.",
    "24.2906629": "Nokia Corporation",
    "24.12094738": "Apple, Inc.",
    "24.12635121": "Mobiwire Mobiles (NingBo) Co., LTD",
    "24.13113021": "Qingdao Hisense Communications Co.,Ltd.",
    "24.9875": "QVidium Technologies, Inc.",
    "24.9874": "Mitsubishi Electric Corporation",
    "24.9877": "ZT Group Int'l Inc",
    "24.318796": "Velocytech",
    "24.9871": "MTA SpA",
    "24.9870": "Alta Solutions, Inc.",
    "24.9873": "Sagemcom Broadband SAS",
    "24.9872": "I Do It",
    "24.13146015": "Nokia Corporation",
    "24.9879": "Alpha Technologies Inc.",
    "24.9878": "NOOLIX Co., Ltd",
    "12.123917689905152": "Guidance Navigation Limited",
    "20.127658777116672": "Socionext Inc.",
    "24.11590393": "ARRIS Group, Inc.",
    "12.346868338688": "Pro Design Electronic GmbH",
    "24.12364668": "TRnP KOREA Co Ltd",
    "24.11058471": "Hewlett Packard Enterprise",
    "20.136823645929472": "mk-messtechnik GmbH",
    "24.11283881": "Tecno Mobile Limited",
    "12.123917681897472": "Altron, a.s.",
    "24.10261424": "Megatronix (Beijing) Technology Co., Ltd.",
    "24.29304": "Cisco Systems, Inc",
    "12.123917684965376": "Beijing Yourong Runda Rechnology Development Co.Ltd.",
    "12.346866741248": "Elan Systems",
    "20.162752344621056": "Austar Hearing Science And Techniligy(Xiamen)Co.,Ltd",
    "24.12625517": "Inspur Group Co., Ltd.",
    "20.111031372218368": "NPG Technology S.A.",
    "24.60205": "Sony Mobile Communications Inc",
    "12.346863828992": "Five9 Network Systems Llc",
    "12.346858967040": "ESA Messtechnik GmbH",
    "24.8178": "VIA Technologies, Inc.",
    "24.13153139": "Cisco-Linksys, LLC",
    "24.7401889": "Liteon Technology Corporation",
    "24.1603796": "Verizon",
    "24.5515803": "Sonos, Inc.",
    "12.123917684813824": "SRS Group s.r.o.",
    "24.9985350": "PeopleNet Communication",
    "12.346858766336": "CIS Corporation",
    "12.346853568512": "nCore, Inc.",
    "20.180394584440832": "Allgo Tech. (Beijing) Co.,Ltd",
    "20.22739796623360": "Trimble LEM",
    "12.123917678088192": "Inspike S.R.L.",
    "24.3425903": "HiTEM Engineering",
    "24.49177": "Leap Technology, Inc.",
    "24.49176": "Lanart Corp",
    "24.49175": "NetAlly",
    "24.49174": "Electronic Theatre Controls",
    "24.49173": "New Media Corp",
    "24.49172": "Telematics Calabasas",
    "24.49171": "Netrix",
    "24.49170": "Netspan Corp",
    "24.12061597": "ROPEX Industrie-Elektronik GmbH",
    "20.9814048505856": "Telstra Corporation Limited",
    "24.49179": "Socket Communications",
    "24.49178": "Corometrics Medical Systems",
    "12.71297883828224": "Bascules Robbe nv",
    "20.70682235895808": "Plus One Global Ltd.",
    "24.10768390": "Shenzhen Huachuang Shidai Technologyco.,Ltd",
    "24.11283310": "Intel Corporate",
    "24.8913176": "BLT Co",
    "12.346860523520": "Karl DUNGS GmbH & Co. KG",
    "12.346859290624": "Widmer Time Recorder Co., Inc.",
    "24.15520154": "Zhuhai Liming Industries Co., Ltd",
    "24.32931": "Lantronix",
    "24.32930": "Creative Electronic Systems",
    "24.32933": "Speed International",
    "24.32932": "Liberty Electronics",
    "24.32935": "Honeywell International Inc",
    "24.32934": "Republic Technology, Inc.",
    "24.32937": "Clearpoint Research",
    "20.5297551704064": "National Radio & Telecommunication Corporation - NRTC",
    "24.32939": "Dukane Network Integration",
    "24.32938": "Maxpeed",
    "24.10262310": "Technicolor",
    "24.7653786": "Ericsson AB",
    "24.3717880": "Intel Corporate",
    "24.10505729": "CENTRAL ENGINEERING co.,ltd.",
    "24.8416395": "Kaeser Kompressoren Ag",
    "24.6060280": "Huawei Device Co., Ltd.",
    "24.7905952": "Nintendo Co., Ltd.",
    "24.9701843": "shenzhen maxtopic technology co.,ltd",
    "12.346859896832": "Ar'S Co., Ltd.",
    "12.123917690458112": "midBit Technologies, LLC",
    "20.88668798713856": "Raven Industries Inc.",
    "12.346856448000": "SELCO s.r.l.",
    "24.5893": "Methode Electronics",
    "24.8715729": "Intel Corporate",
    "24.4474195": "Microsoft",
    "24.833393": "ARRIS Group, Inc.",
    "12.123917690363904": "Galios",
    "12.346854957056": "DEUTA Werke GmbH",
    "24.270074": "BISSELL Homecare, Inc.",
    "24.11042066": "Huawei Technologies Co.,Ltd",
    "24.48224": "Cisco Systems, Inc",
    "12.123917686042624": "Tornado Modular Systems",
    "24.6849662": "Fujitsu Limited",
    "24.7096948": "Aerodisk Llc",
    "24.1605305": "Apple, Inc.",
    "12.346857918464": "Sierra Video Systems",
    "24.4769974": "SysTec GmbH",
    "24.8391783": "Apple, Inc.",
    "24.11311882": "Sony Corporation",
    "20.242213039636480": "Leach International Europe",
    "24.11857692": "360 AI Technology Co.Ltd",
    "20.268341256126464": "Dayang Technology Development Inc.",
    "12.346866360320": "OptoLink Industria e Comercio Ltda",
    "12.346865885184": "Imricor Medical Systems, Inc.",
    "24.8947479": "Canon Inc.",
    "24.11276803": "Intel Corporate",
    "24.4245707": "AM Telecom co., Ltd.",
    "12.119270023168": "TechSolutions A/S",
    "20.264165321080832": "Shenzhen Medica Technology Development Co., Ltd.",
    "24.2159006": "ARRIS Group, Inc.",
    "24.11566407": "Cisco Systems, Inc",
    "20.228844755746816": "Urano Industria De Balancas E Equipamentos Ltda",
    "24.13909351": "Carma Industries Inc.",
    "12.346867888128": "Zephyrus Electronics LTD.",
    "20.206931528515584": "Dexcel Design Pvt Ltd",
    "20.211670372712448": "Beijing Cloud Fly Technology Development Co.Ltd",
    "24.12889010": "vivo Mobile Communication Co., Ltd.",
    "24.10026116": "IEEE Registration Authority",
    "12.346857508864": "DBMCORP, Inc.",
    "24.7629370": "Aperi Corporation",
    "12.346856304640": "Hitachi ABB Power Grids – Grid Automation",
    "24.11313950": "Aruba, a Hewlett Packard Enterprise Company",
    "24.12865542": "Samsung Electronics Co.,Ltd",
    "24.14182595": "Samsung Electronics Co.,Ltd",
    "12.123917690597376": "GridNavigator",
    "24.1634776": "Apple, Inc.",
    "20.136646477479936": "Annapurna labs",
    "20.57353112125440": "ARC Technology Co., Ltd",
    "24.8425448": "Intelbras",
    "20.180577505378304": "Beijing Uniwill Science and Technology Co,Ltd",
    "24.11296950": "Ericsson AB",
    "24.12596669": "Velocytech Aps",
    "12.123917682208768": "Mango DSP, Inc.",
    "12.346868064256": "ACD Elektronik GmbH",
    "24.12618462": "Ruckus Wireless",
    "24.8396202": "Avaya Inc",
    "24.270660": "Sunitec Enterprise Co.,Ltd",
    "24.3980760": "Lg Electronics Inc",
    "24.41168": "Ten X Technology, Inc.",
    "24.41169": "Inventec Corporation",
    "24.41164": "Lite-On",
    "24.41165": "DR. JOHANNES HEIDENHAIN GmbH",
    "24.41166": "Ecessa",
    "24.41167": "Sotas, Inc.",
    "24.41160": "Adtran Inc",
    "24.41161": "Intel (PRO100B and PRO100+)",
    "24.41162": "Fujitsu Denso Ltd.",
    "24.41163": "Ark Telecommunications, Inc.",
    "24.16286208": "CaptionCall",
    "12.346860281856": "Powerlinx, Inc.",
    "20.84187707080704": "Openeye",
    "24.6036079": "Cambridge Industries(Group) Co.,Ltd.",
    "12.123917676584960": "Kitron UAB",
    "12.346867412992": "RPA Electronic Solutions, Inc.",
    "24.11040336": "Samsung Electronics Co.,Ltd",
    "12.123917677174784": "Semiconsoft, inc",
    "20.168210314821632": "go-e GmbH",
    "24.12422": "Transistor Devices, Inc.",
    "12.346864353280": "txtr GmbH",
    "20.39986093096960": "Dodge",
    "24.11327063": "Shaanxi GuoLian Digital TV Technology Co.,Ltd.",
    "24.8388619": "Intel Corporate",
    "12.123917690466304": "Automatica Y Regulacion S.A.",
    "24.13654543": "I-Bt Digital Co.,Ltd",
    "12.123917677256704": "Yaviar",
    "24.10268618": "Cisco Systems, Inc",
    "24.13400235": "Texas Instruments",
    "24.13170498": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "24.12646301": "Powercode",
    "24.10548345": "Huawei Technologies Co.,Ltd",
    "12.346867568640": "Pdu Expert Uk Ltd",
    "24.11837100": "Imagik Int'l Corp",
    "24.6845991": "Zhuhai Dingzhi Electronic Technology Co., Ltd",
    "24.10540597": "WBS Technology",
    "24.9977759": "China SSJ (Suzhou) Network Technology Inc.",
    "24.7867079": "Baofeng TV Co., Ltd.",
    "24.5566977": "Samsung Electronics Co.,Ltd",
    "20.48476183330816": "Lingdong Technology (Beijing) Co. Ltd",
    "20.35227544059904": "Virtium",
    "24.45098": "ORSYS GmbH",
    "12.123917677875200": "mg-sensor GmbH",
    "24.24659": "Toyoda Machine Works, Ltd.",
    "24.24658": "PERIPHERALS ENTERPRISE CO., Ltd.",
    "12.346864443392": "Quarch Technology Ltd",
    "24.3457964": "Samsung Electronics Co.,Ltd",
    "24.24657": "Quality Semiconductor",
    "24.12608921": "Samsung Electronics Co.,Ltd",
    "24.2798": "GCD Hard- & Software GmbH",
    "12.346856882176": "SELKOM GmbH",
    "24.14212605": "Texas Instruments",
    "24.1852819": "Texas Instruments",
    "20.14286624653312": "Beijing WayClouds Technology Co., Ltd.",
    "24.1859794": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.277912": "ChongQing QingJia Electronics CO.,LTD",
    "20.110162578767872": "Hunan Voc Acoustics Technology Co., Ltd.",
    "24.7613199": "Infinidat Ltd.",
    "20.31477532196864": "Schneider-Electric(China)Co.Ltd,Shenzhen Branch",
    "24.6056324": "Nuvico",
    "12.346852794368": "NBO Development Center Sekusui Chemical Co. Ltd.",
    "24.7604924": "LightwaveRF Technology Ltd",
    "24.538": "Zuma Networks",
    "24.13404359": "Aruba, a Hewlett Packard Enterprise Company",
    "12.346867810304": "YUYAMA MFG Co.,Ltd",
    "24.12845229": "Advantech Technology (CHINA) Co., Ltd.",
    "12.346855698432": "F.Imm. S.r.L.",
    "24.9209281": "Aruba, a Hewlett Packard Enterprise Company",
    "24.13109049": "NAKAYO Inc",
    "12.123917678690304": "Sensohive Technologies",
    "24.10014547": "Bbk Educational Electronics Corp.,Ltd.",
    "24.9677": "Skylane Optics",
    "24.9676": "Mobile Communications Korea Incorporated",
    "24.3181819": "Dropcam",
    "24.9675": "Reiner SCT",
    "24.9674": "LS Research, LLC",
    "24.6908": "ModusLink Corporation",
    "24.6909": "Evolis",
    "24.2362248": "Taicang T&W Electronics",
    "24.6904": "Copley Controls Corporation",
    "24.6905": "AeroVIronment (AV Inc)",
    "24.6906": "Welch Allyn, Inc.",
    "24.6907": "Joby Inc.",
    "24.6900": "Handreamnet",
    "24.6901": "Pentaone. Co., Ltd.",
    "24.6902": "Woven Systems, Inc.",
    "12.346852720640": "TELINC Corporation",
    "24.14458843": "Ubiquiti Networks Inc.",
    "12.119269703680": "Dynasthetics",
    "24.9671": "altek Corporation",
    "24.12594670": "Cuff",
    "12.71297884372992": "Debug s.r.l.",
    "24.9670": "kasercorp, ltd",
    "24.10494118": "Espressif Inc.",
    "24.5258922": "Tp-Link Technologies Co.,Ltd.",
    "12.346855796736": "Xycom VME",
    "24.8172059": "Cisco SPVTG",
    "20.216602659258368": "GSM Innovations Pty Ltd",
    "24.8400319": "Cisco Systems, Inc",
    "20.9814041165824": "Eleven Engineering Incorporated",
    "24.1574645": "RAD Data Communications, Ltd.",
    "24.13935421": "Apple, Inc.",
    "24.11294843": "JCT Healthcare",
    "12.346852208640": "IO Limited",
    "24.15734312": "Technicolor (China) Technology Co., Ltd.",
    "24.13696204": "Samsung Electronics Co.,Ltd",
    "20.44768528695296": "Annapurna labs",
    "24.11309871": "Nokia Corporation",
    "24.7897487": "Server Racks Australia Pty Ltd",
    "24.15220583": "Huawei Device Co., Ltd.",
    "24.16278036": "Apple, Inc.",
    "20.31368132165632": "philandro Software GmbH",
    "24.4469290": "Avaya Inc",
    "12.346862260224": "ComAp s.r.o",
    "20.176032233553920": "LDA Technologies",
    "24.9729390": "QinetiQ North America",
    "20.79602635505664": "Winn Technology Co.,Ltd",
    "12.346856595456": "ProDesign GmbH",
    "24.5519003": "1Verge Internet Technology (Beijing) Co., Ltd.",
    "12.123917684191232": "Wyebot, Inc.",
    "24.7876907": "zte corporation",
    "24.1352548": "Samsung Electronics Co.,Ltd",
    "12.123917684482048": "Hangzhou SunTown Intelligent Science & Technology Co.,Ltd.",
    "24.11049687": "Nokia",
    "24.12622994": "Coby",
    "12.346852925440": "Pro-Active",
    "24.5765": "Elisa Oyj",
    "24.5764": "Donjin Co.,Ltd.",
    "24.5767": "Chubb CSC-Vendor AP",
    "24.5766": "Karl Storz Imaging",
    "24.5761": "Vector Informatik GmbH",
    "24.5760": "Bally Gaming + Systems",
    "24.5763": "WEBIO International Co.,.Ltd.",
    "24.5762": "Pro Dex, Inc",
    "20.154680190566400": "Larch Networks",
    "24.5769": "Pilkor Electronics Co., Ltd",
    "24.5768": "ServerEngines LLC",
    "24.7656501": "Giga-Byte Technology Co.,Ltd.",
    "24.5530338": "Apple, Inc.",
    "12.123917678874624": "Global Power Products",
    "12.123917684531200": "Techno Mathematical Co.,Ltd",
    "12.123917681008640": "Infrasafe/ Advantor Systems",
    "24.3725733": "Mikotek Information Inc.",
    "24.6849898": "OuterLink Corporation",
    "24.3996566": "Emcraft Systems LLC",
    "24.15232366": "voestalpine SIGNALING Fareham Ltd.",
    "12.71297883398144": "Schweers informationstechnologie GmbH",
    "24.12601574": "Samsung Electronics Co.,Ltd",
    "24.6045784": "Jefferson Audio Video Systems, Inc.",
    "12.346853982208": "DINEC International",
    "24.12583565": "WINSTAR Display CO.,Ltd",
    "12.346864566272": "Global Security Devices",
    "24.5041391": "Soraa, Inc.",
    "24.10003470": "Snap, Inc.",
    "24.1342725": "Sercomm Philippines Inc",
    "12.123917680238592": "WiTagg, Inc",
    "12.123917677346816": "Quasonix, LLC",
    "12.71297884176384": "MB connect line GmbH Fernwartungssysteme",
    "24.15733699": "Shenzhen Fenda Technology Co., Ltd",
    "24.7137543": "Nokia Corporation",
    "12.71297883361280": "Elgama Sistemos",
    "20.211670373761024": "Fuzhou Fdlinker Technology Co.,LTD",
    "20.97843039698944": "Xmodus Systems GmbH",
    "12.346856591360": "Peleton Photonic Systems",
    "12.123917679042560": "Loop Labs, Inc.",
    "20.225209253625856": "Bejing Nexsec Inc.",
    "24.6305501": "Intel Corporate",
    "12.123917690675200": "Kongsberg Intergrated Tactical Systems",
    "12.71297883865088": "Baudisch Electronic GmbH",
    "12.346864529408": "Arteixo Telecom",
    "12.119269527552": "EUROCONTROL S.p.A.",
    "12.123917681856512": "Prisma Telecom Testing Srl",
    "24.2634066": "Huawei Technologies Co.,Ltd",
    "24.4414": "JL Corporation",
    "24.4415": "Alcatel DI",
    "24.4416": "Nanometrics Inc.",
    "24.4417": "GoodMan Corporation",
    "24.4410": "Shinboram",
    "24.4411": "Micronet Communications Inc.",
    "24.4412": "Micronas GmbH",
    "24.4413": "Kn Soltec Co.,Ltd.",
    "24.4418": "e-SMARTCOM INC.",
    "24.4419": "Dell Inc.",
    "12.71297883762688": "Excelitas",
    "24.9764125": "WhereWhen Corp",
    "24.20649": "Moldat Wireless Technolgies",
    "24.6081883": "Intel Corporate",
    "24.13382449": "Texas Instruments",
    "24.13946552": "Enatel LTD",
    "12.123917685006336": "vision systems safety tech",
    "24.61117": "HTC Corporation",
    "12.123917690830848": "HD Vision Systems GmbH",
    "20.224420108959744": "Cassia Networks",
    "24.12354902": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.6558262": "Technicolor CH USA Inc.",
    "20.22126503395328": "Thales Communications & Security SAS",
    "24.11565296": "Beijing HuaqinWorld Technology Co.,Ltd.",
    "24.15755347": "Integrated Corporation",
    "24.3954645": "FiveCo",
    "24.16447936": "RAID Incorporated",
    "12.123917676240896": "GreenFlux",
    "24.14166406": "Best Wish Technology LTD",
    "12.346853584896": "Dune Networks",
    "24.7266": "LG Electronics (Mobile Communications)",
    "24.7267": "Truen",
    "24.7264": "CSP Frontier Technologies,Inc.",
    "24.7265": "Galaxy Microsystems LImited",
    "24.7262": "ASTON France",
    "24.7263": "Winland Electronics, Inc.",
    "24.7260": "Integrated Medical Systems, Inc.",
    "24.7261": "Leica Microsystems",
    "20.228844756795392": "Cliptech Industria e Comercio Ltda",
    "12.123917691297792": "FRS GmbH & Co. KG",
    "24.7268": "Landis+Gyr",
    "24.7269": "JoeScan, Inc.",
    "12.346861031424": "Phytec Messtechnik GmbH",
    "24.12599054": "Netgear",
    "12.346858172416": "Unic Technologies Inc",
    "24.12897131": "Airista Flow, Inc.",
    "12.123917686693888": "INTEC International GmbH",
    "24.11021261": "Huawei Technologies Co.,Ltd",
    "24.20645": "Capitol Business Systems, Ltd.",
    "20.242213051170816": "SES-imagotag Deutschland GmbH",
    "24.3670750": "Sercomm Corporation.",
    "12.123917687042048": "Doosan Digital Innovation America",
    "24.12353444": "Xiaomi Communications Co Ltd",
    "12.123917689638912": "Beijing Huanyu Zhilian Science &Technology Co., Ltd.",
    "24.49379": "Ositech Communications, Inc.",
    "24.49378": "Calcomp, Inc.",
    "24.49371": "Ipc Corporation (Pte) Ltd.",
    "24.49370": "Nice Systems Ltd.",
    "24.49373": "QLogic Corporation",
    "24.49372": "Eos Technologies, Inc.",
    "24.49375": "KYE Systems Corp.",
    "24.7000": "ACE CAD Enterprise Co., Ltd.",
    "24.49377": "Sonic Solutions",
    "24.49376": "Dsc Communication Corp.",
    "24.7190": "ThyssenKrupp Elevator",
    "24.7191": "Nortel Networks",
    "24.7192": "Sicert S.r.L.",
    "24.7193": "secunet Security Networks AG",
    "24.7194": "Thomas Instrumentation, Inc",
    "24.7195": "Hyperstone GmbH",
    "24.7196": "Center Communication Systems GmbH",
    "24.7197": "Chenzhou Gospell Digital Technology Co.,Ltd",
    "24.7198": "emtrion GmbH",
    "24.7199": "Quest Retail Technology Pty Ltd",
    "24.12068080": "SOYO Technology Development Co., Ltd.",
    "24.4728810": "Samsung Electronics Co.,Ltd",
    "24.13252482": "Storbyte, Inc.",
    "24.64698": "Cisco Systems, Inc",
    "24.11855746": "Vivalnk",
    "12.123917678395392": "Videri Inc.",
    "24.2663796": "Miller Electric Mfg. Co.",
    "24.11054397": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.13434428": "Samsung Electronics Co.,Ltd",
    "20.44213346500608": "APPEAK Technology System Co.Ltd.",
    "24.2906787": "Promate Electronic Co.Ltd",
    "24.15525138": "tado GmbH",
    "24.2016": "Palm Inc.",
    "24.2017": "WIS Communications Co. Ltd.",
    "24.2014": "eCopilt AB",
    "24.2015": "Vbrick Systems Inc.",
    "24.2012": "Atek Co, Ltd.",
    "24.2013": "Cradle Technologies",
    "24.2010": "Neuro Telecom Co., Ltd.",
    "24.2011": "Kirana Networks, Inc.",
    "24.2018": "Bitworks, Inc.",
    "20.44169491906560": "Capintec, Inc.",
    "12.346856697856": "Erhardt+Leimer GmbH",
    "12.123917686796288": "Orbit International",
    "12.71297883291648": "BERG Cloud Limited",
    "24.5788608": "Beijing Guang Runtong Technology Development Company co.,Ltd",
    "24.12898711": "Ventia Utility Services",
    "24.12613752": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "24.14968762": "Danish Interpretation Systems A/S",
    "12.123917686931456": "Zao Zeo",
    "24.16288801": "Tp-Link Technologies Co.,Ltd.",
    "12.123917678989312": "BRS Sistemas Eletrônicos",
    "24.9743969": "Alcatel Lucent",
    "24.15254551": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346862534656": "Pyxis Controls WLL",
    "24.3721843": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "20.132869119803392": "Maddalena S.p.A.",
    "24.3150893": "Piciorgros GmbH",
    "24.12077226": "EM Microelectronic",
    "12.123917677375488": "Rohde&Schwarz Topex SA",
    "24.5038453": "Cisco Systems, Inc",
    "24.5038451": "IEEE Registration Authority",
    "12.123917685772288": "Viper Innovations Ltd",
    "12.346865868800": "Tokyo Weld Co.,Ltd.",
    "24.4723288": "Huawei Technologies Co.,Ltd",
    "24.5038454": "Cisco Systems, Inc",
    "20.273272601051136": "Kokkia Inc",
    "12.123917676605440": "Elektronik & Präzisionsbau Saalfeld GmbH",
    "12.123917675630592": "Schwer+Kopka GmbH",
    "24.455": "Cisco Systems, Inc",
    "24.454": "Quarry Technologies",
    "24.457": "Cisco Systems, Inc",
    "24.456": "Thomas Conrad Corp.",
    "24.451": "Acromag, Inc.",
    "24.450": "ARK Research Corp.",
    "24.453": "Simpler Networks",
    "24.452": "NeoWave, Inc.",
    "24.4200736": "Movon Corporation",
    "24.2936103": "Precor Inc",
    "24.458": "Geocast Network Systems, Inc.",
    "24.9798": "Shenyang Tongfang Multimedia Technology Company Limited",
    "24.3433428": "Bionix Supplychain Technologies Slu",
    "24.7089797": "Sagemcom Broadband SAS",
    "24.1608096": "Wuhan Funshion Online Technologies Co.,Ltd",
    "24.9791": "LIOS Technology GmbH",
    "24.9792": "Baustem Broadband Technologies, Ltd.",
    "24.9793": "ARRIS Group, Inc.",
    "24.9794": "ARRIS Group, Inc.",
    "24.9795": "Alpsalpine Co,.Ltd",
    "24.9796": "Thomson Telecom Belgium",
    "24.9797": "Circontrol S.A.",
    "24.9224102": "Samsung Electronics Co.,Ltd",
    "20.48548674535424": "Institut Dr. Foerster GmbH & Co. KG",
    "24.562996": "Hewlett Packard Enterprise",
    "24.14220126": "Askey Computer Corp",
    "24.9242": "Red Beetle Inc.",
    "24.9243": "iWOW Communications Pte Ltd",
    "24.9240": "Nextwave Semiconductor",
    "24.9241": "Private",
    "24.9246": "Nintendo Co., Ltd.",
    "24.9247": "DCT-Delta GmbH",
    "24.9244": "FuGang Electronic (DG) Co.,Ltd",
    "24.9245": "Giga-Byte Technology Co.,Ltd.",
    "24.4762205": "Store Electronic Systems",
    "20.194363544043520": "EM-Tech",
    "24.9248": "NetUP Inc.",
    "24.9249": "Micro-Star Int'L Co., Ltd.",
    "20.106480351051776": "HindlePower, Inc",
    "24.12115079": "IAI Corporation",
    "12.123917678731264": "China Telecom Fufu Information Technology CO.,LTD",
    "24.9086401": "Ranovus USA",
    "24.6574745": "The Chamberlain Group, Inc",
    "24.1099": "Nvidia",
    "24.1098": "iPolicy Networks, Inc.",
    "24.1097": "Mapletree Networks",
    "24.1096": "Polaroid Corporation",
    "24.1095": "Acrowave Systems Co., Ltd.",
    "24.1094": "CYZENTECH Co., Ltd.",
    "24.1093": "LMS Skalar Instruments GmbH",
    "24.1092": "Western Multiplex Corporation",
    "24.1091": "Agilent Technologies, Inc.",
    "24.1090": "Nact",
    "20.224447326846976": "XConnect Professional Services",
    "12.346868621312": "Preston Industries dba PolyScience",
    "20.136646465945600": "Photosynth Inc.",
    "20.368741187584": "Interaxon Inc",
    "12.123917687484416": "Syscom Instruments SA",
    "24.6049813": "Cisco Systems, Inc",
    "24.3934573": "Powerhouse Dynamics",
    "24.13393855": "Topwise 3G Communication Co., Ltd.",
    "24.6094645": "Wistron Corporation",
    "24.12079061": "Microsoft Corporation",
    "12.123917681041408": "Sono-Tek Corporation",
    "24.13917932": "Beijing Xiaomi Electronics Co., Ltd.",
    "12.346863898624": "Shop Safe AG",
    "24.3981612": "AES Corporation",
    "24.55038": "Cisco Systems, Inc",
    "24.3986646": "WirelessWERX, Inc",
    "24.3843064": "IEEE 802.1 Working Group",
    "12.346860679168": "2N TELEKOMUNIKACE a.s.",
    "24.6063690": "Secret Labs LLC",
    "12.346863644672": "Palomar Products, Inc.",
    "20.53091439214592": "Curb, Inc.",
    "24.7094470": "Nimbus Data, Inc.",
    "24.13911528": "Jiangxi Hongpai Technology Co., Ltd.",
    "20.119143428784128": "choyang powertech",
    "20.137141733556224": "Fortem Technologies, Inc.",
    "24.13661927": "Tp-Link Technologies Co.,Ltd.",
    "24.263982": "VTech Electronics Ltd.",
    "12.123917676707840": "Ufatech Ltd",
    "24.6572995": "Shanghai WOASiS Telecommunications Ltd., Co.",
    "20.233258809819136": "Dolby Laboratories, Inc.",
    "12.123917677719552": "Grossenbacher Systeme AG",
    "24.41121": "Epic Data Inc.",
    "24.4228062": "Zebra Technologies Inc",
    "24.8111": "NextIO, Inc.",
    "24.8766": "IRTrans GmbH",
    "24.8113": "Cybertech Inc.",
    "24.8112": "TimeIPS, Inc.",
    "24.8115": "2Wire Inc",
    "24.8114": "Sontheim Industrie Elektronik GmbH",
    "24.8117": "I/O Interconnect Inc.",
    "24.8116": "SmartShare Systems",
    "24.8119": "WiMate Technologies Corp.",
    "24.8118": "Chi Lin Technology Co., Ltd.",
    "24.316816": "Fortinet, Inc.",
    "24.16306348": "Sony Interactive Entertainment Inc.",
    "24.41127": "Vorax Corporation",
    "12.346863161344": "Vitel Net",
    "24.15517055": "Suzhou Pairlink Network Technology",
    "24.35651": "Rftech",
    "24.4878854": "CAIRE, Inc.",
    "12.346855620608": "Chuo Electric Works Co., LTD.",
    "12.346861961216": "Sensoptics Ltd",
    "12.123917676388352": "Xped Corporation Pty Ltd",
    "24.11280446": "Wuhan Tianyu Information Industry Co., Ltd.",
    "20.193649359978496": "Shenzhen Grid Technology Co.,Ltd",
    "24.10806062": "Silicon & Software Systems Ltd.",
    "24.4478385": "Huawei Technologies Co.,Ltd",
    "24.12900743": "Intel Corporate",
    "24.8180490": "Inventec Corporation",
    "24.1059947": "AMPAK Technology, Inc.",
    "12.123917691478016": "Kronotech Srl",
    "12.123917691015168": "Norbit ODM AS",
    "24.12638562": "Askey Computer Corp",
    "24.8929339": "Huawei Technologies Co.,Ltd",
    "12.119269789696": "Signature Control Systems, Inc.",
    "24.14448692": "Logicom Sa",
    "12.123917686018048": "Sienda New Media Technologies GmbH",
    "20.75247165374464": "Annapurna labs",
    "12.123917690015744": "NooliTIC",
    "24.13937965": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.321429": "Huawei Technologies Co.,Ltd",
    "24.7626670": "Tsl Ppl",
    "24.1851428": "Dell Inc.",
    "24.14182361": "Cisco Systems, Inc",
    "24.14948908": "ZPE Systems, Inc.",
    "12.123917680963584": "JENG IoT BV",
    "20.88668801859584": "Shanghai Pujiang Smart Card Systems Co., Ltd.",
    "24.17017": "Sunitec Enterprise Co.,Ltd",
    "12.123917679558656": "Ongisul Co.,Ltd.",
    "24.5517617": "Raster Vision Ltd",
    "24.835349": "Genetec Inc.",
    "24.9182426": "IEEE Registration Authority",
    "24.3674084": "Dedicated Network Partners Oy",
    "24.9490715": "Palomar Medical Technologies",
    "24.1359201": "Apple, Inc.",
    "12.123917689569280": "Mavili Elektronik Tic. Ve San. A.S.",
    "12.346868699136": "Sakura Seiki Co.,Ltd.",
    "24.7091113": "Magicjack LP",
    "20.154680180080640": "Bertin Technologies",
    "24.5296885": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.12060879": "Intel Corporate",
    "24.5551648": "SUHDOL E&C Co.Ltd.",
    "24.8147017": "Apple, Inc.",
    "12.346861613056": "EcoAxis Systems Pvt. Ltd.",
    "24.5291359": "Tp-Link Technologies Co.,Ltd.",
    "24.8419677": "Infinix mobility limited",
    "12.346863513600": "Peek Traffic Corporation",
    "24.1617807": "Apple, Inc.",
    "24.14470056": "Explora Technologies Inc",
    "24.6839915": "PowerRay Co., Ltd.",
    "12.346854854656": "Chauvin Arnoux",
    "24.5829511": "Hccp",
    "20.150399879217152": "Hugo Techno",
    "20.181338951909376": "Brand New Brand Nordic AB",
    "24.11331056": "Doppler Labs",
    "24.5510397": "Orbbec 3D Technology International",
    "24.11797862": "SP Best Corporation Co., LTD.",
    "24.8238": "Daystar Digital",
    "24.7898612": "Huawei Device Co., Ltd.",
    "24.1842101": "Intel Corporate",
    "24.8239": "Zeta Communications, Ltd.",
    "24.15257785": "Taicang T&W Electronics",
    "24.6614359": "ARRIS Group, Inc.",
    "24.4755290": "Routerboard.com",
    "20.145735984087040": "Noralsy",
    "24.7356076": "Apple, Inc.",
    "24.14162131": "Juniper Networks",
    "20.225209257820160": "Ethernity Networks",
    "24.9482783": "Apple, Inc.",
    "24.6320242": "Shenzhen Hongde Smart Link Technology Co., Ltd",
    "12.123917675864064": "Unmanned Spa",
    "24.4220323": "Sagemcom Broadband SAS",
    "24.7122424": "AzureWave Technology Inc.",
    "24.4214158": "Huawei Technologies Co.,Ltd",
    "24.11068740": "Huawei Technologies Co.,Ltd",
    "20.30931574325248": "Reliatronics Inc.",
    "24.16540137": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.31368094416896": "Nichigaku",
    "12.346857496576": "Sowa Electric Co., Ltd.",
    "24.5029898": "vivo Mobile Communication Co., Ltd.",
    "24.13956423": "Google, Inc.",
    "24.8234": "N.V. Dzine",
    "12.346863259648": "INNOVACIONES Microelectrónicas SL (AnaFocus)",
    "12.346868629504": "MC-monitoring SA",
    "24.13380224": "Micro-Biz intelligence solutions Co.,Ltd",
    "24.8235": "ATML (Advanced Telecommunications Modules, Ltd.)",
    "24.7090446": "Cisco Systems, Inc",
    "24.12592431": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.13654500": "Samsung Electronics Co.,Ltd",
    "24.1094136": "Huawei Technologies Co.,Ltd",
    "24.16273704": "SimpliSafe",
    "24.49255": "United Barcode Industries",
    "24.9437834": "Shenzhen Shidean Legrand Electronic Products Co.,Ltd",
    "20.154019625435136": "Private",
    "12.71297884340224": "DEUTA-WERKE GmbH",
    "12.119269748736": "Hitechlab Inc",
    "20.220327875444736": "Freeus LLC",
    "24.9711203": "Finsécur",
    "24.2232": "E.F. Johnson",
    "24.13640210": "IEEE Registration Authority",
    "12.346861178880": "Inico Technologies Ltd.",
    "24.2233": "Kaonmedia CO., LTD.",
    "24.13653908": "Sagemcom Broadband SAS",
    "24.14946030": "Huawei Technologies Co.,Ltd",
    "24.2362818": "TCT mobile ltd",
    "24.2231": "HIT Incorporated",
    "24.2236": "Ilevo AB",
    "24.8182404": "Dongnian Networks Inc.",
    "24.2237": "Tepg-Us",
    "24.4248590": "Biodata Ltd",
    "24.13300315": "Lexar Electronics(Shenzhen) Co.,Ltd.",
    "24.2377067": "Huawei Device Co., Ltd.",
    "24.36931": "Tattile SRL",
    "24.2235": "NetExcell",
    "24.13422723": "Aruba, a Hewlett Packard Enterprise Company",
    "24.1365870": "CONCH ELECTRONIC Co.,Ltd",
    "12.346866352128": "T+A elektroakustik GmbH & Co.KG",
    "20.159304579547136": "DevCom spol. s r.o.",
    "20.128510961123328": "emz-Hanauer GmbH & Co. KGaA",
    "12.123917681971200": "Akenori PTE LTD",
    "24.8388996": "HTC Corporation",
    "24.8413191": "Rim",
    "20.84531188072448": "Ersúles Limited",
    "24.8975496": "cellon communications technology(shenzhen)Co.,Ltd.",
    "24.16004336": "Apple, Inc.",
    "24.6341959": "Apple, Inc.",
    "24.6345437": "Shenzhen Baitong Putian Technology Co.,Ltd.",
    "24.16265956": "ASUSTek COMPUTER INC.",
    "24.808939": "Mueller Systems",
    "24.9979234": "Shanghai Baud Data Communication Co.,Ltd.",
    "20.84197033115648": "4TheWall - 4D Sistem A.S",
    "24.3438711": "O-Net Communications (Shenzhen) Limited",
    "24.1989": "Gcom, Inc.",
    "12.123917682380800": "Aml",
    "24.1988": "JEAN Co. Ltd.",
    "12.346854215680": "Abest Communication Corp.",
    "24.1341248": "Huawei Device Co., Ltd.",
    "12.346866114560": "Keith & Koep GmbH",
    "24.1985": "Overture Networks, Inc.",
    "12.123917691088896": "Aplex Technology Inc.",
    "24.3529": "THALES Elektronik Systeme GmbH",
    "24.3528": "AirMagnet, Inc",
    "24.1984": "NetZerver Inc.",
    "24.3525": "EchoStar Global B.V.",
    "24.3524": "Emcore Corporation",
    "24.3527": "Cosmic Engineering Inc.",
    "24.3526": "DigiRose Technology Co., Ltd.",
    "24.3521": "SafeWeb Inc",
    "24.5406": "ETHERNET Powerlink Standarization Group (EPSG)",
    "24.3523": "First Communication, Inc.",
    "24.3522": "Private",
    "24.1986": "Netsys Telecom",
    "24.11580333": "HUMAX Co., Ltd.",
    "24.1981": "Radionet Ltd.",
    "24.8450654": "Kyynel",
    "24.5401": "StoreAge Networking Technologies",
    "24.2150318": "Cisco Systems, Inc",
    "24.1983": "Armillaire Technologies, Inc.",
    "24.5403": "Isilon Systems Inc.",
    "24.4981280": "Xiaomi Communications Co Ltd",
    "24.13428703": "American Magnetics, Inc.",
    "12.123917684457472": "PHB Eletronica Ltda.",
    "24.11797528": "Smartchip Integrated Inc.",
    "24.2940077": "Wistron Neweb Corporation",
    "24.13379055": "Apple, Inc.",
    "24.5008403": "IoT Company Solucoes Tecnologicas Ltda",
    "24.10501091": "Apple, Inc.",
    "12.346867695616": "Nippon Systemware Co.,Ltd.",
    "24.8690240": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917676617728": "NSP Europe Ltd",
    "20.264165327372288": "Postium Korea Co., Ltd.",
    "24.9460804": "Compal Broadband Networks, Inc.",
    "24.271973": "Apple, Inc.",
    "24.16046587": "Samsung Electronics Co.,Ltd",
    "24.1323005": "Lt Security Inc",
    "24.41086": "Avid Technology, Inc.",
    "24.41085": "Seeq Technology, Inc.",
    "24.41084": "Tonyang Nylon Co., Ltd.",
    "24.41083": "Dawn Computer Incorporation",
    "24.41082": "Advanced Peripherals Technologies, Inc.",
    "24.41081": "Alps Electric (Usa), Inc.",
    "24.41080": "Marconi Communications",
    "24.11051353": "ASRock Incorporation",
    "24.41089": "Alcatel Data Networks",
    "24.41088": "Tattile SRL",
    "20.198611826638848": "ShenZhen Lark Acoustics Co., Ltd.",
    "12.123917684793344": "AeroVision Avionics, Inc.",
    "24.589930": "AT&T",
    "24.11297890": "zte corporation",
    "24.7375308": "China Mobile Group Device Co.,Ltd.",
    "24.3979386": "Private",
    "24.15772500": "Mitsumi Electric Co.,Ltd.",
    "12.346852937728": "Tattile",
    "12.346867838976": "Smart Power Electronics GmbH & Co. KG",
    "24.49259": "Osi Plus Corporation",
    "24.5243353": "Huawei Technologies Co.,Ltd",
    "12.346853756928": "Optical Wireless Link Inc.",
    "24.11155655": "Kaloom inc",
    "24.4993653": "Apple, Inc.",
    "24.6703": "Cisco Systems, Inc",
    "24.10254764": "Suntec Software(Shanghai) Co., Ltd.",
    "12.123917687615488": "Areco",
    "12.123917681479680": "elements",
    "12.123917680685056": "S.E.I. Co.,Ltd.",
    "24.10514988": "HUMAX Co., Ltd.",
    "24.49355": "Control Technology Corporation",
    "24.2137721": "Universal Electronics, Inc.",
    "24.805171": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "24.6009": "QuickTel",
    "12.346866331648": "Blankom",
    "24.6005": "TTE Germany GmbH",
    "24.6004": "Elesta GmbH",
    "24.6007": "Obsidian Research Corporation",
    "24.6006": "Meso Scale Diagnostics, LLC",
    "24.6001": "APD Communications Ltd",
    "24.6000": "Arti Industrial Electronics Ltd.",
    "24.6003": "Laketune Technologies Co. Ltd",
    "12.346863386624": "n3k Informatik GmbH",
    "12.123917685420032": "Cottonwood Creek Technologies, Inc.",
    "24.13419498": "Motorola Mobility LLC, a Lenovo Company",
    "24.8131006": "Apple, Inc.",
    "20.225135227305984": "EDAG Engineering GmbH",
    "24.10001081": "zte corporation",
    "12.123917687894016": "Anker-East",
    "12.346854420480": "Intelligent Photonics Control",
    "24.8133710": "LG Innotek",
    "12.346855350272": "Innings Telecom Inc.",
    "24.4741945": "ASUSTek COMPUTER INC.",
    "12.123917682257920": "Quantaflow",
    "12.123917675397120": "RCH Vietnam Limited Liability Company",
    "24.7639485": "Four systems Co.,Ltd.",
    "24.4481330": "ecobee inc",
    "12.346855358464": "Teamlog",
    "24.3695547": "Hewlett Packard",
    "24.1865785": "Shenzhen Tencent Computer System Co., Ltd.",
    "12.346865545216": "viscount systems inc.",
    "12.346867011584": "Fr. Sauter AG",
    "12.346853797888": "Matsui Mfg Co.,Ltd",
    "24.14690946": "Universal Global Scientific Industrial Co., Ltd.",
    "24.7371152": "Huawei Technologies Co.,Ltd",
    "24.12600547": "Shenzhen Sinkna Electronics Co., LTD",
    "24.8164394": "Huawei Technologies Co.,Ltd",
    "24.2393863": "SEnergy Corporation",
    "24.14474831": "Gionee Communication Equipment Co Ltd",
    "12.346857218048": "Twoway CATV SERVICE INC.",
    "12.123917686808576": "Samwell International Inc",
    "24.3956997": "Espressif Inc.",
    "24.3956996": "Juniper Networks",
    "24.14184781": "Texas Instruments",
    "20.136823633346560": "XD-GE Automation CO.,LTD",
    "24.9179130": "Hutec co.,ltd",
    "24.1259": "Paxonet Communications, Inc.",
    "24.1258": "Hewlett Packard",
    "24.1255": "Lightpointe Communications, Inc",
    "24.1254": "Banyan Network Private Limited",
    "24.1257": "Infiniswitch Corporation",
    "24.1256": "IER, Inc.",
    "24.1251": "Accton Technology Corp",
    "24.1250": "SMC Networks, Inc.",
    "24.1253": "Glonet Systems, Inc.",
    "24.1252": "Daeryung Ind., Inc.",
    "24.16288350": "Texas Instruments",
    "24.14738300": "Huawei Device Co., Ltd.",
    "24.3967344": "Neosfar",
    "24.6051784": "Nokia Corporation",
    "24.8432999": "Kasda Networks Inc",
    "12.123917679308800": "SYS TEC electronic GmbH",
    "24.6970421": "Secrétariat général de la défense et de la sécurité nationale",
    "12.123917675319296": "Sicon srl",
    "24.7886944": "Huawei Technologies Co.,Ltd",
    "24.15230281": "Cisco Systems, Inc",
    "24.7887091": "Vachen Co.,Ltd",
    "20.136646472237056": "BlueSmart Technology Corporation",
    "24.11016024": "Elektronik System i Umeå AB",
    "20.132869116657664": "Salunda Ltd",
    "24.6079286": "ittim",
    "24.6831852": "Ereca",
    "24.16003211": "Xiaomi Communications Co Ltd",
    "12.346852511744": "San Castle Technologies, Inc.",
    "24.53343": "Valcom, Inc.",
    "24.53342": "Stratabeam Technology, Inc.",
    "24.53341": "Intelliworxx, Inc.",
    "24.53340": "KATHREIN TechnoTrend GmbH",
    "24.53347": "Cisco Systems, Inc",
    "24.53346": "Digigram",
    "24.53345": "Tremon Enterprises Co., Ltd.",
    "24.53344": "Panasonic Europe Ltd.",
    "24.53349": "Toko Electric",
    "24.53348": "Multitel",
    "20.150262236839936": "Thomas & Darden, Inc",
    "12.123917689626624": "Aml",
    "20.141268005945344": "Efco",
    "24.15232218": "Supercomputers and Neurocomputers Research Center",
    "12.123917682270208": "Hankook Ctec Co,. Ltd.",
    "24.7662280": "New H3C Technologies Co., Ltd",
    "24.7662283": "New H3C Technologies Co., Ltd",
    "12.123917682401280": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "24.3165891": "Tianjin Techua Technology Co., Ltd.",
    "12.123917682528256": "Kron Medidores",
    "12.123917679988736": "Quercus Technologies, S. L.",
    "12.123917683695616": "MOG Laboratories Pty Ltd",
    "20.198151832076288": "Nanjing yuekong Intelligent Technology",
    "20.220327868104704": "Meyer Electronics Limited",
    "24.11829845": "Research Products Corporation",
    "20.233258802479104": "Eware Information Technology com.,Ltd",
    "12.346865299456": "LECO Corporation",
    "24.15756234": "Samsung Electronics Co.,Ltd",
    "24.1595497": "Sailer Electronic Co., Ltd",
    "12.346855878656": "InterBridge,Inc.",
    "12.346855870464": "Exavio Inc.",
    "12.123917675720704": "KMtronic ltd",
    "24.4335": "Dbtel Incorporated",
    "24.4334": "Cti Products, Inc.",
    "24.4337": "I-O Corporation",
    "24.4336": "RITTAL-WERK RUDOLF LOH GmbH & Co.",
    "24.4331": "Selsius Systems, Inc.",
    "24.4330": "Adept Technology",
    "24.4333": "Sundance Technology, Inc.",
    "24.4332": "Rpcg, Llc",
    "24.4339": "Nexcom International Co., Ltd.",
    "24.4338": "Antec",
    "24.3644": "Transact Technologies Inc",
    "24.3645": "Televic N.V.",
    "24.3646": "Sun Optronics Inc",
    "24.3647": "Soronti, Inc.",
    "24.3640": "Cisco Systems, Inc",
    "24.3641": "Cisco Systems, Inc",
    "24.3642": "Cirrus Logic",
    "12.346866868224": "Ebner Electronic GmbH",
    "24.14970643": "D-Link International",
    "24.2121463": "Samsung Electronics Co.,Ltd",
    "24.3648": "Nortel Networks",
    "24.3649": "Nihon Mechatronics Co.,Ltd.",
    "24.11817598": "Sony Mobile Communications Inc",
    "12.123917685260288": "Applied Systems Engineering, Inc.",
    "12.346864771072": "Analog Way",
    "24.12102767": "Dell Inc.",
    "24.7867576": "Intel Corporate",
    "12.346867851264": "Netline Communication Technologies",
    "12.123917682331648": "Core Akıllı Ev Sistemleri",
    "24.1371620": "Kaiam Corporation",
    "24.9445570": "Aruba, a Hewlett Packard Enterprise Company",
    "24.12857546": "SD Biosensor",
    "24.5534616": "Toyo Electronics Corporation",
    "24.1851624": "Shenzhen Progress&Win Technology Co.,Ltd",
    "12.123917678002176": "Bolide Technology Group, Inc.",
    "24.9171": "AirLink WiFi Networking Corp.",
    "24.12631362": "ZPA Smart Energy a.s.",
    "12.71297883217920": "Sarana Sistem Mikro",
    "12.123917686448128": "BBR Verkehrstechnik GmbH",
    "12.346863349760": "BRS Sistemas Eletrônicos",
    "24.16265218": "Texas Instruments",
    "20.31477544779776": "RDA Microelectronics Technologies (Shanghai) Co. , Ltd",
    "12.346866774016": "Powersense A/S",
    "24.10243246": "Mesa Labs",
    "24.12596677": "Prosoft Systems LTD",
    "12.123917680549888": "Digital Instrument Transformers",
    "24.10008764": "Samsung Electronics Co.,Ltd",
    "24.7094542": "Cisco Systems, Inc",
    "24.10008762": "LG Electronics (Mobile Communications)",
    "24.3466967": "Dell Inc.",
    "20.211776057638912": "Nuctech Company Limited",
    "12.346863222784": "Redwire, LLC",
    "24.10009497": "Phicomm (Sichuan) Co.,Ltd.",
    "24.11563869": "Huawei Device Co., Ltd.",
    "24.16031471": "Taicang T&W Electronics",
    "12.123917690126336": "Fusar Technologies inc",
    "12.123917686321152": "Tattile Srl",
    "24.9422": "Exeltech Inc",
    "24.9423": "Inscape Data Corporation",
    "24.9420": "Fascinations Toys and Gifts, Inc.",
    "24.9421": "Willow Garage, Inc.",
    "24.9426": "Askey Computer Corp",
    "24.9427": "QUALICA Inc.",
    "24.9424": "Shenzhen SOGOOD Industry CO.,LTD.",
    "24.9425": "Thomson Inc.",
    "24.15524180": "Beijing Unisound Information Technology Co.,Ltd.",
    "24.9428": "Freebox Sas",
    "24.9429": "Winward Industrial Limited",
    "20.198151838367744": "Private",
    "24.14992957": "Apple, Inc.",
    "24.5270347": "Mellanox Technologies, Inc.",
    "24.7908420": "Dell Inc.",
    "24.9735105": "Samsung Electronics Co.,Ltd",
    "24.14461135": "New Spin Golf, LLC.",
    "12.123917677305856": "Uconsys",
    "24.2652910": "ARRIS Group, Inc.",
    "24.6059439": "Fitbit, Inc.",
    "12.123917689352192": "Innovative Concepts And Design Llc",
    "24.2649095": "XIAOMI Electronics,CO.,LTD",
    "24.2199": "Quake Technologies",
    "24.2198": "Printronix, Inc.",
    "24.2193": "Lyan Inc.",
    "24.2192": "Avilinks Sa",
    "24.2191": "Advanced Digital Technology",
    "24.2190": "Nihon Computer Co., Ltd.",
    "24.2197": "DIRC Technologie GmbH & Co.KG",
    "24.2196": "InnoVISION Multimedia Ltd.",
    "24.2195": "Le Information Communication Inc.",
    "24.2194": "EM Solutions",
    "24.14166217": "General Electric Consumer and Industrial",
    "12.346864644096": "Genetec Inc.",
    "24.7014": "Sennheiser electronic GmbH & Co. KG",
    "24.7017": "Equaline Corporation",
    "24.7016": "Modnnet Co., Ltd",
    "24.7011": "Apple, Inc.",
    "24.7010": "JHT Optoelectronics Co.,Ltd.",
    "12.346868752384": "Autonomic Controls. Inc",
    "24.7012": "IsaacLandKorea Co., Ltd,",
    "24.12081567": "Mellanox Technologies, Inc.",
    "20.238070222815232": "Krspace",
    "24.7019": "Swyx Solutions AG",
    "24.7018": "Powerwave Technologies Sweden AB",
    "24.13157114": "Huawei Technologies Co.,Ltd",
    "24.1333696": "Netgear",
    "24.1333699": "Creative Chips GmbH",
    "20.273458019696640": "Shenzhen New-Bund Technology Co., Ltd.",
    "24.13677076": "PoeWit Inc",
    "12.123917686960128": "RedWave Labs Ltd",
    "24.16558234": "Sunitec Enterprise Co.,Ltd",
    "24.49508": "Cisco Systems, Inc",
    "24.3178187": "Sfr",
    "24.835776": "Magneti Marelli Sistemas Electronicos Mexico",
    "12.123917679878144": "Hzhy Technology",
    "24.2123697": "Pluto inc.",
    "12.346867576832": "AR RF/Microwave Instrumentation",
    "24.8771": "AzureWave Technology Inc.",
    "24.8770": "Alacron Inc.",
    "24.8773": "Leine & Linde AB",
    "24.8772": "Chengdu Linkon Communications Device Co., Ltd",
    "24.8775": "Dac Engineering Co., Ltd.",
    "24.8774": "Evoc Intelligent Technology Co.,Ltd.",
    "24.8777": "Home Multienergy Sl",
    "24.8776": "Microsoft Corporation",
    "24.8779": "Airtech Technologies, Inc.",
    "24.8778": "Raylase Ag",
    "24.7084661": "Huawei Device Co., Ltd.",
    "24.10284538": "EVUlution AG",
    "24.10009145": "Doro AB",
    "24.1050081": "Nokia",
    "24.13690189": "ARRIS Group, Inc.",
    "24.2140430": "TCT mobile ltd",
    "24.6052": "Hewlett Packard",
    "12.123917683924992": "Simon Tech, S.L.",
    "24.13372903": "B Meters S.R.L.",
    "24.14451446": "iPort",
    "24.9735817": "Industrias Unidas Sa De Cv",
    "24.13670869": "Alcatel-Lucent",
    "12.123917686595584": "Servotronix Motion Control",
    "24.1095670": "Plastoform Industries Ltd.",
    "24.10529617": "Weifang Goertek Electronics Co.,Ltd",
    "12.123917680185344": "LG Electronics",
    "12.123917690687488": "DEUTA-WERKE GmbH",
    "20.88010619092992": "thumbzup UK Limited",
    "24.36467": "Cisco Systems, Inc",
    "24.2224": "HUBER+SUHNER BKtel GmbH",
    "24.6766": "Impro Technologies",
    "24.6767": "MI.TEL s.r.l.",
    "24.2221": "Toyo-Linx Co., Ltd.",
    "24.6761": "Wuhan Yangtze Optical Technology CO.,Ltd.",
    "24.2223": "Novatec Corporation",
    "24.2222": "PacketFront Network Products AB",
    "24.6601192": "Apple, Inc.",
    "20.228784609427456": "Ahnnet",
    "24.1053264": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.6768": "Cisco-Linksys, LLC",
    "24.6769": "Diostech Co., Ltd.",
    "24.1754": "ITRAN Communications Ltd.",
    "24.1755": "ICHIPS Co., Ltd.",
    "24.1756": "Syabas Technology (Amquest)",
    "24.1757": "AT & T Laboratories - Cambridge Ltd",
    "24.289563": "SYSDINE Co., Ltd.",
    "24.1751": "Cisco Systems, Inc",
    "24.1877872": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.1753": "IPM-Net S.p.A.",
    "24.1758": "Flash Technology",
    "24.1759": "AIDONIC Corporation",
    "12.346855383040": "Dese Technologies SL",
    "24.4244620": "N-iTUS CO.,LTD.",
    "24.7607433": "SRT Wireless",
    "24.14735843": "Eltex Enterprise Ltd.",
    "24.172233": "Intel",
    "24.14188150": "Itel Mobile Limited",
    "12.346862817280": "Luminex Corporation",
    "24.3431488": "Cargt Holdings LLC",
    "24.836519": "Meritec",
    "20.194600868249600": "Temco Japan Co., Ltd.",
    "24.6329888": "Private",
    "12.71297884282880": "HKS-Prozesstechnik GmbH",
    "20.31873094909952": "ShenZhen DeLippo Technology Co., LTD",
    "24.13110578": "Holoplot GmbH",
    "24.8182176": "Huawei Technologies Co.,Ltd",
    "12.346853969920": "EmCom Technology Inc.",
    "12.346857345024": "Packet Island Inc.",
    "20.30931581665280": "Private",
    "24.9472162": "Ionics Technology Me Ltda",
    "12.123917686243328": "Horiba Abx Sas",
    "24.5302998": "Apple, Inc.",
    "24.1845923": "Shenzhen HongRui Optical Technology Co., Ltd.",
    "12.123917678039040": "Fläkt Woods AB",
    "24.11284392": "Humannix Co.,Ltd.",
    "20.106196537180160": "VNS Inc.",
    "24.4198773": "IEEE Registration Authority",
    "12.123917682675712": "Gnf",
    "12.123917676847104": "CENSIS, Uiversity of Glasgow",
    "24.9167": "Cummins-Allison Corp.",
    "24.9166": "Kita Denshi Corporation",
    "24.9165": "Tp-Link Technologies Co.,Ltd.",
    "24.9164": "Nintendo Co., Ltd.",
    "24.9163": "Shenzhen Full-join Technology Co.,Ltd",
    "24.9162": "Behind The Set, LLC",
    "24.9161": "Sichuan Tianyi Information Science & Technology Stock CO.,LTD",
    "12.346865532928": "World Time Solutions Limited",
    "12.123917684596736": "Advice",
    "24.6297736": "zte corporation",
    "24.37334": "Crystal Group, Inc.",
    "24.9169": "Trg",
    "24.9168": "Uniloc USA Inc.",
    "24.6838061": "Tuya Smart Inc.",
    "24.7918415": "Pace-O-Matic, Inc.",
    "20.207147364253696": "Global China Technology Limited",
    "12.123917686468608": "Baumtec GmbH",
    "24.7372181": "Shenzhen City LinwlanTechnology Co. Ltd.",
    "12.123917688279040": "Nxcontrol system Co., Ltd.",
    "24.14689840": "Intrigue Technologies, Inc.",
    "24.1848538": "Mellanox Technologies, Inc.",
    "24.14689846": "Nortel Networks",
    "12.346861617152": "R2Sonic, LLC",
    "12.123917691457536": "Tomei Tsushin Kogyo Co,.Ltd",
    "24.5823597": "Exceptional Innovation, Inc.",
    "12.346860982272": "Funkwerk plettac electronic GmbH",
    "24.12599798": "Lg Electronics Inc",
    "24.1617229": "Polostar Technology Corporation",
    "12.123917678632960": "Magenta Labs, Inc.",
    "24.11277701": "silergy corp",
    "24.12338310": "Samsung Electronics Co.,Ltd",
    "24.1335599": "Avaya Inc",
    "24.11322200": "Continental Automotive Czech Republic s.r.o.",
    "24.1101946": "AMPAK Technology, Inc.",
    "20.61822325489664": "Mission Embedded GmbH",
    "12.346859237376": "Optoplan as",
    "24.4499081": "Promax Electronica Sa",
    "24.5542745": "Shenzhen Twowing Technologies Co.,Ltd.",
    "24.4206193": "Texas Instruments",
    "24.12095559": "Apple, Inc.",
    "12.71297883873280": "Thermal Imaging Radar, LLC",
    "20.229631752929280": "mycable GmbH",
    "12.123917683597312": "Potter Electric Signal Co. LLC",
    "12.71297883394048": "Mesotech International, Inc.",
    "20.176772034330624": "Spacepath Communications Ltd",
    "12.346854166528": "Ranch Networks",
    "12.346863403008": "AHM Limited (CLiKAPAD)",
    "20.88383265177600": "ShenZhen ChuangMo Electronics Technology Co., Ltd",
    "12.346852605952": "WaveCom Electronics, Inc.",
    "12.123917677350912": "Circle Consult ApS",
    "24.1050550": "Samsung Electronics Co.,Ltd",
    "24.2387752": "Espressif Inc.",
    "24.6319824": "Seal Ag",
    "24.13956207": "Apple, Inc.",
    "24.7900279": "Lexmark International, Inc.",
    "24.3461275": "MobilMAX Technology Inc.",
    "24.11332202": "Genix Infocomm Co., Ltd.",
    "12.346867830784": "Saia-Burgess Controls AG",
    "24.12328065": "Sichuan iLink Technology Co., Ltd.",
    "24.10224999": "Honor Device Co., Ltd.",
    "24.9207039": "Brocade Communications Systems LLC",
    "24.3977779": "Huawei Device Co., Ltd.",
    "12.346866081792": "Thales Nederland BV",
    "24.9440715": "ARRIS Group, Inc.",
    "24.53483": "Lightera Networks, Inc.",
    "12.123917680926720": "TimeMachines Inc.",
    "20.44768532889600": "Shenzhen PUAS Industrial Co.,LTD",
    "24.1366761": "ASUSTek COMPUTER INC.",
    "12.346864738304": "Ingeteam Paneles S.A.U.",
    "20.167152472555520": "HwaCom Systems Inc.",
    "24.13908526": "Shenzhen Mtc Co Ltd",
    "12.346858536960": "Proserv",
    "24.12063836": "Huawei Device Co., Ltd.",
    "12.123917688147968": "Ard",
    "24.53484": "NAKAYO Inc",
    "24.45190": "LocSoft Limited",
    "24.525122": "Palo Alto Networks",
    "24.15241487": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917691564032": "Tieline Research Pty Ltd",
    "24.45198": "Cisco Systems, Inc",
    "12.346860589056": "Varma Electronics Oy",
    "24.218": "Atex",
    "24.12883362": "Shenzhen Weijiu Industry And Trade Development Co., Ltd",
    "24.9738497": "Shenzhen YITOA Digital Appliance CO.,LTD",
    "20.154055551746048": "K Technology Corporation",
    "12.123917691265024": "Preston Industries dba PolyScience",
    "24.8435334": "Huawei Technologies Co.,Ltd",
    "24.1111681": "vivo Mobile Communication Co., Ltd.",
    "12.71297884160000": "Redwood Systems",
    "24.9750161": "EliteGroup Computer Systems Co., LTD",
    "24.12370490": "Thomson Video Networks",
    "24.16010519": "Goldcard High-Tech Co.,Ltd.",
    "24.212": "Pure Data Ltd.",
    "12.123917690494976": "Esii",
    "12.346866622464": "Tele Data Control",
    "20.14223575875584": "Engineering Center ENERGOSERVICE",
    "24.5253063": "TECHNART Co.,Ltd",
    "24.2365085": "Cisco Systems, Inc",
    "20.97843044941824": "Private",
    "12.346854510592": "Gauging Systems Inc",
    "12.346863005696": "Juice Technologies, LLC",
    "24.4753411": "ManTechnology Inc.",
    "24.1883048": "OJSC \"Ufimskiy Zavod \"Promsvyaz\"",
    "24.13410851": "Huawei Technologies Co.,Ltd",
    "24.7639972": "Zebra Technologies Corp.",
    "12.346859585536": "T.E.Am., S. A.",
    "24.7123630": "Cisco Systems, Inc",
    "12.346860756992": "S1nn GmbH & Co. KG",
    "12.123917687959552": "ACD Elektronik GmbH",
    "20.255553895923712": "Shenzhen Yibaifen Industrial Co.,Ltd.",
    "24.14729862": "Aisai Communication Technology Co., Ltd.",
    "24.8427291": "Altenergy Power System,Inc.",
    "24.4674627": "GTC (Not registered!)",
    "24.14974443": "Shanghai Notion Information Technology CO.,LTD.",
    "24.3427011": "HuNan ZiKun Information Technology CO., Ltd",
    "24.16264968": "Molex CMS",
    "12.123917680377856": "promedias AG",
    "24.12349028": "Sony Mobile Communications Inc",
    "24.11056214": "Cisco Systems, Inc",
    "24.15225245": "Yeonhab Precision Co.,LTD",
    "24.2107580": "Visionscape Co,. Ltd.",
    "24.15740497": "Casa Systems",
    "24.3720424": "NSS Sp. z o.o.",
    "12.123917689896960": "Gamber Johnson-LLC",
    "12.119269601280": "konzeptpark GmbH",
    "12.123917682606080": "Prisma Telecom Testing Srl",
    "12.123917679853568": "Fludia",
    "24.3184099": "Shanghai Simcom Limited",
    "24.5277577": "Cisco Systems, Inc",
    "24.13162141": "Arab International Optronics",
    "12.123917684641792": "Marine Instruments, S.A.",
    "24.16055473": "JJ Corp",
    "12.346852573184": "Private",
    "24.5273263": "Intel Corporate",
    "20.194600869298176": "MartinLogan, Ltd.",
    "24.40478": "Cisco Systems, Inc",
    "24.3711079": "Nokia Solutions and Networks GmbH & Co. KG",
    "12.346864480256": "Phytec Messtechnik GmbH",
    "12.346864787456": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.8421923": "Apple, Inc.",
    "20.250817679458304": "Safety Vision, LLC",
    "12.123917682249728": "Krontech",
    "24.1573083": "Fitbit Inc.",
    "24.13685756": "Mios, Ltd.",
    "24.9197939": "Zyxel Communications Corporation",
    "12.123917678665728": "Elinkgate Jsc",
    "20.31368186691584": "Hangzhou Xiaowen Intelligent Technology Co., Ltd.",
    "24.6068211": "Apple, Inc.",
    "12.346856124416": "Salent Technologies Ltd",
    "24.13618": "Electro-Metrics Corporation",
    "24.11859484": "Apple, Inc.",
    "20.212188447899648": "LIXIL Corporation",
    "24.9198944": "UCI Corporation Co.,Ltd.",
    "20.216145530454016": "Shenzhen Xtooltech Co., Ltd",
    "24.15789342": "Bilkon Bilgisayar Kontrollu Cih. Im.Ltd.",
    "24.9466498": "Sagemcom Broadband SAS",
    "12.71297883701248": "Honeywell",
    "24.3694531": "Yu Jeong System, Co.Ltd",
    "24.1891818": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.45539": "Cisco Systems, Inc",
    "20.88914847072256": "Private",
    "12.346853978112": "Email Metering",
    "24.4133": "Grayhill, Inc",
    "12.346860408832": "LS Elektronik AB",
    "24.4131": "Network Equipment Technologies",
    "24.4130": "SatCom Media Corporation",
    "24.4137": "Cisco Systems",
    "24.4136": "Computer Technica, Inc.",
    "24.4135": "L-3 Communications East",
    "24.53265": "Prism Video, Inc.",
    "24.4139": "Umax Data Systems, Inc.",
    "24.4138": "Zf Microsystems, Inc.",
    "24.5260874": "Huawei Technologies Co.,Ltd",
    "24.3482": "Infotec Ltd",
    "24.3483": "Heraeus Electro-Nite International N.V.",
    "24.3480": "S.W.A.C. Schmitt-Walter Automation Consult GmbH",
    "24.3481": "Orbital Sciences Corp.; Launch Systems Group",
    "24.3486": "TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd.",
    "24.3487": "RF Micro Devices",
    "24.3484": "Elan GmbH & Co KG",
    "24.3485": "Hewlett Packard",
    "24.3488": "Nedap N.V.",
    "24.3489": "MIRAE ITS Co.,LTD.",
    "12.346855227392": "Chengdu Book Digital Co., Ltd",
    "12.123917686317056": "Tecnint HTE SRL",
    "24.3464756": "zte corporation",
    "24.53263": "Speech Design Gmbh",
    "24.47120": "Yichip Microelectronics (Hangzhou) Co.,Ltd",
    "12.346855641088": "Finesystem Co., Ltd",
    "24.4689": "Silink",
    "24.4688": "Tokyo Aircaft Instrument Co., Ltd.",
    "24.4685": "Inducon BV",
    "24.4684": "BBWM Corporation",
    "24.4687": "nVent",
    "24.4686": "Xac Automation Corp.",
    "12.346867941376": "Skychannel Ltd",
    "24.4680": "Dell EMC",
    "24.4683": "Texas Instruments",
    "24.5492": "Horizon Semiconductors Ltd.",
    "12.123917675081728": "Sportsbeams Lighting, Inc.",
    "12.346868076544": "Robert Bosch Healthcare Systems, Inc.",
    "24.11592974": "Nrg Systems Inc",
    "24.5563561": "BHR Tech GmbH",
    "12.123917676097536": "SECUREAN CO.,Ltd",
    "20.176094516871168": "Shanghai Nohmi Secom Fire Protection Equipment Co.,Ltd.",
    "12.71297883566080": "EPSa GmbH",
    "12.123917682802688": "Strega",
    "24.16016806": "Apple, Inc.",
    "12.123917679448064": "Veryx Technologies Private Limited",
    "12.71297884762112": "EMAC, Inc.",
    "24.10288337": "Aetheris Technology (Shanghai) Co., Ltd.",
    "24.6037712": "Asoni Communication Co., Ltd.",
    "24.15758959": "Apple, Inc.",
    "24.10253588": "Wildix srl",
    "24.11027917": "Siselectron Technology LTD.",
    "24.14190798": "RF Technology Pty Ltd",
    "12.346862624768": "Teranex A Division of Silicon Optix",
    "24.2665752": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.119269736448": "head",
    "12.123917678358528": "Orlaco Products B.V.",
    "12.346866651136": "Brankamp GmbH",
    "20.194600864055296": "DNESO TEN Ltd.",
    "12.123917683187712": "Aplex Technology Inc.",
    "24.7364987": "Cisco Systems, Inc",
    "12.346855559168": "GD Mission Systems",
    "24.2136997": "JIAXING GLEAD Electronics Co.,Ltd",
    "24.8439112": "Shenzhen Zowee Technology Co.,Ltd",
    "20.277981941465088": "Univer S.p.A.",
    "20.149935430303744": "ShenZhen Yuyangsheng technology company LTD",
    "24.6035411": "Lge",
    "24.11853077": "ControlThings Oy Ab",
    "20.97400609832960": "Loctek Ergonomic Technology Corp.",
    "24.7646838": "iNovo Broadband, Inc.",
    "12.123917683023872": "Electrolux",
    "24.13930652": "Apple, Inc.",
    "20.132869122949120": "CL International",
    "20.96896777453568": "Annapurna labs",
    "12.346865016832": "Saia-Burgess Controls AG",
    "24.1614824": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.9208799": "Nokia",
    "12.123917682368512": "Smith Meter, Inc.",
    "24.3194427": "Mitsubishi Electric Corporation",
    "12.346860957696": "Advanced Technology Solutions",
    "20.61822324441088": "Pactron",
    "24.8656879": "Technicolor CH USA Inc.",
    "24.12851433": "Samsung Electronics Co.,Ltd",
    "20.62335516409856": "Star Electronics GmbH & CoKG",
    "24.7391895": "Ruckus Wireless",
    "24.7391899": "Cisco Systems, Inc",
    "12.346866659328": "Rohde&Schwarz Topex SA",
    "24.7914981": "Texas Instruments",
    "24.10270292": "Hewlett Packard",
    "24.4747218": "StorSimple Inc",
    "24.1053364": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.6976381": "25G Ethernet Consortium",
    "24.7879907": "Kai-EE",
    "12.346861555712": "tec5 AG",
    "12.123917691473920": "Orange Tree Technologies Ltd",
    "24.648": "Global Village (PCcard in Mac portable)",
    "24.649": "DNE Technologies",
    "24.644": "UK Grid Solutions Limited",
    "24.645": "Riverstone Networks",
    "24.646": "Occam Networks",
    "24.647": "Adapcom",
    "24.640": "Mu Net, Inc.",
    "24.641": "Madge Ltd.",
    "24.642": "ViaClix, Inc.",
    "24.643": "Spectrum Controls, Inc.",
    "20.198611822444544": "Katerra Inc",
    "24.13934085": "zte corporation",
    "12.346862940160": "Primetech Engineering Corp.",
    "24.3181303": "Huawei Device Co., Ltd.",
    "24.7916587": "Sinewy Technology Co., Ltd",
    "24.7569": "Digitize, Inc",
    "24.7568": "EMCO Flow Systems",
    "24.7565": "Fluke Process Instruments GmbH",
    "24.7564": "La Crosse Technology LTD",
    "24.7567": "PureWave Networks",
    "24.7566": "Alereon, Inc.",
    "24.7561": "VaultStor Corporation",
    "24.7560": "Clearwire",
    "24.7563": "ADB Broadband Italia",
    "24.7562": "TechTrex Inc",
    "24.7088528": "WiZ Connected Lighting Company Limited",
    "24.5809294": "PixArt Imaging Inc.",
    "24.3891": "DUALi Inc.",
    "12.123917681430528": "Sertec Srl",
    "24.5531244": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.16588": "Silcom Manufacturing Technology Inc",
    "12.346856005632": "Networked Robotics Corporation",
    "12.346853081088": "EMAC, Inc.",
    "24.7631602": "Chipsip Technology Co., Ltd.",
    "24.1326275": "Huawei Technologies Co.,Ltd",
    "24.7886103": "SankyuElectronics",
    "24.2423": "Brunner Elektronik AG",
    "24.2422": "Datasoft ISDN Systems GmbH",
    "24.2421": "fSONA Communications Corporation",
    "24.2420": "Innopia Technologies, Inc.",
    "24.2427": "Cisco Systems, Inc",
    "24.2426": "Louis Design Labs.",
    "24.2425": "Advanced Television Systems Committee, Inc.",
    "24.2424": "AIJI System Co., Ltd.",
    "24.2429": "SecWell Networks Oy",
    "24.2428": "Cisco Systems, Inc",
    "12.346858983424": "Owasys Advanced Wireless Devices",
    "12.123917676507136": "NimbeLink Corp",
    "24.1192": "Broadmax Technologies, Inc.",
    "24.1193": "SandStream Technologies, Inc.",
    "24.1190": "SAF Tehnika Ltd.",
    "24.1191": "FabiaTech Corporation",
    "24.1196": "IBM",
    "24.1197": "Malibu Networks",
    "24.1194": "Jetstream Communications",
    "24.1195": "Mavenir Inc.",
    "24.14439045": "Apple, Inc.",
    "24.1198": "Sullair Corporation",
    "24.1199": "Digital Fountain, Inc.",
    "12.123917691432960": "Open System Solutions Limited",
    "24.2658": "Crinis Networks, Inc.",
    "24.1091161": "Mobile Create Co.,Ltd.",
    "12.71297883312128": "Smt D.O.O.",
    "12.123917679570944": "Komz - Izmereniya",
    "24.12383877": "D-Link International",
    "24.2659": "DHD GmbH",
    "20.220092032876544": "Merpa Bilgi Islem Ltd.Sti",
    "24.6583542": "Shooter Detection Systems",
    "24.12874526": "Samsung Electronics Co.,Ltd",
    "20.273272607342592": "Fara As",
    "12.346854719488": "S4 Technology Pty Ltd",
    "24.9492459": "Texas Instruments",
    "24.12121683": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "12.123917682438144": "Secure Systems & Services",
    "24.64844": "Nevatec",
    "24.2904751": "Huawei Technologies Co.,Ltd",
    "24.6332": "ZAO NVP Bolid",
    "24.1879505": "Bosung Electronics Co., Ltd.",
    "24.6218": "Catcher, Inc.",
    "24.6219": "Las Vegas Gaming, Inc.",
    "24.6214": "Crypto S.A.",
    "24.6215": "AceNet Technology Inc.",
    "24.6216": "Vecima Networks Inc.",
    "24.6217": "nVent, Schroff GmbH",
    "24.6210": "Nokia Danmark A/S",
    "24.6211": "Dawevision Ltd",
    "24.6212": "Heads Up Technologies, Inc.",
    "24.6213": "Pulsar-Telecom LLC.",
    "24.10527672": "Wiscloud",
    "20.48688652091392": "Phasor Solutions Ltd",
    "24.5726": "Precision I/O",
    "24.8368": "Gateway Devices, Inc.",
    "24.8369": "Comtech Research Inc.",
    "24.8362": "Ericsson Television Limited",
    "24.8363": "Micro Industries Corp.",
    "24.8360": "Sast Technology Corp.",
    "24.8361": "White Horse Industrial",
    "24.8366": "Ornet Data Communication Tech.",
    "24.8367": "3Com",
    "24.8364": "Interflex Datensysteme Gmbh",
    "24.8365": "Linq Systems",
    "24.13679809": "Weifang Goertek Electronics Co.,Ltd",
    "24.14692357": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.12625373": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "12.71297885085696": "InventLab s.c.",
    "12.346858303488": "Secure Systems & Services",
    "12.346859724800": "Nika Ltd",
    "12.71297883803648": "idcell co.ltd",
    "24.1629270": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "24.14161165": "Tp-Link Technologies Co.,Ltd.",
    "24.7636491": "Cisco Systems, Inc",
    "24.14698989": "Audeze LLC",
    "12.346861318144": "smtag international ag",
    "24.9218766": "Xiaomi Communications Co Ltd",
    "24.9495226": "Intel Corporate",
    "20.136646469091328": "RLC Electronics Systems",
    "12.346862931968": "Sofradir-EC",
    "24.7645070": "zte corporation",
    "24.8958312": "Delta Control GmbH",
    "24.850368": "BSkyB Ltd",
    "20.127937507491840": "Smartiply Inc.",
    "12.123917677117440": "Twc",
    "12.123917676429312": "Woods Hole Oceanographic Institution",
    "24.1593386": "Soarnex",
    "20.273458028085248": "SinePulse GmbH",
    "20.163808574177280": "Skyring Smart Technologies(Shenzhen) Co., Ltd.",
    "12.346868273152": "Smartb Technologies",
    "12.123917686161408": "Pullnet Technology,S.L.",
    "24.10537981": "Samsung Electronics Co.,Ltd",
    "24.10529897": "Samsung Electronics Co.,Ltd",
    "24.7361221": "Avaya Inc",
    "24.5539854": "Samsung Electro-Mechanics(Thailand)",
    "12.346865725440": "Lupatecnologia e Sistemas Ltda",
    "12.123917677785088": "Plasmability, LLC",
    "24.4849": "Ifotec",
    "12.123917678190592": "HongSeok Ltd.",
    "12.346856742912": "Halliburton - Sperry Drilling Service",
    "20.128510965317632": "APM Technologies (DongGuan) Ltd",
    "24.4463634": "Technicolor CH USA Inc.",
    "24.10244856": "Huawei Technologies Co.,Ltd",
    "24.10544331": "Zyxel Communications Corporation",
    "12.346854428672": "eko systems inc.",
    "24.5537832": "Hewlett Packard Enterprise",
    "24.10275307": "Netgear",
    "24.4223092": "Life Technology (China) Co., Ltd",
    "12.346865389568": "Radius Sweden AB",
    "12.71297883910144": "T.W.S. srl",
    "20.277958199607296": "Spedos ADS a.s.",
    "12.346856738816": "Digital Engineering, Inc.",
    "24.3954974": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "24.9986082": "EMW Co., Ltd.",
    "12.346863464448": "WaveIP",
    "20.4471482482688": "Alethea Communications Technologies Pvt. Ltd.",
    "20.57176924094464": "Fotonic i Norden AB",
    "12.123917690920960": "Shanghai JCY Technology Company",
    "24.969": "TECOM Co., Ltd.",
    "24.968": "CML Emergency Services",
    "24.2671325": "AzureWave Technology Inc.",
    "24.6880986": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.14739643": "Unicom Vsens Telecommunications Co., Ltd.",
    "12.346863685632": "HomeScenario, Inc.",
    "12.346852892672": "R F Micro Devices",
    "24.4199995": "Samsung Electronics Co.,Ltd",
    "24.961": "Packet Dynamics Ltd",
    "24.9995953": "Flyaudio corporation (China)",
    "24.13381088": "Routerboard.com",
    "24.57391": "Mcns Holdings, L.P.",
    "24.3761": "Newcotech,Ltd",
    "24.3465681": "IEEE Registration Authority",
    "24.57393": "Hagiwara Electric Co., Ltd.",
    "24.965": "Mobotix AG",
    "24.3671468": "Piller Group GmbH",
    "24.57395": "E.E.P.D. GmbH",
    "12.123917688266752": "Axing Ag",
    "24.7657552": "Evrisko Systems",
    "24.967": "hopf Elektronik GmbH",
    "24.16530853": "Arcadyan Corporation",
    "12.123917681774592": "alfamation spa",
    "12.123917686423552": "Nuance Hearing Ltd.",
    "24.966": "ICUE Systems, Inc.",
    "24.3953282": "Hewlett Packard",
    "20.180394585489408": "Pax",
    "24.3938974": "VitalThings AS",
    "24.7137478": "Earda Technologies co Ltd",
    "24.11571412": "Shenzhen Hoin Internet Technology Co., Ltd",
    "12.346858369024": "Garper Telecomunicaciones, S.L.",
    "24.1841226": "ShenZhen RicherLink Technologies Co.,LTD",
    "20.264625145774080": "Smart Power Technology Co., Ltd.",
    "20.4471489822720": "AC Power Distribution / ACT Entmt.",
    "24.5008728": "Juniper Networks",
    "24.6859474": "Datecs Ltd.,",
    "12.346862616576": "Austco Marketing & Service (USA) ltd.",
    "24.13676268": "Alpha Networks Inc.",
    "12.123917682597888": "Trident RFID Pty Ltd",
    "24.10275971": "Juniper Networks",
    "20.368743284736": "Arrow Electronics,Inc.",
    "24.3161352": "Glintt Tes S.A.",
    "12.346854301696": "Ferrari electronic AG",
    "24.10783067": "Ark Infosolutions Pvt Ltd",
    "12.123917677039616": "DIEHL Connectivity Solutions",
    "12.123917689249792": "Thermo Fisher Scientific Inc.",
    "24.16275246": "Core Brands, LLC",
    "20.247076402233344": "Huizhou GISUN Industrial CO. LTD",
    "20.31368188788736": "AllThingsTalk",
    "24.6861662": "Shenzhen Neostra Technology Co.Ltd",
    "24.3200391": "Samsung Electronics Co.,Ltd",
    "12.123917689561088": "ACD Elekronik GmbH",
    "24.4519143": "Cohesity Inc",
    "24.3191351": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "24.12354141": "Mitsumi Electric Co.,Ltd.",
    "24.11593556": "2Wire Inc",
    "24.12628823": "Yoshiki Electronics Industry Ltd.",
    "24.9177588": "ARRIS Group, Inc.",
    "12.123917687566336": "Sensor 42",
    "24.7618750": "LG Innotek",
    "24.2889696": "SYSTEMES ET TECHNOLOGIES IDENTIFICATION (STid)",
    "12.123917678899200": "Paul Scherrer Institut (PSI)",
    "24.4245034": "Shanghai Jing Ren Electronic Technology Co., Ltd.",
    "24.8661803": "Dell Inc.",
    "24.6029930": "Applied Vision Corporation",
    "12.346864844800": "Linuxstamp Designs, LLC",
    "20.110162587156480": "ATG UV Technology",
    "20.111031373266944": "hiQview Corporation",
    "24.6029938": "Silicon Laboratories",
    "24.9218415": "D&M Holdings Inc.",
    "24.7604": "Kumho Eng Co.,Ltd",
    "24.7605": "Juniper Networks",
    "24.7606": "BestComm Networks, Inc.",
    "24.7607": "Tendril Networks, Inc.",
    "24.7600": "FuJian HengTong Information Technology Co.,Ltd",
    "24.7601": "Crescendo Networks",
    "24.7602": "Hokkaido Electric Engineering Co.,Ltd.",
    "24.7603": "HPN Supply Chain",
    "24.7608": "Intoto Inc.",
    "24.7609": "Wellspring Wireless",
    "12.346855374848": "Cybertron Co., Ltd.",
    "24.62467": "Orbis Systems Oy",
    "24.5028515": "Bison Electronics Inc.",
    "24.5263785": "Texas Instruments",
    "24.6690": "eQ-3 Entwicklung GmbH",
    "24.7903580": "Nationz Technologies Inc",
    "24.5265320": "Cisco Systems, Inc",
    "12.346861117440": "Continental Electronics Corp.",
    "24.6692": "Galaxy Telecom Technologies Ltd",
    "24.8710936": "Texas Instruments",
    "24.4753895": "Intel Corporate",
    "24.6877477": "Data Controls Inc.",
    "12.346854748160": "T/R Systems, Inc.",
    "12.123917681229824": "managee GmbH & Co KG",
    "24.3706490": "Autoit",
    "24.6090124": "Hewlett Packard Enterprise",
    "24.13920333": "Ruckus Wireless",
    "24.4248494": "Samsung Electronics Co.,Ltd",
    "24.15500112": "Microsoft Corporation",
    "20.31346171838464": "Jump Trading",
    "20.44169478275072": "Astronics AES",
    "24.326105": "Speaker Electronic(Jiashan) Co.,Ltd",
    "24.9997945": "Qudelix, Inc.",
    "24.3435475": "Huawei Technologies Co.,Ltd",
    "24.264928": "Xmit Ag Computer Networks",
    "24.16541150": "Amazon Technologies Inc.",
    "24.1367452": "vivo Mobile Communication Co., Ltd.",
    "24.834897": "TCT mobile ltd",
    "24.13651137": "SINTRONES Technology Corp.",
    "12.346866049024": "Synatec Electronic GmbH",
    "20.216145526259712": "Marble Automation",
    "12.346868781056": "Figment Design Laboratories",
    "24.1098619": "ASUSTek COMPUTER INC.",
    "24.10235763": "Universal Tiancheng Technology (Beijing) Co., Ltd.",
    "24.4471964": "Pintsch Tiefenbach GmbH",
    "24.2641298": "Luminator",
    "20.150262237888512": "Volterman Inc.",
    "24.4486252": "Sony Mobile Communications Inc",
    "24.2675690": "Huawei Device Co., Ltd.",
    "20.57176932483072": "Shenzhen Eternal Idea Tech Co.,Ltd",
    "24.11796802": "GCI Science & Technology Co.,LTD",
    "12.123917677797376": "Flirtey Inc",
    "24.15254057": "H-Displays (MSC) Bhd",
    "12.346861899776": "taskit GmbH",
    "24.5038833": "sclak srl",
    "24.14177338": "Texas Instruments",
    "12.123917686607872": "Cab S.R.L.",
    "24.2382517": "Samsung Electronics Co.,Ltd",
    "12.123917686992896": "Integra Metering SAS",
    "12.123917676195840": "Amfitech ApS",
    "24.13930410": "Sogecam Industrial, S.A.",
    "20.273272598953984": "Shenzhen Shengyuan Tech Ltd.",
    "24.272216": "Huawei Technologies Co.,Ltd",
    "12.123917690228736": "Explorer Inc.",
    "24.14453815": "Cisco Systems, Inc",
    "24.6346723": "IEEE Registration Authority",
    "24.15244980": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.3422041": "zte corporation",
    "12.346854481920": "JLCooper Electronics",
    "12.346867593216": "Saab AB",
    "20.75247158034432": "BAYKON Endüstriyel Kontrol Sistemleri San. ve Tic. A.Ş.",
    "24.11299756": "Jenny Science AG",
    "24.14428970": "Apple, Inc.",
    "24.15237166": "Apple, Inc.",
    "24.3712316": "COMECER Netherlands",
    "24.46544": "Samsung Electronics Co.,Ltd",
    "24.2100424": "Netgear",
    "20.154039760191488": "Greenfield Technology",
    "24.9713655": "Huawei Device Co., Ltd.",
    "12.123917676257280": "Hagenuk KMT Kabelmesstechnik GmbH",
    "24.11291239": "Cisco Systems, Inc",
    "24.1314127": "Flextronics International",
    "12.123917676957696": "Rf Creations Ltd",
    "12.123917677105152": "Leontech Limited",
    "20.228844763086848": "Shenzhen Zhongxi Security Co.,Ltd",
    "24.14453123": "Samsung Electronics Co.,Ltd",
    "12.119269470208": "Promixis, LLC",
    "24.8144764": "IEEE Registration Authority",
    "24.16538141": "Hitron Technologies. Inc",
    "12.71297884106752": "Medigus Ltd",
    "24.7138226": "Dongguan Sen DongLv Electronics Co.,Ltd",
    "24.2108599": "Kong Yue Electronics & Information Industry (Xinhui) Ltd.",
    "24.56586": "Ungermann-Bass Inc.",
    "24.10023043": "Huawei Technologies Co.,Ltd",
    "24.285": "Centillium Communications",
    "24.286": "Precidia Technologies, Inc.",
    "12.123917686214656": "ISRV Zrt.",
    "24.1589789": "Private",
    "24.56585": "Ungermann-Bass Inc.",
    "24.1077055": "Buffalo.Inc",
    "24.280": "EZ Digital Co., Ltd.",
    "12.123917678288896": "PuS GmbH und Co. KG",
    "12.123917687705600": "Kranze Technology Solutions",
    "20.154019634872320": "Bluemega Document & Print Services",
    "24.281": "RTUnet (Australia)",
    "24.282": "Hoffmann und Burmeister GbR",
    "24.14206058": "Murata Manufacturing Co., Ltd.",
    "24.56581": "Ungermann-Bass Inc.",
    "24.426": "Airspan Communications, Ltd.",
    "24.2921004": "Kishu Giken Kogyou Company Ltd,.",
    "12.346868514816": "Infotech North America",
    "24.11052514": "MSF-Vathauer Antriebstechnik GmbH & Co KG",
    "24.15789958": "Google, Inc.",
    "24.4851670": "IEEE 802.1 Working Group",
    "24.16031572": "Samsung Electronics Co.,Ltd",
    "12.346861658112": "Phytec Messtechnik GmbH",
    "24.14204748": "Victory Concept Electronics Limited",
    "24.8184319": "Computer Performance, Inc. DBA Digital Loggers, Inc.",
    "24.8173900": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "12.71297884823552": "GD Mission Systems",
    "24.10516666": "Pantech Co., Ltd.",
    "24.10532169": "Arcadyan Corporation",
    "24.14954486": "Unicore communication Inc.",
    "24.1315654": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "20.168210323210240": "Fath Mechatronics",
    "12.123917679927296": "Certus Operations Ltd",
    "24.15749430": "TCT mobile ltd",
    "24.9440947": "Shenzhen Amediatech Technology Co., Ltd.",
    "24.11584970": "IEEE Registration Authority",
    "24.429": "Coach Master International d.b.a. CMI Worldwide, Inc.",
    "20.233258816110592": "Coperion",
    "12.346865590272": "Mac-Gray Corporation",
    "24.1516": "Mosaic Systems Inc.",
    "24.6347541": "New H3C Technologies Co., Ltd",
    "24.7882180": "Dell Inc.",
    "24.1839585": "Wartsila Finland Oy",
    "12.346865680384": "Tektronik",
    "24.9437403": "Samsung Electronics Co.,Ltd",
    "24.11860552": "Apple, Inc.",
    "24.10762444": "Nintendo Co.,Ltd",
    "12.123917687447552": "CAST Group of Companies Inc.",
    "24.12584386": "ARRIS Group, Inc.",
    "12.346868097024": "Net4Things",
    "12.123917682532352": "Maharsystem",
    "12.123917679472640": "Slot3 GmbH",
    "24.15743333": "Samsung Electronics Co.,Ltd",
    "12.123917691375616": "Teledyne API",
    "24.15767608": "Huawei Technologies Co.,Ltd",
    "12.123917675106304": "Transit Solutions, LLC.",
    "24.11808409": "Giga-Byte Technology Co.,Ltd.",
    "24.13947268": "ADB Broadband Italia",
    "24.8946525": "Fn-Link Technology Limited",
    "12.346862268416": "Computerwise, Inc.",
    "24.13155049": "Qdis",
    "12.346852229120": "Moisture Systems",
    "24.13648111": "Murata Manufacturing Co., Ltd.",
    "24.24656": "Internix Inc.",
    "20.66219392630784": "RO.VE.R. Laboratories S.p.A",
    "24.12443": "Smartware",
    "24.12442": "Astro Terra Corp.",
    "24.12441": "Boenig Und Kallenbach Ohg",
    "24.12440": "Global Converging Technologies",
    "24.12447": "Amber Networks",
    "24.12446": "Workbit Corporation.",
    "24.12445": "Nimble Microsystems, Inc.",
    "24.12444": "Timing Applications, Inc.",
    "24.12449": "WEBGATE Inc.",
    "24.12448": "Tyco Submarine Systems, Ltd.",
    "24.5814625": "SOLEM Electronique",
    "24.3454321": "Plds",
    "12.346865020928": "Cammegh Limited",
    "24.6339660": "EWM Hightec Welding GmbH",
    "24.16297947": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16541758": "Directed Perception, Inc",
    "12.71297884725248": "Argus-Spectrum",
    "24.578110": "Intel Corporate",
    "24.14690663": "HMD Global Oy",
    "20.154238422351872": "Oxford Nanopore Technologies Ltd.",
    "24.2915969": "ARRIS Group, Inc.",
    "24.7374925": "Japan Radio Co., Ltd.",
    "12.123917690273792": "Getein Biotechnology Co.,ltd",
    "24.4521379": "Everysight LTD.",
    "12.123917676802048": "ULSee Inc",
    "24.6592872": "Elentec",
    "24.16294039": "ARRIS Group, Inc.",
    "20.132855981146112": "shanghai baudcom communication device co.,ltd",
    "24.5453": "Netronome Systems, Inc.",
    "24.5452": "Saunders Electronics",
    "24.5451": "Wonde Proud Technology Co., Ltd",
    "24.5450": "Wanshih Electronic Co., Ltd",
    "24.5457": "RadioPulse Inc.",
    "24.5456": "Nits Technology Inc",
    "24.5455": "one RF Technology",
    "24.5454": "Iec",
    "24.285804": "Juniper Networks",
    "24.5459": "Cytyc Corporation",
    "24.5458": "Wi-Gear Inc.",
    "24.3413087": "Samsung Electronics Co.,Ltd",
    "24.14984071": "Control Solutions LLC",
    "24.8657783": "Intel Corporate",
    "24.9715913": "Hewlett Packard Enterprise",
    "24.4981385": "Lex Computech Co., Ltd",
    "20.207147374739456": "Sidus Novum Sp. z o. o.",
    "24.8429995": "Intellisis",
    "24.12592013": "Samsung Electronics Co.,Ltd",
    "24.3859": "Nisca corporation",
    "24.3858": "Panasonic Europe Ltd.",
    "24.3855": "Real ID Technology Co., Ltd.",
    "24.3854": "WaveSplitter Technologies, Inc.",
    "24.3857": "Prodrive B.V.",
    "24.3856": "RDM Corporation",
    "24.3851": "Kentima Technologies AB",
    "24.3850": "Clear Edge Networks",
    "24.3853": "Hunt Electronic Co., Ltd.",
    "24.3852": "Synchronic Engineering",
    "24.14470516": "Japan E.M.Solutions Co., Ltd.",
    "24.6310826": "Magic Leap, Inc.",
    "24.4498": "Cisco Systems, Inc",
    "24.4499": "Cisco Systems, Inc",
    "12.123917679325184": "Orion S.r.l.",
    "24.4494": "Halytech Mace",
    "24.4495": "Eutech Instruments Pte. Ltd.",
    "24.4496": "Digital Design Corporation",
    "24.4497": "CTS-Clima Temperatur Systeme GmbH",
    "24.4490": "Viewtran Technology Limited",
    "24.4491": "Alcatel-Lucent Enterprise",
    "24.4492": "Missouri Department of Transportation",
    "24.4493": "Hanchang System Corp.",
    "24.57358": "Avalon Imaging Systems, Inc.",
    "24.57359": "Shanghai Baud Data Communication Co.,Ltd.",
    "24.57354": "Diba, Inc.",
    "24.57355": "Rooftop Communications Corp.",
    "24.57356": "Motorola",
    "24.57357": "Radiant Systems",
    "24.57350": "Silicon Integrated Sys. Corp.",
    "24.57351": "Avaya ECS Ltd",
    "24.57352": "Amazing Controls! Inc.",
    "24.57353": "Stratus Technologies",
    "24.11554755": "Shenzhen NVC Cloud Technology Co., Ltd.",
    "24.6851312": "zte corporation",
    "12.123917675569152": "MB connect line GmbH Fernwartungssysteme",
    "24.9984944": "Kmdata Inc.",
    "24.16268158": "Huawei Device Co., Ltd.",
    "24.13161471": "Hewlett Packard",
    "12.346864447488": "GD Mission Systems",
    "24.7631390": "Edison Labs Inc.",
    "12.123917675065344": "Neusoft Reach Automotive Technology (Shenyang) Co.,Ltd",
    "24.7609266": "Apple, Inc.",
    "24.5563581": "Fn-Link Technology Limited",
    "24.4227869": "Apple, Inc.",
    "24.15222815": "Advanced Automotive Antennas",
    "24.6080208": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.4776425": "Chengdu Meross Technology Co., Ltd.",
    "24.8670135": "Beijing Sankuai Online Technology Co.,Ltd",
    "24.1094511": "Bowei Technology Company Limited",
    "24.10004802": "Guangzhou Tozed Kangwei Intelligent Technology Co., LTD",
    "24.14435367": "IEEE Registration Authority",
    "24.5245841": "IEEE Registration Authority",
    "24.7118460": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3703255": "Texas Instruments",
    "12.346866839552": "Tantal Electronica, Sl",
    "12.123917675913216": "Grossenbacher Systeme AG",
    "12.71297883193344": "CR Magnetics, Inc.",
    "24.14686548": "zte corporation",
    "24.4739187": "Microsoft Corporation",
    "24.8948": "AMPAK Technology, Inc.",
    "24.8949": "Advanced Realtime Tracking GmbH",
    "12.346860662784": "Brain Tech Co., Ltd",
    "24.8942": "Algo Communication Products Ltd",
    "24.8943": "iWDL Technologies",
    "24.8940": "Idealbt Technology Corporation",
    "24.8941": "TSI Power Corporation",
    "24.8946": "SunPower Corp",
    "24.8947": "SHARP Corporation",
    "24.8944": "3 Greens Aviation Limited",
    "24.8945": "Private",
    "24.7084431": "D-Link International",
    "24.3149": "Curtiss-Wright Controls Avionics & Electronics",
    "24.3148": "Arcor AG&Co.",
    "24.3143": "SK Teletech(R&D Planning Team)",
    "24.3142": "Allied Telesyn Inc.",
    "24.3141": "Animation Technologies Inc.",
    "24.3140": "Automated Interfaces, Inc.",
    "24.3147": "Cheops Elektronik",
    "24.3146": "Cygnus Microsystems (P) Limited",
    "24.3145": "Dangaard Telecom Denmark A/S",
    "24.3144": "QoStek Corporation",
    "12.346861744128": "Amicus Wireless",
    "24.13370651": "Samsung Electronics Co.,Ltd",
    "24.8949769": "Juniper Networks",
    "24.10496952": "Hewlett Packard",
    "12.346863411200": "Saia-Burgess Controls AG",
    "24.13408401": "Cisco Systems, Inc",
    "24.10243616": "Cisco Systems, Inc",
    "24.4733752": "Apple, Inc.",
    "24.12070184": "AMPAK Technology,Inc.",
    "24.412": "JDS Uniphase Inc.",
    "12.346853199872": "Inducomp Corporation",
    "24.5534173": "Huawei Device Co., Ltd.",
    "20.149945497681920": "Opto Engineering",
    "12.346857328640": "Lab X Technologies, LLC",
    "20.168198566576128": "hangzhou soar security technologies limited liability company",
    "24.6539": "Novera Optics Korea, Inc.",
    "24.6538": "Northrop Grumman Systems Corp.",
    "24.6535": "Panasonic Mobile Communications Co.,Ltd.",
    "24.6534": "Cheng Hongjian",
    "24.6537": "Sonitrol Corporation",
    "24.6536": "Wi2Wi, Inc",
    "24.6531": "CCT R&D Limited",
    "24.6530": "SmarDTV",
    "24.6533": "IT Watchdogs, Inc",
    "24.6532": "ESTIC Corporation",
    "24.12845062": "Lipi Data Systems Ltd.",
    "24.8694078": "Hewlett Packard",
    "12.346866716672": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "12.346864496640": "Phytec Messtechnik GmbH",
    "12.346863874048": "CosmoData Informatica Ltda.",
    "24.9764705": "China Mobile Group Device Co.,Ltd.",
    "12.346861125632": "Z-App Systems",
    "12.71297884418048": "Private",
    "24.16047075": "Apple, Inc.",
    "24.12644804": "Pacidal Corporation Ltd.",
    "24.16047078": "Cisco Systems, Inc",
    "20.123731223511040": "RHXTune Technology Co.,Ltd",
    "24.15769596": "SHARP Corporation",
    "24.15476426": "Tp-Link Technologies Co.,Ltd.",
    "20.31657729982464": "Gahdeung Elecom",
    "24.5244088": "Huawei Technologies Co.,Ltd",
    "12.123917691035648": "Lacs Srl",
    "24.6876101": "Angstrem Telecom",
    "24.16283044": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.6391": "Kameleon Technologies",
    "24.6390": "Thomson Telecom Belgium",
    "24.6393": "VVOND, Inc.",
    "24.6392": "Cisco-Linksys, LLC",
    "24.6395": "Compro Technology",
    "24.6394": "Yushin Precision Equipment Co.,Ltd.",
    "24.6397": "Optimal Technologies International Inc.",
    "24.6396": "Altec Electronic AG",
    "24.6399": "PowerQuattro Co.",
    "24.6398": "Hewlett Packard",
    "20.176094526308352": "FlexLink AB",
    "24.9710": "Avtex Ltd",
    "24.9711": "I-TEC Co., Ltd.",
    "24.9712": "Suga Electronics Limited",
    "24.9713": "ARRIS Group, Inc.",
    "24.9714": "ARRIS Group, Inc.",
    "24.9715": "Nordwestdeutsche Zählerrevision",
    "24.9716": "KoCo Connector AG",
    "24.9717": "DVS Korea, Co., Ltd",
    "24.9718": "netTALK.com, Inc.",
    "24.9719": "Ansaldo STS USA",
    "24.6353289": "Murata Manufacturing Co., Ltd.",
    "24.15226582": "NCTech Ltd",
    "24.11034806": "Apple, Inc.",
    "24.10538696": "Ce Link Limited",
    "24.1549": "Wave7 Optics",
    "24.1548": "Melco Industries, Inc.",
    "24.5299452": "Ecocom",
    "24.1545": "Crossport Systems",
    "24.1544": "At-Sky SAS",
    "24.1547": "Artesyn Embedded Technologies",
    "24.1546": "Blue2space",
    "24.1541": "Inncom International, Inc.",
    "24.1540": "@Track Communications, Inc.",
    "24.1543": "Omni Directional Control Technology Inc.",
    "24.1542": "RapidWAN, Inc.",
    "24.16442650": "RSAE Labs Inc",
    "24.16534": "Cisco Systems, Inc",
    "12.346855419904": "Datacon",
    "24.11851197": "Intel Corporate",
    "24.16532": "Shographics, Inc.",
    "24.8407888": "Silicon Laboratories",
    "24.6845148": "CETORY.TV Company Limited",
    "24.8403700": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.810805": "Niagara Video Corporation",
    "24.12605607": "Pico Systems Co., Ltd.",
    "24.7371064": "Wuxi Zhanrui Electronic Technology Co.,LTD",
    "24.15241504": "Motorola Mobility LLC, a Lenovo Company",
    "24.4494588": "Netgear",
    "12.123917682515968": "MRC Systems GmbH",
    "12.123917677965312": "Hensoldt Sensors GmbH",
    "24.5555006": "Ciena Corporation",
    "24.8430985": "AzureWave Technology Inc.",
    "24.8695035": "Crouzet Automatismes",
    "24.14444779": "Nintendo Co.,Ltd",
    "24.3460608": "XIAOMI Electronics,CO.,LTD",
    "12.346863730688": "Goerlitz AG",
    "20.159304568012800": "Shenzhen Hisource Technology Development CO.,Ltd.",
    "24.13410101": "Transbit Sp. z o.o.",
    "12.346866335744": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "20.171812093689856": "HK ELEPHONE Communication Tech Co.,Limited",
    "24.8665872": "Lv switch Inc.",
    "24.7637131": "ADB Broadband Italia",
    "24.10281844": "Huawei Technologies Co.,Ltd",
    "12.123917677506560": "Axing Ag",
    "24.14707093": "Pegatron Corporation",
    "12.346867093504": "Baudisch Electronic GmbH",
    "12.123917685542912": "Globalcom Engineering SPA",
    "24.9179081": "Huawei Device Co., Ltd.",
    "24.14476595": "IEEE Registration Authority",
    "24.7625669": "IEEE Registration Authority",
    "24.4502402": "Kuang-chi Institute of Advanced Technology",
    "24.8449": "Aplicaciones Electronicas Quasar (AEQ)",
    "24.8448": "Gemtek Technology Co., Ltd.",
    "24.8447": "Symmetrical Technologies",
    "24.8446": "Topware Inc. / Grand Computer",
    "24.8445": "Itv Technologies, Inc.",
    "24.8444": "Matrox",
    "24.8443": "Octel Communications Corp.",
    "24.8442": "Gde Systems, Inc.",
    "24.8441": "Paralink Networks, Inc.",
    "24.8440": "Carrera Computers, Inc.",
    "24.533426": "vivo Mobile Communication Co., Ltd.",
    "24.16007510": "Arrikto Inc.",
    "12.346857893888": "Advanced Software Technologies",
    "20.203238154436608": "Glamo Inc.",
    "24.14439264": "Intel Corporate",
    "24.8146085": "BSkyB Ltd",
    "12.123917676253184": "Inova Design Solutions Ltd",
    "12.346854768640": "Danlaw Inc",
    "24.7651233": "Hyunteck",
    "20.127937498054656": "Siglent Technologies Co., Ltd.",
    "24.12877126": "Fujitsu Limited",
    "12.123917682343936": "University Of Groningen",
    "12.346863579136": "Psd",
    "12.123917691686912": "Camsat Przemysław Gralak",
    "12.346853670912": "Taishodo Seiko Co., Ltd.",
    "20.150399881314304": "Divelbiss Corporation",
    "24.4825": "Cisco Systems, Inc",
    "24.4824": "International Games System Co., Ltd.",
    "24.4827": "ZIEHL industrie-elektronik GmbH + Co KG",
    "24.4826": "Cisco Systems, Inc",
    "24.4821": "Motion Reality Inc.",
    "24.4820": "Princeton Technology, Ltd",
    "24.4823": "Invento Networks, Inc.",
    "24.4822": "Jiangsu Yitong High-Tech Co.,Ltd",
    "24.8191": "Respironics, Inc.",
    "24.8190": "HPN Supply Chain",
    "24.8193": "Dsp Solutions, Inc.",
    "24.8192": "Lexmark (Print Server)",
    "24.4829": "Shengqu Information Technology (Shanghai) Co., Ltd.",
    "24.4828": "SunCorp Industrial Limited",
    "24.8197": "Simple Technology",
    "24.8196": "Yamatake-Honeywell Co., Ltd.",
    "24.11577130": "Ensemble Designs, Inc.",
    "12.346866573312": "WITHSYSTEM Co.,Ltd",
    "12.123917686415360": "Wartsila Voyage Limited",
    "12.71297883705344": "Sicon srl",
    "24.13371533": "Apple, Inc.",
    "24.7636266": "Pegatron Corporation",
    "12.346863702016": "HSR Harald L. Reuter",
    "24.813756": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346862604288": "Commex Technologies",
    "24.7344558": "Music Life LTD",
    "24.6567196": "e-CON SYSTEMS INDIA PVT LTD",
    "12.123917677678592": "Technica Engineering GmbH",
    "24.8191036": "Visteon Corporation",
    "12.346866139136": "ABtrack s.r.l.",
    "20.198237807968256": "Shenzhen Cudy Technology Co., Ltd.",
    "12.346866147328": "SELEX Communications Limited",
    "12.123917688774656": "DEK Technologies",
    "24.10784773": "China Greatwall Computer Shenzhen Co.,Ltd",
    "24.316336": "Intel Corporate",
    "24.8684342": "Newrun",
    "20.216602664501248": "viRaTec GmbH",
    "12.346861625344": "Gidel",
    "12.123917680742400": "Chelsea Technologies Group Ltd",
    "24.16022220": "SolidFire, Inc.",
    "24.3469238": "Edgecore Networks Corporation",
    "24.13910256": "Huawei Technologies Co.,Ltd",
    "12.346866429952": "Intuitive Surgical, Inc",
    "24.4734258": "Syscor Controls &amp; Automation",
    "24.12336974": "Teleepoch Ltd",
    "24.10539909": "AUMA Riester GmbH & Co. KG",
    "20.128510962171904": "open joint stock company \"YUG-SISTEMA plus\"",
    "24.8178658": "IEEE Registration Authority",
    "24.10786737": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.8713363": "OMS spol. s.r.o.",
    "12.346868793344": "Quercus Technologies, S.L.",
    "24.8493": "Scimolex Corporation",
    "24.8940356": "Sunnovo International Limited",
    "12.71297883521024": "Vitec",
    "20.198151827881984": "Stratom, Inc.",
    "24.9196498": "China Mobile Group Device Co.,Ltd.",
    "24.7865459": "Texas Instruments",
    "20.150399883411456": "Gefran Drive & Motion srl",
    "24.13393239": "Information System Research Institute,Inc.",
    "24.8666566": "Cisco Systems, Inc",
    "24.551621": "Intel Corporate",
    "12.346867019776": "Plant Integrity Limited",
    "24.14973759": "Beijing Co-Cloud Technology Ltd.",
    "12.346863357952": "Update Systems Inc.",
    "24.829076": "Sunitec Enterprise Co.,Ltd",
    "12.346868166656": "Special Systems Engineering Center LLC",
    "24.5506037": "EBN Technology Corp.",
    "20.67047988920320": "Shenzhen Vplus Communication Intelligent Co., Ltd.",
    "12.346866487296": "Tieline Research Pty Ltd",
    "12.123917676400640": "Sanwa New Tec Co.,Ltd",
    "24.6304151": "Zyxel Communications Corporation",
    "12.123917675851776": "sohonet ltd",
    "12.123917686792192": "Paradigm Communication Systems Ltd",
    "24.2406141": "Dell Inc.",
    "12.123917683408896": "PMT Corporation",
    "20.114975472353280": "Ambitio LLC",
    "12.123917686276096": "Ritec",
    "24.8126730": "Texas Instruments",
    "12.346862948352": "Wayne Dalton Corp.",
    "24.1315092": "Tecno Mobile Limited",
    "20.168198556090368": "Beijing Horizon Information Technology Co., Ltd",
    "12.123917685903360": "Creative",
    "12.123917683937280": "QuirkLogic",
    "24.581579": "Systrome Networks",
    "24.7107104": "D-Link International",
    "12.123917685637120": "Booz Allen Hamilton",
    "24.11540054": "Huawei Technologies Co.,Ltd",
    "12.346854465536": "Simet",
    "12.123917679767552": "Tattile Srl",
    "12.346855018496": "Richmond Sound Design Ltd.",
    "12.123917684916224": "Jireh Industries Ltd.",
    "24.4461633": "AudioControl Inc.",
    "12.346857889792": "Visi-tech Systems Ltd",
    "24.1048808": "National Semiconductor",
    "24.6052551": "Cisco Systems, Inc",
    "12.123917688750080": "Specialised Imaging Limited",
    "24.1048800": "Apple A/UX",
    "24.1604737": "Valeo Vision Systems",
    "24.12902450": "IEEE 1904.1 Working Group",
    "24.7369229": "Lenovo Mobile Communication Technology Ltd.",
    "24.15782223": "Linear Llc",
    "24.1586": "Mesco Engineering GmbH",
    "24.12900989": "Ivium Technologies B.V.",
    "24.14962294": "Championtech Enterprise (Shenzhen) Inc",
    "12.123917690052608": "Cardinal Scales Manufacturing Co",
    "12.123917680885760": "Dave Srl",
    "24.16531371": "Henan Lanxin Technology Co., Ltd",
    "24.32874": "Eri (Empac Research Inc.)",
    "24.32875": "Schmid Telecommunication",
    "24.32876": "Cegelec Projects Ltd",
    "24.32877": "Century Systems Corp.",
    "24.32870": "Arcom Control Systems, Ltd.",
    "24.32871": "Square D Company",
    "24.32872": "Yamatech Scientific Ltd.",
    "24.32873": "Computone Systems",
    "24.10750957": "Carry Technology Co.,Ltd",
    "24.32878": "Nippon Steel Corporation",
    "24.32879": "Onelan Ltd.",
    "24.14951532": "Zhejiang Dahua Technology Co., Ltd.",
    "24.6588913": "Samsung Electronics Co.,Ltd",
    "24.13124905": "IMK Networks Co.,Ltd",
    "24.4491467": "Camco Technologies NV",
    "24.9191924": "IPmotion GmbH",
    "12.123917677207552": "Acutronic Link Robotics AG",
    "24.9191922": "RDA Technologies Ltd.",
    "20.255191445143552": "Siliconcube",
    "24.7403751": "IEEE Registration Authority",
    "24.6093296": "Atomos Engineering P/L",
    "24.9744650": "Samsung Electronics Co.,Ltd",
    "24.6557828": "HEXIUM Technical Development Co., Ltd.",
    "12.346867040256": "Goennheimer Elektronic GmbH",
    "12.123917675401216": "IntelliDesign Pty Ltd",
    "24.8674547": "Intel Corporate",
    "24.11820196": "Thing-talk Wireless Communication Technologies Corporation Limited",
    "12.346867077120": "Telemetrics Inc.",
    "12.123917685444608": "Vickers Electronics Ltd",
    "24.8959595": "easynetworks",
    "12.346864840704": "Axis-Shield PoC AS",
    "24.12863544": "Satcom Direct, Inc.",
    "24.14979626": "Dbg Holdings Limited",
    "20.70559715033088": "gridX GmbH",
    "24.7368172": "Wifi-soft LLC",
    "24.8972559": "innomdlelab",
    "12.346861146112": "Admiral Secure Products, Ltd.",
    "20.97843036553216": "Open Roads Consulting, Inc.",
    "12.123917690134528": "Lieron BVBA",
    "24.14432623": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.10810469": "Itel Mobile Limited",
    "24.11016984": "Xts Corp",
    "20.189244305833984": "Wesco Integrated Supply",
    "24.15488046": "Shanghai XiMei Electronic Technology Co. Ltd",
    "24.7352711": "ACX GmbH",
    "24.10749342": "STA Infinity LLP",
    "24.14203993": "Shenzhen DAPU Microelectronics Co., Ltd",
    "12.123917681983488": "Vitsch Electronics",
    "12.71297884524544": "Engage Technologies",
    "12.346859307008": "EDS Systemtechnik",
    "24.16308159": "REV Ritter GmbH",
    "20.127937500151808": "uGrid Network Inc.",
    "24.15252285": "Sino Prime-Tech Limited",
    "24.3968774": "Shenzhen Libtor Technology Co.,Ltd",
    "24.11031730": "Wistron Neweb Corporation",
    "24.3734218": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.11044845": "ARC Wireless LLC",
    "12.346860851200": "Design Analysis Associates, Inc.",
    "12.71297883693056": "Tescom Corporation",
    "24.11853627": "Chromlech",
    "12.123917683691520": "RF-Tuote Oy",
    "24.9998036": "Itel Mobile Limited",
    "20.273458022842368": "etectRx",
    "24.13644137": "Samsung Electronics Co.,Ltd",
    "20.141720411963392": "Dalian Roiland Technology Co.,Ltd",
    "24.12588877": "Samsung Electronics Co.,Ltd",
    "12.123917676576768": "LHA Systems (Pty) Ltd",
    "12.346860269568": "InRay Solutions Ltd.",
    "24.16301074": "Dell Inc.",
    "12.346859401216": "Ace Axis Limited",
    "24.3730247": "Huawei Technologies Co.,Ltd",
    "12.346860306432": "MoBaCon",
    "12.346858090496": "Phytec Messtechnik GmbH",
    "24.6319420": "Luxshare Precision Industry Company Limited",
    "12.71297885020160": "Digital Display Systems",
    "12.123917685665792": "Clever Devices",
    "12.346868252672": "IEEE Registration Authority",
    "12.346861256704": "Telsa s.r.l",
    "24.10503131": "Sitael S.p.A.",
    "24.8442208": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.2584": "Vichel Inc.",
    "12.346863960064": "Tecnint HTE SRL",
    "24.7927269": "BEGA Gantenbrink-Leuchten KG",
    "24.2586": "Imerge Ltd",
    "24.3934238": "Beijing Yupont Electric Power Technology Co.,Ltd",
    "24.12856399": "Qingdao Hisense Mobile Communication Technology Co,Ltd",
    "24.7353045": "Athena Wireless Communications Inc",
    "24.2587": "Stream Labs",
    "20.71414659940352": "Palazzetti Lelio Spa",
    "24.2580": "TECO a.s.",
    "24.2581": "Silicon Data, Inc",
    "24.8660942": "Corporation of the Presiding Bishop of The Church of Jesus Christ of Latter-day Saints",
    "24.2386250": "Jasmine Systems Inc.",
    "24.10509931": "MELPER Co., Ltd.",
    "24.2583": "Nestar Communications, Inc",
    "12.123917687050240": "LOGIX ITS Inc",
    "24.12612781": "Grandstream Networks, Inc.",
    "24.41009": "Hazeltine Corporation, Ms 1-17",
    "24.41008": "Captor Nv/Sa",
    "24.41007": "ADB Broadband Italia",
    "24.41006": "Brand Communications, Ltd.",
    "24.41005": "1394 Trade Association",
    "24.41004": "interWAVE Communications",
    "24.41003": "Transitions Research Corp.",
    "24.41002": "Trancell Systems",
    "24.41001": "Coulter Corporation",
    "24.41000": "Conner Peripherals",
    "20.75685681954816": "Vuro Llc",
    "12.346864525312": "Aitecsystem Co.,Ltd.",
    "20.251330461433856": "Tacom Projetos Bilhetagem Inteligente ltda",
    "24.15732060": "JinQianMao Technology Co.,Ltd.",
    "24.16276557": "Nokia",
    "24.12900152": "Huawei Device Co., Ltd.",
    "12.346857451520": "Logic Beach Inc",
    "20.260167864156160": "DRB Systems",
    "24.12596822": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6828542": "Teladin Co.,Ltd.",
    "24.11832089": "Apple, Inc.",
    "12.123917680607232": "Multiple Access Communications Ltd",
    "24.1050": "Ines Test and Measurement GmbH & CoKG",
    "24.13654421": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346852175872": "Microsoft",
    "12.123917678907392": "BirdDog Australia",
    "24.5558452": "Xiamen Four-Faith Communication Technology Co.,Ltd",
    "12.346864947200": "4D Technology Corporation",
    "20.53091433971712": "Atli World Limited",
    "24.8678216": "ShenZhen EepuLink Co., Ltd.",
    "24.12624798": "EarthCam, Inc.",
    "24.14428833": "MedHab LLC",
    "12.346865770496": "Grupo Epelsa S.L.",
    "12.71297883308032": "ENTEC Electric & Electronic CO., LTD.",
    "24.14977919": "Apple, Inc.",
    "24.12523": "Turbonet Communications, Inc.",
    "20.247076408524800": "Motiveprime Consumer Electronics Pvt Ltd",
    "12.123917682851840": "HUSTY M.Styczen J.Hupert Sp.J.",
    "24.7100790": "Shanghai Tricheer Technology Co.,Ltd.",
    "12.123917681340416": "Edgeware AB",
    "24.15772887": "IEEE Registration Authority",
    "24.15501900": "zte corporation",
    "20.48543318409216": "GTA Electronics Co., Ltd.",
    "24.12345360": "Qingdao Hisense Communications Co.,Ltd.",
    "24.10494686": "vivo Mobile Communication Co., Ltd.",
    "24.800830": "Fanvil Technology Co., Ltd.",
    "24.7386069": "IEEE Registration Authority",
    "24.299758": "iway AG",
    "20.62636958941184": "Fibergate Inc.",
    "12.346858688512": "Qstreams Networks Inc.",
    "24.43522": "Intel Corporation",
    "24.43520": "Intel Corporation",
    "24.43521": "Intel Corporation",
    "24.45201": "Transmeta Corp.",
    "12.123917690347520": "Transportal Solutions Ltd",
    "24.45204": "Alaris, Inc.",
    "12.346857086976": "TESLA, a.s.",
    "24.8177879": "Damalisk",
    "24.5254644": "Phytec Messtechnik GmbH",
    "24.10494357": "Samsung Electronics Co.,Ltd",
    "24.4485112": "Dyson Limited",
    "12.346859106304": "Total RF, LLC",
    "24.8162017": "Compal Information (Kunshan) Co., Ltd.",
    "24.1350447": "JiangSu ZhongXie Intelligent Technology co., LTD",
    "24.8177872": "Tianjin Yaan Technology Co., Ltd.",
    "24.7938": "Pixelmetrix Corporation Pte Ltd",
    "24.7939": "Num Ag",
    "24.7932": "Jsc \"Massa-K\"",
    "24.7933": "Microbit 2.0 AB",
    "24.7930": "PROTEI Ltd.",
    "24.7931": "Trio Motion Technology Ltd",
    "24.7936": "Nokia Danmark A/S",
    "24.7937": "Nokia Danmark A/S",
    "24.7934": "LEVEL s.r.o.",
    "24.7935": "Mueller-Elektronik GmbH & Co. KG",
    "12.123917687988224": "Wiznova",
    "12.123917685796864": "McKay Brothers LLC",
    "24.6595122": "OOO Samlight",
    "24.12353823": "Ingate Systems",
    "24.10245322": "Zhengzhou VCOM Science and Technology Co.,Ltd",
    "20.13592321589248": "Shenzhen C & D Electronics Co., Ltd.",
    "12.346854477824": "Polimar Elektronik Ltd.",
    "24.10803600": "Samsung Electronics Co.,Ltd",
    "24.1357879": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.9754331": "NexFi",
    "24.1863286": "Quarion Technology Inc",
    "24.16629": "Oem Engines",
    "24.16628": "Cameo Communications, Inc.",
    "24.13910224": "OCOSMOS Co., LTD",
    "24.16623": "Hypercom, Inc.",
    "24.16622": "Optimem",
    "24.16621": "Network Controls International Inc",
    "24.16620": "Mikasa System Engineering",
    "24.16627": "Netcor",
    "24.16626": "Janich & Klass Computertechnik",
    "24.16625": "Chuo Electronics Co., Ltd.",
    "24.16624": "Micro Systems Inc",
    "24.6297969": "ARRIS Group, Inc.",
    "24.6297968": "Huizhou Qiaoxing Electronics Technology Co., Ltd.",
    "12.346859499520": "St. Michael Strategies Inc.",
    "24.13908408": "Ciena Corporation",
    "12.71297884090368": "Xadi Inc",
    "12.346855002112": "Hobbes Computer Network Accessories",
    "24.16515452": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5274197": "Apple, Inc.",
    "24.805716": "Tp-Link Technologies Co.,Ltd.",
    "24.3412968": "Intel Corporate",
    "24.581407": "Bose Corporation",
    "24.7377295": "Weber-Stephen Products LLC",
    "12.346865655808": "Axotec Technologies GmbH",
    "24.11585874": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1839": "Intransa, Inc.",
    "24.5399": "Intel Corporate",
    "24.13664196": "Ou Wei Technology Co.，Ltd. of Shenzhen City",
    "24.1831": "Zi Corporation (HK) Ltd.",
    "24.5391": "mingjong",
    "24.5392": "Techsphere Co., Ltd",
    "24.5393": "Data Center Systems",
    "24.5394": "Zurich University of Applied Sciences",
    "24.5395": "EFS sas",
    "24.5396": "Hu Zhou NAVA Networks&Electronics Ltd.",
    "24.583130": "Nanjing Fujitsu Computer Products Co.,Ltd.",
    "24.12910306": "AMICCOM Electronics Corporation",
    "20.184991245926400": "Neos Ventures Limited",
    "24.6086902": "NIC.br- Nucleo de Informacao e Coordenacao do Ponto BR",
    "12.346854232064": "Fr. Sauter AG",
    "24.3941847": "Nokia Corporation",
    "24.6875074": "Samsung Electronics Co.,Ltd",
    "24.11807037": "Shenzhen Urovo Technology Co.,Ltd.",
    "24.10288801": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12626536": "OSASI Technos Inc.",
    "24.281599": "GuangZhou Hedy Digital Technology Co., Ltd",
    "24.3180953": "Guangdong East Power Co.,",
    "24.9722119": "Shanghai HITE-BELDEN Network Technology Co., Ltd.",
    "12.346865242112": "CODESYSTEM Co.,Ltd",
    "24.5642": "SWEEX Europe BV",
    "24.5643": "TVWorks LLC",
    "24.5640": "Sequans Communications",
    "24.5641": "Unitech electronics co., ltd.",
    "24.5646": "Optica Technologies Inc.",
    "24.5647": "Badger Meter Inc",
    "24.5644": "Lpl Development S.A. De C.V",
    "24.5645": "Be Here Corporation",
    "24.5648": "Carina Technology",
    "24.5649": "Altecon Srl",
    "24.542672": "Nokia Shanghai Bell Co., Ltd.",
    "12.346855694336": "Jobin Yvon,Inc",
    "20.66129351409664": "Sivat Technology Co.,Ltd.",
    "24.6680": "Advanced Simulation Technology inc.",
    "12.346862911488": "Mosberger Consulting LLC",
    "24.12110059": "Itel Mobile Limited",
    "24.13132564": "IEEE Registration Authority",
    "24.6555980": "Beijing Superbee Wireless Technology Co.,Ltd",
    "24.3690313": "Juniper Networks",
    "20.277958196461568": "Aroma Retail",
    "24.11050988": "ARRIS Group, Inc.",
    "24.57497": "Samson Ag",
    "24.57496": "AboCom",
    "24.57495": "Carrier Access Corporation",
    "24.57494": "Shimadzu Corporation",
    "24.57493": "Advanced-Vision Technolgies Corp.",
    "24.57492": "Osai Srl",
    "24.57491": "Ackfin Networks",
    "24.57490": "Admtek Incorporated",
    "12.346868613120": "Tateishi Kobisha Co.LTD",
    "12.123917683269632": "Depro Électronique inc",
    "24.57499": "Engage Networks, Inc.",
    "24.57498": "Positron Inc.",
    "24.14191163": "Unit-Em",
    "24.16542023": "Texas Instruments",
    "12.346861907968": "Piller engineering Ltd.",
    "24.6826170": "Dejai",
    "12.123917682040832": "INRADIOS GmbH",
    "24.8962005": "Simple Audio Ltd",
    "24.13432120": "3isysnetworks",
    "12.346855731200": "CANMAX Technology Ltd.",
    "24.14459810": "Feng mi(Beijing)technology co., LTD",
    "24.4240158": "Universal Electronics, Inc.",
    "12.346858078208": "Federal State Unitary Enterprise Experimental Factory for Sc",
    "24.1329596": "DrayTek Corp.",
    "12.123917679906816": "Gaci",
    "24.7875398": "Intel Corporate",
    "24.9930": "Cisco Systems, Inc",
    "24.9931": "Cisco Systems, Inc",
    "24.9932": "Nokia Danmark A/S",
    "24.9933": "PurpleComm, Inc.",
    "24.9934": "Kozumi USA Corp.",
    "24.9935": "Deka R&D",
    "24.9936": "Semihalf",
    "24.9937": "S Squared Innovations Inc.",
    "24.9938": "Pcube Systems, Inc.",
    "24.9939": "Zeno Information System",
    "20.89058723233792": "Sea Eagle Optoelectronic Information Technology(Tianjin)co,Ltd",
    "24.4511": "Nokia Danmark A/S",
    "24.4510": "Solectron Brazil",
    "24.4513": "Vision Netware Co.,Ltd",
    "24.4512": "Vtech Engineering Canada Ltd",
    "24.4515": "LanReady Technologies Inc.",
    "24.4514": "Manufacturing Technology Inc",
    "24.4517": "Fortuna Electronic Corp.",
    "24.4516": "JStream Technologies Inc.",
    "24.4519": "Infilco Degremont Inc.",
    "24.4518": "Sypixx Networks",
    "24.3732": "Maas International BV",
    "12.346867691520": "Taiwan Hiplus Corporation",
    "12.346856554496": "Arcom Digital",
    "24.12103278": "Nintendo Co., Ltd.",
    "24.1052854": "McCain Inc",
    "24.11598179": "Hangzhou H3C Technologies Co., Limited",
    "12.123917688164352": "KST technology",
    "12.346855165952": "Westek Technology Ltd",
    "20.128510970560512": "Wuhan Shenghong Laser Projection Technology Co.,LTD",
    "12.123917681758208": "Sea Shell Corporation",
    "24.20587": "Spx-Ateg",
    "24.20586": "Edeva, Inc.",
    "24.20585": "PixStream Incorporated",
    "24.20584": "Electronic Industries Association",
    "24.20583": "Aerocomm, Inc.",
    "24.20582": "AtecoM GmbH advanced telecomunication modules",
    "24.20581": "TDK-Lambda Corporation",
    "24.20580": "Cae Electronics",
    "24.20589": "Videojet Systems",
    "24.20588": "Beijer Electronics Products AB",
    "24.8420031": "HTC Corporation",
    "24.11026337": "IEEE Registration Authority",
    "24.12089726": "Secure Meters (UK) Limited",
    "24.16553068": "Ubivelox",
    "24.21186": "peiker acustic GmbH",
    "24.12874068": "Tp-Link Technologies Co.,Ltd.",
    "24.828458": "OB Telecom Electronic Technology Co., Ltd",
    "12.346866315264": "HMI Technologies",
    "12.123917680439296": "R.C. Systems Inc",
    "24.14166296": "Huawei Technologies Co.,Ltd",
    "20.36069852577792": "Ariston Thermo s.p.a.",
    "24.14166294": "Ascent Communication Technology",
    "24.20639": "Horizon Computer",
    "24.20638": "Les Technologies SoftAcoustik Inc.",
    "20.180577511669760": "Deatronic srl",
    "24.20635": "Switchcore Ab",
    "24.20634": "Tag Electronic Systems",
    "24.20637": "The Industree B.V.",
    "24.20636": "Beta Research",
    "24.20631": "MMC-EMBEDDED COMPUTERTECHNIK GmbH",
    "24.20630": "Salix Technologies, Inc.",
    "24.20633": "3Com Europe Ltd",
    "24.20632": "Globaloop, Ltd.",
    "24.7099": "RFTech Co.,Ltd",
    "24.7098": "Nortel Networks",
    "24.7095": "Alta Heights Technology Corp.",
    "24.7094": "Bird Electronic Corp.",
    "24.7097": "Elitegroup Computer Systems Co.,Ltd.",
    "24.7096": "Blueway Electronic Co;Ltd",
    "24.7091": "Condalo GmbH",
    "24.7090": "Intellect International NV",
    "24.7093": "Cherry GmbH",
    "24.7092": "Airvod Limited",
    "20.150262243131392": "Zhejiang Haoteng Electronic Technology Co.,Ltd.",
    "12.123917680529408": "Airity Technologies Inc.",
    "12.123917678628864": "Adel System srl",
    "24.9734854": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.2139660": "Xiaomi Communications Co Ltd",
    "24.8943000": "Samsung Electronics Co.,Ltd",
    "24.49585": "Cisco Systems, Inc",
    "24.9480899": "Hitron Technologies. Inc",
    "12.123917679611904": "Energisme",
    "24.6058147": "LG Electronics (Mobile Communications)",
    "24.4518427": "Apple, Inc.",
    "24.7362578": "Panasonic Corporation AVC Networks Company",
    "24.36174": "Cjsc Nii Stt",
    "24.7147": "DMP Electronics INC.",
    "24.12896748": "Gugaoo HK Limited",
    "12.71297884663808": "Techway",
    "24.32897": "Kendall Square Research Corp.",
    "24.4602": "Rane Corporation",
    "24.7145": "Broadcom",
    "12.123917678694400": "Kamacho Scale Co., Ltd.",
    "12.123917691580416": "Acc+Ess Ltd",
    "24.6091947": "Zyxel Communications Corporation",
    "24.32895": "Dy-4 Incorporated",
    "24.32892": "Fibercom, Inc.",
    "12.123917685108736": "Shanghai Hourui Technology Co., Ltd.",
    "24.28001": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "24.32893": "Equinox Systems Inc.",
    "12.123917680844800": "Crde",
    "20.13692006563840": "Tiinlab Acoustic Technology (Shenzhen) Co., Ltd.",
    "24.7141": "802automation Limited",
    "24.3192788": "Hitron Technologies. Inc",
    "24.32891": "Artel Communications Corp.",
    "24.322368": "Cisco Systems, Inc",
    "24.16531971": "Samsung Electronics Co.,Ltd",
    "12.346852470784": "Tecstar Demo Systems Division",
    "12.123917679521792": "Velvac Incorporated",
    "12.346861948928": "Roessmann Engineering",
    "24.2928733": "Netgear",
    "24.32898": "Pep Modular Computers Gmbh",
    "24.9232102": "Samsung Electronics Co.,Ltd",
    "24.32899": "Amdahl",
    "24.16556586": "PT. Callysta Multi Engineering",
    "24.2922477": "Sony Imaging Products & Solutions Inc.",
    "20.176189954064384": "Shenzhen Neostra Technology Co.Ltd",
    "12.123917684989952": "Aplex Technology Inc.",
    "12.346868477952": "Computerwise, Inc.",
    "12.123917679181824": "TangRen C&S CO., Ltd",
    "24.24639": "Patapsco Designs",
    "24.24638": "Cisco",
    "24.6329806": "IEEE Registration Authority",
    "24.6811": "ADEC & Parter AG",
    "24.6841419": "Apple, Inc.",
    "24.24631": "NXP Semiconductors",
    "24.24630": "AIT Austrian Institute of Technology GmbH",
    "24.24633": "SanCom Technology, Inc.",
    "24.24632": "Nortel Networks",
    "24.24635": "AMTEC spa",
    "24.24634": "Quick Controls Ltd.",
    "24.24637": "3Cx",
    "24.24636": "Hagiwara Sys-Com Co., Ltd.",
    "24.550": "XESystems, Inc.",
    "24.551": "ESD Electronic System Design GmbH",
    "24.552": "Necsom, Ltd.",
    "24.553": "Adtec Corporation",
    "24.554": "Asound Electronic",
    "24.555": "SAXA, Inc.",
    "24.556": "ABB Bomem, Inc.",
    "24.557": "Agere Systems",
    "24.558": "TEAC Corp. R& D",
    "24.559": "P-Cube, Ltd.",
    "24.6815": "A-Link Ltd",
    "24.9699": "Hanshinit Inc.",
    "24.9698": "Everspring Industry Co., Ltd.",
    "24.4254414": "Founder Broadband Network Service Co.,Ltd",
    "12.123917686550528": "Sprintshield d.o.o.",
    "24.15991408": "Dell Inc.",
    "24.11854095": "Dell Inc.",
    "24.6335335": "Xiaomi Communications Co Ltd",
    "12.346854203392": "Tattile srl",
    "24.2916": "Kieback & Peter GmbH & Co KG",
    "12.123917682147328": "Code Blue Corporation",
    "24.2801": "Clarity Design, Inc.",
    "24.2800": "Shin-Oh Electronics Co., Ltd. R&D",
    "24.2803": "Cisco Systems, Inc",
    "24.2802": "NeoAxiom Corp.",
    "24.2805": "Airgo Networks, Inc.",
    "24.2804": "Cisco Systems, Inc",
    "24.2807": "Broadcom",
    "24.2806": "Emerson Climate Technologies Retail Solutions, Inc.",
    "24.2809": "HiConnect, Inc.",
    "24.2808": "American Telecare Inc.",
    "12.346863996928": "Aplex Technology Inc.",
    "24.10549397": "Shenzhen TINNO Mobile Technology Corp.",
    "12.123917687455744": "Cognosos, Inc.",
    "24.2376743": "Huawei Technologies Co.,Ltd",
    "24.1894493": "Cisco Systems, Inc",
    "12.123917676322816": "Dave Srl",
    "24.12593651": "XySystem",
    "12.123917686259712": "BKM-Micronic Richtfunkanlagen GmbH",
    "24.12083345": "Onnet Technologies and Innovations LLC",
    "12.346867277824": "Aplex Technology Inc.",
    "12.123917680119808": "Grupo Epelsa S.L.",
    "24.9450059": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.31372485853184": "Orion Labs inc",
    "24.3435355": "New H3C Technologies Co., Ltd",
    "24.10505732": "Nokia Corporation",
    "24.10025627": "LCFC(HeFei) Electronics Technology co., ltd",
    "20.22739798720512": "MTA Systems",
    "12.346862428160": "ESAB-ATAS GmbH",
    "24.16012768": "Niceway CNC Technology Co.,Ltd.Hunan Province",
    "24.10272157": "SkyDisk, Inc.",
    "12.123917691047936": "Honeywell",
    "24.15477488": "Ypsomed AG",
    "24.12633735": "Cisco SPVTG",
    "12.123917685825536": "APEX Stabilizations GmbH",
    "24.12645942": "Hangzhou Kuaiyue Technologies, Ltd.",
    "12.346862456832": "Bladelius Design Group AB",
    "12.123917681885184": "Sicon srl",
    "12.71297883447296": "Mango Communicaitons Inc.",
    "24.12907282": "Texas Instruments",
    "12.346862080000": "Pyramid Technical Consultants",
    "20.211776050298880": "Jsc Npk Atronik",
    "24.12601172": "vivo Mobile Communication Co., Ltd.",
    "24.1074595": "Panamax LLC",
    "12.346860789760": "Syr-Tec Engineering & Marketing",
    "12.71297884172288": "ECON Systems Inc.",
    "24.323902": "Apple, Inc.",
    "24.3937730": "Apple, Inc.",
    "24.3969396": "Along Communication Technology",
    "12.346865610752": "Covidence A/S",
    "20.171977811689472": "Shenzhen jiahua zhongli technology co.LTD",
    "12.346862415872": "Kumera Drives Oy",
    "24.3466732": "Palo Alto Networks",
    "12.71297883869184": "Apantac LLC",
    "24.8258": "Datametrics Corp.",
    "24.8259": "Neuron Company Limited",
    "24.8256": "ARRIS Group, Inc.",
    "24.8257": "Data Net",
    "24.8254": "LogiCan Technologies, Inc.",
    "24.8255": "Juki Corporation",
    "24.8252": "Eurotime Ab",
    "24.8253": "Honeywell Environmental & Combustion Controls",
    "24.8250": "Digital Bi0Metrics Inc.",
    "24.8251": "Wisdm Ltd.",
    "12.123917690093568": "Hong Jiang Electronics Co., Ltd.",
    "12.346852868096": "Honeywell GNO",
    "12.71297883971584": "SC Techswarm SRL",
    "20.75247167471616": "Beijing gpthink technology co.,LTD.",
    "24.36917": "Alpha Telecom, Inc.",
    "24.36916": "Imagic, Inc.",
    "24.36915": "Innovaphone Ag",
    "24.36914": "Pelcombe Group Ltd.",
    "24.36913": "Mysticom, Ltd.",
    "24.36912": "Honeywell-Dating",
    "24.36911": "Netcore Systems, Inc.",
    "24.36910": "Namco Limited",
    "12.346864336896": "Dave Srl",
    "24.36918": "ens, inc.",
    "24.13927780": "Hewlett Packard",
    "24.7614832": "Arcadyan Technology Corporation",
    "24.3428188": "R&amp;M AG",
    "12.123917680766976": "MBio Diagnostics, Inc.",
    "24.9707927": "Stalmart Technology Limited",
    "24.12339040": "Samsung Electronics Co.,Ltd",
    "24.8701959": "ADVA Optical Networking Ltd.",
    "24.11305968": "ImmediaTV Corporation",
    "12.123917679177728": "Liberty Reach",
    "20.162752345669632": "Lao Industria Ltda",
    "12.346856624128": "Sammi Information Systems Co.,Ltd",
    "12.123917678092288": "Health Care Originals, Inc.",
    "24.6304059": "Sunnovo International Limited",
    "24.9221711": "Cisco Systems, Inc",
    "12.123917689135104": "Speedifi Inc",
    "20.31368187740160": "Beijing Geedeen Technology Co., Ltd",
    "24.3199878": "zte corporation",
    "24.2368526": "Apple, Inc.",
    "12.346856038400": "Tennessee Valley Authority",
    "24.13923355": "Beijing HuaDa ZhiBao Electronic System Co.,Ltd.",
    "24.2108707": "Collinear Networks Inc.",
    "24.8401116": "Texas Instruments",
    "24.12361711": "Apple, Inc.",
    "12.346861293568": "Keith & Koep GmbH",
    "24.10242431": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.137149114482688": "Continental",
    "24.6613701": "SiboTech Automation Co., Ltd.",
    "12.346863878144": "Concepteers, LLC",
    "12.346860396544": "HTEC Limited",
    "24.6092961": "Murata Manufacturing Co., Ltd.",
    "24.3453431": "Texas Instruments",
    "12.346855124992": "Gerber Scientific Products, Inc.",
    "24.16303736": "Carefusion",
    "12.346866995200": "ACD Elektronik GmbH",
    "24.9441844": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.6341527": "2Wire Inc",
    "24.9750251": "NOVA electronics, Inc.",
    "12.119269560320": "QQ Navigation AB",
    "20.273458030182400": "3SI Security Systems, Inc",
    "12.123917685510144": "Wartsila Voyage Limited",
    "24.5300357": "Intel Corporate",
    "24.5786": "Quadrics Ltd",
    "12.346866823168": "Bettini Srl",
    "24.5562688": "Ingenico",
    "12.346853355520": "Seranoa Networks, Inc.",
    "24.5821806": "D-Link International",
    "12.71297884987392": "Creative Lighting And Sound Systems Pty Ltd",
    "12.123917684953088": "RCH Vietnam Limited Liability Company",
    "24.3210552": "Google, Inc.",
    "24.9223716": "Tashang Semiconductor(Shanghai) Co., Ltd.",
    "12.346862100480": "KST Technology Co., Ltd",
    "24.12324651": "Fortune Techgroup Co.,Ltd",
    "12.346859683840": "Richard Griessbach Feinmechanik GmbH",
    "12.346853543936": "Redux Communications Ltd.",
    "24.14175673": "Wu Qi Technologies,Inc.",
    "24.4782016": "Technicolor CH USA Inc.",
    "24.15737973": "Apple, Inc.",
    "24.16017197": "ShenZhen Topstar Technology Company",
    "12.123917688856576": "Insitu Inc.",
    "24.8396546": "Edimax Technology Co. Ltd.",
    "24.6853473": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.2674922": "Intel Corporate",
    "24.1835545": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.10009753": "Edeltech Co.,Ltd.",
    "24.3670423": "TSST Global,Inc",
    "24.3670421": "Samsung Electronics Co.,Ltd",
    "12.123917682032640": "Nudron IoT Solutions LLP",
    "24.13652809": "Liteon Technology Corporation",
    "24.12323171": "Apple, Inc.",
    "12.346854649856": "Albert Handtmann Maschinenfabrik GmbH&Co.KG",
    "24.16020277": "Logitech Far East",
    "24.5028021": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.14705637": "Sony Mobile Communications Inc",
    "24.9976206": "Samsung Electronics Co.,Ltd",
    "24.5261547": "Beijing Haihejinsheng Network Technology Co. Ltd.",
    "24.7915870": "Midmark RTLS",
    "12.123917688827904": "L.I.F.E. Corporation SA",
    "20.176069202149376": "TMGcore LLC",
    "12.346861707264": "Xtone Networks",
    "12.119269367808": "Cjsc Stc Simos",
    "24.6565036": "Huawei Technologies Co.,Ltd",
    "24.8939173": "Apple, Inc.",
    "24.8919304": "Apple, Inc.",
    "24.14439811": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.9996781": "Anadem Information Inc.",
    "24.10262008": "SmartDoor Systems, LLC",
    "12.346868436992": "Enetics, Inc.",
    "24.3183548": "Sagemcom Broadband SAS",
    "24.4236236": "Intel Corporate",
    "20.168210317967360": "Sferrum GmbH",
    "24.10769285": "Weifang Goertek Electronics Co.,Ltd",
    "12.346852413440": "ISDN Gateway Technology AG",
    "24.13902114": "Sercomm Corporation.",
    "24.7907970": "New H3C Technologies Co., Ltd",
    "24.9725340": "Intel Corporate",
    "24.6574342": "Hewlett Packard",
    "24.13648670": "Dell Inc.",
    "24.10010332": "Honor Device Co., Ltd.",
    "24.16044002": "Cisco Systems, Inc",
    "20.132855979048960": "Famar Fueguina S.A.",
    "12.346853699584": "Unicoi Systems",
    "24.8705042": "Palo Alto Networks",
    "12.346864115712": "8185",
    "12.123917691678720": "Veilux Inc.",
    "24.9205522": "Samsung Electronics Co.,Ltd",
    "24.10790099": "Bluebank Communication Technology Co.Ltd",
    "20.154314900242432": "Idro-Elettrica S.P.A.",
    "24.9205526": "Longcheer Telecommunication Limited",
    "24.40963": "Siemens Switzerland Ltd., I B T HVP",
    "24.40962": "Leeds & Northrup Australia Pty Ltd",
    "24.40961": "DRS Signal Solutions",
    "24.40960": "Centillion Networks, Inc.",
    "24.40967": "Apexx Technology, Inc.",
    "24.805945": "MitraStar Technology Corp.",
    "24.40965": "Daniel Instruments, Ltd.",
    "24.40964": "Netpower, Inc.",
    "24.40969": "Whitetree Network",
    "24.40968": "Netcorp",
    "24.8668260": "ServerU Inc",
    "24.13400394": "BLU Products Inc.",
    "12.123917680181248": "Li Seng Technology Ltd.,",
    "24.5806191": "Revolution Display",
    "24.5251014": "Apple, Inc.",
    "24.9498329": "Universal De Desarrollos Electrónicos, Sa",
    "20.62636951601152": "Nordi Telekommunikatsiooni Oü",
    "20.250817674215424": "SFC Energy AG",
    "24.5247893": "Sichuan AI-Link Technology Co., Ltd.",
    "12.346856460288": "Mecsel Oy",
    "24.13383002": "SecuGen Corporation",
    "24.4210855": "Sony Mobile Communications Inc",
    "12.123917689573376": "Asml",
    "24.7360343": "Tp-Link Technologies Co.,Ltd.",
    "12.71297884676096": "Secuinfo Co.Ltd",
    "24.3408": "Galazar Networks",
    "24.3409": "DIVR Systems, Inc.",
    "12.123917687574528": "Lumentum",
    "24.9484512": "Shenzhen Yanray Technology Co.,Ltd",
    "24.3402": "Steag ETA-Optik",
    "24.3403": "Roku, Inc.",
    "24.3400": "AEWIN Technologies Co., Ltd.",
    "24.3401": "Triton Systems of Delaware, Inc.",
    "24.3406": "NDR Co.,LTD.",
    "24.3407": "Kenwood Corporation",
    "24.3404": "Outline Electronics Ltd.",
    "24.3405": "Ninelanes",
    "24.14964202": "Dedicated Computing",
    "20.57353108979712": "Schnick-Schnack-Systems GmbH",
    "12.346856898560": "Kooltech LLC",
    "24.13423390": "IEEE Registration Authority",
    "12.346865311744": "Automated Media Services, Inc.",
    "24.15736751": "Dell Inc.",
    "12.123917679751168": "ECS s.r.l.",
    "24.841405": "Huawei Technologies Co.,Ltd",
    "12.346865414144": "CT Company",
    "20.229108128677888": "PartnerNET LTD",
    "24.11033520": "Shenzhen Dehoo Technology Co.,Ltd",
    "24.3724832": "Limidea Concept Pte. Ltd.",
    "20.48548670341120": "Shijiazhuang King Transportation Equipment Co.,Ltd",
    "12.123917680693248": "Pro-Digital Projetos Eletronicos Ltda",
    "12.346861785088": "APProSoftware.com",
    "24.8140104": "Transcend Information",
    "24.5547": "Pro Co Sound Inc",
    "12.123917675085824": "FRAKO Kondensatoren und Anlagenbau GmbH",
    "12.123917675290624": "Shenzhen Rihuida Electronics Co,. Ltd",
    "24.171406": "Cardiac Recorders, Inc.",
    "12.346862088192": "Saia-Burgess Controls AG",
    "24.8127164": "Hansung Electronics Co. LTD",
    "12.123917686231040": "BroadSoft Inc",
    "24.10253423": "Huawei Technologies Co.,Ltd",
    "24.11839531": "Histar Digital Electronics Co., Ltd.",
    "24.4747294": "Texas Instruments",
    "24.1169": "Technovision, Inc.",
    "24.9703486": "Belkin International Inc.",
    "12.123917678202880": "Fiberbase",
    "12.346865360896": "CyberCraft",
    "24.14452683": "Beijing Perfectek Technologies Co., Ltd.",
    "12.123917683761152": "Brushies",
    "24.3698338": "vivo Mobile Communication Co., Ltd.",
    "24.2127360": "Actiontec Electronics, Inc",
    "24.14688237": "Nokia Shanghai Bell Co., Ltd.",
    "12.123917681696768": "Icon Industrial Engineering",
    "24.11297999": "Fn-Link Technology Limited",
    "20.211525441683456": "Uhnder",
    "24.3696197": "OOSIC Technology CO.,Ltd",
    "24.6553706": "Dell Inc.",
    "12.346855952384": "DigitalDeck, Inc.",
    "24.1161": "YAFO Networks, Inc.",
    "24.7927760": "Silverbrook Research",
    "12.346854608896": "STMicroelectronics SRL",
    "24.1113837": "Tp-Link Technologies Co.,Ltd.",
    "24.15470715": "Samsung Electronics Co.,Ltd",
    "12.346868862976": "LOGISOL Kft.",
    "24.41": "Imc Networks Corp.",
    "24.40": "Prodigy Systems Corporation",
    "24.43": "Crisp Automation, Inc",
    "24.42": "Trw - Sedd/Inp",
    "24.45": "Chromatics Inc",
    "24.44": "Autotote Limited",
    "24.47": "Timeplex Inc.",
    "24.46": "Societe Evira",
    "24.49": "Qpsx Communications, Ltd.",
    "24.48": "Vg Laboratory Systems Ltd",
    "24.9985171": "SBG Systems SAS",
    "12.71297884192768": "Better Place",
    "24.3705689": "Google, Inc.",
    "12.123917677715456": "Rohde&Schwarz Topex SA",
    "24.8966673": "Eko Devices, Inc.",
    "24.7881846": "Zioncom Electronics (Shenzhen) Ltd.",
    "24.7879": "2Wire Inc",
    "24.7878": "Obvius Holdings LLC",
    "24.7877": "Middle Atlantic Products Inc",
    "24.7876": "Celio Corp",
    "24.7875": "Kozio, Inc.",
    "24.7874": "Apple, Inc.",
    "24.7873": "3Com Europe Ltd",
    "24.7872": "Microchip Technology Inc.",
    "24.7871": "Haas Automation Inc.",
    "24.7870": "Cisco Systems, Inc",
    "24.5780774": "Deeplet Technology Corp",
    "24.12629463": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.9759082": "Apple, Inc.",
    "24.10006373": "Intel Corporate",
    "24.12101807": "Logic S.p.A.",
    "20.52821119467520": "Beijing Ruiteng Zhongtian TECH Ltd.,Co",
    "24.2919026": "HTC Corporation",
    "12.346853662720": "Pmc",
    "20.229631761317888": "Hangzhou xiaoben technology co.,Ltd",
    "24.5530841": "Sagemcom Broadband SAS",
    "24.3705534": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.14987297": "Samsung Electronics Co.,Ltd",
    "12.346853187584": "GE Transportation Systems",
    "12.123917679640576": "Shanghai Chenzhu Instrument Co., Ltd.",
    "20.57176934580224": "Shenzhen Wxl Electronics Co., Ltd.",
    "12.123917676277760": "Subject Link Inc",
    "24.12622449": "Xiamen Meitu Mobile Technology Co.Ltd",
    "24.15758469": "NGD Systems, Inc.",
    "24.8861": "Pyung-Hwa Ind.Co.,Ltd",
    "24.7625011": "Danal Entertainment",
    "12.71297884966912": "A+EC Klein Ingenieurbuero",
    "24.13641112": "Apple, Inc.",
    "24.8957469": "Compal Information (Kunshan) Co., Ltd.",
    "24.10251063": "Renesas Electronics (Penang) Sdn. Bhd.",
    "24.15256877": "Huawei Technologies Co.,Ltd",
    "12.346857799680": "Procon Electronics",
    "20.79602636554240": "Cnu",
    "24.16039983": "BlueBite",
    "12.346853285888": "Compuworx",
    "24.2162161": "3Com Europe Ltd",
    "12.123917676552192": "Gencoa Ltd",
    "24.13663539": "Huawei Device Co., Ltd.",
    "24.13823": "Texas Instruments",
    "24.5024278": "AzureWave Technologies (Shanghai) Inc.",
    "12.346859270144": "QuickCircuit Ltd.",
    "12.346868899840": "Booyco Electronics",
    "24.11329552": "Liteon Technology Corporation",
    "20.203238170165248": "Private",
    "24.1586940": "Elecsys International Corporation",
    "24.6557134": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.6330833": "Vuzix / Lenovo",
    "24.3966058": "Tp-Link Technologies Co.,Ltd.",
    "24.1118": "PolyTrax Information Technology AG",
    "24.1119": "Avalue Technology, Inc.",
    "12.346858004480": "PaloDEx Group Oy",
    "24.1112": "Fusion X Co., Ltd.",
    "24.1113": "Veristar Corporation",
    "24.1110": "Cambium Networks Limited",
    "24.1111": "Universal Access Technology, Inc.",
    "24.1116": "Mobiwave Pte Ltd",
    "24.1117": "BEKA Elektronik",
    "24.1114": "The Linksys Group, Inc.",
    "24.1115": "Techsan Electronics Co., Ltd.",
    "12.346857046016": "CYJAYA Korea",
    "24.10526097": "SoundBridge",
    "24.6051865": "Jingsheng Technology Co., Ltd.",
    "24.9763762": "Shenzhen Gongjin Electronics Co.,Lt",
    "20.137149110288384": "AG Neovo",
    "24.6838": "Texas Instruments",
    "24.6839": "Ethos Networks LTD.",
    "20.149935442886656": "Hexaglobe",
    "24.11026557": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "12.123917685641216": "PulseTor LLC",
    "24.6830": "Savant Systems LLC",
    "24.6831": "Blusens Technology",
    "24.6832": "Signal Networks Pvt. Ltd.,",
    "24.6833": "Asia Pacific Satellite Industries Co., Ltd.",
    "24.6834": "Cyber Solutions Inc.",
    "24.6835": "Visionite Inc.",
    "24.6836": "FFEI Ltd.",
    "24.6837": "Home Network System",
    "12.123917676519424": "LumiGrow, Inc",
    "24.16528223": "Polyera",
    "24.13658655": "Bse Co.,Ltd.",
    "24.7887198": "Samsung Electronics Co.,Ltd",
    "24.12909796": "DishTV NZ Ltd",
    "24.2363685": "Hutek Co., Ltd.",
    "24.5536873": "Cisco Systems, Inc",
    "24.13662901": "Huawei Technologies Co.,Ltd",
    "20.220327863910400": "Autonics Co., Ltd.",
    "24.15242758": "testo Instruments (Shenzhen) Co., Ltd.",
    "24.9751556": "IEEE Registration Authority",
    "20.62663539294208": "Edge I&D Co., Ltd.",
    "24.1079506": "Elitegroup Computer Systems Co.,Ltd.",
    "20.48548673486848": "FOCAL-JMLab",
    "24.13947606": "Fn-Link Technology Limited",
    "24.149": "Sony Tektronix Corp.",
    "24.148": "Asante",
    "24.143": "Raytheon",
    "24.142": "Solbourne Computer, Inc.",
    "24.141": "Cryptek Inc.",
    "24.140": "Alloy Computer Products (Australia) Pty Ltd",
    "24.147": "Proteon Inc.",
    "24.146": "Cogent Data Technologies",
    "24.145": "Anritsu Corporation",
    "24.144": "Microcom",
    "20.207357808214016": "Gaodi Rus",
    "24.50476": "Juniper Networks",
    "24.578603": "Samsung Electronics Co.,Ltd",
    "12.123917691740160": "Elektro Adrian",
    "24.8432896": "CNET Technology Inc. (Probably an error, see instead 0080AD)",
    "24.56133": "Thamway Co.,Ltd.",
    "24.59112": "Netzin Technology Corporation,.Ltd.",
    "12.346856353792": "Cast Iron Systems",
    "12.123917685436416": "IEEE Registration Authority",
    "24.5567": "technicob",
    "24.5566": "Iqua Ltd.",
    "24.5565": "Group 4 Technology Ltd",
    "24.5564": "Develco",
    "24.5563": "SMA Solar Technology AG",
    "24.5562": "iba AG",
    "24.5561": "Samsung Electronics Co.,Ltd",
    "24.5560": "Tahoe",
    "24.5271448": "Sehaj Synergy Technologies Private Limited",
    "24.14686680": "Bh Technologies",
    "24.5569": "Sony Interactive Entertainment Inc.",
    "24.5568": "Digital Telemedia Co.,Ltd.",
    "20.75685689294848": "SYS TEC electronic GmbH",
    "24.12846376": "Huawei Technologies Co.,Ltd",
    "24.4494891": "Aidon Oy",
    "20.127937499103232": "Oxon Ag",
    "24.11563711": "Murata Manufacturing Co., Ltd.",
    "12.123917687164928": "DomoSafety S.A.",
    "24.7876833": "UltraClenz, LLC",
    "12.71297884295168": "Grupo Epelsa S.L.",
    "12.346857959424": "Dynalco",
    "24.15483894": "Zyxel Communications Corporation",
    "24.8954462": "Juniper Networks",
    "12.346862153728": "Joinsoon Electronics MFG. Co., Ltd",
    "24.11831553": "Huawei Technologies Co.,Ltd",
    "24.4234189": "D-Link International",
    "12.123917683281920": "Megaflex Oy",
    "24.11548752": "Nanjing CAS-ZDC IOT SYSTEM CO.,LTD",
    "24.13381004": "Lg Electronics Inc",
    "24.9225652": "Sagemcom Broadband SAS",
    "20.162752333086720": "Qingdao Maotran Electronics co., ltd",
    "24.13941734": "Huawei Device Co., Ltd.",
    "24.11803568": "Apple, Inc.",
    "24.8675744": "ARRIS Group, Inc.",
    "24.859": "BridgeWave Communications",
    "24.858": "Photron Limited",
    "20.277981938319360": "T Chip Digital Technology Co.Ltd",
    "24.851": "Mitac, Inc.",
    "24.850": "Colubris Networks",
    "24.853": "TeraBeam Internet Systems",
    "24.852": "Fiber Logic Communications",
    "24.855": "Intervoice-Brite, Inc.",
    "24.854": "Wincor Nixdorf International GmbH",
    "24.857": "DigitalSis",
    "24.856": "Hanyang Digitech Co.Ltd",
    "24.16553187": "Technicolor CH USA Inc.",
    "12.123917691064320": "IoT Routers Limited",
    "24.9230806": "Xiaomi Communications Co Ltd",
    "24.5002680": "Turkcell Teknoloji",
    "24.11544104": "Broadcom Limited",
    "24.11571490": "Texas Instruments",
    "20.71414653648896": "Johnson Matthey",
    "24.10809170": "Telewave Co., Ltd.",
    "12.71297884741632": "Tokhatec",
    "12.123917691367424": "EvoLogics GmbH",
    "24.7875053": "Askey Computer Corp",
    "12.123917686419456": "Zhejiang Wellsun Electric Meter Co.,Ltd",
    "24.2906191": "ARRIS Group, Inc.",
    "24.4239412": "Hewlett Packard",
    "20.162752339378176": "ZIGPOS GmbH",
    "12.346854789120": "Cabtronix AG",
    "20.31477540585472": "Henrich Electronics Corporation",
    "20.137215122341888": "Shanghai Institute of Applied Physics, Chinese Academy of Sciences",
    "24.1857496": "Kraftway Corporation PLC",
    "12.346858266624": "DA-Design Oy",
    "12.346855854080": "SUNGJIN NEOTECH Co.Ltd.",
    "24.7877057": "Apple, Inc.",
    "12.346860322816": "Fr. Sauter AG",
    "12.123917683560448": "Guan Show Technologe Co., Ltd.",
    "12.119269462016": "Euresys S.A.",
    "12.346862333952": "Kahler Automation",
    "12.123917682483200": "Uman",
    "24.6458": "Oesolutions",
    "24.15736252": "Microsoft Corporation",
    "24.3732287": "Tatsuno Corporation",
    "12.123917681569792": "Marques,S.A.",
    "24.8651395": "HUMAX Co., Ltd.",
    "12.123917679108096": "Dave Srl",
    "24.2419264": "Helmholz GmbH & Co. KG",
    "24.15245113": "Nokia",
    "24.6459": "LigoWave",
    "24.10775793": "Samsung Electronics Co.,Ltd",
    "24.6830622": "Cisco Meraki",
    "20.97755448999936": "Xiamen Cacamle Technology Co.,Ltd.",
    "12.346859638784": "Spectrum Communications FZE",
    "24.3981152": "Apple, Inc.",
    "24.7659958": "Apple, Inc.",
    "24.15480975": "Huawei Technologies Co.,Ltd",
    "12.123917691011072": "Magneti Marelli S.p.A. Electronics",
    "24.7901758": "Nokia Corporation",
    "24.2421072": "Guangzhou Qi'an Technology Co., Ltd.",
    "20.124214338125824": "Eclipse Security",
    "24.9210555": "Beijing Orient View Technology Co., Ltd.",
    "24.2056": "PPT Vision, Inc.",
    "24.3466112": "Genexis B.V.",
    "24.7688": "Centec Networks Inc",
    "24.7689": "ZEFATEK Co.,LTD",
    "12.123917683298304": "Mecsel Oy",
    "24.7684": "Hanson Research Corporation",
    "24.7685": "Xseed Technologies & Computing",
    "24.7686": "Wibrain",
    "24.7687": "Winy Technology Co., Ltd.",
    "24.7680": "Shantou Institute of Ultrasonic Instruments",
    "24.7681": "Renesas Technology Sales Co., Ltd.",
    "24.7682": "Sougou Keikaku Kougyou Co.,Ltd.",
    "24.7683": "LiComm Co., Ltd.",
    "24.5540538": "Cisco Systems, Inc",
    "24.13128779": "Zyxel Communications Corporation",
    "12.123917690712064": "PXM sp.k.",
    "24.11272906": "HI Solutions, Inc.",
    "24.2057": "Systemonic AG",
    "12.346862800896": "Saia-Burgess Controls AG",
    "24.11824199": "Panasonic Appliances Company",
    "24.42818": "Cisco Systems, Inc",
    "12.346860896256": "Ratioplast-Optoelectronics GmbH",
    "24.7879276": "Tecno Mobile Limited",
    "24.8176326": "TP-Link Corporation Limited",
    "24.268502": "Ubiquiti Networks Inc.",
    "24.4510046": "Shanghai Kingto Information Technology Ltd",
    "24.16566980": "LifeHealth, LLC",
    "12.346868715520": "Mehta Tech, Inc.",
    "20.268902723485696": "Avilution",
    "24.326275": "Qingdao Haier Technology Co.,Ltd",
    "20.124214341271552": "Photonfocus AG",
    "12.346867163136": "Institute For Information Industry",
    "12.123917677920256": "Slat",
    "24.2302": "UNIK C&C Co.,Ltd.",
    "24.2303": "Trilogy Communications Ltd",
    "24.2300": "Gigaphoton Inc.",
    "24.2301": "BlueKorea Co., Ltd.",
    "24.2306": "Redline Communications Inc.",
    "24.2307": "Panasas, Inc",
    "24.2304": "Tmt",
    "24.2305": "Shenzhen Shixuntong Information & Technoligy Co",
    "24.6489": "Staccato Communications Inc.",
    "24.6488": "Bluetooth SIG, Inc.",
    "24.2308": "MONDIAL electronic",
    "24.2309": "iTEC Technologies Ltd.",
    "24.5570434": "Davit Solution co.",
    "24.1651": "TKH Security Solutions USA",
    "24.1650": "Netezza",
    "24.1653": "Banderacom, Inc.",
    "24.1652": "Spectrum Control, Inc.",
    "24.1655": "Sick Ag",
    "24.1654": "Novra Technologies Inc.",
    "24.1657": "Konami Corporation",
    "24.1656": "D&M Holdings Inc.",
    "24.1659": "Toplink C&C Corporation",
    "24.1658": "JMP Systems",
    "24.12893399": "New Ryatek",
    "24.6046782": "Skullcandy",
    "24.2686336": "IEEE Registration Authority",
    "24.10266269": "Whaley Technology Co.Ltd",
    "12.123917689176064": "Sentry360",
    "24.6159": "Nokia Danmark A/S",
    "24.6158": "Avega Systems",
    "24.4199140": "Compass-EOS",
    "24.6153": "Cresyn",
    "24.6152": "SightLogix, Inc.",
    "24.6151": "Fanstel Corp.",
    "24.6150": "Hokkei Industries Co., Ltd.",
    "24.6157": "Terabytes Server Storage Tech Corp",
    "12.346867159040": "Pcsc",
    "24.6155": "Brilliant Telecommunications",
    "24.6154": "Cisco Meraki",
    "24.12103196": "Smart Cube., Ltd",
    "20.268902726631424": "Beijing Penslink Co., Ltd.",
    "12.119269883904": "Silvair",
    "24.9701459": "Liteon Technology Corporation",
    "24.1059161": "Apple, Inc.",
    "20.14223574827008": "Lumel S.A.",
    "24.48442": "Nokia Corporation",
    "20.88914844975104": "Shenzhen Vipstech Co., Ltd",
    "24.9084": "Neotion",
    "24.9085": "Hewlett Packard",
    "24.37079": "NetBoost Corp.",
    "24.37078": "Crystal Group, Inc.",
    "24.9080": "GN Netcom A/S",
    "24.9081": "Union Business Machines Co. Ltd.",
    "24.9082": "Rim",
    "24.9083": "Whdi Llc",
    "24.37073": "Leichu Enterprise Co., Ltd.",
    "24.37072": "Thomson Telecom Belgium",
    "24.37071": "Nortel",
    "24.37070": "avateramedical Mechatronics GmbH",
    "24.9088": "Nanoteq",
    "24.37076": "BindView Development Corp.",
    "24.37075": "GIESECKE & DEVRIENT GmbH",
    "24.37074": "Artel Video Systems",
    "24.1371805": "AirNav Systems LLC",
    "24.5251564": "Huawei Device Co., Ltd.",
    "24.10224542": "Beijing Winchannel Software Technology Co., Ltd",
    "24.12063763": "Keen High Holding(HK) Ltd.",
    "24.7118061": "Cisco Systems, Inc",
    "12.346868207616": "Knowhow Infocom Inc.",
    "12.346857598976": "Gesellschaft für Rationalisierung und Rechentechnik mbH",
    "24.14734436": "Intel Corporate",
    "12.123917691662336": "CSM MACHINERY srl",
    "20.224420100571136": "IC RealTech",
    "24.1841933": "Giga-Byte Technology Co.,Ltd.",
    "24.14734434": "Huawei Device Co., Ltd.",
    "24.9338": "FU YI CHENG Technology Co., Ltd.",
    "24.9339": "Actiontec Electronics, Inc",
    "24.9336": "Mag Tech Electronics Co Limited",
    "24.9337": "Optec Displays, Inc.",
    "24.9334": "TAP.tv",
    "24.9335": "Tibbo Technology",
    "24.9332": "Autronica Fire And Securirty",
    "24.9333": "Compass System(Embedded Dept.)",
    "24.9330": "ReDriven Power Inc.",
    "24.9331": "3Com Europe Ltd",
    "24.2112686": "Apple, Inc.",
    "20.220515727835136": "AP Sensing GmbH",
    "12.346863009792": "Epl",
    "12.346859114496": "Pricol Limited",
    "24.14963944": "Dell Inc.",
    "24.7898172": "Sony Corporation",
    "24.2378625": "Samsung Electronics Co.,Ltd",
    "24.4749909": "ALE International",
    "24.9470795": "Beijing Yunyi Times Technology Co,.Ltd",
    "20.36069841043456": "Shanghai Digicube Info&Tech Co.,Ltd.",
    "24.8152150": "Samsung Electronics Co.,Ltd",
    "24.12894623": "National Oilwell Varco Instrumentation, Monitoring, and Optimization (NOV IMO)",
    "12.346854932480": "Xipher Embedded Networking",
    "24.7124809": "Huawei Technologies Co.,Ltd",
    "12.346867843072": "RO.VE.R. Laboratories S.p.A",
    "24.12362042": "SES-imagotag",
    "24.16009711": "Emstone",
    "24.16540858": "Xiaomi Communications Co Ltd",
    "24.6573110": "Intel Corporate",
    "24.6858977": "Wacom Co.,Ltd.",
    "12.346858205184": "KS System GmbH",
    "24.524378": "IBM Corp",
    "24.524379": "Vta Technologies Inc.",
    "12.346862518272": "DST Swiss AG",
    "24.14216824": "Moog Protokraft",
    "12.346852737024": "IP Unity",
    "24.524370": "Insystec",
    "24.524371": "Middle East Tech. University",
    "24.524376": "Systems Concepts",
    "24.524377": "A/S Mycron",
    "24.524374": "Stanford Linear Accel. Center",
    "24.524375": "Evans & Sutherland",
    "20.224420094279680": "Microtech System,Inc",
    "24.8660249": "Samsung Electronics",
    "24.9440614": "Digimore Electronics Co., Ltd",
    "24.10508019": "Apple, Inc.",
    "12.123917677768704": "ATT Nussbaum Prüftechnik GmbH",
    "24.3980147": "Shenzhen Xinguodu Technology Co., Ltd.",
    "12.123917681283072": "Windar Photonics",
    "20.212188457336832": "Tiandi(Changzhou) Automation Co., Ltd.",
    "24.16472": "Kronos, Inc.",
    "24.16473": "Yoshida Kogyo K. K.",
    "24.16470": "Mcm Japan Ltd.",
    "24.16471": "Lockheed - Sanders",
    "24.16476": "Future Systems, Inc.",
    "24.16477": "Star-Tek, Inc.",
    "24.16474": "Goldstar Information & Comm.",
    "24.16475": "Funasset Limited",
    "24.16478": "North Hills Israel",
    "24.16479": "Afe Computers Ltd.",
    "24.5295843": "Gigafirm.co.LTD",
    "12.346864623616": "Convergent Bioscience Ltd.",
    "24.14183523": "Google, Inc.",
    "24.2101370": "Gemtek Technology Co., Ltd.",
    "24.10508964": "Grand Products Nevada, Inc.",
    "12.123917678960640": "ACD Elekronik GmbH",
    "24.3973223": "Liteon Technology Corporation",
    "24.848710": "Xiaomi Communications Co Ltd",
    "24.11037377": "HanbitEDS Co., Ltd.",
    "24.11037371": "Sagemcom Broadband SAS",
    "20.150262246277120": "Avlink Industrial Co., Ltd",
    "24.2935": "Cogent Systems, Inc.",
    "12.123917677326336": "Sikom AS",
    "12.346852454400": "B. R. Electronics",
    "24.2934": "ET&T Technology Co. Ltd.",
    "12.123917689442304": "Excel Medical Electronics LLC",
    "24.2937": "X-COM, Inc.",
    "24.2936": "Taifatech Inc.",
    "24.13133356": "Beijing Haitai Fangyuan High Technology Co,.Ltd.",
    "24.16539672": "Zhejiang Kangtai Electric Co., Ltd.",
    "24.2931": "Kodeos Communications",
    "24.14167521": "Tricascade Inc.",
    "24.2930": "Lawo AG",
    "24.2933": "Iosoft Ltd.",
    "24.16531465": "zte corporation",
    "24.2932": "Kingwave Technology Co., Ltd.",
    "12.123917687386112": "APG Cash Drawer, LLC",
    "12.123917686640640": "Brayden Automation Corp",
    "24.8165972": "Technicolor CH USA Inc.",
    "24.11022509": "Weifang Goertek Electronics Co.,Ltd",
    "24.7350411": "Samsung Electronics Co.,Ltd",
    "24.14290232": "Farpointe Data, Inc.",
    "24.2939": "Test-Um Inc.",
    "12.123917690388480": "Kentech Instruments Limited",
    "24.827500": "Industrial Cyber Sensing Inc.",
    "24.5010105": "Pegatron Corporation",
    "24.13942655": "Upvel",
    "24.879": "Telsey SPA",
    "12.123917677867008": "Automation Networks & Solutions LLC",
    "12.346861285376": "VTQ Videtronik GmbH",
    "24.44324": "D-Link International",
    "24.136": "Brocade Communications Systems LLC",
    "24.16549270": "Axonne Inc.",
    "24.32096": "Apple, Inc.",
    "20.132869111414784": "Shenzhen Scodeno Technology Co,. Ltd.",
    "12.346864054272": "Phytec Messtechnik GmbH",
    "24.7622009": "Arrive Systems Inc.",
    "24.14163314": "Apple, Inc.",
    "24.6334896": "Merchandising Technologies, Inc",
    "24.398": "Logitec Corporation",
    "24.13913960": "Eidetic Communications Inc",
    "24.48853": "New H3C Technologies Co., Ltd",
    "24.6332702": "Wuhan Maxsine Electric Co.,Ltd.",
    "12.346862452736": "J. DITTRICH ELEKTRONIC GmbH & Co. KG",
    "24.399": "Kenetec, Inc.",
    "12.123917679464448": "Videoport S.A.",
    "24.13136456": "Aplicom Oy",
    "24.7615488": "MTG Co., Ltd.",
    "24.10521621": "Gerhard D. Wempe KG",
    "12.123917685706752": "Sicon srl",
    "24.1342917": "Murata Manufacturing Co., Ltd.",
    "20.198151829979136": "GE Power Management",
    "24.395": "NetLinks Co., Ltd.",
    "12.123917689114624": "Hyundai MNSOFT",
    "12.346859413504": "Firstec Sa",
    "24.6087606": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917680099328": "Jk Device Corporation",
    "24.544447": "Sumitomo Electric Industries, Ltd",
    "24.12887851": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.12592546": "NEC Platforms, Ltd.",
    "24.281168": "Ramaxel Technology (Shenzhen) limited company",
    "12.346858409984": "Genesis inc",
    "24.7906129": "Shenzhen Zhibotong Electronics Co.,Ltd",
    "24.13633424": "Texas Instruments",
    "24.13909963": "ARRIS Group, Inc.",
    "24.8189542": "Tuya Smart Inc.",
    "24.58401": "Sony Interactive Entertainment Inc.",
    "12.71297883635712": "JSC Electrical Equipment Factory",
    "24.16264846": "Nanjing Kechen Electric Co., Ltd.",
    "12.123917683847168": "Llc \"Ntc Actor\"",
    "12.123917680345088": "CRUXELL Corp.",
    "24.3984252": "TCT mobile ltd",
    "24.12867359": "June Life Inc",
    "24.3937787": "Huawei Technologies Co.,Ltd",
    "12.346857463808": "Fibresavers Corporation",
    "24.6558513": "Bosch Car Multimedia (Wuhu) Co. Ltd.",
    "24.13898147": "Samsung Electronics Co.,Ltd",
    "12.123917687443456": "Cdr Srl",
    "24.3944185": "Apple, Inc.",
    "24.13943292": "Nokia Corporation",
    "24.12856016": "Apple, Inc.",
    "12.71297885007872": "shanghai mingding information tech co.Ltd",
    "24.12347164": "Cisco Systems, Inc",
    "24.13423929": "Cisco Systems, Inc",
    "24.37077": "Euphonix, Inc.",
    "24.12118327": "Sonos, Inc.",
    "24.4240589": "Amazon Technologies Inc.",
    "12.123917680586752": "TAALEX Systemtechnik GmbH",
    "24.14729425": "CK Telecom (Shenzhen) Limited",
    "20.127654896336896": "Bach Icon ApS",
    "12.123917676843008": "Sphere of economical technologies Ltd",
    "24.2140709": "Ip-Net Llc",
    "24.1061752": "FLECTRON Co., LTD",
    "24.9212115": "Nokia",
    "12.346858639360": "Procon Electronics",
    "20.193649356832768": "Private",
    "24.2900926": "Sunnovo International Limited",
    "12.123917684924416": "Atonarp Micro-Systems India Pvt. Ltd.",
    "12.346853785600": "Technodrive srl",
    "24.1858059": "Tegile Systems",
    "24.4768": "NeoMeridian Sdn Bhd",
    "24.4769": "BluePacket Communications Co., Ltd.",
    "24.4762": "IRT Electronics Pty Ltd",
    "24.4763": "E2S Electronic Engineering Solutions, S.L.",
    "24.4760": "Mico Electric(Shenzhen) Limited",
    "24.4761": "Ktech Telecommunications Inc",
    "24.4766": "Surf Communications Inc.",
    "24.4767": "RAE Systems",
    "24.4764": "Yulinet",
    "24.4765": "First International Computer do Brasil",
    "24.3239": "Metro (Suzhou) Technologies Co., Ltd.",
    "24.3238": "Mintera Corporation",
    "12.346866966528": "Lo-Q plc",
    "24.3231": "NKE Corporation",
    "24.2406960": "Technical Consumer Products, Inc.",
    "24.3233": "SIGMACOM Co., LTD.",
    "24.3232": "StorCase Technology, Inc.",
    "24.3235": "Rancho Technology, Inc.",
    "24.3234": "Harmonic Video Network",
    "24.3237": "Naman NZ LTd",
    "24.3236": "Prompttec Product Management GmbH",
    "20.70443735187456": "NanJing HuaStart Network Technology Co.,Ltd.",
    "24.584256": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.14179660": "Tp-Link Technologies Co.,Ltd.",
    "24.1094598": "Cisco Systems, Inc",
    "24.7927293": "Huawei Technologies Co.,Ltd",
    "24.10546425": "Cisco Systems, Inc",
    "24.11332736": "ARRIS Group, Inc.",
    "24.4484460": "Samsung Electronics Co.,Ltd",
    "24.8701735": "Gnodal Ltd",
    "20.207147370545152": "VSN Mobil",
    "12.346856042496": "Media Lab., Inc.",
    "12.123917682929664": "SoFiHa",
    "20.105645766344704": "Beijing Kuangshi Technology Co., Ltd",
    "24.8680982": "Addat s.r.o.",
    "24.1632119": "vivo Mobile Communication Co., Ltd.",
    "12.71297883901952": "Nanjing TIANSU Automation Control System Co., Ltd.",
    "24.4229344": "Beijing Ereneben Information Technology Limited Shenzhen Branch",
    "24.7366777": "Saijo Denki International Co., Ltd.",
    "12.346861076480": "Creation Technologies Chicago",
    "20.132777853845504": "KORF Inc.",
    "12.346860834816": "VisualSonics Inc.",
    "12.346859868160": "Sequentric Energy Systems, LLC",
    "24.576485": "R3 - Reliable Realtime Radio Communications GmbH",
    "24.14447139": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.5548500": "Minibar Systems",
    "12.123917691187200": "Smashtag Ltd",
    "20.31477534294016": "Leviathan Solutions Ltd.",
    "24.3713596": "Samsung Electro Mechanics Co., Ltd.",
    "12.123917685854208": "Muuntosähkö Oy - Trafox",
    "12.346858840064": "Cantion A/S",
    "12.346864775168": "CIO Informatique Industrielle",
    "12.123917688565760": "Ellego Powertec Oy",
    "12.123917686210560": "Osr R&D Israel Ltd",
    "24.1053208": "Korins Inc.",
    "24.11064472": "Avira Operations GmbH & Co. KG",
    "24.11033312": "Huawei Device Co., Ltd.",
    "24.3447309": "ZBD Displays Ltd",
    "20.84187706032128": "mtekvision",
    "24.1053202": "Vivo International Corporation Pty Ltd",
    "20.264165330518016": "Powervault Ltd",
    "24.729": "Reliable Controls",
    "24.728": "BRECIS Communications Corporation",
    "20.216602650869760": "Visatech C0., Ltd.",
    "24.723": "NetBotz, Inc.",
    "24.722": "Workstation AG",
    "24.721": "Vivotek, Inc.",
    "24.720": "Comdial Corporation",
    "24.727": "EMPEG Ltd",
    "24.726": "NICE Systems",
    "24.725": "Acr",
    "24.724": "PDA Peripherals, Inc.",
    "24.15752648": "Duracell Powermat",
    "24.2903380": "ASUSTek COMPUTER INC.",
    "24.7448": "Power Innovation GmbH",
    "24.7449": "Arcadyan Technology Corporation",
    "24.49402": "Canary Communications, Inc.",
    "24.7442": "Rohm Co., Ltd.",
    "24.7443": "NextGTV",
    "24.7440": "LightHaus Logic, Inc.",
    "24.7441": "Analogue & Micro Ltd",
    "24.7446": "Sfr",
    "24.7447": "Digital Sky Corporation",
    "24.7444": "Speradtone Information Technology Limited",
    "24.7445": "Shenzhen Dolphin Electronic Co., Ltd",
    "20.229558875848704": "Imin Technology Pte Ltd",
    "24.7358990": "AMPAK Technology,Inc.",
    "12.346856759296": "CT Company",
    "20.255553907458048": "Annapurna labs",
    "12.346865483776": "GEA Farm Technologies GmbH",
    "24.1581694": "Samsung Electronics Co.,Ltd",
    "24.15480198": "Apple, Inc.",
    "24.9473092": "H.K Zongmu Technology Co., Ltd.",
    "24.2662111": "Danfoss Turbocor Compressors, Inc",
    "24.9473091": "Apple, Inc.",
    "12.346864381952": "RaumComputer Entwicklungs- und Vertriebs GmbH",
    "24.1589063": "Samsung Electronics Co.,Ltd",
    "12.71297883328512": "Rosemount Analytical",
    "24.11836674": "Samsung Electronics Co.,Ltd",
    "20.154794723377152": "Strongbyte Solutions Limited",
    "24.9203802": "Intel Corporate",
    "12.123917685813248": "Weltek Technologies Co. Ltd.",
    "24.7081702": "Chengdu Xiyida Electronic Technology Co,.Ltd",
    "24.7875966": "TRENDnet, Inc.",
    "24.9983955": "Hon Hai-Ccpbg Precision Ind.Co.,Ltd.",
    "24.7631536": "Guangzhou Shiyuan Electronics Co., Ltd.",
    "24.16314388": "RackWare Inc.",
    "12.71297884610560": "Prodco International Inc.",
    "24.11316159": "AthenTek Inc.",
    "24.9747030": "Shenzhen Coship Electronics Co., Ltd.",
    "12.346868039680": "CT Company",
    "24.317172": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "24.10005244": "IEEE Registration Authority",
    "24.1363858": "Tp-Link Technologies Co.,Ltd.",
    "24.845605": "Microsoft Corporation",
    "24.1363853": "Ohsung",
    "20.154019624386560": "Nio",
    "24.13663784": "Hewlett Packard",
    "12.123917684371456": "Beeper Communications Ltd.",
    "12.346863071232": "GSS Avionics Limited",
    "24.13389852": "KVH Industries, Inc.",
    "20.176726065807360": "Xiamen Kehua Hengsheng Co.,Ltd",
    "24.11292206": "Shenzhen JingHanDa Electronics Co.Ltd",
    "24.6319": "Samsung Electronics Co.,Ltd",
    "24.6318": "Tvt Co.,Ltd",
    "24.2678": "Beida Jade Bird Huaguang Technology Co.,Ltd",
    "24.2679": "Bluewire Technologies LLC",
    "24.15211096": "Technologic Systems",
    "12.123917684830208": "WiFi Nation Ltd",
    "24.2672": "MPLS Forum",
    "24.2673": "Avrio Technologies, Inc",
    "24.2670": "Harmonic, Inc",
    "24.2671": "ZyFLEX Technologies Inc",
    "24.2676": "Manticom Networks Inc.",
    "24.2677": "Caterpillar, Inc",
    "24.2674": "Stec, Inc.",
    "24.2675": "Scientific Atlanta",
    "24.1329": "Cisco Systems, Inc",
    "24.1328": "Andiamo Systems, Inc.",
    "24.10795951": "Lime Instruments",
    "24.1321": "Shanghai Broadan Communication Technology Co., Ltd",
    "24.1320": "New Focus, Inc.",
    "24.1323": "HORIBA, Ltd.",
    "24.1322": "Ikegami Tsushinki Co., Ltd.",
    "24.1325": "Zoltrix International Limited",
    "24.1324": "Supreme Magic Corporation",
    "24.1327": "Leviton Network Solutions",
    "24.1326": "Cinta Networks",
    "24.14961863": "IEEE Registration Authority",
    "24.811281": "Anda Technologies SAC",
    "24.11574228": "GNH Software India Private Limited",
    "24.1627754": "Inmarsat",
    "24.4746867": "Pica8, Inc.",
    "24.15268960": "ELCOM Innovations Private Limited",
    "24.4746864": "Zhejiang Tmall Technology Co., Ltd.",
    "24.323617": "Compex Systems Pte Ltd",
    "24.1849935": "AccuSpec Electronics, LLC",
    "24.7383116": "MONAD., Inc.",
    "12.346861395968": "Phytec Messtechnik GmbH",
    "24.8681611": "Apple, Inc.",
    "24.11584857": "Samsung Electronics Co.,Ltd",
    "24.4474828": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917689409536": "Translite Global Llc",
    "24.10788169": "Ketra, Inc.",
    "24.8922110": "TTTech Computertechnik AG",
    "20.31372474318848": "Shenzhen Xiaoxi Technology Co., Ltd.",
    "12.346859397120": "Lanetco",
    "24.15778428": "Mianyang Netop Telecom Equipment Co.,Ltd.",
    "12.346856566784": "Idetech Europe S.A.",
    "12.346862768128": "SAMH Engineering Services",
    "12.123917688586240": "Harbin Cheng Tian Technology Development Co., Ltd.",
    "24.15998558": "RtBrick Inc.",
    "24.14438458": "Nanjing Ticom Tech Co., Ltd.",
    "12.123917676883968": "MatchX GmbH",
    "20.13692008660992": "Husty M.Styczen J.Hupert Sp.J.",
    "24.2909043": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.123917687230464": "Alma",
    "24.14447493": "Huawei Device Co., Ltd.",
    "24.3152230": "Samsung Electronics Co.,Ltd",
    "24.815453": "Samsung Electronics Co.,Ltd",
    "20.67047994163200": "Mirico",
    "12.123917679722496": "Emergency Lighting Products Limited",
    "24.6871892": "Phicomm (Shanghai) Co., Ltd.",
    "24.4735834": "Huawei Technologies Co.,Ltd",
    "12.123917676417024": "RCH Vietnam Limited Liability Company",
    "12.123917688475648": "Peter Huber Kaeltemaschinenbau GmbH",
    "20.48543315263488": "Griessbach",
    "24.13661141": "Lifi Labs Management Pty Ltd",
    "12.346855006208": "Softier Inc.",
    "24.4256284": "DASAN Zhone Solutions",
    "24.8954583": "Huawei Technologies Co.,Ltd",
    "12.123917681364992": "Avionica",
    "24.15360118": "CloudSimple, Inc.",
    "24.268": "System Talks Inc.",
    "24.269": "Teledyne DALSA Inc.",
    "24.266": "Cis Technology Inc.",
    "24.267": "Space CyberLink, Inc.",
    "24.264": "AVLAB Technology, Inc.",
    "24.265": "Nagano Japan Radio Co., Ltd.",
    "24.262": "Tews Datentechnik GmbH",
    "24.263": "Leiser GmbH",
    "24.260": "DVICO Co., Ltd.",
    "24.261": "Beckhoff Automation GmbH",
    "24.5152": "G-Links networking company",
    "24.5153": "Total Wireless Technologies Pte. Ltd.",
    "24.5150": "P.A. Semi, Inc.",
    "24.5151": "SunKwang Electronics Co., Ltd",
    "24.5156": "Merry Electrics CO., LTD.",
    "24.5157": "Galactic Computing Corp.",
    "24.5154": "Dell Inc.",
    "24.5155": "J-S Co. NEUROCOM",
    "24.5158": "NL Technology",
    "24.5159": "JazzMutant",
    "24.4201": "Helioss Communications, Inc.",
    "24.4200": "Comos Telecom",
    "24.4203": "Sonus Networks, Inc.",
    "24.4202": "Digital Microwave Corporation",
    "24.4205": "Axxcelera Broadband Wireless",
    "12.346857291776": "Embigence GmbH",
    "24.4207": "Trenton Technology Inc.",
    "24.4206": "Tadiran Com. Ltd.",
    "24.4209": "Advanet Inc.",
    "24.4208": "Caradon Trend Ltd.",
    "24.3189476": "Huawei Device Co., Ltd.",
    "24.9738640": "VTC Telecommunications",
    "12.123917687762944": "YUYAMA MFG Co.,Ltd",
    "24.7899507": "Cmc",
    "24.11058000": "WiZ IoT Company Limited",
    "12.123917688524800": "GridBridge Inc",
    "24.14985398": "Huawei Technologies Co.,Ltd",
    "24.10277682": "PLC Technology Ltd",
    "20.154794720231424": "Beijing KXWELL Technology CO., LTD",
    "12.123917685030912": "Energy Wall",
    "12.123917688668160": "Veeco Instruments",
    "12.71297884291072": "Aplex Technology Inc.",
    "12.123917685407744": "eSoftThings",
    "24.16023865": "Canon Inc.",
    "12.123917687287808": "Innoflight, Inc.",
    "24.10804414": "Huawei Technologies Co.,Ltd",
    "24.14208924": "Apple, Inc.",
    "24.11074264": "Apple, Inc.",
    "12.123917679415296": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "24.4512971": "Semindia Systems Pvt Ltd",
    "24.7922688": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.14191436": "KingTing Tech.",
    "12.346852216832": "Opus Telecom Inc.",
    "24.6093162": "Huawei Technologies Co.,Ltd",
    "24.10780214": "EM Microelectronic",
    "24.10780217": "zte corporation",
    "12.123917678505984": "Movimento Group AB",
    "24.8687616": "Arnold&Richter Cine Technik GmbH & Co. Betriebs KG",
    "24.8416444": "NET New Electronic Technology GmbH",
    "12.123917677436928": "Sierra Nevada Corporation",
    "24.9463580": "Huawei Technologies Co.,Ltd",
    "12.123917675642880": "Cardinal Kinetic",
    "24.8951033": "Gemicom Technology, Inc.",
    "12.123917687652352": "Scanvaegt Systems A/S",
    "24.3195865": "Alpsalpine Co,.Ltd",
    "12.346868277248": "Y-cam Solutions Ltd",
    "20.62070500360192": "Eyesight(Shanghai)Communication Technology Co.,Ltd.",
    "20.176094515822592": "Ordercube GmbH",
    "24.10793449": "Technicolor",
    "12.346863210496": "CA Traffic Ltd",
    "20.18540137545728": "Milesight Taiwan",
    "12.71297883578368": "Tech Source Inc",
    "12.123917680787456": "Sertone, a division of Opti-Knights Ltd",
    "24.35836": "mixi,Inc.",
    "20.216145534648320": "Newland Era Edu Hi-Tech(BeiJing)Co.,Ltd",
    "24.14689279": "Fortinet, Inc.",
    "12.346857062400": "Doramu Co.,Ltd.",
    "24.2420991": "GHT Co., Ltd.",
    "12.123917685923840": "Privafy, Inc",
    "24.8440149": "Redpine Signals, Inc.",
    "24.1333456": "Hewlett Packard",
    "20.66129347215360": "Inter-Coastal Electronics",
    "12.346860630016": "Intelleflex Corporation",
    "24.12900902": "Noblex Sa",
    "12.346854490112": "Schwer+Kopka GmbH",
    "12.71297883992064": "insensiv GmbH",
    "24.13677859": "Bestcare Cloucal Corp.",
    "24.7880469": "EasySYNC Ltd.",
    "24.15505516": "Lufft Mess- und Regeltechnik GmbH",
    "20.123731225608192": "MAX4G, Inc.",
    "24.16580290": "Invensys Controls UK Limited",
    "12.119269343232": "Coolit Systems, Inc.",
    "24.5015389": "Bose Corporation",
    "24.11545155": "Google, Inc.",
    "24.7141756": "Nanoptix Inc.",
    "24.6820": "Future University-Hakodate",
    "12.123917690769408": "Mictrotrac Retsch GmbH",
    "24.820241": "A.O. Smith Water Products",
    "24.16308980": "Taishan Online Technology Co., Ltd.",
    "12.123917689069568": "KnowRoaming",
    "24.7885338": "Samsung Electronics Co.,Ltd",
    "24.6318491": "isepos GmbH",
    "12.123917686624256": "Eta Beta Srl",
    "24.5302641": "Jibo, Inc.",
    "12.123917691174912": "VisioGreen",
    "24.3678663": "Aruba, a Hewlett Packard Enterprise Company",
    "24.7344071": "Shenzhen Ikinloop Technology Co.,Ltd.",
    "24.9518": "Cisco SPVTG",
    "24.9519": "Energy, Inc.",
    "24.9516": "Entourage Systems, Inc.",
    "24.9517": "Kiryung Electronics",
    "24.9514": "Chengdu GeeYa Technology Co.,LTD",
    "24.9515": "Stirling Energy Systems",
    "24.9512": "Daido Signal Co., Ltd.",
    "24.9513": "Comelit Group S.P.A",
    "24.9510": "Genuine Technologies Co., Ltd.",
    "24.9511": "Bitrode Corp.",
    "24.15756664": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346859143168": "Delcan Technologies, Inc",
    "24.10751531": "Facebook Inc",
    "24.8412864": "Yealink(Xiamen) Network Technology Co.,Ltd.",
    "24.9965800": "Apple, Inc.",
    "24.10536329": "Intel Corporate",
    "24.4778595": "Zhejiang Uniview Technologies Co., Ltd.",
    "24.30925": "Ignition Design Labs",
    "24.3954540": "Samsung Electronics Co.,Ltd",
    "24.8152733": "Texas Instruments",
    "24.11838892": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7611067": "Huawei Device Co., Ltd.",
    "24.8440850": "Huawei Device Co., Ltd.",
    "24.5304323": "Open Stack, Inc.",
    "24.7161": "Intellitect Water Ltd",
    "24.7160": "Digitrax Inc.",
    "24.7163": "Alpsalpine Co,.Ltd",
    "24.7162": "G.i.N. mbH",
    "24.7165": "Dignsys Inc.",
    "24.7164": "ASUSTek COMPUTER INC.",
    "24.7167": "Millennia Media inc.",
    "24.7166": "Zavio Inc.",
    "24.7169": "ABB Oy Drives",
    "24.7168": "Entry Point, LLC",
    "24.7881733": "FUJITU(HONG KONG) ELECTRONIC Co.,LTD.",
    "24.2531": "Angel Iglesias S.A.",
    "24.2530": "Sinbon Electronics Co., Ltd.",
    "24.2533": "Hottinger Baldwin Messtechnik GmbH",
    "24.2532": "K Tech Infosystem Inc.",
    "24.2535": "ADC Techonology",
    "24.2534": "Cyber Switching Inc.",
    "24.2537": "Cisco Systems, Inc",
    "24.2536": "Cisco Systems, Inc",
    "24.2539": "HuMANDATA LTD.",
    "24.2538": "YEM Inc.",
    "24.14975692": "Jumptronic GmbH",
    "24.12122243": "Tp-Link Technologies Co.,Ltd.",
    "24.3730712": "Cisco Systems, Inc",
    "12.123917690511360": "Transtronic AB",
    "20.66426717077504": "Beijing Donghua Hongtai Polytron Technologies Inc",
    "24.1578": "Cisco Systems, Inc",
    "12.123917685776384": "Foxconn 4Tech",
    "24.7660867": "Liteon Technology Corporation",
    "24.11581653": "Nanjing Nengrui Auto Equipment CO.,Ltd",
    "24.6308479": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.2677130": "Hangzhou Konke Technology Co.,Ltd.",
    "24.10509799": "DIRECTV, Inc.",
    "24.14733683": "Cisco Systems, Inc",
    "20.277958188072960": "Siren Care(Shanghai) information and technology company",
    "24.8666770": "Huawei Technologies Co.,Ltd",
    "24.12343894": "Cisco Systems, Inc",
    "24.9193826": "Roku, Inc",
    "24.2674440": "Essys",
    "12.123917685792768": "Beijing Arrow SEED Technology Co,.Ltd.",
    "24.12356813": "Samsung Electro-Mechanics(Thailand)",
    "24.8090": "Nortel Networks",
    "12.123917680156672": "Vocality international T/A Cubic",
    "20.140782586560512": "Dongguan I-Chime electrinics Co.,Ltd",
    "24.8093": "Cisco Systems, Inc",
    "24.8176517": "Huawei Technologies Co.,Ltd",
    "24.2127954": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.10800313": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "12.123917678354432": "InnoSenT",
    "24.3673677": "Cisco Systems, Inc",
    "12.123917689724928": "Acouva, Inc.",
    "24.323217": "x-fabric GmbH",
    "24.8153590": "Cisco Systems, Inc",
    "24.8692301": "Birds Eye Systems Private Limited",
    "24.4214326": "HTC Corporation",
    "24.10534736": "S.C. Add-Production S.R.L.",
    "20.181256787591168": "Hydro Electronic Devices, Inc.",
    "24.5032949": "zte corporation",
    "12.123917675679744": "Fuelcellpower",
    "12.346862985216": "The VON Corporation",
    "24.5568102": "Berthold Technologies GmbH and Co.KG",
    "24.2114618": "Schneider Electric Asia Pacific Ltd",
    "12.123917682470912": "Grayshift",
    "24.9721409": "Palo Alto Networks",
    "12.346855821312": "VeroTrak Inc.",
    "24.832335": "Shenzhen Xiaoqi Intelligent Technology Co., Ltd.",
    "20.273272596856832": "Carefree of Colorado",
    "24.2631615": "Samsung Electronics Co.,Ltd",
    "24.1843419": "Huawei Technologies Co.,Ltd",
    "24.2111067": "Megafone Limited",
    "24.8974455": "Cisco Systems, Inc",
    "20.96896787939328": "Conductix-Wampfler",
    "24.536553": "Huawei Technologies Co.,Ltd",
    "20.180577519009792": "The Magstim Company Ltd.",
    "12.123917686919168": "Shandhai Landleaf Architecture Technology Co.,Ltd",
    "24.5258725": "Cisco Systems, Inc",
    "24.13635236": "Porsche Cars North America",
    "24.6091398": "IEEE Registration Authority",
    "24.5033012": "Motorola Solutions Inc.",
    "24.3408862": "Texas Instruments",
    "24.2382412": "Ubiquiti Networks Inc.",
    "24.7346343": "Cisco Systems, Inc",
    "24.12345332": "ASRock Incorporation",
    "12.123917677395968": "Gl Tech Co.,Ltd",
    "12.346855677952": "Advanced Si-Net Co., LTD.",
    "24.13170512": "Dell Inc.",
    "24.2651853": "Funshion Online Technologies Co.,Ltd",
    "20.79215075524608": "Beijing Dragon Resources Limited.",
    "24.11299491": "Shenzhen Kertong Technology Co.,Ltd",
    "12.123917684445184": "Ats-Convers",
    "12.346863951872": "Sage Consultants",
    "12.123917678149632": "Novo DR Ltd.",
    "24.2893783": "Hewlett Packard",
    "24.10225262": "Hytera Communications Corporation Limited",
    "12.346863738880": "Indutherm Giesstechnologie GmbH",
    "24.11551581": "NuLEDs, Inc.",
    "20.277783361093632": "Oberix Group Pty Ltd",
    "24.15498897": "Itel Mobile Limited",
    "20.70682229604352": "RL Controls LLC.",
    "24.3972438": "Huawei Technologies Co.,Ltd",
    "24.6063486": "IEEE Registration Authority",
    "12.71297884209152": "eumig industrie-tv GmbH",
    "24.3973389": "Samsung Electronics Co.,Ltd",
    "12.346857205760": "Ziehl-Abegg AG",
    "24.10020675": "Dell Inc.",
    "24.11279839": "IEEE Registration Authority",
    "24.5786042": "Chitai Electronic Corp.",
    "24.1350114": "Apple, Inc.",
    "24.2891787": "Apple, Inc.",
    "24.8701865": "C3Po S.A.",
    "24.11334341": "Cisco Systems, Inc",
    "24.9989429": "IEEE Registration Authority",
    "12.123917677789184": "Dynamic Perspective GmbH",
    "24.11039876": "zte corporation",
    "24.3732823": "Jolata, Inc.",
    "24.3732820": "Hisense Electric Co.,Ltd",
    "12.71297883185152": "Hangzhou Chenxiao Technologies Co. Ltd.",
    "12.346853826560": "Netsynt Spa",
    "24.13150243": "Samsung Electronics Co.,Ltd",
    "12.346858405888": "TeamProjects BV",
    "24.5771843": "AcSiP Technology Corp.",
    "20.242185276489728": "EK-TEAM Elektronik- u. Kunststoff-Technik GmbH",
    "24.6579975": "Beijing Omnific Technology, Ltd.",
    "24.6867211": "Texas Instruments",
    "24.5017943": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "12.123917690556416": "hangzhou battle link technology Co.,Ltd",
    "12.123917679587328": "Cloud4Wi",
    "12.123917680828416": "Veilux Inc.",
    "24.12892835": "Beijing Winicssec Technologies Co., Ltd.",
    "24.3727786": "zte corporation",
    "24.8708899": "Green Wave Telecommunication SDN BHD",
    "24.4212586": "Astro Gaming",
    "24.13676840": "Samsung Electronics Co.,Ltd",
    "12.123917682077696": "Otto Bihler Maschinenfabrik GmbH & Co. KG",
    "24.5815439": "Cognitive Systems Corp.",
    "24.10223889": "Shenzhen Newabel Electronic Co., Ltd.",
    "12.346854100992": "Edc",
    "20.233258814013440": "POSNET Polska S.A.",
    "24.7099968": "BLU Products Inc",
    "24.9236166": "Precepscion Pty. Ltd.",
    "24.1312520": "Private",
    "24.7919282": "Toshiba",
    "24.16306449": "Tp-Link Technologies Co.,Ltd.",
    "24.15779980": "LeddarTech Inc.",
    "12.346852372480": "Doble Engineering",
    "24.10021989": "Cosesy ApS",
    "24.45287": "British Federal Ltd.",
    "24.12058648": "Htel",
    "12.123917677441024": "Wimate Technology Solutions Private Limited",
    "12.123917685207040": "Jinga-hi, Inc.",
    "20.184991241732096": "Zhejiang Wellsun Intelligent Technology Co.,Ltd.",
    "12.123917687533568": "Cg-Wireless",
    "12.123917675360256": "Rch Italia Spa",
    "12.123917689401344": "Fuhr GmbH Filtertechnik",
    "24.13147053": "Honor Device Co., Ltd.",
    "12.346858598400": "Safeline Sl",
    "24.15775467": "Poslab Technology Co., Ltd.",
    "24.550431": "Palo Alto Networks",
    "12.71297884594176": "Dorlet Sau",
    "24.3692048": "CANDY HOUSE, Inc.",
    "12.123917676740608": "STC \"Rainbow\" Ltd.",
    "24.10808654": "ARRIS Group, Inc.",
    "24.5787878": "Best Buy Corporation",
    "24.15211706": "Fortinet, Inc.",
    "20.140782588657664": "Beijing VControl Technology Co., Ltd.",
    "12.123917678735360": "Sorynorydotcom Inc",
    "24.9755465": "Sitcorp",
    "12.346868641792": "Pounce Consulting",
    "12.346866810880": "Tommotek (WA) Pty Ltd.",
    "24.7352117": "Huawei Technologies Co.,Ltd",
    "24.30002": "Inid Bv",
    "24.15743563": "Bloombase, Inc.",
    "12.346864504832": "Toptechnology SRL",
    "24.3410108": "Apple, Inc.",
    "12.346863198208": "Limab Ab",
    "20.145735979892736": "exodraft a/s",
    "12.71297884909568": "ENTEC Electric & Electronic CO., LTD",
    "12.123917688397824": "MB connect line GmbH Fernwartungssysteme",
    "24.4200663": "Smartronix, Inc.",
    "24.7662138": "Tp-Link Technologies Co.,Ltd.",
    "24.2666493": "Cisco Systems, Inc",
    "24.6843836": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.15737641": "Showa Denki Co.,Ltd.",
    "24.7394169": "Cisco Systems, Inc",
    "24.4987034": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.16265364": "Alcatel-Lucent Telecom Limited",
    "24.6850721": "Universal Electronics, Inc.",
    "24.5310": "Wink communication technology CO.LTD",
    "24.5311": "Cisco-Linksys, LLC",
    "24.5312": "Symstream Technology Group Ltd",
    "24.5313": "U.S. Robotics Corporation",
    "24.5314": "Hewlett Packard",
    "24.5315": "Seagate Technology",
    "12.346862288896": "Domis Sa",
    "24.5317": "Alive Technologies Pty Ltd",
    "24.5318": "Quixant Ltd",
    "24.5319": "Nortel Networks",
    "24.4049": "Applied Wireless Identifications Group, Inc.",
    "24.4048": "Astri",
    "24.4047": "DataWind Research",
    "24.4046": "Kikusui Electronics Corp.",
    "24.4045": "Nortel Networks",
    "24.4044": "ARRIS Group, Inc.",
    "24.4043": "3Com Ltd",
    "24.4042": "A-Jin Techline Co, Ltd",
    "24.4041": "Allnet GmbH",
    "24.4040": "Chantry Networks",
    "24.8948068": "GSI Electronics Inc.",
    "12.346854371328": "GD Mission Systems",
    "24.3681768": "Samsung Electronics Co.,Ltd",
    "24.587420": "Apple, Inc.",
    "20.229108125532160": "Beijing Core Shield Group Co., Ltd.",
    "12.123917691723776": "tiga.eleven GmbH",
    "24.15759848": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5281604": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917688287232": "NOTICE Co., Ltd.",
    "24.13390511": "Universal Global Scientific Industrial Co., Ltd.",
    "12.123917682176000": "MB connect line GmbH Fernwartungssysteme",
    "20.216039227916288": "Awiselink Co., Ltd.",
    "24.4205686": "IEEE Registration Authority",
    "24.5569776": "Samsung Electronics Co.,Ltd",
    "24.7361716": "Vestel Elektronik San ve Tic. A.Ş.",
    "20.268735395921920": "Nantong ZYDZ Electronic.,Co.Ltd",
    "24.2404170": "Alcatel-Lucent IPD",
    "12.346858270720": "Wexiodisk AB",
    "24.4599": "Shenzhen Forward Industry Co., Ltd",
    "24.4598": "Asia Pacific Microsystems , Inc.",
    "24.838412": "iSYS RTS GmbH",
    "24.4591": "Conitec Datensysteme GmbH",
    "24.4590": "Estari, Inc.",
    "24.4593": "QinetiQ Ltd",
    "24.4592": "Wideful Limited",
    "24.4595": "NeoMedia Europe AG",
    "24.4594": "Institute of Network Technologies",
    "24.4597": "Askey Computer Corp",
    "24.4596": "woori-net",
    "24.7120984": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12601856": "Tp-Link Technologies Co.,Ltd.",
    "12.123917684690944": "Elmeasure India Pvt Ltd",
    "12.346868187136": "Phoenix Audio Technologies",
    "24.57419": "JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH",
    "24.57418": "ZX Technologies, Inc",
    "24.57417": "MICROWI ELECTRONIC GmbH",
    "24.57416": "Sdl Communications, Inc.",
    "24.57415": "InFocus Corporation",
    "24.57414": "Bently Nevada Corp.",
    "24.57413": "Touchwave, Inc.",
    "24.57412": "Lsics Corporation",
    "24.57411": "VitalCom",
    "24.57410": "Pacom Systems Ltd.",
    "24.4985262": "Espressif Inc.",
    "24.7121587": "Ruckus Wireless",
    "12.123917687001088": "Optiver Pty Ltd",
    "12.346855030784": "Uster Technologies AG",
    "20.272689490034688": "3Shape Holding A/S",
    "12.346856112128": "Sypris Electronics",
    "24.8929331": "Huawei Technologies Co.,Ltd",
    "24.12346066": "Genexis International B.V.",
    "24.2374": "Cluster Labs GmbH",
    "24.12346062": "Shenzhen Netis Technology Co.,Ltd",
    "12.346857005056": "Armorlink Co.Ltd",
    "12.123917684998144": "Arris",
    "24.7618747": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346867994624": "Netlink Bilisim Sistemleri San. ve Tic. Ltd. Sti.",
    "12.123917687316480": "Sunrise Systems Electronics Co. Inc.",
    "24.3446779": "Advanced Rf Technologies Inc",
    "24.15237336": "GNTEK Electronics Co.,Ltd.",
    "24.6058253": "Cisco Systems, Inc",
    "24.3446774": "ASUSTek COMPUTER INC.",
    "20.167860149157888": "BEAM Authentic",
    "12.123917680934912": "Farmobile, LLC",
    "24.3068": "Cisco Systems, Inc",
    "24.3069": "Cisco Systems, Inc",
    "24.1324742": "Lenovo Mobile Communication Technology Ltd.",
    "20.102196709425152": "Shenzhen VST Automotive Electronics Co., LTD",
    "24.3060": "Private",
    "24.3061": "Shanghai Sibo Telecom Technology Co.,Ltd",
    "24.3062": "Nitgen Co., Ltd",
    "24.3063": "Nidek Co.,Ltd",
    "24.3064": "Infinera",
    "24.3065": "Gemstone Communications, Inc.",
    "24.3066": "Exemys Srl",
    "24.3067": "D-NET International Corporation",
    "12.346858778624": "MA Lighting Technology GmbH",
    "12.346857828352": "Tess GmbH",
    "24.10542597": "JSC \"Irbis-T\"",
    "20.119143420395520": "HTI Co., LTD.",
    "24.20508": "Jatom Systems, Inc.",
    "24.20507": "Abl Canada, Inc.",
    "24.20506": "IQinVision",
    "24.20505": "Spring Tide Networks, Inc.",
    "24.20504": "AMIT, Inc.",
    "24.20503": "Rsr S.R.L.",
    "24.20502": "Molex Canada Ltd",
    "24.20501": "Bright Star Engineering",
    "24.20500": "Cisco Systems, Inc",
    "24.9967340": "IC Intracom",
    "24.2945186": "Apple, Inc.",
    "24.5802586": "Dell Inc.",
    "12.346856148992": "Micro-Measurements",
    "20.145735983038464": "Dunst tronic GmbH",
    "24.7135408": "Sound4",
    "24.4245289": "Sagemcom Broadband SAS",
    "24.11805066": "Dog Hunter LLC",
    "24.12646725": "Toshiba Toko Meter Systems Co., LTD.",
    "24.4993833": "Sweroam",
    "24.3689307": "Ztron Technology Limited",
    "24.4476525": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "20.137149111336960": "Xiamen Mage Information Technology Co.,Ltd.",
    "12.346863190016": "Lino Manfrotto +Co spa",
    "24.8415552": "Lexar Co.,Limited",
    "24.15729006": "Tianyi Telecom Terminals Company Limited",
    "24.7607778": "Tri-Sen Systems Corporation",
    "24.8389276": "Gemtek Technology Co., Ltd.",
    "24.4761063": "Nintendo Co.,Ltd",
    "12.346855469056": "Dornier GmbH",
    "24.8700904": "Vaillant GmbH",
    "20.70682227507200": "Fast Programming",
    "24.8962187": "Apple, Inc.",
    "24.7355395": "RadiAnt Co.,Ltd",
    "24.13139573": "Samsung Electronics Co.,Ltd",
    "12.119269961728": "FSM Solutions Limited",
    "12.123917678600192": "Electrónica Falcón S.A.U",
    "24.4510637": "Shenzhen TINNO Mobile Technology Corp.",
    "24.15998856": "zte corporation",
    "24.12068277": "Trakm8 Ltd",
    "12.123917690400768": "Sicon srl",
    "24.7769": "Silicon Turnkey Express, LLC",
    "24.7768": "D-Link Corporation",
    "24.7767": "ALCOMA, spol. s r.o.",
    "24.7766": "Bally Wulff Entertainment GmbH",
    "24.7765": "COWON SYSTEMS,Inc.",
    "24.7764": "TOYO ELECTRIC Corporation",
    "24.7763": "Further Tech Co., LTD",
    "24.2149742": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.7761": "Converter Industry Srl",
    "24.7760": "Battistoni Research",
    "24.7088985": "Dell Inc.",
    "24.11289842": "Yamaha Corporation",
    "24.4738048": "Network Solutions",
    "20.71070632640512": "FAOD Co.,Ltd.",
    "24.2739": "Fa. GIRA",
    "24.2738": "Fresnel Wireless Systems",
    "24.2737": "GENETEC Corporation",
    "24.2736": "LOYTEC electronics GmbH",
    "24.2735": "Pipal Systems",
    "24.2734": "Rosemount Process Analytical",
    "24.2733": "Stargames Corporation",
    "24.2732": "TerraTec Electronic GmbH",
    "24.2731": "Toyota Technical Development Corporation",
    "24.2730": "AltiGen Communications Inc.",
    "24.9619": "DatNet Informatikai Kft.",
    "24.9618": "Guangzhou Shirui Electronic Co., Ltd",
    "24.9615": "Trident Microsystems, Inc.",
    "24.9614": "The Weather Channel",
    "24.9617": "NEXTEK, Inc.",
    "24.9616": "Super Micro Computer, Inc.",
    "24.9611": "Mellanox Technologies, Inc.",
    "24.9610": "Pole/Zero Corporation",
    "24.9613": "Haier",
    "24.9612": "Esus Elektronik San. Ve Dis. Tic. Ltd. Sti.",
    "24.12599300": "Medicaroid Corporation",
    "24.1428": "HMS Industrial Networks",
    "24.1429": "Alesis Corporation",
    "24.1099603": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "12.71297883860992": "First Design System Inc.",
    "24.1422": "Flextronics International GmbH & Co. Nfg. KG",
    "24.1423": "CLCsoft co.",
    "24.1420": "Opentech Inc.",
    "24.1421": "Lynx Photonic Networks, Inc.",
    "24.1426": "Pultek Corp.",
    "24.1427": "Grammar Engine Inc.",
    "24.1424": "Swissvoice Ltd.",
    "24.1425": "Active Silicon Ltd",
    "24.8443397": "Apple, Inc.",
    "24.11539046": "Futaba-Kikaku",
    "24.12379772": "Cambium Networks Limited",
    "24.5831054": "SECUDOS GmbH",
    "12.123917691424768": "CorDes, LLC",
    "24.9233856": "Samsung Electronics Co.,Ltd",
    "24.2889": "RF-Link System Inc.",
    "24.2888": "sofrel",
    "24.5824876": "Ying Hua Information Technology (Shanghai)Co., LTD",
    "24.2881": "Ing. Büro Dr. Beutlhauser",
    "24.2880": "Cambridge Industries Group (CIG)",
    "24.2883": "Microscan Systems, Inc.",
    "24.2882": "commax Co., Ltd.",
    "24.2885": "Cisco Systems, Inc",
    "24.2884": "Concord IDea Corp.",
    "24.2887": "Advanced Energy",
    "24.2886": "Cisco Systems, Inc",
    "12.346868686848": "Elta",
    "24.13155689": "Samsung Electronics Co.,Ltd",
    "24.14992515": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7371187": "Cisco Systems, Inc",
    "12.346864021504": "Sencon UK Ltd.",
    "20.268902720339968": "ICE Gateway GmbH",
    "24.8951653": "exands",
    "24.10275152": "Baumer Holding",
    "24.5024563": "KST technology",
    "24.7639120": "Renesas Electronics Corporation",
    "12.346864697344": "Monarch Innovative Technologies Pvt Ltd",
    "12.123917684609024": "OutSys",
    "20.127658788651008": "bistos.co.ltd",
    "24.4219626": "China Mobile Group Device Co.,Ltd.",
    "12.123917686476800": "Emka Technologies",
    "24.1288": "Inetcam, Inc.",
    "24.1289": "AVOC Nishimura Ltd.",
    "24.1286": "Reddo Networks AB",
    "24.1287": "Fine Appliance Corp.",
    "24.1284": "Naray Information & Communication Enterprise",
    "24.1285": "Systems Integration Solutions, Inc.",
    "24.1282": "Apple (PCI bus Macs)",
    "24.1283": "Iconag",
    "24.1280": "Cisco Systems, Inc",
    "24.1281": "Cisco Systems, Inc",
    "24.5387": "Sage Infotech Ltd.",
    "24.7347583": "Comtech Technology Co., Ltd.",
    "24.8568": "Matuschek Messtechnik GmbH",
    "24.8569": "IOGEAR, Inc.",
    "24.8564": "AvaLAN Wireless",
    "24.8565": "Pacific Satellite International Ltd.",
    "24.8566": "YMax Telecom Ltd.",
    "24.8567": "W. L. Gore & Associates",
    "24.8560": "Dell Inc.",
    "24.8561": "Wesung TNC Co., Ltd.",
    "24.8562": "Seoultek Valley",
    "24.8563": "Ion Torrent Systems, Inc.",
    "24.545979": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.4490002": "Star-Net",
    "12.346853957632": "DTS, Inc.",
    "20.39672890785792": "Topgolf Sweden AB",
    "24.15744259": "Tecno Mobile Limited",
    "24.5820559": "IEEE 1904.1 Working Group",
    "24.5057": "Asoka USA Corporation",
    "24.5056": "Trix Tecnologia Ltda.",
    "24.5055": "Media System Planning Corp.",
    "24.5054": "Virtual Conexions",
    "24.5053": "Hymatom Sa",
    "24.5052": "Artimi Ltd",
    "24.5051": "Smartvue Corporation",
    "24.5050": "ReadyLinks Inc",
    "12.346859155456": "Softwareentwicklung",
    "24.5059": "Cisco Systems, Inc",
    "24.5058": "WACOM Co.,Ltd",
    "12.123917688532992": "June Automation Singapore Pte. Ltd.",
    "24.4235995": "Amazon Technologies Inc.",
    "12.71297884672000": "GD Mission Systems",
    "12.123917683814400": "Specialized Communications Corp.",
    "20.57176923045888": "Globex 99 LTD",
    "24.6594551": "Kone OYj",
    "24.4902": "ECM Systems Ltd",
    "24.4903": "Data Acquisitions limited",
    "24.4900": "Schneider Electric Ultra Terminal",
    "24.4901": "Cortina Systems Inc",
    "24.4906": "Sitronics Telecom Solutions",
    "24.4907": "Phoenix Digital",
    "24.4904": "Westech Korea Inc.,",
    "24.4905": "VSST Co., LTD",
    "24.4908": "MAZ Brandenburg GmbH",
    "24.4909": "iWise Communications",
    "12.123917677453312": "Telco Antennas Pty Ltd",
    "20.31657719496704": "Insigma Inc",
    "24.57471": "LOGISTISTEM s.r.l.",
    "24.11577077": "Xaptum, Inc.",
    "12.346860531712": "Qtron Pty Ltd",
    "20.49277252403200": "IntelliLUM",
    "24.22059": "Cisco Systems, Inc",
    "24.4463743": "Motorola Mobility LLC, a Lenovo Company",
    "24.9229033": "Yokote Seiko Co., Ltd.",
    "12.71297884057600": "Avant Technologies",
    "24.11015513": "Breathometer, Inc.",
    "24.12364064": "Apple, Inc.",
    "12.71297883709440": "ard sa",
    "24.5518575": "Huawei Technologies Co.,Ltd",
    "24.6333516": "ASUSTek COMPUTER INC.",
    "24.1869485": "Mct Co., Ltd.",
    "24.1360206": "Google, Inc.",
    "24.9502049": "Apple, Inc.",
    "12.123917686272000": "AGR International",
    "12.346854756352": "PATLITE Corporation",
    "24.12361794": "Shanghai Flyco Electrical Appliance Co.,Ltd",
    "24.2155527": "Spark srl",
    "12.346862841856": "DragonFly Scientific LLC",
    "24.5545747": "Apple, Inc.",
    "24.8940259": "Alpha Networks Inc.",
    "12.346863095808": "DTV Innovations",
    "12.346861817856": "Image Video",
    "24.3152791": "Lierda Science & Technology Group Co.,Ltd",
    "24.8434313": "Forworld Electronics Ltd.",
    "24.5567124": "Huawei Device Co., Ltd.",
    "24.36999": "Itis",
    "24.36998": "Cisco Systems, Inc",
    "24.36997": "Golden Enterprises, Inc.",
    "24.36996": "Atech System",
    "24.36995": "Turbo Communication, Inc.",
    "24.36994": "Force Institute",
    "24.36993": "Aloha Networks, Inc.",
    "24.36992": "Not Limited, Inc.",
    "24.36991": "WatchGuard Technologies, Inc.",
    "24.36990": "Vetronix Corp.",
    "20.198151837319168": "Union Tecnologica Noxium S.L.",
    "12.346860871680": "Draco Systems",
    "20.176772037476352": "CoolR Group Inc",
    "24.6078624": "Huawei Technologies Co.,Ltd",
    "24.3718145": "Huawei Technologies Co.,Ltd",
    "12.123917680214016": "Private",
    "24.5303612": "Apple, Inc.",
    "24.15752217": "Aruba, a Hewlett Packard Enterprise Company",
    "20.36069844189184": "Annapurna labs",
    "20.18540143837184": "Eyeball Fintech Company",
    "12.123917675786240": "Den Automation",
    "24.11576039": "Shenzhen TINNO Mobile Technology Corp.",
    "12.123917686280192": "AEM Singapore Pte. Ltd.",
    "24.14220049": "Axacore",
    "24.13689867": "Wearable Inc.",
    "12.123917679804416": "Schildknecht AG",
    "12.123917684633600": "SiFive",
    "24.12364179": "Cambium Networks Limited",
    "24.1617761": "Apple, Inc.",
    "24.6568682": "Cisco Systems, Inc",
    "20.198237816356864": "Shenzhen Huabai Intelligent Technology Co., Ltd.",
    "20.229108121337856": "Huaqin Telecom Hong Kong Ltd",
    "24.8706988": "Huawei Technologies Co.,Ltd",
    "24.36878": "Handlink Technologies, Inc.",
    "20.273458020745216": "Dongwoo Engineering Co.,Ltd",
    "24.36879": "Kawasaki Heavy Industries, Ltd",
    "12.346861133824": "Cardinal Scale Mfg Co",
    "12.346866135040": "Telemerkki Oy",
    "24.14210933": "Sagemcom Broadband SAS",
    "24.6054857": "Intellithings Ltd.",
    "24.7866168": "Z.U.K. Elzab S.A.",
    "24.14479632": "Nokia Corporation",
    "12.123917686366208": "Lemonade Lab Inc",
    "24.2669961": "Google, Inc.",
    "24.294224": "Shenzhen Kang Ying Technology Co.Ltd.",
    "20.172584086798336": "RYEEX Technology Co.,Ltd.",
    "24.15511008": "D-Link International",
    "24.5013690": "Wuhan Tianyu Information Industry Co., Ltd.",
    "24.524358": "Sony Corporation",
    "24.8416663": "Private",
    "24.524359": "Sequent Computer Systems Inc.",
    "24.10274769": "SHARP Corporation",
    "24.16534249": "Castlenet Technology Inc.",
    "24.8941619": "Hangzhou Silan Microelectronic Inc",
    "24.7374529": "Texas Instruments",
    "12.346855665664": "Athena Semiconductor",
    "24.6565303": "Seungil Electronics",
    "24.10488993": "SKTB SKiT",
    "24.15525700": "Omntec mfg. inc",
    "12.346854592512": "DGT Sp. z o.o.",
    "24.3733641": "Huawei Technologies Co.,Ltd",
    "24.15256626": "Micronet LTD",
    "24.7885386": "Ensenso GmbH",
    "20.39919491743744": "Rcc Time Co.,Limited",
    "24.13115736": "Intel Corporate",
    "24.5551955": "Hunan Fenghui Yinjia Science And Technology Co.,Ltd",
    "24.14194528": "C2 Development, Inc.",
    "24.4198021": "Samsung Electro-Mechanics(Thailand)",
    "24.9701687": "HUMAX Co., Ltd.",
    "12.123917675950080": "Pure Air Filtration",
    "12.123917683146752": "Mect Srl",
    "24.2912963": "Soundmatters",
    "24.7637812": "Shanghai Smart System Technology Co., Ltd",
    "24.16528870": "Visteon corp",
    "24.4754531": "Huawei Device Co., Ltd.",
    "12.123917685338112": "CAS Medical Systems, Inc",
    "20.9814036971520": "Jiangsu Logread Network Technology Co., LTD.",
    "24.7667581": "Wren Sound Systems, LLC",
    "12.346858496000": "Gradual Tecnologia Ltda.",
    "12.346853113856": "Audio Design Associates, Inc.",
    "24.8678125": "Wireless Tsukamoto.,co.LTD",
    "20.110162575622144": "Koangyow Integration Machine Co., Ltd.",
    "12.346857164800": "eXray Broadband Inc.",
    "24.15023": "BlueBit Ltd.",
    "24.4258261": "Tecore Networks",
    "24.13400624": "CMAX Wireless Co., Ltd.",
    "24.3677219": "Hilan Technology CO.,LTD",
    "20.71070629494784": "Embrionix Design Inc.",
    "24.3677210": "Cisco Systems, Inc",
    "12.346858860544": "COE Limited",
    "24.9193326": "IEEE Registration Authority",
    "12.123917679058944": "ELOMAC Elektronik GmbH",
    "24.13164779": "Apple, Inc.",
    "24.9963309": "Texas Instruments",
    "20.62636954746880": "Arunas Pty Ltd",
    "12.346855288832": "Arthur Industries Inc., dba On Hold Media Group",
    "12.346868387840": "Tattile srl",
    "12.123917691551744": "Moduware PTY LTD",
    "24.16256352": "Canon Inc.",
    "24.14216421": "Zyxel Communications Corporation",
    "12.123917688868864": "ASHIDA Electronics Pvt. Ltd",
    "12.346867658752": "Liquid Robotics, Inc",
    "24.2642681": "Zhongxin Intelligent Times (Shenzhen) Co., Ltd.",
    "12.346863865856": "Integra LifeSciences (Ireland) Ltd",
    "24.9484669": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "24.16270893": "SYNTEC Incorporation",
    "24.8417200": "Texas Instruments",
    "24.14159769": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8665691": "Inventec(Chongqing) Corporation",
    "20.66426716028928": "Xiamen Smarttek CO., Ltd.",
    "24.2159643": "Shenzhen Bilian electronic CO.,LTD",
    "24.5006760": "IEEE Registration Authority",
    "24.3706962": "Huawei Technologies Co.,Ltd",
    "12.71297883381760": "Bettini Srl",
    "12.123917675036672": "ESYSE GmbH Embedded Systems Engineering",
    "24.16431749": "APAT Optoelectronics Components Co., Ltd.",
    "24.830469": "Unitend Technologies Inc.",
    "24.3707387": "Xenox Holding BV",
    "12.346867748864": "Ebneuro Spa",
    "12.123917682540544": "Videogenix",
    "12.71297883516928": "Amantys Ltd",
    "24.7331": "Terra",
    "24.13650315": "Zoom Server",
    "20.31873101201408": "Meshbox Foundation Pte. Ltd.",
    "24.6047015": "Juniper Networks",
    "12.346854191104": "M2 Technology Pty Ltd",
    "12.123917683654656": "xm",
    "24.9464003": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917689622528": "CoXlab Inc.",
    "24.11287813": "Instorescreen Aisa",
    "24.12336517": "Huawei Technologies Co.,Ltd",
    "24.8698427": "Canon Inc.",
    "24.15734201": "PlayFusion Limited",
    "20.39672882397184": "SMaBiT GmbH",
    "12.346860318720": "Pyrosequencing AB",
    "12.123917689524224": "MaNima Technologies BV",
    "24.8156776": "Huawei Technologies Co.,Ltd",
    "20.88914838683648": "Oceancctv Ltd",
    "24.3699392": "Comtrend Corporation",
    "12.71297884758016": "Georg Neumann GmbH",
    "24.11072102": "Huawei Device Co., Ltd.",
    "12.123917682302976": "Acco Brands Europe",
    "24.2158594": "MTData Developments Pty. Ltd.",
    "20.106196535083008": "Synamedia",
    "24.12884096": "Apple, Inc.",
    "24.5257557": "Cisco SPVTG",
    "24.14958278": "Huawei Technologies Co.,Ltd",
    "24.7365054": "Wistron Neweb Corporation",
    "24.807729": "ETH Zurich",
    "12.123917682581504": "Dolby Laboratories, Inc.",
    "24.9733844": "Surveillance Pro Corporation",
    "24.1101995": "zte corporation",
    "12.346857955328": "Genetix Ltd",
    "24.16015209": "Matsufu Electronics distribution Company",
    "12.346867552256": "Rohde&Schwarz Topex SA",
    "24.10753937": "Shenzhen Decnta Technology Co.,LTD.",
    "24.10753931": "Huawei Technologies Co.,Ltd",
    "12.346856800256": "CAMCO GmbH",
    "12.346852458496": "Cybectec Inc.",
    "24.8420635": "Neosystem Co. Ltd.",
    "24.2742": "Compunetix, Inc",
    "24.3942566": "Alcatel-Lucent Enterprise (China)",
    "24.2388576": "IOTTECH Corp",
    "24.3200601": "Merging Technologies SA",
    "24.12121906": "Aryaka Networks Inc",
    "20.127654885851136": "Symtop Instrument Co.",
    "24.2740": "ETIC Telecommunications",
    "24.6042": "D-Link Corporation",
    "12.123917684019200": "Viotec USA",
    "24.10267757": "Universal Electronics, Inc.",
    "24.2746": "Arcon Technology Limited",
    "24.5679": "Geutebrück GmbH",
    "24.2747": "Taiwan Secom Co,. Ltd",
    "12.123917688860672": "Sicon srl",
    "24.2744": "Cisco Systems, Inc",
    "24.12861954": "Samsung Electronics Co.,Ltd",
    "24.2745": "Astera Technologies Corp.",
    "24.13133662": "Extreme Networks, Inc.",
    "24.16010367": "Huawei Technologies Co.,Ltd",
    "12.123917681790976": "EMAC, Inc.",
    "12.346856673280": "Electro-Matic Products, Inc.",
    "24.5673": "Nivus GmbH",
    "24.10760501": "Apple, Inc.",
    "20.66256289923072": "snap40 Ltd",
    "24.8136154": "Fn-Link Technology Limited",
    "24.8935197": "Seongji Industry Company",
    "24.13649628": "Southwest Research Institute",
    "12.123917675511808": "Isac Srl",
    "24.8652538": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.32936": "Vitacom Corporation",
    "12.123917677015040": "Xavant",
    "24.7082965": "Cisco Systems, Inc",
    "12.123917689888768": "Herutu Electronics Corporation",
    "12.71297884991488": "Micromega Dynamics SA",
    "24.12084869": "Sagemcom Broadband SAS",
    "24.8945189": "Extreme Networks, Inc.",
    "24.1213": "ARRIS Group, Inc.",
    "24.9479599": "Huawei Technologies Co.,Ltd",
    "20.119143425638400": "IskraUralTEL",
    "24.4986019": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "12.123917688725504": "Monnit Corporation",
    "24.1215": "VersaLogic Corp.",
    "24.45128": "Marconi Communications Inc.",
    "24.7117035": "Riverbed Technology, Inc.",
    "24.5812436": "ZuniData Systems Inc.",
    "20.66256293068800": "Robox Smart Motion (Wuhu) Co.,Ltd",
    "24.11327958": "tci GmbH",
    "24.1217": "Cisco Systems, Inc",
    "24.44831": "Cisco Systems, Inc",
    "20.84197037309952": "Sercomm Corporation.",
    "20.145727917391872": "Shinkawa Ltd.",
    "24.9719628": "Sound United LLC",
    "12.346855063552": "Invensys Energy Systens (NZ) Limited",
    "24.2916453": "HARTING Inc. of North America",
    "24.15734176": "KyungDong One Co., Ltd.",
    "24.10276226": "Cheng Uei Precision Industry Co.,Ltd",
    "24.9240030": "Sagemcom Broadband SAS",
    "20.167594813292544": "Pdahl",
    "12.123917681020928": "nexgenwave",
    "20.154019639066624": "Electrical & Automation Larsen & Toubro Limited",
    "12.346859139072": "Hoer GmbH & Co. Industrie-Electronic KG",
    "12.346864214016": "Naretrends",
    "24.5306146": "Cisco Systems, Inc",
    "24.12104440": "Apple, Inc.",
    "24.4901438": "Zeus",
    "12.123917681147904": "FactoryLab B.V.",
    "24.547802": "CliniCare LTD",
    "24.9234355": "Sonardyne International Ltd",
    "24.9975277": "Huawei Technologies Co.,Ltd",
    "24.1880212": "HTC Corporation",
    "12.123917676691456": "Kubu, Inc.",
    "20.176069208440832": "IST ElektronikgesmbH",
    "24.10255343": "Hewlett Packard",
    "20.35758076329984": "Great Lite International",
    "12.346865168384": "Practical Control Ltd",
    "24.14185621": "zte corporation",
    "24.11312004": "Smak Tecnologia e Automacao",
    "24.2679583": "Xiaomi Communications Co Ltd",
    "24.16548": "Rose Electronics",
    "24.16549": "Clinicomp Intl.",
    "24.16546": "Kingstar Technology Inc.",
    "24.16547": "Microunity Systems Engineering",
    "24.16544": "Goldstar Co., Ltd.",
    "24.16545": "Ergo Computing",
    "24.16542": "Concurrent Technologies Ltd.",
    "24.16543": "Telco Systems, Inc.",
    "24.16540": "Transware",
    "24.16541": "DigiBoard",
    "24.13682231": "Intel Corporate",
    "12.119269822464": "3green ApS",
    "20.39672883445760": "Nile Global Inc",
    "20.145383537770496": "Cheng Du virtual world Technology Limited.",
    "12.123917689094144": "Rational Production srl Unipersonale",
    "12.346862223360": "Phytec Messtechnik GmbH",
    "24.11320370": "Apple, Inc.",
    "24.10250331": "Octonion SA",
    "24.50164": "Samsung Electronics Co.,Ltd",
    "24.9500228": "Huawei Device Co., Ltd.",
    "12.123917683036160": "adidas AG",
    "24.13684696": "Huizhou Boshijie Technology Co.,Ltd",
    "24.13384289": "Samsung Electro Mechanics Co., Ltd.",
    "24.2904201": "Shenzhen Kaixuan Visual Technology Co.,Limited",
    "20.149935437643776": "Creative Sensor Inc.",
    "24.8162250": "Tp-Link Technologies Co.,Ltd.",
    "12.346852904960": "Extreme Copper, Inc.",
    "24.8145387": "XIAOMI Electronics,CO.,LTD",
    "24.5295": "Datasym POS Inc.",
    "24.5294": "Wizlogics Co., Ltd.",
    "24.5297": "Axell Wireless Limited",
    "24.5296": "Naeil Community",
    "24.5291": "Senhai Electronic Technology Co., Ltd.",
    "24.5290": "Ashly Audio, Inc.",
    "24.5293": "Gassner Wiege- und Meßtechnik GmbH",
    "24.5292": "Bountiful WiFi",
    "24.16025457": "Infoblox",
    "24.5299": "CoreStar International Corp",
    "24.5298": "mCubelogics Corporation",
    "12.346855702528": "Digital Receiver Technology",
    "24.16006813": "Benu Networks, Inc.",
    "24.16310184": "zte corporation",
    "12.123917683552256": "Metromatics Pty Ltd",
    "12.123917680476160": "sydetion UG (h.b.)",
    "24.15734827": "LG Chem",
    "12.123917690114048": "Fire4 Systems UK Ltd",
    "24.8432812": "Juniper Networks",
    "24.41710": "Cisco Systems, Inc",
    "24.41717": "Guangzhou Yuanyun Network Technology Co.,Ltd",
    "24.7924537": "Instituto Nacional de Tecnología Industrial",
    "24.13903243": "Intel Corporate",
    "12.123917680107520": "Oso Technologies, Inc.",
    "20.105645773684736": "Remowireless Communication International Co.,Limited",
    "24.13147356": "Elitegroup Computer Systems Co.,Ltd.",
    "24.16294580": "RHEWA-WAAGENFABRIK August Freudewald GmbH &amp;Co. KG",
    "24.7642194": "Huizhou Desay SV Automotive Co., Ltd.",
    "24.14724094": "Orb Networks, Inc.",
    "24.8165917": "Apple, Inc.",
    "24.7882525": "Samsung Electronics Co.,Ltd",
    "12.346867359744": "Swareflex GmbH",
    "24.3680602": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917675991040": "Beijing One City Science & Technology Co., LTD",
    "20.52821120516096": "Advanced Electronic Designs, Inc.",
    "24.12902780": "U2S co.",
    "24.11294581": "HMD Global Oy",
    "24.1112352": "Cisco Systems, Inc",
    "20.124214343368704": "System-on-Chip engineering",
    "20.176094528405504": "Audix",
    "20.119709259268096": "Beijing Fimi Technology Co., Ltd.",
    "20.162752347766784": "BAE Systems",
    "24.5010567": "Leader Phone Communication Technology Co., Ltd.",
    "24.7123150": "Netgear",
    "24.12593081": "Beijing National Railway Research & Design Institute of Signal & Communication Co., Ltd.",
    "20.176032231456768": "Adomi",
    "20.48835653009408": "Sensity Systems",
    "24.13131776": "ASUSTek COMPUTER INC.",
    "24.16300002": "Cisco Systems, Inc",
    "24.9709836": "Shenyang wisdom Foundation Technology Development Co., Ltd.",
    "20.101728421675008": "Beijing HZFD Technology Co., Ltd",
    "12.123917687783424": "Polskie Sady Nowe Podole Sp. z o.o.",
    "24.1854199": "Amon Inc",
    "20.163808572080128": "Solaborate Inc.",
    "24.14219439": "Daontec",
    "24.3959463": "Intel Corporate",
    "24.585531": "MCS Logic Inc.",
    "24.9457754": "unGlue, Inc",
    "12.123917690331136": "MSB Elektronik und Gerätebau GmbH",
    "24.4984302": "Jabil Circuit (Shanghai) Ltd.",
    "24.15769187": "Cisco Systems, Inc",
    "12.123917675687936": "Traffic and Parking Control Co, Inc.",
    "24.8439906": "Openpeak, Inc",
    "12.123917685432320": "Acorde Technologies",
    "24.5270718": "InnosiliconTechnology Ltd",
    "24.14428212": "HAKKO Corporation",
    "24.16282110": "The Goldman Sachs Group, Inc.",
    "12.346866184192": "Embedded Solution Bank Co., Ltd.",
    "24.8686733": "Apple, Inc.",
    "24.11804957": "Beijing GuangXin Technology Co., Ltd",
    "24.4503230": "Cisco Systems, Inc",
    "24.4658": "LeWiz Communications Inc.",
    "24.4659": "JRC TOKKI Co.,Ltd.",
    "24.4656": "Picaso Infocommunication CO., LTD.",
    "24.4657": "Motion Control Systems, Inc.",
    "24.4654": "Signal Technology - AISD",
    "24.4655": "Sanei Electric Inc.",
    "24.4652": "Soenen Controls N.V.",
    "24.4653": "SiNett Corporation",
    "24.4650": "VTech Telecommunications Ltd.",
    "24.4651": "Virbiage Pty Ltd",
    "12.346854010880": "TT electronics plc",
    "24.960": "RFTNC Co., Ltd.",
    "24.3929": "Phonak AG",
    "24.3928": "Adder Technology Limited",
    "24.57598": "Cisco Systems, Inc",
    "24.57599": "SECURITY DYNAMICS TECHNOLOGIES, Inc.",
    "24.2628356": "Zalliant LLC",
    "24.9962200": "IEEE Registration Authority",
    "24.3921": "Azul Systems, Inc.",
    "24.3920": "StreamScale Limited",
    "24.3923": "Solarflare Communications Inc.",
    "24.3922": "YORK Refrigeration, Marine & Controls",
    "24.57596": "Huawei Technologies Co.,Ltd",
    "24.3924": "Entrelogic Corporation",
    "24.3927": "CABLELOGIC Co., Ltd.",
    "24.57595": "Leightronix, Inc.",
    "24.298006": "Huawei Technologies Co.,Ltd",
    "12.123917675835392": "Nireco Corporation",
    "20.58071051141120": "NumberFour AG",
    "24.8169392": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.16018135": "Microsoft Corporation",
    "24.13129315": "Sunflex Europe GmbH",
    "24.7087051": "Wattty Corporation",
    "24.12070859": "CMS Electracom",
    "24.14472490": "Sunny Heart Limited",
    "24.6598822": "Apple, Inc.",
    "24.7127367": "Xiamen Hanin Electronic Technology Co., Ltd",
    "24.1326783": "zte corporation",
    "24.962": "Solphone K.K.",
    "24.12597559": "Gree Electric Appliances, Inc. Of Zhuhai",
    "12.346858516480": "Kommunikations- & Sicherheitssysteme Gesellschaft m.b.H",
    "20.171812087398400": "Optris GmbH",
    "12.346867720192": "Cat AB",
    "12.123917691240448": "Medicomp, Inc",
    "12.346856796160": "EASYTECH GmbH",
    "12.123917686493184": "Net And Print Inc.",
    "24.6073420": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.71297884680192": "BrightLeaf Power",
    "24.7358": "Nintendo Co., Ltd.",
    "24.7359": "Intel Corporate",
    "24.7352": "CBC Co., Ltd",
    "24.7353": "Kwang Sung Electronics Co., Ltd.",
    "24.7350": "Duzon CNT Co., Ltd.",
    "24.7351": "USC DigiArk Corporation",
    "24.7356": "CastGrabber, LLC",
    "24.7357": "Ezze Mobile Tech., Inc.",
    "24.7354": "VerScient, Inc.",
    "24.7355": "MusicianLink",
    "24.15227888": "Imaging Diagnostics",
    "24.12874238": "Cisco Systems, Inc",
    "24.11307932": "Primera Technology, Inc.",
    "24.964": "Tomra Systems ASA",
    "24.1324467": "Future Designs, Inc.",
    "24.322867": "Intel Corporate",
    "24.16031731": "Huawei Technologies Co.,Ltd",
    "24.1362592": "Hu&Co",
    "12.119269376000": "Annecy Electronique Sas",
    "12.346857308160": "SATEL sp. z o.o.",
    "24.11550857": "Senient Systems LTD",
    "24.8836": "Desay A&V Science And Technology Co.,Ltd",
    "24.8837": "Nomus Comm Systems",
    "24.8834": "8086 Consultancy",
    "24.8835": "Juniper Networks",
    "24.8832": "A2B Electronics AB",
    "24.8833": "Daintree Networks Pty",
    "24.8830": "Chengdu 30Kaitian Communication Industry Co.Ltd",
    "24.8831": "Ruckus Wireless",
    "24.13122967": "Nokia Corporation",
    "24.12367954": "Cybera, Inc.",
    "24.8838": "Astron",
    "24.8839": "Titan Wireless LLC",
    "24.53459": "Cisco Systems, Inc",
    "24.53458": "Epilog Corporation",
    "24.53457": "Sycamore Networks",
    "24.53456": "Zhongxing Telecom Ltd.",
    "24.53455": "Moreton Bay",
    "24.53454": "iSystem Labs",
    "24.53453": "Atan Technology Inc.",
    "24.53452": "Technologies Lyre Inc.",
    "24.53451": "Dasan Co., Ltd.",
    "24.53450": "Intrinsyc Software International Inc.",
    "12.123917688815616": "microWerk GmbH",
    "24.12082442": "Apple, Inc.",
    "24.7370288": "ARRIS Group, Inc.",
    "24.14954530": "Hanwha Techwin Security Vietnam",
    "24.11059434": "Pepwave Limited",
    "12.346866266112": "Saia-Burgess Controls AG",
    "24.4511794": "AzureWave Technology Inc.",
    "24.9200674": "Sony Mobile Communications Inc",
    "12.346865160192": "Micro I/O Servicos de Electronica, Lda",
    "12.119269478400": "rioxo GmbH",
    "12.346852990976": "J D Richards",
    "24.7640125": "AgJunction",
    "24.4491043": "Hoya Service Corporation",
    "24.12591696": "Koss Corporation",
    "24.4739525": "Intel Corporate",
    "24.9229218": "VisSim AS",
    "24.7348051": "Cisco Systems, Inc",
    "12.346860572672": "MultimediaLED",
    "24.6408": "Duaxes Corporation",
    "24.2388": "AMiT spol. s. r. o.",
    "24.2389": "Young Generation International Corp.",
    "24.2382": "Bartech Systems International, Inc",
    "24.2383": "elmegt GmbH & Co. KG",
    "24.2380": "Communication Weaver Co.,Ltd.",
    "24.2381": "Braintree Communications Pty Ltd",
    "24.2386": "Auerswald GmbH & Co. KG",
    "24.2387": "Linkage System Integration Co.Ltd.",
    "24.2384": "Independent Storage Corporation",
    "24.2385": "Apogee Imaging Systems",
    "24.7371881": "Avaya Inc",
    "12.346855604224": "KoolSpan, Inc.",
    "12.123917683703808": "CoreKinect",
    "24.821895": "AgLogica Holdings, Inc",
    "24.4774142": "ClarIDy Solutions, Inc.",
    "12.119269613568": "Ecomed-Complex",
    "12.123917677682688": "Tex Computer Srl",
    "24.6065453": "Shanghai Wellpay Information Technology Co., Ltd",
    "24.3442781": "Logitech Far East",
    "24.24778": "Harmonic Systems Incorporated",
    "24.24779": "Hitachi Zosen Corporation",
    "24.24774": "Dcs Ag",
    "24.24775": "Amati Communications Corp.",
    "24.24776": "Kuka Welding Systems & Robots",
    "24.24777": "ControlNet, Inc.",
    "24.24770": "Mpl Ag",
    "24.24771": "Netvision Corporation",
    "24.24772": "Soliton Systems K.K.",
    "24.24773": "Ancot Corp.",
    "24.11821196": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.281839": "Fujian Sanao Technology Co.,Ltd",
    "12.346859122688": "CreaTech Electronics Co.",
    "24.699": "Continuous Computing Corp",
    "24.698": "Cisco Systems, Inc",
    "24.697": "Cisco Systems, Inc",
    "24.696": "Whi Konsult Ab",
    "24.695": "Watanabe Electric Industry Co., Ltd.",
    "24.694": "Acrosser Technology Co., Ltd.",
    "24.693": "Avnet, Inc.",
    "24.692": "Daphne",
    "24.691": "Intel Corporation",
    "24.690": "Cablevision",
    "24.2903238": "Murata Manufacturing Co., Ltd.",
    "24.14419905": "HangZhou QiYang Technology Co.,Ltd.",
    "24.10548989": "Texas Instruments",
    "24.73": "Apricot Computers, Ltd",
    "12.346854658048": "Tattile Srl",
    "12.346864181248": "ITF Fröschl GmbH",
    "12.71297884807168": "Kerun Visual Technology Co., Ltd.(Shenzhen)",
    "24.1310845": "zte corporation",
    "24.15269264": "Turbostor",
    "24.16578743": "TRONTEQ Electronic",
    "24.16033805": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "24.13423777": "MitraStar Technology Corp.",
    "12.346863673344": "Absolute Fire Solutions Inc.",
    "24.2968": "NiceTechVision",
    "24.2969": "SensAble Technologies, Inc.",
    "24.2966": "Innotrac Diagnostics Oy",
    "24.2967": "Matsushita Electric Industrial Co.,Ltd.",
    "24.2964": "Digital Monitoring Products, Inc.",
    "24.2965": "eBet Gaming Systems Pty Ltd",
    "24.2962": "Ascom Danmark A/S",
    "24.2963": "Ritter Elektronik",
    "24.2960": "ADVA Optical Networking Ltd.",
    "24.2961": "Aglaia Gesellschaft für Bildverarbeitung und Kommunikation mbH",
    "24.349": "Oracle Corporation",
    "24.348": "Cadant Inc.",
    "24.5244420": "Cisco Systems, Inc",
    "24.9008": "Dizipia, Inc.",
    "24.9009": "Nintendo Co., Ltd.",
    "24.14426460": "Intel Corporate",
    "24.341": "Promise Technology, Inc.",
    "24.340": "G3M Corporation",
    "24.343": "Syswave Co., Ltd",
    "24.342": "Firewiredirect.Com, Inc.",
    "24.345": "S1 Corporation",
    "24.344": "Electro Industries/Gauge Tech",
    "24.347": "ITALTEL S.p.A/RF-UP-I",
    "24.346": "Digital Video Broadcasting",
    "12.346863165440": "Densitron Technologies Ltd",
    "24.12886353": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.2927524": "Cisco SPVTG",
    "20.93065832300544": "Nederman Holding AB",
    "12.346854617088": "Digicast Networks",
    "20.136646475382784": "Dspread Technology (Beijing) Inc.",
    "12.123917686800384": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "24.532467": "Cisco Systems, Inc",
    "24.10793529": "Cisco Systems, Inc",
    "24.3631": "Roche Diagnostics GmbH",
    "12.123917679902720": "Microsoft Research",
    "12.123917684428800": "sonatest",
    "24.12076004": "Vlatacom",
    "20.141268006993920": "Quickte Technology Co.,Ltd",
    "24.11337196": "Apple, Inc.",
    "12.123917686652928": "Serveron / Qualitrol",
    "24.2634768": "Enigma Diagnostics Limited",
    "24.13150070": "Huawei Technologies Co.,Ltd",
    "12.346860011520": "Atec Srl",
    "24.15208494": "Really Simple Software, Inc",
    "12.346864136192": "Phytec Messtechnik GmbH",
    "24.4502579": "tide.co.,ltd",
    "12.123917682425856": "Alturna Networks",
    "12.123917684240384": "Dainichi Denshi Co.,LTD",
    "24.8128729": "Shanghai B-Star Technology Co",
    "24.14159331": "Fxi Technologies As",
    "12.346856071168": "Phytec Messtechnik GmbH",
    "24.14463924": "Honeywell Environmental & Combustion Controls (Tianjin) Co., Ltd.",
    "12.346864824320": "Dantec Dynamics A/S",
    "12.123917681221632": "Mikrotron Mikrocomputer, Digital- und Analogtechnik GmbH",
    "12.346863489024": "MeshWorks Wireless Oy",
    "24.9491": "Cxp Digital Bv",
    "24.532287": "WondaLink Inc.",
    "12.346862030848": "Gehrke Kommunikationssysteme GmbH",
    "24.8958433": "Mojo Networks, Inc.",
    "24.8411333": "NovaTec Kommunikationstechnik GmbH",
    "24.16051256": "Tp-Link Technologies Co.,Ltd.",
    "24.8065": "Accel Semiconductor Corp",
    "24.8064": "Lucas Holding bv",
    "24.8067": "Teleplan Technology Services Sdn Bhd",
    "24.8066": "Cal-Comp Electronics & Communications Company Ltd.",
    "24.8061": "Embedded Wireless GmbH",
    "24.8060": "Witelcom AS",
    "24.8063": "Phabrix Limited",
    "24.8062": "ARRIS Group, Inc.",
    "24.8069": "Apriva ISS, LLC",
    "24.8068": "Gigle Semiconductor",
    "24.16303006": "Huawei Technologies Co.,Ltd",
    "12.346863390720": "Mettler Toledo",
    "24.16308054": "Simm Tronic Limited",
    "24.16011219": "shenzhen hemuwei technology co.,ltd",
    "24.2685600": "vivo Mobile Communication Co., Ltd.",
    "20.137149122871296": "Tibit Communications",
    "20.180680383266816": "Stellamore",
    "20.242213042782208": "Tattile Srl",
    "12.346856955904": "Trafsys As",
    "12.346858319872": "Gnutek Ltd.",
    "20.145735976747008": "CCX Technologies Inc.",
    "12.123917680353280": "Shachihata Inc.",
    "20.238070221766656": "Inspur Group Co., Ltd.",
    "24.9736914": "KCF Technologies, Inc.",
    "20.137149121822720": "Guangzhou Puppyrobot Technology Co.Ltd Beijing Branch",
    "24.13904362": "Zimory GmbH",
    "12.123917689110528": "Sandia National Laboratories",
    "24.15510968": "Apple, Inc.",
    "24.13896361": "ARRIS Group, Inc.",
    "24.9718166": "WiSilica Inc",
    "24.7627743": "VTS GmbH",
    "20.203238159679488": "Xiamen Xindeco Ltd.",
    "12.71297884942336": "Vigitron Inc.",
    "12.123917688131584": "Solid State Disks Ltd",
    "24.6852022": "Aetek Inc.",
    "24.13399657": "Seetech",
    "24.4372": "EverFocus Electronics Corp.",
    "12.346862039040": "Electronic Media Services Ltd",
    "24.3455782": "Motorola Mobility LLC, a Lenovo Company",
    "20.158667126079488": "Shanghai JaWay Information Technology Co., Ltd.",
    "24.816322": "Apple, Inc.",
    "24.5793466": "Hangzhou H3C Technologies Co., Limited",
    "24.9997638": "Intel Corporate",
    "12.123917675552768": "Husty M.Styczen J.Hupert Sp.J.",
    "24.5818486": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "20.229108124483584": "Cyrus Technology GmbH",
    "12.346867740672": "Belik S.P.R.L.",
    "12.123917685301248": "Acs Motion Control",
    "20.106480356294656": "Quantronix, Inc.",
    "24.7381393": "Trendsetter Medical, LLC",
    "24.9828": "Core System Japan",
    "24.9829": "ProtectedLogic Corporation",
    "24.9826": "Actiontec Electronics, Inc",
    "24.9827": "Shenzhen Huitaiwei Tech. Ltd, co.",
    "24.9824": "Logiways",
    "24.9825": "Irumtek Co., Ltd.",
    "24.9822": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9823": "Samsung Electronics Co.,Ltd",
    "24.9820": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9821": "Samsung Electronics Co.,Ltd",
    "24.4724986": "Nocsys",
    "24.13956391": "Siemens Ag",
    "24.6858792": "Huawei Technologies Co.,Ltd",
    "24.9735504": "Beamex Oy Ab",
    "20.27478311567360": "Shenzhen Rui jiali Electronic Technology Co. Ltd.",
    "24.567362": "Huawei Device Co., Ltd.",
    "24.1881900": "ASUSTek COMPUTER INC.",
    "24.12902011": "Huawei Device Co., Ltd.",
    "24.15259673": "AzureWave Technology Inc.",
    "12.346860609536": "Bruno International Ltd.",
    "24.11593694": "Homa Technologies JSC",
    "12.346866749440": "Dacom West GmbH",
    "24.11560291": "Shanghai Railway Communication Factory",
    "24.3210344": "Tp-Link Technologies Co.,Ltd.",
    "12.346865754112": "Epec Oy",
    "24.15748082": "JTECH Communications, Inc.",
    "24.14163065": "Apple, Inc.",
    "24.16254236": "DRS Imaging and Targeting Solutions",
    "24.8672479": "Huawei Device Co., Ltd.",
    "12.123917676367872": "2-Observe",
    "24.7340469": "Cisco Systems, Inc",
    "24.4479459": "Huawei Technologies Co.,Ltd",
    "12.123917680877568": "Wagner Group GmbH",
    "20.5239430184960": "Shenzhen Shuotian Information Technology Co., LTD",
    "12.346854731776": "Servicios Condumex, S. A. de C. V.",
    "12.123917685252096": "Overspeed SARL",
    "24.11039773": "PHOENIX CONTACT Electronics GmbH",
    "24.16304492": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.15782313": "Apple, Inc.",
    "24.9233512": "Guangzhou Sageran Technology Co., Ltd.",
    "24.6064150": "Rockwell Automation",
    "12.346852450304": "Private",
    "12.346862448640": "Intuitive Surgical, Inc",
    "24.309731": "Samsung Electronics Co.,Ltd",
    "24.32909": "Westcove Technology BV",
    "24.14696758": "MitraStar Technology Corp.",
    "24.3439200": "ARRIS Group, Inc.",
    "24.32900": "The Cloud Inc.",
    "24.32901": "H-Three Systems Corporation",
    "24.32902": "Computer Generation Inc.",
    "24.32903": "Oki Electric Industry Co., Ltd",
    "24.32904": "Victor Company Of Japan, Ltd.",
    "24.32905": "Tecnetics (Pty) Ltd.",
    "24.32906": "Summit",
    "24.32907": "Dacoll Limited",
    "24.1589807": "Alcatel Lucent",
    "20.137141742993408": "Alpha Technologies, Llc",
    "24.8191362": "Guangdong Genius Technology Co., Ltd.",
    "12.123917690904576": "WIBOND Informationssysteme GmbH",
    "24.10758327": "Yangtze Memory Technologies Co., Ltd.",
    "24.13391277": "Rim",
    "24.16286962": "Samsung Electronics Co.,Ltd",
    "24.4194567": "Arista Corp",
    "24.15499273": "NovaSparks",
    "12.123917681639424": "EMAC, Inc.",
    "24.9182": "Ansync Inc.",
    "24.12588093": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9219660": "Plugable Technologies",
    "20.198151834173440": "NSI Co., Ltd.",
    "24.11566436": "Intel Corporate",
    "24.11566434": "Dipl.-Ing. H. Horstmann GmbH",
    "24.13909305": "Dialog Semiconductor",
    "24.16016477": "Toshiba",
    "12.123917685374976": "K&A Electronics Inc.",
    "24.9437989": "Huawei Technologies Co.,Ltd",
    "24.9725229": "OnePlus Technology (Shenzhen) Co., Ltd",
    "12.346863046656": "Fr. Sauter AG",
    "20.89058719039488": "Garrison Technology",
    "24.5785313": "Baxter International Inc",
    "12.346857553920": "Motec Pty Ltd",
    "12.123917685129216": "Egag, LLC",
    "24.20605": "Ifp",
    "24.20606": "Newer Technology",
    "24.49260": "Svec Computer Corp.",
    "12.346853429248": "Elad Srl",
    "24.20600": "Megaton House, Ltd.",
    "24.9194329": "3D Imaging & Simulations Corp",
    "24.20601": "Private",
    "24.49265": "Areanex Communications, Inc.",
    "24.20603": "Merlot Communications",
    "24.6871333": "ProSys Development Services",
    "12.123917687984128": "Jc Hunter Technologies",
    "12.71297883340800": "Integrated Control Corp.",
    "24.8945411": "Comper Technology Investment Limited",
    "24.15728767": "Janz - Contadores de Energia, SA",
    "12.346856861696": "Pitney Bowes, Inc",
    "24.5253771": "Telekom Research and Development Sdn Bhd",
    "24.7093221": "Hewlett Packard",
    "24.14429222": "Iton Technology Limited",
    "24.1878785": "Innovolt",
    "20.176189953015808": "Incogniteam Ltd.",
    "24.5251687": "PixeLINK",
    "20.211776054493184": "Shenzhen Wins Electronic Technology Co., Ltd",
    "12.346861244416": "Intelligent Security Systems",
    "24.3207965": "zte corporation",
    "24.841366": "Amimon Ltd",
    "24.12882905": "Samsung Electronics Co.,Ltd",
    "12.346860085248": "Globalcom Engineering Srl",
    "24.11281215": "Shenzhen Minew Technologies Co., Ltd.",
    "24.12869212": "Huawei Technologies Co.,Ltd",
    "12.346856824832": "KOP Ltd",
    "20.168198567624704": "NC-LINK Technology Co., Ltd.",
    "24.5534287": "Apple, Inc.",
    "12.346859044864": "GHL Systems Berhad",
    "24.12885485": "Microsoft Corporation",
    "20.224641224278016": "Watertech S.p.A.",
    "12.123917687013376": "OnYield Inc Ltd",
    "12.71297884336128": "nanoTRONIC GmbH",
    "24.10225179": "Hangzhou H3C Technologies Co., Limited",
    "24.8202": "Source-Comm Corp.",
    "24.14208152": "Huawei Device Co., Ltd.",
    "24.3693224": "Beijing Zhongdun Security Technology Development Co.",
    "12.346857619456": "Hyundai J. Comm",
    "12.123917677223936": "Hildebrand Technology Limited",
    "24.41133": "Marconi Spa",
    "24.41132": "Gilat Satellite Networks, Ltd.",
    "24.41131": "Netcs Informationstechnik Gmbh",
    "24.41130": "Spacelabs Medical",
    "24.41137": "First Virtual Corporation",
    "24.41136": "I-O Data Device,Inc.",
    "24.41135": "Wms Industries",
    "24.41134": "Nucom Systems, Inc.",
    "24.41139": "Zykronix",
    "24.41138": "Shima Seiki",
    "24.11308308": "Smartrove Inc",
    "12.346858459136": "MedAvant Healthcare",
    "12.346857676800": "AVerMedia Technologies, Inc.",
    "12.346854625280": "Omicron Ceti AB",
    "24.9443135": "Huawei Technologies Co.,Ltd",
    "20.105645760053248": "S Labs sp. z o.o.",
    "24.2364688": "IEEE Registration Authority",
    "24.4242017": "ARIMA Communications Corp.",
    "12.346856792064": "ISE GmbH",
    "24.8931952": "Wacom Co.,Ltd.",
    "12.123917691183104": "Streambox Inc",
    "20.114975483887616": "Outstanding Technology Co., Ltd.",
    "20.5239431233536": "Advanced Digital Technologies, s.r.o.",
    "12.123917688975360": "BeiLi eTek (Zhangjiagang) Co., Ltd.",
    "12.123917677883392": "Foshan Vohom",
    "20.52819396657152": "Beijing Mydreamplus Information Technology Co., Ltd.",
    "12.346854805504": "Littwin GmbH & Co KG",
    "12.346868289536": "Brauch Elektronik GmbH&Co.KG",
    "20.22739802914816": "Sercomm Corporation.",
    "24.2375854": "NIIC Technology Co., Ltd.",
    "24.8970593": "Art Beijing Science and Technology Development Co., Ltd.",
    "20.207357810311168": "Amap Information Technology Co., Ltd",
    "24.11307261": "Apple, Inc.",
    "24.6712": "Sanmina-SCI",
    "12.346865598464": "MME Mueller Mikroelektronik",
    "20.154314897096704": "ecom instruments GmbH",
    "24.6854475": "Shenzhen Costar Electronics Co. Ltd.",
    "20.132811178639360": "Apc",
    "24.2623006": "Ningbo Global Useful Electric Co.,Ltd",
    "12.346853052416": "Loewe Opta GmbH",
    "20.220327872299008": "Maxcom S.A.",
    "12.346868936704": "MSR-Solutions GmbH",
    "24.12350679": "HangZhou JuRu Technology CO.,LTD",
    "24.293835": "Universal Global Scientific Industrial Co., Ltd.",
    "24.4218109": "Dell Inc.",
    "12.346862850048": "Coolit Systems, Inc.",
    "24.1587757": "Samsung Electronics Co.,Ltd",
    "24.12876707": "NAVIS Inc.",
    "24.8725": "ASUSTek COMPUTER INC.",
    "24.3994514": "Virtualtek. Co. Ltd",
    "24.15260330": "VideoHome Technology Corp.",
    "12.123917678964736": "Softcreate Corp.",
    "20.251018412556288": "Ce Labs, Llc",
    "24.11859998": "Kinova",
    "24.7630617": "Icarvisions (Shenzhen) Technology Co., Ltd.",
    "24.13411503": "Shenzhen Sowell Technology Co., LTD",
    "20.168210327404544": "Jazwares LLC",
    "24.7085169": "Zhejiang Dahua Technology Co., Ltd.",
    "24.8134105": "Xiaomi Communications Co Ltd",
    "24.9474740": "Diehl BGT Defence GmbH & Co. KG",
    "24.9478672": "United Telecoms Ltd",
    "24.13123239": "KUNBUS GmbH",
    "24.14955976": "Huawei Technologies Co.,Ltd",
    "20.163808569982976": "Elkron",
    "20.246679832887296": "Mountz, Inc.",
    "12.346857779200": "Matrix Telecom Pvt. Ltd.",
    "20.159182266302464": "Cinet Inc",
    "24.11586814": "Huawei Device Co., Ltd.",
    "24.14220435": "Intel Corporate",
    "12.123917688438784": "Ellenex Pty Ltd",
    "24.15005935": "Samsung Electronics Co.,Ltd",
    "12.346854662144": "Shanghai Gaozhi Science&Technology Development Ltd.",
    "12.123917687205888": "Tiama",
    "24.8452331": "RayTight",
    "24.2914621": "Boditech Med",
    "24.283379": "Apple, Inc.",
    "20.158667122933760": "CommandScape, Inc.",
    "20.132855984291840": "Tnb",
    "24.568516": "FMTech",
    "24.1573361": "Xiaomi Communications Co Ltd",
    "24.6830968": "Cisco Systems, Inc",
    "24.2373452": "Cisco SPVTG",
    "12.123917690720256": "Cambridge Technology, Inc.",
    "24.281286": "Aipon Electronics Co., Ltd",
    "24.10525083": "Apple, Inc.",
    "12.123917677133824": "Dsp Design Ltd",
    "24.6846540": "Nortel Networks",
    "24.10779849": "Huawei Technologies Co.,Ltd",
    "24.565143": "Leroy Automation",
    "12.346868797440": "Toptech Systems, Inc.",
    "24.4219278": "Stella-Green Co",
    "24.6297897": "Voltronic Power Technology(Shenzhen) Corp.",
    "24.4219270": "Micro-Star Int'L Co.,Ltd",
    "12.119269818368": "Seven Solutions S.L",
    "24.6348341": "GITSN, Inc.",
    "24.7912892": "Samsung Electronics Co.,Ltd",
    "24.13657894": "Hewlett Packard Enterprise",
    "24.293303": "Texas Instruments",
    "24.1354693": "Intel Corporate",
    "20.216145529405440": "Xinjiang Golden Calf Energy IOT Technology Co., Ltd",
    "12.123917675208704": "iFreecomm Technology Co., Ltd",
    "20.127654887948288": "Cloudvue Technologies Corporation",
    "20.49277256597504": "Bowei Technology Company Limited",
    "12.346864250880": "EMAC, Inc.",
    "12.346862358528": "ISA - Intelligent Sensing Anywhere, S.A.",
    "24.282391": "HUMAX Co., Ltd.",
    "24.3957248": "Samsung Electronics Co.,Ltd",
    "24.41690": "INAT GmbH",
    "24.841836": "Shenzhen Fast Technologies Co.,Ltd",
    "20.89058722185216": "Dolphin Concepts Limited",
    "20.242213045927936": "AiSight GmbH",
    "24.53279": "Senetas Corporation Ltd",
    "24.53278": "Pingtel Corp.",
    "24.53277": "Furuno Electric Co., Ltd.",
    "24.53276": "Sbs Technologies,",
    "24.53275": "Mimaki Engineering Co., Ltd.",
    "24.53274": "Urmet Tlc S.P.A.",
    "24.53273": "Dainippon Screen Corporate",
    "24.53272": "Qwes. Com, Inc.",
    "24.53271": "Syntech Information Co., Ltd.",
    "24.53270": "Scm Microsystems, Inc.",
    "24.5484": "Sane System Co., Ltd",
    "24.1904": "Ubiquoss Inc",
    "24.5486": "A. W. Communication Systems Ltd",
    "24.5487": "Xiranet Communications GmbH",
    "24.1901": "Flexlight Networks",
    "24.1900": "Daehanet, Inc.",
    "24.5482": "DG2L Technologies Pvt. Ltd.",
    "24.1902": "Sinetica Corporation Limited",
    "24.1067185": "Beijing Duokan Technology Limited",
    "24.1909": "Valence Semiconductor, Inc.",
    "24.5489": "Nolan Systems",
    "24.10246785": "Xiaomi Communications Co Ltd",
    "24.14444146": "Samsung Electronics Co.,Ltd",
    "12.123917682868224": "Nidec Leroy-Somer",
    "12.346862772224": "MAMAC Systems, Inc.",
    "12.123917685256192": "YUYAMA MFG Co.,Ltd",
    "24.12381219": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.3888": "Raza Microelectronics Inc",
    "24.3889": "Allied Vision Technologies Canada Inc",
    "24.3886": "Megapower International Corp.",
    "24.3887": "W-Linx Technology Co., Ltd.",
    "24.3884": "Uplogix, Inc.",
    "24.3885": "Chung-Hsin Electric & Machinery Mfg.Corp.",
    "24.3882": "Cableware Electronics",
    "24.3883": "Greenbell Systems",
    "24.3880": "Itronix Corporation",
    "24.3881": "Augmentix Corporation",
    "20.159304572207104": "Fair Winds Digital srl",
    "24.8142123": "Huawei Device Co., Ltd.",
    "24.5738": "Tps",
    "24.5739": "Samsung Electronics Co.,Ltd",
    "24.5736": "Eishin Electronics",
    "24.5737": "MRV Communication (Networks) LTD",
    "24.5734": "Quantier Communication Inc.",
    "24.5735": "A-TEC Subsystem INC.",
    "24.5732": "Prod-El SpA",
    "24.5733": "Cellon France",
    "24.5730": "Liyuh Technology Ltd.",
    "24.5731": "KBT Mobile",
    "24.10778464": "Nokia Corporation",
    "20.35227553497088": "Bently & EL Co. Ltd.",
    "24.4736247": "Samsung Electronics Co.,Ltd",
    "24.3955255": "Samsung Electronics Co.,Ltd",
    "12.123917689688064": "EarTex",
    "24.4289": "Oi Electric Co.,Ltd",
    "24.4288": "ARMA, Inc.",
    "24.10491484": "Triteka LTD",
    "24.4281": "Maxtor Corp.",
    "24.4280": "Ishigaki Computer System Co.",
    "24.4283": "Data & Information Technology",
    "24.4282": "Martinho-Davis Systems, Inc.",
    "24.4285": "The Telecommunication Technology Committee (Ttc)",
    "24.4284": "Aastra Telecom",
    "24.4287": "InterAir Wireless",
    "24.4286": "March Networks Corporation",
    "24.3778": "Lowrance Electronics, Inc.",
    "24.3779": "Logic Controls, Inc.",
    "24.57389": "InnoMediaLogic, Inc.",
    "24.57388": "AST - built into 5166M PC motherboard (win95 id's as Intel)",
    "24.11576404": "Tripwire Inc.",
    "24.57383": "Dux, Inc.",
    "24.3771": "Everbee Networks",
    "24.57381": "dit Co., Ltd.",
    "24.3773": "Burdick, a Quinton Compny",
    "24.3774": "B&B Electronics Manufacturing Co.",
    "24.3775": "Remsdaq Limited",
    "24.57385": "SMC EtherPower II 10/100",
    "24.57384": "Aptix Corporation",
    "24.4742496": "AzureWave Technology Inc.",
    "24.3938831": "ClearSky Data",
    "12.346867314688": "ACS Motion Control Ltd.",
    "12.123917689495552": "Insitu, Inc",
    "24.11858394": "LG Electronics (Mobile Communications)",
    "24.14180531": "Guangdong Global Electronic Technology CO.，LTD",
    "24.12095106": "Huawei Device Co., Ltd.",
    "12.346868883456": "Globalcom Engineering Srl",
    "24.11055572": "Cisco Systems, Inc",
    "24.14731111": "Apple, Inc.",
    "12.346858102784": "Transenna AB",
    "12.346868772864": "DEUTA-WERKE GmbH",
    "20.181338949812224": "Paragon Business Solutions Ltd.",
    "24.21693": "Swelaser AB",
    "24.2119785": "Ruckus Wireless",
    "24.14682559": "SHENZHEN TONG BO WEI TECHNOLOGY Co.,LTD",
    "24.6055532": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.8655775": "zte corporation",
    "24.12877861": "Huawei Device Co., Ltd.",
    "24.16009557": "MIMO TECH Co., Ltd.",
    "12.123917689425920": "Metralight, Inc.",
    "20.255861683388416": "Anduril Industries",
    "24.14473856": "New H3C Technologies Co., Ltd",
    "24.6042407": "Spazio Italia srl",
    "24.954": "Oracle Corporation",
    "24.955": "Signal Communications Limited",
    "24.956": "COT GmbH",
    "24.957": "OmniCluster Technologies, Inc.",
    "24.950": "QSI Corporation",
    "24.951": "ZACCESS Systems",
    "24.952": "NetKit Solutions, LLC",
    "24.953": "Hualong Telecom Co., Ltd.",
    "24.3735350": "Ruckus Wireless",
    "24.7385141": "Shenzhen Zowee Technology Co., Ltd",
    "24.958": "Netility",
    "24.959": "Centerpoint Broadband Technologies, Inc.",
    "24.6572000": "Feitian Technologies Co., Ltd",
    "12.346861764608": "Cinetix GmbH",
    "24.9596": "Huachentel Technology Development Co., Ltd",
    "24.9597": "PointRed Telecom Private Ltd.",
    "12.346860797952": "hema electronic GmbH",
    "24.2664155": "Huawei Technologies Co.,Ltd",
    "24.9592": "JSC \"Concern \"Sozvezdie\"",
    "24.9593": "J & F Labs",
    "24.9590": "Neli Technologies",
    "24.9591": "D-BOX Technologies",
    "12.123917679067136": "Gogo Business Aviation",
    "24.9598": "New Pos Technology Limited",
    "24.9599": "CallTechSolution Co.,Ltd",
    "24.4740444": "Wu Qi Technologies,Inc.",
    "24.1326578": "Beijing Shidai Hongyuan Network Communication Co.,Ltd",
    "20.167594814341120": "Vitronic Dr.-Ing. Stein Bildverarbeitungssysteme GmbH",
    "24.12109482": "Earda Technologies co Ltd",
    "24.7259": "Chubb Electronic Security Systems Ltd",
    "24.7258": "Advanced Relay Corporation",
    "24.7257": "Devon It",
    "24.7256": "Cisco Systems, Inc",
    "24.7255": "Cisco Systems, Inc",
    "24.7254": "Pado Systems, Inc.",
    "24.7253": "Shenzhen Kaifa Technology Co.",
    "24.7252": "Hillstone Networks Inc",
    "24.7251": "Synergy Lighting Controls",
    "24.7250": "Visionee Srl",
    "24.10787829": "Hybridserver Tec GmbH",
    "24.6834610": "Carlo Gavazzi Ltd",
    "12.346862436352": "Hirotech, Inc",
    "24.3166923": "Liteon Technology Corporation",
    "24.15224613": "Murata Manufacturing Co., Ltd.",
    "24.5006553": "Guangdong Leawin Group Co., Ltd",
    "12.346867085312": "Nanosolution Inc.",
    "24.1097045": "Q-Lab Corporation",
    "12.346853232640": "Lumentis AB",
    "12.346856452096": "QT systems ab",
    "20.368729653248": "Shinko Technos co.,ltd.",
    "24.49341": "Inex Technologies, Inc.",
    "24.49342": "Alcatel - Sel",
    "24.49343": "Technology Concepts, Ltd.",
    "24.49344": "Shore Microsystems, Inc.",
    "24.49345": "Quad/Graphics, Inc.",
    "24.49346": "Infinite Networks Ltd.",
    "24.49347": "Acuson Computed Sonography",
    "24.49348": "Computer Operational",
    "24.49349": "Sid Informatica",
    "24.1331856": "KNS Group LLC (YADRO Company)",
    "24.15762976": "Arcadyan Corporation",
    "24.1888698": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11810295": "Zhejiang Pearmain Electronics Co.ltd.",
    "24.10281520": "Julong Co,.Ltd.",
    "12.123917685817344": "Lecip Corporation",
    "24.16312094": "Ruckus Wireless",
    "24.793632": "Kakao Corp",
    "24.10227116": "Tymphany HK Limited",
    "24.12877135": "Cisco Systems, Inc",
    "12.123917682905088": "SEMEX-EngCon GmbH",
    "24.8645": "3DSP Corp",
    "24.8644": "Consilium AB",
    "24.8647": "Russound",
    "24.8646": "CSJ Global, Inc.",
    "24.8641": "ABB Oy / Medium Voltage Products",
    "24.8640": "Mobile Appliance, Inc.",
    "24.8643": "CORNELL Communications, Inc.",
    "24.8642": "GL Communications Inc",
    "24.12364246": "Cyber-Rain, Inc.",
    "24.8649": "Wavecom Asia Pacific Limited",
    "24.8648": "LOHUIS Networks",
    "24.1373786": "Xiaomi Communications Co Ltd",
    "12.123917690728448": "Carlo Gavazzi Industri",
    "24.3936690": "Fraunhofer FIT",
    "24.13142855": "Nolangroup S.P.A con Socio Unico",
    "24.4257070": "Leica Microsystems (Schweiz) AG",
    "24.2029": "Altera Corporation",
    "24.2028": "Cisco Systems, Inc",
    "24.2027": "Cisco Systems, Inc",
    "24.2026": "Massana, Inc.",
    "24.2025": "Intel Corporation",
    "24.2024": "EdgeWave",
    "24.2023": "FreeWave Technologies",
    "24.2022": "edgeflow Canada Inc.",
    "24.2021": "Coup Corporation",
    "24.2020": "SoftRadio Co., Ltd.",
    "12.123917678653440": "Power Ltd.",
    "24.1598": "Opthos Inc.",
    "24.1599": "Everex Communications Inc.",
    "24.1596": "Intrinsyc Software International Inc.",
    "24.1597": "Microwave Data Systems Inc.",
    "24.1594": "Dura Micro, Inc.",
    "24.1595": "Arcturus Networks Inc.",
    "24.1592": "Sungjin C&C Co., Ltd.",
    "24.1593": "Newtec",
    "24.1590": "Jedai Broadband Networks",
    "24.1591": "Toptrend-Meta Information (ShenZhen) Inc.",
    "24.4773637": "Apple, Inc.",
    "20.216602662404096": "KyongBo Electric Co., Ltd.",
    "24.12209642": "Environexus",
    "20.211670376906752": "Shanghai Charmhope Information Technology Co.,Ltd.",
    "24.13396977": "Sound Masking Inc.",
    "24.36594": "Netgear",
    "24.10781050": "Cig Shanghai Co Ltd",
    "12.346860781568": "T Proje Muhendislik Dis. Tic. Ltd. Sti.",
    "24.6618": "Welltrans O&E Technology Co. , Ltd.",
    "24.6619": "Micro-Star International Co., Ltd.",
    "24.6610": "Intel Corporate",
    "24.6611": "TRAK Microwave",
    "24.6612": "ICX Technologies",
    "24.6613": "IP Innovations, Inc.",
    "24.6614": "LS Cable and System Ltd.",
    "24.6615": "Fortunetek Co., Ltd",
    "24.6616": "Maxfor",
    "24.6617": "Zeutschel GmbH",
    "24.1048": "Teltronic S.A.U.",
    "24.1049": "Fibercycle Networks, Inc.",
    "12.346860126208": "Indefia",
    "12.71297883250688": "Sencon Inc.",
    "20.163808571031552": "Shenzhen Golden Star Technology Ltd",
    "24.1040": "Spinnaker Networks, Inc.",
    "24.1041": "Inkra Networks, Inc.",
    "24.1042": "WaveSmith Networks, Inc.",
    "24.1043": "snom technology GmbH",
    "24.1044": "Umezawa Musen Denki Co., Ltd.",
    "24.1045": "Rasteme Systems Co., Ltd.",
    "24.1046": "Parks S/A Comunicacoes Digitais",
    "24.1047": "Elau Ag",
    "24.2927411": "Texas Instruments",
    "24.3687078": "Ingenieurbuero Stahlkopf",
    "24.4761404": "Sichuan tianyi kanghe communications co., LTD",
    "20.149935432400896": "CPAC Systems",
    "24.2936131": "Summit Technology Group",
    "24.15783359": "Intel Corporate",
    "24.12472": "RiverDelta Networks",
    "20.105645763198976": "Osi Technology Co.,Ltd.",
    "20.154680189517824": "Parametric GmbH",
    "24.2397130": "Voxtronic Austria",
    "20.154238416060416": "Chipsafer Pte. Ltd.",
    "12.123917678858240": "CT Company",
    "12.346866495488": "Aplex Technology Inc.",
    "24.7928132": "Private",
    "24.289272": "Apple, Inc.",
    "24.9279": "Storwize, Inc.",
    "12.119269277696": "Quantum Technology Sciences, Inc.",
    "24.9273": "Digital Barriers Advanced Technologies",
    "24.9272": "Brocade Communications Systems LLC",
    "24.9271": "Motorola - BSG",
    "24.9270": "Apple, Inc.",
    "24.9277": "Emerson Appliance Motors and Controls",
    "24.9276": "S.A.A.A.",
    "24.9275": "CSSI (S) Pte Ltd",
    "24.9274": "Ludl Electronic Products",
    "24.12863001": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8167175": "Cig Shanghai Co Ltd",
    "24.1048720": "HP",
    "24.4465578": "Farmage Co., Ltd.",
    "24.1610711": "Texas Instruments",
    "12.346853548032": "AirVast Technology Inc.",
    "24.6048294": "Enguity Technology Corp",
    "24.13684811": "Huawei Technologies Co.,Ltd",
    "24.16260325": "Telefonbau Behnke GmbH",
    "24.12121777": "Apple, Inc.",
    "12.123917689663488": "Yamamoto Works Ltd.",
    "24.13958163": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917681500160": "Power Electronics Espana, S.L.",
    "12.123917679595520": "TESEC Corporation",
    "20.35227552448512": "sehwa",
    "12.346852552704": "Malden Electronics Ltd",
    "12.346867392512": "Multisuns Corporation",
    "12.123917678776320": "SPE Smartico, LLC",
    "12.123917677125632": "ard sa",
    "20.225207123968000": "Haishu Technology LIMITED",
    "24.11550102": "Vision Valley FZ LLC",
    "24.10224755": "Tecmobile (International) Ltd.",
    "24.15210492": "Fujitsu Cloud Technologies Limited",
    "24.7372217": "Cisco Systems, Inc",
    "12.123917690249216": "Melecs EWS GmbH",
    "12.346856505344": "Metrolab Technology SA",
    "24.4462671": "Fitview",
    "12.119269838848": "PREVAC sp. z o.o.",
    "24.7095320": "Zappware",
    "24.8958609": "Apple, Inc.",
    "24.4467000": "NietZsche enterprise Co.Ltd.",
    "20.176069206343680": "EISST International Ltd",
    "24.9226192": "zhejiang ebang communication co.,ltd",
    "20.255553901166592": "Lighthouse EIP",
    "20.13592318443520": "The Raymond Corporation",
    "24.13949645": "BSkyB Ltd",
    "12.346853695488": "Wavium AB",
    "24.8183713": "Espressif Inc.",
    "12.123917680427008": "Centuryarks Ltd.,",
    "12.123917676650496": "Aspt, Inc.",
    "24.848052": "Globalsat International Technology Ltd",
    "12.123917685645312": "Hear Gear, Inc.",
    "24.13393118": "China Mobile Group Device Co.,Ltd.",
    "24.2647779": "AzureWave Technology Inc.",
    "24.11324558": "Axis Communications AB",
    "24.13653257": "Nintendo Co.,Ltd",
    "24.319664": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.8715559": "Silicon Laboratories",
    "12.123917687181312": "Conclusive Engineering",
    "12.123917683322880": "Cravis Co., Limited",
    "24.16573783": "Nokia Corporation",
    "24.9226445": "Samsung Electronics Co.,Ltd",
    "12.123917684449280": "Unlimiterhear co.,ltd. taiwan branch",
    "12.71297884999680": "Horiba Abx Sas",
    "24.16408846": "Laboratory for Computational Sensing and Robotics, Johns Hopkins University",
    "24.11335415": "LG Electronics (Mobile Communications)",
    "20.27478318907392": "Energie Ip",
    "12.71297884872704": "Thirdwayv Inc.",
    "24.5816298": "Cisco Systems, Inc",
    "24.11823081": "Qingdao GoerTek Technology Co., Ltd.",
    "12.346856185856": "Comlab Inc.",
    "12.346862755840": "Ddl",
    "24.5815922": "Huawei Technologies Co.,Ltd",
    "24.11013322": "Shenzhen Sundray Technologies Company Limited",
    "24.4755795": "Weinmann Geräte für Medizin GmbH + Co. KG",
    "20.268902727680000": "Fractyl Labs",
    "24.8915375": "Masimo Corporation",
    "24.16554311": "Cisco Systems, Inc",
    "24.12616840": "Finis Inc",
    "24.13687602": "Nokia Corporation",
    "12.346859425792": "Sechan Electronics, Inc.",
    "12.346853924864": "Home Director, Inc",
    "24.4753561": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24.4505948": "Texas Instruments",
    "12.123917680390144": "Tibit Communications",
    "12.123917679255552": "Liaoyun Information Technology Co., Ltd.",
    "12.346852204544": "Tharsys",
    "12.123917675159552": "Telstra",
    "24.14962221": "Cisco Systems, Inc",
    "24.1322104": "GooWi Wireless Technology Co., Limited",
    "12.123917684772864": "Trident Systems Inc",
    "20.224447325798400": "Apeiron Data Systems",
    "24.12847416": "Fujian Star-Net Communication Co.,Ltd",
    "24.6850411": "INQ Mobile Limited",
    "24.1860535": "OpenProducts 237 AB",
    "12.123917685587968": "Embedded Systems Lukasz Panasiuk",
    "12.346855927808": "MANGO DSP Ltd.",
    "24.14445290": "Infinix mobility limited",
    "24.2659503": "Samhwa Telecom",
    "24.12071349": "Microsoft Corporation",
    "20.22739789283328": "Henfred Technology Co., Ltd.",
    "12.123917686362112": "Birdland Audio",
    "12.346860425216": "Zeltiq Aesthetics, Inc.",
    "20.220515729932288": "Shanghai Bwave Technology Co.,Ltd",
    "24.4517839": "UGENE SOLUTION inc.",
    "24.10751078": "Shenzhen Colorful Yugong Technology and Development Co., Ltd.",
    "24.3672724": "Samsung Electronics Co.,Ltd",
    "24.6601745": "CombiQ AB",
    "24.8451509": "Texas Instruments",
    "20.84531181780992": "Nexoforge Inc.",
    "12.346860417024": "Private",
    "24.11043221": "Samsung Electronics Co.,Ltd",
    "24.11797853": "Inspur Electronic Information Industry Co.,Ltd.",
    "12.123917680177152": "CMI, Inc.",
    "24.3944559": "Huawei Technologies Co.,Ltd",
    "12.346856157184": "Technovare Systems, Inc.",
    "24.14974201": "Samsung Electronics Co.,Ltd",
    "24.4228654": "Jdm Mobile Internet Solution Co., Ltd.",
    "12.123917687709696": "Kron Medidores",
    "24.804997": "LG Electronics (Mobile Communications)",
    "24.5028113": "Raisecom Technology CO.,LTD",
    "24.5028112": "Apple, Inc.",
    "12.346866647040": "IDC Solutions Pty Ltd",
    "24.15790034": "Tf Payment Service Co., Ltd",
    "12.346862796800": "Grupo Epelsa S.L.",
    "24.9138": "Intelligent Mechatronic Systems Inc",
    "24.5441": "StrataLight Communications, Inc.",
    "12.346862592000": "Keith & Koep GmbH",
    "24.9197884": "IEEE Registration Authority",
    "24.3461828": "RENA GmbH Print Systeme",
    "24.13943198": "Ruckus Wireless",
    "24.14457630": "Intercom, Inc.",
    "24.1720": "Bandspeed Pty Ltd",
    "24.4732511": "Wintecronics Ltd.",
    "12.123917681811456": "Season Electronics Ltd",
    "24.6345401": "Realand Bio Co., Ltd.",
    "12.123917677826048": "WIXCON Co., Ltd",
    "24.8659550": "Shenzhen Technewchip Technology Co.,Ltd.",
    "24.9437865": "Zhejiang Dahua Technology Co., Ltd.",
    "24.16012693": "HENGBAO Corporation LTD.",
    "24.9134": "Dell Inc.",
    "24.9135": "ARRIS Group, Inc.",
    "24.7644962": "LG Electronics (Mobile Communications)",
    "24.9136": "COMXION Technology Inc.",
    "24.7655467": "ASUSTek COMPUTER INC.",
    "20.79602631311360": "VideoStitch, Inc",
    "24.11853818": "Litemax Electronics Inc.",
    "12.346860093440": "GE Security-Kampro",
    "24.12085698": "Sunitec Enterprise Co., Ltd.",
    "24.5293453": "Juniper Networks",
    "12.346862096384": "Far South Networks (Pty) Ltd",
    "24.4475025": "HDMI Licensing, LLC",
    "24.2151030": "Itel Mobile Limited",
    "24.4733300": "Proware Technologies Co., Ltd.",
    "24.8178445": "Antaira Technologies, LLC",
    "24.13669114": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.4718": "Seidel Elektronik GmbH Nfg.KG",
    "12.71297884073984": "Top Connect OU",
    "20.264809159327744": "Wren Associates, LTD",
    "24.822141": "Vizio, Inc",
    "24.8972415": "Apple, Inc.",
    "12.123917691076608": "Revolution Retail Systems",
    "12.346856431616": "TSB Solutions Inc.",
    "24.7630839": "Wistron Neweb Corporation",
    "12.346854330368": "Hochschule für Technik, Wirtschaft und Kultur Leipzig (FH)",
    "20.163808562642944": "Guang Dong Takstar Electronic Co.,Ltd.",
    "24.2903890": "Cisco Systems, Inc",
    "24.9239829": "Imagine Marketing Private Limited",
    "12.123917690445824": "Last Mile Gear",
    "12.346863566848": "AT4 wireless.S.A",
    "24.6035255": "I-View Now, LLC.",
    "24.12347951": "Henge Docks LLC",
    "24.13939774": "Messcomp Datentechnik GmbH",
    "24.7644341": "Powerleader Science and Technology Co. Ltd.",
    "24.3576": "ORGA Kartensysteme GmbH",
    "24.3577": "NDS Limited",
    "24.3574": "Technology Thesaurus Corp.",
    "24.3575": "Space Dynamics Lab",
    "24.3572": "Watertek Co.",
    "24.3573": "Teletronics International Inc.",
    "24.3570": "Private",
    "24.3571": "Asmax Solutions",
    "24.12342524": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.13119848": "eZEX corporation",
    "24.3578": "Micro Control Systems Ltd.",
    "24.3579": "Komax AG",
    "12.123917677486080": "Rfhic",
    "24.3698209": "Wasion Group Ltd.",
    "24.2260": "IneoQuest Technologies, Inc",
    "20.272689492131840": "Electric Objects",
    "24.12359": "Nissei Electric Co., Ltd.",
    "24.12358": "Controlled Electronic Manageme",
    "24.12357": "Village Networks, Inc. (VNI)",
    "24.12356": "CradlePoint, Inc",
    "24.12355": "Idream Technologies, Pte. Ltd.",
    "24.12354": "DeTeWe-Deutsche Telephonwerke",
    "24.12353": "Saejin T & M Co., Ltd.",
    "24.12352": "Cisco Systems, Inc",
    "24.12351": "TurboComm Tech Inc.",
    "24.12350": "Radcom Ltd.",
    "24.4242738": "Texas Instruments",
    "24.11853361": "Cisco Systems, Inc",
    "24.1860761": "Comtrend Corporation",
    "12.346864017408": "AixControl GmbH",
    "20.66256284680192": "Geoplan Korea",
    "24.1050181": "Sagemcom Broadband SAS",
    "24.6868558": "L-3 Communications Infrared Products",
    "12.123917679157248": "Sycomp Electronic GmbH",
    "24.4489079": "Connected Home",
    "24.11033452": "Robert Bosch Gmbh, CM-CI2",
    "24.12885569": "G24 Power Limited",
    "20.247076412719104": "Itel Mobile Limited",
    "24.5286847": "Hoseo Telecom",
    "20.93020695298048": "Tite, Inc.",
    "24.10243935": "TAP Sound System",
    "24.9232025": "Comlab Inc.",
    "12.123917684862976": "Tattile Srl",
    "20.93065820766208": "I-Moon Technology Co., Limited",
    "20.89058716942336": "Yongjing Shanghai Electronic Science and Technology",
    "12.346858201088": "Intuitive Surgical, Inc",
    "24.6903": "dataschalt e+a GmbH",
    "12.346861338624": "S.C.E. s.r.l.",
    "24.16278629": "Anova Applied Electronics, Inc.",
    "24.8698040": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.346861330432": "The Pennsylvania State University",
    "24.11033331": "Shanghai Siflower Communication Technology Co., Ltd",
    "24.6593214": "Apple, Inc.",
    "12.346862391296": "Tattile srl",
    "24.4196000": "Texas Instruments",
    "20.66256282583040": "Shenzhen VETAS Communication Technology Co , Ltd.",
    "12.123917681577984": "Project Decibel, Inc.",
    "24.6609824": "Select Comfort",
    "24.1372814": "ShenYang ZhongKe-Allwin Technology Co.LTD",
    "24.12583808": "Juniper Networks",
    "24.1321492": "ShenZhen Selenview Digital Technology Co.,Ltd",
    "24.7658449": "Ebay Inc",
    "20.48835656155136": "Private",
    "24.14714099": "High Grade Controls Corporation",
    "20.176032220971008": "Vast Production Services",
    "24.16580215": "Hitachi Reftechno, Inc.",
    "20.39986081562624": "Shenzhen Hailuck Electronic Technology CO.,LTD",
    "24.16037858": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917683412992": "Eurotronik Kranj d.o.o.",
    "24.4251309": "Juniper Networks",
    "12.123917687422976": "MB connect line GmbH Fernwartungssysteme",
    "24.16030124": "WEBER Schraubautomaten GmbH",
    "24.7989": "Air802 Llc",
    "24.7988": "Lung Hwa Electronics Co., Ltd.",
    "24.7983": "Berker GmbH & Co. KG",
    "24.7982": "Triangle Research Int'l Pte Ltd",
    "24.7981": "Electro-Optical Imaging, Inc.",
    "24.7980": "Starbridge Networks",
    "24.7987": "Netgear",
    "24.7986": "Nintendo Co., Ltd.",
    "24.7985": "Radiocomp",
    "24.7984": "Travelping",
    "24.6056": "EDM Corporation",
    "24.6057": "Sentivision",
    "24.6054": "Yosin Electronics Co., Ltd.",
    "24.6055": "Mobile Computing Promotion Consortium",
    "20.22333151510528": "Emerson Network Power (India) Pvt. Ltd.",
    "24.6053": "Ralink Technology Corp",
    "24.6050": "Camrivox Ltd.",
    "24.6051": "MIX s.r.l.",
    "20.194284240240640": "Rizhao SUNWAM International Co., Ltd.",
    "24.7360264": "Shenzhen Huisheng Information Technology Co., Ltd.",
    "24.6058": "elab-experience inc.",
    "24.6059": "Nintendo Co., Ltd.",
    "24.9459452": "Tecno Mobile Limited",
    "12.71297883430912": "Vishay Celtron Technologies, Inc.",
    "24.10548161": "Tp-Link Technologies Co.,Ltd.",
    "24.8454056": "Unidis",
    "12.346853179392": "Tecnint HTE SRL",
    "24.286038": "camtron industrial inc.",
    "24.11055834": "Fujitsu Limited",
    "20.242876974891008": "Suzhou ATES electronic technology co.LTD",
    "24.8689766": "Samsung Electronics Co.,Ltd",
    "24.10248026": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.102196701036544": "iSon Tech",
    "24.8429483": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.5782726": "Omneality Ltd.",
    "12.346858942464": "ProconX Pty Ltd",
    "12.346852306944": "Zabacom, Inc.",
    "12.346854502400": "Envara",
    "20.176772029087744": "Quantlab Financial, LLC",
    "24.8711475": "BBMC Co.,Ltd",
    "12.123917681541120": "Pivothead",
    "24.15750292": "Honeywell Connected Building",
    "24.8406493": "Intel Corporate",
    "20.62070509797376": "Shanghai ZoomSmart Technology Co., Ltd.",
    "24.10783963": "Lenovo",
    "24.2946098": "Espressif Inc.",
    "20.154039761240064": "Charmlink Tech(HK) Co.,Limited",
    "24.4988": "Kaicom co., Ltd.",
    "24.4989": "Dynalab, Inc.",
    "20.30931569082368": "Microview Science and Technology Co.,Ltd",
    "24.4982": "Tabor Electronics Ltd.",
    "24.4983": "Samsung Electronics Co.,Ltd",
    "24.4980": "Atheros Communications, Inc.",
    "24.4981": "American Security Products Co.",
    "24.4986": "Netvox Technology Co., Ltd.",
    "24.4987": "Movon Corporation",
    "24.4984": "Qsan Technology, Inc.",
    "24.4985": "Ponder Information Industries Ltd.",
    "12.123917676625920": "CMC Industrial Electronics Ltd",
    "24.5518283": "Apple, Inc.",
    "24.12104750": "Gionee Communication Equipment Co,Ltd.ShenZhen",
    "24.12644091": "OnePlus Tech (Shenzhen) Ltd",
    "24.1206": "Stratex Networks, Inc.",
    "24.1207": "AMB i.t. Holding",
    "24.1204": "Ciac",
    "24.1205": "Equitrac Corporation",
    "24.1202": "Essegi Srl",
    "24.1203": "Videotek, Inc.",
    "24.1200": "ELESIGN Co., Ltd.",
    "24.1201": "Signal Technology, Inc.",
    "24.7920105": "Momentum Iot",
    "24.4755503": "LG Electronics (Mobile Communications)",
    "24.1208": "Kumahira Co., Ltd.",
    "24.1209": "S.I. Soubou, Inc.",
    "24.9488274": "ARRIS Group, Inc.",
    "24.10753767": "Huawei Technologies Co.,Ltd",
    "24.5267381": "Askey Computer Corp.",
    "24.5277033": "Samsung Electronics Co.,Ltd",
    "20.224641222180864": "Spark Biomedical",
    "24.3967398": "Kapelse",
    "24.9729719": "Samsung Electronics Co.,Ltd",
    "12.346858397696": "ESEM Grünau GmbH & Co. KG",
    "12.346860089344": "ELMAR electronic",
    "20.224641228472320": "Az-Technology Sdn Bhd",
    "20.48835647766528": "Sm Dsp Co.,Ltd.",
    "24.11852809": "Guangzhou Dawei Communication Co.,Ltd",
    "12.123917686337536": "APG Cash Drawer, LLC",
    "12.71297884069888": "Devialet SA",
    "24.3438807": "Gionee Communication Equipment Co.,Ltd.",
    "12.123917685317632": "Crown Solar Power Fencing Systems",
    "24.4368": "Maxanna Technology Co., Ltd.",
    "24.4369": "Intel Corporation",
    "24.4366": "Tsurusaki Sealand Transportation Co. Ltd.",
    "24.4367": "netplat,Inc.",
    "24.4364": "Atmark Techno, Inc.",
    "24.4365": "SANBlaze Technology, Inc.",
    "24.4362": "Hewlett Packard",
    "24.4363": "Franklin Technology Systems",
    "24.4360": "Orbital Data Corporation",
    "24.4361": "Micro-Star International",
    "24.10773409": "Inventus Power Eletronica do Brasil LTDA",
    "24.1079216": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.1079217": "Samsung Electronics Co.,Ltd",
    "12.123917675061248": "Magosys Systems LTD",
    "24.12103405": "Elitegroup Computer Systems Co.,Ltd.",
    "24.6042309": "Teracom Ltd.",
    "24.8694554": "Apple, Inc.",
    "12.119269609472": "JSC Prominform",
    "24.4471236": "Universal Global Scientific Industrial Co., Ltd.",
    "12.123917677993984": "Skywave Networks Private Limited",
    "24.5028696": "Chicony Electronics Co., Ltd.",
    "24.15245184": "Belkin International Inc.",
    "12.346857734144": "LJU Automatisierungstechnik GmbH",
    "20.49277260791808": "Fiberroad Technology Co., Ltd.",
    "24.8427041": "Lenovo",
    "24.12357690": "Robert Bosch GmbH",
    "12.346854989824": "Smiths Detection",
    "24.13417470": "Huawei Technologies Co.,Ltd",
    "12.346858561536": "Communication Components Inc.",
    "24.2947057": "Seeed Technology Inc.",
    "12.346858250240": "Roke Manor Research Ltd",
    "20.154055546503168": "Geomc",
    "12.119269236736": "GORAMO - Janusz Gorecki",
    "20.132869114560512": "Alango Technologies Ltd",
    "12.346856402944": "Center VOSPI JSC",
    "24.9208828": "Axioma Metering UAB",
    "24.1324549": "Nokia Corporation",
    "24.302178": "BMT Medical Technology s.r.o.",
    "12.346856103936": "Casabyte Inc.",
    "12.346854838272": "Axode Sa",
    "24.5526548": "Digital RF Corea, Inc",
    "12.123917690216448": "RF Track",
    "24.13661972": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917685477376": "amakidenki",
    "24.9489631": "Intel Corporate",
    "12.346853675008": "Quantronix, Inc.",
    "24.8952633": "Samsung Electronics Co.,Ltd",
    "12.123917680697344": "Power Electronics Espana, S.L.",
    "20.224447329992704": "Siemens AG Austria",
    "24.8430039": "Shanghai DareGlobal Technologies Co.,Ltd",
    "20.150262235791360": "Mikroelektronika",
    "12.346864599040": "SS Systems LLC",
    "12.346862149632": "Dsci",
    "24.3733971": "Apple, Inc.",
    "24.14196765": "Espressif Inc.",
    "20.216145533599744": "SolidGear Corporation",
    "24.824315": "BNS Solutions",
    "20.229631751880704": "Perfant Technology Co., Ltd",
    "24.2635868": "Flextronics",
    "24.12906868": "eero inc.",
    "24.15753741": "Shenzhen Egreat Tech Corp.,Ltd",
    "12.123917691154432": "winsun AG",
    "24.1105803": "Shenzhen CareDear Communication Technology Co.,Ltd",
    "12.346864869376": "Peperoni-Light",
    "24.14194618": "Pegatron Corporation",
    "20.62335521652736": "Aeroespacial Guosheng Technology Co., Ltd",
    "24.1599594": "zte corporation",
    "12.346855759872": "Minicom Advanced Systems Ltd",
    "24.49242": "Semaphore Communications Corp.",
    "24.15243919": "Quanta Computer Inc.",
    "24.1327014": "Sony Home Entertainment&Sound Products Inc",
    "12.346860167168": "RaySat Israel LTD",
    "12.123917686710272": "Myro Control, LLC",
    "24.7631101": "Intel Corporate",
    "24.63532": "Cisco Systems, Inc",
    "12.346859053056": "Mate Media Access Technologies",
    "24.15205741": "Nortel Networks",
    "20.66129357701120": "Getmobit Llc",
    "20.127658778165248": "Nura Holdings Pty Ltd",
    "24.13400159": "Apple, Inc.",
    "12.346857713664": "Innopsys",
    "12.123917684883456": "Weble Sàrl",
    "24.1342899": "Joa Telecom.Co.,Ltd",
    "24.3965098": "Ransnet Singapore Pte Ltd",
    "24.4759957": "Apple, Inc.",
    "24.9419": "Autonet Mobile",
    "24.9418": "Tobii Technology AB",
    "12.346860716032": "SLICAN sp. z o.o.",
    "24.9413": "Meridian Audio Limited",
    "24.9412": "Cisco Systems, Inc",
    "24.9411": "Cisco Systems, Inc",
    "24.9410": "Asumo Co.,Ltd.",
    "24.9417": "Broadband Solutions Group",
    "24.9416": "Broadband Solutions Group",
    "24.9415": "Mobilarm Ltd",
    "24.9414": "Hager Electro SAS",
    "24.1604619": "Ruckus Wireless",
    "24.6578826": "evon GmbH",
    "12.346861350912": "Byte Paradigm",
    "12.346853060608": "Cyberex",
    "12.346865778688": "Tritium Pty Ltd",
    "24.15527950": "AboCom",
    "24.36968": "Dvt Corp.",
    "24.6856688": "zte corporation",
    "24.8397025": "BVBA DPTechnics",
    "24.3191221": "Arcadyan Corporation",
    "24.7400676": "Apple, Inc.",
    "12.123917675155456": "Elsuhd Net Ltd Co.",
    "24.10235343": "Shishi Tongyun Technology(Chengdu)Co.,Ltd.",
    "24.7048": "Divinet Access Technologies Ltd",
    "24.7049": "EMZA Visual Sense Ltd.",
    "24.7046": "Bosch Access Systems GmbH",
    "24.7047": "Deepsound Tech. Co., Ltd",
    "24.7044": "Scan Engineering Telecom",
    "24.7045": "MAN Energy Solutions",
    "24.7042": "Taiwan Semiconductor Co., Ltd.",
    "24.7043": "Finsoft Ltd",
    "24.7040": "LORD Corporation",
    "24.7041": "DATAQ Instruments, Inc.",
    "12.346860642304": "Intelight Inc.",
    "20.242213036490752": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "24.2278": "Littlefeet",
    "24.2279": "SHI ControlSystems,Ltd.",
    "24.2276": "Envenergy Inc",
    "24.2277": "IDK Corporation",
    "24.2274": "Cisco Systems, Inc",
    "24.2275": "Cisco Systems, Inc",
    "24.2272": "ATO Technology Ltd.",
    "24.2273": "Barix AG",
    "24.2270": "3UP Systems",
    "24.2271": "Alistel Inc.",
    "24.13907229": "Technicolor",
    "24.9707603": "Nokia Corporation",
    "24.2672241": "Yota Devices OY",
    "24.15515754": "PowerChord Group Limited",
    "12.346864058368": "Real D",
    "24.11827527": "Nutanix",
    "24.7140211": "Samsung Electronics Co.,Ltd",
    "24.6345244": "Sunnovo International Limited",
    "24.16523033": "Samsung Electro Mechanics Co., Ltd.",
    "20.31873092812800": "A&B Technology",
    "24.16045600": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346857910272": "ptswitch",
    "24.268470": "Private",
    "24.7879300": "Apple, Inc.",
    "12.346861441024": "Industrial Control and Communication Limited",
    "24.14201719": "HTC Corporation",
    "24.4736763": "Huawei Technologies Co.,Ltd",
    "24.4201603": "Intel Corporate",
    "24.2937451": "Fn-Link Technology Limited",
    "12.71297883598848": "Tronic Control ltd.",
    "24.14426553": "Sentec Co.Ltd",
    "24.10146403": "FirmTek, LLC",
    "24.15784370": "Exo S.A.",
    "20.216602660306944": "Advanced Navigation",
    "12.346864123904": "Electronic Systems Protection, Inc.",
    "20.83999499223040": "TEL-Electronics Ltd",
    "24.1315929": "Vodafone Italia S.p.A.",
    "24.3715024": "Nevro",
    "12.123917678510080": "Nevotek",
    "24.12898531": "Intel Corporate",
    "24.5568226": "Huawei Technologies Co.,Ltd",
    "24.9997876": "Zhejiang Boxsam Electronic Co.,Ltd",
    "24.2627245": "Intel Corporate",
    "24.24688": "Cisco",
    "24.24689": "Midas Lab, Inc.",
    "24.24680": "Dialogic Corporation",
    "24.24681": "Brocade Communications Systems LLC",
    "24.24682": "Mitsubishi Wireless Communications. Inc.",
    "24.24683": "Synclayer Inc.",
    "24.24684": "Arescom",
    "24.24685": "Digital Equipment Corp.",
    "24.24686": "Davicom Semiconductor, Inc.",
    "24.24687": "Clarion Corporation Of America",
    "24.16315698": "ARRIS Group, Inc.",
    "24.9240244": "Vsoontech Electronics Co., Limited",
    "24.11552703": "Samsung Electronics Co.,Ltd",
    "24.9452890": "Shenzhen Wision Technology Holding Limited",
    "24.3948623": "Guangdong Pisen Electronics Co.,Ltd",
    "24.15991556": "Google, Inc.",
    "24.832183": "Texas Instruments",
    "24.4750920": "Earda Technologies co Ltd",
    "12.123917675593728": "Powercraft Electronics Pvt. Ltd.",
    "24.14212329": "Kunshan Erlab ductless filtration system Co.,Ltd",
    "24.16312579": "D-Link International",
    "12.346854543360": "Hauppauge Computer Works, Inc.",
    "12.346867421184": "SystemAdvanced Co,Ltd",
    "24.4465301": "China Mobile Iot Limited company",
    "24.12860063": "Siconix Inc.",
    "20.114869205467136": "Stephan Electronics SARL",
    "24.3938654": "Samsung Electronics Co.,Ltd",
    "24.13957161": "Sagemcom Broadband SAS",
    "24.3716429": "Apple, Inc.",
    "24.7865665": "Queclink Wireless Solutions Co., Ltd",
    "24.11299599": "Cadence Design Systems Inc",
    "24.8429622": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "24.15780769": "Apple, Inc.",
    "24.837292": "Dags",
    "20.48688648945664": "Collatz+Trojan GmbH",
    "24.12124001": "Apple, Inc.",
    "24.6374": "Computer Hardware Design SIA",
    "12.123917689274368": "Subinitial LLC",
    "12.346861989888": "Metronic Aparatura Kontrolno - Pomiarowa",
    "12.346856374272": "NanShanBridge Co.Ltd",
    "24.16277744": "Aruba, a Hewlett Packard Enterprise Company",
    "20.52821133099008": "Imageo s.r.o.",
    "24.6355613": "Huawei Technologies Co.,Ltd",
    "24.3428159": "IO-Power Technology Co., Ltd.",
    "12.123917679063040": "MultiDyne",
    "24.12069003": "Parker Hannifin Manufacturing (UK) Ltd",
    "24.11286097": "Jiangsu Hengtong Terahertz Technology Co., Ltd.",
    "24.3150": "Winbest Technology CO,LT",
    "12.123917691703296": "Niron systems & Projects",
    "24.1866529": "Sony Mobile Communications Inc",
    "12.346860888064": "eInfochips Ltd.",
    "24.3153": "Scientific Technologies Inc.",
    "24.3943472": "Dell Inc.",
    "20.268735389630464": "Rosco, Inc",
    "24.12240005": "Fraunhofer IIS",
    "24.11046436": "Apple, Inc.",
    "24.6837441": "ColorTokens, Inc.",
    "24.8706353": "Hangzhou H3C Technologies Co., Limited",
    "24.5017868": "Lanix Internacional, S.A. de C.V.",
    "24.29188": "Samsung Electronics Co., Ltd. ARTIK",
    "24.2389629": "Cisco SPVTG",
    "24.8968546": "Canopus Systems US LLC",
    "24.7616649": "ANNAX Anzeigesysteme GmbH",
    "12.123917684715520": "Abbott Medical Optics Inc.",
    "12.123917689655296": "Korea Plant Maintenance",
    "24.11301897": "Hangzhou H3C Technologies Co., Limited",
    "24.14713207": "GreenBytes, Inc.",
    "12.123917679267840": "Mi-Fi Networks Pvt Ltd",
    "24.15531093": "A. Eberle GmbH & Co. KG",
    "24.10263232": "LEGO System A/S",
    "24.9746629": "RuggedCom Inc.",
    "12.123917679824896": "Pycom Ltd",
    "12.71297884135424": "A-Z-E",
    "20.149945498730496": "Sonnet Labs Inc.",
    "20.247076410621952": "Moog Crossbow",
    "12.346855202816": "Dalsa",
    "12.123917683474432": "Ingenieurbuero SOMTRONIK",
    "24.560940": "Amber Technology Ltd.",
    "24.2890116": "IDN Telecom, Inc.",
    "24.10767059": "ST Electronics(Shanghai) Co.,Ltd",
    "24.8695583": "Beat System Service Co,. Ltd.",
    "20.180698120978432": "CoAsia Microelectronics Corp.",
    "24.8133372": "Dalian Co-Edifice Video Technology Co., Ltd",
    "24.7350528": "Shenzhen ChipTrip Technology Co,Ltd",
    "12.346854985728": "Private",
    "24.14191737": "Google, Inc.",
    "12.346852442112": "Private",
    "12.123917686677504": "YUYAMA MFG Co.,Ltd",
    "24.14158275": "Cercacor Labs",
    "20.225209248382976": "Shenzhen Chenggu Technology Co., Ltd",
    "24.8401120": "Hewlett Packard Enterprise",
    "12.123917687689216": "Zumbach Electronic AG",
    "24.14962923": "Apple, Inc.",
    "20.17622853746688": "Private",
    "24.4511876": "Apple, Inc.",
    "12.71297885114368": "Commercial Wireless Systems International LLC.",
    "24.16039326": "Cisco Systems, Inc",
    "24.302590": "Hivesystem",
    "24.286631": "Shenzhen Yichen Technology Development Co.,LTD",
    "24.1634147": "Cisco Systems, Inc",
    "24.6593945": "Fs Com Inc",
    "24.13938033": "Cisco Systems, Inc",
    "24.13951788": "S. Siedle & Sohne",
    "24.8448044": "Hewlett Packard",
    "12.123917691641856": "Aeronautical & General Instruments Ltd.",
    "24.10511099": "Hisense Visual Technology Co.,Ltd",
    "24.10543617": "AVTrace Ltd.(China)",
    "20.114975482839040": "Fuzhou x-speed information technology Co.,Ltd.",
    "12.123917681516544": "Globalcom Engineering SPA",
    "12.123917675819008": "Peek Traffic",
    "24.6596177": "Huawei Technologies Co.,Ltd",
    "24.7127575": "Hewlett Packard",
    "12.346867752960": "Smart Grid Networks",
    "12.71297884430336": "AeroVision Avionics, Inc",
    "12.346855866368": "IEE Inc",
    "12.346861158400": "Shockfish SA",
    "24.3175698": "Sony Mobile Communications Inc",
    "12.123917689851904": "Invent Vision - iVision Sistemas de Imagem e Visão S.A.",
    "24.1900199": "IDentytech Solutins Ltd.",
    "12.123917675421696": "North Pole Engineering, Inc.",
    "24.9197963": "C Technologies AB",
    "12.346864431104": "ClimateWell AB (publ)",
    "12.346861150208": "Trakce a.s.",
    "24.15998101": "Beijing Yunyi Times Technology Co,.Ltd",
    "24.2923172": "Eolo SpA",
    "12.123917678080000": "Iem Sa",
    "24.6327590": "Arcadyan Corporation",
    "24.14212986": "Quest Software, Inc.",
    "20.39986083659776": "ContactProximity Inc",
    "12.346867286016": "SAMSUNG Electronics Co.,Ltd.(LED Division)",
    "20.176069211586560": "Digifocus Technology Inc.",
    "24.11842998": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.11842991": "Minsung Electronics",
    "12.346858782720": "Liquid Breaker, LLC",
    "24.15470887": "Texas Instruments",
    "12.123917676498944": "BluB0X Security, Inc.",
    "24.5219": "Idcs N.V.",
    "24.5218": "Digiwell Technology, inc",
    "24.15470880": "FloDesign Wind Turbine Corporation",
    "24.5215": "Aditec Co. Ltd",
    "24.5214": "IBM Corp",
    "24.5217": "Corona Corporation",
    "24.5216": "Kyocera Wireless Corp.",
    "24.5211": "SeekerNet Inc.",
    "24.5210": "Westermo Neratec AG",
    "24.5213": "WJ Communications, Inc.",
    "24.5212": "Intronics B.V.",
    "24.11599549": "Private",
    "24.4168": "Htrc Automation, Inc.",
    "24.4169": "ShoreTel, Inc",
    "24.4164": "InnoLabs Corporation",
    "24.4165": "Nortel Networks",
    "24.4166": "Alcorn Mcbride Inc.",
    "24.4167": "Echo Eletric Co. Ltd.",
    "24.4160": "Intermec Corporation",
    "24.4161": "Bristol Babcock, Inc.",
    "24.4162": "Alacritech, Inc.",
    "24.4163": "A2 Corporation",
    "12.123917675241472": "Ochno AB",
    "24.2411495": "MYK, Inc.",
    "12.346856054784": "Arecont Vision",
    "24.13132796": "ARRIS Group, Inc.",
    "24.1878098": "Sensaio Pte Ltd",
    "24.12627709": "Shenzhen HC-WLAN Technology Co.,Ltd",
    "12.123917684256768": "Performance Controls, Inc.",
    "20.176094521065472": "Xiamen Caimore Communication Technology Co.,Ltd.",
    "12.123917676167168": "Samriddi Automations Pvt. Ltd.",
    "12.123917677813760": "Alere Technologies AS",
    "24.16300378": "LG Innotek",
    "12.71297883570176": "Aplex Technology Inc.",
    "12.123917680279552": "INSEVIS GmbH",
    "24.3943863": "Texas Instruments",
    "12.346855649280": "Crossing Informationssysteme GmbH",
    "24.12868230": "Huawei Device Co., Ltd.",
    "24.57512": "SAT GmbH & Co.",
    "24.57513": "Funai Electric Co., Ltd.",
    "24.57510": "Telogy Networks, Inc.",
    "24.57511": "Ipc Information Systems, Inc.",
    "24.57516": "Midsco, Inc.",
    "24.57517": "Ees Technology, Ltd.",
    "24.57514": "Electrosonic Ltd.",
    "24.57515": "Dimat S.A.",
    "24.478": "Trango Systems, Inc.",
    "24.57518": "Xaqti Corporation",
    "24.57519": "General Dynamics Information Systems",
    "24.3958550": "Lily Robotics",
    "12.346862342144": "Telegamma srl",
    "12.346859196416": "Triumf",
    "12.346853408768": "U.S. Traffic Corporation",
    "12.346856845312": "SEV Tidsystem AB",
    "12.346853400576": "Runcom",
    "12.346853036032": "Zelax",
    "24.8705544": "Wingtech Mobile Communications Co., Ltd.",
    "24.8177835": "Acro Associates, Inc.",
    "12.346864549888": "EDC wifi",
    "20.35758080524288": "IOG Products LLC",
    "12.346860544000": "Wireless Cables Inc.",
    "24.5026332": "Maxphotonics Co.,Ltd",
    "24.7367957": "Cisco Systems, Inc",
    "12.346866356224": "Visual Telecommunication Network, Inc",
    "24.16556350": "Samsung Electronics Co.,Ltd",
    "12.123917685047296": "KeFF Networks",
    "24.9753623": "Gpi Korea Inc.",
    "24.5395627": "REALTEK (a Realtek 8029 based PCI Card)",
    "20.220327874396160": "Shenzhen Lihewei Electronics Co.,Ltd.Hunan Branch",
    "24.848901": "Beijing Signalway Technologies Co.,Ltd",
    "24.4746683": "Vestel Elektronik San ve Tic. A.Ş.",
    "24.3305": "Bloomberg L.P.",
    "24.3304": "GuangZhou AnJuBao Co., Ltd",
    "24.3307": "CNMP Networks, Inc.",
    "24.3306": "aphona Kommunikationssysteme",
    "24.3301": "ARRIS Group, Inc.",
    "24.3300": "NeuroCom International, Inc.",
    "24.3303": "MediaTek Inc.",
    "24.3302": "Fortinet Inc.",
    "24.12382866": "Plantronics, Inc.",
    "24.3309": "Real Digital Media",
    "24.3308": "Spectracom Corp.",
    "20.277981935173632": "Oviss Labs Inc.",
    "12.71297884110848": "Arago Systems",
    "24.793080": "Apple, Inc.",
    "24.16008584": "Huawei Technologies Co.,Ltd",
    "24.3180505": "Ruckus Wireless",
    "20.74781679419392": "Stara S/A Indústria de Implementos Agrícolas",
    "12.346860748800": "CompuShop Services LLC",
    "20.247076399087616": "BeSTAR Corporation",
    "24.14701975": "Emergent Vision Technologies Inc.",
    "20.31477541634048": "BMK professional electronics GmbH",
    "24.819581": "EEP Elektro-Elektronik Pranjic GmbH",
    "20.97400605638656": "Shenzhen C & D Electronics Co., Ltd.",
    "24.6073718": "Shenzhen TCL New Technology Co., Ltd",
    "12.123917687558144": "Okolab Srl",
    "12.346862923776": "NKS Co.Ltd.",
    "12.346862346240": "Actech",
    "12.123917679759360": "ard sa",
    "24.1850855": "Sigma Koki Co.,Ltd.",
    "24.10499705": "Huawei Technologies Co.,Ltd",
    "24.14982942": "ARRIS Group, Inc.",
    "24.13414997": "D-Link International",
    "24.16270876": "Sony Interactive Entertainment Inc.",
    "24.2636126": "Samsung Electronics Co.,Ltd",
    "12.346863247360": "Sicon srl",
    "24.472": "Teltronics, Inc.",
    "24.15996413": "Shanghai Pateo Electronic Equipment Manufacturing Co., Ltd.",
    "12.123917690425344": "Justek Inc",
    "24.28162": "Xovis AG",
    "12.123917683793920": "TripleOre",
    "12.346863616000": "Fritz Pauker Ingenieure GmbH",
    "24.7916163": "IEEE Registration Authority",
    "24.7382044": "Advanced Wireless Dynamics S.L.",
    "24.13914862": "BSkyB Ltd",
    "12.123917676888064": "AooGee Controls Co., LTD.",
    "24.3454973": "Guangzhou Younghead Electronic Technology Co.,Ltd",
    "20.181256790736896": "Grundig",
    "24.96": "Kontron Elektronik Gmbh",
    "24.97": "Gateway Communications",
    "24.94": "ICANN, IANA Department",
    "24.95": "Sumitomo Electric Industries, Ltd",
    "24.92": "Telematics International Inc.",
    "24.93": "Cs Telecom",
    "24.90": "SK",
    "24.91": "Eltec Elektronik Ag",
    "24.98": "Bull Hn Information Systems",
    "24.99": "Barco Control Rooms Gmbh",
    "12.346866831360": "Wotbox ltd",
    "12.346864173056": "AXED Jakubowski Wojciechowski sp.j.",
    "24.4211241": "Layer3TV, Inc",
    "24.13115534": "Storagedata",
    "20.88668805005312": "Changsha SinoCare, Inc",
    "12.346859532288": "DTech Labs Inc",
    "24.617": "Nadatel Co., Ltd",
    "24.616": "Harris Government Communications",
    "24.615": "Node Runner, Inc.",
    "24.614": "Thermalogic Corporation",
    "24.613": "Virditech Co. Ltd.",
    "24.2479": "e-generis",
    "24.611": "UPS Manufacturing SRL",
    "24.610": "Soyo Group Soyo Com Tech Co., Ltd",
    "24.2474": "Data Comm for Business, Inc.",
    "24.2475": "Netcontrol Oy",
    "24.2476": "Lanvoice",
    "24.2477": "Hyundai Syscomm, Inc.",
    "24.2470": "Ignis Optics, Inc.",
    "24.2471": "Bang & Olufsen A/S",
    "24.2472": "Eastmode Pte Ltd",
    "24.2473": "Ikanos Communications",
    "24.3937770": "Tescom Co., Ltd.",
    "24.5789130": "Apple, Inc.",
    "24.11811129": "Shenzhen TINNO Mobile Technology Corp.",
    "24.9985410": "Arista Networks",
    "24.5829532": "Cisco Systems, Inc",
    "24.7536": "Cisco Systems, Inc",
    "24.7537": "Cisco Systems, Inc",
    "24.7534": "Nokia Danmark A/S",
    "24.7535": "Chainzone Technology Co., Ltd",
    "24.7532": "ClariPhy Communications, Inc.",
    "24.7533": "Confidant International LLC",
    "24.7530": "Alpha Networks Inc.",
    "24.7531": "ARRIS Group, Inc.",
    "12.123917689987072": "ICP NewTech Ltd",
    "12.346868826112": "RTT Mobile Interpretation",
    "24.7538": "Wistron Corporation",
    "24.7539": "Buffalo.Inc",
    "20.31657718448128": "Pga Electronic",
    "12.123917684510720": "Equatel",
    "24.10244058": "Apple, Inc.",
    "24.12340796": "Core Staff Co., Ltd.",
    "24.13658576": "Verto Medical Solutions, LLC",
    "24.6333623": "TP-Link Corporation Limited",
    "20.31368095465472": "Silora R&D",
    "12.123917686763520": "CODEC Co., Ltd.",
    "24.11025611": "Rossma",
    "20.228844750503936": "Aim",
    "24.5970": "DAGS, Inc",
    "24.5971": "nFore Technology Inc.",
    "24.5972": "Arkino HiTOP Corporation Limited",
    "24.5973": "GE Security",
    "24.5974": "Vinci Labs Oy",
    "24.5975": "Rix Technology Limited",
    "24.5976": "ThruVision Ltd",
    "24.5977": "Cisco Systems, Inc",
    "24.5978": "Cisco Systems, Inc",
    "24.5979": "ACS Solutions Switzerland Ltd.",
    "24.14987269": "Beijing IQIYI Science & Technology Co., Ltd.",
    "20.137141734604800": "Fossil Power Systems Inc",
    "12.346856341504": "Weinberger Deutschland GmbH",
    "24.12059397": "Intel Corporate",
    "24.12096722": "Retail Innovation HTT AB",
    "24.2900385": "Bose Corporation",
    "24.13682136": "Latecoere",
    "24.16053723": "Xiaomi Communications Co Ltd",
    "24.13401974": "Cisco Systems, Inc",
    "20.124214339174400": "Kontech Electronics Co., Ltd",
    "24.9708398": "Shenzhen Junlan Electronic Ltd",
    "20.229558881091584": "Shenzhen xiaosha Intelligence Technology Co. Ltd",
    "12.123917682278400": "iTelaSoft Pvt Ltd",
    "24.7347464": "99IOT Shenzhen co.,ltd",
    "24.12088768": "PayPal, Inc.",
    "12.123917675683840": "Pa Consulting Services",
    "24.9744112": "Cisco Systems, Inc",
    "20.136646474334208": "DaLian Cheering Tech Co.,Ltd",
    "24.14418039": "Tp-Link Technologies Co.,Ltd.",
    "12.346866753536": "Cerner Corporation",
    "24.8950118": "Viewcooper Corp.",
    "24.4493664": "Murata Manufacturing Co., Ltd.",
    "12.71297883537408": "Energy Team S.p.A.",
    "24.10779300": "ARRIS Group, Inc.",
    "24.8353": "Dovatron",
    "24.8352": "Oa Laboratory Co., Ltd.",
    "24.8351": "Mercury Computer Systems, Inc.",
    "24.8350": "Brown'S Operating System Services, Ltd.",
    "24.8357": "Api Engineering",
    "24.8356": "Multipoint Networks",
    "24.8355": "Harmonic, Inc",
    "24.8354": "Galcom Networking Ltd.",
    "24.8359": "Pairgain Technologies, Inc.",
    "24.8358": "Proxim Inc",
    "24.7435": "Power Standards Lab",
    "20.149945502924800": "Red Technologies, LLC.",
    "12.123917677547520": "Sorion Electronics ltd",
    "12.346865373184": "HJPC Corporation dba Pactron",
    "20.70559713984512": "Beijing Kuaiyu Electronic Co., Ltd.",
    "24.10549321": "Cisco Systems, Inc",
    "12.123917679304704": "HYOSUNG Power & Industrial Systems",
    "24.6059669": "Technicolor CH USA Inc.",
    "24.5769701": "Kivic Inc.",
    "24.6616763": "Fibocom Wireless Inc.",
    "20.31873088618496": "HiHi Ltd",
    "24.12068843": "Raspberry Pi Foundation",
    "24.1313204": "zte corporation",
    "20.136823635443712": "FLEXIM GmbH",
    "24.7430": "HM Electronics, Inc.",
    "24.8127326": "Xiaomi Communications Co Ltd",
    "24.12868599": "ants",
    "20.110373795528704": "Shenzhen Jie Shi Lian Industrial Co., LTD",
    "24.13156668": "RuggedTek Hangzhou Co., Ltd",
    "24.3192365": "Mojo Networks, Inc.",
    "24.4758485": "Extreme Networks, Inc.",
    "24.15206024": "Apple, Inc.",
    "24.6063238": "Brightsource Industries Israel LTD",
    "24.6388": "EO TECHNICS Co., Ltd.",
    "24.5794580": "Apple, Inc.",
    "24.6389": "Shenzhen Streaming Video Technology Company Limited",
    "24.14445330": "worldcns inc.",
    "24.524404": "Casio Computer Co. Ltd.",
    "24.2607": "Artnix Inc.",
    "24.524406": "Pc Lan Technologies",
    "24.524407": "TSL (now Retix)",
    "24.524400": "Mitsubishi Precision Co.,LTd.",
    "24.524401": "Matra (Dsie)",
    "24.524402": "Xerox Corp Univ Grant Program",
    "24.2606": "Maple Networks Co., Ltd",
    "24.1586641": "ARRIS Group, Inc.",
    "24.327329": "Fihonest communication co.,Ltd",
    "24.2097839": "Murata Manufacturing Co., Ltd.",
    "24.524409": "The Droid Works",
    "24.2125105": "Firewalla Inc",
    "24.3960645": "Fiberpro Inc.",
    "24.2604": "Active Tchnology Corporation",
    "24.2603": "Etherstuff",
    "24.14706296": "Apple, Inc.",
    "24.2602": "QSI Systems Inc.",
    "24.6093626": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.2601": "Pan Dacom Networking AG",
    "24.11567244": "Laser Light Engines",
    "24.2600": "Motorola",
    "24.14420244": "Talk-A-Phone Co.",
    "24.12378435": "Apple, Inc.",
    "24.16259687": "Tp-Link Technologies Co.,Ltd.",
    "24.36892": "mps Software Gmbh",
    "24.36893": "Pec (Nz) Ltd.",
    "24.36890": "Unisphere Solutions",
    "24.36891": "Digital Controls",
    "24.36896": "Philips Analytical X-Ray B.V.",
    "24.36897": "Cisco Systems, Inc",
    "24.36894": "Selesta Ingegneria S.p.A.",
    "24.36895": "Adtec Productions, Inc.",
    "20.22739799769088": "Ntc Soft",
    "24.36898": "Ivex",
    "24.36899": "Zilog Inc.",
    "24.4764253": "Huawei Technologies Co.,Ltd",
    "24.180364": "3Com",
    "12.346857684992": "Zetera",
    "24.3731765": "SensorTec-Canada",
    "24.14994589": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.11799238": "DEXON Systems Ltd.",
    "24.14952305": "Smartlabs",
    "12.346865664000": "Industrial Control Links, Inc.",
    "24.3154691": "Belkin International Inc.",
    "12.346852929536": "MAZet GmbH",
    "24.5303153": "Intel Corporate",
    "12.71297883746304": "MeshWorks Wireless Oy",
    "24.1892007": "Cisco Systems, Inc",
    "24.15752311": "Google, Inc.",
    "12.123917677711360": "Cello Electronics (UK) Ltd",
    "20.93020690055168": "Royal Boon Edam International BV",
    "12.123917687033856": "Dadacon GmbH",
    "24.1880055": "D-Link International",
    "20.277783360045056": "Karry Communication Limited",
    "24.1332878": "Samsung Electronics Co.,Ltd",
    "24.11277869": "Hewlett Packard",
    "12.346858106880": "Eaton Corp. Electrical Group Data Center Solutions - Pulizzi",
    "24.9486790": "Apple, Inc.",
    "24.2632140": "Corsa Technology Incorporated",
    "12.123917675462656": "Lumiplan Duhamel",
    "12.123917683204096": "SDK Kristall",
    "12.346853761024": "Pantec Engineering AG",
    "12.123917678546944": "Parent Power",
    "24.1599960": "Tuya Smart Inc.",
    "24.1599962": "China Mobile Group Device Co.,Ltd.",
    "24.1552": "Abeona Networks Inc",
    "24.41071": "Color Sentinel Systems, LLC",
    "12.71297885003776": "Peek Traffic",
    "24.1553": "Zeus Wireless, Inc.",
    "12.123917679960064": "Sinftech LLC",
    "24.1550": "IGYS Systems, Inc.",
    "24.12354437": "Microsoft Corporation",
    "24.1551": "Narad Networks Inc",
    "12.346862170112": "Ti2000 Tecnologia Informatica 2000",
    "24.1556": "Prism Holdings",
    "24.1557": "Kimoto Electric Co., Ltd.",
    "24.1554": "Accusys, Inc.",
    "24.1555": "Kawasaki Microelectronics Incorporated",
    "20.154794726522880": "PTYPE Co., LTD.",
    "24.2154425": "Google, Inc.",
    "24.3687756": "Light Labs, Inc.",
    "24.589070": "Samsung Electronics Co.,Ltd",
    "24.11565460": "Motorola Mobility LLC, a Lenovo Company",
    "24.6050334": "Nintendo Co.,Ltd",
    "12.123917686714368": "Vtron Pty Ltd",
    "24.8703075": "Huawei Device Co., Ltd.",
    "24.49186": "Lasermaster Technologies, Inc.",
    "24.49187": "Tutankhamon Electronics",
    "24.49184": "Arco Electronic, Control Ltd.",
    "12.346863845376": "Teledyne Defence Limited",
    "24.49182": "La Francaise Des Jeux",
    "24.49183": "S.E.R.C.E.L.",
    "24.49180": "Interlink Communications Ltd.",
    "24.49181": "Grand Junction Networks, Inc.",
    "24.4267": "Koito Electric Industries, Ltd.",
    "20.31368178302976": "SMARTMOVT TECHNOLOGY Co.， LTD",
    "24.49188": "Eden Sistemas De Computacao Sa",
    "24.49189": "Dataproducts Corporation",
    "20.13592325783552": "Hunan Newman Car NetworKing Technology Co.,Ltd",
    "12.346858954752": "CC Systems AB",
    "24.14194400": "Alibaba Cloud Computing Ltd.",
    "24.13902541": "Xsens Technologies B.V.",
    "24.8950927": "Xi'an Zhisensor Technologies Co.,Ltd",
    "24.20731": "Vsk Electronics",
    "24.5535184": "Cisco Systems, Inc",
    "12.346856202240": "M5 Data Limited",
    "20.83999495028736": "Beijing Bluehalo Internet Inc.",
    "24.277005": "SM Optics S.r.l.",
    "20.119709250879488": "Aaon",
    "24.6044229": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.8029424": "McDowell Signal Processing, LLC (dba McDSP)",
    "24.20732": "Edimax Technology Co. Ltd.",
    "24.6044221": "zte corporation",
    "24.15528502": "Neomontana Electronics",
    "12.346856288256": "PAT Industries, DBA Pacific Advanced Technology",
    "20.18540134400000": "Cal-Comp Industria E Comercio De Eletronicos E Informatica Ltda",
    "12.346862784512": "Isac Srl",
    "24.5772243": "Gemtek Technology Co., Ltd.",
    "24.15246944": "Huawei Technologies Co.,Ltd",
    "24.20733": "Visioncomm Co., Ltd.",
    "20.75247161180160": "Zhejiang Ship Electronics & Technology Co., Ltd.",
    "12.123917686837248": "Analogics Tech India Ltd",
    "24.11061306": "Huawei Technologies Co.,Ltd",
    "24.6347672": "Calix Inc.",
    "24.11285998": "Fn-Link Technology Limited",
    "24.9211176": "Intel Corporate",
    "12.346866032640": "WellSense Technologies",
    "24.9482548": "ARRIS Group, Inc.",
    "20.176189948821504": "Friday Lab, UAB",
    "24.12377538": "Espressif Inc.",
    "24.6344326": "Sercomm Corporation.",
    "12.123917690130432": "Domus S.C.",
    "20.128612177018880": "Shenzhen Ruishi Information Technology Co.,Ltd.",
    "20.176726064758784": "ManTech International Corporation",
    "12.123917688487936": "Dat-Con d.o.o.",
    "20.220092027633664": "Repp Health",
    "24.5560145": "Proximus",
    "12.123917687291904": "Al Kamel Systems S.L.",
    "24.8392228": "ForgetBox",
    "20.61822318149632": "OOO NPP Uraltechnologiya",
    "24.32980": "Chase Limited",
    "24.32981": "Cadre Technologies",
    "24.32982": "Nuvotech, Inc.",
    "24.32983": "Fantum Electronics",
    "24.32984": "Network Peripherals",
    "24.32985": "EMK Elektronik GmbH & Co. KG",
    "24.32986": "Bruel & Kjaer",
    "24.32987": "Graphon Corporation",
    "24.32988": "Picker International",
    "24.32989": "Gmx Inc/Gimix",
    "12.123917684178944": "SenseNL",
    "12.346860486656": "Network Harbor, Inc.",
    "12.346867965952": "General Industrial Controls Pvt Ltd",
    "24.2124775": "Tp-Link Technologies Co.,Ltd.",
    "12.123917676613632": "Dave Srl",
    "24.9484496": "Joyent, Inc.",
    "24.11331707": "Huawei Technologies Co.,Ltd",
    "20.114975475499008": "femrice",
    "12.123917685821440": "RF Code",
    "20.57194439507968": "Harbin Yantuo Science and Technology Development Co., Ltd",
    "12.346853888000": "Norco",
    "12.123917687746560": "GD Mission Systems",
    "12.346853076992": "Innovation Institute, Inc.",
    "24.11330259": "Hewlett Packard",
    "12.346864508928": "MireroTack",
    "24.2924539": "Wistron Neweb Corporation",
    "20.260167872544768": "flexlog GmbH",
    "24.12863295": "Huawei Technologies Co.,Ltd",
    "12.123917675970560": "Acs Motion Control",
    "24.49374": "Zcomm, Inc.",
    "24.11033644": "Apple, Inc.",
    "12.123917688664064": "Waversa Systems",
    "20.66219398922240": "Vanstone Electronic (Beijing)Co,. Ltd.",
    "24.13893997": "Tp-Link Technologies Co.,Ltd.",
    "24.5263402": "Egardia",
    "24.10237940": "Huawei Technologies Co.,Ltd",
    "20.114975476547584": "Omniimpex GmbH",
    "24.15004612": "Samsung Electronics Co.,Ltd",
    "12.123917677137920": "SmartNodes",
    "12.123917682655232": "Kwant Controls BV",
    "24.6319559": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.11295252": "Samsung Electronics Co.,Ltd",
    "20.79215078670336": "Xiamen Rongta Technology Co.,Ltd.",
    "24.4508942": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "24.16559978": "Qulsar Inc",
    "24.6317061": "Texas Instruments",
    "24.12600065": "Epec Oy",
    "24.16534996": "Universal Global Scientific Industrial Co., Ltd.",
    "24.15208754": "Samsung Electronics Co.,Ltd",
    "24.8448662": "Apple, Inc.",
    "24.8944177": "Velankani Electronics Pvt. Ltd.",
    "24.10537637": "Intelligence Technology Of Cec Co., Ltd",
    "20.35227545108480": "Shenzhen Zhangyue Technology Co.,Ltd",
    "24.5005694": "Huawei Device Co., Ltd.",
    "24.8153938": "Tigaro Wireless",
    "12.123917679919104": "Tecogen Inc.",
    "12.346861174784": "Veeco Process Equipment, Inc.",
    "20.189244301639680": "Shenzheng SenseTime Technology Co. Ltd",
    "24.10526940": "Huawei Device Co., Ltd.",
    "24.16296586": "Axview Technology (Shenzhen) Co.,Ltd",
    "12.123917680463872": "SITA Messtechnik GmbH",
    "24.15785186": "Apple, Inc.",
    "20.220092040216576": "Fränkische Rohrwerke Gebr. Kirchner GmbH & Co. KG",
    "24.12231376": "Wireless Broadband Alliance Ltd",
    "24.5258657": "Samsung Electronics Co.,Ltd",
    "12.123917681405952": "Nubewell Networks Pvt Ltd",
    "24.2684059": "Leetek",
    "12.346864066560": "GREEN Center s.r.o.",
    "12.123917676597248": "R&D Gran-System-S LLC",
    "24.14743623": "Apple, Inc.",
    "24.10499099": "Adero Inc",
    "24.8410218": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.4854": "Mdk Co.,Ltd.",
    "24.12582912": "Western Digital (may be reversed 00 00 C0?)",
    "24.9212470": "Aus.Linx Technology Co., Ltd.",
    "24.5264781": "China Mobile IOT Company Limited",
    "24.2898664": "Espressif Inc.",
    "12.123917683380224": "SYS TEC electronic GmbH",
    "24.12374373": "Cisco SPVTG",
    "24.5557520": "Panasonic Mobile Communications Co.,Ltd.",
    "12.123917686829056": "DSIT Solutions LTD",
    "24.4216365": "Smartron India Pvt ltd",
    "12.123917689917440": "Thinnect, Inc,",
    "24.10016503": "LG Electronics (Mobile Communications)",
    "20.48476172845056": "Curtiss-Wright Drive Technology",
    "24.12616537": "IEEE Registration Authority",
    "24.1071863": "ADB Italia",
    "24.16276283": "Askey Computer Corp",
    "12.346868334592": "Pro Design Electronic GmbH",
    "24.4240833": "Cisco Systems, Inc",
    "24.13419786": "Shenzhen Dajiahao Technology Co.,Ltd",
    "24.9753861": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346862649344": "Technovare Systems",
    "24.315276": "XIAOMI Electronics,CO.,LTD",
    "12.346864893952": "Qualnetics Corporation",
    "24.9498706": "MEIZU Technology Co., Ltd.",
    "12.123917683945472": "Entec Solar S.L.",
    "12.123917689053184": "Ascendent Technology Group",
    "24.5298688": "Hangzhou H3C Technologies Co., Limited",
    "12.123917683789824": "KST technology",
    "24.34917": "Apple, Inc.",
    "12.346853519360": "HECUBA Elektronik",
    "12.346864984064": "Sicon srl",
    "12.346852405248": "VNR Electronique SA",
    "24.13901580": "JAI Manufacturing",
    "24.13686659": "Huawei Technologies Co.,Ltd",
    "12.123917678792704": "Comark Interactive Solutions",
    "24.10231128": "Texas Instruments",
    "12.123917680951296": "LM-Instruments Oy",
    "20.119709249830912": "Greenbird Vertriebs GmbH",
    "24.13692865": "Huawei Technologies Co.,Ltd",
    "12.346858364928": "Micro Elektronische Producten",
    "24.5827809": "Newport Corporation",
    "12.346865057792": "Lancier Monitoring GmbH",
    "20.184991250120704": "Laonz Co.,Ltd",
    "20.260167866253312": "WisIOE",
    "24.814671": "PrimeVOLT Co., Ltd.",
    "24.13426883": "EXTEN Technologies, Inc.",
    "24.11328741": "Procter & Gamble Company",
    "24.7340342": "FATEK Automation Corporation",
    "24.12341758": "Swann communications Pty Ltd",
    "24.16041928": "Kelvin Inc.",
    "12.123917675634688": "Akse srl",
    "24.8688396": "InCoax Networks Europe AB",
    "24.12059669": "Bayan Audio",
    "12.346861834240": "Mosaic Dynamic Solutions",
    "24.12332440": "ThinGlobal LLC",
    "24.293387": "Beijing Xiaomi Electronics Co., Ltd.",
    "24.47283": "Cisco Systems, Inc",
    "24.12846723": "Huawei Technologies Co.,Ltd",
    "12.346866216960": "KLING & FREITAG GmbH",
    "24.47286": "Motorola Mobility LLC, a Lenovo Company",
    "24.2363492": "Shenzhen Ecsino Tecnical Co. Ltd",
    "24.540915": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.2131347": "IEEE Registration Authority",
    "24.2131340": "Assa",
    "24.3728789": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917689614336": "ISG Nordic AB",
    "24.6979": "Beijing DG Telecommunications equipment Co.,Ltd",
    "24.6978": "Wise & Blue",
    "24.6975": "ProCurve Networking by HP",
    "24.6974": "Curtis, Inc.",
    "24.6977": "General Infinity Co.,Ltd.",
    "24.6976": "Network Automation mxc AB",
    "24.6971": "Yi-Qing CO., LTD",
    "24.6970": "SIMS Corp.",
    "24.6973": "EuroTel Spa",
    "24.6972": "Software Technologies Group,Inc.",
    "24.5408": "Radiocrafts AS",
    "24.5409": "Horoquartz",
    "24.9762592": "Tianjin Deviser Electronics Instrument Co., Ltd",
    "24.5404": "Leneco",
    "24.5405": "M2I Corporation",
    "24.1987": "Thomson",
    "24.5407": "Multivision Intelligent Surveillance (Hong Kong) Ltd",
    "24.5400": "Shenzhen 10MOONS Technology Development CO.,Ltd",
    "24.285838": "gosund GROUP CO.,LTD",
    "24.5402": "Hunter Engineering Company",
    "24.1982": "DataLogic SpA",
    "24.14200878": "Guangzhou Zonerich Business Machine Co., LTD.",
    "24.6831668": "MEIZU Technology Co., Ltd.",
    "24.803489": "Mellanox Technologies, Inc.",
    "20.154019628580864": "Nanjing bilian information Technology Co.,Ltd.",
    "12.346868215808": "BAP Precision Ltd.",
    "24.9497719": "silicom",
    "24.13120952": "Ericsson, EAB/RWI/K",
    "24.1852396": "Japan Circuit Co.,Ltd",
    "24.5829782": "Source Chain",
    "24.10241085": "Chengdu Xuguang Technology Co, Ltd",
    "12.71297883287552": "Audio Enhancement",
    "24.17407": "Ketron S.R.L.",
    "20.159304575352832": "Power Engineering & Manufacturing, Inc.",
    "12.123917691052032": "Hunan Lianzhong Technology Co.,Ltd.",
    "24.5800477": "H. Schomäcker GmbH",
    "12.123917679820800": "Xgem Sas",
    "24.4859": "Samsung Electronics Co.,Ltd",
    "24.8932074": "Texas Instruments",
    "24.23457": "shanghai huayuan chuangxin software CO., LTD.",
    "12.346859761664": "MEDAV GmbH",
    "24.15232338": "ARRIS Group, Inc.",
    "20.220958196498432": "Volansys technologies pvt ltd",
    "20.62366870929408": "Aina Wireless Inc",
    "12.123917675732992": "ndb technologies",
    "20.229206574235648": "ENCORED Technologies, Inc.",
    "24.6870146": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.346859040768": "Bertronic Srl",
    "24.9976719": "Intel Corporate",
    "24.13431815": "Eukrea Electromatique Sarl",
    "24.12630758": "Application Solutions (Electronics and Vision) Ltd",
    "24.3428809": "Apple, Inc.",
    "12.346860040192": "NORMA systems GmbH",
    "24.6580852": "Auth-Servers, Llc",
    "24.7659850": "IEEE Registration Authority",
    "20.268341260320768": "Zeepro Inc.",
    "24.3806": "REMEC, Inc.",
    "24.3807": "PLX Technology",
    "24.3804": "Tellion INC.",
    "24.3805": "Shure Incorporated",
    "24.3802": "C-Tech United Corp.",
    "24.3803": "XiNCOM Corp.",
    "24.3800": "Positron Access Solutions Corp",
    "24.3801": "Aksys, Ltd.",
    "12.346859974656": "Pointe Conception Medical Inc.",
    "24.3808": "Mcharge",
    "24.3809": "ExtremeSpeed Inc.",
    "24.13898198": "ShotSpotter, Inc.",
    "24.13155865": "D-Link International",
    "12.123917688987648": "Kronotech Srl",
    "24.10512756": "Honor Device Co., Ltd.",
    "24.4465": "DEXTER Communications, Inc.",
    "24.4464": "Gsc Srl",
    "24.4467": "SMART Storage Systems",
    "24.4466": "Cotron Corporation",
    "24.4461": "American Time and Signal",
    "24.4460": "Nanwang Multimedia Inc.,Ltd",
    "24.4463": "Netforyou Co., LTD.",
    "24.4462": "Peplink International Ltd.",
    "24.4734938": "Espressif Inc.",
    "24.10798414": "Huawei Device Co., Ltd.",
    "24.4469": "Intel Corporation",
    "24.4468": "Mojo Networks, Inc.",
    "24.15761164": "Leopold Kostal GmbH &Co. KG",
    "12.123917685882880": "Pneumax S.p.A.",
    "20.229108129726464": "North American Blue Tiger Company, LLC",
    "12.123917679321088": "Redler Computers",
    "24.13473": "Rf-Lambda Usa Inc.",
    "12.346867679232": "Saia-Burgess Controls AG",
    "12.71297883852800": "WooshCom Corporation",
    "24.7122016": "Venetex Corp",
    "24.5042690": "Xiaomi Communications Co Ltd",
    "20.194363555577856": "Risecomm (Hk) Technology Co. Limited",
    "24.2627534": "Omnisense Ltd",
    "12.346861772800": "Rohde&Schwarz Topex SA",
    "24.574945": "Samsung Electro-Mechanics(Thailand)",
    "24.11583385": "Wizitdongdo",
    "24.7354606": "Avaya Inc",
    "12.346853933056": "BLANKOM Antennentechnik GmbH",
    "12.123917681324032": "Ateme",
    "24.4470553": "2 Save Energy Ltd",
    "24.15216140": "NETINT Technologies Inc.",
    "24.9961670": "Apple, Inc.",
    "12.123917686833152": "DWQ Informatikai Tanacsado es Vezerlestechnikai KFT",
    "24.8973": "Nortel Networks",
    "24.8972": "Clover Electronics Co.,Ltd.",
    "24.8971": "ARRIS Group, Inc.",
    "24.8970": "ARBURG GmbH & Co KG",
    "24.8977": "Gloscom Co., Ltd.",
    "24.8976": "LNC Technology Co., Ltd.",
    "24.8975": "Hirsch Electronics Corporation",
    "24.8974": "Gorba AG",
    "24.13382294": "Huawei Device Co., Ltd.",
    "24.8979": "Qool Technologies Ltd.",
    "24.8978": "Apple, Inc.",
    "24.1865806": "China Mobile Iot Limited company",
    "12.346858590208": "Version-T",
    "24.3114": "OCTTEL Communication Co., Ltd.",
    "24.3115": "ELIAS Technology, Inc.",
    "24.3116": "Enwiser Inc.",
    "24.3117": "FullWave Technology Co., Ltd.",
    "24.3110": "Weintek Labs. Inc.",
    "24.3111": "Sammy Corporation",
    "24.3112": "Rifatron",
    "24.3113": "VMware, Inc.",
    "12.346865348608": "SENSAIR Pty Ltd",
    "12.123917683535872": "SP Controls",
    "24.3118": "Openet information technology(shenzhen) Co., Ltd.",
    "24.3119": "SeorimTechnology Co.,Ltd.",
    "24.2653403": "Hefei Toycloud Technology Co.,ltd",
    "24.3180336": "Huawei Technologies Co.,Ltd",
    "24.4749406": "Amazon Technologies Inc.",
    "24.6575537": "ARRIS Group, Inc.",
    "12.123917682065408": "EA Elektroautomatik GmbH & Co. KG",
    "20.31614739415040": "Kids Wireless Inc",
    "12.346857222144": "ELPROC sp. z o.o.",
    "12.119269761024": "TGS Geophysical Company (UK) Limited",
    "24.547598": "Fortinet, Inc.",
    "24.8925": "Protecta Electronics Ltd",
    "20.57194449993728": "ND SatCom GmbH",
    "12.123917688897536": "Crde",
    "24.10776185": "DFT System Co.Ltd",
    "24.2686782": "zte corporation",
    "24.2682200": "JAG Jakob AG",
    "24.2686780": "Apple, Inc.",
    "24.6566": "ARRIS Group, Inc.",
    "24.6567": "Itu-T",
    "24.6564": "Austar Technology (hang zhou) Co.,Ltd",
    "24.6565": "RadarFind Corporation",
    "24.6562": "Ordyn Technologies",
    "24.6563": "asteel electronique atlantique",
    "24.6560": "Nihon Data Systens, Inc.",
    "24.6561": "Lg Information & Comm.",
    "24.6292232": "Apple, Inc.",
    "20.132869118754816": "Insignal Co., Ltd.",
    "24.6568": "WiQuest Communications",
    "24.6569": "Cisco Systems, Inc",
    "12.346865758208": "Armour Home Electronics LTD",
    "12.346855186432": "Ballard Technology, Inc,",
    "12.71297884520448": "Seiko Time Systems Inc.",
    "24.40996": "3Com",
    "12.346860703744": "Funkwerk Information Technologies Karlsfeld GmbH",
    "24.9459793": "Technicolor CH USA Inc.",
    "24.1622719": "Buffalo.Inc",
    "24.14205032": "Netgenetech.co.,ltd.",
    "24.8182260": "UAB \"Teltonika Telematics\"",
    "24.14205034": "Hunantv.com Interactive Entertainment Media Co.,Ltd.",
    "24.7136759": "New H3C Technologies Co., Ltd",
    "24.13902414": "Alcatel Lucent",
    "12.346867445760": "Perceptive Pixel Inc.",
    "24.13913255": "Informtekhnika & Communication, LLC",
    "24.16551515": "China Mobile Iot Limited company",
    "20.58071046946816": "Blustream Pty Ltd",
    "24.6691": "Ice Qube, Inc",
    "12.346862243840": "Powersense A/S",
    "24.6693": "Delta Dore",
    "24.6694": "Deltanode Solutions AB",
    "24.6695": "Ubistar",
    "24.6696": "ASWT Co., LTD. Taiwan Branch H.K.",
    "24.6697": "Johnson Outdoors Marine Electronics d/b/a Minnkota",
    "24.6698": "Arcadyan Technology Corporation",
    "24.6699": "Ayecom Technology Co., Ltd.",
    "24.798": "Optranet, Inc.",
    "24.799": "Condev Ltd.",
    "24.792": "Cyras Systems, Inc.",
    "24.793": "Infineon AG",
    "24.790": "Nobell Communications, Inc.",
    "24.791": "Merlin Systems, Inc.",
    "24.796": "Svenska Hardvarufabriken AB",
    "24.797": "Taiwan Commate Computer, Inc.",
    "24.794": "Beijing Broad Telecom Ltd., China",
    "24.795": "Cellvision Systems, Inc.",
    "20.71389211000832": "TeraTron GmbH",
    "24.1331582": "New H3C Technologies Co., Ltd",
    "24.12106354": "Cynove",
    "24.8175537": "ARRIS Group, Inc.",
    "24.424": "Welltech Computer Co., Ltd.",
    "24.425": "Bmw Ag",
    "24.2157940": "Ability enterprise co.,Ltd.",
    "24.427": "Main Street Networks",
    "24.420": "Microlink Corporation",
    "24.421": "Nextcomm, Inc.",
    "24.422": "Scientific-Atlanta Arcodan A/S",
    "24.423": "Unex Technology Corporation",
    "24.428": "Sitara Networks, Inc.",
    "12.346866384896": "Dumps Electronic",
    "24.9769": "Juphoon System Software Inc.",
    "24.9768": "companytec automação e controle ltda.",
    "24.1514": "Rednix",
    "24.1515": "Blue Ridge Networks, Inc.",
    "24.1512": "TurboWave, Inc.",
    "24.1513": "Unicess Network, Inc.",
    "24.1510": "Egenera, Inc.",
    "24.1511": "Netrake an AudioCodes Company",
    "24.9761": "InteliCloud Technology Inc.",
    "24.9760": "ISGUS GmbH",
    "24.9763": "JRD Communication Inc",
    "24.9762": "Compal Information (Kunshan) Co., Ltd.",
    "24.9765": "MediaSputnik",
    "24.9764": "Thomson Inc.",
    "24.9767": "Truesell",
    "24.9766": "Geophysical Survey Systems, Inc.",
    "24.10502471": "Huawei Device Co., Ltd.",
    "24.6876483": "BlackBerry RTS",
    "12.119269867520": "S.I.C.E.S. srl",
    "12.346864259072": "ENTEC Electric & Electronic Co., LTD.",
    "24.2637028": "Huawei Technologies Co.,Ltd",
    "12.123917681078272": "ECoCoMS Ltd.",
    "12.346860986368": "USM Systems, Ltd",
    "12.123917689167872": "Sun Electronics Co.,Ltd.",
    "12.123917683568640": "Shangnuo company",
    "24.11838187": "IEEE Registration Authority",
    "24.8403754": "ABB Xiamen Low Voltage Equipment Co.,Ltd.",
    "24.849013": "Zliide Technologies ApS",
    "20.35758071087104": "Mid Continent Controls, Inc.",
    "24.4766286": "Hewlett Packard",
    "20.212188448948224": "Dongguan Chuan OptoElectronics Limited",
    "24.811126": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1897959": "Turtle Industry Co., Ltd.",
    "20.220515732029440": "Aventics GmbH",
    "20.180659907723264": "Htd",
    "20.216039224770560": "Robostar",
    "24.24741": "Performance Telecom Corp.",
    "24.6064596": "Beijing Banner Electric Co.,Ltd",
    "24.5804557": "Kinetic Avionics Limited",
    "12.346855550976": "MultimediaLED",
    "24.4516200": "Huawei Technologies Co.,Ltd",
    "12.123917686165504": "MB connect line GmbH Fernwartungssysteme",
    "24.15791721": "Motion Analysis Corporation",
    "20.106480348954624": "Ameli s.r.l.",
    "24.15207541": "Syncbak, Inc.",
    "20.137141738799104": "Annapurna labs",
    "24.8702129": "Incognito Software Systems Inc.",
    "24.4493787": "Shanghai Huaqin Telecom Technology Co.,Ltd",
    "24.14705851": "DigiView S.r.l.",
    "24.16027934": "Zhuhai Ewpe Information Technology Inc",
    "20.220515719446528": "Rotronic Ag",
    "24.8498": "Masterclock, Inc.",
    "24.8499": "Building B, Inc",
    "12.123917686534144": "digital-spice",
    "24.8490": "Audiovox Corporation",
    "24.8491": "MSA Auer",
    "24.8492": "SemIndia System Private Limited",
    "12.346862145536": "Earnestcom Sdn Bhd",
    "24.8494": "dresden-elektronik",
    "24.8495": "Phoebe Micro Inc.",
    "24.8496": "Keico Hightech Inc.",
    "24.8497": "Blynke Inc.",
    "24.528790": "Intel Corporate",
    "24.10239658": "EnvyLogic Co.,Ltd.",
    "12.346866520064": "RF Code, Inc",
    "24.7910740": "Huawei Device Co., Ltd.",
    "24.9992724": "Microsoft Corporation",
    "24.7651264": "Canon Inc.",
    "24.9992720": "Ericsson AB",
    "24.14438178": "Espressif Inc.",
    "20.48548672438272": "Rutledge Omni Services Pte Ltd",
    "24.8142": "Qtech Llc",
    "24.8143": "MSI Technology GmbH",
    "24.8140": "Samsung Electronics Co.,Ltd",
    "24.8141": "Samsung Electronics Co.,Ltd",
    "24.8146": "Commtech Technology Macao Commercial Offshore Ltd.",
    "24.8147": "RIVA Networks Inc.",
    "24.8144": "Giga-Byte Technology Co.,Ltd.",
    "24.8145": "Optex Co.,Ltd.",
    "24.8148": "4Ipnet, Inc.",
    "24.8149": "MICRORISC s.r.o.",
    "12.346854588416": "ACD Elektronik GmbH",
    "12.123917684707328": "Triax A/S",
    "24.4878": "Focusrite Audio Engineering Limited",
    "24.4879": "EGEMEN Bilgisayar Muh San ve Tic LTD STI",
    "24.4876": "HF System Corporation",
    "24.4877": "Galileo Avionica",
    "24.4874": "Nortel Networks",
    "24.4875": "Mextal B.V.",
    "24.4872": "Nuvera Fuel Cells",
    "24.4873": "Ocean Broadband Networks",
    "24.4870": "Always On Wireless",
    "24.4871": "Paravirtual Corporation",
    "20.18540138594304": "Prolan Zrt.",
    "12.123917679341568": "Harman Connected Services Corporation India Pvt. Ltd.",
    "12.123917681197056": "Vitec",
    "24.10512412": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "20.228844764135424": "u::Lux GmbH",
    "24.8187035": "Fuzhou Teraway Information Technology Co.,Ltd",
    "24.669212": "Smardii",
    "24.573473": "Huawei Technologies Co.,Ltd",
    "24.3439": "Ember Corporation",
    "12.346864500736": "Wheatstone Corporation",
    "24.16525980": "Apple, Inc.",
    "24.14438132": "Shenzhen MTN Electronics CO., Ltd",
    "12.346856484864": "Avionica, Inc",
    "24.12620084": "ProCurve Networking by HP",
    "24.9185591": "Apple, Inc.",
    "24.12620082": "Patriot Memory",
    "24.459": "Evr",
    "20.212188456288256": "zxsolution",
    "20.75685683003392": "Appotronics Co., Ltd",
    "24.8660501": "ADB Broadband Italia",
    "12.123917689192448": "Nidec asi spa",
    "24.9799": "Wfe Technology Corp.",
    "12.346868736000": "Koyo Electric",
    "24.14959078": "Ottec Technology GmbH",
    "12.123917690671104": "Nanotok LLC",
    "24.5813795": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.2906597": "Hewlett Packard",
    "24.4760854": "Huawei Device Co., Ltd.",
    "24.3429436": "Takaoka Toko Co.,Ltd.",
    "24.14482248": "Intel Corporate",
    "24.12094048": "Intel Corporate",
    "24.5510259": "Wingtech Group (HongKong）Limited",
    "12.346857230336": "SEBA Design Pty Ltd",
    "24.304115": "Emicon",
    "24.6352476": "Shenzhen Fast Technologies Co.,Ltd",
    "24.3148281": "IEEE Registration Authority",
    "24.9790": "Trapeze Networks",
    "24.1069170": "Sercomm Corporation.",
    "12.346868543488": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "24.7659930": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.315706": "Xiaomi Communications Co Ltd",
    "12.346853302272": "DSP Design, Ltd.",
    "24.15753857": "ProCurve Networking by HP",
    "24.12599261": "Huawei Technologies Co.,Ltd",
    "24.6091632": "CC&C Technologies, Inc",
    "12.346864037888": "Prodco",
    "12.71297884168192": "SigmaPhi Electronics",
    "12.346855432192": "C3-ilex, LLC",
    "20.97843042844672": "Xiamen Leelen Technology Co.,Ltd",
    "24.11287143": "Cisco Systems, Inc",
    "20.251330462482432": "Private",
    "24.8919961": "Shenzhen Xin Fei Jia Electronic Co. Ltd.",
    "12.123917676969984": "TOPROOTTechnology Corp. Ltd.",
    "24.15224526": "Nintendo Co., Ltd.",
    "24.5306533": "eWBM Co., Ltd.",
    "24.16010757": "Cisco Systems, Inc",
    "24.14732536": "Xiaomi Communications Co Ltd",
    "24.13671546": "Xiaomi Communications Co Ltd",
    "24.13152798": "Chipsea Technologies (Shenzhen) Corp.",
    "12.346868400128": "Atonometrics, Inc.",
    "24.16531324": "Huawei Technologies Co.,Ltd",
    "24.12093063": "Liteon Technology Corporation",
    "24.5566276": "Lumi United Technology Co., Ltd",
    "24.6579355": "Juniper Networks",
    "24.15261682": "PRF Co., Ltd.",
    "12.123917685166080": "Aplex Technology Inc.",
    "24.6087308": "Zyxel Communications Corporation",
    "24.16519366": "Taicang T&W Electronics",
    "12.123917690941440": "Beijing Vizum Technology Co.,Ltd.",
    "12.346863509504": "Rafael",
    "20.154314888708096": "Beida Jade Bird Universal Fire Alarm Device CO.,LTD.",
    "24.12878703": "Ciena Corporation",
    "12.123917684645888": "Rohde&Schwarz Topex SA",
    "20.48835641475072": "ELKO EP, s.r.o.",
    "12.123917680824320": "Dorlet Sau",
    "24.14176184": "Lg Electronics",
    "20.4471495065600": "Julida Limited",
    "12.123917691658240": "Galileo Tıp Teknolojileri San. ve Tic. A.S.",
    "24.8669461": "vArmour Networks, Inc.",
    "12.123917682008064": "Hangzhou Weimu Technology Co,.Ltd.",
    "24.7380521": "Sony Interactive Entertainment Inc.",
    "24.8708184": "ARRIS Group, Inc.",
    "24.9192527": "HUMAX Co., Ltd.",
    "24.7401622": "Cisco Systems, Inc",
    "24.7340632": "01Db-Metravib",
    "24.8153451": "Atmosic Technologies",
    "12.123917688782848": "Microtronics Engineering GmbH",
    "24.2395104": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.16270075": "Yasuda Joho Co.,ltd.",
    "24.10018009": "UNITEC Co., Ltd.",
    "24.6300096": "Murata Manufacturing Co., Ltd.",
    "24.13108638": "Hefei Symboltek Co.,Ltd",
    "24.32829": "Surigiken Co., Ltd.",
    "24.32828": "Tvs Electronics Ltd",
    "24.10811344": "LAN Accessories Co., Ltd.",
    "24.2659326": "Cisco Systems, Inc",
    "24.32823": "Ericsson Business Comm.",
    "24.32822": "Reflex Manufacturing Systems",
    "24.32821": "Technology Works",
    "24.32820": "Smt Goupil",
    "24.32827": "Apt Communications, Inc.",
    "24.32826": "Varityper, Inc.",
    "24.32825": "Alcatel Stc Australia",
    "24.32824": "Data Research & Applications",
    "24.1616885": "Ultra Electronics Electrics",
    "24.4492735": "Rhino Mobility LLC",
    "24.5005525": "airPointe of New Hampshire",
    "24.5515096": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.123917688086528": "ENTEC Electric & Electronic Co., LTD.",
    "24.16286841": "Yaojin Technology(Shenzhen)Co.,Ltd",
    "12.123917689958400": "SL Audio A/S",
    "12.123917690281984": "Fuji Data System Co.,Ltd.",
    "24.24668": "Cisco Systems, Inc",
    "12.346854256640": "Product Design Dept., Sohwa Corporation",
    "24.10250942": "QEES ApS.",
    "24.6093563": "Acubit",
    "24.15205619": "Throughtek Co., Ltd.",
    "24.15469978": "Mellanox Technologies, Inc.",
    "20.162752346718208": "Sunthink S&T Development Co.,Ltd",
    "12.346862227456": "Cambustion Ltd",
    "12.346852839424": "Mobiltex Data Ltd.",
    "12.346868281344": "Boryeu Technology Co.,Ltd",
    "24.12867010": "Bach-Simpson",
    "24.5295352": "Samsung Electro Mechanics Co., Ltd.",
    "24.9700790": "Liling FullRiver Electronics & Technology Ltd",
    "24.16820": "Wuxi Zhongxing Optoelectronics Technology Co.,Ltd.",
    "24.9700795": "IEEE Registration Authority",
    "24.11293932": "IEEE P1823 Standards Working Group",
    "12.346859450368": "E3Switch LLC",
    "12.346860802048": "beroNet GmbH",
    "12.123917681520640": "Ineo-Sense",
    "24.9483485": "Private",
    "24.8969897": "Roku, Inc.",
    "24.8938560": "Apple, Inc.",
    "24.13423662": "Arcadyan Corporation",
    "20.45069251903488": "Airbus Defence and Space Oy",
    "24.4750638": "Tp-Link Technologies Co.,Ltd.",
    "20.48476177039360": "Beijing Zhijian Link Technology Co., Ltd.",
    "24.11599115": "IEEE Registration Authority",
    "12.123917676789760": "5nines",
    "12.123917686730752": "Hermann Automation GmbH",
    "12.123917683019776": "Triton Electronics Ltd",
    "24.846134": "ELIMOS srl",
    "24.6345482": "Quatius Limited",
    "24.16570391": "Beijing Hesun Technologies Co.Ltd.",
    "24.11598861": "Amazon Technologies Inc.",
    "24.13652121": "ASRock Incorporation",
    "24.13390260": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.9752108": "Sony Mobile Communications Inc",
    "12.346864582656": "b.a.b-technologie gmbh",
    "24.7890150": "Green Motive Technology Limited",
    "12.346853167104": "Spectra Technologies Holding Co., Ltd.",
    "12.71297884528640": "GD Mission Systems",
    "12.123917691461632": "KJ3 Elektronik AB",
    "24.5821706": "Murata Manufacturing Co., Ltd.",
    "12.123917679771648": "Integrated Systems Engineering, Inc.",
    "24.132865": "Racal-Datacom",
    "12.123917681676288": "Gega Electronique",
    "24.6039508": "Shenzhen Qihu Intelligent Technology Company Limited",
    "12.123917688848384": "G-Philos Co.,Ltd",
    "24.16016159": "Huawei Technologies Co.,Ltd",
    "24.16037811": "vivo Mobile Communication Co., Ltd.",
    "24.6612952": "Samsung Electronics Co.,Ltd",
    "24.8698729": "ARRIS Group, Inc.",
    "24.16045192": "Apple, Inc.",
    "12.346858094592": "Etap Nv",
    "24.9999308": "Lg Electronics Inc",
    "20.44213340209152": "Innovative Technology Ltd",
    "24.8696603": "Kinexon GmbH",
    "24.8447061": "New H3C Technologies Co., Ltd",
    "24.9199432": "Continental Intelligent Transportation Systems LLC",
    "20.167860148109312": "Beijing Tiandi-Marco Electro-Hydraulic Control System Company Ltd.",
    "24.15742881": "Huike Electronics (SHENZHEN) CO., LTD.",
    "12.346852716544": "Monitor Business Machines Ltd.",
    "24.8442251": "Hangzhou I'converge Technology Co.,Ltd",
    "12.346865254400": "Keysight Technologies Inc.",
    "24.13169433": "LG Electronics (Mobile Communications)",
    "12.346864963584": "Insitu, Inc",
    "20.106196531937280": "Jlztlink Industry(ShenZhen)Co.,Ltd.",
    "24.2679116": "Apple, Inc.",
    "24.1068968": "Sony Corporation",
    "12.346857357312": "O2Run",
    "20.124214329737216": "SHENZHEN Xin JiuNing Electronics Co Ltd",
    "12.123917682286592": "P&C Micro's Pty Ltd",
    "24.14160075": "Andrea Informatique",
    "24.12092804": "Dell Inc.",
    "24.15508123": "Kemppi Oy",
    "12.346859925504": "Eid",
    "20.189549213908992": "Micro Connect Pty Ltd",
    "24.41076": "Perception Technology",
    "24.41077": "Micron Technology, Inc.",
    "24.41074": "Ovation Systems Ltd.",
    "24.41075": "Com21, Inc.",
    "24.41072": "Coastcom",
    "24.41073": "Video Lottery Technologies,Inc",
    "24.41070": "Austron, Inc.",
    "12.346866528256": "Lauda Dr. R. Wobser Gmbh & Co. Kg",
    "24.14206103": "Quanta Computer Inc.",
    "24.41078": "Cardware Lab, Inc.",
    "24.41079": "Fujitsu Nexion, Inc.",
    "12.123917675069440": "Hanwa Electronic Ind.Co.,Ltd.",
    "12.123917686427648": "BeatCraft, Inc.",
    "12.119269322752": "cPacket Networks",
    "24.7134314": "Promethean Limited",
    "24.12618607": "S I Sistemas Inteligentes Eletrônicos Ltda",
    "12.346865479680": "DEUTA-WERKE GmbH",
    "24.13652311": "LEAX Arkivator Telecom",
    "12.71297883271168": "Franke Aquarotter GmbH",
    "24.7921923": "Amazon Technologies Inc.",
    "12.123917681471488": "Silicann Systems GmbH",
    "24.280271": "Beijing Venustech Cybervision Co.,Ltd.",
    "12.346863169536": "Phytec Messtechnik GmbH",
    "24.15209742": "Nokia Corporation",
    "24.3441349": "Samsung Electronics Co.,Ltd",
    "12.346855133184": "Pixy Ag",
    "12.346862198784": "IDT Sound Processing Corporation",
    "24.11038286": "Huawei Device Co., Ltd.",
    "12.346860974080": "Syntronics",
    "20.70443724701696": "Beijing Hexinruitong Electric Power Technology Co., Ltd.",
    "24.6293520": "Huawei Technologies Co.,Ltd",
    "24.5287795": "Shenzhen Bitland Information Technology Co., Ltd.",
    "24.3722503": "Beijing FaceCam Technology Co., Ltd.",
    "12.71297884225536": "Task Sistemas De Computacao S.A.",
    "24.4203970": "Intel Corporate",
    "24.4484837": "Huawei Technologies Co.,Ltd",
    "20.39672889737216": "Private",
    "24.9984138": "SYSGRATION Ltd.",
    "24.8939098": "Apple, Inc.",
    "24.8691141": "Centera Photonics Inc.",
    "24.1861229": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.225135230451712": "Ability Enterprise Co., Ltd",
    "20.224641219035136": "lesswire GmbH",
    "20.175931704475648": "Encroute AB",
    "24.5283495": "Samsung Electronics Co.,Ltd",
    "12.346860130304": "Aavd",
    "24.12345334": "Mercury Communication Technologies Co.,Ltd.",
    "24.16043590": "Hewlett Packard",
    "24.4740447": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.9493070": "Avanu",
    "24.11309618": "Huawei Technologies Co.,Ltd",
    "20.216039222673408": "Star2Star Communications, LLC",
    "12.123917682536448": "AVA Technologies Inc.",
    "24.9499050": "Samsung Electronics Co.,Ltd",
    "24.6858861": "Apple, Inc.",
    "24.1584308": "Nectarsoft Co., Ltd.",
    "24.57524": "Techno Scope Co., Ltd.",
    "20.272689491083264": "CLARUS Korea Co., Ltd",
    "24.45230": "Symmetricom",
    "24.9499056": "Hangzhou Anheng Info&Tech CO.,LTD",
    "24.45235": "Xstreamis Plc",
    "24.21937": "Shanghai Baud Data Communication Co.,Ltd.",
    "24.13399522": "ARRIS Group, Inc.",
    "12.123917681278976": "Surtec",
    "24.6557332": "Pluribus Networks, Inc.",
    "12.123917691731968": "Serveron Corporation",
    "24.7903": "Master Industrialization Center Kista",
    "24.7902": "Byd Company Limited",
    "24.7901": "Wasko S.A.",
    "24.7900": "Sony Mobile Communications Inc",
    "24.7907": "T&W Electronics (ShenZhen) Co.,Ltd",
    "24.7906": "Samsung Electronics Co.,Ltd",
    "24.7905": "Samsung Electronics Co.,Ltd",
    "24.7904": "Urmet SpA",
    "24.15480383": "Markov Corporation",
    "24.7909": "Cisco-Linksys, LLC",
    "24.7908": "ACS Solutions France",
    "24.3979083": "Technicolor CH USA Inc.",
    "24.7875193": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.14722313": "Bitmain Technologies Inc",
    "24.16265534": "endeavour GmbH",
    "24.15530499": "Fca",
    "24.12845489": "SeekTech INC",
    "12.71297883488256": "Serveron Corporation",
    "12.123917683302400": "Waterkotte GmbH",
    "12.346852569088": "Ticketmaster Corp",
    "24.1878836": "New H3C Technologies Co., Ltd",
    "12.123917688528896": "Dynim Oy",
    "24.11836683": "Bq",
    "12.346864324608": "Greenlight Innovation Corp.",
    "20.268735382290432": "Mossman Limited",
    "24.13410533": "DZG Metering GmbH",
    "12.346853449728": "Tex Computer SRL",
    "24.11026469": "Oxide Computer Company",
    "12.346865582080": "Smartech-technology",
    "24.8132813": "E-TRON Co.,Ltd.",
    "24.9734228": "Texas Instruments",
    "24.9462028": "Fida International (S) Pte Ltd",
    "24.16038055": "zte corporation",
    "20.31873086521344": "InSeat Solutions, LLC",
    "20.83999505514496": "ZMIN Technologies",
    "20.264625136336896": "Sercomm Corporation.",
    "24.2410568": "mywerk Portal GmbH",
    "24.9460143": "Texas Instruments",
    "24.16028351": "Ubiquiti Networks Inc.",
    "24.4255453": "Huawei Technologies Co.,Ltd",
    "24.3691034": "Nokia",
    "24.2397332": "Hong Kong Bouffalo Lab Limited",
    "24.2397331": "FibRSol Global Network Limited",
    "24.53332": "Sas Institute Inc.",
    "24.53333": "Kathrein-Werke Kg",
    "24.53330": "Ascend Communications, Inc.",
    "24.53331": "Connected Systems",
    "24.53336": "Cisco Systems, Inc",
    "24.53337": "Ambit Microsystems Corp.",
    "24.53334": "Somat Corporation",
    "24.53335": "Ultrak, Inc.",
    "20.66219386339328": "Hannstar Display Corp",
    "24.53338": "Symbionics, Ltd.",
    "24.53339": "Acroloop Motion Control",
    "20.114869214904320": "Globus Infocom Limited",
    "24.14985213": "Cellco Partnership",
    "24.6307781": "Cox Co., Ltd",
    "24.10516337": "Vialis BV",
    "20.31346178129920": "Applied Concepts, Inc.",
    "24.15467585": "ShenZhen TIGO Semiconductor Co., Ltd.",
    "24.6871911": "Nantong Coship Electronics Co., Ltd.",
    "24.14425556": "Microstep-MIS spol. s r.o.",
    "24.3464574": "Umeox Innovations Co.,Ltd",
    "12.71297885106176": "Wheatstone Corporation",
    "24.5538093": "PAX Computer Technology(Shenzhen) Ltd.",
    "24.8926540": "Transics",
    "24.2154120": "IC Realtime LLC",
    "24.15228267": "Luminate Wireless",
    "12.123917680726016": "Konrad GmbH",
    "12.346854424576": "Intelligent Media Technologies, Inc.",
    "24.5678": "Space Shuttle Hi-Tech Co., Ltd.",
    "24.3471126": "Tp-Link Technologies Co.,Ltd.",
    "24.5672": "Magicard Ltd",
    "24.5671": "embedded-logic DESIGN AND MORE GmbH",
    "24.5670": "ARRIS Group, Inc.",
    "24.5677": "STNet Co., Ltd.",
    "24.5676": "Xanboo",
    "24.5675": "Togami Electric Mfg.co.,Ltd.",
    "24.5674": "Antik computers & communications s.r.o.",
    "24.7871997": "Jabil Inc",
    "24.1840006": "Huawei Device Co., Ltd.",
    "24.25766": "Maquet CardioVascular",
    "24.3680733": "ONvocal Inc",
    "12.346859626496": "ASKI Industrie Elektronik Ges.m.b.H.",
    "20.31873098055680": "guangzhou huiqun intelligent technology co. LTD",
    "20.251018415702016": "Iag Group Ltd",
    "24.9705075": "Point Core SARL",
    "12.346864451584": "MoeTronix",
    "20.66219394727936": "Martem AS",
    "24.573100": "Edimax Technology Co. Ltd.",
    "12.346866536448": "Dsp Design Ltd",
    "24.1896510": "Wearhaus Inc.",
    "24.1365581": "D-Link International",
    "20.79602626068480": "Mission Microwave Technologies, Inc",
    "24.9468657": "Wally",
    "24.14690401": "Huawei Technologies Co.,Ltd",
    "24.11580904": "Ruroc LTD",
    "24.819494": "Juniper Networks",
    "24.9493853": "Apple, Inc.",
    "12.346861260800": "Thales Communications Inc",
    "24.16268622": "Softlink Automation System Co., Ltd",
    "24.6559359": "Polycom",
    "24.11554579": "Dell Inc.",
    "24.13953202": "Samsung Electronics Co.,Ltd",
    "24.9496250": "ASUSTek COMPUTER INC.",
    "12.346852749312": "General Industries Argentina",
    "12.346868887552": "CONTROL SYSTEMS Srl",
    "24.1584536": "Jinwoo Industrial system",
    "24.9949": "Fival Science & Technology Co.,Ltd.",
    "24.9948": "Optical Systems Design",
    "12.346853339136": "CSS Industrie Computer GmbH",
    "24.15001417": "Hewlett Packard",
    "24.9941": "Ory Solucoes em Comercio de Informatica Ltda.",
    "24.9940": "IRCA SpA",
    "24.9943": "KM Electornic Technology Co., Ltd.",
    "24.9942": "Ningbo Andy Optoelectronic Co., Ltd.",
    "24.9945": "ARRIS Group, Inc.",
    "24.9944": "Magic Point Inc.",
    "24.9947": "Ionics EMS Inc.",
    "24.9946": "Universal Media Corporation /Slovakia/ s.r.o.",
    "24.6051565": "3pleplay Electronics Private Limited",
    "24.2654243": "Hewlett Packard",
    "24.8110": "Blick South Africa (Pty) Ltd",
    "24.3635": "Shuko Electronics Co.,Ltd",
    "24.3634": "Kontron Medical",
    "24.3637": "Intel Corporation",
    "24.3636": "NexGen City, LP",
    "24.587964": "Apple, Inc.",
    "24.3630": "Edimax Technology Co. Ltd.",
    "24.3633": "Olympus Soft Imaging Solutions GmbH",
    "24.3632": "AERAS Networks, Inc.",
    "24.6581376": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.123917685878784": "Daatrics LTD",
    "24.1058805": "TP-Link Corporation Limited",
    "24.3639": "Harms & Wende GmbH & Co.KG",
    "24.3638": "HEINESYS, Inc.",
    "24.9493": "Sfr",
    "24.9492": "PC Worth Int'l Co., Ltd.",
    "24.298138": "Huawei Device Co., Ltd.",
    "24.9490": "zte corporation",
    "24.9497": "Viaas Inc",
    "24.9496": "Power PLUS Communications AG",
    "24.9495": "Venntis, LLC",
    "24.9494": "Integrated Design Tools, Inc.",
    "24.9499": "Philips CareServant",
    "24.9498": "Psiber Data Systems Inc.",
    "12.346858549248": "Narinet, Inc.",
    "12.123917682708480": "Aml",
    "24.12646082": "Huawei Technologies Co.,Ltd",
    "24.8391505": "Chengdu XGimi Technology Co.,Ltd",
    "24.9764386": "Huawei Technologies Co.,Ltd",
    "12.123917676318720": "M.T. S.R.L.",
    "24.13152227": "Hefei Radio Communication Technology Co., Ltd",
    "12.123917685612544": "Monnit Corporation",
    "12.119270035456": "Gill Instruments Ltd",
    "24.4760109": "Shenzhen Huaxuchang Telecom Technology Co.,Ltd",
    "24.2132356": "Compal Information (Kunshan) Co., Ltd.",
    "12.346857775104": "Valde Systems, Inc.",
    "24.7640631": "Todaair Electronic Co., Ltd",
    "24.2132358": "zte corporation",
    "24.49263": "Komatsu Ltd.",
    "24.49262": "Haft Technology, Inc.",
    "24.49261": "Boca Research, Inc.",
    "24.20607": "DrayTek Corp.",
    "24.49267": "Xedia Corporation",
    "24.49266": "Knx Ltd.",
    "24.2630911": "Wistron Neweb Corporation",
    "24.49264": "Sectra Secure-Transmission Ab",
    "24.5817227": "Samsung Electronics Co.,Ltd",
    "24.49269": "Xante Corporation",
    "24.49268": "Toyoda Automatic Loom Works Ltd",
    "24.20608": "Cisco Systems, Inc",
    "24.20609": "Murata Machinery, Ltd.",
    "12.346867585024": "Covidence A/S",
    "12.346859929600": "Control Service do Brasil Ltda",
    "24.2627704": "Microsoft Corporation",
    "24.6581994": "Iskratel d.o.o.",
    "20.216602655064064": "comtime GmbH",
    "24.7374144": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.123917681184768": "Haiyang Olix Co.,Ltd.",
    "24.8722": "CAI Networks, Inc.",
    "24.8723": "Pci Corporation",
    "24.8720": "ARRIS Group, Inc.",
    "24.8721": "Rohati Systems",
    "24.8726": "Shibaura Vending Machine Corporation",
    "24.8727": "Neat Electronics",
    "24.8724": "Rinnai Korea",
    "12.346856226816": "Oy Stinghorn Ltd",
    "24.8728": "Akamai Technologies Inc",
    "24.8729": "Dell Inc.",
    "24.8650797": "Pegatron Corporation",
    "20.246679825547264": "Fibrain",
    "24.13398253": "Cisco Systems, Inc",
    "24.2148": "Fasy S.p.A.",
    "24.2149": "Jascom Co., Ltd",
    "24.7102011": "Cisco Systems, Inc",
    "24.2144": "LodgeNet Entertainment Corp.",
    "24.2145": "SoftEnergy Co., Ltd.",
    "24.2146": "NEC Eluminant Technologies, Inc.",
    "24.2147": "Entrisphere Inc.",
    "24.2140": "Shanghai Dare Technologies Co. Ltd.",
    "24.2141": "Mitel Corporation",
    "24.2142": "Pco Ag",
    "24.2143": "Picanol N.V.",
    "24.1499": "PSI Nentec GmbH",
    "24.1498": "Apex Automationstechnik",
    "24.8953988": "Formation Data Systems",
    "24.1493": "Speedcom Wireless",
    "24.1492": "FutureSmart Networks, Inc.",
    "24.1491": "eProduction Solutions, Inc.",
    "24.1490": "DAP Technologies",
    "24.1497": "Techno Valley, Inc.",
    "24.1496": "Arescom, Inc.",
    "24.1495": "Vista Imaging, Inc.",
    "24.1494": "L-3 Linkabit",
    "24.6583266": "Arbiter Systems, Inc.",
    "24.3330": "NEC Platforms, Ltd.",
    "24.11064329": "USA 111 Inc",
    "24.2682412": "Frontline Test Equipment",
    "24.5500": "Dave Networks, Inc.",
    "24.6715": "Doah Elecom Inc.",
    "24.6714": "Dongahelecomm",
    "24.6717": "Ajin Vision Co.,Ltd",
    "24.6716": "Technowave Ltd.",
    "24.6711": "Lear Corporation",
    "24.6710": "Aipermon GmbH & Co. KG",
    "24.6713": "Merten GmbH&CoKG",
    "24.2113541": "feno GmbH",
    "24.5544470": "Uzushio Electric Co.,Ltd.",
    "24.6719": "Intelbras",
    "24.6718": "Faster Technology LLC",
    "24.8173363": "Askey Computer Corp",
    "24.3336": "AboveCable, Inc.",
    "24.8694250": "Career Technologies USA",
    "20.31346172887040": "Shenzhen Aoa Technology Co.,Ltd",
    "12.346859511808": "SPM Instrument AB",
    "24.6340292": "Apple, Inc.",
    "12.123917686353920": "Warecube,Inc",
    "24.1107599": "Kwangsung Electronics Korea Co.,Ltd.",
    "20.149945509216256": "zhejiang yuanwang communication technolgy co.,ltd",
    "24.10792602": "Maestro Wireless Solutions ltd.",
    "24.24600": "Stellar One Corporation",
    "24.24601": "Roche Diagnostics",
    "24.24602": "Keithley Instruments",
    "24.24603": "Mesa Electronics",
    "24.24604": "Telxon Corporation",
    "24.24605": "Lucent Technologies",
    "24.24606": "Softlab, Inc.",
    "24.24607": "Stallion Technologies",
    "24.24608": "Pivotal Networking, Inc.",
    "24.24609": "Dsc Corporation",
    "24.2392714": "Prowave Technologies Ltd.",
    "20.31368099659776": "Ing Buero Ziegler",
    "24.6063920": "Apple, Inc.",
    "24.7916332": "Join Digital, Inc.",
    "12.346864721920": "Rohde&Schwarz Topex SA",
    "24.7402389": "Universal Global Scientific Industrial Co., Ltd.",
    "20.22126506541056": "Black Moth Technologies",
    "24.5034122": "HUMAX Co., Ltd.",
    "20.207357800873984": "comtac AG",
    "12.346866364416": "Sicon srl",
    "20.268341264515072": "Nihon Mega Logic Co.,Ltd.",
    "24.6601740": "LG Electronics (Mobile Communications)",
    "24.6598280": "Polytec GmbH",
    "24.1725": "BNTECHNOLOGY Co., Ltd.",
    "24.1724": "Macrolink, Inc.",
    "24.1727": "Accella Technologies Co., Ltd.",
    "24.1726": "Baumer Optronic GmbH",
    "24.1721": "A5TEK Corp.",
    "12.346857697280": "BLANKOM Antennentechnik GmbH",
    "24.1723": "ATI Technologies Inc.",
    "24.1722": "Westwave Communications",
    "24.9130": "HFR, Inc.",
    "24.9131": "Cisco Systems, Inc",
    "24.9132": "Cisco Systems, Inc",
    "24.9133": "Xmark Corporation",
    "24.1729": "Cisco Systems, Inc",
    "24.1728": "United Internetworks, Inc.",
    "12.346862882816": "Industrial Data Products Ltd",
    "24.9137": "Longcheer Technology (Singapore) Pte Ltd",
    "24.284164": "Gionee Communication Equipment Co.,Ltd.",
    "24.16053162": "JBL GmbH & Co. KG",
    "12.346856747008": "T2C Marketing AB",
    "12.346868330496": "Algodue Elettronica Srl",
    "12.123917677780992": "GSI Technology",
    "12.346863362048": "OUTLINE srl",
    "24.12124083": "MitraStar Technology Corp.",
    "20.273272605245440": "withus",
    "24.565547": "ShenZhen EZL Technology Co., Ltd",
    "12.346867212288": "Futec Inc.",
    "24.5300814": "zte corporation",
    "12.346854866944": "EPSa GmbH",
    "24.8937818": "Siano Mobile Silicon Ltd.",
    "12.346860404736": "aiXtrusion GmbH",
    "24.14482968": "Tp-Link Technologies Co.,Ltd.",
    "24.13140756": "Tempo Communications",
    "24.6855792": "Apple, Inc.",
    "12.123917678497792": "Hongin., Ltd",
    "24.11839640": "Huawei Device Co., Ltd.",
    "24.12362977": "Nabto",
    "24.15530219": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.71297883365376": "Circuitec Ind. Equip. Eletr. Ltda",
    "24.14979835": "Samsung Electronics Co.,Ltd",
    "24.13907927": "zte corporation",
    "12.346866552832": "Peek Traffic Corporation",
    "20.198151835222016": "Nanjing Putian Telecommunications Technology Co.,Ltd.",
    "24.3420093": "Nintendo Co.,Ltd",
    "24.8267": "Autocomputer Co., Ltd.",
    "24.8266": "Pronet Gmbh",
    "24.8265": "Comtron, Inc.",
    "24.8264": "Marconi Communications",
    "24.8263": "Steinbrecher Corp.",
    "24.8262": "Ciprico, Inc.",
    "24.8261": "ION Networks, Inc.",
    "24.8260": "Genitech Pty Ltd",
    "12.346852945920": "Private",
    "24.3162688": "zte corporation",
    "24.8269": "Inovis Gmbh",
    "24.8268": "Mitron Computer Pte Ltd.",
    "12.346866671616": "Wartsila Voyage Limited",
    "20.224641220083712": "MatchX GmbH",
    "24.3450958": "Cisco Systems, Inc",
    "24.36969": "Juniper Networks",
    "24.36966": "Troika Networks, Inc.",
    "24.36967": "WalkAbout Computers, Inc.",
    "24.36964": "Thomson Inc.",
    "24.36965": "Finisar Corporation",
    "24.36962": "ICP VORTEX COMPUTERSYSTEME GmbH",
    "24.36963": "Coherent Communications Systems Corporation",
    "24.36960": "System Create Corp.",
    "24.36961": "Pacific Research & Engineering Corporation",
    "24.308808": "Zenner",
    "24.13653426": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.61822322343936": "Skylark Wireless LLC",
    "12.123917685678080": "Kokam Co., Ltd",
    "12.123917676457984": "Bharat Heavy Electricals Limited",
    "24.11038047": "Raisecom Technology CO., LTD",
    "24.315006": "NXP France Semiconductors France",
    "24.1048576": "Private",
    "20.163808573128704": "Rosenberger Technologies Co.,Ltd.",
    "20.273272600002560": "Go-Link Technology Co., Ltd.",
    "12.123917688729600": "Power Element",
    "12.346867515392": "head",
    "24.7919324": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.534478": "Nagano Keiki Co., Ltd.",
    "20.35227543011328": "Clearly IP Inc",
    "12.346867478528": "Hentschel System GmbH",
    "24.13393013": "Weightech Com. Imp. Exp. Equip. Pesagem Ltda",
    "12.346865315840": "Ocas As",
    "24.7384868": "TP Vision Belgium NV",
    "12.123917679939584": "Toolplanet Co., Ltd.",
    "24.11564571": "Huawei Technologies Co.,Ltd",
    "24.10023280": "Murata Manufacturing Co., Ltd.",
    "20.27478317858816": "SKA Organisation",
    "20.172584096235520": "Lighting New Energy Technology Co., Ltd.",
    "12.346853109760": "Century Geophysical Corp.",
    "24.12893004": "Zebra Information Tech Co. Ltd",
    "24.9741561": "Huawei Technologies Co.,Ltd",
    "24.3177566": "Juniper Networks",
    "24.4753197": "Shen Zhen Da Xia Long Que Technology Co.,Ltd",
    "12.346856693760": "Scame Sistemi srl",
    "24.2683736": "2C - Trifonov & Co",
    "24.2118961": "Samsung Electronics Co.,Ltd",
    "24.8932153": "Siemens AG, Healthcare Sector",
    "24.2118962": "Gotech International Technology Limited",
    "24.13897935": "Huanshun Network Science and Technology Co., Ltd.",
    "12.123917676777472": "Shanghai Danyan Information Technology Co., Ltd.",
    "12.346852282368": "Princeton Gamma Tech",
    "24.16010544": "Elitegroup Computer Systems Co.,Ltd.",
    "24.13402859": "Kyocera Corporation",
    "20.172584093089792": "CAMA（Luoyang）Electronics Co.，Ltd",
    "24.14456364": "NST Audio Ltd",
    "12.346866962432": "Holdline Tecnologia e Sistemas Ltda",
    "24.3968147": "Juniper Networks",
    "12.123917677322240": "Monnit Corporation",
    "12.346852950016": "Imigix Ltd.",
    "12.346854723584": "Vlsip Technologies Inc.",
    "24.4750187": "Huawei Technologies Co.,Ltd",
    "24.3677460": "Skydio Inc.",
    "12.346868211712": "Heinzinger electronic GmbH",
    "12.123917691645952": "Nara Controls Inc.",
    "24.16573244": "Huawei Technologies Co.,Ltd",
    "20.175931706572800": "IoTecha Corp",
    "12.346853916672": "Beckmann+Egle GmbH",
    "24.15000585": "Leifheit Ag",
    "24.16009002": "Nanjing Rousing Sci. and Tech. Industrial Co., Ltd",
    "24.10259672": "Hewlett Packard Enterprise",
    "24.5791839": "Huawei Technologies Co.,Ltd",
    "24.311208": "ISB Corporation",
    "20.175931695038464": "SomaDetect Inc",
    "24.10264842": "Scle Sfe",
    "24.8975726": "Huawei Technologies Co.,Ltd",
    "24.7102758": "Rinicom Holdings",
    "24.1103201": "Tuya Smart Inc.",
    "12.346852941824": "Stella Electronics & Tagging",
    "24.16555160": "Arlo Technology",
    "24.14423730": "Huawei Technologies Co.,Ltd",
    "20.30931571179520": "B-Scada Inc.",
    "24.809372": "Juniper Networks",
    "24.15529771": "Hd Digital Tech Co., Ltd.",
    "12.123917675294720": "Ooo \"Orion-R\"",
    "20.150262247325696": "Impact Distribution",
    "24.14704742": "Sercomm Corporation.",
    "24.15773927": "Apple, Inc.",
    "24.10807268": "Connex GmbH",
    "12.346865803264": "Survalent Technology Corp",
    "12.346857349120": "Tantronic AG",
    "20.189244308979712": "Elekon AG",
    "12.346855256064": "Crossbow Technology, Inc.",
    "24.8677990": "Sumitomo Kizai Co.,Ltd.",
    "24.13898307": "AMPAK Technology, Inc.",
    "20.141268003848192": "Zhuhai TOP Intelligence Electric Co., Ltd.",
    "24.4782373": "Maestronic",
    "24.16041898": "Marvell Semiconductors",
    "12.346852610048": "Advanced Signal Corp.",
    "20.211776058687488": "Pinpark Inc.",
    "12.346857426944": "Computerwise, Inc.",
    "24.5257853": "AlphaTech PLC Int’l Co., Ltd.",
    "12.346862284800": "Thales Communications & Security S.A.",
    "24.16044296": "Intel Corporate",
    "20.251330447802368": "Tband srl",
    "20.30931576422400": "Lg Cns",
    "20.167137983332352": "United Power Research Technology Corp.",
    "24.13937742": "BODi rS, LLC",
    "24.2622021": "Konze System Technology Co.,Ltd.",
    "24.1589972": "Wuhan Hosan Telecommunication Technology Co.,Ltd",
    "24.13651244": "Roku, Inc.",
    "24.8389343": "ORA Inc.",
    "24.87": "Scitex Corporation Ltd.",
    "24.13380085": "Apple, Inc.",
    "24.8662144": "Sichuan Changhong Electric Ltd.",
    "24.12114716": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917686706176": "WiViCom Co., Ltd.",
    "24.5564373": "Sun Cupid Technology (HK) LTD",
    "20.84372898185216": "Shenzhen Cogitation Technology Co.,Ltd.",
    "24.13147933": "Shenzhen Communication Technologies Co.,Ltd",
    "24.13370329": "Cisco Meraki",
    "20.101728420626432": "ProdataKey",
    "20.31368139505664": "Qivivo",
    "12.123917675884544": "Russian Telecom Equipment Company",
    "24.10506117": "LG Electronics (Mobile Communications)",
    "20.4471492968448": "Haerbin Donglin Technology Co., Ltd.",
    "20.31368185643008": "Istria soluciones de criptografia, S. A.",
    "24.833234": "D-Link International",
    "12.119269785600": "UAB Kitron",
    "24.11049158": "Shinbo Co., Ltd.",
    "12.346854846464": "Spellman High Voltage Electronics Corp",
    "24.1063769": "Samsung Electronics Co.,Ltd",
    "12.123917676724224": "Davey Bickford",
    "24.7905542": "Tp-Link Technologies Co.,Ltd.",
    "12.123917690007552": "RWS Automation GmbH",
    "24.88": "Racore Computer Products Inc.",
    "24.40998": "Teldat, S.A.",
    "24.40999": "Firepower Systems, Inc.",
    "12.346861322240": "Honeywell Sensotec",
    "24.40992": "Citicorp/Tti",
    "24.40993": "General Dynamics Mission Systems",
    "24.40990": "Est Corporation",
    "24.40991": "Tricord Systems, Inc.",
    "24.1630666": "wanze",
    "24.40997": "Redcom Labs Inc.",
    "24.40994": "Centre For Development Of Advanced Computing",
    "24.40995": "Applied Creative Technology, Inc.",
    "20.264165332615168": "ATN Media Group FZ LLC",
    "24.13370492": "G-WAY Microwave",
    "24.1068888": "Aruba, a Hewlett Packard Enterprise Company",
    "24.15471538": "Netonix",
    "12.346853023744": "Real World Computing Partnership",
    "24.12347851": "Panasonic Life Solutions Networks Co., Ltd.",
    "20.5239433330688": "Ekin Teknoloji San ve Tic A.S.",
    "24.11583902": "zte corporation",
    "24.15791957": "Cisco Systems, Inc",
    "20.127937497006080": "Beijing Inspiry Technology Co., Ltd.",
    "24.9498311": "Samsung Electronics Co.,Ltd",
    "20.31657720545280": "Shenzhen Smart Device Technology Co.,LTD",
    "24.11015501": "Tp-Link Technologies Co.,Ltd.",
    "24.8170277": "Mesmo Technology Inc.",
    "24.13656736": "China Essence Technology (Zhumadian) Co., Ltd.",
    "24.1866769": "Eid",
    "24.3433": "TMT&D Corporation",
    "24.3432": "Vinci Systems, Inc.",
    "24.3431": "Ericsson",
    "24.3430": "Cisco Systems, Inc",
    "24.3437": "K-Tech Devices Corp.",
    "24.3436": "M-Audio",
    "24.3435": "Mita-Teknik A/S",
    "24.3434": "Redwood Technologies LTD",
    "12.346855686144": "Ratotec GmbH",
    "24.3438": "K-Patents Oy",
    "24.11555350": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.300613": "Wuxi Sky Chip Interconnection Technology Co.,Ltd.",
    "24.11589573": "Logipix Ltd",
    "24.10760314": "Samsung Electronics Co.,Ltd",
    "24.13406320": "Cisco Systems, Inc",
    "24.3706": "GemWon Communications Co., Ltd.",
    "12.346858209280": "ProMik GmbH",
    "12.346860949504": "Grantronics Pty Ltd",
    "24.15489258": "Ruckus Wireless",
    "24.4253488": "DEY Storage Systems, Inc.",
    "24.15506482": "Sichuan AI-Link Technology Co., Ltd.",
    "12.346868916224": "Nupoint Systems Inc.",
    "24.3389": "Hammerhead Systems, Inc.",
    "24.3388": "i.Tech Dynamic Ltd",
    "24.10499834": "Ettus Research LLC",
    "24.3385": "Network Electronics",
    "24.3384": "Nissin Inc.",
    "24.3387": "Microelectronics Technology Inc.",
    "24.3386": "Microsoft Corp.",
    "24.3381": "PAC International Ltd",
    "24.3380": "Shell International Exploration and Production, Inc.",
    "24.3383": "Wiplug",
    "24.3382": "Wu Han Routon Electronic Co., Ltd",
    "24.18": "Information Technology Limited",
    "24.19": "Camex",
    "24.16": "Sytek Inc.",
    "24.17": "Normerel Systemes",
    "24.14": "Fujitsu Limited",
    "24.15": "Next, Inc.",
    "24.12": "Cisco Systems, Inc",
    "24.13": "Fibronics Ltd.",
    "24.10": "Omron Tateisi Electronics Co.",
    "24.11": "Matrix Corporation",
    "20.31614747803648": "NewLand (NZ) Communication Tech Limited",
    "24.9724190": "Wata Electronics Co.,Ltd.",
    "24.9724196": "Pason Systems",
    "20.233630182932480": "Innoviz Technologies LTD",
    "24.11802833": "Apple, Inc.",
    "24.13125007": "Wyler AG",
    "24.2930925": "SonicWall",
    "24.8410905": "Samsung Electronics Co.,Ltd",
    "20.207147366350848": "Solectria Renewables, LLC",
    "12.123917684264960": "Guangzhou Wanglu",
    "24.6845": "Impatica Inc.",
    "24.6844": "U4EA Technologies Ltd",
    "24.1067846": "Mitsubishi Electric Corporation",
    "24.3730087": "Hewlett Packard",
    "24.8683565": "Cisco Systems, Inc",
    "20.167295465816064": "Thorlabs GmbH",
    "24.8949901": "Cisco Systems, Inc",
    "24.8436454": "Neets",
    "24.2467": "Leadfly Techologies Corp. Ltd.",
    "24.7916089": "Apple, Inc.",
    "24.9473123": "GZ Weedong Networks Technology Co. , Ltd",
    "24.2415259": "Motorola Mobility LLC, a Lenovo Company",
    "24.14443704": "Shenzhen JingHanDa Electronics Co.Ltd",
    "24.14723760": "Suntaili Enterprise Co. Ltd,",
    "12.123917685915648": "Zhejiang Zhaolong Interconnect Technology Co.,Ltd",
    "20.123598310211584": "Beijing Security Union Information Technology Co.,Ltd",
    "24.7631670": "MICRODIGTAL Inc",
    "20.101728425869312": "HHCC Plant Technology Co., Ltd.",
    "24.805062": "Celestica Inc.",
    "24.10514857": "Nokia Corporation",
    "20.97843040747520": "Swarco Traffic Systems Gmbh",
    "24.576069": "Toyou Feiji Electronics Co., Ltd.",
    "24.6609921": "Static Systems Group PLC",
    "20.255191450386432": "Shenzhen Hipad Telecommunication Technology Co.,Ltd",
    "24.1578697": "Samsung Electronics Co.,Ltd",
    "24.9986250": "Apple, Inc.",
    "24.8414330": "Ola Sense Inc",
    "24.5243537": "Espressif Inc.",
    "24.16001075": "MMPC Inc.",
    "24.1853689": "GN Netcom A/S",
    "24.5254862": "Asahi Electronics Co.,Ltd",
    "24.2638278": "Huawei Technologies Co.,Ltd",
    "24.15514366": "Giroptic",
    "24.7916339": "DHC Software Co.,Ltd",
    "24.3181977": "Hewlett Packard",
    "24.600": "Flying Packets Communications",
    "24.798640": "Samsung Electronics Co.,Ltd",
    "24.11855405": "Espressif Inc.",
    "24.11855402": "LG Innotek",
    "24.6559577": "Intellivision Holdings, LLC",
    "12.123917682049024": "Bt9",
    "24.6269": "Armorlink Co.Ltd",
    "24.6268": "Intercross, Llc",
    "24.4777946": "AzureWave Technology Inc.",
    "20.105645764247552": "Zaptec",
    "24.6265": "dSys",
    "24.6264": "Mackware GmbH",
    "24.6267": "4NSYS Co. Ltd.",
    "24.6266": "Wiremold",
    "24.6261": "AnaCise Testnology Pte Ltd",
    "24.6260": "Cisco Systems, Inc",
    "24.6263": "Amplex A/S",
    "24.6262": "WowWee Ltd.",
    "20.180698122027008": "Astro, Inc",
    "12.346868101120": "Applied Micro Electronics AME bv",
    "24.1893068": "Texas Instruments",
    "12.346866941952": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.2672916": "Taimag Corporation",
    "24.6847701": "Y Soft Corporation, a.s.",
    "24.8656633": "Tp-Link Technologies Co.,Ltd.",
    "24.12380463": "HP Inc.",
    "20.39672885542912": "Annapurna labs",
    "24.1588642": "Intel Corporate",
    "24.14421665": "Compal Information (Kunshan) Co., Ltd.",
    "24.1066074": "Shaanxi Hitech Electronic Co., LTD",
    "24.12359296": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.8171869": "Huawei Technologies Co.,Ltd",
    "24.13696920": "Huawei Technologies Co.,Ltd",
    "24.1052286": "Juniper Networks",
    "24.6339985": "Move Innovation",
    "24.1123": "Bosch Security Systems",
    "24.1122": "DAKOS Data & Communication Co., Ltd.",
    "24.1121": "EPOX Computer Co., Ltd.",
    "24.1120": "Knilink Technology, Inc.",
    "24.1127": "Wuhan Research Institute of MII",
    "24.1126": "ARMITEL Co.",
    "24.1125": "i.s.t isdn-support technik GmbH",
    "24.1124": "Pulse-Link Inc",
    "24.4205473": "Sony Mobile Communications Inc",
    "24.1129": "Innocom, Inc.",
    "24.1128": "Vivity, Inc.",
    "12.123917681295360": "Stellwerk GmbH",
    "12.123917677551616": "HI-TECH SYSTEM Co. Ltd.",
    "12.123917690757120": "Warecube,Inc",
    "12.123917682282496": "Discover Battery",
    "24.11559048": "Intel Corporate",
    "24.1587374": "Conspin Solution",
    "24.6841": "Pmc",
    "24.6840": "Anseri Corporation",
    "24.6843": "Fontal Technology Incorporation",
    "24.6842": "Caton Overseas Limited",
    "24.3467019": "HAN Networks Co., Ltd",
    "24.1618441": "Torrey Pines Logic, Inc",
    "24.6847": "TRUMPF Laser Marking Systems AG",
    "24.6846": "Computer Hi-Tech Inc.",
    "24.6849": "3Com Ltd",
    "24.6848": "Joybien Technologies Co., Ltd.",
    "24.11052616": "Qingdao Hisense Communications Co.,Ltd.",
    "24.1625748": "Samsung Electronics Co.,Ltd",
    "20.31372483756032": "Accriva",
    "20.175931698184192": "Xiaojie Technology (Shenzhen) Co., Ltd",
    "12.346856329216": "Sintium Ltd",
    "24.11553119": "OMRON HEALTHCARE Co., Ltd.",
    "12.346861039616": "Motion Analysis Corp",
    "12.123917684785152": "Dot System S.r.l.",
    "24.5775449": "Xiaomi Communications Co Ltd",
    "12.346865471488": "ZK Celltest Inc",
    "24.1618992": "Nest Labs Inc.",
    "24.11590144": "Cera Electronique",
    "24.172": "Conware Netzpartner",
    "24.173": "Bruker Instruments Inc.",
    "24.170": "Xerox",
    "24.171": "Logic Modeling Corporation",
    "24.176": "Rnd-Rad Network Devices",
    "24.177": "Alpha Micro",
    "24.174": "Dassault Electronique",
    "24.175": "Canberra Industries, Inc.",
    "24.14720305": "Intel Corporate",
    "24.178": "Televideo Systems, Inc.",
    "24.179": "Cimlinc Incorporated",
    "12.346862092288": "LOGITAL DIGITAL MEDIA srl",
    "12.123917689380864": "csintech",
    "24.11543795": "Progeny Systems",
    "24.6081050": "Microsoft Mobile Oy",
    "20.128510959026176": "Cerevo Inc.",
    "20.368731750400": "Beijing Connected Information Technology Co.,Ltd.",
    "12.123917686616064": "Omwave",
    "12.119269285888": "Vigor Electric Corp",
    "24.11578618": "Fujitsu Limited",
    "24.12073595": "Worldplay (Canada) Inc.",
    "24.11276847": "Fantasia Trading LLC",
    "12.123917677248512": "Urbana Smart Solutions Pte Ltd",
    "24.7955": "S.& A.S. Ltd.",
    "24.297493": "Avaya Inc",
    "24.16255055": "Sagemcom Broadband SAS",
    "24.14699333": "Hi-P Electronics Pte Ltd",
    "24.7377079": "Huawei Device Co., Ltd.",
    "12.346858278912": "Cannon Technologies",
    "24.4988876": "Huawei Technologies Co.,Ltd",
    "20.52912295247872": "Private",
    "24.3460542": "Huawei Technologies Co.,Ltd",
    "24.294670": "Barrot Technology Limited",
    "24.7956": "NexG",
    "20.4471483531264": "Suzhou Ribao Technology Co.,Ltd.",
    "12.123917685989376": "Fluid Management Technology",
    "12.123917682192384": "Techimp - Altanova group Srl",
    "24.3157480": "JDA, LLC (JDA Systems)",
    "24.2900485": "Lytx",
    "24.7871818": "zte corporation",
    "24.8156139": "Huawei Device Co., Ltd.",
    "24.16036290": "Zhejiang Dahua Technology Co., Ltd.",
    "24.8684810": "Hella Sonnen- und Wetterschutztechnik GmbH",
    "12.123917676814336": "Guan Show Technologe Co., Ltd.",
    "24.3151501": "ProLon",
    "20.52912291053568": "Beijing Surestar Technology Co. Ltd,",
    "12.119269892096": "Hettich Benelux",
    "20.141720414060544": "Private",
    "12.346867601408": "Alpha-Mos",
    "24.2662891": "Etek Technology (Shenzhen) Co.,Ltd",
    "12.346863017984": "Itas A/S",
    "24.2636365": "Cloud Network Technology (Samoa) Limited",
    "24.16553166": "zte corporation",
    "12.346856136704": "ABB Inc.",
    "12.346864590848": "Digital Lumens",
    "20.13592314249216": "avxav Electronic Trading LLC",
    "24.10026644": "Apple, Inc.",
    "12.123917682114560": "Sicon srl",
    "24.820": "Omega Engineering Inc.",
    "24.821": "Mirae Technology",
    "24.822": "Zetes Technologies",
    "24.823": "Vaone, Inc.",
    "24.824": "Oak Technology",
    "24.825": "Eurologic Systems, Ltd.",
    "24.826": "Silicon Wave, Inc.",
    "24.827": "TAMI Tech Co., Ltd.",
    "24.828": "Daiden Co., Ltd.",
    "24.829": "ILSHin Lab",
    "24.13676749": "Moen",
    "24.1334012": "Liteon Technology Corporation",
    "24.319218": "Shenzhen ECOM Technology Co. Ltd",
    "24.7370891": "vivo Mobile Communication Co., Ltd.",
    "12.123917682188288": "Enovative Networks, Inc.",
    "12.123917685784576": "Applied Satellite Engineering",
    "12.346860232704": "A.D.I Video technologies",
    "20.88914833440768": "Clear Flow by Antiference",
    "12.346861916160": "C&H technology ltd.",
    "24.11798578": "TC Communications",
    "12.123917684490240": "ANTEK GmbH",
    "24.9222188": "IPitomy Communications",
    "24.16544763": "Huawei Technologies Co.,Ltd",
    "12.123917691744256": "Cybercom AB",
    "24.8690854": "Arcadyan Technology Corporation",
    "12.123917690351616": "Gridless Power Corperation",
    "12.346865303552": "CSI Controles e Sistemas Industriais Ltda.",
    "20.225209252577280": "Obelisk Inc.",
    "12.346866688000": "rbz robot design s.l.",
    "24.6862588": "Hitron Technologies. Inc",
    "12.346854469632": "EdenTree Technologies, Inc.",
    "20.110373786091520": "Bühler AG",
    "12.123917679534080": "Italiana Ponti Radio Srl",
    "24.20714": "Xel Communications, Inc.",
    "20.57353111076864": "Keystone Electronic Solutions",
    "12.123917679468544": "Blair Companies",
    "24.37501": "Ficosa Internationa(Taicang) C0.,Ltd.",
    "24.1576822": "D-Link International",
    "24.10531072": "ShenZhen Cando Electronics Co.,Ltd",
    "24.11028125": "Cisco Meraki",
    "24.4997256": "Sanshin Electronics Co.,Ltd.",
    "24.12336659": "Accordance Systems Inc.",
    "12.123917681135616": "olympus-ossa",
    "12.123917685907456": "Trapeze Software Group Inc",
    "20.102196708376576": "Shenzhen Hivt Technology Co.,Ltd",
    "24.14197596": "Samsung Electronics Co.,Ltd",
    "12.346867367936": "Saia-Burgess Controls AG",
    "12.346863067136": "DVTech",
    "12.346852429824": "Damler Chrysler Rail System (Signal) AB",
    "24.9487199": "Nanjing Jiahao Technology Co., Ltd.",
    "12.123917688348672": "YUYAMA MFG Co.,Ltd",
    "24.16291791": "Daeshin-Information Technology Co., Ltd.",
    "12.346864279552": "WAC (Israel) Ltd.",
    "24.5267496": "Xirrus Inc.",
    "12.123917681086464": "MVT Video Technologies R + H Maedler GbR",
    "24.2686898": "Toshiba Corp.",
    "12.346862641152": "Spectra Embedded Systems",
    "24.16312337": "Huawei Technologies Co.,Ltd",
    "24.9455382": "Rorze Corporation",
    "12.123917680218112": "OptoPrecision GmbH",
    "24.2339": "Heaman System Co., Ltd",
    "24.2338": "TST Biometrics GmbH",
    "24.11811368": "Samsung Electronics Co.,Ltd",
    "24.2333": "Proteam Computer Corporation",
    "24.2332": "CacheVision, Inc",
    "24.2331": "Digital Generation Inc.",
    "24.2330": "Macat Optics & Electronics Co., Ltd.",
    "24.2337": "Planmeca Oy",
    "24.2336": "EpoX COMPUTER CO.,LTD.",
    "24.2335": "A&D Co., Ltd.",
    "24.2334": "Firstech Technology Corp.",
    "12.123917677617152": "Mettler Toledo",
    "12.346853158912": "Young Computer Co., Ltd.",
    "24.5028251": "Weifang Goertek Electronics Co.,Ltd",
    "20.79215069233152": "Baja Electronics Technology Limited",
    "24.7400026": "Apple, Inc.",
    "24.9470250": "Apple, Inc.",
    "24.7675": "NETCLEUS Systems Corporation",
    "24.7674": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "24.7677": "Nokia Danmark A/S",
    "24.7676": "Ksic",
    "24.7671": "R. STAHL Schaltgeräte GmbH",
    "24.7670": "Samsung Electronics Co.,Ltd",
    "24.7673": "Cybiotronics (Far East) Limited",
    "24.7672": "Webpro Vision Technology Corporation",
    "12.346862723072": "HCE Engineering S.r.l.",
    "24.15792194": "KEEBOX, Inc.",
    "24.7679": "Network Critical Solutions Ltd",
    "24.7678": "Palm, Inc",
    "24.8889": "Analogix Seminconductor, Inc",
    "24.8888": "Norcott",
    "24.16274143": "VNL Europe AB",
    "24.1892709": "Marshal Corporation",
    "12.123917689970688": "7thSense Design Limited",
    "20.167137990672384": "Navroom Beijing, China",
    "24.4762888": "Huawei Technologies Co.,Ltd",
    "12.346861264896": "Desarrollo De Sistemas Integrados De Control S.A.",
    "20.233630191321088": "Suzhou Wan Dian Zhang Network Technology Co., Ltd",
    "24.6860563": "Futura Mobility",
    "24.7116066": "Scalys",
    "24.4766990": "Quanta Storage Inc.",
    "12.346853629952": "Infineer Ltd.",
    "20.268902728728576": "Private",
    "24.14166549": "Leitner SpA",
    "24.16256496": "zte corporation",
    "24.16256497": "Sontex SA",
    "24.16256490": "ZyCast Technology Inc.",
    "24.3153818": "IEEE Registration Authority",
    "12.123917686288384": "Novoptel GmbH",
    "12.123917678084096": "Clock-O-Matic",
    "20.216602652966912": "Shenzhen Kalif Electronics Co.,Ltd",
    "20.71414656794624": "Embedded IQ",
    "24.6610921": "Ateme",
    "24.14953558": "Lilee Systems, Ltd.",
    "24.7920914": "Arcadyan Corporation",
    "20.44169488760832": "ThirdReality, Inc",
    "24.9443244": "Huawei Technologies Co.,Ltd",
    "12.346856075264": "IMAGO Technologies GmbH",
    "24.7380614": "X6D Limited",
    "12.346865696768": "Thomas & Betts",
    "24.2927595": "Cisco Systems, Inc",
    "24.2682625": "Amazon Technologies Inc.",
    "12.346854076416": "Synopsys",
    "24.6065486": "Apple, Inc.",
    "24.6104": "Magnum Semiconductor, Inc.",
    "24.6105": "AAI Corporation",
    "24.2155876": "ARRIS Group, Inc.",
    "24.6107": "Canko Technologies Inc.",
    "24.6100": "Monsoon Multimedia, Inc",
    "24.6101": "Samsung Electronics Co.,Ltd",
    "24.6102": "Bluechips Microhouse Co.,Ltd.",
    "24.6103": "ION Geophysical Corporation Inc.",
    "24.3687273": "Patrol Products Consortium LLC",
    "24.10508602": "V.D.S. Video Display Systems srl",
    "24.6108": "Daemyung Zero1",
    "24.6109": "Clipsal Australia",
    "12.123917689208832": "Damerell Design Limited (DCL)",
    "12.123917678043136": "IPG Photonics Corporation",
    "24.12121575": "WayTools, LLC",
    "12.123917687054336": "Elbit Systems of America",
    "24.11594895": "Gmx Sas",
    "12.346852323328": "Industrial Control Links",
    "24.16274217": "Huawei Technologies Co.,Ltd",
    "24.13632331": "Apple, Inc.",
    "12.123917691199488": "Hanshin Electronics",
    "24.12363942": "Intel Corporate",
    "24.6577554": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.24811": "Fourthtrack Systems",
    "24.24810": "StreamLogic",
    "24.24813": "Ricardo Test Automation Ltd.",
    "24.24812": "Hermary Opto Electronics Inc.",
    "24.24815": "Flytech Technology Co., Ltd.",
    "24.24814": "Apollo",
    "24.24817": "Exp Computer, Inc.",
    "24.24816": "Johnson & Johnson Medical, Inc",
    "24.24819": "Performance Analysis Broadband, Spirent plc",
    "24.24818": "Lasergraphics, Inc.",
    "12.346867580928": "Wishtek Technology, Inc.",
    "24.16528194": "Juniper Networks",
    "24.11313708": "Baycity Technologies Ltd",
    "24.6854632": "Society of Motion Picture &amp; Television Engineers",
    "12.123917682847744": "AeroVision Avionics, Inc.",
    "20.229206578429952": "PelKorea",
    "24.4206596": "Apple, Inc.",
    "24.16027882": "Deciso B.V.",
    "12.346863058944": "tetronik GmbH AEN",
    "12.346860941312": "GD Mission Systems",
    "24.7865740": "mMax Communications, Inc.",
    "20.14286625701888": "Yantai Dongfang Wisdom Electic Co.,Ltd.",
    "20.61822319198208": "Shenzhen Smart-core Technology co., Ltd.",
    "20.96896780599296": "Aggregate Co.,Ltd.",
    "20.74781678370816": "Shen Zhen Huawang Technology Co; Ltd",
    "12.123917685211136": "Radian Research, Inc.",
    "24.9307": "Raidon Technology, Inc.",
    "24.9306": "Nanjing Panda Electronics Company Limited",
    "24.9305": "ABB Automation products GmbH",
    "24.9304": "PA Bastion CC",
    "24.8438126": "Hewlett Packard",
    "24.9302": "2Wire Inc",
    "24.9301": "MuLogic BV",
    "24.9300": "Samsung Electronics Co.,Ltd",
    "24.5516119": "Night Owl SP",
    "24.9309": "Terberg besturingstechniek B.V.",
    "24.9308": "Design-Com Technologies Pty. Ltd.",
    "20.105645768441856": "Yangzhou Wanfang Electronic Technology,CO.,Ltd.",
    "20.52821123661824": "A9",
    "24.6346777": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.15219218": "Samsung Electronics Co.,Ltd",
    "24.300139": "Tecno Mobile Limited",
    "24.12851665": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "12.346853679104": "SAET I.S. S.r.l.",
    "24.14978063": "Discovery Insure",
    "24.9997066": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.53431": "Intel Corporation",
    "24.7906947": "Precidata",
    "24.53430": "Crescent Networks, Inc.",
    "24.14211666": "Apator S.A.",
    "12.346853920768": "Remia s.r.o.",
    "24.15509611": "Elitegroup Computer Systems Co.,Ltd.",
    "24.8817": "Jäger Computergesteuerte Meßtechnik GmbH.",
    "24.12882688": "8Devices",
    "24.53435": "Cisco Systems, Inc",
    "20.180577506426880": "Powell Industries",
    "24.8811": "Cisco-Linksys, LLC",
    "12.346861670400": "Vapor Rail",
    "24.1884096": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.123917681446912": "VX Instruments GmbH",
    "24.8813": "Shenzhen GIEC Electronics Co., Ltd.",
    "24.2101582": "MeteRSit S.R.L.",
    "12.123917678338048": "A&T Corporation",
    "12.123917682491392": "Quan International Co., Ltd.",
    "24.538789": "Funkwerk plettac electronic GmbH",
    "24.8439749": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3970942": "IPS Technology Limited",
    "24.2685907": "Ragentek Technology Group",
    "20.212014914863104": "Private",
    "24.10758533": "Sichuan AI-Link Technology Co., Ltd.",
    "24.8398097": "Itel Mobile Limited",
    "24.12848709": "ACK Networks,Inc.",
    "24.10811876": "AirVine Scientific, Inc.",
    "24.824065": "PT. Prasimax Inovasi Teknologi",
    "24.2948324": "CTEK Sweden AB",
    "24.1589758": "Digital 14",
    "24.9469488": "Lars",
    "12.346856034304": "Specstroy-Svyaz Ltd",
    "24.10767317": "Xiaomi Communications Co Ltd",
    "12.346858803200": "Physik Instrumente (PI) GmbH&Co.KG",
    "24.16429": "Harris Adacom Corporation",
    "24.16428": "Isis Distributed Systems, Inc.",
    "12.123917680640000": "DaiShin Information & Communications Co., Ltd",
    "24.16421": "Molecular Dynamics",
    "24.16420": "Compac Inc.",
    "24.16423": "SMC Massachusetts",
    "24.16422": "Buffalo.Inc",
    "24.16425": "Compex",
    "24.2679623": "Liteon Technology Corporation",
    "24.16427": "Trigem Computer, Inc.",
    "24.16426": "Canoga-Perkins",
    "24.15003652": "Dell Inc.",
    "12.123917689868288": "Private",
    "20.268902719291392": "Delem Bv",
    "24.10001355": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "20.79215074476032": "Annapurna labs",
    "24.14727653": "Technicolor",
    "12.123917682221056": "E-Controls",
    "24.13398804": "Hong Kong Wheatek Technology Limited",
    "24.9739052": "Extreme Networks, Inc.",
    "24.842305": "Hangzhou H3C Technologies Co., Limited",
    "24.314141": "Traka plc",
    "20.71389215195136": "Hangzhou GANX Technology Co.,Ltd.",
    "24.13400885": "zte corporation",
    "12.346860638208": "microC Design SRL",
    "24.6853072": "IEEE Registration Authority",
    "24.12906732": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346864852992": "SAM Co., Ltd.",
    "12.346865516544": "Cp Eletronica Sa",
    "24.8703330": "ShenZhen IDWELL Technology CO.,Ltd",
    "20.167295456378880": "Dspread Technology (Beijing) Inc.",
    "24.3950491": "Theissen Training Systems, Inc.",
    "24.16575706": "Ubiquiti Networks Inc.",
    "24.8708340": "IEEE Registration Authority",
    "24.4477403": "Raytheon BBN Technologies",
    "12.123917684252672": "Wizapply Co.,Ltd",
    "24.6608090": "Intel Corporate",
    "20.74781672079360": "Musashi Seimitsu Industry Co.,Ltd",
    "24.2670235": "Technicolor CH USA Inc.",
    "24.14710141": "Motorola Mobility LLC, a Lenovo Company",
    "24.15772171": "Arra Networks/ Spectramesh",
    "24.7629739": "Guangdong Enok Communication Co., Ltd",
    "24.10507742": "ARRIS Group, Inc.",
    "24.5025516": "Guangzhou limee technology co.,LTD",
    "20.216145535696896": "Canare Electric Co., Ltd.",
    "24.5028930": "Shenzhen Hangsheng Electronics Co.,Ltd.",
    "12.346856083456": "Bigbang L.T.D.",
    "24.1579327": "Tamtron Oy",
    "24.5028936": "Cisco Systems, Inc",
    "24.3984694": "Apple, Inc.",
    "12.123917691125760": "MPM Micro Präzision Marx GmbH",
    "24.12432": "Cyra Technologies, Inc.",
    "24.12433": "Taiwan First Line Elec. Corp.",
    "24.12430": "Cross Match Technologies, Inc.",
    "24.12431": "MICRILOR, Inc.",
    "24.12436": "Cisco Systems, Inc",
    "24.12437": "Procomp Informatics, Ltd.",
    "24.12434": "Kontron Electronics AG",
    "12.346864754688": "Hanning Elektro-Werke GmbH & Co. KG",
    "24.12438": "Cisco Systems, Inc",
    "24.12439": "AB Regin",
    "12.346861686784": "Verint Systems Ltd.",
    "20.119143423541248": "Soundking Electronics&Sound Co., Ltd.",
    "24.8947203": "Huawei Technologies Co.,Ltd",
    "20.250817673166848": "Aeroel srl",
    "12.346860113920": "Ameli Spa",
    "12.123917681061888": "QSnet Visual Technologies Ltd",
    "24.8451630": "Hangzhou H3C Technologies Co., Limited",
    "24.5285340": "ARRIS Group, Inc.",
    "24.7873965": "Samsung Electronics Co.,Ltd",
    "24.12084369": "CK Telecom Ltd",
    "24.13120485": "Huawei Technologies Co.,Ltd",
    "24.5013792": "Taicang T&W Electronics",
    "24.556644": "Technicolor CH USA Inc.",
    "24.5521993": "Sony Corporation",
    "12.346863280128": "Absolutron. LLC",
    "24.14711236": "iRay Technology Company Limited",
    "12.346861752320": "PHB Eletronica Ltda.",
    "12.346852249600": "Leroy Automation",
    "24.7656069": "Texas Instruments",
    "12.123917690523648": "SBS-Feintechnik GmbH & Co. KG",
    "12.346852634624": "Alstom Signalling Ltd.",
    "24.10236355": "BSkyB Ltd",
    "12.346856464384": "Bluewater Systems Ltd",
    "24.16301833": "Huawei Technologies Co.,Ltd",
    "20.211525439586304": "Victron Energy B.V.",
    "24.10769166": "Cisco Systems, Inc",
    "24.11048909": "ARRIS Group, Inc.",
    "24.11592062": "Huawei Technologies Co.,Ltd",
    "20.211670370615296": "Ants",
    "20.242213037539328": "Advanced Electronics Ltd",
    "12.123917690355712": "Grupo Epelsa S.L.",
    "24.16292079": "Huawei Technologies Co.,Ltd",
    "24.7121741": "Digital Payment Technologies",
    "24.2892081": "Juniper Networks",
    "24.16540056": "NEC Personal Products, Ltd",
    "24.15235827": "S5 Tech S.r.l.",
    "12.346858758144": "CPqD",
    "24.10768454": "Xiaomi Communications Co Ltd",
    "24.6879574": "Objetivos y Servicio de Valor Añadido",
    "24.6555070": "New Light Led",
    "24.6345314": "Tzukuri Pty Ltd",
    "24.6422": "PayTec AG",
    "12.123917681876992": "CEA Technologies Pty Ltd",
    "12.123917678518272": "Applied Radar, Inc.",
    "24.542294": "Veo-Labs",
    "24.3938903": "Cardiopulmonary Corp",
    "12.346859077632": "Navtech Radar Ltd",
    "24.4985177": "Vision Information & Communications",
    "24.4985172": "Mobiwire Mobiles (NingBo) Co., LTD",
    "24.4753": "KWS Computersysteme GmbH",
    "24.4752": "KYOWA Electric & Machinery Corp.",
    "24.4751": "Montilio",
    "24.4750": "Q-Free ASA",
    "24.4757": "Aiware Inc.",
    "24.4756": "Sumitomo Electric Device Innovations, Inc",
    "24.4755": "ABB Power Protection (CH)",
    "24.4754": "Griffin Technology",
    "24.4759": "O2Micro, Inc.",
    "24.4758": "Addlogix",
    "20.206931537952768": "Hangzhou Linker Digital Technology Co., Ltd",
    "24.9203140": "Megapixel VR",
    "20.36069847334912": "Beijing Huadianzhongxin Tech.Co.,Ltd",
    "24.2412767": "Apple, Inc.",
    "24.12608740": "Cisco Systems, Inc",
    "12.346856849408": "Metafix Inc.",
    "12.346866860032": "Sec.Eng Systems Pty Ltd",
    "24.300403": "zte corporation",
    "12.123917690621952": "Beijing Hzhytech Technology Co.Ltd",
    "12.123917688471552": "Nexus Electric S.A.",
    "24.16558992": "Huawei Technologies Co.,Ltd",
    "24.6307451": "Gafachi, Inc.",
    "24.15263606": "Shenzhen Kootion Technology Co., Ltd",
    "12.71297883377664": "Dommel GmbH",
    "24.15263600": "Warp9 Tech Design, Inc.",
    "24.13384510": "Lester Electrical",
    "12.123917680254976": "Glidewell Laboratories",
    "24.6588671": "Sichuan Changhong Electric Ltd.",
    "12.123917690761216": "Duval Messien",
    "20.159304564867072": "Pars Ertebat Afzar Co.",
    "12.346865111040": "SKD System AB",
    "24.13945148": "Cisco Systems, Inc",
    "20.88010618044416": "New Audio LLC",
    "24.4484439": "Liteon Technology Corporation",
    "12.123917681057792": "C4I Systems Ltd",
    "24.8705224": "Widex A/S",
    "12.123917690171392": "Optix Jsc",
    "24.13372914": "Motorola Mobility LLC, a Lenovo Company",
    "24.3198": "Tellium Incorporated",
    "24.3199": "synertronixx GmbH",
    "24.3194": "DaTARIUS Technologies GmbH",
    "24.3195": "ALPHA PROJECT Co.,Ltd.",
    "24.3196": "Internet Information Image Inc.",
    "24.3197": "Teikoku Electric Mfg. Co., Ltd",
    "24.3190": "Micro-Star International Co., Ltd.",
    "24.3191": "Life Racing Ltd",
    "24.3192": "In-Tech Electronics Limited",
    "24.3193": "Extel Communications P/L",
    "12.346860711936": "Enero Solutions inc.",
    "24.10772058": "ACTIVIO Inc.",
    "12.123917677936640": "WeWork Companies, Inc.",
    "24.13661839": "Calix Inc.",
    "24.9187238": "Solid Optics B.V.",
    "12.346862706688": "ENGICAM s.r.l.",
    "20.207357814505472": "B4ComTechnologies LLC",
    "20.255191441997824": "Guangzhou Tianyi Electronics Co., Ltd",
    "24.8169835": "Syrotech Networks. Ltd.",
    "12.123917678194688": "Ash Technologies",
    "12.346868789248": "The Security Center Inc",
    "24.13387478": "Cisco Systems, Inc",
    "12.123917681905664": "GSP Sprachtechnologie GmbH",
    "24.1625123": "Philio Technology Corporation",
    "24.3678376": "ColorTokens, Inc.",
    "24.2934227": "Ruckus Wireless",
    "24.13929017": "Samsung Electronics Co.,Ltd",
    "24.6582274": "Tp-Link Technologies Co.,Ltd.",
    "24.1596826": "BobjGear LLC",
    "24.1862053": "BlackBerry RTS",
    "24.14478228": "Cisco Systems, Inc",
    "24.15237458": "Apple, Inc.",
    "24.7115878": "Intel Corporate",
    "24.10268055": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5274882": "Biodit",
    "24.712": "Technocom Communications Technology (pte) Ltd",
    "24.713": "Mellanox Technologies, Inc.",
    "24.710": "Data Track Technology PLC",
    "24.711": "Alpsalpine Co,.Ltd",
    "24.716": "M.C.C.I",
    "24.717": "TeleDream, Inc.",
    "24.714": "EndPoints, Inc.",
    "24.715": "TriState Ltd.",
    "24.718": "FoxJet, Inc.",
    "24.719": "ZyGate Communications, Inc.",
    "20.272805462540288": "SigmaConnectivityAB",
    "20.171812091592704": "JNL Technologies Inc",
    "24.16527211": "Everspin Technologies, Inc.",
    "24.1338226": "Shenzhen Fortune Ship Technology Co., Ltd.",
    "20.268902731874304": "Simavita (Aust) Pty Ltd",
    "20.84197039407104": "Hanwang Technology Co.,Ltd",
    "24.4740866": "Huawei Technologies Co.,Ltd",
    "24.7473": "Highpro International R&D Co,.Ltd.",
    "24.7472": "YX Wireless S.A.",
    "24.7471": "QuantumVision Corporation",
    "24.7470": "Ruckus Wireless",
    "24.7477": "Viconics Electronics Inc.",
    "24.7476": "SYRIS Technology Corp",
    "24.7475": "Maverick Systems Inc.",
    "24.7474": "Longkay Communication & Technology (Shanghai) Co. Ltd",
    "24.7479": "Thales-Panda Transportation System",
    "24.7478": "Electronics Corporation Of India Limited",
    "12.346863964160": "Tata Power Company, Strategic Electronics Division",
    "24.5249842": "Taicang T&W Electronics",
    "12.71297885065216": "YXLON International A/S",
    "24.12073480": "Tenda Technology Co.,Ltd.Dongguan branch",
    "12.123917689262080": "Nilar AB",
    "24.13375912": "PT Inovação e Sistemas SA",
    "12.346853117952": "Smartmedia LLC",
    "24.5839": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5838": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.71297884041216": "LECO Corporation",
    "24.5835": "Apple, Inc.",
    "24.5834": "Nortel Networks",
    "24.5837": "Hiji High-Tech Co., Ltd.",
    "24.5836": "Xcute Mobile Corp.",
    "24.5831": "Cisco Systems, Inc",
    "24.5830": "North Atlantic Industries",
    "24.5833": "NAT Seattle, Inc.",
    "24.5832": "Cisco Systems, Inc",
    "24.572364": "AK-NORD EDV VERTRIEBSGES. mbH",
    "12.346863652864": "Kramara s.r.o.",
    "24.4213251": "Zyxel Communications Corporation",
    "24.12899384": "Huawei Technologies Co.,Ltd",
    "20.264165326323712": "Nanchang BlackShark Co.,Ltd.",
    "24.7639322": "Ruckus Wireless",
    "20.141720398331904": "Wheatstone Corporation",
    "24.7871578": "SHARP Corporation",
    "24.9475827": "Buffalo.Inc",
    "24.10800858": "Arcadyan Corporation",
    "12.346854002688": "InfinitiNetworks Inc.",
    "12.346855235584": "SHF Communication Technologies AG",
    "24.14419247": "National Products Inc.",
    "12.123917687582720": "Sunsa, Inc",
    "12.346863472640": "Steinbeis-Transferzentrum Embedded Design und Networking",
    "24.5044938": "Cambridge Industries(Group) Co.,Ltd.",
    "12.346857943040": "Junger Audio-Studiotechnik GmbH",
    "24.2644748": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "24.10492713": "Sagemcom Broadband SAS",
    "12.346859692032": "Sncf Eim Pays De Loire",
    "12.123917680881664": "Automatizacion Y Conectividad Sa De Cv",
    "24.2643": "Intronics, Incorporated",
    "24.2642": "AsiaRF Ltd.",
    "24.2641": "GyroSignal Technology Co., Ltd.",
    "24.2640": "Remotek Corporation",
    "24.2647": "Hewlett Packard",
    "24.2646": "HITACHI Maxell Ltd.",
    "24.2645": "MARKEM Corporation",
    "24.2644": "Laguna Hills, Inc.",
    "24.2649": "HW server",
    "24.2648": "Freyer & Siegel Elektronik GmbH & Co. KG",
    "24.7920031": "NuCom HK Ltd.",
    "24.7643978": "Zimi Corporation",
    "24.9218958": "Shanghai Feixun Communication Co.,Ltd.",
    "24.10527547": "Huawei Technologies Co.,Ltd",
    "24.8410": "Alcatel-Lucent Enterprise",
    "24.8411": "Xnet Technology, Inc.",
    "24.8412": "Densitron Taiwan Ltd.",
    "24.8413": "Cybertec Pty Ltd",
    "24.8414": "Japan Digital Laborat'Y Co.Ltd",
    "24.8415": "Kyosan Electric Mfg. Co., Ltd.",
    "24.8416": "Actiontec Electronics, Inc",
    "24.8417": "Alamar Electronics",
    "24.8418": "Information Resource Engineering",
    "24.8419": "Mcd Kencom Corporation",
    "12.123917675470848": "Lightdrop",
    "24.1098426": "UTT Co., Ltd.",
    "20.31657727885312": "Sirea",
    "24.11307725": "ROGER D.Wensker, G.Wensker sp.j.",
    "12.346856267776": "Alro Information Systems SA",
    "24.9736892": "Syntech(Hk) Technology Limited",
    "20.136646476431360": "Hangzhou Yiyitaidi Information Technology Co., Ltd.",
    "24.10497299": "AirTies Wireless Networks",
    "12.346856423424": "Grossenbacher Systeme AG",
    "12.346863443968": "OpenXS B.V.",
    "24.5169": "PDL Electronics Ltd",
    "24.5168": "ViPowER, Inc",
    "24.5163": "Edata Communication Inc.",
    "24.5162": "Elitegroup Computer Systems Co.,Ltd.",
    "24.5161": "V Center Technologies Co., Ltd.",
    "24.5160": "Vocollect Inc",
    "24.5167": "Savvius",
    "24.5166": "77 Elektronika Kft.",
    "24.5165": "Toradex AG",
    "24.5164": "Koncept International, Inc.",
    "24.3734292": "Huawei Technologies Co.,Ltd",
    "24.1587237": "Wuhan Lingjiu High-tech Co.,Ltd.",
    "12.123917690822656": "BRS Sistemas Eletrônicos",
    "24.11577143": "Roku, Inc.",
    "24.11540614": "SmarDTV",
    "24.159923": "Data Recall Ltd.",
    "24.12109642": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.16255934": "ARRIS Group, Inc.",
    "24.159920": "M/A-Com Inc. Companies",
    "24.3434570": "Teraworks Co., Ltd.",
    "24.12110340": "Holtek Semiconductor Inc.",
    "24.7928244": "Nokia Solutions and Networks GmbH & Co. KG",
    "12.123917687214080": "fpgalabs.com",
    "24.7377795": "Intelligent Optical Network High Tech CO.,LTD.",
    "12.123917675962368": "AEV Broadcast Srl",
    "20.260167870447616": "Zhejiang HEJU Communication Technology Co., Ltd",
    "20.62335524798464": "Dallas Delta Corporation",
    "24.15245292": "Chengdu Kt Electronic Hi-Tech Co.,Ltd",
    "24.4722639": "Hewlett Packard",
    "20.31368102805504": "Nebbiolo Technologies",
    "20.167137993818112": "AGV spa",
    "24.13382587": "Sagemcom Broadband SAS",
    "24.2120542": "Shenzhen Wonhe Technology Co., Ltd",
    "12.346861731840": "Rogue Engineering Inc.",
    "24.316367": "Apple, Inc.",
    "24.7870261": "Nokia Shanghai Bell Co., Ltd.",
    "24.9180741": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.14427943": "Huawei Device Co., Ltd.",
    "24.2419865": "Askey Computer Corp",
    "12.123917677162496": "Tieline Research Pty Ltd",
    "12.123917680144384": "Digital Domain",
    "24.8187745": "STR Elektronik Josef Schlechtinger GmbH",
    "24.12880474": "Jfcontrol",
    "24.2401415": "Huawei Device Co., Ltd.",
    "24.5299431": "Amazon Technologies Inc.",
    "12.123917675515904": "Panoramic Power",
    "20.88383267274752": "XSLAB Inc.",
    "24.10779549": "Espressif Inc.",
    "12.346852261888": "CAD-UL GmbH",
    "20.149945501876224": "Shenzhen Xin Kingbrand Enterprises Co.,Ltd",
    "20.93020700540928": "Hangzhou duotin Technology Co., Ltd.",
    "20.137215113953280": "1000eyes GmbH",
    "24.53288": "Harmonic, Inc",
    "24.53289": "Wakefern Food Corporation",
    "24.53286": "Hirschmann Austria Gmbh",
    "24.53287": "Applied Automation, Inc.",
    "24.53284": "Cognex Corporation",
    "24.53285": "Xrosstech, Inc.",
    "24.53282": "Incredible Technologies, Inc.",
    "24.53283": "Infortrend Technology, Inc.",
    "24.53280": "Aim System, Inc.",
    "24.2949": "Cisco Systems, Inc",
    "12.123917687554048": "Rch Italia Spa",
    "24.13150716": "Goyoo Networks Inc.",
    "24.313351": "Xiaomi Communications Co Ltd",
    "24.12366600": "Hongkong Ragentek Communication Technology Co.,Limited",
    "24.8154169": "Pixsys Srl",
    "24.11073867": "Eltex Enterprise Ltd.",
    "24.8155355": "Intel Corporate",
    "24.26925": "Sunnovo International Limited",
    "24.3814721": "Raspberry Pi (Trading) Ltd",
    "24.14160031": "Huawei Technologies Co.,Ltd",
    "24.8434228": "Technicolor CH USA Inc.",
    "24.13152717": "Apple, Inc.",
    "24.13107332": "Cisco Systems, Inc",
    "24.16268799": "Huawei Technologies Co.,Ltd",
    "24.6353290": "Huawei Technologies Co.,Ltd",
    "24.185": "Mcdonnell Douglas Computer Sys",
    "24.4200625": "Extreme Networks, Inc.",
    "24.7929687": "Intel Corporate",
    "24.184": "Seikosha Co., Ltd.",
    "24.3520": "Spagat AS",
    "24.37015": "Sycamore Networks",
    "12.346859986944": "Integrated Security Corporation",
    "24.37014": "Askey Computer Corp",
    "12.123917686599680": "merkur Funksysteme AG",
    "20.39986078416896": "CompanyDeep",
    "24.37017": "Allied Telesis, K.K.",
    "12.71297884635136": "NFT Automatisierungssysteme GmbH",
    "24.6323913": "Apple, Inc.",
    "12.346854862848": "IP Unity",
    "24.37016": "Sbc Designs, Inc.",
    "24.323": "IEEE 802",
    "12.123917680365568": "Tecnint HTE SRL",
    "24.322": "Cisco Systems, Inc",
    "24.14736264": "Open Standard Digital-IF Interface for SATCOM Systems",
    "24.1610950": "ShenZhen Chenyee Technology Co., Ltd.",
    "12.123917683388416": "eumig industrie-TV GmbH.",
    "24.321": "Cable Print",
    "24.37012": "Osprey Technologies, Inc.",
    "24.1074639": "Iqsim",
    "12.346864128000": "Thetis S.p.A.",
    "24.13658085": "Dell Inc.",
    "12.123917682126848": "NAJIN automation",
    "20.141268001751040": "Hangzhou Synway Information Engineering Co., Ltd",
    "12.346862399488": "GE Security-Kampro",
    "24.3973960": "vivo Mobile Communication Co., Ltd.",
    "24.4773973": "Telvent",
    "24.11037613": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.31346164498432": "Shandong Luneng Intelligence Technology CO., Ltd",
    "12.123917691146240": "Markus Labs",
    "24.5000040": "Mobile Device, Inc.",
    "12.123917682376704": "IBM Research GmbH",
    "24.14958882": "OnePlus Technology (Shenzhen) Co., Ltd",
    "12.346853380096": "TTPCom Ltd.",
    "24.11804143": "Internet Laboratories, Inc.",
    "24.10523473": "Cisco Systems, Inc",
    "12.123917687791616": "Infrasafe/ Advantor Systems",
    "20.163808564740096": "Cavity Eye",
    "24.13433965": "Riz Transmitters",
    "24.5993": "Cymphonix Corp",
    "24.8158180": "Z'Sedai Kenkyusho Corporation",
    "12.119269416960": "Bettini Srl",
    "24.1599175": "Shenzhen Libre Technology Co., Ltd",
    "12.346860060672": "Hosa Technology, Inc.",
    "24.6855906": "Cisco Systems, Inc",
    "20.176726075244544": "Private",
    "24.9527": "Runcom Technologies Ltd.",
    "24.9526": "Oki Electric Industry Co., Ltd.",
    "24.9525": "Minimax GmbH & Co KG",
    "24.9523": "Wittenstein Ag",
    "24.9522": "Digital Recorders",
    "24.9521": "Cloud Engines, Inc.",
    "24.9520": "Aetas Systems Inc.",
    "24.8127774": "Rafael Ltd.",
    "24.9529": "IfTA GmbH",
    "24.9528": "Samsung Electronics Co., Ltd., Memory Division",
    "12.123917687885824": "BTG Instruments AB",
    "24.3704322": "Flexoptix GmbH",
    "12.123917676441600": "Savroni̇k Elektroni̇k",
    "24.16044641": "SEMOCON Co., Ltd",
    "12.123917685944320": "8Cups",
    "24.3681745": "Samsung Electronics Co.,Ltd",
    "24.11058127": "Apple, Inc.",
    "24.12329874": "BYD Precision Manufacture Company Ltd.",
    "24.265922": "ViewSonic Mobile China Limited",
    "24.14701885": "Cisco Meraki",
    "24.2394631": "Mellanox Technologies, Inc.",
    "24.16562160": "Idaho National Laboratory",
    "24.2128152": "Intel Corporate",
    "24.14955667": "Huawei Technologies Co.,Ltd",
    "24.9711127": "Schneider Electric Canada Inc",
    "24.8452870": "Huawei Technologies Co.,Ltd",
    "20.167860141817856": "Shenzhen UniStrong Science & Technology Co., Ltd",
    "20.17622849552384": "Fujian Quanzhou Dong Ang Electronics Co., Ltd.",
    "24.16310241": "MyLight Systems",
    "12.71297884889088": "Hipodromo De Agua Caliente, S.A. De C.V.",
    "12.119269715968": "Momentum Data Systems",
    "24.787765": "Juniper Systems",
    "24.3708141": "Netgear",
    "24.14174855": "Oi Electric Co.,Ltd",
    "24.15261435": "Mesotic Sas",
    "12.123917681668096": "Roush",
    "24.62575": "Samsung Electronics Co.,Ltd",
    "24.2159736": "Xiaomi Communications Co Ltd",
    "12.346867712000": "Tamagawa Electronics Co.,Ltd.",
    "12.123917675270144": "Fibernet Ltd",
    "12.123917679710208": "Shenyang TECHE Technology Co.,Ltd",
    "24.7660855": "Radspin",
    "12.123917688066048": "BIT Group USA, Inc.",
    "12.346864459776": "Systemteq Limited",
    "24.10509761": "TMCT Co., LTD.",
    "24.4236964": "PassivSystems Ltd",
    "24.11598571": "Cisco Systems, Inc",
    "24.11019448": "Nokia",
    "24.3448699": "Canon Inc.",
    "24.15006850": "Juniper Networks",
    "20.75247168520192": "Changzhou Haitu Electronic Technology Co.,Ltd",
    "24.6562180": "Nippon Denki Kagaku Co.,LTD",
    "24.14711134": "Wuxi Xiaohu Technology Co.,Ltd.",
    "12.123917675606016": "Legrand Electric Ltd",
    "24.5263524": "Samsung Electronics Co.,Ltd",
    "12.346852495360": "JSR Ultrasonics",
    "12.346852515840": "Base 2",
    "12.346864078848": "Plitron Manufacturing Inc.",
    "24.16021030": "Viltechmeda UAB",
    "24.11037564": "Shenzhen Fenglian Technology Co., Ltd.",
    "24.9397": "Nortel Networks",
    "24.10761433": "Intel Corporate",
    "24.4200062": "ASUSTek COMPUTER INC.",
    "24.2894007": "Hangzhou Ruiying technology co., LTD",
    "20.212188446851072": "Xerox Corporation",
    "12.123917690376192": "Zhuhai Lonl electric Co.,Ltd.",
    "12.123917690634240": "robert juliat",
    "12.123917689491456": "em-tec GmbH",
    "24.8157841": "Intel Corporate",
    "24.11039365": "Idt, Inc.",
    "24.2114625": "Remote Solution",
    "20.273272610488320": "Avateq Corp.",
    "24.10266048": "vivo Mobile Communication Co., Ltd.",
    "24.7401123": "Eoptolink Technology Inc. Ltd,",
    "24.8887": "GSS Grundig SAT-Systems GmbH",
    "24.4207386": "Apple, Inc.",
    "24.5563104": "ARRIS Group, Inc.",
    "24.8886": "Superflow Technologies Group",
    "12.123917691301888": "IOOOTA Srl",
    "24.15762633": "zte corporation",
    "24.8885": "Novita",
    "24.4768172": "Plantronics, Inc.",
    "24.8151443": "HTC Corporation",
    "24.5293277": "Buffalo.Inc",
    "24.8884": "ARRIS Group, Inc.",
    "24.8883": "Sei S.p.A.",
    "24.8882": "4RF Communications Ltd",
    "24.8881": "Elbit Systems Ltd.",
    "24.8880": "D-Link Corporation",
    "24.8697153": "Samsung Electronics Co.,Ltd",
    "12.123917688786944": "ENGenesis",
    "20.220958194401280": "United Barcode Systems",
    "24.9745423": "Aruba, a Hewlett Packard Enterprise Company",
    "24.16015373": "Panoptic Technology, Inc",
    "24.10808919": "Seiko Epson Corporation",
    "12.346865250304": "Honeywell",
    "20.224447323701248": "MEDCOM sp. z o.o.",
    "24.576616": "Cisco Systems, Inc",
    "24.272168": "Microsoft Corporation",
    "24.13414568": "Huawei Device Co., Ltd.",
    "24.5783380": "LG Electronics (Mobile Communications)",
    "24.5022049": "Rain Bird Corporation",
    "12.346867617792": "Droplet Measurement Technologies",
    "20.146101714812928": "Logos01 Srl",
    "12.346860777472": "ID-KARTA s.r.o.",
    "24.14170364": "Ruckus Wireless",
    "24.32552": "Actiontec Electronics, Inc",
    "24.1587612": "Skorpios Technologies",
    "24.13405402": "Shenzhen Wei Da Intelligent Technology Go.,Ltd",
    "12.346868191232": "Green Instruments A/S",
    "24.3469124": "2Wire Inc",
    "24.41087": "Gsm-Syntel, Ltd.",
    "24.2944491": "Alpheus Digital Company Limited",
    "24.16571733": "Shenzhen WeWins wireless Co.,Ltd",
    "24.9440436": "Alinket Electronic Technology Co., Ltd",
    "12.71297884401664": "Mango DSP, Inc.",
    "24.4980866": "Cisco Systems, Inc",
    "24.3456166": "Beijing Ding Qing Technology, Ltd.",
    "24.16614": "C.A.E.N.",
    "24.7889276": "Mitsumi Electric Co.,Ltd.",
    "24.1350160": "Microsoft Corporation",
    "12.346863837184": "FarSite Communications Limited",
    "12.346862747648": "Eoslink",
    "24.7898350": "Indra Espacio S.A.",
    "24.5289621": "Micropoint Biotechnologies,Inc.",
    "20.216602663452672": "Beijing KDF information technology co. LTD.",
    "20.4884638203904": "Shenzhen WayOS Technology Crop., Ltd.",
    "20.180577514815488": "Plus One Japan Ltd.",
    "24.9492780": "Hug-Witschi Ag",
    "12.346861805568": "SMAVIS Inc.",
    "24.14465032": "Extreme Networks, Inc.",
    "24.1052031": "Netgear",
    "24.3704862": "Huawei Technologies Co.,Ltd",
    "24.10801617": "ECOtality North America",
    "12.71297885011968": "NPB Automation AB",
    "24.11809208": "Aviwest",
    "24.12596608": "A&R Tech",
    "24.9715874": "Anywave Communication Technologies, Inc.",
    "24.8682832": "Holley Metering Limited",
    "24.16384": "Pci Componentes Da Amzonia Ltd",
    "24.16385": "Zero One Technology Co Ltd (ZyXEL?)",
    "24.16386": "Perle Systems Limited",
    "24.16387": "Emerson Process Management Power & Water Solutions, Inc.",
    "24.9465004": "Fortinet, Inc.",
    "24.16388": "Icm Co. Ltd.",
    "24.16389": "Ani Communications Inc.",
    "12.346865930240": "Innovative Circuit Technology",
    "12.123917689196544": "Resolution Systems",
    "24.7139710": "Zhejiang Uniview Technologies Co.,Ltd.",
    "12.123917675278336": "cal4care Pte Ltd",
    "24.10005052": "Will i-tech Co., Ltd.",
    "12.123917679230976": "Comrod AS",
    "24.10767125": "Sun Cupid Technology (HK) LTD",
    "24.14182126": "Boxin Communication Co.,Ltd.",
    "12.71297884188672": "Comline Elektronik Elektrotechnik GmbH",
    "12.71297883344896": "Depro Electronique",
    "20.255861673951232": "Webfleet Solutions B.V.",
    "24.11197511": "Cachengo, Inc.",
    "24.7917561": "Huawei Device Co., Ltd.",
    "24.4999651": "Xiaomi Communications Co Ltd",
    "12.123917683466240": "TESCAN Brno, s.r.o.",
    "24.11317742": "Shenzhen Fenda Technology Co., Ltd",
    "12.346867625984": "Civolution",
    "20.251330456190976": "eZeLink LLC",
    "24.1635920": "Multimedia Pacific Limited",
    "24.1839457": "Ciena Corporation",
    "20.242185274392576": "Nex Technologies PTY LTD",
    "20.49277251354624": "Ezee Systems Limited",
    "12.346858926080": "Ferrari electronic AG",
    "24.3459389": "Intel Corporate",
    "12.346857758720": "Freiburger BlickZentrum",
    "24.8967436": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5249733": "Microsoft",
    "24.15211736": "Apple, Inc.",
    "12.346866454528": "MangoDSP",
    "12.346857652224": "Marposs SPA",
    "24.2103310": "Shenzhen Sunchip Technology Co., Ltd",
    "24.5369": "Vantage Controls",
    "24.5368": "Scientific Atlanta",
    "24.1868": "Beicom Inc.",
    "24.1869": "Zebra Technologies Corp.",
    "24.7882723": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.14221027": "D-Link International",
    "24.5361": "Cisco Systems, Inc",
    "24.5360": "Business Security OL AB",
    "24.5363": "ViXS Systems Inc",
    "24.5362": "Cisco Systems, Inc",
    "24.5365": "OSI Security Devices",
    "24.1867": "Daihen Corporation",
    "24.5367": "CREVIS Co., LTD",
    "24.1865": "CENiX Inc.",
    "24.13654133": "Active Control Technology Inc.",
    "12.346854834176": "Futaba Corporation",
    "24.4750070": "Ncs Electrical Sdn Bhd",
    "12.123917687250944": "Warecube,Inc",
    "24.4750079": "Essys",
    "24.4987453": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917684944896": "WIKA Alexander Wiegand SE & Co. KG",
    "24.559046": "Ingram Micro Services",
    "24.815660": "Tp-Link Technologies Co.,Ltd.",
    "12.123917690650624": "Impolux GmbH",
    "24.5025308": "SaiNXT Technologies LLP",
    "12.346858422272": "Grupo Epelsa S.L.",
    "24.7356671": "Huawei Device Co., Ltd.",
    "24.14736337": "Technicolor CH USA Inc.",
    "24.4475611": "Apple, Inc.",
    "12.123917685010432": "Richard Paul Russell Ltd",
    "12.346852564992": "Private",
    "24.12881857": "Deeptrack S.L.U.",
    "24.9457195": "Huawei Technologies Co.,Ltd",
    "12.346856357888": "SmartMotor AS",
    "12.346857074688": "Orbis Oy",
    "12.346859548672": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "24.11543697": "Huawei Device Co., Ltd.",
    "24.11273927": "ServerNet S.r.l.",
    "24.1605776": "Cisco Systems, Inc",
    "24.16576850": "Sony Corporation",
    "24.5552138": "D-Link International",
    "12.71297883951104": "iCOGNIZE GmbH",
    "24.15498422": "Samsung Electronics Co.,Ltd",
    "24.7081412": "Beijing Xiaomi Electronics Co., Ltd.",
    "24.9978204": "Nintendo Co.,Ltd",
    "24.4098": "Actia",
    "24.4099": "Imatron, Inc.",
    "24.8954046": "Huawei Technologies Co.,Ltd",
    "24.16561821": "Zhejiang Dahua Technology Co., Ltd.",
    "12.123917680590848": "Teletypes Manufacturing Plant",
    "24.4090": "Optinel Systems, Inc.",
    "24.4091": "Nippon Denso Industry Co., Ltd.",
    "24.4092": "Merit Li-Lin Ent.",
    "24.4093": "Glorytek Network Inc.",
    "24.4094": "G-Pro Computer",
    "24.4095": "Control4",
    "24.4096": "Cable Television Laboratories, Inc.",
    "24.4097": "Citel",
    "24.4777664": "SIMCom Wireless Solutions Co.,Ltd.",
    "12.123917683077120": "BroadSoft Inc",
    "24.5281650": "Westinghouse Digital",
    "24.12880101": "Samsung Electronics Co.,Ltd",
    "20.452703813632": "Command Alkon, Inc",
    "20.44169484566528": "Telecom and Microelectonic Industries",
    "20.70443730993152": "ADH Guardian USA",
    "24.2159477": "ARRIS Group, Inc.",
    "24.13424444": "Huawei Technologies Co.,Ltd",
    "24.4542": "AGP Telecom Co. Ltd",
    "24.4543": "AESYS S.p.A.",
    "24.4540": "Cisco Systems, Inc",
    "24.4541": "Bombardier Transportation",
    "24.4546": "United Fiber Optic Communication",
    "24.4547": "Transceiving System Technology Corporation",
    "24.4544": "Aday Technology Inc",
    "24.1827": "ELCON Systemtechnik GmbH",
    "24.4548": "Terminales de Telecomunicacion Terrestre, S.L.",
    "24.4549": "TEN Technology",
    "24.11853535": "zte corporation",
    "24.13945266": "Quanergy Systems Inc",
    "12.346853793792": "Sparr Electronics Ltd",
    "12.346857857024": "Lab-El Elektronika Laboratoryjna S.J.",
    "12.123917683953664": "Innovative Circuit Technology",
    "20.111031374315520": "Bronkhorst High-Tech BV",
    "24.13530": "LG Electronics (Mobile Communications)",
    "24.2661111": "ADVA Optical Networking Ltd.",
    "12.123917684551680": "Dolotron d.o.o.",
    "24.2661114": "TCT mobile ltd",
    "24.57466": "Mikrodidakt Ab",
    "24.57467": "Bay Networks",
    "24.1062474": "Boston Dynamics",
    "24.57465": "A.T.N.R.",
    "24.57462": "Development Concepts, Inc.",
    "24.57463": "Webgear, Inc.",
    "24.57460": "Tiernan Communications, Inc.",
    "24.57461": "Verilink Corporation",
    "24.13131064": "HUMAX Co., Ltd.",
    "24.1077795": "Tellescom Industria E Comercio Em Telecomunicacao",
    "24.57468": "Mettler-Toledo, Inc.",
    "24.57469": "Encore (Netronix?)",
    "24.20576": "Tandberg Telecom As",
    "24.20574": "Digitek Micrologic S.A.",
    "24.20575": "Brand Innovators",
    "24.20572": "Tundo Corporation",
    "24.20570": "Network Alchemy, Inc.",
    "24.20571": "Kawasaki Lsi U.S.A., Inc.",
    "24.1890855": "vivo Mobile Communication Co., Ltd.",
    "24.12590128": "Shenzhen WIFI-3L Technology Co.,Ltd",
    "24.20578": "Kouwell Electronics Corp. **",
    "24.20579": "Oy Comsel System Ab",
    "24.11549390": "Huawei Device Co., Ltd.",
    "20.31346167644160": "C&A Marketing, INC.",
    "12.123917680160768": "Infosoft Digital Design & Services Private Limited",
    "24.15789755": "VIPAR GmbH",
    "24.5026024": "Beijing RongZhi xinghua technology co., LTD",
    "24.22591": "PC Aquarius",
    "20.9814047457280": "Origami Energy Ltd",
    "24.6870938": "Ubiquiti Networks Inc.",
    "24.12874032": "Fon Technology S.L.",
    "24.6615803": "Hangzhou Ezviz Software Co.,Ltd.",
    "24.848878": "EM Microelectronic",
    "12.346865283072": "Gronic Systems GmbH",
    "24.7880275": "BSkyB Ltd",
    "24.5042006": "Shenzhen Sundray Technologies Company Limited",
    "20.101728415383552": "Sichuan C.H Control Technology Co., Ltd.",
    "20.145727906906112": "thousand star tech LTD.",
    "24.6032330": "Tunstall Nordic AB",
    "24.3011": "Multiplex, Inc.",
    "24.3010": "Corinex Communication Corp.",
    "24.3013": "SMC Networks, Inc.",
    "24.3012": "BIOTRONIK GmbH & Co",
    "24.3015": "ICET S.p.A.",
    "24.3014": "ISAC, Inc.",
    "24.2417727": "Wenzhou Kunmei Communication Technology Co.,Ltd.",
    "24.3016": "AirFlow Networks",
    "24.3019": "Fagor Automation , S. Coop",
    "24.3018": "Datavan Tc",
    "24.6825783": "Samsung Electronics Co.,Ltd",
    "24.3188242": "ShenZhen Hugsun Technology Co.,Ltd.",
    "24.801893": "Dome Imaging Inc",
    "24.4744686": "IEEE Registration Authority",
    "12.71297884618752": "Thermo Fisher Sceintific",
    "24.10015379": "Google, Inc.",
    "12.123917682704384": "Grossenbacher Systeme AG",
    "24.12090309": "Apple, Inc.",
    "12.346859405312": "Anymax",
    "20.62070502457344": "Light Blue Optics Ltd.",
    "24.6838655": "Alcatel Lucent",
    "12.346852409344": "BrainBoxes Ltd",
    "24.15525205": "Hirschmann Automation",
    "24.6086196": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.14470844": "Shenzhen JSR Technology Co.,Ltd.",
    "24.14470849": "Cisco Systems, Inc",
    "12.123917689118720": "attocube systems AG",
    "12.346865586176": "FUJI DATA SYSTEM Co.,Ltd.",
    "24.576551": "Motorola Mobility LLC, a Lenovo Company",
    "24.6082869": "Xtreme Power Systems",
    "12.123917688332288": "Kuebrich Ingeniergesellschaft mbh & Co. KG",
    "24.7886610": "Mobile Integration Workgroup",
    "12.346863521792": "Mighty Lube Systematic Lubrication, Inc.",
    "24.15204749": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16005047": "Apple, Inc.",
    "24.16296709": "Sagemcom Broadband SAS",
    "24.8930423": "Huawei Technologies Co.,Ltd",
    "24.16558774": "Samsung Electronics Co.,Ltd",
    "24.14717532": "Apple, Inc.",
    "12.346852356096": "Sinetica Corp",
    "24.20688": "Minerva Systems",
    "12.346853007360": "RFL Electronics, Inc.",
    "24.3689334": "AIRTAME ApS",
    "24.20684": "Seagate Cloud Systems Inc",
    "24.20685": "Digianswer A/S",
    "24.20686": "Lg International Corp.",
    "24.20687": "Vanlink Communication Technology Research Institute",
    "24.20680": "Addonics Technologies, Inc.",
    "24.20681": "Maspro Denkoh Corp.",
    "24.20682": "Net To Net Technologies",
    "24.20683": "Jetter",
    "12.123917676916736": "Airobot Oü",
    "24.3936486": "PHAZR Inc.",
    "20.106196534034432": "Beijing Sinomedisite Bio-tech Co.,Ltd",
    "24.6066685": "Jaewoncnc",
    "24.6799": "Nortel Networks",
    "24.6798": "3Way Networks Ltd",
    "24.2923806": "Huawei Technologies Co.,Ltd",
    "24.6795": "Chunil Electric Ind., Co.",
    "24.6794": "Samsung Electronics Co.,Ltd",
    "24.6797": "AVECS Bergen GmbH",
    "24.6796": "Sophos Ltd",
    "24.6791": "Canhold International Limited",
    "24.6790": "New Wave Design & Verification",
    "24.6793": "Nokia Danmark A/S",
    "24.6792": "Venergy,Co,Ltd",
    "24.14469011": "Nokia Corporation",
    "24.6088835": "Huawei Technologies Co.,Ltd",
    "24.6106": "Spans Logic",
    "12.123917690048512": "Fracarro srl",
    "24.16306215": "Seiko Epson Corporation",
    "12.123917677924352": "Season Design Technology",
    "24.8141139": "zte corporation",
    "24.2148075": "Panasonic Corporation AVC Networks Company",
    "24.13655908": "IEEE Registration Authority",
    "12.123917684776960": "Requea",
    "12.346853421056": "ChipWrights, Inc.",
    "24.8154616": "Apple, Inc.",
    "12.123917676204032": "Securolytics, Inc.",
    "24.3708306": "Beijing Tendyron Corporation",
    "24.6589854": "IVT Electronic Co.,Ltd",
    "24.9646": "Microsoft Corporation",
    "24.9647": "COMFILE Technology",
    "24.9644": "I-Tech corporation",
    "24.9645": "Manufacturing Resources International",
    "24.9642": "Beijing Soul Technology Co.,Ltd.",
    "24.9643": "Aio Lcd Pc Bu / Tpv",
    "24.9640": "Kontron (BeiJing) Technology Co.,Ltd",
    "24.9641": "Shanghai Embedway Information Technologies Co.,Ltd",
    "24.509": "Digital Voice Systems, Inc.",
    "24.508": "Keyence Corporation",
    "24.9648": "Schmartz Inc",
    "24.9649": "Maya-Creation Corporation",
    "24.1413": "Juniper Networks",
    "24.1412": "AbsoluteValue Systems, Inc.",
    "24.1411": "ImageCom Limited",
    "24.1410": "ClearCube Technology",
    "24.1417": "National Datacomputer",
    "24.1416": "Sensoria Corp.",
    "24.1415": "Locus, Incorporated",
    "24.1414": "Lucent Technologies",
    "24.5820513": "Huawei Technologies Co.,Ltd",
    "12.346860998656": "Tieline Research Pty Ltd",
    "24.1418": "Netcom Co., Ltd.",
    "24.10522232": "Sagemcom Broadband SAS",
    "24.8127696": "Apple, Inc.",
    "24.5002975": "Targus US LLC",
    "24.15475173": "Toshiba",
    "24.2852": "AirLogic",
    "24.2853": "Aeluros",
    "24.2850": "Environmental Systems and Services",
    "24.2851": "Siemens Home & Office Comm. Devices",
    "24.2856": "Quatech Inc.",
    "24.2857": "LS(LG) Industrial Systems co.,Ltd",
    "24.2854": "Wetek Corporation",
    "24.2855": "Scion Corporation",
    "24.14162942": "Twinlinx Corporation",
    "24.2858": "HOWTEL Co., Ltd.",
    "24.2859": "Hostnet Corporation",
    "24.13421772": "Silicon Laboratories",
    "12.123917684846592": "Triax A/S",
    "24.6002": "ASTRO Strobel Kommunikationssysteme GmbH",
    "24.8134756": "Alcatel-Lucent IPD",
    "24.12599082": "Biscotti, Inc.",
    "12.123917691117568": "Selex ES Inc.",
    "24.5536338": "CTE International srl",
    "20.150262241034240": "psb intralogistics GmbH",
    "24.14473343": "Shenzhen JoinCyber Telecom Equipment Ltd",
    "12.123917685153792": "Volansys technologies pvt ltd",
    "24.1096183": "Niko-Servodan",
    "24.798089": "QiiQ Communications Inc.",
    "12.346866016256": "Enatel Limited",
    "12.123917675696128": "ProConnections, Inc.",
    "24.8695128": "Huawei Technologies Co.,Ltd",
    "12.346867101696": "Visito S.R.L.",
    "24.15767709": "Apple, Inc.",
    "24.6310337": "Avaya Inc",
    "12.346853257216": "MarekMicro GmbH",
    "24.54834": "GE Energy",
    "24.1074818": "NEC Platforms, Ltd.",
    "24.16253746": "Khomp",
    "24.7632200": "Amazon Technologies Inc.",
    "24.7666122": "Nintendo Co.,Ltd",
    "24.15791644": "Apple, Inc.",
    "24.8595": "Videofon MV",
    "24.8594": "Baoding Galaxy Electronic Technology Co.,Ltd",
    "24.8597": "GWD Media Limited",
    "24.8596": "Ping Communication",
    "24.8591": "Avantgarde Acoustic Lautsprechersysteme GmbH",
    "24.8590": "Mekics Co., Ltd.",
    "24.8593": "D-Link Corporation",
    "24.8592": "Goliath Solutions",
    "24.3726267": "Sunbow Telecom Co., Ltd.",
    "12.123917688209408": "Trust Automation",
    "24.8599": "Elitegroup Computer Systems Co.,Ltd.",
    "24.8598": "Telsey S.p.A.",
    "24.11276755": "Suzhou HOTEK Video Technology Co. Ltd",
    "20.132855974854656": "Ekoenergetyka - Polska S.A.",
    "24.7127659": "Apple, Inc.",
    "12.346866028544": "Apex NV",
    "24.533794": "Advansee",
    "24.533797": "Xiaomi Communications Co Ltd",
    "24.13933216": "Vnpt Technology",
    "12.123917685874688": "ConectaIP Tecnologia S.L.",
    "24.15770149": "Amazon Technologies Inc.",
    "24.3969323": "Vexata Inc",
    "12.123917688070144": "Ela Innovation",
    "24.3420014": "Anywire corporation",
    "24.7648950": "eero inc.",
    "20.88914835537920": "Traxens",
    "20.238070230155264": "Vrinda Nano Technologies Pvt Ltd",
    "12.123917688115200": "Dsp Design",
    "24.9468456": "Beijing Morncloud Information And Technology Co. Ltd.",
    "24.1627574": "SMG Holdings LLC",
    "24.9234316": "DK Networks",
    "24.4939": "ToGoldenNet Technology Inc.",
    "24.4938": "Engim, Inc.",
    "24.4933": "Eaton Corporation",
    "24.2941970": "Sagemcom Broadband SAS",
    "24.4931": "Matsushita Electronic Components (Europe) GmbH",
    "24.4930": "Vision Research, Inc.",
    "24.4937": "Zyxel Communications Corporation",
    "24.4936": "Artila Electronics Co., Ltd.",
    "24.4935": "Red Lion Controls, LP",
    "24.4934": "D-Link Corporation",
    "24.11569408": "Huawei Technologies Co.,Ltd",
    "24.13895064": "ARRIS Group, Inc.",
    "24.11014279": "Texas Instruments",
    "20.149935440789504": "Premier Merchandises Limited",
    "24.7348356": "Hon Hai Precision Ind. Co., Ltd.",
    "24.2925236": "Fortify Technologies, LLC",
    "24.6300580": "Sichuan AI-Link Technology Co., Ltd.",
    "12.123917687816192": "Cresprit Inc.",
    "12.123917683679232": "National Time & Signal Corp.",
    "24.7895663": "Juice Technology AG",
    "24.14721495": "Sfr",
    "24.5513173": "Advantage Electronics",
    "12.123917676683264": "WiSuite USA",
    "24.12905443": "Rrcn Sas",
    "24.5947": "Cisco Systems, Inc",
    "24.2424077": "Intelbras",
    "24.4727320": "Shenzhen Yipingfang Network Technology Co., Ltd.",
    "24.2684422": "Syes srl",
    "12.346865033216": "QNE GmbH & Co. KG",
    "20.140782601240576": "Private",
    "12.346858143744": "Terrascience Systems Ltd.",
    "12.346861359104": "Vortex Engineering pvt ltd",
    "12.346864246784": "Asto",
    "24.8660624": "Beijing Thought Science Co.,Ltd.",
    "24.6087948": "Beijing Xiaomi Mobile Software Co., Ltd",
    "20.180698119929856": "Shenzhen City billion Leiden science and Technology Co., Ltd.",
    "20.48543326797824": "Aaton Digital",
    "20.52821125758976": "Newtons4th Ltd",
    "12.71297885081600": "Founder Broadband Network Service Co.,Ltd.",
    "24.3168603": "streamnow AG",
    "24.2639786": "Nokia Corporation",
    "24.11065868": "Servercom (India) Private Limited",
    "12.123917684744192": "Shenzhen Wisewing Internet Technology Co.,Ltd",
    "20.132855971708928": "Burisch Elektronik Bauteile GmbH",
    "24.4995948": "Garo Ab",
    "20.180698118881280": "Engineered SA",
    "20.71389214146560": "BloomSky,Inc.",
    "12.346857738240": "Innovative Neurotroncs",
    "12.123917675347968": "Groupeer Technologies",
    "24.3934036": "Apple, Inc.",
    "20.250817676312576": "SAGE Glass",
    "24.1612277": "Sichuan Changhong Electric Ltd.",
    "20.31614741512192": "Shenzhen Kinstone D&T Develop Co.,Ltd",
    "24.13143427": "Huawei Technologies Co.,Ltd",
    "24.9704997": "Apple, Inc.",
    "12.123917676220416": "OOO \"Microlink-Svyaz\"",
    "24.10286222": "Apple, Inc.",
    "12.123917685719040": "Aditec GmbH",
    "20.31368098611200": "Schawbel Technologies LLC",
    "24.4779392": "daesung eltec",
    "24.9222218": "Samsung Electro-Mechanics(Thailand)",
    "24.15233421": "Elsys Equipamentos Eletronicos Ltda",
    "24.15523642": "vivo Mobile Communication Co., Ltd.",
    "24.4511218": "IEEE Registration Authority",
    "24.9748420": "Ruckus Wireless",
    "24.2380394": "Zhejiang Dahua Technology Co., Ltd.",
    "24.60093": "Cisco Systems, Inc",
    "24.4462114": "Microsoft Corporation",
    "24.7103085": "Micro-Star INT'L CO., LTD",
    "24.15790688": "Mobitec AB",
    "12.346854043648": "quantumBEAM Limited",
    "24.14213304": "Bulat Llc",
    "24.16518213": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "12.346866327552": "Audio Authority Corp",
    "12.123917676183552": "GSP Sprachtechnologie GmbH",
    "20.225207125016576": "Elk Products",
    "24.9188986": "Palo Alto Networks",
    "24.11061375": "Roqos, Inc.",
    "12.123917677101056": "NextEV Co., Ltd.",
    "12.346860244992": "DIT-MCO International",
    "12.346862555136": "Ciemme Sistemi Spa",
    "24.8442397": "AzureWave Technology Inc.",
    "20.22739794526208": "TMG TE GmbH",
    "12.123917679312896": "Twoway Communications, Inc.",
    "24.12332861": "vivo Mobile Communication Co., Ltd.",
    "12.346855751680": "RISCO Group",
    "24.6853408": "New H3C Technologies Co., Ltd",
    "24.3151128": "Ubiquitous Communication Co. ltd.",
    "12.346868420608": "Plda",
    "20.211525442732032": "Zhejiang Haikang Science And Technology Co.,Ltd",
    "20.83889191124992": "Glonexs",
    "20.88668796616704": "Seneco A/S",
    "24.3989921": "Universal Global Scientific Industrial Co., Ltd.",
    "24.7614312": "AzureWave Technology Inc.",
    "24.7106617": "Novar GmbH",
    "24.6578564": "Velux",
    "12.123917677461504": "BAYKON Endüstriyel Kontrol Sistemleri San. ve Tic. A.Ş.",
    "24.14194576": "Commonwealth Scientific and Industrial Research Organisation",
    "24.7617560": "Taicang T&W Electronics",
    "20.172584090992640": "KXT Technology Co., Ltd.",
    "12.123917681418240": "PKE Electronics AG",
    "24.13432629": "Amazon Technologies Inc.",
    "24.16575056": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917686108160": "Bizwerks, LLC",
    "20.93020699492352": "Xi'an Hua Fan Technology Co.,Ltd.",
    "24.16537052": "Baltic Latvian Universal Electronics LLC",
    "12.123917680398336": "Plantiga Technologies Inc",
    "20.242876962308096": "Ambi Labs Limited",
    "20.97400610881536": "Alunos AG",
    "12.346856386560": "AIE Etudes",
    "24.15002243": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917678436352": "Ultamation Limited",
    "12.123917677694976": "Sequent AG",
    "24.12062932": "Masimo Corporation",
    "12.123917679095808": "JNR Sports Holdings, LLC",
    "24.16257079": "Atopia Systems, LP",
    "20.198611823493120": "Canaan Creative Co.,Ltd.",
    "24.13407904": "Huawei Technologies Co.,Ltd",
    "12.346859749376": "Simicon",
    "12.346860769280": "Communications Laboratories Inc",
    "24.15515231": "Vestel Elektronik San ve Tic. A.Ş.",
    "24.14216476": "Magna Technology SL",
    "24.7665446": "Neuron Robotics",
    "12.346860388352": "InnoScan K/S",
    "12.346852900864": "Wavefly Corporation",
    "12.346860843008": "Lars Morich Kommunikationstechnik GmbH",
    "12.123917691490304": "Eyedea",
    "24.2924887": "acromate, Inc.",
    "24.269242": "Samsung Electronics Co.,Ltd",
    "24.12888737": "SUMMIT DEVELOPMENT, spol.s r.o.",
    "24.5031894": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "24.7368717": "Apple, Inc.",
    "20.52821128904704": "Ampetronic Ltd",
    "24.9217759": "Tp-Link Technologies Co.,Ltd.",
    "12.71297884614656": "CT Company",
    "24.14947830": "Texas Instruments",
    "24.3438023": "Avaya Inc",
    "24.8399610": "Germaneers GmbH",
    "12.123917687504896": "Tokyo Communication Equipment MFG Co.,ltd.",
    "12.123917686870016": "Critical Link LLC",
    "24.2382091": "White Sky Inc. Limited",
    "24.7655168": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24.12093294": "Yandex Services AG",
    "24.4240142": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.4245449": "Naviit Inc.",
    "24.3958806": "VXi Corporation",
    "24.15793771": "Shanghai High-Flying Electronics Technology Co., Ltd",
    "24.10506196": "ADB Broadband Italia",
    "24.8412751": "Fn-Link Technology Limited",
    "24.16579319": "Shanghai Baud Data Communication Co.,Ltd.",
    "24.8928548": "Google, Inc.",
    "24.8156720": "Shenzhen YOUHUA Technology Co., Ltd",
    "20.5297546461184": "ETL Elektrotechnik Lauter GmbH",
    "24.8156725": "Intel Corporate",
    "24.8188699": "LG Electronics (Mobile Communications)",
    "12.123917686976512": "Cambria Corporation",
    "24.2674665": "Apple, Inc.",
    "24.10788745": "Shanghai Rui Rui Communication Technology Co.Ltd.",
    "24.4471602": "Intelbras",
    "12.123917684322304": "Dorlet Sau",
    "12.346861326336": "Matrix Switch Corporation",
    "12.123917677817856": "Enlaps",
    "24.8966847": "German Autolabs",
    "12.123917681799168": "Stesalit Systems Ltd",
    "24.9190115": "LG Electronics (Mobile Communications)",
    "12.123917689270272": "Russian Telecom Equipment Company",
    "20.154794729668608": "Evaporcool Solutions",
    "24.14161963": "Conti Temic Microelectronic GmbH",
    "24.16011005": "Actions Semiconductor Co.,Ltd.(Cayman Islands)",
    "24.5525830": "AMG Systems Ltd.",
    "24.3934454": "Cisco Systems, Inc",
    "24.12091898": "Apple, Inc.",
    "24.539272": "Universal Global Scientific Industrial Co., Ltd.",
    "24.3701575": "AKELA, Inc.",
    "24.9756494": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "20.127658787602432": "Shen Zhen Yingjiachuang Electronics Technology Co.,Ltd.",
    "20.141268000702464": "Shiroshita Industrial Co., Ltd.",
    "24.15470052": "Amazon Technologies Inc.",
    "24.4253259": "Apple, Inc.",
    "24.15730929": "Samsung Electronics Co.,Ltd",
    "12.123917690957824": "Mitsubishi Electric Micro-Computer Application Software Co.,Ltd.",
    "24.4214507": "Higher Way Electronic Co., Ltd.",
    "12.346863382528": "Compañía de Instrumentacion y control, S.L.",
    "24.3158837": "Boosty",
    "12.123917688594432": "Tornado Modular Systems",
    "24.1083931": "RAONIX Inc.",
    "12.346865958912": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "20.194363546140672": "LOWOTEC GmbH",
    "12.123917677727744": "KMtronic ltd",
    "24.8390024": "Ruijie Networks Co.,LTD",
    "12.346868121600": "AeroControl, Inc.",
    "24.4990695": "Welgate Co., Ltd.",
    "24.11551764": "New H3C Technologies Co., Ltd",
    "24.13165615": "Technical Research Design and Development",
    "12.346865688576": "Aplica Technologies",
    "24.14969889": "messMa GmbH",
    "24.3985011": "Cisco Systems, Inc",
    "24.11040214": "FreeTek International Co., Ltd.",
    "24.4505": "2wcom Systems GmbH",
    "24.4223363": "Juniper Networks",
    "12.123917684068352": "KST technology",
    "24.4473936": "OttoQ",
    "12.346861236224": "Pixeye Ltd",
    "24.288101": "Testop",
    "24.5791960": "ARRIS Group, Inc.",
    "24.13124932": "Asia Pacific CIS (Wuxi) Co, Ltd",
    "12.346862465024": "Telemetrie Elektronik GmbH",
    "24.7650824": "ATEK Products, LLC",
    "24.12882383": "Luxul",
    "12.346862026752": "BuLogics, Inc.",
    "24.1105396": "Maxway Electronics CO.,LTD",
    "24.16010352": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "20.111031370121216": "XIMO Communication Technology Co., Ltd",
    "24.13121648": "Samsung Electronics Co.,Ltd",
    "20.260167871496192": "Fci",
    "12.346868404224": "D-Hike Electroncs Technology Co.,Ltd",
    "24.1367465": "ASUSTek COMPUTER INC.",
    "24.9213930": "Freebox Sas",
    "24.11296092": "Fn-Link Technology Limited",
    "12.346865295360": "Bruckner & Jarosch Ingenieurgesellschaft mbH",
    "24.5771447": "Infinix mobility limited",
    "24.16036196": "Lightning Telecommunications Technology Co. Ltd",
    "20.211670379003904": "Private",
    "24.10764859": "Intel Corporate",
    "24.10749673": "Texas Instruments",
    "24.482": "Ando Electric Corporation",
    "24.7746": "Teltonika",
    "12.346862276608": "Conolog Corporation",
    "24.8682651": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "12.123917691654144": "Shenzhen Zhiting Technology Co.,Ltd",
    "24.12635853": "Apple, Inc.",
    "24.483": "Siemens AG",
    "12.123917690839040": "Sator Controls s.r.o.",
    "24.8968591": "Juniper Networks",
    "24.8936519": "Alcatel Lucent",
    "24.7820": "ASUSTek COMPUTER INC.",
    "24.7821": "ARRIS Group, Inc.",
    "24.7822": "Hunkeler AG",
    "24.7823": "Canon Inc.",
    "24.7824": "Elitegroup Computer Systems Co.,Ltd.",
    "24.7825": "KIMIN Electronic Co., Ltd.",
    "24.7826": "Jeulin S.A.",
    "24.7827": "CiriTech Systems Inc",
    "24.7828": "Supercom Technology Corporation",
    "24.7829": "Sigmalink",
    "20.207147378933760": "Private",
    "20.211776046104576": "Annapurna labs",
    "12.346860568576": "SoftSwitching Technologies",
    "24.3147597": "Samsung Electro-Mechanics(Thailand)",
    "12.346860584960": "ITB CompuPhase",
    "24.1052203": "NEC CASIO Mobile Communications",
    "12.346866708480": "Device GmbH",
    "12.123917687947264": "RMI Laser LLC",
    "12.123917680263168": "Xsight Systems Ltd.",
    "24.16536270": "Control iD",
    "24.1878976": "Hitron Technologies. Inc",
    "12.346852593664": "EC Elettronica S.R.L.",
    "24.10255314": "NEOLAB Convergence",
    "24.16018980": "NEC Personal Computers, Ltd.",
    "24.279969": "NIRIT- Xinwei Telecom Technology Co., Ltd.",
    "24.16299927": "NEC Platforms, Ltd.",
    "24.1051954": "Embedian, Inc.",
    "12.123917679730688": "Aeryon Labs Inc",
    "24.1858656": "Shenzhen Belzon Technology Co.,LTD.",
    "24.1051951": "Online Security Pty. Ltd.",
    "24.4472327": "Electrolux",
    "24.3941115": "Apple, Inc.",
    "24.8709908": "Liang Herng Enterprise,Co.Ltd.",
    "24.11582437": "Sagemcom Broadband SAS",
    "24.4474441": "DFI (Diamond Flower Industries)",
    "24.2103106": "LG Electronics",
    "24.8663189": "Hon Hai Precision IND.CO.,LTD",
    "20.167152459972608": "Zoleo Inc.",
    "24.12338478": "Knowledge Development for POF S.L.",
    "20.250817680506880": "Teletask Belgium",
    "12.123917686788096": "Inras GmbH",
    "24.10255779": "Huawei Technologies Co.,Ltd",
    "24.12069545": "Apple, Inc.",
    "12.346868850688": "Brunel GmbH Section Communications",
    "24.2943708": "Askey Computer Corp",
    "24.16545254": "Handreamnet",
    "20.31614737317888": "Hangzhou Softel Optic Co., Ltd",
    "24.7921110": "c-scape",
    "24.7921113": "Guangzhou Shiyuan Electronics Co., Ltd.",
    "12.346865123328": "Wavecom Elektronik Ag",
    "24.16264767": "Huawei Technologies Co.,Ltd",
    "20.167152469409792": "NAB co,.LTD",
    "12.123917685936128": "Metek Meteorologische Messtechnik GmbH",
    "24.13688137": "Juniper Networks",
    "24.6305139": "Earda Technologies co Ltd",
    "24.1580011": "Actiontec Electronics, Inc",
    "24.1351065": "Apple, Inc.",
    "20.233258803527680": "MusicLens Inc.",
    "24.14985157": "Huawei Technologies Co.,Ltd",
    "20.39672887640064": "Shandong Kehui Power Automation Co. Ltd.",
    "24.14686327": "Fujitsu Limited",
    "12.123917682335744": "Nto Ire-Polus",
    "12.123917686845440": "Network Customizing Technologies Inc",
    "24.15238309": "Intel Corporate",
    "12.346864316416": "ikerlan",
    "12.119269855232": "UAB Kitron",
    "24.6556631": "Apple, Inc.",
    "24.15764008": "JIANGSU HENGSION ELECTRONIC S and T CO.,LTD",
    "12.346859376640": "NewOnSys Ltd.",
    "24.11051416": "TCT mobile ltd",
    "24.9752589": "Huawei Technologies Co.,Ltd",
    "12.123917677301760": "Delta Tau Data Systems, Inc.",
    "24.5397": "Leipold+Co.GmbH",
    "24.7363244": "Samsung Electronics Co.,Ltd",
    "24.9762467": "Apple, Inc.",
    "24.5538": "ARRIS Group, Inc.",
    "24.5539": "ARRIS Group, Inc.",
    "12.346860204032": "General Microsystems Sdn Bhd",
    "12.123917689122816": "Portrait Displays, Inc.",
    "12.123917691715584": "E-MetroTel",
    "20.212014916960256": "Regulus Co.,Ltd.",
    "24.5530": "ARRIS Group, Inc.",
    "24.5531": "Nortel Networks",
    "24.5532": "B-KYUNG SYSTEM Co.,Ltd.",
    "24.5533": "Tripp Lite",
    "24.5534": "Mad Catz Interactive Inc",
    "24.5535": "Terascala, Inc.",
    "24.5536": "Nokia Danmark A/S",
    "24.5537": "Eca-Sinters",
    "24.7623098": "Xiaomi Communications Co Ltd",
    "24.3728325": "Taicang T&W Electronics",
    "20.172584089944064": "Shenzhen Xtooltech Co., Ltd",
    "20.189549218103296": "HMicro Inc",
    "12.346865987584": "Key Systems, Inc.",
    "24.812028": "Nanjing Signway Technology Co., Ltd",
    "24.5825318": "Compass Technologies Inc.",
    "24.6831902": "EFFECT Photonics B.V.",
    "24.10525463": "Taicang T&W Electronics",
    "20.62366877220864": "NHS Sistemas de Energia",
    "24.13378606": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "20.97755445854208": "Shenzhen Digissin Technology",
    "20.31346170789888": "Cgi It Uk Limited",
    "12.123917677232128": "Private",
    "12.123917690408960": "CP contech electronic GmbH",
    "20.66426712883200": "Metro",
    "12.346861060096": "Arcontia AB",
    "24.1094251": "base Co.,Ltd.",
    "24.2138481": "Amazon Technologies Inc.",
    "20.127654898434048": "Volacomm Co., Ltd",
    "24.9488795": "Tesorion Nederland B.V.",
    "12.123917689335808": "SET Power Systems GmbH",
    "20.84187714420736": "Shenzhen bankledger Technology Co, Ltd",
    "12.123917675667456": "TechSigno srl",
    "24.5021699": "Vitec",
    "24.10253843": "Ring LLC",
    "20.193649358929920": "Shenzhen Yifang Digital Technology Co.,Ltd.",
    "20.106196529840128": "Cadmo Soluciones SAC",
    "12.123917682712576": "Standard Backhaul Communications",
    "12.71297884381184": "Tti Ltd",
    "24.14212814": "Isung Co.,Ltd",
    "24.11588341": "Vello Systems, Inc.",
    "12.346856783872": "Network I/O",
    "24.4667": "KeRo Systems ApS",
    "12.346853163008": "Sysnet Co., Ltd.",
    "24.4665": "S Net Systems Inc.",
    "24.4664": "SetaBox Technology Co., Ltd.",
    "24.4663": "Texas Instruments",
    "24.4662": "ConSentry Networks",
    "24.4661": "Andrew Corporation",
    "24.4660": "Camille Bauer",
    "24.12596186": "Shenzhen JRUN Technologies CO., LTD",
    "12.123917682245632": "Blu Wireless Technology Ltd",
    "24.1349349": "Samsung Electronics Co.,Ltd",
    "24.4669": "GES Co, Ltd",
    "24.4668": "Second Rule LLC",
    "24.11015028": "Panasonic Corporation AVC Networks Company",
    "20.88914842877952": "Tannak International AB",
    "24.5513629": "Samsung Electronics Co.,Ltd",
    "12.346865270784": "Reko-vek",
    "12.346857299968": "Photonic Bridges Inc.",
    "12.123917689647104": "ScopeSensor Oy",
    "12.71297885077504": "Webeasy BV",
    "12.123917690961920": "Scanvaegt Nordic A/S",
    "24.15530740": "SenRa Tech Pvt. Ltd",
    "24.3733453": "IEEE Registration Authority",
    "24.13139132": "Valink Co., Ltd.",
    "24.7928852": "Family Zone Cyber Safety Ltd",
    "12.71297883934720": "WxBR Sistemas de Telecomunicacoes Ltda",
    "12.346860580864": "Technomark",
    "12.346861965312": "DECTRIS Ltd.",
    "20.66129346166784": "Wisycom",
    "24.2361956": "AzureWave Technology Inc.",
    "24.2361955": "ARRIS Group, Inc.",
    "24.5506451": "WOORI ELEC Co.,Ltd",
    "24.10796587": "Huawei Technologies Co.,Ltd",
    "24.3176540": "Partow Tamas Novin (Parman)",
    "20.194363545092096": "Ivk-Sayany",
    "24.12070048": "Resideo",
    "20.163808565788672": "Sercomm Corporation.",
    "12.123917678997504": "Systems Engineering Arts Pty Ltd",
    "24.20727": "Venture Manufacturing (Singapore) Ltd.",
    "24.7362": "Part II Research, Inc.",
    "24.7361": "ARRIS Group, Inc.",
    "24.7360": "Intel Corporate",
    "24.20723": "GLOBAL NET INFORMATION CO., Ltd.",
    "24.20722": "Microsoft Corp.",
    "24.20721": "Maxlinear, Inc",
    "24.20720": "Cisco Systems, Inc",
    "24.7369": "Kaise Electronic Technology Co., Ltd.",
    "24.7368": "INDUSTRONIC Industrie-Electronic GmbH & Co. KG",
    "24.20729": "Sensormatic Electronics LLC",
    "24.20728": "Entrega Technologies, Inc.",
    "12.123917687685120": "Sanmina Israel",
    "12.123917688012800": "Mini Solution Co. Ltd.",
    "24.11856898": "Alpsalpine Co,.Ltd",
    "24.13633482": "Juniper Networks",
    "12.123917680861184": "Mencom Corporation",
    "24.49227": "Creative Microsystems",
    "24.13682131": "AzureWave Technology Inc.",
    "24.14991354": "Technicolor CH USA Inc.",
    "20.176189958258688": "Nanjing zhanyi software technology co., LTD",
    "20.136646478528512": "Speedifi Inc",
    "24.2897814": "Cybo Co.,Ltd.",
    "24.7359725": "TMRG, Inc.",
    "24.9699778": "Samsung Electronics Co.,Ltd",
    "12.346862018560": "PCM Industries",
    "24.8809": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8808": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8807": "Nortel Networks",
    "24.8806": "Nokia Danmark A/S",
    "24.8805": "Nokia Danmark A/S",
    "24.49220": "Emcom Corporation",
    "24.8803": "Koos Technical Services, Inc.",
    "24.8802": "BEP Marine",
    "24.8801": "Frontier Silicon Ltd",
    "24.8800": "AFREEY Inc.",
    "24.53426": "Xiotech Corporation",
    "24.53427": "DRS Technologies Canada Ltd",
    "24.53424": "Bitswitch Ltd.",
    "24.53425": "Omega Electronics Sa",
    "24.1056192": "Samsung Electronics Co.,Ltd",
    "24.53423": "Cutler-Hammer, Inc.",
    "24.53420": "Commscope, Inc",
    "24.53421": "Tl Industries",
    "24.53428": "Katsujima Co., Ltd.",
    "24.53429": "IPricot formerly DotCom",
    "20.154238425497600": "Xertified AB",
    "24.1330740": "Remote Solution",
    "20.119143427735552": "Reconova Technologies",
    "24.318162": "Cisco Systems, Inc",
    "24.12630147": "Intel Corporate",
    "24.7587": "SabiOso",
    "24.7586": "Cisco Systems, Inc",
    "24.7585": "Cisco Systems, Inc",
    "24.7584": "Heng Yu Electronic Manufacturing Company Limited",
    "24.7583": "MATT R.P.Traczynscy Sp.J.",
    "24.7582": "Axion Technologies",
    "24.7581": "Artjoy International Limited",
    "24.7580": "Rockwell Automation",
    "24.7589": "WB Electronics",
    "24.7588": "Hangzhou System Technology CO., LTD",
    "24.8177534": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.1086795": "Megabyte Ltd.",
    "24.11827656": "Ruckus Wireless",
    "24.14437667": "MRV Comunications",
    "12.346859982848": "Laserdyne Technologies",
    "24.3435932": "Carrier Corporation",
    "24.15486029": "ZAO NPK RoTeK",
    "24.13913984": "Kemper Digital GmbH",
    "24.4239478": "ASUSTek COMPUTER INC.",
    "24.16424": "Netcomm Limited",
    "24.7135227": "IEEE Registration Authority",
    "24.6872537": "HMD Global Oy",
    "20.225207120822272": "Shenzhen Decnta Technology Co.,LTD.",
    "12.346867302400": "Dorlet Sau",
    "20.48476180185088": "Shenzhen elink smart Co., ltd",
    "12.123917677215744": "Enyx SA",
    "24.13162265": "Cisco-Linksys, LLC",
    "12.123917678469120": "Risk Expert sarl",
    "12.119269875712": "K+K Messtechnik GmbH",
    "24.3168020": "Apple, Inc.",
    "24.2420378": "Intel Corporate",
    "24.24723": "Varian",
    "24.24722": "Micro/Sys, Inc.",
    "24.24721": "First Pacific Networks, Inc.",
    "24.24720": "Artiza Networks Inc",
    "24.24727": "3Com",
    "24.24726": "T.S. Microtech Inc.",
    "24.24725": "Accu-Time Systems, Inc.",
    "24.24724": "AMD PCNET PCI",
    "24.6184": "e2v technologies (UK) ltd.",
    "24.6185": "Gatsometer",
    "24.24729": "SBE, Inc.",
    "24.24728": "Ht Communications",
    "24.6180": "Kimaldi Electronics, S.L.",
    "24.6181": "Private",
    "24.6182": "Cale Access AB",
    "24.6183": "Nec Unified Solutions Nederland B.V.",
    "12.123917687095296": "ARP Corporation",
    "24.1638159": "Intel Corporate",
    "20.176772041670656": "Glooko inc",
    "24.7085680": "Guangzhou YBDS IT Co.,Ltd",
    "20.162752340426752": "LTE-X, Inc",
    "12.346853744640": "wolf-inf-tec",
    "24.2957": "Avvio Networks",
    "24.2956": "Flextronics",
    "24.2955": "Kerajet, S.A.",
    "24.2954": "MITEQ Inc.",
    "24.2953": "Top Global Technology, Ltd.",
    "24.2952": "Vidisco ltd.",
    "24.2951": "American Reliance Inc.",
    "24.2950": "Aruba, a Hewlett Packard Enterprise Company",
    "20.137215125487616": "optilink networks pvt ltd",
    "24.8143759": "E-Band Communications Corp.",
    "24.2959": "Akita Electronics Systems Co.,Ltd.",
    "24.2958": "Ascent Corporation",
    "24.330": "Sony Corporation",
    "24.331": "Ennovate Networks, Inc.",
    "24.37008": "I-Bus",
    "24.37009": "DigitalScape, Inc.",
    "24.334": "WIN Enterprises, Inc.",
    "24.335": "Adtran Inc",
    "24.336": "Gilat Communications, Ltd.",
    "24.337": "Ensemble Communications",
    "24.338": "Chromatek Inc.",
    "24.37003": "Tattile SRL",
    "24.37000": "Baxall Security Ltd.",
    "24.37001": "Softcom Microsystems, Inc.",
    "24.37006": "Nortel Networks Broadband Access",
    "24.37007": "Audio Codes Ltd.",
    "24.37004": "Etrend Electronics, Inc.",
    "24.1055756": "Apple, Inc.",
    "24.1602": "Genetel Systems Inc.",
    "24.1603": "SONO Computer Co., Ltd.",
    "24.1600": "White Rock Networks",
    "24.1601": "Itcn",
    "24.1606": "ShenZhen XunBao Network Technology Co Ltd",
    "24.1607": "Etrali S.A.",
    "24.1604": "NextGen Business Solutions, Inc",
    "24.1605": "Meisei Electric Co. Ltd.",
    "12.346866696192": "Vision & Control GmbH",
    "24.5988": "ATMedia GmbH",
    "24.1608": "Seedsware, Inc.",
    "24.1609": "3M Deutschland GmbH",
    "24.548317": "Mercury Corporation",
    "24.3435202": "Huawei Technologies Co.,Ltd",
    "12.346861608960": "China Railway Signal & Communication Corp.",
    "24.11280080": "Temic Automotive Philippines Inc.",
    "24.10531895": "GD Mission Systems",
    "24.10531897": "Cisco Systems, Inc",
    "24.15245816": "zte corporation",
    "24.11799589": "Palo Alto Networks",
    "24.16564797": "Atmel Corporation",
    "20.260167873593344": "Crrc Qingdao Sifang Rolling Stock Research Institute Co.,Ltd",
    "24.316471": "Znv",
    "12.123917675888640": "Laser Imagineering GmbH",
    "12.346868682752": "Motec Pty Ltd",
    "12.123917683003392": "Excelfore Corporation",
    "24.9389": "Adolf Thies Gmbh & Co. KG",
    "24.9388": "Hangzhou DPtech Technologies Co., Ltd.",
    "24.9387": "A7 Engineering, Inc.",
    "24.9386": "Dycor Technologies Ltd.",
    "24.9385": "Ag Leader Technology",
    "24.9384": "ProCurve Networking by HP",
    "24.9383": "Advanced Video Communications Inc.",
    "24.9382": "TELESTAR DIGITAL GmbH",
    "24.9381": "Buffalo.Inc",
    "24.9380": "Siklu Communication",
    "24.1099390": "Shenzhen Juchin Technology Co., Ltd",
    "24.6063809": "Dongguan Solum Electronics Co.,Ltd",
    "24.12369310": "Itel Mobile Limited",
    "20.128510971609088": "Knog Pty Ltd",
    "24.4720098": "Huawei Device Co., Ltd.",
    "24.11337166": "Intel Corporate",
    "12.346864795648": "Altierre",
    "12.123917691498496": "Vena Engineering Corporation",
    "24.3163271": "Hefei Radio Communication Technology Co., Ltd",
    "24.4728665": "Levven Electronics Ltd.",
    "12.123917688467456": "AutomationX GmbH",
    "24.12335205": "Jwcnetworks",
    "24.3187794": "Arrival Elements BV",
    "12.123917681115136": "Rch Italia Spa",
    "24.8384": "Pulse Electronics, Inc.",
    "24.8385": "SAXA, Inc.",
    "24.8386": "Texas Memory Systems, Inc.",
    "24.8387": "Counter Solutions Ltd.",
    "24.8380": "Long Reach Networks Pty Ltd",
    "24.8381": "Niobrara R & D Corporation",
    "24.8382": "Lan Access Corp.",
    "24.8383": "Aehr Test Systems",
    "12.123917690195968": "Nanjing Dandick Science&technology development co., LTD",
    "12.346857979904": "Lattice Semiconductor Corp. (LPA)",
    "24.8388": "Inet,Inc.",
    "24.8389": "Eagle NE2000",
    "24.524309": "Stc Business Systems",
    "24.524308": "Excelan",
    "24.3469514": "Shenzhen Linghangyuan Digital Technology Co.,Ltd.",
    "12.123917677293568": "Instec, Inc.",
    "24.524303": "Mitel Corporation",
    "24.524302": "Ncr/At&T",
    "24.524301": "ICL (International Computers, Ltd.)",
    "12.346854346752": "VoiceCom AG",
    "24.524307": "Exxon",
    "24.524306": "Bell Atlantic Integrated Syst.",
    "24.524305": "Tektronix Inc.",
    "24.524304": "AT&T [misrepresentation of 800010?]",
    "20.127658779213824": "Philips Personal Health Solutions",
    "12.123917685858304": "RCS Energy Management Ltd",
    "20.39986086805504": "Cosmicnode",
    "24.5263252": "Loxone Electronics GmbH",
    "24.536971": "Huawei Technologies Co.,Ltd",
    "12.346858909696": "Nilan A/S",
    "24.13949080": "Korea CNO Tech Co., Ltd",
    "12.346857480192": "Linear Acoustic, Inc",
    "24.16264136": "Jiangsu Switter Co., Ltd",
    "12.123917689974784": "DEUTA-WERKE GmbH",
    "24.12111271": "Maxeler Technologies Ltd.",
    "12.346865291264": "Peek Traffic Corporation",
    "24.8039": "Hitachi,Ltd.",
    "12.346853351424": "Diva Systems",
    "24.8037": "Korea Electric Terminal Co., Ltd.",
    "24.8034": "JSC \"Stilsoft\"",
    "24.8035": "JSC Goodwin-Europa",
    "24.8032": "Compass Systems Corp.",
    "24.8033": "Talent Communication Networks Inc.",
    "24.8030": "Dyna Technology Co.,Ltd.",
    "24.8031": "Blatand GmbH",
    "24.11817597": "Sony Mobile Communications Inc",
    "12.123917676470272": "Serial Image Inc.",
    "20.163603969736704": "CircuitWerkes, Inc.",
    "20.45033656942592": "Unicair Communication Tec Co., Ltd.",
    "24.3424866": "Apple, Inc.",
    "24.15790665": "Hitron Technologies. Inc",
    "24.12906229": "II-VI Incorporated",
    "24.3643": "Hawking Technologies, Inc.",
    "24.10806161": "Deny Fontaine",
    "12.346854506496": "Wheatstone Corporation",
    "12.346865467392": "Konsmetal S.A.",
    "24.10807697": "Sistemas Audiovisuales Itelsis S.L.",
    "24.7628791": "Extreme Networks, Inc.",
    "24.14745591": "Softiron Inc.",
    "24.1068681": "Garmin International",
    "12.346853863424": "Fleetwood Electronics Ltd",
    "24.9749181": "Tecnobit",
    "24.15214711": "TMY Co., Ltd.",
    "12.346863104000": "Saia-Burgess Controls AG",
    "24.2383839": "Kyocera Corporation",
    "24.6821331": "Zaklady Elektroniki i Mechaniki Precyzyjnej R&G S.A.",
    "24.11849696": "Sichuan tianyi kanghe communications co., LTD",
    "24.11849691": "Shenzhen Jiuzhou Electric Co.,LTD",
    "20.114869207564288": "Copper Labs, Inc.",
    "12.123917680672768": "Small Data Garden Oy",
    "24.14989690": "Dell Inc.",
    "12.123917682364416": "Digital Matter Pty Ltd",
    "24.3427492": "amazipoint technology Ltd.",
    "24.6062991": "Cisco Systems, Inc",
    "24.9817": "Nintendo Co., Ltd.",
    "24.9816": "T-Platforms (Cyprus) Limited",
    "24.30878": "Sagemcom Broadband SAS",
    "24.9814": "Sansonic Electronics USA",
    "24.9813": "Hewlett Packard",
    "24.9812": "3Com",
    "24.9811": "DaySequerra Corporation",
    "24.9810": "Cisco Systems, Inc",
    "12.346862186496": "Belik S.P.R.L.",
    "24.9819": "Hitron Technologies. Inc",
    "24.9818": "D-Link Corporation",
    "24.12865933": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.9210478": "Estun Automation Technoloy Co., Ltd",
    "24.10520093": "Chengdu Fuhuaxin Technology co.,Ltd",
    "24.12324708": "Intel Corporate",
    "12.123917686059008": "Giant Power Technology Biomedical Corporation",
    "24.1333151": "Huawei Technologies Co.,Ltd",
    "24.11324241": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.9991386": "Infinix mobility limited",
    "24.12337409": "Shenzhen TINNO Mobile Technology Corp.",
    "12.346863038464": "Alga Microwave Inc",
    "24.16536205": "Technicolor CH USA Inc.",
    "20.97755444805632": "TEM Mobile Limited",
    "24.15737864": "Talaris (Sweden) AB",
    "24.3155064": "Sagemcom Broadband SAS",
    "12.123917682024448": "Crde",
    "24.15737861": "OPUS High Technology Corporation",
    "24.3184144": "Huawei Device Co., Ltd.",
    "24.9754713": "TCT mobile ltd",
    "12.123917681410048": "Earth Works",
    "24.6866490": "Espressif Inc.",
    "24.305194": "Nokia Corporation",
    "12.123917688479744": "Vitec",
    "24.10769922": "fenglian Technology Co.,Ltd.",
    "20.31477536391168": "Hanjen.Chin Co., Ltd.",
    "24.7628241": "Atlinks",
    "24.9219657": "IEEE Registration Authority",
    "24.15767833": "Garmin International",
    "24.12891410": "General Electric Digital Energy",
    "12.346853904384": "Axotec Technologies GmbH",
    "24.4480755": "Tonalite Holding B.V.",
    "24.554230": "Winterhalter Gastronom GmbH",
    "24.15003714": "Google, Inc.",
    "24.13956671": "Iea S.R.L.",
    "24.15748309": "Maxlinear, Inc",
    "24.11848121": "Sichuan AI-Link Technology Co., Ltd.",
    "24.6556032": "Xiaomi Communications Co Ltd",
    "12.346864791552": "ProSourcing GmbH",
    "12.123917689528320": "Hubbell Power Systems",
    "24.7340926": "Technicolor CH USA Inc.",
    "24.15782328": "Ledvance",
    "24.7914510": "H&D Wireless",
    "24.8669029": "Huawei Technologies Co.,Ltd",
    "12.346863931392": "siXis, Inc.",
    "12.346853535744": "Telephonics Corp.",
    "24.12061066": "Apple, Inc.",
    "24.11073382": "ITE Tech Inc",
    "24.7363087": "Hewlett Packard",
    "24.3450633": "Trevil srl",
    "24.1077501": "Mellanox Technologies, Inc.",
    "12.123917683052544": "Insitu, Inc",
    "24.1077504": "Huawei Device Co., Ltd.",
    "20.111031379558400": "Private",
    "24.808101": "Pegatron Corporation",
    "24.13406440": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.5307440": "Treehouse Labs",
    "24.3423640": "JinQianMao Technology Co.,Ltd.",
    "24.8191339": "Xiaomi Communications Co Ltd",
    "12.346861830144": "Postec Data Systems Ltd",
    "12.123917684035584": "e-Matix Corporation",
    "24.8393896": "Guangzhou V-SOLUTION Electronic Technology Co., Ltd.",
    "12.346867949568": "Micro Technic A/S",
    "24.13389660": "IEEE Registration Authority",
    "12.123917684015104": "Salupo Sas",
    "12.346856255488": "Revolution Education Ltd",
    "12.346855174144": "Micrel-Nke",
    "12.346857832448": "GeoFocus, LLC",
    "24.5012040": "Nippon Seiki (Europe) B.V.",
    "24.11847577": "Extreme Networks, Inc.",
    "24.1107119": "Apr, Llc",
    "12.123917691510784": "Hanbat National University",
    "12.346867355648": "AEL Microsystems Limited",
    "24.13149197": "Cooler Master Technology Inc",
    "24.7394546": "Rim",
    "20.13692007612416": "U-Pass.Co.,Ltd",
    "24.15510176": "Pegatron Corporation",
    "20.167860150206464": "SENKO Co.,Ltd.",
    "24.15514889": "FUSION Electronics Ltd",
    "12.123917676138496": "Welltec Corp.",
    "24.3458002": "Fn-Link Technology Limited",
    "24.13918682": "Apple, Inc.",
    "24.269165": "LG Electronics (Mobile Communications)",
    "24.6350593": "Huawei Technologies Co.,Ltd",
    "24.797318": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.66219387387904": "BEWATEC Kommunikationstechnik GmbH",
    "24.14446903": "zte corporation",
    "12.346860548096": "Shenzhen MaiWei Cable TV Equipment CO.,LTD.",
    "12.119269535744": "Silicon Controls",
    "24.10762857": "Vers Inc",
    "20.167295463718912": "RayTron, INC.",
    "12.123917683912704": "NAC Planning Co., Ltd.",
    "24.6823744": "Blu Wireless Technology Ltd",
    "24.11062165": "East Best Co., Ltd.",
    "24.11040623": "Zonoff",
    "24.10761457": "Texas Instruments",
    "12.346854195200": "EXI Wireless Systems Inc.",
    "12.346863599616": "Bit-Lab PTY LTD",
    "24.7094280": "Apple, Inc.",
    "12.346857603072": "Innovation, Institute, Inc",
    "24.3192305": "Aitexin Technology Co., Ltd",
    "24.13936209": "Tuya Smart Inc.",
    "24.8405286": "Huawei Technologies Co.,Ltd",
    "24.11050148": "Furrion Limited",
    "12.346862915584": "Meiryo Denshi Corp.",
    "20.167137982283776": "Shenzhen Ateko Photoelectricity Co Ltd",
    "24.847740": "AnaCom Inc",
    "24.12617036": "Ragentek Technology Group",
    "24.3268": "Tiptel AG",
    "24.3269": "Nextlink Co., Ltd.",
    "24.4493079": "Samsung Electronics Co., Ltd. ARTIK",
    "24.11592605": "Cat System Co.,Ltd.",
    "12.346864144384": "Sound Metrics Corp",
    "24.3262": "Innominate Security Technologies AG",
    "24.3263": "Holy Stone Ent. Co., Ltd.",
    "24.839928": "Cisco Systems, Inc",
    "24.3261": "Interface Masters, Inc",
    "24.3266": "ControlNet (India) Private Limited",
    "24.3267": "BeWAN systems",
    "24.3264": "Genera Oy",
    "24.3265": "Eaton Corporation",
    "12.123917675491328": "InAccess Networks SA",
    "24.843247": "Nokia Corporation",
    "24.4480717": "Razer Inc",
    "12.346864414720": "MangoDSP",
    "24.8926828": "EFM Networks",
    "24.16018415": "Sagemcom Broadband SAS",
    "24.3946045": "Nokia Corporation",
    "20.277958190170112": "Xmitech Technology Co., Limited",
    "24.3440185": "Nokia Danmark A/S",
    "24.12346443": "ARRIS Group, Inc.",
    "24.10802101": "GLITEL Stropkov, s.r.o.",
    "24.801997": "Universal Global Scientific Industrial Co., Ltd.",
    "12.346852651008": "Saco Smartvision Inc.",
    "24.2932459": "Nothing Technology Limited",
    "24.11302084": "Maytronics Ltd.",
    "12.119269953536": "Roslen Eco-Networking Products",
    "24.271813": "Huawei Technologies Co.,Ltd",
    "24.4482410": "Mega Video Electronic(HK) Industry Co., Ltd",
    "24.15234431": "FIRS Technologies(Shenzhen) Co., Ltd",
    "24.14429025": "Apple, Inc.",
    "12.123917683142656": "Flexim Security Oy",
    "12.123917687492608": "Telecast Inc.",
    "24.16302999": "NZXT Corp. Ltd.",
    "24.12435": "Sonnet Technologies, Inc",
    "24.13422110": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "12.123917685055488": "XpertSea Solutions inc.",
    "12.346857398272": "GEM ELETTRONICA Srl",
    "12.123917677490176": "Coppernic Sas",
    "24.47968": "Intel Corporate",
    "12.123917684473856": "Accusonic",
    "24.3974678": "vivo Mobile Communication Co., Ltd.",
    "24.2662140": "Sagemcom Broadband SAS",
    "24.6583142": "Shenzhen Siera Technology Ltd",
    "24.16316371": "International Communications Corporation",
    "24.6034709": "Advan",
    "12.123917691617280": "MB connect line GmbH Fernwartungssysteme",
    "24.13654528": "Technicolor CH USA Inc.",
    "24.13418756": "Applied Technical Systems",
    "12.346853044224": "Summit Avionics, Inc.",
    "24.6082431": "Espressif Inc.",
    "24.16522628": "Autobase",
    "24.10269362": "Huawei Technologies Co.,Ltd",
    "12.123917686747136": "Aplex Technology Inc.",
    "12.123917690413056": "Shooter Detection Systems",
    "20.224641227423744": "Dtrovision",
    "12.123917680050176": "Codewerk GmbH",
    "12.123917686681600": "Supervision Test et Pilotage",
    "24.7901852": "Intel Corporate",
    "12.346861821952": "Margaritis Engineering",
    "12.123917687160832": "GSF Corporation Pte Ltd",
    "12.346856669184": "Computime Systems UK Ltd.",
    "24.540302": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.14943736": "Bytedance",
    "12.123917675859968": "Common Sense Monitoring Solutions Ltd.",
    "12.346854961152": "AUM Infotech Private Limited",
    "20.70443729944576": "Beijing Gemotech Intelligent Technology Co., Ltd.",
    "12.123917687717888": "Mipot S.p.a.",
    "12.346861436928": "Array Technologies Inc",
    "24.6348": "Axiohm Sas",
    "24.6349": "Erae Electronics Industry Co., Ltd",
    "20.18540135448576": "Hanacns",
    "24.6342": "OPW Fuel Management Systems",
    "24.6343": "Real Time Automation",
    "24.6340": "Raba Technologies LLC",
    "24.6341": "Nokia Danmark A/S",
    "12.346866761728": "VendNovation LLC",
    "24.6347": "Tecobest Technology Limited",
    "24.6344": "ISONAS Inc.",
    "24.6345": "EOps Technology Limited",
    "24.16040520": "Samsung Electronics Co.,Ltd",
    "24.13685873": "MVTECH co., Ltd",
    "20.62335515361280": "Basalte BVBA",
    "24.1584273": "Concept Development, Inc.",
    "24.8138429": "Google, Inc.",
    "24.11546824": "Teal Drones, Inc.",
    "12.346862485504": "Samsung Heavy Industries Co.,Ltd.",
    "12.346853171200": "Carl Baasel Lasertechnik GmbH",
    "12.123917675450368": "Panoramic Power",
    "12.123917682110464": "Becton Dickinson",
    "24.8681644": "Cisco Systems, Inc",
    "24.6297603": "Daikin Air-conditioning (Shanghai) Co., Ltd.",
    "24.292462": "Alpsalpine Co,.Ltd",
    "24.549": "One Stop Systems",
    "24.9685": "Robonica (Pty) Ltd",
    "12.346867769344": "System Industrie Electronic GmbH",
    "24.5041630": "Askey Computer Corp",
    "12.123917686079488": "ERA a.s.",
    "24.11316109": "Lyngso Marine A/S",
    "24.14955923": "Hangzhou GoTo technology Co.Ltd",
    "24.9688": "Korea Maintenance",
    "24.4987782": "Arcadyan Corporation",
    "24.52157": "Cambridge Broadband Networks Group",
    "24.9689": "DataFab Systems Inc.",
    "24.12351287": "Intel Corporate",
    "24.6607425": "Keith & Koep GmbH",
    "20.13592315297792": "HongKong Blossom Limited",
    "12.71297884442624": "Globalcom Engineering Srl",
    "12.123917683978240": "Quaesta Instruments, LLC",
    "24.16001864": "Ubiqam",
    "24.12845793": "Khwahish Technologies Private Limited",
    "24.11582347": "Wavetel Technology Limited",
    "24.13658526": "LUMINEX Lighting Control Equipment",
    "20.114975473401856": "G-TECH Instruments Inc.",
    "24.1378": "Digital View Limited",
    "24.1379": "J-Works, Inc.",
    "24.1372": "Kowa Company, Ltd.",
    "24.1373": "D-Link Systems, Inc.",
    "24.1370": "Power Dsine Ltd.",
    "24.1371": "Charles Industries, Ltd.",
    "24.1376": "Leader Comm.Co., Ltd",
    "24.1377": "nac Image Technology, Inc.",
    "24.1374": "Cisco Systems, Inc",
    "24.1375": "Cisco Systems, Inc",
    "24.16568162": "IBM Corp",
    "24.14956786": "Advantage Controls",
    "12.123917690036224": "Hipodromo De Agua Caliente, S.A. De C.V.",
    "24.14432481": "Juniper Networks",
    "12.346863546368": "GO engineering GmbH",
    "24.231": "Star Gate Technologies",
    "24.230": "Aptor Produits De Comm Indust",
    "24.233": "Isicad, Inc.",
    "24.232": "Accton Technology Corp.",
    "24.235": "Matsushita Comm. Ind. Co. Ltd.",
    "24.234": "Upnod Ab",
    "24.237": "April",
    "24.236": "Microprocess",
    "24.239": "Kti",
    "24.238": "Network Designers Limited [also KNX Ltd, a former division]",
    "24.12612651": "Shenzhen Xunlong Software Co.,Limited",
    "24.1455": "InnoScan Computing A/S",
    "24.16543185": "APRESIA Systems, Ltd.",
    "12.346861187072": "Sensata Technologies, Inc.",
    "20.154019638018048": "Shenzhen Lanxus technology Co. Ltd.",
    "12.346854842368": "Tattile srl",
    "24.16521745": "vivo Mobile Communication Co., Ltd.",
    "24.4221582": "Hangzhou Puwell OE Tech Ltd.",
    "12.346856828928": "Sensus Metering Systems Israel",
    "20.22333147316224": "Fls Finland Oy",
    "24.1574517": "Dilax Intelcom GmbH",
    "12.123917681139712": "Valk Welding B.V.",
    "24.1894939": "Nokia",
    "12.346860208128": "Iusa Sa De Cv",
    "24.3460461": "CommSky Technologies",
    "24.14691046": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.2391868": "Huawei Technologies Co.,Ltd",
    "24.1958": "Leviton Manufacturing Co., Inc.",
    "24.1959": "A-Z Inc.",
    "24.1956": "GN Netcom Ltd.",
    "24.1957": "Y.D.K Co. Ltd.",
    "24.1954": "Opteon Corporation",
    "24.1955": "Ositis Software, Inc.",
    "24.1952": "e-Watch Inc.",
    "24.1953": "VIASYS Healthcare GmbH",
    "24.1950": "Ilinx Co., Ltd.",
    "24.1951": "Action Digital Inc.",
    "24.9488344": "zte corporation",
    "12.346866126848": "Bettini Srl",
    "24.4989203": "Nokia Shanghai Bell Co., Ltd.",
    "24.16028774": "CountMax, ltd",
    "24.41937": "Cisco Systems, Inc",
    "12.346852188160": "Project Management Enterprises, Inc.",
    "12.119269638144": "Enkora Oy Ltd",
    "24.2947746": "Alcatel-Lucent Enterprise",
    "24.6353213": "JABLOCOM s.r.o.",
    "20.66256286777344": "GJS Co., Ltd.",
    "24.5707": "Quorion Data Systems GmbH",
    "24.5706": "Vibration Technology Limited",
    "24.5705": "SetOne GmbH",
    "24.5704": "SSD Company Limited",
    "24.5703": "Cisco Systems, Inc",
    "24.5702": "Cisco Systems, Inc",
    "24.5701": "Power Distribution, Inc.",
    "24.5700": "LITE-ON Technology Corp.",
    "12.123917686513664": "Power Security Systems Ltd.",
    "24.5709": "Alcatel-Lucent IPD",
    "24.5708": "PLANET INT Co., Ltd",
    "24.11851998": "iota Computing, Inc.",
    "24.10778419": "Google, Inc.",
    "24.14186888": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.14721432": "NOJA Power Switchgear Pty Ltd",
    "20.9814044311552": "Victiana SRL",
    "12.346859618304": "Elmec Inc.",
    "24.4252": "M-System Co., Ltd.",
    "24.4253": "Clarinet Systems, Inc.",
    "24.4250": "Netline",
    "20.17622839066624": "Beijing Assem Technology Co., ltd",
    "24.4256": "Innovex Technologies, Inc.",
    "24.4257": "Kendin Semiconductor, Inc.",
    "24.4254": "Aware, Inc.",
    "24.4255": "Pavo, Inc.",
    "24.14946651": "Hewlett Packard",
    "24.4258": "Tns",
    "24.4259": "Omnitronix, Inc.",
    "24.3701": "New York Air Brake Corp.",
    "24.3700": "Solar Telecom. Tech",
    "24.3703": "Decru, Inc.",
    "24.3702": "Gemsoc Innovision Inc.",
    "24.3705": "Ample Communications Inc.",
    "24.3704": "Amtelco",
    "24.909": "PCS Revenue Control Systems, Inc.",
    "20.52912288956416": "Micomsoft Co.,Ltd.",
    "24.907": "PLUS-ONE I&T, Inc.",
    "24.906": "America Online, Inc.",
    "24.905": "Plantronics, Inc.",
    "24.904": "Fastfame Technology Co., Ltd.",
    "24.903": "Blaze Network Products",
    "24.902": "Ho Net, Inc.",
    "24.901": "Actelis Networks, Inc.",
    "24.900": "Aeta",
    "24.15744703": "Gogoro Taiwan Limited",
    "24.5043676": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917681934336": "Keepen",
    "24.13902399": "Lenovo Mobile Communication Technology Ltd.",
    "12.346855067648": "Elman srl",
    "12.346852601856": "NCast Corporation",
    "12.346860118016": "Dorlet Sau",
    "20.83889185882112": "Shenzhen HommPro Technology Co.,Ltd",
    "24.5040612": "New H3C Technologies Co., Ltd",
    "24.816573": "Cisco Systems, Inc",
    "24.9486613": "Sony Mobile Communications Inc",
    "24.10548249": "Nokia Corporation",
    "12.346860445696": "Empirix Italy S.p.A.",
    "24.8932559": "Pulzze Systems, Inc",
    "12.123917675900928": "Cryptotronix LLC",
    "24.7620256": "Hewlett Packard",
    "20.220515733078016": "Private",
    "24.4235207": "BAEKCHUN I&C Co., Ltd.",
    "24.16553322": "Octagon Systems Corp.",
    "12.123917680803840": "LLC NPO Svyazkomplektservis",
    "24.4511457": "Snuza International Pty. Ltd.",
    "12.346855886848": "A&G Soluzioni Digitali",
    "12.346862710784": "Visualware Inc",
    "12.346860576768": "Cet",
    "12.346858594304": "2N TELEKOMUNIKACE a.s.",
    "24.9764142": "Shanghai Uniscope Technologies Co.,Ltd",
    "24.14692816": "SZ Telstar CO., LTD",
    "20.154680182177792": "Gati Information Technolog(Kunshan)Co.,Ltd.",
    "24.6051492": "Wanan Hongsheng Electronic Co.Ltd",
    "24.2660280": "myenergi Ltd",
    "20.198237817405440": "DongYoung media",
    "24.2664108": "seca gmbh & co. kg",
    "24.6080739": "Shintec Hozumi co.ltd.",
    "12.123917690486784": "hmt telematik GmbH",
    "24.6845123": "Samsung Electronics Co.,Ltd",
    "12.123917680447488": "Ateme",
    "24.6325107": "Apple, Inc.",
    "24.5278053": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "12.123917680443392": "iSiS-Ex Limited",
    "12.346868428800": "GSP Sprachtechnologie GmbH",
    "20.154680176934912": "Ouman Oy",
    "12.123917681598464": "Netbric Technology Co.,Ltd.",
    "24.3952910": "Cisco Systems, Inc",
    "24.7087235": "Microsoft Mobile Oy",
    "12.119269232640": "OpenRB.com, Direct SIA",
    "24.7288": "Wyplay Sas",
    "24.7289": "Cohesive Financial Technologies LLC",
    "24.7280": "Novacomm Ltda",
    "24.7281": "Emergent Electronics",
    "24.7282": "Mayer & Cie GmbH & Co KG",
    "24.7283": "Arista Networks",
    "24.7284": "Syswan Technologies Inc.",
    "24.7285": "Segnet Ltd.",
    "12.346867113984": "Wings for Media SL",
    "24.7287": "Prodys",
    "24.7015": "Cisco Systems Inc",
    "24.8965029": "Suzhou Torchstar Intelligent Technology Co.,Ltd",
    "24.14477084": "AUG Elektronik GmbH",
    "24.11835834": "Hewlett Packard",
    "24.2634110": "Hongkong Nano IC Technologies Co., Ltd",
    "24.12590747": "Pixavi AS",
    "12.346867122176": "DEUTA-WERKE GmbH",
    "24.12899465": "JiangSu Joyque Information Industry Co.,Ltd",
    "24.3954511": "China Mobile Group Device Co.,Ltd.",
    "24.799196": "Huawei Technologies Co.,Ltd",
    "24.7660580": "Apiste Corporation",
    "12.123917690900480": "SamabaNova Systems",
    "24.10775972": "Huawei Technologies Co.,Ltd",
    "24.7132": "Vencer Co., Ltd.",
    "24.7133": "ARRIS Group, Inc.",
    "24.7130": "UTStarcom Inc",
    "24.7131": "Valeo VECS",
    "24.7136": "TELENOT ELECTRONIC GmbH",
    "24.7137": "ViaLogy",
    "24.7134": "Renkus-Heinz, Inc.",
    "24.7135": "Iskra Sistemi d.d.",
    "24.7138": "AhnLab,Inc.",
    "24.7139": "Health Hero Network, Inc.",
    "24.49399": "Engage Communication, Inc.",
    "24.49398": "Celan Technology Inc.",
    "24.5831712": "Altice Labs S.A.",
    "24.820295": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346854916096": "Sterling Industry Consult GmbH",
    "20.128510967414784": "Kanto Aircraft Instrument Co., Ltd.",
    "20.31372472221696": "ANDRA Sp. z o. o.",
    "24.6301468": "VIDEOR E. Hartig GmbH",
    "12.123917679104000": "APOLLO GIKEN Co.,Ltd.",
    "24.2386238": "innodisk Corporation",
    "20.31368176205824": "Wurm GmbH & Co. KG Elektronische Systeme",
    "24.7619199": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.11842149": "Daeho I&T",
    "12.346866806784": "Combilent",
    "12.346865336320": "RealD",
    "24.6319103": "Nintendo Co.,Ltd",
    "24.4758993": "Samsung Electronics Co.,Ltd",
    "12.123917681823744": "DEUTA-WERKE GmbH",
    "24.8618": "Nokia Danmark A/S",
    "24.8619": "Nokia Danmark A/S",
    "24.8616": "Telephonics Corporation",
    "24.1086615": "Logitech Hong Kong",
    "24.8614": "Videotec Spa",
    "24.8615": "Hantle System Co., Ltd.",
    "24.8612": "Dbii Networks",
    "24.8613": "ERLPhase Power Technologies Ltd.",
    "24.8610": "EKE-Electronics Ltd.",
    "24.8611": "Micromint",
    "24.14155923": "Aurender Inc.",
    "20.194363549286400": "Systovi",
    "24.4257004": "Cisco Systems, Inc",
    "12.123917690507264": "Qlinx Technologies",
    "12.123917677060096": "LinkAV Technology Co., Ltd",
    "24.3676099": "Hewlett Packard Enterprise",
    "24.6040565": "Vivint Wireless Inc.",
    "24.10227274": "Shenzhen Vastking Electronic Co.,Ltd.",
    "20.154794716037120": "Lanhomex Technology(Shen Zhen)Co.,Ltd.",
    "24.8176545": "Apple, Inc.",
    "12.346862178304": "Mecos AG",
    "24.4491227": "Tymphany Acoustic Technology (Huizhou) Co., Ltd.",
    "24.2910104": "InTalTech Ltd.",
    "20.242213044879360": "Methodex Systems Pvt. Ltd.",
    "24.15206745": "Guoguang Electric Co.,Ltd",
    "24.2562": "Annso Co., Ltd.",
    "24.2563": "Endesa Servicios, S.L.",
    "24.2560": "Mediatek Corp.",
    "24.2561": "SOHOware, Inc.",
    "24.2566": "Teledex LLC",
    "24.2567": "WebWayOne Ltd",
    "24.2564": "3Com Ltd",
    "24.2565": "Widax Corp.",
    "24.6669": "HandHeld entertainment, Inc.",
    "24.6668": "Swe-Dish Satellite Systems AB",
    "24.2568": "Alpine Electronics, Inc.",
    "24.2569": "TaraCom Integrated Products, Inc.",
    "24.2633666": "Automotive Data Solutions",
    "24.1031": "Topcon Positioning Systems, Inc.",
    "24.1030": "Fa. Metabox AG",
    "24.1033": "Cratos Networks",
    "24.1032": "Sanko Electronics Co., Ltd.",
    "24.1035": "3Com Europe Ltd",
    "24.1034": "Sage Systems",
    "24.1037": "Avaya Inc",
    "24.1036": "Kanno Works, Ltd.",
    "24.1039": "Asus Network Technologies, Inc.",
    "24.1038": "AVM GmbH",
    "12.346859352064": "Gebhardt Ventilatoren GmbH",
    "24.1091024": "Murata Manufacturing Co., Ltd.",
    "24.2672590": "Cisco Systems, Inc",
    "24.14157521": "Honeywell Fire System (Shanghai) Co,. Ltd.",
    "12.346860400640": "Mueller Elektronik",
    "24.48514": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.2377486": "PentronicAB",
    "24.2936169": "Aqavi.com",
    "12.123917688823808": "Coheros Oy",
    "20.31346165547008": "3xLOGIC Inc.",
    "24.24913": "Huawei Technologies Co.,Ltd",
    "24.6310428": "SUYIN Corporation",
    "12.123917688029184": "McQ Inc",
    "12.346865352704": "Isis Engineering, S.A.",
    "24.12864324": "Omniprint Inc.",
    "24.12371984": "Cisco SPVTG",
    "24.14712712": "EVIDENCE Network SIA",
    "12.346857644032": "Image Control Design Limited",
    "24.6063635": "Beijing Zhoenet Technology Co., Ltd",
    "12.123917690982400": "KAYA Instruments",
    "12.123917677948928": "MB connect line GmbH Fernwartungssysteme",
    "12.346863890432": "IntelliVision Technologies, Corp",
    "24.9747374": "Husqvarna AB",
    "24.8918059": "Protonic Holland",
    "20.102196710473728": "Unfors Raysafe AB",
    "24.3430142": "Cisco Meraki",
    "12.123917676478464": "Biwave Technologies, Inc.",
    "24.12331052": "e-Smart Systems Pvt. Ltd",
    "24.9224": "Pacific Biosciences",
    "24.9225": "The Toro Company",
    "24.9226": "US Beverage Net",
    "24.9227": "Virtual Computer Inc.",
    "24.9220": "Nokia Danmark A/S",
    "24.9221": "Dilog Nordic AB",
    "24.9222": "Pointmobile",
    "24.9223": "Telem Sas",
    "20.48543323652096": "Brüel & Kjaer Vibro GmbH",
    "24.9228": "DELEC GmbH",
    "24.9229": "OnePath Networks LTD.",
    "20.106196522500096": "Siema Applications",
    "20.127658781310976": "Fujian Taili Communication Technology Co.,Ltd",
    "12.123917685460992": "Fluid Components Intl",
    "24.9748816": "2Wire Inc",
    "24.14448460": "Cisco Systems, Inc",
    "12.346859167744": "Qualisys AB",
    "20.57176925143040": "Federal Aviation Administration",
    "24.9211881": "Apple, Inc.",
    "24.9719879": "Rechnerbetriebsgruppe",
    "24.12628793": "Comigo Ltd.",
    "20.79602628165632": "shenzhen sunflower technologies CO., LIMITED",
    "24.5826102": "EVRsafe Technologies",
    "24.6573040": "CalDigit, Inc",
    "24.6700": "SATEC Co.,LTD",
    "24.8701839": "APS Networks GmbH",
    "24.9841": "AUTOVISION Co., Ltd",
    "20.88668799762432": "TREXOM S.r.l.",
    "12.346858311680": "SomerData ltd",
    "24.7345156": "Limited Liability Company",
    "12.123917691596800": "AKIS technologies",
    "12.346860367872": "Highrail Systems Limited",
    "12.346860785664": "IMS Röntgensysteme GmbH",
    "20.273458032279552": "ZAO &quot;RADIUS Avtomatika&quot;",
    "20.127937506443264": "Fournie Grospaud Energie SASU",
    "12.346859261952": "Oerlikon Contraves AG",
    "24.6706": "Activa Multimedia",
    "12.123917681958912": "Root Automation",
    "12.346868482048": "SecureTech Systems, Inc.",
    "12.123917675540480": "Zao Zeo",
    "24.6612517": "Woxu Wireless Co., Ltd",
    "24.12629779": "Beijing Xiaoyuer Technology Co. Ltd.",
    "20.17622841163776": "Tongfang computer co.Ltd.",
    "12.346854174720": "Data Respons A/S",
    "24.2890289": "Electronics Company Limited",
    "24.13396496": "The Chamberlain Group, Inc",
    "24.6707": "ASI Communications, Inc.",
    "24.13938181": "Samsung Electronics Co.,Ltd",
    "24.10023447": "Castlenet Technology Inc.",
    "24.7921380": "Texas Instruments",
    "24.2227": "Fastwel",
    "12.346855141376": "Neo Electronics Ltd",
    "24.7900244": "Ping Communication",
    "24.2226": "Shenzhen Compass Technology Development Co.,Ltd",
    "24.11567576": "I-sys Corp",
    "24.7107005": "Ezelink Telecom",
    "20.44169483517952": "Lookman Electroplast Industries Ltd",
    "24.1872249": "Integrated System Technologies Ltd",
    "24.9725822": "Belden India Private Limited",
    "12.123917675524096": "Abalance Corporation",
    "20.194600861958144": "Fasii Information Technology (Shanghai) Ltd.",
    "24.557490": "Logitech (China) Technology Co., Ltd",
    "12.123917691162624": "Copper Labs, Inc.",
    "12.123917683040256": "Sadel S.p.A.",
    "24.9438066": "Longnan Junya Digital Technology Co. Ltd.",
    "24.831101": "Texas Instruments",
    "24.16274893": "Visteon Corporation",
    "24.6837485": "Alcatel-Lucent",
    "24.16528535": "ASUSTek COMPUTER INC.",
    "12.346860904448": "CT Company",
    "20.149945507119104": "Shenzhen ORVIBO Technology Co., Ltd",
    "12.346859388928": "MangoDSP",
    "20.229631763415040": "MAX Smart Home, LLC",
    "20.150399884460032": "Shenzhen MMUI Co.,Ltd",
    "24.46961": "Cisco Systems, Inc",
    "24.13433701": "Nintendo Co., Ltd.",
    "12.346860883968": "SPJ Embedded Technologies Pvt. Ltd.",
    "20.225209255723008": "Q-Branch Labs, Inc.",
    "20.83999496077312": "Orica Europe Pty Ltd & Co KG",
    "20.176032234602496": "RYD Electronic Technology Co.,Ltd.",
    "24.6709": "BARTEC GmbH",
    "12.346854739968": "Extreme Engineering Solutions",
    "24.8188090": "Linkwell Telesystems Pvt Ltd",
    "20.124214344417280": "Cuav",
    "24.11297059": "Drivven, Inc.",
    "12.346862055424": "Varec Inc.",
    "24.267602": "Apple, Inc.",
    "12.123917682937856": "Romteck Australia",
    "12.123917690646528": "shenzhen suofeixiang technology Co.,Ltd",
    "12.123917681258496": "JFA Electronics Industry and Commerce EIRELI",
    "12.346861051904": "Spectrum Management, LC",
    "24.16055224": "Huawei Technologies Co.,Ltd",
    "24.5256075": "Texas Instruments",
    "24.9213132": "Sfr",
    "24.8425245": "Tp-Link Technologies Co.,Ltd.",
    "12.123917688872960": "Promomed Rus Llc",
    "24.11585883": "Beijing Symtech CO.,LTD",
    "20.167594815389696": "Zhejiang Hanshow Technology Co., Ltd.",
    "24.14950267": "Cisco Systems, Inc",
    "24.4975": "PacketMotion, Inc.",
    "24.4974": "Techmetro Corp.",
    "20.176069210537984": "Standartoptic, Limited Liability Company",
    "24.1859806": "Mercury Communication Technologies Co.,Ltd.",
    "24.7881985": "Biamp Systems",
    "20.71070623203328": "SKS-Kinkel Elektronik GmbH",
    "24.6851343": "China Mobile IOT Company Limited",
    "24.12095252": "Analytica GmbH",
    "24.7881981": "Tp-Link Technologies Co.,Ltd.",
    "24.5799782": "Huawei Technologies Co.,Ltd",
    "24.1071410": "Roku, Inc",
    "24.2943089": "Alert Metalguard ApS",
    "24.7659906": "Texas Instruments",
    "20.368739090432": "Quantum Communication Technology Co., Ltd.,Anhui",
    "24.3459568": "LM Technologies Ltd",
    "12.346852376576": "Ing. Buero W. Kanis GmbH",
    "12.346860826624": "Xtensor Systems Inc.",
    "24.8225": "Algorithms Software Pvt. Ltd.",
    "24.3413406": "Apple, Inc.",
    "12.123917680304128": "Micro Trend Automation Co., LTD",
    "24.14733443": "Samsung Electronics Co.,Ltd",
    "24.3729368": "HTC Corporation",
    "24.14200817": "Panasonic Appliances Company",
    "12.123917677494272": "ModuSystems, Inc",
    "12.346863517696": "Global Rainmakers Inc.",
    "24.2225": "ProQuent Systems",
    "24.11810148": "Fujian Tian Cheng Electron Science & Technical Development Co.,Ltd.",
    "24.6765": "Cisco Systems, Inc",
    "24.9206930": "Apple, Inc.",
    "24.2362289": "KOSTAL Industrie Elektrik GmbH",
    "24.8424599": "Kaonmedia CO., LTD.",
    "24.4010": "Nexus Technologies",
    "24.4011": "Kyushu Electronics Systems Inc.",
    "24.4012": "Ieee 802.11",
    "24.4013": "FMN communications GmbH",
    "24.4014": "E2O Communications",
    "24.4015": "Dialog Inc.",
    "24.4016": "Compal Electronics INC.",
    "24.4017": "Cognio Inc.",
    "24.4018": "Broadband Pacenet (India) Pvt. Ltd.",
    "12.346862362624": "Connor-Winfield",
    "12.346867134464": "Enika.Cz",
    "12.346860953600": "Elsys AG",
    "24.2220": "Eltromat GmbH",
    "24.3699166": "Apple, Inc.",
    "20.13692017049600": "D-Link （Shanghai）Limited Corp.",
    "24.4490945": "Siemens Low Voltage & Products",
    "24.6762": "Tranzas, Inc.",
    "24.3549": "Profilo Telra Elektronik Sanayi ve Ticaret. A.Ş",
    "20.45033660088320": "Atomrock LLC",
    "24.3547": "Airwave Technologies Inc.",
    "24.3546": "Allied Telesis K.K.",
    "24.3545": "Anton Paar GmbH",
    "24.3544": "Bbn",
    "24.3543": "Bright",
    "24.3542": "Iti Ltd",
    "24.3541": "O'Rite Technology Co.,Ltd",
    "24.3540": "Symantec Corporation",
    "24.36924": "Atlantic Network Systems",
    "12.346861457408": "Gimcon",
    "24.13384415": "Private",
    "24.2229": "Tai Guen Enterprise Co., Ltd",
    "24.7362495": "New H3C Technologies Co., Ltd",
    "24.2228": "Syspol",
    "12.346858606592": "EBNeuro SpA",
    "24.14740288": "Apple, Inc.",
    "24.10248875": "Sumavision Technologies Co.,Ltd",
    "12.346859368448": "QR Sciences Ltd",
    "12.123917688455168": "Southern Ground Audio LLC",
    "24.10260124": "Huawei Device Co., Ltd.",
    "24.2930333": "RED Digital Cinema",
    "20.31614746755072": "Nitto Seiko",
    "24.3099": "ORACOM Co, Ltd.",
    "24.3098": "Quest Technical Solutions Inc.",
    "24.555404": "Huawei Technologies Co.,Ltd",
    "24.3090": "Micro-Optronic-Messtechnik GmbH",
    "24.3093": "CyberPower Systems, Inc.",
    "24.3092": "Diagnostic Instruments, Inc.",
    "24.3095": "AJA Video Systems Inc",
    "24.3094": "Concorde Microsystems Inc.",
    "24.3097": "Telio Communications GmbH",
    "24.3096": "Zenisu Keisoku Inc.",
    "24.13160473": "Shanghai Tigercel Communication Technology Co.,Ltd",
    "24.1750": "Cisco Systems, Inc",
    "20.136823646978048": "Hoperun Mmax Digital Pte. Ltd.",
    "20.57353101639680": "ISRA Vision AG",
    "24.13154964": "Samsung Electro-Mechanics(Thailand)",
    "24.1752": "Maple Optical Systems",
    "24.12347926": "tdvine",
    "20.132811187027968": "Eksagate Elektronik Mühendislik ve Bilgisayar San. Tic. A.Ş.",
    "20.31657728933888": "Dream Visions Co., LTD",
    "24.16263059": "Apple, Inc.",
    "12.346864959488": "Kyuhen",
    "24.5416": "Beacon Medical Products LLC d.b.a. BeaconMedaes",
    "12.123917681225728": "Star Systems International",
    "24.13171332": "Trusonus corp.",
    "12.346859462656": "AV SatCom AS",
    "24.9461116": "New H3C Technologies Co., Ltd",
    "24.5414": "Remote Technologies Inc",
    "12.346856579072": "Satellite Services BV",
    "24.15256600": "D-Link International",
    "24.5285064": "Samsung Electronics Co.,Ltd",
    "20.255553903263744": "CoxSpace",
    "24.1066488": "LNT-Automation GmbH",
    "24.14689592": "Titan Pet Products",
    "12.71297884033024": "Cantada Inc",
    "24.535789": "Technity Solutions Inc.",
    "12.346866262016": "Scale-Tron, Inc.",
    "20.123731222462464": "Shenzhen Coolhear Information Technology Co., Ltd.",
    "20.119143419346944": "Kwong Ming Electrical Manufactory Limited",
    "24.12326": "HeiTel Digital Video GmbH",
    "24.12327": "Kerbango, Inc.",
    "24.12324": "Cisco Systems, Inc",
    "24.12325": "Checkout Computer Systems, Ltd",
    "24.12322": "Fong Kai Industrial Co., Ltd.",
    "24.12323": "Cogent Computer Systems, Inc.",
    "24.12320": "TSI, Inc..",
    "24.12321": "Hsing Tech. Enterprise Co.,Ltd",
    "24.14478342": "Heimi Network Technology Co., Ltd.",
    "24.12328": "FASE Saldatura srl",
    "24.12329": "Opicom",
    "24.5261744": "Samsung Electronics Co.,Ltd",
    "12.346852999168": "AlphaMedia Co., Ltd",
    "24.16277152": "Intel Corporate",
    "24.841542": "Apple, Inc.",
    "24.9459166": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.212014907523072": "Vernier Software & Technology",
    "12.123917681192960": "Heroic Technologies Inc.",
    "12.123917681987584": "Ohmori Electric Industries Co.Ltd",
    "24.15489670": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "20.88914832392192": "Telic AG",
    "24.7730": "Zensys",
    "24.7731": "Inventec Corporation",
    "24.7732": "CryptoMetrics",
    "24.7733": "Nintendo Co., Ltd.",
    "24.7734": "Ipte",
    "24.7735": "Universal Global Scientific Industrial Co., Ltd.",
    "24.7736": "Bluecard Software Technology Co., Ltd.",
    "24.7737": "Comsys Communication Ltd.",
    "24.7738": "Nokia Danmark A/S",
    "24.2410974": "Genoray",
    "24.5282374": "Safetrust Inc",
    "24.5831556": "Intel Corporate",
    "20.4884639252480": "Griesser Electronic AG",
    "24.64033": "Samsung Electronics Co.,Ltd",
    "24.589": "Mannesman Dematic Colby Pty. Ltd.",
    "24.588": "SiByte, Inc.",
    "24.583": "Great Dragon Information Technology (Group) Co., Ltd.",
    "24.582": "All-Win Tech Co., Ltd.",
    "24.581": "Lampus Co, Ltd.",
    "24.580": "SURECOM Technology Co.",
    "24.587": "Cisco Systems, Inc",
    "24.586": "Cisco Systems, Inc",
    "24.585": "Aviv Infocom Co, Ltd.",
    "24.584": "Pilz GmbH & Co.",
    "20.88383277760512": "Shinsoft Co., Ltd.",
    "24.12484": "Canon Imaging Systems Inc.",
    "24.8139630": "MEG Electronics Inc.",
    "24.7375369": "Huawei Technologies Co.,Ltd",
    "24.9722059": "Nintendo Co.,Ltd",
    "12.346856636416": "Offshore Systems Ltd",
    "20.35758072135680": "Unilumin Group Co.,Ltd",
    "24.1321090": "Midicom Electronics Co.Ltd",
    "12.346853703680": "Partner Voxstream A/S",
    "12.123917685415936": "Lockheed Martin - THAAD",
    "12.123917679403008": "DEUTA-WERKE GmbH",
    "24.9186730": "Apple, Inc.",
    "24.9238853": "Power Automation pte Ltd",
    "24.7370625": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7657194": "Texas Instruments",
    "12.123917689315328": "Redfish Group Pty Ltd",
    "12.346858835968": "Redfone Communications LLC",
    "24.14439639": "Amazon Technologies Inc.",
    "12.123917675196416": "Legendaire Technology Co., Ltd.",
    "24.15007045": "Intel Corporate",
    "12.346858315776": "Magal Senstar Inc.",
    "12.123917675974656": "Lupa Tecnologia e Sistemas Ltda",
    "24.3979051": "PLUMgrid Inc",
    "24.2900011": "Smart iBlue Technology Limited",
    "12.71297883856896": "Alumbra Produtos Elétricos e Eletrônicos Ltda",
    "12.123917681065984": "YUYAMA MFG Co.,Ltd",
    "24.11303490": "iConnectivity",
    "24.14943289": "TomTom Software Ltd",
    "24.1580522": "Intel Corporate",
    "24.2760": "ZPSYS CO.,LTD. (Planning&Management)",
    "24.2761": "Zambeel Inc",
    "24.2762": "YOKOYAMA SHOKAI CO.,Ltd.",
    "24.2763": "XPAK MSA Group",
    "24.2764": "Winnow Networks, Inc.",
    "24.2765": "Sunrich Technology Limited",
    "24.6069": "Peerless Systems Corporation",
    "24.6068": "Remote Security Systems, LLC",
    "24.6067": "Aftek Infosys Limited",
    "24.6066": "SK Telesys",
    "24.6065": "ACIST Medical Systems, Inc.",
    "24.6064": "Nokia Danmark A/S",
    "24.6063": "Enermet",
    "24.6062": "GAI-Tronics",
    "24.6061": "AceNet Corporation",
    "24.6060": "O'Neil Product Development Inc.",
    "24.15496378": "Hirschmann Automation and Control GmbH",
    "24.1239": "Omitec Instrumentation Ltd.",
    "24.1238": "Takagi Industrial Co., Ltd.",
    "24.1237": "Hitachi Information & Communication Engineering, Ltd.",
    "24.1236": "Proview Electronics Co., Ltd.",
    "24.1235": "Toyokeiki Co., Ltd.",
    "24.1234": "Adcon Telemetry GmbH",
    "24.1233": "Drew Technologies, Inc.",
    "24.1232": "Softlink s.r.o.",
    "24.1231": "Seagate Technology",
    "24.1230": "Patria Ailon",
    "24.15392136": "IEEE PES-PSRC Working Group H3, PC37.237",
    "24.8966201": "TCL Technoly Electronics(Huizhou).,Ltd",
    "24.8519": "Nintendo Co., Ltd.",
    "24.8518": "Sanmina-SCI",
    "24.8515": "ARRIS Group, Inc.",
    "24.8514": "Advanced Control Systems doo",
    "24.8517": "Semptian Technologies Ltd.",
    "24.8516": "SS Telecoms",
    "24.8511": "A-Team Technology Ltd.",
    "24.8510": "TomTom International BV",
    "24.8513": "Radlive",
    "12.346853597184": "Celite Systems, Inc.",
    "24.9749737": "PowerLayer Microsystems HongKong Limited",
    "24.9722750": "Trilobit Ltda.",
    "12.346865475584": "Moravian Instruments",
    "12.346862022656": "Watthour Engineering Co., Inc.",
    "12.346854801408": "Micronet Ltd.",
    "20.368736993280": "Lucistechnologies（Shanghai）Co.,Ltd",
    "12.346853666816": "Dickson Technologies",
    "12.346855657472": "Ihm",
    "24.5008": "Termtek Computer Co., Ltd",
    "24.5009": "Ouen Co.,Ltd.",
    "24.10798923": "Huawei Device Co., Ltd.",
    "20.264625139482624": "Hanju Network Technologies Co.",
    "20.268902725582848": "Teq Sa",
    "12.123917684752384": "Honeywell/Intelligrated",
    "24.5000": "WiMedia Alliance",
    "24.5001": "Redes de Telefonía Móvil S.A.",
    "24.5002": "Qingdao GoerTek Technology Co., Ltd.",
    "24.5003": "Phantom Technologies LLC",
    "24.5004": "Kumyoung.Co.Ltd",
    "24.5005": "Kinghold",
    "24.5006": "FOAB Elektronik AB",
    "24.5007": "Asiarock Technology Limited",
    "24.15264362": "StarTech.com",
    "24.2397387": "ARRIS Group, Inc.",
    "24.13162962": "Hewlett Packard",
    "24.10783936": "JLG Industries, Inc.",
    "24.7117193": "Cisco Systems, Inc",
    "12.71297885036544": "Device Solutions Ltd",
    "24.12076249": "Apple, Inc.",
    "24.14949451": "V2 Technology, Inc.",
    "24.16027145": "Nokia Corporation",
    "24.1586802": "Shaoxing ShunChuang Technology CO.,LTD",
    "24.12110261": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.71297883201536": "Bactest Limited",
    "24.16570045": "Robert Bosch GmbH",
    "24.804282": "Huawei Technologies Co.,Ltd",
    "12.71297884151808": "Siegmar Zander HuSWare",
    "24.16520628": "Hewlett Packard",
    "12.346860019712": "Cominfo, Inc.",
    "24.1593138": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8451331": "ARRIS Group, Inc.",
    "24.10250705": "Huawei Technologies Co.,Ltd",
    "24.9973096": "Samsung Electronics Co., Ltd",
    "24.3683835": "Sagemcom Broadband SAS",
    "24.7100055": "Amazon Technologies Inc.",
    "24.5569174": "Nokia Solutions and Networks GmbH & Co. KG",
    "20.176069197955072": "Kyung In Electronics",
    "24.15521418": "Shenzhen JMicron Intelligent Technology Developmen",
    "24.15757126": "Ubiik",
    "12.123917688238080": "Triax A/S",
    "20.268341259272192": "Shenzhen Grandsun Electronic Co.,Ltd.",
    "24.8172866": "ACES Technology",
    "24.15482377": "Performance Designed Products, Llc",
    "24.1868720": "Linked IP GmbH",
    "12.346857193472": "ImpediMed Limited",
    "24.12605372": "Avaya Inc",
    "20.88668800811008": "PointGrab",
    "24.1632040": "Cisco Systems, Inc",
    "12.123917680562176": "Qingdao Haitian Weiye Automation Control System Co., Ltd",
    "24.9758917": "ARRIS Group, Inc.",
    "24.10522985": "LG Electronics (Mobile Communications)",
    "24.4357": "Sunplus Technology Co., Ltd.",
    "24.4356": "Telexy",
    "24.4355": "kawamura electric inc.",
    "24.4354": "Aurora Multimedia Corp.",
    "24.4353": "CET Technologies Pte Ltd",
    "24.4352": "Schneider Electric",
    "24.4351": "Cisco Systems, Inc",
    "24.4350": "Digital Equipment Corporation",
    "24.3436870": "Huawei Device Co., Ltd.",
    "24.4359": "RGB Networks Inc.",
    "24.4358": "Siemens NV (Belgium)",
    "20.84187712323584": "AvertX",
    "12.346854338560": "SysAware S.A.R.L.",
    "24.8940209": "vivo Mobile Communication Co., Ltd.",
    "12.123917677449216": "Asystems Corporation",
    "24.9732516": "Azuray Technologies",
    "12.123917681004544": "Eizo Rugged Solutions",
    "24.10500725": "PSS Belgium N.V.",
    "12.71297884733440": "Critical Link LLC",
    "12.346859233280": "ESTEC Co.,Ltd.",
    "20.57176931434496": "Project Engineering srl",
    "20.84187703934976": "Smart Access",
    "24.12354847": "Samsung Electronics Co.,Ltd",
    "12.71297885052928": "Rigel Engineering",
    "20.18540145934336": "LeoLabs",
    "24.5529277": "NL Technologies",
    "12.346868199424": "Supranet",
    "12.346854797312": "Ansitex Corp.",
    "24.15223894": "Intercept Services Limited",
    "24.12359817": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.7079638": "Huawei Device Co., Ltd.",
    "12.123917675806720": "Tiama",
    "24.14731554": "Jireh Energy Tech., Ltd.",
    "24.57709": "Cisco Systems, Inc",
    "24.12111606": "Mellanox Technologies, Inc.",
    "20.176772039573504": "Serious Integrated, Inc.",
    "20.31368092319744": "Philips Personal Health Solutions",
    "12.71297884590080": "PLATINUM GmbH",
    "12.346853068800": "Interisa Electronica, S.A.",
    "20.203238160728064": "Vonger Electronic Technology Co.,Ltd.",
    "12.346853097472": "Collabo Tec. Co., Ltd.",
    "24.35924": "ADB Broadband Italia",
    "24.14961772": "Shenzhen Guo Wei Electronic Co,. Ltd.",
    "24.308265": "Samsung Electronics Co.,Ltd",
    "20.127937510637568": "Qingdao Wintec System Co., Ltd",
    "12.71297884934144": "Wicher Digital Technik",
    "12.123917676560384": "Farmpro Ltd",
    "24.4995285": "Mitac Computing Technology Corporation",
    "24.6862233": "Hewlett Packard",
    "24.5513240": "Tely Labs",
    "12.123917686120448": "Amphenol Advanced Sensors",
    "24.11817641": "Texas Instruments",
    "24.9160": "Team-R",
    "12.346859163648": "Heim- & Bürokommunikation",
    "12.346855755776": "Oplink Communications",
    "12.123917680058368": "Biotage Sweden AB",
    "24.310407": "Shenzhen JustLink Technology Co., LTD",
    "12.346866417664": "Rong Shun Xuan Corp.",
    "24.12648404": "Netgear",
    "24.12093382": "Prudential Technology co.,LTD",
    "24.15227739": "Lg Electronics Inc",
    "12.346867933184": "HASCOM International Pty Ltd",
    "24.4511293": "Talari Networks",
    "24.2683150": "Dell Inc.",
    "24.9468176": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346852970496": "SAIA Burgess Controls AG",
    "24.12123442": "Zhejiang ROICX Microelectronics",
    "12.346856493056": "Efsys",
    "24.10289086": "OTSL Inc.",
    "24.8950897": "Brocade Communications Systems LLC",
    "24.9444": "Withings",
    "24.9445": "Seer Technology, Inc",
    "24.9446": "In Motion Technology Inc.",
    "24.9447": "Plaster Networks",
    "24.9440": "DS Tech, LLC",
    "24.9441": "Convey Computer Corp.",
    "24.9442": "Hasegawa Electric Co.,Ltd.",
    "24.9443": "CAO Group",
    "24.9448": "Dell Inc.",
    "24.9449": "Samsung Electronics Co.,Ltd",
    "12.346859966464": "ZigBee Alliance",
    "24.7375821": "ASUSTek COMPUTER INC.",
    "24.13255788": "Cypress Computer Systems, Inc.",
    "24.14992939": "Airware",
    "20.106196526694400": "Untangle, Inc.",
    "24.13667509": "Texas Instruments",
    "24.7660172": "Microsoft Corporation",
    "24.7140223": "Aruba, a Hewlett Packard Enterprise Company",
    "24.1359934": "ARRIS Group, Inc.",
    "24.8417567": "Juniper Networks",
    "20.62366867783680": "Bumjin C&L Co., Ltd.",
    "12.346860191744": "Integrated Design Tools, Inc.",
    "24.2419123": "Cisco Systems, Inc",
    "24.8397053": "Samsung Electronics Co.,Ltd",
    "24.5510537": "MCS Logic Inc.",
    "12.123917679456256": "biosilver.co.,ltd",
    "24.7077": "MyungMin Systems, Inc.",
    "24.7076": "S.A.E Afikim",
    "24.7075": "Flexit Group GmbH",
    "24.7074": "IDS Imaging Development Systems GmbH",
    "24.7073": "Åmic AB",
    "24.7072": "Awox",
    "24.7071": "Calyptech Pty Ltd",
    "24.7070": "Askey Computer Corp",
    "24.7079": "Lorica Solutions",
    "24.2883635": "EControls, LLC",
    "24.2247": "Hewlett Packard",
    "24.2246": "Philips Consumer Communications",
    "24.2245": "Liontech Co., Ltd.",
    "24.2244": "Hikari Co.,Ltd.",
    "24.2243": "Contex A/S",
    "24.2242": "Cisco Systems, Inc",
    "24.2241": "Avistar Communications Corporation",
    "24.2240": "Asa Systems",
    "12.346856120320": "Net Optics",
    "24.10230802": "Aruba, a Hewlett Packard Enterprise Company",
    "24.9108246": "TeraByte, Inc.",
    "24.2249": "TechniSat Digital GmbH Daun",
    "24.2248": "Soneticom, Inc.",
    "12.346861252608": "Nanomotion Ltd.",
    "24.2152710": "Iota, Inc.",
    "24.1885574": "Absolute Acoustics",
    "20.154039771725824": "Elcon AB",
    "12.346856570880": "ImperativeNetworks",
    "12.346855456768": "Odixion",
    "24.13429276": "DCONWORKS Co., Ltd",
    "12.123917687209984": "Eiwa Giken Inc.",
    "24.8406833": "Packet Design, Inc.",
    "20.132777844408320": "Ory Laboratory Co., Ltd.",
    "12.346864455680": "Toptech Systems, Inc.",
    "24.316942": "Funai Electric Co., Ltd.",
    "24.14979198": "Motorola Mobility LLC, a Lenovo Company",
    "24.9096100": "Resideo",
    "12.123917684248576": "Field Design Inc.",
    "24.14707991": "Apple, Inc.",
    "24.3450205": "Technosoft International Srl",
    "12.123917690028032": "Beijing PanGu Company",
    "12.123917690732544": "Aplex Technology Inc.",
    "12.71297884581888": "Hitachi Power Solutions Co., Ltd.",
    "24.14165772": "MaxTronic International Co., Ltd.",
    "12.123917678415872": "Vtron Pty Ltd",
    "20.189244295348224": "PiOctave Solutions Pvt Ltd",
    "12.123917691092992": "Cdr Srl",
    "12.123917689511936": "Umweltanalytik Holbach GmbH",
    "24.571192": "Lite-On Technogy Corp.",
    "12.71297883799552": "Dave Srl",
    "24.4200186": "EKM Metering",
    "24.15740888": "Bi2-Vision",
    "12.346853707776": "Verifiber LLC",
    "12.346864160768": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "12.346857971712": "Dixell S.p.a.",
    "12.123917679996928": "EA Elektroautomatik GmbH & Co. KG",
    "24.14688770": "ARRIS Group, Inc.",
    "24.6600227": "CCS Care Communication Solutions GmbH",
    "24.9190194": "Microfan B.V.",
    "24.8664593": "hyungseul publishing networks",
    "24.1315915": "Ruijie Networks Co.,LTD",
    "24.5035575": "Qsono Electronics Co., Ltd",
    "12.123917687226368": "Controlled Power Company",
    "12.346861756416": "DiTEST Fahrzeugdiagnose GmbH",
    "20.168198562381824": "Koala Technology CO., LTD.",
    "12.123917689946112": "Siemens Mobility GmbH - MO TI SPA",
    "24.15227914": "Cisco Systems, Inc",
    "24.3969408": "Liteon Technology Corporation",
    "24.15253992": "Shenzhen Xiao Bi En Culture Education Technology Co.,Ltd.",
    "12.123917683613696": "Authenticdata",
    "12.346862043136": "Aqua Cooler Pty Ltd",
    "24.3810": "Custom Engineering",
    "24.14188601": "Microchip Technology Inc.",
    "24.14188604": "Anhui Huami Information Technology Company Limited",
    "24.1075263": "LG Electronics (Mobile Communications)",
    "12.123917680074752": "Operational Technology Solutions",
    "12.346861547520": "Curtis Door Systems Inc",
    "12.346855780352": "Companion Worlds, inc.",
    "24.7918228": "Netgear",
    "24.11047754": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.13144126": "Pakton Technologies",
    "24.14217165": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.5005909": "Sanmina-Sci System De Mexico S.A. De C.V.",
    "12.123917691523072": "InDiCor",
    "24.13681282": "Cisco Systems, Inc",
    "12.123917688172544": "Trident Infosol Pvt Ltd",
    "12.123917682130944": "Arinax",
    "12.346852659200": "Flextel S.p.A",
    "24.11849280": "Samsung Electronics Co.,Ltd",
    "24.13669248": "Ducere Technologies Pvt. Ltd.",
    "12.123917684895744": "4CAM GmbH",
    "12.71297884270592": "So-Cool Corporation.",
    "12.346864390144": "SISS Technology Inc.",
    "12.123917681754112": "Particle sizing systems",
    "12.346861842432": "Reliatronics Inc.",
    "12.346859614208": "ADVA Optical Networking",
    "24.5808821": "LG Electronics (Mobile Communications)",
    "24.13684739": "Samsung Electronics Co.,LTD",
    "12.346857418752": "Respironics Inc.",
    "24.9180340": "zte corporation",
    "12.123917680373760": "Valeo Cda",
    "24.11273227": "Peloton Interactive, Inc",
    "24.16047181": "Beijing CCD Digital Technology Co., Ltd",
    "24.3714857": "Zhejiang Dahua Technology Co., Ltd.",
    "24.11593197": "Huawei Technologies Co.,Ltd",
    "24.3422100": "Hamee Corp.",
    "12.346862530560": "SAI Informationstechnik",
    "24.8976319": "vivo Mobile Communication Co., Ltd.",
    "20.58144482918400": "ASA Innovation & Technology Ltd.",
    "12.123917679656960": "Kitron Uab",
    "20.93020696346624": "Alfen BV",
    "24.11071544": "Shen Zhen Shi Jin Hua Tai Electronics Co.,Ltd",
    "12.346866999296": "Cinetix s.r.l.",
    "12.346864922624": "Z-App Systems, Inc.",
    "24.313817": "Dish Technologies Corp",
    "12.346855895040": "TTC Telecom",
    "12.123917691600896": "Getraline",
    "20.93020702638080": "Private",
    "24.283605": "Sysorex Global Holdings",
    "12.123917690343424": "Qntra Technology",
    "24.5037885": "Team Engineers Advance Technologies India Pvt Ltd",
    "24.5017819": "JL Audio",
    "24.11050450": "Shanghai DareGlobal Technologies Co.,Ltd",
    "24.4454": "Taelim Electronics Co., Ltd.",
    "20.180698128318464": "Homebeaver",
    "12.123917688766464": "Rheonics GmbH",
    "12.123917687787520": "German Power GmbH",
    "24.8682166": "AltoBeam (China) Inc.",
    "20.58071049043968": "EQPlay Intelligent Technology(Kunshan) Co,Ltd.",
    "12.123917679910912": "Reinhardt System- und Messelectronic GmbH",
    "12.346857484288": "Genie Network Resource Management Inc.",
    "12.71297883844608": "Telefrank GmbH",
    "12.346859253760": "iTRACS Corporation",
    "24.8934293": "Apple, Inc.",
    "24.8939612": "Shenzhen ChuangDao & Perpetual Eternal Technology Co.,Ltd",
    "24.2373615": "EMC Electronic Media Communication SA",
    "24.1111194": "T3 Innovation",
    "24.16519": "Ubitrex Corporation",
    "24.16518": "Michels & Kleberhoff Computer",
    "24.16515": "Tda Industria De Produtos",
    "24.16514": "Laboratory Equipment Corp.",
    "24.16517": "Saab Instruments Ab",
    "24.16516": "Honeywell International HPS",
    "24.16511": "FLIR Systems",
    "24.1580720": "Samsung Electronics Co.,Ltd",
    "24.16513": "Mannesmann Scangraphic Gmbh",
    "24.16512": "Athenix Corporation",
    "12.346852364288": "Eloquence Ltd",
    "12.123917689253888": "BRS Sistemas Eletrônicos",
    "20.273272604196864": "Cadmus Electronic Co.,Ltd.",
    "24.9443002": "Huawei Technologies Co.,Ltd",
    "24.9238547": "Orange Polska",
    "12.346854248448": "SouthWing S.L.",
    "12.119269687296": "Siemens Industries, Inc, Retail & Commercial Systems",
    "12.346860171264": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.3415280": "Xiaomi Communications Co Ltd",
    "24.14206833": "Huawei Technologies Co.,Ltd",
    "12.346862125056": "Digital Payment Technologies",
    "24.2143129": "Enterasys",
    "24.7638280": "Apple, Inc.",
    "24.286649": "Cisco Systems, Inc",
    "24.1356070": "Industrial Software Co",
    "12.346859835392": "e&s Engineering & Software GmbH",
    "24.12610842": "Tianjin Henxinhuifeng Technology Co.,Ltd.",
    "24.11292668": "SVS-VISTEK GmbH",
    "20.264809158279168": "Dongguan Gedi Electrons Techeology Co.,LTD",
    "24.7917359": "Huawei Technologies Co.,Ltd",
    "24.19443": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "20.137141741944832": "SIGMA-ELEKTRO GmbH",
    "12.346856161280": "Micro-Robotics Limited",
    "20.154039767531520": "TeleAlarm SA",
    "24.5565096": "Apple, Inc.",
    "24.9754481": "Samsung Electronics Co.,Ltd",
    "12.346853834752": "PotsTek, Inc",
    "24.13651838": "Apple, Inc.",
    "24.15257883": "Askey Computer Corp",
    "24.553360": "Intel Corporate",
    "12.346868559872": "Xemex NV",
    "24.16318303": "Shenzhen Communication Technology Co.,Ltd",
    "24.4776899": "JENOPTIK Advanced Systems GmbH",
    "12.346861334528": "ARISTO Graphic Systeme GmbH & Co. KG",
    "24.12375911": "Private",
    "24.9751055": "Honeywell Analytics",
    "12.123917676924928": "MIVO Technology AB",
    "24.8452699": "Clevo Co.",
    "12.346854707200": "Dsp Design",
    "20.128510973706240": "Private",
    "24.5012575": "Apple, Inc.",
    "20.45069247709184": "Jasco Products Company",
    "24.4229786": "TITENG CO., Ltd.",
    "24.12506": "Comtrend Corporation",
    "12.346856247296": "Vacon Plc",
    "24.1577329": "Guangzhou Doctorpai Education & Technology Co.,Ltd",
    "24.12505": "Datacore Software Corp.",
    "24.12502": "Msc Vertriebs Gmbh",
    "24.12503": "Innovative Systems, L.L.C.",
    "24.12500": "AAE Systems, Inc.",
    "24.12501": "DResearch GmbH",
    "24.9457091": "Flonidan A/S",
    "20.149945499779072": "Beijing laiwei Technology Co.,Ltd",
    "24.12508": "Rightech Corporation",
    "24.12509": "Indigita Corporation",
    "24.1313477": "Amazon Technologies Inc.",
    "24.6074378": "Huawei Technologies Co.,Ltd",
    "12.123917683507200": "QIT Co., Ltd.",
    "24.5246": "InnerWireless",
    "24.5247": "Thomson Telecom Belgium",
    "24.5244": "3Com Ltd",
    "24.5245": "Aeon Digital International",
    "24.5242": "Eubus GmbH",
    "24.5243": "Iteris, Inc.",
    "24.5240": "Tp-Link Technologies Co.,Ltd.",
    "24.5241": "NEC Magnus Communications,Ltd.",
    "24.12086504": "Samsung Electronics Co.,Ltd",
    "24.5248": "Hitachi-LG Data Storage Korea, Inc",
    "24.5249": "Multilink Inc",
    "24.4195": "Starguide Digital Networks",
    "24.4194": "Nx Server, Ilnc.",
    "24.4197": "Radyne Corporation",
    "24.4196": "Dnpg, Llc",
    "24.1360413": "Sabtech Industries",
    "24.4190": "Spirent plc, Service Assurance Broadband",
    "24.4193": "Hostlink Corp.",
    "24.4192": "Billington",
    "24.4199": "Ericsson",
    "24.4198": "Advanced Control Systems, Inc.",
    "12.346858643456": "TDM Ingénierie",
    "24.8653692": "Hitron Technologies. Inc",
    "12.346856390656": "TERN, Inc.",
    "12.346858127360": "INCOTEC GmbH",
    "24.12386009": "Apple, Inc.",
    "12.123917691699200": "Finder SpA",
    "12.123917682085888": "Labtronik s.r.l.",
    "20.88010613850112": "Security Alarms & Co. S.A.",
    "24.16547825": "Aruba, a Hewlett Packard Enterprise Company",
    "12.123917681348608": "Link Electric & Safety Control Co.",
    "24.7399212": "Apple, Inc.",
    "24.9756886": "China Dragon Technology Limited",
    "24.9756880": "HealthStream Taiwan Inc.",
    "24.10512510": "ARRIS Group, Inc.",
    "24.10528973": "Intel Corporate",
    "24.15225198": "TSMART Inc.",
    "24.15740747": "Apple, Inc.",
    "24.2911101": "Texas Instruments",
    "24.10513644": "Rim",
    "12.123917677760512": "Blink Services AB",
    "24.12622053": "University of Michigan",
    "24.16311355": "ASIX Electronics Corporation",
    "24.13423553": "Vestel Elektronik San ve Tic. A.Ş.",
    "24.5801188": "IP500 Alliance e.V.",
    "24.1858944": "Mitubishi Hitachi Power Systems Industries Co., Ltd.",
    "20.36069853626368": "Lauda Dr R Wobser Gmbh & Co Kg",
    "12.346853281792": "Endusis Limited",
    "24.57549": "Saab Sensis Corporation",
    "24.57548": "Hero Systems, Ltd.",
    "24.3978": "WideView",
    "24.3979": "Orion MultiSystems Inc",
    "24.57541": "Bcom Electronics Inc.",
    "24.57540": "Horner Electric, Inc.",
    "24.57543": "Eurotech Srl",
    "24.57542": "Link2It, L.L.C.",
    "24.3976": "AMETEK, Inc.",
    "24.3977": "Winnertec System Co., Ltd.",
    "24.57547": "Reson, Inc.",
    "24.57546": "Best Data Products",
    "12.123917685755904": "THELIGHT Luminary for Cine and TV S.L.",
    "12.346860556288": "JANUS srl",
    "24.5526735": "Probedigital Co.,Ltd",
    "12.123917691305984": "European Advanced Technologies",
    "24.4748675": "Intellion AG",
    "20.39919495938048": "Owasys Advanced Wireless Devices",
    "24.11838338": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "20.62636950552576": "Vanguard",
    "12.123917682855936": "APG Cash Drawer, LLC",
    "24.8420126": "Corsair Memory, Inc.",
    "12.123917684748288": "RF Code",
    "12.123917688733696": "MetCom Solutions GmbH",
    "20.35227549302784": "Welzek (Beijing) Technologies Co, Ltd",
    "24.10249310": "Harman Consumer Group",
    "24.12085748": "Routerboard.com",
    "12.119269732352": "QCORE Medical",
    "12.346867564544": "Pulse Communication Systems Pvt. Ltd.",
    "12.346855763968": "XIMEA s.r.o.",
    "12.123917679235072": "Beijing Yutian Technology Co., Ltd.",
    "24.7387472": "Texas Instruments",
    "20.17622840115200": "Diginet Control Systems Pty Ltd",
    "12.346852532224": "Sonifex Ltd",
    "24.3359": "AV Digital",
    "24.3356": "Amesys Defense",
    "24.3357": "High-Tek Harness Ent. Co., Ltd.",
    "24.3354": "Mustek System Inc.",
    "24.3355": "Kyoto Electronics Manufacturing Co., Ltd.",
    "24.3352": "Mega-Trend Electronics CO., LTD.",
    "24.3353": "ROBE Show lighting",
    "24.3350": "UHS Systems Pty Ltd",
    "24.3351": "Turbo Networks Co.Ltd",
    "12.346864758784": "Volumatic Limited.",
    "24.1836630": "IDY Corporation",
    "24.15208906": "Shandong Kaer Electric.Co.,Ltd",
    "20.48688649994240": "Shanghai Visteon Automotive Electronics System CO. Ltd.",
    "12.346860466176": "Sesol Industrial Computer",
    "12.346863833088": "Phytron-Elektronik GmbH",
    "24.13427071": "Juniper Networks",
    "12.123917687349248": "Hatteland Display AS",
    "24.5534302": "Unionman Technology Co.,Ltd",
    "24.5780953": "Cisco Systems, Inc",
    "24.6817137": "Samsung Electronics Co.,Ltd",
    "24.14437374": "Shenzhen Belon Technology CO.,LTD",
    "24.11841559": "Avaya Inc",
    "24.9467551": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "12.119269666816": "Verified Energy, LLC.",
    "24.49230": "Comtrol Corporation",
    "24.6879016": "LCFC(HeFei) Electronics Technology co., ltd",
    "20.88383274614784": "Chengdu Cove Technology Co.,Ltd",
    "20.154680188469248": "Suzhou Guowang Electronics Technology Co., Ltd.",
    "24.28155": "Vutrix Technologies Ltd",
    "12.71297884352512": "KOMPAN Pawel Sokolowski",
    "24.10775745": "LTi REEnergy GmbH",
    "12.346858733568": "Phytec Messtechnik GmbH",
    "24.9480129": "Pakedge Device and Software Inc.",
    "24.12289": "Smp",
    "24.12288": "Allwell Technology Corp.",
    "24.15254237": "Infinix mobility limited",
    "12.123917688078336": "NETA Elektronik AS",
    "24.279713": "Telecon Galicia,S.A.",
    "12.123917681827840": "Rosslare Enterprises Limited",
    "24.8906": "Anviz Biometric Tech. Co., Ltd.",
    "12.123917681549312": "Riken Optech Corporation",
    "24.6593032": "Shenzhen SuperElectron Technology Co.,LTD",
    "24.12330464": "Huawei Technologies Co.,Ltd",
    "24.12904714": "D-Link International",
    "24.8907": "IONODES Inc.",
    "20.84187711275008": "Erlab DFS SAS",
    "24.2676086": "Premier Wireless, Inc.",
    "24.2933344": "Oracle Corporation",
    "24.49336": "Fraser'S Hill Ltd.",
    "24.7393097": "Intel Corporate",
    "24.6452": "Trendon Touch Technology Corp.",
    "24.6453": "Duerr Dental Ag",
    "24.6450": "Gude Systems GmbH",
    "24.6451": "Strix Systems, Inc.",
    "24.6456": "UMB Communications Co., Ltd.",
    "24.6457": "Gigamips",
    "24.6454": "Sterlite Optical Technologies Limited",
    "24.6455": "CommerceGuard AB",
    "20.154238427594752": "Faravid Communication&Data Analysis",
    "20.49277250306048": "iCIRROUND Inc",
    "24.7912185": "Cisco Systems, Inc",
    "24.49332": "Myson Technology, Inc.",
    "12.346855448576": "BQT Solutions (Australia) Limited",
    "24.5803273": "Cisco Systems, Inc",
    "12.123917679812608": "Profcon AB",
    "24.14738990": "TCT mobile ltd",
    "24.49233": "Advanced Integration Research",
    "24.14738993": "Snb Technologies Limited",
    "12.123917680418816": "Dataflex International BV",
    "12.123917688635392": "DKI Technology Co., Ltd",
    "12.123917686665216": "Tecnowatt",
    "24.2485": "3J Tech. Co., Ltd.",
    "24.2484": "Kisan Telecom Co., Ltd.",
    "24.2487": "Cisco Systems, Inc",
    "24.2486": "Cisco Systems, Inc",
    "24.2481": "Kanematsu Electronics, Ltd.",
    "24.2480": "Onkyo Corporation",
    "24.2483": "MCM Systems Ltd",
    "24.2482": "L&F Inc.",
    "24.2489": "Action Imaging Solutions",
    "24.2488": "Entise Systems",
    "24.8689368": "ARRIS Group, Inc.",
    "12.346856935424": "Metron Sp. z o.o.",
    "24.628": "Tommy Technologies Corp.",
    "24.629": "SMART Technologies, Inc.",
    "24.54157": "Hotel Technology Next Generation",
    "24.627": "Coriolis Networks",
    "24.624": "Crewave Co., Ltd.",
    "24.625": "Zhone Technologies",
    "24.622": "NeGeN Access, Inc.",
    "24.37731": "Uni-Link Technology Co., Ltd.",
    "24.620": "Philips CFT",
    "24.1619703": "New Pos Technology Limited",
    "24.12083743": "Cisco Systems, Inc",
    "24.14198984": "zte corporation",
    "20.189244300591104": "Shenzhen Ouzheng Electronic Tech Co,.Ltd",
    "20.110162586107904": "Zhejiang Moorgen Intelligent Technology Co.,Ltd",
    "12.346867494912": "DataSoft Corporation",
    "24.4233294": "Texas Instruments",
    "24.4233292": "Casacom Solutions AG",
    "24.7509": "ZANTAZ, Inc",
    "24.7508": "Sunnic Technology & Merchandise INC.",
    "24.7507": "S&O Electronics (Malaysia) Sdn. Bhd.",
    "24.7506": "Defzone B.V.",
    "24.7505": "Babcock & Wilcox Power Generation Group, Inc",
    "24.7504": "Spinetix Sa",
    "24.7503": "Apple, Inc.",
    "24.7502": "TCM Mobile LLC",
    "24.7501": "Adaptive Recognition Hungary, Inc",
    "24.7500": "Alcatel-Lucent",
    "24.5570511": "Mopria Alliance",
    "24.10758531": "Boeing Defence Australia",
    "24.7360001": "Kwangwon Tech Co., Ltd.",
    "24.7899946": "Zhen Shi Information Technology (Shanghai) Co., Ltd.",
    "24.16313765": "ARRIS Group, Inc.",
    "24.10488762": "Samsung Electro Mechanics Co., Ltd.",
    "24.37082": "Dynarc, Inc.",
    "24.37083": "Next Level Communications",
    "24.37080": "Whitecross Systems",
    "24.37081": "Cisco Systems, Inc",
    "24.37086": "Cardkey Systems, Inc.",
    "24.37087": "Mitsubishi Chemical America, Inc.",
    "24.37084": "Teco Information Systems",
    "24.37085": "MIHARU COMMUNICATIONS Inc",
    "24.9075": "GridIron Systems, Inc.",
    "24.9074": "More Star Industrial Group Limited",
    "12.346858221568": "InSync Technology Ltd",
    "24.37089": "Telena S.P.A.",
    "24.9071": "DAQ System",
    "24.9070": "Burster GmbH & Co KG",
    "24.9073": "SOAM Systel",
    "24.9072": "Snell",
    "12.346856538112": "Trapeze Software Group Inc",
    "24.5908": "BR Controls Nederland bv",
    "24.1688": "egnite GmbH",
    "24.1689": "Vida Design Co.",
    "12.346854387712": "Monitor Business Machines Ltd",
    "24.8957394": "Samsung Electronics Co.,Ltd",
    "24.5901": "Dust Networks Inc.",
    "24.5900": "Twig Com Ltd.",
    "24.5903": "Cisco Systems, Inc",
    "24.5902": "Cisco Systems, Inc",
    "24.5905": "Cytiva Sweden AB",
    "24.5904": "Casa Systems Inc.",
    "20.4884636106752": "Neurio Technology Inc.",
    "24.5906": "ISCO International",
    "24.8164851": "Cisco Systems, Inc",
    "12.346861416448": "IFAM GmbH",
    "24.7625386": "Huawei Technologies Co.,Ltd",
    "24.15766330": "NxtConect",
    "24.12871998": "Turbulence Design Inc.",
    "20.17622848503808": "Tessera Technology Inc.",
    "12.123917690748928": "Scame Sistemi srl",
    "12.346868662272": "Leroy Somer",
    "24.12356437": "NPP ELIKS America Inc. DBA T&M Atlantic",
    "24.14206609": "Hichan Technology Corp.",
    "12.123917678325760": "RTA srl",
    "24.294435": "China Mobile IOT Company Limited",
    "24.12906158": "VSS Monitoring",
    "20.229558873751552": "Forgamers Inc.",
    "24.3985656": "Apple, Inc.",
    "24.11857138": "Shanghai Listent Medical Tech Co., Ltd.",
    "20.272689493180416": "Witium Co., Ltd",
    "24.16316415": "Syn-Tech Systems Inc",
    "12.123917682225152": "Shen Zhen Qls Electronic Technology Co.,Ltd.",
    "24.4199831": "Protect America, Inc.",
    "24.1363938": "ARRIS Group, Inc.",
    "24.6877117": "Cisco Systems, Inc",
    "12.346856222720": "BiODE Inc.",
    "12.119269711872": "PLAiR Media, Inc",
    "24.9179348": "Sagemcom Broadband SAS",
    "12.123917678137344": "Special Projects Group, Inc",
    "24.8308": "Sungwoon Systems",
    "24.8309": "Motorola Communication Israel",
    "24.8304": "Hynet, Ltd.",
    "24.5718": "Nintendo Co., Ltd.",
    "24.8306": "Worklink Innovations",
    "24.8307": "Fusion Systems Corporation",
    "24.8300": "Evergreen Technology Corp.",
    "24.8301": "Data Race, Inc.",
    "24.8302": "Xact, Inc.",
    "24.8303": "Flowpoint Corporation",
    "12.346854555648": "Dixtal Biomedica Ind. Com. Ltda.",
    "24.2366092": "Squarehead Technology AS",
    "12.119269994496": "Triax A/S",
    "24.11814901": "Earda Technologies co Ltd",
    "24.16560044": "Socionext Inc.",
    "20.154055548600320": "Anntec （Beijing） Technology Co.,Ltd.",
    "20.88383268323328": "Suzhou Ruixinjie Information Technology Co.,Ltd",
    "24.13695116": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.123917679480832": "Advanced Devices SpA",
    "24.8943963": "Konka Group Co., Ltd.",
    "24.12115550": "Longcheer Telecommunication Limited",
    "24.16525218": "Trans Electric Co., Ltd.",
    "24.531963": "Shanghai Mexon Communication Technology Co.,Ltd",
    "24.4230175": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.52912285810688": "Ilsan Electronics",
    "24.5248008": "AiNET",
    "12.123917676269568": "Dorlet Sau",
    "24.16018914": "zte corporation",
    "24.524431": "Chipcom Corporation",
    "24.524430": "Tandem / Solbourne Computer ?",
    "24.524432": "Sonoma Systems",
    "24.4475493": "Silverflare Ltd.",
    "24.10277741": "Netgear",
    "12.123917681016832": "LAMTEC Meß- und Regeltechnik für Feuerungen GmbH & Co. KG",
    "24.6594791": "Cisco Systems, Inc",
    "20.154680181129216": "Jiangsu Sixingda Information Technology Co., Ltd.",
    "12.71297883451392": "SchulerControl GmbH",
    "24.10757403": "Avaya Inc",
    "12.346862579712": "WoKa-Elektronik GmbH",
    "20.229206567944192": "Daiken Automacao Ltda",
    "24.8402369": "Xiaomi Communications Co Ltd",
    "24.12081911": "Ouya, Inc",
    "24.9673": "Shenzhen Huapu Digital Co., Ltd",
    "24.524383": "Saber Technology Corp.",
    "24.524382": "Counterpoint Computer Inc.",
    "24.524381": "Gould Inc.",
    "24.524380": "Four Phase Systems",
    "24.524387": "Plessey",
    "24.524386": "General Dynamics",
    "24.524385": "Jarogate Ltd.",
    "24.524384": "Industrial Networking Inc.",
    "24.524389": "Genrad Inc.",
    "24.524388": "Sitasys AG",
    "24.3732993": "Solid State Storage Technology Corporation",
    "12.346863267840": "Microsystemes",
    "24.4233339": "Aisino Corporation",
    "24.1848078": "PernixData",
    "12.71297884585984": "Exibea AB",
    "24.551053": "New H3C Technologies Co., Ltd",
    "24.12081918": "Handaer Communication Technology (Beijing) Co., Ltd",
    "24.12875691": "Beijing ASU Tech Co.,Ltd",
    "24.12875695": "Advanced Digital Broadcast SA",
    "24.8405326": "Bbk Educational Electronics Corp.,Ltd.",
    "12.123917684637696": "The Institute of Mine Seismology",
    "24.15224326": "Edup International (Hk) Co., Ltd",
    "24.15745478": "Heat Tech Company, Ltd.",
    "12.123917678878720": "Sicon srl",
    "24.8411652": "LG Electronics (Mobile Communications)",
    "24.9899": "Seiko Epson Corporation",
    "24.9898": "Kenmec Mechanical Engineering Co., Ltd.",
    "24.9897": "Strong Technologies Pty Ltd",
    "24.9896": "Daehap Hyper-Tech",
    "24.9895": "Connect Srl",
    "24.9894": "Trixell",
    "24.9893": "Microrobot.Co.,Ltd",
    "24.9892": "Novus Produtos Eletronicos Ltda",
    "24.9891": "FQ Ingenieria Electronica S.A.",
    "24.9890": "Instrumentation Technology Systems",
    "24.8399865": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.11141122": "DEC",
    "24.11141123": "DEC",
    "24.11141120": "DEC",
    "24.9748500": "Sorter Sp. j. Konrad Grzeszczyk MichaA, Ziomek",
    "12.346856808448": "Avenir Technologies Inc.",
    "12.123917675192320": "Monnit Corporation",
    "24.11058490": "Unionman Technology Co.,Ltd",
    "24.8126541": "Huawei Technologies Co.,Ltd",
    "12.346858913792": "Naim Audio",
    "12.346864386048": "Peek Traffic Corporation",
    "12.123917684797440": "EA Elektroautomatik GmbH & Co. KG",
    "24.972": "Momentum Computer, Inc.",
    "24.13671472": "Foster Electric Company, Limited",
    "24.3717": "Catalyst Enterprises, Inc.",
    "20.31477538488320": "Zillnk",
    "24.3714": "Commtech Wireless",
    "24.3715": "Cisco Systems, Inc",
    "12.346866241536": "Pro-Digital Industria Eletronica",
    "24.3712": "Thomson Technology Inc",
    "12.346856722432": "Phytec Messtechnik GmbH",
    "24.3713": "Devicescape Software, Inc.",
    "12.346865737728": "Weiss Robotics GmbH & Co. KG",
    "24.7886707": "Huawei Technologies Co.,Ltd",
    "12.346855198720": "Transtech DSP",
    "24.3710": "ionSign Oy",
    "24.3463508": "WiBotic Inc.",
    "24.1074120": "Liteon Technology Corporation",
    "24.5823253": "Tecno Mobile Limited",
    "12.123917688373248": "Avionica",
    "24.521": "Shenzhen SED Information Technology Co., Ltd.",
    "12.123917686812672": "Zao Zeo",
    "24.12113127": "Aruba, a Hewlett Packard Enterprise Company",
    "12.346868346880": "Aplex Technology Inc.",
    "20.176189944627200": "Business Support Consultant Co.,Ltd",
    "24.2052": "ICA Inc.",
    "24.2053": "Techno-Holon Corporation",
    "12.123917681049600": "Tattile Srl",
    "12.123917689982976": "Kendrion Mechatronics Center GmbH",
    "24.2050": "Hewlett Packard",
    "24.2051": "Cos Tron",
    "24.49159": "Pinnacle Data Systems, Inc.",
    "24.49158": "Nippon Avionics Co., Ltd.",
    "24.49155": "Globalnet Communications",
    "24.49154": "Sercomm Corporation.",
    "24.49157": "Livingston Enterprises Inc",
    "24.49156": "Japan Business Computer Co.Ltd",
    "12.346858065920": "Vestfold Butikkdata AS",
    "24.49153": "Diatek Patient Managment",
    "24.49152": "Lanoptics, Ltd.",
    "24.2054": "Raonet Systems, Inc.",
    "24.4514103": "ARRIS Group, Inc.",
    "24.12862351": "LG Electronics (Mobile Communications)",
    "24.2055": "Access Devices Limited",
    "24.7106719": "Apple, Inc.",
    "24.11035615": "Displaire Corporation",
    "20.180577513766912": "Shen Zhen Pasun Tech Co.Ltd.",
    "12.123917679251456": "Grupo Epelsa S.L.",
    "24.3942509": "Google, Inc.",
    "12.346854825984": "Real Time Systems",
    "24.1340816": "Tp-Link Technologies Co.,Ltd.",
    "12.346864332800": "Telventy Energia S.A.",
    "24.8950910": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11578794": "Avaya Inc",
    "24.7629449": "Rezolt Corporation",
    "24.13907513": "Texas Instruments",
    "24.8411384": "Motorola Mobility LLC, a Lenovo Company",
    "24.32957": "The Furukawa Electric Co., Ltd",
    "24.32956": "Hitachi Engineering Co., Ltd",
    "24.32955": "Hughes Lan Systems",
    "24.32954": "Specialix (Asia) Pte, Ltd",
    "24.32953": "Arche Technoligies Inc.",
    "24.32952": "Dmg Mori B.U.G. Co., Ltd.",
    "24.32951": "Stellar Computer",
    "24.32950": "Themis corporation",
    "24.32959": "Takaoka Electric Mfg. Co. Ltd.",
    "24.32958": "Aries Research",
    "12.346859257856": "REXXON GmbH",
    "24.2682010": "Texas Instruments",
    "12.346859728896": "Teradici Corporation",
    "12.123917687955456": "Commsignia Ltd.",
    "24.16044082": "Yunnan Ideal Information&Technology.,Ltd",
    "24.11561190": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.8418436": "ALL Winner (Hong Kong) Limited",
    "24.5034401": "Huawei Technologies Co.,Ltd",
    "12.123917677056000": "Martec S.p.A.",
    "24.3450508": "Shine Profit Development Limited",
    "24.2390314": "Ruckus Wireless",
    "24.8946297": "Shenzhen RB-LINK Intelligent Technology Co.Ltd",
    "12.123917679497216": "Edgeware AB",
    "24.8158525": "Huawei Technologies Co.,Ltd",
    "24.8158529": "Jinmuyu Electronics Co., Ltd.",
    "24.5033517": "Danlaw Inc",
    "20.452704862208": "Shenzhen Xiao Bi En Culture Education Technology Co.,Ltd.",
    "24.7661071": "Tecno Mobile Limited",
    "24.822308": "Shenzhen Bilian Electronic Co.，Ltd",
    "12.71297883582464": "Protonic Holland",
    "20.44213334966272": "Shenzhen Gospell Smarthome Electronic Co., Ltd.",
    "20.229631764463616": "Arrowave Technologies Limited",
    "24.6596239": "Zhongshan Readboy Electronics Co.,Ltd",
    "24.4460802": "EDMI Europe Ltd",
    "24.6316876": "D-Link International",
    "24.13407797": "LVS Co.,Ltd.",
    "12.346857381888": "Cambridge Technology, Inc.",
    "24.3704789": "Intel Corporate",
    "24.7616315": "UNINET Co.,Ltd.",
    "24.1611090": "1More",
    "12.346858377216": "Aquarotter A Franke Company",
    "24.11017066": "Texas Instruments",
    "24.34464": "Private",
    "20.145735974649856": "SouXin Corporate",
    "24.34460": "Palo Alto Networks",
    "24.4456464": "Apple, Inc.",
    "20.268902729777152": "A-dec Inc.",
    "24.1347896": "Amazon Technologies Inc.",
    "12.346852585472": "Riverlink Computers, Ltd.",
    "12.123917689835520": "GJD Manufacturing",
    "24.5773015": "Alpsalpine Co,.Ltd",
    "24.15264529": "Texas Instruments",
    "24.8705733": "SolarEdge Technologies",
    "24.10279687": "Thum+Mahr GmbH",
    "24.9724521": "ARRIS Group, Inc.",
    "24.2913184": "Cisco Systems, Inc",
    "12.123917679947776": "Aplex Technology Inc.",
    "12.346857365504": "Real D",
    "20.31477070823424": "DSM Messtechnik GmbH",
    "24.8947988": "All Components Incorporated",
    "12.346868744192": "Simple Solutions",
    "12.123917683490816": "Crde",
    "24.7395633": "Cambridge Industries(Group) Co.,Ltd.",
    "12.346853376000": "Pycon, Inc.",
    "24.9190014": "Universal Electronics, Inc.",
    "12.346853142528": "Movaz Networks, Inc.",
    "12.123917683683328": "Ai-Lynx",
    "24.14451291": "JANUS, spol. s r.o.",
    "24.24423": "TP-Link Corporation Limited",
    "24.15785713": "Wistron Infocomm (Zhongshan) Corporation",
    "24.5030406": "Somfy",
    "24.3672484": "Firefly Integrations",
    "24.5297023": "Yu Fly Mikly Way Science and Technology Co., Ltd.",
    "24.6317053": "Transcend Communication Beijing Co.,Ltd.",
    "12.346858110976": "Kinemetrics, Inc.",
    "20.57194446848000": "Eginity, Inc.",
    "24.13900351": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.13170256": "Apple, Inc.",
    "24.11594865": "Samsung Electronics Co.,Ltd",
    "24.9749476": "Atlas Copco IAS GmbH",
    "24.12618445": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "24.11857721": "Samsung Electronics Co.,Ltd",
    "12.123917676732416": "Robotelf Technologies (Chengdu) Co., Ltd.",
    "12.346864951296": "Procon Electronics",
    "12.123917676376064": "Mobile Devices Unlimited",
    "24.5261918": "Masimo Corporation",
    "24.41142": "Sanritz Automation Co., Ltd.",
    "24.41143": "Cordant, Inc.",
    "24.41140": "Texas Microsystems, Inc.",
    "24.41141": "3H Technology",
    "24.41146": "Patton Electronics Co.",
    "24.41147": "Hilan Gmbh",
    "24.41144": "NetApp",
    "24.41145": "Eagle Technology, Inc.",
    "12.119269482496": "Daylight Solutions, Inc",
    "24.41148": "Viasat, Incorporated",
    "24.41149": "I-Tech Corp.",
    "24.282842": "Qiku Internet Network Scientific (Shenzhen) Co., Ltd",
    "24.4767277": "Symanitron",
    "12.346860687360": "MISCO Refractometer",
    "24.1321938": "Armtel Ltd.",
    "12.123917684400128": "Dofuntech Co.,LTD.",
    "24.5772838": "Avaya Inc",
    "12.346868129792": "Saia-Burgess Controls AG",
    "24.5292657": "Securetech Inc",
    "12.123917679738880": "Hermann Sewerin GmbH",
    "24.13161153": "Jetlun (Shenzhen) Corporation",
    "24.6576301": "China Mobile IOT Company Limited",
    "24.7348028": "Samsung Electronics Co.,Ltd",
    "20.242185282781184": "Pyrexx Technologies GmbH",
    "24.8147786": "Metamako",
    "12.123917688328192": "Atlas Lighting Products",
    "20.9814034874368": "Imperx, Inc",
    "12.123917676965888": "ENTEC Electric & Electronic Co., LTD.",
    "20.58071058481152": "NTX Embedded",
    "24.12851692": "Qualisys AB",
    "12.346852974592": "Advanced Medical Information Technologies, Inc.",
    "12.346858504192": "Identic Ab",
    "12.123917675749376": "Landis Gyr",
    "24.8676310": "Microsoft Corporation",
    "12.123917678854144": "RF-Tuote Oy",
    "24.6950432": "GeoFrenzy, Inc",
    "12.346866737152": "Fr. Sauter AG",
    "24.16047518": "GoPro",
    "12.346855022592": "Norco Industrial Technology Inc",
    "24.13382871": "Gewiss S.P.A.",
    "20.45069246660608": "University of York",
    "24.9201722": "Huawei Technologies Co.,Ltd",
    "24.847590": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917678342144": "Movidius SRL",
    "24.16298461": "Apple, Inc.",
    "12.346860773376": "Dsp Design",
    "12.346852614144": "Avtron Manufacturing Inc.",
    "12.123917681778688": "samwooeleco",
    "12.123917675966464": "Tomahawk Robotics",
    "24.6616823": "Anhui Dynamic Power Co., Ltd.",
    "12.123917691613184": "Centum Adetel Group",
    "20.127654894239744": "Tanjarine",
    "20.84197032067072": "Commsignia, Ltd.",
    "24.1325890": "Realfit(Shenzhen) Intelligent Technology Co., Ltd",
    "24.6068952": "Fujitsu Limited",
    "24.16498": "Applied Innovation",
    "24.16499": "Bass Associates",
    "20.216039223721984": "Watec Co., Ltd.",
    "24.16490": "Kentek Information Systems,Inc",
    "24.16491": "Sysgen",
    "24.16492": "Copernique",
    "24.16493": "Lanco, Inc.",
    "24.16494": "Corollary, Inc.",
    "24.16495": "Sync Research Inc.",
    "24.16496": "Interware Co., Ltd.",
    "24.16497": "Atm Computer Gmbh",
    "24.15263927": "Murata Manufacturing Co., Ltd.",
    "24.14207389": "Ea Display Limited",
    "24.10498713": "Lenovo (Beijing) Co., Ltd.",
    "12.346852528128": "Austco Marketing & Service (USA) ltd.",
    "12.346854809600": "Ateme",
    "24.7089964": "Samsung Electronics Co.,Ltd",
    "24.4480233": "Samsung Electronics Co.,Ltd",
    "24.1061823": "Technicolor CH USA Inc.",
    "12.123917675704320": "KST technology",
    "12.346867834880": "Specialty Microwave Corp",
    "24.13696848": "Texas Instruments",
    "24.11858091": "Apple, Inc.",
    "24.12351173": "Huawei Technologies Co.,Ltd",
    "24.14418840": "LG Innotek",
    "24.14722816": "Verkada Inc",
    "24.37103": "Integrix, Inc.",
    "24.418": "Logical Co., Ltd.",
    "12.123917686300672": "SMTC Corporation",
    "12.346853031936": "Palm, Inc.",
    "24.6297868": "Rramac",
    "24.20973": "LG Innotek",
    "20.229558871654400": "YUAN High-Tech Development Co., Ltd.",
    "24.8680195": "Letv Mobile and Intelligent Information Technology (Beijing) Corporation Ltd.",
    "24.9758": "Qingbang Elec(Sz) Co., Ltd",
    "24.15734936": "Apple, Inc.",
    "24.11572597": "Tp-Link Technologies Co.,Ltd.",
    "24.9759": "SAE Magnetics (H.K.) Ltd.",
    "24.4213460": "Widex A/S",
    "24.9197565": "LVX Western",
    "24.6608061": "Alpsalpine Co,.Ltd",
    "24.10526145": "China Dragon Technology Limited",
    "24.6926": "InoTec GmbH Organisationssysteme",
    "24.6927": "Petratec",
    "24.6924": "Cisco Systems, Inc",
    "24.6925": "Cisco Systems, Inc",
    "24.6922": "Intelligent Distributed Controls Ltd",
    "24.6923": "Phidgets Inc.",
    "24.6920": "Danfoss Drives A/S",
    "24.6921": "Matrix Telecom Pvt. Ltd.",
    "24.9756": "Neovia Inc.",
    "24.6928": "ShenZhen Kang Hui Technology Co.,ltd",
    "24.6929": "D-Link Corporation",
    "12.123917682405376": "Alion Science & Technology",
    "24.9757": "Cop Security System Corp.",
    "24.415": "ReadyNet",
    "24.7882436": "Daehap Hyper-Tech",
    "24.414": "ESS Technology, Inc.",
    "12.123917678452736": "Pavo Tasarim Üreti̇m Ti̇c A.Ş.",
    "24.1109721": "Canoga Perkins Corporation",
    "24.417": "Mag-Tek, Inc.",
    "24.9723920": "Huawei Device Co., Ltd.",
    "24.3187062": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.4985701": "Emplus Technologies",
    "24.1509": "Renishaw PLC",
    "24.2625659": "D-Link International",
    "12.123917676343296": "ROMO Wind A/S",
    "24.1508": "Red Lion Controls Inc.",
    "12.123917681102848": "Software Motor Corp",
    "12.123917682352128": "Sensor Highway Ltd",
    "24.8692020": "Apple, Inc.",
    "24.9755726": "AEV, spol. s r. o.",
    "24.3945216": "Shenzhen Bilian electronic CO.,LTD",
    "24.1050892": "Janome Sewing Machine Co., Ltd.",
    "24.5789": "Cisco Systems, Inc",
    "24.5788": "Cisco Systems, Inc",
    "24.5787": "Alstom Transport",
    "24.1891927": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.5785": "Tonic DVB Marketing Ltd",
    "24.5784": "T&A Mobile Phones",
    "24.5783": "NEC Corporation",
    "24.5782": "QDI Technology (H.K.) Limited",
    "24.5781": "AVC Technology (International) Limited",
    "24.5780": "Sennheiser Communications A/S",
    "24.1501": "Cisco Systems, Inc",
    "24.15758058": "Google, Inc.",
    "24.573344": "Samsung Electronics Co.,Ltd",
    "24.1500": "Cisco Systems, Inc",
    "24.2134410": "Profalux",
    "24.1503": "Electronic Innovation, Inc.",
    "24.5508653": "vivo Mobile Communication Co., Ltd.",
    "12.346857213952": "SLICAN sp. z o.o.",
    "12.123917678399488": "Schneider Electric Motion USA",
    "12.346863468544": "Palladio Systeme GmbH",
    "24.1505": "Trellis Photonics, Ltd.",
    "12.123917681012736": "Olympus NDT Canada",
    "24.1066880": "Huawei Technologies Co.,Ltd",
    "12.346859130880": "EATON FHF Funke + Huster Fernsig GmbH",
    "24.9466432": "Apple, Inc.",
    "24.1507": "LightSand Communications, Inc.",
    "24.263402": "Valens Semiconductor Ltd.",
    "24.1506": "Creativ Network Technologies",
    "12.123917680783360": "Ducommun Inc.",
    "24.320878": "THUB Co., ltd.",
    "12.346859536384": "Smartek d.o.o.",
    "24.4784": "Efore Oyj (Plc)",
    "12.346868518912": "Dr. Neumann elektronik GmbH",
    "24.4786": "Avolites Ltd.",
    "24.4787": "Advance Wireless Technology Corp.",
    "24.4780": "Ontimetek Inc.",
    "24.4781": "Vivavis Ag",
    "24.4782": "HLS HARD-LINE Solutions Inc.",
    "24.4783": "ELPRO Technologies",
    "24.4788": "Work Microwave GmbH",
    "24.4789": "Vialta, Inc.",
    "20.14286627799040": "Antailiye Technology Co.,Ltd",
    "24.585355": "Samsung Electronics Co.,Ltd",
    "24.16002687": "Aruba, a Hewlett Packard Enterprise Company",
    "12.346852761600": "DigitAll World Co., Ltd",
    "24.9457787": "Advanced PANMOBIL Systems GmbH & Co. KG",
    "12.123917678440448": "OAS Sweden AB",
    "12.346855682048": "Vlsip Technologies, Inc",
    "24.5273073": "ARRIS Group, Inc.",
    "24.3839": "Megasolution,Inc.",
    "24.3838": "EndRun Technologies LLC",
    "24.3837": "Fujinon Corporation",
    "24.3836": "JTAG Technologies B.V.",
    "24.3835": "Macey Enterprises",
    "20.48688640557056": "Rheonik Messtechnik GmbH",
    "24.3833": "REA Elektronik GmbH",
    "24.3832": "Sbc Asi",
    "24.3831": "Vulcan Portals Inc",
    "24.3830": "E-TEN Information Systems Co., Ltd.",
    "12.346863722496": "CompuDesigns, Inc.",
    "24.14168596": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.14950163": "IBM Corp",
    "20.220958182866944": "Meiryo Technica Corporation",
    "24.15761158": "Cisco Systems, Inc",
    "24.4438": "Pharos Systems NZ",
    "24.4439": "Oki Electric Industry Co., Ltd.",
    "24.4436": "Webpro Technologies Inc.",
    "24.4437": "Sevis Systems",
    "24.4434": "Eidsvoll Electronics AS",
    "24.4435": "Trident Tek, Inc.",
    "24.4432": "Belkin Corporation",
    "24.4433": "Mykotronx",
    "24.4430": "690885 Ontario Inc.",
    "24.4431": "US Digital Television, Inc",
    "24.987": "Apogee Electronics Corp.",
    "12.346866167808": "Infinition Inc.",
    "24.985": "Secheron SA",
    "24.984": "iMPath Networks, Inc.",
    "24.983": "NextNet Wireless, Inc.",
    "24.844004": "Plantronics, Inc.",
    "24.981": "Advanced Communications Co., Ltd.",
    "24.980": "Alloptic, Inc.",
    "24.12593163": "Honor Device Co., Ltd.",
    "12.346858233856": "Buyang Electronics Industrial Co., Ltd.",
    "24.588882": "OpenXS BV",
    "24.15224651": "Shenzhen Delos Electronic Co., Ltd",
    "24.989": "Comark Interactive Solutions",
    "24.988": "Lexar Media, Inc.",
    "24.3789": "Skov A/S",
    "24.3788": "Tableau, LLC",
    "24.8942934": "Cisco Systems, Inc",
    "24.3781": "Digital Multitools Inc",
    "24.3780": "Iskra Transmission d.d.",
    "24.3783": "Motorola Korea",
    "24.3782": "Asix Electronics Corp.",
    "24.3785": "YOKO Technology Corp.",
    "24.3784": "Zoran Corporation",
    "24.3787": "VineSys Technology",
    "24.3786": "WTSS Inc",
    "20.146101703278592": "Hangzhou Uni-Ubi Co.,Ltd.",
    "12.346868088832": "Miris AB",
    "24.13896474": "HTC Corporation",
    "24.1064220": "Intel Corporate",
    "24.60426": "Xiaomi Communications Co Ltd",
    "24.7200": "CLB Benelux",
    "24.7201": "Nucsafe Inc.",
    "24.7202": "Aeris Elettronica s.r.l.",
    "24.7203": "Dell Inc.",
    "24.7204": "Formosa Wireless Systems Corp.",
    "24.7205": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7206": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7207": "Sunell Electronics Co.",
    "24.7208": "Sphairon Technologies GmbH",
    "24.7209": "Core Digital Electronics Co., Ltd",
    "24.4743798": "Huawei Technologies Co.,Ltd",
    "24.53281": "Regent Electronics Corp.",
    "20.216039233159168": "INOTEC Sicherheitstechnik GmbH",
    "24.11021241": "Samsung Electronics Co.,Ltd",
    "24.7632935": "Dell Inc.",
    "24.7087270": "vivo Mobile Communication Co., Ltd.",
    "24.22465": "LG Electronics (Mobile Communications)",
    "12.346856947712": "Cloanto Corporation",
    "24.57083": "Cisco Systems, Inc",
    "12.123917679419392": "Henrich Electronics Corporation",
    "12.123917689966592": "Momentum Data Systems",
    "24.6295415": "SilverPlus, Inc",
    "24.9209": "Double-Take Software, INC.",
    "24.6034913": "Aidc Technology (S) Pte Ltd",
    "24.8407840": "Ventilation Control",
    "24.49319": "Seel Ltd.",
    "24.49318": "Exicom Australia Pty. Ltd",
    "24.49317": "Dickens Data Systems",
    "24.49316": "Unigraf Oy",
    "24.49315": "Dual Enterprises Corporation",
    "24.49314": "Intermedium A/S",
    "24.49313": "Tokyo Denshi Sekei Co.",
    "24.49312": "Advance Micro Research, Inc.",
    "24.49311": "Quanta Computer Inc.",
    "24.49310": "Cache Computers, Inc.",
    "24.6569612": "Huawei Technologies Co.,Ltd",
    "12.123917676404736": "Rivercity Innovations Ltd.",
    "24.8928": "Atlantic Software Technologies S.r.L.",
    "24.8929": "ZORT Labs, LLC.",
    "24.11069816": "Huawei Device Co., Ltd.",
    "24.53501": "Optima Tele.Com, Inc.",
    "24.53500": "Granite Microsystems",
    "24.53503": "Cisco Systems, Inc",
    "24.53502": "Astral Point",
    "24.8920": "Shenzhen GST Security and Safety Technology Limited",
    "24.8921": "Fortex Industrial Ltd.",
    "24.8922": "Anatek, Llc",
    "24.8923": "Translogic Corporation",
    "24.10009374": "BYD Precision Manufacture Company Ltd.",
    "20.198611827687424": "Kona I",
    "12.123917681680384": "Season Electronics Ltd",
    "12.123917676556288": "Vistec Electron Beam GmbH",
    "20.176726062661632": "WiteRiver Technology LLC",
    "24.8696": "Enseo, Inc.",
    "24.8697": "WIRECOM Technologies",
    "24.8694": "Oracle Corporation",
    "24.8695": "HPN Supply Chain",
    "24.8692": "INRange Systems, Inc",
    "24.8693": "Western Engravers Supply, Inc.",
    "24.8690": "EASY3CALL Technology Limited",
    "24.8691": "Si14 SpA",
    "24.4739433": "Samsung Electronics Co.,Ltd",
    "24.8698": "A4SP Technologies Ltd.",
    "24.8699": "LG Electronics (Mobile Communications)",
    "24.13372634": "Miljovakt AS",
    "20.171812081106944": "Symfun Telecom Ltd",
    "24.530458": "zte corporation",
    "24.2070": "Bluelon ApS",
    "24.2071": "EmergeCore Networks LLC",
    "24.2072": "Pixelworks, Inc.",
    "24.2073": "Banksys",
    "24.2074": "Sanrad Intelligence Storage Communications (2000) Ltd.",
    "24.2075": "Windigo Systems",
    "24.6559": "Dkt A/S",
    "24.6558": "Nifty",
    "24.6557": "Vizio, Inc",
    "24.6556": "Ctring",
    "24.815913": "Sercomm Corporation.",
    "24.6554": "Edo-Evi",
    "24.6553": "Fujitsu Technology Solutions GmbH",
    "24.6552": "Sato Corporation",
    "24.6551": "Soft Device Sdn Bhd",
    "24.6550": "TurboChef Technologies Inc.",
    "24.15240800": "NSD Corporation",
    "24.8966513": "Beghelli S.P.A",
    "24.6302165": "DAVOLINK Inc.",
    "12.71297883766784": "Axxess Identification Ltd",
    "24.11330069": "Huawei Technologies Co.,Ltd",
    "12.346859446272": "Laser Electronics Ltd",
    "12.346866700288": "A&A General Srl",
    "24.2120954": "Yangzhou ChangLian Network Technology Co,ltd.",
    "24.5540062": "Cisco Systems, Inc",
    "24.1856826": "QianGua Corp.",
    "24.322021": "Pjrc.Com, Llc",
    "24.1625224": "Hitachi Johnson Controls Air",
    "20.194600863006720": "Taian Yuqi Communication Technology Co., Ltd",
    "20.58071047995392": "Tascent, Inc.",
    "24.6352838": "Shenzhen Chima Technologies Co Limited",
    "12.123917676392448": "Schneider Electric Motion USA",
    "24.11829695": "Meta-Networks, LLC",
    "24.15769538": "Ubiquiti Networks Inc.",
    "20.220327866007552": "TrackMan",
    "12.123917686726656": "Green Access Ltd",
    "24.7652897": "Zhejiang Hite Renewable Energy Co.,LTD",
    "12.123917683048448": "Verity Studios AG",
    "24.9620": "Eurodesign BG LTD",
    "24.2695": "Integrated Micromachines Inc.",
    "24.2696": "InCypher S.A.",
    "12.346854019072": "Flander Oy",
    "24.2690": "Tatsuta System Electronics Co.,Ltd.",
    "24.2691": "Salto Systems S.L.",
    "24.2692": "Rainsun Enterprise Co., Ltd.",
    "24.2693": "PLAT'C2,Inc",
    "12.346866130944": "F+D Feinwerk- und Drucktechnik GmbH",
    "24.2699": "Cisco Systems, Inc",
    "24.20604": "Videocon Ag",
    "12.346865557504": "Kdt",
    "24.4737769": "Hewlett Packard Enterprise",
    "24.479": "ISDN Communications, Ltd.",
    "12.346859872256": "Gaisler Research AB",
    "24.477": "Avail Networks",
    "24.476": "Activetelco",
    "24.475": "Freecom Technologies GmbH",
    "24.474": "WINCOMM Corporation",
    "24.473": "Sigma, Inc.",
    "24.2381934": "zte corporation",
    "24.471": "F5 Networks, Inc.",
    "24.470": "manroland AG",
    "24.8167101": "Espressif Inc.",
    "24.1529": "TOA Corporation",
    "24.1528": "Real Time Access, Inc.",
    "24.1527": "Analog Devices, Inc.",
    "24.1526": "Young Chang Co. Ltd.",
    "24.1525": "Geospace Technologies",
    "24.1524": "System Base Co., Ltd.",
    "24.1523": "Webyn",
    "24.1522": "Power R, Inc.",
    "24.1521": "Vrcom, Inc.",
    "24.1520": "Satec",
    "12.346862776320": "Flight Data Systems Pty Ltd",
    "12.346856468480": "LogiM GmbH Software und Entwicklung",
    "24.39634": "Cisco Systems, Inc",
    "12.123917678739456": "Tex Computer Srl",
    "20.255191452483584": "Acopian Technical Company",
    "24.9743326": "OMX Technology - FZE",
    "24.7402016": "Actiontec Electronics, Inc",
    "24.7126965": "Noon Technology Co., Ltd",
    "20.167594820632576": "my-PV GmbH",
    "20.181256789688320": "Malldon Technology Limited",
    "24.3937228": "Cisco Systems, Inc",
    "24.13109929": "Quanta Computer Inc.",
    "24.45107": "OAO \"Izhevskiy radiozavod\"",
    "24.2885155": "Win Leader Inc.",
    "24.1099295": "Huawei Technologies Co.,Ltd",
    "24.2680783": "Apple, Inc.",
    "20.141720409866240": "Nanjing LILO Technology Co. Ltd.",
    "24.15473597": "Silicon Laboratories",
    "20.154039759142912": "DataRemote Inc.",
    "24.6317217": "RADiflow Ltd.",
    "20.220515728883712": "Advanced Micro Controls Inc.",
    "12.123917681868800": "Nd Meter",
    "20.212188455239680": "Dongmengling",
    "12.123917675302912": "APP Engineering, Inc.",
    "12.123917681545216": "Topic Embedded Products B.V.",
    "24.13955159": "Nintendo Co.,Ltd",
    "24.5266118": "Guangdong Super Telecom Co.,Ltd.",
    "20.220092025536512": "Fungible, Inc.",
    "12.123917679697920": "GlooVir Inc.",
    "12.123917677707264": "RCH Vietnam Limited Liability Company",
    "12.346854121472": "ReliOn Inc.",
    "24.9973564": "Sichuan Changhong Electric Ltd.",
    "24.1369828": "Tp-Link Technologies Co.,Ltd.",
    "24.4545": "4P Mobile Data Processing",
    "24.3452388": "Shanghai Chint Power Systems Co., Ltd.",
    "12.346853150720": "Steelcase Inc.",
    "24.20602": "Xpeed, Inc.",
    "12.346857431040": "7+ Kft",
    "20.101728423772160": "Express LUCK Industrial Ltd.",
    "12.346863435776": "DS PRO Audio Ltda",
    "24.15003980": "Private",
    "24.6323917": "Huawei Technologies Co.,Ltd",
    "24.6816309": "Konten Networks Inc.",
    "24.8435285": "Intel Corporate",
    "24.4847": "OneAccess SA",
    "24.4846": "Sony Mobile Communications Inc",
    "24.4845": "AVG Advanced Technologies",
    "12.346854821888": "Rototype S.p.A.",
    "24.4843": "PDH Solutions, LLC",
    "24.4842": "Trane",
    "24.4841": "Abbey Systems Ltd",
    "24.4840": "Fraunhofer IMS",
    "24.8173": "Tecan Systems Inc.",
    "24.8172": "Synapse Électronique",
    "24.8171": "Trio Datacom Pty Ltd",
    "24.8170": "Applied Media Technologies Corporation",
    "24.8177": "Paradox Hellas S.A.",
    "24.8176": "Audio Partnership",
    "24.8175": "Shinsei Industries Co.,Ltd",
    "24.8174": "ubisys technologies GmbH",
    "20.181256781299712": "T2T System",
    "24.8153907": "Tenyu Tech Co. Ltd.",
    "12.123917675282432": "Don Electronics Ltd",
    "24.3942101": "Sotera Wireless",
    "24.5263068": "Ping Communication",
    "12.346864373760": "InterCel Pty Ltd",
    "12.346859188224": "4RF Communications Ltd",
    "24.4132": "Nagoya Electric Works Co., Ltd",
    "24.10806043": "Nokia",
    "12.123917687451648": "H2O-Yug Llc",
    "24.11044787": "Samsung Electronics Co.,Ltd",
    "20.146101713764352": "Aimtron Corporation",
    "24.12865055": "BlueN",
    "12.119269670912": "Luxcon System Limited",
    "24.4134": "Accelerated Networks, Inc.",
    "24.8669": "Northstar Systems Corp",
    "24.8482": "Chip-pro Ltd.",
    "24.10539969": "Hewlett Packard",
    "12.71297884037120": "Elan Systems",
    "24.5799521": "Qingdao Hisense Communications Co.,Ltd.",
    "24.12858988": "LG Innotek",
    "12.346860683264": "TESCAN, s.r.o.",
    "24.5776757": "Huawei Technologies Co.,Ltd",
    "12.346863943680": "JanasCard",
    "24.15747946": "Scientific Production Association Siberian Arsenal, Ltd.",
    "12.346852556800": "Private",
    "12.346855669760": "Alcea",
    "12.346866282496": "Fr.SauterAG",
    "24.3150052": "Apple, Inc.",
    "20.141720399380480": "Guangzhou Pinzhong Electronic Technology CO., LTD",
    "24.13920442": "Shenzhen Sundray Technologies Company Limited",
    "24.3167725": "Trex Network LLC",
    "24.325944": "Huawei Technologies Co.,Ltd",
    "24.16268949": "Huawei Technologies Co.,Ltd",
    "20.132855980097536": "Xiamen Cheerzing IOT Technology Co.,Ltd.",
    "24.10534821": "Coresys",
    "24.9750447": "Raylios Technology",
    "24.33273": "Texas Instruments",
    "12.346863656960": "Energid",
    "20.114975481790464": "Spraying Systems Co.",
    "24.6296203": "Canon Inc.",
    "24.15778863": "Huawei Device Co., Ltd.",
    "24.3675952": "Ulrich Lippert GmbH & Co KG",
    "12.346867294208": "Power-One Italia S.p.A",
    "24.1607226": "Digital Art System",
    "24.7902699": "Itel Mobile Limited",
    "12.123917686947840": "Osung Lst Co.,Ltd.",
    "12.346855583744": "ABB Xiamen Switchgear Co. Ltd.",
    "24.10231773": "Accupix Inc.",
    "12.119269928960": "Evondos Oy",
    "20.211776060784640": "Continental Automotive Component Malaysia Sdn.Bhd.",
    "24.12370280": "DinBox Sverige AB",
    "12.123917675466752": "Fsr, Inc.",
    "12.119269937152": "Tierra Japan Co.,Ltd",
    "24.3148700": "Delta Mobile Systems, Inc.",
    "24.5510009": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9971586": "IEEE Registration Authority",
    "20.74781673127936": "Atari, Inc.",
    "12.123917689995264": "IAI-Israel Aerospace Industries MBT",
    "24.3210475": "LG Electronics (Mobile Communications)",
    "24.7623805": "Cisco SPVTG",
    "24.1617823": "DIGITRONIC Automationsanlagen GmbH",
    "24.2385055": "Huawei Device Co., Ltd.",
    "24.10801300": "Vivavis Ag",
    "24.11842863": "Hewlett Packard",
    "24.15004648": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "12.346867007488": "Teplovodokhran",
    "24.2380735": "Enernet",
    "24.15004643": "Shanghai iComhome Co.,Ltd.",
    "24.16284538": "ARRIS Group, Inc.",
    "12.346867544064": "Warp9 Tech Design, Inc.",
    "24.7097990": "Third Millennium Systems Ltd.",
    "24.8185195": "ESEN Optoelectronics Technology Co.,Ltd.",
    "12.346859438080": "Stanley Security Solutions, Inc.",
    "24.16054609": "Apple, Inc.",
    "24.14960346": "Xiaomi Communications Co Ltd",
    "24.3410752": "Mios Elettronica Srl",
    "20.167594808049664": "Shenzhen MALATA Mobile Communication Co.,LTD",
    "20.141720410914816": "EVER Sp. z o.o.",
    "12.123917680467968": "Larimart Spa",
    "24.3420902": "Panasonic Industrial Devices Europe GmbH",
    "24.808225": "Axiros GmbH",
    "20.154238426546176": "AU Optronics Corporation",
    "24.13640185": "Ubiquiti Networks Inc.",
    "24.12618155": "ARRIS Group, Inc.",
    "12.123917677158400": "Amrehn & Partner EDV-Service GmbH",
    "20.162752336232448": "Neutrik AG",
    "12.123917682421760": "Coloet S.r.l.",
    "20.45069254000640": "T-Radio AS",
    "24.10523754": "Shenzhen XGTEC Technology Co,.Ltd.",
    "20.229384972664832": "Shenzhen eloT Technology Co.,Ltd",
    "20.137215111856128": "Heyuan Yongyida Technology Holdings Co.,Ltd.",
    "12.346861969408": "TechnoAP",
    "12.71297883815936": "Keyware Solutions Inc.",
    "24.2403201": "Apple, Inc.",
    "24.1606591": "Arcadyan Technology Corporation",
    "12.123917686308864": "Heartland.Data Inc.",
    "12.123917687996416": "SYS TEC electronic GmbH",
    "20.48835649863680": "Logic IO Aps",
    "20.127658784456704": "Broadcast Wireless Systems Ltd",
    "24.7647244": "Network Video Technologies, Inc",
    "24.16037307": "Ceragon Networks",
    "24.1875659": "Beijing Nari Smartchip Microelectronics Company Limited",
    "20.83999501320192": "Wuhan MoreQuick Network Technology Co., Ltd.",
    "20.189549212860416": "Pfdc Elancyl",
    "20.31657731031040": "eSight Corporation",
    "24.4860": "PLANET System Co.,LTD",
    "12.346866044928": "Tellabs Operations Inc.",
    "24.2407960": "Dadoutek Company Limited",
    "12.123917689991168": "Delta",
    "20.62335518507008": "Beijing Memblaze Technology Co Ltd",
    "12.346865025024": "Beijing Zhongherongzhi Elec.&Tech.Co.,Ltd.",
    "24.9759097": "Liteon Technology Corporation",
    "24.4475073": "Hewlett Packard Enterprise",
    "24.11848937": "Prosyst",
    "12.123917685276672": "Zaxcom Inc",
    "12.123917681844224": "Access Protocol Pty Ltd",
    "24.15787902": "Samsung Electronics Co.,Ltd",
    "24.4681": "Delta Elettronica S.p.A.",
    "24.9716161": "Mini-Cam Limited",
    "12.123917685596160": "Nueon - The COR",
    "24.10023603": "Hewlett Packard Enterprise",
    "20.163808560545792": "Inaxsys Security Systems inc.",
    "24.14440167": "Apple, Inc.",
    "24.8397432": "China Mobile IOT Company Limited",
    "24.4682": "Dedicated Devices, Inc.",
    "20.154039762288640": "Elco(Tianjin)Electronics Co.,Ltd.",
    "24.8391174": "COMTEC co.,ltd",
    "12.123917682573312": "Gryphon Secure Inc",
    "12.123917681303552": "Quantum Opus, LLC",
    "20.264809150939136": "Anord Mardix (USA) Inc.",
    "12.123917684051968": "Tiama",
    "24.4245535": "Shenzhen Xinguodu Technology Co., Ltd.",
    "24.10020853": "Huawei Technologies Co.,Ltd",
    "24.10020852": "Hewlett Packard",
    "20.127937502248960": "Yekani Manufacturing PTY Ltd",
    "24.14964544": "ARRIS Group, Inc.",
    "24.13652322": "Sony Mobile Communications Inc",
    "24.6678": "Nokia Danmark A/S",
    "24.2158120": "Apple, Inc.",
    "24.6679": "Teak Technologies, Inc.",
    "12.346862743552": "Winners Satellite Electronics Corp.",
    "12.123917691232256": "Senso2Me",
    "24.8655298": "IEEE Registration Authority",
    "24.6676": "Xin Hua Control Engineering Co.,Ltd.",
    "24.15214805": "Cots Technology",
    "24.6677": "gemalto e-Payment",
    "24.6674": "Essilor",
    "24.6675": "Wanlida Group Co., LTD",
    "24.6672": "Lucent Trans Electronics Co.,Ltd",
    "12.346866814976": "Jenny Science AG",
    "24.5567926": "Oriental Pacific International Limited",
    "12.346868248576": "Reach Technologies Inc.",
    "24.6670": "Cheng Uei Precision Industry Co.,Ltd",
    "24.6671": "Sistemas Avanzados de Control, S.A.",
    "24.5259705": "I-O Data Device,Inc.",
    "24.12378725": "Huawei Technologies Co.,Ltd",
    "12.123917687468032": "Saratov Electrounit Production Plant named after Sergo Ordzhonikidze, OJSC",
    "24.2384590": "Aruba, a Hewlett Packard Enterprise Company",
    "24.12639338": "Qingdao Eastsoft Communication Technology Co.,LTD",
    "12.123917684150272": "Toptech Systems, Inc.",
    "12.123917676437504": "Season Electronics Ltd",
    "20.238070231203840": "Digital Concepts",
    "20.52821126807552": "Bronkhorst High-Tech BV",
    "12.346863935488": "Nikon Systems Inc.",
    "12.123917683511296": "DEUTA-WERKE GmbH",
    "24.8448519": "Tp-Link Technologies Co.,Ltd.",
    "24.11295470": "China Mobile Group Device Co.,Ltd.",
    "12.123917690740736": "ABECO Industrie Computer GmbH",
    "12.346853851136": "Artec Design",
    "24.13414682": "Samsung Electronics Co.,Ltd",
    "24.7124980": "Samsung Electronics Co.,Ltd",
    "20.48543317360640": "shenzhen Clever Electronic Co., Ltd.",
    "12.346865328128": "Sdd Itg",
    "24.10539228": "Amazon Technologies Inc.",
    "12.123917685116928": "Softel",
    "24.4985984": "Noregon Systems, Inc",
    "20.145727919489024": "Shenzhen Intellifusion Technologies Co., Ltd.",
    "24.3955": "RS Automation Co., Ltd",
    "24.15738669": "Amazon Technologies Inc.",
    "24.1023": "Microsoft Corporation",
    "12.123917677969408": "CT Company",
    "24.11559367": "Ericsson-LG Enterprise",
    "24.7092385": "Ubee Interactive Co., Limited",
    "12.119269466112": "MindMade Sp. z o.o.",
    "24.14476037": "ZAO \"NPK Rotek\"",
    "24.13913244": "Shenzhen YOOBAO Technology Co.Ltd",
    "24.14433412": "Ticom Geomatics, Inc.",
    "24.4252630": "Aruba, a Hewlett Packard Enterprise Company",
    "20.93065831251968": "Assem Technology Co.,Ltd.",
    "24.1861464": "Huawei Technologies Co.,Ltd",
    "24.2413260": "SmartDrive Systems Inc.",
    "24.16276605": "Shenzhen Honesty Electronics Co.,Ltd.",
    "12.123917677834240": "HAS co.,ltd.",
    "24.4775708": "Wuhan NEC Fibre Optic Communications industry Co. Ltd",
    "24.3977599": "Wattwatchers Pty Ld",
    "20.31368134262784": "Guangzhou Video-Star Electronics Co.,Ltd.",
    "20.136646464897024": "BungBungame Inc",
    "24.7958": "Wistron Corporation",
    "24.286372": "Shenzhen Netis Technology Co.,Ltd",
    "24.7954": "Juniper Networks",
    "24.845577": "Fox Crypto B.V.",
    "24.2364423": "Xiamen Sigmastar Technology Ltd.",
    "24.7957": "Bioscrypt Inc",
    "24.7950": "Japan Kyastem Co., Ltd",
    "24.7951": "Select Engineered Systems",
    "24.7952": "Toledo Do Brasil Industria De Balancas Ltda",
    "24.7953": "Openmoko, Inc.",
    "24.7354961": "Xiaomi Communications Co Ltd",
    "24.10269190": "Procentec",
    "24.10548196": "Alcatel-Lucent IPD",
    "24.1322379": "Incipio Technologies, Inc",
    "20.88914840780800": "Ivativ, Inc",
    "20.35758079475712": "H3 Industries, Inc.",
    "24.8131021": "QianTang Technology",
    "24.7364266": "Action Target, Inc.",
    "24.2945539": "Emko Elektronik San Ve Tic As",
    "20.224420105814016": "ART&CORE Inc",
    "24.10279486": "Intel Corporate",
    "24.5804371": "China Mobile IOT Company Limited",
    "24.6833993": "Texas Instruments",
    "24.5828302": "Icon Time Systems",
    "20.140782585511936": "Golana Technology (Shenzhen) Co., Ltd.",
    "12.346865102848": "High Speed Design, Inc.",
    "24.14993126": "Mophie, LLC",
    "12.123917685735424": "Wave Music Ltd",
    "12.346861469696": "Shenzhen Pennda Technologies Co., Ltd.",
    "24.4783523": "Xiaomi Communications Co Ltd",
    "20.171977806446592": "EA Technology Ltd",
    "20.102196707328000": "Access IS",
    "24.8408688": "Samsung Electronics Co.,Ltd",
    "12.119269318656": "Energotechnica OOO NPP Ltd",
    "20.110162572476416": "Shenzhen Emeet Intelligent Technology Co., Ltd.",
    "12.346859429888": "Interactive Electronic Systems",
    "12.123917677002752": "Gogo BA",
    "12.346867683328": "Peek Traffic Corporation",
    "24.819299": "Tp-Link Technologies Co.,Ltd.",
    "12.123917683179520": "Mitsubishi Heavy Industries Thermal Systems, Ltd.",
    "24.16516679": "Cortland Research, LLC",
    "24.2666250": "Sirius XM Radio Inc",
    "24.5798889": "AirPro Technology India Pvt. Ltd",
    "24.14220344": "Giantec Semiconductor Inc",
    "24.6584209": "Samsung Electronics Co.,Ltd",
    "24.6056576": "Zyxel Communications Corporation",
    "24.790887": "Cisco Systems, Inc",
    "24.6056573": "Kentkart Ege Elektronik San. Ve Tic. Ltd. Sti.",
    "20.176094529454080": "Precision Planting, LLC.",
    "12.346868453376": "CSA Engineering AG",
    "24.12340089": "SensingTek",
    "24.1357816": "Technicolor CH USA Inc.",
    "24.14724845": "Loenk",
    "24.6067798": "AzureWave Technology Inc.",
    "24.16265061": "Huawei Device Co., Ltd.",
    "24.8681015": "Huawei Technologies Co.,Ltd",
    "24.6830878": "Countwise LTD",
    "24.16265066": "Itel Mobile Limited",
    "24.1857278": "Motorola Mobility LLC, a Lenovo Company",
    "12.346868297728": "Triax A/S",
    "24.8405628": "Adolf Tedsen GmbH & Co. KG",
    "12.119269679104": "Two Dimensional Instruments, LLC",
    "24.4772047": "Universal Electronics, Inc.",
    "20.22739791380480": "Iflytek Co.,Ltd.",
    "24.545344": "EbV Elektronikbau- und Vertriebs GmbH",
    "12.346867716096": "RFL Electronics, Inc.",
    "12.123917689806848": "Umano Medical Inc.",
    "24.16261765": "Cypress Technology CO., LTD.",
    "20.181256780251136": "General Electric Company",
    "24.5080": "Princeton Instruments",
    "24.5081": "Matrix Product Development, Inc.",
    "24.5082": "Diskware Co., Ltd",
    "24.5083": "SHOEI Electric Co.,Ltd",
    "24.5084": "Ibtek Inc.",
    "24.5085": "Abbott Diagnostics",
    "24.5086": "Adapt4, LLC",
    "24.5087": "Ryvor Corp.",
    "24.5088": "Murata Manufacturing Co., Ltd.",
    "24.5089": "Iprobe AB",
    "24.6326705": "Key Digital Systems",
    "24.53369": "Cisco Systems, Inc",
    "24.53368": "Eltex of Sweden AB",
    "24.53361": "Echelon Corp.",
    "24.53360": "Long Well Electronics Corp.",
    "24.53363": "Acn Advanced Communications",
    "24.53362": "Broadlogic",
    "24.53365": "Alaris Medical Systems, Inc.",
    "24.53364": "Taqua Systems, Inc.",
    "24.53367": "Lucent Technologies",
    "24.53366": "Bank of America",
    "24.5001964": "SOLARWATT GmbH",
    "12.346861940736": "Grupo Epelsa S.L.",
    "24.11582881": "Zaklad Elektroniczny Sims",
    "24.7925580": "Unetconvergence Co., Ltd.",
    "24.1595702": "Xiaomi Communications Co Ltd",
    "24.8816": "ABK North America, LLC",
    "24.16550807": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.12846101": "Netgear",
    "20.84372899233792": "Dongguan SmartAction Technology Co.,Ltd",
    "24.10516305": "ASMedia Technology Inc.",
    "24.1326887": "Noccela Oy",
    "12.123917675237376": "DoWoo Digitech",
    "24.10762678": "Shenzhen Peizhe Microelectronics Co.Ltd",
    "24.41870": "Cisco Systems, Inc",
    "24.5624": "Kingtronics Industrial Co. Ltd.",
    "24.5625": "Cisco Systems, Inc",
    "24.5626": "Cisco Systems, Inc",
    "12.346864975872": "Computer Automation Technology Inc",
    "24.5620": "Eventide",
    "24.5621": "Sustainable Energy Systems",
    "24.5622": "Science And Engineering Services, Inc.",
    "24.5623": "Wintecronics Ltd.",
    "24.10015537": "Shenzhen Bolutek Technology Co.,Ltd.",
    "24.5628": "Littelfuse Startco",
    "24.5629": "Complete Media Systems",
    "24.7648370": "Ciesse",
    "12.123917685223424": "Xiris Automation Inc.",
    "24.10778758": "Avaya Inc",
    "24.3955380": "Google, Inc.",
    "12.123917677891584": "Avlinkpro",
    "24.10526480": "Shenzhen Bilian Electronic Co.，Ltd",
    "12.119269486592": "ELTRADE Ltd",
    "12.123917688020992": "Private",
    "20.89058717990912": "IPC Global",
    "20.198151823687680": "Alturna Networks",
    "12.346864140288": "Drinelec",
    "24.3732030": "Microsoft Mobile Oy",
    "24.3668": "AlphaCell Wireless Ltd.",
    "24.3669": "Auvitran",
    "24.3666": "Optium Corporation",
    "24.3667": "Av Tech Corporation",
    "24.3664": "Thomson Telecom Belgium",
    "24.3665": "tecna elettronica srl",
    "24.3662": "Waveplus Technology Co., Ltd.",
    "24.3663": "Trajet GmbH",
    "24.3660": "Bermai Inc.",
    "24.3661": "Numesa Inc.",
    "24.14459820": "RBcloudtech",
    "24.5246965": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.16019364": "Physik Instrumente GmbH & Co. KG",
    "24.1896545": "SCAPS GmbH",
    "12.346856665088": "Solace Systems, Inc.",
    "20.52819392462848": "Beijing Netswift Technology Co.,Ltd.",
    "12.123917688344576": "Jsc «Satis-Tl-94»",
    "12.71297885040640": "Wexiodisk AB",
    "24.15518512": "Huawei Technologies Co.,Ltd",
    "24.13661630": "Reno A&E",
    "24.14736639": "Xiaomi Communications Co Ltd",
    "20.127654891094016": "Heptagon Systems PTY. LTD.",
    "12.346860064768": "Rosslare Enterprises Limited",
    "12.123917690892288": "NIRIT- Xinwei Telecom Technology Co., Ltd.",
    "24.4225819": "Mettle Networks Inc.",
    "12.346855608320": "SSI Schaefer Peem",
    "24.9993": "Nintendo Co., Ltd.",
    "24.9990": "Yoisys",
    "24.9991": "Lift Complex DS, JSC",
    "24.9996": "Cisco Systems, Inc",
    "24.9997": "Cisco Systems, Inc",
    "24.9994": "Iee S.A.",
    "24.9995": "Adura Technologies",
    "24.9998": "Intel Corporate",
    "24.9999": "Envisionnovation Inc",
    "12.123917679968256": "Tiama",
    "24.9490287": "Dlogixs Co Ltd",
    "24.1327043": "SnapAV",
    "20.180680386412544": "Williamson Corporation",
    "24.15783855": "IEEE Registration Authority",
    "24.5506699": "Ningbo Deli Kebei Technology Co.LTD",
    "24.534167": "Zyxel Communications Corporation",
    "24.8920993": "Apple, Inc.",
    "24.7875625": "New H3C Technologies Co., Ltd",
    "20.194600857763840": "TAE HYUNG Industrial Electronics Co., Ltd.",
    "12.346861182976": "Neptune Technology Group Inc.",
    "24.6080471": "Azroad Technology Company Limited",
    "24.3191140": "Power Electronics International Inc.",
    "12.123917684162560": "GSI GeoSolutions International Ltd",
    "24.5041659": "ASUSTek COMPUTER INC.",
    "12.119270014976": "Digital Loggers, Inc.",
    "24.2885772": "HUMAX Co., Ltd.",
    "24.6830057": "Amazon Technologies Inc.",
    "20.154314896048128": "Genis",
    "12.71297883734016": "Jel System Co., Ltd.",
    "12.346864934912": "Dismuntel Sal",
    "24.16293133": "Control Technology Inc.",
    "24.7880932": "Samsung Electronics Co.,Ltd",
    "24.8444604": "Megafone Limited",
    "24.9217441": "Oregano Systems - Design & Consulting GmbH",
    "24.20659": "Voiceboard Corporation",
    "24.20658": "BRODEL GmbH",
    "12.346853842944": "Azio Technology Co.",
    "24.20653": "CommUnique Wireless Corp.",
    "24.20652": "Maple Computer Corporation",
    "24.20651": "NALTEC, Inc.",
    "24.20650": "Konica Minolta Holdings, Inc.",
    "24.20657": "Giddings & Lewis",
    "24.20656": "Technology Atlanta Corporation",
    "24.20655": "Intergon, Inc.",
    "24.20654": "FDK Co., Ltd",
    "24.9452690": "Apple, Inc.",
    "12.346857984000": "Zao",
    "12.123917687238656": "Axotec Technologies GmbH",
    "24.6583895": "Innovative Security Designs",
    "12.71297883639808": "Tattile Srl",
    "12.123917687382016": "Yawata Electric Industrial Co.,Ltd.",
    "24.6845009": "Ubiquiti Networks Inc.",
    "24.10772881": "NamJunSa",
    "24.6825753": "Microchip Technology Inc.",
    "24.4466851": "Jiangsu fulian Communication Technology Co., Ltd.",
    "12.123917683335168": "Tci",
    "24.3716563": "SecuWorks",
    "20.193649361027072": "COMOTA Co., Ltd.",
    "24.10275545": "Nokia Corporation",
    "12.346862669824": "Adgil Design Inc.",
    "20.79602621874176": "DefPower Ltd",
    "24.8719": "MoCA (Multimedia over Coax Alliance)",
    "24.8718": "Indigo Security Co., Ltd.",
    "24.8713": "Omron Healthcare Co., Ltd",
    "24.8712": "Certicom Corp",
    "24.8711": "Inteno Broadband Technology AB",
    "24.8710": "Cyberdyne Inc.",
    "24.8717": "Cisco Systems, Inc",
    "24.8716": "Cisco Systems, Inc",
    "24.8715": "National Source Coding Center",
    "24.8714": "OnLive, Inc",
    "24.3670296": "ULVAC,Inc.",
    "20.31657726836736": "Shin-Yosha Corporation",
    "20.61822320246784": "Chengdu Krosslan Technology Inc.",
    "12.346857607168": "Orion Technologies,LLC",
    "12.346856546304": "Oelmann Elektronik GmbH",
    "12.123917683712000": "Chromalox, Inc.",
    "12.123917685518336": "IoTrek Technology Private Limited",
    "12.346868355072": "RF Code, Inc",
    "12.119269646336": "Manufacturas y transformados AB",
    "24.4773589": "Google, Inc.",
    "24.16041687": "blackned GmbH",
    "24.8710893": "Roku, Inc",
    "12.346864717824": "Vitel Net",
    "12.346868412416": "Dynamic Design",
    "24.11055278": "Leoni",
    "12.71297883283456": "Meggitt Defense Systems Inc.",
    "24.9700508": "Huawei Technologies Co.,Ltd",
    "24.6748": "Euchner GmbH+Co. KG",
    "24.6749": "Mobinnova Corp.",
    "24.6746": "Korea Electric Power Data Network (KDN) Co., Ltd",
    "24.6747": "NetCare Service Co., Ltd.",
    "24.6744": "CCV Deutschland GmbH - Celectronic eHealth Div.",
    "24.6745": "Ircona",
    "24.6742": "ViewTel Co,. Ltd.",
    "24.6743": "Matrix Design Group, LLC",
    "24.6740": "Hip Shing Electronics Ltd.",
    "24.6741": "ACA-Digital Corporation",
    "24.1099157": "Lenovo",
    "24.1778": "Platys Communications",
    "24.1779": "AcceLight Networks",
    "24.1776": "Digeo, Inc.",
    "24.1777": "Optillion",
    "24.1774": "Shenyang Neu-era Information & Technology Stock Co., Ltd",
    "24.1775": "Maxxan Systems, Inc.",
    "24.1772": "Harris Corporation",
    "24.1773": "Inara Networks",
    "24.1770": "ELZET80 Mikrocomputer GmbH&Co. KG",
    "24.1771": "Global Data",
    "24.12637073": "IEEE Registration Authority",
    "12.123917678657536": "Merus Power Dynamics Ltd.",
    "24.6800": "Trópico Sistemas e Telecomunicações da Amazônia LTDA.",
    "12.123917687177216": "EDCO Technology 1993 ltd",
    "20.146101701181440": "ShenZhen Panrich Technology Limited",
    "24.9495292": "IEEE Registration Authority",
    "12.346859958272": "Control and Robotics Solutions",
    "12.346854158336": "Smith Meter, Inc.",
    "24.4479981": "Aruba, a Hewlett Packard Enterprise Company",
    "24.30692": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.14165111": "Universal Electric Corporation",
    "12.123917685567488": "Uplevel Systems Inc",
    "24.6573940": "LENUS Co., Ltd.",
    "20.75685678809088": "TIBA Research & Development (1986) LTD",
    "12.346855243776": "LECO Corporation",
    "12.123917683564544": "C W F Hamilton & Co Ltd",
    "12.123917685731328": "Grace Design/Lunatec LLC",
    "12.123917682036736": "Art Spa",
    "12.346854871040": "Logosol, Inc.",
    "12.346867531776": "MB connect line GmbH Fernwartungssysteme",
    "24.8190036": "Samsung Electronics Co.,Ltd",
    "24.7637544": "HMD Global Oy",
    "24.9141": "Ortana Ltd",
    "24.9140": "Nokia Danmark A/S",
    "24.9143": "Q-Light Co., Ltd.",
    "24.9142": "Securite Communications / Honeywell",
    "24.9145": "Airbus Defence and Space Deutschland GmbH",
    "24.9144": "Sichuan Jiuzhou Electronic Technology Co.,Ltd",
    "24.9147": "Accretech SBS, Inc.",
    "24.9146": "Chroma",
    "24.9149": "Digital Ally, Inc.",
    "24.9148": "EQ-SYS GmbH",
    "24.11582747": "Dongguan Liesheng Electronic Co., Ltd.",
    "12.123917678215168": "Conserv Solutions",
    "12.123917682995200": "Green Instruments A/S",
    "24.6335688": "KunTeng Inc.",
    "20.198151825784832": "Dimton Co.,Ltd.",
    "12.123917677703168": "Solar RIg Technologies",
    "24.311555": "Clover Network, Inc.",
    "24.13159794": "Sagemcom Broadband SAS",
    "24.3434643": "Tecnovideo Srl",
    "24.15217726": "Beijing Infosec Technologies Co., LTD.",
    "24.3434645": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.119269965824": "Coban Srl",
    "12.123917684850688": "Visility",
    "12.123917675941888": "Delta Solutions LLC",
    "24.12864208": "Fireflies Systems",
    "24.1844012": "Samsung Electronics Co.,Ltd",
    "24.13430489": "VAHLE Automation GmbH",
    "20.225135226257408": "Netradyne, Inc.",
    "12.346867245056": "SAE IT-systems GmbH & Co. KG",
    "24.7890265": "Sagemcom Broadband SAS",
    "12.346855378944": "Comodo",
    "24.6585300": "Texas Instruments",
    "24.16291152": "Proton Products Chengdu Ltd",
    "12.119269359616": "Momentum Data Systems",
    "24.9746541": "Intel Corporate",
    "24.5557742": "ShenZhen Apexis Electronic Co.,Ltd",
    "24.6345961": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.12888093": "D-Link International",
    "24.1882006": "Huawei Technologies Co.,Ltd",
    "24.2383051": "AXIe Consortium, Inc.",
    "24.2684210": "ADD-Engineering BV",
    "12.346868125696": "InForce Computing, Inc.",
    "24.6868078": "Huawei Technologies Co.,Ltd",
    "20.132811177590784": "Excelocity Inc.",
    "24.12366209": "Sharp Corporation",
    "12.71297884020736": "EST Analytical",
    "24.3981025": "Nikon Corporation",
    "20.158667125030912": "Kaertech Limited",
    "24.10690": "Cisco Systems, Inc",
    "24.13653921": "Werma Signaltechnik GmbH & Co. KG",
    "24.7129068": "Samsung Electro-Mechanics(Thailand)",
    "20.154314901291008": "Shenzhen Tian-Power Technology Co.,Ltd.",
    "12.346853601280": "Pulsar GmbH",
    "24.10270611": "Creatcomm Technology Inc.",
    "12.346860838912": "ESP-Planning Co.",
    "24.4986516": "shenzhen sibituo Technology Co., Ltd",
    "12.123917688483840": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "12.123917678280704": "CYNIX Systems Inc",
    "24.16020426": "Conversion Sound Inc.",
    "24.3431696": "Wytek",
    "24.7918407": "Ericsson AB",
    "24.4502165": "Sichuan AI-Link Technology Co., Ltd.",
    "12.123917679173632": "Zaklad Energoelektroniki Twerd",
    "24.9178247": "Nokia",
    "12.123917691101184": "HAMEG GmbH",
    "24.16595": "Kimpsion International Corp.",
    "24.16594": "Pagine Corporation",
    "24.16597": "Sartorius Mechatronics T&H GmbH",
    "24.16596": "Gage Talker Corp.",
    "24.16591": "Strawberry Tree, Inc.",
    "24.16590": "Net-Source, Inc.",
    "24.16593": "Fukuda Denshi Co., Ltd.",
    "24.16592": "Mitac International Corp.",
    "12.123917691314176": "Isabellenhütte Heusler Gmbh &Co KG",
    "12.346855473152": "I.C.S. Electronics Limited",
    "24.16599": "Studio Gen Inc.",
    "24.16598": "Locamation B.V.",
    "12.346863325184": "Tampere University of Technology",
    "24.5283623": "Huawei Technologies Co.,Ltd",
    "12.123917686104064": "Östling Marking Systems GmbH",
    "24.13142659": "Dongguan HuaHong Electronics Co.,Ltd",
    "24.5520176": "duagon AG",
    "24.11305971": "AMPAK Technology, Inc.",
    "24.14195815": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.14286631993344": "Fujian Jieyu Computer Technology Co., Ltd.",
    "24.13914705": "IBT Ingenieurbureau Broennimann Thun",
    "24.5306712": "Huawei Device Co., Ltd.",
    "20.89058721136640": "Honeywell International",
    "24.15528491": "Tecno Mobile Limited",
    "12.346855170048": "Research Applications",
    "12.71297884651520": "OPASCA Systems GmbH",
    "24.11799364": "Smartisan Technology Co., Ltd.",
    "12.123917685714944": "Mitsubishi Heavy Industries Thermal Systems, Ltd.",
    "24.13942046": "5VT Technologies,Taiwan LTd.",
    "24.7907005": "DAEYEON Control&Instrument Co,.Ltd",
    "12.123917688496128": "Multipure International",
    "12.346854600704": "DSM-Messtechnik GmbH",
    "24.9440193": "Sprocomm Technologies CO.,Ltd",
    "20.102196702085120": "Shanghai Notion Information Technology CO.,LTD.",
    "24.6061406": "Huawei Technologies Co.,Ltd",
    "24.1839023": "Lucid Vision Labs",
    "24.8434474": "UK Grid Solutions Ltd",
    "12.346855575552": "Novx Systems",
    "24.7628336": "T:mi Ytti",
    "24.7127506": "ARRIS Group, Inc.",
    "24.6859658": "RF IDeas",
    "20.167860140769280": "Surtec",
    "12.123917676396544": "Walk Horizon Technology (Beijing) Co., Ltd.",
    "24.5557671": "Fujian Shenzhou Electronic Co.,Ltd",
    "24.12333147": "Dell Inc.",
    "24.3995084": "New H3C Technologies Co., Ltd",
    "24.14957850": "Broadcom Limited",
    "24.551434": "Askey Computer Corp",
    "12.123917690503168": "Panasonic Life Solutions Elektri̇k Sanayi̇ Ve Ti̇care",
    "24.5285503": "Apple, Inc.",
    "24.1839792": "Amazon Technologies Inc.",
    "20.93065828106240": "Wonders Technology Co., Ltd.",
    "24.1835074": "NARI Technology Co., Ltd.",
    "12.123917677916160": "Worldsensing",
    "24.13385501": "Intesis Software SL",
    "20.57194442653696": "Pebble Technology",
    "24.1605838": "Barberry Solutions Ltd",
    "12.346859696128": "Priority Electronics Ltd",
    "24.4485272": "Ming Hong Technology (Shen Zhen) Limited",
    "24.11043198": "Apple, Inc.",
    "24.9754403": "Shanghai DareGlobal Technologies Co.,Ltd",
    "24.1069426": "Huawei Technologies Co.,Ltd",
    "24.8663663": "Guangzhou Ava Electronics Technology Co.,Ltd",
    "24.13424833": "Cisco Systems, Inc",
    "12.123917688758272": "Houston Radar LLC",
    "24.3187232": "ARG Telecom",
    "12.346867666944": "Amsterdam Scientific Instruments BV",
    "24.9716557": "Ciena Corporation",
    "20.70559702450176": "Lista AG",
    "24.14686367": "EM Microelectronic",
    "24.7369748": "Murata Manufacturing Co., Ltd.",
    "24.12386081": "Smart Code(shenzhen)Technology Co.,Ltd",
    "24.8165074": "Fihonest communication co.,Ltd",
    "24.554520": "ViE Technologies Sdn. Bhd.",
    "24.14423496": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "12.123917679390720": "Sintokogio, Ltd",
    "12.123917678493696": "IMP-Computer Systems",
    "12.119269900288": "Radmor S.A.",
    "12.346856116224": "Silicon Optix Canada Inc.",
    "12.123917684326400": "Aplex Technology Inc.",
    "24.6827131": "Cisco Systems, Inc",
    "24.10257013": "Yichip Microelectronics (Hangzhou) Co.,Ltd",
    "24.6071740": "eero inc.",
    "12.123917689438208": "Peter Huber Kaeltemaschinenbau AG",
    "24.8665675": "Intel Corporate",
    "12.346858786816": "Staer Spa",
    "12.123917689749504": "Leidos",
    "24.7083570": "Huawei Technologies Co.,Ltd",
    "12.346859278336": "Netami",
    "24.9456769": "Hewlett Packard Enterprise",
    "12.71297883680768": "IPS Technology Limited",
    "12.123917682610176": "How-E",
    "24.558182": "Novartis Pharma AG",
    "20.176726061613056": "DirectOut GmbH",
    "24.6338414": "Google, Inc.",
    "24.11065384": "Ascensia Diabetes Care",
    "24.10259147": "Juniper Networks",
    "12.123917676445696": "European Synchrotron Radiation Facility",
    "24.4119": "Bosch Access Systems GmbH",
    "24.4118": "T.Sqware",
    "24.4220538": "mediola - connected living AG",
    "24.4115": "Kontron America, Inc.",
    "24.4114": "Processor Systems (I) Pvt Ltd",
    "24.4117": "OOmon Inc.",
    "24.4116": "Cisco Systems, Inc",
    "24.4111": "Industrial Cpu Systems",
    "24.4110": "Micro Linear Coporation",
    "24.4113": "Cisco Systems",
    "24.4112": "Initio Corporation",
    "24.8421368": "Juniper Networks",
    "24.3468": "Shanghai Wedone Digital Ltd. CO.",
    "24.3469": "Prosoft Technology, Inc",
    "24.3464": "D-Link Corporation",
    "24.3465": "Bils Technology Inc",
    "24.3466": "Winners Electronics Co., Ltd.",
    "24.3467": "T&D Corporation",
    "24.3460": "Makus Inc.",
    "24.3461": "Tapwave, Inc.",
    "24.3462": "Huber + Suhner AG",
    "24.3463": "Elitegroup Computer Systems Co.,Ltd.",
    "24.14450596": "2Wire Inc",
    "20.88668806053888": "Zheng Dian Electronics Limited",
    "20.272689486888960": "Digatron Power Electronics GmbH",
    "12.71297884364800": "ATX Networks Ltd.",
    "24.13168294": "Shenzhen SHX Technology Co., Ltd",
    "12.346861207552": "Specialized Communications Corp.",
    "20.88914839732224": "Tianjin Natianal Health Technology Co.,Ltd",
    "12.346867462144": "Arista Systems Corporation",
    "20.31477071872000": "Guang Dong He Zheng Network Technology Co.,Ltd",
    "12.123917680947200": "Sea Air and Land Communications Ltd",
    "24.14710480": "AMPAK Technology, Inc.",
    "20.224447335235584": "Tolomatic, Inc.",
    "24.11592917": "Google, Inc.",
    "12.123917682458624": "Konzept Informationssysteme GmbH",
    "12.123917680771072": "Private",
    "12.346867404800": "Proteqsen",
    "24.2940393": "Arista Networks",
    "24.15264142": "SOLAR controls s.r.o.",
    "24.16318402": "Apple, Inc.",
    "24.7392657": "Teracom Telematica S.A",
    "24.10810658": "Chofu Seisakusho Co.,Ltd",
    "20.154314887659520": "Future Robot Technology Co., Limited",
    "24.14446612": "Private",
    "12.71297883418624": "Telcomkorea",
    "12.123917691396096": "Sarokal Test Systems Oy",
    "12.346862956544": "L-3 Communications Mobile-Vision, Inc.",
    "12.123917679120384": "Ultra Electronics Sonar System Division",
    "24.15768763": "RaonThink Inc.",
    "24.13919912": "Riedo Networks Ltd",
    "12.346852913152": "Rymic",
    "24.13378103": "IEEE Registration Authority",
    "12.346859491328": "SAIA Burgess Controls AG",
    "12.123917684674560": "Invertek Drives Ltd",
    "24.2408893": "Crrc Dalian R&D Co.,Ltd.",
    "24.13406355": "Hansong Tehnologies",
    "24.3418865": "Texas Instruments",
    "24.5816882": "NEC Corporation",
    "12.346859220992": "Ele.Mag S.r.l.",
    "12.346859655168": "Dsp Design",
    "20.159182260011008": "Shenzhen Photon Broadband Technology CO., LTD",
    "12.123917683720192": "Reo Ag",
    "12.123917689462784": "DSan Corporation",
    "24.2105147": "Google, Inc.",
    "24.16277058": "Technicolor CH USA Inc.",
    "20.79602622922752": "Gopod Group Limited",
    "24.9479727": "Naver",
    "12.346858385408": "pikkerton GmbH",
    "12.346857185280": "Yarg Biometrics Limited",
    "12.346857021440": "SPECS GmbH",
    "24.3148835": "zte corporation",
    "24.11580124": "Zyxel Communications Corporation",
    "24.1080228": "Olive & Dove Co.,Ltd.",
    "12.346861027328": "Arcos Technologies Ltd.",
    "24.16009288": "Amscreen Group Ltd",
    "12.346865975296": "Schneider Electric Motion USA",
    "24.11019499": "ZAO NPO Introtest",
    "24.15738254": "Huawei Technologies Co.,Ltd",
    "24.4516427": "Actlas Inc.",
    "20.13692016001024": "Shenzhen Samchung Video Technology Co., Ltd.",
    "24.12894048": "Tecno Mobile Limited",
    "24.10770972": "smart-electronic GmbH",
    "24.11291390": "Hisense Broadband Multimedia Technology Co.,Ltd.",
    "12.346856415232": "ads-tec GmbH",
    "24.788660": "HUMAX Co., Ltd.",
    "24.3733426": "Seojun Electric",
    "24.8656954": "Ruckus Wireless",
    "24.15759312": "Xcellen",
    "24.13155201": "Huawei Device Co., Ltd.",
    "24.1846": "Data Video Technologies Co., Ltd.",
    "24.1848817": "NEW Lift Neue Elektronische Wege Steuerungsbau GmbH",
    "12.71297884123136": "Lightstar",
    "20.123731219316736": "Church & Dwight Co., Inc.",
    "12.346853056512": "Motion Analysis Corp.",
    "24.53937": "TPV Display Technology (Xiamen) Co.,Ltd.",
    "24.7365304": "D-Link International",
    "24.4218970": "Hawkeye Tech Co. Ltd",
    "24.6356729": "Thomas & Betts",
    "24.11303841": "Intel Corporate",
    "12.346860298240": "DSR Information Technologies Ltd.",
    "20.251018421993472": "FLK information security technology Co,. Ltd",
    "12.346866225152": "GHL Systems Bhd",
    "24.6558245": "Cisco Systems, Inc",
    "24.2662039": "Huawei Technologies Co.,Ltd",
    "12.119269916672": "Link Precision",
    "12.346867482624": "SPARQ systems",
    "20.137215112904704": "CeoTronics AG",
    "24.7603133": "Buffalo.Inc",
    "24.12850938": "Prysm Inc",
    "12.123917683810304": "MEPS Realtime",
    "24.8853": "SGM Technology for lighting spa",
    "24.3990950": "Hangzhou H3C Technologies Co., Limited",
    "24.13434033": "Wireless Technology, Inc.",
    "24.3979154": "Hitachi Maxell, Ltd., Optronics Division",
    "24.9178720": "Nanjing Juplink Intelligent Technologies Co., Ltd.",
    "24.2422142": "Hwh Co., Ltd.",
    "12.346867118080": "Technica Engineering GmbH",
    "12.71297884573696": "Telefrang AB",
    "12.346867867648": "Predictive Sensor Technology",
    "24.15494303": "Chengdu Volans Technology CO.,LTD",
    "24.14158746": "Samsung Electronics Co.,Ltd",
    "24.2409": "Meret Optical Communications",
    "24.2408": "Technoventure, Inc.",
    "24.2405": "HyunJu Computer Co., Ltd.",
    "24.2404": "Hi-Techniques, Inc.",
    "24.2407": "Tachyon, Inc",
    "24.2406": "Trimble Europe Bv",
    "24.2401": "Switchgear and Instrumentation Ltd",
    "24.2400": "YOZAN Inc.",
    "24.2403": "Dominion Lasercom Inc.",
    "24.2402": "Sonitor Technologies AS",
    "24.1178": "Cisco Systems, Inc",
    "24.1179": "Cisco Systems, Inc",
    "12.123917678223360": "SMITEC S.p.A.",
    "24.1174": "Extreme Networks, Inc.",
    "24.1175": "MacroSystem Digital Video AG",
    "24.1176": "Mahi Networks",
    "24.1177": "Chino Corporation",
    "24.1170": "Hive Internet, Ltd.",
    "24.1171": "Tsinghua Unisplendour Co., Ltd.",
    "24.1172": "Breezecom, Ltd.",
    "24.1173": "Tejas Networks India Limited",
    "12.123917675843584": "AML Oceanographic",
    "12.346864820224": "Oki Denki Bohsai Co.,Ltd.",
    "24.842134": "Cisco Systems, Inc",
    "12.346861228032": "Mocon, Inc.",
    "20.4884650786816": "Gimso Mobile Ltd",
    "24.6236": "EDSLAB Technologies",
    "24.6237": "Taiguen Technology (Shen-Zhen) Co., Ltd.",
    "24.6234": "uControl, Inc.",
    "24.6235": "Network Chemistry, Inc",
    "24.6232": "TagMaster AB",
    "24.6233": "Strawberry Linux Co.,Ltd.",
    "24.6230": "EyeFi, Inc",
    "24.6231": "Unilever R&D",
    "24.6238": "Nexterm Inc.",
    "24.6239": "TAC Inc.",
    "24.16296414": "Puissance Plus",
    "24.15211339": "amnimo Inc.",
    "20.49277264986112": "Cita Smart Solutions Ltd",
    "24.9231141": "ESG Solutions",
    "24.1853504": "IMS Messsysteme GmbH",
    "12.123917686099968": "DEUTA-WERKE GmbH",
    "24.15231001": "Huawei Technologies Co.,Ltd",
    "12.346862231552": "Miromico AG",
    "24.5026253": "Ruckus Wireless",
    "20.49277253451776": "AOptix Technologies, Inc",
    "24.11569159": "Strata Worldwide",
    "12.123917679202304": "Private",
    "24.6557158": "Petra Systems",
    "24.12365227": "Avaya Inc",
    "24.573675": "Mellanox Technologies, Inc.",
    "24.16028575": "Hon Hai Precision Ind. Co., Ltd.",
    "12.346861080576": "InnoTrans Communications, Inc.",
    "24.13389112": "Carnegie Technologies",
    "24.6607556": "Apple, Inc.",
    "12.123917680308224": "LLC Sarov Innovative Technologies (WIZOLUTION)",
    "24.5271781": "He Shan World Fair Electronics Technology Limited",
    "12.346858872832": "Xworks NZ Limited",
    "24.1868609": "Hefei Bitland Information Technology Co.Ltd",
    "20.48476186476544": "Molex Incorporated",
    "24.10012123": "Ericsson AB",
    "24.12104135": "Bt&Com Co.,Ltd",
    "24.6595393": "Wonderlan (Beijing) Technology Co., Ltd.",
    "20.159304569061376": "Dongguan Kangyong electronics technology Co. Ltd",
    "24.4237910": "China Dragon Technology Limited",
    "20.39672884494336": "Kaiyun",
    "12.346866929664": "IHI Scube Co.,Ltd",
    "24.9736174": "Verizon Telematics",
    "24.5275722": "Quectel Wireless Solutions Co., Ltd.",
    "24.15492077": "Hyundai Autoever Corp.",
    "12.123917688672256": "Rishaad Brown",
    "20.238070227009536": "Camtrace",
    "24.3686963": "Wildeboer Bauteile GmbH",
    "12.123917688545280": "Computerwise, Inc.",
    "24.61996": "Shanghai B-star Technology Co.,Ltd.",
    "24.6881147": "Tp-Link Technologies Co.,Ltd.",
    "12.123917677498368": "Potter Electric Signal Co. LLC",
    "24.129": "Bay Networks",
    "24.128": "Cray Communications (formerly Dowty Network Services)",
    "24.125": "Oracle Corporation",
    "24.124": "Ampere Incorporated",
    "24.127": "Linotype-Hell AG",
    "24.126": "Clustrix Corporation",
    "24.121": "Networth Incorporated",
    "24.120": "Labtam Limited",
    "24.123": "Research Machines",
    "24.122": "Dana Computer Inc.",
    "24.15739177": "Cisco Systems, Inc",
    "24.9988925": "ARRIS Group, Inc.",
    "24.14727610": "Apple, Inc.",
    "24.13416616": "AMPAK Technology, Inc.",
    "12.346853613568": "Teseda Corporation",
    "24.6880435": "Next Level Security Systems, Inc.",
    "24.9706780": "TLab West Systems AB",
    "24.10512896": "Verilink Corporation",
    "20.27056769335296": "Airprotec",
    "24.14684287": "Ruckus Wireless",
    "24.11300687": "Enspert Inc",
    "24.13432897": "Lumewave",
    "24.7899245": "Apple, Inc.",
    "24.10782835": "Cisco Systems, Inc",
    "12.123917689933824": "Leo Bodnar Electronics Ltd",
    "24.10780063": "Shenzhen iComm Semiconductor CO.,LTD",
    "24.4225130": "Motorola Mobility LLC, a Lenovo Company",
    "24.9196535": "A&D Engineering Co., Ltd.",
    "12.123917687500800": "Potter Electric Signal Co. LLC",
    "12.346857054208": "HEINEN ELEKTRONIK GmbH",
    "24.3465517": "Intel Corporate",
    "24.4989112": "Apple, Inc.",
    "24.6842670": "Zycoo Co.,Ltd",
    "12.346868707328": "SERCOM Regeltechniek",
    "20.176069209489408": "All Inspire Health Inc.",
    "12.123917683920896": "Tattile Srl",
    "24.26859": "HP Inc.",
    "24.7922541": "Espressif Inc.",
    "24.14999718": "Apple, Inc.",
    "24.9705786": "Elvaco AB",
    "24.5278839": "DIRMEIER Schanktechnik GmbH &Co KG",
    "24.8133523": "Huawei Device Co., Ltd.",
    "20.70559704547328": "Annapurna labs",
    "12.346857304064": "Computerized Elevator Contol",
    "24.15506315": "Hewlett Packard Enterprise",
    "12.123917675855872": "ProHound Controles Eirelli",
    "24.13394405": "Motorola Mobility LLC, a Lenovo Company",
    "24.1845213": "Datang Gohighsec(zhejiang)Information Technology Co.,Ltd.",
    "24.9969177": "Sagemcom Broadband SAS",
    "12.123917691150336": "Seznam.cz, a.s., CZ26168685",
    "20.31657721593856": "Sunray Medical Apparatus Co.,Ltd.",
    "24.16036737": "WindowMaster A/S",
    "24.588936": "Samsung Electronics Co.,Ltd",
    "24.13113050": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.832807": "Huawei Technologies Co.,Ltd",
    "24.16026859": "D-Link International",
    "20.31477539536896": "Atlas Aerospace",
    "20.124214336028672": "Flexim Security Oy",
    "24.4235528": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.123917681709056": "CertUsus GmbH",
    "12.346858864640": "Westel Wireless Systems",
    "20.31477073969152": "ERATO (HK) Corporation Limited",
    "24.13153314": "Askey Computer Corp",
    "12.346867343360": "Lightel",
    "20.167295455330304": "Danfoss Power Solutions",
    "24.9701575": "Huawei Device Co., Ltd.",
    "12.123917677723648": "Reflexion Medical",
    "20.225135237791744": "Toong In Electronic Corp.",
    "20.180394587586560": "Carbon, Inc.",
    "24.5787819": "TLS Corporation",
    "20.75247162228736": "Anhui GuDao Tech",
    "24.14477716": "Cloud Network Technology Singapore Pte. Ltd.",
    "24.8715436": "Apple, Inc.",
    "24.13919230": "Arcadyan Corporation",
    "12.346867257344": "IB Elektronik GmbH",
    "24.16056059": "Samsung Electronics Co.,Ltd",
    "24.4759800": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346868383744": "Gets MSS S.A.",
    "12.123917675409408": "PoolDigital GmbH & Co. KG",
    "24.6817557": "Intel Corporate",
    "12.71297884983296": "Beking Industrieele automatisering",
    "24.7655644": "Ericsson AB",
    "24.21016": "Wuxi Keboda Electron Co.Ltd",
    "12.123917683310592": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "12.346867920896": "Icon Research Ltd",
    "24.3211254": "HangZhou KuoHeng Technology Co.,ltd",
    "12.346866548736": "Comsat VertriebsgmbH",
    "24.1876179": "IEEE Registration Authority",
    "24.11565168": "Wi-NEXT, Inc.",
    "24.10486627": "Robert Bosch Healthcare GmbH",
    "20.88010616995840": "Machfu, Inc.",
    "24.9452718": "Yunnan KSEC Digital Technology Co.,Ltd.",
    "12.346855526400": "Nimcat Networks",
    "24.2364": "Jacques Technologies P/L",
    "24.2365": "Newisys,Inc.",
    "24.2366": "C&I Technologies",
    "24.2367": "Double-Win Enterpirse CO., LTD",
    "24.2360": "Allot Communications",
    "24.2361": "ShibaSoku Co.,Ltd.",
    "24.2362": "Molex CMS",
    "24.2363": "Hyundai Networks Inc.",
    "24.2368": "AGFEO GmbH & Co. KG",
    "24.2369": "Allied Telesis R&D Center K.K.",
    "24.2405975": "Cisco Systems, Inc",
    "24.15256481": "Nokia Corporation",
    "24.2419793": "ADF Technologies Sdn Bhd",
    "24.13164420": "Texas Instruments",
    "20.216602657161216": "Critical Link",
    "24.7136474": "NEC Platforms, Ltd.",
    "12.123917681557504": "Electronic Equipment Company Pvt. Ltd.",
    "24.1842535": "Huawei Technologies Co.,Ltd",
    "24.49487": "DDL Co,.ltd.",
    "24.10006585": "Nokia",
    "24.7628": "Ayon Cyber Security, Inc",
    "24.7629": "ARRIS Group, Inc.",
    "24.7626": "PAV Electronics Limited",
    "24.7627": "Exéns Development Oy",
    "24.7624": "Navionics Research Inc., dba SCADAmetrics",
    "24.7625": "GainSpan Corp.",
    "24.7622": "SNR Inc.",
    "24.7623": "L-3 Communications Geneva Aerospace",
    "24.7620": "AIOI Systems Co., Ltd.",
    "24.7621": "Beijing Jiaxun Feihong Electricial Co., Ltd.",
    "12.346859782144": "Avonaco Systems, Inc.",
    "24.5295733": "Measy Electronics Co., Ltd.",
    "24.7085712": "Hansol Technics Co., Ltd.",
    "24.11821175": "Sichuan Changhong Electric Ltd.",
    "24.5820529": "BW Broadcast",
    "20.189244304785408": "Solare Datensysteme GmbH",
    "24.6570516": "Swisscom Energy Solutions AG",
    "20.128612189601792": "Bernard Krone Holding GmbH & Co. KG",
    "24.1894061": "Huawei Device Co., Ltd.",
    "24.10530885": "Halong Mining",
    "24.1099583": "Midas Klark Teknik Ltd",
    "12.346854764544": "LinkSecurity A/S",
    "12.346867687424": "easii ic adiis",
    "24.2677054": "Telecor Inc.",
    "20.110373790285824": "Chunghwa System Integration Co., Ltd.",
    "24.3163409": "Texas Instruments",
    "24.13669368": "Stonestreet One LLC",
    "24.14214979": "Wush, Inc",
    "24.8447194": "IEEE Registration Authority",
    "24.2673948": "Espotel Oy",
    "12.346855653376": "Advanced IP Communications",
    "24.11534451": "Wistron Neweb Corporation",
    "12.123917687021568": "Atl-Sd",
    "24.1843665": "IEEE Registration Authority",
    "12.346865119232": "Micro Technica",
    "12.346868690944": "HSDC Sp. z o.o.",
    "12.123917683838976": "Census Digital Incorporated",
    "24.7904487": "Shenzhen Aikede Technology Co., Ltd",
    "24.4505571": "Shenzhen Longtech Electronics Co.,Ltd",
    "24.15214262": "Inside Biometrics International Limited",
    "24.13144902": "MXCHIP Company Limited",
    "20.216145523113984": "Shen Zhen Euse Technology Co.,Ltd",
    "12.123917685841920": "Dorlet Sau",
    "24.16284399": "Rosonix Technology, Inc.",
    "24.8442011": "Huawei Technologies Co.,Ltd",
    "20.75247160131584": "lb Lautsprecher gmbH",
    "24.24828": "Conservation Through Innovation Ltd.",
    "24.24829": "NetICs, Inc.",
    "24.3207994": "+plugg srl",
    "24.567893": "Motorola Mobility LLC, a Lenovo Company",
    "24.3933281": "ARRIS Group, Inc.",
    "24.24820": "ADVANCED COMPUTER SOLUTIONS, Inc.",
    "24.24821": "Icon West, Inc.",
    "24.24822": "Nextest Communications Products, Inc.",
    "24.24823": "Datafusion Systems",
    "24.24824": "Loran International Technologies Inc.",
    "24.24825": "Diamond Lane Communications",
    "24.24826": "Educational Technology Resources, Inc.",
    "24.24827": "Packeteer, Inc.",
    "24.9477955": "Accutron Instruments Inc.",
    "24.8183057": "Chongqing MAS SCI&TECH.Co.,Ltd",
    "12.123917681975296": "WOW System",
    "24.14976257": "Geberit International AG",
    "24.6063708": "Samsung Electronics Co.,Ltd",
    "12.346852184064": "GD California, Inc.",
    "24.7342927": "Cisco Systems, Inc",
    "24.292001": "Aligera Equipamentos Digitais Ltda",
    "24.32791": "Pfu Limited",
    "24.32790": "Wandel & Goltermann",
    "12.346867986432": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.32792": "Kobe Steel, Ltd.",
    "24.32795": "Kodiak Technology",
    "24.32794": "Bell Atlantic",
    "24.32797": "Integrated Inference Machines",
    "24.32796": "Newport Systems Solutions",
    "24.32799": "Krupp Atlas Electronik Gmbh",
    "24.32798": "Xinetron, Inc.",
    "24.311452": "Tellabs Inc.",
    "24.11333278": "Samsung Electronics Co.,Ltd",
    "12.346863632384": "Fr. Sauter AG",
    "24.5250451": "Huawei Technologies Co.,Ltd",
    "24.5281849": "Xiaomi Communications Co Ltd",
    "12.123917676285952": "Tunstall A/S",
    "12.123917678841856": "Vivalnk",
    "12.346855440384": "DTASENSOR S.p.A.",
    "24.2678501": "Huawei Technologies Co.,Ltd",
    "24.2108353": "Huawei Technologies Co.,Ltd",
    "12.346855305216": "Patria Advanced Solutions",
    "24.13669478": "Dell Inc.",
    "12.346861068288": "Scm Products, Inc.",
    "24.15466978": "Foxconn Interconnect Technology",
    "24.13928408": "Samsung Electronics Co.,Ltd",
    "12.346854694912": "Fg Synerys",
    "12.346855153664": "PYRAMID Computer GmbH",
    "24.5282035": "Rheem Australia Pty Ltd",
    "20.154019635920896": "Bausch Datacom NV/SA",
    "24.5813293": "YSTen Technology Co.,Ltd",
    "24.10541117": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.13117738": "Barun Electronics",
    "24.4471944": "Ficosa Maroc International",
    "24.16037193": "Xiamen Yeastar Information Technology Co., Ltd.",
    "24.3422391": "HUMAX Co., Ltd.",
    "12.346857156608": "Quantum3D, Inc.",
    "24.8972198": "iKnowledge Integration Corp.",
    "24.16275968": "Sanford LP",
    "12.346857824256": "ubinetsys.co..ltd",
    "24.5292038": "Carmanah Signs",
    "24.847513": "Amazon Technologies Inc.",
    "24.14698671": "Premietech Limited",
    "12.346855616512": "Ecutel Systems, Inc.",
    "24.9470048": "Nilfisk A/S",
    "12.346854027264": "Twin Development S.A.",
    "12.123917677907968": "Mect Srl",
    "24.2817386": "Charter Communications",
    "24.7610003": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11072940": "Huawei Technologies Co.,Ltd",
    "24.11278082": "Fibar Group sp. z o.o.",
    "24.13170436": "Building Block Video",
    "24.2115321": "Advanced Network Devices (dba:AND)",
    "24.6821381": "Systems And Electronic Development FZCO",
    "12.123917691195392": "Sonel S.A.",
    "12.346863132672": "Roesch & Walter Industrie-Elektronik GmbH",
    "12.346853093376": "Lanex S.A.",
    "24.3946212": "ARRIS Group, Inc.",
    "24.16410": "Fuji Electric Co., Ltd.",
    "24.16411": "Printer Systems Corp.",
    "24.16412": "AST",
    "24.16413": "Invisible Software, Inc.",
    "24.16414": "Icc",
    "24.16415": "Colorgraph Ltd",
    "24.16416": "CommScope Inc",
    "24.16417": "Raster Graphics",
    "24.16418": "Klever Computers, Inc.",
    "24.16419": "Logic Corporation",
    "12.346866679808": "Leonardo MW Ltd (Land & Naval Defence Electronic)",
    "12.346859737088": "Tattile srl",
    "24.8173920": "Shanghai X-Cheng telecom LTD",
    "24.3711": "Hewlett Packard",
    "24.1315671": "ATP Electronics, Inc.",
    "24.11557060": "Broadcast Microwave Services, Inc",
    "24.11328714": "Huawei Technologies Co.,Ltd",
    "20.110373783994368": "FaceHeart Inc.",
    "24.7093099": "Routerboard.com",
    "24.11812274": "Degreane Horizon",
    "20.198151826833408": "Komsis Elektronik Sistemleri San. Tic. Ltd.Sti",
    "24.2912081": "Herospeed Digital Technology Limited",
    "20.163603963445248": "Nanjing Yacer Communication Technology Co. Ltd.",
    "24.11042538": "Intel Corporate",
    "24.2374688": "Dynamode Group",
    "24.867": "Miraesys Co., Ltd.",
    "24.2362732": "Smnd",
    "20.48476173893632": "Aimco",
    "12.123917685485568": "Network Integrity Systems",
    "12.346863800320": "Polytron Corporation",
    "12.346864668672": "Touch Revolution Inc.",
    "12.123917680734208": "Dsilog",
    "24.1841107": "Cisco Systems, Inc",
    "12.123917689815040": "Signature Control Systems, LLC.",
    "24.4774957": "Huawei Technologies Co.,Ltd",
    "24.6605057": "INVENTEC Corporation",
    "24.3204054": "Spotify USA Inc.",
    "24.7868499": "China Mobile Group Device Co.,Ltd.",
    "20.106480342663168": "Avalun",
    "24.4241073": "SUNGSAM CO,.Ltd",
    "24.363": "LightChip, Inc.",
    "24.362": "Alitec",
    "24.10018690": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.361": "Celestix Networks Pte Ltd.",
    "24.6592276": "eero inc.",
    "24.360": "Vitana Corporation",
    "20.132855972757504": "Cartender",
    "24.367": "Inkel Corp.",
    "24.366": "Conklin Corporation",
    "24.12469": "Tadiran Microwave Networks",
    "24.12468": "Intersil Corp.",
    "12.346863755264": "KYAB Lulea AB",
    "24.365": "CarrierComm Inc.",
    "24.12461": "Shanghai Communication",
    "24.12460": "Systeme Lauer GmbH & Co., Ltd.",
    "24.12463": "Honeywell GmbH",
    "24.364": "Foxconn",
    "24.12465": "TrunkNet",
    "24.12464": "Convergenet Technologies",
    "24.12467": "San Valley Systems, Inc.",
    "24.12466": "L-3 Sonoma EO",
    "24.7377395": "Universal Electronics, Inc.",
    "24.369": "Allied Data Technologies",
    "24.2903": "Silicon Laboratories",
    "24.5801804": "Lite-On Clean Energy Technology Corp.",
    "24.815178": "Huawei Technologies Co.,Ltd",
    "24.9028": "Objective Interface Systems, Inc.",
    "24.2907": "Rincon Research Corporation",
    "24.6347562": "Hns",
    "24.12347102": "Shadow Creator Information Technology Co.,Ltd.",
    "20.224641225326592": "Smiths US Innovation LLC",
    "24.14428692": "Shanghai Longjing Technology Co.",
    "12.123917675073536": "Sumer Data S.L",
    "24.4210156": "HUMAX Co., Ltd.",
    "24.15252392": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.2147236": "RetailNext",
    "24.9476196": "Impex-Sat GmbH&amp;Co KG",
    "24.1322590": "Sercomm Corporation.",
    "12.346861604864": "Research Applications Incorp",
    "24.12080829": "Itt Llc",
    "24.14423515": "Ge Ruili Intelligent Technology ( Beijing ) Co., Ltd.",
    "24.5479": "RADWIN Inc.",
    "24.5478": "A-First Technology Co., Ltd.",
    "24.5475": "Cisco Systems, Inc",
    "24.5474": "Cisco Systems, Inc",
    "24.5477": "Xiamen Yealink Network Technology Co.,Ltd",
    "24.5476": "BEHRINGER Spezielle Studiotechnik GmbH",
    "24.5471": "GreenPeak Technologies",
    "24.5470": "Morgan Stanley",
    "24.5473": "JJPlus Corporation",
    "24.5472": "Hewlett Packard",
    "24.10779679": "Cobham plc",
    "24.4708": "daum electronic gmbh",
    "24.4709": "Enerdyne Technologies, Inc.",
    "24.4704": "Stanton Magnetics,inc.",
    "24.4705": "Adaptix, Inc",
    "24.4706": "Nokia Danmark A/S",
    "24.4707": "Data Voice Technologies GmbH",
    "24.4700": "Green Hills Software, Inc.",
    "24.4701": "CyberNet Inc.",
    "24.4702": "Caen",
    "24.4703": "AWIND Inc.",
    "24.6071460": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.1353611": "Zhejiang Dahua Technology Co., Ltd.",
    "24.14178519": "CZ.NIC, z.s.p.o.",
    "12.346856427520": "Multimessage Systems Ltd.",
    "24.13684989": "Cisco Systems, Inc",
    "24.2901019": "Spectrum Medical Limited",
    "24.6558572": "zte corporation",
    "24.9475082": "Analog Devices, Inc",
    "24.5281543": "Xiamen Paperang Technology Co.,Ltd.",
    "12.123917690580992": "Netfort Solutions",
    "24.554751": "Thomson Telecom Belgium",
    "12.123917682081792": "LLC \"NTZ \"Mekhanotronika\"",
    "24.7121713": "Apple, Inc.",
    "20.71070626349056": "Pella Corporation",
    "20.238070225960960": "Teplovodokhran Ltd.",
    "12.123917681393664": "Motec Pty Ltd",
    "12.123917680943104": "Chugoku Electric Manufacturing Co.,Inc",
    "24.8975504": "Jetmobile Pte Ltd",
    "12.346859556864": "ela-soft GmbH & Co. KG",
    "20.198237819502592": "Elleta Solutions Ltd",
    "12.346852655104": "Reselec AG",
    "24.6299037": "Sichuan AI-Link Technology Co., Ltd.",
    "24.15789113": "Essec",
    "24.5801378": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "24.57372": "Cradlepoint, Inc",
    "12.346864394240": "NetworkFX Communications, LLC",
    "24.57370": "Comtec Systems. Co., Ltd.",
    "24.57371": "Sphere Communications, Inc.",
    "24.57376": "Tecnomen Oy",
    "24.57377": "Freegate Corp.",
    "24.57374": "Cisco Systems, Inc",
    "24.57375": "AVIDIA Systems, Inc.",
    "24.57378": "Analog Devices, Inc.",
    "24.57379": "Telrad",
    "24.4252208": "AzureWave Technology Inc.",
    "12.346865106944": "Foerster-Technik GmbH",
    "12.119269806080": "Eurek Srl",
    "24.4989465": "Yuanfudao Hk Limted",
    "24.13659870": "Sagemcom Broadband SAS",
    "24.3468555": "Shanghai XZ-COM.CO.,Ltd.",
    "24.5810031": "iD corporation",
    "24.8439359": "Remec Broadband Wireless , LLC",
    "12.123917682233344": "DEUTA-WERKE GmbH",
    "20.62663549779968": "Management Service Corporation",
    "24.13171425": "ARQ Digital LLC",
    "24.12608710": "Nokia Corporation",
    "24.7323": "FEIG ELECTRONIC GmbH",
    "12.346863239168": "S.two Corporation",
    "12.346868600832": "Finishing Brands",
    "12.123917689298944": "ACD Elekronik GmbH",
    "12.123917687128064": "Amico Corporation",
    "24.1351928": "Huawei Technologies Co.,Ltd",
    "24.10779418": "Huawei Device Co., Ltd.",
    "24.589604": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.14948459": "Zyxel Communications Corporation",
    "12.346857082880": "NSFOCUS Information Technology Co., Ltd.",
    "20.45033667428352": "Matricx Singapore Pte Ltd",
    "12.123917680017408": "Hosokawa Micron Powder Systems",
    "24.6038649": "Baltech AG",
    "12.346866593792": "ZED Ziegler Electronic Devices GmbH",
    "24.3164": "GTN Systems B.V.",
    "24.3167": "Avtec, Inc.",
    "24.3166": "Calypso Medical",
    "24.3161": "Indyme Electronics, Inc.",
    "24.3160": "M&S Systems",
    "24.3163": "Hanwang Technology Co.,Ltd",
    "24.3162": "IBSmm Embedded Electronics Consulting",
    "24.3169": "AC Tech corporation DBA Advanced Digital",
    "24.3168": "ACM Systems",
    "12.346864861184": "Becton Dickinson",
    "20.114975474450432": "NIPK Electron Co.",
    "12.123917683064832": "Medisafe International",
    "24.10515857": "Samsung Electronics Co.,Ltd",
    "12.346865668096": "Satellink Inc.",
    "24.10288129": "Apple, Inc.",
    "20.171977809592320": "Globalcom Engineering SPA",
    "24.10254851": "Ciena Corporation",
    "20.176726066855936": "SIMTEC Elektronik GmbH",
    "24.8713804": "Cross Point BV",
    "24.2118774": "Sony Mobile Communications Inc",
    "24.11335772": "Chengdu Higon Integrated Circuit Design Co,. Ltd.",
    "24.1895538": "Allradio Co., Ltd",
    "12.346853126144": "iREZ Technologies LLC",
    "24.8964999": "Apple, Inc.",
    "24.6077850": "Huawei Device Co., Ltd.",
    "24.1625112": "Intel Corporate",
    "24.6077854": "Hongkong Miracle Eagle Technology(Group) Limited",
    "20.224447331041280": "Shanghai Doit IOT Technology Co.,Ltd.",
    "24.1060527": "Texas Instruments",
    "20.74781680467968": "Gemmy Electronics (Shenzhen) Co, Ltd",
    "12.346863808512": "Integrated Control Corp.",
    "24.14738895": "Texas Instruments",
    "24.16558145": "Avaya Inc",
    "24.6083755": "Zektor",
    "12.123917685391360": "Zerospace ICT Services B.V.",
    "24.12616048": "Effigis GeoSolutions",
    "24.3472031": "Ruckus Wireless",
    "24.1619660": "We Corporation Inc.",
    "24.749": "DXO Telecom Co., Ltd.",
    "24.748": "Maschoff Design Engineering",
    "24.745": "CS Systemes De Securite - C3S",
    "24.744": "E.D.&A.",
    "24.747": "Pico Communications",
    "24.746": "Focus Enhancements",
    "24.741": "Timeware Ltd.",
    "24.740": "JC HYUN Systems, Inc.",
    "24.743": "CAB GmbH & Co KG",
    "24.742": "Gould Instrument Systems, Inc.",
    "12.123917677592576": "Private",
    "12.346865938432": "Bluestone Technology GmbH",
    "12.123917691514880": "SenSys (Design Electronics Ltd)",
    "12.123917677502464": "OOO Research and Production Center \"Computer Technologies\"",
    "24.15759299": "Intel Corporate",
    "24.2138855": "Lee-Dickens Ltd",
    "24.7428": "Zipit Wireless, Inc.",
    "24.7429": "Cooper Lighting Solutions",
    "24.540534": "Intellian Technologies, Inc.",
    "24.7424": "Brivo Systems, LLC",
    "24.7425": "Neptune Digital",
    "24.7426": "Cybertech Telecom Development",
    "24.2149689": "Texas Instruments",
    "24.7420": "Sumitomo Electric Industries, Ltd",
    "24.7421": "Universal Electronics, Inc.",
    "24.7422": "Quartics Inc",
    "24.7423": "Napera Networks Inc",
    "24.7102964": "Sfr",
    "24.2618": "J-THREE INTERNATIONAL Holding Co., Ltd.",
    "24.2619": "GCT Semiconductor, Inc",
    "24.2614": "Synelec Telecom Multimedia",
    "24.2615": "Procera Networks, Inc.",
    "24.2616": "Apani Networks",
    "24.2617": "LoPA Information Technology",
    "24.2610": "Xsido Corporation",
    "24.2611": "Emulex Corporation",
    "24.2612": "Identicard Systems Incorporated",
    "24.2613": "Xilinx",
    "24.5866": "Intel Corporate",
    "24.5867": "Intel Corporate",
    "24.5864": "Sigma Designs, Inc.",
    "24.5865": "Tiba Medical Inc",
    "24.5862": "Giga-Byte Technology Co.,Ltd.",
    "24.5863": "Dynamix Promotions Limited",
    "24.5860": "Vanguard Security Engineering Corp.",
    "24.5861": "Fordley Development Limited",
    "24.9732": "Audio Processing Technology Ltd",
    "24.9733": "CC Systems AB",
    "24.9730": "SMART Temps LLC",
    "24.9731": "Shenzhen Wistar Technology Co., Ltd",
    "24.9736": "Apple, Inc.",
    "24.9737": "Phyllis Co., Ltd.",
    "24.5868": "Elitegroup Computer Systems Co.,Ltd.",
    "24.5869": "Utility, Inc",
    "24.1331281": "SHARP Corporation",
    "24.13151900": "Shanghai TYD Elecronic Technology Co. Ltd",
    "12.123917685551104": "Osaki Datatech Co., Ltd.",
    "24.16314498": "Nag Llc",
    "12.123917690613760": "Lithe Technology, LLC",
    "12.71297883406336": "Special Measurements Labs LLC",
    "20.216145525211136": "GL Solutions Inc.",
    "24.6593665": "Qualcomm Inc.",
    "24.7634358": "Aliwei Communications, Inc",
    "24.13920097": "XonTel Technology Co.",
    "20.228784617816064": "Tecnosoft srl",
    "12.123917685571584": "aelettronica group srl",
    "12.346864304128": "Tritech International Ltd",
    "24.14168959": "Office FA.com Co.,Ltd.",
    "12.123917675724800": "Raven Systems Design, Inc",
    "24.12600016": "Juniper Networks",
    "24.7646393": "Ubiquiti Networks Inc.",
    "24.3548": "Vac",
    "20.53091444457472": "Hangzhou Ev-Tech Co.,Ltd",
    "24.7351327": "Wisol",
    "24.5250605": "StreamUnlimited Engineering GmbH",
    "24.13389834": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.10502445": "Lansen Systems AB",
    "12.123917686132736": "Vitec Multimedia",
    "24.12342319": "Intel Corporate",
    "24.7120081": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12110394": "Dell Inc.",
    "12.123917691039744": "Chengdu Lingya Technology Co., Ltd.",
    "12.123917690368000": "S.I.C.E.S. srl",
    "24.15495643": "Shenzhen Baichuan Digital Technology Co., Ltd.",
    "24.3197944": "Samsung Electronics Co.,Ltd",
    "20.145211810381824": "Kessel Ag",
    "12.123917683757056": "Han Chang",
    "24.8469": "PHYWE Systeme GmbH & Co. KG",
    "24.8468": "Hylab Technology Inc.",
    "24.4758082": "Technomate Ltd",
    "24.8461": "Samsin Innotec",
    "24.8460": "Cymtec Systems, Inc.",
    "24.8463": "Cernium Corp",
    "24.8462": "Orpak Systems L.T.D.",
    "24.8465": "Uniphone Inc.",
    "24.8464": "Clearbox Systems",
    "24.8467": "Padtec S/A",
    "24.8466": "Wiscom System Co.,Ltd",
    "24.2631334": "PBR electronics GmbH",
    "24.3472569": "Hewlett Packard Enterprise",
    "24.9969806": "Definium Technologies",
    "24.5134": "Nortel Networks",
    "24.5135": "Federal State Unitary Enterprise Leningrad R&D Institute of",
    "24.5136": "Suzhou Keda Technology CO.,Ltd",
    "24.5137": "Deutschmann Automation GmbH & Co. KG",
    "24.5130": "WEPIO Co., Ltd.",
    "24.5131": "First International Computer, Inc.",
    "24.5132": "Gkb Cctv Co., Ltd.",
    "24.5133": "Nortel Networks",
    "24.6307530": "Cambridge Medical Robotics Ltd",
    "12.123917682978816": "Soniclean Pty Ltd",
    "24.5138": "S-TEC electronics AG",
    "24.5139": "Trebing & Himstedt Prozeßautomation GmbH & Co. KG",
    "12.123917684408320": "Wi6labs",
    "24.11305751": "Shenzhen Furtunetel Communication Co., Ltd",
    "24.13400929": "Nikkiso Co., Ltd.",
    "24.8186798": "Ridgeline Instruments",
    "24.8962967": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.346863734784": "Harper Chalice Group Limited",
    "24.13164233": "Coriant",
    "24.7662312": "ARRIS Group, Inc.",
    "24.14735290": "Texas Instruments",
    "24.14984687": "Tron Link Electronics Co., Ltd.",
    "24.6563414": "Shenzhen Fanweitai Technology Service Co.,Ltd",
    "24.52476": "Cisco Systems, Inc",
    "24.6051610": "ARRIS Group, Inc.",
    "24.13926858": "iDevices, LLC",
    "20.128510963220480": "UTU Oy",
    "24.3996227": "Huawei Technologies Co.,Ltd",
    "24.6816440": "Compal Broadband Networks, Inc.",
    "24.3434218": "Vertu International Corporation Limited",
    "24.2414881": "Avaya Inc",
    "24.11036287": "Apple, Inc.",
    "12.346862440448": "Uwe Schneider GmbH",
    "24.2898369": "Ciena Corporation",
    "24.1583877": "8D Technologies",
    "24.7918833": "IEEE Registration Authority",
    "12.346852368384": "MotionIO",
    "24.5556703": "Fn-Link Technology Limited",
    "12.123917688201216": "Uni Dimenxi Sdn Bhd",
    "12.123917687721984": "Aplex Technology Inc.",
    "12.346864517120": "AGWTech Ltd",
    "20.272805461491712": "AVPro Global Holdings LLC",
    "12.71297883824128": "NanoPulse, Inc.",
    "24.9737632": "Google, Inc.",
    "24.2649812": "Huawei Technologies Co.,Ltd",
    "20.273272597905408": "KLATU Networks Inc",
    "24.15991336": "Samsung Electro-Mechanics(Thailand)",
    "24.14195515": "Huawei Technologies Co.,Ltd",
    "12.123917682737152": "Qualitteq Llc",
    "12.123917684899840": "Bintel AB",
    "24.315758": "IEEE Registration Authority",
    "12.123917689372672": "YUYAMA MFG Co.,Ltd",
    "24.1840415": "Huawei Technologies Co.,Ltd",
    "12.346866978816": "Norwia AS",
    "12.123917691019264": "Season Electronics Ltd",
    "24.16521108": "InterCreative Co., Ltd",
    "24.15998502": "Simon-Kaloi Engineering",
    "20.268902722437120": "OmniNet",
    "24.10807715": "Honest Technology Co., Ltd",
    "12.123917676912640": "Xacti Corporation",
    "24.7081834": "JDC Tech Co., Ltd.",
    "12.123917678882816": "Movicom Electric LLC",
    "24.11829957": "Relay2, Inc.",
    "24.3688380": "Huawei Technologies Co.,Ltd",
    "24.14683866": "Cisco Systems, Inc",
    "12.123917678292992": "Tattile Srl",
    "12.71297883590656": "Parallel Wireless",
    "24.14471969": "HUMAX Co., Ltd.",
    "24.13654467": "CeraMicro Technology Corporation",
    "24.4204057": "Apple, Inc.",
    "24.16405871": "SymbolicIO",
    "12.123917681172480": "Grossenbacher Systeme AG",
    "24.14736213": "Dell Inc.",
    "20.5297553801216": "Elotec Fischer Elektronik GmbH",
    "24.2418090": "Philips Oral Healthcare, Inc.",
    "24.15241892": "LG Electronics (Mobile Communications)",
    "12.123917689516032": "DETECT Australia",
    "24.16296291": "Compal Information (Kunshan) Co., Ltd.",
    "24.35856": "Black Box Corp.",
    "12.123917685686272": "Shen Zhen Huawang Technology Co; Ltd",
    "12.123917682417664": "APG Cash Drawer, LLC",
    "24.15752585": "Dycon Limited",
    "12.123917680041984": "Agilack",
    "12.123917684027392": "ControlWorks, Inc.",
    "24.9746868": "Aptos Technology",
    "24.11810769": "IEEE Registration Authority",
    "24.10789995": "Huawei Technologies Co.,Ltd",
    "24.12631496": "Samsung Electronics Co.,Ltd",
    "24.9211050": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.1616669": "Samsung Electronics Co.,Ltd",
    "12.346864218112": "Enatel",
    "24.11059004": "HDV Phoelectron Technology Limited",
    "12.123917680209920": "Foxtel srl",
    "24.4510410": "Anvia TV Oy",
    "24.13640108": "Yo Labs LLC",
    "24.14467291": "Shenzhen Kaiboer Technology Co., Ltd.",
    "24.7135262": "Modcam AB",
    "12.123917682200576": "Abbott Diagnostics Technologies AS",
    "24.10488140": "CenturyLink",
    "24.3451797": "Espressif Inc.",
    "24.3936320": "daesung network",
    "24.14698707": "Mobiwire Mobiles (Ningbo) Co.,Ltd",
    "24.8127403": "Xiaomi Communications Co Ltd",
    "24.13156350": "Huawei Device Co., Ltd.",
    "24.13141153": "Cisco Systems, Inc",
    "24.7084049": "Decatur Electronics",
    "12.123917690990592": "comtac AG",
    "12.346859769856": "Dain",
    "24.15205274": "Samsung Electronics Co.,Ltd",
    "24.16253998": "Samsung Electro-Mechanics(Thailand)",
    "20.229631756075008": "peiker CEE",
    "12.123917685067776": "Logicube Inc",
    "20.452700667904": "miliwave",
    "20.14286629896192": "CTK Contact Electronics co., Ltd.",
    "24.6349607": "Tp-Link Technologies Co.,Ltd.",
    "24.3172997": "Samsung Electronics Co.,Ltd",
    "12.346867732480": "Plasmatronics pty ltd",
    "12.119269908480": "Balter Security GmbH",
    "12.123917682204672": "Ametek Solidstate Controls",
    "24.11839673": "Cisco Systems, Inc",
    "24.9439802": "PSG Plastic Service GmbH",
    "12.123917683638272": "Sinoembed",
    "24.10282549": "Nintendo Co., Ltd.",
    "24.11335884": "ARRIS Group, Inc.",
    "24.15745845": "DVN(Shanghai)Ltd.",
    "24.1862": "TURCK, Inc.",
    "24.32858": "Tulip Computers International BV",
    "24.32859": "Condor Systems, Inc.",
    "24.9482720": "Beijing Nebula Link Technology Co., Ltd",
    "24.1863": "Mecalc",
    "24.32852": "Frontier Technologies Corp.",
    "24.32853": "Fermilab",
    "24.32850": "Technically Elite Concepts",
    "24.32851": "Intellicom, Inc.",
    "24.32856": "Printer Systems Corp.",
    "24.1860": "Unico, Inc.",
    "12.346862919680": "RealVision Inc.",
    "24.32855": "Adsoft, Ltd.",
    "24.1861": "Radlan Computer Communications Ltd.",
    "20.452702765056": "Suzhou Radiant Lighting Technology Co.,Ltd",
    "24.1866": "Carl Valentin GmbH",
    "20.88010608607232": "SPD Development Company Ltd",
    "20.277783362142208": "Shenzhen Elebao Technology Co., Ltd",
    "24.5364": "DekTec Digital Video B.V.",
    "24.1864": "The Imaging Source Europe",
    "24.5366": "Juniper Networks",
    "24.6566921": "BITwave Pte Ltd",
    "24.2634437": "HUMAX Co., Ltd.",
    "12.123917687414784": "Consarc Corporation",
    "24.11566704": "Zadara Storage Ltd.",
    "12.71297884659712": "Aplex Technology Inc.",
    "20.149935439740928": "Shenzhen Speedrun Technologies Co.,Ltd.",
    "12.346865172480": "Broadsoft PacketSmart, Inc.",
    "12.346854813696": "TECNEW Electronics Engineering Cr., Ltd.",
    "24.14178287": "Samsung Electronics Co.,Ltd",
    "24.8081": "DBS Lodging Technologies, LLC",
    "24.8080": "Actiontec Electronics, Inc",
    "24.1844705": "China Mobile IOT Company Limited",
    "24.5523973": "wenglor sensoric gmbh",
    "12.123917690097664": "Birtech Technology",
    "12.123917679591424": "Guilin Tryin Technology Co.,Ltd",
    "12.123917685469184": "Unicom Global, Inc.",
    "24.2117863": "AMPAK Technology,Inc.",
    "24.8973596": "Cudo Communication Co., Ltd.",
    "24.14681830": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.3725258": "7Hugs Labs",
    "20.220092034973696": "BIOT Sp. z o.o.",
    "12.346852438016": "Curtis, Inc.",
    "12.123917679747072": "ChongQing JianTao Technology Co., Ltd.",
    "24.9477939": "EFM Networks",
    "24.8177417": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.11030079": "Hitron Technologies. Inc",
    "24.14973": "Cisco Systems, Inc",
    "20.83999497125888": "Roost",
    "12.123917681246208": "FDSTiming",
    "24.2404873": "Avaya Inc",
    "24.2892895": "Babolat VS",
    "24.15785585": "Shanghai EDO Technologies Co.,Ltd.",
    "24.13396888": "Minetec Wireless Technologies",
    "12.346858373120": "ASiS Technologies Pte Ltd",
    "24.41025": "Inficon",
    "24.41024": "Apple (PCI Mac)",
    "24.41027": "American Technology Labs, Inc.",
    "24.41026": "Spur Products Corp.",
    "24.41021": "Opto-22",
    "24.41020": "Eg&G Nuclear Instruments",
    "24.41023": "Computer Society Microprocessor & Microprocessor Standards C",
    "24.41022": "Atm Forum",
    "24.1345170": "Tp-Link Technologies Co.,Ltd.",
    "24.41029": "PHOENIX CONTACT Electronics GmbH",
    "24.41028": "Ntt It Co., Ltd.",
    "24.8183158": "Suzhou Hanming Technologies Co., Ltd.",
    "24.11298653": "Intel Corporate",
    "24.1861758": "Shenzhen Qihu Intelligent Technology Company Limited",
    "24.8706863": "Sierra Wireless",
    "24.15478350": "Hitachi-Lg Data Storage Inc",
    "12.346861580288": "Alfred Kuhse GmbH",
    "24.1088890": "Huawei Device Co., Ltd.",
    "20.220327873347584": "Optictimes Co.,Ltd",
    "24.11042872": "ShenZhen MovingComm Technology Co., Limited",
    "12.123917680082944": "Versilis Inc.",
    "24.5013346": "Nokia Corporation",
    "20.225207122919424": "Siemens AG, MO MLT BG",
    "24.10022270": "eero inc.",
    "20.264036220403712": "Raysgem Electronics and Technology Co.Ltd",
    "24.7123792": "Anhui comhigher tech co.,ltd",
    "20.110162579816448": "Dongguan Huili electroacoustic Industrial Co.,ltd",
    "24.11020620": "Precision Optical Transceivers, Inc.",
    "24.1057401": "ZeroDesktop, Inc.",
    "20.88010611752960": "Sinope technologies Inc",
    "24.2110060": "Samsung Electronics Co.,Ltd",
    "20.149945508167680": "Inor Process AB",
    "12.123917680095232": "Temperature@lert",
    "24.6144": "Unigrand Ltd",
    "24.14217563": "Espressif Inc.",
    "24.15773730": "TOHO Electronics INC.",
    "12.123917682548736": "Tofwerk Ag",
    "24.6313060": "Eyedro Green Solutions Inc.",
    "24.13171993": "Texas Instruments",
    "24.13377476": "InVue",
    "12.123917675479040": "Orlaco Products B.V.",
    "20.105645772636160": "PSS Co., Ltd",
    "12.346862854144": "Harris Designs of NRV, Inc.",
    "12.71297883545600": "Ultra Electronics Flightline Systems",
    "12.346868609024": "Vremya-CH JSC",
    "24.2374274": "Irts",
    "12.123917690626048": "Vaunix Technology Corporation",
    "24.13110804": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "12.123917689909248": "Eurotempest AB",
    "12.346861477888": "Wireless Cables Inc.",
    "24.12332527": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.1588407": "Huawei Device Co., Ltd.",
    "24.45262": "Viveris Technologies",
    "24.14988027": "Apple, Inc.",
    "24.5275229": "Arcadyan Technology Corporation",
    "24.1097852": "Blu-ray Disc Association",
    "24.44794": "Murata Manufacturing Co., Ltd.",
    "24.7890606": "ZTEC Instruments, Inc.",
    "24.6076619": "Allis Communications",
    "24.14421037": "Weifang Goertek Electronics Co.,Ltd",
    "24.1108206": "PhaseSpace",
    "12.346862891008": "IPcontrols GmbH",
    "20.229631759220736": "Mitsubishi Electric US, Inc.",
    "20.150399875022848": "shenzhen franklin ESS technology CO.,Ltd",
    "24.10500344": "OURA Health Oy",
    "24.9240151": "Apple, Inc.",
    "12.346854219776": "BAE SYSTEMS Controls",
    "12.346856976384": "Advanced Driver Information Technology GmbH",
    "12.71297884016640": "Logical Product",
    "12.123917675307008": "Gecko Robotics Inc",
    "20.84187715469312": "Annapurna labs",
    "24.12327652": "Huawei Device Co., Ltd.",
    "20.220515726786560": "Focalcrest, Ltd.",
    "24.9462587": "Samsung Electronics Co.,Ltd",
    "12.346862977024": "i-RED Infrarot Systeme GmbH",
    "24.13657412": "Apple, Inc.",
    "24.15262178": "Energotest",
    "24.15262177": "Gemtek Technology Co., Ltd.",
    "20.154019636969472": "Reynaers Aluminium",
    "24.10803620": "neXus ID Solutions AB",
    "20.45033664282624": "Herbert Waldmann GmbH & Co. KG",
    "12.123917678837760": "JamHub Corp.",
    "24.11836639": "Apple, Inc.",
    "24.9483898": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.16601": "American Megatrends Inc.",
    "24.16600": "Ocean Office Automation Ltd.",
    "24.16603": "Advanced Technical Solutions",
    "24.16602": "Telspec Ltd",
    "24.16605": "Hong Technologies",
    "24.16604": "Tritec Electronic Gmbh",
    "24.16607": "Digalog Systems, Inc.",
    "24.16606": "Elsag Datamat spa",
    "24.16609": "Marner International, Inc.",
    "24.16608": "Atomwide Ltd.",
    "12.123917687365632": "Deymed Diagnostic",
    "12.346858356736": "Info-Chip Communications Ltd.",
    "20.167137988575232": "Ormazabal Protection&Automation",
    "12.346864168960": "ARKRAY, Inc. Kyoto Laboratory",
    "12.346867818496": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.1900075": "Amazon Technologies Inc.",
    "12.346866302976": "Microcubs Systems Pvt Ltd",
    "12.346857590784": "Smartcom-Bulgaria AD",
    "24.10804016": "Liteon Technology Corporation",
    "24.10804014": "Kingspan Environmental Ltd",
    "24.4250781": "Hajen",
    "24.14694463": "zte corporation",
    "12.346854240256": "Willowglen Systems Inc.",
    "24.11032640": "Cambridge Industries(Group) Co.,Ltd.",
    "24.1351375": "Dell Inc.",
    "24.5509356": "Microchip Technology Inc.",
    "24.4768688": "Pharos Co.Ltd",
    "24.1853": "Nanjing Postel Telecommunications Co., Ltd.",
    "24.1852": "Telecom Design",
    "24.1851": "Tenovis GmbH & Co KG",
    "24.1850": "Inventel",
    "24.1857": "Sierra Automated Systems",
    "24.1856": "Buffalo.Inc",
    "24.1855": "Woojyun Systec Co., Ltd.",
    "24.1854": "China Great-Wall Computer Shenzhen Co., Ltd.",
    "24.40686": "Positivo Tecnologia S.A.",
    "24.1859": "Chelsio Communications",
    "24.1858": "Ormazabal",
    "12.71297883963392": "Act",
    "20.229108132872192": "Montblanc-Simplo GmbH",
    "20.206931534807040": "Auralic Limited",
    "12.123917677740032": "Teko Telecom Srl",
    "24.9228041": "Dell EMC",
    "24.9759532": "Google, Inc.",
    "24.4750042": "Hangzhou H3C Technologies Co., Limited",
    "20.168210326355968": "Shenzhen Incar Technology Co., Ltd.",
    "24.2102969": "Intel Corporate",
    "24.16301121": "Rosslare Enterprises Limited",
    "12.123917690679296": "octoScope",
    "24.794031": "Xiaomi Communications Co Ltd",
    "24.13415770": "Fraunhofer ITWM",
    "24.7363374": "M2Communication Inc.",
    "24.4745290": "Intel Corporate",
    "24.7867632": "Cisco Systems, Inc",
    "24.16016080": "Not for Radio, LLC",
    "24.13685424": "Apple, Inc.",
    "24.8951381": "Zitte corporation",
    "12.123917682159616": "DEUTA-WERKE GmbH",
    "12.123917687136256": "N A Communications LLC",
    "24.12614": "Juniper Networks",
    "12.346852347904": "Innocor LTD",
    "20.52819393511424": "OOO \"Microlink-Svyaz\"",
    "24.14203692": "Apple, Inc.",
    "24.59706": "AzureWave Technology Inc.",
    "12.123917691629568": "Design SHIFT",
    "24.8660443": "Samsung Electronics Co.,Ltd",
    "24.6075737": "CNEX Labs",
    "24.6313465": "Espressif Inc.",
    "24.10763258": "IEEE Registration Authority",
    "24.11585709": "People Power Company",
    "12.123917684453376": "Cymtec Ltd",
    "12.123917676134400": "Project H Pty Ltd",
    "12.346864132096": "Phytec Messtechnik GmbH",
    "24.155788": "3Com",
    "12.123917675839488": "ADMiTAS CCTV Taiwan Co. Ltd",
    "24.155782": "Logic Replacement Tech. Ltd.",
    "24.4732206": "Shenzhen Wolck Network Product Co.,LTD",
    "24.16039660": "Apple, Inc.",
    "12.123917684285440": "MicroElectronics System Co.Ltd",
    "24.9478747": "Apple, Inc.",
    "24.6609793": "Syszone Co., Ltd.",
    "24.2908662": "Apple, Inc.",
    "20.97400598298624": "shenzhen UDD Technologies,co.,Ltd",
    "24.9918": "Schoonderbeek Elektronica Systemen B.V.",
    "24.9919": "ShenZhen Temobi Science&Tech Development Co.,Ltd",
    "24.9912": "Actiontec Electronics, Inc",
    "24.9913": "Dell Inc.",
    "24.9910": "Askey Computer Corp",
    "24.9911": "Kingston Technology Company, Inc.",
    "24.9916": "General Jack Technology Ltd.",
    "24.9917": "JTEC Card &amp; Communication Co., Ltd",
    "24.9914": "ARRIS Group, Inc.",
    "24.9915": "Apple, Inc.",
    "24.4579": "Thomson, Inc.",
    "24.4578": "Hua Jung Components Co., Ltd.",
    "12.346865717248": "AeroMechanical Services Ltd, FLYHT",
    "12.346867306496": "traffic network solutions s.l",
    "24.4573": "FROMUS TEC. Co., Ltd.",
    "24.4572": "Glunz & Jensen",
    "24.4571": "Land-Cellular Corporation",
    "24.4570": "Vivaas Technology Inc.",
    "24.4577": "Arcelik A.S",
    "24.4576": "U-MEDIA Communications, Inc.",
    "24.4575": "Current Energy",
    "24.4574": "Eurilogic",
    "24.10247573": "VTC Electronics Corp.",
    "24.7658120": "Tp-Link Technologies Co.,Ltd.",
    "20.198611830833152": "Hengkang（Hangzhou）Co.,Ltd",
    "24.13566": "Huawei Technologies Co.,Ltd",
    "12.123917679087616": "Ateme",
    "24.13382697": "Tp-Link Technologies Co.,Ltd.",
    "24.10525458": "China Mobile IOT Company Limited",
    "20.277958197510144": "Scopus International-Belgium",
    "12.123917677842432": "Dosepack India LLP",
    "24.7083118": "Huawei Technologies Co.,Ltd",
    "12.123917684555776": "Alere Technologies AS",
    "12.123917683990528": "Abb",
    "12.123917684224000": "WELT Corporation",
    "24.1594948": "Huawei Technologies Co.,Ltd",
    "12.346858795008": "Doble Engineering",
    "24.7909208": "Gimasi Sa",
    "24.49212": "Tower Tech S.R.L.",
    "24.49213": "Wiesemann & Theis Gmbh",
    "24.49210": "Men-Mikro Elektronik Gmbh",
    "24.49211": "Multiaccess Computing Corp.",
    "24.49216": "Ecci",
    "24.49217": "Digital Transmission Systems",
    "24.49214": "Fa. Gebr. Heller Gmbh",
    "24.49215": "Stores Automated Systems, Inc.",
    "12.123917679292416": "Figment Design Laboratories",
    "24.49218": "Datalux Corp.",
    "24.49219": "Stratacom",
    "24.13119047": "Suzhou SmartChip Semiconductor Co., LTD",
    "20.31873089667072": "Umeox Innovations Co.,Ltd",
    "12.123917679407104": "Flexsolution APS",
    "12.123917682057216": "Integrotech sp. z o.o.",
    "12.346858999808": "Datamars SA",
    "24.13666901": "Skullcandy",
    "24.6617456": "Kenade Electronics Technology Co.,LTD.",
    "20.13692014952448": "Synaccess Networks",
    "12.346866069504": "DSP4YOU LTd",
    "12.346864103424": "RO.VE.R. Laboratories S.p.A",
    "12.123917678346240": "iOne",
    "24.2907105": "Centripetal Networks, Inc",
    "20.22126509686784": "Swiss Electronic (Shenzhen) Co., Ltd",
    "12.346866720768": "Sequtec Inc",
    "12.123917685051392": "LayTec AG",
    "24.1051561": "Intel Corporate",
    "24.14435510": "Samsung Electronics Co.,Ltd",
    "24.8793": "Guangzhou New Postcom Equipment Co.,Ltd.",
    "24.8792": "Taiyo Yuden Co., Ltd.",
    "24.8791": "3Com Europe Ltd",
    "24.8790": "Cisco Systems, Inc",
    "24.8797": "Digicable Network India Pvt. Ltd.",
    "24.8796": "Multimedia & Communication Technology",
    "24.8795": "Teradici Corporation",
    "24.8794": "Garde Security AB",
    "24.8799": "Liteon Technology Corporation",
    "24.8798": "Uwin Technologies Co.,LTD",
    "24.8431510": "Neurotek LLC",
    "12.123917688246272": "Waldo System",
    "12.346867068928": "FUJI DATA SYSTEM Co., Ltd",
    "24.2937153": "IEEE Registration Authority",
    "24.8400969": "Liteon Technology Corporation",
    "12.346853687296": "N&P Technologies",
    "24.11064264": "Topcon Electronics GmbH & Co. KG",
    "24.14481593": "zte corporation",
    "24.2139": "Hanbit Electronics Co., Ltd.",
    "24.2138": "IntiGate Inc.",
    "24.1066430": "Norphonic AS",
    "24.12063538": "AMPAK Technology,Inc.",
    "24.2135": "Polaris Networks, Inc.",
    "24.2134": "Gamatronic Electronic Industries Ltd.",
    "24.2137": "ShenZhen Unitone Electronics Co., Ltd.",
    "24.2136": "Novatechnology Inc.",
    "24.2131": "Schleicher GmbH & Co. Relaiswerke KG",
    "24.2130": "Davolink Co. Inc.",
    "24.2133": "NASA-Goddard Space Flight Center",
    "24.2132": "Netronix, Inc.",
    "24.3182966": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346853986304": "AIOI Systems Co., Ltd.",
    "24.2136809": "Volacomm Co., Ltd",
    "24.4774736": "Huawei Technologies Co.,Ltd",
    "24.2635320": "IEEE Registration Authority",
    "24.12379711": "Samsung Electronics Co.,Ltd",
    "24.5002909": "Apple, Inc.",
    "20.45069240369152": "Galileo, Inc.",
    "20.62366871977984": "Umlogics",
    "12.123917675335680": "Jt",
    "20.268341255077888": "Shenzhen headsun technology",
    "24.5294015": "Tp-Link Technologies Co.,Ltd.",
    "12.346860650496": "Wireless Value BV",
    "12.123917689700352": "Combilent",
    "24.10785830": "Elgama-Elektronika Ltd.",
    "12.123917690843136": "RoyalShield Technologies India Private Limited",
    "24.16534950": "Huawei Technologies Co.,Ltd",
    "24.11553337": "Beijing Roborock Technology Co., Ltd.",
    "24.9232121": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.123917682798592": "Oti Ltd",
    "12.123917687103488": "Nexus Tech. VN",
    "20.159304566964224": "ShenZhen Temwey Innovation Technology Co.,Ltd.",
    "24.7603621": "Tp-Link Technologies Co.,Ltd.",
    "24.10527941": "Intel Corporate",
    "20.194363559772160": "Private",
    "24.2791": "Eliop S.A.",
    "24.2790": "Elitegroup Computer Systems Co.,Ltd.",
    "24.2793": "AirVast Technology Inc.",
    "24.2792": "Cathay Roxus Information Technology Co. LTD",
    "24.2795": "Tp-Link Technologies Co.,Ltd.",
    "24.2794": "Adam Elektronik Ltd. Şti",
    "24.2797": "HARTING Electronics GmbH",
    "24.2796": "Koatsu Gas Kogyo Co., Ltd.",
    "24.2799": "Otrum Asa",
    "24.297055": "Huawei Technologies Co.,Ltd",
    "24.24655": "Tattile SRL",
    "24.24654": "Cycle Computer (Sun MotherBoard Replacements)",
    "24.24653": "Mmc Networks, Inc.",
    "24.24652": "Sagemcom Broadband SAS",
    "24.24651": "Safe-com GmbH & Co. KG",
    "24.24650": "Saic Ideas Group",
    "24.3442218": "Heinz Lackmann GmbH & Co KG",
    "24.539": "Kollmorgen-Servotronix",
    "24.9679": "Nokia Danmark A/S",
    "24.9678": "InnerSpace",
    "24.532": "Dtvro",
    "24.533": "Cotas Computer Technology A/B",
    "24.530": "SierraCom",
    "24.531": "S.D.E.L.",
    "24.536": "Advanced Scientific Corp",
    "24.537": "Paralon Technologies",
    "24.534": "ESI (Extended Systems, Inc)",
    "24.535": "Cisco Systems, Inc",
    "12.123917683392512": "HeadsafeIP PTY LTD",
    "24.1875084": "Juniper Networks",
    "24.1448": "Wyle Electronics",
    "24.1449": "Princeton Networks, Inc.",
    "24.1444": "Lucid Voice Ltd.",
    "24.1445": "Kott",
    "24.1446": "Extron Electronics",
    "24.1447": "HYPERCHIP Inc.",
    "24.1440": "MOBILINE Kft.",
    "24.1441": "Zenocom",
    "24.1442": "CELOX Networks",
    "24.1443": "QEI, Inc.",
    "12.123917680664576": "Data Informs LLC",
    "24.8172124": "Acacia Communications",
    "12.123917682151424": "Visual Engineering Technologies Ltd",
    "20.5297545412608": "Dspread Technology (Beijing) Inc.",
    "24.8422023": "ATCOM Technology Co.Ltd.",
    "24.11581834": "Netgear",
    "24.12113839": "Murata Manufacturing Co., Ltd.",
    "24.8429467": "Sichuan AI-Link Technology Co., Ltd.",
    "24.9457866": "Hitron Technologies. Inc",
    "24.2869": "Quad Bit System co., Ltd.",
    "24.2868": "ShangHai Broadband Technologies CO.LTD",
    "24.2863": "bplan GmbH",
    "24.2862": "Cal-Comp Electronics & Communications Company Ltd.",
    "24.2861": "Danfoss Inc.",
    "24.2860": "Eiki Industrial Co. Ltd.",
    "24.2867": "Vivato Technologies",
    "24.2866": "Vormetric, Inc.",
    "24.2865": "Yantai ZhiYang Scientific and technology industry CO., LTD",
    "24.2864": "Beijing Gongye Science & Technology Co.,Ltd",
    "24.8715934": "RTC Industries, Inc.",
    "12.346856710144": "DRDC Valcartier",
    "12.346863230976": "BFI Industrie-Elektronik GmbH & Co.KG",
    "12.346853363712": "Tecwings GmBh",
    "24.10491944": "Apple, Inc.",
    "24.9723700": "Renesas Electronics (Penang) Sdn. Bhd.",
    "20.250817677361152": "B Meters S.R.L.",
    "24.1597996": "Sagemcom Broadband SAS",
    "12.346863947776": "Ams",
    "24.10231460": "Renesas Electronics (Penang) Sdn. Bhd.",
    "12.346866286592": "P4Q Electronics",
    "20.52912298393600": "Origami Group Limited",
    "24.4203331": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.7641461": "IBM Corporation",
    "24.3191992": "LG Electronics",
    "24.8447568": "Apple, Inc.",
    "24.8129104": "J-MEX Inc.",
    "24.2670810": "Juniper Networks",
    "24.14991686": "Objetivos y Servicios de Valor A",
    "24.6093692": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "24.4465532": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.346861854720": "RCS Communication Test Systems Ltd.",
    "24.13947474": "Intel Corporate",
    "24.13157962": "Flextronics Tech.(Ind) Pvt Ltd",
    "24.3962186": "ARRIS Group, Inc.",
    "24.12089283": "METER Group",
    "24.6837493": "enLighted Inc",
    "24.8891": "beyerdynamic GmbH & Co. KG",
    "24.3938685": "Ericsson AB",
    "24.2669017": "Radisys Corporation",
    "24.7101662": "SunReports, Inc.",
    "24.2680240": "Huawei Technologies Co.,Ltd",
    "24.4964": "Paradigm Technology Inc..",
    "24.4965": "Nortel Networks",
    "24.4966": "Neturity Technologies Inc.",
    "24.4967": "Narayon. Co., Ltd.",
    "24.4960": "Cisco Systems, Inc",
    "24.4961": "Biospace Co., Ltd.",
    "24.4962": "ShinHeung Precision Co., Ltd.",
    "24.4963": "Verascape, Inc.",
    "24.8230": "Amkly Systems, Inc.",
    "24.8231": "Ming Fortune Industry Co., Ltd",
    "24.8232": "West Egg Systems, Inc.",
    "24.8233": "TeleProcessing CSU/DSU (now owned by ADC/Kentrox)",
    "24.4968": "Saab Danmark A/S",
    "24.4969": "Honda Electron Co., LED.",
    "24.8236": "Welltronix Co., Ltd.",
    "24.8237": "Taiyo Corporation",
    "24.7078665": "Cisco Systems, Inc",
    "24.36939": "Gemtek Technology Co., Ltd.",
    "24.36938": "Concur System Technologies",
    "24.36935": "Giga Fast E. Ltd.",
    "24.36934": "Dexdyne, Ltd.",
    "24.36937": "Entridia Corporation",
    "24.36936": "Zeal Corporation",
    "12.346855596032": "biokeysystem",
    "24.36930": "ECCS, Inc.",
    "24.36933": "Marconi Communications",
    "24.36932": "Assured Digital, Inc.",
    "12.346866667520": "Friend Spring Industrial Co., Ltd.",
    "24.5792099": "Quantum Networks (SG) Pte. Ltd.",
    "24.5566815": "Sichuan AI-Link Technology Co., Ltd.",
    "24.1869795": "Tecno Mobile Limited",
    "24.10511503": "Askey Computer Corp",
    "24.57464": "Berkeley Networks",
    "24.3725359": "zte corporation",
    "24.12366379": "EM-Tech",
    "20.110162585059328": "SHEN ZHEN FUCHANG TECHNOLOGY Co.,Ltd.",
    "12.346860457984": "Sicon srl",
    "12.123917685297152": "OS42 UG (haftungsbeschraenkt)",
    "24.1887446": "Xiaomi Communications Co Ltd",
    "20.277981932027904": "Grandway Technology (Shenzhen) Limited",
    "24.8181828": "Enmotus Inc",
    "12.123917679648768": "Zamir Recognition Systems Ltd.",
    "24.9193406": "Dell Inc.",
    "12.123917675565056": "Kazdream Technologies LLP",
    "24.4722412": "Tp-Link Technologies Co.,Ltd.",
    "24.11037295": "Rim",
    "20.35227541962752": "Netinovo Technologies(Shenzhen) Ltd",
    "24.11322651": "Zhuhai Pantum Electronics Co., Ltd.",
    "24.2948523": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.346862419968": "ethermetrics",
    "24.10806679": "Gessler GmbH",
    "24.10757717": "LTI Motion (Shanghai) Co., Ltd.",
    "24.13916850": "Galleon Systems",
    "24.2920743": "Eintechno Corporation",
    "24.11065857": "Shenzhen NUOLIJIA Digital Technology Co.,Ltd",
    "20.124214342320128": "Fixstars Corporation",
    "24.14208570": "Xiaomi Communications Co Ltd",
    "24.14420300": "Huawei Technologies Co.,Ltd",
    "12.346853433344": "Tensilica, Inc.",
    "24.14432082": "Ge",
    "20.62663552925696": "FUBA Automotive Electronics GmbH",
    "24.3675857": "Samsung Electronics Co.,Ltd",
    "12.346852782080": "IMV Invertomatic",
    "20.212014918008832": "Samsara Networks Inc",
    "20.4471481434112": "Acentury",
    "24.2926748": "Creatz inc.",
    "12.346852986880": "DSS Networks, Inc.",
    "24.3453450": "Huawei Device Co., Ltd.",
    "24.5553747": "zte corporation",
    "24.588409": "vivo Mobile Communication Co., Ltd.",
    "24.11047910": "Jiangxi Jinggangshan Cking Communication Technology Co.,Ltd",
    "24.10259360": "Apple, Inc.",
    "24.12093352": "Step Ahead Innovations Inc.",
    "24.6863884": "Cisco Systems, Inc",
    "12.123917688696832": "Alto Aviation",
    "12.346867220480": "MCOPIA Co., Ltd",
    "12.123917681213440": "Creative Electronics Ltd",
    "20.193649355784192": "Uvax Concepts",
    "24.14168372": "Roku, Inc",
    "24.14429500": "Huawei Device Co., Ltd.",
    "12.346861289472": "Sandar Telecast AS",
    "12.346856194048": "Key Systems , Inc.",
    "24.10520300": "SAEHAN HITEC Co., Ltd",
    "12.346860818432": "Connection Electronics Ltd.",
    "12.123917678374912": "Laborie",
    "12.123917676523520": "National TeleConsultants LLC",
    "24.15240488": "Apple, Inc.",
    "12.123917679964160": "Plazma-T",
    "24.15779920": "Huawei Technologies Co.,Ltd",
    "24.9745100": "Pioneer Corporation",
    "24.13950807": "Alpinion Medical Systems",
    "12.123917684121600": "NPF Modem, LLC",
    "24.5798498": "Texas Instruments",
    "24.1078922": "EoCell",
    "24.14474247": "TRP Systems BV",
    "24.8396562": "Microchip Technology Inc.",
    "24.15507149": "Artesyn Embedded Technologies",
    "24.524342": "Intergraph",
    "12.123917677400064": "Kospel S.A.",
    "24.12340829": "ZhongMiao Technology Co., Ltd.",
    "24.6339096": "nextLAP GmbH",
    "24.3709144": "Meritech Co.,Ltd",
    "24.9972926": "2Wire Inc",
    "24.10500891": "Inspire Tech",
    "24.8426143": "Apple, Inc.",
    "12.123917683245056": "Software Systems Plus",
    "20.242213038587904": "Annapurna labs",
    "12.346857320448": "Mandozzi Elettronica SA",
    "24.13425129": "SCR Engineers Ltd.",
    "12.346856718336": "HRZ data GmbH",
    "24.1878535": "Cisco Systems, Inc",
    "12.123917683163136": "Crde",
    "24.13424657": "Aiconn Technology Corporation",
    "12.346855424000": "Casa Systems Inc.",
    "24.2127503": "Apple, Inc.",
    "24.2127507": "Lenovo (Beijing) Limited.",
    "24.3155919": "Private",
    "24.11596195": "Fengfan (BeiJing) Technology Co., Ltd.",
    "24.15738295": "Samsung Electro-Mechanics(Thailand)",
    "20.84197034164224": "Private",
    "24.1072319": "DuroByte Inc",
    "12.346856529920": "Dsp Design",
    "24.9717680": "Beijing Eswin Computing Technology Co., Ltd",
    "12.123917677379584": "DAT Informatics Pvt Ltd",
    "24.1842915": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.123917688242176": "Xirgo Technologies LLC",
    "12.346867806208": "fibrisTerre GmbH",
    "24.6869575": "Portalis LC",
    "12.346863595520": "Redwood Systems",
    "24.713841": "Caterpillar Inc.",
    "24.14740322": "Shanghai Hulu Devices Co., Ltd",
    "12.346857517056": "Beta Lasermike Ltd",
    "24.2146413": "Shenzhen Spacetek Technology Co.,Ltd",
    "20.66426714980352": "Eltov System",
    "24.10533112": "Amgen U.S.A. Inc.",
    "24.4215309": "Pico Technology",
    "24.6610409": "Xiaomi Communications Co Ltd",
    "20.83999507611648": "Qingping Technology (Beijing) Co., Ltd.",
    "12.123917686013952": "Evco Spa",
    "24.5811755": "Huawei Device Co., Ltd.",
    "24.15784572": "Rlh Industries,Inc.",
    "24.5254040": "Es-tech International",
    "20.110373789237248": "Leontech Limited",
    "20.264625138434048": "Suzhou Pairlink Network Technology",
    "12.346855014400": "Admiral Secure Products, Ltd.",
    "20.57353100591104": "Beijing Vorx Telecommunications Co., Ltd.",
    "24.5038523": "Zhuhai HiFocus Technology Co., Ltd.",
    "12.123917687439360": "EDF Lab",
    "24.1313244": "Huawei Technologies Co.,Ltd",
    "20.31477075017728": "Exicom Tele-Systems Ltd.",
    "24.11587740": "Samsung Electronics Co.,Ltd",
    "24.12859706": "SMC Networks Inc",
    "12.123917686251520": "CL International",
    "20.159304570109952": "Totalone Technology Co., Ltd.",
    "12.346864685056": "Amatic Industries GmbH",
    "24.8147162": "Private",
    "24.14179835": "Private",
    "24.3967309": "Dr. Ing. h.c. F. Porsche AG",
    "24.11836053": "Shenzhen Boomtech Industrial Corporation",
    "12.71297883877376": "Enel doo Belgrade",
    "24.4242846": "Physio-Control, Inc",
    "24.11321838": "Xiaomi Communications Co Ltd",
    "24.7351936": "DIEHL Connectivity Solutions",
    "12.123917684727808": "GREATWALL Infotech Co., Ltd.",
    "24.6031215": "Thought Creator",
    "24.15517138": "Huawei Device Co., Ltd.",
    "12.346859184128": "Edgeware AB",
    "24.6581911": "Tp-Link Technologies Co.,Ltd.",
    "24.7081268": "Nokia",
    "24.8662548": "Silicon Laboratories",
    "12.71297883938816": "PCH Engineering A/S",
    "24.12120073": "Espressif Inc.",
    "24.3017": "Electroline Equipment",
    "12.346859687936": "Physical Acoustics Corporation",
    "12.71297883426816": "Tekelek Europe Ltd",
    "24.1073865": "Adatis GmbH & Co. KG",
    "24.1634629": "NetComm Wireless Limited",
    "24.11842193": "LG Electronics",
    "12.123917678972928": "Duerkopp-Adler",
    "12.123917686865920": "iGrid T&D",
    "24.10801548": "Apple, Inc.",
    "20.9814039068672": "Vigitron Inc.",
    "24.9467886": "Cisco Systems, Inc",
    "12.123917684219904": "Allied Data Systems",
    "24.2759": "Unication Group",
    "20.175931693989888": "Danriver Technologies Corp.",
    "24.14991857": "SHENZHEN SPOTMAU INFORMATION TECHNOLIGY CO., Ltd",
    "24.15384231": "Sonos, Inc.",
    "24.2758": "Overture Networks.",
    "24.15339869": "Autovision Technology Inc.",
    "24.1837906": "Epicom S.A",
    "20.53091447603200": "IoTmaxx GmbH",
    "24.6872296": "PacketStorm Communications",
    "24.2754": "Wuhan FiberHome Digital Technology Co.,Ltd.",
    "24.2757": "Color Kinetics",
    "12.346856017920": "Spirent Communications (Scotland) Limited",
    "24.2756": "Daewoo Teletech Co., Ltd",
    "24.9208733": "Shenzhen Xinyupeng Electronic Technology Co., Ltd",
    "24.15475560": "IntelliVoice Co.,Ltd.",
    "12.123917683744768": "Shanghai Westwell Information and Technology Company Ltd",
    "24.6070": "Aquantia Corporation",
    "24.2750": "OPNET Technologies CO., LTD.",
    "24.8930273": "Cisco Systems, Inc",
    "24.2753": "Futuretel",
    "24.69": "Ford Aerospace & Comm. Corp.",
    "24.68": "Castelle Corporation",
    "24.67": "Micro Technology",
    "24.66": "Metier Management Systems Ltd.",
    "24.65": "Ice Corporation",
    "24.64": "Applicon, Inc.",
    "24.63": "Syntrex, Inc.",
    "24.62": "Simpact",
    "24.61": "Unisys",
    "24.60": "Auspex Systems Inc.",
    "24.7859": "Primex Wireless",
    "24.7858": "LG Innotek",
    "24.3974703": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.7851": "TeleWell Oy",
    "24.7850": "E-Senza Technologies GmbH",
    "24.7853": "Wingtech Group Limited",
    "24.7852": "Armadeus Systems",
    "24.7855": "Ophir Optronics Ltd",
    "24.7854": "Continental Automotive Systems Inc.",
    "24.7857": "Cryptsoft Pty Ltd",
    "24.7856": "ImesD Electronica S.L.",
    "12.123917680975872": "Peter Huber Kaeltemaschinenbau AG",
    "12.346855157760": "AutoTOOLS group Co. Ltd.",
    "12.346860429312": "Digitrol Ltd",
    "24.6848372": "Cisco-Linksys, LLC",
    "24.2895176": "bct electronic GesmbH",
    "12.346867339264": "MCS MICRONIC Computer Systeme GmbH",
    "24.317941": "ASUSTek COMPUTER INC.",
    "24.16537850": "Shen Zhen Shi Xin Zhong Xin Technology Co.,Ltd.",
    "20.62335523749888": "Gesellschaft industrieller Technologien",
    "24.11302173": "Huawei Technologies Co.,Ltd",
    "12.346856779776": "United Western Technologies",
    "24.8141781": "Imago Group",
    "24.6596663": "Juni Korea Co., Ltd",
    "24.8138624": "Gigaset Communications GmbH",
    "24.38249": "LSD Science and Technology Co.,Ltd.",
    "24.1105636": "automationNEXT GmbH",
    "24.23097": "Shenzhen Fast Technologies Co.,Ltd",
    "12.346860597248": "Open Video, Inc.",
    "24.14158294": "Zexelon Co., Ltd.",
    "24.9465624": "Private",
    "12.71297884221440": "YUYAMA MFG Co.,Ltd",
    "24.12847819": "Cisco Systems, Inc",
    "12.346853801984": "Goerlitz AG",
    "24.1319475": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3728093": "zte corporation",
    "12.346854711296": "ZP Engineering SEL",
    "24.7926430": "Pegatron Corporation",
    "12.71297884483584": "Zhejiang Wellsun Electric Meter Co.,Ltd",
    "24.14159127": "Tp-Link Technologies Co.,Ltd.",
    "24.16295365": "Cisco Systems, Inc",
    "24.14446336": "Livescribe, Inc.",
    "24.6057833": "Broadcom Limited",
    "20.136646471188480": "Zerosystem LTD.Co",
    "24.7128277": "HMD Global Oy",
    "20.176726063710208": "COMSYS Communications Systems Service GmbH",
    "24.13933088": "Apple, Inc.",
    "24.4218242": "Netgear",
    "24.9205975": "Shenzhen Fast Technologies Co.,Ltd",
    "24.1594336": "Hanyang Digitech Co.Ltd",
    "12.123917688877056": "NimbeLink Corp",
    "24.12104427": "Googol Technology (HK) Limited",
    "24.2162381": "System In Frontier Inc.",
    "24.12857232": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.7367609": "Cisco Systems, Inc",
    "20.175931691892736": "Zhejiang Hechuan Technology Co.,Ltd",
    "20.45033668476928": "Skyrockettoys LLC",
    "24.9736244": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "20.97755439562752": "Probit Srl",
    "24.1341939": "China Mobile Group Device Co.,Ltd.",
    "12.346855862272": "Inoteska s.r.o.",
    "20.171812082155520": "Haesung Ds",
    "12.123917677170688": "Fracarro srl",
    "24.7655965": "Huawei Technologies Co.,Ltd",
    "24.6812": "RightHand Technologies, Inc.",
    "24.6813": "Skipper Wireless, Inc.",
    "24.6810": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346862120960": "C&I Co.Ltd",
    "24.6816": "Dell Inc.",
    "24.6817": "Cisco Systems, Inc",
    "24.6814": "ICON Digital International Limited",
    "24.3147071": "JTI Co.,Ltd.",
    "12.123917675098112": "FTG Corporation",
    "24.6818": "Cisco Systems, Inc",
    "24.6819": "Delorme",
    "24.11856073": "XADA Technologies",
    "12.123917691584512": "Alcohol Countermeasure Systems",
    "12.346863480832": "U-Care Inc.",
    "24.12594725": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.3194907": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "24.12594726": "Private",
    "24.11052648": "zte corporation",
    "24.6836663": "PowerCloud Systems, Inc.",
    "24.3694711": "Pegatron Corporation",
    "24.9754126": "shenzhen yunmao information technologies co., ltd",
    "24.3946632": "ConnectQuest, llc",
    "12.346854817792": "RPM Systems Corporation",
    "24.3203": "Logical Solutions",
    "24.11305961": "Beijing Zile Technology Co., Ltd",
    "12.71297884913664": "Multiobrabotka",
    "24.13919479": "Chengdu Usee Digital Technology Co., Ltd",
    "24.13947621": "BKAV Corporation",
    "12.123917679378432": "D.Marchiori Srl",
    "24.10249197": "zte corporation",
    "24.19762": "Andon Health Co.,Ltd.",
    "24.13940489": "zte corporation",
    "24.15242083": "Nokia",
    "12.346853728256": "Binary Wave Technologies Inc.",
    "24.3170339": "ARRIS Group, Inc.",
    "24.11320688": "Huawei Device Co., Ltd.",
    "24.578666": "LG Electronics (Mobile Communications)",
    "24.2410707": "McWane India Pvt Ltd",
    "24.14744300": "Platan sp. z o.o. sp. k.",
    "12.123917683605504": "NCE Network Consulting Engineering srl",
    "12.123917687910400": "AE Van de Vliet BVBA",
    "12.123917675765760": "Ayre Acoustics, Inc.",
    "24.2650500": "Spire Payments",
    "12.346856558592": "Octatron, Inc.",
    "24.1586077": "Cisco Systems, Inc",
    "24.5541": "DCI Co., Ltd.",
    "24.5540": "ARRIS Group, Inc.",
    "24.5543": "Robatech AG",
    "24.5542": "Digital Electronics Products Ltd.",
    "24.5545": "Kwang Woo I&C Co.,Ltd",
    "24.5544": "ARRIS Group, Inc.",
    "12.346855948288": "Tiyoda Mfg Co.,Ltd.",
    "24.5546": "Rextechnik International Co.,",
    "24.5549": "Accedian Networks",
    "24.5548": "Capelon AB",
    "24.7909348": "Comau S.p.A",
    "12.346858029056": "Bundesamt für Strahlenschutz",
    "12.346866606080": "Fluid Components International",
    "24.2905404": "Gainspeed, Inc.",
    "12.346862878720": "Tred Displays",
    "12.123917690236928": "STACKFORCE GmbH",
    "12.123917681672192": "Aplex Technology Inc.",
    "12.346854584320": "Macronet s.r.l.",
    "24.11331147": "Shenzhen Baojia Battery Technology Co., Ltd.",
    "12.123917679431680": "TriLED",
    "12.123917675503616": "Zao Zeo",
    "24.11807246": "Technicolor CH USA Inc.",
    "24.11800214": "Heran",
    "24.16311523": "Intel Corporate",
    "20.198611833978880": "SALZBRENNER media GmbH",
    "24.10490941": "Parrot Sa",
    "24.9979708": "Shanghai Sunmon Communication Technogy Co.,Ltd",
    "24.8183072": "IOXOS Technologies S.A.",
    "12.123917690380288": "Druck Ltd.",
    "20.255191440949248": "DongGuan Pengxun Electronics Technology Co., Ltd.",
    "12.123917688541184": "Kinemetrics",
    "24.1352986": "Huawei Technologies Co.,Ltd",
    "24.2928171": "Samsung Electronics Co.,Ltd",
    "12.123917680668672": "DogWatch Inc",
    "24.11323988": "Telldus Technologies AB",
    "24.20689": "Cisco Systems, Inc",
    "24.7078624": "HP Inc.",
    "24.12211692": "Ieee 802.15",
    "24.24076": "HMD Global Oy",
    "20.111031365926912": "Seven Solutions S.L",
    "12.346863570944": "Evercare",
    "12.123917679755264": "Cardinal Scale Mfg Co",
    "24.3697436": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.8178719": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.813059": "Cisco Systems, Inc",
    "24.3190734": "vivo Mobile Communication Co., Ltd.",
    "24.11834446": "WeTelecom Co., Ltd.",
    "24.3149539": "Aquantia Corporation",
    "12.346864488448": "Sentec Ltd",
    "20.114869212807168": "Wingtech Mobile Communications Co., Ltd.",
    "24.14163916": "Brocade Communications Systems LLC",
    "24.13653893": "Pantech Co., Ltd.",
    "24.15233010": "Actiontec Electronics, Inc",
    "24.7358441": "vivo Mobile Communication Co., Ltd.",
    "24.5278214": "Tuya Smart Inc.",
    "12.346866569216": "Margento R&D",
    "12.346853326848": "Stonefly Networks",
    "24.10769968": "Cisco Systems, Inc",
    "12.123917684338688": "Reo Ag",
    "24.12877962": "Cloud Diagnostics Canada ULC",
    "12.123917681319936": "OOO Research and Production Center \"Computer Technologies\"",
    "24.7087033": "Sony Mobile Communications Inc",
    "12.123917682999296": "Reactec Ltd",
    "24.7350909": "EpSpot AB",
    "24.873": "Nippon Antenna Co., Ltd.",
    "24.872": "Embedone Co., Ltd.",
    "24.871": "Jasmine Networks, Inc.",
    "24.870": "ASM Pacific Technology",
    "24.877": "Runtop, Inc.",
    "24.876": "Cisco Systems, Inc",
    "24.875": "Cisco Systems, Inc",
    "24.874": "Mainnet, Ltd.",
    "24.1113172": "Shany Electronic Co., Ltd.",
    "24.878": "Nicon Systems (Pty) Limited",
    "12.123917688954880": "FLSmidth",
    "12.123917687402496": "Fotona D.D.",
    "24.15739427": "Creative Next Design",
    "20.206931533758464": "Shenzhen PHilorise Technical Limited",
    "24.10798064": "Intel Corporate",
    "12.123917690937344": "Elektronik Art",
    "24.6309730": "Beijing Sensoro Technology Co.,Ltd.",
    "24.8420229": "IEEE Registration Authority",
    "24.7338": "Bellon Pty Ltd",
    "24.7339": "Meyer Sound Laboratories, Inc.",
    "24.16047169": "Youngzone Culture (Shanghai) Corp",
    "24.7334": "Win4NET",
    "24.7335": "International Quartz Limited",
    "24.7336": "AirTies Wireless Networks",
    "24.7337": "Audiomatica Srl",
    "24.7330": "ADB Broadband Italia",
    "24.2639942": "GridCentric Inc.",
    "24.7332": "Sony Mobile Communications Inc",
    "24.7333": "Zygo Corporation",
    "24.29581": "Shenzhen TINNO Mobile Technology Corp.",
    "12.123917688340480": "Verti Tecnologia",
    "24.4199385": "Global ES",
    "12.346860990464": "OMB Sistemas Electronicos S.A.",
    "24.15207187": "Akib Systems Taiwan, INC",
    "24.1692": "Transmode Systems AB",
    "24.49403": "Advanced Technology Labs",
    "20.39672886591488": "Ganzhou Dehuida Technology Co., Ltd",
    "24.49401": "Artesyn Embedded Technologies",
    "24.49400": "About Computing Inc.",
    "24.49407": "Seagate Cloud Systems Inc",
    "24.49406": "Aptec Computer Systems, Inc.",
    "24.49405": "Prosum",
    "24.49404": "ASDG Incorporated",
    "24.6578790": "IEEE Registration Authority",
    "24.5042222": "Vifa Denmark A/S",
    "24.7117570": "Nokia Corporation",
    "24.14480385": "Dell Inc.",
    "24.13937613": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.36090": "Inventec Corporation",
    "24.10776790": "Motorola Mobility LLC, a Lenovo Company",
    "12.346863378432": "ProCom Systems, Inc.",
    "24.15225988": "NEO Information Systems Co., Ltd.",
    "24.13108213": "Ruckus Wireless",
    "24.11811286": "ProCurve Networking by HP",
    "24.4496565": "Alcomp, Inc",
    "20.114869215952896": "Neurolab",
    "12.346868727808": "Soudronic AG",
    "12.346862845952": "D-TA Systems",
    "12.346859151360": "Ifox - Industria e Comercio Ltda",
    "24.9980020": "Huawei Technologies Co.,Ltd",
    "24.5028978": "Primex Wireless",
    "20.229631753977856": "Teco Image Systems Co., Ltd.",
    "24.2124436": "Hitron Technologies. Inc",
    "12.123917688156160": "Asia Pacific Satellite Coummunication Inc.",
    "20.61822312906752": "R3C Information(Shenzhen) Co.，Ltd.",
    "12.346857136128": "Digital Dynamics, Inc.",
    "24.11293812": "China Mobile IOT Company Limited",
    "24.16006758": "Bee Computing (HK) Limited",
    "24.1639": "Tripp Lite",
    "24.1638": "Roving Networks",
    "24.1633": "NIA Home Technologies Corp.",
    "24.1632": "NADEX Co., Ltd.",
    "24.1631": "ECI Telecom Ltd.",
    "24.1630": "Photuris, Inc.",
    "24.1637": "Sunny Giken, Inc.",
    "24.1636": "Fostex Corporation",
    "24.1635": "Human Technology Co., Ltd.",
    "24.1634": "MBM Technology Ltd.",
    "24.8153779": "Ibc Technologies Inc.",
    "20.132855975903232": "BYD Auto lndustry Co.,Ltd",
    "12.346854699008": "Continental Gateway Limited",
    "12.123917681954816": "Glial Technology",
    "24.6175": "Palmmicro Communications",
    "24.6174": "GDX Technologies Ltd.",
    "24.6177": "Sindoricoh",
    "24.6176": "w5networks",
    "24.6171": "TaiJin Metal Co., Ltd.",
    "24.6170": "AVerMedia Information Inc.",
    "24.6173": "Asia Electronics Co.,Ltd",
    "24.6172": "Exterity Limited",
    "24.6179": "Delta Electronics, Inc.",
    "24.6178": "Cec Telecom Co.,Ltd.",
    "20.27056773529600": "Mantra Softech India Pvt Ltd",
    "24.8972708": "Hewlett Packard Enterprise",
    "20.154055552794624": "Atol Llc",
    "24.1060901": "Apple, Inc.",
    "12.123917675585536": "Beijing CONvision Technology Co.,Ltd",
    "12.346866171904": "Ibetor S.L.",
    "24.2677633": "Shanghai Guao Electronic Technology Co., Ltd",
    "20.154680179032064": "SEVERIN Elektrogeräte GmbH",
    "24.15471933": "Beijing Yaxunhongda Technology Co., Ltd.",
    "24.5546612": "Cisco Systems, Inc",
    "12.123917690437632": "Emporia Renewable Energy Corp",
    "24.6335252": "LG Innotek",
    "12.71297884545024": "Marposs S.p.A",
    "12.123917678424064": "Elektro-System s.c.",
    "24.9484338": "Extreme Networks, Inc.",
    "12.346866118656": "SECOM GmbH",
    "24.14723256": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "24.37051": "TAINET COMMUNICATION SYSTEM Corp.",
    "24.37050": "Valid Networks, Inc.",
    "24.37053": "Omnia Communications, Inc.",
    "24.37052": "Telemann Co., Ltd.",
    "24.37055": "Cisco Systems, Inc",
    "24.37054": "Ibc/Integrated Business Computers",
    "24.37057": "Peco II, Inc.",
    "24.37056": "K.J. Law Engineers, Inc.",
    "24.37059": "Topic Semiconductor Corp.",
    "24.37058": "JK microsystems, Inc.",
    "24.6092739": "Syntech (Hk) Technology Limited",
    "24.16033066": "Hawa Technologies Inc",
    "12.346858024960": "Nautel Limited",
    "24.1632271": "Viking Electronics Inc.",
    "24.16281295": "Arcx Inc",
    "24.6091271": "Speco Technologies",
    "24.8918340": "Cisco Meraki",
    "24.13681073": "Samsung Electronics Co.,Ltd",
    "12.346855542784": "Secure Systems & Services",
    "24.9743032": "Joview(Beijing) Technology Co. Ltd.",
    "24.15209252": "GuangZhou Bonsoninfo System CO.,LTD",
    "24.16293816": "YAZAKI Energy System Corporation",
    "24.15787170": "Huawei Technologies Co.,Ltd",
    "24.11537109": "zte corporation",
    "24.14436809": "Casco Signal Ltd",
    "20.163603967639552": "Sam Nazarko Trading Ltd",
    "12.71297884819456": "Soukai Electric",
    "20.123598308114432": "SelectTech GeoSpatial, LLC",
    "24.11841621": "Huawei Technologies Co.,Ltd",
    "24.11277409": "ATAW Co., Ltd.",
    "24.1084266": "Antailiye Technology Co.,Ltd",
    "24.9358": "Infoware ZRt.",
    "24.9359": "Do-Monix",
    "12.346855919616": "vidisys gmbh",
    "12.123917680627712": "LunaNexus Inc",
    "24.9350": "DesignArt Networks",
    "24.9351": "Transact Campus, Inc.",
    "24.9352": "Centre For Development Of Telematics",
    "24.9353": "Vodafone Omnitel N.V.",
    "24.9354": "Kaga Electronics Co., Ltd.",
    "24.9355": "Hybus Co., Ltd.",
    "24.9356": "ASUSTek COMPUTER INC.",
    "24.9357": "Sony Interactive Entertainment Inc.",
    "20.198611832930304": "Shanghai Shenou Communication Equipment Co., Ltd.",
    "24.8446399": "Huawei Technologies Co.,Ltd",
    "24.2155176": "Apple, Inc.",
    "24.1882764": "Texas Instruments",
    "24.503": "Image Display Systems, Inc.",
    "24.15783850": "Motorola Mobility LLC, a Lenovo Company",
    "24.5006315": "Application Solutions (Electronics and Vision) Ltd",
    "24.502": "Association of Musical Electronics Industry",
    "12.346865451008": "PURVIS Systems Incorporated",
    "24.501": "Erim S.A.",
    "24.500": "Enterasys",
    "24.14688010": "Xslent Energy Technologies. LLC",
    "24.507": "DoTop Technology, Inc.",
    "12.123917683580928": "PlayAlive A/S",
    "20.97400599347200": "Sercomm Corporation.",
    "12.346852589568": "Seabridge",
    "20.220958195449856": "Eltako GmbH",
    "24.5250239": "Cisco Systems, Inc",
    "24.505": "TeraGlobal Communications Corp.",
    "24.504": "Texio Technology Corporation",
    "24.10767458": "Hangzhou Microimage Software Co., Ltd",
    "24.13914666": "TangoWiFi.com",
    "24.524350": "Codex Corporation",
    "24.524351": "Fred Koschara Enterprises",
    "24.524352": "Ferranti Computer Sys. Limited",
    "24.524353": "Racal-Milgo Information Sys..",
    "24.524354": "Japan Macnics Corp.",
    "24.524355": "Pixel Computer Inc.",
    "24.524356": "David Systems Inc.",
    "24.524357": "Concurrent Computer Corp.",
    "24.36870": "Hamamatsu Photonics K.K.",
    "24.36871": "Domex Technology Corp.",
    "24.36872": "HanA Systems Inc.",
    "24.36873": "I Controls, Inc.",
    "24.36874": "Proton Electronic Industrial Co., Ltd.",
    "24.36875": "Lanner Electronics, Inc.",
    "24.36876": "Cisco Systems, Inc",
    "24.36877": "Overland Storage Inc.",
    "12.71297884848128": "devboards GmbH",
    "20.145383540916224": "Neat S.r.l.",
    "24.16563214": "Zhejiang Cainiao Supply Chain Management Co., Ltd",
    "24.8007": "MCS Logic Inc.",
    "24.8006": "Nortel Networks",
    "24.8005": "Enterasys",
    "24.8004": "GE Transportation Systems",
    "24.8003": "Entes Elektronik",
    "24.8002": "Etherstack plc",
    "24.8001": "Ruckus Wireless",
    "24.8000": "Speakercraft Inc.",
    "12.123917679882240": "aelettronica group srl",
    "24.8009": "Manhattan TV Ltd",
    "24.8008": "Mojix Inc.",
    "12.123917675659264": "Cominfo, Inc.",
    "24.7373061": "Cisco Systems, Inc",
    "12.346862653440": "Patronics International LTD",
    "12.71297885130752": "BaOpt Benelux bv",
    "24.5244221": "CyWee Group Ltd",
    "12.123917687312384": "Providius Corp",
    "12.123917689339904": "Dalian Laike Technology Development Co., Ltd",
    "12.346857414656": "Criterion Systems Limited",
    "12.346864050176": "Sinovia Sa",
    "24.9209659": "Leica Camera AG",
    "20.57176921997312": "RPE \"Monitor\"",
    "12.346859630592": "ASC telecom AG",
    "24.7920290": "Cynosure Technologies Co.,Ltd",
    "24.12060075": "zte corporation",
    "12.123917679071232": "Barnstormer Softworks",
    "12.71297884479488": "hanatech",
    "24.10268371": "Seiko Epson Corporation",
    "24.9439770": "Taicang T&W Electronics",
    "24.848737": "Java Information",
    "24.1840307": "Airwire Technologies",
    "24.1419": "IPmental, Inc.",
    "12.346866003968": "Maddalena S.p.A",
    "24.10014132": "Virident Systems, Inc.",
    "24.8969366": "SENAO Networks, Inc.",
    "24.16015019": "Texas Instruments",
    "20.88668803956736": "Oem Production Inc.",
    "24.825922": "Fn-Link Technology Limited",
    "24.7871930": "Huawei Technologies Co.,Ltd",
    "12.123917678456832": "Leaff Engineering Srl",
    "24.6857668": "Liteon Technology Corporation",
    "24.10539898": "Intel Corporate",
    "12.119269982208": "Private",
    "12.123917678985216": "TerOpta Ltd",
    "12.346855817216": "MODIA SYSTEMS Co., Ltd",
    "24.11022524": "Samsung Electronics Co.,Ltd",
    "24.8394621": "Nortel Networks",
    "24.3168826": "ASUSTek COMPUTER INC.",
    "20.44169477226496": "SAKATA DENKI Co., Ltd.",
    "24.4349": "Cocom A/S",
    "24.7620046": "Cresyn",
    "12.123917679362048": "Presentation Switchers, Inc.",
    "24.15005601": "Datafox GmbH",
    "24.4992131": "Zhejiang KaNong Network Technology Co.,Ltd.",
    "12.71297884278784": "Rite-Tech Industrial Co., Ltd.",
    "12.123917689290752": "Jiangsu Etern Compamy Limited",
    "12.346862825472": "Aprilia Racing S.R.L.",
    "24.1090750": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.1103170": "Samsung Electronics Co.,Ltd",
    "20.4884649738240": "Shenzhen BoClouds Technology Co.,Ltd.",
    "24.1342938": "Apple, Inc.",
    "24.9710771": "HUMAX Co., Ltd.",
    "12.123917678264320": "FINANCIERE DE L'OMBREE (eolane)",
    "12.123917675778048": "Andium",
    "20.79215077621760": "Microprogram Information Co., Ltd",
    "12.346860494848": "Logotherm Regelsysteme GmbH",
    "24.16532931": "Texas Instruments",
    "12.346853441536": "Delphin Technology AG",
    "20.137149113434112": "Shanghai Yitu Technology Co. Ltd",
    "24.12589786": "Apple, Inc.",
    "24.11561142": "Hangzhou OYE Technology Co. Ltd",
    "12.123917686689792": "Hifocus Electronics India Private Limited",
    "24.7108192": "Kyocera Corporation",
    "12.346855911424": "SYS TEC electronic GmbH",
    "24.8417658": "Huawei Technologies Co.,Ltd",
    "24.5784292": "Baxter International Inc",
    "12.346859315200": "Lumistar Incorporated",
    "20.62663547682816": "Indra Navia AS",
    "12.123917678448640": "Slat",
    "24.13907675": "Jiangsu Toppower Automotive Electronics Co., Ltd",
    "24.13686476": "Wintop",
    "24.12356328": "Qing Dao Haier Telecom Co.,Ltd.",
    "24.5815335": "Cisco Systems, Inc",
    "24.11043328": "Hisense Electric Co.,Ltd",
    "24.3715373": "Sonotronic Nagel GmbH",
    "24.791584": "Samsung Electronics Co.,Ltd",
    "24.2399915": "Huawei Technologies Co.,Ltd",
    "24.5789416": "ARRIS Group, Inc.",
    "20.242185284878336": "VerifEye Technologies",
    "24.1062673": "Simlink AS",
    "24.12587185": "HMD Global Oy",
    "24.14944487": "Quectel Wireless Solutions Co.,Ltd.",
    "24.11162763": "Tintri",
    "12.123917677662208": "Shenzhen Rongda Computer Co.,Ltd",
    "12.123917680635904": "Deep BV",
    "24.803305": "Huawei Technologies Co.,Ltd",
    "20.264625137385472": "Pavo Tasarim Uretim Ticaret A.S.",
    "24.7883655": "Murata Manufacturing Co., Ltd.",
    "24.3700433": "Euronda SpA",
    "24.5799880": "S2M",
    "24.15783138": "Dell Inc.",
    "12.346862387200": "Stuyts Engineering Haarlem BV",
    "24.16536478": "Shanghai Wind Technologies Co.,Ltd",
    "24.1606284": "Arcadyan Corporation",
    "24.8412684": "Yealink(Xiamen) Network Technology Co.,Ltd.",
    "24.2622688": "Fermax Electronica S.A.U.",
    "12.346858577920": "Lester Electronics Ltd",
    "24.49270": "I-Data International A-S",
    "24.49271": "Daewoo Telecom Ltd.",
    "24.49272": "Computer Systems Engineering",
    "20.123598315454464": "Liquid Instruments Pty Ltd",
    "12.123917690699776": "DKS Dienstl.ges. f. Komm.anl. d. Stadt- u. Reg.verk. mbH",
    "24.7896403": "Extreme Networks, Inc.",
    "24.49273": "Fonsys Co.,Ltd.",
    "24.14162382": "Telesquare",
    "24.14473340": "Beijing Jingdong Century Trading Co., LTD.",
    "24.49275": "Ascend Communications",
    "24.13370548": "Select Comfort",
    "24.49276": "Hightech Information",
    "24.4481436": "FONsystem co. ltd.",
    "12.346863783936": "PhotoTelesis LP",
    "24.49277": "Risc Developments Ltd.",
    "12.123917678055424": "NeurIT s.r.o.",
    "24.49278": "Kubota Corporation Electronic",
    "24.1896137": "Elo touch solutions",
    "24.14188209": "x.o.ware, inc.",
    "24.49279": "Nupon Computing Corp.",
    "24.14712298": "Texas Instruments",
    "20.119143417249792": "Shenzhen Justek Technology Co., Ltd",
    "20.31873100152832": "Tianjin Keyvia Electric Co.,Ltd",
    "12.123917680189440": "Compal Electronics, Inc.",
    "12.346855251968": "Eurotek Srl",
    "12.346862678016": "Dynon Instruments",
    "20.203238167019520": "Yuwei Info&Tech Development Co.,Ltd",
    "24.2117233": "Juniper Networks",
    "12.123917678268416": "AVA Monitoring AB",
    "12.346864156672": "Nisshin Electronics co.,ltd.",
    "12.123917676187648": "Shenzhen Luxurite Smart Home Ltd",
    "24.7654387": "Lava international limited",
    "24.3416967": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.12124": "Cisco Systems, Inc",
    "12.346862350336": "St. Michael Strategies",
    "12.123917690667008": "Pnetworks",
    "12.123917675122688": "Weigl Elektronik & Mediaprojekte",
    "24.3253": "Premier Technolgies, Inc",
    "24.3252": "AutoCell Laboratories, Inc.",
    "24.3251": "ROUND Co.,Ltd.",
    "24.3250": "UNION co., ltd.",
    "24.3257": "Lea",
    "24.3256": "Medion Ag",
    "24.3255": "Nanjing Huazhuo Electronics Co., Ltd.",
    "24.3254": "Nanjing Seu Mobile & Internet Technology Co.,Ltd",
    "24.9470989": "Apple, Inc.",
    "24.3259": "Iskraemeco",
    "24.3258": "Jamex, Inc.",
    "24.47930": "Amazon Technologies Inc.",
    "24.6088639": "New Singularity International Technical Development Co.,Ltd",
    "12.123917679017984": "Sanmina Israel",
    "12.71297884053504": "deskontrol electronics",
    "24.8926815": "LG Electronics (Mobile Communications)",
    "12.71297883471872": "GD Mission Systems",
    "20.97400601444352": "Unity Surveillance, Inc.",
    "24.12904382": "SCSpro Co.,Ltd",
    "24.3190436": "Espressif Inc.",
    "12.123917681336320": "Tata Power Sed",
    "24.6084127": "Qardio, Inc",
    "24.11853264": "Continental Automotive Hungary Kft",
    "20.180394582343680": "Adetel Equipment",
    "20.45033666379776": "LogiM GmbH Software und Entwicklung",
    "24.11318035": "ARRIS Group, Inc.",
    "24.12617395": "Shenzhen Voxtech Co., Ltd.",
    "24.15761779": "Amazon Technologies Inc.",
    "12.123917679837184": "AC Power Corp.",
    "24.8146869": "Arcadyan Technology Corporation",
    "12.346860994560": "Condigi Televagt A/S",
    "12.123917679984640": "Technica Engineering GmbH",
    "24.12374391": "Tp-Link Technologies Co.,Ltd.",
    "12.71297885151232": "ELNEC s.r.o.",
    "24.13408395": "SONY Visual Products Inc.",
    "24.6076108": "NovaComm Technologies Inc.",
    "12.346863583232": "Advanced Electronic Designs, Inc.",
    "24.13934189": "Wuhan Zhongyuan Huadian Science & Technology Co.,",
    "24.41191": "Central Data Corporation",
    "24.41190": "Dialogic Corporation",
    "24.41193": "Electronic Retailing Systems International",
    "24.41192": "Reuters Holdings Plc",
    "24.41195": "Encore Networks, Inc.",
    "24.41194": "Ethercom Corp.",
    "24.41197": "Brooks Automation, Inc.",
    "24.41196": "Transmitton Ltd.",
    "24.41199": "Lucidata Ltd.",
    "12.346858655744": "Fraunhofer IIS",
    "24.9208801": "Samsung Electronics Co.,Ltd",
    "24.7142025": "Cisco Systems, Inc",
    "20.216039234207744": "Anhui GuangXing Linked-Video Communication Technology Co, Ltd.",
    "12.119269228544": "Converging Systems Inc.",
    "24.2389763": "Intel Corporate",
    "12.346866401280": "Software Effect Enterprises, Inc",
    "24.6821673": "Intel Corporate",
    "20.171977813786624": "Skydock do Brasil Ltda",
    "12.346854928384": "West-Com Nurse Call Systems, Inc.",
    "24.1053489": "Technicolor",
    "12.71297884254208": "Smith Meter, Inc",
    "24.8689281": "Cathay Communication Co.,Ltd",
    "24.13932299": "Xiaomi Communications Co Ltd",
    "24.5243243": "Huawei Technologies Co.,Ltd",
    "24.2122802": "Samsung Electro Mechanics Co., Ltd.",
    "24.16039292": "Chengdu jinshi communication Co., LTD",
    "24.12107651": "Intel Corporate",
    "12.346857635840": "EtherTek Circuits",
    "24.7910506": "Huawei Technologies Co.,Ltd",
    "24.15754278": "Extreme Networks, Inc.",
    "24.9209232": "Apple, Inc.",
    "12.346864472064": "ITS Telecom",
    "24.6818087": "Apple, Inc.",
    "24.9225857": "Intel Corporate",
    "24.549478": "ASUSTek COMPUTER INC.",
    "24.1597895": "lemonbeat GmbH",
    "24.2627240": "Microsoft Corporation",
    "24.12099337": "Wistron Neweb Corporation",
    "24.6373": "Adhoco AG",
    "24.6372": "Yiguang",
    "24.6371": "Visualgate Systems, Inc.",
    "24.6370": "Topdata Sistemas de Automacao Ltda",
    "24.6377": "Numata Corporation",
    "24.6376": "Hacetron Corporation",
    "24.6375": "Cameo Communications, INC.",
    "24.8734": "Media Devices Co., Ltd.",
    "24.6379": "Blue Zen Enterprises Private Limited",
    "24.6378": "Alltec GmbH",
    "12.346855104512": "Ross Video Limited",
    "24.9720624": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9232513": "zte corporation",
    "12.123917687496704": "Virtuosys Ltd",
    "24.16574610": "Hangzhou Lancable Technology Co.,Ltd",
    "24.9181485": "IEEE Registration Authority",
    "24.2365469": "Samsung Electro-Mechanics(Thailand)",
    "24.5285675": "Huawei Technologies Co.,Ltd",
    "24.41118": "Ictv",
    "24.3209175": "Thread Technology Co., Ltd",
    "20.242876966502400": "Controls Inc",
    "24.1849952": "Ruckus Wireless",
    "12.346868035584": "Electro-Motive Diesel",
    "12.346856787968": "BiPOM Electronics, Inc.",
    "20.242185272295424": "Skywave Technology Co,.Ltd.",
    "24.15791809": "Sonos, Inc.",
    "24.10010859": "Global Regency Ltd",
    "24.1575762": "Nanotron Technologies GmbH",
    "24.7089444": "Zhen Shi Information Technology (Shanghai) Co., Ltd.",
    "12.346855739392": "Always On Wireless",
    "24.5813244": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.6997": "Hurco Automation Ltd.",
    "24.6996": "Cisco Systems, Inc",
    "24.6995": "Cisco Systems, Inc",
    "24.6994": "ARRIS Group, Inc.",
    "24.6993": "Vector Technology Corp.",
    "24.6992": "Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)",
    "24.6991": "Avaya Inc",
    "24.6990": "Navman New Zealand",
    "24.9763625": "Zebra Technologies Inc.",
    "24.2919857": "Intel Corporate",
    "24.6999": "Semindia Systems Private Limited",
    "24.6998": "Tehuti Networks Ltd.",
    "12.346860326912": "Bluebell Opticom Limited",
    "12.123917685268480": "Komsis Elektronik Sistemleri San. Tic. Ltd.Sti",
    "12.346865152000": "Potter Electric Signal Co. LLC",
    "24.1343": "VisionTek, Inc.",
    "24.1342": "KID Systeme GmbH",
    "24.1341": "Agere Systems",
    "24.1340": "Xircom",
    "24.1347": "IQ Wireless GmbH",
    "24.1346": "Otari, Inc.",
    "24.1345": "Advanced Systems Co., Ltd.",
    "24.1344": "FAST Corporation",
    "24.1349": "Internet Photonics",
    "24.1348": "Valley Technologies, Inc.",
    "24.12063721": "Trace Live Network",
    "12.123917679169536": "Herrick Tech Labs",
    "12.123917690118144": "MAC Solutions (UK) Ltd",
    "12.123917682860032": "Blue Marble Communications, Inc.",
    "24.11145093": "Yamaichi Electronics",
    "20.61822315003904": "Shenzhen HS Fiber Communication Equipment CO., LTD",
    "24.248": "Digital Equipment Corporation",
    "24.249": "Quotron Systems Inc.",
    "24.11026722": "China Mobile (Hangzhou) Information Technology Co.,Ltd.",
    "24.240": "Samsung Electronics Co.,Ltd",
    "24.241": "Magna Computer Corporation",
    "24.242": "Spider Communications",
    "24.243": "Gandalf Data Ltd. - Canada",
    "24.244": "Allied Telesis, Inc.",
    "24.245": "Diamond Sales Limited",
    "24.246": "Applied Microsystems Corp.",
    "24.247": "Youth Keep Enterprise Co Ltd",
    "24.8692838": "Samsung Electronics Co.,Ltd",
    "24.1576236": "Intelbras",
    "24.8693988": "Huawei Technologies Co.,Ltd",
    "24.2931386": "Samsung Electronics Co.,Ltd",
    "24.4269": "Softronics Usb, Inc.",
    "24.4268": "Imci Technologies",
    "24.16288796": "Kaishun Electronic Technology Co., Ltd. Beijing",
    "24.4263": "Unex Technology Corporation",
    "24.4262": "Cisco Systems, Inc",
    "24.4261": "Oxford Instruments",
    "24.4260": "Xircom",
    "24.56094": "Albedo Telecom SL",
    "24.4266": "Media4, Inc.",
    "24.4265": "Adhoc Technologies",
    "24.4264": "Reliance Computer Corp.",
    "12.346868457472": "EMAC, Inc.",
    "12.346862690304": "ACD Elektronik GmbH",
    "24.1967": "Red Lion Controls, LP",
    "24.1966": "Britestream Networks, Inc.",
    "24.1965": "Pentacon GmbH Foto-und Feinwerktechnik",
    "24.1964": "Eolring",
    "24.1963": "Samsung Electronics Co.,Ltd",
    "24.1962": "Quantum Data Inc.",
    "24.1961": "Novasonics",
    "24.1960": "Haier Group Technologies Ltd",
    "20.171812085301248": "ProMOS Technologies Inc.",
    "24.1969": "Equator Technologies",
    "24.1968": "Office Details, Inc.",
    "12.123917678542848": "PT. Emsonic Indonesia",
    "24.13416081": "NECMagnusCommunications",
    "24.6078995": "Fr. Sauter AG",
    "24.13160913": "AGAiT Technology Corporation",
    "12.123917684695040": "Systems With Intelligence Inc.",
    "24.320248": "AEP Ticketing solutions srl",
    "12.71297883779072": "Sentry 360 Security",
    "24.5271619": "BSkyB Ltd",
    "24.15006009": "Minxon Hotel Technology INC.",
    "24.29283": "Netcore Technology Inc.",
    "24.53437": "Lattice Semiconductor Corp. (LPA)",
    "24.10752355": "IEEE Registration Authority",
    "24.11857177": "Pie Digital, Inc.",
    "24.6852398": "Hitron Technologies. Inc",
    "24.16297069": "Deltenna Ltd",
    "12.123917680615424": "Sontay Ltd.",
    "24.16297061": "Intel Corporate",
    "24.5780087": "Reliance Communications LLC",
    "24.936": "IDOT Computers, Inc.",
    "24.937": "AXCENT Media AG",
    "24.934": "Traxit Technology, Inc.",
    "24.935": "Unixtar Technology, Inc.",
    "24.932": "Imation Corp.",
    "24.933": "Medea Corporation",
    "24.930": "Catapult Communications",
    "24.931": "MAVIX, Ltd.",
    "24.938": "Watlow",
    "24.939": "Bridge Information Systems",
    "12.346857963520": "Federal Electronics",
    "12.346860728320": "Keith & Koep GmbH",
    "12.346857476096": "Tecna Srl",
    "24.16563356": "Vimar Spa",
    "24.15227531": "Xiaomi Communications Co Ltd",
    "24.16258375": "Avaya Inc",
    "24.14954278": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917679693824": "Lineage Power Pvt Ltd.,",
    "20.44213347549184": "SCA Hygiene Products AB",
    "12.346857312256": "Seachange international",
    "24.12861751": "Quectel Wireless Solutions Co., Ltd.",
    "24.16312224": "vivo Mobile Communication Co., Ltd.",
    "20.220092029730816": "iWave Systems Tech Pvt Ltd",
    "24.6041298": "ABC(XiSheng) Electronics Co.,Ltd",
    "20.137215124439040": "mirakonta s.l.",
    "24.12890881": "Apple, Inc.",
    "12.123917684379648": "Paramount Bed Holdings Co., Ltd.",
    "20.66219391582208": "Attrackting AG",
    "24.15499206": "Eccel Corporation Sas",
    "24.9995491": "Texas Instruments",
    "24.7624969": "Huawei Technologies Co.,Ltd",
    "24.4233169": "BK Electronics cc",
    "24.15513082": "Philips Lighting BV",
    "24.7137667": "Gastron Co., LTD.",
    "24.12106062": "Shenzhen iBaby Labs, Inc.",
    "24.8686541": "IEEE Registration Authority",
    "20.255553904312320": "MORNSUN Guangzhou Science & Technology Co., Ltd.",
    "20.181338947715072": "Heyuan intelligence technology CO.,Ltd",
    "24.11818681": "Teraspek Technologies Co.,Ltd",
    "24.3670053": "Intel Corporate",
    "24.64600": "WebSilicon Ltd.",
    "20.145727911100416": "POCT biotechnology",
    "12.119269904384": "Tesla Controls",
    "20.127937503297536": "Shenzhen Atx Technology Co.,Ltd",
    "24.11844732": "Texas Instruments",
    "12.346855362560": "Setaram",
    "24.4932": "Fargo Electronics Inc.",
    "24.9995145": "Samsung Electronics Co.,Ltd",
    "12.123917675708416": "RoboCore Tecnologia",
    "24.8711960": "Intel Corporate",
    "24.9578": "inIT - Institut Industrial IT",
    "24.9579": "ATENIX E.E. s.r.l.",
    "24.12251676": "Iso/Tc22/Sc31",
    "24.6087302": "Nortel Networks",
    "24.9570": "interbro Co. Ltd.",
    "24.9571": "Luxtera Inc",
    "24.9572": "Dell Inc.",
    "24.9573": "Vizimax Inc.",
    "24.9574": "Samsung Electronics Co.,Ltd",
    "24.9575": "Samsung Electronics Co.,Ltd",
    "24.9576": "Huawei Technologies Co.,Ltd",
    "24.9577": "Sagemcom Broadband SAS",
    "12.123917680676864": "Nuance Hearing Ltd.",
    "24.4249685": "IEEE Registration Authority",
    "24.1618281": "Sonicwall",
    "24.2081": "Cisco Systems, Inc",
    "24.2080": "Cisco Systems, Inc",
    "24.2083": "Texa Corp.",
    "24.2082": "InPro Comm",
    "24.2085": "Acme Packet",
    "24.2084": "Nuance Document Imaging",
    "24.2087": "ADB Broadband Italia",
    "24.2086": "Colorado Med Tech",
    "24.2089": "Tokyo Electron Device Nagasaki Limited",
    "24.2088": "Koei Engineering Ltd.",
    "20.220958192304128": "Shanghai Mo xiang Network Technology CO.,Ltd",
    "12.346853347328": "COE Limited",
    "12.123917678317568": "Primalucelab isrl",
    "24.13677252": "Technicolor CH USA Inc.",
    "24.2648274": "Huawei Technologies Co.,Ltd",
    "20.70682240090112": "NOX Systems AG",
    "20.22126502346752": "Inttelix Brasil Tecnologia e Sistemas Ltda",
    "24.13632996": "Huawei Device Co., Ltd.",
    "24.10793639": "Adaxys Solutions AG",
    "24.7109": "IEEE Registration Authority",
    "24.7108": "Ultratec, Inc.",
    "20.176189950918656": "SinoGrid Software Systems Inc.",
    "24.7103": "Sagemcom Broadband SAS",
    "24.7102": "ICOP Digital",
    "24.7101": "FMC Kongsberg Subsea AS",
    "24.7100": "Silver Peak Systems, Inc.",
    "24.7107": "Mobisolution Co.,Ltd",
    "24.7106": "Integrated Control Technology Limitied",
    "24.7105": "HOLUX Technology, Inc.",
    "24.7104": "Juniper Networks",
    "24.2559": "X.net 2000 GmbH",
    "24.2558": "Daisy Technologies, Inc.",
    "24.2553": "Art Japan Co., Ltd.",
    "24.2552": "Unimo Technology Co., Ltd.",
    "24.2551": "SED, a division of Calian",
    "24.2550": "Shenzhen Eastern Digital Tech Ltd.",
    "24.2557": "Ubinetics Limited",
    "24.2556": "IPFLEX Inc.",
    "24.2555": "Philips Patient Monitoring",
    "20.189549210763264": "Kpnetworks Ltd.",
    "20.74781677322240": "Norden Communication UK Ltd.",
    "12.346864005120": "Chitose Co.,Ltd",
    "24.13912926": "Taiyo Yuden Co., Ltd.",
    "24.7903485": "Q9 Networks Inc.",
    "24.8627": "Ross Controls",
    "24.8626": "Fiberblaze A/S",
    "24.8625": "Digital Solutions Ltd",
    "24.8624": "Tyco Telecommunications",
    "24.8623": "Radio Frequency Systems",
    "24.8622": "Alcatel-Lucent France - Wtd",
    "24.8621": "Nordic ID Oy",
    "24.8620": "Infrared Integrated Systems Ltd",
    "24.16001661": "Tp-Link Technologies Co.,Ltd.",
    "20.194363558723584": "Audio Elektronik İthalat İhracat San ve Tic A.Ş.",
    "24.8629": "Galvanic Ltd",
    "24.8628": "Apro Media Co., Ltd",
    "12.123917690744832": "GMI Ltd",
    "24.13647186": "Ava Security Limited",
    "24.15512023": "Hewlett Packard",
    "24.11838691": "Cisco Systems, Inc",
    "12.346858729472": "Enatel Limited",
    "24.12879593": "Huawei Technologies Co.,Ltd",
    "24.6857853": "Sun Cupid Technology (HK) LTD",
    "24.12119673": "YWire Technologies, Inc.",
    "12.346863099904": "Screen Technics Pty Limited",
    "12.123917680316416": "Next Sight srl",
    "24.15526392": "Guang Zhou TRI-SUN Electronics Technology Co., Ltd",
    "24.6578232": "Hirschmann Automation and Control GmbH",
    "12.346866556928": "M.P. Electronics",
    "12.346856906752": "Safegate International AB",
    "24.16281326": "Huawei Technologies Co.,Ltd",
    "24.1856022": "Dongguan Hele Electronics Co., Ltd",
    "20.163603962396672": "Gocoax, Inc",
    "20.368745381888": "Private",
    "24.9998045": "TE Connectivity Limerick",
    "12.123917689565184": "ETL Elektrotechnik Lauter GmbH",
    "24.7612847": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346863534080": "Incyma",
    "24.10752277": "Robert Bosch Engineering and Business Solutions pvt. Ltd.",
    "12.346865070080": "Powersense A/S",
    "24.9964881": "Huawei Device Co., Ltd.",
    "24.2151312": "Samsung Electronics Co.,Ltd",
    "24.4994184": "Intel Corporate",
    "12.346866073600": "Saia-Burgess Controls AG",
    "24.38049": "F5 Networks, Inc.",
    "24.14432020": "Apple, Inc.",
    "12.123917679046656": "KST technology",
    "12.346861363200": "Gradual Tecnologia Ltda.",
    "12.123917689327616": "RCH Vietnam Limited Liability Company",
    "12.346865897472": "Applied Medical Technologies, Inc DBA AirClean Systems",
    "24.9719945": "SimonsVoss Technologies GmbH",
    "12.123917685198848": "Aethera Technologies",
    "20.57353099542528": "Shenzhen Advance River System Technology Co., Ltd",
    "24.24945": "Apple, Inc.",
    "12.123917687877632": "Abb",
    "24.6300865": "Jiangsu Zhongxun Electronic Technology Co., Ltd",
    "24.3414818": "Grandbeing Technology Co., Ltd",
    "24.11596268": "AMPAK Technology, Inc.",
    "24.15513863": "CloudGenix Inc",
    "20.62663544537088": "CaptiveAire Systems Inc.",
    "12.346866855936": "ICU Scandinavia Schweiz GmbH",
    "24.12368841": "Kellendonk Elektronik GmbH",
    "12.346861379584": "Altair semiconductor Ltd",
    "24.6092508": "Samsung Electronics Co.,Ltd",
    "12.71297884557312": "Progress Rail Services, Inspection and Information Systems",
    "20.198151828930560": "Yireh Auto Tech Co.,Ltd.",
    "24.3434887": "zte corporation",
    "24.2673161": "ThyssenKrupp Elevators (Shanghai) Co.,Ltd",
    "24.3698700": "ARRIS Group, Inc.",
    "24.10791678": "Huawei Device Co., Ltd.",
    "12.346865455104": "Pacific Coast Engineering",
    "24.12321354": "HMD Global Oy",
    "24.586923": "Apple, Inc.",
    "24.5255080": "Cisco Systems, Inc",
    "24.13905349": "Panasonic i-PRO Sensing Solutions Co., Ltd.",
    "24.3712237": "Xiaomi Communications Co Ltd",
    "24.43632": "LG Electronics (Mobile Communications)",
    "24.43630": "Cisco Systems, Inc",
    "24.6033403": "Amino Communications Ltd",
    "24.1601815": "eta plus electronic gmbh",
    "24.12332616": "ARRIS Group, Inc.",
    "24.3469195": "NTT Communications Corporation",
    "24.1316510": "Wingtech Group (HongKong）Limited",
    "24.4512231": "Ubiquiti Networks Inc.",
    "12.123917679214592": "RANIX,Inc.",
    "12.123917683208192": "Oledcomm",
    "24.8923315": "Huawei Technologies Co.,Ltd",
    "24.16579579": "Cisco Systems, Inc",
    "20.48543329943552": "Private",
    "24.14464172": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "12.123917687971840": "Frog Cellsat Limited",
    "24.538034": "CoreEdge Networks Co., Ltd",
    "24.8968755": "Beijing Xiaoyuer Network Technology Co., Ltd",
    "24.1311058": "Samsung Electronics Co.,Ltd",
    "12.123917687889920": "Tiama",
    "24.10810343": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.13397668": "Samsung Electronics Co.,Ltd",
    "12.346854670336": "Confidence Direct Ltd",
    "24.8425920": "AgileMesh, Inc.",
    "24.7919442": "Huawei Technologies Co.,Ltd",
    "12.346868678656": "Rohde&Schwarz Topex SA",
    "24.9192045": "Huawei Technologies Co.,Ltd",
    "12.123917687074816": "Matrix Telematics Limited",
    "24.1074402": "ADFweb.com s.r.l.",
    "12.346858487808": "Silex Industrial Automation Ltd.",
    "12.123917686915072": "Prozess Technologie",
    "24.7621928": "Routerboard.com",
    "24.8670979": "Ablelink Electronics Ltd",
    "24.2634914": "Cisco Systems, Inc",
    "24.5520086": "Samsung Electronics Co.,Ltd",
    "12.346866950144": "Rosslare Enterprises Limited",
    "24.5523265": "Xcheng Holding",
    "24.800583": "zte corporation",
    "24.4755958": "Shenzhen Reach software technology CO.,LTD",
    "12.346866933760": "Jetstream Ltd.",
    "24.9178486": "Huawei Technologies Co.,Ltd",
    "24.14995087": "Apple, Inc.",
    "12.123917683089408": "Amada Miyachi America Inc.",
    "24.11017860": "Quanta Computer Inc.",
    "24.9181401": "Shenzhen RF Technology Co., Ltd",
    "12.346859659264": "eta systemi CKB",
    "20.128510968463360": "AStar Design Service Technologies Co., Ltd.",
    "12.123917681999872": "Ketronixs Sdn Bhd",
    "24.14210686": "Gsk Cnc Equipment Co.,Ltd",
    "12.123917683736576": "Adimec Advanced Image Systems",
    "12.123917675626496": "Alcodex Technologies Private Limited",
    "12.346867253248": "Edgeware AB",
    "24.15467123": "Aruba, a Hewlett Packard Enterprise Company",
    "20.110373791334400": "Sensoro Co., Ltd.",
    "24.47606": "Shenzhen Super Rich Electronics Co.,Ltd",
    "24.4458251": "Google, Inc.",
    "20.48548680826880": "Amaryllo International Inc.",
    "24.16283016": "Huawei Technologies Co.,Ltd",
    "12.346852241408": "Sensys Technologies Inc.",
    "20.105645774733312": "Annapurna labs",
    "24.2679447": "Shanghai InfoTM Microelectronics Co.,Ltd.",
    "20.145735987232768": "Wormit",
    "24.8145576": "MindTree Wireless PVT Ltd",
    "20.273272595808256": "Art Spa",
    "24.3975668": "Intel Corporate",
    "24.12350836": "Huawei Technologies Co.,Ltd",
    "12.346867744768": "Eyelock Corporation",
    "24.3427818": "zte corporation",
    "24.10234175": "Apple, Inc.",
    "24.12093667": "Compal Information (Kunshan) Co., Ltd.",
    "24.1607767": "Beijing Jinhong Xi-Dian Information Technology Corp.",
    "12.346858622976": "MacDonald Humfrey (Products) Ltd",
    "24.16039063": "Shenzhen Crave Communication Co., LTD",
    "12.346859859968": "E S P Technologies Ltd",
    "24.3174151": "Nations Technologies Inc.",
    "12.123917678071808": "Signals and systems india pvt ltd",
    "20.163603974979584": "Hanzhuo Information Technology(Shanghai) Ltd.",
    "24.11808504": "Eline Technology co.Ltd",
    "12.346867044352": "ENSIS Co., Ltd.",
    "24.5045236": "Optimal Audio Ltd",
    "24.14263": "Sagemcom Broadband SAS",
    "24.5305579": "Apple, Inc.",
    "24.7873914": "LEO Innovation Lab",
    "24.4229544": "WiredIQ, LLC",
    "24.6820644": "Huawei Device Co., Ltd.",
    "24.2666700": "Xenya d.o.o.",
    "24.10099490": "InfoCore Co.,Ltd.",
    "24.7352139": "Steelcase Inc.",
    "24.12622836": "IEEE Registration Authority",
    "24.8426387": "Xapt GmbH",
    "24.525697": "Roku, Inc.",
    "12.123917680922624": "Rehwork GmbH",
    "24.3424437": "Ibm",
    "24.7112724": "Intel Corporate",
    "24.7654998": "Packet Force Technology Limited Company",
    "24.3224": "LETEK Communications Inc.",
    "20.167295466864640": "SureFlap Ltd",
    "20.220958186012672": "BBPOS International Limited",
    "24.9482193": "netKTI Co., Ltd",
    "24.7080406": "Digiquest Electronics LTD",
    "24.14732270": "Samsung Electronics Co.,Ltd",
    "24.7882081": "CyberTAN Technology Inc.",
    "24.12690": "TP-Link Corporation Limited",
    "24.13389548": "HUMAX Co., Ltd.",
    "20.70443733090304": "Chongqing IQIYI Intelligence Technology Co., Ltd.",
    "12.123917686980608": "GHM-Messtechnik GmbH (Standort IMTRON)",
    "12.346854445056": "asetek Inc.",
    "20.251330449899520": "Shanghai DGE Co., Ltd",
    "12.123917684523008": "Contec Americas Inc.",
    "20.132869109317632": "Shenzhen Envicool Information Technology Co., Ltd",
    "20.61822317101056": "Dongguan Innovation Technology Co Ltd",
    "24.5338": "Huntleigh Healthcare",
    "24.5339": "Elma Trenew Electronic GmbH",
    "24.5332": "K Technology Corporation",
    "24.5333": "Datang Telecom Technology CO. , LCD,Optical Communication Br",
    "24.5330": "Kyuden Technosystems Corporation",
    "24.5331": "Sepsa",
    "24.5336": "bio-logic SA",
    "24.5337": "IP Fabrics, Inc.",
    "24.5334": "Jeongmin Electronics Co.,Ltd.",
    "24.5335": "Datastore Technology Corp",
    "24.4069": "Mercury Security Corporation",
    "24.4068": "Pantech Co.,Ltd",
    "24.4061": "Sordin Ab",
    "24.4060": "Ueda Japan Radio Co., Ltd.",
    "24.4063": "SOLOMON Technology Corp.",
    "24.4062": "Sony Mobile Communications Inc",
    "24.4065": "Id Digital Corporation",
    "24.4064": "NComputing Co.,Ltd.",
    "24.4067": "Damm Cellular Systems A/S",
    "24.4066": "Hangzhou H3C Technologies Co., Limited",
    "12.71297884516352": "Aplex Technology Inc.",
    "24.9752693": "CIS Crypto",
    "24.8453498": "BLU Products Inc",
    "12.123917682667520": "United States Technologies Inc.",
    "24.5533010": "Topview Optronics Corp.",
    "12.123917677686784": "Overline Systems",
    "24.10774278": "Apple, Inc.",
    "24.1842429": "Dalian Hi-Think Computer Technology, Corp",
    "24.7653673": "Zhejiang Dahua Technology Co., Ltd.",
    "12.123917684314112": "WoKa-Elektronik GmbH",
    "24.8413125": "Apple, Inc.",
    "24.11325880": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1854536": "Itel Mobile Limited",
    "24.14958498": "Wuxi DSP Technologies Inc.",
    "24.7882452": "Samsung Electronics Co.,Ltd",
    "12.123917677510656": "ifak technology + service GmbH",
    "24.57435": "West End Systems Corp.",
    "24.57434": "Galea Network Security",
    "24.57437": "Unitec Co., Ltd.",
    "24.57436": "PHC Corporation",
    "24.57431": "Han Microtelecom. Co., Ltd.",
    "24.57430": "Holontech Corporation",
    "24.57433": "Controlled Environments, Ltd.",
    "24.57432": "Phase One Denmark A/S",
    "12.346855579648": "St. Michael Strategies",
    "24.57439": "e-Net, Inc.",
    "24.57438": "Japan Aviation Electronics Industry, Ltd.",
    "24.6038294": "Squirrels Research Labs LLC",
    "24.13399192": "Filmetrics Inc.",
    "12.123917678166016": "Wartsila Voyage Limited",
    "24.4210945": "Rockwell Automation",
    "24.3432824": "The Boeing Company",
    "12.346855444480": "Uxp",
    "12.346860150784": "Benshaw Canada Controls, Inc.",
    "24.3598": "ESA elettronica S.P.A.",
    "24.3599": "Ermme",
    "24.2650821": "Smartmatic Corp",
    "12.123917690187776": "Shenzhen SanYeCao Electronics Co.,Ltd",
    "24.3590": "Team Simoco Ltd",
    "24.3591": "Sony Mobile Communications Inc",
    "24.3592": "Cisco-Linksys, LLC",
    "24.3593": "Shenzhen Coship Software Co.,LTD.",
    "24.3594": "Sakuma Design Office",
    "24.3595": "Netac Technology Co., Ltd.",
    "24.3596": "Intel Corporation",
    "24.3597": "Hesch Schröder GmbH",
    "24.8949352": "Huawei Device Co., Ltd.",
    "20.264809154084864": "New IT Project LLC",
    "24.4238477": "Data Management, Inc.",
    "24.7065": "KS System GmbH",
    "24.15392985": "Berk-tek LLC",
    "12.346853769216": "dresden-elektronik",
    "12.346858602496": "Comtest Networks",
    "12.346852687872": "Aim",
    "24.7067": "Hose-McCann Communications",
    "24.11591482": "Samsung Electronics Co.,Ltd",
    "12.123917681942528": "PTYPE Co., LTD.",
    "12.346857009152": "GCS, Inc",
    "24.15490152": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.3042": "Lumenera Corporation",
    "24.3043": "Key Stream Co., Ltd.",
    "24.3040": "SercoNet Ltd.",
    "24.3041": "Nokia NET Product Operations",
    "24.3046": "Datel Electronics",
    "24.3047": "Comflux Technology Inc.",
    "24.3044": "Hosiden Corporation",
    "24.3045": "HIMS International Corporation",
    "24.3048": "Aoip",
    "24.3049": "Actel Corporation",
    "20.268735394873344": "Huaqin Telecom Technology Co.,Ltd.",
    "12.346867396608": "Lamson Safes & Security",
    "24.13164967": "Vertu Corporation Limited",
    "12.123917678051328": "Telefire",
    "12.123917680979968": "STENTORIUS by ADI",
    "24.16262958": "Mercku",
    "12.119269298176": "OOO NPP Mera",
    "24.9474144": "Rsi Video Technologies",
    "12.346860937216": "Radiometer Medical ApS",
    "24.20525": "Accel, Inc.",
    "24.20524": "Soyo Computer, Inc.",
    "24.20527": "TollBridge Technologies, Inc.",
    "24.20526": "Cambex Corporation",
    "24.20521": "1394 Printer Working Group",
    "24.20520": "Aval Communications",
    "24.20523": "Genrad Ltd.",
    "24.20522": "Cisco Systems, Inc",
    "20.137141729361920": "Changsha SUNYE Electric Co., Ltd.",
    "24.16307298": "Pumatronix Equipamentos Eletronicos Ltda.",
    "24.20529": "Aeroflex Laboratories, Inc.",
    "24.20528": "Future Plus Systems",
    "12.346852843520": "Arula Systems, Inc.",
    "12.123917681094656": "PROEL TSI s.r.l.",
    "24.12604967": "Belkin International Inc.",
    "24.6322146": "Samsung Electronics Co.,Ltd",
    "12.123917682388992": "Alere Technologies AS",
    "24.6879381": "Redflow Limited",
    "24.12845129": "Kamama",
    "20.70443734138880": "disguise Technologies Limited",
    "12.346866905088": "Trentino Systems",
    "20.176069205295104": "Applied Information, Inc.",
    "24.10503088": "Hangzhou BroadLink Technology Co.,Ltd",
    "12.346862002176": "Shockfish SA",
    "24.8917582": "Qualcomm Inc.",
    "24.6824779": "Nihon Dengyo Kousaku",
    "20.57353097445376": "BlackEdge Capital",
    "24.6916": "Affinity International S.p.a",
    "24.9206685": "Apple, Inc.",
    "24.530001": "Shenzhen Sea Star Technology Co.,Ltd",
    "24.8687047": "Cisco SPVTG",
    "12.123917680381952": "Selex ES Inc.",
    "24.8676713": "New H3C Technologies Co., Ltd",
    "12.123917686185984": "Xemex NV",
    "24.6033035": "Extreme Networks, Inc.",
    "24.6841168": "Hella India Automotive Pvt Ltd",
    "12.346853560320": "Enea Real Time AB",
    "24.1060771": "Shenzhen Uvision-tech Technology Co.Ltd",
    "12.123917688659968": "YUYAMA MFG Co.,Ltd",
    "24.8956327": "IEEE Registration Authority",
    "20.35227550351360": "Irsap",
    "24.10494542": "IEEE Registration Authority",
    "24.8149607": "JNC Systems, Inc.",
    "20.58071052189696": "Shenzhen Rikomagic Tech Corp.,Ltd",
    "12.71297884975104": "TEWS Elektronik GmbH & Co. KG",
    "12.123917691535360": "AERIAL CAMERA SYSTEMS Ltd",
    "24.7087993": "Microsoft Mobile Oy",
    "24.7749": "Sony Mobile Communications Inc",
    "24.7748": "Santec",
    "24.12068250": "Thalmic Labs",
    "12.123917686898688": "Internet Protocolo Logica Sl",
    "24.7741": "Alpsalpine Co,.Ltd",
    "24.7740": "Lyngbox Media AB",
    "24.7743": "TrellisWare Technologies, Inc.",
    "24.7742": "KMW Inc.",
    "24.7745": "Microwave Communication & Component, Inc.",
    "24.7744": "Shanghai DareGlobal Technologies Co.,Ltd",
    "24.7747": "Aisin Aw Co.,Ltd.",
    "24.540478": "WSH GmbH",
    "24.2719": "Pannaway Technologies, Inc.",
    "24.2718": "BroadWeb Corportation",
    "24.6584816": "Intel Corporate",
    "24.1087683": "Murata Manufacturing Co., Ltd.",
    "24.11800499": "vivo Mobile Communication Co., Ltd.",
    "24.8436060": "ELFTECH Co., Ltd.",
    "24.2711": "SONICblue, Inc.",
    "24.2710": "Mewtel Technology Inc.",
    "24.2713": "Calamp Wireless Networks Inc",
    "24.2712": "M+F Gwinner GmbH & Co",
    "24.2715": "TB Group Inc",
    "24.2714": "Aiptek International Inc",
    "24.2717": "King Young Technology Co. Ltd.",
    "24.2716": "Server Technology, Inc.",
    "12.123917684572160": "Slat",
    "24.9736143": "ARRIS Group, Inc.",
    "24.12338352": "Elastifile",
    "12.346858442752": "MEG Electronic Inc.",
    "12.346859335680": "Mavenir System, Inc.",
    "24.7887912": "Prime Motion Inc.",
    "12.71297884921856": "Broachlink Technology Co.,Limited",
    "24.11539464": "Lightspeed International Co.",
    "24.14185994": "Escort, Inc.",
    "12.123917684826112": "Blocksi Llc",
    "24.8970142": "New H3C Technologies Co., Ltd",
    "24.14180528": "bioMérieux Italia S.p.A.",
    "20.189244299542528": "Motec Pty Ltd",
    "12.123917683126272": "Stone Three",
    "24.3174834": "Hangzhou Prevail Optoelectronic Equipment Co.,LTD.",
    "20.97755441659904": "DivioTec Inc.",
    "12.346860224512": "IMP Telekom",
    "24.1327897": "Creative&Link Technology Limited",
    "24.317098": "Samsung Electro-Mechanics(Thailand)",
    "12.346863288320": "Star Electronics GmbH & Co. KG",
    "24.14443644": "C.R.S. iiMotion GmbH",
    "12.123917682524160": "NuRi&G Engineering co,.Ltd.",
    "12.346862788608": "Nippon Manufacturing Service Corporation (abbreviated as 'nms')",
    "24.6299905": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.6299907": "I4Vine, Inc",
    "24.14714801": "Nata-Info Ltd.",
    "20.172584097284096": "Foshan Synwit Technology Co.,Ltd.",
    "24.2037": "Bridgeco Co AG",
    "20.220327867056128": "Shenzhen Zero Zero Infinity Technology Co.，Ltd.",
    "24.12871605": "Libratone A/S",
    "12.346860498944": "Sorec",
    "24.12083096": "Amazon Technologies Inc.",
    "24.8546": "Nortel Networks",
    "24.8547": "Askey Computer Corp",
    "24.8544": "Hidea Solutions Co. Ltd.",
    "24.8545": "Yournet Inc.",
    "24.8542": "IBM Corp",
    "24.8543": "IHSE GmbH",
    "24.8540": "Intel Corporate",
    "24.8541": "Intel Corporate",
    "24.6312129": "Kinetek Sports",
    "24.8548": "Special Design Bureau for Seismic Instrumentation",
    "24.8549": "Presstek Inc.",
    "24.15492817": "B&W Group LTD",
    "24.10776948": "Huawei Technologies Co.,Ltd",
    "24.8432764": "Sichuan AI-Link Technology Co., Ltd.",
    "12.119269642240": "Plair Media Inc.",
    "24.1592980": "Messoa Technologies Inc.",
    "24.2883831": "Xos",
    "24.13129920": "Intel Corporate",
    "24.4772675": "ARRIS Group, Inc.",
    "20.22126499201024": "Deutsche Energieversorgung GmbH",
    "24.14475948": "Lumens Digital Optics Inc.",
    "24.5071": "4Access Communications",
    "24.5070": "Intel Corporate",
    "24.5073": "KIRK telecom A/S",
    "24.5072": "t+ Medical Ltd",
    "24.5075": "Micro-Star International Co., Ltd.",
    "24.5074": "Page Iberica, S.A.",
    "24.5077": "RuggedCom",
    "24.5076": "ASUSTek COMPUTER INC.",
    "24.5079": "SPIDCOM Technologies SA",
    "24.5078": "Tii Network Technologies, Inc.",
    "24.12363177": "Apple, Inc.",
    "24.2633900": "Frontiir Co. Ltd.",
    "24.8955001": "Apple, Inc.",
    "24.4743263": "LG Electronics (Mobile Communications)",
    "24.2905133": "Cisco Systems, Inc",
    "12.71297883697152": "Wheatstone Corporation",
    "24.811471": "Texas Instruments",
    "24.14475499": "Nanjing Aozheng Information Technology Co.Ltd",
    "24.1098287": "China Entropy Co., Ltd.",
    "12.346853580800": "Plextek Limited",
    "12.346858651648": "Aurora Flight Sciences",
    "20.247076406427648": "Shenzhen Xrinda Technology Ltd",
    "12.123917686767616": "GoTrustID Inc.",
    "20.255191449337856": "Protek Electronics Group Co.,LTD",
    "24.578572": "Intel Corporate",
    "24.13664634": "Apple, Inc.",
    "24.9477501": "ARRIS Group, Inc.",
    "12.123917677129728": "Savari Inc",
    "12.123917688741888": "Monnit Corporation",
    "24.2120490": "Private",
    "24.1851012": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917681741824": "Sanmina Israel",
    "12.346859466752": "infoplan Gesellschaftfür Informationssysteme mbH",
    "24.2124938": "Extreme Networks, Inc.",
    "24.1362976": "Tp-Link Technologies Co.,Ltd.",
    "24.5801954": "Shenzhen Coship Electronics Co., Ltd.",
    "12.123917680791552": "Nefteavtomatika",
    "24.2656533": "CashGuard Sverige AB",
    "12.346853974016": "Interzeag Ag",
    "24.3465423": "zte corporation",
    "12.123917682237440": "Eleven Engineering Incorporated",
    "24.10238618": "Shenzhen Sundray Technologies Company Limited",
    "12.346859933696": "BES GmbH",
    "12.123917690056704": "Argosy Labs Inc.",
    "20.176772036427776": "Viettronimex JSC",
    "24.6430": "Beyondwiz Co., Ltd.",
    "24.9502067": "zte corporation",
    "24.1618874": "Netlogic AB",
    "24.3193264": "Intracom Asia Co., Ltd",
    "24.8684595": "Paradox Engineering SA",
    "24.9726640": "Arcadyan Corporation",
    "12.346862194688": "UGL Limited",
    "24.14195252": "Beijing SHENQI Technology Co., Ltd.",
    "24.15522257": "Shenzhen TG-NET Botone Technology Co.,Ltd.",
    "24.6054297": "Cloud Network Technology Singapore Pte. Ltd.",
    "20.168210313773056": "Leia, Inc",
    "12.346854903808": "Globe Wireless",
    "20.176069201100800": "ProPhotonix",
    "24.551120": "Japan System Design",
    "24.14688136": "Xiaomi Communications Co Ltd",
    "12.119269519360": "PT. Amanindo Nusapadu",
    "24.3997656": "Huawei Technologies Co.,Ltd",
    "24.7383267": "Huawei Technologies Co.,Ltd",
    "12.346868236288": "Hella Gutmann Solutions GmbH",
    "24.11030875": "Samsung Electronics Co.,Ltd",
    "12.71297884286976": "Cavalry Storage Inc",
    "20.175931702378496": "Guangdong Jinpeng Technology Co. LTD",
    "12.71297884258304": "Alpha Design Co.,Ltd.",
    "20.39919501180928": "Mighty Audio, Inc.",
    "24.570019": "Cynny Italia S.r.L.",
    "12.123917691080704": "Kohler Mira Ltd",
    "24.10787143": "Huawei Technologies Co.,Ltd",
    "12.346857095168": "Matrix Audio Designs",
    "12.346865512448": "Coptonix GmbH",
    "12.346862174208": "MGM-Devices Oy",
    "24.11571508": "Taleo",
    "12.123917689630720": "SPC Bioclinicum",
    "12.346865963008": "Axible Technologies",
    "24.2230": "RouteFree, Inc.",
    "12.123917677903872": "Hodwa Co., Ltd",
    "24.5544818": "Ericsson AB",
    "24.8418207": "Kyocera Corporation",
    "24.36314": "Link One Co., Ltd.",
    "12.346859917312": "Phytec Messtechnik GmbH",
    "12.123917675827200": "Lineage Power Pvt Ltd.,",
    "24.10770124": "Technicolor CH USA Inc.",
    "24.15241752": "Arcontia International AB",
    "24.3460136": "Nexpring Co. LTD.,",
    "24.7872475": "Samsung Electronics Co.,Ltd",
    "24.15793568": "Acurix Networks Pty Ltd",
    "24.16037605": "TerraSem Co.,Ltd",
    "24.11860297": "optilink networks pvt ltd",
    "24.14461188": "Apple, Inc.",
    "12.346855514112": "Punjab Communications Ltd",
    "24.10532314": "HongKong THTF Co., Ltd",
    "20.84197029969920": "Atrie Technology Fzc",
    "24.2674983": "Beijing Xiaomi Mobile Software Co., Ltd",
    "20.96896785842176": "SAMIL CTS Co., Ltd.",
    "24.5540248": "Huawei Technologies Co.,Ltd",
    "24.11564365": "Nokia",
    "12.346865463296": "Fr. Sauter AG",
    "12.346860511232": "Saia-Burgess Controls AG",
    "12.346857820160": "Workpower Tecnologia Eletronica Ltda-Epp",
    "12.346857574400": "Phytec Messtechnik GmbH",
    "24.6601816": "Intel Corporate",
    "24.7665883": "IEEE Registration Authority",
    "12.346862264320": "SAMsystems GmbH",
    "12.123917676179456": "Geppetto Electronics",
    "12.346864119808": "RC Systems Co. Inc.",
    "24.9222244": "AcSiP Technology Corp.",
    "24.10758583": "bluesky",
    "24.550384": "JM Zengge Co., Ltd",
    "24.2234": "Erskine Systems Ltd",
    "24.1884213": "Planex Communications Inc.",
    "24.16033593": "Juniper Networks",
    "20.193649357881344": "Sound United",
    "24.10794041": "Cisco Systems, Inc",
    "12.346857238528": "Delta Tau Data Systems, Inc.",
    "12.123917676298240": "Detec Systems Ltd",
    "12.346858242048": "Enseo, Inc.",
    "24.12350870": "Beijing Broadwit Technology Co., Ltd.",
    "12.346858749952": "Fidelity Comtech, Inc.",
    "24.1893917": "Samsung Electronics Co.,Ltd",
    "24.15002": "Cisco Systems, Inc",
    "24.15003": "Cisco Systems, Inc",
    "24.15000": "Cisco Systems, Inc",
    "24.15001": "Cisco Systems, Inc",
    "24.15004": "Cisco Systems, Inc",
    "24.15005": "NEC Platforms, Ltd.",
    "12.123917676048384": "Internet Protocolo Logica Sl",
    "24.5557865": "Hikari Trading Co.,Ltd.",
    "12.346857861120": "Gma, Llc",
    "24.36952": "Ultra Electronics Command & Control Systems",
    "24.4728364": "Apple, Inc.",
    "24.5515116": "Jiangsu Houge Technology Corp.",
    "24.5253235": "Huawei Device Co., Ltd.",
    "12.346860052480": "Keith & Koep GmbH",
    "24.14966873": "Pingtek Co., Ltd.",
    "24.1630145": "Aetheros",
    "24.269767": "zte corporation",
    "24.1593939": "Pixord Corporation",
    "12.346865618944": "Arcos Technologies Ltd.",
    "24.8679943": "I&C Technology",
    "12.346865176576": "Rebo Co.,Ltd.",
    "24.4995288": "ARRIS Group, Inc.",
    "12.119269564416": "Engineering Center ENERGOSERVICE",
    "24.13922854": "zte corporation",
    "24.1875262": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.189244306882560": "FINEpowerX INC",
    "24.6089245": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7115604": "Yaojin Technology (Shenzhen) Co., LTD.",
    "24.6074118": "LG Electronics (Mobile Communications)",
    "24.15783783": "Axema Passagekontroll AB",
    "12.346863087616": "Explorer Inc.",
    "20.162752337281024": "Shenzhen Baolijie Technology Co., Ltd.",
    "24.9465288": "DLG Automação Industrial Ltda",
    "24.13109363": "Ruckus Wireless",
    "24.6562581": "Huawei Device Co., Ltd.",
    "24.10795392": "Parking BOXX Inc.",
    "20.39986084708352": "Guangzhou Lango Electronics Technology Co.,Ltd.",
    "24.306244": "Holtek Semiconductor Inc.",
    "24.13904911": "ARRIS Group, Inc.",
    "12.123917686145024": "Simple Works, Inc.",
    "24.12325889": "Hitron Technologies. Inc",
    "24.49293": "Tronix Product Development",
    "12.123917678551040": "Asiga",
    "20.14286628847616": "Vermes Microdispensing GmbH",
    "24.3960823": "EnTek Systems, Inc.",
    "24.7904976": "Samsung Electronics Co.,Ltd",
    "24.12613681": "Huawei Device Co., Ltd.",
    "24.5821329": "Quectel Wireless Solutions Co., Ltd.",
    "24.8183876": "NEON Inc",
    "24.6863315": "Shenzhen Trolink Technology CO, LTD",
    "24.558237": "Amazon Technologies Inc.",
    "20.251330459336704": "Shenzhen Arronna Telecom Co.,Ltd",
    "24.16515090": "Toshiba Samsung Storage Technolgoy Korea Corporation",
    "24.2673643": "One",
    "24.562861": "Cisco Systems, Inc",
    "12.346865266688": "Kinects Solutions Inc",
    "12.346864095232": "Nayos LTD",
    "24.11033587": "Audivo GmbH",
    "12.123917682614272": "YUYAMA MFG Co.,Ltd",
    "12.123917691576320": "Fx Technology Limited",
    "24.16532639": "zte corporation",
    "12.123917691203584": "ASL Holdings",
    "24.14700477": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.16532633": "Swarco LEA d.o.o.",
    "12.123917688807424": "ITsynergy Ltd",
    "24.4244694": "ChongQing Camyu Technology Development Co.,Ltd.",
    "24.1634532": "Xiaomi Communications Co Ltd",
    "24.13145298": "Jiangsu Datang Electronic Products Co., Ltd",
    "12.346856280064": "The Board Room Inc.",
    "24.11319119": "METEL s.r.o.",
    "20.145211806187520": "Futurecom Systems Group",
    "24.7912474": "Cisco Systems, Inc",
    "20.216145521016832": "BA International Electronics Co. Ltd.",
    "12.123917687009280": "Cetto Industries",
    "12.346855960576": "Isensix",
    "12.71297884450816": "GDE Polska",
    "12.346852577280": "Clever Devices",
    "24.6574941": "Frauscher Sensortechnik",
    "24.539228": "Junilab, Inc.",
    "20.277958193315840": "Annapurna labs",
    "24.5822163": "Dairy Cheq Inc",
    "12.123917687107584": "Profound Medical Inc.",
    "24.5826678": "IEEE Registration Authority",
    "24.5826675": "HANGZHOU DANGBEI NETWORK TECH.Co.,Ltd",
    "24.4765859": "Apple, Inc.",
    "20.181338945617920": "Wuxi Junction Infomation Technology Incorporated Company",
    "24.5000757": "Cisco Systems, Inc",
    "24.15217421": "Xaptec GmbH",
    "12.346853445632": "Sr Research Ltd",
    "24.8147545": "Sichuan Jiuzhou Electronic Technology Co., Ltd.",
    "24.10256509": "SYSCABLE Korea Inc.",
    "12.346864709632": "Phytec Messtechnik GmbH",
    "24.6303": "Lenntek Corporation",
    "24.8707645": "Crystal Vision Ltd",
    "12.123917675790336": "Molu Technology Inc., LTD.",
    "12.346853617664": "Computerwise, Inc.",
    "24.16520711": "Taian Technology(Wuxi) Co.,Ltd.",
    "20.62636953698304": "Fast Cotton(Beijing) Limited",
    "24.275333": "Nanchang BlackShark Co.,Ltd.",
    "24.4198876": "Sonance",
    "24.275337": "Huawei Technologies Co.,Ltd",
    "24.13127913": "Raisecom Technology CO., LTD",
    "12.123917688717312": "Ligpt",
    "24.14730812": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.2893407": "IEEE Registration Authority",
    "24.12091800": "Intel Corporate",
    "24.7123495": "Sony Video & Sound Products Inc.",
    "24.2387852": "Huawei Device Co., Ltd.",
    "24.16292472": "Huawei Technologies Co.,Ltd",
    "24.6559472": "Huawei Technologies Co.,Ltd",
    "24.4206120": "MiXTelematics",
    "20.52912294199296": "Fine Triumph Technology Corp.,Ltd.",
    "24.4767037": "New H3C Technologies Co., Ltd",
    "24.16557601": "Samsung Electronics Co.,Ltd",
    "12.123917685575680": "Intercore GmbH",
    "24.14728094": "Seiko Epson Corporation",
    "24.9178851": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1863525": "Giga-Byte Technology Co.,Ltd.",
    "12.346857771008": "Fail Safe Solutions LLC",
    "24.5287314": "NX Technologies",
    "24.9465893": "Garea Microsys Co.,Ltd.",
    "12.71297884065792": "Yuko Electric Co.,Ltd",
    "12.123917689761792": "Suzhou ZhiCai Co.,Ltd.",
    "24.14736832": "vivo Mobile Communication Co., Ltd.",
    "24.9226315": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.2142209": "Technicolor",
    "24.14196153": "eMegatech International Corp.",
    "24.5254748": "HTC Corporation",
    "12.123917676326912": "A & T Technologies",
    "24.13939322": "ARRIS Group, Inc.",
    "24.12106779": "Huawei Technologies Co.,Ltd",
    "24.12369935": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12379551": "WATERWORLD Technology Co.,LTD",
    "12.123917680812032": "ITK Dr. Kassen GmbH",
    "24.1105329": "Apple, Inc.",
    "12.346865442816": "Dsp Design Ltd",
    "24.2365037": "Huawei Technologies Co.,Ltd",
    "24.13901933": "HTC Corporation",
    "24.11335667": "Xiaomi Communications Co Ltd",
    "20.22739800817664": "IO Industries Inc.",
    "24.8681336": "Cochlear Limited",
    "24.14478724": "Haverford Systems Inc",
    "24.808540": "HyBroad Vision (Hong Kong) Technology Co Ltd",
    "12.346865623040": "Arktan Systems",
    "12.71297884127232": "Heitec Ag",
    "20.62335512215552": "Freedompro Srl",
    "24.16524": "Axis Communications Ab",
    "24.16525": "The Goodyear Tire & Rubber Co.",
    "24.16526": "Tattile SRL",
    "24.16527": "Wm-Data Minfo Ab",
    "24.16520": "Mobuis",
    "24.16521": "Meidensha Corporation",
    "24.16522": "Tps Teleprocessing Sys. Gmbh",
    "24.16523": "Raylan Corporation",
    "24.45101": "ViaGate Technologies, Inc.",
    "24.16528": "Ansel Communications",
    "24.16529": "Procomp Industria Eletronica",
    "24.14481496": "Lorent Networks, Inc.",
    "24.4219441": "Gifa",
    "12.71297883987968": "comtime GmbH",
    "24.4219444": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "24.14219395": "Intel Corporate",
    "24.14014": "Northwest Towers",
    "24.12636893": "Samsung Electronics Co.,Ltd",
    "24.16515533": "Fundacion Tekniker",
    "24.9500242": "Tp-Link Technologies Co.,Ltd.",
    "24.58374": "Huawei Technologies Co.,Ltd",
    "24.16535696": "SIMEX Sp. z o.o.",
    "24.2915162": "Milper Ltd",
    "24.3951824": "Ceragon Networks",
    "24.526774": "Masimo Corp",
    "24.5279": "System and Chips, Inc.",
    "24.5278": "UbONE Co., Ltd",
    "24.5277": "Sound ID Inc.",
    "24.5276": "HF Company",
    "24.5275": "Nokota Communications, LLC",
    "24.5274": "ARRIS Group, Inc.",
    "24.5273": "Helicomm Inc",
    "24.5272": "Viking Design Technology",
    "24.5271": "ZHIYUAN Eletronics co.,ltd.",
    "24.5270": "Phonic Corp.",
    "12.346866388992": "Chauvin Arnoux",
    "12.119269294080": "Softel SA de CV",
    "12.346865762304": "beks Kommunikacios Technika kft",
    "12.123917681119232": "CT Company",
    "24.5509169": "Smarto",
    "24.12382577": "Intel Corporate",
    "24.4731869": "Zennio Avance Y Tecnologia, S.L.",
    "24.6898": "Dynavisions Schweiz AG",
    "24.6899": "Samyoung Electronics",
    "12.71297884438528": "digitech GmbH &amp; Co. KG",
    "24.6892": "Keumbee Electronics Co.,Ltd.",
    "24.6893": "INCOTEC GmbH",
    "24.6890": "Radio Terminal Systems Pty Ltd",
    "24.6891": "Allied Telesis R&D Center K.K.",
    "24.6896": "Alcatel-Lucent IPD",
    "24.6897": "Embedded Artists AB",
    "24.6894": "Shenztech Ltd",
    "24.6895": "Loopcomm Technology, Inc.",
    "24.4226319": "Huawei Technologies Co.,Ltd",
    "24.4475741": "GE Healthcare",
    "24.12326310": "Taiwan Jantek Electronics,Ltd.",
    "24.12326316": "Vodafone Italia S.p.A.",
    "24.11548045": "Nokia Corporation",
    "24.8190080": "Cisco Systems, Inc",
    "24.14352235": "Team Precision Public Company Limited",
    "24.14715262": "Apple, Inc.",
    "24.6327575": "Sentrus Government Systems Division, Inc",
    "12.123917685043200": "ioThings",
    "12.346860302336": "Ast Incorporated",
    "12.123917678243840": "Iotopia Solutions",
    "12.71297884839936": "Advanced Micro Controls Inc.",
    "24.5034187": "Huawei Technologies Co.,Ltd",
    "12.123917680201728": "Orlaco Products B.V.",
    "24.11848535": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.71297883316224": "Digimerge Technology Inc",
    "12.123917678624768": "OOO NPP Mars-Energo",
    "12.123917689282560": "Grupo Epelsa S.L.",
    "12.123917690040320": "Iradimed",
    "24.9720067": "Samsung Electronics Co.,Ltd",
    "24.10248477": "Panasonic Corporation of North America",
    "12.346861273088": "BK Innovation, Inc.",
    "24.9185934": "DongGuan Ramaxel Memory Technology",
    "24.10257599": "Apple, Inc.",
    "24.16546818": "Phicomm (Shanghai) Co., Ltd.",
    "24.13421924": "SM-Electronic GmbH",
    "12.123917689884672": "NEUROPHET, Inc.",
    "24.8923465": "Renesas Electronics (Penang) Sdn. Bhd.",
    "24.15482615": "Juniper Networks",
    "24.2121031": "vivo Mobile Communication Co., Ltd.",
    "24.7363230": "Technicolor CH USA Inc.",
    "12.119269830656": "Ambient Devices, Inc.",
    "12.346855776256": "ELV Elektronik AG",
    "12.123917686181888": "Datamars SA",
    "20.211525440634880": "JAM-Labs Corp",
    "24.851255": "SUSE Linux GmbH",
    "24.4638": "Juniper Networks",
    "24.4639": "Harding Instruments",
    "24.10247310": "ASUSTek COMPUTER INC.",
    "24.4630": "ICP Internet Communication Payment AG",
    "24.4631": "Cisco-Linksys, LLC",
    "24.4632": "ARUZE Corporation",
    "24.4633": "General Datacomm LLC",
    "24.4634": "Techno Soft Systemnics Inc.",
    "24.4635": "Sound Devices, LLC",
    "24.4636": "Parrot Sa",
    "24.4637": "Netfabric Corporation",
    "24.3943": "West Instruments",
    "24.3942": "Cisco-Linksys, LLC",
    "24.3941": "icube Corp.",
    "24.3940": "D&R Electronica Weesp BV",
    "24.3947": "GateWare Communications GmbH",
    "24.3946": "Nortel Networks",
    "24.3945": "SEW Eurodrive GmbH & Co. KG",
    "24.3944": "Vavic Network Technology, Inc.",
    "24.3994873": "Moda-InnoChips",
    "24.3949": "Midas Engineering",
    "24.3948": "ADDI-DATA GmbH",
    "24.14449556": "Cisco Systems, Inc",
    "24.57570": "Innova Corp.",
    "24.57571": "Sk-Elektronik Gmbh",
    "24.57572": "FANUC ROBOTICS NORTH AMERICA, Inc.",
    "24.57573": "Cinco Networks, Inc.",
    "24.57574": "INCAA Computers",
    "24.57575": "Raytheon E-Systems, Inc.",
    "24.57576": "GRETACODER Data Systems AG",
    "24.57577": "Data Labs, Inc.",
    "24.57578": "Innovat Communications, Inc.",
    "24.57579": "Digicom Systems, Incorporated",
    "20.242185277538304": "EcoGuard AB",
    "24.13945008": "Prime Electronics & Satellitics Inc.",
    "12.123917681909760": "Open Grow",
    "24.11327529": "Bouffalo Lab (Nanjing) Co., Ltd.",
    "24.16299114": "Hewlett Packard",
    "12.123917679828992": "Haxe Systeme",
    "24.6834806": "eero inc.",
    "12.346864672768": "Private",
    "12.346854752256": "Replicom Ltd.",
    "12.346854629376": "Zendex Corporation",
    "24.1595992": "Dell Inc.",
    "12.346868834304": "Exibea AB",
    "20.272689494228992": "Bettini Srl",
    "24.2119599": "Shenzhen FH-NET OPTOELECTRONICS CO.,LTD",
    "12.123917675917312": "B.E.A. sa",
    "12.346857177088": "Neets",
    "12.346852265984": "Emtac Technology Corp.",
    "24.3367": "MICROPLEX Printware AG",
    "24.3366": "Primagraphics Limited",
    "24.3365": "Sanden Corporation",
    "24.3364": "Sentec E&E Co., Ltd.",
    "24.3363": "Smart Solution, Inc",
    "24.3362": "Unitronics LTD",
    "24.3361": "WISCORE Inc.",
    "24.3360": "Asahikasei Technosystem Co.,Ltd.",
    "24.13920211": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.3369": "Cisco Systems, Inc",
    "24.3368": "Cisco Systems, Inc",
    "12.346859847680": "Abeo Corporation",
    "24.11584620": "Senseit",
    "12.71297883820032": "EMAC, Inc.",
    "24.14691305": "Intel Corporate",
    "24.14998905": "En-Vision America, Inc.",
    "20.273458023890944": "Package Guard, Inc",
    "12.123917676085248": "Tex Computer Srl",
    "24.2413554": "zte corporation",
    "24.822269": "Intel Corporate",
    "24.8850": "Cinetal",
    "24.8851": "zte corporation",
    "24.8852": "Kyocera Corporation",
    "12.346865848320": "DECA Card Engineering GmbH",
    "24.8854": "LinoWave Corporation",
    "24.8855": "XMOS Semiconductor",
    "24.8856": "Sony Mobile Communications Inc",
    "24.8857": "SeaMicro Inc.",
    "24.8858": "Lastar, Inc.",
    "24.8859": "AverLogic Technologies, Inc.",
    "12.123917690695680": "Dave Srl",
    "12.123917688967168": "Azcom Technology S.r.l.",
    "24.53475": "Ele-Chem Engineering Co., Ltd.",
    "24.53474": "Mrt Micro, Inc.",
    "24.53477": "Solidum Systems Corp.",
    "24.53476": "Cisco Systems, Inc",
    "24.53471": "Kuzumi Electronics, Inc.",
    "24.53470": "Philips Multimedia Network",
    "24.53473": "Avionitek Israel Inc.",
    "24.53472": "Dooin Electronics Co.",
    "12.119269658624": "Datasat Digital Entertainment",
    "12.123917676716032": "Xirgo Technologies LLC",
    "24.53479": "Vcon Telecommunication Ltd.",
    "24.53478": "Ibond Inc.",
    "12.346853830656": "EPM Tecnologia e Equipamentos",
    "24.537969": "CASwell INC.",
    "12.346852274176": "Cross Products Ltd.",
    "24.3441734": "Ericsson AB",
    "24.14717244": "Kyeungin CNS Co., Ltd.",
    "24.10797281": "Nintendo Co., Ltd.",
    "24.8713978": "Miovision Technologies Incorporated",
    "12.346865213440": "Quail",
    "24.8692177": "Sagemcom Broadband SAS",
    "24.13374630": "Yichun MyEnergy Domain, Inc",
    "24.11570128": "Cisco Systems, Inc",
    "24.11570127": "Cisco Systems, Inc",
    "12.346856972288": "Acutelogic Corporation",
    "24.6463": "RDI technology(Shenzhen) Co.,LTD",
    "24.6462": "ADB Broadband Italia",
    "24.6461": "GMC Guardian Mobility Corp.",
    "24.6460": "HighPoint Technologies Incorporated",
    "24.6467": "Belden",
    "24.6466": "On Software International Limited",
    "24.6465": "Pitney Bowes, Inc",
    "24.6464": "Rackable Systems",
    "12.123917686431744": "Ya Batho Trading (Pty) Ltd",
    "24.6469": "RF COncepts, LLC",
    "24.6468": "Fossil Partners, L.P.",
    "24.16269137": "Samsung Electronics Co.,Ltd",
    "20.229558880043008": "Nanjing Magewell Electronics Co.,Ltd",
    "24.16557178": "IEEE Registration Authority",
    "20.167860146012160": "Mekotronics Co., Ltd",
    "12.346855690240": "NetTest A/S",
    "12.71297883242496": "APG Cash Drawer, LLC",
    "12.123917677658112": "Bresslergroup",
    "12.71297883197440": "Monarch Instrument",
    "24.48757": "Cisco Systems, Inc",
    "12.346865541120": "ConectaIP Tecnologia S.L.",
    "24.24752": "Hewlett Packard",
    "24.24753": "Input/Output, Inc.",
    "24.24750": "Trio Information Systems Ab",
    "24.24751": "Pacific Micro Data, Inc.",
    "24.24756": "Glenayre R&D Inc.",
    "24.24757": "KEBA GmbH",
    "24.24754": "Process Control Corp.",
    "24.24755": "Z-Com, Inc.",
    "24.13433320": "Samsung Electronics Co.,Ltd",
    "24.24758": "Land Computer Co., Ltd.",
    "24.24759": "Channelmatic, Inc.",
    "24.6092045": "Institute of microelectronic applications",
    "24.8433725": "Apple, Inc.",
    "12.346862292992": "GD Mission Systems",
    "24.9493318": "E-Lead Electronic Co., Ltd",
    "24.6287": "Montgomery Technology, Inc.",
    "24.6286": "Ekahau, Inc.",
    "24.6285": "Nokia Danmark A/S",
    "24.6284": "Mobile Action Technology Inc.",
    "24.6283": "Dell Inc.",
    "24.6282": "Infinova LLC",
    "24.6281": "WinNet Solutions Limited",
    "24.6280": "GOTIVE a.s.",
    "24.6289": "Zhongshan General K-mate Electronics Co., Ltd",
    "24.6288": "RadioCOM, s.r.o.",
    "24.9026": "Coffee Equipment Company",
    "24.9027": "Tem Ag",
    "24.9024": "MiXTelematics",
    "24.9025": "Vanderbilt International (SWE) AB",
    "24.9022": "Alcatel-Lucent IPD",
    "24.9023": "Purechoice Inc",
    "24.9020": "Alflex",
    "24.9021": "Laird Technologies",
    "24.2900": "BiTMICRO Networks, Inc.",
    "24.2901": "ADInstruments",
    "24.2902": "Cybernetics",
    "24.368": "ESE Embedded System Engineer'g",
    "24.2904": "Astronautics C.A LTD",
    "24.2905": "ScriptPro, LLC",
    "12.346864340992": "W. Vershoven GmbH",
    "24.9029": "Sony Mobile Communications Inc",
    "24.7624600": "TRUMPF Laser GmbH + Co. KG",
    "24.2112361": "vivo Mobile Communication Co., Ltd.",
    "12.346863112192": "Purite Ltd",
    "24.10527024": "DLI Taiwan Branch office",
    "24.9704570": "vivo Mobile Communication Co., Ltd.",
    "20.211670381101056": "Cyber Sciences, Inc.",
    "24.12862823": "SAMBON PRECISON and ELECTRONICS",
    "20.194284241289216": "Ad Hoc Developments S.L",
    "20.250817678409728": "FireAngel Safety Technology Ltd",
    "24.10279191": "Motorola Mobility LLC, a Lenovo Company",
    "12.71297885102080": "EMS Computers Pty Ltd",
    "24.8419967": "ABB Genway Xiamen Electrical Equipment CO., LTD",
    "24.12109909": "Shanghai GBCOM Communication Technology Co.,Ltd.",
    "24.4251159": "Shenzhen Lanfeng Times Industrial Co.,Ltd.",
    "24.4769379": "GTO Access Systems LLC",
    "24.8129478": "Samsung Electronics Co.,Ltd",
    "24.12064706": "Apple, Inc.",
    "24.9489846": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7345318": "Apple, Inc.",
    "12.123917677084672": "Guan Show Technologe Co., Ltd.",
    "12.346854727680": "Verex Technology",
    "24.6856203": "Cisco Systems, Inc",
    "20.30931575373824": "Dynojet Research",
    "24.14466426": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "12.123917688102912": "BirdDog Australia",
    "24.7101501": "IEEE Registration Authority",
    "12.346863251456": "SPX Dehydration & Filtration",
    "24.6046183": "Wiatec International Ltd.",
    "24.16027850": "Tensorcom",
    "24.16027851": "IEEE Registration Authority",
    "24.16574058": "Industrial Software Co",
    "12.346858287104": "Synrad, Inc.",
    "12.123917684281344": "MB connect line GmbH Fernwartungssysteme",
    "24.13926974": "Argosy Technologies, Ltd.",
    "24.11564972": "ARRIS Group, Inc.",
    "24.14705296": "Jinan Jovision Science & Technology Co., Ltd.",
    "12.346852802560": "RF Applications, Inc.",
    "24.1868900": "Formosa Wireless Communication Corp.",
    "24.1616216": "Provision Thai Co., Ltd.",
    "12.346862596096": "Vision Technologies, Inc.",
    "24.9439801": "Wiio, Inc.",
    "24.8089": "SERONICS co.ltd",
    "24.8088": "Daiichi-Dentsu Ltd.",
    "24.8087": "BERTANA srl",
    "24.8086": "Aprotech Co.Ltd",
    "24.8085": "Sagemcom Broadband SAS",
    "24.8084": "Lascar Electronics Ltd",
    "24.8083": "Xiotech Corporation",
    "24.8082": "Motorola Solutions Inc.",
    "24.52276": "Juniper Networks",
    "12.346857435136": "Micro Connect Pty Ltd",
    "12.346867015680": "Rohde&Schwarz Topex SA",
    "24.1098098": "Huawei Technologies Co.,Ltd",
    "12.123917689417728": "Infodev Electronic Designers Intl.",
    "24.8442678": "Cern",
    "20.66256288874496": "Taitex Corporation",
    "12.123917688061952": "Qtechnology A/S",
    "20.102196712570880": "Sunpet Industries Limited",
    "20.180698126221312": "Guangzhou Optical Bridge Communication Equipment Co.,Ltd.",
    "24.269946": "DSPWorks",
    "24.3416067": "Shenzhen Feitengyun Technology Co.,LTD",
    "12.346852671488": "PLLB elettronica spa",
    "24.6333475": "Silicon Laboratories",
    "24.5542590": "Samsung Electronics Co.,Ltd",
    "24.2948407": "Blue Calypso, Inc.",
    "24.14742033": "Digitalwatt",
    "24.4462845": "Apple, Inc.",
    "24.4250789": "Huawei Device Co., Ltd.",
    "12.346860216320": "Global Tel-Link",
    "12.123917688647680": "Digitella Inc.",
    "12.346863292416": "Joehl & Koeferli AG",
    "24.4730016": "Xiaomi Communications Co Ltd",
    "12.123917687590912": "SmartGuard LLC",
    "24.9480501": "JWEntertainment",
    "12.123917675614208": "plc-tec AG",
    "24.5544999": "Plasma Cloud Limited",
    "12.346853867520": "SCA Data Systems",
    "24.291147": "IEEE Registration Authority",
    "24.14960839": "Cisco SPVTG",
    "24.5029016": "IEEE Registration Authority",
    "24.11282956": "Csr Zhuzhou Institute Co.,Ltd.",
    "24.8953770": "Hella Gutmann Solutions GmbH",
    "24.16572048": "Samsung Electronics Co.,Ltd",
    "12.71297883750400": "Magnescale Co.,Ltd",
    "20.168210316918784": "Shanghai ZeXin Information Technologies Co. Ltd.",
    "24.8917135": "Huawei Technologies Co.,Ltd",
    "24.4511633": "Huawei Technologies Co.,Ltd",
    "24.12112649": "Cox Communications, Inc",
    "12.346863624192": "IPtec, Inc.",
    "12.346865954816": "Fr.Sauter AG",
    "24.2906639": "Cisco Systems, Inc",
    "24.15261879": "Toshiba",
    "24.11038885": "UniComm Inc.",
    "24.9848": "Logic Instrument SA",
    "24.9849": "Euphonic Technologies, Inc.",
    "24.9840": "Cinch Connectors",
    "24.5848": "Senea AB",
    "24.9842": "AAMP of America",
    "24.9843": "Ricoh Company,Ltd.",
    "24.9844": "Hunter Douglas",
    "24.9845": "Aztech Electronics Pte Ltd",
    "24.9846": "COMMidt AS",
    "24.9847": "Deif A/S",
    "12.71297884930048": "Elan Systems",
    "24.1873861": "Ynomia Pty Ltd",
    "24.2913930": "Hewlett Packard",
    "24.9740706": "AMPAK Technology, Inc.",
    "20.52821122613248": "Avic Jonhon Optronic Technology Co., Ltd.",
    "12.123917685473280": "Truecom Telesoft Private Limited",
    "24.16302081": "Juniper Networks",
    "24.5844": "Compal Communications, Inc.",
    "12.346855784448": "Owasys Advanced Wireless Devices",
    "24.5845": "Synccom Co., Ltd",
    "24.10807669": "Apple, Inc.",
    "24.5846": "TDA Tech Pty Ltd",
    "24.5847": "Sunways AG",
    "20.89058727428096": "Shenzhen Haipengxin Electronic Co., Ltd.",
    "24.5840": "ATech elektronika d.o.o.",
    "24.5841": "ZAT a.s.",
    "12.346864807936": "Dycec, S.A.",
    "24.1620801": "Huawei Device Co., Ltd.",
    "24.5842": "Caspian",
    "24.8401960": "Microchip Technology Inc.",
    "24.5843": "Wistron Corporation",
    "24.2378107": "Innovative Converged Devices Inc",
    "24.947": "IA Link Systems Co., Ltd.",
    "12.123917676036096": "vitalcare",
    "12.123917676290048": "DEUTA-WERKE GmbH",
    "12.123917686849536": "AmericanPharma Technologies",
    "20.229558884237312": "Dante Security Inc.",
    "24.14696010": "Cavanagh Group International",
    "24.4212": "Aten International Co., Ltd.",
    "12.71297884143616": "Sentry 360 Security",
    "24.49164": "Relia Technolgies",
    "24.49165": "Advanced Logic Research, Inc.",
    "24.49166": "Psitech, Inc.",
    "24.49167": "QNX Software Systems Ltd.",
    "24.49160": "Seco Srl",
    "24.49161": "KT Technology (s) Pte Inc",
    "24.49162": "Micro Craft",
    "24.49163": "Norcontrol A.S.",
    "24.5561250": "Fibrain",
    "24.49168": "Hirakawa Hewtech Corp.",
    "24.49169": "Interactive Computing Devices",
    "12.346859708416": "Schlumberger K.K.",
    "20.264036226695168": "G3 Technologies< Inc",
    "24.6853667": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7657015": "Amazon Technologies Inc.",
    "12.71297883258880": "Toni Studio",
    "20.167137996963840": "Private",
    "12.346854637568": "Softhill Technologies Ltd.",
    "24.12066654": "Apption Labs Limited",
    "24.3199042": "Dell Inc.",
    "12.123917684387840": "Tattile Srl",
    "24.14958948": "Nokia",
    "20.70682241138688": "Smart Sensor Devices Ab",
    "24.9754940": "Enelps",
    "24.32928": "Hewlett Packard",
    "24.32929": "Microtest",
    "24.32926": "Datus Gmbh",
    "24.32927": "Alcatel Business Systems",
    "24.32924": "Luxcom, Inc.",
    "24.32925": "Commscraft Ltd.",
    "24.32922": "Novus Networks Ltd",
    "24.32923": "Justsystem Corporation",
    "24.32920": "Tdk Corporation",
    "24.32921": "Eaton Industries GmbH",
    "24.12850824": "Huawei Device Co., Ltd.",
    "24.11021270": "Shina System Co., Ltd",
    "12.123917681930240": "Horizon Co., Ltd",
    "12.71297883299840": "A2S",
    "12.346859839488": "Lober, S.A.",
    "20.212014912765952": "xxter bv",
    "24.10523042": "OnePlus Electronics (Shenzhen) Co., Ltd.",
    "12.123917684957184": "Public Joint Stock Company Morion",
    "24.8403866": "ghe-ces electronic ag",
    "24.12862589": "Huawei Technologies Co.,Ltd",
    "24.2156674": "zte corporation",
    "12.71297884229632": "e.p.g. Elettronica Srl",
    "20.128612182261760": "Wuhan Tianyu Information Industry Co., Ltd.",
    "24.7661049": "Intel Corporate",
    "24.6869434": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.12118698": "ICG NETWORKS CO.,ltd",
    "24.12331883": "Beijing Haier IC Design Co.,Ltd",
    "12.71297884737536": "Kiwigrid GmbH",
    "12.71297884790784": "Aplex Technology Inc.",
    "24.822377": "Shenzhen elink smart Co., ltd",
    "12.346856992768": "vg controls, inc",
    "12.123917684350976": "JLCooper Electronics",
    "24.1072187": "Perma-Pipe, Inc.",
    "20.52912300490752": "Shenzhen Fengliyuan Energy Conservating Technology Co. Ltd",
    "24.3949182": "IEEE Registration Authority",
    "12.346868326400": "OFI Inc. (dba 2D2C)",
    "24.13931752": "Huawei Technologies Co.,Ltd",
    "12.123917681704960": "Sankyo Intec co.,ltd",
    "24.1635434": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.8388624": "AT&T [misrepresented as 080010? One source claims this is correct]",
    "20.184991237537792": "Imecon Engineering SrL",
    "24.12348677": "Dusun Electron Co.,Ltd.",
    "12.346862313472": "Esensors, Inc.",
    "12.123917676081152": "OOO \"Alyans\"",
    "24.8938463": "Apple, Inc.",
    "20.206931532709888": "Matica Technologies Ag",
    "20.176189956161536": "Qunar.com",
    "20.194600866152448": "eSenseLab Ltd.",
    "12.123917679026176": "Ariston Thermo s.p.a.",
    "12.123917680226304": "MAS Elettronica sas di Mascetti Sandro e C.",
    "24.1345995": "Huawei Technologies Co.,Ltd",
    "24.9979943": "Tp-Link Technologies Co.,Ltd.",
    "24.11032398": "Tp-Link Technologies Co.,Ltd.",
    "24.13156646": "ZPM Industria e Comercio Ltda",
    "12.346857410560": "IBEX UK Limited",
    "24.14453787": "vivo Mobile Communication Co., Ltd.",
    "24.847791": "IEEE Registration Authority",
    "24.7388137": "Xiaomi Communications Co Ltd",
    "12.346863456256": "E-Motion System, Inc.",
    "24.12867958": "Fugoo Coorporation",
    "12.123917690912768": "Hengen Technologies GmbH",
    "12.71297884606464": "Actronic Technologies",
    "24.837663": "Summer Infant, Inc.",
    "24.11290395": "Huawei Device Co., Ltd.",
    "24.7183": "Cisco Systems, Inc",
    "24.7182": "Cisco Systems, Inc",
    "24.49364": "Axon Networks, Inc.",
    "24.7180": "TANITA Corporation",
    "24.49362": "Syntellect, Inc.",
    "24.16309068": "PNY Technologies, INC.",
    "12.123917684088832": "Tattile Srl",
    "24.49363": "Olympus Image Systems, Inc.",
    "24.49360": "Ratoc System Inc.",
    "24.7184": "Cisco-Linksys, LLC",
    "24.2111420": "Kuipers Electronic Engineering BV",
    "20.17622844309504": "Estone Technology Inc",
    "12.123917685280768": "WEPTECH elektronik GmbH",
    "12.123917689348096": "Lge",
    "20.136823639638016": "Nomad Digital Ltd.",
    "20.247076405379072": "POMCube Inc.",
    "24.9467969": "Intel Corporate",
    "24.41119": "Commvision Corp.",
    "20.4471486676992": "ZPD technology Co., Ltd",
    "24.41111": "Jc Information Systems",
    "24.41110": "Mitsumi Electric Co.,Ltd.",
    "24.41113": "K-Net Ltd.",
    "24.41112": "NetApp",
    "24.41115": "Qpsx Communications, Ltd.",
    "24.41114": "Nihon Kohden America",
    "24.41117": "Johnathon Freeman Technologies",
    "24.41116": "Xyplex, Inc.",
    "24.4241724": "Hewlett Packard Enterprise",
    "12.346862714880": "Essential Design & Integration P/L",
    "24.13422144": "Janteq Corp",
    "12.346863665152": "Global Satellite Engineering",
    "24.13418097": "OptiLogix BV",
    "24.12643044": "Sonicwall",
    "20.162752341475328": "iungo",
    "12.71297884413952": "Canfield Scientific, Inc.",
    "20.225207116627968": "Ken A/S",
    "24.13913124": "Vuppalamritha Magnetic Components LTD",
    "12.346860199936": "Grupo Epelsa S.L.",
    "24.11280143": "Texas Instruments",
    "12.346860601344": "Luxpert Technologies Co., Ltd.",
    "12.119269351424": "Private",
    "20.172584094138368": "AVI Pty Ltd",
    "24.8957121": "Generiton Co., Ltd.",
    "24.8951841": "Teraon",
    "24.16306117": "Motorola Mobility LLC, a Lenovo Company",
    "12.346852626432": "NetVision Telecom",
    "20.62070503505920": "Istcontrol",
    "24.13170483": "Intel Corporate",
    "24.510": "Digital Equipment Corporation",
    "12.346868023296": "Unfors Instruments AB",
    "12.346866917376": "Kago Electronics BV",
    "24.8703935": "Fairphone",
    "24.9728428": "TCT mobile ltd",
    "12.123917682089984": "Tex Computer Srl",
    "24.8708903": "Tailyn Technologies Inc",
    "24.13694779": "Helmut Mauell GmbH Werk Weida",
    "24.2915556": "Wuhan Tianyu Information Industry Co., Ltd.",
    "12.123917682053120": "Becton Dickinson",
    "24.13409954": "Amazon Technologies Inc.",
    "20.67047993114624": "AMobile Solutions (Xiamen) CO. , LTD.",
    "12.346862583808": "Veracity UK Ltd",
    "24.2108679": "Samsung Electronics Co.,Ltd",
    "24.809026": "DME Micro",
    "24.7368843": "Cisco Systems, Inc",
    "12.71297883738112": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "24.4769165": "Lear Corporation GmbH",
    "24.15735827": "LG Electronics (Mobile Communications)",
    "12.346866479104": "Capton",
    "24.9473033": "Total Phase",
    "12.346864730112": "Altronix Corporation",
    "24.11297726": "Amazon Technologies Inc.",
    "24.16271613": "China Mobile Group Device Co.,Ltd.",
    "12.123917680259072": "Itest communication Tech Co., LTD",
    "12.123917691695104": "Les distributions Multi-Secure incorporee",
    "24.11010413": "Aiwa Corporation",
    "24.16270563": "Taicang T&W Electronics",
    "12.123917689024512": "Core Balance Co., Ltd.",
    "24.1319643": "eero inc.",
    "20.145211804090368": "Kazdream Technologies LLP",
    "24.2949102": "vivo Mobile Communication Co., Ltd.",
    "12.346857639936": "Detection Technology Inc.",
    "24.13388795": "Hellberg Safety AB",
    "12.123917688639488": "Vision4ce Ltd",
    "24.13392190": "Sensium Healthcare Limited",
    "24.15732716": "Huawei Technologies Co.,Ltd",
    "24.8966040": "Huawei Technologies Co.,Ltd",
    "12.346868117504": "Rosslare Enterprises Limited",
    "24.6919": "Mendocino Software",
    "24.6918": "Ateliers R. LAUMONIER",
    "24.6917": "Ymc Ag",
    "12.346864885760": "Audient Ltd",
    "24.6915": "Action Technology (SZ) Co., Ltd",
    "24.6914": "ED Co.Ltd",
    "24.6913": "Applied Radio Technologies",
    "24.6912": "Neopost Technologies",
    "24.6911": "Wizyoung Tech.",
    "24.6910": "Sofacreal",
    "12.123917678419968": "Solectrix",
    "24.5293833": "Juniper Networks",
    "24.13657709": "Shenzhen Bus-Lan Technology Co., Ltd.",
    "24.6595210": "Huawei Device Co., Ltd.",
    "20.45069255049216": "Private",
    "24.792863": "Inform Electronik",
    "24.10511249": "Netgear",
    "24.15489511": "Microsoft Corporation",
    "12.123917680287744": "AMEDTEC Medizintechnik Aue GmbH",
    "24.8680490": "zte corporation",
    "24.9479274": "Sisnet Co., Ltd",
    "24.3992315": "Nse Ag",
    "24.8921106": "LMI Technologies",
    "20.31368097562624": "Xiaoxinge (Tangshan) Electronic Technology Co., Ltd.",
    "12.346867703808": "Klangspektrum GmbH",
    "12.123917676109824": "Colorimetry Research, Inc",
    "24.1574655": "Acer Computer(Shanghai) Limited.",
    "24.3471058": "Panasonic Taiwan Co.,Ltd.",
    "24.6295961": "Siama Systems Inc",
    "24.16295779": "Zhejiang Tmall Technology Co., Ltd.",
    "20.128612190650368": "Private",
    "20.167137994866688": "Promicon Elektronik GmbH + Co.KG",
    "12.346866339840": "Vitec",
    "12.123917689499648": "Vitec",
    "24.11311401": "Huawei Technologies Co.,Ltd",
    "24.53255": "Mic Associates, Inc.",
    "24.53254": "Cisco Systems, Inc",
    "24.53257": "Hsing Tech. Enterprise Co. Ltd",
    "24.53256": "Mactell Corporation",
    "24.53251": "Comda Enterprises Corp.",
    "24.53250": "Ditech Corporation",
    "24.53253": "Zhs Zeitmanagementsysteme",
    "24.53252": "Pentacom Ltd.",
    "12.123917683666944": "OOO Research and Production Center \"Computer Technologies\"",
    "24.53259": "Rhk Technology, Inc.",
    "24.53258": "Lanaccess Telecom S.A.",
    "24.7126761": "Alcatel-Lucent IPD",
    "24.12598970": "Dell Inc.",
    "24.570287": "vivo Mobile Communication Co., Ltd.",
    "24.16314854": "Samsung Electronics Co.,Ltd",
    "24.1864221": "Dell Inc.",
    "24.16266907": "Texas Instruments",
    "24.1869612": "ASUSTek COMPUTER INC.",
    "20.141720400429056": "Thurlby Thandar Instruments LTD",
    "12.346853015552": "Telefrang AB",
    "24.12381277": "Apple, Inc.",
    "12.123917690023936": "Mi Inc.",
    "24.16289738": "Google, Inc.",
    "24.5758": "Diboss.Co.,Ltd",
    "24.5759": "Bluebird Soft Inc.",
    "12.346860236800": "Elmec, Inc.",
    "24.6599536": "PowerComm Solutions LLC",
    "24.5750": "Intel Corporate",
    "24.5751": "Bihl + Wiedemann GmbH",
    "24.5752": "Shenzhen Baoan Gaoke Electronics Co., Ltd",
    "24.5753": "eOn Communications",
    "24.5754": "Skyworth Overseas Development Ltd.",
    "24.5755": "Haver&Boecker",
    "24.5756": "iRex Technologies BV",
    "24.5757": "Sky-Line Information Co., Ltd.",
    "24.2009": "Splicecom",
    "24.11305174": "Hexatronic AB",
    "12.123917685121024": "Qunu Labs Private Limited",
    "24.2008": "Hitron Technologies. Inc",
    "12.346865999872": "RealTime Systems Ltd",
    "24.1592494": "Continental",
    "24.8183273": "ATOM tech Inc.",
    "24.2947227": "Cisco Systems, Inc",
    "24.9702169": "Huawei Technologies Co.,Ltd",
    "12.123917682814976": "Rugged Science",
    "24.320822": "Apple, Inc.",
    "12.123917681733632": "Innitive B.V.",
    "12.123917690814464": "Sanyu Switch Co., Ltd.",
    "24.2001": "Spectrum Signal Processing Inc.",
    "24.10003460": "Ericsson AB",
    "12.346853871616": "Portalplayer, Inc",
    "24.2000": "Automat Engenharia de Automação Ltda.",
    "12.346866982912": "Michael Riedel Transformatorenbau GmbH",
    "24.2003": "SPGPrints B.V.",
    "24.65224": "Cisco Systems, Inc",
    "24.2002": "Logopak Systeme GmbH & Co. KG",
    "24.8133123": "8Locations Co., Ltd.",
    "24.2005": "3e Technologies Int;., Inc.",
    "12.123917688295424": "YUYAMA MFG Co.,Ltd",
    "20.159304578498560": "Beijing Lanxum Computer Technology CO.,LTD.",
    "12.346867097600": "Monnit Corp.",
    "24.2004": "Zhejiang Yutong Network Communication Co Ltd.",
    "20.31368106999808": "Quest Integrity",
    "24.9489444": "Synaptics, Inc",
    "24.6586": "Paradox Security Systems Ltd",
    "24.15220978": "Pegatron Corporation",
    "24.12608107": "Cisco Systems, Inc",
    "24.2006": "Commil Ltd.",
    "24.16308664": "Open Mesh, Inc.",
    "12.346852270080": "Skylake Talix",
    "12.346864189440": "Folink",
    "24.5283849": "Xiaomi Communications Co Ltd",
    "24.4407": "Aichi Electric Co., Ltd.",
    "24.4406": "Goodrich Sensor Systems",
    "24.4405": "Grandeye Ltd",
    "24.4404": "MediaCell, Inc.",
    "24.4403": "Siemens AG Austria",
    "24.4402": "Synology Incorporated",
    "24.4401": "Unatech. Co.,Ltd",
    "24.4400": "Allied Telesis (Hong Kong) Ltd.",
    "24.295323": "BSkyB Ltd",
    "24.4409": "STOEBER ANTRIEBSTECHNIK GmbH + Co. KG.",
    "24.4408": "Taishin Co., Ltd.",
    "20.176094520016896": "JGR Optics Inc",
    "24.16266579": "Magenta Research Ltd.",
    "24.3430918": "SHARP Corporation",
    "24.2906344": "Huawei Technologies Co.,Ltd",
    "24.4520794": "zte corporation",
    "12.346862473216": "Netpower Labs AB",
    "24.3752": "United Technologists Europe Limited",
    "24.3753": "Shanghai Xun Shi Communications Equipment Ltd. Co.",
    "24.3750": "ASUSTek COMPUTER INC.",
    "24.3751": "Endace Technology",
    "24.3756": "Mintron Enterprise Co., Ltd.",
    "24.3757": "Metanoia Technologies, Inc.",
    "24.3754": "Scalent Systems, Inc.",
    "24.3755": "Cray Inc",
    "24.3758": "Gawell Technologies Corp.",
    "24.3759": "Castel",
    "24.10495926": "Samsung Electronics Co.,Ltd",
    "24.16002134": "Senor Tech Co Ltd",
    "24.8199": "Sfa, Inc.",
    "24.14435607": "ARRIS Group, Inc.",
    "24.8198": "Garrett Communications, Inc.",
    "12.123917684101120": "Global Communications Technology LLC",
    "24.20491": "Cisco Systems, Inc",
    "24.20490": "Iris Technologies, Inc.",
    "24.20493": "Satori Electoric Co., Ltd.",
    "24.20492": "e-Tek Labs, Inc.",
    "24.20495": "Cisco Systems, Inc",
    "24.20494": "Chromatis Networks, Inc.",
    "24.20496": "NovaNET Learning, Inc.",
    "24.20499": "Seagate Cloud Systems Inc",
    "24.20498": "Cbl - Gmbh",
    "24.7279": "Emfit Ltd",
    "24.7278": "Newbury Networks, Inc.",
    "24.7271": "Pumpkin Networks, Inc.",
    "24.7270": "Ucamp Co.,Ltd",
    "24.7273": "Packet Vision Ltd",
    "24.7272": "Anhui Sun Create Electronics Co., Ltd",
    "24.7275": "COVAX Co. Ltd",
    "24.7274": "Weiss Engineering Ltd.",
    "24.7277": "Kyohritsu Electronic Industry Co., Ltd.",
    "24.7276": "30805",
    "24.9468504": "Zegna-Daidong Limited",
    "12.123917691191296": "Honeywell",
    "20.48543328894976": "Hysentel Technology Co., Ltd",
    "12.123917680492544": "Airthings",
    "20.168210328453120": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "24.8195": "Pixel Power Ltd.",
    "20.58071057432576": "Glory Mark Electronic Ltd. Taiwan Branch (B.V.I.)",
    "24.1627927": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.1335188": "Huawei Technologies Co.,Ltd",
    "24.49366": "J1 Systems, Inc.",
    "24.49367": "Taiwan Trading Center Dba",
    "24.7181": "G-Technology, Inc.",
    "24.49365": "Quancom Electronic Gmbh",
    "24.7187": "Optsys Technology Co., Ltd.",
    "24.7186": "ARRIS Group, Inc.",
    "24.7185": "ARRIS Group, Inc.",
    "24.49361": "Comtree Technology Corporation (EFA also reported)",
    "24.6859529": "Nokia",
    "24.7189": "iPhotonix LLC",
    "24.7188": "VMware, Inc.",
    "24.49368": "Universal Data Systems",
    "24.49369": "Quinte Network Confidentiality Equipment Inc",
    "24.5830267": "Xia Men UnionCore Technology LTD.",
    "24.53532": "Acetel",
    "12.346853814272": "LaserBit Communications Corp.",
    "24.1348655": "Usys Co.,Ltd.",
    "24.8957120": "Zyxel Communications Corporation",
    "12.346854973440": "Quest Retail Technology Pty Ltd",
    "24.2366227": "Shanghai Nutshell Electronic Co., Ltd.",
    "24.11286312": "Taicang T&W Electronics",
    "24.7084480": "Apple, Inc.",
    "24.6325170": "GkWare e.K.",
    "24.12642402": "Apple, Inc.",
    "24.419": "Genesys Logic, Inc.",
    "20.180698130415616": "Finetree Communications Inc",
    "12.123917677150208": "Siemens Healthcare Diagnostics",
    "12.346866511872": "eumig industrie-tv GmbH",
    "24.4239610": "LG Electronics (Mobile Communications)",
    "24.16025030": "FDT Technologies",
    "24.13115326": "Huawei Technologies Co.,Ltd",
    "12.346863276032": "Haivision Systems Inc",
    "24.12113053": "M Seven System Ltd.",
    "24.16312842": "Dipl.-Math. Michael Rauch",
    "24.530420": "IBM Corp",
    "24.4257056": "Espressif Inc.",
    "24.6831654": "Intel Corporate",
    "24.6588": "Electro Chance Srl",
    "24.6589": "New Media Life",
    "24.4199597": "Huawei Device Co., Ltd.",
    "24.6580": "Intellio Ltd",
    "24.6581": "Famar Fueguina S.A.",
    "24.6582": "Euro Emme s.r.l.",
    "24.6583": "Nokia Danmark A/S",
    "24.6584": "Boundary Devices",
    "24.6585": "Dell Inc.",
    "24.2007": "Caporis Networks AG",
    "24.6587": "Hewlett Packard",
    "12.346858401792": "POSNET Polska S.A.",
    "24.4510276": "Seiko Epson Corporation",
    "24.836734": "EUCAST Co., Ltd.",
    "24.15770546": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.123917687070720": "Lookout Portable Security",
    "24.836730": "Super Micro Computer, Inc.",
    "24.8661401": "Apple, Inc.",
    "24.11047229": "Apple, Inc.",
    "24.8922515": "Tp-Link Technologies Co.,Ltd.",
    "24.6632": "Cisco Systems, Inc",
    "24.6633": "S-Information Technolgy, Co., Ltd.",
    "24.6630": "Toyo Medic Co.,Ltd.",
    "24.6631": "Cisco Systems, Inc",
    "24.6636": "Sagamore Systems, Inc.",
    "24.6637": "Axesstel Inc.",
    "24.6634": "TeraMage Technologies Co., Ltd.",
    "24.6635": "Pyronix Ltd",
    "24.6638": "CARLO GAVAZZI CONTROLS SPA-Controls Division",
    "24.6639": "Shenzhen Linnking Electronics Co.,Ltd",
    "24.9185053": "QuantHouse",
    "12.123917687640064": "Clarity Medical Pvt Ltd",
    "12.346852818944": "Cynaps",
    "24.14978917": "Yelatma Instrument Making Enterprise, JSC",
    "20.225207117676544": "PJG Systementwicklung GmbH",
    "24.7634477": "Beijing Thomson CITIC Digital Technology Co. LTD.",
    "24.10280707": "Harman/Becker Automotive Systems GmbH",
    "20.149935444983808": "Private",
    "24.14972884": "Minrray Industry Co.,Ltd",
    "20.71389209952256": "Guangzhou Auric Intelligent Technology Co.,Ltd.",
    "24.446": "Gigalink Co., Ltd.",
    "24.447": "Teleforce Co., Ltd.",
    "24.444": "Brains Corporation",
    "24.445": "Peterson Electro-Musical Products, Inc.",
    "24.442": "IC-Net, Inc.",
    "24.443": "Frequentis",
    "24.440": "Netsensity, Inc.",
    "24.441": "SKF (U.K.) Limited",
    "24.15246180": "Signal Path International / Peachtree Audio",
    "24.12078574": "Hisense broadband multimedia technology Co.,Ltd",
    "24.448": "CompuLab, Ltd.",
    "24.449": "Vitesse Semiconductor Corporation",
    "24.9789": "MIA Corporation",
    "24.9788": "Bachmann Technology GmbH & Co. KG",
    "24.9783": "Samsung Electro Mechanics Co., Ltd.",
    "24.9782": "ARRIS Group, Inc.",
    "24.9781": "Bluetechnix GmbH",
    "24.9780": "Infineta Systems, Inc",
    "24.9787": "Onbnetech",
    "24.9786": "Digitec Systems",
    "24.9785": "T.M. Electronics, Inc.",
    "24.9784": "Xia Men Joyatech Co., Ltd.",
    "24.10283051": "vivo Mobile Communication Co., Ltd.",
    "24.3989437": "Huawei Technologies Co.,Ltd",
    "20.62366882463744": "Wyres SAS",
    "12.346859593728": "Mitrol S.R.L.",
    "24.6335442": "Bose Corporation",
    "24.564358": "Nokia Shanghai Bell Co., Ltd.",
    "12.346864222208": "iScreen LLC",
    "24.1316220": "Tokyo Cosmos Electric Co.,Ltd.",
    "24.5794156": "Ruijie Networks Co.,LTD",
    "24.9255": "Ssi Computer Corp",
    "24.9254": "Nohmi Bosai Ltd.",
    "24.9257": "Mk Master Inc.",
    "24.9256": "EnergyICT",
    "24.9251": "AzureWave Technologies (Shanghai) Inc.",
    "24.9250": "Knapp Logistik Automation GmbH",
    "24.9253": "Shenzhenshi chuangzhicheng Technology Co.,Ltd",
    "24.9252": "Ace Axis Limited",
    "24.411": "Kyoto Microcomputer Co., Ltd.",
    "24.9259": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9258": "Hittite Microwave Corporation",
    "24.1062": "Autosys",
    "24.1063": "Cisco Systems, Inc",
    "24.1060": "TMC s.r.l.",
    "24.1061": "Atmel Corporation",
    "24.1066": "Wireless Networks, Inc.",
    "24.1067": "IT Access Co., Ltd.",
    "24.1064": "Cisco Systems, Inc",
    "24.1065": "Pixord Corporation",
    "24.1068": "Minet, Inc.",
    "24.1069": "Sarian Systems, Ltd.",
    "12.346868416512": "SYS TEC electronic GmbH",
    "20.31346179178496": "Microtronics Engineering GmbH",
    "12.123917675204608": "Shenzhen Gaona Electronic Co.Ltd",
    "24.410": "LEUNIG GmbH",
    "12.119269429248": "Adixein Limited",
    "12.123917689679872": "Spanawave Corporation",
    "24.1342380": "Nokia",
    "24.1370056": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346853392384": "Delta Tau Data Systems, Inc.",
    "24.10793834": "JSC SDO Chromatec",
    "24.4725380": "Pointer Telocation Ltd",
    "24.413": "E-Control Systems, Inc.",
    "24.2376380": "Alinco,incorporated",
    "24.14743221": "Nintendo Co.,Ltd",
    "12.123917684375552": "ERAESEEDS co.,ltd.",
    "12.346854137856": "DIGITRONIC Automationsanlagen GmbH",
    "12.123917679845376": "CAPTEMP, Lda",
    "24.2398936": "Yealink(Xiamen) Network Technology Co.,Ltd.",
    "24.8424284": "Shenzhen Runhuicheng Technology Co., Ltd",
    "24.1059971": "Ximea",
    "12.346852831232": "BaSyTec GmbH",
    "24.10858": "Cisco Systems, Inc",
    "24.4720234": "Tempered Networks, Inc.",
    "24.14996339": "Huawei Technologies Co.,Ltd",
    "24.1074480": "Dell Inc.",
    "24.3205978": "Landis + Gyr",
    "24.536683": "Palo Alto Networks",
    "24.6076972": "Hewlett Packard Enterprise",
    "12.71297883295744": "Scharco Elektronik GmbH",
    "24.11824515": "Intel Corporate",
    "24.5817731": "Itel Mobile Limited",
    "24.822747": "Cisco Meraki",
    "12.123917680300032": "CT Company",
    "24.1616399": "Huawei Device Co., Ltd.",
    "24.8124": "EVGA Corporation",
    "12.119269629952": "Sulaon Oy",
    "24.8126": "Shenzhen Mopnet Industrial Co.,Ltd",
    "24.8127": "Fulhua Microelectronics Corp. Taiwan Branch",
    "24.8120": "Universal Remote Control, Inc.",
    "24.8121": "Paltronics",
    "24.8122": "Boyoung Tech",
    "24.8123": "Xenatech Co.,LTD",
    "12.123917677764608": "Visualware, Inc.",
    "24.3463873": "SAE Technologies Development(Dongguan) Co., Ltd.",
    "24.8129": "Hanlong Technology Co.,LTD",
    "24.14718329": "ORTHOsoft inc, d/b/a Zimmer CAS",
    "20.97755442708480": "Chronos Technology Ltd.",
    "24.13930927": "Electroacustica General Iberica, S.A.",
    "12.119269699584": "tado GmbH",
    "24.1633926": "Novatel Wireless Solutions, Inc.",
    "24.2406610": "Opzoon Technology Co.,Ltd.",
    "12.123917685927936": "xmi systems",
    "12.346857578496": "Omnitronics Pty Ltd",
    "24.11280430": "Hitron Technologies. Inc",
    "20.264625141579776": "U3storage Technologies Co., Ltd",
    "20.225209258868736": "Shanghai tongli information technology co. LTD",
    "24.2905555": "Huawei Technologies Co.,Ltd",
    "24.16029236": "Intel Corporate",
    "12.123917683367936": "meoENERGY",
    "24.6599795": "Xiaomi Communications Co Ltd",
    "24.416": "Infinilink Corporation",
    "24.8963045": "Betop Techonologies",
    "12.346857492480": "Monaghan Engineering, Inc.",
    "12.123917683011584": "Impulse Automation",
    "24.4761422": "zte corporation",
    "12.123917679128576": "Vtron Pty Ltd",
    "24.4498766": "Qorvo International Pte. Ltd.",
    "12.346858430464": "LEA d.o.o.",
    "24.3455605": "Everest Networks, Inc",
    "12.346852790272": "Case Information & Communications",
    "12.123917682462720": "Connido Limited",
    "12.346853941248": "SIGOS Systemintegration GmbH",
    "24.5768534": "Elettronica GF S.r.L.",
    "12.346867847168": "Human Network Labs, Inc.",
    "12.123917676158976": "Private",
    "12.346852646912": "Litton Guidance & Control Systems",
    "24.3172492": "Reach Technology Inc.",
    "12.346857807872": "C21 Systems Limited",
    "24.13153719": "Apple, Inc.",
    "20.44768537083904": "AnyLink LLC",
    "20.242185280684032": "Shanghai Huahong Integrated Circuit Co.,Ltd",
    "24.15250544": "IEEE Registration Authority",
    "24.6304496": "Mplus technology",
    "24.7887208": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917688188928": "GMI Ltd",
    "24.9569": "ProCurve Networking by HP",
    "24.11034890": "Arduino Ag",
    "24.14964244": "Suttle Apparatus",
    "24.17000": "Cisco Systems, Inc",
    "20.272689497374720": "Luxul Technology Inc",
    "12.123917687963648": "Layer Logic Inc",
    "12.123917691764736": "Symetrics Industries d.b.a. Extant Aerospace",
    "24.2401340": "Ubiquiti Networks Inc.",
    "20.106480346857472": "Hemisphere GNSS",
    "24.11297293": "Jabil Circuit(Wuxi) Co.,Ltd",
    "24.4724802": "Shanghai Winaas Co. Equipment Co. Ltd.",
    "24.12083567": "Accton Technology Corp",
    "12.346853482496": "Instrumeter A/S",
    "24.303859": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917691228160": "Firehose Labs, Inc.",
    "20.149945495584768": "Jungheinrich Norderstedt AG & Co. KG",
    "12.346860507136": "ACE Carwash Systems",
    "24.14693262": "Apple, Inc.",
    "20.27478315761664": "Gosuncn Technology Group Co.,LTD.",
    "12.346861142016": "BAE Systems Hägglunds AB",
    "12.123917677281280": "Hiquel Elektronik- und Anlagenbau GmbH",
    "12.123917689319424": "Network Innovations",
    "20.264809146744832": "Blacknight Internet Solutions Limited",
    "12.71297884426240": "EMAC, Inc.",
    "24.6008": "Central Music Co.",
    "12.71297884979200": "Orion Systems, Inc",
    "24.11280137": "shenzhen RAKwireless technology Co.,Ltd",
    "24.351": "DIGITAL DESIGN GmbH",
    "24.3204315": "Cisco Systems, Inc",
    "24.7642588": "2Wire Inc",
    "12.346865934336": "eMDee Technology, Inc.",
    "24.13682624": "Tp-Link Technologies Co.,Ltd.",
    "12.123917678612480": "BEDEROV GmbH",
    "24.12371675": "Nokia Corporation",
    "24.2159939": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "20.79602632359936": "Shenzhen Inpor cloud Computing Co., Ltd.",
    "24.13656014": "Hitachi Data Systems",
    "24.5510387": "Intel Corporate",
    "12.123917687795712": "Dandong Dongfang Measurement & Control Technology Co., Ltd.",
    "24.356": "Cisco Systems, Inc",
    "12.123917681401856": "Rocket Lab Ltd.",
    "12.346854400000": "imo-elektronik GmbH",
    "24.270900": "Wireless Standard Extensions",
    "12.346854133760": "Automaatiotekniikka Seppo Saari Oy",
    "24.357": "AirSwitch Corporation",
    "24.4783850": "Homa B.V.",
    "24.13172520": "Liteon Technology Corporation",
    "24.15229310": "Liteon Technology Corporation",
    "12.346856022016": "Hankuk Tapi Computer Co., Ltd",
    "12.123917678030848": "Meiknologic GmbH",
    "24.16032404": "Eagle World Development Co., Limited",
    "12.346857107456": "GFI Chrono Time",
    "12.346853371904": "TPA Traffic & Parking Automation BV",
    "24.7122415": "KZ Broadband Technologies, Ltd.",
    "12.123917678710784": "Private",
    "24.4214143": "Apple, Inc.",
    "24.7643183": "Cisco Systems, Inc",
    "24.12330525": "Hong Kong Tecon Technology",
    "24.15511447": "Git",
    "20.207147377885184": "Lucent Trans Electronics Co., Ltd",
    "24.7113452": "Nantong Coship Electronics Co., Ltd.",
    "12.123917688107008": "Tattile Srl",
    "24.11848999": "Huawei Technologies Co.,Ltd",
    "24.8709673": "Bluwan SA",
    "24.545070": "Orion Diagnostica Oy",
    "24.16569014": "IEEE Registration Authority",
    "20.163603970785280": "ENTEC Electric & Electronic Co., LTD.",
    "12.123917682429952": "Riot",
    "24.20710": "Hakusan Corporation",
    "24.13937407": "Micro World",
    "20.203238155485184": "Votem",
    "24.2622766": "Dematic Corp",
    "24.2643057": "Huawei Device Co., Ltd.",
    "20.93020701589504": "Beijing HTSmartech Co.,Ltd",
    "24.309901": "Samsung Electronics Co.,Ltd",
    "24.11319385": "Uniband Electronic Corp,",
    "24.6038212": "Dae Eun Eletronics Co., Ltd",
    "24.9216921": "Servercom (India) Private Limited",
    "12.346864705536": "Nolam Embedded Systems",
    "24.11049321": "Dell Inc.",
    "24.4487230": "Samsung Electronics Co.,Ltd",
    "20.4471484579840": "Bolicom Innovation Technology (BeiJing) Co.,LTD.",
    "12.123917675184128": "Iylus Inc.",
    "12.71297883574272": "Cominfo, Inc.",
    "24.5290120": "wi2be Tecnologia S/A",
    "12.346859892736": "B. Rexroth the identity company GmbH",
    "24.12627028": "Sagemcom Broadband SAS",
    "12.346852753408": "Invensys Controls Network Systems",
    "12.346868895744": "Flexkom Internet Pazarlama Bilipim ve Eoitim Hiz.Inp.Mim.Muh.Oto.Enerji San. Tic. A.p.",
    "20.272689496326144": "Beijing Redcdn Technology, Co., Ltd",
    "24.13901365": "TOHO Electronics INC.",
    "12.123917690232832": "YUYAMA MFG Co.,Ltd",
    "12.346862632960": "Ectec Inc.",
    "20.31657724739584": "Bird Home Automation GmbH",
    "12.346858467328": "Cerus Corp",
    "24.13171073": "Seneca s.r.l.",
    "12.119269240832": "MicroSigns Technologies Inc",
    "24.3163802": "Netgear",
    "12.123917678301184": "BlueMark Innovations BV",
    "24.9057468": "Fiberworks AS",
    "12.346867707904": "Eco Monitoring Utility Systems Ltd",
    "24.10288085": "vivo Mobile Communication Co., Ltd.",
    "20.225135225208832": "Tecnoideal Srl",
    "24.9975571": "PT.Navicom Indonesia",
    "24.12347972": "Commend International GmbH",
    "24.1093496": "Shenzhen ATUE Technology Co., Ltd",
    "24.7644327": "QRS Music Technologies, Inc.",
    "24.9437499": "Sagemcom Broadband SAS",
    "12.346864025600": "Selco",
    "12.123917684211712": "Kronotech Srl",
    "12.123917676892160": "ELEUSI GmbH",
    "20.70559709790208": "Zhejiang Guoli Security Technology Co., Ltd.",
    "12.346867060736": "Omnica Corporation",
    "12.123917681512448": "Cosylab d.d.",
    "24.3510": "Broadcom",
    "24.3511": "Sanko Electric Co,.Ltd",
    "24.3512": "Schiller Ag",
    "24.3513": "PC Engines GmbH",
    "24.3514": "Océ Document Technologies GmbH",
    "24.3515": "Nippon Dentsu Co.,Ltd.",
    "24.3516": "Cisco Systems, Inc",
    "24.3517": "Cisco Systems, Inc",
    "24.3518": "Bel Fuse Europe Ltd.,UK",
    "24.3519": "TekTone Sound & Signal Mfg., Inc.",
    "20.132811185979392": "Louroe Electronics",
    "24.10550213": "Apple, Inc.",
    "12.123917685702656": "Rch Italia Spa",
    "24.47873": "Octothorpe Corp.",
    "24.4243223": "Digistar Telecom. SA",
    "20.53091436068864": "Morgan Schaffer Inc.",
    "20.255553900118016": "Huaqin Technology Co.,Ltd",
    "24.12375": "QTelNet, Inc.",
    "24.12374": "HMS Industrial Networks",
    "24.12377": "Kontron Compact Computers Ag",
    "24.12376": "Api Motion",
    "24.12371": "Basler AG",
    "24.12370": "Elastic Networks",
    "24.12373": "Renesas Technology America, Inc.",
    "24.12372": "Castlenet Technology Inc.",
    "20.45033661136896": "MyOmega Systems GmbH",
    "24.12379": "Toko Inc.",
    "24.12378": "Telgen Corporation",
    "12.119269941248": "Techlan Reti s.r.l.",
    "24.6760": "Weltec Enterprise Co., Ltd.",
    "24.7351842": "zte corporation",
    "24.7099597": "Lampex Electronics Limited",
    "12.346854703104": "Crystal Vision Ltd",
    "24.3972111": "Intel Corporate",
    "24.2101973": "Scientech Materials Corporation",
    "20.22126505492480": "Shenzhen CATIC Information Technology Industry Co.,Ltd",
    "24.8128811": "Bekey A/S",
    "24.11845877": "Shenzhen TINNO Mobile Technology Corp.",
    "24.13169670": "Avaya Inc",
    "20.224641230569472": "Beijing Cotytech Technology Co.,LTD.",
    "24.802463": "Apple, Inc.",
    "24.12344993": "PsiKick, Inc.",
    "20.124214337077248": "NST Technology Limited Co.,Ltd.",
    "24.41094": "Amber Wave Systems, Inc.",
    "24.41095": "Microsemi Corporation",
    "24.41096": "Essential Communications",
    "24.41097": "Xpoint Technologies, Inc.",
    "24.1323012": "Huawei Technologies Co.,Ltd",
    "24.41091": "Asimmphony Turkey",
    "24.41092": "Dataplex Pty Ltd",
    "12.346855743488": "Optronic Partner pr AB",
    "24.300193": "Catch The Wind Inc",
    "12.123917684559872": "Cubitech",
    "24.41098": "Brooktrout Technology, Inc.",
    "24.41099": "Aston Electronic Designs Ltd.",
    "24.16260831": "Apple, Inc.",
    "24.16298739": "Guangzhou Bosma Technology Co.,Ltd",
    "12.123917686382592": "Private",
    "24.1620229": "Hong Kong Bouffalo Lab Limited",
    "12.123917682872320": "Project Service S.a.s.",
    "24.7658202": "D-Link International",
    "24.12598159": "Minicom Digital Signage",
    "24.11058332": "Cloud Light Technology Limited",
    "12.346865864704": "Aplex Technology Inc.",
    "12.123917678936064": "DK-Technologies A/S",
    "24.4482670": "Ip-Line",
    "24.9980385": "Boeing Defence Australia",
    "20.84531182829568": "Lenovo Data Center Group",
    "24.11579796": "zte corporation",
    "12.346861875200": "QuanZhou TDX Electronics Co., Ltd.",
    "24.11808960": "York Instruments Ltd",
    "12.123917677465600": "Zebra Elektronik A.S.",
    "24.9731134": "Polewall Norge AS",
    "12.123917679669248": "KeyProd",
    "12.346860961792": "ATG Automatisierungstechnik GERA GmbH",
    "24.13433172": "Avaya Inc",
    "24.13433175": "u-blox AG",
    "24.12384952": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "24.1581461": "Samsung Electronics Co.,Ltd",
    "20.70682233798656": "Cloud4Wi",
    "24.3974014": "Huawei Technologies Co.,Ltd",
    "12.346859843584": "Abb",
    "24.6059894": "TCT mobile ltd",
    "12.346861309952": "ACD Elektronik GmbH",
    "24.13933710": "University of FUKUI",
    "24.1071239": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6030": "Gunnebo Cash Automation AB",
    "24.6031": "Ningbo Yidong Electronic Co.,Ltd.",
    "24.6032": "HYUNDAI DIGITECH Co, Ltd.",
    "24.6033": "LinTech GmbH",
    "24.6034": "Falcom Wireless Comunications Gmbh",
    "24.6035": "Tigi Corporation",
    "24.6036": "Cisco Systems, Inc",
    "24.6037": "Cisco Systems, Inc",
    "24.6038": "Rittmeyer AG",
    "24.6039": "Telsy Elettronica S.p.A.",
    "24.11324046": "ODA Technologies",
    "24.3441092": "AVM GmbH",
    "24.13122373": "Jri",
    "24.8131019": "Huawei Technologies Co.,Ltd",
    "20.44768536035328": "Vehant Technologies Pvt Ltd.",
    "12.346867548160": "Alias ip",
    "24.13123028": "CyberTAN Technology Inc.",
    "24.57531": "Nbx Corporation",
    "20.163808559497216": "Reichert Inc.",
    "12.346856439808": "Talleres de Escoriaza SA",
    "24.57532": "Symon Communications, Inc.",
    "24.9465531": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917677613056": "Telenor Connexion AB",
    "24.3445570": "TTE Corporation",
    "24.3164710": "Shenzhen Trolink Technology CO, LTD",
    "12.123917680709632": "Echogear",
    "24.9709456": "TCT mobile ltd",
    "12.123917690925056": "Teleplatforms",
    "12.346867896320": "Mango DSP, Inc",
    "24.794662": "noax Technologies AG",
    "24.14446029": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.44749": "Pensando Systems",
    "20.163603965542400": "ProConnections, Inc.",
    "24.12088391": "Convergence Technologies",
    "12.123917680832512": "LSL systems",
    "24.7131834": "zte corporation",
    "24.4727605": "Dialog Semiconductor Hellas SA",
    "24.14184791": "Lenovo Mobile Communication Technology Ltd.",
    "12.123917686464512": "DiTEST Fahrzeugdiagnose GmbH",
    "12.123917687656448": "Ariston Thermo s.p.a.",
    "24.1867493": "D-Link International",
    "12.123917689421824": "Ateme",
    "24.1260": "Memobox SA",
    "24.1261": "Billion Electric Co., Ltd.",
    "24.1262": "Lincoln Electric Company",
    "24.1263": "Polestar Corp.",
    "24.1264": "International Computers, Ltd",
    "24.1265": "WhereNet",
    "24.1266": "Polycom",
    "24.1267": "FS FORTH-SYSTEME GmbH",
    "24.1268": "Infinite Electronics Inc.",
    "24.1269": "SnowShore Networks, Inc.",
    "24.14962842": "HW Communications Ltd",
    "24.3984730": "Technische Alternative GmbH",
    "24.15996259": "F5 Networks, Inc.",
    "24.6876995": "Apple, Inc.",
    "24.3984733": "Huawei Technologies Co.,Ltd",
    "24.15483632": "ADL Embedded Solutions, Inc.",
    "12.123917691281408": "Wireless Systems Solutions LLC",
    "20.132855978000384": "Guangzhou Kingray information technology Co.,Ltd.",
    "12.119269998592": "kuwatec, Inc.",
    "24.3147100": "Brother industries, LTD.",
    "24.565641": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.3721308": "Cisco SPVTG",
    "24.3467774": "Metis Co., Ltd.",
    "24.1090126": "Gold3Link Electronics Co., Ltd",
    "24.2378499": "Samsung Electronics Co.,Ltd",
    "24.6584189": "Hitron Technologies. Inc",
    "24.5823373": "Fast Co., Ltd.",
    "20.242185278586880": "Wharton Electronics Ltd",
    "24.15212770": "HUMAX Co., Ltd.",
    "24.6043098": "There Corporation Oy",
    "24.2928917": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.4308": "Storage Computer Corporation",
    "24.4309": "Imasde Canarias, S.A.",
    "24.11325276": "Apple, Inc.",
    "24.4300": "CLP COMPUTER LOGISTIK PLANUNG GmbH",
    "24.4301": "Interface Concept",
    "24.4302": "Volamp, Ltd.",
    "24.4303": "Fiberlane Communications",
    "24.4304": "Witcom, Ltd.",
    "24.4305": "Top Layer Networks, Inc.",
    "24.4306": "Nitto Tsushinki Co., Ltd",
    "24.4307": "Grips Electronic Gmbh",
    "24.53350": "Wintriss Engineering Corp.",
    "24.53351": "Campio Communications",
    "24.53352": "Iwill Corporation",
    "24.53353": "Technologic Systems",
    "24.53354": "Linkup Systems Corporation",
    "24.53355": "Sr Telecom Inc.",
    "24.53356": "Sharewave, Inc.",
    "24.53357": "Acrison, Inc.",
    "24.53358": "Trendview Recorders Ltd.",
    "24.53359": "Kmc Controls",
    "24.2367211": "Apple, Inc.",
    "24.16292112": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346863607808": "Embedia",
    "24.1884": "Eastman Kodak Company",
    "24.1885": "Celleritas Inc.",
    "24.1886": "Ametek Power Instruments",
    "24.1887": "VCS Video Communication Systems AG",
    "24.1880": "DragonWave Inc.",
    "24.1881": "Boris Manufacturing Corp.",
    "24.1882": "Air Products and Chemicals, Inc.",
    "24.1883": "Gibson Guitars",
    "24.9235579": "Apple, Inc.",
    "24.1888": "TOMIS Information & Telecom Corp.",
    "24.1889": "29530",
    "12.123917687488512": "AndFun Co.,Ltd.",
    "24.10807089": "Nokia Corporation",
    "24.13913508": "Murata Manufacturing Co., Ltd.",
    "12.346864877568": "SRC Computers, LLC",
    "12.123917675769856": "AnaPico AG",
    "12.346857988096": "Penny & Giles Aerospace Ltd",
    "12.346866819072": "MAC Valves, Inc.",
    "24.6334718": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.1583739": "Nintendo Co., Ltd.",
    "24.6307054": "LG Electronics (Mobile Communications)",
    "24.8904": "Applied Instruments B.V.",
    "20.70559712935936": "NowTechnologies Zrt",
    "24.8963683": "Apple, Inc.",
    "24.535248": "SRD Innovations Inc.",
    "24.3657": "Forsway Scandinavia AB",
    "24.3656": "Lipman TransAction Solutions",
    "24.3655": "NCI System Co.,Ltd.",
    "24.3654": "Niigata Seimitsu Co.,Ltd.",
    "24.3653": "Beijing Newtry Electronic Technology Ltd",
    "24.3652": "Digital 5, Inc.",
    "24.3651": "G-Tek Electronics Sdn. Bhd.",
    "24.3650": "Motic Incoporation Ltd.",
    "24.4737962": "Apple, Inc.",
    "24.3659": "atrium c and i",
    "24.3658": "Changchun Huayu WEBPAD Co.,LTD",
    "12.123917680648192": "JD Squared, Inc.",
    "12.123917679661056": "SYSCO Sicherheitssysteme GmbH",
    "24.8435061": "Huawei Technologies Co.,Ltd",
    "24.1060246": "Looxcie Inc.",
    "12.346868260864": "Sentry 360 Security",
    "24.8902": "Sutus Inc",
    "24.6580558": "Texas Instruments",
    "24.1592287": "Caavo Inc",
    "20.145727913197568": "Arts Digital Technology (HK) Ltd.",
    "12.346854227968": "Ultracker Technology",
    "24.1850580": "Huawei Technologies Co.,Ltd",
    "24.3732395": "LCFC(HeFei) Electronics Technology co., ltd",
    "12.346852814848": "MetaWave Vedeo Systems",
    "24.8903": "SEGGER Microcontroller GmbH & Co. KG",
    "24.8153795": "GatesAir, Inc",
    "24.7608824": "IEEE Registration Authority",
    "24.8125": "Kyocera Wireless Corp.",
    "24.8900": "epro GmbH",
    "24.11567033": "Netgear",
    "24.14960317": "C&C Technic Taiwan Co., Ltd.",
    "12.123917679714304": "Fiber Optika Technologies Pvt. Ltd.",
    "24.12336047": "Apple, Inc.",
    "12.123917691666432": "Care Pvt Ltd",
    "24.8901": "INFORSON Co,Ltd.",
    "12.123917685170176": "CONTINENT, Ltd",
    "24.12333273": "Arcadyan Corporation",
    "24.9435": "Alcohol Monitoring Systems",
    "24.9434": "Innovar Systems Limited",
    "24.9437": "Centrak, Inc.",
    "24.9436": "Juniper Networks",
    "24.9431": "Intel Corporate",
    "24.9430": "Intel Corporate",
    "24.9433": "BICOM, Inc.",
    "24.9432": "IlSung Precision",
    "24.15215338": "Intel Corporate",
    "24.16253815": "Apple, Inc.",
    "24.9439": "Digitalbox Europe GmbH",
    "24.9438": "GLOBAL Technology Inc.",
    "12.123917683355648": "JASCO Applied Sciences Canada Ltd",
    "24.1372357": "Xtremio Ltd.",
    "12.123917688152064": "WTE Limited",
    "12.346855931904": "Inhand Electronics, Inc.",
    "24.8128": "Control Express Finland Oy",
    "12.123917678866432": "Turbo Technologies Corporation",
    "24.13129285": "Intermas France",
    "24.5264082": "Hangzhou Telin Technologies Co., Limited",
    "12.123917677412352": "Astronomical Research Cameras, Inc.",
    "12.346865913856": "Moco Media Pty Ltd",
    "24.319337": "Car Connectivity Consortium",
    "24.6604202": "South African Broadcasting Corporation",
    "12.346859704320": "Spirent Communications",
    "24.7102074": "Ubiquitous Internet Telecom Co., Ltd",
    "12.123917685739520": "SMART IN OVATION GmbH",
    "24.16284556": "Amped Wireless",
    "24.13112441": "Samsung Electronics Co.,Ltd",
    "12.346857570304": "Tattile srl",
    "12.123917690359808": "Lotes Tm Ooo",
    "12.346867630080": "UgMO Technologies",
    "24.13109672": "Intel Corporate",
    "24.7626640": "LAM Technologies",
    "24.2127055": "Shenzhen Voxtech Co.,Ltd",
    "24.27633": "Cisco Systems, Inc",
    "12.123917675274240": "Coveloz Technologies Inc.",
    "24.7028": "MiraLink Corporation",
    "24.7029": "Hypermedia Systems",
    "24.13157109": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "12.123917679652864": "serva transport systems GmbH",
    "24.7020": "LookX Digital Media BV",
    "24.7021": "Midtronics, Inc.",
    "24.7022": "Keysight Technologies, Inc.",
    "24.7023": "Teletrak Ltd",
    "24.7024": "IRI Ubiteq, INC.",
    "24.7025": "Telular Corp.",
    "24.7026": "Sicep s.p.a.",
    "24.63516": "Huawei Technologies Co.,Ltd",
    "24.2218": "Karam",
    "24.2219": "EnerLinx.com, Inc.",
    "24.2210": "ADI Engineering, Inc.",
    "24.2211": "Cisco Systems, Inc",
    "24.2212": "Cisco Systems, Inc",
    "24.2213": "Peninsula Systems Inc.",
    "24.2214": "Multiware & Image Co., Ltd.",
    "24.2215": "iLogic Inc.",
    "24.2216": "Systec Co., Ltd.",
    "24.2217": "SangSang Technology, Inc.",
    "24.15992485": "Hangzhou Bianfeng Networking Technology Co., Ltd.",
    "12.123917678428160": "IHI Inspection & Instrumentation Co., Ltd.",
    "24.8744": "Breeze Innovations Ltd.",
    "24.8745": "Compumedics Ltd",
    "24.8746": "SoundEar A/S",
    "24.8747": "Nucomm, Inc.",
    "24.8740": "Good Will Instrument Co., Ltd.",
    "24.8741": "Thales Avionics Ltd",
    "24.8742": "Avaak, Inc.",
    "24.8743": "uv-electronic GmbH",
    "24.14952075": "Huawei Device Co., Ltd.",
    "24.4461337": "Wkk Technology Ltd.",
    "24.8748": "Ceton Corp",
    "24.8749": "SMC Networks Inc.",
    "12.346867261440": "EMAC, Inc.",
    "12.346862792704": "Agility Mfg, Inc.",
    "24.12876941": "IEEE Registration Authority",
    "24.16048414": "Texas Instruments",
    "24.15491296": "Avi-On Labs",
    "24.1324825": "PT Prakarsa Visi Valutama",
    "24.8665144": "Samsung Electro-Mechanics(Thailand)",
    "24.2386096": "Samsung Electronics Co.,Ltd",
    "24.8665141": "Apple, Inc.",
    "12.123917687898112": "VTC Digicom",
    "24.8176276": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.16280579": "Shenzhen Teleone Technology Co., Ltd",
    "24.6655": "Finnzymes",
    "12.123917688053760": "Scharco Elektronik GmbH",
    "20.48688642654208": "Geartech Ltd",
    "24.1332794": "Huawei Device Co., Ltd.",
    "24.2672191": "Intel Corporate",
    "24.6777": "Telecomunication Technologies Ltd.",
    "24.6776": "ubtos",
    "24.6775": "ARRIS Group, Inc.",
    "24.6774": "SDT information Technology Co.,LTD.",
    "24.6773": "Sony Mobile Communications Inc",
    "24.6772": "Procare International Co",
    "24.6771": "Gemtek Technology Co., Ltd.",
    "24.6770": "Mosart Semiconductor Corp.",
    "12.346859020288": "Zhuhai Jiahe Electronics Co.,LTD",
    "24.9991450": "Huawei Device Co., Ltd.",
    "24.6779": "Teleco, Inc.",
    "24.6778": "Lismore Instruments Limited",
    "24.1747": "Alpha Telecom, Inc. U.S.A.",
    "24.1746": "Tundra Semiconductor Corp.",
    "24.1745": "Tahoe Networks, Inc.",
    "24.1744": "Elgar Electronics Corp.",
    "24.1743": "Thales Avionics In-Flight Systems, LLC",
    "24.1742": "Dateno",
    "24.1741": "Leaf Imaging Ltd.",
    "24.1740": "JMI Electronics Co., Ltd.",
    "24.4248765": "SK Networks Service CO., LTD.",
    "24.4506374": "SIFROM Inc.",
    "12.71297884405760": "NSP Europe Ltd",
    "24.1749": "Diamond Systems Corp.",
    "24.1748": "Interactive Objects, Inc.",
    "12.123917682442240": "Peek Traffic",
    "24.9453227": "ARRIS Group, Inc.",
    "24.4499650": "Sewoo Tech Co., Ltd",
    "24.7380177": "Intel Corporate",
    "24.9744415": "Apple, Inc.",
    "24.13954567": "HIWIFI Co., Ltd.",
    "24.1332242": "Entis Co., Ltd.",
    "24.7401216": "Apple, Inc.",
    "24.12325800": "Shenzhen YOUHUA Technology Co., Ltd",
    "12.346863980544": "Efftronics Systems (P) Ltd",
    "12.123917680705536": "Bayern Engineering GmbH & Co. KG",
    "12.123917679702016": "Monixo",
    "12.123917677928448": "Tattile Srl",
    "24.11316014": "Beijing LasNubes Technology Co., Ltd.",
    "12.346868711424": "Far South Networks",
    "24.10288360": "Intel Corporate",
    "12.123917687259136": "Wicell Technology",
    "12.123917676564480": "Entech Electronics",
    "12.123917687930880": "Flexsolution APS",
    "24.4494842": "Qingdao Santong Digital Technology Co.Ltd",
    "24.16548806": "N-Radio Technologies Co., Ltd.",
    "24.16385854": "NBASE-T Alliance",
    "24.9963063": "IEEE Registration Authority",
    "24.2893335": "Oculus VR, LLC",
    "24.7637517": "ARRIS Group, Inc.",
    "24.3848759": "Cirrent",
    "24.3162661": "Burg-Wächter Kg",
    "12.123917688369152": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "24.9963068": "Samsung Electronics Co.,Ltd",
    "24.284106": "BriView (Xiamen) Corp.",
    "24.32405": "Cisco Systems, Inc",
    "12.123917681872896": "\"Meta-chrom\" Co. Ltd.",
    "12.346864349184": "Audioscope 2K Srl",
    "24.12329723": "RF Industries",
    "24.8167991": "King Slide Technology CO., LTD.",
    "12.123917683531776": "Angora Networks",
    "12.346861924352": "IT-Designers GmbH",
    "24.15209326": "Huawei Technologies Co.,Ltd",
    "24.8167998": "Huawei Technologies Co.,Ltd",
    "24.16526686": "zte corporation",
    "12.123917678886912": "Yousung",
    "24.7340959": "Espressif Inc.",
    "24.10025699": "Xiaomi Communications Co Ltd",
    "24.15253944": "Nanjing Bangzhong Electronic Commerce Limited",
    "24.12105687": "2GIG Technologies",
    "20.220327871250432": "Thinci, Inc.",
    "24.7391087": "Inno S",
    "20.158667115593728": "Apollo Video Technology",
    "20.228784604184576": "Private",
    "12.123917681438720": "Beijing HuaLian Technology Co, Ltd.",
    "12.123917687934976": "Intelligent Energy Ltd",
    "24.12882508": "Keisokuki Center Co.,Ltd.",
    "24.14431729": "Positivo Tecnologia S.A.",
    "24.13948744": "Cisco Systems, Inc",
    "24.8936955": "zte corporation",
    "12.123917687480320": "CCII Systems (Pty) Ltd",
    "24.15778289": "Apple, Inc.",
    "12.119269552128": "Orbital Systems, Ltd.",
    "24.10252602": "Huawei Technologies Co.,Ltd",
    "24.12624094": "Multi Touch Oy",
    "24.1085152": "Samsung Electronics Co.,Ltd",
    "24.7650259": "Shenzhen xeme Communication Co., Ltd.",
    "12.123917687676928": "Southern Innovation",
    "12.123917683347456": "Incusense",
    "24.12624098": "Eden Innovations",
    "24.1858110": "Samsung Electronics Co.,Ltd",
    "24.9998464": "Linkpower Network System Inc Ltd.",
    "12.346867953664": "AeroVision Avionics, Inc.",
    "24.3968": "Trinity Security Systems,Inc.",
    "12.346864377856": "Jomitek",
    "12.346856435712": "Goodrich",
    "12.123917690433536": "Skreens",
    "24.8039287": "Orange",
    "20.225135235694592": "Nortek Security & Control",
    "20.154055550697472": "LocoLabs LLC",
    "24.1839055": "Sypro Optics GmbH",
    "24.15777265": "Sipod Inc.",
    "24.49987": "E-T-A Circuit Breakers Ltd",
    "12.346867781632": "Cytec Zylindertechnik GmbH",
    "24.7888328": "LG Electronics",
    "12.123917679030272": "Remote Sensing Solutions, Inc.",
    "24.12629117": "Huawei Device Co., Ltd.",
    "24.7919386": "Ciena Corporation",
    "24.13952695": "Samsung Electronics Co.,Ltd",
    "24.8916752": "Huami Information Technology Co.,Ltd.",
    "24.15528190": "zte corporation",
    "12.346868731904": "Jinyoung Contech",
    "24.45184": "Mannesmann Ipulsys B.V.",
    "24.5812495": "Samsung Electronics Co.,Ltd",
    "24.1317457": "Treetech Sistemas Digitais",
    "12.346854412288": "nVent, Schroff GmbH",
    "20.211525434343424": "IPG Automotive GmbH",
    "20.251330453045248": "Shenzhen Joyetech Electronics Co., Ltd.",
    "24.2642226": "Shenzhen Prayfly Technology Co.,Ltd",
    "12.346861953024": "Sicon srl",
    "12.123917690441728": "Thiel Audio Products Company, LLC",
    "24.11557191": "Shenzhen Qihu Intelligent Technology Company Limited",
    "24.6857742": "BSH Hausgeräte GmbH",
    "24.1315165": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346861715456": "MangoDSP",
    "24.282507": "Adapteva, Inc.",
    "24.13377768": "Apple, Inc.",
    "12.123917680066560": "Nortek Global HVAC",
    "12.346857259008": "Ace Electronics, Inc.",
    "12.123917678645248": "Vensi, Inc.",
    "24.8658470": "KERNEL-I Co.,LTD",
    "12.123917685014528": "University of Wisconsin Madison - Department of High Energy Physics",
    "24.4498534": "Groupe Ldlc",
    "24.9232285": "NEC Corporation",
    "24.14740762": "vastriver Technology Co., Ltd",
    "12.71297885069312": "Peter Huber",
    "12.346863886336": "Act",
    "24.12370715": "Kygo Life A",
    "24.14697435": "Shenzhen ViewAt Technology Co.,Ltd.",
    "24.525794": "Juniper Networks",
    "24.16570440": "Apple, Inc.",
    "20.71414665183232": "Clixxo Broadband Private Limited",
    "24.10750197": "Sagemcom Broadband SAS",
    "12.123917689733120": "Communication Systems Solutions",
    "12.71297884778496": "CDEX Corp.",
    "24.4205392": "ARRIS Group, Inc.",
    "20.53091440263168": "Shenzhen iTG robot Co.,Ltd.",
    "24.13645665": "Texas Instruments",
    "24.3414581": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917684097024": "​ASUNG TECHNO CO.,Ltd",
    "20.229558878994432": "Eco Mobile",
    "12.123917686910976": "DynaMount LLC",
    "12.346860109824": "Comsonics, Inc.",
    "24.8417892": "Huawei Device Co., Ltd.",
    "24.11796502": "Ingenico Terminals Sas",
    "12.346865405952": "Wayne Kerr Electronics",
    "24.1345854": "Vixtel Technologies Limted",
    "12.346866937856": "Gall Tankdatensysteme GmbH",
    "24.7395996": "Tecsen",
    "24.4221611": "Rim",
    "20.136823643832320": "Tohan Engineering Corporation",
    "24.15212178": "Huawei Device Co., Ltd.",
    "12.123917688205312": "Csinfotel",
    "24.3187267": "Shenzhen Prifox Innovation Technology Co., Ltd.",
    "20.180659905626112": "Innovations in Optics, Inc.",
    "20.66219390533632": "University of British Columbia",
    "24.11050938": "Samsung Electronics Co.,Ltd",
    "24.15260160": "Kivo Technology, Inc.",
    "24.9239712": "Shenzhen Smart Device Technology Co., LTD.",
    "24.13908124": "Sony Mobile Communications Inc",
    "12.346860032000": "Starling Advanced Communications",
    "24.7642799": "Apple, Inc.",
    "24.10519071": "Samsung Electronics Co.,Ltd",
    "24.12091471": "Texas Instruments",
    "20.83999498174464": "Plus One Japan Limited",
    "20.189244307931136": "Beijing Chunhong Technology Co., Ltd.",
    "12.346860023808": "ACD Elektronik GmbH",
    "24.12331991": "Revogi Innovation Co., Ltd.",
    "24.11068097": "Texas Instruments",
    "24.3187194": "Huawei Technologies Co.,Ltd",
    "20.132811180736512": "Pinhole (Beijing) Technology Co., Ltd.",
    "24.11068099": "Shenzhen YOUHUA Technology Co., Ltd",
    "20.70682231701504": "The 52nd Research Institute of China Electronic Technology Group Corporation",
    "24.7663725": "AzureWave Technology Inc.",
    "20.44213333917696": "Panasonic System Solutions Europe",
    "24.11322978": "MitraStar Technology Corp.",
    "20.36069845237760": "Zaber Technologies Inc.",
    "24.3958572": "Sciovid Inc.",
    "24.1606449": "Samsung Electronics Co.,Ltd",
    "24.4146": "Alta Technology",
    "24.4147": "Accesslan Communications, Inc.",
    "24.4144": "EION Inc.",
    "24.4145": "Objective Communications, Inc.",
    "24.4142": "Network Systems & Technologies Pvt. Ltd.",
    "24.4143": "Cisco Systems",
    "24.4140": "Lasat Networks A/S",
    "24.4141": "Hitachi Software Engineering",
    "12.123917683634176": "Telefonix Incorporated",
    "24.4148": "GNP Computers",
    "24.4149": "Elitegroup Computer Systems Co.,Ltd.",
    "24.3495": "Private",
    "24.3494": "Universal Switching Corporation",
    "24.3497": "T.E.A.M. S.L.",
    "24.3496": "Teletronics Technology Corporation",
    "24.3491": "Emerging Technologies Limited",
    "24.3490": "Infrant Technologies, Inc.",
    "24.3493": "Fabric7 Systems, Inc",
    "24.3492": "Dosch & Amand Systems Ag",
    "24.1364329": "Huawei Technologies Co.,Ltd",
    "24.16517194": "Fujitsu Limited",
    "24.3499": "Parker Hannifin GmbH Electromechanical Division Europe",
    "24.3498": "S.A.Tehnology co.,Ltd.",
    "24.285333": "Nokia Corporation",
    "20.211670366420992": "Chongqing Jiuyu Smart Technology Co.Ltd.",
    "12.71297884950528": "Logos 01 S.r.l.",
    "24.13384767": "SA.S.S. Datentechnik AG",
    "12.123917686153216": "eumig industrie-TV GmbH.",
    "20.4884645543936": "Lighthouse AI, Inc",
    "24.7387425": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11847696": "UMPI Elettronica",
    "12.346855944192": "YUYAMA MFG Co.,Ltd",
    "20.128612178067456": "InnoTrans Communications",
    "24.1112558": "Nokia Corporation",
    "24.1112555": "Industria Fueguina de Relojería Electrónica s.a.",
    "12.123917690753024": "Motomuto Aps",
    "24.8694161": "Cyber Trans Japan Co.,Ltd.",
    "12.71297883467776": "Dos&Donts SRL",
    "24.2910203": "ShenZhen Zhengjili Electronics Co., LTD",
    "24.10514346": "LG Innotek",
    "20.273272603148288": "Josh.ai",
    "24.13164848": "Milkyway Group Ltd",
    "24.13160746": "Comtrend Corporation",
    "12.346862084096": "Sands Instrumentation India Pvt Ltd",
    "24.4229167": "Extreme Networks, Inc.",
    "20.203238168068096": "Lam Research",
    "24.4666": "Posystech Inc., Co.",
    "12.346860007424": "Itibia Technologies",
    "24.12596404": "Gigastone Corporation",
    "24.3456456": "Cisco Systems, Inc",
    "12.123917676687360": "Global Technical Systems",
    "24.13143761": "IEEE Registration Authority",
    "20.4884640301056": "Apparatebau Gauting GmbH",
    "24.4255253": "Zioncom Electronics (Shenzhen) Ltd.",
    "12.346863923200": "IAdea Corporation",
    "24.13428908": "SOYEA Technology Co.,Ltd.",
    "12.346866954240": "Embedded Labs",
    "12.346856902656": "Phytec Messtechnik GmbH",
    "24.14981104": "Shanghai VLC Technologies Ltd. Co.",
    "12.346866847744": "Tyco Traffic & Transportation",
    "12.346853765120": "Cyan Technology Ltd",
    "20.228844754698240": "Shanghai Routech Co., Ltd",
    "12.119269945344": "Enerwise Solutions Ltd.",
    "20.114869210710016": "ShangHai Aigentoo Information Technology Co., Ltd",
    "24.6035407": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.5258474": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.11817058": "EmBestor Technology Inc.",
    "12.123917675261952": "Elesar Limited",
    "12.123917679005696": "Imenco Ltd",
    "20.233258805624832": "Barobo, Inc.",
    "24.6292390": "Inteno Broadband Technology AB",
    "20.242185270198272": "Suritel",
    "12.123917678641152": "synchrotron SOLEIL",
    "24.584962": "IEEE Registration Authority",
    "24.11585178": "Juniper Networks",
    "24.8926927": "Huawei Device Co., Ltd.",
    "12.346853781504": "Teradyne Inc.",
    "20.211525429100544": "KidKraft",
    "24.2654344": "Netgear",
    "24.7649771": "JinQianMao Technology Co.,Ltd.",
    "24.12081779": "Samsung Electronics Co.,Ltd",
    "12.123917676527616": "Carlson Wireless Technologies Inc.",
    "24.6839619": "Intel Corporate",
    "12.346867236864": "Horizon Co., Ltd",
    "24.8137697": "Shenzhen Ferex Electrical Co.,Ltd",
    "24.2137037": "Apple, Inc.",
    "24.671": "L-3 Communication Aviation Recorders",
    "24.670": "Information Equipment Co., Ltd.",
    "24.673": "World Wide Packets",
    "24.672": "Flatstack Ltd.",
    "24.675": "ABB Switzerland Ltd, Power Systems",
    "24.674": "Hilscher GmbH",
    "24.677": "Hewlett Packard",
    "24.2493": "Epygi Technologies, Ltd.",
    "24.679": "Vivace Networks",
    "12.346855034880": "StorLink Semi",
    "12.346856857600": "Efficient Channel Coding",
    "12.123917690138624": "Amazon Technologies Inc.",
    "12.71297884499968": "Tokyo Drawing Ltd.",
    "24.3991076": "LG Display",
    "24.12074318": "Shenzhen Cultraview Digital Technology Co.,Ltd Shanghai Branch",
    "24.2637673": "Sony Mobile Communications Inc",
    "12.123917675560960": "OptiScan Biomedical Corp.",
    "24.3433641": "Hewlett Packard",
    "24.11303501": "Alpsalpine Co,.Ltd",
    "24.7550": "Cisco-Linksys, LLC",
    "24.7551": "Tekron International Ltd",
    "24.7552": "Beijing Huahuan Eletronics Co.,Ltd",
    "12.346858774528": "Northern Power",
    "24.7554": "GN Netcom A/S",
    "24.7555": "Emitech Corporation",
    "24.7556": "Gateway, Inc.",
    "24.7557": "Call Direct Cellular Solutions",
    "24.7558": "Shinwa Industries(China) Ltd.",
    "24.7559": "VigTech Labs Sdn Bhd",
    "20.52821121564672": "Private",
    "24.7875531": "Dell Inc.",
    "20.48543327846400": "E Core Corporation",
    "24.292341": "Csst",
    "24.4738182": "Huawei Device Co., Ltd.",
    "12.123917681590272": "Magnamed Tecnologia Medica S/A",
    "20.154794721280000": "SHENZHEN D-light Technolgy Limited",
    "24.4230220": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "24.10785599": "Huawei Technologies Co.,Ltd",
    "24.14962200": "Gardasoft VisionLimited",
    "12.346861072384": "Optoelettronica Italia S.r.l.",
    "24.10748130": "Huawei Technologies Co.,Ltd",
    "24.5958": "Freedom9 Inc.",
    "24.5959": "Trimble",
    "20.66256287825920": "Ubtech Robotics Corp",
    "24.5952": "Bluberi Gaming Technologies Inc",
    "24.5953": "Defidev",
    "24.5950": "LeucotronEquipamentos Ltda.",
    "24.5951": "Belkin International Inc.",
    "24.5956": "Araneo Ltd.",
    "24.5957": "INNOTZ CO., Ltd",
    "24.5954": "Fujitsu Limited",
    "24.5955": "Deck Srl",
    "24.4482887": "Huawei Technologies Co.,Ltd",
    "12.123917685149696": "Fortuna Impex Pvt ltd",
    "24.6296678": "zte corporation",
    "24.5776268": "Technicolor CH USA Inc.",
    "12.123917678178304": "Energi innovation Aps",
    "24.2161339": "Cambridge Executive Limited",
    "24.2456": "Capinfo Company Limited",
    "24.2457": "Cp Georges Renault",
    "24.2454": "Rdi",
    "24.2455": "Nortel Networks",
    "24.2452": "Cronyx Engineering",
    "24.2453": "Castle Technology Ltd",
    "24.2450": "InterEpoch Technology,INC.",
    "24.2451": "Visteon Corporation",
    "24.1636804": "BAE Systems",
    "24.7370480": "LevelOne Communications (India) Private Limited",
    "24.2458": "Elmo Company, Limited",
    "24.2459": "Western Telematic Inc.",
    "24.1185": "Pathway Connectivity",
    "24.1184": "Verity Instruments, Inc.",
    "24.1187": "Microchip Technology Inc.",
    "24.1186": "L.S.I. Japan Co., Ltd.",
    "24.1181": "Ipanema Technologies",
    "24.1180": "Surgient Networks, Inc.",
    "24.1183": "Freescale Semiconductor",
    "24.1182": "Wirelink Co., Ltd.",
    "24.38124": "Huawei Device Co., Ltd.",
    "12.346859458560": "Kitron Microelectronics AB",
    "24.5001335": "Huawei Device Co., Ltd.",
    "24.1189": "Barco Projection Systems NV",
    "24.1188": "NetEnabled, Inc.",
    "24.9973760": "Beijing KEMACOM Technologies Co., Ltd.",
    "24.13904085": "TCT mobile ltd",
    "24.2410134": "Samsung Electronics Co.,Ltd",
    "24.4255822": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346868666368": "EATON FHF Funke + Huster Fernsig GmbH",
    "12.123917680336896": "Lineage Power Pvt Ltd.,",
    "24.2980": "Shiron Satellite Communications Ltd. (1996)",
    "24.2981": "Quasar Cipta Mandiri, PT",
    "24.2982": "Miyakawa Electric Works Ltd.",
    "24.2983": "Maranti Networks",
    "24.2984": "Hanback Electronics Co., Ltd.",
    "24.2985": "CloudShield Technologies, Inc.",
    "24.2986": "Aiphone co.,Ltd",
    "24.2987": "Advantech Technology (CHINA) Co., Ltd.",
    "24.2988": "3Com Ltd",
    "24.2989": "PC-PoS Inc.",
    "24.6205": "Vertex Link Corporation",
    "24.6204": "Encore Software Limited",
    "24.6203": "CENITS Co., Ltd.",
    "24.6202": "Westell Technologies Inc.",
    "24.6201": "Cisco-Linksys, LLC",
    "24.6200": "PanAccess Communications,Inc.",
    "12.123917685018624": "Motohaus Powersports Limited",
    "24.2680328": "Tokheim",
    "24.7887166": "RAFI GmbH & Co.KG",
    "20.31477543731200": "Shenzhen Liandian Communication Technology Co.LTD",
    "24.8955325": "Qpcom Inc.",
    "12.123917677297664": "CT Company",
    "24.9982641": "Samsung Electronics Co.,Ltd",
    "24.14207882": "Micro-Star INTL CO., LTD.",
    "24.15754690": "Yanfeng Visteon Electronics Technology (Shanghai) Co.,Ltd.",
    "24.8379": "Zax Corporation",
    "24.8378": "Center For High Performance",
    "24.11536501": "Huawei Technologies Co.,Ltd",
    "24.8375": "Namaqua Computerware",
    "24.8374": "Agile Networks, Inc.",
    "24.8377": "Metricom, Inc.",
    "24.8376": "Prime Option, Inc.",
    "24.8371": "Tattile SRL",
    "24.8370": "GKD Gesellschaft Fur Kommunikation Und Datentechnik",
    "24.8373": "Yaskawa Electric Corporation",
    "24.8372": "Terma Elektronik As",
    "24.5797493": "Beijing ECHO Technologies Co.,Ltd",
    "24.5769760": "Cisco Systems, Inc",
    "12.123917678297088": "Thales Nederland BV",
    "24.13128085": "Huawei Technologies Co.,Ltd",
    "24.11023634": "Huawei Device Co., Ltd.",
    "24.12355779": "Ningbo Dooya Mechanic & Electronic Technology Co., Ltd",
    "12.346859433984": "Pneumopartners LaenneXT SA",
    "12.346857029632": "Ieee P1609 Wg",
    "24.15993418": "INDUSNET Communication Technology Co.,LTD",
    "12.346854129664": "Cognex Corporation",
    "20.58071044849664": "Captec Ltd",
    "12.346864902144": "Austco Marketing & Service (USA) ltd.",
    "24.10781745": "Samsung Electronics Co.,Ltd",
    "24.327309": "Huawei Technologies Co.,Ltd",
    "20.242185281732608": "Nautilus Infotech CO., Ltd.",
    "24.32849": "ADC Fibermux",
    "12.123917689589760": "EkspertStroyProekt LLC",
    "12.123917675220992": "HKW-Elektronik GmbH",
    "12.346854473728": "Silverback Systems",
    "24.8170868": "Cisco Systems, Inc",
    "12.123917681176576": "Imecon Engineering SrL",
    "24.3428778": "Jid Global",
    "20.212188459433984": "Shenzhen Elsky Technology Co., Ltd",
    "24.9488002": "IEEE Registration Authority",
    "12.123917687058432": "JSK System",
    "12.123917683879936": "Polynet Telecommunications Consulting and Contractor Ltd.",
    "12.346855997440": "Engim, Inc.",
    "24.1625892": "Huawei Technologies Co.,Ltd",
    "24.16311547": "Actiontec Electronics, Inc",
    "20.48548679778304": "Jiangsu JianHu Science & Technology Co., Ltd.",
    "24.5016368": "Actiontec Electronics, Inc",
    "24.7510": "Kramer Electronics Ltd.",
    "24.5790146": "Extreme Networks, Inc.",
    "24.2367392": "Huawei Technologies Co.,Ltd",
    "12.346854162432": "Metrotech",
    "24.11569553": "Lge",
    "24.2925326": "Power Dragon Development Limited",
    "20.212188460482560": "Dropbeats Technology Co., Ltd.",
    "24.2905905": "Wingtech Group (HongKong）Limited",
    "24.7650011": "Longconn Electornics(shenzhen)Co.,Ltd",
    "24.15994499": "ARRIS Group, Inc.",
    "24.9980897": "Hewlett Packard",
    "12.346856275968": "Teneros",
    "24.5506214": "ASUSTek COMPUTER INC.",
    "12.123917676302336": "Elsist Srl",
    "24.1079063": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.11824941": "Wahoo Fitness",
    "12.123917686124544": "Osmozis",
    "12.123917688045568": "Magnetek",
    "24.6852553": "Zhuolian (Shenzhen) Communication Co., Ltd",
    "24.5832113": "LG Electronics",
    "24.14686613": "Nutanix",
    "12.346860621824": "Open System Solutions Limited",
    "20.180680385363968": "Larva.io OÜ",
    "24.14999749": "Samsung Electronics Co.,Ltd",
    "24.9223870": "Xiaomi Communications Co Ltd",
    "24.14168162": "Apple, Inc.",
    "20.176772031184896": "Speedgoat GmbH",
    "24.14218203": "Post CH AG",
    "24.5916": "NT MicroSystems, Inc.",
    "24.2113882": "Smarteh d.o.o.",
    "24.37094": "ALi Corporation",
    "24.5914": "Winegard Company",
    "24.9069": "ResMed Ltd",
    "24.9734165": "Infinique Worldwide Inc",
    "24.37091": "Avex Electronics Inc.",
    "24.9222693": "Unionman Technology Co.,Ltd",
    "12.346858680320": "Omnitrol Networks, Inc.",
    "24.37090": "Distributed Processing Technology",
    "24.3970261": "Juniper Networks",
    "12.346853859328": "Paltronics, Inc.",
    "24.7651910": "Amazon Technologies Inc.",
    "24.37093": "Teknema, Inc.",
    "24.37092": "Nec America, Inc.",
    "12.346855194624": "iamba LTD.",
    "12.346853212160": "Corelatus A.B.",
    "20.198237809016832": "Huizhou Sunoda Technology Co. Ltd",
    "12.123917685530624": "Capsys",
    "20.52912296296448": "Hunan Changsha Hengjian Technoldgy Develpment Co.,Ltd.",
    "12.123917685239808": "Sevensense Robotics AG",
    "12.346864967680": "SET GmbH",
    "24.9989365": "zte corporation",
    "24.37099": "Sentry Telecom Systems",
    "24.38911": "Heimann Sensor GmbH",
    "12.123917678014464": "Eastern Science & Technology Co., Ltd",
    "24.37098": "Alpha Technologies, Inc.",
    "24.9064": "Zebra Technologies Inc",
    "24.14727501": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.12349889": "XTrillion, Inc.",
    "24.9741258": "KonnectONE, LLC",
    "24.9065": "Cisco-Linksys, LLC",
    "24.14191324": "Huawei Device Co., Ltd.",
    "24.14207494": "Titan DataCenters France",
    "24.12095945": "Handreamnet",
    "24.7103514": "Penguin Computing",
    "12.346855215104": "SRI International",
    "12.123917689450496": "NUBURU Inc.",
    "12.71297884971008": "Magic Systems",
    "20.180394597023744": "tinylogics",
    "24.8181094": "Amazon Technologies Inc.",
    "12.123917678370816": "Synaccess Networks Inc.",
    "24.7363": "ARRIS Group, Inc.",
    "24.20726": "Pan-International Industrial Corp.",
    "24.20724": "Sigmatek Gmbh & Co. Kg",
    "20.224641226375168": "Feelmore Labs",
    "12.123917690576896": "Breas Medical AB",
    "24.7367": "Rembrandt Technologies, LLC d/b/a REMSTREAM",
    "24.7366": "ProStor Systems",
    "24.14177442": "Extreme Networks, Inc.",
    "24.7365": "3Com Ltd",
    "24.2686565": "DongGuan Siyoto Electronics Co., Ltd",
    "24.7364": "Hewlett Packard",
    "24.14974618": "zte corporation",
    "12.346860158976": "MedAvant Healthcare",
    "20.57194445799424": "i3 International Inc.",
    "12.123917681860608": "Advanced Telecommunications Research Institute International",
    "12.123917682806784": "Apator Miitors ApS",
    "24.6579748": "Sagemcom Broadband SAS",
    "20.242876970696704": "Tiertime Corporation",
    "24.6087412": "AcSiP Technology Corp.",
    "24.9482524": "Dell Inc.",
    "24.7616695": "Canon Inc.",
    "24.13657723": "Samsung Electronics Co.,Ltd",
    "12.346856308736": "MC TECHNOLOGY GmbH",
    "20.220958191255552": "Yarward Electronics Co., Ltd.",
    "24.262858": "Shenzhen Vtsonic Co.,ltd",
    "12.123917689475072": "C Tech Bilisim Teknolojileri San. Ve Tic. A.S.",
    "24.16274616": "Amazon Technologies Inc.",
    "12.346860179456": "Quantum Medical Imaging",
    "24.1314975": "Apple, Inc.",
    "24.15778380": "Zhejiang FeiYue Digital Technology Co., Ltd",
    "24.14683577": "Cree, Inc.",
    "24.7906276": "Apple, Inc.",
    "24.14732995": "Askey Computer Corp",
    "24.7617": "Audinate Pty L",
    "24.7616": "Enphase Energy",
    "24.7615": "Radiient Technologies, Inc.",
    "24.7614": "ARRIS Group, Inc.",
    "20.22126500249600": "Private",
    "24.7612": "Nintendo Co., Ltd.",
    "24.7611": "Dynamic System Electronics Corp.",
    "24.7610": "Sony Corporation",
    "20.4884647641088": "Digibest Technology Co., Ltd.",
    "24.3686160": "DBL Technology Ltd.",
    "12.123917686091776": "Mes Communication Co., Ltd",
    "24.7619": "RIKOR TV, Ltd",
    "24.7618": "Xortec Oy",
    "24.13393230": "Zyxel Communications Corporation",
    "12.123917678047232": "Atos spa",
    "12.346858299392": "Tonna Electronique",
    "24.9188144": "EmFirst Co., Ltd.",
    "24.42884": "ITX security",
    "24.11824124": "Intel Corporate",
    "24.2421375": "Hewlett Packard Enterprise",
    "24.3964170": "Apple, Inc.",
    "12.123917681250304": "Microvision",
    "24.2897568": "Capisco Limited",
    "24.15512339": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.13915011": "Murata Manufacturing Co., Ltd.",
    "12.123917685829632": "GS Industrie-Elektronik GmbH",
    "12.123917687091200": "CAITRON GmbH",
    "12.346852560896": "Simple Network Magic Corporation",
    "24.11019262": "LG Electronics",
    "24.11314791": "Electronic Systems Protection, Inc.",
    "24.8679089": "Park Assist LLC",
    "24.1633511": "Buffalo.Inc",
    "12.71297884246016": "Tieline Research Pty Ltd",
    "24.6610947": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.6850933": "nuoxc",
    "24.9202317": "Technicolor CH USA Inc.",
    "12.346859110400": "GFP Lab S.r.l.",
    "12.123917675425792": "Neopath Integrated Systems Ltda",
    "12.346862997504": "Arrowvale Electronics",
    "24.16532579": "Universal Audio, Inc",
    "24.274704": "Inspur Group Co., Ltd.",
    "12.71297884348416": "CEMSI, Inc.",
    "24.3431355": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "20.132777854894080": "Huwomobility",
    "24.3156306": "Hillstone Networks Inc",
    "12.119269797888": "Nistica",
    "24.3988873": "Somo Holdings & Tech. Co.,Ltd.",
    "12.346854879232": "Kinetics",
    "24.13938679": "Arista Networks",
    "24.588512": "Fohhn Audio AG",
    "24.11298013": "IEEE Registration Authority",
    "24.14423954": "Captivate Network",
    "24.8393602": "Huawei Technologies Co.,Ltd",
    "24.15264333": "Huawei Technologies Co.,Ltd",
    "24.7870632": "Huawei Device Co., Ltd.",
    "24.2908420": "Shenzhen Fenglian Technology Co., Ltd.",
    "24.7403885": "Hangzhou H3C Technologies Co., Limited",
    "24.15778673": "Apple, Inc.",
    "24.11068754": "JUWEL Aquarium AG & Co. KG",
    "24.10224280": "Samsung Electronics Co.,Ltd",
    "24.12326840": "Intel Corporate",
    "12.123917690908672": "SENSYS GmbH",
    "24.6573890": "JETTER CO., Ltd.",
    "20.171977808543744": "Appareo Systems, LLC",
    "24.11023466": "Samsung Electronics Co.,Ltd",
    "24.5775343": "Tuttnaer LTD",
    "24.10262409": "1More",
    "12.123917685456896": "Asymmetric Technologies",
    "20.224420107911168": "Newstar (Hk) Electronic Development Limited",
    "12.346857152512": "Elbit Systems Ltd.",
    "12.123917683290112": "Atessa, Inc.",
    "12.123917683875840": "Ionetech",
    "24.15492055": "Intel Corporate",
    "20.255861672902656": "internet domain name system beijing engineering research center ltd",
    "12.123917682409472": "Guardian Controls International Ltd",
    "24.5032271": "Nokia",
    "24.13917506": "Nokia Corporation",
    "24.8136636": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.14964967": "Samsung Electronics Co.,Ltd",
    "12.346859945984": "EFG CZ spol. s r.o.",
    "24.8804": "Hewlett Packard",
    "12.123917690888192": "Xj Electric Co., Ltd.",
    "24.46550": "Omnibit Inc.",
    "24.12647417": "IEEE Registration Authority",
    "24.9500817": "Kaonmedia CO., LTD.",
    "12.123917690867712": "dds",
    "24.12882707": "100fio networks technology llc",
    "24.12342353": "Samsung Electronics Co.,Ltd",
    "24.6067106": "Liteon Technology Corporation",
    "24.7117153": "Sagemcom Broadband SAS",
    "24.6613328": "Cisco Systems, Inc",
    "24.14966529": "Raspberry Pi Trading Ltd",
    "12.71297885061120": "NKT Photonics A/S",
    "24.16039808": "Huawei Technologies Co.,Ltd",
    "24.5562112": "Ulterius Technologies, LLC",
    "24.7640961": "zte corporation",
    "20.101728422723584": "BeiJing Xinsheng Technology Co.,Ltd",
    "24.53422": "Oresis Communications, Inc.",
    "24.16279041": "Suzhou Chi-tek information technology Co., Ltd",
    "24.8398473": "D-Link International",
    "12.71297885097984": "FST21 Ltd.",
    "24.7911105": "AOBO Telecom Co.,Ltd",
    "24.11287415": "Sagemcom Broadband SAS",
    "24.9989576": "Toshiba Mitsubishi-Electric Industrial Systems Corporation",
    "20.111031376412672": "Bridgeport Instruments, LLC",
    "12.123917689999360": "Vulcan Wireless Inc.",
    "20.163808563691520": "Uoi Technology Corporation",
    "20.158667116642304": "North Pole Engineering, Inc.",
    "12.123917689544704": "Abbott Diagnostics Technologies AS",
    "24.7138230": "Huawei Technologies Co.,Ltd",
    "24.12638260": "xvtec ltd",
    "20.89058713796608": "Simicon",
    "12.123917681750016": "Simplified MFG",
    "24.12108911": "Primmcon Industries Inc",
    "24.13408534": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346853478400": "Time Terminal Adductor Group AB",
    "24.1069319": "Intel Corporate",
    "12.123917680267264": "MC2-Technologies",
    "12.123917678718976": "Lookman Electroplast Industries Ltd",
    "20.168198563430400": "MSB Elektronik und Gerätebau GmbH",
    "12.346864152576": "CT Company",
    "24.14430989": "Qing Dao Haier Telecom Co.,Ltd.",
    "20.273272609439744": "THK Co.,LTD.",
    "24.9494768": "IEEE 1722a Working Group",
    "20.159182264205312": "S.A.E.T. S.R.L.",
    "24.2945262": "Apple, Inc.",
    "12.123917676044288": "Vocality International Ltd",
    "24.9734012": "ARRIS Group, Inc.",
    "12.123917678596096": "Cyberteam Sp z o o",
    "24.5523686": "Beijing Xiaomi Mobile Software Co., Ltd",
    "20.84372900282368": "Heliotis AG",
    "12.346861199360": "Broadcast Electronics",
    "24.9179595": "ABUS Security-Center GmbH & Co. KG",
    "20.67047980531712": "Annapurna labs",
    "12.123917677568000": "Telecom Sante",
    "12.346854772736": "Allied Telesis K.K.",
    "24.3721641": "SMART High Reliability Solutions, Inc.",
    "12.123917684293632": "Niveo International BV",
    "24.4498477": "TCT mobile ltd",
    "24.12854178": "PT. Emsonic Indonesia",
    "20.264625134239744": "Quantum Power Systems",
    "24.12340420": "Apple, Inc.",
    "24.8142006": "Shenzhen Homecare Technology Co.,Ltd.",
    "24.584564": "HUMAX Co., Ltd.",
    "12.123917680402432": "FlowNet LLC",
    "24.14436926": "Hewlett Packard",
    "24.13376480": "IEEE Registration Authority",
    "20.57194447896576": "ClearCaptions LLC",
    "24.14459168": "Nokia",
    "12.346867900416": "Bmr",
    "12.346854182912": "EMKA-electronic AG",
    "24.3208889": "Ecocentric Energy",
    "24.6861882": "WaterFurnace International, Inc.",
    "12.346856734720": "Deister Electronic GmbH",
    "12.123917690793984": "Kyoto Denkiki",
    "24.4872815": "Applied Materials",
    "24.12595185": "Volta Networks",
    "24.10246724": "Compal Information (Kunshan) Co., Ltd.",
    "12.123917686358016": "Invissys",
    "12.123917681532928": "Dave Srl",
    "24.544761": "Cisco Systems, Inc",
    "24.12450": "Lightner Engineering",
    "24.12451": "Cisco Systems, Inc",
    "24.12452": "Woodwind Communications System",
    "24.12453": "Active Power",
    "24.12454": "Vianet Technologies, Ltd.",
    "24.12455": "Schweitzer Engineering",
    "24.12456": "Ol'E Communications, Inc.",
    "24.12457": "Netiverse, Inc.",
    "12.346857050112": "Faber Electronics BV",
    "24.12459": "Delta Networks, Inc.",
    "20.141268016431104": "Private",
    "24.14983983": "Shanghai Artimen Technology Co., Ltd.",
    "12.123917679259648": "Nexelec",
    "20.167152461021184": "E. P. Schlumberger",
    "24.6865997": "Motorola Mobility LLC, a Lenovo Company",
    "12.346864009216": "ElectroCom",
    "12.71297885073408": "Omnik New Energy Co., Ltd",
    "24.14472098": "Apple, Inc.",
    "12.123917688270848": "i-View Communication Inc.",
    "24.804669": "Hitron Technologies. Inc",
    "20.110162577719296": "Shenzhen He&e Technology Co.,Ltd.",
    "24.5767355": "Juniper Networks",
    "24.12355731": "VILLBAU Ltd.",
    "24.9494523": "Homerider Systems",
    "12.346863271936": "Sentry 360 Security",
    "24.14177653": "Samsung Electronics Co.,Ltd",
    "24.11810531": "Kbvision Group",
    "24.16516124": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.58144491307008": "CREW by True Rowing, Inc.",
    "20.180698115735552": "Innopro",
    "24.13933812": "Palo Alto Networks",
    "20.452699619328": "Annapurna labs",
    "12.346860617728": "Alphion Corporation",
    "24.10022811": "Ohsung",
    "24.6592861": "Lge",
    "12.123917685993472": "SmartSafe",
    "24.5426": "Consumer Technologies Group, LLC",
    "24.5427": "Nadam.Co.,Ltd",
    "24.5424": "Dell EMC",
    "24.5425": "Kocom",
    "24.5422": "PacketHop, Inc.",
    "24.5423": "ARRIS Group, Inc.",
    "24.5420": "Cisco Systems, Inc",
    "24.5421": "TenX Networks, LLC",
    "24.5428": "A Beltrónica-Companhia de Comunicações, Lda",
    "24.5429": "OTE Spa",
    "24.13931416": "Nokia Corporation",
    "24.2900993": "Samsung Electronics Co.,Ltd",
    "24.2637850": "C8 MediSensors, Inc.",
    "20.264165324226560": "XI'AN MEI SHANG MEI WIRELESS TECHNOLOGY.Co., Ltd.",
    "24.7875428": "Bose Corporation",
    "24.15260310": "Zhuhai Tianrui Electrical Power Tech. Co., Ltd.",
    "12.346864410624": "Fiber Connections Inc.",
    "24.3860": "Mindray Co., Ltd.",
    "24.3861": "Icotera A/S",
    "24.3862": "Jay How Technology Co.,",
    "24.3863": "Insta Elektro GmbH",
    "24.3864": "Industrial Control Systems",
    "12.346864046080": "Sanyo Electronic Industries Co.,Ltd",
    "24.3866": "Gaming Support B.V.",
    "24.3867": "Ego Systems Inc.",
    "24.3868": "DigitAll World Co., Ltd",
    "24.3869": "Cosmo Techs Co., Ltd.",
    "24.7638861": "duagon Germany GmbH",
    "24.10512774": "Wellav Technologies Ltd",
    "20.180394593878016": "Moog Music Inc.",
    "24.6609803": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.10794526": "Huawei Device Co., Ltd.",
    "24.4220068": "The Furukawa Electric Co., Ltd",
    "24.4489": "Aerotech Inc",
    "24.4488": "Enterasys",
    "24.4487": "Category Solutions, Inc",
    "24.4486": "Prime Systems, Inc.",
    "24.4485": "Hewlett Packard",
    "24.2671135": "Samsung Electro-Mechanics(Thailand)",
    "24.4483": "Datalogic ADC, Inc.",
    "24.4482": "IMI Norgren Ltd",
    "24.4481": "InterEnergy Co.Ltd,",
    "24.4480": "ARRIS Group, Inc.",
    "24.7120890": "Youngbo Engineering Inc.",
    "24.5801326": "Qsc Ag",
    "12.123917679550464": "Technological Application and Production One Member Liability Company (Tecapro company)",
    "24.7358018": "Chyng Hong Electronic Co., Ltd.",
    "24.29132": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.180698125172736": "Ksenia Security srl",
    "12.123917687701504": "D.E.M. Spa",
    "20.255861676048384": "Alperia Fiber srl",
    "12.346859991040": "Euro Display Srl",
    "24.8423703": "Tp-Link Technologies Co.,Ltd.",
    "20.181338948763648": "Guangzhou Maxfaith Communication Technology Co.,LTD.",
    "24.3284": "Positron Public Safety Systems inc.",
    "24.3285": "Passave Inc.",
    "24.3286": "Partner Tech",
    "24.3287": "Nallatech Ltd",
    "24.3280": "Symetrix",
    "24.3281": "SFOM Technology Corp.",
    "24.3282": "Schaffner EMV AG",
    "24.3283": "Prettl Elektronik Radeberg GmbH",
    "24.3288": "M. K. Juchheim GmbH & Co",
    "24.3289": "Itcare Co., Ltd",
    "24.6825514": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.14691222": "Apple, Inc.",
    "20.167594809098240": "Shenzhen cositea electronics technology co.,LTD",
    "12.346854576128": "Mettler Toledo",
    "12.346863181824": "CT Company",
    "24.8959": "Nivis Llc",
    "24.8958": "Advanced Illumination",
    "24.8955": "Intel Corporate",
    "24.8954": "Intel Corporate",
    "24.8957": "Nokia Danmark A/S",
    "24.8956": "Nokia Danmark A/S",
    "24.8951": "Conceptronic",
    "24.8950": "Syracuse Research Corporation",
    "24.8953": "Pollin Electronic GmbH",
    "24.8952": "PIMA Electronic Systems Ltd.",
    "24.8154103": "NTI co., ltd.",
    "24.16296511": "DWnet Technologies(Suzhou) Corporation",
    "24.3138": "Routerboard.com",
    "24.3139": "Ralink Technology, Corp.",
    "24.3136": "Altech Controls",
    "24.3137": "Cisco-Linksys, LLC",
    "24.3134": "Crest Audio",
    "24.3135": "Cogent Defence & Security Networks,",
    "24.3445103": "UserGate Ltd.",
    "24.3133": "Glsystech Co., Ltd.",
    "24.3130": "Oxance",
    "24.3131": "Orion Electric Co., Ltd.",
    "12.346856243200": "CUE, a.s.",
    "12.346867138560": "ITW Reyflex North America",
    "20.58144485015552": "Ningbo Hua Gao Mdt Info Tech Ltd",
    "24.15248371": "Huawei Technologies Co.,Ltd",
    "24.15484528": "Meinberg Funkuhren GmbH & Co. KG",
    "20.167137992769536": "Hanshin Medical Co., Ltd.",
    "12.346864300032": "Emc Partner Ag",
    "24.1064254": "China Mobile Group Device Co.,Ltd.",
    "12.119269896192": "P A Network Laboratory Co.,Ltd",
    "20.159182265253888": "Nanjing Jiexi Technologies Co., Ltd.",
    "24.8691637": "Huawei Technologies Co.,Ltd",
    "24.6500": "Doorking Inc.",
    "24.3674325": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.6502": "Asiarock Technology Limited",
    "12.346865750016": "Gogo BA",
    "24.6504": "Digital Video Networks(Shanghai) CO. LTD.",
    "24.6505": "Nortel Networks",
    "24.6506": "MikroM GmbH",
    "24.6507": "Danpex Corporation",
    "24.6508": "Etrovision Technology",
    "24.6509": "Raybit Systems Korea, Inc",
    "12.123917685059584": "ProTom International",
    "24.16005385": "Hewlett Packard",
    "24.16001409": "vivo Mobile Communication Co., Ltd.",
    "24.12879112": "Intel Corporate",
    "24.5803364": "Xi'an Clevbee Technology Co.,Ltd",
    "12.346862673920": "Cloos Electronic Gmbh",
    "20.101728419577856": "Shanghai Yanhe automation technology co.,LTD",
    "24.12607097": "Shenzhen Huaxun Ark Technologies Co.,Ltd",
    "24.15230420": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.1366917": "S Net Media",
    "24.5265308": "Seiko Epson Corporation",
    "24.5285603": "David Clark Company",
    "24.12593523": "Audyssey Laboratories Inc.",
    "12.346864783360": "Globe Wireless",
    "24.48014": "HME Co., Ltd.",
    "12.346860240896": "T 1 Engineering",
    "20.229206576332800": "Greenwave Scientific",
    "24.8676006": "EuroCB (Phils), Inc.",
    "24.8389508": "Ruckus Wireless",
    "12.346868592640": "Aplex Technology Inc.",
    "12.123917688197120": "Chengdu Meihuan Technology Co., Ltd",
    "12.123917681422336": "Labotect Labor-Technik-Göttingen GmbH",
    "24.9458758": "Tes Electronic Solutions",
    "24.13934139": "Guangzhou Shiyuan Electronic Technology Company Limited",
    "12.346866786304": "Ocean Sonics",
    "24.14680741": "ABB Robotics",
    "24.9703": "Sony Mobile Communications Inc",
    "24.9702": "Belgian Monitoring Systems bvba",
    "24.9701": "LG Electronics (Mobile Communications)",
    "24.9700": "OMNI-WiFi, LLC",
    "24.9707": "Reutech Radar Systems (PTY) Ltd",
    "24.9706": "Iphion BV",
    "12.346857680896": "Digi Sesn Ag",
    "24.9704": "Idaho Technology",
    "24.6188": "Ascend Networks, Inc.",
    "24.9709": "NuVo Technologies LLC",
    "24.9708": "Humanware",
    "24.15991594": "Amazon Technologies Inc.",
    "24.2112007": "Cisco Systems, Inc",
    "24.1570": "Chung Fu Chen Yeh Enterprise Corp.",
    "24.1571": "MGE UPS Systems France",
    "24.1572": "Gentner Communications Corp.",
    "24.323646": "Huawei Device Co., Ltd.",
    "24.1574": "MWE GmbH",
    "12.346866561024": "Esensors, Inc.",
    "24.5859": "Askey Computer Corp",
    "24.5858": "American Fibertek, Inc.",
    "24.5857": "SiliconStor, Inc.",
    "24.5856": "3Com Ltd",
    "24.5855": "Lundinova AB",
    "24.5854": "FAST Inc",
    "24.5853": "Gigabeam Corporation",
    "24.5852": "Archos",
    "24.5851": "Samsung Electronics Co.,Ltd",
    "24.5850": "Futronic Technology Co. Ltd.",
    "12.346853027840": "Lake Technology, Ltd.",
    "12.346852352000": "Turtle Mountain Corp",
    "24.7132810": "LG Electronics (Mobile Communications)",
    "24.7403563": "DWnet Technologies(Suzhou) Corporation",
    "12.346858082304": "Equipos de Telecomunicación Optoelectronicos, S.A.",
    "24.13369472": "Bettini Srl",
    "24.6186": "Taiwan Video & Monitor",
    "24.562903": "AVM GmbH",
    "12.123917675380736": "Amber Kinetics Inc",
    "24.6187": "Softier",
    "20.30931573276672": "Soundtrack Your Brand Sweden AB",
    "24.4495194": "Shenzhen Fast Technologies Co.,Ltd",
    "24.11584135": "Technicolor CH USA Inc.",
    "24.10544888": "Texas Instruments",
    "24.16272691": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346862735360": "Industrial Computing Ltd",
    "12.123917678555136": "Contec Americas Inc.",
    "12.123917675978752": "Picture Elements, Inc.",
    "24.11589021": "Shenzhen Zowee Technology Co., Ltd",
    "12.119269507072": "ZAO &quot;RADIUS Avtomatika&quot;",
    "24.8188056": "Bee Beans Technologies, Inc.",
    "24.4723273": "Luxcom Technologies Inc.",
    "12.346852253696": "DSP Design Ltd.",
    "24.13918621": "Apple, Inc.",
    "24.13410170": "Chiun Mai Communication Systems, Inc",
    "12.123917687037952": "Mahle",
    "20.246679819255808": "Annapurna labs",
    "12.123917684215808": "Arwin Technology Limited",
    "24.6341067": "Fujian Great Power PLC Equipment Co.,Ltd",
    "24.7134889": "Cisco Meraki",
    "12.123917685329920": "ATX Networks Corp",
    "20.228784607330304": "powerall",
    "24.1394": "Deonet Co., Ltd.",
    "24.1395": "Cisco Systems, Inc",
    "24.1396": "Cisco Systems, Inc",
    "24.1397": "CDS-Electronics BV",
    "24.1390": "National Enhance Technology, Inc.",
    "20.4471494017024": "Nuance Hearing Ltd.",
    "24.1392": "Baydel Ltd.",
    "24.1393": "Seiwa Electronics Co.",
    "24.1398": "NSM Technology Ltd.",
    "24.1399": "SM Information & Communication",
    "24.15242486": "Tp-Link Technologies Co.,Ltd.",
    "24.15754717": "Primax Electronics Ltd.",
    "24.4726573": "Shenzhen Jie Shi Lian Industrial Co.,LTD",
    "24.6309926": "Newbridge Technologies Int. Ltd.",
    "24.1581106": "Apple, Inc.",
    "24.6566568": "Intel Corporate",
    "24.10235553": "Xiaomi Communications Co Ltd",
    "24.11298566": "Ruckus Wireless",
    "20.13592319492096": "S2C limited",
    "12.119269515264": "Géant",
    "20.66219395776512": "Zone Controls AB",
    "24.10522846": "Veedims,Llc",
    "24.4778800": "Eterna Technology, Inc.",
    "24.13635437": "T&W Electronics Company",
    "12.123917691138048": "Ars Products",
    "12.123917680361472": "ATX Networks Corp",
    "24.16256900": "Natural Security SAS",
    "12.123917688791040": "MI Inc.",
    "12.123917690085376": "Eon",
    "24.1048788": "DEC",
    "20.211525433294848": "Nanjing Balance Network Technology Co., Ltd",
    "20.14223583215616": "chengdu joyotime Technology Co., Ltd.",
    "24.4818": "Texas Instruments",
    "24.4819": "Zetta Systems, Inc.",
    "12.123917680779264": "VAGLER International Sdn Bhd",
    "24.822825": "Arcadyan Corporation",
    "24.4810": "Mechatronic Brick Aps",
    "24.4811": "CSS Inc.",
    "24.4812": "Bitatek CO., LTD",
    "24.4813": "ASEM SpA",
    "24.4814": "Advanced Cybernetics Group",
    "24.4815": "Accton Technology Corp",
    "24.4816": "Gossen-Metrawatt-GmbH",
    "24.1889762": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.15745564": "Intel Corporate",
    "24.13680172": "CNSLink Co., Ltd.",
    "24.5558704": "Universal Laser Systems, Inc",
    "24.15774930": "D-Link International",
    "20.159182271545344": "Li Seng Technology Ltd.",
    "20.264809147793408": "720?bei jing?Health iTech Co.,Ltd",
    "24.561338": "Danlaw Inc",
    "12.123917687373824": "MB connect line GmbH Fernwartungssysteme",
    "20.452712202240": "Desird Design R&D",
    "24.14471060": "Beijing Rongcheng Hutong Technology Co., Ltd.",
    "24.2635575": "Apple, Inc.",
    "24.2675119": "Nokia Corporation",
    "12.123917680205824": "Dorlet Sau",
    "12.346857885696": "Solystic",
    "12.346868830208": "Scandinova Systems AB",
    "24.3963967": "ASUSTek COMPUTER INC.",
    "12.123917690871808": "Shengli Financial Software Development",
    "12.123917691727872": "Aplex Technology Inc.",
    "24.15232459": "Samsung Electronics Co.,Ltd",
    "20.84187704983552": "LumiGrow Inc.",
    "12.123917685080064": "Enerwise Solutions Ltd.",
    "12.123917688516608": "Dalcnet srl",
    "12.71297884250112": "Orbital A/S",
    "24.13917296": "Wi-Fi Alliance",
    "20.268341265563648": "DXG Technology Corp.",
    "24.6613121": "Aruba, a Hewlett Packard Enterprise Company",
    "12.346858340352": "Buyang Electronics Industrial Co., Ltd.",
    "12.71297884200960": "Marmitek BV",
    "24.15227559": "LLC Emzior",
    "24.538524": "Topaz Co. LTD.",
    "24.3150003": "Liteon Technology Corporation",
    "24.4724556": "MicroPower technologies",
    "24.14735778": "Hippih aps",
    "24.190": "The Nti Group",
    "24.15747586": "Cisco Systems, Inc",
    "24.191": "Symmetric Computer Systems",
    "24.192": "Western Digital now SMC (Std. Microsystems Corp.)",
    "24.193": "Madge Ltd.",
    "12.346860822528": "Quantum Controls BV",
    "12.123917688434688": "Netberg",
    "24.194": "Information Presentation Tech.",
    "24.195": "Harris Corporation",
    "24.3158581": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "24.196": "Waters Div. Of Millipore",
    "24.3973909": "Bless Information & Communications Co., Ltd",
    "24.3464189": "Cisco Systems, Inc",
    "24.332": "Berkeley Process Control",
    "12.346864422912": "Pro4tech",
    "24.4512608": "Nanjing Baihezhengliu Technology Co., Ltd",
    "24.333": "Shin Kin Enterprises Co., Ltd",
    "12.123917678063616": "Brnet Co.,Ltd.",
    "24.22956": "Kpn. B.V.",
    "24.14195393": "Nokia",
    "20.35227551399936": "Amazon Technologies Inc.",
    "24.2153619": "Cheetah Hi-Tech, Inc.",
    "24.11287162": "Roku, Inc.",
    "12.123917678096384": "Scame Sistemi srl",
    "12.346861064192": "AT&T Government Solutions",
    "12.346867408896": "Macronet s.r.l.",
    "24.339": "Archtek Telecom Corporation",
    "12.71297883553792": "Rosslare Enterprises Limited",
    "12.123917682348032": "COMPAR Computer GmbH",
    "24.6575617": "Tp-Link Technologies Co.,Ltd.",
    "12.346864263168": "MicroPoise",
    "24.14471932": "Huawei Technologies Co.,Ltd",
    "24.14196321": "Huawei Device Co., Ltd.",
    "12.346856767488": "XYTAC system technologies",
    "12.71297883643904": "CT Company",
    "24.13916160": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.268804": "WaveIP",
    "24.37005": "Vickers Electronics Systems",
    "24.13401399": "ARRIS Group, Inc.",
    "24.5981": "Dongseo system.",
    "24.14710026": "Alpsalpine Co,.Ltd",
    "24.14681541": "Tp-Link Technologies Co.,Ltd.",
    "24.5980": "Sharp Corporation",
    "24.10754312": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12113654": "Huawei Technologies Co.,Ltd",
    "24.5983": "XENOLINK Communications Co., Ltd.",
    "24.12631194": "Huawei Technologies Co.,Ltd",
    "24.5515932": "LSY Defense, LLC.",
    "12.123917676175360": "Kbpr Llc",
    "12.346865344512": "EMAC, Inc.",
    "12.346857562112": "AC SYSTEMS, s.r.o.",
    "12.119269384192": "DIMEP Sistemas",
    "24.5985": "Private",
    "24.5515938": "Alpha Networks Inc.",
    "20.74781683613696": "Hangzhou Asia Infrastructure Tech. Co., Ltd.",
    "24.5984": "Naito Densei Machida MFG.CO.,LTD",
    "24.2112946": "Huawei Technologies Co.,Ltd",
    "24.5987": "Essentia S.p.A.",
    "24.5269514": "Huawei Technologies Co.,Ltd",
    "12.123917679296512": "DEV Systemtechnik GmbH& Co KG",
    "24.9485798": "Quectel Wireless Solutions Co., Ltd.",
    "24.5986": "Solar Technology, Inc.",
    "20.171977820078080": "Ningbo Shen Link Communication Technology Co., Ltd",
    "24.10534": "Applied Optoelectronics, Inc Taiwan Branch",
    "20.13692005515264": "Deltapath, Inc.",
    "24.6093926": "Cisco Systems, Inc",
    "12.346855530496": "Golden River Traffic",
    "24.10019958": "Zentan",
    "20.154238424449024": "IntelliVIX Co. Ltd.",
    "24.10532005": "Samsung Electronics Co.,Ltd",
    "12.123917675597824": "PROFITT Ltd",
    "24.1086054": "Samsung Electronics Co.,Ltd",
    "24.3461730": "Sz Dji Technology Co.,Ltd",
    "24.7401608": "Nintendo Co.,Ltd",
    "24.2908809": "Regenersis (Glenrothes) Ltd",
    "24.5550369": "Huawei Technologies Co.,Ltd",
    "24.7401607": "Apple, Inc.",
    "24.14724299": "Apple, Inc.",
    "24.32801": "Alcatel Canada Inc.",
    "24.32800": "Network Products",
    "24.32803": "Integrated Business Networks",
    "24.32802": "Scan-Optics",
    "24.32805": "Telit Wireless Solutions GmbH",
    "24.32804": "Kalpana, Inc.",
    "24.32807": "Adaptive Systems, Inc.",
    "24.32806": "Network Products Corporation",
    "24.32809": "Eagle Technology, Inc.",
    "24.32808": "Tradpost (Hk) Ltd",
    "12.346856652800": "Comtech Systems Inc.",
    "12.123917687693312": "Xylon",
    "12.123917676646400": "Algodue Elettronica Srl",
    "12.123917680738304": "Image Tech Co.,Ltd",
    "12.123917684858880": "dA Tomato Limited",
    "12.123917685805056": "Lansitec Technology Co., Ltd",
    "24.6318404": "Apple, Inc.",
    "24.14996394": "Fujitsu Connected Technologies Limited",
    "20.62366878269440": "Secad Sa",
    "24.4054": "Sarotech Co., Ltd",
    "24.8407409": "Apple, Inc.",
    "24.13681043": "Skybell, Inc",
    "24.15251272": "Cisco Systems, Inc",
    "24.12630246": "Ruijie Networks Co.,LTD",
    "12.123917681188864": "Mechatronics Systems Private Limited",
    "12.346864320512": "Southwest Research Institute",
    "12.123917690167296": "Dorsett Technologies, Inc.",
    "24.11330376": "MadgeTech, Inc",
    "24.11330370": "Huawei Technologies Co.,Ltd",
    "12.346863697920": "Saia-Burgess Controls AG",
    "12.346853990400": "Sedia Electronics",
    "24.13385311": "Hewlett Packard",
    "24.10510480": "Samsung Electronics Co.,Ltd",
    "12.346859941888": "Putercom Enterprise Co., LTD.",
    "24.10508259": "Huawei Technologies Co.,Ltd",
    "12.123917686407168": "AnyComm.Co.,Ltd.",
    "24.12867072": "Galleon Embedded Computing",
    "24.827025": "Sankosha Corporation",
    "24.6850215": "Cisco Systems, Inc",
    "20.39986089951232": "Only",
    "24.11535764": "Liteon Technology Corporation",
    "24.12114382": "Samsung Electronics Co.,Ltd",
    "12.346867179520": "Famas System S.p.A.",
    "24.7644456": "Huawei Technologies Co.,Ltd",
    "24.13632810": "Arcadyan Corporation",
    "24.13656909": "Meiko Maschinenbau GmbH &amp; Co. KG",
    "12.123917682925568": "effectas GmbH",
    "12.123917690929152": "ADE Technology Inc.",
    "20.119143430881280": "Clinton Electronics Corporation",
    "24.16051103": "Ellu Company 2019 SL",
    "24.2884255": "3ALogics",
    "24.15745480": "IEEE Registration Authority",
    "24.10752846": "Luxul",
    "12.346868576256": "Exelis Inc.",
    "24.11044872": "Apple, Inc.",
    "20.48548671389696": "AudioNord Distribution A/S",
    "24.7911499": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.2910650": "RF Controls, LLC",
    "24.5568197": "Fujian Star-Net Communication Co.,Ltd",
    "24.12348449": "Apple, Inc.",
    "24.1874156": "Hewlett Packard Enterprise",
    "24.8190687": "Apple, Inc.",
    "24.11573050": "E-Fuel Corporation",
    "20.70559706644480": "Beijing Smarot Technology Co., Ltd.",
    "12.123917677068288": "Convergent Design",
    "24.14189670": "Extreme Networks, Inc.",
    "24.16012320": "Tellescom Industria E Comercio Em Telecomunicacao",
    "24.4237017": "Apple, Inc.",
    "24.10795739": "Dell Inc.",
    "24.16309370": "Variscite LTD",
    "24.9724786": "vivo Mobile Communication Co., Ltd.",
    "24.16015316": "Cisco SPVTG",
    "12.123917688573952": "LightDec GmbH & Co. KG",
    "24.13684193": "Scientech Electronics",
    "20.93065827057664": "Do Easy International Limited",
    "24.6577708": "IRay Technology Co., Ltd.",
    "24.3733706": "OWIN Inc.",
    "24.14181083": "Eno Inc.",
    "24.2383550": "QNAP Systems, Inc.",
    "24.7395456": "Upos System sp. z o.o.",
    "24.2890998": "Alien Green LLC",
    "24.11594973": "Huawei Technologies Co.,Ltd",
    "24.11594977": "Private",
    "24.6090361": "Global Digitech Co LTD",
    "12.346861203456": "Datalink Technologies Gateways Inc.",
    "24.7092293": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6055798": "IDS Imaging Development Systems GmbH",
    "24.9887": "Private",
    "24.41018": "Kubotek Corporation",
    "24.41019": "Toshin Electric Co., Ltd.",
    "24.41014": "Applied Network Technology",
    "24.41015": "Mindray DS USA, Inc.",
    "24.807415": "Chauvin Arnoux",
    "24.41017": "Ross Technology, Inc.",
    "24.41010": "Ges Singapore Pte. Ltd.",
    "24.41011": "imc MeBsysteme GmbH",
    "24.41012": "Axel",
    "24.41013": "Cylink Corporation",
    "24.7385701": "Hiltron s.r.l.",
    "24.13390937": "OnTime Networks AS",
    "24.3457273": "Rockwell Automation",
    "24.45922": "Apple, Inc.",
    "24.7907443": "Samsung Electronics Co.,Ltd",
    "12.346856587264": "Telecom Protection Technologies Limited",
    "20.220092038119424": "Smart Wires Inc",
    "20.220327877541888": "Taylor Dynamometer",
    "12.71297884205056": "InfoMac Sp. z o. o. Sp. k.",
    "24.16272063": "Huawei Technologies Co.,Ltd",
    "24.16268479": "Botato Electronics Sdn Bhd",
    "20.228844760989696": "Schleifenbauer Holding BV",
    "24.9216072": "Beijing NeTopChip Technology Co.,LTD",
    "12.346865356800": "IPSES S.r.l.",
    "20.181338940375040": "Shanghai Mission Information Technologies (Group) Co.,Ltd",
    "12.346865901568": "Gillam-Fei S.A.",
    "12.346859085824": "StoreTech Limited",
    "20.70443736236032": "Nibble",
    "12.123917688885248": "Netzikon GmbH",
    "24.13945327": "Hewlett Packard",
    "24.839163": "EDIC Systems Inc.",
    "20.66219388436480": "HomeWizard B.V.",
    "12.346868019200": "Verified Energy, LLC.",
    "24.11313174": "Cisco Systems, Inc",
    "24.4223221": "Apple, Inc.",
    "20.27478325198848": "StreamLocator",
    "12.346864689152": "Saia-Burgess Controls AG",
    "24.14704569": "Cisco Systems, Inc",
    "24.13161379": "D-Link International",
    "24.11309189": "Apple, Inc.",
    "12.123917685956608": "Business Marketers Group, Inc.",
    "12.346860539904": "Adeneo",
    "20.96896779550720": "Mars Digi Tech Co.,Ltd",
    "24.53493": "Orange Micro, Inc.",
    "24.53492": "Carinthian Tech Institute",
    "24.9476727": "Texas Instruments",
    "24.41090": "Nkt Elektronik A/S",
    "24.13911777": "Apple, Inc.",
    "24.53491": "Solari Di Udine Spa",
    "24.1630856": "STT Condigi",
    "24.53490": "Monterey Networks",
    "20.198237820551168": "Chunghsin International Electronics Co.,Ltd.",
    "12.346863927296": "Grenmore Ltd",
    "24.7355046": "New H3C Technologies Co., Ltd",
    "24.53497": "Acute Communications Corp.",
    "24.41093": "Private",
    "24.53496": "Fujian Star Terminal",
    "20.127658790748160": "ITS Partner (O.B.S) S.L.",
    "24.53495": "Next Nets Corporation",
    "12.346860310528": "Venture Research Inc.",
    "24.53494": "Nokia",
    "24.14174446": "Hangzhou Xueji Technology Co., Ltd.",
    "12.346855149568": "Private",
    "24.1865718": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.45213": "Point Grey Research Inc.",
    "24.45210": "Morrow Technologies Corp.",
    "12.123917675937792": "Walter Müller AG",
    "24.9209374": "Apple, Inc.",
    "12.123917690462208": "Sierra Pacific Innovations Corp",
    "20.136646468042752": "EyeLock LLC",
    "24.7397689": "3view Ltd",
    "12.123917689393152": "PULOON Tech",
    "24.2389590": "Shanghai Net Miles Fiber Optics Technology Co., LTD.",
    "24.540253": "Sagemcom Broadband SAS",
    "12.346855882752": "Epsilon Srl",
    "24.7929": "Pascom Kommunikations systeme GmbH.",
    "24.7928": "Emfinity Inc.",
    "24.7925": "Hitek Automated Inc.",
    "24.7924": "L-3 Communications Display Systems",
    "24.7927": "Cisco Systems, Inc",
    "24.7926": "Cisco Systems, Inc",
    "24.7921": "Servimat",
    "24.7920": "Gigafin Networks",
    "24.7923": "From2",
    "12.346856456192": "Planea Oy",
    "24.4999503": "zte corporation",
    "24.6306016": "Belkin International Inc.",
    "24.10245338": "SkyBell Technologies Inc.",
    "24.6070648": "TableTop Media (dba Ziosk)",
    "12.346864328704": "Talo, NV Inc",
    "12.123917690421248": "Robotic Research, LLC",
    "12.123917686657024": "Safelet BV",
    "24.6070646": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.2131798": "Siemens Ag",
    "12.123917675384832": "KOMS Co.,Ltd.",
    "12.123917690716160": "Absolent AB",
    "20.62636948455424": "NZIA Connect Inc",
    "24.16638": "Symplex Communications",
    "24.16639": "Telebit Corporation",
    "12.346865491968": "Krontek Pty Ltd",
    "24.16630": "Katron Computers Inc.",
    "24.16631": "Polaroid Corporation",
    "24.16632": "Systemhaus Discom",
    "24.16633": "Combinet",
    "24.16634": "Microboards, Inc.",
    "24.16635": "Cascade Communications Corp.",
    "24.16636": "Ibr Computer Technik Gmbh",
    "24.16637": "Lxe",
    "24.8971816": "Shenzhen Kezhonglong Optoelectronic Technology Co.,Ltd",
    "24.8395262": "JianLing Technology CO., LTD",
    "24.16523932": "Han Kyung I Net Co.,Ltd.",
    "12.346867802112": "Positioneering Limited",
    "24.14680944": "ShenZhen Continental Wireless Technology Co., Ltd.",
    "24.4474503": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "24.9462001": "Apple, Inc.",
    "20.146101710618624": "Shenzhen Hcn.Electronics Co.,Ltd.",
    "24.9708769": "Alcatel-Lucent Enterprise",
    "24.584489": "Jiangsu Huitong Group Co.,Ltd.",
    "24.9197213": "Imaging Solutions Group",
    "24.4502546": "Sius Ag",
    "24.9495042": "Texas Instruments",
    "24.9456869": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917684207616": "Henschel-Robotics GmbH",
    "24.5389": "Hoana Medical, Inc.",
    "24.5388": "AVM GmbH",
    "24.1808": "Adax, Inc.",
    "24.1337867": "Cypress Electronics Limited",
    "24.1804": "SVA-Intrusion.com Co. Ltd.",
    "24.1805": "Cisco",
    "24.1806": "Cisco Systems, Inc",
    "24.5380": "Game Plus Co., Ltd.",
    "24.1800": "Bitrage Inc.",
    "24.1801": "Westerstrand Urfabrik AB",
    "24.1802": "Unicom Automation Co., Ltd.",
    "24.1803": "Novabase SGPS, SA",
    "24.7868279": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "12.119269363712": "Openpeak, Inc",
    "24.16318044": "Reciprocal Labs Corp",
    "24.14172844": "Shanghai Feixun Communication Co.,Ltd.",
    "24.300944": "Lartech telecom LLC",
    "24.281581": "Apple, Inc.",
    "24.5655": "Msi",
    "24.5654": "Browan Communication Inc.",
    "24.5657": "Lancelan Technologies S.L.",
    "24.5656": "HIVION Co., Ltd.",
    "24.5651": "LibreStream Technologies Inc.",
    "24.5650": "Otsuka Electronics Co., Ltd.",
    "24.5653": "Nittan Company, Limited",
    "24.5652": "Picosecond Pulse Labs",
    "24.7876335": "Nokia Corporation",
    "24.5659": "Micronet Corporation",
    "24.5658": "Dametric AB",
    "12.71297884602368": "Spectratech Inc.",
    "24.11025934": "HMD Global Oy",
    "24.16265051": "eGauge Systems LLC",
    "24.2104579": "Elatec GmbH",
    "24.4380": "Pleora Technologies Inc.",
    "24.4381": "Hectrix Limited",
    "24.4382": "ETHERNET Powerlink Standarization Group (EPSG)",
    "24.4383": "Doremi Labs, Inc.",
    "24.4384": "Cisco Systems, Inc",
    "24.4385": "Cisco Systems, Inc",
    "24.4386": "CIMSYS Inc",
    "24.4387": "Appointech, Inc.",
    "24.4388": "Apple, Inc.",
    "24.4389": "IBM Corp",
    "12.346856509440": "Pinnacle Technology",
    "20.27056771432448": "Innominds Software Inc",
    "12.346867187712": "Arcos Technologies LTD",
    "24.5010435": "Bq",
    "12.123917686648832": "Technology Link Corporation",
    "24.57484": "Neoparadigm Labs, Inc.",
    "24.57485": "Pressure Systems, Inc.",
    "24.57486": "Utstarcom",
    "24.57487": "Cisco Systems",
    "24.57480": "LTX-Credence CORPORATION",
    "24.57481": "ION Networks, Inc.",
    "24.57482": "Gec Avery, Ltd.",
    "24.57483": "QLogic Corporation",
    "20.5297544364032": "ShenZhen Huafu Information technology Co.?Ltd",
    "24.9709965": "Shanghai AdaptComm Technology Co., Ltd.",
    "24.57488": "Beckman Lab. Automation Div.",
    "24.57489": "LG Electronics",
    "12.346857730048": "Valtronic SA",
    "12.346865426432": "Vidisys GmbH",
    "24.263645": "Shenzhen Cultraview Digital Technology Co., Ltd",
    "24.7873711": "ASUSTek COMPUTER INC.",
    "24.819474": "Private",
    "24.9481719": "vivo Mobile Communication Co., Ltd.",
    "12.346863226880": "Frontier Electronic Systems Corp.",
    "24.6559373": "Cisco Systems, Inc",
    "24.56287": "Intel Corporate",
    "24.13647452": "Huawei Technologies Co.,Ltd",
    "12.123917675008000": "SOREDI touch systems GmbH",
    "24.5273349": "Samsung Electronics Co.,Ltd",
    "20.176772038524928": "Impulse Networks Pte Ltd",
    "20.225135236743168": "Dspread Technology (Beijing) Inc.",
    "24.9923": "Insightek Corp.",
    "24.9922": "SCDI Co. LTD",
    "24.9921": "Artray Co., Ltd.",
    "24.9920": "EnergyHub",
    "24.9927": "Intel Corporate",
    "24.9926": "Intel Corporate",
    "24.9925": "Guangdong Gosun Telecommunications Co.,Ltd",
    "24.9924": "Cadmos microsystems S.r.l.",
    "24.9929": "Proventix Systems, Inc.",
    "24.9928": "System Sensor",
    "24.7379877": "Shenzhen Y&D Electronics Co.,LTD.",
    "24.4524": "Simtec Electronics",
    "24.4525": "Shanghai Ruijie Technology",
    "24.4526": "ARRIS Group, Inc.",
    "24.4527": "Medialink-i,Inc",
    "24.4520": "Quest Technologies",
    "24.4521": "MOIMSTONE Co., LTD",
    "24.4522": "Uniclass Technology, Co., LTD",
    "24.4523": "Trustable Technology Co.,Ltd.",
    "24.4528": "Fortelink Inc.",
    "24.4529": "BlueExpert Technology Corp.",
    "24.11801875": "Huawei Technologies Co.,Ltd",
    "24.3154267": "Shenzhen Ostar Display Electronic Co.,Ltd",
    "24.11584005": "Bionime",
    "12.123917682753536": "SCHMID electronic",
    "24.12623841": "zte corporation",
    "20.181338941423616": "Dongguan Mingji Electronics technology Group Co., Ltd.",
    "12.123917690220544": "Talleres de Escoriaza SA",
    "24.15501813": "Lenovo Mobile Communication Technology Ltd.",
    "24.6575459": "Intelight Inc.",
    "24.20594": "Corvis Corporation",
    "24.20595": "Cisco Systems, Inc",
    "24.20596": "Advanced Hi-Tech Corp.",
    "24.20597": "Kestrel Solutions",
    "24.20590": "Corder Engineering Corporation",
    "24.20591": "G-Connect",
    "24.20592": "Chaintech Computer Co., Ltd.",
    "24.20593": "Aiwa Co., Ltd.",
    "24.5531622": "Shenzhen Mtc Co Ltd",
    "24.16018066": "Shenzhen Fast Technologies Co.,Ltd",
    "24.20598": "IBM Corp",
    "24.20599": "Prolific Technology, Inc.",
    "12.346868461568": "comlet Verteilte Systeme GmbH",
    "24.5398": "Uriel Systems Inc.",
    "12.123917677199360": "Gremesh.com",
    "12.123917685026816": "Spaceflight Industries",
    "24.1838": "North Node AB",
    "12.123917688401920": "SOtM",
    "12.346852544512": "Ignitus Communications, LLC",
    "24.14454920": "Huawei Technologies Co.,Ltd",
    "24.16517024": "LRE Medical GmbH",
    "24.11699": "AMPAK Technology,Inc.",
    "24.10770434": "Shin-Il Tech",
    "12.119269957632": "SKODA ELECTRIC a.s.",
    "24.16553013": "Huawei Technologies Co.,Ltd",
    "24.10802552": "Texas Instruments",
    "12.346859225088": "Paraytec Ltd",
    "24.15505025": "2276427 Ontario Inc",
    "12.346856325120": "Superna Ltd",
    "24.49249": "Solectek Corporation",
    "24.49248": "ID Scandinavia A/S",
    "24.20628": "ARRIS Group, Inc.",
    "24.20629": "Peracom Networks",
    "24.49241": "Denso Corporation",
    "24.49240": "Dataexpert Corp.",
    "24.49243": "Networks Northwest, Inc.",
    "24.20621": "Abit Computer Corporation",
    "24.49245": "L&N Technologies",
    "24.49244": "Elonex Plc",
    "24.49247": "Fine-Pal Company Limited",
    "24.49246": "Vari-Lite, Inc.",
    "24.5390": "Openbrain Technologies Co., Ltd.",
    "12.346857504768": "Christ Elektronik GmbH",
    "12.123917681537024": "Lazer Safe Pty Ltd",
    "24.1830": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.1833": "Kistler Instrumente AG",
    "12.346864271360": "Keith & Koep GmbH",
    "24.1832": "Neo Telecom",
    "24.8432292": "Usag",
    "24.1835": "Jung Myung Telecom Co., Ltd.",
    "20.62636957892608": "Distech Controls",
    "24.7345823": "EtherCAT Technology Group",
    "24.1834": "Innovance Networks",
    "20.4884646592512": "Observables, Inc.",
    "12.123917687767040": "Dromont S.p.A.",
    "12.123917682741248": "Critical Link LLC",
    "24.1837": "CNSystems",
    "24.4582": "Scientific Atlanta",
    "24.1836": "Fabricom",
    "12.346860732416": "Vitel Net",
    "12.123917679452160": "Peloton Technology",
    "24.8038": "Planar Llc",
    "20.220327869153280": "Grinbi Partners",
    "24.11275547": "LG Electronics (Mobile Communications)",
    "24.11041593": "Nokia Corporation",
    "24.8036": "Beijing Autelan Technology Inc.",
    "12.123917681168384": "Arevita",
    "24.14213124": "Vodia Networks Inc",
    "24.530508": "A. S. Thomas, Inc.",
    "24.8432918": "Xiaomi Communications Co Ltd",
    "24.2166": "SDSystem",
    "12.346855723008": "Vpg",
    "24.2164": "Dell Inc.",
    "24.2165": "Acorp Electronics Corp.",
    "24.2162": "Sorenson Communications",
    "24.2163": "DapTechnology B.V.",
    "24.2160": "Rasvia Systems, Inc.",
    "24.2161": "NORTHDATA Co., Ltd.",
    "24.5009677": "Cisco Systems, Inc",
    "24.5009676": "Cisco Systems, Inc",
    "24.2168": "Benchmark Storage Innovations",
    "24.2169": "CEM Corporation",
    "12.123917684396032": "Stmovic",
    "20.71389213097984": "Siebert Industrieelektronik GmbH",
    "24.7885068": "LiveU Ltd.",
    "24.6588296": "Juniper Networks",
    "12.123917689667584": "Electronic Systems Design Sprl",
    "20.246679830790144": "Shenzhen Rongan Networks Technology Co.,Ltd",
    "12.123917676662784": "Ci4Rail",
    "20.277981939367936": "Silicon (Shenzhen) Electronic Technology Co.,Ltd.",
    "24.9973078": "Resolution Audio",
    "24.8922412": "Arcadyan Technology Corporation",
    "24.2290": "C&S Technology",
    "24.2291": "Wany",
    "24.2292": "Bluetake Technology Co., Ltd.",
    "24.2293": "YESTECHNOLOGY Co.,Ltd.",
    "24.2294": "Sumitomo Electric Industries, Ltd",
    "24.2295": "Hitachi Ltd, Semiconductor & Integrated Circuits Gr",
    "24.2296": "Utc Ccs",
    "24.2297": "Artesyn Embedded Technologies",
    "24.2298": "KEB Automation KG",
    "24.2299": "SonoSite, Inc.",
    "20.163603961348096": "Hangzhou Yongkong Technology Co., Ltd.",
    "12.123917677805568": "New Imaging Technologies",
    "24.5535626": "Hewlett Packard Enterprise",
    "24.9181887": "Samsung Electronics Co.,Ltd",
    "12.346868637696": "Argus-Spectrum",
    "12.123917690683392": "Orange Tree Technologies Ltd",
    "24.62649": "Apple, Inc.",
    "24.13430236": "MitraStar Technology Corp.",
    "12.123917686558720": "E-Plus Technology Co., Ltd",
    "20.97755446902784": "Annapurna labs",
    "24.6340277": "Motorola Mobility LLC, a Lenovo Company",
    "24.12862956": "Shanghai Yali Electron Co.,LTD",
    "24.5029029": "Samsung Electronics Co.,Ltd",
    "24.13162361": "Qing Dao Haier Telecom Co.,Ltd.",
    "24.13162360": "BSH Hausgeraete GmbH",
    "24.8157779": "Phytrex Technology Corp.",
    "24.3953623": "Cedes Ag",
    "24.8175496": "Mobilicom LTD",
    "12.123917684936704": "eSMART Technologies SA",
    "12.346859339776": "BL Healthcare, Inc.",
    "12.123917680115712": "Microvideo",
    "24.565": "Paragon Networks International",
    "24.24627": "Acuity Imaging, Inc.",
    "24.567": "Cosmo Research Corp.",
    "24.566": "INIT GmbH",
    "24.561": "Ingersoll-Rand",
    "24.24623": "Cisco Systems, Inc",
    "24.24620": "LINX Data Terminals, Inc.",
    "24.24621": "Alerton Technologies, Inc.",
    "24.10540960": "Huawei Device Co., Ltd.",
    "24.569": "Visicom",
    "24.568": "Serome Technology, Inc.",
    "24.24628": "ROBERT BOSCH GmbH",
    "24.24629": "Dallas Semiconductor, Inc.",
    "24.7085547": "u-blox AG",
    "24.13109016": "TDSi",
    "20.31372484804608": "Beijing Raycores Technology Co.,Ltd",
    "24.4499522": "Dell Inc.",
    "24.11828380": "Amazon Technologies Inc.",
    "24.7865571": "Huawei Device Co., Ltd.",
    "24.12852380": "JiQiDao",
    "24.2834": "NURI Telecom Co., Ltd.",
    "24.2835": "Zetron Inc",
    "24.2836": "ViewSonic Corporation",
    "12.119269257216": "Energy Aware Technology",
    "24.2830": "Trapeze Networks",
    "12.346854457344": "AutoTOOLS group Co. Ltd.",
    "24.2832": "11wave Technonlogy Co.,Ltd",
    "24.2833": "Himeji Abc Trading Co.,Ltd.",
    "20.48688641605632": "Newtrax Technologies Inc",
    "24.15228294": "Chang Yow Technologies International Co.,Ltd.",
    "24.2838": "Communication Machinery Corporation",
    "24.2839": "MKS Instruments",
    "24.9112": "Vutlan sro",
    "24.554645": "Auto Industrial Co., Ltd.",
    "24.9110": "Andes Technology Corporation",
    "24.9111": "Westell Technologies Inc.",
    "24.9116": "Juniper Networks",
    "24.9117": "Mapower Electronics Co., Ltd",
    "24.9114": "EasyData Hardware GmbH",
    "24.9115": "Elster Solutions, LLC",
    "24.9118": "Jiangsu Lemote Technology Corporation Limited",
    "24.9119": "Institut für Prüftechnik",
    "24.29852": "Ruijie Networks Co.,LTD",
    "24.13111411": "Century Opticomm Co.,Ltd",
    "24.9227854": "Espressif Inc.",
    "24.5793885": "Tempo Australia Pty Ltd",
    "24.10502719": "Tyan Computer Corp",
    "24.11854315": "Private",
    "20.128612184358912": "Avantree Corporation",
    "24.16288060": "Inventec Appliances Corp.",
    "24.7652923": "AzureWave Technology Inc.",
    "24.2924127": "ARRIS Group, Inc.",
    "24.2643649": "Harman/Becker Automotive Systems GmbH",
    "24.12067047": "Guangzhou Horizontal Information & Network Integration Co. Ltd",
    "12.346859364352": "Laser Tools & Technics Corp.",
    "24.7622734": "New H3C Technologies Co., Ltd",
    "24.1844047": "EDMI Europe Ltd",
    "24.2634548": "Huawei Device Co., Ltd.",
    "12.123917691486208": "Hyannis Port Research",
    "24.10283157": "New H3C Technologies Co., Ltd",
    "24.16524607": "Apple, Inc.",
    "24.4722787": "NTT Innovation Institute, Inc.",
    "24.4720171": "Private",
    "12.71297883373568": "Tecnologias Plexus",
    "12.346852741120": "Kylink Communications Corp.",
    "20.207357805068288": "Shanghai Laisi Information Technology Co.,Ltd",
    "24.9196874": "Intel Corporate",
    "12.123917677756416": "Airthings",
    "24.2897656": "Cisco Systems, Inc",
    "24.15792103": "Huawei Technologies Co.,Ltd",
    "24.5792598": "Fn-Link Technology Limited",
    "24.9176250": "Dell Inc.",
    "24.8241": "Tattile SRL",
    "24.8240": "Analog & Digital Systems",
    "24.8243": "Synapse Technologies, Inc.",
    "24.8242": "Alcatel Taisel",
    "24.8245": "IBM (International Business Machines)",
    "24.8244": "Rotec Industrieautomation Gmbh",
    "24.8247": "Seagate Technology",
    "24.8246": "Bmc Software",
    "24.8249": "Scinets",
    "24.8248": "Vme Microsystems International Corporation",
    "20.84372901330944": "Elink Technology (Shenzhen) Co., Limited",
    "24.4508066": "Greenwald Industries",
    "12.346864082944": "Palgiken Co.,Ltd.",
    "24.3717355": "IEEE Registration Authority",
    "20.17622838018048": "RippleTek Tech Ltd",
    "20.13692012855296": "Beijing L&S Lancom Platform Tech. Co., Ltd.",
    "24.3950353": "Huawei Technologies Co.,Ltd",
    "12.346854563840": "CDS Rail",
    "24.12374483": "Shenzhen TINNO Mobile Technology Corp.",
    "24.13141794": "Lumenpulse",
    "24.36904": "Nippon Signal Co., Ltd.",
    "24.36905": "Crypto Ag",
    "24.36906": "Communication Devices, Inc.",
    "24.36907": "Cisco",
    "24.36900": "Pipelinks, Inc.",
    "24.36901": "BAE Systems Australia (Electronic Systems) Pty Ltd",
    "24.36902": "Advanced Switching Communications, Inc.",
    "24.36903": "Intel Corporation",
    "24.36908": "Data & Control Equipment Ltd.",
    "24.36909": "Data Electronics (Aust.) Pty, Ltd.",
    "12.346868432896": "Dynon Instruments",
    "20.44213341257728": "Havells India Limited",
    "24.16007823": "Samsung Electronics Co.,Ltd",
    "24.7097681": "Shenzhen Ceres Technology Co., Ltd.",
    "12.123917682544640": "Phoniro Systems AB",
    "24.7890235": "Shaoxing Ourten Electronics Co., Ltd.",
    "24.8928284": "Mercury Corporation",
    "20.97843031310336": "Beseye Cloud Security Co. Ltd.",
    "20.74781684662272": "BIG Climatic Manufacture, Co. LTD, Zhongshan Branch",
    "12.346855321600": "CI Systems Ltd.",
    "12.123917675483136": "Mo-Sys Engineering Ltd",
    "20.111031363829760": "Shanghai Simcom Limited",
    "12.123917688233984": "Fukuda Sangyo Co., Ltd.",
    "24.7131568": "Wing Sing Electronics Hong Kong Limited",
    "12.346854952960": "Dr. Staiger, Mohilo + Co GmbH",
    "24.5768276": "Icomm Hk Limited",
    "24.12902817": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.9699947": "Optictimes Co.,Ltd",
    "24.6584612": "Huawei Device Co., Ltd.",
    "24.11307413": "AzureWave Technology Inc.",
    "24.2891555": "Apple, Inc.",
    "24.2899768": "Cisco Systems, Inc",
    "20.194284250726400": "Beijing Geekplus Technology Co.,Ltd.",
    "24.12584643": "TP-Link Corporation Limited",
    "12.123917679951872": "International Roll-Call Corporation",
    "24.12361700": "Aruba, a Hewlett Packard Enterprise Company",
    "24.567232": "Technicolor CH USA Inc.",
    "24.6843939": "Wi3 Inc.",
    "24.11331967": "IoT Tech Limited",
    "24.4465276": "Huawei Technologies Co.,Ltd",
    "20.57176929337344": "Keruyun Technoligies(Beijing) Corporation Limited",
    "24.5520695": "Ruckus Wireless",
    "24.3426028": "Huawei Device Co., Ltd.",
    "24.44501": "Huawei Device Co., Ltd.",
    "24.8180163": "Apple, Inc.",
    "24.1876207": "IEEE Registration Authority",
    "24.3671366": "Foctek Photonics, Inc.",
    "20.211776051347456": "Infiot Inc.",
    "24.14466960": "Huizhou Qiaoxing Telecommunication Industry Co.,Ltd.",
    "24.14990667": "zte corporation",
    "12.123917677154304": "Engage Technologies",
    "20.124214332882944": "Dr. Simon Consulting GmbH",
    "24.10234499": "Samsung Electronics Co.,Ltd",
    "24.5812277": "Apple, Inc.",
    "24.7615062": "NT-ware Systemprg GmbH",
    "20.154055554891776": "China Potevio Co., Ltd",
    "12.119269814272": "Unilever R&amp;D",
    "12.346852257792": "Hunter Technology Inc.",
    "24.17693": "Cisco Systems, Inc",
    "24.6579274": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.3716901": "Wistron Infocomm (Zhongshan) Corporation",
    "24.4246906": "Qingdao Hisense Communications Co.,Ltd.",
    "20.52819403997184": "Technology for Humankind",
    "24.1607488": "Xiaomi Communications Co Ltd",
    "12.346857525248": "Velux",
    "20.48835648815104": "Milbank Manufacturing Co.",
    "12.346855407616": "Milmega Ltd",
    "12.123917677625344": "Computech International",
    "20.272689488986112": "Innodisk",
    "24.8704835": "Calix Inc.",
    "24.4499173": "Thinking Technology Co.,Ltd",
    "20.167295457427456": "Guangzhou Wuzhou Technology Co, Ltd.",
    "24.8417813": "BSkyB Ltd",
    "24.15516418": "HUMAX Co., Ltd.",
    "24.13249374": "Suzhou Motor-comm Electronic Technology Co.,Ltd",
    "20.206931529564160": "NDSL, Inc.",
    "24.3184007": "Hunan Fn-Link Technology Limited",
    "12.346860924928": "Boreste",
    "24.16439688": "Momentum Data Systems",
    "12.123917686861824": "AvMap srlu",
    "20.250817667923968": "Zavod № 423",
    "20.132777850699776": "Ombitron, Inc.",
    "24.2373068": "Zhongshan Scinan Internet of Things Co.,Ltd.",
    "24.2404521": "Shanghai Mobiletek Communication Ltd.",
    "24.4195340": "A&T",
    "20.211776055541760": "Alcatraz AI Inc.",
    "12.123917676310528": "Farmobile, LLC",
    "24.11560381": "Apple, Inc.",
    "24.1375919": "Sagittar Limited",
    "20.62663543488512": "New Telecom Solutions LLC",
    "12.123917682479104": "Zoe Medical",
    "24.15513968": "Ruijie Networks Co.,LTD",
    "12.123917683650560": "Lumine Lighting Solutions Oy",
    "24.16257017": "Google, Inc.",
    "24.3676754": "Seiko Epson Corporation",
    "12.119270051840": "UAB Kitron",
    "20.71414660988928": "Teleepoch Ltd",
    "24.7898893": "Unifiedgateways India Private Limited",
    "20.224420097425408": "Shenzhen Vanstor Technology Co.,Ltd",
    "24.5293710": "Biwin Semiconductor (HK) Company Limted",
    "12.123917677989888": "Zpas S.A.",
    "12.346868203520": "Enistic Limited",
    "24.1877633": "Huawei Technologies Co.,Ltd",
    "12.346861584384": "Symetrics Industries",
    "24.8395111": "Shanghai Reallytek Information Technology Co.,Ltd",
    "24.1861290": "Samsung Electronics Co.,Ltd",
    "12.123917684383744": "Rimota Limited",
    "20.233630187126784": "LS Communication Co.,Ltd.",
    "24.2892146": "Juniper Networks",
    "24.6409": "DEVI - Danfoss A/S",
    "12.123917678391296": "Seal Shield, LLC",
    "12.123917677264896": "Heitec Ag",
    "12.123917686956032": "Silent Gliss International Ltd",
    "20.242876961259520": "FLYHT Aerospace",
    "24.9715650": "Hewlett Packard Enterprise",
    "12.346862014464": "Emitech Corporation",
    "12.123917681836032": "LECO Corporation",
    "24.16297419": "HMD Global Oy",
    "24.834719": "Apple, Inc.",
    "20.84531195412480": "Plus One Japan Limited",
    "24.2366276": "Hangzhou Tuners Electronics Co., Ltd",
    "24.8418591": "BSkyB Ltd",
    "12.346861543424": "Schneider Electric GmbH",
    "12.123917689577472": "MB connect line GmbH Fernwartungssysteme",
    "12.123917685952512": "Samwell International Inc",
    "12.123917686890496": "Power Electronics Espana, S.L.",
    "24.13412289": "Google, Inc.",
    "24.8697858": "Cisco Systems, Inc",
    "24.13168245": "Pishion International Co. Ltd",
    "24.3947123": "Avaya Inc",
    "24.9712027": "Alphatronics BV",
    "12.71297883357184": "InventLab s.c.",
    "20.52912287907840": "CHISON Medical Technologies Co., Ltd.",
    "24.4470536": "MRV Comunications",
    "24.8972536": "Yong Tai Electronic (Dongguan) Ltd.",
    "12.346856153088": "Ward Leonard Electric Company",
    "24.40970": "Airspan",
    "24.40971": "Computex Co., Ltd.",
    "24.40972": "Kingmax Technology Inc.",
    "24.40973": "The Panda Project",
    "24.40974": "Netscout Systems Inc",
    "24.40975": "Broadband Technologies",
    "24.40976": "Syslogic Datentechnik Ag",
    "24.40977": "Mutoh Industries Ltd.",
    "24.40978": "Telco Systems, Inc.",
    "24.40979": "Teltrend Ltd.",
    "12.123917679079424": "Head",
    "12.346859061248": "Initel srl",
    "12.346860478464": "NH Research",
    "24.2371578": "Toshiba Global Commerce Solutions",
    "24.12623142": "Topicon HK Lmd.",
    "12.123917675257856": "Guan Show Technologe Co., Ltd.",
    "12.123917685911552": "Gstar Creation Co.,Ltd",
    "20.180577516912640": "Shanghai Xietong Technology Inc.",
    "24.8388718": "Apple, Inc.",
    "24.6823407": "Shenzhen CYX Technology Co., Ltd.",
    "24.7392608": "Apple, Inc.",
    "24.7893597": "Cisco Systems, Inc",
    "20.154055547551744": "Structura Technology & Innovation",
    "24.9815": "Ooo Npp Ekra",
    "24.3419": "Smart Empire Investments Limited",
    "24.3418": "Tiesse SpA",
    "24.3415": "Fujitsu I-Network Systems Limited.",
    "24.3414": "Dell Inc.",
    "24.3417": "Amity Systems, Inc.",
    "24.3416": "Private",
    "24.3411": "Beijing 5w Communication Corp.",
    "24.3410": "Comart system",
    "24.3413": "SANYCOM Technology Co.,Ltd",
    "24.3412": "3Com Ltd",
    "24.3981510": "d&b audiotechnik GmbH",
    "24.12634936": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.284463": "Sertel Electronics UK Ltd",
    "12.123917689470976": "Eutron SPA",
    "24.5775274": "Apple, Inc.",
    "24.2105137": "Inteno Broadband Technology AB",
    "24.7900723": "Jiangsu SEUIC Technology Co.,Ltd",
    "24.4520009": "Union Pacific Railroad",
    "24.9713376": "Sichuan Bihong Broadcast &amp; Television New Technologies Co.,Ltd",
    "20.176094518968320": "Firm INFORMTEST Ltd.",
    "12.123917686538240": "Horiba Abx Sas",
    "24.16051551": "Shenzhen Ktc Technology Group",
    "24.2940323": "Point Grey Research Inc.",
    "24.14980462": "IEEE Registration Authority",
    "12.346862505984": "Institut Dr. Foerster",
    "24.9501111": "Huawei Technologies Co.,Ltd",
    "24.20021": "Hewlett Packard Enterprise",
    "24.12881671": "Shenzhen Yihao Hulian Science and Technology Co., Ltd.",
    "24.577719": "Qingdao Hisense Communications Co.,Ltd.",
    "20.106196532985856": "(Un)Manned",
    "24.16266102": "Good Mind Innovation Co., Ltd.",
    "24.546560": "Tp-Link Technologies Co.,Ltd.",
    "12.346863575040": "Power Medical Interventions",
    "24.556089": "Amazon Technologies Inc.",
    "24.3436170": "Ecosense",
    "20.233258810867712": "Takumi Japan Ltd",
    "20.105645762150400": "EDA Technology Co.,LTD",
    "24.32965": "Novellco De Mexico",
    "12.123917689266176": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "12.346854899712": "2N TELEKOMUNIKACE a.s.",
    "12.123917675700224": "Wanco Inc",
    "20.22333142073344": "Annapurna labs",
    "24.3189144": "Huawei Device Co., Ltd.",
    "24.41209": "Bintec Communications Gmbh",
    "12.346858225664": "Georgia Tech Research Institute",
    "24.41205": "Radguard Ltd.",
    "24.41204": "Ge",
    "24.41207": "V.I Computer Corp.",
    "24.41206": "AutoGas Systems Inc.",
    "24.41201": "Mti",
    "24.41200": "Toronto Microelectronics Inc.",
    "24.41203": "Staubli",
    "24.41202": "Infotek Communications, Inc.",
    "12.346856869888": "WIT Inc",
    "24.9449095": "LabJack",
    "24.16554950": "Sumavision Technologies Co.,Ltd",
    "12.346864357376": "Transtechnik GmbH & Co.KG",
    "24.15741925": "Huawei Technologies Co.,Ltd",
    "24.34": "Visual Technology Inc.",
    "24.35": "ABB Automation AB, Dept. Q",
    "24.36": "Connect As",
    "24.37": "Ramtek Corp.",
    "24.30": "Telsist Industria Electronica",
    "24.31": "Telco Systems, Inc.",
    "24.32": "Dataindustrier Diab Ab",
    "24.33": "Sureman Comp. & Commun. Corp.",
    "12.71297884094464": "Richter",
    "24.11325824": "Crexendo Business Solutions, Inc.",
    "24.38": "Sha-Ken Co., Ltd.",
    "24.39": "Japan Radio Company",
    "24.7882": "Nortel Networks",
    "24.7883": "\"RPC \"Energoautomatika\" Ltd",
    "24.7880": "Rapid Mobile (Pty) Ltd",
    "24.299558": "ASUSTek COMPUTER INC.",
    "24.7886": "BISA Technologies (Hong Kong) Limited",
    "20.36069846286336": "Radarxense BV",
    "24.7884": "Cdvi",
    "24.7885": "KYLAND Technology Co. LTD",
    "24.7888": "Ingespace",
    "24.7889": "Keyprocessor B.V.",
    "24.6583000": "GooWi Technology Co.,Limited",
    "20.75247170617344": "Cte",
    "24.14158795": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "24.5268979": "Hewlett Packard",
    "24.13421697": "Huawei Technologies Co.,Ltd",
    "24.8699474": "Huawei Technologies Co.,Ltd",
    "24.13409792": "Nintendo Co., Ltd.",
    "24.10256607": "Arcadyan Technology Corporation",
    "24.9731769": "National Narrowband Network Communications Pty Ltd",
    "24.3705527": "ITC Networks",
    "24.1052440": "Fu Gang Electronic(KunShan)CO.,LTD",
    "12.346854715392": "Digital Recorders Inc",
    "24.7772": "RB GeneralEkonomik",
    "24.3723881": "Texas Instruments",
    "12.123917683027968": "Symbicon Ltd",
    "24.12893546": "SKF GmbH",
    "24.3727502": "Mitsubishi Electric Corporation",
    "24.8970483": "Juniper Networks",
    "12.123917689479168": "Stahl GmbH",
    "12.123917691293696": "Soehnle Industrial Solutions GmbH",
    "12.123917681475584": "OOO \"NTC Rotek\"",
    "24.14459494": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917688692736": "Beijing Aumiwalker technology CO.,LTD",
    "24.6604627": "Apple, Inc.",
    "24.2623117": "Itl, Llc",
    "12.346857148416": "Movaz Networks, Inc.",
    "12.123917686149120": "Shenzhen Weema TV Technology Co.,Ltd.",
    "24.7398766": "Texas Instruments",
    "24.6576632": "Vodafone Omnitel B.V.",
    "20.145735978844160": "Logic Supply",
    "24.2908172": "Quanta Computer Inc.",
    "24.2397250": "Open Road Solutions , Inc.",
    "24.7376059": "Mimodisplaykorea",
    "24.7376054": "Huawei Technologies Co.,Ltd",
    "20.132855983243264": "Jiangsu byzoro intelligent technology Co.,Ltd",
    "24.15261552": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.5308192": "Keenetic Limited",
    "24.10488511": "Wieson Technologies Co., Ltd.",
    "20.137141740896256": "Flying Loft Inc.",
    "20.145383541964800": "Shenzhen IP3 Century Intelligent Technology Co., Ltd",
    "24.7607669": "Xiaomi Communications Co Ltd",
    "24.12587819": "Sichuan Changhong Electric Ltd.",
    "24.8985": "Sielox LLC",
    "12.346856366080": "Numeron Sp. z o.o.",
    "24.1579147": "Fujitsu Connected Technologies Limited",
    "12.346866089984": "Watermark Estate Management Services, LLC",
    "24.1109": "ANTARA.net",
    "24.1108": "Quadriga UK",
    "24.1105": "Medrad, Inc.",
    "24.1104": "DMD Computers SRL",
    "24.1107": "YottaYotta, Inc.",
    "24.1106": "RocketLogix, Inc.",
    "24.1101": "Cisco Systems, Inc",
    "24.1100": "Jenoptik",
    "24.1103": "Schubert System Elektronik Gmbh",
    "24.1102": "Cisco Systems, Inc",
    "12.346865729536": "Inviso B.V.",
    "24.15483572": "ADC Corporation",
    "20.251330450948096": "Guang Lian Zhi Tong Technology Limited",
    "24.5272100": "Texas Instruments",
    "24.3467036": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.6829": "ARRIS Group, Inc.",
    "24.6828": "Corelatus AB",
    "24.9763751": "IEEE Registration Authority",
    "24.9603": "Cisco Systems, Inc",
    "24.6823": "Torian Wireless",
    "24.6822": "Telefunken Radio Communication Systems GmbH &CO.KG",
    "24.6821": "BRN Phoenix",
    "12.346860859392": "Task Sistemas De Computacao Ltda",
    "24.6827": "eWings s.r.l.",
    "24.6826": "Analogic Corp.",
    "24.6825": "Fujian Star-Net Communication Co.,Ltd",
    "24.6824": "Mamiya Digital Imaging Co., Ltd.",
    "20.119143421444096": "Unitel Engineering",
    "20.66129353506816": "Shenzhen Bestway Technology Co., Ltd",
    "12.123917681606656": "Acemis France",
    "24.8659265": "Shenzhen Ginwave Technologies Ltd.",
    "20.17622845358080": "nanoTech Co., Ltd.",
    "24.9703019": "Huawei Technologies Co.,Ltd",
    "24.1099516": "Garmin International",
    "12.123917684740096": "National Radio & Telecommunication Corporation - NRTC",
    "12.123917683908608": "Septentrio NV",
    "24.2408627": "Rsf",
    "24.16259627": "Google, Inc.",
    "24.158": "Marli S.A.",
    "24.159": "Ameristar Technologies Inc.",
    "24.154": "Rc Computer A/S",
    "24.155": "Information International, Inc",
    "24.156": "Rolm Mil-Spec Computers",
    "24.157": "Locus Computing Corporation",
    "24.150": "Marconi Electronics Ltd.",
    "24.151": "Dell EMC",
    "24.152": "Crosscomm Corporation",
    "24.153": "Mtx, Inc.",
    "20.74781682565120": "symplr",
    "12.123917685186560": "Intercomp S.p.A.",
    "12.123917675532288": "IST Technologies (SHENZHEN) Limited",
    "20.67047991017472": "Corelink Technology Co.,Ltd",
    "12.119269617664": "Klingenthaler Musikelektronik GmbH",
    "24.12636915": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "20.176032228311040": "ARIMA Communications Corp.",
    "20.39919503278080": "Church & Dwight Co., Inc.",
    "24.3418602": "Mcd Electronics Sp. Z O.O.",
    "20.106196523548672": "Ponoor Experiments Inc.",
    "24.4225141": "IMBEL - Industria de Material Belico do Brasil",
    "24.5271450": "Wi-Fi Alliance",
    "12.71297883832320": "Quantronix, Inc.",
    "24.16042834": "Huawei Technologies Co.,Ltd",
    "24.14981329": "Microsoft Mobile Oy",
    "24.9762405": "Ruckus Wireless",
    "12.71297883656192": "Teraflops",
    "12.71297884049408": "Shimizu Electric Co., Ltd.",
    "24.8656934": "Osram GmbH",
    "12.346859208704": "Epec Oy",
    "24.12100471": "Cisco Systems, Inc",
    "24.1360383": "ShenZhen QianHai Comlan communication Co.,LTD",
    "24.3994": "Synchrony, Inc.",
    "24.3995": "Ross Video Limited",
    "24.3996": "Panduit Corp",
    "24.3997": "DisplayLink (UK) Ltd",
    "24.3990": "Telco Systems, Inc.",
    "24.3991": "Avanex Corporation",
    "24.3992": "Avamax Co. Ltd.",
    "24.3993": "APAC opto Electronics Inc.",
    "24.3998": "Murrelektronik GmbH",
    "24.3999": "ARRIS Group, Inc.",
    "20.154238430740480": "Annapurna labs",
    "24.3962995": "Airsonics",
    "24.12071080": "Road-Track Telematics Development",
    "24.16019278": "Echowell",
    "24.3966632": "Sangshin elecom.co,, LTD",
    "24.5805163": "Intel Corporate",
    "20.211670371663872": "Viper Design, LLC",
    "24.4736135": "Cisco SPVTG",
    "24.7662770": "Amazon Technologies Inc.",
    "24.13941704": "vivo Mobile Communication Co., Ltd.",
    "24.7137395": "Tp-Link Technologies Co.,Ltd.",
    "24.7137396": "Huawei Technologies Co.,Ltd",
    "12.123917675442176": "U-Tech",
    "20.88668807102464": "Beijing YangLian Networks Technology co., LTD",
    "12.346862252032": "Sietron Elektronik",
    "12.123917677289472": "Private",
    "24.848": "Bticino Spa",
    "24.849": "Diebold, Inc.",
    "24.13637994": "Ciena Corporation",
    "24.15205392": "Private",
    "24.842": "RIAS Corporation",
    "24.843": "Nortel Networks",
    "24.840": "Norscan Instruments, Ltd.",
    "24.841": "Vidicode Datacommunicatie B.V.",
    "24.846": "Pos Data Company, Ltd.",
    "24.847": "Sur-Gard Security",
    "24.844": "Shanghai DigiVision Technology Co., Ltd.",
    "24.845": "Chiaro Networks, Ltd.",
    "24.14727589": "AzureWave Technology Inc.",
    "24.14212877": "beroNet GmbH",
    "24.9761182": "Huizhou Maorong Intelligent Technology Co.,Ltd",
    "24.14471765": "Kinpo Electronics, Inc.",
    "24.5658711": "Aculab plc",
    "12.346865045504": "Elektronic Thoma GmbH",
    "24.1592077": "Ruckus Wireless",
    "20.14223588458496": "Private",
    "24.15528795": "Nokia Corporation",
    "20.181338938277888": "Sweam AB",
    "24.13911898": "ScreenBeam, Inc.",
    "24.15005531": "ARRIS Group, Inc.",
    "12.346861895680": "Electronia",
    "24.6292503": "Posbank Co.,Ltd",
    "24.5535125": "Tp-Link Technologies Co.,Ltd.",
    "24.15500140": "RM Tech Co., Ltd.",
    "24.5278223": "Shenzhen Fise Technology Holding Co.,Ltd.",
    "12.346859282432": "Dynamic Hearing Pty Ltd",
    "24.11301430": "Lexking Technology Co., Ltd.",
    "20.368744333312": "Victorsure Limited",
    "24.3409915": "Ericsson AB",
    "24.2885660": "Ovh",
    "24.2885667": "Sercomm France Sarl",
    "24.5270413": "Nutanix",
    "12.346856980480": "Quartet Technology, Inc.",
    "24.2637430": "Common Networks",
    "12.123917678387200": "AudioTEC LLC",
    "12.346862321664": "Ebru GmbH",
    "12.346854309888": "Communication and Telemechanical Systems Company Limited",
    "24.4492818": "DT Research, Inc.",
    "12.346858700800": "dPict Imaging, Inc.",
    "12.123917685661696": "Redwood Systems",
    "12.346860929024": "B S E",
    "24.49238": "Somelec",
    "12.346865995776": "UNGAVA Technologies Inc.",
    "24.49239": "Myco Electronics",
    "12.123917684486144": "Matrix Switch Corporation",
    "12.123917690200064": "Groupe Citypassenger Inc",
    "24.12109590": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.13157062": "Shanghai Airm2m Communication Technology Co., Ltd",
    "24.49234": "Burr-Brown",
    "12.346855841792": "Colorado vNet",
    "24.15480947": "Juniper Networks",
    "24.49235": "Aspect Software Inc.",
    "24.5000185": "IEEE Registration Authority",
    "24.49236": "Network Peripherals, Ltd.",
    "24.49237": "Modular Computing Technologies",
    "24.13125506": "Areson Technology Corp.",
    "12.346854547456": "Hardmeier",
    "24.49231": "Dell Inc.",
    "24.13143016": "Masimo Corporation",
    "24.49232": "Toyo Denki Seizo K.K.",
    "24.14997277": "Oraya Therapeutics",
    "24.60856": "KYOCERA Corporation",
    "24.6851009": "Huawei Technologies Co.,Ltd",
    "24.7699": "Cisco Systems, Inc",
    "24.7698": "Ecolab",
    "24.7697": "Elelux International Ltd",
    "24.7696": "Huawei Technologies Co.,Ltd",
    "24.7695": "Briot International",
    "24.7694": "Maxi View Holdings Limited",
    "24.7693": "Micran Ltd.",
    "24.7692": "Sherwood Information Partners, Inc.",
    "24.7691": "Hewlett Packard",
    "24.7690": "Syba Tech Limited",
    "24.8660109": "Zebra Technologies Inc",
    "12.123917680594944": "SoftLab-NSK",
    "24.14985962": "Liteon Technology Corporation",
    "24.322747": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3670342": "Shenzhen Bilian Electronic Co.，Ltd",
    "12.123917677424640": "Elbit Systems Bmd And Land Ew - Elisra Ltd",
    "24.14985964": "Tianjin Hualai Technology Co., Ltd",
    "20.272689502617600": "Private",
    "20.255191438852096": "clabsys",
    "12.119269933056": "AVnu Alliance",
    "12.346865651712": "K.C.C. Shokai Limited",
    "24.14193473": "General Electric Global Research",
    "24.11827623": "Samsung Electro-Mechanics(Thailand)",
    "12.346854653952": "Biotage",
    "20.168210325307392": "chiconypower",
    "24.13666961": "Technicolor CH USA Inc.",
    "24.575155": "Qtech Llc",
    "12.71297883492352": "MACHINEPERFORMANCE ApS",
    "24.2315": "MTL Instruments PLC",
    "24.2314": "SnedFar Technology Co., Ltd.",
    "24.2317": "Leader Electronics Corp.",
    "24.2316": "Mayekawa Mfg. Co. Ltd.",
    "24.2311": "Chrysalis Development",
    "24.2310": "Esteem Networks",
    "24.2313": "Telenor Connect A/S",
    "24.2312": "VTech Technology Corp.",
    "24.2319": "Fortinet Inc.",
    "24.2318": "Helix Technology Inc.",
    "24.5009006": "Comnect Technology CO.,LTD",
    "24.1664": "Card Access, Inc.",
    "24.1665": "Goepel Electronic GmbH",
    "12.346862133248": "Triple Ring Technologies, Inc.",
    "24.1667": "Bravara Communications, Inc.",
    "24.1660": "Cisco Systems, Inc",
    "12.346865192960": "Deansoft CO., Ltd.",
    "24.1608159": "CerebrEX Inc.",
    "24.1663": "Digeo, Inc.",
    "12.123917680541696": "Digiverv Inc",
    "24.1668": "Biacore AB",
    "24.1669": "NetNearU Corporation",
    "24.7405404": "Cheerzing Communication(Xiamen)Technology Co.,Ltd",
    "24.15494923": "FADU, Inc.",
    "24.10281230": "NCTech Ltd",
    "24.2155818": "Netgear",
    "20.57194443702272": "Eclipse Information Technologies",
    "24.8162705": "Shanghai Hongzhuo Information Technology co.,LTD",
    "24.24781": "VideoServer, Inc.",
    "24.24780": "Emtrak, Incorporated",
    "24.24783": "Alteon Networks, Inc.",
    "24.24782": "Acclaim Communications",
    "24.24785": "Cascade Communications",
    "24.24784": "Snmp Research Incorporated",
    "24.6128": "SZCOM Broadband Network Technology Co.,Ltd",
    "24.6129": "Renu Electronics Pvt Ltd",
    "24.6126": "ARRIS Group, Inc.",
    "24.24788": "Eldat Communication Ltd.",
    "24.6124": "Texas Instruments",
    "24.6125": "WooJooIT Ltd.",
    "24.6122": "Texas Instruments",
    "24.6123": "Texas Instruments",
    "24.6120": "Texas Instruments",
    "24.6121": "Texas Instruments",
    "12.123917679628288": "Gentec Systems Co.",
    "24.10245905": "Feitian Xunda(Beijing) Aeronautical Information Technology Co., Ltd.",
    "20.207357811359744": "ForoTel",
    "24.4241562": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.154314892902400": "Qbic Technology Co., Ltd",
    "24.13396766": "Cloud Network Technology Singapore Pte. Ltd.",
    "12.123917679599616": "EarTex",
    "20.171977807495168": "MOZI (Shenzhen) Artificial Intelligence Technology Co., Ltd.",
    "24.13932255": "Sungjin C&T Co.,Ltd",
    "24.12331222": "Rowley Associates Limited",
    "24.6317049": "Ciholas, Inc.",
    "24.2924591": "Veroguard Systems Pty Ltd",
    "20.255553905360896": "Limited Liability Company M.S.Korp",
    "12.346856411136": "Afonics Fibreoptics Ltd",
    "24.9097": "Hangzhou H3C Technologies Co., Limited",
    "24.9096": "V.T. Telematica S.p.a.",
    "24.9095": "ThinkFlood, Inc.",
    "24.9094": "IMI Hydronic Engineering international SA",
    "24.9093": "Antipode",
    "24.9092": "GGH Engineering s.r.l.",
    "24.9091": "InMage Systems Inc",
    "24.9090": "Lih Rong electronic Enterprise Co., Ltd.",
    "24.37060": "Javelin Systems, Inc.",
    "24.37061": "Internet Magic, Inc.",
    "24.37062": "Optim Systems, Inc.",
    "24.37063": "Icom Inc.",
    "24.37064": "Waverider Communications (Canada) Inc.",
    "24.37065": "DPAC Technologies",
    "24.9099": "Quanta Computer Inc.",
    "24.9098": "Ciena Corporation",
    "24.6038860": "Avire Global Pte Ltd",
    "20.154039765434368": "Shenzhen Cylan Technology Co.,Ltd",
    "12.123917684514816": "Vertel Digital Private Limited",
    "24.1632244": "Apple, Inc.",
    "24.3712438": "Shenzhen Megmeet Electrical Co.,Ltd",
    "12.346857914368": "éolane",
    "24.311407": "Huawei Technologies Co.,Ltd",
    "24.8390427": "Vsolution Telecommunication Technology Co.,Ltd.",
    "20.167295452184576": "Shenzhen Herofun Bio-Tech Co., Ltd",
    "24.524942": "Netgear",
    "24.9321": "Smart Doorphones",
    "24.9320": "Sumavision Technologies Co.,Ltd",
    "24.9323": "Covia, Inc.",
    "24.9322": "Solid Year Co., Ltd.",
    "24.1868377": "IEEE Registration Authority",
    "24.30385": "Somfy-Protect By Myfox SAS",
    "24.9327": "Onda Communication spa",
    "24.9326": "Phihong USA Corp.",
    "24.9329": "Fusion MultiSystems dba Fusion-io",
    "24.9328": "AUROTECH ultrasound AS.",
    "24.7093869": "Apple, Inc.",
    "24.6350524": "Sino-Telecom Technology Co.,Ltd.",
    "24.12900595": "iZotope",
    "24.34609": "Cisco Systems, Inc",
    "20.137141737750528": "Yongguan Electronic Technology (D.G)LTD",
    "24.11337107": "Weifang Goertek Electronics Co.,Ltd",
    "12.71297883729920": "Bestel China",
    "24.13681711": "Tamagawa Seiki Co.,Ltd.",
    "24.536159": "Hewlett Packard",
    "12.71297884729344": "Luceo",
    "24.13141328": "Apple, Inc.",
    "24.9192289": "Hailo Digital Hub GmbH & Co. KG",
    "24.14420788": "LG Electronics (Mobile Communications)",
    "24.825023": "Huawei Technologies Co.,Ltd",
    "24.11072624": "Fujian Newland Communication Science Technologies Co.,Ltd.",
    "20.251330451996672": "Elan Systems",
    "12.71297884946432": "Shin-ei Electronic Measuring Co.,Ltd.",
    "24.8559": "SymCom, Inc.",
    "24.1880643": "Tdc A/S",
    "12.123917683884032": "Package Guard, Inc",
    "24.524369": "ExperData",
    "24.524368": "Daisy Systems Corp.",
    "24.11552002": "Huawei Device Co., Ltd.",
    "24.13956371": "Texas Instruments",
    "24.524361": "Univation",
    "24.524360": "Eurotherm Gauging Systems",
    "24.524363": "Planning Research Corp.",
    "24.524362": "Banyan Systems Inc.",
    "24.524365": "Corvus Systems Inc.",
    "24.524364": "Hydra Computer Systems Inc.",
    "24.524367": "Cygnet Systems",
    "12.346868924416": "Spirent Communications",
    "20.224420110008320": "Private",
    "24.16283291": "Neopis Co., Ltd.",
    "12.123917683093504": "Lsb - La Salle Blanche",
    "24.3938064": "Sagemcom Broadband SAS",
    "24.14198068": "Spectronix Corporation",
    "24.14694115": "Stage One International Co., Ltd.",
    "24.16529764": "Itel Mobile Limited",
    "24.16449": "Fujikura Ltd.",
    "24.16448": "Ring Access, Inc.",
    "24.16447": "Ssangyong Computer Systems",
    "24.16446": "Raster Ops Corporation",
    "24.16445": "Teradata Corporation",
    "24.16444": "Forks, Inc.",
    "24.16443": "Synerjet International Corp.",
    "24.16442": "Impact Technologies",
    "24.3469648": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.16440": "Talent Electric Incorporated",
    "24.45065": "Grass Valley, A Belden Brand",
    "12.346864013312": "Troll Systems Corporation",
    "24.15746619": "Comcast Cable Corporation",
    "12.346860593152": "Phytec Messtechnik GmbH",
    "24.10504220": "Hewlett Packard",
    "24.7142055": "AMPAK Technology, Inc.",
    "24.4506793": "Opticom Communication, LLC",
    "24.11017494": "AzureWave Technology Inc.",
    "24.8435464": "Blue Danube Systems, Inc",
    "24.7606971": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917691236352": "KST technology",
    "24.3420559": "Apple, Inc.",
    "24.16295007": "Xiaomi Communications Co Ltd",
    "24.7103388": "ARRIS Group, Inc.",
    "12.123917675454464": "Global-King International Co., Ltd.",
    "20.167295459524608": "Kortek Corporation",
    "24.8154886": "Caterpillar Trimble Control Technologies",
    "20.154238419206144": "Shanghai Satellite Communication Technology Co.,Ltd",
    "20.250817672118272": "XPR Group",
    "24.8181602": "Freestyle Technology Pty Ltd",
    "24.16531419": "Hewlett Packard",
    "12.346868809728": "Azurtest",
    "24.10234047": "Continental Automotive Czech Republic s.r.o.",
    "20.58144481869824": "Rinco Ultrasonics AG",
    "24.10024919": "ARRIS Group, Inc.",
    "12.123917683273728": "RedLeaf Security",
    "20.57194437410816": "EFD Induction",
    "24.4493499": "Apple, Inc.",
    "24.8440395": "Shenzhen Gongjin Electronics Co.,Ltd",
    "24.10806456": "BlackBerry RTS",
    "24.6823268": "Sunwave Communications Co., Ltd",
    "12.346858668032": "Sohon Inc",
    "20.62335511166976": "Triple Jump Medical",
    "12.346865049600": "Herrick Technology Laboratories, Inc.",
    "20.167594819584000": "Intech",
    "24.6859708": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.2405086": "Espressif Inc.",
    "24.1070794": "Peplink International Ltd.",
    "12.123917675245568": "SSL - Electrical Aerospace Ground Equipment Section",
    "24.4214965": "Shenzhen New Species Technology Co., Ltd.",
    "24.11314286": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.346858246144": "RADA Electronics Industries Ltd.",
    "24.10527714": "Actiontec Electronics, Inc",
    "12.346865782784": "Airfolc,Inc.",
    "12.346856910848": "Eridon Corporation",
    "24.5560774": "Huawei Device Co., Ltd.",
    "24.313472": "Samtec Inc",
    "24.16027282": "Huawei Technologies Co.,Ltd",
    "24.2651664": "IgniteNet",
    "12.346862551040": "Hammock Corporation",
    "24.13672409": "IEEE Registration Authority",
    "24.9748254": "eflow Inc. / Smart Device Planning and Development Division",
    "12.346856656896": "BLEILE DATENTECHNIK GmbH",
    "24.12625408": "Apple, Inc.",
    "24.9735891": "Arista Networks",
    "12.123917676961792": "Dycec, S.A.",
    "24.12641184": "Renesas Electronics (Penang) Sdn. Bhd.",
    "20.247076407476224": "Yuneec International（China）Co.，Ltd",
    "24.7399491": "Beijing C&W Optical Communication Technology Co.,Ltd.",
    "20.180577507475456": "Circus World Displays Ltd",
    "12.123917690691584": "Chargelib",
    "24.10265968": "eero inc.",
    "20.229108131823616": "Shenzhen Canzone Technology Co.,Ltd.",
    "24.4983252": "Arcadyan Technology Corporation",
    "24.13128673": "Beijing Bytedance Network Technology Co., Ltd",
    "24.13390773": "Huawei Technologies Co.,Ltd",
    "12.346852327424": "The Frensch Corporation (Pty) Ltd.",
    "12.346864799744": "Fr. SauterAG",
    "12.346862497792": "Walter Grotkasten",
    "24.12907900": "Brocade Communications Systems LLC",
    "12.346853048320": "Charlotte's Web Networks",
    "24.6071434": "Cisco Systems, Inc",
    "12.346861649920": "LSS GmbH",
    "24.8143948": "Entertainment Solutions, S.L.",
    "12.346862510080": "Chauvin Arnoux",
    "24.11048924": "Ibm",
    "24.16555480": "Beijing TongTongYiLian Science and Technology Ltd.",
    "24.12124158": "Texas Instruments",
    "12.346862166016": "MEV Limited",
    "20.31368126922752": "EBS Sp. z o.o.",
    "12.346854641664": "Rdtech",
    "12.123917684273152": "Schneider Electric Motion USA",
    "24.3944191": "Apple, Inc.",
    "24.5569086": "Samsung Electronics Co.,Ltd",
    "24.2622168": "Samsung Electronics Co.,Ltd",
    "24.3710810": "C-Kur TV Inc.",
    "24.12862624": "Cisco Systems, Inc",
    "24.12363025": "Netgear",
    "20.48476185427968": "Hangzhou Yanzhi Technology Co.,Ltd.",
    "24.11552069": "YACOUB Automation GmbH",
    "24.14216006": "Google, Inc.",
    "24.1323877": "TEM Mobile Limited",
    "12.123917675507712": "wallbe GmbH",
    "24.7885581": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.13420759": "CIAS Elettronica srl",
    "24.45880": "Kontron Asia Pacific Design Sdn. Bhd",
    "24.7363757": "Konami Gaming Inc",
    "12.123917686312960": "Trinity Solutions LLC",
    "24.15750401": "Huawei Device Co., Ltd.",
    "24.6615618": "Gerdes Aktiengesellschaft",
    "12.346863202304": "Littlemore Scientific",
    "24.4248365": "Apple, Inc.",
    "24.47016": "Heinzinger electronic GmbH",
    "12.346861588480": "Sindoma Müh Mim Ýnþ Elk San Tic Ltd.",
    "12.123917684838400": "Task Sistemas De Computacao Ltda",
    "12.123917678800896": "CT Company",
    "24.1346160": "ADS GmbH",
    "24.4779": "WiLife, Inc.",
    "24.4778": "IEE, Inc.",
    "24.4775": "ISR TECHNOLOGIES Inc",
    "24.4774": "Dolby Australia",
    "24.4777": "3Com Ltd",
    "24.4776": "intec GmbH",
    "20.31368180400128": "Novetta",
    "24.4770": "Vita",
    "24.2402615": "PURE Storage",
    "24.4772": "ThingMagic, LLC",
    "24.11551005": "ITTIM Technologies",
    "12.346852220928": "TTTech",
    "24.3208": "Apache Micro Peripherals, Inc.",
    "24.3209": "AC Electric Vehicles, Ltd.",
    "24.3204": "Eazix, Inc.",
    "24.3205": "Cisco Systems, Inc",
    "24.3206": "Cisco Systems, Inc",
    "24.3207": "Amd",
    "24.3200": "Opelcomm Inc.",
    "24.3201": "Schneider Electric (Australia)",
    "24.3202": "Network Technologies Inc",
    "20.22126511783936": "POS Systema LLC",
    "20.238070234349568": "Yusan Industries Limited",
    "12.346863149056": "DetNet South Africa PTY (LTD)",
    "24.11536858": "Ring Solutions",
    "24.13384536": "Curiouser Products Inc",
    "24.3180609": "Sichuan AI-Link Technology Co., Ltd.",
    "24.584260": "Extreme Networks, Inc.",
    "12.346866057216": "demmel products",
    "20.5297554849792": "Evolute Systems Private Limited",
    "24.1683": "Flexus Computer Technology, Inc.",
    "12.346853527552": "wellink, Ltd.",
    "24.10505895": "Apple, Inc.",
    "24.7357468": "Shenzhen Kaifa Technology Co.,Ltd.",
    "24.15252945": "Huawei Technologies Co.,Ltd",
    "12.123917682745344": "Transformational Security, LLC",
    "24.571938": "Swaive Corporation",
    "24.14157873": "Samsung Electronics Co.,Ltd",
    "24.13379117": "Verifi, LLC",
    "24.3182763": "Apple, Inc.",
    "24.3718959": "Espec Corp.",
    "24.8188637": "Vence Corp",
    "24.1681": "PT Inovacao",
    "20.58071055335424": "Meatest sro",
    "20.14286630944768": "Celerway Communication AS",
    "12.346863550464": "Synthesechemie Dr. Penth GmbH",
    "24.1601595": "Huawei Device Co., Ltd.",
    "12.123917690605568": "allora Factory BVBA",
    "20.22333153607680": "Arkus-ST Ltd",
    "24.1860543": "Shenzhen Broadtel Telecom Co.,Ltd",
    "24.16047353": "Huawei Technologies Co.,Ltd",
    "24.6292295": "Billion Electric Co. Ltd.",
    "12.346867331072": "Fr. Sauter AG",
    "24.1362059": "Apple, Inc.",
    "24.47900": "Huawei Device Co., Ltd.",
    "20.58144493404160": "Annapurna labs",
    "20.79602625019904": "Data Technology Inc.",
    "24.10249721": "AcSiP Technology Corp.",
    "12.346866475008": "Manitowoc Ice",
    "24.6088608": "Nokia",
    "24.2134861": "Fujian Star-Net Communication Co.,Ltd",
    "12.346853240832": "NewHer Systems",
    "24.14205094": "AzureWave Technology Inc.",
    "20.251018409410560": "Doowon Electronics & Telecom Co.,Ltd",
    "24.577695": "Cisco Systems, Inc",
    "24.2144128": "Toshiba Visual Solutions Corporation Co.,Ltd",
    "20.272805459394560": "Teleofis",
    "12.346856804352": "Paul Scherrer Institut (PSI)",
    "24.8146813": "Sawwave",
    "12.71297883213824": "ClearSite Communications Inc.",
    "24.1684": "Mobillian Corporation",
    "24.738": "NDC Infared Engineering",
    "24.2648762": "Apple, Inc.",
    "24.2648760": "Apple, Inc.",
    "24.734": "Astrodesign, Inc.",
    "24.735": "Net Com Systems, Inc.",
    "24.736": "ETAS GmbH",
    "24.737": "Integrated Network Corporation",
    "24.730": "ExiO Communications, Inc.",
    "24.731": "Netsec",
    "24.732": "Fujitsu General Limited",
    "24.733": "Bromax Communications, Ltd.",
    "24.2654159": "Intel Corporate",
    "12.123917686370304": "Octopus IoT srl",
    "24.7459": "Sensus",
    "24.7458": "Foss Analytical A/S",
    "24.7455": "Siauliu Tauro Televizoriai, JSC",
    "24.7454": "Kyushu Ten Co.,Ltd",
    "24.7457": "Alcad SL",
    "24.7456": "Comtrend Corporation",
    "24.7451": "Sangean Electronics Inc.",
    "24.7450": "OvisLink S.A.",
    "24.7453": "Inter-M Corporation",
    "24.7452": "Gennet s.a.",
    "12.346863140864": "Apantac LLC",
    "24.5043547": "Huawei Technologies Co.,Ltd",
    "24.5547779": "Tp-Link Technologies Co.,Ltd.",
    "12.123917681582080": "Xirgo Technologies LLC",
    "24.11575562": "Pivotal Systems Corporation",
    "20.198237818454016": "Impakt S.A.",
    "12.123917677699072": "Boston Dynamics",
    "12.123917683949568": "Zetechtics Ltd",
    "12.346861129728": "L3 Communications Nova Engineering",
    "12.346857467904": "Polystar Instruments AB",
    "24.4213272": "Addrek Smart Solutions",
    "24.8953240": "Allied-telesisK.K.",
    "24.2900280": "Hewlett Packard",
    "24.41438": "ShenZhen ShiHua Technology CO.,LTD",
    "24.16580522": "IEEE Registration Authority",
    "20.49277266034688": "Private",
    "24.9731047": "Samsung Electronics Co.,Ltd",
    "24.14481177": "Cisco Systems, Inc",
    "20.71414655746048": "IntelliDesign Pty Ltd",
    "12.123917678432256": "AT-Automation Technology GmbH",
    "12.71297883533312": "Tammermatic Group Oy",
    "24.7397113": "Fai Wah International (Hong Kong) Limited",
    "24.6600134": "Nintendo Co.,Ltd",
    "24.4736753": "Uros Oy",
    "20.255861677096960": "Elcoma",
    "12.346865430528": "Thermo Fisher Scientific",
    "24.14952289": "Honor Device Co., Ltd.",
    "12.123917686198272": "Bartec GmbH",
    "12.346857545728": "Mark IV IDS Corp.",
    "24.9971997": "Jaguar de Mexico, SA de CV",
    "24.13933404": "Chongqing Miedu Technology Co., Ltd.",
    "24.5780539": "Glovast Technology Ltd.",
    "24.16019627": "vivo Mobile Communication Co., Ltd.",
    "24.6328": "New Voice International AG",
    "24.6329": "Cisco Systems, Inc",
    "24.6324": "Dawon Media Inc.",
    "24.6325": "Magna Carta",
    "24.6326": "S3C, Inc.",
    "24.6327": "D3 Led, Llc",
    "24.6320": "Nortel Networks",
    "24.6321": "IBM Corp",
    "24.6322": "Adeunis Rf",
    "24.6323": "TEC WizHome Co., Ltd.",
    "24.9695": "Taser International Inc.",
    "24.1318": "IPAS GmbH",
    "24.1319": "SJ Tek Co. Ltd",
    "24.14724702": "Alpsalpine Co,.Ltd",
    "24.9694": "Probits Co., LTD.",
    "24.1314": "LEA*D Corporation, Inc.",
    "24.1315": "AVL List GmbH",
    "24.1316": "BTL System (HK) Limited",
    "24.1317": "Puretek Industrial Co., Ltd.",
    "24.1310": "Brocade Communications Systems LLC",
    "24.9697": "Shanghai Seeyoo Electronic & Technology Co., Ltd",
    "24.1312": "Smartronix, Inc.",
    "24.1313": "Control Microsystems",
    "24.8438": "Net Tek & Karlnet Inc",
    "24.8439": "Cyberdata Corporation",
    "24.16037982": "Texas Instruments",
    "24.5804099": "Sagemcom Broadband SAS",
    "24.11828265": "Shenzhen Guzidi Technology Co.,Ltd",
    "24.6847688": "Measurement Systems S. de R.L.",
    "24.8432": "Universal Microelectronics Co.",
    "24.8433": "Altos India Limited",
    "24.8430": "Gtech Corporation",
    "24.8431": "Usc Corporation",
    "24.8436": "Spectrix Corporation",
    "24.8437": "Pandatel Ag",
    "24.8434": "Oracle Corporation",
    "24.8435": "Raynet Corporation",
    "24.9693": "Sunnytek Information Co., Ltd.",
    "12.346857803776": "Seitec Elektronik GmbH",
    "24.13371616": "Apple, Inc.",
    "24.3176": "SigmaTel, Inc.",
    "24.9692": "Sumitomo Electric Industries, Ltd",
    "20.52819399802880": "Bonraybio",
    "24.1852501": "Sieb & Meyer AG",
    "24.2936664": "Huawei Technologies Co.,Ltd",
    "12.123917682728960": "Cardinal Health",
    "24.4890": "Cisco Systems, Inc",
    "24.4891": "BeCell Innovations Corp.",
    "24.4892": "LiteTouch, Inc.",
    "24.4893": "Scanvaegt International A/S",
    "24.4894": "peiker acustic GmbH",
    "24.4895": "NxtPhase T&D, Corp.",
    "24.4896": "Intel Corporate",
    "24.4897": "Hewlett Packard",
    "24.4898": "DAQ Electronics, Inc.",
    "24.4899": "Cap Co., Ltd.",
    "12.346866692096": "Luceo",
    "12.346860793856": "O. Bay AG",
    "24.5794481": "Hangzhou H3C Technologies Co., Limited",
    "24.587512": "GET Engineering",
    "24.16255952": "Datang Telecom communication terminal (Tianjin) Co., Ltd.",
    "24.7877147": "D-Link International",
    "24.4728552": "Tek-Air Systems, Inc.",
    "24.14718982": "Espressif Inc.",
    "24.6355778": "Apple, Inc.",
    "24.13430531": "Hunan Keyshare Communication Technology Co., Ltd.",
    "24.11293166": "Cambridge Communication Systems Ltd",
    "12.123917689942016": "Bestcode Llc",
    "20.123598312308736": "An Phat Information Technology Co., Ltd",
    "12.346860044288": "Lewis Controls Inc.",
    "20.58144487112704": "Hubitat Inc.",
    "24.7924770": "Shanghai Qihui Telecom Technology Co., LTD",
    "12.123917688442880": "Cannex Technology Inc.",
    "24.11583094": "Tp-Link Technologies Co.,Ltd.",
    "24.14432862": "Roku, Inc.",
    "24.7100736": "Mercury Communication Technologies Co.,Ltd.",
    "12.346862968832": "Conduant Corporation",
    "12.346867773440": "Tecnint HTE SRL",
    "24.56591": "Ungermann-Bass Inc.",
    "24.56590": "Ungermann-Bass Inc.",
    "20.141720402526208": "Beijing Yuantel Technolgy Co.,Ltd-Shenzhen Branch",
    "24.316341": "Huawei Device Co., Ltd.",
    "24.293": "Yaesu Musen Co., Ltd.",
    "24.292": "Acer Incorporated",
    "24.291": "Schneider Electric Japan Holdings Ltd.",
    "24.290": "Trend Communications, Ltd.",
    "24.297": "DFI Inc.",
    "24.296": "EnjoyWeb, Inc.",
    "24.295": "OPEN Networks Pty Ltd",
    "24.294": "PAC Labs",
    "24.6881015": "Apple, Inc.",
    "24.299": "TELENET Co., Ltd.",
    "24.298": "Telematica Sistems Inteligente",
    "24.5518640": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5145": "Sidsa",
    "24.5144": "C4Line",
    "24.5147": "Cisco Systems, Inc",
    "24.5146": "Deicy Corporation",
    "24.5141": "Intec Automation inc.",
    "24.5140": "Jumpnode Systems LLC.",
    "24.5143": "RSE Informations Technologie GmbH",
    "24.5142": "Scosche Industries, Inc.",
    "24.14976043": "Hangzhou Softel Optic Co., Ltd",
    "12.123917681770496": "Fortress Cyber Security",
    "24.5149": "KEBA Industrial Automation Germany GmbH",
    "24.5148": "Cisco Systems, Inc",
    "24.4234": "TeraLogic, Inc.",
    "12.346865205248": "Wilke Technology GmbH",
    "24.4236": "Fujitsu Services Ltd",
    "24.4237": "Johnson Controls, Inc.",
    "24.4230": "ATTO Technology, Inc.",
    "24.4231": "Xstreamis Plc",
    "24.4232": "American Networks Inc.",
    "24.4233": "WebSonic",
    "24.4238": "HUGH SYMONS CONCEPT Technologies Ltd.",
    "24.4239": "Raptor Systems",
    "24.8954627": "Samsung Electronics Co.,Ltd",
    "12.123917682311168": "SENSEON Corporation",
    "24.10232170": "Tp-Link Technologies Co.,Ltd.",
    "20.154019626483712": "Agilent S.p.A",
    "12.123917675601920": "inomed Medizintechnik GmbH",
    "24.5016559": "Huawei Technologies Co.,Ltd",
    "12.71297883484160": "Vital Tech Industria e Comercio Ltda",
    "24.12860476": "Cybelec Sa",
    "12.123917684961280": "BISTOS.,Co.,Ltd",
    "20.71389201563648": "Tsinghua Tongfang Co., LTD",
    "12.346853269504": "Unipower AB",
    "12.123917681496064": "Senso2Me",
    "24.1590502": "zte corporation",
    "12.123917691023360": "TXMission Ltd.",
    "12.123917679796224": "Jiangsu Jinheng Information Technology Co.,Ltd.",
    "20.119143422492672": "Hangzhou Netease Yanxuan Trading Co.,Ltd",
    "12.346862059520": "Tsuji Electronics Co.,Ltd",
    "12.123917690953728": "dinosys",
    "24.2662977": "exlar corp",
    "24.15499875": "zte corporation",
    "24.10755954": "Dell Inc.",
    "24.3158676": "W-IE-NE-R Plein & Baus GmbH",
    "24.1583827": "Juniper Networks",
    "12.123917679218688": "MG s.r.l.",
    "24.3671732": "A.D.C. GmbH",
    "24.16289413": "Comtrend Corporation",
    "12.346863767552": "EXEL s.r.l",
    "24.11594110": "Advanced Micro Peripherals",
    "24.6313569": "IXECLOUD Tech",
    "24.12337804": "Alpsalpine Co,.Ltd",
    "20.111031371169792": "Securosys SA",
    "24.5516254": "New H3C Technologies Co., Ltd",
    "12.123917682827264": "Power Ltd.",
    "20.31873091764224": "Beijing Hengxin Rainbow Information Technology Co.,Ltd",
    "24.4773957": "Shenzhen Mainuoke Electronics Co., Ltd",
    "12.123917681504256": "Observer Foundation",
    "24.824207": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.11027585": "Nokia Corporation",
    "12.123917680816128": "Williams Sound LLC",
    "20.193649367318528": "Access Device Integrated Communications Corp.",
    "12.123917687672832": "Sensobox GmbH",
    "20.198237811113984": "ShenZhen Comstar Technology Company",
    "24.1105916": "Siemens AG",
    "12.119269650432": "Embit srl",
    "20.75685684051968": "tiga.eleven GmbH",
    "24.8154466": "Apple, Inc.",
    "12.346864553984": "Ariem Technologies Pvt Ltd",
    "24.567642": "AzureWave Technology Inc.",
    "12.123917688258560": "Netemera Sp. z o.o.",
    "12.346858262528": "Meucci Solutions",
    "24.3260": "Iscutum",
    "12.71297884475392": "Perm Scientific-Industrial Instrument Making Company JSC",
    "24.14736016": "Huawei Technologies Co.,Ltd",
    "12.346867032064": "DS! Ingenieurbuero",
    "24.14175018": "Cognitas Technologies, Inc.",
    "20.45033662185472": "Shenzhen Modern Cowboy Technology Co.,Ltd.",
    "12.123917689286656": "\"Digital Solutions\" JSC",
    "12.123917681287168": "Grossenbacher Systeme AG",
    "24.13689930": "Murata Manufacturing Co., Ltd.",
    "24.10771495": "Nintendo Co., Ltd.",
    "24.6041177": "Samsung Electronics Co.,Ltd",
    "24.4783502": "Huawei Technologies Co.,Ltd",
    "12.123917679034368": "Abitsoftware, Ltd.",
    "24.64653": "Hitron Technologies. Inc",
    "24.64651": "Amazon Technologies Inc.",
    "24.63601": "DGS Denmark A/S",
    "24.11327000": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.12587430": "Fort-Telecom ltd.",
    "12.123917675810816": "Precitec Optronik GmbH",
    "24.3148485": "Ruio telecommunication technologies Co., Limited",
    "24.9501": "DSA Encore, LLC",
    "24.9500": "Edt",
    "24.9503": "Zynus Vision Inc.",
    "24.9502": "Rotel Technologies",
    "24.9505": "Logitek Electronic Systems, Inc.",
    "24.9504": "SMA Railway Technology GmbH",
    "24.9507": "OCP Inc.",
    "24.9506": "ASRock Incorporation",
    "24.9509": "CTERA Networks Ltd.",
    "24.9508": "Lightcomm Technology Co., Ltd",
    "24.12365509": "ASUSTek COMPUTER INC.",
    "24.15516554": "Accuenergy (CANADA) Inc",
    "24.278386": "Mellanox Technologies, Inc.",
    "12.123917678948352": "Aimco",
    "24.6564746": "ALL BEST Industrial Co., Ltd.",
    "20.176094517919744": "Shanghai Xuntai Information Technology Co.,Ltd.",
    "24.12628225": "zte corporation",
    "20.71389208903680": "Vaisala Oyj",
    "24.11546709": "China Mobile IOT Company Limited",
    "24.13118507": "IEEE Registration Authority",
    "24.4995337": "HPL Electric & Power Private Limited",
    "12.346856419328": "Bir,Inc.",
    "24.10282590": "Apple, Inc.",
    "24.32889": "Microbus Designs Ltd.",
    "24.32888": "Practical Peripherals, Inc.",
    "24.32881": "Sai Technology",
    "24.32880": "Computadoras Micron",
    "24.32883": "Dwb Associates",
    "24.32882": "Microplex Systems Ltd.",
    "24.32885": "Parsytec Gmbh",
    "24.32884": "Fisher Controls",
    "24.32887": "Brother industries, LTD.",
    "24.32886": "Mcnc",
    "24.7154": "Kworld Computer Co., Ltd",
    "24.7155": "TRANSRADIO SenderSysteme Berlin AG",
    "24.7156": "Kenwin Industrial(Hk) Ltd.",
    "24.7157": "Tellink Sistemas de Telecomunicación S.L.",
    "24.7150": "Nokia Danmark A/S",
    "24.7151": "Blossoms Digital Technology Co.,Ltd.",
    "24.7152": "Value Platforms Limited",
    "24.7153": "Nanjing SilverNet Software Co., Ltd.",
    "12.346861248512": "ATS Elektronik GmbH",
    "24.7158": "CONWISE Technology Corporation Ltd.",
    "24.7159": "Lund IP Products AB",
    "24.2504": "Sinagawa Tsushin Keisou Service",
    "24.2505": "BlueWINC Co., Ltd.",
    "24.2506": "iMaxNetworks(Shenzhen)Limited.",
    "24.2507": "HBrain",
    "24.2500": "Medicore Co., Ltd",
    "24.2501": "KINGENE Technology Corporation",
    "24.2502": "Visionics Corporation",
    "24.2503": "Movistec",
    "12.123917681127424": "Clockwork Dog",
    "24.2508": "Moog GmbH",
    "24.2509": "Hudson Soft Co.,Ltd.",
    "12.123917676978176": "Shenzhen CAMERAY ELECTRONIC CO., LTD",
    "12.123917684203520": "Onosokki Co.,Ltd",
    "12.346866597888": "Keith & Koep GmbH",
    "24.5298003": "CONELCOM GmbH",
    "24.12344277": "ARRIS Group, Inc.",
    "20.251330455142400": "PT.MLWTelecom",
    "24.13910952": "Changzhou Haojie Electric Co., Ltd.",
    "24.2929178": "Phicomm (Shanghai) Co., Ltd.",
    "24.6328452": "DSSD Inc",
    "24.10789348": "Innotube, Inc.",
    "24.11851945": "BetterBots",
    "12.346867400704": "Pivitec, LLC",
    "12.123917676351488": "eleven-x",
    "24.12878399": "Fujian Newland Auto-ID Tech. Co,.Ltd.",
    "24.6345130": "Vishal Telecommunications Pvt Ltd",
    "24.11550289": "BSkyB Ltd",
    "24.16259844": "Zhong Shan City Richsound Electronic Industrial Ltd",
    "20.176189959307264": "Private",
    "12.123917688631296": "DTECH Labs, Inc.",
    "24.4998518": "China Mobile(Hangzhou) Information Technology Co.,Ltd.",
    "12.123917679288320": "Antlia Systems",
    "24.268303": "Samsung Electronics Co.,Ltd",
    "12.346854969344": "ABB T&D Spa",
    "12.123917680037888": "Power Electronics Espana, S.L.",
    "24.326035": "Infinix mobility limited",
    "20.58071059529728": "Kongqiguanjia (Beijing)Technology co.，ltd",
    "12.123917688561664": "ACRIOS Systems s.r.o.",
    "24.2129268": "Apple, Inc.",
    "24.16484": "Kla Instruments Corporation",
    "12.346867728384": "AVocation Systems, Inc.",
    "24.9756288": "Giga-Byte Technology Co.,Ltd.",
    "24.1872764": "ARRIS Group, Inc.",
    "20.180659908771840": "Guangdong Real-Design Intelligent Technology Co.,Ltd",
    "24.7101109": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "24.7101104": "TP-Link Corporation Limited",
    "24.14960563": "zte corporation",
    "12.346855993344": "Elektrotechnik & Elektronik Oltmann GmbH",
    "24.10237315": "Nipro Diagnostics, Inc",
    "24.6608916": "Cisco Systems, Inc",
    "12.346855800832": "Daintree Networks Pty",
    "20.84531194363904": "Ktc(K-Tel)",
    "24.12884102": "Qorvo International Pte. Ltd.",
    "20.83889187979264": "Shenzhen dingsheng technology co., LTD",
    "12.346867073024": "Aesir Copenhagen",
    "12.346860933120": "Ing. Knauseder Mechatronik GmbH",
    "20.88010622238720": "Shenzhen zhong ju Fiber optical Co.Ltd",
    "12.123917680852992": "ATE Systems Inc",
    "12.346855325696": "COMERSON S.r.l.",
    "12.346852315136": "Teracom Telematica S.A",
    "24.5305882": "Kunshan JADE Technologies co., Ltd.",
    "24.10506694": "Avaya Inc",
    "12.71297883930624": "Ferlin Trading BV",
    "12.123917689827328": "Smith Meter, Inc.",
    "24.15254304": "Austco Marketing & Service (USA) ltd.",
    "24.5307012": "Tp-Link Technologies Co.,Ltd.",
    "24.1083899": "Samsung Electronics Co.,Ltd",
    "12.346858856448": "Asia Pacific Card & System Sdn Bhd",
    "24.5292781": "GN Audio A/S",
    "24.2116766": "Cisco Systems, Inc",
    "12.123917677649920": "Pedax Danmark",
    "12.346854199296": "SKY Computers, Inc.",
    "12.123917689073664": "Uwinloc",
    "24.10252114": "zte corporation",
    "12.346857705472": "Netsynt S.p.A.",
    "12.123917686460416": "Cubitech",
    "12.123917680091136": "Private",
    "12.346867982336": "Peter Huber Kältemaschinenbau GmbH",
    "12.123917687062528": "Bighouse.,Inc.",
    "24.13640982": "Mercury Communication Technologies Co.,Ltd.",
    "12.346854948864": "DVTel Israel Ltd.",
    "20.198151824736256": "Fibersystem AB",
    "12.346861977600": "Resurgent Health & Medical",
    "24.16572991": "Clay Paky Spa",
    "12.71297884897280": "cibite AG",
    "24.11295484": "Intel Corporate",
    "24.13930291": "Microsoft Corporation",
    "24.6560871": "Digibras Industria Do Brasils/A",
    "12.346865369088": "Criticare Systems, Inc",
    "24.12333996": "Silicon Laboratories",
    "12.346853855232": "Electrex S.R.L",
    "12.346861568000": "Rmsd Ltd",
    "12.123917677445120": "Spectrum Brands",
    "24.8439041": "OctoGate IT Security Systems GmbH",
    "24.16011499": "Telechips Inc",
    "12.123917678592000": "Sicon srl",
    "20.189244303736832": "Sichuan Odot Automation System Co.,Ltd.",
    "20.31614740463616": "Monument Labs, Inc.",
    "24.13656204": "zte corporation",
    "24.14728707": "Lite-On Network Communication (Dongguan) Limited",
    "12.346853388288": "Dong A Eltek Co., Ltd.",
    "24.13643187": "Huawei Technologies Co.,Ltd",
    "12.123917675651072": "MobiPromo",
    "20.220515725737984": "Electronic Controls Design, Inc.",
    "12.346858360832": "Micro Technology Services Inc.",
    "24.2115233": "VECOW Co., Ltd",
    "24.14431037": "Huawei Device Co., Ltd.",
    "12.123917685960704": "MobiPromo",
    "24.9196987": "Songwoo Information & Technology Co., Ltd",
    "12.346857402368": "Dakty GmbH",
    "24.12386220": "Topcon Corporation",
    "24.6603604": "Avaya Inc",
    "12.346858819584": "Dorsett Technologies, Inc.",
    "24.14741019": "Panasonic Automotive Systems Company of America",
    "24.14736144": "Samsung Electronics Co.,Ltd",
    "12.123917688500224": "3D Printing Specialists",
    "12.346868244480": "Monsoon Solutions, Inc.",
    "12.346866348032": "Electrolight Shivuk (1994) Ltd.",
    "24.5817803": "Samsung Electronics Co.,Ltd",
    "24.12892825": "I+ME Actia Informatik und Mikro-Elektronik GmbH",
    "24.11074416": "WiseSec L.t.d",
    "20.146101715861504": "Scale-Tec Ltd.",
    "24.7138465": "Shenzhen Clou Electronics Co. Ltd.",
    "24.1578772": "Daewoois",
    "24.14174002": "Tp-Link Technologies Co.,Ltd.",
    "12.346853715968": "Broadstorm Telecom",
    "12.123917688901632": "KSE GmbH",
    "20.132777848602624": "Beijing Coilabs technology co.,ltd",
    "20.84531184926720": "Huizhou Dehong Technology Co., Ltd.",
    "12.123917684346880": "COONTROL Tecnologia em Combustão LTDA EPP",
    "20.247076401184768": "Shanghai- British Information Technology Co., Ltd",
    "24.13663797": "Intel Corporate",
    "24.15266342": "Millson Custom Solutions Inc.",
    "24.12097140": "AllDSP GmbH & Co. KG",
    "24.9492723": "zte corporation",
    "24.45296": "Caly Networks",
    "24.45294": "Ajile Systems, Inc.",
    "24.45292": "Eacem",
    "12.123917678796800": "Lanmark Controls Inc.",
    "24.42249": "WigWag Inc.",
    "12.123917685178368": "MyoungSung System",
    "24.9499841": "Rand McNally",
    "24.10259923": "Leonton Technologies",
    "12.123917682921472": "AVL DiTEST GmbH",
    "24.9982525": "Sunitec Enterprise Co.,Ltd",
    "12.123917680168960": "Panoramic Power",
    "12.123917681385472": "Axxess Identification Ltd",
    "24.10500599": "LG Electronics (Mobile Communications)",
    "24.15006607": "Mobiwire Mobiles (Ningbo) Co.,Ltd",
    "20.97400602492928": "elgris UG",
    "12.346866565120": "Nantes Systems Private Limited",
    "24.16278186": "xn systems",
    "24.10500590": "Sagemcom Broadband SAS",
    "24.12640702": "Huawei Technologies Co.,Ltd",
    "24.10254250": "Nadasnv",
    "20.176032224116736": "Power Diagnostic Service Co., LTD.",
    "24.5787885": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.346866450432": "Precision Remotes",
    "12.346867490816": "American Microsystems, Ltd.",
    "12.346856960000": "Artis GmbH",
    "24.5829372": "Huawei Device Co., Ltd.",
    "24.2400137": "Texas Instruments",
    "24.6043320": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "20.119709248782336": "Lineable Inc",
    "20.44169482469376": "Llc \"Microteh\"",
    "20.452706959360": "Pangaea Solution Inc",
    "24.10543706": "Amicus SK, s.r.o.",
    "12.71297883754496": "Boehme Nachrichtentechnik",
    "20.176032230408192": "Lon Microsystems Inc.",
    "24.14205947": "Detracom",
    "24.5814753": "Crystalfontz America, Inc.",
    "24.5025329": "ShengHai Electronics (Shenzhen) Ltd",
    "24.14956684": "Digital Products Limited",
    "24.15743573": "Omega Elektronik AS",
    "24.9198117": "ARRIS Group, Inc.",
    "24.46989": "Nanjing Shining Electric Automation Co., Ltd",
    "24.16544250": "Trane Technologies",
    "20.5297550655488": "Metra Electronics",
    "12.346867978240": "Sascal Displays Ltd",
    "24.1635907": "Apple, Inc.",
    "24.6344748": "Ruckus Wireless",
    "24.3680793": "Technica Engineering GmbH",
    "12.123917690642432": "Dameca a/s",
    "24.791762": "China Mobile Group Device Co.,Ltd.",
    "24.3691675": "zte corporation",
    "24.10756869": "Open Networking Laboratory",
    "24.3423908": "Ring LLC",
    "12.123917685411840": "Benchmark Electronics BV",
    "20.5239436476416": "Shantou Yingsheng Import & Export Trading Co.,Ltd.",
    "24.11842882": "Hubbell Power Systems, Inc.",
    "24.3468571": "Cisco Systems, Inc",
    "24.5303": "AR Infotek Inc.",
    "24.5302": "Enswer Technology Inc.",
    "24.5301": "Physiometrix,Inc",
    "24.5300": "General Dynamics United Kingdom Ltd",
    "24.5307": "Open Interface North America",
    "24.5306": "Carvers SA de CV",
    "24.5305": "Mstar Semiconductor",
    "24.5304": "Hill-Rom",
    "24.5309": "incNETWORKS, Inc",
    "24.5308": "Synectic Telecom Exports Pvt. Ltd.",
    "12.123917679509504": "ejoin, s.r.o.",
    "24.4720": "NGES Denro Systems",
    "24.1350921": "Huawei Technologies Co.,Ltd",
    "24.16542092": "ANDREAS STIHL AG & Co. KG",
    "24.7865695": "Shenzhen WYC Technology Co., Ltd.",
    "12.346854375424": "Fracarro srl",
    "24.11555685": "China Mobile IOT Company Limited",
    "24.7870967": "Juniper Networks",
    "24.8137265": "Intel Corporate",
    "24.1610404": "Ciena Corporation",
    "12.71297883947008": "Aplex Technology Inc.",
    "24.3944250": "SFORZATO Corp.",
    "24.8191632": "Mellanox Technologies, Inc.",
    "24.4745268": "Silicon Motion, Inc.",
    "24.2130624": "Xiaomi Communications Co Ltd",
    "24.2402429": "Panasonic Automotive Systems Asia Pacific(Thailand)Co.,Ltd.",
    "12.123917689294848": "Qprel srl",
    "12.346868674560": "Mylogic",
    "12.119269920768": "Elektrometal SA",
    "24.16015586": "Xiaomi Communications Co Ltd",
    "24.1858802": "Apple, Inc.",
    "20.224420098473984": "Laserworld (Switzerland) AG",
    "12.346866835456": "Xitek Design Limited",
    "24.57408": "DeskStation Technology, Inc.",
    "24.57409": "Cspi",
    "24.57404": "AdvanSys",
    "24.57405": "Focon Electronic Systems A/S",
    "24.57406": "Alfatech, Inc.",
    "24.57407": "Jaton Corporation",
    "24.57400": "Proxima Corporation",
    "24.57401": "Paradyne 7112 T1 DSU/CSU",
    "24.57402": "Cabletron Systems, Inc.",
    "24.57403": "Prominet Corporation",
    "20.27056776675328": "APANA Inc.",
    "20.35227546157056": "Austin Hughes Electronics Ltd.",
    "12.71297884598272": "SQF Spezialelektronik GmbH",
    "24.13638311": "UniPrint",
    "24.16268061": "Technicolor CH USA Inc.",
    "24.546097": "Routerboard.com",
    "24.12346015": "Telenet Systems P. Ltd.",
    "12.346852581376": "Private",
    "20.167594822729728": "Baycom Opto-Electronics Technolgy Co., Ltd.",
    "24.13644561": "Apple, Inc.",
    "24.584182": "AMPAK Technology,Inc.",
    "24.4776516": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917682810880": "enModus",
    "24.842878": "Espressif Inc.",
    "24.5286366": "Smartcom - Bulgaria AD",
    "24.2101104": "Foxtech",
    "12.123917686771712": "MB connect line GmbH Fernwartungssysteme",
    "12.123917675393024": "Unisor Multisystems Ltd",
    "24.9996100": "Wuxi Hongda Science and Technology Co.,LTD",
    "24.3079": "Iftest AG",
    "24.3078": "Nixvue Systems Pte Ltd",
    "12.123917686935552": "WuXi anktech Co., Ltd",
    "24.3073": "Abatron AG",
    "24.3072": "BEB Industrie-Elektronik AG",
    "24.3071": "Berkeley Camera Engineering",
    "24.3070": "CASTEL Broadband Limited",
    "24.3077": "RPA Reserch Co., Ltd.",
    "24.3076": "Tecnova",
    "24.3075": "HDMI Licensing, LLC",
    "24.3074": "ABB Oy",
    "24.1337762": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.14722672": "Nokia Corporation",
    "24.4235143": "Jide Technology (Hong Kong) Limited",
    "24.15376305": "Philips International B.V.",
    "24.20518": "Cosystems, Inc.",
    "24.36919": "Acucomm, Inc.",
    "24.20514": "Zonet Technology, Inc.",
    "24.20515": "Pg Design Electronics, Inc.",
    "24.20516": "Navic Systems, Inc.",
    "24.10542565": "SHARP Corporation",
    "24.20510": "Grass Valley, A Belden Brand",
    "24.20511": "Mrg Systems, Ltd.",
    "20.48543320506368": "Appostar Technology Co. Ltd",
    "24.20513": "Eis International, Inc.",
    "24.3440330": "Nextwill",
    "24.9455770": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5000550": "Nanjing Jiahao Technology Co., Ltd.",
    "24.13166576": "Juniper Networks",
    "24.13383595": "Edgecore Networks Corporation",
    "24.2388361": "Texas Instruments",
    "12.123917679603712": "Warecube,Inc",
    "12.346855403520": "Cosine Systems, Inc.",
    "12.346857254912": "Alfing Montagetechnik GmbH",
    "12.346862686208": "Innovative American Technology",
    "24.12873247": "Tp-Link Technologies Co.,Ltd.",
    "24.14686954": "Allied Telesis, Inc.",
    "24.8451055": "Facebook Technologies, LLC",
    "24.9714416": "Nokia Corporation",
    "20.167137991720960": "HySecurity",
    "12.346859089920": "Tsien (UK) Ltd",
    "24.14440636": "Thomas-Krenn.AG",
    "24.8683072": "Texas Instruments",
    "24.48112": "Ungermann-Bass Inc.",
    "12.346862542848": "Kumho Electric, Inc.",
    "24.8142493": "Patech",
    "24.1619999": "ARRIS Group, Inc.",
    "24.6873609": "Huawei Technologies Co.,Ltd",
    "24.51941": "Cisco Systems, Inc",
    "24.16318120": "Technico Japan Corporation",
    "24.13918954": "Huawei Technologies Co.,Ltd",
    "12.123917680902144": "Ascon Tecnologic S.r.l.",
    "12.346867908608": "Kinki Roentgen Industrial Co.,Ltd",
    "24.16403575": "Occitaline",
    "12.123917676077056": "Aplex Technology Inc.",
    "12.346859851776": "Teletek Electronics",
    "12.346860273664": "Eidos Spa",
    "24.7792": "Cobham Antenna Systems",
    "24.7793": "MIrcom Group of Companies",
    "24.1050668": "Texas Instruments",
    "24.7791": "Magna-Power Electronics, Inc.",
    "24.7796": "Sagemcom Broadband SAS",
    "24.7797": "LG Electronics (Mobile Communications)",
    "24.7794": "Pcs",
    "24.7795": "zte corporation",
    "24.7798": "Thermo Fisher Scientific",
    "24.7799": "Air2App",
    "24.6876078": "Samsung Electronics Co.,Ltd",
    "24.4517070": "Juniper Networks",
    "24.6327816": "Samsung Electronics Co.,Ltd",
    "24.16292181": "Fortress Technology Inc",
    "24.6082221": "CDYNE Corporation",
    "12.346868264960": "Saia-Burgess Controls AG",
    "12.346858958848": "Sparton Electronics",
    "24.9621": "Northwest Signal Supply, Inc",
    "24.9622": "GIGAVISION srl",
    "24.9623": "Kalki Communication Technologies",
    "24.9624": "Zhong Shan City Litai Electronic Industrial Co. Ltd",
    "24.9625": "Hedon e.d. B.V.",
    "24.9626": "CEStronics GmbH",
    "24.9627": "Beijing PKUNITY Microsystems Technology Co., Ltd",
    "24.9628": "Cisco-Linksys, LLC",
    "24.9629": "Private",
    "12.123917682339840": "Akeo Plus",
    "12.346860359680": "Phytec Messtechnik GmbH",
    "24.10796801": "Intel Corporate",
    "12.123917683015680": "Potter Electric Signal Co. LLC",
    "24.1439": "Yotta Networks, Inc.",
    "24.1438": "Zinwell Corporation",
    "24.1435": "Cisco Systems, Inc",
    "24.1434": "Cisco Systems, Inc",
    "24.1437": "Daniel Computing Systems, Inc.",
    "24.1436": "Kleinknecht GmbH, Ing. Büro",
    "24.1431": "Eagle Traffic Control Systems",
    "24.1430": "Genotech Co., Ltd.",
    "24.1433": "DRS Test and Energy Management or DRS-TEM",
    "20.57176928288768": "Shenzhen Andakai Technologies Co., Ltd.",
    "20.48543321554944": "Coremate Technical Co., Ltd",
    "12.123917690978304": "Gigaray",
    "12.346858852352": "P.C.E.",
    "24.11805927": "Codetek Technology Co.,Ltd",
    "24.1090778": "Huawei Technologies Co.,Ltd",
    "24.7899680": "Ubiquiti Networks Inc.",
    "12.346866884608": "Sicon srl",
    "24.1608298": "AVPro Global Hldgs",
    "24.1857947": "Huawei Technologies Co.,Ltd",
    "24.9198": "ARRIS Group, Inc.",
    "24.9199": "Zuend Systemtechnik AG",
    "20.67047982628864": "Harman Connected Services, Inc.",
    "24.9192": "Demco Corp.",
    "24.1621650": "Delta Networks, Inc.",
    "24.9190": "Innovation Farm, Inc.",
    "24.9191": "Hinke A/S",
    "24.9196": "Algorithmix GmbH",
    "24.9197": "ARRIS Group, Inc.",
    "24.9194": "Cisco Systems, Inc",
    "12.346858934272": "Kenton Research Ltd",
    "24.15994385": "IEEE Registration Authority",
    "24.4251394": "LINE BIZ Plus",
    "12.346863824896": "Atlas Copco IAS GmbH",
    "20.175931703427072": "Xi'an Yingsheng Electric Technology Co.,Ltd.",
    "12.123917685895168": "DesignA Electronics Limited",
    "12.119269490688": "OOO Actidata",
    "12.123917687463936": "Nocix, LLC",
    "24.2393240": "Beijing Jiaoda Microunion Tech.Co.,Ltd.",
    "24.8414553": "EM Microelectronic",
    "24.1893759": "Samsung Electronics Co.,Ltd",
    "24.5010623": "Apple, Inc.",
    "12.346864087040": "North Bridge Technologies",
    "20.83889197416448": "Shenzhen Haichuan Intelligent Information Technology Co., Ltd.",
    "24.10773215": "DS Global. Co., LTD",
    "24.8661268": "EMPORIA TELECOM Produktions- und VertriebsgesmbH & Co KG",
    "24.1291": "SICOM Systems, Inc.",
    "24.1290": "ICS Spa",
    "24.1293": "Midstream Technologies, Inc.",
    "12.346854461440": "Legra Systems, Inc.",
    "24.1295": "Tanaka S/S Ltd.",
    "24.1294": "3ware, Inc.",
    "24.1297": "Complementary Technologies Ltd",
    "24.1296": "Infinite Shanghai Communication Terminals Ltd.",
    "24.1299": "VTLinx Multimedia Systems, Inc.",
    "24.1298": "Zebra Technologies Inc",
    "24.1080303": "Zyxel Communications Corporation",
    "20.127658785505280": "Private",
    "24.31512": "SENTRY Co., LTD.",
    "24.16290163": "Aedle Sas",
    "24.8579": "ANDRITZ HYDRO GmbH",
    "24.8578": "SandLinks Systems, Ltd.",
    "12.346866458624": "metraTec GmbH",
    "24.8576": "ARRIS Group, Inc.",
    "24.8575": "Intraco Technology Pte Ltd",
    "24.8574": "Telit Communication s.p.a",
    "24.8573": "Pyxis S.R.L.",
    "24.8572": "2Wire Inc",
    "24.8571": "Bastec AB",
    "24.8570": "Sejin Electron, Inc.",
    "20.22333143121920": "Zehnder Group AG",
    "24.11854029": "Fusion-io, Inc",
    "12.123917678686208": "LPA Excil Electronics",
    "12.346856968192": "VAZA Elektronik AB",
    "24.265788": "HP Inc.",
    "12.346859855872": "Chesapeake Sciences Corp",
    "20.66426708688896": "La Barrière Automatique",
    "24.11288810": "C&T Solution Inc.",
    "20.207357809262592": "Direct Communication Solutions",
    "12.123917690146816": "SiS Technology",
    "24.4915": "BaudTec Corporation",
    "24.4914": "Beijing Topsec Network Security Technology Co., Ltd.",
    "24.4917": "VS Industry Berhad",
    "24.4916": "Arkados, Inc.",
    "24.4911": "Interactek",
    "24.4910": "ITian Coporation",
    "24.4913": "CellPoint Connect",
    "24.4912": "Euro Protection Surveillance",
    "24.2120192": "Coval",
    "20.88010621190144": "Shenzhen Lucky Sonics Co.,Ltd",
    "24.4919": "Orient Power Home Network Ltd.",
    "24.4918": "Tianjin 712 Communication Broadcasting co., ltd.",
    "12.346864070656": "Timberline Mfg Company",
    "20.146101712715776": "Orchard Electronics Co., Ltd.",
    "24.1090488": "Iskratel d.o.o.",
    "12.123917687570432": "Vishay Nobel AB",
    "24.8135692": "Telechips, Inc.",
    "24.16253203": "Huawei Technologies Co.,Ltd",
    "12.346863505408": "Phytec Messtechnik GmbH",
    "20.66129350361088": "Swissdotnet SA",
    "24.13917663": "Pegatron Corporation",
    "12.346856337408": "nVent, Schroff GmbH",
    "24.5558627": "Max-Tech,Inc",
    "12.346855645184": "Luceo",
    "12.123917689466880": "Mijienetrtech Co.,Ltd",
    "24.2905295": "LG Electronics (Mobile Communications)",
    "20.163603971833856": "hyBee Inc.",
    "12.346856521728": "Roxar A/S",
    "20.206931524321280": "Redvision CCTV",
    "24.2366468": "Shenzhen Jehe Technology Development Co., Ltd.",
    "24.13691651": "Zhehua technology limited",
    "24.10757299": "FlatFrog Laboratories AB",
    "12.346864041984": "APG Cash Drawer, LLC",
    "24.13929945": "Meld Technology, Inc",
    "12.123917675180032": "Scorpion Precision Industry (HK)CO. Ltd.",
    "24.2398189": "Techvision Intelligent Technology Limited",
    "24.11044754": "Broadband Antenna Tracking Systems",
    "24.8660602": "Guangdong Taide Zhilian Technology Co.,Ltd",
    "24.10806652": "Espressif Inc.",
    "24.14735432": "Dell Inc.",
    "20.97755447951360": "Kustom Signals Inc",
    "24.6321800": "Velodyne",
    "24.8674451": "Chabrier Services",
    "24.816107": "IEEE Registration Authority",
    "24.5514902": "Apple, Inc.",
    "20.31614738366464": "Videri Inc.",
    "12.346865307648": "Creative Micro Design",
    "24.12875000": "Hot Pepper, Inc.",
    "20.22126512832512": "Private",
    "24.36988": "Digitalcast, Inc.",
    "24.36989": "Lake Communications",
    "24.36984": "Mer Telemanagement Solutions, Ltd.",
    "24.36985": "ClearOne, Inc.",
    "24.36986": "Spectralink, Inc",
    "24.36987": "E-Tech, Inc.",
    "24.36980": "Argon Networks, Inc.",
    "24.36981": "Nec Do Brasil S.A.",
    "24.36982": "Fmt Aircraft Gate Support Systems Ab",
    "24.36983": "Advanced Fibre Communications",
    "24.3148384": "IEEE Registration Authority",
    "12.123917684084736": "WoKa-Elektronik GmbH",
    "12.71297883222016": "Gogo BA",
    "12.71297883455488": "An Chen Computer Co., Ltd.",
    "12.123917680140288": "Santa Barbara Imaging Systems",
    "24.2117366": "AzureWave Technology Inc.",
    "24.14206305": "CommFront Communications Pte Ltd",
    "24.13646259": "ARRIS Group, Inc.",
    "12.346865565696": "Private",
    "24.15526165": "STI Ltd",
    "24.8443045": "Huawei Technologies Co.,Ltd",
    "12.123917675995136": "TrexEdge, Inc.",
    "24.15258367": "Sagemcom Broadband SAS",
    "24.13150924": "Private",
    "12.346852810752": "Infinitec Networks, Inc.",
    "24.9741324": "Diehl Metering GmbH",
    "20.181338939326464": "Ingelabs S.L.",
    "12.123917684195328": "APE GmbH",
    "12.346864234496": "Advanced Storage Concepts, Inc.",
    "20.35227540914176": "Wideband Systems, Inc.",
    "12.123917685190656": "Itronics Ltd",
    "24.7107033": "AzureWave Technology Inc.",
    "24.13117972": "Apple, Inc.",
    "12.71297884131328": "HAMEG GmbH",
    "12.123917688651776": "Plenty Unlimited Inc",
    "12.346862493696": "NEUTRONIK e.K.",
    "24.10531788": "Hewlett Packard",
    "24.4782065": "Nokia",
    "20.193649370464256": "Private",
    "12.123917683068928": "Panamera",
    "24.10509089": "ENVINET GmbH",
    "24.14717607": "Feitian Technologies Co., Ltd",
    "12.123917677539328": "Sorama BV",
    "24.6342552": "Verifone",
    "12.123917675495424": "OrbiWise SA",
    "24.16291271": "LG Electronics (Mobile Communications)",
    "24.15792370": "Texas Instruments",
    "12.346864660480": "Peek Traffic Corporation",
    "12.123917691441152": "ARC Technology Solutions, LLC",
    "24.10500865": "Kyung In Electronics",
    "24.13645285": "EM Microelectronic",
    "24.6588826": "LG Electronics (Mobile Communications)",
    "12.123917686575104": "First Light Imaging",
    "24.7657163": "New H3C Technologies Co., Ltd",
    "12.123917691707392": "Kawasaki Robot Service,Ltd.",
    "24.9211634": "Apple, Inc.",
    "20.176069203197952": "Zhuhai Cheer Technology Co., LTD.",
    "20.146101705375744": "PetroInTrade",
    "24.9444647": "zte corporation",
    "20.168210324258816": "Shield Inc.",
    "20.272689495277568": "EMBUX Technology Co., Ltd.",
    "20.150399887605760": "Sercomm Corporation.",
    "12.123917680918528": "MB connect line GmbH Fernwartungssysteme",
    "24.3685723": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.264625143676928": "Zhejiang Makepower Electronics,Inc.",
    "24.14706797": "Raybased AB",
    "20.212014913814528": "Alpha Audiotronics, Inc.",
    "24.6309309": "Microsoft",
    "20.111031364878336": "Narrative AB",
    "24.12326021": "Samsung Electronics Co.,Ltd",
    "24.13925296": "Askey Computer Corp",
    "12.123917684289536": "Kratos Analytical Ltd",
    "12.123917687521280": "Alfa Elettronica srl",
    "24.7661282": "Dell Inc.",
    "20.39919496986624": "Nanjing Wanlida Technology Co., Ltd.",
    "12.123917677051904": "Hangzhou Woosiyuan Communication Co.,Ltd.",
    "12.346865823744": "IAF GmbH",
    "24.10763936": "iComm HK LIMITED",
    "12.123917677944832": "Kdt",
    "12.346853085184": "Inspiration Technology P/L",
    "12.123917691482112": "Saice",
    "12.123917687173120": "Elektronik Art",
    "24.16256323": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.31873090715648": "SABIK Offshore GmbH",
    "20.154794728620032": "Beijing Xinxunxintong Eletronics Co.,Ltd",
    "24.7394686": "Scalar Corporation",
    "24.16257800": "Nokia",
    "12.346862702592": "Charlyrobot",
    "12.123917688655872": "Colmek",
    "12.346853732352": "PDQ Manufacturing",
    "24.12080400": "Zioncom Electronics (Shenzhen) Ltd.",
    "24.10025415": "IEEE Registration Authority",
    "24.6329077": "ARRIS Group, Inc.",
    "24.10138360": "SFF Committee",
    "12.123917677363200": "Orlaco Products B.V.",
    "20.111031377461248": "Dongyang unitech.co.ltd",
    "24.8394245": "Vodafone Italia S.p.A.",
    "24.2129039": "Quanta Microsystems,Inc.",
    "20.31657725788160": "OxySec S.r.l.",
    "20.35758083670016": "Dynaudio",
    "20.5239437524992": "Amiosec Ltd",
    "24.10529563": "D-Link International",
    "24.8918469": "Huawei Device Co., Ltd.",
    "12.71297883881472": "Fraunhofer HHI",
    "24.7900237": "Indyme Solutions, LLC",
    "12.346865184768": "SiGarden Sp z o.o.",
    "24.5563312": "JVL Industri Elektronik",
    "24.9210277": "Micro-Star INT'L CO., LTD",
    "24.10547735": "GE Medical System(China) Co., Ltd.",
    "24.13390051": "Espressif Inc.",
    "24.835444": "Morse Micro",
    "20.181256786542592": "Auranext",
    "20.145735985135616": "Sphera Telecom",
    "24.6577623": "Shenzhen Lifesense Medical Electronics Co., Ltd.",
    "24.12889396": "Routerboard.com",
    "24.8186323": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13169515": "Yamato Scale Co.,Ltd.",
    "20.17622847455232": "Wireless input technology Inc.",
    "20.229108127629312": "Shanghai Luying International Trade Co.,Ltd",
    "12.123917689810944": "Theta432",
    "24.13948173": "Southwest Microwave, Inc",
    "24.3176047": "LG Electronics (Mobile Communications)",
    "12.71297884696576": "Contec Steuerungstechnik & Automation GmbH",
    "12.123917686517760": "Shanghai Tianhe Automation Instrumentation Co., Ltd.",
    "24.2140345": "Siemens Ag",
    "20.45069248757760": "Jinlitong International Co.,Ltd",
    "24.8448887": "Wistron Corporation",
    "12.119269593088": "Inpixal",
    "24.14165282": "ARRIS Group, Inc.",
    "20.48688651042816": "Santec Corporation",
    "20.167295458476032": "Western Security Solutions",
    "12.346857377792": "RTDS Technologies Inc.",
    "20.216145527308288": "AKKA Germany GmbH",
    "24.6617226": "China Mobile IOT Company Limited",
    "12.119269863424": "YIK Corporation",
    "24.5509904": "Huawei Technologies Co.,Ltd",
    "12.346856542208": "Argo-Tech",
    "12.71297883557888": "Leica Biosystems",
    "12.123917688414208": "Siemens As",
    "24.528378": "Ksp Inc.",
    "12.123917685497856": "GreenFlux",
    "24.9499256": "Radius Gateway",
    "24.3715547": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.11580243": "IEEE Registration Authority",
    "12.346861502464": "Friedrich Kuhnt GmbH",
    "24.5786649": "Chongqing Guohong Technology Development Company Limited",
    "12.123917683421184": "Camsat Przemysław Gralak",
    "12.123917681377280": "Qualitteq Llc",
    "24.11817555": "Seagate Technology",
    "24.9712605": "Taco Inc",
    "20.71070619009024": "Fin Robotics Inc",
    "24.15488505": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346858901504": "DIGITEK S.p.A.",
    "12.123917681852416": "MG s.r.l.",
    "24.12121930": "Rcntec",
    "12.346864816128": "ART Antriebs- und Regeltechnik GmbH",
    "20.193649364172800": "CIDE Interactive",
    "24.4457383": "Cisco Systems, Inc",
    "24.9714609": "Kaonmedia CO., LTD.",
    "24.9714608": "New H3C Technologies Co., Ltd",
    "12.71297884299264": "Halliburton - Sperry Drilling Service",
    "20.272805452054528": "ADTECHNO Inc.",
    "24.12321280": "Stewart Audio",
    "20.229108122386432": "wallbe GmbH",
    "12.346863136768": "Keith & Koep GmbH",
    "12.123917681201152": "Zehetner-Elektronik GmbH",
    "24.7893404": "ARRIS Group, Inc.",
    "24.6873446": "Private",
    "12.123917683232768": "ATOM GIKEN Co.,Ltd.",
    "12.123917688320000": "KST technology",
    "12.123917688844288": "Vector.Co.,Ltd.",
    "12.123917681528832": "Dorlet Sau",
    "24.12858": "so-logic",
    "24.7400": "Cummins Inc",
    "24.792005": "SDTEC Co., Ltd.",
    "20.119709258219520": "Nanjing Buruike Electronics Technology Co., Ltd.",
    "12.123917682507776": "BCD Audio",
    "12.123917689204736": "Lenoxi Automation s.r.o.",
    "12.71297884884992": "Kronotech Srl",
    "20.52821127856128": "WINTEK System Co., Ltd",
    "12.346854965248": "BBI Engineering, Inc.",
    "12.123917683228672": "Nuand LLC",
    "24.45156": "Cisco Systems, Inc",
    "24.10515250": "Cashmaster International Limited",
    "24.6042460": "Swissphone Telecom AG",
    "12.71297883688960": "Wuxi SiNeng New Energy Co., Ltd.",
    "12.346857906176": "nVent, Schroff GmbH",
    "12.346862481408": "Melex Inc.",
    "12.346863763456": "Marine Entertainment Systems Ltd",
    "12.346863177728": "BPC circuits Ltd",
    "24.4441": "Matisse Networks Inc",
    "24.8955708": "Ragentek Technology Group",
    "24.13646879": "Intel Corporate",
    "12.123917683384320": "Rch Italia Spa",
    "24.3426541": "Dell Inc.",
    "12.346858483712": "Telecom FM",
    "12.71297884389376": "Akse srl",
    "24.7808": "Icotera A/S",
    "24.2644715": "Apple, Inc.",
    "24.7802": "Cisco Systems, Inc",
    "24.7803": "R.I.CO. S.r.l.",
    "24.7800": "Owitek Technology Ltd.,",
    "24.7801": "Cisco Systems, Inc",
    "24.7806": "Nortel Networks",
    "24.7807": "CBM of America",
    "24.7804": "Taiwick Limited",
    "24.7805": "Samsung Electronics Co.,Ltd",
    "24.6332004": "Vivonic GmbH",
    "24.3975405": "smart light technology",
    "20.212014911717376": "Xena Security Limited",
    "20.106480343711744": "Elap s.r.l.",
    "20.75685680906240": "Shenzhen Hebang Electronic Co., Ltd",
    "24.14473120": "Huawei Device Co., Ltd.",
    "24.11840077": "Ensequence, Inc.",
    "12.123917679374336": "Guangzhou Haoxiang Computer Technology Co.,Ltd.",
    "12.123917684854784": "Lineage Power Pvt Ltd.,",
    "20.220092036022272": "Shiftall Inc.",
    "24.1841630": "eyevis GmbH",
    "20.110162576670720": "Active Brains",
    "24.16579": "Fischer And Porter Co.",
    "24.16578": "Applied Computing Devices",
    "24.16573": "Starlight Networks, Inc.",
    "24.16572": "Algorithmics Ltd.",
    "24.16571": "Goldstar Cable Co., Ltd.",
    "24.16570": "Alliant Computer Systems Corp.",
    "24.16577": "Bizerba-Werke Wilheim Kraut",
    "24.16576": "Vista Controls Corporation",
    "24.16575": "Channel Systems Intern'L Inc.",
    "24.16574": "Boeing Defense & Space",
    "12.346852421632": "Celotek Corp",
    "24.14470447": "Seiko Epson Corporation",
    "24.15767525": "Tamio, Inc",
    "24.3208973": "Cisco Systems, Inc",
    "24.15791460": "Samsung Electronics Co.,Ltd",
    "20.154794725474304": "Trilux Group Management GmbH",
    "12.346856648704": "Grupo Epelsa S.L.",
    "24.1839773": "Ieee Pes Psrc/Sub",
    "20.22126498152448": "GloQuad",
    "24.5789378": "Huawei Technologies Co.,Ltd",
    "12.123917677064192": "Morgan Schaffer Inc.",
    "24.15998731": "YAMABISHI Corporation",
    "12.123917684621312": "Dismuntel Sal",
    "24.16041877": "WEY Technology AG",
    "20.145211816673280": "Provision-ISR",
    "12.123917676929024": "MacGray Services",
    "24.9494455": "s.m.s smart microwave sensors GmbH",
    "24.12849290": "Ruckus Wireless",
    "24.1321329": "Nemoa Electronics (Hk) Co. Ltd",
    "20.58144483966976": "Doki Technologies Limited",
    "12.346861682688": "IO-Connect",
    "12.123917691437056": "Integrated Protein Technologies, Inc.",
    "12.346867027968": "Funkwerk IT Karlsfeld GmbH",
    "24.282202": "ARRIS Group, Inc.",
    "24.7884924": "Juniper Networks",
    "24.7924500": "Shenzhen Fast Technologies Co.,Ltd",
    "12.123917688000512": "Peek Traffic",
    "24.9755593": "AzureWave Technology Inc.",
    "20.211776047153152": "Connected Space Management",
    "24.2641870": "Liaoning Wontel Science and Technology Development Co.,Ltd.",
    "24.5512": "Salutica Allied Solutions Sdn Bhd",
    "24.5513": "D-MAX Technology Co.,Ltd",
    "24.5510": "Xiamen Overseas Chinese Electronic Co., Ltd.",
    "24.5511": "Takenaka Seisakusho Co.,Ltd",
    "24.5516": "Liab ApS",
    "24.5517": "Jennic Ltd",
    "24.5514": "SURECOM Technology Corp.",
    "24.5515": "Park Air Systems Ltd",
    "12.346863841280": "Ronyo Technologies s.r.o.",
    "24.5518": "Plustek.INC",
    "24.5519": "NTT Advanced Technology Corporation",
    "24.11320587": "Leimac Ltd.",
    "24.12384273": "Xiamen DNAKE Technology Co.,Ltd",
    "12.346865115136": "Commotive A/S",
    "20.48548675584000": "octoScope, Inc.",
    "12.123917684412416": "IWS Global Pty Ltd",
    "24.6327526": "ARRIS Group, Inc.",
    "24.9194669": "Evoluzn Inc.",
    "24.555946": "Svyazkomplektservice, Llc",
    "12.119269330944": "Dunlop Systems & Components",
    "20.93020697395200": "VendNovation LLC",
    "24.12584778": "Brita GmbH",
    "12.346855391232": "MicroBee Systems, Inc",
    "12.123917685977088": "SPX Radiodetection",
    "12.346868539392": "Matsusada Precision Inc.",
    "24.9237923": "Samsung Electro-Mechanics(Thailand)",
    "24.1060161": "Sichuan AI-Link Technology Co., Ltd.",
    "24.13154003": "Huawei Technologies Co.,Ltd",
    "12.346859503616": "Gem-Med Sl",
    "12.346863771648": "CM Elektronik GmbH",
    "24.2359482": "Huawei Technologies Co.,Ltd",
    "20.198237815308288": "Qstar Technology Co,Ltd",
    "20.211776048201728": "Hitachi High-Tech Materials Corporation",
    "12.123917675814912": "Embedded Arts Co., Ltd.",
    "12.123917675663360": "PTN Electronics Limited",
    "24.15482629": "Institute 706, The Second Academy China Aerospace Science & Industry Corp",
    "24.6074686": "Sagemcom Broadband SAS",
    "12.71297884377088": "Scandyna A/S",
    "24.324434": "Huawei Technologies Co.,Ltd",
    "24.8927792": "Aruba, a Hewlett Packard Enterprise Company",
    "20.207147376836608": "UtilLighting Co.,Ltd.",
    "24.15773959": "Ericsson AB",
    "24.4641": "B.Braun Melsungen AG",
    "24.4640": "Cadco Systems",
    "24.4643": "Pixim",
    "24.4642": "Skardin (UK) Ltd",
    "24.4645": "ARRIS Group, Inc.",
    "24.4644": "NexQL Corporation",
    "24.4647": "Franklin Electric Co., Inc.",
    "24.4646": "Japan Direx Corporation",
    "24.4649": "BroadEasy Technologies Co.,Ltd",
    "24.4648": "Data Ltd.",
    "24.7366018": "Suzhou Hanming Technologies Co., Ltd.",
    "24.5505924": "Hongkong Nano IC Technologies Co., Ltd",
    "12.123917684006912": "Shanghai Longpal Communication Equipment Co., Ltd.",
    "24.6330333": "MicroSys Electronics GmbH",
    "24.6042624": "Hisense Electric Co.,Ltd",
    "24.15232824": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.123917680508928": "KMtronic ltd",
    "12.346865377280": "PACOMP Sp. z o.o.",
    "24.3918": "Cellink",
    "24.3919": "PCS Systemtechnik GmbH",
    "24.3914": "Kyushu-kyohan co.,ltd",
    "24.3915": "Oracle Corporation",
    "24.3916": "Elextech INC",
    "24.3917": "TalkSwitch",
    "24.3910": "Sinar Ag",
    "24.3911": "Robox Spa",
    "24.3912": "Polypix Inc.",
    "24.3913": "Northover Solutions Limited",
    "12.123917677514752": "Rapiot",
    "24.3471936": "Guangzhou Robustel Technologies Co., Limited",
    "24.13427092": "Juniper Networks",
    "24.11554342": "Tp-Link Technologies Co.,Ltd.",
    "12.123917677338624": "Mesa Labs, Inc.",
    "24.16297733": "Huawei Device Co., Ltd.",
    "20.31614742560768": "LX Corporation Pty Ltd",
    "24.11332433": "Universal Electronics, Inc.",
    "24.14726901": "IEEE Registration Authority",
    "12.346861957120": "STRATEC Control Systems",
    "20.251018410459136": "JSC \"Svyaz Inginiring M\"",
    "20.255553894875136": "Xirgo Technologies LLC",
    "20.180698127269888": "Abb Ab Pghv",
    "12.123917687607296": "KST technology",
    "24.5528230": "Espressif Inc.",
    "24.7390892": "Bosch Automotive Aftermarket",
    "24.10797255": "ShenZhen Hitom Communication Technology Co..LTD",
    "24.8654222": "Espressif Inc.",
    "12.346852638720": "Edge Tech Co., Ltd.",
    "24.20708": "Apple, Inc.",
    "12.123917685514240": "Kopis Mobile LLC",
    "24.20705": "Ns Tech Electronics Sdn Bhd",
    "24.20707": "ARRIS Group, Inc.",
    "24.20706": "Cisco Systems, Inc",
    "24.20701": "Serra Soldadura, S.A.",
    "24.20700": "Tas Telefonbau A. Schwabe Gmbh & Co. Kg",
    "24.20703": "AirFiber, Inc.",
    "24.20702": "Signum Systems Corp.",
    "24.6842866": "ComAp s.r.o.",
    "24.7349": "Neihua Network Technology Co.,LTD.(NHN)",
    "24.7348": "Iridium Satellite LLC",
    "12.346862759936": "Realtime Systems Ltd.",
    "24.7344": "Cisco Systems, Inc",
    "24.7347": "Apple, Inc.",
    "24.7346": "Bpt Spa",
    "24.7341": "Wuhan Telecommunication Devices Co.,Ltd",
    "24.7340": "Qniq Technology Corp.",
    "24.7343": "Plato Networks Inc.",
    "24.7342": "WiChorus, Inc.",
    "24.1114282": "Private",
    "12.346859171840": "Nanoradio AB",
    "12.346855555072": "SkipJam",
    "24.12382895": "devolo AG",
    "20.171812094738432": "Phoenix Audio Technologies",
    "20.31368182497280": "Shenzhen Shouxin Tongda Technology Co.,Ltd",
    "24.1319895": "Eutronix Co., Ltd.",
    "24.4232637": "NTmore.Co.,Ltd",
    "12.123917675044864": "FUJICOM Co.,Ltd.",
    "12.123917685506048": "Galea Electric S.L.",
    "24.1627855": "Kurth Electronic GmbH",
    "12.123917688881152": "Hrt",
    "24.8829": "Ye Data Inc.",
    "24.8828": "Woori SMT Co.,ltd",
    "24.16312680": "Egker Kft.",
    "24.53448": "Prevas A/S",
    "24.53449": "Advantech Co., Ltd.",
    "24.53444": "Teratech Corporation",
    "24.53445": "Computational Systems, Inc.",
    "24.53446": "Thomas & Betts Corp.",
    "24.53447": "Pathway, Inc.",
    "24.53440": "Cisco Systems, Inc",
    "24.53441": "Harmonic Data Systems, Ltd.",
    "24.53442": "Balthazar Technology Ab",
    "24.53443": "Vivid Technology Pte, Ltd.",
    "24.9451743": "Changzhou Tiannengbo System Co. Ltd.",
    "24.159745": "Racal-Datacom",
    "24.16005140": "Shanghai Howell Electronic Co.,Ltd",
    "20.30931578519552": "Private",
    "24.15738725": "Murata Manufacturing Co., Ltd.",
    "24.4739511": "Intel Corporate",
    "12.123917676208128": "AddSecure Smart Grids",
    "24.7646616": "Cisco Systems, Inc",
    "24.16557671": "Amazon Technologies Inc.",
    "24.6044711": "Samsung Electronics Co.,Ltd",
    "12.346855567360": "EXYS bvba",
    "24.4778481": "Apple, Inc.",
    "12.123917685760000": "iMAR Navigation GmbH",
    "24.6438": "BitsGen Co., Ltd.",
    "24.6439": "ImCoSys Ltd",
    "24.6434": "CM Comandos Lineares",
    "24.6435": "Phonex Korea Co., LTD.",
    "24.6436": "LBNL Engineering",
    "24.6437": "Intelicis Corporation",
    "20.45033663234048": "1More",
    "24.6431": "Microlink communications Inc.",
    "24.6432": "KUME electric Co.,Ltd.",
    "24.6433": "Elitegroup Computer Systems Co.,Ltd.",
    "24.2099437": "Huawei Technologies Co.,Ltd",
    "24.7355449": "SEAWING Kft",
    "20.260167869399040": "Bei jing Lian Shan times Techonology Co.Ltd",
    "24.11858723": "Petatel Inc.",
    "12.123917675388928": "Vitec",
    "12.123917690966016": "Zenros ApS",
    "24.2672762": "ARRIS Group, Inc.",
    "24.13931060": "NEC Corporation",
    "24.2672764": "zte corporation",
    "20.176772035379200": "ShenZhen JuWangShi Tech",
    "24.14968606": "SHEN ZHEN NUO XIN CHENG TECHNOLOGY co., Ltd.",
    "24.6596573": "Avaya Inc",
    "24.24709": "Storage Concepts",
    "24.24708": "Digital Video",
    "12.346858131456": "Rosta Ltd",
    "24.24701": "Sentient Networks Inc.",
    "24.24700": "WaveAccess, Ltd.",
    "24.24703": "Aurora Technologies, Inc.",
    "24.24702": "Gigalabs, Inc.",
    "24.24705": "Tv/Com International",
    "24.24704": "Microtronix Datacom Ltd.",
    "24.24707": "Cisco Systems, Inc.",
    "24.24706": "Novalink Technologies, Inc.",
    "24.680": "Air Link Technology",
    "24.681": "RACOM, s.r.o.",
    "24.682": "PLcom Co., Ltd.",
    "24.683": "CTC Union Technologies Co., Ltd.",
    "24.684": "3PAR data",
    "24.685": "HOYA Corporation",
    "24.297100": "Eifelwerk Butler Systeme GmbH",
    "24.687": "TeleCruz Technology, Inc.",
    "24.688": "Hokubu Communication & Industrial Co., Ltd.",
    "24.689": "Anritsu, Ltd.",
    "24.15472431": "Tp-Link Technologies Co.,Ltd.",
    "12.123917689012224": "Critical Link LLC",
    "24.13636299": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "12.123917675868160": "Tsat As",
    "24.15519903": "Xiaomi Communications Co Ltd",
    "20.176726073147392": "Shenzhen Talent Technology company limited",
    "24.8662901": "Innokas Group",
    "24.7124331": "HUMAX Co., Ltd.",
    "12.123917685395456": "LiveCopper Inc.",
    "12.346864541696": "TV Portal Co., Ltd.",
    "24.16563559": "Arista Networks",
    "24.7878151": "Cermate Technologies Inc.",
    "24.16561084": "Intel Corporate",
    "24.2971": "Sirius System Co, Ltd.",
    "24.8911": "Planex Communications Inc.",
    "24.2973": "TwinMOS Technologies Inc.",
    "24.2972": "TriBeam Technologies, Inc.",
    "24.2975": "Neue ELSA GmbH",
    "24.2974": "Yasing Technology Corp.",
    "24.2977": "Fujikura Solutions Ltd.",
    "24.2976": "T&L Information Inc.",
    "24.2979": "Siemens AG",
    "24.2978": "Sumitomo Electric Industries, Ltd",
    "24.9452989": "Secure Meters Limited",
    "24.6598385": "Qingdao Hisense Electronics Co.,Ltd.",
    "24.358": "Tc Group A/S",
    "24.359": "Hioki E.E. Corporation",
    "24.9019": "C-Matic Systems Ltd",
    "24.9018": "Samsung Electronics Co.,Ltd",
    "24.9017": "Samsung Electronics Co.,Ltd",
    "24.9016": "OJ-Electronics A/S",
    "24.9015": "Global Star Solutions ULC",
    "24.9014": "METEL s.r.o.",
    "12.346867654656": "Infronics Systems Limited",
    "24.9012": "Cisco Systems, Inc",
    "24.9011": "Cisco Systems, Inc",
    "24.9010": "Apple, Inc.",
    "24.11558106": "HP Inc.",
    "20.149945500827648": "Hauch & Bach ApS",
    "24.14193967": "Texas Instruments",
    "20.62663550828544": "Swedish Adrenaline AB",
    "20.154039770677248": "Pyras Technology Inc.",
    "12.123917678784512": "Iroc Ab",
    "24.16561762": "IC Holdings LLC",
    "24.8403429": "Patlite Corporation",
    "24.7651709": "Infinix mobility limited",
    "12.123917678473216": "Royal Engineering Consultancy Private Limited",
    "20.52912286859264": "Dewesoft d.o.o.",
    "12.123917685501952": "Proemion GmbH",
    "12.119269728256": "Servicios Electronicos Industriales Berbel s.l.",
    "24.8176951": "Apple, Inc.",
    "24.849109": "Ruckus Wireless",
    "20.228844765184000": "Private",
    "24.16522401": "Nokia",
    "12.123917684133888": "xTom GmbH",
    "12.123917676462080": "Tokyo Drawing Ltd.",
    "12.346866290688": "DSI RF Systems, Inc.",
    "24.7125121": "zte corporation",
    "24.16526893": "Lorom Industrial Co.LTD.",
    "12.123917686439936": "Innoknight",
    "24.4493692": "HMD Global Oy",
    "24.3162785": "ilumisys Inc. DBA Toggled",
    "20.150262232645632": "Shenzhenshi kechuangzhixian technology Co.LTD",
    "12.123917681033216": "Shenzhen Lianfaxun Electronic Technology Co., Ltd",
    "20.264165323177984": "Candelic Limited",
    "20.62335517458432": "Thrust Networks",
    "24.4723699": "Bbk Educational Electronics Corp.,Ltd.",
    "24.16008272": "BND Co., Ltd.",
    "12.123917689184256": "WPGSYS Pte Ltd",
    "24.9470095": "Huawei Device Co., Ltd.",
    "24.10527728": "D-Link International",
    "12.123917689978880": "TDI. Co., LTD",
    "12.346867130368": "Industrea Mining Technology",
    "20.255553902215168": "Hangzhou Lanxum Security Technology Co., Ltd",
    "24.2642656": "Layon international Electronic & Telecom Co.,Ltd",
    "20.228844753649664": "Viatron GmbH",
    "24.8439270": "Microsoft Corporation",
    "24.12111251": "Penetek, Inc",
    "24.8058": "WiWide Inc.",
    "24.8059": "TechNexion Ltd.",
    "12.123917682016256": "PTYPE Co., LTD.",
    "12.123917688311808": "Radar",
    "12.123917683470336": "Wavemed srl",
    "24.8050": "QingDao Hiphone Technology Co,.Ltd",
    "24.8051": "Teraview Technology Co., Ltd.",
    "24.8052": "Eigen Development",
    "24.8053": "GiBahn Media",
    "24.8054": "AirLogic Systems Inc.",
    "24.8055": "Heol Design",
    "24.8056": "Blue Fox Porini Textile",
    "24.8057": "Lodam Electronics A/S",
    "12.346852859904": "Log-In, Inc.",
    "12.346853294080": "Quest Innovations",
    "20.4884641349632": "Bureau Electronique Appliquee",
    "24.12097370": "BIOSTAR Microtech Int'l Corp.",
    "24.1081990": "Luxshare Precision Industry Co.,Ltd",
    "24.15774841": "Apple, Inc.",
    "12.123917682184192": "VECTARE Inc",
    "24.10486880": "Netgear",
    "24.10272978": "Conductix-Wampfler GmbH",
    "20.36069854674944": "Funkwerk Systems GmbH",
    "12.123917676875776": "MB connect line GmbH Fernwartungssysteme",
    "12.346855804928": "Control LAN S.A.",
    "12.346867347456": "ATOM GIKEN Co.,Ltd.",
    "24.5507469": "deister electronic GmbH",
    "12.123917676052480": "Gupsy GmbH",
    "24.3179604": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.13692004466688": "Eversec Technology Corporation",
    "24.4251": "Emulex Corporation",
    "12.346852679680": "JL-teknik",
    "24.12106671": "Samsung Electronics Co.,Ltd",
    "12.346861629440": "Cristal Controles Ltee",
    "24.12870378": "Samsung Electronics Co.,Ltd",
    "24.14966299": "Google, Inc.",
    "24.8439498": "Endian s.r.l.",
    "20.180659906674688": "Certi Networks Sdn Bhd",
    "12.346855297024": "American Microsystems, Ltd.",
    "20.31368103854080": "HABEY USA Inc.",
    "24.3158740": "Hanilstm Co., Ltd.",
    "24.13296901": "Mobilitie",
    "12.346867961856": "Organis GmbH",
    "24.2401580": "Huawei Technologies Co.,Ltd",
    "24.10812301": "Hangzhou Dunchong Technology Co.Ltd",
    "24.12091445": "Shenzhen Qihu Intelligent Technology Company Limited",
    "24.49862": "Intel Corporate",
    "24.1107630": "Source Technologies, LLC",
    "24.2891343": "Chengdu Qianli Network Technology Co., Ltd.",
    "24.8401392": "Samsung Electronics Co.,Ltd",
    "24.1358702": "Samsung Electronics Co.,Ltd",
    "20.145383548256256": "EDC Electronic Design Chemnitz GmbH",
    "12.346860105728": "Enertex Bayern GmbH",
    "24.7919029": "Navielektro Ky",
    "24.9739349": "Alta Data Technologies",
    "24.5513568": "Alula",
    "24.6353882": "Logic Way GmbH",
    "24.7091345": "Huawei Technologies Co.,Ltd",
    "12.123917687328768": "Abionic",
    "24.8929177": "Siemens AG",
    "24.9831": "Carecom Co.,Ltd.",
    "24.9830": "EFM Networks",
    "24.9833": "Nokia Danmark A/S",
    "24.9832": "Nokia Danmark A/S",
    "24.9835": "Shine Union Enterprise Limited",
    "24.9834": "Essensium Nv",
    "24.9837": "MobileAccess Networks",
    "24.9836": "Inventec Corporation",
    "24.9839": "Coordiwise Technology Corp.",
    "24.9838": "Nissho-denki Co.,LTD.",
    "24.2153725": "Huawei Device Co., Ltd.",
    "12.123917678813184": "Daifuku CO., Ltd.",
    "24.9718281": "BitWise Controls",
    "24.3707": "Toshiba",
    "24.908": "Total Impact",
    "24.3709": "Electronics Line 3000 Ltd.",
    "24.3708": "Televes S.A.",
    "24.1083598": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.12324762": "D-Link International",
    "24.11329096": "Private",
    "24.556952": "vivo Mobile Communication Co., Ltd.",
    "24.1089851": "Fujikura Rubber Ltd.",
    "12.119269437440": "JE Suunnittelu Oy",
    "12.123917688832000": "Leica Microsystems Ltd. Shanghai",
    "20.255861679194112": "DEHN SE + Co KG",
    "24.9714886": "Espressif Inc.",
    "12.346857721856": "Silent System",
    "12.123917685788672": "Unipower AB",
    "24.6548": "Jorjin Technologies Inc.",
    "24.14971618": "Huawei Technologies Co.,Ltd",
    "24.15258755": "ULTIMATE Europe Transportation Equipment GmbH",
    "12.346860482560": "GFI Chrono Time",
    "24.6549": "Jurong Hi-Tech (Suzhou)Co.ltd",
    "12.123917691604992": "Oceancctv Ltd",
    "12.346859266048": "MedAvant Healthcare",
    "24.566972": "Amazon Technologies Inc.",
    "24.15512808": "Wistron Mexico SA de CV",
    "20.229631765512192": "Duksanmecasys Co., Ltd.",
    "24.6544": "ELM DATA Co., Ltd.",
    "24.6545": "avinfo",
    "24.4472498": "DEOTRON Co., LTD.",
    "24.6546": "Adtran Inc",
    "24.8407572": "Apple, Inc.",
    "24.6547": "Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea",
    "24.12329146": "Inspur (Shandong) Electronic Information Co., Ltd",
    "24.6540": "iXSea",
    "24.6541": "Ocean Optics, Inc.",
    "12.346865569792": "Promess GmbH",
    "24.5269422": "Cisco Systems, Inc",
    "24.6542": "Oticon A/S",
    "24.6543": "Nokia Bell N.V.",
    "24.4782299": "Huawei Technologies Co.,Ltd",
    "24.6094847": "Shenzhen Kezhonglong Optoelectronic Technology Co., Ltd",
    "24.4778850": "Murata Manufacturing Co., Ltd.",
    "20.132855982194688": "Boneng Transmission(Suzhou)Co.,Ltd",
    "12.119269879808": "ENTE Sp. z o.o.",
    "24.10024247": "zte corporation",
    "12.346858717184": "Iac",
    "12.123917678706688": "Xitron",
    "24.14196071": "Hewlett Packard",
    "12.123917677096960": "Audiodo AB",
    "24.13148598": "Shenzhen Longway Technologies Co., Ltd",
    "24.1317821": "Volex Inc.",
    "12.346864238592": "Innovate Software Solutions Pvt Ltd",
    "12.123917685563392": "Quercus Technologies, S.L.",
    "24.14476664": "Experimental Factory of Scientific Engineering and Special Design Department",
    "12.346859675648": "Phytec Messtechnik GmbH",
    "12.123917684424704": "Mianjie Technology",
    "12.123917682954240": "TIECHE Engineered Systems",
    "24.14217458": "Zeebo Inc",
    "24.14216087": "TRENDnet, Inc.",
    "24.12614588": "Cisco Systems, Inc",
    "12.123917679124480": "Laser Imagineering Vertriebs GmbH",
    "20.277981937270784": "Nreal Technology Limited",
    "24.13638716": "Tecno Mobile Limited",
    "12.346866122752": "Solbrig Electronics, Inc.",
    "24.15764086": "Samsung Electronics Co.,Ltd",
    "12.346861408256": "Bachmann Monitoring GmbH",
    "24.9199695": "Cisco Systems, Inc",
    "12.71297883906048": "Ever Trend Technology Development Limited",
    "24.2158972": "Huawei Technologies Co.,Ltd",
    "24.1106887": "Seohwa Telecom",
    "20.62366873026560": "Dojo-Labs Ltd",
    "12.71297884344320": "ACD Elektronik GmbH",
    "24.804809": "Amazon Technologies Inc.",
    "24.16978": "RLX Technologies",
    "20.84531190169600": "Shenzhen Evolution Dynamics Co., Ltd.",
    "12.123917677604864": "Dave Srl",
    "12.123917682782208": "EMPELOR GmbH",
    "12.123917680955392": "Profotech",
    "24.9486617": "Nokia",
    "24.1896168": "Ilshin Elecom",
    "24.12610669": "MagneMotion, Inc.",
    "24.4741085": "Facebook Inc",
    "24.7868805": "NBS Payment Solutions Inc.",
    "24.1602866": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.7352414": "Nanjing Zhongke Menglian Information Technology Co.,LTD",
    "20.158667129225216": "Shenzhen Cloudynamo Internet Technologies Co.,LTD.",
    "24.13171144": "NewSharp Technology(SuZhou)Co,Ltd",
    "24.7352413": "Ubiquoss Inc",
    "24.10805478": "Filizola S.A. Pesagem E Automacao",
    "24.8681843": "Shanghai Baud Data Communication Co.,Ltd.",
    "24.14734568": "Intel Corporate",
    "12.123917680959488": "TinTec Co., Ltd.",
    "24.4253587": "Shenzhen Siviton Technology Co.,Ltd",
    "24.9728210": "Winfirm Technology",
    "24.10542487": "PolarLink Technologies, Ltd",
    "24.8413583": "Shenzhen sangfei consumer communications co.,ltd",
    "12.123917687169024": "Potter Electric Signal Co. LLC",
    "20.132855976951808": "Huaqin Telecom Technology Co.,Ltd.",
    "24.2913120": "Earda Technologies co Ltd",
    "24.12622544": "Apple, Inc.",
    "24.3447643": "Maquet GmbH",
    "24.8936082": "Cisco Systems, Inc",
    "24.11551087": "Shenzhen Maxtang Computer Co.,Ltd",
    "24.13893929": "Broadcom",
    "24.7654668": "Smith Myers Communications Ltd.",
    "12.123917682360320": "Melecs EWS GmbH",
    "24.40966": "Image Data Processing System Group",
    "24.10541167": "Argo Ai, Llc",
    "12.123917683445760": "Data Physics Corporation",
    "12.123917683871744": "Peek Traffic",
    "24.6555638": "Samsung Electronics Co.,Ltd",
    "20.93020692152320": "Elotech Industrieelektronik GmbH",
    "24.10749878": "Samsung Electronics Co.,Ltd",
    "24.7381683": "Apple, Inc.",
    "24.5826568": "Autonics Corporation",
    "24.1089082": "Zhejiang Tmall Technology Co., Ltd.",
    "24.6333648": "Samsung Electronics Co.,Ltd",
    "24.1071383": "Tonal",
    "12.123917685440512": "Tband srl",
    "24.11308340": "Huawei Technologies Co.,Ltd",
    "24.2940172": "Discovergy GmbH",
    "12.123917675171840": "Marimo electronics Co.,Ltd.",
    "12.123917681180672": "Dsp Design",
    "12.123917683200000": "Schneider Electric Motion USA",
    "24.8671386": "Easy Soft TV Co., Ltd",
    "12.123917675847680": "Innominds Software Inc",
    "20.31346169741312": "Fagus-GreCon Greten GmbH & Co. KG",
    "24.41120": "Compact Data, Ltd.",
    "24.271877": "GFR Gesellschaft für Regelungstechnik und Energieeinsparung mbH",
    "24.41122": "Digicom S.P.A.",
    "24.41123": "Reliable Power Meters",
    "24.41124": "Oracle Corporation",
    "24.41125": "Teknor Microsysteme, Inc.",
    "24.41126": "M.I. Systems, K.K.",
    "12.346864812032": "Keith & Koep GmbH",
    "24.41128": "Renex Corporation",
    "24.41129": "Navtel Communications Inc.",
    "20.452710105088": "Zhejiang Holip Electronic Technology Co.,Ltd",
    "12.346857267200": "GD Mission Systems",
    "12.123917689937920": "Sol Welding srl",
    "24.1636474": "i3 International Inc.",
    "12.123917685600256": "Potter Electric Signal Co. LLC",
    "24.11557663": "Thermo Fisher Scientific S.P.A.",
    "24.7361573": "ARRIS Group, Inc.",
    "12.123917676654592": "Era Toys Limited",
    "12.346852954112": "Casabyte",
    "12.123917676371968": "Innosonix GmbH",
    "24.10749294": "Tiinlab Corporation",
    "24.15476475": "Tecc Co.,Ltd.",
    "12.123917687902208": "RADA Electronics Industries Ltd.",
    "24.5785893": "E3 Enterprise",
    "24.1053435": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.7657534": "Intel Corporate",
    "24.1356932": "Samsung Electronics Co.,Ltd",
    "12.346857529344": "Alcon Technologies",
    "24.15470592": "Enance Source Co., Ltd.",
    "24.10548304": "HTC Corporation",
    "24.8423129": "Samsung Electronics Co.,Ltd",
    "24.11025244": "Huawei Technologies Co.,Ltd",
    "24.2638113": "OptiSense Network, LLC",
    "12.123917689675776": "microtec Sicherheitstechnik GmbH",
    "24.13953550": "Avaya Inc",
    "12.346857140224": "Westinghouse Rail Systems Ltd",
    "24.5849": "Ningbo Bird Co.,Ltd.",
    "24.16260367": "Hitron Technologies. Inc",
    "12.123917677629440": "Itg.Co.Ltd",
    "24.11304586": "Cisco Systems, Inc",
    "24.40299": "Murata Manufacturing Co., Ltd.",
    "12.123917679222784": "Acrodea, Inc.",
    "24.1367609": "Huawei Device Co., Ltd.",
    "12.123917677608960": "MOTION LIB,Inc.",
    "24.6325829": "Avery Weigh-Tronix, LLC",
    "12.123917682372608": "Koco Motion US LLC",
    "24.10501486": "Cisco Systems, Inc",
    "24.10501487": "Cisco Systems, Inc",
    "24.9496336": "New H3C Technologies Co., Ltd",
    "24.800732": "Huawei Technologies Co.,Ltd",
    "24.11013133": "Cisco Systems, Inc",
    "24.8686980": "Rajant Corporation",
    "24.5507413": "zte corporation",
    "24.303441": "Netgear",
    "24.8925322": "Murata Manufacturing Co., Ltd.",
    "24.7376711": "vivo Mobile Communication Co., Ltd.",
    "24.4210728": "Ziv",
    "12.346867990528": "Vitec Multimedia",
    "12.123917682475008": "Jeio Tech",
    "24.4210722": "Ziv",
    "24.7604961": "Huawei Device Co., Ltd.",
    "12.346868846592": "MB connect line GmbH Fernwartungssysteme",
    "12.346860285952": "Zoe Medical",
    "24.9972134": "Compal Information (Kunshan) Co., Ltd.",
    "12.123917681262592": "Forcite Helmet Systems Pty Ltd",
    "24.5273779": "zte corporation",
    "24.9736575": "Askey Computer Corp",
    "24.10538997": "zte corporation",
    "12.123917675581440": "Clover Electronics Technology Co., Ltd.",
    "24.13123225": "Texas Instruments",
    "24.9492303": "Bookeen",
    "24.8913394": "Vitec System Engineering Inc.",
    "20.159182268399616": "Act",
    "24.316612": "ASUSTek COMPUTER INC.",
    "24.15993759": "Cig Shanghai Co Ltd",
    "24.10016908": "Nortel Networks",
    "12.71297884774400": "KST technology",
    "24.3721146": "CS Services Co.,Ltd.",
    "12.71297883279360": "Nautel Limited",
    "24.7609056": "IEEE Registration Authority",
    "24.14724786": "Bender GmbH &amp; Co.KG",
    "12.346862010368": "IQ Automation GmbH",
    "12.119269707776": "Kitron GmbH",
    "24.5285683": "Ruckus Wireless",
    "24.13653109": "SAVI Controls",
    "24.4500048": "Juniper Networks",
    "20.124214331834368": "VOXX International",
    "24.4984775": "Earda Technologies co Ltd",
    "24.16270788": "Shenzhen Netforward Micro-Electronic Co., Ltd.",
    "24.11859342": "Huawei Technologies Co.,Ltd",
    "24.14955438": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.184991244877824": "Plejd AB",
    "24.37112": "Mediatrix Telecom",
    "24.3472746": "Apple, Inc.",
    "24.219": "British Telecommunications plc",
    "24.409": "HeiSei Electronics",
    "24.213": "Micrognosis International",
    "12.346852933632": "Infolink Software AG",
    "24.211": "Wang Laboratories Inc.",
    "24.210": "Sbe, Inc.",
    "24.217": "Nippon Telegraph & Telephone",
    "24.216": "old Novell NE1000's (before about 1987?) (also 3Com)",
    "24.215": "Dartmouth College (NED Router)",
    "24.214": "Punch Line Holding",
    "24.8962645": "Texas Instruments",
    "24.9749": "Teracom Limited",
    "12.346865082368": "Smartfield, Inc.",
    "24.37117": "CopperCom, Inc.",
    "24.402": "Texas Digital Systems",
    "12.123917677854720": "WideNorth AS",
    "24.403": "Hanbyul Telecom Co., Ltd.",
    "24.2397444": "Sfr",
    "24.14418533": "Meditech Kft",
    "12.346867605504": "West-Com Nurse Call Systems, Inc.",
    "24.9745": "Licera AB",
    "12.346865504256": "Micro Technic A/S",
    "24.1314626": "Nokia",
    "24.10528348": "Supercomputing Systems AG",
    "24.9744": "Apacewave Technologies",
    "24.53268": "Root, Inc.",
    "24.53269": "Univex Microtechnology Corp.",
    "24.1079387": "Actiontec Electronics, Inc",
    "24.53264": "Convergent Networks, Inc.",
    "24.9743": "Linn Products Ltd",
    "24.53266": "Gateworks Corp.",
    "24.53267": "Primex Aerospace Company",
    "24.53260": "Snijder Micro Systems",
    "24.53261": "Micromeritics Instrument",
    "24.53262": "Pluris, Inc.",
    "24.9742": "Ablaze Systems, LLC",
    "24.5497": "Lunatone Industrielle Elektronik GmbH",
    "24.5496": "Audio / Video Innovations",
    "24.5495": "Allied Telesis, Inc.",
    "24.5494": "LABiTec - Labor Biomedical Technologies GmbH",
    "24.5493": "Nevis Networks Inc.",
    "24.404": "Capital Equipment Corporation",
    "24.5491": "NewSoft Technology Corporation",
    "24.5490": "Red-Lemon",
    "24.9740": "Dataram",
    "24.5499": "Leuze electronic GmbH + Co. KG",
    "24.5498": "Telefin S.p.A.",
    "24.1938": "Sütron Electronic GmbH",
    "24.1939": "Shin Satellite Public Company Limited",
    "24.16314806": "Motorola Mobility LLC, a Lenovo Company",
    "20.207357801922560": "Lattec I/S",
    "24.2391840": "Sagemcom Broadband SAS",
    "24.1930": "Mentor Data System Inc.",
    "24.1931": "Wegener Communications, Inc.",
    "24.1932": "Elektronikspecialisten i Borlange AB",
    "24.1933": "NetEngines Ltd.",
    "24.1934": "Garz & Fricke GmbH",
    "24.1935": "Emkay Innovative Products",
    "24.1936": "Tri-M Technologies (s) Limited",
    "24.1937": "International Data Communications, Inc.",
    "24.851525": "Sony Interactive Entertainment Inc.",
    "20.44169487712256": "Tokin Limited",
    "24.3964337": "Texas Instruments",
    "12.346853937152": "Axes Technologies",
    "24.3890": "Lootom Telcovideo Network Wuxi Co Ltd",
    "24.3893": "Cisco Systems, Inc",
    "24.3892": "Cisco Systems, Inc",
    "24.3895": "Xambala Incorporated",
    "24.3894": "Accurate Techhnologies, Inc.",
    "24.3897": "Iris Sensors",
    "24.3896": "Netstar",
    "24.3899": "Fuji System Machines Co., Ltd.",
    "24.3898": "Hisharp",
    "24.5721": "Z.M.P. Radwag",
    "24.5720": "Fusiontech Technologies Inc.",
    "24.5723": "Grip Audio",
    "24.5722": "Harman Specialty Group",
    "24.5725": "AirDefense, Inc.",
    "24.5724": "Trackflow Ltd.",
    "24.5727": "Fairmount Automation",
    "24.1535": "SNS Solutions, Inc.",
    "24.5729": "Novatium Solutions (P) Ltd",
    "24.5728": "Nortel Networks",
    "24.5891": "MOSDAN Internation Co.,Ltd",
    "24.1537": "Otanikeiki Co., Ltd.",
    "12.123917681131520": "LYNX Technik AG",
    "24.1530": "IPOptical, Inc.",
    "24.1531": "ShareGate, Inc.",
    "12.123917687017472": "Grossenbacher Systeme AG",
    "24.3171179": "Apple, Inc.",
    "12.346862157824": "Pantel International",
    "24.1532": "Schenck Pegasus Corp.",
    "24.13945775": "Nokia Corporation",
    "24.10247795": "Calibre UK LTD",
    "24.1533": "PacketLight Networks Ltd.",
    "20.137141736701952": "Collinear Networks Inc.",
    "24.57398": "Pioneer Corporation",
    "24.57399": "Century Corporation",
    "24.3769": "HASHIMOTO Electronics Industry Co.,Ltd.",
    "24.3768": "Iiga co.,Ltd",
    "24.13671690": "Linkcom",
    "24.3763": "Hewlett Packard",
    "24.3762": "Micro-Research Finland Oy",
    "24.963": "Micronik Multimedia",
    "24.3760": "Solutions Radio BV",
    "24.3767": "Knovative, Inc.",
    "24.3766": "Riverbed Technology, Inc.",
    "24.3765": "Ecastle Electronics Co., Ltd.",
    "24.3764": "Guangzhou Gaoke Communications Technology Co.Ltd.",
    "24.12325470": "Beijing WisVideo INC.",
    "24.28851": "Data Recall Ltd.",
    "12.123917681553408": "DARBS Inc.",
    "24.1064426": "HFC Technology (Beijing) Ltd. Co.",
    "24.26983": "IEEE Registration Authority",
    "24.16542704": "Topwell International Holdinds Limited",
    "24.2935308": "Withus Planet",
    "24.321142": "AMPAK Technology, Inc.",
    "24.1582665": "Intel Corporate",
    "24.11062469": "Saab AB (publ)",
    "24.12353403": "Huawei Device Co., Ltd.",
    "24.12874389": "Cisco Systems, Inc",
    "12.123917678579712": "ALVAT s.r.o.",
    "24.21663": "Avaya Inc",
    "12.123917691068416": "Alere Technologies AS",
    "24.12103527": "Hewlett Packard",
    "24.4254873": "Intel Corporate",
    "12.346861826048": "BigBear",
    "24.9758475": "Intel Corporate",
    "24.16521488": "Samsung Electronics Co.,Ltd",
    "24.2409546": "zte corporation",
    "24.7286": "The Wandsworth Group Ltd",
    "12.346859716608": "CardioMEMS Inc.",
    "24.12858065": "Farlink Technology Limited",
    "12.346866094080": "Smith Meter, Inc",
    "24.6356310": "WOORISYSTEMS CO., Ltd",
    "24.9734915": "EAGET Innovation and Technology Co., Ltd.",
    "24.9589": "FiberPlex Technologies, LLC",
    "24.9588": "KUNIMI MEDIA DEVICE Co., Ltd.",
    "24.9696": "CeedTec Sdn Bhd",
    "24.9581": "Broadband Forum",
    "24.9580": "\"Azimut\" Production Association JSC",
    "24.9583": "Dantherm Power",
    "24.9582": "Van Breda B.V.",
    "24.9585": "Zhejiang Tianle Digital Electric Co.,Ltd",
    "24.9584": "Eastern Communications Company Limited",
    "24.9587": "ST Electronics (Info-Security) Pte Ltd",
    "24.9586": "Nemo-Q International AB",
    "24.6312656": "FACTS Engineering",
    "24.9455287": "Vadaro Pte Ltd",
    "24.9691": "ATI Electronics(Shenzhen) Co., LTD",
    "12.346855129088": "Integrated System Solution Corp.",
    "24.1599408": "Samsung Electronics Co.,Ltd",
    "24.15524265": "Cisco Systems, Inc",
    "24.817685": "Intel Corporate",
    "24.3729804": "Reco S.p.A.",
    "24.9299": "Initra d.o.o.",
    "24.6305430": "The Sapling Company",
    "12.123917685399552": "Blue Storm Associates, Inc.",
    "12.123917688963072": "Cubic ITS, Inc. dba GRIDSMART Technologies",
    "24.4775163": "Nokia Corporation",
    "24.9690": "Secura Key",
    "24.6824287": "Edgecore Networks Corporation",
    "24.8978134": "ShangHai WangYong Software Co., Ltd.",
    "24.4478605": "PNC Technologies Co., Ltd.",
    "12.346858086400": "Plastic Logic",
    "24.8929729": "Orbisat Da Amazonia Ind E Aerol Sa",
    "20.123731224559616": "Beijing Strongleader Science & Technology Co., Ltd.",
    "24.8988": "Fourier Systems Ltd.",
    "24.8989": "Deltacom Electronics Ltd",
    "24.8986": "ITF Co., Ltd.",
    "24.8987": "Danaher Motion - Kollmorgen",
    "24.8984": "Toshiba",
    "24.2660807": "Lindsay Broadband Inc",
    "24.8982": "Kisan Electronics Co",
    "24.8983": "Lasercraft Inc",
    "24.8980": "Intel Corporate",
    "24.8981": "Intel Corporate",
    "24.2098191": "Dell Inc.",
    "12.346861043712": "Acoustic Research Laboratories Pty Ltd",
    "24.9292": "Solartron Metrology Ltd",
    "24.7375736": "citygrow technology co., ltd",
    "20.105645761101824": "RTDS Technologies Inc.",
    "24.13115370": "Avaya Inc",
    "20.58144480821248": "Teton Camera LLC",
    "24.8678": "Starlight Video Limited",
    "24.8679": "Informatics Services Corporation",
    "24.6086000": "Hangzhou H3C Technologies Co., Limited",
    "12.123917685682176": "Botek Systems AB",
    "24.8670": "Firepro Wireless",
    "24.8671": "Martin Christ GmbH",
    "24.8672": "CommAgility Ltd",
    "24.8673": "Nortel Networks",
    "24.8674": "visago Systems & Controls GmbH & Co. KG",
    "24.8675": "SerialTek LLC",
    "24.8676": "I-Win",
    "24.8677": "Display Solution AG",
    "20.128612175970304": "GHL Advanced Technology GmbH & Co. KG",
    "24.3199789": "devolo AG",
    "24.4761298": "GJsun Optical Science and Tech Co.,Ltd.",
    "20.132811176542208": "Daincube",
    "12.123917680394240": "Wuxi New Optical Communication Co.,Ltd.",
    "24.11805184": "Brother Industries, LTD.",
    "24.1581": "TouchStar Technologies, L.L.C.",
    "24.1580": "Bivio Networks",
    "24.1583": "Pivotech Systems Inc.",
    "24.1582": "Aristos Logic Corp.",
    "24.1585": "Calix Inc.",
    "24.1584": "Adtranz Sweden",
    "24.1587": "Cross Match Technologies GmbH",
    "12.346868740096": "DINTEK Shanghai Electronic Ltd",
    "24.1589": "PacketAir Networks, Inc.",
    "24.1588": "GTE Airfone Inc.",
    "24.49395": "Network Communications Corp.",
    "12.346861113344": "ACS Motion Control Ltd.",
    "20.88383266226176": "Hypertech Advance Co., LTD",
    "24.49394": "Transition Engineering Inc",
    "24.16291306": "Apple, Inc.",
    "24.49393": "Shinko Electric Co., Ltd.",
    "24.49392": "Kingston Technology Corporation",
    "24.15206725": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.8181345": "Xiaomi Communications Co Ltd",
    "24.49391": "Abit Corporation",
    "24.13956591": "Hewlett Packard Enterprise",
    "12.346863906816": "Numcore Ltd",
    "12.123917690589184": "Solar Network & Partners",
    "24.12889618": "Samsung Electronics Co.,Ltd",
    "24.6609": "Intel Corporate",
    "24.6608": "Cathexis",
    "24.2588": "Bridge Information Co., Ltd.",
    "24.2589": "Optical Communications Products Inc.",
    "24.8418963": "Newag SA",
    "24.6603": "Zyxel Communications Corporation",
    "24.6602": "Broadata Communications, Inc",
    "24.6601": "S&C Electric Company",
    "24.6600": "AnyDATA Corporation",
    "24.6607": "Salicru, S.A.",
    "24.6606": "Progressive Gaming International",
    "24.6605": "Chengdu ethercom information technology Ltd.",
    "24.6604": "RCG (HK) Ltd",
    "24.1099276": "Domino UK Ltd",
    "24.1059": "Intel Corporation",
    "24.1058": "Studio Technologies, Inc",
    "24.1053": "Corega of America",
    "24.1052": "ipDialog, Inc.",
    "24.1051": "Bridgeworks Ltd.",
    "24.2680724": "Microtime Computer Inc.",
    "24.1057": "Ocular Networks",
    "24.1056": "Slim Devices, Inc.",
    "24.1055": "Sony Interactive Entertainment Inc.",
    "24.1054": "Shikoku Instrumentation Co., Ltd.",
    "24.7902388": "Sercomm Corporation.",
    "24.5038494": "Tecno Mobile Limited",
    "24.567945": "zte corporation",
    "24.13110472": "Huawei Technologies Co.,Ltd",
    "24.16550": "Cray Research Inc.",
    "20.229206575284224": "Accumulate AB",
    "24.13395466": "Apple, Inc.",
    "24.14210489": "Rainforest Automation, Inc.",
    "12.346853593088": "Cepheid",
    "24.12641325": "Tp-Link Technologies Co.,Ltd.",
    "24.9471643": "zte corporation",
    "12.123917677588480": "Voltaware Limited",
    "12.346852212736": "Vbrick Systems Inc.",
    "24.7143358": "MPB Communications Inc.",
    "24.16281537": "Apple, Inc.",
    "24.12331078": "NextBIT Computing Pvt. Ltd.",
    "12.123917680807936": "Cityntel OU",
    "12.123917677555712": "Compal Electronics, Inc.",
    "24.5033066": "Micro-Star INTL CO., LTD.",
    "12.123917676142592": "Momentum Data Systems",
    "12.346868092928": "Ruetz Technologies GmbH",
    "24.11332010": "Hay Systems Ltd",
    "24.11861912": "Huawei Technologies Co.,Ltd",
    "24.2681230": "Mitsubishi Electric Corporation",
    "24.10286618": "Carrier Fire & Security",
    "20.180659901431808": "DGC Access AB",
    "12.346856058880": "Mindspeed Technologies",
    "24.9208": "Zyxel Communications Corporation",
    "20.61822321295360": "Alicat Scientific",
    "24.9206": "Softwell Technology Co., Ltd.",
    "24.9207": "Private",
    "24.9204": "Masternaut",
    "24.9205": "Wilo Se",
    "24.9202": "TVLogic",
    "24.9203": "Glocom, Inc.",
    "24.9200": "Shanghai Jinghan Weighing Apparatus Co. Ltd.",
    "24.9201": "Sony Mobile Communications Inc",
    "12.123917690470400": "ProtoConvert Pty Ltd",
    "24.16289894": "Nain Inc.",
    "24.2378750": "ASUSTek COMPUTER INC.",
    "12.346859900928": "Elbit Systems of America",
    "12.123917684461568": "Wexiodisk AB",
    "24.2378759": "Huawei Technologies Co.,Ltd",
    "20.194284246532096": "Voxiscom",
    "24.849316": "Cisco Systems, Inc",
    "20.48548667195392": "Electronique Bluewave Inc.",
    "24.15259857": "HP Inc.",
    "24.12904989": "Technicolor",
    "24.8183184": "Shenzhen Ogemray Technology Co., Ltd.",
    "24.2675766": "Jiangsu dewosi electric co., LTD",
    "20.22126508638208": "Newings Communication CO., LTD.",
    "12.71297884139520": "WANTECH Networks",
    "12.346858975232": "Phytec Messtechnik GmbH",
    "12.71297884164096": "Grossenbacher Systeme AG",
    "24.13914775": "nSTREAMS Technologies, Inc.",
    "24.524289": "Computervision Corporation",
    "24.14706611": "Shenzhen C-Data Technology Co., Ltd",
    "12.123917688385536": "Fidalia Networks Inc",
    "24.3205925": "Intek Digital",
    "12.346867609600": "TimeTerminal Adductor Group AB",
    "12.346858188800": "Techimp Systems S.r.l.",
    "20.229108118192128": "Decathlon SA",
    "24.14294049": "Sesam GmbH",
    "24.14988056": "Intel Corporate",
    "24.4457260": "Intel Corporate",
    "24.14685109": "vivo Mobile Communication Co., Ltd.",
    "24.14964648": "Stuart Manufacturing, Inc.",
    "12.123917678620672": "Hangzhou Weimu Technology Co.,Ltd.",
    "24.4462663": "Hunan Scrown Electronic Information Tech.Co.,Ltd",
    "24.569208": "Totus Solutions, Inc.",
    "24.2892602": "Hewlett Packard",
    "12.346867384320": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.9994478": "Samsung Electronics Co.,Ltd",
    "24.12629753": "Intel Corporate",
    "24.16052126": "Sgsg Science & Technology Co. Ltd",
    "24.8668926": "Huawei Technologies Co.,Ltd",
    "24.2657464": "zte corporation",
    "24.5268053": "IEEE Registration Authority",
    "24.5810233": "Intel Corporate",
    "24.10262944": "Xiaomi Communications Co Ltd",
    "12.346853404672": "Haag-Streit AG",
    "24.14456896": "Microsoft Corporation",
    "12.123917677481984": "Mimo Networks",
    "24.5507344": "Apple, Inc.",
    "20.18540142788608": "Pickering Interfaces Ltd",
    "12.123917680009216": "BlueBox Video Limited",
    "12.123917675327488": "EVPU Defence a.s.",
    "20.154055559086080": "Electronic Controlled Systems, Inc.",
    "12.123917686489088": "Tattile Srl",
    "24.16024545": "Shanghai Clouder Semiconductor Co.,Ltd",
    "24.506": "Horoscas",
    "24.3159469": "Apple, Inc.",
    "24.14167451": "Shenzhen G.Credit Communication Technology Co., Ltd",
    "24.15991585": "BeNeXt B.V.",
    "20.225209245237248": "Evoko Unlimited AB",
    "12.346859876352": "DMT System S.p.A.",
    "12.123917679009792": "Imagine Inc.",
    "24.4506523": "Ooo Rubezh Npo",
    "24.2365638": "Hunan Fn-Link Technology Limited",
    "12.346857066496": "OpenPeak, Inc.",
    "12.123917686669312": "ICT BUSINESS GROUP of Humanrights Center for disabled people",
    "12.346852425728": "MiSPO Co., Ltd.",
    "24.8617": "Mobilink Telecom Co.,Ltd",
    "24.11312663": "ASUSTek COMPUTER INC.",
    "12.123917678981120": "GeoSpectrum Technologies Inc",
    "24.3179416": "Espressif Inc.",
    "24.6555113": "Shenzhen WayOS Technology Crop., Ltd.",
    "24.11324090": "Midokura Co., Ltd.",
    "24.1884950": "MileSight Technology Co., Ltd.",
    "12.123917683462144": "Cjsc «Interset»",
    "20.136823634395136": "Digital Lumens",
    "12.346868240384": "Honeywell",
    "20.154055556988928": "Alcidae Inc",
    "24.12105713": "Wistron Neweb Corporation",
    "24.4760723": "Taiyo Yuden Co.,Ltd",
    "20.88383273566208": "CCTV Manufacturer",
    "24.6347168": "Lenovo Mobile Communication Technology Ltd.",
    "12.123917684342784": "Preo Industries Far East Ltd",
    "12.346863620096": "3Roam",
    "12.346854313984": "Varisys Ltd",
    "20.149935433449472": "Wuhan Strong Electronics Co., Ltd",
    "12.346868871168": "Task Sistemas de Computacao",
    "12.123917688324096": "Planewave Instruments",
    "12.123917684199424": "HMicro Inc",
    "24.2625960": "Intel Corporate",
    "20.242876973842432": "Brck",
    "12.123917686009856": "Audio Visual Digital Systems",
    "24.7391218": "Huawei Technologies Co.,Ltd",
    "24.11069470": "Atw Technology, Inc.",
    "12.123917687726080": "Skyriver Communications Inc.",
    "24.11069476": "Inim Electronics S.R.L.",
    "12.346864848896": "Ibercomp SA",
    "20.110373793431552": "Jiangsu Aisida Electronic Co.,Ltd",
    "12.346857611264": "NVE Corporation",
    "24.6347299": "Estech Co.,Ltd",
    "12.346852339712": "Nova Engineering Inc.",
    "24.586218": "Hewlett Packard Enterprise",
    "12.123917690859520": "HKC Limited",
    "24.4500200": "Nanotec Electronic GmbH & Co. KG",
    "24.526570": "Amsc",
    "12.346852331520": "Grossenbacher Systeme AG",
    "24.14732220": "Cisco Meraki",
    "24.13397487": "TJK Tietolaite Oy",
    "24.14217712": "Pepxim International Limited",
    "24.13938960": "Huawei Technologies Co.,Ltd",
    "24.545044": "Qingdao Topscomm Communication Co., Ltd",
    "24.7367033": "IEEE Registration Authority",
    "24.14957157": "MofiNetwork Inc",
    "24.10544436": "Stratec Biomedical AG",
    "12.346856689664": "HAMEG GmbH",
    "12.123917689540608": "Bio Rad Laboratories",
    "12.346863591424": "Vecom Usa",
    "24.301542": "Shenzhen Yoostar Technology Co., Ltd",
    "12.71297884409856": "Biotage Sweden AB",
    "24.554354": "Obelux Oy",
    "12.123917686140928": "Master Meter Inc.",
    "20.31477069774848": "Cirque Audio Technology Co., Ltd",
    "24.12618848": "Cisco Systems, Inc",
    "24.10261326": "Xerox Corporation",
    "12.346853953536": "Thrane & Thrane A/S",
    "12.346860691456": "Winner Technology Co, Ltd.",
    "24.1073108": "Tendyron Corporation",
    "20.224447327895552": "Shanghai Cargoa M.&E.Equipment Co.Ltd",
    "12.346861576192": "TELIUM s.c.",
    "24.4718641": "Huawei Technologies Co.,Ltd",
    "24.4718643": "Technicolor CH USA Inc.",
    "12.346854322176": "OMICRON electronics GmbH",
    "24.13410912": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.16533805": "Amazon Technologies Inc.",
    "24.13387523": "Intercon Systems Co., Ltd.",
    "24.5805218": "KETEK GmbH",
    "24.1897114": "Google, Inc.",
    "24.543439": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.14189006": "Ahn Inc.",
    "24.4457335": "IEEE Registration Authority",
    "24.2416490": "Huawei Technologies Co.,Ltd",
    "24.6331181": "JMACS Japan Co., Ltd.",
    "24.8430133": "Edgecore Networks Corporation",
    "12.346861473792": "Deutscher Weterdienst",
    "12.123917676634112": "Nippon Seiki Co., Ltd.",
    "12.123917682577408": "Zigencorp, Inc",
    "12.346853416960": "Procon Electronics",
    "20.132869120851968": "Shenzhen Chenzhuo Technology Co., Ltd.",
    "12.119269400576": "Analytical Instrument Systems, Inc.",
    "12.71297884794880": "Diagnosys Test Systems Ltd",
    "24.4216985": "Logitech Far East",
    "12.123917685981184": "Bionics co.,ltd.",
    "24.13594": "Cisco Systems, Inc",
    "12.123917688700928": "Contec Americas Inc.",
    "24.840338": "Intel Corporate",
    "24.2934811": "Routerboard.com",
    "12.346864701440": "Ailes Electronics Co., Ltd.",
    "24.3694438": "Cisco SPVTG",
    "12.346855395328": "ifak system GmbH",
    "24.4855": "Xiamen Xinglian Electronics Co., Ltd.",
    "12.346857865216": "Ellex Medical Pty Ltd",
    "12.123917682262016": "Crde",
    "24.4038": "Eurocom Industries A/S",
    "24.4039": "Dionica R&D Ltd.",
    "12.346864574464": "BG Systems, Inc.",
    "24.11154644": "CDVI Wireless SpA",
    "24.4032": "DELCOMp",
    "24.4033": "WAVE Corporation",
    "24.4030": "e-w/you Inc.",
    "24.4031": "DGT Sp. z o.o.",
    "24.4036": "NST co.,LTD.",
    "24.4037": "KeyMed Ltd",
    "24.4034": "Uniwell Corporation",
    "24.4035": "PalmPalm Technology, Inc.",
    "24.3569": "Ionix Inc.",
    "24.3568": "Qcom Technology Inc.",
    "24.3561": "Napatech Aps",
    "24.3560": "Nasaco Electronics Pte. Ltd",
    "24.3563": "CompXs Limited",
    "24.3562": "Kingtel Telecommunication Corp.",
    "24.3565": "Cisco Systems, Inc",
    "24.3564": "Cisco Systems, Inc",
    "24.3567": "Soc. Coop. Bilanciai",
    "24.3566": "Andrew RF Power Amplifier Group",
    "24.14179716": "CAx soft GmbH",
    "12.346867269632": "Kyoritsu Electric Corporation",
    "12.346854580224": "Scharff Weisberg Systems Integration Inc",
    "24.2151007": "SmartCap Technologies",
    "24.6833111": "Agilecom Photonics Solutions Guangdong Limited",
    "24.3443573": "Lava International(H.K) Limited",
    "12.123917688934400": "Sicon srl",
    "24.3201497": "Apple, Inc.",
    "24.4858": "Thx Ltd",
    "24.557113": "Cisco SPVTG",
    "24.14203841": "Micro-Star INTL CO., LTD.",
    "12.123917687377920": "Sasken Technologies Ltd",
    "24.5529513": "Teracom Limited",
    "24.13933608": "JayBird LLC",
    "24.15744656": "Ruckus Wireless",
    "20.211525436440576": "Nanjing SinoVatio Technology Co., Ltd",
    "24.14423898": "Hitachi High-Technologies Corporation",
    "24.2387082": "YUKAI Engineering",
    "20.220092028682240": "RF Engineering and Energy Resource",
    "12.123917675266048": "Savari Inc",
    "24.1896398": "bebro electronic GmbH",
    "24.6661": "Optibase Ltd",
    "24.12347945": "Texas Instruments",
    "24.9239832": "Huawei Technologies Co.,Ltd",
    "24.6660": "Interay Solutions BV",
    "24.5037437": "LHP Telematics LLC",
    "24.6663": "Arecont Vision",
    "12.123917688082432": "MB connect line GmbH Fernwartungssysteme",
    "24.6662": "OpVista, Inc.",
    "24.1364255": "Huawei Technologies Co.,Ltd",
    "24.6665": "Wayfarer Transit Systems Ltd",
    "12.123917679575040": "AlyTech",
    "24.7086166": "Cisco Systems, Inc",
    "24.6664": "Simoco Ltd.",
    "24.14204864": "Espressif Inc.",
    "24.12335314": "Pandachip Limited",
    "24.6667": "Bona Technology Inc.",
    "24.8436396": "TeleAdapt Ltd",
    "24.6666": "Adaptive Micro-Ware Inc.",
    "20.141268014333952": "Kaynes Technology India Pvt Ltd",
    "24.12348": "Onnto Corp.",
    "24.12349": "Iva Corporation",
    "24.12344": "Xcp, Inc.",
    "24.12345": "Softbook Press",
    "24.12346": "Maatel",
    "24.12347": "PowerCom Technology",
    "24.12340": "Set Engineering",
    "24.12341": "Corning Incorporated",
    "24.12342": "Rmp Elektroniksysteme Gmbh",
    "24.12343": "Packard Bell Nec Services",
    "24.5247467": "SilverNet Ltd",
    "20.13592316346368": "iH&S Technology Limited",
    "24.303050": "Huawei Technologies Co.,Ltd",
    "12.346862256128": "Trs Systems, Inc.",
    "12.123917688676352": "Private",
    "12.346853609472": "J&B Engineering Group S.L.",
    "24.15227831": "Ample Systems Inc.",
    "24.13127558": "Sophos Ltd",
    "24.4488319": "Innolight Technology Corporation",
    "20.88383270420480": "AVTECH Software, Inc.",
    "24.1110021": "Intel Corporate",
    "24.4229895": "Actiontec Electronics, Inc",
    "20.171812089495552": "\"CONTINENT\" Co. Ltd",
    "20.52819390365696": "Hurray Cloud Technology Co., Ltd.",
    "12.123917680721920": "Cloud Intelligence Pty Ltd",
    "24.7718": "Digifriends Co. Ltd",
    "24.7719": "SBN TECH Co.,Ltd.",
    "24.8177348": "Kolff Computer Supplies b.v.",
    "20.176772040622080": "Oray.com co., LTD.",
    "24.7712": "Intertain Inc.",
    "24.7713": "Qisda Corporation",
    "24.7710": "Honeywell Life Safety",
    "24.7711": "Nortel Networks",
    "24.7716": "Zhejiang Bell Technology Co.,ltd",
    "24.7717": "Intek Digital",
    "24.7714": "ARVOO Imaging Products BV",
    "24.7715": "Electronic Educational Devices, Inc",
    "12.346862047232": "Keene Electronics Ltd.",
    "12.123917677473792": "GID Industrial",
    "12.123917684768768": "CMT Medical technologies",
    "20.71070631591936": "Onion Corporation",
    "24.8129782": "Guangdong Huiwei High-tech Co., Ltd.",
    "24.3723978": "Zhejiang Tmall Technology Co., Ltd.",
    "24.7355096": "Shenzhen Afoundry Electronic Co., Ltd",
    "12.123917689225216": "Aplex Technology Inc.",
    "24.2929722": "Apple, Inc.",
    "24.2375472": "Oxygen Broadband s.a.",
    "24.9238871": "RuiXingHengFang Network (Shenzhen) Co.,Ltd",
    "24.12646832": "Huawei Technologies Co.,Ltd",
    "24.14481238": "Samsung Electronics Co.,Ltd",
    "24.14481237": "Sitronik",
    "24.7388990": "Charles River Laboratories",
    "24.13921498": "CSM GmbH",
    "24.7139270": "Shenzhen Twowing Technologies Co.,Ltd.",
    "24.6050925": "Huawei Technologies Co.,Ltd",
    "12.346854776832": "AnaLogic Computers Ltd.",
    "24.2903677": "Chunghua Intelligent Network Equipment Inc.",
    "24.2415121": "NO NDA Inc",
    "24.11328328": "ARRIS Group, Inc.",
    "24.8695181": "Apple, Inc.",
    "24.4726128": "Cisco SPVTG",
    "24.6059863": "Haivision Network Video",
    "24.8951622": "Sichuan AI-Link Technology Co., Ltd.",
    "24.7399962": "Cisco Systems, Inc",
    "24.2904906": "Shenzhen Winyao Electronic Limited",
    "12.346854273024": "Vocality International Ltd",
    "24.13420005": "Dell Inc.",
    "24.4195721": "T-Mobile, USA",
    "24.6049": "3soft inc.",
    "24.6048": "RoboTech srl",
    "24.2748": "Seabridge Ltd.",
    "24.2749": "Rupprecht & Patashnick Co.",
    "24.8436090": "eero inc.",
    "12.123917684105216": "Fareco",
    "24.6041": "SmarTire Systems Inc.",
    "24.6040": "Azonic Technology Co., LTD",
    "24.6043": "Chant Sincere Co.,Ltd",
    "24.2741": "Digital Electronic Network",
    "24.6045": "Kelman Limited",
    "24.6044": "DEPRAG SCHULZ GMBH u. CO.",
    "24.6047": "Apricorn",
    "24.6046": "Sirit Inc",
    "24.1219": "CASTOR Informatique",
    "24.1218": "Magnipix, Inc.",
    "12.346862526464": "E-ViEWS SAFETY SYSTEMS, INC",
    "24.2123933": "Liteon Technology Corporation",
    "24.1211": "Bardac Corporation",
    "24.1210": "KDD Media Will Corporation",
    "12.119269556224": "TeliSwitch Solutions",
    "24.1212": "Giantec, Inc.",
    "24.286027": "Sony Corporation",
    "24.1214": "OptXCon, Inc.",
    "12.346858237952": "Kobold Sistemi s.r.l.",
    "24.1216": "Cisco Systems, Inc",
    "24.14206185": "Samsung Electronics Co.,Ltd",
    "20.14286633041920": "Yinuo-Link Limited",
    "12.123917677596672": "Medicomp, Inc",
    "24.14993186": "Cisco Systems, Inc",
    "24.2162395": "M2M Solution S.A.S.",
    "20.146101711667200": "iSolution Technologies Co.,Ltd.",
    "24.16296933": "shenzhen worldelite electronics co., LTD",
    "24.12359488": "IEEE Registration Authority",
    "12.71297883586560": "Hipodromo De Agua Caliente, S.A. De C.V.",
    "24.3933375": "PRAVIS SYSTEMS Co.Ltd.,",
    "24.10785201": "Technicolor",
    "24.3169592": "Beissbarth",
    "12.346861703168": "AHV Systems, Inc.",
    "24.3995556": "Samsung Electronics Co.,Ltd",
    "12.346866446336": "BitWise Controls",
    "24.16014473": "Apple, Inc.",
    "24.1856670": "Universal Electronics, Inc.",
    "12.346856837120": "ISEPOS GmbH",
    "20.106480354197504": "Nextivity",
    "24.4999": "27M Technologies AB",
    "24.4998": "ABB Inc/Totalflow",
    "24.4995": "Application Technologies and Engineering Research Laboratory",
    "24.4994": "Cetacea Networks Corporation",
    "24.4997": "Add-On Technology Co., LTD.",
    "24.4996": "Advanced Motion Controls",
    "24.4991": "Cisco Systems, Inc",
    "24.4990": "CorEdge Networks, Inc.",
    "24.4993": "CHIPS & Systems, Inc.",
    "24.4992": "Cisco Systems, Inc",
    "20.137141732507648": "Sun Asia Trade Co.",
    "24.4983357": "Adnacom Inc.",
    "24.12845436": "Ruckus Wireless",
    "24.8174724": "IEEE Registration Authority",
    "24.15519085": "Allied Telesis, Inc.",
    "24.12086051": "Toshiba",
    "24.55761": "Sony Interactive Entertainment Inc.",
    "20.224447334187008": "shenzhen zonglian network technology limited",
    "12.119270002688": "Bridge Diagnostics, Inc.",
    "12.346864398336": "iDeal Teknoloji Bilisim Cozumleri A.S.",
    "24.56498": "Extreme Networks, Inc.",
    "24.3967380": "BSkyB Ltd",
    "24.11026497": "Dragino Technology Co., Limited",
    "12.123917676150784": "Macromatic Industrial Controls, Inc.",
    "12.123917675429888": "Onethinx BV",
    "24.22221": "Apple, Inc.",
    "24.2419926": "CSG Science & Technology Co.,Ltd.Hefei",
    "12.123917690294272": "BroadSoft Inc",
    "24.5022": "Ciara Technologies Inc.",
    "24.5023": "Electronics Design Services, Co., Ltd.",
    "24.5020": "Exavera Technologies, Inc.",
    "24.5021": "MaxLinear Hispania S.L.U.",
    "24.5026": "MaxStream, Inc",
    "24.5027": "Siemens Home & Office Comm. Devices",
    "24.5024": "ALGOSYSTEM Co., Ltd.",
    "24.5025": "Crow Electronic Engeneering",
    "24.5028": "KeyEye Communications",
    "24.5029": "General Solutions, LTD.",
    "24.15756850": "Microtel Innovation S.R.L.",
    "24.4371": "Fraunhofer FOKUS",
    "24.4370": "Honeywell CMSS",
    "24.4373": "EPIN Technologies, Inc.",
    "24.3438870": "Huawei Technologies Co.,Ltd",
    "24.4375": "Cesnet",
    "24.4374": "Coteau Vert Co., Ltd.",
    "24.4377": "Solteras, Inc.",
    "24.4376": "BLX IC Design Corp., Ltd.",
    "24.4379": "Targa Systems Div L-3 Communications",
    "24.4378": "ARRIS Group, Inc.",
    "12.123917687832576": "Peek Traffic",
    "24.12116974": "Universal Electronics, Inc.",
    "24.12081014": "lignex1",
    "20.119143426686976": "krtkl inc.",
    "24.53387": "ADVA Optical Networking Ltd.",
    "24.53386": "Photron Usa",
    "24.53385": "Dynacolor, Inc.",
    "24.53384": "ARRIS Group, Inc.",
    "24.53383": "Microfirst Inc.",
    "24.53382": "Foveon, Inc.",
    "24.53381": "Otis Elevator Company",
    "24.53380": "Nexcomm Systems, Inc.",
    "24.5026201": "Apple, Inc.",
    "12.346854334464": "4Links Limited",
    "24.53389": "Phoenix Group, Inc.",
    "24.53388": "Genoa Technology, Inc.",
    "12.123917681352704": "Hdanywhere",
    "12.123917691691008": "Heng Dian Technology Co., Ltd",
    "24.7908283": "Samsung Electronics Co.,Ltd",
    "24.16524069": "EosTek (Shenzhen) Co., Ltd.",
    "12.346861637632": "Elisra",
    "12.346862682112": "LabJack Corporation",
    "20.163603976028160": "SynchronicIT BV",
    "24.6293955": "u-blox AG",
    "24.1626309": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.9208833": "Private",
    "24.1583703": "Huawei Technologies Co.,Ltd",
    "24.8928139": "Cheering Connection Co. Ltd.",
    "20.207357807165440": "Airfi Oy AB",
    "24.16262066": "Huawei Technologies Co.,Ltd",
    "24.13931812": "Clover Network, Inc.",
    "12.123917687771136": "Ensotech Limited",
    "12.346866143232": "Sterna Security",
    "24.3466925": "Intel Corporate",
    "24.14995748": "Mojo Networks, Inc.",
    "24.13394107": "Empower RF Systems",
    "12.346854690816": "Technisyst Pty Ltd",
    "24.13389348": "Brocade Communications Systems LLC",
    "24.10789314": "Ericsson AB",
    "24.7374750": "Beken Corporation",
    "12.123917688999936": "WyreStorm Technologies Ltd",
    "24.12368966": "SKS Welding Systems GmbH",
    "24.11816130": "Nr Electric Co., Ltd",
    "12.346864185344": "Kristech",
    "24.4492929": "Vig",
    "12.123917687410688": "Synics AG",
    "12.123917689483264": "KLS Netherlands B.V.",
    "24.5822915": "Motorola Mobility LLC, a Lenovo Company",
    "20.66129343021056": "Shenzhen Pinsida Technology Co.,Ltd.",
    "12.123917687508992": "Itrinegy Ltd.",
    "24.14196287": "Apple, Inc.",
    "12.346860908544": "Grupo Epelsa S.L.",
    "12.123917677240320": "Marioff Corporation Oy",
    "24.16551805": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917677178880": "ETON Deutschland Electro Acoustic GmbH",
    "24.14297324": "HG Genuine",
    "24.567457": "Cyclotronics Power Concepts, Inc",
    "24.5552854": "Huawei Technologies Co.,Ltd",
    "24.6829411": "Shenzhen Liown Electronics Co.,Ltd.",
    "24.14192502": "Apple, Inc.",
    "24.16553539": "Juniper Networks",
    "24.828148": "Chameleon Technology (UK) Limited",
    "20.154238417108992": "TelWare Corporation",
    "24.10282941": "Winduskorea co., Ltd",
    "24.9468": "QuoPin Co., Ltd.",
    "24.9469": "Accedian Networks Inc",
    "24.9466": "Hilger u. Kern GMBH",
    "24.9467": "Private",
    "24.9464": "Technical Solutions Company Ltd.",
    "24.9465": "Cisco Systems, Inc",
    "24.9462": "Miyoshi Electronics Corporation",
    "24.9463": "Cisco Systems, Inc",
    "24.9460": "Kaminario, Ltd.",
    "24.9461": "NDS Surgical Imaging",
    "24.12616829": "AMPAK Technology, Inc.",
    "24.3689616": "ARRIS Group, Inc.",
    "24.7660151": "Vizmonet Pte Ltd",
    "24.3970313": "Liteon Technology Corporation",
    "24.12098505": "SMC Networks Inc",
    "24.5515149": "NXP (China) Management Ltd.",
    "24.10286385": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7344239": "Hewlett Packard Enterprise",
    "24.7404687": "Freebox Sas",
    "12.346862370816": "Vester Elektronik GmbH",
    "20.162752343572480": "SolarEdge Technologies",
    "24.12337039": "Huawei Technologies Co.,Ltd",
    "20.163808568934400": "Shanghai Hyco Genyong Technology Co., Ltd.",
    "24.7616109": "Vodafone Italia S.p.A.",
    "24.14474532": "Energica Motor Company SpA",
    "24.7051": "NEC Platforms, Ltd.",
    "24.7050": "2M Electronic A/S",
    "24.7053": "Electronic Computer Systems, Inc.",
    "24.7052": "JMicron Technology Corp.",
    "24.7055": "Cisco Systems, Inc",
    "24.7054": "Hulu Sweden AB",
    "24.7057": "Efkon Ag",
    "24.7056": "Cisco Systems, Inc",
    "24.7059": "JC Decaux SA DNT",
    "24.7058": "l-acoustics",
    "24.2261": "Vanguard Networks Solutions, LLC",
    "12.346863157248": "Pneu-Logic Corporation",
    "24.2263": "How Corporation",
    "24.2262": "HASSNET Inc.",
    "24.2265": "Mitadenshi Co.,LTD",
    "24.2264": "Dowkey Microwave",
    "24.2267": "Corrigent Systems",
    "24.2266": "SofaWare Technologies Ltd.",
    "24.2269": "Telena Communications, Inc.",
    "24.2268": "Wiznet",
    "24.4498974": "Ingram Micro Services",
    "24.3730660": "Samsung Electronics Co.,Ltd",
    "24.12067856": "Magneti Marelli Slovakia s.r.o.",
    "24.7906564": "Texas Instruments",
    "24.3984007": "Iders Incorporated",
    "12.123917679140864": "H3D, Inc.",
    "24.12363147": "Samsung Electronics Co.,Ltd",
    "24.6344105": "Abloomy",
    "24.11822744": "Zealabs srl",
    "12.119269335040": "Aba Electronics Technology Co.,Ltd",
    "24.14975411": "Shenzhen ACT Industrial Co.,Ltd.",
    "12.346865496064": "inotech GmbH",
    "24.2386048": "Braveridge.co.,ltd.",
    "24.10788571": "AutoCrib, Inc.",
    "24.13407306": "Pfeiffer Vacuum GmbH",
    "20.31368135311360": "Tokyo Drawing Ltd.",
    "12.346857336832": "Creative Retail Entertainment",
    "24.14434729": "Intel Corporate",
    "24.2421469": "Radiant Zemax LLC",
    "12.346856476672": "Altronic, Inc.",
    "24.13669022": "Microsoft Corporation",
    "24.14998595": "Huawei Device Co., Ltd.",
    "20.4471487725568": "Herrick Tech Labs",
    "20.168210315870208": "Juketek Co., Ltd.",
    "12.123917685215232": "Signal Processing Devices Sweden AB",
    "24.10259441": "The Warehouse Limited",
    "12.123917675368448": "Pro-Digital Projetos Eletronicos Ltda",
    "24.3707045": "Cisco Systems, Inc",
    "12.346859679744": "Sequestered Solutions",
    "24.16521276": "Apple, Inc.",
    "20.150399876071424": "Fortive Setra-ICG(Tianjin)Co.,Ltd",
    "24.12334565": "Hydro Systems Company",
    "24.2755": "eM Technics Co., Ltd.",
    "12.346860048384": "Icon Time Systems",
    "12.346866503680": "Keith & Koep GmbH",
    "12.123917686587392": "Vigilate srl",
    "24.4994638": "Panasonic Corporation Connected Solutions Company",
    "24.16003523": "Apple, Inc.",
    "24.2635077": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.2158525": "Roku, Inc",
    "24.9755967": "A+V Link Technologies, Corp.",
    "12.346856239104": "CSIRO - Division of Exploration and Mining",
    "12.346857811968": "Ortrat, S.L.",
    "12.346852827136": "Jäger Computergesteuerte Meßtechnik GmbH.",
    "24.16312654": "Apple, Inc.",
    "24.15253975": "Philips",
    "24.3937": "Hewlett Packard",
    "12.123917686296576": "ATOM GIKEN Co.,Ltd.",
    "24.2925881": "Parallel Wireless, Inc",
    "12.123917690081280": "Adeli",
    "24.2376076": "Intel Corporate",
    "24.6063992": "Cybertelbridge co.,ltd",
    "12.346868002816": "Saia-Burgess Controls AG",
    "12.123917678362624": "Dokuen Co. Ltd.",
    "12.346864283648": "Gentec",
    "12.123917676265472": "Conjing Networks Inc.",
    "24.12358251": "Apple, Inc.",
    "12.123917683970048": "LG Electronics",
    "24.14740935": "Shenzhen Friendcom Technology Development Co., Ltd",
    "12.123917691506688": "MB connect line GmbH Fernwartungssysteme",
    "12.346856988672": "TelASIC Communications, Inc.",
    "12.71297883541504": "COLONIAL ASSEMBLY and DESIGN",
    "20.145211808284672": "Llc Stc Mzta",
    "24.3163542": "Huawei Technologies Co.,Ltd",
    "12.123917691736064": "Prolan Process Control Co.",
    "20.58144479772672": "SAMA NextGen PTE Limited",
    "24.6316511": "Z-meta Research LLC",
    "12.346866319360": "Keith & Koep GmbH",
    "12.123917687369728": "Smart Vision Lights",
    "24.3989618": "Apple, Inc.",
    "12.123917686501376": "DimoSystems BV",
    "24.13640667": "Apple, Inc.",
    "24.6553841": "Cisco Systems, Inc",
    "24.676": "AddPac Technology Co., Ltd.",
    "24.4242547": "Cronoplast S.L.",
    "24.2150192": "ASUSTek COMPUTER INC.",
    "24.16294509": "Canon Inc.",
    "12.346859118592": "Cadi Scientific Pte Ltd",
    "12.346860564480": "dB Broadcast Limited",
    "24.678": "Effinet Systems Co., Ltd.",
    "12.346859012096": "Elk Products",
    "12.346864766976": "Doppler Systems LLC",
    "24.12904836": "Tp-Link Technologies Co.,Ltd.",
    "24.11323199": "Vitsmo Co., Ltd.",
    "24.15478424": "AzureWave Technology Inc.",
    "24.7384938": "Shenzhen Fenglian Technology Co., Ltd.",
    "12.346863747072": "Smartb Technologies",
    "12.346854993920": "Embedded Systems Design",
    "20.35758073184256": "Kloudspot Inc",
    "12.346865831936": "Guardtec,Inc",
    "24.6867456": "Octopus Systems Limited",
    "24.11043455": "CIBN Oriental Network(Beijing) CO.,Ltd",
    "20.158667119788032": "mcf88 SRL",
    "24.2752": "Fuyoh Video Industry CO., LTD.",
    "24.13401973": "Cisco Systems, Inc",
    "12.346868842496": "Mango DSP, Inc.",
    "12.123917679808512": "Fiem Industries Ltd.",
    "20.167860151255040": "Resonant Systems Inc.",
    "24.2643546": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13130656": "Shenzhen Qihu Intelligent Technology Company Limited",
    "24.10504798": "Microsoft Corporation",
    "24.8445981": "Cisco Systems, Inc",
    "24.4515504": "China Mobile IOT Company Limited",
    "20.141268002799616": "Phoenix Co.,Ltd.",
    "24.2656": "Autostar Technology Pte Ltd",
    "24.2145222": "Jabil Circuit Hungary Ltd.",
    "24.2370046": "KYOCERA Corporation",
    "24.9478455": "Beijing Splendidtel Communication Technology Co,. Ltd",
    "12.123917688569856": "Shenzhen Witlink Co.,Ltd.",
    "20.136823642783744": "Ametek VIS",
    "12.346856574976": "Adatel Telecomunicaciones S.A.",
    "24.13148208": "Texas Instruments",
    "12.346852978688": "TransComm Technology System, Inc.",
    "12.346855292928": "Blackline Systems Corporation",
    "24.528518": "NEC Platforms, Ltd.",
    "24.2657": "Cellinx Systems Inc.",
    "20.132777842311168": "RONIX incorporated",
    "12.123917676474368": "Eiden Co.,Ltd.",
    "24.14710947": "Shanghai Winner Information Technology Co.,Inc",
    "24.13636162": "BIOS Corporation",
    "24.15470276": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.11830599": "Amptown System Company GmbH",
    "12.346859773952": "Fantuzzi Reggiane",
    "24.7397090": "Apple, Inc.",
    "24.7553": "Guangzhou Gateway Electronics Co., Ltd",
    "24.14425503": "U & B tech",
    "24.13684574": "Energy Micro AS",
    "12.123917677522944": "Zehntner Testing Instruments",
    "20.83889186930688": "Power Active Co., Ltd",
    "12.123917690073088": "CesiumAstro Inc.",
    "24.525169": "Krg Corporate",
    "12.123917689856000": "CONTES, spol. s r.o.",
    "24.20627": "Boeing",
    "24.12330657": "FACTORY FIVE Corporation",
    "24.16270765": "Konka Group Co., Ltd.",
    "24.5283643": "Oi Electric Co.,Ltd",
    "24.1319483": "Google, Inc.",
    "24.16272892": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.15735853": "Juniper Networks",
    "24.16046002": "LGS Innovations, LLC",
    "24.11023002": "Digicom Futuristic Technologies Ltd.",
    "12.123917689331712": "AmTote Australasia",
    "12.346860380160": "International Mining Technologies",
    "24.2993": "Super Star Technology Co., Ltd.",
    "20.62335519555584": "BoCo Inc.",
    "12.346856067072": "Gilbarco Autotank AB",
    "24.282538": "Ruckus Wireless",
    "24.4982729": "COMPUTER OFFICE Co.,Ltd.",
    "12.123917689344000": "GD Mission Systems",
    "24.10545627": "Ningbo FreeWings Technologies Co.,Ltd",
    "24.4258953": "ARRIS Group, Inc.",
    "24.12327527": "YF Technology Co., Ltd",
    "24.4236919": "Juniper Networks",
    "24.8712489": "Metrascale Inc.",
    "20.22333144170496": "FedEx Services OTI",
    "24.1079502": "Hanvit SI, Inc.",
    "12.346866892800": "Youho Electric Ind.,Ltd.",
    "12.123917683916800": "Skopei B.V.",
    "24.282287": "LG Innotek",
    "12.123917679583232": "eSOL Co.,Ltd.",
    "24.7352025": "Guangzhou Shiyuan Electronics Co., Ltd.",
    "20.45069242466304": "NUUO, Inc.",
    "24.9198019": "ADB Italia",
    "24.5789453": "Danfoss Solar Inverters",
    "12.123917676113920": "neQis",
    "12.346867322880": "Fsm Ag",
    "20.242185275441152": "Century Audio, Inc.",
    "24.5800616": "Microsoft",
    "24.1622093": "Giga-Byte Technology Co.,Ltd.",
    "12.123917683994624": "Cubitech",
    "12.123917675200512": "Tresent Technologies",
    "12.346857742336": "Elfiq Inc.",
    "20.31657733128192": "Private",
    "24.11841882": "Siemens AG Energy Management Division",
    "24.10018282": "Infinix mobility limited",
    "20.83889192173568": "Zivid AS",
    "24.3451630": "Mikrovisatos Servisas UAB",
    "24.3992313": "Jubixcoltd",
    "24.826710": "Consort Controls Ltd",
    "12.346859274240": "B&S MEDIA Co., LTD.",
    "24.9213135": "Encell Technology, Inc.",
    "12.123917682061312": "Luceo",
    "24.5220": "Cryptosoft",
    "24.5221": "Novo Nordisk A/S",
    "24.5222": "Kleinhenz Elektronik GmbH",
    "24.5223": "ArrowSpan Inc.",
    "24.5224": "CelPlan International, Inc.",
    "24.5225": "Cisco Systems, Inc",
    "24.5226": "Cisco Systems, Inc",
    "24.5227": "Anagran, Inc.",
    "24.5228": "Netgear",
    "24.5229": "RF Technologies",
    "24.14732410": "Huawei Technologies Co.,Ltd",
    "24.5798477": "Stonesoft Corporation",
    "24.3414604": "Shenzhen Weibu Electronics Co.,Ltd.",
    "24.11556614": "Vallox Oy",
    "24.4179": "Computer Technology Corp.",
    "24.4178": "Mettler-Toledo (Albstadt) Gmbh",
    "24.4177": "Cmicro Corporation",
    "24.4176": "Rion Co., Ltd.",
    "24.4175": "Oracle Corporation",
    "24.4174": "Ceologic",
    "24.4173": "Surtec Industries, Inc.",
    "24.4172": "Teledyne LeCroy, Inc",
    "24.4171": "3Com",
    "24.4170": "The Parvus Corporation",
    "20.154019632775168": "V2 S.p.A.",
    "12.123917680533504": "Sensefarm AB",
    "24.13927172": "Shenzhen Fast Technologies Co.,Ltd",
    "24.13425412": "DataRemote Inc.",
    "12.346853138432": "Industrial Indexing Systems, Inc.",
    "20.5239423893504": "Dreamkas Llc",
    "12.123917675352064": "Bae Systems",
    "24.5598": "Nokia Danmark A/S",
    "24.5599": "Clivet S.p.A.",
    "24.37011": "EIZO Corporation",
    "24.5592": "Interlink Electronics",
    "24.5593": "PKC Electronics Oy",
    "24.5590": "OSLiNK Sp. z o.o.",
    "24.5591": "Reti Corporation",
    "24.5596": "KT&C Co., Ltd.",
    "24.5597": "IP Control Systems Ltd.",
    "24.5594": "Iritel A.D.",
    "24.5595": "Canesta Inc.",
    "24.7658218": "Texas Instruments",
    "12.123917676154880": "Private",
    "24.1061502": "Huawei Device Co., Ltd.",
    "12.346852691968": "Matuschek Messtechnik GmbH",
    "24.12089382": "Nintendo Co.,Ltd",
    "20.128510960074752": "KLIMAT SOLEC Sp. z o.o.",
    "24.8133873": "Huawei Technologies Co.,Ltd",
    "24.3724615": "ASUSTek COMPUTER INC.",
    "12.346867154944": "Applied Micro Electronics AME bv",
    "24.57529": "Byas Systems",
    "24.57528": "AMD PCNet",
    "24.57527": "Cosworth Electronics Ltd",
    "24.57526": "Entrada Networks",
    "24.57525": "Ardent Communications Corp.",
    "24.3176659": "Virgilant Technologies Ltd.",
    "24.57523": "EtherWAN Systems, Inc.",
    "24.57522": "Telmax Communications Corp.",
    "24.57521": "Alcatel-Lucent Enterprise",
    "24.57520": "Cisco Systems",
    "24.14201232": "Cisco Systems, Inc",
    "24.7122239": "Hubbell Building Automation, Inc.",
    "24.4749126": "Calix Inc.",
    "12.123917687234560": "Device Solutions Ltd",
    "24.1363123": "Ao \"Gk Nateks\"",
    "24.2665790": "Shenzhen TONG BO WEI Technology CO.,LTD",
    "12.346867560448": "Gogo BA",
    "24.16292025": "Huawei Technologies Co.,Ltd",
    "24.8687254": "Embertec Pty Ltd",
    "24.11588968": "Shenzhen Cultraview Digital Technology Co., Ltd",
    "20.180659904577536": "Protean Payment",
    "12.123917688840192": "Euklis by GSG International",
    "12.71297884893184": "Noritake Itron Corporation",
    "12.123917683834880": "eeas gmbh",
    "12.346866188288": "Nihon Kessho Koogaku Co., Ltd.",
    "24.2417428": "Apple, Inc.",
    "24.13432159": "E Focus Instruments India Private Limited",
    "12.123917691559936": "M2M Cybernetics Pvt Ltd",
    "24.3338": "Barco Projection Systems NV",
    "24.3339": "Buffalo.Inc",
    "24.4496248": "The Now Factory",
    "24.2408826": "Beijing Iactive Network Co.,Ltd",
    "12.123917684981760": "Sanmina Israel",
    "12.346865209344": "Sequoia Technology Group Ltd",
    "24.3331": "Matrics, Inc.",
    "24.3332": "Foxboro Eckardt Development GmbH",
    "24.3333": "cybernet manufacturing inc.",
    "24.3334": "Compulogic Limited",
    "24.3335": "Calrec Audio Ltd",
    "24.300518": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.3337": "Yuehua(Zhuhai) Electronic CO. LTD",
    "12.123917677932544": "Yongtong tech",
    "24.11571316": "Fulan Electronics Limited",
    "24.9721765": "Hewlett Packard",
    "12.123917679943680": "Veilux Inc.",
    "24.16256243": "LG Electronics (Mobile Communications)",
    "12.346857406464": "Orenco Systems, Inc.",
    "24.844960": "Huawei Device Co., Ltd.",
    "24.16025541": "Huawei Device Co., Ltd.",
    "20.48543322603520": "Itus Networks, LLC",
    "20.146101704327168": "ASL Intercom B.V.",
    "24.13660892": "Cisco Systems, Inc",
    "24.9979402": "Apple, Inc.",
    "24.13123036": "Xiaomi Communications Co Ltd",
    "24.9475223": "Huawei Technologies Co.,Ltd",
    "24.16558612": "Giga-Byte Technology Co.,Ltd.",
    "24.1109121": "Samsung Electronics Co.,Ltd",
    "24.8941524": "Dell Inc.",
    "20.4471491919872": "Hubei Baobao Intelligent Technology Co.,LTD",
    "12.346865774592": "New Embedded Technology",
    "24.15506419": "Samsung Electro-Mechanics(Thailand)",
    "12.71297884917760": "Lubino s.r.o.",
    "20.128612186456064": "Capwave Technologies Inc",
    "24.85": "Commissariat A L`Energie Atom.",
    "24.84": "Schneider Electric",
    "12.346868858880": "First System Technology Co., Ltd.",
    "24.86": "Dr. B. Struck",
    "24.81": "Hob Electronic Gmbh & Co. Kg",
    "24.80": "Radisys Corporation",
    "24.83": "Compucorp",
    "24.82": "Intrusion.com, Inc.",
    "24.89": "Hellige GMBH",
    "24.2423653": "Huawei Technologies Co.,Ltd",
    "12.346855874560": "Embedtronics Enterprise",
    "12.123917676830720": "Potter Electric Signal Co. LLC",
    "12.346861461504": "Coherix, Inc",
    "24.4233506": "AzureWave Technology Inc.",
    "24.11578066": "Sichuan tianyi kanghe communications co., LTD",
    "20.14223580069888": "Syntrans AB",
    "24.8687465": "Dell Inc.",
    "24.6084021": "Shenzhen WiSiYiLink Technology Co.,Ltd",
    "24.10137034": "Credo",
    "12.123917677973504": "Teuco Guzzini",
    "20.189549217054720": "Beijing Hamigua Technology Co., Ltd.",
    "24.9445085": "Great Computer Corporation",
    "24.2469": "Hansung Eletronic Industries Development Co., Ltd",
    "24.2468": "HARTEC Corporation",
    "24.297186": "Beats Electronics LLC",
    "24.2466": "Interface Co., Ltd.",
    "24.2465": "Telewise Communications, Inc.",
    "24.2464": "Microtechno Corporation",
    "24.2463": "Videx Inc.",
    "24.2462": "Testech, Inc.",
    "24.2461": "Haliplex Communications",
    "24.2460": "Naval Research Laboratory",
    "24.608": "Accordion Networks, Inc.",
    "24.609": "Tilgin AB",
    "24.1885756": "Arcadyan Technology Corporation",
    "24.846234": "Atls Altec",
    "24.601": "Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group",
    "24.602": "Catena Networks",
    "24.603": "Cambridge Silicon Radio",
    "24.604": "SCI Systems (Kunshan) Co., Ltd.",
    "24.605": "Calix Networks",
    "24.606": "High Technology Ltd",
    "24.607": "Nortel Networks",
    "12.346868867072": "Sparks Instruments SA",
    "24.4480350": "SHENZHEN Coolkit Technology CO.,LTD",
    "24.7529": "Knorr-Bremse IT-Services GmbH",
    "24.7528": "Thomson Telecom Belgium",
    "24.7521": "BIJ Corporation",
    "24.7520": "ASUSTek COMPUTER INC.",
    "24.7523": "Miele & Cie. KG",
    "24.7522": "InPhase Technologies",
    "24.7525": "Microwave Radio Communications",
    "24.7524": "Adam Communications Systems Int Ltd",
    "24.7527": "Amec",
    "24.7526": "Hyundai Telecom",
    "12.346857881600": "Ingeniería AlmudÃ­ S.L.",
    "24.12854112": "Intel Corporate",
    "24.10497078": "Fn-Link Technology Limited",
    "12.123917675012096": "Gogo BA",
    "24.5963": "Nokia Danmark A/S",
    "24.5962": "Socomec",
    "24.5961": "Hyundae Yong-O-Sa Co.,Ltd",
    "24.5960": "Neokoros Brasil Ltda",
    "24.5967": "iCatch Inc.",
    "24.5966": "Parama-tech Co.,Ltd.",
    "24.5965": "Dynamic Network Factory, Inc.",
    "24.5964": "Millipore",
    "12.346866663424": "Systech",
    "24.5968": "GSI Group, MicroE Systems",
    "24.12350774": "Alpsalpine Co,.Ltd",
    "12.346861678592": "Leonardo MW Ltd (Land & Naval Defence Electronic)",
    "20.194284252823552": "Nanjing Yining Intelligent Technology Co., Ltd.",
    "24.8402803": "Netgear",
    "12.123917681573888": "ATX Networks Corp",
    "24.12846895": "Huawei Technologies Co.,Ltd",
    "24.15484732": "Redwire, LLC",
    "24.12872628": "myIDkey",
    "24.15477986": "Apple, Inc.",
    "20.14286635139072": "Maksat Technologies P Ltd",
    "24.9473400": "D-Link International",
    "12.346863263744": "Fep Srl",
    "12.346862379008": "Xad Communications Ltd",
    "24.9973752": "Huawei Device Co., Ltd.",
    "24.53462": "Aethra Telecomunicazioni",
    "24.10013235": "Apple, Inc.",
    "24.10283292": "zte corporation",
    "12.123917684613120": "Miravue",
    "24.10232334": "TASCAN Systems GmbH",
    "24.16296400": "LG Electronics (Mobile Communications)",
    "24.1356517": "Shenzhen iComm Semiconductor CO.,LTD",
    "24.9492414": "Wavelab Global Inc.",
    "24.11285288": "Huawei Device Co., Ltd.",
    "24.9452904": "G-Printec, Inc.",
    "24.9452907": "Zicon Technology Corp.",
    "12.123917675921408": "PLCiS",
    "20.141720407769088": "Elcus",
    "24.7920307": "GBO Technology",
    "24.8326": "Microtech Electronics Limited",
    "24.8327": "Memotec, Inc.",
    "24.8324": "Oce Printing Systems, Gmbh",
    "24.8325": "Eaton Corporation",
    "24.8322": "Oneac Corporation",
    "24.8323": "Presticom Incorporated",
    "24.8320": "Synergy (Uk) Ltd.",
    "24.8321": "Titan Electronics",
    "24.8328": "Global Village Communication",
    "24.8329": "T3Plus Networking, Inc.",
    "24.564737": "Quanta Computer Inc.",
    "24.5245746": "Foxda Technology Industrial(ShenZhen)Co.,LTD",
    "24.15211369": "Sercomm Corporation.",
    "24.2120096": "Paragon Technologies Inc.",
    "12.346862866432": "Appareo Systems, LLC",
    "12.346856443904": "SensoTech GmbH",
    "24.15246529": "Deep Sea Electronics Ltd",
    "12.123917683253248": "Telaeris, Inc.",
    "24.6209": "High Tech Computer Corp",
    "24.6208": "3 Phoenix, Inc.",
    "12.346853191680": "PIUSYS Co., Ltd.",
    "12.123917677027328": "Meggitt",
    "24.6207": "2Wire Inc",
    "24.3985774": "Telepower Communication Co., Ltd",
    "24.6206": "Digilent, Inc",
    "20.250817681555456": "ROMO Wind A/S",
    "24.28999": "Amazon Technologies Inc.",
    "20.132811183882240": "Beijing CarePulse Electronic Technology",
    "12.123917683961856": "WUHAN EASYLINKIN TECHNOLOGY co.,LTD",
    "24.16578119": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.10788741": "Lyve Minds, Inc",
    "12.71297883402240": "Software Workshop",
    "24.524412": "Vitalink",
    "24.524411": "Sanyo Electric Co. Ltd.",
    "24.524410": "Indata",
    "24.524417": "Crosfield Electronics",
    "24.524416": "Aes Data Inc.",
    "24.524415": "Carnegie-Mellon University",
    "24.524414": "Amalgamated Wireless(Aus) Ltd",
    "24.524419": "Seiko Denshi",
    "24.524418": "Veritas Software",
    "24.3416036": "ARRIS Group, Inc.",
    "24.6316063": "Sz Dji Technology Co.,Ltd",
    "12.123917686386688": "RF Code",
    "12.123917690568704": "Wartsila Voyage Limited",
    "24.14738150": "Espressif Inc.",
    "24.7633552": "Murata Manufacturing Co., Ltd.",
    "24.816062": "Dongguan Haimai Electronie Technology Co.,Ltd",
    "24.10234103": "Xiaomi Communications Co Ltd",
    "24.16288567": "ARRIS Group, Inc.",
    "24.1609465": "G2C Co. Ltd.",
    "12.346866180096": "infinitec co., ltd.",
    "24.13920677": "Miura Systems Ltd.",
    "24.6486": "Cisco Systems, Inc",
    "12.123917678682112": "Naval Group",
    "24.16273773": "Denwa Technology Corp.",
    "12.346858946560": "Ihm",
    "24.7626088": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.189": "Mitsubishi Cable Industries, Ltd. / Ryosei Systems",
    "24.188": "Rockwell Automation",
    "24.187": "TRI-DATA Systems Inc.",
    "24.186": "Siig, Inc.",
    "24.1338890": "Private",
    "24.536996": "Huawei Device Co., Ltd.",
    "24.183": "Dove",
    "24.182": "Micro-Matic Research",
    "24.181": "Datability",
    "24.180": "Edimax Technology Co. Ltd.",
    "20.145383547207680": "Shenzhen Horn Audio Co.,Ltd.",
    "12.346865840128": "CODACO ELECTRONIC s.r.o.",
    "24.2126677": "ARRIS Group, Inc.",
    "24.1602005": "Hazens Automotive Electronics(SZ)Co.,Ltd.",
    "24.11044311": "Sangfor Technologies Inc.",
    "24.11563021": "Nokia",
    "24.3952199": "Etronic A/S",
    "24.325880": "Edgecore Networks Corporation",
    "12.123917681635328": "Luxar Tech, Inc.",
    "20.150399877120000": "Hasbro Inc",
    "24.11595333": "AzureWave Technology Inc.",
    "24.3429780": "Fujian Star-Net Communication Co.,Ltd",
    "24.11798833": "Huawei Technologies Co.,Ltd",
    "20.27478324150272": "Ark Vision Systems GmbH & Co. KG",
    "24.12884482": "LG Electronics (Mobile Communications)",
    "12.123917686202368": "Nuviz Oy",
    "24.11279622": "Apple, Inc.",
    "12.123917676871680": "LDA audio video profesional S.L.",
    "24.14181781": "Toy's Myth Inc.",
    "24.9222135": "Shenzhen UniStrong Science & Technology Co., Ltd",
    "12.346861719552": "Rosendahl Studiotechnik GmbH",
    "24.1332805": "Savitech Corp.",
    "20.123731217219584": "Hori Co., Ltd.",
    "24.6764": "Cisco Systems, Inc",
    "24.16532886": "Compal Information (Kunshan) Co., Ltd.",
    "24.11034181": "ASUSTek COMPUTER INC.",
    "24.7079008": "RBH Access Technologies Inc.",
    "24.15244031": "Fujian LANDI Commercial Equipment Co.,Ltd",
    "24.13645634": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "12.346862567424": "Rohde&Schwarz Topex SA",
    "24.14174475": "Huawei Technologies Co.,Ltd",
    "24.16290803": "Volans",
    "12.346867855360": "Amstelland Electronic BV",
    "12.346860670976": "It Salux Co., Ltd.",
    "24.2125804": "Braemac CA LLC",
    "20.238070224912384": "Auvidea GmbH",
    "12.346855936000": "Advanced Micro Controls Inc.",
    "20.123598306017280": "Graphcore Ltd",
    "24.2662802": "GERP Solution",
    "20.127658789699584": "Voltaware Services Limited",
    "12.123917676511232": "Mutelcor GmbH",
    "24.6328855": "Apple, Inc.",
    "12.346856030208": "Microtarget Tecnologia Digital Ltda.",
    "24.3972": "Astute Networks, Inc.",
    "20.171977817980928": "Guangdong Hanwei intergration Co.,Ltd",
    "24.14705255": "Xiaomi Communications Co Ltd",
    "24.12359649": "Broadcom Limited",
    "12.346852700160": "ard sa",
    "24.9192704": "Murata Manufacturing Co., Ltd.",
    "12.123917680869376": "Ruag Defence France SAS",
    "20.31372479561728": "Toshiba Toko Meter Systems Co., LTD.",
    "24.6317856": "Intel Corporate",
    "24.16283576": "Samsung Electronics Co.,Ltd",
    "24.3973": "ADDO-Japan Corporation",
    "12.346864750592": "Quercus Technologies, S.L.",
    "24.8441957": "CKS Corporation",
    "24.4993870": "Hightech",
    "12.123917689200640": "Jiangsu Horaintel Co.,Ltd",
    "24.1613": "Sencore",
    "24.2907539": "Ruckus Wireless",
    "24.9764509": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.346859819008": "Beceem Communications, Inc.",
    "12.346853228544": "Adescom Inc.",
    "20.114975480741888": "Altis Technology",
    "24.14461321": "Macandc",
    "20.149935429255168": "Foshan Huaguo Optical Co.,Ltd",
    "24.11844114": "China Mobile (Hangzhou) Information Technology Co.,Ltd.",
    "24.11035634": "Texas Instruments",
    "24.7657044": "Gint",
    "24.832751": "Digience Co.,Ltd.",
    "12.346863316992": "Peek Traffic Corporation",
    "12.71297883262976": "BITMILL srl",
    "12.346867470336": "UV Networks, Inc.",
    "20.93065829154816": "Advice",
    "24.11059746": "TM-Research Inc.",
    "12.346865332224": "Keith & Koep GmbH",
    "20.88383263080448": "Ufanet SC",
    "20.31477542682624": "LLC \"Gagar.In\"",
    "24.8158091": "Control Concepts, Inc.",
    "12.123917683363840": "Communication Systems Solutions",
    "24.12870484": "U-Raku, Inc.",
    "12.346864463872": "GHL Systems Berhad",
    "24.2658890": "Hewlett Packard",
    "24.6763": "Universal Global Scientific Industrial Co., Ltd.",
    "24.3458714": "Enecsys Ltd",
    "24.3455544": "Pal Mohan Electronics Pvt Ltd",
    "12.346853892096": "RF Code",
    "24.11541649": "Elim Co",
    "12.123917683499008": "Joehl & Koeferli AG",
    "24.3974": "BlackBerry RTS",
    "12.346855026688": "TA Instruments Ltd",
    "12.346856382464": "CoreStreet, Ltd",
    "20.44768524500992": "Shenzhen Eview Gps Technology",
    "12.71297884233728": "Power Electronics Espana, S.L.",
    "12.346867974144": "Vision Systems Aeronautic",
    "24.13172272": "Bejing DAYO Mobile Communication Technology Ltd.",
    "24.6089576": "Cheerstar Technology Co., Ltd",
    "24.4758808": "Flashbay Limited",
    "20.31372475367424": "Ultraflux",
    "24.5253453": "Nanjing Iot Sensor Technology Co,Ltd",
    "24.8661840": "Huria Co.,Ltd.",
    "12.123917680746496": "Rhythm Engineering, LLC.",
    "24.32979": "Shiva",
    "24.32978": "Shinnihondenko Co., Ltd.",
    "24.32975": "Embedded Performance Inc.",
    "24.32974": "Broadcast Television Systems",
    "24.32977": "Kimtron Corporation",
    "24.32976": "Computer Products International",
    "24.32971": "Falco Data Products",
    "24.32970": "Netcom Research Incorporated",
    "24.32973": "Micronics Computer, Inc.",
    "24.32972": "Microwave Bypass Systems",
    "12.123917683376128": "Surion (Pty) Ltd",
    "24.10236006": "RWE Effizienz GmbH",
    "24.7610233": "Youngkook Electronics Co.,Ltd",
    "12.346865221632": "Wapice Ltd.",
    "24.4236783": "Shenzhen Four Seas Global Link Network Technology Co., Ltd.",
    "24.10259566": "Samsung Electronics Co.,Ltd",
    "12.346864881664": "Private",
    "24.15529661": "Sncf Mobilités",
    "24.2371924": "Huawei Technologies Co.,Ltd",
    "24.9440899": "Monico Monitoring, Inc.",
    "24.14182086": "Shenzhen Daystar Technology Co.,ltd",
    "12.346864201728": "Quartech Corporation",
    "12.123917680291840": "Nordson Corporation",
    "24.8710974": "Vivint Smart Home",
    "24.8972050": "Whirlpool Corporation",
    "12.123917678256128": "Terratel Technology s.r.o.",
    "24.3173980": "Validus Technologies",
    "24.9194937": "Unmonday Ltd",
    "24.13405411": "Texas Instruments",
    "24.7395619": "vivo Mobile Communication Co., Ltd.",
    "24.1598833": "Top Victory Electronics (Taiwan) Co., Ltd.",
    "24.12121296": "Herrmann Ultraschalltechnik GmbH & Co. Kg",
    "24.5255067": "Intel Corporate",
    "20.35227554545664": "HANGZHOU DANGBEI NETWORK TECH.Co.,Ltd",
    "24.6993650": "Eli Technology Inc",
    "20.242876965453824": "shenzhen bangying electronics co,.ltd",
    "24.1632477": "Moduletek",
    "20.220092026585088": "Galgus",
    "12.346865979392": "Cadi Scientific Pte Ltd",
    "20.189549207617536": "infypower Co., Ltd",
    "24.2674076": "Shenzhen Dynamax Software Development Co.,Ltd.",
    "24.539192": "New H3C Technologies Co., Ltd",
    "12.346866581504": "Security Services Group (SSG)",
    "20.119709247733760": "Beijing Ainemo Co Ltd",
    "24.15782823": "CobaltRay Co., Ltd",
    "24.2140153": "Shenzhen Olanboa Technology Co., Ltd",
    "24.5261934": "Netgear",
    "24.5822074": "TCPlink",
    "24.3716087": "Huawei Device Co., Ltd.",
    "24.4244037": "Shenzhen Hexicom Technology Co., Ltd.",
    "24.10789450": "Cisco SPVTG",
    "24.8437253": "Apple, Inc.",
    "24.14946953": "topsystem Systemhaus GmbH",
    "24.9446380": "Availink, Inc.",
    "24.16307320": "Flextronics Tech.(Ind) Pvt Ltd",
    "24.3448208": "Heinzmann GmbH & CO. KG",
    "24.15217537": "ARRIS Group, Inc.",
    "24.10235726": "zte corporation",
    "12.123917688360960": "Preo Industries Far East Limited",
    "24.555016": "Samsung Electronics Co.,Ltd",
    "24.10502209": "SAMWONFA Co.,Ltd.",
    "24.15346395": "Labs Network Industrie 4. 0 e.V.",
    "24.15729224": "SmarteBuilding",
    "12.123917689892864": "DEUTA-WERKE GmbH",
    "24.2371216": "Palit Microsystems, Ltd",
    "12.123917690802176": "BroadSoft Inc",
    "24.1632297": "Ubiquiti Networks Inc.",
    "20.180698117832704": "BLOKS. GmbH",
    "12.123917687119872": "Ur Fog S.R.L.",
    "12.123917677219840": "iRF - Intelligent RF Solutions, LLC",
    "24.7393409": "Beijing Netpower Technologies Inc.",
    "24.2116611": "Aruba, a Hewlett Packard Enterprise Company",
    "24.1624295": "Shenzhen Hualistone Technology Co.,Ltd",
    "12.346852864000": "Fastware.Net, LLC",
    "24.16260499": "Longdhua(Beijing) Controls Technology Co.,Ltd",
    "24.16260496": "Solidwintech",
    "12.346866507776": "Spang Power Electronics",
    "12.346856701952": "Brivo Systems, LLC",
    "20.150399880265728": "Robert Bosch JuP1",
    "24.5031426": "Radios, Inc.",
    "20.229108120289280": "Shanghai Zhongmi Communication Technology Co.,Ltd",
    "24.15466990": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.4507644": "Huawei Device Co., Ltd.",
    "20.66426709737472": "Homegear GmbH",
    "20.238070229106688": "Grünbeck Wasseraufbereitung GmbH",
    "12.123917688909824": "EZSYS Co., Ltd.",
    "24.10767921": "Intel Corporate",
    "20.71414652600320": "SubPac",
    "12.71297884454912": "Calon Associates Limited",
    "24.16285846": "Elsys Equipamentos Eletrônicos Ltda",
    "12.123917688410112": "Rapiscan Systems",
    "24.5562148": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16043408": "Vispiron Rotec GmbH",
    "24.7920431": "Texas Instruments",
    "12.123917688619008": "Veilux Inc.",
    "24.16277290": "Nokia Corporation",
    "12.123917689237504": "PAMIR Inc",
    "12.123917675782144": "ChamSys Ltd",
    "24.10264177": "Huawei Device Co., Ltd.",
    "24.11589068": "Texas Instruments",
    "24.10787789": "Cisco Systems, Inc",
    "24.12629592": "Apple, Inc.",
    "24.1319690": "Huawei Technologies Co.,Ltd",
    "24.6056783": "Hello Inc.",
    "24.5284004": "Nokia",
    "24.10234095": "Huawei Technologies Co.,Ltd",
    "24.10023000": "Lynxspring, Incl.",
    "24.14971269": "Securifi Ltd",
    "12.123917688918016": "Vemco Sp. z o. o.",
    "24.15254906": "Ufispace Co., LTD.",
    "20.62070506651648": "Rock Electronic Co., Ltd.",
    "12.346855636992": "Kinesys Projects Limited",
    "12.123917683122176": "Phytron GmbH",
    "24.4780823": "Private",
    "24.5011279": "Embedded Wireless Labs",
    "24.14174139": "Huawei Device Co., Ltd.",
    "24.16530665": "Tsingtong Technologies Co, Ltd.",
    "24.15507213": "IEEE Registration Authority",
    "12.346866298880": "OWITA GmbH",
    "24.15007449": "EDMI Europe Ltd",
    "24.12108049": "Apple, Inc.",
    "12.123917680701440": "Radikal d.o.o.",
    "20.184991238586368": "GTDevice LLC",
    "24.15269039": "Netgear",
    "12.346862215168": "Commercial Timesharing Inc.",
    "12.346864545792": "Epiko, elektronski sistemi d.o.o.",
    "24.4782460": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.31346177081344": "Evondos Oy",
    "24.13169200": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7120514": "EDAM information & communications",
    "12.346857877504": "DATAMICRO Co., Ltd.",
    "24.13936936": "GreenWave Reality Inc",
    "12.346853625856": "Ipcast",
    "20.176069204246528": "MESIT asd, s.r.o.",
    "12.346865733632": "TelGaAs, Inc.",
    "24.16543168": "Bme Corporation",
    "24.14691571": "MRS Electronic GmbH",
    "12.123917681217536": "EA Elektroautomatik GmbH & Co. KG",
    "24.47298": "Heights Telecom T ltd",
    "24.1088221": "Apple, Inc.",
    "24.12108194": "Dragon Path Technologies Co., Limited",
    "24.2919": "Topview Technology Corporation",
    "24.557500": "HongKong Ipro Technology Co., Limited",
    "24.6940": "Coherent",
    "24.6941": "Phoenix International Co., Ltd",
    "24.6942": "HART Communication Foundation",
    "24.6943": "DELTA - Danish Electronics, Light & Acoustics",
    "24.6944": "TPine Technology",
    "24.6945": "Intel Corporate",
    "24.6946": "Palit Microsystems ( H.K.) Ltd.",
    "24.6947": "SimpleComTools",
    "24.6948": "Quanta Computer Inc.",
    "24.6949": "Nortel Networks",
    "24.10260121": "Hewlett Packard",
    "24.11800699": "Texas Instruments",
    "24.7888716": "Argox Information Co., Ltd.",
    "12.346861105152": "Inventis Technology Pty Limited",
    "24.5418": "Nokia Corporation",
    "24.5417": "N3 Corporation",
    "12.346859712512": "Cimetrics Research Pty Ltd",
    "24.5415": "Balboa Instruments",
    "12.346865676288": "Industrial Automatics Design Bureau",
    "24.5413": "Chamberlain Access Solutions",
    "24.5412": "Numatics, Inc.",
    "24.5411": "Meteor Communications Corporation",
    "24.5410": "Dea Security",
    "24.13654208": "Qingdao Haier Multimedia Limited.",
    "24.9175149": "Apple, Inc.",
    "24.6338867": "Deutron Electronics Corp.",
    "24.3209709": "Sony Corporation",
    "12.123917684871168": "Emtel System Sp. z o.o.",
    "20.255861670805504": "DongGuan Ramaxel Memory Technology",
    "24.12487": "Macromate Corp.",
    "24.12486": "Control Solutions, Inc.",
    "24.12485": "Cadence Design Systems, Inc.",
    "24.2910": "Audio Engineering Society Inc.",
    "24.12483": "Flueckiger Elektronik Ag",
    "20.61822313955328": "HOBART GmbH",
    "24.12481": "Hewlett Packard",
    "24.12480": "Lara Technology, Inc.",
    "24.2917": "Sy.A.C. srl",
    "24.4220966": "Thales UK Limited",
    "24.11325071": "Hwa Yao Technologies Co., Ltd",
    "24.12489": "LuxN, N",
    "24.12488": "Gad Line, Ltd.",
    "24.15252080": "Cisco Systems, Inc",
    "24.2914": "ib-mohnen KG",
    "24.12630193": "BitBox Ltd",
    "20.66256281534464": "Starloop Tech Co., Ltd.",
    "24.13164113": "IntegraOptics",
    "24.1575956": "iSonea Limited",
    "12.123917691518976": "CODESYSTEM Co.,Ltd",
    "24.2104265": "Juniper Networks",
    "24.14714973": "Technicolor CH USA Inc.",
    "12.123917688217600": "Todd Digital Limited",
    "12.123917688225792": "Connected Response",
    "24.11805488": "Itron Inc",
    "12.123917680558080": "Husty M.Styczen J.Hupert Sp.J.",
    "12.71297883725824": "Dsp Design",
    "12.123917684064256": "Phyton, Inc. Microsystems and Development Tools",
    "24.6849856": "IGI Mobile, Inc.",
    "20.238070223863808": "Get SAT",
    "24.5811320": "Cisco Systems, Inc",
    "24.8392151": "Latticework, Inc",
    "12.346854293504": "nNovia, Inc.",
    "24.3465002": "Fujikon Industrial Co.,Limited",
    "24.4882292": "INTEC Inc.",
    "24.12636563": "Huawei Device Co., Ltd.",
    "12.346857201664": "Open Date Equipment Limited",
    "24.7121263": "TransPacket AS",
    "24.6075618": "Inspur Software Group Ltd.",
    "24.15500245": "GIRD Systems Inc",
    "24.6330470": "Texas Instruments",
    "24.15487420": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.819760": "Shenzhen Magnus Technologies Co.,Ltd",
    "24.8396975": "Trade FIDES, a.s.",
    "24.14731165": "Texas Instruments",
    "24.10007535": "Huawei Device Co., Ltd.",
    "24.794936": "Hengstler GmbH",
    "24.3819": "Sandmartin(zhong shan)Electronics Co.,Ltd",
    "24.3818": "Shadong Luneng Jicheng Electronics,Co.,Ltd",
    "24.2107009": "Tecno Mobile Limited",
    "24.3811": "Chiyu Technology Co.,Ltd",
    "24.2159523": "Huawei Technologies Co.,Ltd",
    "24.3813": "bitWallet, Inc.",
    "24.3812": "Boe Technology Group Co.,Ltd",
    "24.3815": "Aac Electronics Corp.",
    "24.3814": "Adimos Systems LTD",
    "24.3817": "WayTech Development, Inc.",
    "24.3816": "Zioncom Electronics (Shenzhen) Ltd.",
    "12.346862538752": "Halliburton Far East Pte Ltd",
    "12.123917690658816": "Sociedad Iberica De Construcciones Electricas, S.A. (Sice)",
    "24.2130936": "Advanced Digital Broadcast SA",
    "24.4458": "Domo Ltd",
    "24.4459": "Digital Data Communications Asia Co.,Ltd",
    "24.13135030": "Optcom Co., Ltd.",
    "24.4450": "Star Micronics Co.,Ltd.",
    "24.4451": "System Spa Dept. Electronics",
    "24.4452": "ACARD Technology Corp.",
    "24.4453": "ZNYX Networks, Inc.",
    "24.3681354": "ASUSTek COMPUTER INC.",
    "24.4455": "Integrated System Solution Corp.",
    "24.4456": "HomeLogic LLC",
    "24.4457": "EMS Satcom",
    "24.10025153": "IDT Technology Limited",
    "24.10796484": "Huawei Technologies Co.,Ltd",
    "20.154238429691904": "innolectric AG",
    "24.1064202": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.346859790336": "Realtronix Company",
    "12.346867523584": "Powersense A/S",
    "12.71297884880896": "Futaba Corporation",
    "12.346867904512": "Wren Sound Systems",
    "24.2918": "Teralink Communications",
    "24.7228": "Seon Design Inc.",
    "24.7229": "WaveStorm",
    "24.5529970": "Zodiac Aerospace Sas",
    "12.346852237312": "Floware System Solutions Ltd.",
    "24.7222": "iNEWiT NV",
    "24.7223": "Callpod, Inc.",
    "24.7220": "Huey Chiao International Co., Ltd.",
    "24.7221": "Nokia Danmark A/S",
    "24.7226": "Element Labs, Inc.",
    "24.7227": "AmRoad Technology Inc.",
    "24.7224": "Bio-Rad Laboratories, Inc.",
    "24.7225": "S Netsystems Inc.",
    "24.22482": "Cisco Systems, Inc",
    "12.346859507712": "Keith & Koep GmbH",
    "24.12382992": "Extreme Networks, Inc.",
    "24.6325437": "Buffalo.Inc",
    "24.12345870": "Huawei Technologies Co.,Ltd",
    "24.49335": "American Power Conversion Corp",
    "24.49334": "HVE, Inc.",
    "24.49337": "Funk Software, Inc.",
    "24.8905": "Lenord, Bauer & Co GmbH",
    "24.49331": "Comstat Datacomm Corporation",
    "24.49330": "Norand Corporation",
    "24.49333": "Corporate Network Systems,Inc.",
    "12.346859745280": "On Air Networks",
    "24.49339": "Forval Creative, Inc.",
    "24.49338": "Netvantage",
    "24.8908": "SciLog, Inc.",
    "24.8909": "Ared Technology Co., Ltd.",
    "24.3107": "Beijing Lanchuan Tech. Co., Ltd.",
    "24.3106": "Double D Electronics Ltd",
    "24.3105": "Faculty of Science and Technology, Keio University",
    "24.3104": "Fi WIn, Inc.",
    "24.3103": "Glimmerglass Networks",
    "24.3102": "Global Cache",
    "24.3101": "Mettler & Fuchs AG",
    "24.3100": "MicroWeb Co., Ltd.",
    "24.3109": "Allied Telesis Labs, Inc.",
    "24.3108": "Anator",
    "24.793687": "Texas Instruments",
    "24.16035815": "Apple, Inc.",
    "12.346868269056": "Dsp Design Ltd",
    "20.22739793477632": "Az-Technology Sdn Bhd",
    "24.3955953": "eero inc.",
    "24.6578175": "Nicolet Neuro",
    "24.7488": "Intel – GE Care Innovations LLC",
    "24.7489": "Hardy Instruments",
    "24.7486": "Saka Techno Science Co.,Ltd",
    "24.7487": "Mitron Pty Ltd",
    "24.7484": "Muscle Corporation",
    "24.7485": "Avidyne Corporation",
    "24.7482": "mh acoustics LLC",
    "24.7483": "Nokia Danmark A/S",
    "24.7480": "Seagate Technology",
    "24.7481": "Moohadigital Co., Ltd",
    "24.8400862": "Zurich Instruments AG",
    "24.1884638": "Hewlett Packard",
    "12.71297884852224": "PRESSOL Schmiergeraete GmbH",
    "24.6579": "Stanford Research Systems",
    "24.6578": "XYnetsoft Co.,Ltd",
    "24.2058": "Espera-Werke GmbH",
    "24.2059": "Birka BPA Informationssystem AB",
    "24.5544291": "Apple, Inc.",
    "24.6571": "Raycom CO., LTD",
    "24.6570": "Cisco Systems, Inc",
    "24.6573": "Bobst Sa",
    "24.6572": "GSP SYSTEMS Inc.",
    "24.6575": "Rigol Technologies, Inc.",
    "24.6574": "Hopling Technologies b.v.",
    "24.6577": "Arrow7 Corporation",
    "24.6576": "HanYang System",
    "24.12873339": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "12.123917680644096": "Liberator Pty Ltd",
    "12.346858618880": "DARE Development",
    "24.6594968": "Huawei Device Co., Ltd.",
    "12.123917684842496": "Senquire Pte. Ltd",
    "12.346862219264": "Depro Électronique",
    "24.540040": "Samsung Electronics Co.,Ltd",
    "24.14187903": "Sagemcom Broadband SAS",
    "24.11568126": "Dell Inc.",
    "24.3160358": "Sony Mobile Communications Inc",
    "24.6683": "ARRIS Group, Inc.",
    "24.6682": "Gentex Corporation/Electro-Acoustic Products",
    "24.6681": "Computer Engineering Limited",
    "24.48065": "Canon Inc.",
    "24.6687": "Coastal Environmental Systems",
    "24.6686": "Aruba, a Hewlett Packard Enterprise Company",
    "24.6685": "PChome Online Inc.",
    "24.6684": "GT&T Engineering Pte Ltd",
    "24.6689": "Brookhuis Applied Technologies BV",
    "24.6688": "CMOTECH Co. Ltd.",
    "24.1585826": "Laon Technology Co., Ltd.",
    "24.10493451": "MINIX Technology Limited",
    "12.123917684174848": "Hangzhou Leaper Technology Co. Ltd.",
    "24.10285013": "Panda Wireless, Inc.",
    "24.7085783": "vivo Mobile Communication Co., Ltd.",
    "24.12112958": "vivo Mobile Communication Co., Ltd.",
    "24.9190317": "Netgear",
    "24.15490851": "Qinghai Kimascend Electronics Technology Co. Ltd.",
    "24.2124160": "Apple, Inc.",
    "24.2669654": "EMAC, Inc.",
    "24.2420884": "Apple, Inc.",
    "12.119269597184": "Postec Data Systems",
    "24.5794297": "Fusion Transactive Ltd.",
    "24.16569590": "Messana Air.Ray Conditioning s.r.l.",
    "20.145735977795584": "Annapurna labs",
    "24.1087542": "Dell Inc.",
    "24.3472545": "Micronet union Technology(Chengdu)Co., Ltd.",
    "24.37102": "Personal Communications Technologies",
    "24.37101": "Central System Research Co., Ltd.",
    "24.37100": "Pyrescom",
    "24.37107": "Aspect Communications",
    "24.37106": "Cisco",
    "24.37105": "Seagate Cloud Systems Inc",
    "24.37104": "Harmonic Video Systems Ltd.",
    "24.9754": "Femtocomm System Technology Corp.",
    "24.9755": "Laurel Bank Machines Co., Ltd.",
    "24.37109": "Clevo Co.",
    "24.37108": "Lightning Instrumentation",
    "24.9750": "Rosemount Inc.",
    "24.9751": "OEM Worldwide",
    "24.9752": "ASUSTek COMPUTER INC.",
    "24.9753": "Frc",
    "24.5888": "ARRIS Group, Inc.",
    "24.5889": "KDE, Inc.",
    "20.88668809199616": "Sagetech Corporation",
    "20.13592317394944": "Annapurna labs",
    "24.12322115": "AVM GmbH",
    "12.346867924992": "Aplex Technology Inc.",
    "24.13123508": "ARRIS Group, Inc.",
    "24.5880": "Aviqtech Technology Co., Ltd.",
    "24.5881": "CETRTA POT, d.o.o., Kranj",
    "24.5882": "ECI Telecom Ltd.",
    "24.5883": "Shenzhen Mtc Co Ltd",
    "24.5884": "Tohken Co.,Ltd.",
    "24.5885": "Jaty Electronics",
    "24.5886": "Alpsalpine Co,.Ltd",
    "24.5887": "Wamin Optocomm Mfg Corp",
    "24.1874263": "Intel Corporate",
    "24.12329124": "Samsung Electronics Co.,Ltd",
    "24.10270416": "Rivet Networks",
    "20.154794724425728": "Swift Navigation Inc",
    "24.16533743": "Huawei Technologies Co.,Ltd",
    "24.9286": "MMB Research Inc.",
    "24.9287": "Kaztek Systems",
    "24.9284": "Nintendo Co., Ltd.",
    "24.9285": "Adtran Inc",
    "24.9282": "Axona Limited",
    "24.9283": "Nortel Networks",
    "24.9280": "Halo Monitoring, Inc.",
    "24.9281": "Wanzl Metallwarenfabrik GmbH",
    "24.11046126": "MicroMade Galka i Drozdz sp.j.",
    "24.16253560": "IEEE Registration Authority",
    "24.9288": "SpiderCloud Wireless, Inc",
    "24.9289": "Shen Zhen Lite Star Electronics Technology Co., Ltd",
    "24.15238475": "Apple, Inc.",
    "12.123917691252736": "ABL Space Systems",
    "12.346867535872": "Mettler Toledo",
    "24.16522563": "Apple, Inc.",
    "24.6872010": "Apple, Inc.",
    "12.119269978112": "Autelis, LLC",
    "12.123917683941376": "MB connect line GmbH Fernwartungssysteme",
    "24.6854267": "Apple, Inc.",
    "24.572448": "Hangzhou Royal Cloud Technology Co., Ltd",
    "12.346854277120": "Algodue Elettronica Srl",
    "24.7637179": "Cisco Systems, Inc",
    "12.346857631744": "Dst Control Ab",
    "12.123917684977664": "ENS Engineered Network Systems",
    "24.1892754": "Qisda Corporation",
    "12.123917680148480": "Copious Imaging LLC",
    "24.8153819": "SafeTone Technology Co.,Ltd",
    "24.16259223": "2Wire Inc",
    "24.15783900": "Wesine (Wuhan) Technology Co., Ltd.",
    "24.41058": "Aes Prodata",
    "12.346855792640": "Cleode",
    "20.171812080058368": "Antailiye Technology Co.,Ltd",
    "12.123917684002816": "Integrated Control Corp.",
    "24.15784952": "Apple, Inc.",
    "12.346863120384": "KYOEI ENGINEERING Co.,Ltd.",
    "24.8489": "Cisco-Linksys, LLC",
    "24.8488": "Oracle Corporation",
    "24.8483": "Aerosat Avionics",
    "24.3151816": "Sony Mobile Communications Inc",
    "24.8481": "VRmagic GmbH",
    "24.8480": "Sequel Technologies",
    "24.8487": "Tp-Link Technologies Co.,Ltd.",
    "24.8486": "Shenzhen Torch Equipment Co., Ltd.",
    "24.8485": "KUK JE TONG SHIN Co.,LTD",
    "24.8484": "Optos Plc",
    "12.123917683539968": "Songwoo Information & Technology Co., Ltd",
    "12.123917675249664": "Ultimate Software",
    "24.7382634": "Prox Dynamics AS",
    "24.10752404": "Lenovo",
    "24.8151398": "Amazon Technologies Inc.",
    "24.8155": "Network Supply Corp.,",
    "24.8154": "Nortel Networks",
    "24.8157": "Gdi Llc",
    "24.8156": "Mobile Safe Track Ltd",
    "24.8151": "Telerad Sa",
    "24.8150": "Shenzhen Allywll",
    "24.8153": "RSD Communications Ltd",
    "24.8152": "A-Trust Computer Corporation",
    "24.8159": "Nokia Danmark A/S",
    "24.8158": "Nokia Danmark A/S",
    "24.4861": "Optimus Ic S.A.",
    "12.346865721344": "Embed-It Og",
    "24.4863": "Lely Industries N.V.",
    "24.4862": "Lenovo Mobile Communication Technology Ltd.",
    "24.4865": "IronGate S.L.",
    "24.4864": "It-Factory, Inc.",
    "24.4867": "GateConnect",
    "24.4866": "Intel Corporate",
    "24.4869": "Epicom, Inc.",
    "24.4868": "Flaircomm Technologies Co. LTD",
    "12.123917681434624": "PCS Inc.",
    "12.123917685616640": "GlobalTest LLC",
    "12.123917675372544": "Uni Control System Sp. z o. o.",
    "24.13396474": "Apple, Inc.",
    "20.70682237992960": "Magenta Labs, Inc.",
    "20.216039221624832": "Lynx Innovation Litimed",
    "24.7089158": "OOO NPP Systemotechnika-NN",
    "12.123917680480256": "Tempris GmbH",
    "12.123917681655808": "emperor brands",
    "24.8704810": "Ieee 1905.1",
    "24.7891593": "Huawei Technologies Co.,Ltd",
    "12.346861699072": "Design Lightning Corp",
    "12.123917679673344": "Ctrogers Llc",
    "12.123917676908544": "Eurotek Srl",
    "12.346866757632": "Ruwisch & Kollegen GmbH",
    "12.123917689794560": "Neuron GmbH",
    "12.346853466112": "AVerMedia Systems, Inc.",
    "20.167152473604096": "VR Technology(Shenzhen) Limited",
    "24.4463354": "New H3C Technologies Co., Ltd",
    "12.123917681967104": "Acrodea, Inc.",
    "12.123917675089920": "EN ElectronicNetwork Hamburg GmbH",
    "12.123917680128000": "adnexo GmbH",
    "24.4478690": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.7141976": "Avaya Inc",
    "24.14995056": "Health & Life co., Ltd.",
    "24.14967377": "Hao Cheng Group Limited",
    "24.7403815": "Samsung Electronics Co.,Ltd",
    "24.8940398": "Apple, Inc.",
    "24.6053735": "Intel Corporate",
    "12.71297884835840": "Tonna Electronique",
    "12.71297884688384": "Sicon srl",
    "24.12360124": "FonSee Technology Inc.",
    "12.346864361472": "Logical Tools s.r.l.",
    "24.13115022": "ADV Security (S) Pte Ltd",
    "20.22739797671936": "Veo Technologies",
    "24.5042879": "Cambridge Industries(Group) Co.,Ltd.",
    "24.32342": "China Dragon Technology Limited",
    "24.1617595": "Siemens Convergence Creators GmbH&Co.KG",
    "24.14164724": "Avnet Silica",
    "12.346864369664": "Custos Mobile S.L.",
    "24.7893074": "Welotec GmbH",
    "24.10271640": "Shen Zhen RND Electronic Co.,LTD",
    "24.314248": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "12.123917689147392": "BESO sp. z o.o.",
    "12.123917683548160": "Elbit Systems of America",
    "12.346855714816": "Septentrio nv/sa",
    "24.6606893": "Next Generation Integration (NGI)",
    "20.141720405671936": "Shortcut Labs",
    "24.13897650": "Applied Micro Electronics AME bv",
    "24.13397408": "Roku, Inc.",
    "24.7621793": "Liteon Technology Corporation",
    "24.4019": "Actiontec Electronics, Inc",
    "12.346858917888": "Skyline Products, Inc",
    "24.9748352": "zte corporation",
    "24.15777687": "Sony Corporation",
    "12.346866733056": "Neets",
    "20.88668797665280": "Guangzhou Hysoon Electronic Co., Ltd.",
    "24.12113190": "Zyxel Communications Corporation",
    "24.9189724": "Bit4id Srl",
    "12.346857332736": "Beijing Corelogic Communication Co., Ltd.",
    "12.123917686112256": "Pantec AG",
    "24.10761175": "MitraStar Technology Corp.",
    "20.48688653139968": "Irootech Technology Co.,Ltd",
    "24.6348531": "Huawei Technologies Co.,Ltd",
    "24.15004661": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "12.123917676281856": "SMITEC S.p.A.",
    "24.6481": "NETCONS, s.r.o.",
    "24.13172599": "Dyson Limited",
    "24.6480": "Harman Multimedia",
    "24.14974298": "Beijing Hanbang Technology Corp.",
    "20.5297543315456": "Beijing Huaxia Qixin Technology Co., Ltd.",
    "24.6483": "Chainleader Communications Corp.",
    "12.123917690970112": "OOO Research and Production Center \"Computer Technologies\"",
    "12.71297883615232": "BRS Sistemas Eletrônicos",
    "24.6482": "ACOGITO Co., Ltd",
    "24.14958818": "Samsung Electronics Co.,Ltd",
    "12.346855809024": "IDA Corporation",
    "24.15745781": "Huawei Device Co., Ltd.",
    "24.6485": "Cisco Systems, Inc",
    "20.176094514774016": "GERSYS GmbH",
    "24.6484": "Leaf Corporation.",
    "24.32838": "Tattile SRL",
    "24.32839": "In-Net Corp.",
    "24.2390520": "KUPSON spol. s r.o.",
    "24.6487": "Saafnet Canada Inc.",
    "24.32830": "Synernetics",
    "24.32831": "Tatung Company",
    "24.32832": "John Fluke Manufacturing Co.",
    "24.32833": "Veb Kombinat Robotron",
    "24.32834": "Force Computers",
    "24.32835": "Networld, Inc.",
    "24.32836": "Systech Computer Corp.",
    "24.32837": "Matsushita Electric Ind. Co",
    "24.7614204": "DirectPacket Research, Inc,",
    "20.67047984726016": "Gulf Security Technology Co., Ltd",
    "24.12897108": "Fibocom Auto Inc.",
    "24.4738430": "Dell Inc.",
    "24.3717923": "OzVision America LLC",
    "24.5243020": "Hong Kong Telecommunications (HKT) Limited",
    "12.346858393600": "Technomarine JSC",
    "24.5005534": "Netgear",
    "24.132102": "BBN",
    "12.346866012160": "UG Systems GmbH & Co. KG",
    "20.277958187024384": "Ningbo Bull Digital Technology Co., LTD",
    "24.8709592": "Guangdong UNIPOE IoT Technology Co.,Ltd.",
    "12.123917677010944": "Sense For Innovation",
    "24.5305632": "Samsung Electronics Co.,Ltd",
    "24.3203715": "Texas Instruments",
    "24.5358": "Western Digital Technologies, Inc.",
    "24.10806805": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.1878": "Juyoung Telecom",
    "24.2407941": "Hewlett Packard",
    "12.123917677641728": "Globalcom Engineering SPA",
    "12.123917679136768": "Olympus NDT Canada",
    "24.1875": "Beijing Qxcomm Technology Co., Ltd.",
    "20.180394588635136": "Beijing XiaoRui Technology Co., Ltd",
    "24.12588216": "Nokia",
    "24.5355": "AwarePoint Corporation",
    "24.5356": "Acro Telecom",
    "12.346858098688": "AYC Telecom Ltd",
    "24.1876": "Xyterra Computing, Inc.",
    "24.5523990": "Texas Instruments",
    "12.123917685690368": "Private",
    "24.1871": "Cisco Systems, Inc",
    "24.3948369": "2Crsi",
    "24.1870": "IPFRONT Inc",
    "24.12586366": "Additech, Inc.",
    "24.11860985": "Huawei Technologies Co.,Ltd",
    "24.1873": "m-u-t AG",
    "24.12624740": "3D Systems Massachusetts",
    "24.5353": "Nortech International",
    "12.346855096320": "Embedded Toolsmiths",
    "24.9205441": "Goden Tech Limited",
    "24.13951118": "ValueHD Corporation",
    "12.123917683957760": "Perceptics, LLC",
    "24.8709337": "Shenzhen NEED technology Ltd.",
    "12.123917681274880": "Teleplatforms",
    "24.16287729": "Apple, Inc.",
    "12.123917686157312": "Lineage Power Pvt Ltd.,",
    "12.346858532864": "Tattile Srl",
    "24.3713131": "Orga BV",
    "12.71297883660288": "Internet Protocolo Logica Sl",
    "24.307431": "Huawei Technologies Co.,Ltd",
    "12.346854178816": "COSMO co.,ltd.",
    "24.2372460": "Private",
    "12.346865627136": "Saia-Burgess Controls AG",
    "12.123917690257408": "Fracarro srl",
    "24.7111526": "Nanjing SAC Power Grid Automation Co., Ltd.",
    "24.10263616": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.16529195": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.71297883422720": "Computer System Co.,Ltd",
    "24.13161872": "Flight Data Systems",
    "24.13395110": "Huawei Technologies Co.,Ltd",
    "12.123917685727232": "EPSOFT Co., Ltd",
    "24.5259094": "Syncmold Enterprise Corp",
    "20.31477072920576": "LYT inc.",
    "24.12622852": "Shaanxi GuoLian Digital TV Technology Co.,Ltd.",
    "12.123917686444032": "Cari Electronic",
    "24.15479149": "Hansgrohe",
    "20.225135233597440": "Shenzhen Uyesee Technology Co.,Ltd",
    "24.13654781": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "20.27478321004544": "CreyNox GmbH",
    "24.2646086": "Lantech Communications Global, Inc.",
    "12.123917683888128": "ZIGPOS GmbH",
    "12.346865078272": "InForce Computing, Inc.",
    "24.9722157": "EKE Building Technology Systems Ltd",
    "12.346862006272": "Lécureux SA",
    "24.14710566": "Huawei Device Co., Ltd.",
    "24.13170376": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11580488": "Texas Instruments",
    "12.346858962944": "Gogo BA",
    "24.1980": "Identix Inc.",
    "12.346868011008": "Grossenbacher Systeme AG",
    "24.12865331": "Espressif Inc.",
    "24.41043": "Compact Devices, Inc.",
    "24.41042": "Stanilite Electronics Pty. Ltd",
    "24.41041": "Angia Communications. Inc.",
    "24.41040": "Cypress Semiconductor",
    "24.41047": "LANCOM Systems GmbH",
    "24.41046": "Micropross",
    "24.41045": "Data Device Corporation",
    "24.41044": "Private",
    "24.41049": "Hamilton Hallmark",
    "24.41048": "Glory, Ltd.",
    "12.123917686194176": "SensoTec GmbH",
    "24.13924139": "Nanjing Ztlink Network Technology Co.,Ltd",
    "24.3091": "MediaQ",
    "24.4457540": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.16571169": "Samsara Networks Inc",
    "20.111031369072640": "Kay Schulze & Karsten Pohle GbR",
    "12.346860462080": "Monitor Business Machines Ltd.",
    "24.4203698": "Sichuan AI-Link Technology Co., Ltd.",
    "24.4216121": "Cisco Systems, Inc",
    "24.9705602": "Hewlett Packard Enterprise",
    "24.1318842": "IEEE Registration Authority",
    "12.346854891520": "Packet Techniques Inc.",
    "24.546981": "Beijing Vrv Software Corpoaration Limited.",
    "24.49292": "Performance Technologies, Inc.",
    "24.16517239": "Prentke Romich Company",
    "24.831322": "Genus Power Infrastructures Limited",
    "24.13935769": "InView Technology Corporation",
    "24.11059794": "Huawei Device Co., Ltd.",
    "24.6156": "Optelian Access Networks",
    "24.5546779": "Shenzhen Linkworld Technology Co,.LTD",
    "24.15483878": "AWCER Ltd.",
    "12.119270010880": "TN Core Co.,Ltd.",
    "12.346863448064": "Diversified Control, Inc.",
    "24.8960639": "Qfiednet Co., Ltd.",
    "24.12624750": "BSkyB Ltd",
    "24.14161418": "Nippon Electro-Sensory Devices",
    "20.5239435427840": "Flintec UK Ltd.",
    "24.6041918": "Texas Instruments",
    "24.4728167": "Poly",
    "24.9980487": "CHG Hospital Beds",
    "24.10749008": "nFore Technology Inc.",
    "12.119269433344": "Osborne Coinage Co",
    "24.292099": "Huawei Technologies Co.,Ltd",
    "24.7917865": "Vicos",
    "12.346860134400": "JMC America, LLC",
    "24.1367525": "Mpmkvvcl",
    "20.150262242082816": "FlashForge Corporation",
    "24.833847": "Ubee Interactive Co., Limited",
    "20.27056774578176": "Beijing Xinertel Technology Co., Ltd.",
    "12.346867859456": "HCL Technologies",
    "24.14200528": "Shanghai Engineering Science & Technology Co.,LTD CGNPC",
    "24.2108101": "Petite-En",
    "24.12350129": "Samsung Electronics Co.,Ltd",
    "12.346867429376": "STT Condigi A/S",
    "12.346861383680": "Beonic Corporation",
    "24.815833": "zte corporation",
    "24.9181264": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "20.224447332089856": "Safilo S.p.A.",
    "24.7638536": "Bestek Corp.",
    "12.346857558016": "VASTech",
    "24.8677496": "Apple, Inc.",
    "20.172584088895488": "b8ta Inc.",
    "24.7976": "HPN Supply Chain",
    "24.7977": "Hewlett Packard",
    "24.7974": "Cisco Systems, Inc",
    "24.7975": "Cisco Systems, Inc",
    "24.7972": "Digitview Technology Co., Ltd.",
    "24.7973": "MBS GmbH",
    "24.7970": "Source Photonics, Inc.",
    "24.7971": "Interacoustics",
    "24.7361746": "Pegatron Corporation",
    "24.8966772": "zte corporation",
    "24.5553529": "Samsung Electronics Co.,Ltd",
    "24.7978": "Accm",
    "24.7979": "Orange Logic",
    "24.1588178": "BYD Precision Manufacture Company Ltd.",
    "12.123917680451584": "Talleres de Escoriaza SA",
    "12.346854207488": "Radionor Communications",
    "24.6332317": "CloudSwitch",
    "12.346854744064": "Tieline Research Pty Ltd",
    "24.8146441": "Shenzhen Skyworth Wireless Technology Co.,Ltd",
    "24.2362122": "Palo Alto Networks",
    "24.15732415": "ZettaHash Inc.",
    "12.123917684232192": "EA Elektroautomatik GmbH & Co. KG",
    "24.13419232": "Raisecom Technology CO., LTD",
    "20.264625132142592": "Guilin glsun Science and Tech Co.,LTD",
    "24.3729631": "b gmbh medien + datenbanken",
    "24.14482951": "Pegatron Corporation",
    "12.346866987008": "Vitec",
    "24.9231757": "Wifly-City System Inc.",
    "24.1752059": "UHD Alliance",
    "24.11024708": "Actiontec Electronics, Inc",
    "24.10537376": "Murata Manufacturing Co., Ltd.",
    "12.346863075328": "X-Pert Paint Mixing Systems",
    "24.3180918": "Dalian Lamba Technology Co.,Ltd",
    "24.3956481": "Synerchip Co., Ltd.",
    "24.9716804": "LG Innotek",
    "24.4984686": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.2943615": "Dell Inc.",
    "24.12605155": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.11806961": "E-Prime Co., Ltd.",
    "24.10521848": "Huawei Technologies Co.,Ltd",
    "24.3992143": "2Wire Inc",
    "24.1576108": "Canon Inc.",
    "24.15236971": "Samsung Electronics Co.,Ltd",
    "20.136823640686592": "Mennekes Elektrotechnik GmbH & Co. KG",
    "24.14202582": "Blu Tether Limited",
    "12.346865811456": "Tehama Wireless",
    "24.6556490": "Digital Telecom Technology Limited",
    "24.10765593": "Xiaomi Communications Co Ltd",
    "24.53309": "Galileo Technology, Ltd.",
    "24.53308": "Vieo, Inc.",
    "24.53307": "Vision Products Pty. Ltd.",
    "24.53306": "Zoneworx, Inc.",
    "24.53305": "Utilicom, Inc.",
    "24.53304": "Fivemere, Ltd.",
    "24.53303": "ARRIS Group, Inc.",
    "24.53302": "Technology Atlanta Corp.",
    "24.53301": "Behavior Tech. Computer Corp.",
    "24.53300": "Ormec Systems Corp.",
    "12.123917682733056": "Vocality international T/A Cubic",
    "24.9972924": "Intel Corporate",
    "20.277981940416512": "Bee Smart(Changzhou) Information Technology Co., Ltd",
    "20.224420095328256": "Beijing Daotongtianxia Co.Ltd.",
    "12.346863333376": "Endeas Oy",
    "24.9228029": "Shenzhen zhouhai technology co.,LTD",
    "24.3164475": "Nanjing Z-Com Wireless Co.,Ltd",
    "24.12358437": "Ningbo Joyson Preh Car Connect Co.,Ltd.",
    "20.159182272593920": "PowerShield Limited",
    "24.16026704": "Intel Corporate",
    "24.9086": "Elster Gmbh",
    "24.6057248": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9087": "Plantronics, Inc.",
    "24.3983238": "Dongguan Huarong Communication Technologies Co.,Ltd.",
    "12.123917677645824": "Bentec GmbH Drilling & Oilfield Systems",
    "24.282118": "Ericsson AB",
    "24.16288316": "IEEE Registration Authority",
    "20.45069252952064": "Grandway Technology (Shenzhen) Limited",
    "24.6613071": "Serialway Communication Technology Co. Ltd",
    "24.5280591": "Gradual Tecnologia Ltda.",
    "12.123917689786368": "BAB TECHNOLOGIE GmbH",
    "24.4776367": "Vity",
    "24.5608": "Nortel Networks",
    "24.5609": "D-Link Corporation",
    "24.5606": "MOBILE TECHNIKA Inc.",
    "24.5607": "Quantec Tontechnik",
    "24.5604": "Zimmer Elektromedizin",
    "24.5605": "Cheertek Inc.",
    "24.5602": "Dr.Ing. Herbert Knauer GmbH",
    "24.5603": "Dream Technologies Corporation",
    "24.5600": "Ericsson",
    "24.5601": "Picochip Ltd",
    "24.16037290": "zte corporation",
    "20.44213339160576": "Georg Neumann GmbH",
    "24.3182202": "Shenzhen iComm Semiconductor CO.,LTD",
    "24.5027884": "Cambridge Mobile Telematics, Inc.",
    "12.346867183616": "NEXT video systems Hard- and Software Development GmbH",
    "24.7379228": "Shenzhen Honesty Electronics Co.,Ltd",
    "24.11572059": "ARRIS Group, Inc.",
    "24.9089": "Lengda Technology(Xiamen) Co.,Ltd.",
    "24.8152994": "Intel Corporate",
    "24.16003257": "Hewlett Packard",
    "24.3608": "MyA Technology",
    "24.3609": "LogicaCMG Pty Ltd",
    "24.3600": "C-guys, Inc.",
    "24.3601": "BDT Büro und Datentechnik GmbH & Co.KG",
    "24.3602": "Adaptive Micro Systems Inc.",
    "24.3603": "Accu-Sort Systems inc.",
    "24.3604": "Visionary Solutions, Inc.",
    "24.3605": "Tadlys LTD",
    "24.3606": "SouthWing S.L.",
    "24.3607": "Private",
    "24.2360712": "Google, Inc.",
    "20.44768526598144": "Zhuhai RongBang Electronic Technology Co., Ltd.",
    "24.7662464": "Samsung Electronics Co.,Ltd",
    "12.346862247936": "Engage Technologies",
    "20.71070621106176": "TOPROOTTechnology Corp. Ltd.",
    "12.123917684305920": "Dorlet Sau",
    "20.62070501408768": "Proch plastic Co., Ltd.",
    "12.346860343296": "Precision MicroControl Corporation",
    "24.14731642": "Apple, Inc.",
    "12.123917677387776": "Saline Lectronics, Inc.",
    "24.1597476": "Hewlett Packard",
    "12.346852433920": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "24.8686060": "IEEE Registration Authority",
    "24.1328202": "Apollo Seiko Ltd.",
    "12.123917677895680": "Quantum Detectors",
    "12.123917686267904": "Mart Network Solutions Ltd",
    "12.123917677527040": "Cardinal Scale Mfg Co",
    "12.346852302848": "Ashling Microsystems Ltd.",
    "24.9978": "BandRich Inc.",
    "24.9979": "AirDio Wireless, Inc.",
    "20.220958197547008": "Heitec Ag",
    "24.9974": "Military Communication Institute",
    "24.9975": "Nivetti Systems Pvt. Ltd.",
    "24.9976": "Golden Highway Industry Development Co., Ltd.",
    "24.9977": "S.E.M. srl",
    "24.9970": "Netgear",
    "24.9971": "SMC Networks",
    "24.9972": "Nesslab",
    "24.9973": "XRPLUS Inc.",
    "12.346857263104": "Truteq Wireless (PTY) Ltd.",
    "12.346859372544": "FIRSTTRUST Co.,Ltd.",
    "24.9460788": "Sirius Electronic Systems Srl",
    "24.2409465": "Securitas Direct AB",
    "24.5525061": "Private",
    "12.346862608384": "Ion Sense Inc.",
    "20.167137985429504": "Zedi, Inc.",
    "24.11289392": "Versa Networks",
    "12.346854895616": "ACD Elektronik GmbH",
    "24.16564958": "Murata Manufacturing Co., Ltd.",
    "24.8406269": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "20.128510957977600": "Altenburger Electronic GmbH",
    "24.49397": "Metacomp, Inc.",
    "24.4511141": "AddOn Computer",
    "24.6575487": "NSFOCUS Information Technology Co., Ltd.",
    "24.3199358": "Huawei Technologies Co.,Ltd",
    "24.15793352": "ARRIS Group, Inc.",
    "12.123917682819072": "Micram Instruments Ltd",
    "24.12876591": "Beijing JoinHope Image Technology Ltd.",
    "24.49396": "Interlink System Co., Ltd.",
    "12.123917677953024": "ijin co.,ltd.",
    "24.5008118": "HMD Global Oy",
    "12.123917675999232": "MonsoonRF, Inc.",
    "24.4086": "Darfon Lighting Corp",
    "24.828418": "Alcatel-Lucent IPD",
    "24.12597497": "Nokia Danmark A/S",
    "24.3196752": "MIC Technology Group",
    "24.4488613": "Vtion Information Technology (Fujian) Co.,Ltd",
    "12.119269388288": "CAMEA, spol. s r.o.",
    "24.20671": "Metalligence Technology Corp.",
    "24.20670": "Fast Multimedia Ag",
    "24.20673": "Gemflex Networks, Ltd.",
    "24.20672": "Gatan, Inc.",
    "24.49274": "Priva BV",
    "24.20674": "IEEE Registration Authority",
    "24.20677": "ADS Technologies, Inc",
    "24.20676": "Imd",
    "24.20679": "Private",
    "24.20678": "Loop Telecommunication International, Inc.",
    "24.13693941": "Huawei Device Co., Ltd.",
    "12.123917679718400": "ASE GmbH",
    "24.1599295": "Hanover Displays Limited",
    "24.10521223": "HuiZhou KaiYue Electronic Co.,Ltd",
    "20.5239428087808": "Innovusion Inc.",
    "24.13924274": "Netix Global B.V.",
    "24.9465957": "Texas Instruments",
    "12.346863304704": "Smith Meter, Inc.",
    "12.346866421760": "T-Industry, s.r.o.",
    "24.3676463": "Nokia Corporation",
    "12.123917689872384": "Merz s.r.o.",
    "24.8735": "eSang Technologies Co., Ltd.",
    "12.346858745856": "R&D Technology Solutionz Limited",
    "24.8737": "Itoh Denki Co,Ltd.",
    "24.8736": "Mitac Technology Corp",
    "24.8731": "Morega Systems",
    "24.8730": "Audio Precision",
    "24.8733": "Freegene Technology LTD",
    "24.8732": "Private",
    "24.15484022": "Cisco Systems, Inc",
    "24.8739": "TimeKeeping Systems, Inc.",
    "24.8738": "Schaffner Deutschland GmbH",
    "12.123917681315840": "Aplex Technology Inc.",
    "24.14730931": "MilDef AB",
    "24.16004833": "Abilis Systems SARL",
    "24.11806301": "Taicang T&W Electronics",
    "12.123917682982912": "Adde",
    "24.1056081": "8Mesh Networks Limited",
    "12.346861666304": "Autotank AB",
    "24.11857588": "Askey Computer Corp",
    "12.123917676826624": "Shenzhen Siera Technology Ltd",
    "24.2159": "Resources Computer Network Ltd.",
    "24.2158": "Hyglo AB",
    "24.2157": "Missouri FreeNet",
    "24.2156": "Plasmon LMS",
    "24.2155": "Mipsys",
    "24.2154": "Securiton Gmbh",
    "24.2153": "Command-e Technology Co.,Ltd.",
    "24.2152": "PurOptix",
    "24.2151": "Uptime Devices",
    "24.2150": "DSX Access Systems, Inc.",
    "24.6876357": "Intel Corporate",
    "24.49390": "Kyocera Corporation",
    "24.13152923": "Intel Corporate",
    "24.1871502": "DB Communication & Systems Co., ltd.",
    "24.10752354": "Arlo Technology",
    "20.110373781897216": "Annapurna labs",
    "12.123917679329280": "Ecoinet",
    "24.8923548": "Samsung Electronics Co.,Ltd",
    "20.83999504465920": "Suzhou Embedded Electronic Technology Co., Ltd.",
    "24.14725873": "Fn-Link Technology Limited",
    "20.212188454191104": "LongSung Technology (Shanghai) Co.,Ltd.",
    "24.5544465": "IEEE Registration Authority",
    "24.6728": "Takacom Corporation",
    "24.6729": "Micro Vision Co.,LTD",
    "24.11821741": "Elysia Germany GmbH",
    "24.5562797": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.5688": "TECOM Co., Ltd.",
    "24.6720": "A-Four Tech Co., Ltd.",
    "24.6721": "INOCOVA Co.,Ltd",
    "24.6722": "Techcity Technology co., Ltd.",
    "24.6723": "Logical Link Communications",
    "24.6724": "JWTrading Co., Ltd",
    "24.6725": "GN Netcom A/S",
    "24.6726": "Digital Multimedia Technology Co., Ltd",
    "24.6727": "Agami Systems, Inc.",
    "24.1718": "Nir-Or Israel Ltd.",
    "24.1719": "TELEM GmbH",
    "24.4748804": "Private",
    "24.6338054": "Phorus",
    "24.1710": "Himachal Futuristic Communications Ltd",
    "24.1711": "Xalted Networks",
    "24.1712": "Comtech EF Data Corp.",
    "24.1713": "Sonicwall",
    "24.1714": "Linxtek Co.",
    "24.1715": "Diagraph Corporation",
    "24.1716": "Vorne Industries, Inc.",
    "24.1717": "Source Photonics, Inc.",
    "24.2420221": "Siemens Canada Limited",
    "24.15206196": "Champion Optical Network Engineering, LLC",
    "24.9852": "Metz-Werke GmbH & Co KG",
    "24.14194325": "Apple, Inc.",
    "12.123917682966528": "Tiama",
    "24.6591852": "LG Electronics",
    "24.6048287": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.14979177": "Rockwell Automation",
    "20.260167876739072": "MAX Technologies",
    "12.346858180608": "Nomus Comm Systems",
    "24.11826247": "CoreOS",
    "24.11826243": "Samsung Electronics Co.,Ltd",
    "24.2153702": "Tp-Link Technologies Co.,Ltd.",
    "24.11821293": "Beijing Xiaomi Mobile Software Co., Ltd",
    "24.16273507": "Verathon",
    "24.7613": "Versamed Inc.",
    "12.123917684297728": "Vertex Co.,Ltd.",
    "24.8426604": "Tokyo Radar Corporation",
    "24.3854277": "Quickline AG",
    "24.15526840": "Hewlett Packard Enterprise",
    "24.3160676": "Intel Corporate",
    "24.5267926": "Indu-Sol GmbH",
    "24.14998381": "Beijing Xiaomi Electronics Co., Ltd.",
    "20.242213049073664": "Hefei Datang Storage Technology Co.,Ltd",
    "24.16515999": "Samsung Electronics Co.,Ltd",
    "12.123917676449792": "MB connect line GmbH Fernwartungssysteme",
    "24.5513976": "zte corporation",
    "20.272689498423296": "Rosemount Analytical",
    "24.1333891": "Logi-D inc",
    "20.35227547205632": "Tecnint HTE SRL",
    "12.123917687713792": "Behr Technologies Inc",
    "24.2392490": "KSH International Co., Ltd.",
    "24.9129": "Beijing Detianquan Electromechanical Equipment Co., Ltd",
    "24.9128": "Marshall Electronics",
    "24.37370": "Synapse Product Development",
    "12.71297883385856": "Ermes Elettronica s.r.l.",
    "24.7739": "Nokia Danmark A/S",
    "24.9123": "ARRIS Group, Inc.",
    "24.9122": "ARRIS Group, Inc.",
    "24.9121": "Trend Electronics Ltd",
    "24.9120": "Hana CNS Co., LTD.",
    "24.9127": "Redpine Signals, Inc.",
    "24.9126": "E-Mon",
    "24.9125": "SageTV, LLC",
    "24.9124": "New Concepts Development Corp.",
    "20.52819398754304": "essence security",
    "20.79215081816064": "M2Lab Ltd.",
    "24.837205": "Wuxi YSTen Technology Co.,Ltd.",
    "24.3943156": "Brother Industries, LTD.",
    "24.4779532": "D-Link International",
    "12.346852397056": "Apex Signal Corp",
    "24.13113847": "Samsung Electronics Co.,Ltd",
    "12.123917682167808": "Potter Electric Signal Co. LLC",
    "24.6355661": "Apple, Inc.",
    "12.346861342720": "Heraeus Noblelight GmbH",
    "24.10272110": "DERA Co., Ltd",
    "20.119709246685184": "Sercomm Corporation.",
    "24.7908876": "Far South Networks",
    "24.9702552": "Apple, Inc.",
    "12.123917677211648": "Lab241 Co.,Ltd.",
    "24.12859957": "Fn-Link Technology Limited",
    "24.3683": "Lemke Diagnostics GmbH",
    "12.346857091072": "ANSA Corporation",
    "24.13947465": "Power Ethernet",
    "24.323945": "Huawei Device Co., Ltd.",
    "24.2383039": "Ncse",
    "24.2137847": "Extreme Networks, Inc.",
    "24.8420628": "Huawei Technologies Co.,Ltd",
    "24.7365655": "Orbis Tecnologia Electrica S.A.",
    "24.11283875": "TXTR GmbH",
    "24.4516005": "Myreka Technologies Sdn. Bhd.",
    "12.71297884782592": "Array Corporation",
    "12.71297884487680": "InnoTrans Communications, Inc",
    "24.3411814": "Web Sensing LLC",
    "12.346859102208": "Ideus AB",
    "24.5545781": "Dell Inc.",
    "24.5818677": "Chiun Mai Communication Systems, Inc",
    "24.15756811": "Microsoft Corporation",
    "24.9206965": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.36959": "Cisco Systems, Inc",
    "24.36958": "Rauland-Borg Corporation",
    "24.11861109": "SEMA Electronics(HK) CO.,LTD",
    "12.123917679226880": "Idosens",
    "24.36953": "Telecom Device K.K.",
    "12.346866712576": "Internet Protocolo Logica Sl",
    "24.36951": "AANetcom, Inc.",
    "24.36950": "Telestream, Inc.",
    "24.36957": "Netcom Sicherheitstechnik Gmbh",
    "24.36956": "Edmi",
    "24.36955": "Raymond And Lae Engineering",
    "24.36954": "Dearborn Group, Inc.",
    "24.310664": "Aruba, a Hewlett Packard Enterprise Company",
    "24.1616133": "Hewlett Packard",
    "24.2889845": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.312294": "IEEE Registration Authority",
    "24.535734": "Apple, Inc.",
    "24.15792839": "Huawei Device Co., Ltd.",
    "24.9985563": "ConversDigital Co., Ltd.",
    "24.14328089": "Google, Inc.",
    "24.57454": "FAR SYSTEMS S.p.A.",
    "12.346860515328": "MangoDSP",
    "24.6077167": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.16444270": "xRAN.org",
    "24.3199831": "Logosol, Inc.",
    "24.15732880": "New H3C Technologies Co., Ltd",
    "12.123917688098816": "Metatronics B.V.",
    "12.123917684973568": "3S - Sensors, Signal Processing, Systems GmbH",
    "24.8127751": "Pegatron Corporation",
    "24.312740": "Cisco Systems, Inc",
    "24.5569368": "WISEWARE, Lda",
    "24.13893645": "Phoenix Broadband Technologies, LLC.",
    "12.71297884954624": "AGE A. Gilg Elektronik",
    "24.15773006": "Globalscale Technologies, Inc.",
    "12.346860765184": "ANTEK GmbH",
    "24.2385135": "Cyg Sunri Co.,Ltd.",
    "12.71297884655616": "Photop Koncent",
    "24.5780843": "Tecno Mobile Limited",
    "20.277958198558720": "HEAD-DIRECT (KUNSHAN) Co. Ltd",
    "20.49277263937536": "Square Inc.",
    "24.816402": "Shenzhen Kunlun TongTai Technology Co.,Ltd.",
    "20.57194435313664": "GoChip Inc.",
    "24.9231954": "ISSC Technologies Corp.",
    "24.12870934": "Hewlett Packard",
    "24.6337783": "Plume Design Inc",
    "24.15517826": "Cisco Systems, Inc",
    "24.10535902": "Triton Electronic Systems Ltd.",
    "24.5268867": "Texas Instruments",
    "12.346852446208": "Chase Manhattan Bank",
    "24.8942053": "Amazon Technologies Inc.",
    "24.3453780": "Huawei Technologies Co.,Ltd",
    "12.123917681926144": "Altaneos",
    "24.6347395": "Hangzhou H3C Technologies Co., Limited",
    "12.123917675831296": "Vitec",
    "12.71297884864512": "Parris Service Corporation",
    "24.6092": "Alcatel-Lucent",
    "24.16299417": "Guangzhou CHNAVS Digital Technology Co.,Ltd",
    "24.2886091": "Cobs Ab",
    "24.2660475": "Samsung Electronics Co.,Ltd",
    "12.346861096960": "Ooo \"Ntk \"Imos\"",
    "12.123917687246848": "Technik & Design GmbH",
    "12.71297884536832": "Plt",
    "24.11286035": "Samsung Electronics Co.,Ltd",
    "24.12893908": "Avaya Inc",
    "24.524373": "Stanford Telecomm. Inc.",
    "12.123917686546432": "Nordic Automation Systems AS",
    "20.97400604590080": "Epiphan Systems Inc",
    "24.53319": "Xn Technologies",
    "24.15267412": "Huawei Technologies Co.,Ltd",
    "24.10233920": "Discovery Technology,LTD..",
    "24.1107910": "Hewlett Packard",
    "12.346861645824": "E.ON ES Sverige AB",
    "24.2927891": "Shenzhen Zhilu Technology Co.,Ltd",
    "24.9197467": "Wistron Neweb Corporation",
    "24.4752327": "Sagemcom Broadband SAS",
    "24.10265893": "Shandong USR IOT Technology Limited",
    "24.15789584": "Samsung Electronics Co.,Ltd",
    "20.71389206806528": "Cape",
    "12.346865074176": "Beceem Communications, Inc.",
    "24.6036953": "Apple, Inc.",
    "12.346856833024": "Venue 1, Inc.",
    "24.9469793": "Chicony Electronics Co., Ltd.",
    "24.53312": "Sysmate Co., Ltd.",
    "24.4740117": "Net Rules Tecnologia Eireli",
    "20.66129345118208": "Laipac Technology Inc.",
    "20.277958189121536": "Loupedeck Oy",
    "24.4244422": "SnapRoute",
    "24.16264923": "RTW GmbH & Co. KG",
    "24.5300463": "Nokia",
    "12.346862317568": "KRESS-NET Krzysztof Rutecki",
    "24.527748": "GECO, Inc.",
    "24.1072045": "Mega Well Limited",
    "12.123917679894528": "Twoway Communications, Inc.",
    "24.7083513": "Nautronix Limited",
    "24.15994369": "Apple, Inc.",
    "24.6331316": "Huawei Technologies Co.,Ltd",
    "12.346857762816": "Netcomsec Co Ltd",
    "24.274262": "ATOCS (Shenzhen) LTD",
    "12.346864197632": "Schnoor Industrieelektronik GmbH & Co. KG",
    "24.4205812": "Universal Global Scientific Industrial Co., Ltd.",
    "24.9235398": "Huawei Technologies Co.,Ltd",
    "24.14559": "Cisco Systems, Inc",
    "24.5307551": "Samsung Electronics Co.,Ltd",
    "24.4201311": "Weifang Goertek Electronics Co.,Ltd",
    "24.16052631": "EMTAKE Inc",
    "20.106480347906048": "Revol Technologies inc",
    "24.9211510": "taskit GmbH",
    "12.123917680472064": "Biennebi s.r.l.",
    "24.10544617": "enimai Inc",
    "24.8917565": "Suzhou Aquila Solutions Inc.",
    "24.11310083": "Envision Peripherals Inc",
    "12.346860875776": "Gamber-Johnson LLC",
    "12.346861088768": "Neocontrol Soluções em Automação",
    "24.9199583": "Beijing Railway Signal Factory",
    "12.123917678407680": "Xantia Sa",
    "24.14463106": "Nokia",
    "24.13275045": "Advanced TS Migrations, Co",
    "24.9969167": "Jeelan (Shanghai Jeelan Technology Information Inc",
    "12.123917678538752": "Aplex Technology Inc.",
    "24.552257": "Apple, Inc.",
    "24.8972567": "Tamaggo",
    "24.5011822": "Intel Corporate",
    "12.123917683974144": "RCH Vietnam Limited Liability Company",
    "24.8182270": "New Cosmos Electric Co., Ltd.",
    "24.9756174": "SmartOptics AS",
    "12.123917677600768": "Td Ecophisika",
    "24.3468007": "Shanghai High-Flying Electronics Technology Co., Ltd",
    "24.13160715": "Nokia Corporation",
    "20.176726067904512": "AutarcTech GmbH",
    "24.14726445": "Apple, Inc.",
    "20.233258807721984": "Nanjing LES Information Technology Co., Ltd",
    "12.346862952448": "DITRON S.r.l.",
    "24.9757882": "Huawei Device Co., Ltd.",
    "12.123917677260800": "Montalvo",
    "12.119269392384": "Stechwin.Co.Ltd.",
    "12.346855079936": "Selco",
    "12.346857324544": "Thales Defence and Security Systems GmbH",
    "24.3446": "Hokuto Denshi Co,. Ltd.",
    "24.3447": "FalconStor Software",
    "24.3444": "Sand Network Systems, Inc.",
    "24.3445": "Kobian Pte Ltd - Taiwan Branch",
    "24.3442": "2Wire Inc",
    "24.3443": "Technical Support, Inc.",
    "24.3440": "Datamax Corporation",
    "24.3441": "boca systems",
    "20.75247169568768": "Scaime",
    "24.3448": "Engineering & Security",
    "24.3449": "Dynamic Solutions Co,.Ltd.",
    "24.6325044": "Huawei Technologies Co.,Ltd",
    "24.3673723": "V.P.S. Thai Co., Ltd",
    "24.10000337": "MitraStar Technology Corp.",
    "24.13387834": "Dell Inc.",
    "12.346866368512": "iRphotonics",
    "12.123917678026752": "Spark Lasers",
    "12.123917688250368": "De Haardt bv",
    "24.10781289": "Datrium, Inc.",
    "24.12073629": "Alarm.com",
    "24.7119104": "Intel Corporate",
    "24.8191528": "Salutron Inc.",
    "24.13110903": "Le Shi Zhi Xin Electronic Technology (Tianjin) Limited",
    "24.4780592": "Ubizcore Co.,LTD",
    "12.346856681472": "kippdata GmbH",
    "24.2646164": "Capelec",
    "12.123917684736000": "Vision Systems Aurtomotive (Safety Tech)",
    "24.8702406": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.549729": "Huawei Technologies Co.,Ltd",
    "24.11296574": "Samsung Electro-Mechanics(Thailand)",
    "12.346857836544": "Twinwin Technplogy Co.,Ltd.",
    "24.6582323": "Apple, Inc.",
    "12.346858168320": "SAIA Burgess Controls AG",
    "24.9757222": "D. ORtiz Consulting, LLC",
    "20.14223581118464": "BSX Athletics",
    "24.12084539": "Bolymin, Inc.",
    "24.3190518": "Radio Mobile Access",
    "24.9712906": "Samsung Electronics Co.,Ltd",
    "24.10542869": "Huawei Technologies Co.,Ltd",
    "12.346856951808": "Fr. Sauter AG",
    "24.10268143": "Obihai Technology, Inc.",
    "24.14980782": "ALTOGRAPHICS Inc.",
    "24.3029": "Nvergence, Inc.",
    "24.8414297": "Nimbus Inc.",
    "24.13924074": "zte corporation",
    "24.2383775": "Huawei Device Co., Ltd.",
    "12.346860814336": "Residential Control Systems",
    "20.137149124968448": "Beijing Topnew Group Co., Ltd",
    "24.2766": "Radiantech, Inc.",
    "24.843684": "Samsung Electronics Co.,Ltd",
    "24.2145142": "COL GIOVANNI PAOLO SpA",
    "24.2767": "PROVIDEO Multimedia Co. Ltd.",
    "24.2768": "Niigata Develoment Center, F.I.T. Co., Ltd.",
    "20.66426706591744": "Xi'An Yep Telecom Technology Co.,Ltd",
    "24.2769": "Mws",
    "24.3444519": "Ruckus Wireless",
    "12.123917678931968": "Star Electronics GmbH & Co. KG",
    "20.224447333138432": "E Ink Corp",
    "24.4241463": "Sony Mobile Communications Inc",
    "24.5026890": "NANOWAVE Technologies Inc.",
    "24.16047269": "Dawon Dns",
    "24.9499575": "Kirisun Communications Co., Ltd.",
    "20.49277259743232": "Beijing Hexing Chuangxiang Technology Co., Ltd.",
    "12.71297884749824": "BroadSoft Inc",
    "24.2908147": "Pertronic Industries",
    "24.8173435": "Paradigm Electronics Inc",
    "24.5815801": "Sigrand",
    "12.123917683675136": "Harborside Technology",
    "24.8919214": "Tamron Co., Ltd",
    "24.10492933": "Nimax Telecom Co.,Ltd.",
    "24.10502181": "Actioncable, Inc.",
    "24.7631645": "ifm electronic gmbh",
    "12.346852171776": "Integrated Automation Solutions",
    "12.123917687152640": "Oculii",
    "24.3863841": "Single Digits",
    "24.15494325": "zte corporation",
    "24.4251089": "Beautiful Card Corporation",
    "24.13128002": "Samsung Electronics Co.,Ltd",
    "24.7877199": "Millennium Group, Inc.",
    "24.5530883": "Quectel Wireless Solutions Co., Ltd.",
    "24.3441140": "SST Taiwan Ltd.",
    "24.8700353": "Juniper Networks",
    "12.346854187008": "Perto Periféricos de Automação S.A.",
    "24.13158224": "Motorola Mobility LLC, a Lenovo Company",
    "24.564043": "iKuai Networks",
    "12.346864828416": "Spectronix Corporation",
    "24.2102240": "Samsung Electronics Co.,Ltd",
    "24.1615017": "Vanu Inc.",
    "24.64802": "Cisco Systems, Inc",
    "12.346857852928": "Computrol Fuel Systems Inc.",
    "12.346854363136": "Aeroflex GmbH",
    "24.6250": "Global Link Digital Technology Co,.LTD",
    "24.6251": "Sambu Communics CO., LTD.",
    "24.6252": "Neonode AB",
    "24.6253": "Zhenjiang Sapphire Electronic Industry CO.",
    "24.6254": "3Com Ltd",
    "24.6255": "Setha Industria Eletronica LTDA",
    "24.6256": "E28 Shanghai Limited",
    "24.6257": "Hewlett Packard",
    "24.6258": "Expertise Engineering",
    "24.6259": "Cisco Systems, Inc",
    "20.229558886334464": "E-T-A Elektrotechnische Apparate GmbH",
    "24.8408705": "Samsung Electronics Co.,Ltd",
    "24.8403132": "Huawei Technologies Co.,Ltd",
    "24.14183415": "Nintendo Co., Ltd.",
    "24.9228667": "CG Mobile",
    "24.8685997": "Apple, Inc.",
    "24.12640119": "Conrad Electronic SE",
    "24.12625214": "Apple, Inc.",
    "24.16311834": "Samsung Electronics Co.,Ltd",
    "12.346854920192": "Infinetix Corp",
    "24.15268841": "Apple, Inc.",
    "12.123917682098176": "Veo Robotics, Inc.",
    "24.6327370": "Apple, Inc.",
    "24.13411426": "ARRIS Group, Inc.",
    "12.123917683748864": "Rch Italia Spa",
    "12.346852474880": "New Standard Engineering NV",
    "12.346854359040": "Nansen S. A. - Instrumentos de Precisão",
    "24.1156": "Amann GmbH",
    "24.1157": "PicoLight",
    "24.1154": "Medialogic Corp.",
    "24.1155": "Deltron Technology, Inc.",
    "24.1152": "Brocade Communications Systems LLC",
    "24.1153": "Econolite Control Products, Inc.",
    "24.1150": "Siqura B.V.",
    "24.1151": "Chr. Mayr GmbH & Co. KG",
    "12.123917690777600": "RF Industries",
    "24.8512": "EN Technologies Inc.",
    "24.1158": "ITTC, University of Kansas",
    "24.1159": "Cogency Semiconductor, Inc.",
    "12.71297884803072": "Zoe Medical",
    "24.7865015": "ShenZhen Ultra Easy Technology CO.,LTD",
    "24.10490512": "Avaya Inc",
    "12.346867789824": "syes srl",
    "20.242876968599552": "SAN Engineering",
    "12.123917680758784": "WICOM1 GmbH",
    "24.6874": "Biz-2-Me Inc.",
    "24.6875": "ARRIS Group, Inc.",
    "24.6876": "Nokia Danmark A/S",
    "24.6877": "PePWave Ltd",
    "24.6870": "Jiagnsu Aetna Electric Co.,Ltd",
    "24.6871": "Christie Digital Systems, Inc.",
    "24.6872": "AlsterAero GmbH",
    "24.6873": "International Broadband Electric Communications, Inc.",
    "24.6878": "ARRIS Group, Inc.",
    "24.6879": "Interactivetv Pty Limited",
    "24.11801737": "Cisco Systems, Inc",
    "24.3196175": "Huawei Technologies Co.,Ltd",
    "12.71297884114944": "Triton Electronics LTD",
    "24.7914259": "China Mobile Group Device Co.,Ltd.",
    "12.123917681938432": "JPEmbedded Mazan Filipek Sp. J.",
    "20.58144488161280": "Biamp",
    "12.346864930816": "Digitale Analoge COMponenten West Electronic Vertriebs GmbH",
    "20.75247163277312": "Itc",
    "24.6831997": "Ingram Micro Services",
    "24.13120654": "Intel Corporate",
    "24.107": "Silicon Graphics",
    "24.106": "Computer Consoles Inc.",
    "24.105": "Concord Communications Inc",
    "24.104": "Rosemount Controls",
    "24.103": "Soft * Rite, Inc.",
    "24.102": "Talaris Systems, Inc.",
    "24.101": "Network General Corporation",
    "24.100": "Yokogawa Digital Computer Corporation",
    "24.6053547": "Juniper Networks",
    "24.13665236": "Xtel Wireless ApS",
    "24.109": "Cray Communications, Ltd.",
    "24.108": "Private",
    "12.123917686702080": "HUSTY M.Styczen J.Hupert sp.j.",
    "24.13902000": "Mist Systems, Inc.",
    "24.10277451": "zte corporation",
    "24.27836": "Cisco Systems, Inc",
    "20.228844757843968": "Shenzhen SIC Technology. Co., Ltd.",
    "20.48476175990784": "Sunit Oy",
    "20.84187701837824": "Inster Tecnología y Comunicaciones SAU",
    "24.56176": "Apple, Inc.",
    "24.3735038": "IEEE Registration Authority",
    "24.7885668": "SHIFT GmbH",
    "12.123917689692160": "Yehl & Jordan Llc",
    "24.1626225": "Dasan Co., Ltd.",
    "24.3958668": "Huawei Technologies Co.,Ltd",
    "24.8955590": "Sagemcom Broadband SAS",
    "20.53091442360320": "Shanghai gatang technology CO.,LTD",
    "24.6354332": "CRU-Dataport",
    "12.123917687279616": "Grossenbacher Systeme AG",
    "24.2905468": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.6331044": "Gvi Security Inc.",
    "24.8170831": "Cisco Systems, Inc",
    "24.12100486": "D-Link International",
    "20.96896781647872": "Tring",
    "20.198151833124864": "Axiomatic Technologies Corporation",
    "20.180577515864064": "ALSTOM Strongwish (Shenzhen) Co., Ltd",
    "24.8684806": "Apple, Inc.",
    "12.123917686095872": "ETA Technology Pvt Ltd",
    "24.11277812": "Apple, Inc.",
    "20.119709256122368": "CELL System Co.,Ltd.",
    "24.3962947": "Huawei Technologies Co.,Ltd",
    "12.346866102272": "Indra Australia",
    "20.96896792133632": "UPM Technology, Inc",
    "24.12384754": "Teko",
    "24.14460106": "Apple, Inc.",
    "24.3961911": "Rim",
    "24.9225697": "ShenZhen Konka Telecommunication Technology Co.,Ltd",
    "24.12398": "Hewlett Packard",
    "24.5002393": "Huawei Technologies Co.,Ltd",
    "24.12376014": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "12.123917687345152": "U&R GmbH Hardware- und Systemdesign",
    "24.815": "Global Sun Technology, Inc.",
    "24.814": "Scope Information Management, Ltd.",
    "24.817": "Cisco Systems, Inc",
    "24.816": "Imagenics, Co., Ltd.",
    "24.811": "GAI Datenfunksysteme GmbH",
    "24.810": "UniData Communication Systems, Inc.",
    "24.813": "IBASE Technology, Inc.",
    "24.812": "ABB Switzerland Ltd",
    "24.7401846": "Data Modul AG",
    "24.819": "Digitel Co., Ltd.",
    "24.818": "Cisco Systems, Inc",
    "12.346856599552": "A3Ip",
    "24.1585599": "ASUSTek COMPUTER INC.",
    "20.48476179136512": "Sct Optronics Co., Ltd",
    "12.123917688578048": "tinnos",
    "24.3938937": "Huayuan Technology CO.,LTD",
    "24.2887873": "Nintendo Co., Ltd.",
    "20.167860147060736": "Shenzhen Hubsan Technology Co.，LTD.",
    "24.320920": "Xiaomi Communications Co Ltd",
    "24.12397": "Lucent Technologies",
    "12.346856128512": "Wheels of Zeus Inc.",
    "12.123917683781632": "YUYAMA MFG Co.,Ltd",
    "12.123917687922688": "sensorway",
    "12.346854604800": "Thales Communications Ltd",
    "12.346853130240": "Keith & Koep GmbH",
    "24.14194491": "Artesyn Embedded Technologies",
    "24.1599313": "KOMEG Industrielle Messtechnik GmbH",
    "20.88668808151040": "Axel Technology",
    "24.16021856": "Huawei Technologies Co.,Ltd",
    "12.346857132032": "Argon ST",
    "24.24583": "Acres Gaming, Inc.",
    "24.24582": "Sotec Co., Ltd",
    "24.24581": "Feedback Data Ltd.",
    "24.24580": "Computadores Modulares Sa",
    "24.24587": "LOGWARE GmbH",
    "24.24586": "Sord Computer Corporation",
    "24.24585": "Cisco",
    "24.24584": "3Com",
    "12.123917691256832": "Guan Show Technologe Co., Ltd.",
    "20.264625146822656": "Fiziico Co., Ltd.",
    "24.24589": "Digital Logic GmbH",
    "24.24588": "Eurotech Inc.",
    "24.5278274": "Uptmate Technology Co., LTD",
    "20.48548666146816": "Changzhou WEBO Weighing Device & System CO.,LTD",
    "24.15501455": "Tp-Link Technologies Co.,Ltd.",
    "12.71297884459008": "WOW System",
    "24.15004455": "Atol Llc",
    "24.3990503": "China Mobile Group Device Co.,Ltd.",
    "24.3467828": "HangZhou Gubei Electronics Technology Co.,Ltd",
    "12.123917688537088": "Grossenbacher Systeme AG",
    "24.3191318": "Hitachi ABB Power Grids – Grid Automation",
    "12.123917682679808": "Guan Show Technologe Co., Ltd.",
    "24.1876152": "Hon Hai Precision Ind. Co., Ltd.",
    "24.5899194": "Wireless Broadband Alliance Ltd",
    "24.5299718": "Apple, Inc.",
    "24.5304531": "Samsung Electronics Co.,Ltd",
    "24.7087545": "Bbk Educational Electronics Corp.,Ltd.",
    "24.7398": "Innes",
    "24.7399": "Rocon PLC Research Centre",
    "24.7396": "EleSy JSC",
    "24.7397": "MBS Electronic Systems GmbH",
    "24.7394": "Attero Tech, LLC.",
    "24.7395": "Optimedical Systems",
    "24.7392": "Dasan Tps",
    "24.7393": "Indra Sistemas, S.A.",
    "24.7390": "Interactive Multimedia eXchange Inc.",
    "24.7391": "Belkin International Inc.",
    "12.123917691133952": "FRS GmbH & Co. KG",
    "24.4194682": "Cisco Systems, Inc",
    "12.123917681893376": "Hosin Industrial Limited",
    "24.2348": "Hitpoint Inc.",
    "24.2349": "HTC Corporation",
    "24.2346": "MYTECS Co.,Ltd.",
    "24.2347": "iQstor Networks, Inc.",
    "24.2344": "Telecore",
    "24.2345": "Sanyo Industries (UK) Limited",
    "24.2342": "Yoda Communications, Inc.",
    "24.2343": "Toyokeiki Co.,Ltd.",
    "24.2340": "Telebau GmbH",
    "24.2341": "VSN Systemen BV",
    "12.346854354944": "Linn Products Ltd",
    "24.8444435": "eero inc.",
    "24.8150264": "Intel Corporate",
    "24.7903631": "Mediline Italia Srl",
    "24.8406918": "Samsung Electronics Co.,Ltd",
    "20.132869115609088": "NimbeLink Corp",
    "24.5545902": "iBASE Gaming Inc",
    "20.57176920948736": "ZQAM Communications",
    "24.7648": "Intel Corporate",
    "24.7649": "Intel Corporate",
    "24.53499": "Tek Microsystems, Incorporated",
    "24.53498": "Thales e-Security Ltd.",
    "12.123917683716096": "Mettler Toledo",
    "24.7640": "Microsoft Corporation",
    "24.7641": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7642": "Mikroelektronika spol. s r. o.",
    "24.7643": "C-BEL Corporation",
    "24.7644": "HangZhou DeChangLong Tech&Info Co.,Ltd",
    "24.7645": "Dat H.K. Limited",
    "24.7646": "Zhejiang Broadcast&Television Technology Co.,Ltd.",
    "24.7647": "Sunitec Enterprise Co.,Ltd",
    "24.62427": "WOO Sports",
    "12.123917676879872": "inomatic GmbH",
    "24.3676236": "Sony Home Entertainment&Sound Products Inc",
    "20.128510966366208": "aritec gmbh",
    "24.10282727": "Samsung Electronics Co.,Ltd",
    "24.16396009": "OCA Alliance, Inc.",
    "20.225209251528704": "Glenair",
    "24.3431098": "tcloud intelligence",
    "24.2937306": "Sanjole, Inc.",
    "12.346854682624": "Allen Martin Conservation Ltd",
    "12.71297883324416": "Electrical Geodesics Incorporated",
    "24.14952612": "Prama India Private Limited",
    "12.346855047168": "Magellan Technology Pty Limited",
    "12.119269265408": "Solomon Systech Pte Ltd",
    "12.123917683257344": "RDT Ltd",
    "12.123917691334656": "Hella Gutmann Solutions GmbH",
    "24.14214955": "Netscout Systems Inc",
    "20.225207113482240": "SAMIM Co",
    "20.70443728896000": "Guangzhou RALID Information System Co.Ltd",
    "24.16050470": "Tianjin Zanpu Technology Inc.",
    "24.4499708": "Apple, Inc.",
    "12.123917685202944": "ATIM Radiocommunication",
    "12.123917684125696": "Firefly RFID Solutions",
    "24.3463026": "Xiamen Yudian Automation Technology Co., Ltd",
    "20.141268010139648": "Smart Electronics Nz Limited",
    "12.123917679955968": "Shen Zhen Ttk Technology Co,Ltd",
    "24.13114110": "DLOGIC GmbH",
    "24.6117": "Texas Instruments",
    "24.6116": "Texas Instruments",
    "24.6115": "Texas Instruments",
    "24.6114": "ARRIS Group, Inc.",
    "24.6113": "DACOS Technologies Co., Ltd.",
    "24.6112": "Cisco Systems, Inc",
    "24.6111": "Cisco Systems, Inc",
    "24.6110": "Advantage Six Ltd",
    "12.346856730624": "Matra Electronique",
    "24.6119": "Texas Instruments",
    "24.6118": "Texas Instruments",
    "12.123917684310016": "Soudax Equipements",
    "24.2101847": "Most Lucky Trading Ltd",
    "24.174652": "Olivetti Telecomm Spa (Olteco)",
    "24.9199874": "Beijing Baofengmojing Technologies Co., Ltd",
    "24.6605858": "Arcadyan Corporation",
    "12.346863853568": "Elko Systems",
    "24.6954860": "iPass, Inc.",
    "24.6577542": "Intel Corporate",
    "12.346868654080": "Elbit Systems of America",
    "24.15768504": "Huawei Technologies Co.,Ltd",
    "20.127658786553856": "Saiercom Corporation",
    "24.2392519": "Huawei Device Co., Ltd.",
    "24.4248407": "Ison Technology Co., Ltd.",
    "24.24808": "Hitachi Computer Products (America), Inc.",
    "24.24809": "Atop Technologies, Inc.",
    "24.24806": "Shomiti Systems Incorporated",
    "24.24807": "Randata",
    "24.24804": "Compuserve, Inc.",
    "24.24805": "Fuji Automation Co., Ltd.",
    "24.24802": "Quest Engineering & Development",
    "24.24803": "Arbin Instruments",
    "24.24800": "Axiom Technology Co., Ltd.",
    "24.24801": "Orckit Communications Ltd.",
    "24.3935357": "Tiny Mesh AS",
    "24.3938581": "GFI Chrono Time",
    "24.5307161": "Chipsea Technologies (Shenzhen) Corp.",
    "24.8924052": "MADOKA SYSTEM Co.,Ltd.",
    "24.5522967": "zte corporation",
    "12.346866925568": "Calixto Systems Pvt Ltd",
    "24.7386521": "Embedded Technologies s.r.o.",
    "24.3152079": "DEOS control systems GmbH",
    "24.8190222": "Samsung Electronics Co.,Ltd",
    "20.75685688246272": "Auctus Technologies Co.,Ltd.",
    "12.346852962304": "Tonbu, Inc.",
    "12.123917691342848": "Get SAT",
    "12.123917690880000": "Daavlin",
    "24.16293416": "Cisco Meraki",
    "24.12624493": "Abbott Point of Care",
    "24.7372355": "ARRIS Group, Inc.",
    "24.8439282": "AzureWave Technology Inc.",
    "24.13401831": "Panasonic Corporation AVC Networks Company",
    "24.2911877": "Intel Corporate",
    "20.229558883188736": "Chunghsin International Electronics Co.,Ltd.",
    "24.12855702": "Signify B.V.",
    "24.24675": "Psion Dacom Plc.",
    "12.346858541056": "Axis Electronics",
    "20.97843041796096": "Excenon Mobile Technology Co., Ltd.",
    "24.515": "Woonsang Telecom, Inc.",
    "24.24677": "B&R Industrial Automation GmbH",
    "24.1613564": "Apple, Inc.",
    "24.24676": "Netcomm Limited",
    "12.346858635264": "AirMatrix, Inc.",
    "24.8441529": "Samsung Electronics Co.,Ltd",
    "24.511": "Data Direct Networks, Inc.",
    "12.346860527616": "AutomationX GmbH",
    "24.11323701": "Ness Corporation",
    "24.512": "Net & Sys Co., Ltd.",
    "12.346854559744": "Bender GmbH & Co. KG",
    "24.3172683": "Rim",
    "24.513": "IFM Electronic gmbh",
    "12.123917680574464": "Merten GmbH&CoKG",
    "12.346855825408": "Ni.Co. S.r.l.",
    "24.1886947": "IEEE Registration Authority",
    "24.3417606": "CarePredict, Inc.",
    "12.346867511296": "Subsea Systems, Inc.",
    "24.518": "Telital R&D Denmark A/S",
    "24.11336062": "Elesys Inc.",
    "24.4725714": "Intron Scientific co., ltd.",
    "24.519": "VisionGlobal Network Corp.",
    "20.224420106862592": "Guangzhou Southelectric Power Science Technology Development Co.,Ltd.",
    "24.273122": "Cisco Systems, Inc",
    "24.1466": "Area Netwoeks, Inc.",
    "12.346862780416": "Quorum Technologies Ltd",
    "24.1467": "Myspace AB",
    "24.569511": "Truebeyond Co., Ltd",
    "24.12873899": "Ruijie Networks Co.,LTD",
    "24.14703964": "Oy Everon Ab",
    "24.11557594": "Hewlett Packard",
    "24.11597104": "Hitron Technologies. Inc",
    "12.123917685092352": "Confed Holding B.V.",
    "24.302977": "Netscout Systems Inc",
    "24.11291222": "Cisco Systems, Inc",
    "12.123917680173056": "NOA Co., Ltd.",
    "24.11846908": "Xiaomi Communications Co Ltd",
    "12.346858676224": "dSPACE GmbH",
    "24.16436": "Bustek Corporation",
    "24.16437": "Opcom",
    "24.16434": "Digital Communications",
    "24.16435": "Addtron Technology Co., Ltd.",
    "24.16432": "Gk Computer",
    "24.16433": "Kokusai Electric Co., Ltd",
    "24.16430": "Precision Software, Inc.",
    "24.16431": "Xlnt Designs Inc (XDI)",
    "20.22333149413376": "Shenzhen V-Streaming Technology Co., Ltd.",
    "24.16438": "Zoom Telephonics, Inc",
    "24.16439": "Sea-Ilan, Inc.",
    "24.12093898": "ILJIN ELECTRIC Co., Ltd.",
    "12.346865577984": "CaptiveAire Systems Inc.",
    "20.35758068989952": "Hemina Spa",
    "12.346863374336": "Bitmanufaktur GmbH",
    "20.206931530612736": "Cameron",
    "24.2667592": "Huawei Technologies Co.,Ltd",
    "24.5553911": "Pegatron Corporation",
    "24.9655": "Costar electronics, inc.,",
    "24.13938109": "Cisco Systems, Inc",
    "24.1468": "Resource Data Management Ltd",
    "24.15748514": "Dell Inc.",
    "24.538729": "Hong Kong AMobile Intelligent Corp. Limited Taiwan Branch",
    "24.5036431": "Dell Inc.",
    "24.1469": "Roax Bv",
    "24.10278027": "Innolux Technology Europe BV",
    "24.2910831": "IEEE Registration Authority",
    "24.13137803": "Mercury Systems – Trusted Mission Solutions, Inc.",
    "24.15523389": "Lamprey Networks, Inc.",
    "12.346865909760": "Recon Dynamics, LLC",
    "24.4227040": "Glory Star Technics (ShenZhen) Limited",
    "24.15214785": "Eltex Enterprise Ltd.",
    "24.6557480": "2Wire Inc",
    "24.10023083": "Apple, Inc.",
    "24.10501739": "IEEE Registration Authority",
    "12.123917676249088": "MG s.r.l.",
    "20.66129348263936": "CASKY eTech Co., Ltd.",
    "24.11324169": "Hefcom Metering (Pty) Ltd",
    "12.346854170624": "Dave Srl",
    "12.123917681831936": "Decyben",
    "24.9990376": "Innatech Sdn Bhd",
    "24.16304729": "NetComm Wireless",
    "24.9446324": "New H3C Technologies Co., Ltd",
    "12.123917683933184": "LG Electronics",
    "24.12910171": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5793663": "Clare Controls Inc.",
    "24.3941616": "IEEE Registration Authority",
    "24.5814996": "Huawei Technologies Co.,Ltd",
    "12.346863529984": "Air Monitors Ltd",
    "12.123917683556352": "Xacti Corporation",
    "20.128510972657664": "Diamond Kinetics",
    "12.346857500672": "Solid State Logic",
    "24.7098445": "Cisco Systems, Inc",
    "12.123917684822016": "Giordano Controls Spa",
    "24.13115866": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.1573550": "vivo Mobile Communication Co., Ltd.",
    "24.3430240": "MitraStar Technology Corp.",
    "24.6820501": "Lupine Lighting Systems GmbH",
    "24.8185002": "Private",
    "12.346868183040": "cadac,inc.",
    "24.12409": "Cqos, Inc.",
    "24.12408": "Cisco Systems, Inc",
    "24.12407": "Onprem Networks",
    "24.12406": "Akamba Corporation",
    "24.12405": "Adtech",
    "24.12404": "Equiinet Ltd.",
    "24.12403": "International Microsystems, In",
    "24.12402": "Intellibyte Inc.",
    "24.12401": "Cisco Systems, Inc",
    "24.12400": "1Net Corporation",
    "24.2883851": "NASCENT Technology, LLC - RemKon",
    "20.198151831027712": "eInfochips Limited",
    "24.43848": "eero inc.",
    "12.123917677801472": "NASA Johnson Space Center",
    "24.11318653": "Liteon Technology Corporation",
    "24.2623245": "Guangzhou Winsound Information Technology Co.,Ltd.",
    "24.15782641": "Amazon Technologies Inc.",
    "20.229206568992768": "Happo Solutions Oy",
    "24.7900023": "Standar Telecom",
    "24.13634219": "Yokogawa Digital Computer Corporation",
    "24.15229315": "Black Diamond Advanced Technology, LLC",
    "24.5025299": "Huawei Technologies Co.,Ltd",
    "24.9748269": "Apple, Inc.",
    "24.16310037": "Sunitec Enterprise Co.,Ltd",
    "12.346854326272": "Innovics Wireless Inc",
    "12.71297883414528": "Socus networks",
    "24.1634962": "Shannon Systems",
    "20.233258808770560": "BlueCats US, LLC",
    "12.123917690945536": "MacGray Services",
    "24.9238288": "AMPAK Technology, Inc.",
    "12.123917680275456": "Human Oriented Technology, Inc.",
    "24.13163977": "Lenovo Mobile Communication Technology Ltd.",
    "20.97400607735808": "Peak Communications Limited",
    "24.2147215": "Texas Instruments",
    "20.62335514312704": "Avo Development Ltd",
    "24.4983220": "zte corporation",
    "12.346865602560": "ACD Elektronik GmbH",
    "24.6609568": "Robert Bosch Smart Home GmbH",
    "20.220515717349376": "Aisworld Private Limited",
    "24.2939655": "Intel Corporate",
    "12.346866765824": "Lee Laser",
    "12.123917680566272": "RFL Electronics, Inc.",
    "24.15760331": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346852663296": "RadioTel",
    "24.10521755": "Xtreme Technologies Corp",
    "24.10234230": "Intel Corporate",
    "24.1734139": "Autosar",
    "24.16986": "Cisco Systems, Inc",
    "12.346855415808": "Zodiak Data Systems",
    "24.10519212": "Linear DMS Solutions Sdn. Bhd.",
    "24.14462468": "CELOXICA Ltd",
    "12.346862424064": "LUMINEX Lighting Control Equipment",
    "24.10128": "Cisco Systems, Inc",
    "24.13171193": "Cisco Systems, Inc",
    "12.346863108096": "Reliable System Services Corp",
    "24.5775670": "KMB systems, s.r.o.",
    "24.7349542": "Nokia",
    "24.7663146": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.12642239": "Sichuan AI-Link Technology Co., Ltd.",
    "12.123917688381440": "LSC Lighting Systems (Aust) Pty Ltd",
    "12.123917684109312": "Qbic Communications Dmcc",
    "12.71297884815360": "Spider Tecnologia Ind. e Com Ltda",
    "24.8687116": "Apple, Inc.",
    "20.5239425990656": "SiS Technology",
    "12.123917676703744": "Koizumi Lighting Technology Corp.",
    "12.346862354432": "Sound Player Systems e.K.",
    "24.4726": "CG Power Systems Ireland Limited",
    "24.4727": "Korenix Technologies Co., Ltd.",
    "24.4724": "NIT lab",
    "24.4725": "Sentilla Corporation",
    "24.4722": "Redux Communications Ltd.",
    "24.4723": "Stoke Inc",
    "24.3443374": "Sagemcom Broadband SAS",
    "24.4721": "Measurement Computing Corp",
    "24.4728": "International Bar Code",
    "24.4729": "Hewlett Packard",
    "24.8929235": "Huawei Technologies Co.,Ltd",
    "24.16028955": "Hefei Radio Communication Technology Co., Ltd",
    "24.6324523": "Astronics Custom Control Concepts",
    "24.16546059": "Hitachi High Technologies America, Inc.",
    "24.20666": "D-Link Corporation",
    "12.123917689880576": "Quiss Ag",
    "24.20667": "Cms Technologies",
    "24.20664": "Inova Computers Gmbh & Co. Kg",
    "24.20665": "Xitron Technologies, Inc.",
    "24.20662": "Good Way Ind. Co., Ltd.",
    "12.346864254976": "Gigatronik Köln GmbH",
    "24.7121192": "HMD Global Oy",
    "24.1110801": "Cisco Systems, Inc",
    "24.20663": "Boser Technology Co., Ltd.",
    "24.5529609": "Google, Inc.",
    "24.20660": "Satchwell Control Systems, Ltd",
    "24.20661": "Fichet Securite Electronique",
    "12.346865926144": "7+ Kft",
    "24.11801982": "Celona Inc.",
    "24.13903697": "Infopia Co., Ltd",
    "12.123917675053056": "EXARA Group",
    "24.3189": "Oriental integrated electronics. LTD",
    "24.3188": "Rivertec Corporation",
    "24.3187": "Telson Electronics Co., Ltd",
    "24.3186": "Tempearl Industrial Co., Ltd.",
    "24.3185": "Wybron, Inc",
    "24.3184": "ACC GmbH",
    "24.3183": "Amtek system co.,LTD.",
    "24.3182": "ASUSTek COMPUTER INC.",
    "24.3181": "Edwards Ltd.",
    "24.3180": "Eve Systems GmbH",
    "24.4738812": "ARRIS Group, Inc.",
    "24.8693640": "Perples",
    "24.12869700": "Everex Electronics Limited",
    "24.7648647": "Apple, Inc.",
    "24.7915195": "Innovasic, Inc.",
    "24.6616398": "EM Microelectronic",
    "20.132777851748352": "Wan Chao An (Beijing) Technology Co., Ltd.",
    "12.346868879360": "DiTEST Fahrzeugdiagnose GmbH",
    "24.11577975": "Cisco Systems, Inc",
    "24.14182752": "Steinsvik",
    "20.268735396970496": "ORtek Technology, Inc.",
    "20.176189945675776": "Videx Electronics S.p.A.",
    "24.3718963": "NEC CASIO Mobile Communications",
    "24.5541024": "Liteon Technology Corporation",
    "24.4780415": "Intel Corporate",
    "12.123917679665152": "Quadio Devices Private Limited",
    "20.119709257170944": "Toucan Systems Ltd",
    "12.123917688049664": "The Engineerix Group",
    "24.9464182": "Withrobot Inc.",
    "24.8431709": "Osterhout Design Group",
    "12.346854539264": "intec GmbH",
    "12.346854633472": "Winford Engineering",
    "24.767": "Handan BroadInfoCom",
    "24.766": "Viditec, Inc.",
    "24.765": "Cisco Systems, Inc",
    "24.764": "Cisco Systems, Inc",
    "24.763": "Baumuller Aulugen-Systemtechnik GmbH",
    "24.762": "DX Antenna Co., Ltd.",
    "24.761": "MIMOS Berhad",
    "24.760": "SEAKR Engineering, Inc.",
    "12.346854068224": "Benchmark Electronics",
    "24.1319006": "Apple, Inc.",
    "24.769": "Exfo",
    "24.768": "Barracuda Networks, Inc.",
    "24.10285097": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.346868908032": "KST technology",
    "24.14727860": "Arrcus, Inc",
    "24.16291010": "Intel Corporate",
    "24.9450539": "Gatekeeper Systems, Inc.",
    "12.346854060032": "Lifetime Memory Products, Inc.",
    "24.4737479": "Samsung Electronics Co.,Ltd",
    "24.7406": "SHARP Corporation",
    "24.7407": "Primax Electronics Ltd.",
    "24.7404": "Mobilesoft (Aust.) Pty Ltd",
    "24.7405": "Environnement Sa",
    "24.7402": "Scientific-Atlanta, Inc",
    "24.7403": "Nortel Networks",
    "12.119269888000": "Hangzhou Zhiping Technology Co., Ltd.",
    "24.7401": "Galaxy Technology Limited",
    "24.7408": "D-Link Corporation",
    "24.7409": "SUPoX Technology Co. , LTD.",
    "24.1888519": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.8137435": "Xiaomi Communications Co Ltd",
    "24.491": "C-COM Corporation",
    "24.490": "Cirilium Corp.",
    "24.493": "SETA Corp.",
    "24.492": "Ericsson Group",
    "24.495": "Camtel Technology Corp.",
    "24.494": "Comtrol Europe, Ltd.",
    "24.497": "Innovative Concepts, Inc.",
    "24.496": "Tridium, Inc.",
    "24.499": "QPS, Inc.",
    "24.498": "Mark of the Unicorn, Inc.",
    "20.246679827644416": "Fujian Newland Auto-ID Tech. Co.,Ltd.",
    "24.5800": "Cwt Co., Ltd.",
    "24.5801": "2Ei",
    "24.5802": "Kei Communication Technology Inc.",
    "24.5803": "Dansensor A/S",
    "24.5804": "Toho Technology Corp.",
    "24.5805": "BT-Links Company Limited",
    "24.5806": "Inventel",
    "24.5807": "Shenzhen Union Networks Equipment Co.,Ltd.",
    "24.5808": "VK Corporation",
    "24.5809": "Kbs",
    "24.5536117": "Nokia Corporation",
    "20.137141739847680": "Briowireless Inc.",
    "24.1623434": "Huawei Technologies Co.,Ltd",
    "12.123917690208256": "Changzhou Rapid Information Technology Co,Ltd",
    "12.346855510016": "Broadcast Sports Inc",
    "24.5802995": "Zyxel Communications Corporation",
    "12.123917676666880": "YUYAMA MFG Co.,Ltd",
    "20.75247156985856": "Shenzhen Furuilian Electronic Co.,Ltd.",
    "12.346853961728": "MosChip USA",
    "20.84197042552832": "Cantronic Systems (Canada) Inc",
    "24.5547291": "Huawei Technologies Co.,Ltd",
    "12.123917678764032": "CTG sp. z o. o.",
    "24.10005207": "Blue Wave Networking Co Ltd",
    "12.123917689098240": "RZB Rudolf Zimmermann, Bamberg GmbH",
    "12.123917684236288": "CORES Corporation",
    "12.123917689503744": "ResIOT UBLSOFTWARE SRL",
    "20.154039757045760": "Noritsu Precision Co., Ltd.",
    "24.2422391": "Apple, Inc.",
    "24.15500468": "Cig Shanghai Co Ltd",
    "12.123917688274944": "Grupo Epelsa S.L.",
    "20.162752334135296": "Dongguan Kingtron Electronics Tech Co., Ltd",
    "24.2636": "Molecular Devices Corporation",
    "24.2637": "Noritz Corporation",
    "24.2634": "Targa Systems Ltd.",
    "24.2635": "DataPower Technology, Inc.",
    "24.2632": "Albatron Technology",
    "24.2633": "F5 Networks, Inc.",
    "24.2630": "Aro Welding Technologies Sas",
    "24.2631": "Allied Vision Technologies",
    "24.7897144": "Funai Electric Co., Ltd.",
    "24.7360102": "Shenzhen Fast Technologies Co.,Ltd",
    "24.2638": "UNITEK Electronics INC.",
    "24.2639": "Brain Boxes Limited",
    "24.8710290": "Actiontec Electronics, Inc",
    "24.12340731": "Hitron Technologies. Inc",
    "24.8403": "OST (Ouet Standard Telematique)",
    "24.8402": "Rad Data Communications, Ltd.",
    "24.8401": "Microcomputer Systems (M) Sdn.",
    "24.8400": "Versalynx Corp.",
    "24.8407": "JAPAN MINICOMPUTER SYSTEMS CO., Ltd.",
    "24.8406": "Breezecom, Ltd.",
    "24.8405": "Vipa Gmbh",
    "24.8404": "Cabletron Systems, Inc.",
    "12.123917688283136": "Bel Power Solutions GmbH",
    "24.8409": "Panasonic Technologies, Inc./Mieco-Us",
    "24.8408": "Nortel Networks",
    "24.1615607": "Tp-Link Technologies Co.,Ltd.",
    "24.13371643": "Tp-Link Technologies Co.,Ltd.",
    "24.11839528": "Shenzhen Concox Information Technology Co., Ltd",
    "20.75685693489152": "Joint-Stock Company Research and Development Center \"ELVEES\"",
    "24.5118": "Grandtec Electronic Corp.",
    "24.5119": "Dage-MTI of MC, Inc.",
    "24.5116": "SiCortex, Inc",
    "24.5117": "Nokia Danmark A/S",
    "24.5114": "LifeSize Communications, Inc",
    "24.5115": "Rkc Instrument Inc.",
    "24.5112": "Dex Security Solutions",
    "24.5113": "Cavera Systems",
    "24.5110": "Cintech",
    "24.5111": "SMC Networks, Inc.",
    "24.3192258": "Tp-Link Technologies Co.,Ltd.",
    "12.346863054848": "EMS Industries",
    "24.8970326": "Huawei Technologies Co.,Ltd",
    "24.1056628": "Hewlett Packard",
    "24.2392064": "Westcontrol AS",
    "24.13107878": "Beijing Newmine Technology",
    "24.5297399": "Tp-Link Technologies Co.,Ltd.",
    "12.123917688430592": "Proconex 2010 Inc.",
    "12.346852487168": "Luma Corporation",
    "24.11066115": "Samsung Electro-Mechanics(Thailand)",
    "12.123917686816768": "ENTEC Electric & Electronic Co., LTD.",
    "24.6078935": "Samsung Electronics Co.,Ltd",
    "24.13944678": "Acentic GmbH",
    "20.124214334980096": "Beijing Eehuu Technology Co.,Ltd.",
    "24.3691420": "Apple, Inc.",
    "24.8706": "Excito Elektronik i Skåne AB",
    "24.15266224": "Sagemcom Broadband SAS",
    "24.15497714": "Startel",
    "24.6306428": "Tp-Link Technologies Co.,Ltd.",
    "24.3464092": "Sagemcom Broadband SAS",
    "24.7107853": "WiFiSONG",
    "24.11048586": "Apple, Inc.",
    "24.532337": "Tp-Link Technologies Co.,Ltd.",
    "20.220958184964096": "Qbic Technology Co., Ltd",
    "24.16265128": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346852388864": "E.I.S.M.",
    "12.346852360192": "Lockheed Martin Tactical Defense Systems Eagan",
    "24.2401429": "Thales Canada Inc.",
    "12.123917680513024": "Nanjing Pingguang Electronic Technology Co., Ltd",
    "12.346863669248": "Ingenieurbuero Bickele und Buehler GmbH",
    "24.16579656": "Apple, Inc.",
    "24.1581222": "Sage Co., Ltd.",
    "24.9702189": "NetView Technologies(Shenzhen) Co., Ltd",
    "12.346860195840": "Delta Tau Data Systems, Inc.",
    "24.13921806": "Tp-Link Technologies Co.,Ltd.",
    "12.346854031360": "Euphony technology CO., LTD.",
    "24.13636938": "zte corporation",
    "12.123917677142016": "Golden Grid Systems",
    "24.2906506": "LG Electronics (Mobile Communications)",
    "24.13393272": "JTD Consulting",
    "24.1892178": "Texas Instruments",
    "24.297697": "Flextronics Manufacturing(Zhuhai)Co.,Ltd.",
    "24.5303066": "Brocade Communications Systems LLC",
    "12.123917687906304": "ARKRAY, Inc. Kyoto Laboratory",
    "20.154039769628672": "You Zhengcheng co.,ltd",
    "24.3724299": "Samsung Electronics Co.,Ltd",
    "24.6852484": "Huawei Technologies Co.,Ltd",
    "24.5538221": "Eagle Research Corporation",
    "24.13148878": "Oasis Media Systems LLC",
    "24.11849462": "HTC Corporation",
    "24.5559407": "Cisco SPVTG",
    "24.3430464": "Huawei Technologies Co.,Ltd",
    "24.7623719": "Shenzhen Fast Technologies Co.,Ltd",
    "24.2632070": "APC by Schneider Electric",
    "12.123917677330432": "Arete Associates",
    "24.11288866": "Eclipse Electronic Systems Inc.",
    "12.123917689929728": "Densitron Technologies Ltd",
    "24.10787201": "FuJian Elite Power Tech CO.,LTD.",
    "12.346861760512": "ATIS group s.r.o.",
    "24.14157102": "Skyviia Corporation",
    "24.14460605": "Beijing Lanbo Technology Co., Ltd.",
    "24.16280537": "Cisco Systems, Inc",
    "12.123917677359104": "Eta-Usa",
    "12.346858254336": "Innomed Medical Inc",
    "12.123917687836672": "CISTECH Solutions",
    "24.13905699": "Akenori PTE Ltd",
    "24.12065177": "Nesys",
    "12.346863218688": "Navicron Oy",
    "24.9229951": "EM Microelectronic",
    "20.141268008042496": "Chendu Ningshui Technology Co.,Ltd",
    "24.11306182": "Tp-Link Technologies Co.,Ltd.",
    "24.1091890": "Beijing Cyber Cloud Technology Co. ,Ltd.",
    "24.15481946": "Shen Zhen Heng Sheng Hui Digital Technology Co.,Ltd",
    "24.8703144": "Espressif Inc.",
    "24.530229": "Cisco Systems, Inc",
    "12.346867503104": "Tex Computer Srl",
    "24.16271999": "Innometriks Inc",
    "24.6576877": "Apple, Inc.",
    "24.3700834": "Sony Mobile Communications Inc",
    "12.346864418816": "Gainbrain",
    "20.181256785494016": "Original Products Pvt. Ltd.",
    "24.4761884": "Apple, Inc.",
    "24.14480645": "AzureWave Technology Inc.",
    "24.13653836": "Cisco Systems, Inc",
    "24.3183350": "Shanghai Sunmon Communication Technogy Co.,Ltd",
    "24.4719045": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.15240693": "ZNYX Networks, Inc.",
    "24.11334111": "D-Link International",
    "24.14974566": "Huawei Technologies Co.,Ltd",
    "12.123917684080640": "megatec electronic GmbH",
    "24.3709686": "Samsung Electronics Co.,Ltd",
    "12.119269408768": "Care Everywhere LLC",
    "12.123917685141504": "Groupe Paris-Turf",
    "12.346866085888": "Rohde&Schwarz Topex SA",
    "24.12097712": "Atlona Inc.",
    "24.9211787": "China Mobile Chongqing branch",
    "24.12906625": "Huawei Technologies Co.,Ltd",
    "12.123917680271360": "Tonbo Imaging Pte Ltd",
    "20.180659911917568": "Neotech Systems Pvt. Ltd.",
    "24.8942488": "K2E Tekpoint",
    "12.123917691346944": "Ecologicsense",
    "24.6343040": "Trymus",
    "24.4940": "YDT Technology International",
    "24.3984917": "Mercedes-Benz USA, LLC",
    "20.220327864958976": "Tymphany Acoustic Technology (Huizhou) Co., Ltd.",
    "20.277783364239360": "Shenzhen ALFEYE Technology CO.,Ltd",
    "24.14742986": "Cheng Uei Precision Industry Co.,Ltd.",
    "12.71297884086272": "Vpg",
    "24.14742982": "Apple, Inc.",
    "24.4783286": "Lava International(H.K) Limited",
    "24.11534974": "Muller Services",
    "24.4783288": "Woodstream Corporation",
    "12.346858352640": "Eads Test & Services",
    "24.11306185": "Sagemcom Broadband SAS",
    "20.35758070038528": "Networking Services Corp",
    "24.15514909": "Cisco Systems, Inc",
    "24.12333663": "Zhejiang Dahua Technology Co., Ltd.",
    "24.5515038": "AzureWave Technology Inc.",
    "12.346858672128": "Shockfish SA",
    "24.3976809": "ESA Elektroschaltanlagen Grimma GmbH",
    "12.71297884323840": "University of Nebraska -- Lincoln",
    "24.7404684": "OneAccess SA",
    "12.346861268992": "Xelerated",
    "24.15745863": "Huawei Technologies Co.,Ltd",
    "24.16520345": "Aimore Acoustics Incorporation",
    "24.7891191": "YSTen Technology Co.,Ltd",
    "24.3230": "MemoryLink Corp.",
    "12.346862829568": "KWS-Electronic GmbH",
    "20.211525430149120": "Grup Arge Enerji ve Kontrol Sistemleri",
    "24.6860948": "Inesa Electron Co.,Ltd",
    "24.11326308": "Abb Spa, Abb Sace Div.",
    "24.3963588": "Chemtronics",
    "12.346868146176": "Vtron Pty Ltd",
    "24.8188969": "NUUO Inc.",
    "12.123917687304192": "YUYAMA MFG Co.,Ltd",
    "20.14223586361344": "CJSC «Svyaz Engineering»",
    "24.2386280": "Tp-Link Technologies Co.,Ltd.",
    "24.2115565": "BSkyB Ltd",
    "24.5806013": "Cisco Systems, Inc",
    "24.3409054": "IEEE Registration Authority",
    "24.11857620": "Texas Instruments",
    "24.7903464": "D-Link International",
    "20.167594818535424": "iWave Japan, Inc.",
    "20.49277257646080": "XiaMen 35.com Technology Co,.Ltd.",
    "20.172584098332672": "Shanxi ZhuoZhi fei High Electronic Technology Co. Ltd.",
    "24.7874762": "Sonos, Inc.",
    "12.346858135552": "Syntronic AB",
    "12.346857037824": "Buyang Electronics Industrial Co., Ltd.",
    "24.2377416": "Motorola Mobility LLC, a Lenovo Company",
    "24.14192072": "Atil Technology Co., LTD",
    "20.168198553993216": "HighSecLabs",
    "24.7343978": "Cisco Systems, Inc",
    "20.220515724689408": "Ube, Inc. (dba Plum)",
    "24.3684235": "Huawei Technologies Co.,Ltd",
    "24.3676006": "Promzakaz Ltd.",
    "12.123917678333952": "Hanwell Technology Co., Ltd.",
    "12.123917682720768": "UPM Technology, Inc",
    "12.123917676548096": "Altran Uk",
    "20.167860142866432": "Nexus Electrical(Jiaxing) Limited",
    "20.123731218268160": "Chengdu Ophylink Communication Technology Ltd.",
    "12.346859552768": "Exertus",
    "24.2672677": "DellKing Industrial Co., Ltd",
    "20.62070498263040": "L-3 Communications Mobile-Vision, Inc.",
    "24.3672586": "Sky-City Communication and Electronics Limited Company",
    "24.15740839": "Huawei Technologies Co.,Ltd",
    "24.2394": "Racewood Technology",
    "12.123917682503680": "LP Technologies Inc.",
    "24.16300991": "eero inc.",
    "12.123917687967744": "Foshan Shilantian Network S.T. Co., Ltd.",
    "24.11330606": "SK hynix",
    "24.11858664": "ARRIS Group, Inc.",
    "24.8448714": "Dialog Semiconductor Hellas SA",
    "12.346861367296": "Tattile Srl",
    "24.13904050": "ioBridge, Inc.",
    "24.11035298": "Jiwumedia Co., Ltd.",
    "20.48835654057984": "Holjeron",
    "24.11552890": "Tp-Link Technologies Co.,Ltd.",
    "24.7905100": "HOERBIGER Elektronik GmbH",
    "24.14432087": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917690163200": "Nke",
    "24.11552794": "Apple, Inc.",
    "24.11049619": "Sagemcom Broadband SAS",
    "24.11298671": "Electrocompaniet A.S.",
    "24.539320": "Shinoda Plasma Co., Ltd.",
    "24.9756504": "IJ Electron CO.,Ltd.",
    "24.12458": "Axus Microsystems, Inc.",
    "12.346852462592": "Computer Concepts Corp",
    "24.8147337": "SG Wireless Limited",
    "24.11291592": "Juniper Networks",
    "20.247076413767680": "Advatek Lighting Pty Ltd",
    "24.11280235": "Super Micro Computer, Inc.",
    "20.4884643446784": "Omylis Pte Ltd",
    "12.346858479616": "Shenzhen Huazhong Technology Inc",
    "20.145735975698432": "Shenzhen LTIME In-Vehicle Entertainment System Company Limited",
    "24.12090057": "Siemens Ltd.",
    "12.123917686206464": "Exi Flow Measurement Ltd",
    "12.123917688008704": "Collini Dienstleistungs GmbH",
    "12.123917680046080": "Green House Co., Ltd.",
    "12.346857627648": "C2 Diagnostics",
    "24.2116717": "Hugo Brennenstuhl Gmbh & Co. KG.",
    "24.2582": "Lassen Research",
    "12.346860552192": "Delphi Display Systems, Inc.",
    "12.346853412864": "InBus Engineering, Inc.",
    "24.12324261": "Texas Instruments",
    "24.16528792": "Favite Inc.",
    "24.2110003": "Shenzhen Bilian Electronic Co.，Ltd",
    "20.180577517961216": "JSC “Component-ASU”",
    "12.123917682778112": "Swiftnet SOC Ltd",
    "24.12870839": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917685850112": "MCQ TECH GmbH",
    "12.71297884078080": "NVS Technologies Inc",
    "24.15773716": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.4507764": "China Mobile Group Device Co.,Ltd.",
    "12.123917689077760": "GP Systems GmbH",
    "12.346867646464": "Logical Electromechanical Sys Inc.",
    "12.123917678559232": "ADAMCZEWSKI elektronische Messtechnik GmbH",
    "12.123917684817920": "Wilson Electronics",
    "12.71297883983872": "M.M. Elektrolab",
    "24.10760465": "Ziv",
    "20.167860139720704": "Dalian Eastern Display Co., Ltd.",
    "24.9757678": "zte corporation",
    "20.48835646717952": "Beep, Inc.",
    "24.11322170": "Samsung Electronics Co.,Ltd",
    "12.346861801472": "Korea Telecom Internet Solutions (KTIS)",
    "24.11050273": "Cisco Systems, Inc",
    "12.123917688463360": "VideoRay LLC",
    "24.12336843": "vivo Mobile Communication Co., Ltd.",
    "24.12850193": "Apple, Inc.",
    "12.123917683773440": "Athreya Inc",
    "20.264036228792320": "Annapurna labs",
    "24.10511640": "Vnpt Technology",
    "24.14465020": "Alps Electric (Ireland) Ltd",
    "12.346859597824": "MB connect line GmbH Fernwartungssysteme",
    "24.10494391": "Netgear",
    "24.13419105": "IEEE Registration Authority",
    "12.346859347968": "Techno Fittings S.r.l.",
    "24.6076673": "Hewlett Packard",
    "24.3946374": "Netgear",
    "12.346863243264": "Ceron Tech Co.,LTD",
    "12.346853437440": "Uniwell Systems (UK) Ltd.",
    "12.346860351488": "Emitech Corporation",
    "24.1578789": "Cameo Communications, Inc.",
    "12.71297883496448": "Honeywell International",
    "24.3173566": "Skymotion Technology (HK) Limited",
    "20.96896783745024": "Shangyin Intelligence Technology Shandong Co.,Ltd",
    "24.16017730": "Askey Computer Corp",
    "20.22126507589632": "Winsonic Electronics Co., Ltd.",
    "20.206931536904192": "Parlay Labs dba Highfive",
    "24.800254": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.8680544": "zte corporation",
    "12.346868056064": "Vertex Antennentechnik GmbH",
    "24.13137243": "Quantify Technology Pty. Ltd.",
    "24.5024040": "Insensi",
    "24.7917572": "Neousys Technology Inc.",
    "24.1356656": "Gigaset Digital Technology (Shenzhen) Co., Ltd.",
    "20.44768529743872": "GROTHE GmbH",
    "20.31873087569920": "Shenzhen SEWO Technology Co.,Ltd.",
    "24.6342056": "Beijing LT Honway Technology Co.,Ltd",
    "20.44768530792448": "Shanghai Taiji Software Co.,Limited",
    "12.123917685182464": "Ideetron b.v.",
    "12.123917680754688": "Automata GmbH & Co. KG",
    "12.123917691609088": "Visual Fan",
    "12.346867970048": "Packet Plus, Inc.",
    "24.282444": "Huawei Technologies Co.,Ltd",
    "24.4241395": "ARRIS Group, Inc.",
    "24.9764596": "Sagemcom Broadband SAS",
    "24.16535972": "ARRIS Group, Inc.",
    "12.346868633600": "Assembly Contracts Limited",
    "20.31657723691008": "Tokai Rika Co., Ltd.",
    "20.75685686149120": "Shenzhen Qiutian Technology Co.,Ltd",
    "24.5249701": "GN Netcom A/S",
    "24.1879": "Topcall International AG",
    "24.5359": "TZero Technologies, Inc.",
    "12.346864570368": "RF-Embedded GmbH",
    "12.346859302912": "Microway",
    "24.1874": "Rhythm Watch Co., Ltd.",
    "24.1877": "Lafon",
    "24.5357": "Airak, Inc.",
    "24.5350": "AIM Infrarotmodule GmbH",
    "24.5351": "Stolinx,. Inc",
    "24.5352": "ARRIS Group, Inc.",
    "24.1872": "Cisco Systems, Inc",
    "24.6338946": "RO.VE.R. Laboratories S.p.A.",
    "24.9472351": "Weifang Goertek Electronics Co.,Ltd",
    "24.8951414": "TTC MARCONI s.r.o.",
    "24.11294779": "Human Assembler, Inc.",
    "24.6617985": "IEEE Registration Authority",
    "24.4731965": "IEP GmbH",
    "12.123917678321664": "Wuhan Xingtuxinke ELectronic Co.,Ltd",
    "12.346853908480": "BitBox Ltd",
    "20.273458025988096": "Maven Wireless AB",
    "24.794050": "SeAH Networks",
    "24.2936922": "Liteon Technology Corporation",
    "24.9199293": "Huawei Device Co., Ltd.",
    "12.346860412928": "Cascade Technologies Ltd",
    "12.123917675589632": "DEUTA-WERKE GmbH",
    "20.251018419896320": "EPS Bio",
    "20.189549206568960": "Shenzhen PuHua Technology Co., Ltd",
    "24.14440166": "Shenzhen Bococom Technology Co.,LTD",
    "20.71070625300480": "Securiton AG",
    "24.10016443": "Apple, Inc.",
    "24.9194716": "Planex Communications Inc.",
    "24.6031963": "Samsung Electro Mechanics Co., Ltd.",
    "12.346853621760": "Acunia N.V.",
    "24.4989528": "cozybit, Inc.",
    "24.12088895": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.12076998": "SanJet Technology Corp.",
    "24.4089": "Valcretec, Inc.",
    "24.4088": "Cisco Systems, Inc",
    "24.4083": "Jung Myoung Communications&Technology",
    "24.4082": "Loud Technologies Inc.",
    "24.4081": "nex-G Systems Pte.Ltd",
    "24.4080": "Sunray Co. Ltd.",
    "24.4087": "Cisco Systems, Inc",
    "12.346861789184": "Goanna Technologies Pty Ltd",
    "24.4085": "GN&S company",
    "24.4084": "Guntermann & Drunck GmbH",
    "24.12623682": "Apple, Inc.",
    "12.346855735296": "Aspel S.A.",
    "24.5686": "Quanta Computer Inc.",
    "24.5687": "CITEL SpA",
    "24.5684": "Mathtech, Inc.",
    "24.5685": "Hewlett Packard",
    "24.5682": "Samsung Electronics Co.,Ltd",
    "24.5683": "Oxford Diagnostics Ltd.",
    "24.5680": "Vativ Technologies",
    "24.5681": "Xteam",
    "12.346865639424": "taskit GmbH",
    "24.5689": "Ubiquam Co., Ltd.",
    "24.4555": "Jacobsons AB",
    "24.4554": "Long Range Systems, Inc.",
    "24.4557": "Axsun Technologies",
    "24.4556": "Guangzhou Jinpeng Group Co.,Ltd.",
    "24.4551": "Raymarine UK Ltd",
    "24.4550": "Seagate Technology",
    "24.4553": "MTT Corporation",
    "24.4552": "Powercom Co., Ltd.",
    "24.10542084": "Becker-Antriebe GmbH",
    "24.4559": "Thrane & Thrane A/S",
    "24.4558": "Ubisense Limited",
    "24.57390": "Spc Electronics Corporation",
    "24.14942875": "Intel Corporate",
    "24.13371848": "Imaqliq Ltd",
    "24.2901245": "Hewlett Packard",
    "24.3680": "360SUN Digital Broadband Corporation",
    "24.3681": "Microtrol Limited",
    "24.3682": "Nortel Networks",
    "12.346853367808": "MaxLinear Hispania S.L.U.",
    "24.3684": "Elphel, Inc",
    "24.3685": "TransCore",
    "24.3686": "Hitachi Industry & Control Solutions, Ltd.",
    "24.3687": "Eltis Microelectronics Ltd.",
    "24.3688": "E-TOP Network Technology Inc.",
    "24.3689": "China Electric Power Research Institute",
    "12.346859409408": "Bando electronic communication Co.Lltd",
    "24.6606446": "Sierra Wireless",
    "24.8953448": "Technicolor CH USA Inc.",
    "24.9972031": "Fujian Start Computer Equipment Co.,Ltd",
    "24.57453": "Compuware Corporation",
    "24.57452": "Ultra Electronics Command & Control Systems",
    "24.57451": "W&G Special Products",
    "24.57450": "Kapsch Ag",
    "24.57457": "Epis Microcomputer",
    "24.57456": "Dh Technology",
    "24.57455": "ARRIS Group, Inc.",
    "12.346858610688": "Winsys Informatica ltda",
    "24.57459": "National Amusement Network, Inc.",
    "24.57458": "Lynk",
    "24.2371832": "Huawei Device Co., Ltd.",
    "24.15219167": "Askey Computer Corp",
    "24.20543": "Anchor Games",
    "24.20542": "Cisco Systems, Inc",
    "24.20540": "Tsinghua Novel Electronics",
    "24.20547": "Marvell Semiconductor, Inc.",
    "24.20546": "Sci Manufacturing Singapore Pte, Ltd.",
    "24.20545": "Coretronic Corporation",
    "24.20544": "Panasonic Electric Works Co., Ltd.",
    "24.20549": "Rioworks Solutions, Inc.",
    "24.20548": "Asaca Corporation",
    "12.123917684703232": "Paradigm Technology Services B.V.",
    "12.123917675876352": "Kahler Automation",
    "12.346866913280": "RadioMobile Inc",
    "24.1065833": "Soundmax Electronic Limited",
    "24.3419405": "Samsung Electronics Co.,Ltd",
    "24.9978842": "Intertech",
    "24.8687327": "Sony Mobile Communications Inc",
    "12.346867621888": "Global Tecnologia LTDA.",
    "24.3697595": "ARRIS Group, Inc.",
    "24.8176879": "Devialet",
    "24.3981814": "Melange Systems Pvt. Ltd.",
    "24.57394": "Misys Financial Systems, Ltd.",
    "24.296074": "7Inova Technology Limited",
    "12.123917688315904": "Seclab",
    "24.9444110": "Fujitsu Technology Solutions GmbH",
    "24.3024": "XiMeta Technology Americas Inc.",
    "24.3025": "Aeronix, Inc.",
    "24.3026": "Remopro Technology Inc.",
    "24.3027": "cd3o",
    "24.3020": "Jusan, S.A.",
    "24.3021": "Hewlett Packard",
    "24.3022": "Free2move AB",
    "24.3023": "Agfa Ndt Inc.",
    "24.6617479": "Avvasi Inc.",
    "12.123917676793856": "StarBridge, Inc.",
    "24.3028": "Beijing Wise Technology & Science Development Co.Ltd",
    "12.346857078784": "Esmart Distribution Pte Ltd",
    "24.1585161": "Woojin Industrial Systems Co., Ltd.",
    "24.1585164": "Hisense Electric Co.,Ltd",
    "12.123917686632448": "Intesens",
    "20.172584083652608": "Annapurna labs",
    "12.346860519424": "Wise Industria de Telecomunicações Ldta.",
    "24.8130581": "Brilliant Home Technology, Inc.",
    "24.7078": "intotech inc.",
    "24.57396": "Cisco Systems, Inc",
    "12.123917691588608": "RedRat Ltd",
    "24.2682336": "AMPAK Technology, Inc.",
    "24.12880365": "Solid Optics EU N.V.",
    "24.13115238": "Dell Inc.",
    "12.123917678608384": "Inphi Corporation",
    "24.11807890": "Zhejiang Weirong Electronic Co., Ltd",
    "24.6079956": "Intel Corporate",
    "12.346868314112": "Celestial Audio",
    "24.57397": "Artesyn Embedded Technologies",
    "24.7630564": "Asia Vital Components Co.,Ltd.",
    "24.7096889": "Bita",
    "12.346868768768": "City Computing Ltd",
    "24.12399": "Seyeon Tech. Co., Ltd.",
    "24.796022": "Longcheer Telecommunication Limited",
    "20.189549204471808": "Jia-Teng",
    "24.12393": "Impacct Technology Corp.",
    "24.12392": "Cybernetics Tech. Co., Ltd.",
    "24.12391": "BIOSTAR Microtech Int'l Corp.",
    "24.12390": "Rfm",
    "12.346860163072": "Field Design Service",
    "24.12396": "Hitex Holding GmbH",
    "24.12395": "Cmos Systems, Inc.",
    "24.12394": "Penta Media Co., Ltd.",
    "24.15766196": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "20.272805464637440": "Tofino",
    "24.63343": "Apple, Inc.",
    "24.6356677": "Apple, Inc.",
    "24.3865": "Boston Scientific",
    "20.272689501569024": "Lit Technologies",
    "24.1060715": "Microsoft Corporation",
    "12.123917687275520": "Integritech",
    "20.167594821681152": "Praesideo B.V.",
    "24.8956343": "Apple, Inc.",
    "24.15523995": "Samsung Electronics Co.,Ltd",
    "12.123917679149056": "Wincode Technology Co., Ltd.",
    "24.1581269": "ARRIS Group, Inc.",
    "24.4207468": "Godrej & Boyce Mfg. co. ltd",
    "12.346853511168": "Haag-Streit AG",
    "12.123917678465024": "Equos Research Co., Ltd",
    "12.346865537024": "Level 3 Communications",
    "24.514": "Amino Communications, Ltd.",
    "24.24674": "Telesync, Inc.",
    "24.516": "Bodmann Industries Elektronik GmbH",
    "24.517": "Hitachi Denshi, Ltd.",
    "24.24671": "Nippon Unisoft Corporation",
    "24.24670": "Liberty Technology Networking",
    "24.24673": "Whistle Communications Corp.",
    "24.24672": "Data Innovations North America",
    "24.24679": "Acer Lan",
    "24.24678": "LACROIX Trafic",
    "24.287347": "Cisco Systems, Inc",
    "24.9659": "INNERINT Co., Ltd.",
    "24.9658": "Alcatel-Lucent IPD",
    "24.1464": "Electronic Design Associates, Inc.",
    "24.1465": "Airvana, Inc.",
    "24.1462": "INSYS Microelectronics GmbH",
    "24.1463": "Arbor Technology Corp.",
    "24.1460": "Aceex Corporation",
    "24.1461": "Broadcom Technologies",
    "24.9651": "Hewlett Packard",
    "24.9650": "MBDA Deutschland GmbH",
    "24.9653": "Cisco Systems, Inc",
    "24.9652": "Cisco Systems, Inc",
    "12.346852884480": "LMI Technologies",
    "24.9654": "Telecom FM",
    "24.9657": "Cypress Solutions Inc",
    "24.9656": "Agile Communications, Inc.",
    "24.13123625": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16006528": "FAG Industrial Services GmbH",
    "24.8954828": "Amatis Controls",
    "24.2845": "LayerZero Power Systems, Inc.",
    "24.2844": "SIBCO bv",
    "24.2847": "I CON Computer Co.",
    "24.2846": "KAPPA opto-electronics GmbH",
    "24.2841": "Vernier Networks, Inc.",
    "24.2840": "Private",
    "24.2843": "Systronix, Inc.",
    "24.2842": "Industrial Defender, Inc.",
    "12.123917676101632": "Vocality International Ltd",
    "20.207147372642304": "Shenzhen General Measure Technology Co., Ltd",
    "24.6584743": "Phison Electronics Corp.",
    "24.2849": "G-Star Communications Inc.",
    "24.2848": "Hirata corporation",
    "12.123917688057856": "C.D.N.Corporation",
    "24.12640280": "Huawei Technologies Co.,Ltd",
    "24.16389657": "trendytech",
    "12.123917675798528": "Look Easy International Limited",
    "24.1790": "Ambrado, Inc",
    "24.1791": "Sheba Systems Co., Ltd.",
    "24.1792": "Zettamedia Korea",
    "24.1793": "Racal-Datacom",
    "24.1794": "Varex Imaging",
    "24.1795": "CSEE Transport",
    "24.1796": "Alpsalpine Co,.Ltd",
    "24.1797": "Endress & Hauser GmbH & Co",
    "24.1798": "Sanritz Corporation",
    "24.1799": "Interalia Inc.",
    "24.8134728": "KoamTac",
    "24.1096190": "Lika srl",
    "20.180680390606848": "T-Touching Co., Ltd.",
    "24.14978845": "Huawei Device Co., Ltd.",
    "24.14168803": "Xiaomi Communications Co Ltd",
    "24.11856291": "port industrial automation GmbH",
    "12.123917683818496": "U-JIN Mesco Co., Ltd.",
    "24.6317139": "AOD Co.,Ltd.",
    "12.123917684756480": "Season Design Technology",
    "24.4772388": "Liteon Technology Corporation",
    "24.1855518": "Sunplus Technology Co., Ltd.",
    "24.12119947": "Shenzhen Cannice Technology Co.,Ltd",
    "24.1335554": "Alpine Electronics, Inc.",
    "20.207147365302272": "Process-Electronic Sp. z o.o.",
    "12.123917679316992": "Azmoon Keifiat",
    "12.346855092224": "FACTS Engineering LLC",
    "24.7636923": "Ciena Corporation",
    "12.123917676539904": "Wired Broadcast Ltd",
    "12.123917688745984": "Grupo Epelsa S.L.",
    "24.14214509": "TCT mobile ltd",
    "24.549269": "Rockwell Automation",
    "24.2655029": "Samsung Electronics Co.,Ltd",
    "12.123917682143232": "Private",
    "24.15744297": "Actility",
    "12.123917676490752": "IOT Engineering",
    "12.123917680553984": "Brakels IT",
    "12.346859487232": "Acte Sp. z o.o.",
    "24.4504134": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.346857115648": "Rose Technologies",
    "24.9978438": "Sol Industry Pte., Ltd",
    "24.11028298": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.16563409": "Huawei Technologies Co.,Ltd",
    "12.123917684891648": "Myostat Motion Control Inc",
    "20.48688644751360": "Exertus Oy",
    "24.8212": "Global View Co., Ltd.",
    "24.8213": "Actis Computer Sa",
    "24.8210": "Camtronics Medical Systems",
    "24.8211": "Diversified Technology, Inc.",
    "24.8216": "Cis Technology Inc.",
    "24.8217": "Ohler Gmbh",
    "24.8214": "Showa Electric Wire & Cable Co",
    "24.8215": "Orbotech",
    "24.8218": "MRV Communications, Inc.",
    "24.8219": "Northern Telecom/Network",
    "12.123917682003968": "Connit",
    "24.4948": "Zcomax Technologies, Inc.",
    "24.4949": "TOMEN Cyber-business Solutions, Inc.",
    "24.4946": "Naztec, Inc.",
    "24.4947": "HYDAC Filtertechnik GMBH",
    "24.4944": "Silver Spring Networks, Inc",
    "24.4945": "Niles Audio Corporation",
    "24.4942": "Valox Systems, Inc.",
    "24.4943": "Rapidus Wireless Networks Inc.",
    "24.2366182": "Huawei Device Co., Ltd.",
    "24.4941": "Inepro BV",
    "12.71297884463104": "Supplier Ind. e Com de Eletroeletrônicos",
    "12.123917675143168": "Ravelin Ltd",
    "20.74781676273664": "Lenovo Image(Tianjin) Technology Ltd.",
    "12.123917678252032": "Iso/Tc 22/Sc 31",
    "12.346853838848": "Survalent Technology Corporation",
    "12.346866077696": "Technagon GmbH",
    "20.255861674999808": "Yawata Electric Industrial Co.,Ltd.",
    "24.6593316": "V Technology Co., Ltd.",
    "24.4463784": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917682139136": "Eredi Giuseppe Mercuri SPA",
    "24.5558653": "Samsung Electronics Co.,Ltd",
    "24.2675268": "LCFC(HeFei) Electronics Technology Co., Ltd.",
    "12.123917678723072": "MiraeSignal Co., Ltd",
    "20.272689487937536": "Reason Tecnologia SA",
    "12.123917684670464": "JETI Technische Instrumente GmbH",
    "24.1593181": "JRC Mobility Inc.",
    "12.346855038976": "Lipowsky Industrie-Elektronik GmbH",
    "12.71297884307456": "GD Mission Systems",
    "24.6880126": "Apple, Inc.",
    "24.15244812": "Max8Usa Distributors Inc.",
    "12.123917690109952": "Transit Solutions, LLC.",
    "12.346854350848": "Level Control Systems",
    "12.123917683220480": "Yukilab",
    "24.1889981": "Leedarson Lighting Co., Ltd.",
    "24.1889982": "Wistron Neweb Corporation",
    "24.10272887": "PrintCounts, LLC",
    "24.8187277": "Microsoft",
    "24.12860650": "Buffalo.Inc",
    "12.346852745216": "Bsquare",
    "12.346859216896": "Fertron Controle e Automacao Industrial Ltda.",
    "12.71297883758592": "Lyngsoe Systems",
    "20.220958193352704": "HENAN FOXSTAR DIGITAL DISPLAY Co.,Ltd.",
    "20.176189946724352": "iLoda Solutions Limited",
    "12.123917677752320": "Code Hardware SA",
    "12.346859048960": "Interopix, Inc.",
    "20.229206582624256": "Private",
    "24.13940577": "Sichuan AI-Link Technology Co., Ltd.",
    "20.114975471304704": "Shenzhen NeaTech Intelligence Technology Co., Ltd.",
    "24.6566201": "IEEE Registration Authority",
    "12.123917690306560": "JSC Kaluga Astral",
    "24.15482365": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.11274079": "Huawei Technologies Co.,Ltd",
    "24.7112326": "Technonia",
    "20.224447337332736": "Shenzhen Hooenergy Technology Co.,Ltd",
    "12.123917689536512": "Syscom Instruments SA",
    "24.291004": "Globalstar Inc.",
    "24.3731608": "Vapor Stone Rail Systems",
    "12.123917687320576": "Ensys srl",
    "24.551146": "Eito Electronics Co., Ltd.",
    "12.346862489600": "CNS Systems, Inc.",
    "12.123917690916864": "ID Lock AS",
    "12.123917679779840": "Allim System Co,.Ltd.",
    "24.15224272": "Huawei Technologies Co.,Ltd",
    "24.9710143": "Diversey Inc",
    "12.346855464960": "Navionics S.p.A.",
    "24.13112203": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.9699334": "jinyoung",
    "20.62366874075136": "Matrixstream Technologies, Inc.",
    "20.145727905857536": "Research Electronics International, LLC.",
    "12.346857025536": "Protium Technologies, Inc.",
    "24.3210513": "Macrotech (Usa) Inc.",
    "24.569944": "Juniper Networks",
    "24.6319206": "Intel Corporate",
    "12.346854047744": "WaveIP",
    "24.6319203": "Hitron Technologies. Inc",
    "24.5773816": "ARRIS Group, Inc.",
    "24.3411181": "Moduel AB",
    "24.7636602": "Dell Inc.",
    "12.346859057152": "Tech Comm, Inc.",
    "12.123917676810240": "RELISTE Ges.m.b.H.",
    "12.123917678170112": "CAITRON Industrial Solutions GmbH",
    "20.277981929930752": "Coet Costruzioni Elettrotecniche",
    "24.1850078": "Samsung Electronics Co.,Ltd",
    "20.48543314214912": "Xiamen Vorlink Iot Technology Co.,Ltd.",
    "24.14996465": "Cisco Systems, Inc",
    "12.346861711360": "Embedded Data Systems, LLC",
    "24.13677661": "Huawei Device Co., Ltd.",
    "24.8673159": "DVR C&C Co., Ltd.",
    "20.27056777723904": "Shenzhen Fionexx Technologies Ltd.",
    "24.1842024": "ARRIS Group, Inc.",
    "12.346867814400": "Future Design Controls, Inc",
    "24.7351684": "i4C Innovations",
    "24.832990": "Alcatel Lucent",
    "24.11564300": "QA Cafe",
    "24.7401756": "Shenzhen Ogemray Technology Co.,Ltd",
    "24.7374278": "ARRIS Group, Inc.",
    "20.71389205757952": "Kendrion Kuhnke Automation GmbH",
    "24.6586507": "VG Controls, Inc.",
    "24.14996797": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7374274": "ASRock Incorporation",
    "24.2907444": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.7611786": "Hangzhou H3C Technologies Co., Limited",
    "24.11287925": "Hangzhou Zhiway Technologies Co.,Ltd.",
    "24.10399": "Semptian Co., Ltd.",
    "24.3709161": "Huawei Device Co., Ltd.",
    "12.346865229824": "GridManager A/S",
    "12.123917685321728": "KM OptoElektronik GmbH",
    "24.12115819": "SpotCam Co., Ltd.",
    "20.84372891893760": "Nemon Co., Ltd.",
    "24.10230210": "Dongguan Liesheng Electronic Co., Ltd.",
    "12.123917689233408": "Zamir Recognition Systems Ltd.",
    "24.7116495": "Apple, Inc.",
    "24.3205518": "Huawei Technologies Co.,Ltd",
    "24.5042112": "Amazon Technologies Inc.",
    "24.1330535": "Zioncom Electronics (Shenzhen) Ltd.",
    "24.9218741": "Espressif Inc.",
    "24.12890986": "Cisco Systems, Inc",
    "24.8957251": "Pegatron Corporation",
    "20.137215120244736": "Polarteknik Oy",
    "12.346862211072": "Inter Control Hermann Köhler Elektrik GmbH&Co.KG",
    "24.14716908": "Repotec Co., Ltd.",
    "12.123917686755328": "Rexxam Co.,Ltd.",
    "12.346852163584": "T.L.S. Corp.",
    "24.5779447": "Sagemcom Broadband SAS",
    "24.2895976": "Pantech Co.,Ltd",
    "24.16291898": "Leeman International (HongKong) Limited",
    "12.346867638272": "Ixonos Plc",
    "24.12351917": "Samsung Electronics Co.,Ltd",
    "20.89058714845184": "Coyote Sytem",
    "12.346858147840": "Rafael",
    "20.216039230013440": "Airbus DS - SLC",
    "24.12098202": "Xin Shi Jia Technology (Beijing) Co.,Ltd",
    "24.8937448": "IEEE Registration Authority",
    "24.7384279": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.13370999": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.71297883926528": "ACD Elektronik GmbH",
    "24.12119203": "Zyxel Communications Corporation",
    "24.13148602": "Neul Ltd",
    "24.13424671": "Maipu Communication Technology Co.,Ltd.",
    "24.15737312": "eero inc.",
    "24.6610460": "Kingnetic Pte Ltd",
    "24.8127433": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.838694": "Airenetworks",
    "24.12097966": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "12.71297883897856": "Micrologic",
    "24.11018342": "Apple, Inc.",
    "24.3707349": "Hangzhou H3C Technologies Co., Limited",
    "20.225135228354560": "Viper Design, LLC",
    "20.13692009709568": "Green Fox Electro AS",
    "24.11592421": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917678780416": "Videotrend srl",
    "12.123917688143872": "SINED srl",
    "20.97400606687232": "Loftie",
    "24.15264474": "Denkovi Assembly Electronics LTD",
    "20.229631760269312": "Apption Labs Limited",
    "24.13373603": "Beijing Nan Bao Technology Co., Ltd.",
    "20.123731213025280": "Veracity UK Ltd",
    "24.1346634": "Huawei Technologies Co.,Ltd",
    "24.552434": "Shenzhen MIMOWAVE Technology Co.,Ltd",
    "24.5775207": "Open-m technology limited",
    "24.13169542": "Shenzhen Xiaoniao Technology Co.,Ltd",
    "24.5784728": "Xiaomi Communications Co Ltd",
    "24.7918477": "Hongkong Yunlink Technology Limited",
    "12.123917681373184": "Suprock Technologies",
    "24.556321": "Altasec technology corporation",
    "24.3447153": "Quanta Storage Inc.",
    "24.3447151": "VPI Engineering",
    "12.346856488960": "Boundless Technologies",
    "24.14173702": "Silicon Valley Global Marketing",
    "24.42431": "Cisco Systems, Inc",
    "24.16266305": "Intel Corporate",
    "24.13160798": "Huawei Technologies Co.,Ltd",
    "12.346867208192": "Saia-Burgess Controls AG",
    "24.4773177": "Huawei Technologies Co.,Ltd",
    "20.216039235256320": "Labor Strauss Sicherungsanlagenbau GmbH",
    "24.275394": "Intel Corporate",
    "12.346860806144": "KPE spol. s r.o.",
    "12.123917677871104": "Equipos De Telecomunicación Optoelectrónicos, S.A.",
    "24.4760448": "Gionee Communication Equipment Co.,Ltd.",
    "24.13641028": "Samsung Electro-Mechanics(Thailand)",
    "24.2934086": "Huawei Device Co., Ltd.",
    "24.2934088": "IAdea Corporation",
    "24.5561373": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "20.31657722642432": "TengFeng",
    "12.119270006784": "YESpay International Ltd",
    "24.15511814": "Acuro Networks, Inc",
    "20.206931531661312": "Q-PRODUCTS a. s.",
    "24.5748": "EuroCB (Phils.), Inc.",
    "24.9175802": "COMMANDO Networks Limited",
    "24.12617707": "Samsung Electronics Co.,Ltd",
    "20.242876971745280": "Amazinglayer Network Co., Ltd.",
    "12.346853724160": "Cognex Corporation",
    "24.4508555": "LG Innotek",
    "24.1313910": "Freebox Sas",
    "12.71297884684288": "ICS Eletronics",
    "12.123917682876416": "thou&tech",
    "24.16014195": "Wanjiaan Interconnected Technology Co., Ltd",
    "12.123917681381376": "Jsc \"Otzvuk\"",
    "24.12883973": "Minieum Networks, Inc",
    "24.3441410": "iFORCOM Co., Ltd",
    "24.311768": "Huawei Device Co., Ltd.",
    "12.346860699648": "SMT d.o.o.",
    "12.123917676072960": "Beijing Nacao Technology Co., Ltd.",
    "24.305944": "Elecom Co.,Ltd.",
    "12.71297884631040": "KMtronic ltd",
    "24.1620774": "Fn-Link Technology Limited",
    "12.346858586112": "Harman/Becker Automotive Systems GmbH",
    "24.3132": "MediaChorus, Inc.",
    "24.5031553": "Shenzhen Aisat Electronic Co., Ltd.",
    "12.123917679386624": "Dr. Zinngrebe GmbH",
    "24.7603440": "Mobiwire Mobiles (NingBo) Co., LTD",
    "24.13928592": "Samsung Electronics Co.,Ltd",
    "12.123917675945984": "Nasdaq",
    "12.123917691207680": "Tessa Agritech Srl",
    "24.8715518": "Apple, Inc.",
    "24.15237013": "Apple, Inc.",
    "12.123917681803264": "Benchmark Electronics - Secure Technology",
    "12.346855849984": "Wireless Reading Systems Holding ASA",
    "24.1592854": "Samsung Electronics Co.,Ltd",
    "24.12097968": "Cohere Technologies",
    "24.45161": "Honewell Oy",
    "24.12359561": "Huawei Device Co., Ltd.",
    "24.45165": "Jones Futurex Inc.",
    "24.7138641": "NEXCONTROL Co.,Ltd",
    "24.3180077": "Arista Network, Inc.",
    "24.6311": "Yoggie Security Systems LTD.",
    "24.6056144": "Aurora Innovation Inc.",
    "24.6310": "Persistent Systems, LLC",
    "24.6613222": "global moisture management system",
    "24.6313": "Ethernet Direct Corporation",
    "12.346865217536": "Industry Controls, Inc.",
    "24.15248378": "Shenzhen Reecam Tech.Ltd.",
    "24.9501531": "Avaya Inc",
    "24.6312": "AnNeal Technology Inc.",
    "24.4983610": "TCT mobile ltd",
    "24.13689671": "Yoga",
    "20.39919490695168": "sonoscape",
    "24.6314": "Protec Fire Detection plc",
    "24.7833": "Vantanol Industrial Corporation",
    "20.31368131117056": "Zhuhai MYZR Technology Co.,Ltd",
    "24.7831": "Medium Link System Technology CO., LTD,",
    "24.7830": "Sepura Plc",
    "24.7837": "Recall Technologies, Inc.",
    "24.6317": "Nidec Sankyo Corporation",
    "24.7835": "San-Eisha, Ltd.",
    "24.7834": "HAMILTON Bonaduz AG",
    "24.7839": "Visioneering Systems, Inc.",
    "24.6316": "Shanghai Jiao Da HISYS Technology Co. Ltd.",
    "20.154039766482944": "ViaWear, Inc.",
    "12.123917676244992": "DSP4YOU LTd",
    "24.12082811": "Samsung Electronics Co.,Ltd",
    "20.71414667280384": "BBB Inc.",
    "12.123917687115776": "JungwooEng Co., Ltd",
    "24.4503956": "Douglas Lighting Controls",
    "24.5308146": "Sify Technologies Ltd",
    "24.1353260": "S.M. Dezac S.A.",
    "12.346857660416": "Honeywell",
    "24.5742": "Arbitron Inc.",
    "12.71297883893760": "EnVerv Inc.",
    "12.346858045440": "Delsat Group S.A.",
    "24.13691957": "Cisco Systems, Inc",
    "20.193649362075648": "Aether Services, Inc.",
    "12.123917686255616": "David Horn Communications Ltd",
    "24.2941712": "Stratacache",
    "24.10772379": "Nexell",
    "24.10013604": "Benchmark Electronics",
    "24.1096357": "Gana I&C Co., Ltd",
    "24.1858645": "PRIMA Cinema, Inc",
    "24.10778456": "Ningbo Freewings Technologies Co.,Ltd",
    "24.16290090": "GLP German Light Products GmbH",
    "12.123917687083008": "Fenotech Inc.",
    "24.5788298": "Mitsubishi Electric Corporation",
    "12.71297884811264": "Beat Sensing co. , ltd.",
    "20.71070622154752": "Omnitracs, LLC",
    "24.3209157": "Apple, Inc.",
    "24.14078": "SuperVision",
    "24.14072": "Conti Temic microelectronic GmbH",
    "24.8009194": "SHARP Corporation",
    "24.13954060": "Harley-Davidson Motor Company",
    "24.15269173": "Huawei Device Co., Ltd.",
    "12.123917684658176": "Sensor Developments",
    "12.123917688213504": "Remote Diagnostic Technologies Ltd",
    "24.1573245": "Harbin Arteor technology co., LTD",
    "20.132811188076544": "Zhejiang Science Electronic Tech Co., Ltd",
    "20.145383549304832": "Shenzhen Lidaxun Digital Technology Co.,Ltd",
    "24.15208392": "Universal Electronics, Inc.",
    "24.1852259": "Huawei Technologies Co.,Ltd",
    "24.1595008": "Intel Corporate",
    "24.9973178": "Fibergate Inc.",
    "12.123917686734848": "Systolé Hardware B.V.",
    "24.16288210": "Cloud Network Technology Singapore Pte. Ltd.",
    "12.123917682020352": "TTC TELEKOMUNIKACE, s.r.o.",
    "12.346859159552": "Thales UK",
    "24.14730103": "Samsung Electronics Co.,Ltd",
    "24.4480890": "Shihlin Electric & Engineering Corp.",
    "20.66129349312512": "Inter Action Corporation",
    "12.71297883664384": "Toec Technology Co.,Ltd",
    "24.5529": "Samsung Electronics Co.,Ltd",
    "24.5528": "Kolektor group",
    "20.44169489809408": "Epoch International Enterprises, Inc.",
    "24.5004812": "Routerboard.com",
    "24.13420015": "Co-Comm Servicios Telecomunicaciones S.L.",
    "24.5523": "U4EA Technologies Inc.",
    "24.5522": "Facom UK Ltd (Melksham)",
    "24.5521": "RLW Inc.",
    "24.5520": "Hectronic GmbH",
    "24.5527": "Aeta Audio Systems",
    "24.5526": "ARRIS Group, Inc.",
    "24.5525": "Quester Tangent Corporation",
    "24.5524": "Bixolon Co.,Ltd",
    "24.16037152": "Biostar Microtech international corp.",
    "24.6501": "YuHua TelTech (ShangHai) Co., Ltd.",
    "20.30931580616704": "p2-plus inc.",
    "20.45033669525504": "Performance Motion Devices",
    "20.268902730825728": "Cheetah Medical",
    "24.12075359": "Asp Ag",
    "24.6503": "TELDAT Sp.J.",
    "12.123917689778176": "SA Photonics",
    "12.346858708992": "T2 Communication Ltd",
    "20.27478316810240": "Accel Robotics",
    "24.5538618": "Zyxel Communications Corporation",
    "12.123917676236800": "S.E.I. Co.,Ltd.",
    "12.123917689159680": "AMMT GmbH",
    "12.346859323392": "Etani Electronics Co.,Ltd.",
    "24.9709871": "New H3C Technologies Co., Ltd",
    "24.13381047": "Apple, Inc.",
    "24.13409342": "Cisco Meraki",
    "24.10492183": "Bertel S.p.a.",
    "20.71070620057600": "TW-TeamWare",
    "24.11548713": "Siliconware Precision Industries Co., Ltd.",
    "24.3959421": "Niigata Power Systems Co., Ltd.",
    "24.14474096": "Tonfunk Systementwicklung und Service GmbH",
    "24.10769808": "Xiaomi Communications Co Ltd",
    "12.123917686239232": "Siglead Inc",
    "24.5801449": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "12.123917676675072": "Beijing Muniulinghang Technology Co., Ltd",
    "20.39672888688640": "SuZhou A-rack Information Technology Co.,Ltd",
    "20.62663542439936": "Siemens AG, PG IE R&D",
    "24.1067645": "Intel Corporate",
    "24.6561409": "Dowslake Microsystems",
    "12.123917688582144": "Mesh Motion Inc",
    "12.346856407040": "Applied Materials UK Ltd",
    "12.123917682094080": "Beijing Zgh Security Research Institute Co., Ltd",
    "24.13161982": "Shenzhen Zowee Technology Co., Ltd",
    "24.11306622": "Create New Technology (HK) Limited Company",
    "24.7649566": "Nanjing Bestway Automation System Co., Ltd",
    "20.127937501200384": "SpringCard",
    "24.14739688": "Olive Telecommunication Pvt. Ltd",
    "24.14739686": "Shenzhen C-Data Technology Co., Ltd.",
    "20.67047981580288": "Naruida Technology Ltd.",
    "20.31477533245440": "Sequent AG",
    "24.324796": "Xena Networks",
    "12.123917688193024": "Vars Technology",
    "24.14727089": "ARRIS Group, Inc.",
    "24.12380007": "Quanzhou TDX Electronics Co., Ltd",
    "20.145211805138944": "Beijing Dayu Technology Co., Ltd.",
    "20.228784613621760": "Abb Evi Spa",
    "12.123917683441664": "Algra tec AG",
    "12.346854793216": "Telvent",
    "24.7130582": "Netgear",
    "24.7908544": "Hewlett Packard",
    "12.123917676535808": "Larraioz Elektronika",
    "24.9452262": "Parrot Sa",
    "24.14197525": "vivo Mobile Communication Co., Ltd.",
    "24.1850671": "Huawei Technologies Co.,Ltd",
    "24.12597594": "Zhejiang Dahua Technology Co., Ltd.",
    "24.899": "Metera Networks, Inc.",
    "24.898": "A-One Co., Ltd.",
    "24.895": "Atheros Communications, Inc.",
    "24.894": "PORTech Communications, Inc.",
    "24.897": "Ingenico International",
    "24.896": "SSH Communications Security Corp.",
    "24.891": "IDEC IZUMI Corporation",
    "24.890": "Taiyo Yuden Co., Ltd.",
    "24.893": "Stellcom",
    "24.892": "Coax Media",
    "24.2112870": "ARRIS Group, Inc.",
    "12.346862403584": "Vitel Net",
    "24.3682553": "LG Electronics (Mobile Communications)",
    "24.16545652": "Intel Corporate",
    "12.346854936576": "Septier Communication Ltd",
    "12.123917687799808": "Technical Panels Co. Ltd.",
    "24.7316": "LI-COR Biosciences",
    "24.7317": "Opticomm Corporation",
    "24.7314": "Tervela",
    "24.7315": "ExaDigm Inc",
    "24.20730": "Oxtel, Ltd.",
    "24.7313": "Gefen LLC",
    "24.7310": "Alcatel-Lucent IPD",
    "24.7311": "Advanced Electronic Design, Inc.",
    "24.6830664": "SAMJIN Co., Ltd.",
    "24.7318": "Linkwise Technology Pte Ltd",
    "24.7319": "Enzytek Technology Inc.,",
    "24.12897666": "Hangzhou Lowan Information Technology Co., Ltd.",
    "24.1866821": "Vitek Industrial Video Products, Inc.",
    "20.14223585312768": "GainStrong Industry Co.,Ltd",
    "24.13152064": "marco Systemanalyse und Entwicklung GmbH",
    "24.10281334": "Cisco Systems, Inc",
    "24.7131186": "LG Electronics",
    "20.452714299392": "Tianjin Lianwu Technology Co., Ltd.",
    "12.346852855808": "Radiant Networks Plc",
    "20.93020689006592": "Torrap Design Limited",
    "12.346857996288": "EM Motorsport Ltd",
    "24.14965460": "Eltex Enterprise Ltd.",
    "24.2421032": "Telstra",
    "20.171977814835200": "Teptron AB",
    "12.346859753472": "cryptiris",
    "20.44768525549568": "SolarEdge Technologies",
    "24.53413": "American Arium",
    "24.53412": "Alantro Communications",
    "24.53411": "Vocal Data, Inc.",
    "24.53410": "Integrated Device",
    "24.53417": "Shinano Kenshi Co., Ltd.",
    "24.53416": "Network Engines, Inc.",
    "24.53415": "Tokyo Sokki Kenkyujo Co., Ltd.",
    "24.53414": "Lanbird Technology Co., Ltd.",
    "24.53419": "Deltakabel Telecom Cv",
    "24.53418": "Chase Communications",
    "24.11272943": "Comsis",
    "24.5271186": "Innocent Technology Co., Ltd.",
    "24.12107129": "TrendPoint Systems",
    "24.5009767": "PoLabs d.o.o.",
    "24.10001495": "Huawei Technologies Co.,Ltd",
    "24.6876258": "YODO Technology Corp. Ltd.",
    "24.7361343": "Alfa Instrumentos Eletronicos Ltda.",
    "24.7922613": "Hewlett Packard",
    "24.7646647": "China Mobile Group Device Co.,Ltd.",
    "24.1590370": "Riava Networks, Inc.",
    "24.9744099": "Belden Hirschmann Industries (Suzhou) Ltd.",
    "12.123917687427072": "SolwayTech",
    "12.346856820736": "Pixtree Technologies, inc.",
    "24.5820235": "Lufkin Industries",
    "24.14962473": "MA Lighting Technology GmbH",
    "12.123917684183040": "Rievtech Electronic Co.,Ltd",
    "12.346867539968": "BAE Systems",
    "24.7629698": "Movek",
    "24.11838959": "Sercomm Corporation.",
    "24.5018534": "IEEE Registration Authority",
    "24.5034927": "HMD Global Oy",
    "24.15242316": "Cogent Healthcare Systems Ltd",
    "24.7653925": "Calxeda, Inc.",
    "24.8169853": "Objenious",
    "24.7124682": "DIVUS GmbH",
    "24.11296016": "Pace Americas",
    "24.24730": "Njk Techno Co.",
    "24.24731": "AstroNova, Inc",
    "24.24732": "Perkin-Elmer Incorporated",
    "24.24733": "Pmi Food Equipment Group",
    "24.24734": "Asc X3 - Information Technology Standards Secretariats",
    "24.24735": "Phast Corporation",
    "24.24736": "Switched Network Technologies, Inc.",
    "24.6198": "REJ Co.,Ltd",
    "24.24738": "Nihon Unisys Limited Co.",
    "24.24739": "Continuum Technology Corp.",
    "24.6195": "Texas Instruments",
    "24.6194": "Texas Instruments",
    "24.6193": "Texas Instruments",
    "24.6192": "Texas Instruments",
    "24.6191": "Texas Instruments",
    "24.6190": "XStreamHD",
    "24.10785600": "Beijing Supvan Information Technology Co.,Ltd.",
    "24.6562326": "Shandong Taixin Electronic co.,Ltd",
    "24.1894087": "Cisco Systems, Inc",
    "24.2423539": "Shanghai Flexem Technology Co.,Ltd.",
    "20.193649368367104": "TAIWAN Anjie Electronics Co.,Ltd.",
    "12.123917682843648": "Tecsag Innovation AG",
    "12.123917686349824": "Wart-Elektronik",
    "20.58071050092544": "eesy-innovation GmbH",
    "24.15251138": "Juniper Networks",
    "24.16541391": "Apple, Inc.",
    "24.2937924": "Fujitsu Limited",
    "20.88914837635072": "Bliq B.V.",
    "24.311963": "Aura Home, Inc.",
    "12.123917689552896": "eCozy GmbH",
    "20.167295453233152": "INFODAS GmbH",
    "12.123917679632384": "Shenzhen Vikings Technology Co., Ltd.",
    "12.346858389504": "DIAS Infrared GmbH",
    "24.305": "Bosch Security Systems, Inc.",
    "24.304": "Extreme Networks, Inc.",
    "24.307": "KYOWA Electronic Instruments C",
    "24.306": "Dranetz - BMI",
    "24.301": "Komodo Technology",
    "24.300": "Aravox Technologies, Inc.",
    "24.303": "Twinhead International Corp",
    "24.302": "PC Partner Ltd.",
    "24.309": "KDC Corp.",
    "20.22333150461952": "NPort Networks Inc.,",
    "24.6297031": "IdaTech",
    "20.5239427039232": "Extech Electronics Co., LTD.",
    "24.1615": "PRO-NETS Technology Corporation",
    "24.37036": "Optivision, Inc.",
    "24.37035": "Cisco Systems, Inc",
    "24.37034": "Indigo Active Vision Systems Limited",
    "24.37033": "Western Digital",
    "24.37032": "NineTiles Networks, Ltd.",
    "24.37031": "Clientec Corporation",
    "24.37030": "Cisco Systems, Inc",
    "24.1619": "Cisco Systems, Inc",
    "24.1618": "Cisco Systems, Inc",
    "24.37039": "J. Morita Mfg. Corp.",
    "24.37038": "ITALTEL S.p.A/RF-UP-I",
    "24.15348916": "ThinkRF Inc.",
    "24.32779": "Csk Corporation",
    "24.32778": "Japan Computer Corp.",
    "24.32777": "Jupiter Systems (older MX-600 series machines)",
    "24.32776": "Dynatech Computer Systems",
    "24.32775": "Dlog Nc-Systeme",
    "24.32774": "Compuadd Corporation",
    "24.32773": "Cactus Computer Inc.",
    "24.32772": "Antlow Computers, Ltd.",
    "24.32771": "Hytec Electronics Ltd.",
    "24.32770": "Satelcom (Uk) Ltd",
    "24.2886633": "Cisco Systems, Inc",
    "24.837324": "Huawei Technologies Co.,Ltd",
    "24.6081791": "Techroutes Network Pvt Ltd",
    "24.5562393": "Ring LLC",
    "24.16572751": "BRK Brands, Inc.",
    "20.30931579568128": "Liscotech System Co., Ltd.",
    "24.9373": "NES Technology Inc.",
    "24.9370": "Beijing Zhongchuang Telecommunication Test Co., Ltd.",
    "24.9371": "Action Star Enterprise Co., Ltd.",
    "24.9376": "ARRIS Group, Inc.",
    "24.9377": "ARRIS Group, Inc.",
    "24.9374": "ADC-Elektronik GmbH",
    "24.9375": "RIM Testing Services",
    "24.9378": "Hong Kong Middleware Technology Limited",
    "24.9379": "Sonim Technologies Inc",
    "24.2019": "Navcom Technology, Inc.",
    "20.48548676632576": "Exegy Inc",
    "20.184991248023552": "Shanghai East China Computer Co., Ltd",
    "20.247076409573376": "Folksam AB",
    "24.15269234": "Shanghai Linguo Technology Co., Ltd.",
    "24.3731101": "Anedo Ltd.",
    "12.123917690474496": "Xafax Nederland bv",
    "24.11573986": "STEMMER IMAGING GmbH",
    "20.84372888748032": "Jsc Nic",
    "24.3163209": "PLATH GmbH",
    "12.123917678018560": "Peter Huber",
    "12.346866614272": "T2M2 GmbH",
    "12.346853720064": "Jeffress Engineering Pty Ltd",
    "24.5538939": "Digital Devices GmbH",
    "24.6562816": "Xorcom Ltd.",
    "24.5032531": "Skyera, Inc.",
    "24.8397": "Hybrid Networks, Inc.",
    "24.8396": "Digital Services, Ltd.",
    "24.8395": "Pretec Electronics Corp.",
    "24.8394": "Digital Ocean",
    "24.8393": "Victron Bv",
    "24.8392": "Larscom Incorporated",
    "24.8391": "AKAI Professional M.I. Corp.",
    "24.8390": "Nectec",
    "12.346859954176": "Crinia Corporation",
    "24.8399": "Test & Measurement Systems Inc",
    "24.8398": "Logical Design Group, Inc.",
    "24.10259836": "Alpsalpine Co,.Ltd",
    "24.6612377": "EFM Networks",
    "20.5297549606912": "Fuzhou Zhuoyi Electronic Co.,Ltd",
    "24.524336": "Network Research Corporation",
    "24.524337": "Little Machines Inc.",
    "24.524334": "Metaphor Computer Systems",
    "24.524335": "Prime Computer",
    "24.524332": "Britton Lee Inc.",
    "24.524333": "Lan-Tec Inc.",
    "24.524330": "Mosaic Technologies Inc.",
    "24.524331": "Digital Equipment Corporation",
    "12.123917677674496": "Step Technica Co., Ltd.",
    "24.11824437": "Dalian Seasky Automation Co;Ltd",
    "24.524338": "Tigan Incorporated",
    "24.524339": "Bausch & Lomb",
    "24.3420612": "AVM GmbH",
    "24.943": "Paragea Communications",
    "24.9443452": "Samsung Electro Mechanics Co., Ltd.",
    "24.5032286": "Samsung Electronics Co.,Ltd",
    "24.942": "Allied Advanced Manufacturing Pte, Ltd.",
    "24.1317059": "Intel Corporate",
    "24.7604652": "Quext, LLC",
    "24.941": "Emerson Energy Systems AB",
    "24.8021": "Honeywell Security (China) Co., Ltd.",
    "24.8020": "Lorex Technology Inc.",
    "24.8023": "Phonik Innovation Co.,LTD",
    "24.8022": "Digital Forecast",
    "24.8025": "Kronback Tracers",
    "24.8024": "EMH Energiemesstechnik GmbH",
    "24.8027": "Apple, Inc.",
    "24.8026": "Beckwith Electric Co.",
    "24.8029": "Nokia Danmark A/S",
    "24.8028": "Nokia Danmark A/S",
    "24.946": "Radware",
    "24.11595387": "Roku, Inc",
    "12.346868580352": "Yash SiQure Technologies India Pvt. Ltd.",
    "24.12332117": "Bear Flag Design, Inc.",
    "24.15790223": "Nextek Solutions Pte Ltd",
    "24.944": "Xsense Technology Corp.",
    "24.2649983": "Cisco Systems, Inc",
    "24.8702386": "D-Link International",
    "12.346863366144": "Santa Barbara Instrument Group",
    "12.123917676593152": "XV360 Optical Information Systems Ltd.",
    "24.949": "Entra Technology Co.",
    "24.5779266": "Universal Electric Corporation",
    "24.11838213": "Xiamen Yaxon Network Co., Ltd.",
    "24.5779264": "Nintendo Co.,Ltd",
    "12.123917685235712": "Sportsbeams Lighting, Inc.",
    "24.14190091": "IEEE Registration Authority",
    "12.123917686050816": "Gbs Communications, Llc",
    "24.15991599": "Reduxio Systems",
    "20.246679824498688": "Tryen",
    "24.15250753": "zte corporation",
    "12.123917681786880": "The Dini Group, La Jolla inc.",
    "20.268902732922880": "RSAE Labs Inc",
    "24.9705": "i-mate Development, Inc.",
    "24.9742858": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917691408384": "Shenzhen Hui Rui Tianyan Technology Co., Ltd.",
    "24.12903327": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346861486080": "MaCaPS International Limited",
    "24.13676470": "Linktop Technology Co., LTD",
    "24.2682998": "Apple, Inc.",
    "20.180577504329728": "Chengdu Territory Technology Co.,Ltd",
    "12.346867589120": "Monsoon Solutions, Inc.",
    "12.346867634176": "ANT Group s.r.l",
    "24.5244587": "Cisco Systems, Inc",
    "12.123917689384960": "Pengo Technology Co., Ltd",
    "12.123917682987008": "Editech Co., Ltd",
    "24.2926645": "Rim",
    "24.11324075": "Virtual Electric Inc",
    "24.12095412": "Jabil Circuit Italia S.R.L",
    "12.123917680517120": "RTDS Technologies Inc.",
    "24.13953291": "Cvt Co.,Ltd",
    "20.31614735220736": "Hangzhou Kaierda Electric Welding Machine Co.,Ltd",
    "20.31477530099712": "Tangshan Liulin Automation Equipment Co., Ltd.",
    "24.7352344": "Avaya Inc",
    "24.2893869": "Bbk Educational Electronics Corp.,Ltd.",
    "12.123917683982336": "EIDOS s.r.l.",
    "24.1599618": "Beward R&D Co., Ltd.",
    "20.272805457297408": "Zengge Co., Limited",
    "24.1573": "The Linksys Group, Inc.",
    "24.7356855": "Edwards Lifesciences LLC",
    "24.13957537": "Huawei Technologies Co.,Ltd",
    "12.123917675745280": "Capgemini Netherlands",
    "24.6867684": "Cisco Systems, Inc",
    "20.36069850480640": "MeshPlusPlus, Inc.",
    "24.1575": "Uniwide Technologies, Inc.",
    "12.346865643520": "Rutter Inc",
    "24.1576": "Cisco Systems, Inc",
    "12.346861662208": "INTEGRA Biosciences AG",
    "24.10803286": "Gimbal, Inc",
    "12.123917682647040": "Mettler Toledo",
    "24.1577": "IBM RISC6000 system",
    "12.346855276544": "SavvyCorp.com Ltd",
    "20.167594816438272": "Beijing 3CAVI Tech Co.,Ltd",
    "24.10487704": "Samsung Electronics Co.,Ltd",
    "24.1579": "Intraserver Technology",
    "24.5288803": "Digitron da Amazonia S/A",
    "12.346863915008": "haneron",
    "24.13425068": "Intel Corporate",
    "12.346864857088": "Graesslin GmbH",
    "24.12893828": "Texas Instruments",
    "24.11028813": "Huawei Technologies Co.,Ltd",
    "24.8707865": "Chuango Security Technology Corporation",
    "20.31368101756928": "Wide World Trade HK ltd.",
    "12.123917682786304": "Aural Ltd",
    "12.123917690241024": "Breas Medical AB",
    "12.123917680582656": "ALTIT.CO.,Ltd.",
    "24.14974053": "Sunstar Communication Technology Co., Ltd",
    "24.4510944": "Sony Mobile Communications Inc",
    "24.4755010": "DIGALOG GmbH",
    "24.11010779": "zte corporation",
    "24.8126865": "Apple, Inc.",
    "24.3459075": "Nokia Corporation",
    "24.14160793": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.6043872": "Shanghai Super Electronics Technology Co.,LTD",
    "24.9198576": "ARRIS Group, Inc.",
    "12.346857234432": "Ingenieurbüro Schober",
    "12.123917689413632": "Hiber",
    "12.123917686571008": "Secuinfo Co. Ltd",
    "12.123917687824384": "Han Chang",
    "24.4757706": "Sichuan AI-Link Technology Co., Ltd.",
    "12.123917681442816": "EBE Mobility & Green Energy GmbH",
    "20.106480344760320": "Novo innovations Ltd",
    "24.10211": "Cisco Systems, Inc",
    "24.9441036": "Cloud Network Technology Singapore Pte. Ltd.",
    "20.233630196563968": "Shenzhen Anysec Technology Co. Ltd",
    "24.12614865": "Pegatron Corporation",
    "12.123917682761728": "Tirasoft Nederland",
    "24.3427152": "zte corporation",
    "24.7246": "TASA International Limited",
    "24.14457486": "Nanjing Cocomm electronics co., LTD",
    "24.13407703": "Vizio, Inc",
    "24.4500953": "Cisco Systems, Inc",
    "24.1317872": "Intellimedia Systems Ltd",
    "24.14197342": "Apple, Inc.",
    "24.15512897": "SHINANO E and E Co.Ltd.",
    "24.33011": "Sun Electronics Corp.",
    "24.33010": "Raycom Systems Inc",
    "24.33013": "Quantel Ltd",
    "24.33012": "Telemechanique Electrique",
    "24.33015": "Zenith Communications Products",
    "24.33014": "Synergy Microsystems",
    "24.33017": "Heurikon Corporation",
    "24.33016": "Mizar, Inc.",
    "24.33019": "Bvm Limited",
    "12.346863394816": "Gygax Embedded Engineering GEE.ch",
    "12.123917679992832": "hera Laborsysteme GmbH",
    "20.31657732079616": "Dabi Atlante S/A Industrias Medico Odontológicas",
    "24.4501029": "Cisco Systems, Inc",
    "24.8665045": "Samsung Electronics Co.,Ltd",
    "20.180659902480384": "Shanghai KuanYu Industrial Network Equipment Co.,Ltd",
    "20.212188453142528": "IVT corporation",
    "24.15783159": "varram system",
    "24.2140747": "Cisco-Linksys, LLC",
    "12.346859806720": "Monitor Electronics Ltd",
    "12.346853273600": "Perceptics Corp.",
    "24.11843673": "Texa Spa",
    "12.123917688754176": "Stuyts Engineering Haarlem BV",
    "12.346856169472": "Adilec Enginyeria SL",
    "24.15261184": "ChongQing GuanFang Technology Co.,LTD",
    "24.7086498": "AMPAK Technology, Inc.",
    "24.8186858": "Asct",
    "24.13132999": "zte corporation",
    "12.123917687861248": "Future Skies",
    "12.123917675675648": "Communication Technology Ltd.",
    "12.346856333312": "Tumsan",
    "12.346856923136": "Mobile Matrix, Inc.",
    "24.14972548": "Apple, Inc.",
    "24.16552186": "Independent Technologies",
    "12.346858930176": "Protech Optronics Co., Ltd.",
    "24.3456506": "Cisco SPVTG",
    "24.3456505": "Shanghai WDK Industrial Co.,Ltd.",
    "24.16043938": "Espressif Inc.",
    "12.119269912576": "L.G.L. Electronics S.p.a.",
    "12.346858938368": "Eds",
    "24.4470758": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5258876": "LeiShen Intelligent System Co.Ltd",
    "12.123917677203456": "D-E-K GmbH & Co.KG",
    "12.123917677477888": "Arclan'System",
    "12.346858754048": "Pan-STARRS",
    "24.9977702": "Wuhan Funshion Online Technologies Co.,Ltd",
    "24.12608404": "Apple, Inc.",
    "12.123917691002880": "Matsuhisa Corporation",
    "24.14215506": "Keopsys",
    "24.7384404": "Malvern Instruments Ltd",
    "12.123917679529984": "Perform3-D LLC",
    "24.6554315": "ARRIS Group, Inc.",
    "24.6038595": "O-Sung Telecom Co., Ltd.",
    "12.346863341568": "SAN GIORGIO S.E.I.N. srl",
    "24.13423259": "Shenzhen Bopengfa Elec&Technology CO.,Ltd",
    "24.11816854": "Zhejiang Xinzailing Technology co., ltd",
    "20.39672894980096": "Helen of Troy",
    "12.346853576704": "Standard Comm. Corp.",
    "24.838086": "Samwin Hong Kong Limited",
    "24.7113519": "zte corporation",
    "12.119269634048": "Check-It Solutions Inc",
    "24.6590657": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.346857275392": "Telematrix",
    "24.3672747": "Formlabs",
    "24.49350": "Personal Media Corp.",
    "24.3279": "Cisco Systems, Inc",
    "24.3278": "Cisco Systems, Inc",
    "24.3275": "Design Combus Ltd",
    "24.3274": "HGST a Western Digital Company",
    "24.3277": "Iec - Tc57",
    "24.3276": "Aeroscout Ltd.",
    "24.3271": "Intelligent Computer Solutions Inc.",
    "24.3270": "Ka-Ro electronics GmbH",
    "24.3273": "Ilwoo Data & Technology Co.,Ltd",
    "24.3272": "Xytronix Research & Design, Inc.",
    "20.58071045898240": "Shenzhen Bao Lai Wei Intelligent Technology Co., L",
    "24.2662790": "enblink",
    "12.123917687156736": "Canam Technology, Inc.",
    "12.346859970560": "IES Elektronikentwicklung",
    "24.4501288": "Intel Corporate",
    "20.264165329469440": "Telstra",
    "24.13428632": "My Social Stuff",
    "20.225135224160256": "NWL Inc.",
    "12.346866896896": "Unixmedia Srl",
    "24.555596": "Huawei Technologies Co.,Ltd",
    "24.13950157": "Apple, Inc.",
    "24.10487919": "Huawei Technologies Co.,Ltd",
    "12.346853003264": "Linet Oy",
    "12.346856259584": "Conformative Systems, Inc.",
    "12.346855354368": "VX Technologies Inc.",
    "24.16001107": "Zioncom Electronics (Shenzhen) Ltd.",
    "12.346858520576": "FDT Manufacturing, LLC",
    "24.12639447": "Huawei Device Co., Ltd.",
    "24.2916047": "Onzo Ltd",
    "24.8409334": "Itel Mobile Limited",
    "20.268341252980736": "Beijing Dongjin Aero-Tech Co., Ltd",
    "24.16302962": "Tsuzuki Denki",
    "12.346863882240": "Datasat Digital Entertainment",
    "24.11540934": "Apple, Inc.",
    "24.13170797": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "20.106480357343232": "HuBDIC CO.,LTD",
    "24.11291422": "Integri-Sys.Com LLC",
    "12.123917687578624": "Aplex Technology Inc.",
    "12.123917683724288": "EXASCEND (Wuhan) Co., Ltd",
    "24.3675214": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346858176512": "Guangzhou Hui Si Information Technologies Inc.",
    "20.71414664134656": "URMET Home & Building Solutions Pty Ltd",
    "24.7371752": "Huawei Technologies Co.,Ltd",
    "24.16510": "Evergreen Systems, Inc.",
    "24.10268680": "Ubiquitous Computing Technology Corporation",
    "12.123917679935488": "Hi Tech Systems Ltd",
    "12.123917675126784": "ePOINT Embedded Computing Limited",
    "24.9203846": "Gesellschaft für Sonder-EDV-Anlagen mbH",
    "24.3419332": "Compal Broadband Networks, Inc.",
    "20.71389207855104": "Shanghai Broadwan Communications Co.,Ltd",
    "24.5010341": "Kove",
    "20.31873095958528": "Cobham Slip Rings",
    "24.13122101": "Tenda Technology Co., Ltd.",
    "12.123917680865280": "Vape Rail International",
    "24.5268243": "Hon Hai Precision Ind. Co.,Ltd.",
    "20.181256783396864": "DURATECH Enterprise,LLC",
    "12.346856644608": "Micro Technic A-S",
    "20.93065817620480": "Intelligent Surveillance Corp",
    "20.154680186372096": "TTR Corporation",
    "24.12638711": "ShanDong Domor Intelligent S&T CO.,Ltd",
    "24.15994779": "Wavelink",
    "24.6359": "JAVAD GNSS, Inc.",
    "24.6358": "Swirlnet A/S",
    "24.6355": "Teamcast",
    "24.6354": "High-Gain Antennas LLC",
    "24.6357": "Reigncom",
    "24.6356": "Unified Display Interface SIG",
    "24.6351": "Baldor Electric Company",
    "24.6350": "Dreamtech Co., Ltd",
    "24.6353": "Siemens Home & Office Comm. Devices",
    "24.6352": "AtRoad, A Trimble Company",
    "24.8977685": "Lineeye Co., Ltd",
    "12.346854285312": "Windmill Innovations",
    "24.1391": "Innomedia Technologies Pvt. Ltd.",
    "12.346861371392": "Pumatronix Equipamentos Eletronicos Ltda.",
    "12.123917681291264": "Vaisala Oyj",
    "12.123917676347392": "Power Electronics Espana, S.L.",
    "24.14180215": "Telink Semiconductor (Shanghai) Co., Ltd.",
    "12.123917687271424": "APG Cash Drawer, LLC",
    "20.198237814259712": "Arnouse Digital Devices Corp",
    "24.800079": "Nokia",
    "24.10006129": "VVDN Technologies Pvt Ltd",
    "24.5025800": "Shenzhen Gwelltimes Technology Co.,Ltd",
    "24.6056754": "Texas Instruments",
    "12.123917690564608": "Wave",
    "24.7637609": "Korea Image Technology Co., Ltd",
    "12.346858807296": "LDA Audio Video Profesional",
    "24.3995464": "Shenzhen Linsn Technology Development Co.,Ltd",
    "20.5297540169728": "INTRIPLE, a.s.",
    "24.9209216": "Smart Innovation LLC",
    "24.16518987": "Texas Instruments",
    "12.123917676130304": "iREA System Industry",
    "12.346861445120": "Docutemp, Inc",
    "12.346866585600": "Acrux Technology Limited",
    "12.71297883435008": "UniSVR Global Information Technology Corp.",
    "12.71297883791360": "Comtel Electronics GmbH",
    "24.1356232": "InfiniWing, Inc.",
    "24.52148": "Shenzhen Ateko Photoelectricity Co.,Ltd",
    "12.123917688606720": "Breas Medical AB",
    "24.11014652": "ARRIS Group, Inc.",
    "24.14986621": "SCL Elements",
    "24.4727148": "China Mobile IOT Company Limited",
    "12.123917689434112": "Fath Mechatronics",
    "20.39986087854080": "Torgovyy Dom Tehnologiy Llc",
    "12.346861932544": "EL.C.A. soc. coop.",
    "12.346866872320": "Protagon Process Technologies GmbH",
    "24.6870765": "GooWi Wireless Technology Co., Limited",
    "24.9179040": "di-soric GmbH & Co. KG",
    "24.1369": "Intracom S.A.",
    "24.1368": "Synchronous, Inc.",
    "24.1365": "Japan Cash Machine Co., Ltd.",
    "24.1364": "Rangestar Wireless",
    "24.1367": "Agile TV Corporation",
    "24.1366": "360 Systems",
    "24.1361": "F & S Elektronik Systeme GmbH",
    "24.1360": "Vcomms Connect Limited",
    "24.1363": "DVC Company, Inc.",
    "24.1362": "Xycotec Computer GmbH",
    "12.71297883811840": "Tokyo Denki Gijutsu Kogyo",
    "24.12382706": "Cisco Systems, Inc",
    "24.12412": "Adid Sa",
    "12.346862235648": "Bettini srl",
    "24.222": "Cetia",
    "24.223": "Bell & Howell Pub Sys Div",
    "24.220": "Hayes Microcomputer Products",
    "24.221": "Tcl Incorporated",
    "24.226": "Acer Technologies Corp.",
    "24.227": "Integrated Micro Products Ltd",
    "24.224": "Quadram Corp.",
    "24.225": "Grid Systems",
    "24.228": "In2 Groupe Intertechnique",
    "24.229": "Sigmex Ltd.",
    "24.15232489": "Huawei Technologies Co.,Ltd",
    "24.5196": "General Meters Corp.",
    "24.5197": "Intelligent Systems",
    "24.5194": "Taiwan Thick-Film Ind. Corp.",
    "24.5195": "Hifn, Inc.",
    "24.5192": "Inventec Multimedia & Telecom Corporation",
    "24.5193": "Sichuan Changhong Electric Ltd.",
    "24.5190": "SuperVision Solutions LLC",
    "24.5191": "BOAZ Inc.",
    "24.1854832": "Intel Corporate",
    "24.14703263": "IEEE Registration Authority",
    "12.123917676756992": "SVP Broadcast Microwave S.L.",
    "24.5198": "Srisa",
    "24.5199": "Oracle Corporation",
    "12.119269851136": "Cubic Systems, Inc.",
    "24.5257263": "ASE Group Chung-Li",
    "24.14707878": "Creotech Instruments S.A.",
    "20.141268013285376": "Ningbo Plus and Popscreens electronic Technology Co.,LTD",
    "12.123917684719616": "Chromateq",
    "24.11325556": "Espressif Inc.",
    "12.346859663360": "Pelweckyj Videotechnik GmbH",
    "20.31873093861376": "sunweit industrial limited",
    "24.4103": "Cisco Systems",
    "12.123917686173696": "Ateme",
    "24.1941": "Elitegroup Computer Systems Co.,Ltd.",
    "24.1940": "Simple Devices, Inc.",
    "24.1943": "Netpower Co., Ltd.",
    "24.1942": "LSI Systems, Inc.",
    "24.1945": "Tipping Point Technologies, Inc.",
    "24.1944": "Selea SRL",
    "24.1947": "Aurora Networks",
    "24.1946": "Verint Systems Inc",
    "24.1949": "Musashi Co., Ltd.",
    "24.1948": "Golden Electronics Technology Co., Ltd.",
    "24.15519758": "MiraeRecognition Co., Ltd.",
    "24.4107": "Cisco Systems, Inc",
    "24.16028769": "NexGen Storage",
    "24.7372510": "Nastec Ltd.",
    "20.277783352705024": "Shenzhen VMAX New Energy Co., Ltd.",
    "12.123917691084800": "VirtualHere Pty. Ltd.",
    "24.9759466": "Tellescom Industria E Comercio Em Telecomunicacao",
    "12.346866958336": "Saia-Burgess Controls AG",
    "24.15482043": "Huawei Device Co., Ltd.",
    "12.123917688770560": "biosilver.co.,ltd",
    "24.1590550": "Texas Instruments",
    "24.5260791": "Liuhe Intelligence Technology Ltd.",
    "24.11558117": "Nokia Corporation",
    "20.264809156182016": "Evco Spa",
    "24.4736288": "Intel Corporate",
    "24.5004604": "Cisco Systems, Inc",
    "24.9733060": "GL Technologies (Hong Kong) Limited",
    "24.2126932": "vivo Mobile Communication Co., Ltd.",
    "24.8435236": "Ivs",
    "24.9439511": "Far-sighted mobile",
    "24.4245": "Thomson Inc.",
    "24.4244": "Performance Analysis Broadband, Spirent plc",
    "24.4247": "WinNet Metropolitan Communications Systems, Inc.",
    "24.4246": "Tracewell Systems, Inc.",
    "24.4241": "No Wires Needed Bv",
    "24.4240": "Cimetrics, Inc.",
    "24.4243": "Cms Computers, Ltd.",
    "24.4242": "Netcore Inc.",
    "12.123917688913920": "ic-automation GmbH",
    "20.181256782348288": "Wyze Labs Inc",
    "20.220327862861824": "Western Reserve Controls, Inc.",
    "24.4249": "InnoMedia, Inc.",
    "24.4248": "Starnet Technologies, Inc.",
    "24.3189437": "Shanghai Reallytek Information Technology Co.,Ltd",
    "24.910": "Atoga Systems, Inc.",
    "24.911": "Weinschel Corporation",
    "24.912": "Digital Video Communications, Inc.",
    "24.913": "Advanced Digital Broadcast, Ltd.",
    "24.914": "Hyundai Teletek Co., Ltd.",
    "24.915": "Apple, Inc.",
    "24.916": "Connect One",
    "24.917": "California Amplifier",
    "24.918": "EZ Cast Co., Ltd.",
    "24.919": "FireBrick Limited",
    "12.123917684170752": "Connoiseur Electronics Private Limited",
    "24.3738": "Boe Technology Group Co.,Ltd",
    "24.3739": "Ambit Microsystems Corporation",
    "12.346864562176": "Postjet Systems Ltd",
    "24.6593": "Alpsalpine Co,.Ltd",
    "24.6592": "ARRIS Group, Inc.",
    "24.6591": "Citiway technology Co.,ltd",
    "20.159304565915648": "Yite technology",
    "24.4742454": "Verizon",
    "12.346855788544": "TIR Systems Ltd.",
    "24.6597": "Sony Interactive Entertainment Inc.",
    "24.1371579": "2Wire Inc",
    "24.6595": "Qualitrol",
    "24.23729": "Gospell DIGITAL TECHNOLOGY CO., LTD",
    "24.7101795": "ShenZhen Rapoo Technology Co., Ltd.",
    "24.6594": "Equustek Solutions, Inc.",
    "12.123917685964800": "Sadel S.p.A.",
    "12.123917683359744": "Solvera Lynx d.d.",
    "24.7343040": "Dewav Technology Company",
    "24.14991085": "Netcore Technology Inc.",
    "24.3408010": "IEEE Registration Authority",
    "12.123917679337472": "SUS Corporation",
    "24.11310450": "Jovision Technology Co., Ltd.",
    "20.30931568033792": "Varaani Works Oy",
    "12.123917685075968": "Rch Italia Spa",
    "24.3678599": "Midea Group Co., Ltd.",
    "24.2624552": "Unigen DataStorage Corporation",
    "24.11545119": "Wingtech Group (HongKong）Limited",
    "24.7624947": "Amazon Technologies Inc.",
    "24.8178872": "Intel Corporate",
    "20.84372897136640": "Airtex Manufacturing Partnership",
    "12.71297884704768": "AHB Systeme GmbH",
    "12.123917679726592": "STEP sarl",
    "24.9739607": "Panasonic do Brasil Limitada",
    "24.6052661": "Mist Systems, Inc.",
    "24.16285978": "Overkiz",
    "12.123917691318272": "Dimastec Gestao De Ponto E Acesso Eireli-Me",
    "24.9552": "Riverbed Technology, Inc.",
    "24.9553": "SE-Elektronic GmbH",
    "24.9550": "Vertex Wireless Co., Ltd.",
    "24.9551": "ELETTROLAB Srl",
    "24.9556": "Pixel8 Networks",
    "24.9557": "Visonic Technologies 1993 Ltd.",
    "24.9554": "VXi Corporation",
    "24.9555": "ADB Broadband Italia",
    "24.9558": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9559": "BlackBerry RTS",
    "20.31477061386240": "Jabil circuit italia srl",
    "20.119709255073792": "Guilin Zhishen Information TechonlogyCO.,Ltd",
    "12.123917682028544": "DuraComm Corporation",
    "24.7299": "New Level Telecom Co., Ltd.",
    "24.7298": "Genew Technologies",
    "24.9740212": "ChengDu JiaFaAnTai Technology Co.,Ltd",
    "24.10231931": "Apple, Inc.",
    "24.7293": "Excelpoint Manufacturing Pte Ltd",
    "24.7292": "Perq Systems Corporation",
    "24.7291": "Castlenet Technology Inc.",
    "24.7290": "Perfectone Netware Company Ltd",
    "24.7297": "NextGen Venturi LTD",
    "24.7296": "New Business Division/Rhea-Information CO., LTD.",
    "24.7295": "Check Point Software Technologies",
    "24.7294": "Toshiba",
    "24.4995344": "Newtek Electronics co., Ltd.",
    "24.5831574": "Ruckus Wireless",
    "20.22333146267648": "Red Technology Limited",
    "24.8685030": "Guangdong Asano Technology CO.,Ltd.",
    "20.88914834489344": "Spii Spa",
    "24.7903171": "Dingxin Information Technology Co.,Ltd",
    "20.140782590754816": "Shenzhen Zidoo Technology Co., Ltd.",
    "24.7125": "Cisco Systems, Inc",
    "24.7124": "Cisco Systems, Inc",
    "24.7127": "Cisco SPVTG",
    "24.7126": "Kelvin Hughes Ltd",
    "24.7121": "Sogestmatic",
    "24.7120": "Identec Solutions",
    "24.7123": "Panasonic Corporation AVC Networks Company",
    "24.7122": "ULTRA-X ASIA PACIFIC Inc.",
    "24.16312245": "µTech Tecnologia LTDA",
    "24.7129": "Edgewater Wireless Systems Inc",
    "24.7128": "FLIR Systems Inc",
    "12.123917685334016": "Blake UK",
    "24.15247346": "sTraffic",
    "24.811303": "Actiontec Electronics, Inc",
    "24.64624": "Intrepid Control Systems, Inc.",
    "24.12878950": "Apple, Inc.",
    "20.137141728313344": "TGT Limited",
    "20.268902718242816": "Epitel, Inc.",
    "24.2669401": "RNET Technologies, Inc.",
    "24.8601": "Vacon Plc",
    "24.8600": "Thai Radio Co, LTD",
    "24.8603": "Dell Inc.",
    "24.8602": "Cambridge Visual Networks Ltd",
    "24.8605": "Adesys BV",
    "24.8604": "Honeywld Technology Corp.",
    "24.8607": "Satel Oy",
    "24.8606": "Sony Mobile Communications Inc",
    "24.8609": "Cisco Systems, Inc",
    "24.8608": "Cisco Systems, Inc",
    "12.123917685002240": "Dcns",
    "12.346860474368": "Genesis Automation Inc.",
    "24.14467329": "SOLiD Technologies, Inc.",
    "20.242876972793856": "Tintel Hongkong Co.Ltd",
    "24.2924284": "Canon Inc.",
    "12.123917678460928": "GLT Exports Ltd",
    "12.346861481984": "bioMérieux Italia S.p.A.",
    "24.5544722": "Samsung Electronics Co.,Ltd",
    "12.119269720064": "Donbass Soft Ltd and Co.KG",
    "24.6629": "Lynx Studio Technology, Inc.",
    "24.6860320": "Apple, Inc.",
    "24.1637535": "Changhe Electronics Co., Ltd.",
    "24.12637291": "Huawei Device Co., Ltd.",
    "12.346862960640": "VasoNova, Inc.",
    "24.6588375": "ADB Broadband Italia",
    "24.16577832": "Zyxel Communications Corporation",
    "12.346868658176": "ACIDA GmbH",
    "24.2575": "Ilryung Telesys, Inc",
    "24.2574": "Invivo Research Inc.",
    "24.2577": "ExPet Technologies, Inc",
    "24.2576": "FAST media integrations AG",
    "24.2571": "Sealevel Systems, Inc.",
    "24.2570": "SUNIX Co., Ltd.",
    "24.2573": "Amphenol",
    "24.2572": "Scientific Research Corporation",
    "20.149945503973376": "Changsha Xiangji-Haidun Technology Co., Ltd",
    "12.123917680136192": "viZaar industrial imaging AG",
    "24.2579": "Honeywell Video Systems",
    "24.2578": "Azylex Technology, Inc",
    "24.1004": "ICG Research, Inc.",
    "24.1005": "Shinkawa Electric Co., Ltd.",
    "24.1006": "MKNet Corporation",
    "24.1007": "Oneline AG",
    "24.1000": "Wavesight Limited",
    "24.1001": "Akara Canada, Inc.",
    "24.1002": "Mega System Technologies, Inc.",
    "24.1003": "Atrica",
    "24.14972380": "Arcadyan Corporation",
    "24.1008": "Redfern Broadband Networks",
    "24.1009": "Cicada Semiconductor, Inc.",
    "12.123917680545792": "Procon automatic systems GmbH",
    "12.119269793792": "Topicon",
    "24.31789": "Samsung Electronics Co.,Ltd",
    "24.4478144": "Thompson Aerospace",
    "24.12637120": "Samsung Electronics Co.,Ltd",
    "24.11041793": "Samsung Electronics Co.,Ltd",
    "24.13894218": "Delphian Systems LLC",
    "24.15007137": "Huawei Technologies Co.,Ltd",
    "12.123917683351552": "IDZ Ltd",
    "24.7665249": "Schneider Electric Fire & Security Oy",
    "24.12119653": "Liteon Technology Corporation",
    "12.71297884639232": "Techboard Srl",
    "20.268902721388544": "Ricker Lyman Robotic",
    "24.5257888": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.12641314": "Texas Instruments",
    "24.14471062": "Samsung Electronics Co.,Ltd",
    "24.15757964": "Samsung Electronics Co.,Ltd",
    "12.123917684043776": "Levelup Holding, Inc.",
    "24.6063608": "Shenzhen Kaifa Technology Co.,Ltd.",
    "24.8178010": "Dell EMC",
    "12.71297884901376": "DemoPad",
    "24.8190300": "U.I. Lapp GmbH",
    "12.346865766400": "Elbit Systems of America",
    "24.9237": "Magnetic Autocontrol GmbH",
    "24.9236": "Cisco Systems, Inc",
    "24.9235": "Cisco Systems, Inc",
    "24.9234": "Benign Technologies Co, Ltd.",
    "24.9233": "PharmaSmart LLC",
    "24.9232": "NUETEQ Technology,Inc.",
    "24.9231": "Ishii Tool & Engineering Corporation",
    "24.9230": "Inventec Besta Co., Ltd.",
    "12.346854887424": "CAD-UL GmbH",
    "24.9239": "Thomson Telecom Belgium",
    "24.9238": "Any Use",
    "12.346866864128": "Nocosium",
    "12.346859421696": "Netistix Technologies Corporation",
    "24.3716541": "E.G.O. Elektro-Ger",
    "24.15004517": "Time-O-Matic, Inc.",
    "12.123917685489664": "Atx Networks Ltd",
    "12.346868232192": "Baycom Opto-Electronics Technolgy Co., Ltd.",
    "12.123917680340992": "Ispher",
    "24.15773520": "Phantom Intelligence",
    "24.12064475": "Chant Sincere Co.,Ltd",
    "12.346866630656": "Instrumentel Limited",
    "24.16572923": "Array Networks",
    "24.8143266": "Nokia",
    "20.233258812964864": "Veea",
    "24.12507": "Mindready Solutions, Inc.",
    "24.562474": "Technicolor CH USA Inc.",
    "24.12504": "Sitek",
    "24.13917989": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.8441442": "Hewlett Packard",
    "12.123917690327040": "Intech",
    "12.346859786240": "Acisa",
    "20.84372895039488": "Voegtlin Instruments GmbH",
    "20.127937495957504": "IRS Systementwicklung GmbH",
    "12.123917689180160": "BTG Instruments AB",
    "24.11038122": "Intel Corporate",
    "24.8968730": "Redpine Signals, Inc.",
    "24.3449219": "AWare, Inc",
    "24.14438110": "Shinyei Technology Co., Ltd.",
    "24.3970830": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "24.16275825": "Intel Corporate",
    "12.346868670464": "senTec Elektronik GmbH",
    "24.3943577": "Edgecore Networks Corporation",
    "12.123917678637056": "Monnit Corporation",
    "24.2685870": "Huawei Technologies Co.,Ltd",
    "24.3943572": "杭州德澜科技有限公司（HangZhou Delan Technology Co.,Ltd）",
    "12.123917691465728": "3PS Inc",
    "24.7911638": "Zebra Technologies Inc.",
    "12.123917685481472": "Suruga Seiki Co., Ltd.",
    "12.123917680357376": "Vitrea Smart Home Technologies",
    "24.3205919": "Skylab M&C Technology Co.,Ltd",
    "12.123917677088768": "BRS Sistemas Eletrônicos",
    "24.801616": "Apple, Inc.",
    "24.14200908": "Jinan USR IOT Technology Co., Ltd.",
    "12.123917679517696": "Solace Systems Inc.",
    "24.13953152": "Cisco Systems, Inc",
    "24.11830753": "Apple, Inc.",
    "24.2389541": "Texas Instruments",
    "20.13592327880704": "Munters Europe AB",
    "24.2920875": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.12373839": "Zyxel Communications Corporation",
    "12.123917677166592": "Eastone Century Technology Co.,Ltd.",
    "20.273458018648064": "PT. Eyro Digital Teknologi",
    "24.10231094": "Huawei Technologies Co.,Ltd",
    "24.14201122": "Juniper Networks",
    "24.14987169": "Hes-So",
    "24.16547670": "CoSyst Control Systems GmbH",
    "20.163808561594368": "Beijing Leja Tech co., Ltd.",
    "24.3450810": "Fischer International Systems Corporation",
    "24.7619978": "Samsung Electronics Co.,Ltd",
    "24.4993161": "Thales Transportation Systems GmbH",
    "24.5032097": "Cisco Meraki",
    "12.346861092864": "IP Thinking A/S",
    "12.346866802688": "Hino Engineering, Inc",
    "12.123917679349760": "SinterCast",
    "24.14717429": "Netgear",
    "24.1863244": "Logistic Service & Engineering Co.,Ltd",
    "24.1111003": "Gridco Systems, Inc.",
    "20.176032232505344": "HangZhou iMagic Technology Co., Ltd",
    "24.1351606": "Guangdong Genius Technology Co., Ltd.",
    "12.346859905024": "Assembly Contracts Limited",
    "24.3427831": "LG Electronics (Mobile Communications)",
    "24.11051145": "Tactical Communications",
    "24.1068551": "Shanghai Genvision Industries Co.,Ltd",
    "12.71297883410432": "CI Systems Ltd",
    "12.346865147904": "Zeus Systems Private Limited",
    "24.13396150": "TP-Link Corporation Limited",
    "12.123917689720832": "L-3 communications ComCept Division",
    "12.71297884549120": "TDS Software Solutions Pty Ltd",
    "12.123917682659328": "Speech Technology Center Limited",
    "12.346867822592": "IDTRONIC GmbH",
    "24.7872273": "RAB Lighting",
    "24.8138253": "Blackmagic Design",
    "24.6605731": "Pointmobile",
    "24.13250751": "IEEE 802.1 Working Group",
    "24.1340433": "Rim",
    "24.6304433": "Tp-Link Technologies Co.,Ltd.",
    "24.4227670": "Continental Automotive GmbH",
    "24.8658595": "Sagemcom Broadband SAS",
    "24.16389222": "Allegion PLC",
    "24.15263861": "iS5 Communications Inc.",
    "12.346856615936": "ARVOO Imaging Products BV",
    "24.12349046": "Green Energy Options Ltd",
    "20.242213048025088": "Maxvision Technology Corp.",
    "24.8185124": "Quirky, Inc.",
    "24.5254587": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "24.15557": "WONWOO Engineering Co., Ltd",
    "24.15513712": "Beijing Heweinet Technology Co.,Ltd.",
    "12.71297884385280": "Lightstar",
    "12.123917675520000": "ENTEC Electric & Electronic CO., LTD",
    "12.346865647616": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "20.31368140554240": "Artis GmbH",
    "24.4191": "Zodiac Data Systems",
    "20.220092030779392": "Dalco Ag",
    "24.7832": "GreenLine Communications",
    "24.14984305": "Intel Corporate",
    "24.4246842": "Z3 Technology",
    "24.13387709": "Rhombus Systems",
    "24.14729911": "Masimo Corporation",
    "12.346858725376": "Stanton Technologies Sdn Bhd",
    "24.7836": "Fidustron INC",
    "20.97400600395776": "Tonnet Telecommunication International Co., Ltd.",
    "20.132811191222272": "Konecranes",
    "12.123917681922048": "Flextronics International Kft",
    "24.6604463": "AXERRA Networks Ltd",
    "12.123917680893952": "i2-electronics",
    "24.7838": "ddm hopt + schuler Gmbh + Co. KG",
    "24.16576822": "Samsung Electronics Co.,Ltd",
    "20.44213343354880": "Bluekey Pty Ltd",
    "24.5798506": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5519839": "Huawei Technologies Co.,Ltd",
    "24.12117541": "2Wire Inc",
    "24.9748038": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.12326501": "Cisco Systems, Inc",
    "24.16315492": "Rawe Electonic GmbH",
    "24.5290146": "ImTech Technologies LLC,",
    "12.346853195776": "Kimmon Manufacturing Co., Ltd.",
    "24.5805263": "Vertex Standard LMR, Inc.",
    "24.6609731": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.15729548": "AzureWave Technology Inc.",
    "24.6348814": "Shinsei Electronics Co Ltd",
    "24.583282": "Jebsee Electronics Co.,Ltd.",
    "24.7353609": "Cisco Systems, Inc",
    "24.10001": "LanPro Inc",
    "24.10000": "Intel Corporate",
    "24.10003": "Universal Global Scientific Industrial Co., Ltd.",
    "24.10002": "MaxVision LLC",
    "24.10005": "Rebound Telecom. Co., Ltd",
    "24.10004": "Grainmustards, Co,ltd.",
    "24.10007": "CE Digital(Zhenjiang)Co.,Ltd",
    "24.10006": "Adachi-Syokai Co., Ltd.",
    "24.10009": "Tp-Link Technologies Co.,Ltd.",
    "24.10008": "Suzhou NEW SEAUNION Video Technology Co.,Ltd",
    "12.346858721280": "NEO Information Systems Co., Ltd.",
    "12.123917685383168": "Blocksi Llc",
    "24.15775185": "Texas Instruments",
    "24.1623297": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917688393728": "Intecom",
    "12.123917679734784": "Litum Bilgi Teknolojileri San. Ve Tic. A.S.",
    "24.8654661": "Shanghai GMT Digital Technologies Co., Ltd",
    "12.123917686394880": "GHL Systems Berhad",
    "12.123917684465664": "Aplex Technology Inc.",
    "24.6353647": "VisionVera International Co., Ltd.",
    "20.264036229840896": "Q Core Medical Ltd",
    "24.803134": "Microsoft Corporation",
    "24.5395456": "Realtek (UpTech? also reported)",
    "24.2645807": "RNware Co.,Ltd.",
    "24.4003": "Alpha Networks Inc.",
    "24.4002": "2xWireless",
    "24.4001": "Gigabit Systems Inc.",
    "24.4000": "Canon Korea Business Solutions Inc.",
    "24.4007": "Raptor Networks Technology",
    "24.4006": "S2 Security Corporation",
    "24.4005": "BWA Technology GmbH",
    "24.4004": "Sprecher Automation GmbH",
    "24.11853853": "Conception Electronique",
    "24.4009": "PC Fabrik",
    "24.4008": "Photometrics, Inc.",
    "24.6598156": "Cisco Systems, Inc",
    "24.12087540": "Compal Information (Kunshan) Co., Ltd.",
    "12.346858164224": "Taiwan Video & Monitor",
    "12.346868441088": "Audio Power Labs",
    "12.123917690544128": "PCTEL, Inc.",
    "20.251330448850944": "Shanghai Hoping Technology Co., Ltd.",
    "12.123917680197632": "WiTagg, Inc",
    "20.31368133214208": "Corporate Systems Engineering",
    "24.10761169": "Fibrlink Communications Co.,Ltd.",
    "24.3703929": "Cisco Meraki",
    "20.154794727571456": "Shenzhen KSTAR Science and Technology Co., Ltd",
    "24.1094614": "Cisco Systems, Inc",
    "24.1094613": "Cisco Systems, Inc",
    "24.1075": "Cyberboard A/S",
    "24.3981891": "Nokia Corporation",
    "24.7092509": "Beijing ZhongHuaHun Network Information center",
    "24.1074": "Voyetra Turtle Beach, Inc.",
    "24.8969852": "Askey Computer Corp.",
    "24.10021114": "Nintendo Co.,Ltd",
    "24.2141000": "Huawei Technologies Co.,Ltd",
    "12.346853654528": "Brijing Embedor Embedded Internet Tech. Co. Ltd.",
    "24.555417": "AIM GmbH",
    "12.71297884647424": "Assembly Contracts Ltd",
    "12.346852536320": "Radiometer Medical A/S",
    "24.14724943": "Deutsche Telekom AG",
    "24.25566": "Cloudwalk Technology Co.,Ltd",
    "20.216039225819136": "PASCAL Co., Ltd.",
    "24.1079": "Powin Information Technology, Inc.",
    "24.1078": "ELANsat Technologies, Inc.",
    "12.346857848832": "Targeted Technologies, LLC",
    "24.325108": "Proxim Wireless",
    "12.123917688705024": "EREE Electronique",
    "24.13414618": "Liteon Technology Corporation",
    "12.123917682327552": "MI Inc.",
    "24.12345511": "Sony Interactive Entertainment Inc.",
    "24.15769162": "Intel Corporate",
    "24.792561": "Telecsys",
    "24.547798": "Intel Corporate",
    "24.12313": "Cisco Systems, Inc",
    "24.12312": "Jetway Information Co., Ltd.",
    "24.12311": "BlueArc UK Ltd",
    "24.12310": "Ishida Co., Ltd.",
    "24.12317": "Skystream, Inc.",
    "24.12316": "Altvater Airdata Systems",
    "24.12315": "Shuttle, Inc.",
    "24.12314": "Smartbridges Pte. Ltd.",
    "12.346866106368": "DALOG Diagnosesysteme GmbH",
    "24.12319": "Optical Networks, Inc.",
    "24.12318": "3Com Europe Ltd",
    "20.57194441605120": "Life Interface Co., Ltd.",
    "24.11318197": "Huawei Technologies Co.,Ltd",
    "24.10012856": "vivo Mobile Communication Co., Ltd.",
    "24.10514000": "Nanotek Elektronik Sistemler Ltd. Sti.",
    "12.346855899136": "TC Communications",
    "24.16049233": "Huawei Technologies Co.,Ltd",
    "12.346857840640": "Grupo Epelsa S.L.",
    "24.11306915": "Apple, Inc.",
    "24.843044": "Intel Corporate",
    "12.123917679333376": "North Star Bestech Co.,",
    "12.123917683249152": "Excel Medical Electronics LLC",
    "24.7723": "Radio Systems Design, Inc.",
    "24.7722": "Netgear",
    "24.7721": "Hypertherm Inc",
    "24.7720": "Lumexis Corporation",
    "24.7727": "DiMoto Pty Ltd",
    "24.7726": "SIRTI S.p.A.",
    "24.7725": "Stim",
    "24.7724": "CyVerse Corporation",
    "24.7729": "infomark",
    "24.7728": "Shireen Inc",
    "12.123917684301824": "Indutherm Giesstechnologie GmbH",
    "24.5303700": "Egatel Sl",
    "24.13666274": "Samsung Electronics Co.,Ltd",
    "24.23811": "Xilinx, Inc",
    "12.346860867584": "Bmti",
    "24.598": "Alpha Processor, Inc.",
    "24.599": "Microcom Corp.",
    "24.12072805": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.594": "Carrier Corporation",
    "24.595": "Televideo, Inc.",
    "24.596": "WorldGate",
    "24.597": "IBM Corp",
    "24.590": "Datacard Group",
    "24.591": "IPM Datacom S.R.L.",
    "24.592": "Geyser Networks, Inc.",
    "24.593": "Soma Networks, Inc.",
    "24.13643435": "Texas Instruments",
    "12.346863431680": "MCC Computer Company",
    "12.71297884311552": "MESA Electronic GmbH",
    "24.2889405": "IEEE Registration Authority",
    "24.11316761": "Roku, Inc",
    "24.12607768": "Tp-Link Technologies Co.,Ltd.",
    "12.123917682831360": "Quercus Technologies, S.L.",
    "24.10245556": "I.S.E. S.r.l.",
    "24.11272514": "Uriel Technologies SIA",
    "24.8666958": "Tri-Tech Manufacturing, Inc.",
    "24.8715996": "Borqs Beijing Ltd.",
    "12.123917684146176": "GDI Technology Inc",
    "24.13922114": "WAXESS USA Inc",
    "24.15250632": "Samsung Electronics Co.,Ltd",
    "12.346857615360": "NanShanBridge Co.Ltd",
    "12.123917687918592": "Innotas Elektronik GmbH",
    "24.9184517": "Huawei Technologies Co.,Ltd",
    "12.346854244352": "EDD Srl",
    "24.7913142": "Enhanced Vision",
    "24.1864717": "Zyxel Communications Corporation",
    "20.96896784793600": "pureLiFi Ltd",
    "24.13654769": "Shenzhen Pulier Tech CO.,Ltd",
    "24.5804495": "Intel Corporate",
    "24.6296755": "CyberTAN Technology Inc.",
    "24.7913149": "STULZ GmbH",
    "20.207147375788032": "Shenzhen Crave Communication Co.,ltd",
    "24.2773": "Brainchild Electronic Co., Ltd.",
    "24.2772": "CoreBell Systems Inc.",
    "24.2771": "INITECH Co., Ltd",
    "24.2770": "JEPICO Corporation",
    "24.2777": "Sony Mobile Communications Inc",
    "24.2776": "IPCserv Technology Corp.",
    "24.6094": "Screen Service Spa",
    "24.6095": "iMCA-GmbH",
    "24.2779": "Trilliant",
    "24.2778": "Vindicator Technologies",
    "24.4738004": "Technicolor CH USA Inc.",
    "24.3970": "Mortara Instrument, Inc.",
    "24.3971": "Brainium Technologies Inc.",
    "24.57545": "AutomatedLogic Corporation",
    "24.6029368": "Viasat Group S.p.A.",
    "24.57544": "Virtual Access, Ltd.",
    "24.8520": "Kaco Solar Korea",
    "24.8521": "China Daheng Group ,Inc.",
    "24.8522": "Pixel Velocity, Inc",
    "24.8523": "Shenzhen HAMP Science & Technology Co.,Ltd",
    "24.8524": "Samsung Electronics Co.,Ltd",
    "24.8525": "Guangzhou Skytone Transmission Technology Com. Ltd.",
    "24.8526": "GS Yuasa Power Supply Ltd.",
    "24.2680553": "SIS Sat Internet Services GmbH",
    "24.8528": "Eyeview Electronics",
    "24.8529": "Millinet Co., Ltd.",
    "24.3975": "Maxcess International",
    "12.346867449856": "ANA-U GmbH",
    "24.4521914": "zte corporation",
    "12.123917682909184": "Hills Health Solutions",
    "24.16262209": "Yeelink",
    "12.346855976960": "Star Link Communication Pvt. Ltd.",
    "24.5019": "ioIMAGE Ltd.",
    "24.5018": "K-ubique ID Corp.",
    "24.10496237": "HMD Global Oy",
    "24.5013": "congatec AG",
    "24.5012": "Infohand Co.,Ltd",
    "24.5011": "Panta Systems, Inc.",
    "24.5010": "Ruckus Wireless",
    "24.5017": "STAC Corporation.",
    "24.5016": "TrafficSim Co.,Ltd",
    "24.5015": "Oracle Corporation",
    "24.5014": "Acbel Polytech Inc.",
    "24.8298": "Osaka Computer Corp.",
    "24.8299": "Konica Minolta Holdings, Inc.",
    "24.12079105": "Extreme Networks, Inc.",
    "12.123917676064768": "Hanyoung Nux Co.,Ltd",
    "24.8292": "Protec Microsystems, Inc.",
    "24.8293": "Supernet Networking Inc.",
    "24.8290": "Scorpion Logic, Ltd.",
    "24.8291": "Wipro Infotech Ltd.",
    "24.8296": "Isdyne",
    "24.8297": "Isdn Systems Corporation",
    "24.8294": "General Magic, Inc.",
    "24.8295": "Node Runner Inc",
    "24.7376137": "Nokia Corporation",
    "20.74781686759424": "Bolin Technology Co., Ltd",
    "12.71297884356608": "ObjectFab GmbH",
    "20.74781674176512": "Exsom Computers LLC",
    "24.7637034": "Huawei Technologies Co.,Ltd",
    "24.4469927": "ARRIS Group, Inc.",
    "12.346857422848": "Aphex Systems Ltd.",
    "24.6846957": "SHARP Corporation",
    "24.14474954": "Akyllor",
    "24.265422": "Apple, Inc.",
    "24.11578917": "Varikorea",
    "24.10774716": "Oakley Inc.",
    "24.13918435": "Sercomm Corporation.",
    "24.4256520": "Murata Manufacturing Co., Ltd.",
    "24.2909215": "Vizio, Inc",
    "24.3440819": "Xiaomi Communications Co Ltd",
    "24.2663918": "Shenzhen SDGI CATV Co., Ltd",
    "24.3152424": "Mako Networks Ltd",
    "12.123917678219264": "Günther Spelsberg GmbH + Co. KG",
    "24.8958225": "Intel Corporate",
    "24.14164127": "Cubro Acronet GesmbH",
    "20.66219393679360": "Sensor to Image GmbH",
    "24.12886003": "Mciao Technologies, Inc.",
    "24.4255564": "Fihonest communication co.,Ltd",
    "12.346853216256": "LAUD Electronic Design AS",
    "20.203238169116672": "ZheJiang FangTai Electirc Co., Ltd",
    "24.10238639": "Samsung Electronics Co.,Ltd",
    "12.123917679616000": "SYLink Technologie",
    "20.39919498035200": "Cyber1st",
    "24.7915649": "Sony Interactive Entertainment Inc.",
    "24.4497809": "Huawei Technologies Co.,Ltd",
    "24.7134512": "Cisco Systems, Inc",
    "12.346853642240": "Nordson Corp.",
    "20.57176927240192": "uberGARD Pte. Ltd.",
    "24.1370546": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.2139853": "Hewlett Packard Enterprise",
    "20.167594817486848": "Private",
    "24.4737584": "George Robotics Limited",
    "24.4721586": "IEEE Registration Authority",
    "24.27901": "Sichuan Changhong Electric Ltd.",
    "24.11034340": "12Sided Technology, LLC",
    "24.16575929": "Arrayent",
    "24.2130538": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346865922048": "Olympus NDT Canada Inc.",
    "24.3706599": "Fanhattan LLC",
    "20.264165331566592": "Shanghai Think-Force Electronic Technology Co. Ltd",
    "12.346855968768": "Samsoft",
    "24.11300795": "TATUNG Technology Inc.",
    "24.14419317": "Siemens Energy Automation",
    "20.198611831881728": "Quantitec GmbH",
    "12.123917684592640": "Diamante Lighting Srl",
    "24.5526509": "Sony Corporation",
    "24.4464280": "Ruckus Wireless",
    "24.57717": "AK-Systems Ltd",
    "24.13407050": "Sierra Wireless",
    "24.15475517": "Huawei Technologies Co.,Ltd",
    "12.123917681602560": "Kalfire",
    "24.4464289": "Hewlett Packard",
    "12.346868510720": "Sicon srl",
    "24.16027192": "Dell Inc.",
    "24.3932160": "3Com",
    "24.3210597": "Huawei Technologies Co.,Ltd",
    "12.123917683150848": "Projects Unlimited Inc.",
    "24.12352977": "Radio Data Comms",
    "24.7929409": "Socus networks",
    "24.8712519": "Cisco Systems, Inc",
    "24.13660612": "zte corporation",
    "24.7929405": "Juniper Networks",
    "12.346857197568": "Securiton AG",
    "24.2154652": "Nokia",
    "24.7920238": "Cisco Systems, Inc",
    "20.176032229359616": "MIS Industrie Systeme GmbH & Co. KG",
    "12.123917677838336": "Foerster-Technik GmbH",
    "24.294267": "Quanta Computer Inc.",
    "12.123917689004032": "WyreStorm Technologies Ltd",
    "24.4515054": "Wistron Neweb Corporation",
    "20.27056778772480": "Eutron SPA",
    "24.9699440": "Nokia Corporation",
    "24.2638896": "GenesisTechnical Systems (UK) Ltd",
    "24.4773887": "BLANKOM Antennentechnik GmbH",
    "24.15792631": "IES GmbH & Co. KG",
    "20.45033665331200": "Honeywell Safety Products USA, Inc",
    "12.123917683658752": "Stanet Co.,Ltd",
    "20.17622846406656": "Ion Professional Solutions",
    "24.11810625": "Consert, Inc.",
    "24.4721737": "NAKAYO Inc",
    "12.346862444544": "Ronan Engineering",
    "12.123917690585088": "J.D. Koftinoff Software, Ltd.",
    "24.13635063": "Huawei Device Co., Ltd.",
    "24.10274726": "AVM GmbH",
    "12.123917682298880": "Cominfo, Inc.",
    "24.16563105": "Shenzhen Minicreate Technology Co.,Ltd",
    "24.8136": "Up-Today Industrial Co., Ltd.",
    "24.3026478": "LAA (Locally Administered Address) for Meditech Systems",
    "24.3733687": "V2Com Participacoes S.A.",
    "12.346859003904": "ANNAX Anzeigesysteme GmbH",
    "24.4224150": "aFUN TECHNOLOGY INC.",
    "24.8132": "ARRIS Group, Inc.",
    "12.123917685547008": "Vetaphone A/S",
    "24.16312439": "Harman/Becker Automotive Systems GmbH",
    "24.4779873": "Huawei Device Co., Ltd.",
    "24.13906534": "Fike Corporation",
    "24.12335577": "Z-Tec",
    "12.71297884512256": "Pleiger Elektronik GmbH and Co. KG",
    "12.123917691568128": "DIgSILENT GmbH",
    "12.123917686333440": "Farho Domotica Sl",
    "20.84372890845184": "Machine Max",
    "12.123917678768128": "Guangxi Hunter Information Industry Co.,Ltd",
    "24.1616453": "Fon Technology",
    "24.9457": "Shenzhen Fanhai Sanjiang Electronics Co., Ltd.",
    "12.346859962368": "Cadex Electronics Inc.",
    "24.9455": "Sony Mobile Communications Inc",
    "24.9454": "Wynmax Inc.",
    "24.9453": "YT Elec. Co,.Ltd.",
    "24.9452": "United Information Technology Co.,Ltd.",
    "24.9451": "ClearPath Networks, Inc.",
    "24.9450": "iris-GmbH infrared & intelligent sensors",
    "20.167152470458368": "Angora Networks",
    "24.9459": "Nintendo Co., Ltd.",
    "24.9458": "Uniphone Telecommunication Co., Ltd.",
    "24.1893931": "Apple, Inc.",
    "24.13955895": "Xunison Ltd.",
    "24.8139": "NIW Solutions",
    "24.10775594": "Cisco Systems, Inc",
    "12.346861944832": "Phytec Messtechnik GmbH",
    "12.346866798592": "EtherLight",
    "24.3677258": "SIMCom Wireless Solutions Co.,Ltd.",
    "12.123917688307712": "Ossiaco",
    "12.123917677146112": "KST technology",
    "24.3451545": "Nokia",
    "24.12098532": "ABB Power Systems Power Generation",
    "24.14990041": "360 Fly Inc.",
    "12.346859515904": "SAE IT-systems GmbH & Co. KG",
    "24.9741950": "Netgear",
    "20.83889194270720": "Electrolux Professional AB",
    "24.13896633": "Solid Semecs bv.",
    "24.15737422": "Esan electronic co.",
    "24.11593209": "Apple, Inc.",
    "12.123917682106368": "Abbott Diagnostics Technologies AS",
    "24.15226457": "Advanced-Connectek Inc.",
    "24.12098285": "OceanServer Technology, Inc",
    "24.7138394": "Hon Hai Precision Ind. CO.,Ltd.",
    "20.62663541391360": "Smart Solution Technology, Inc",
    "24.15000747": "Apple, Inc.",
    "12.123917675954176": "Grossenbacher Systeme AG",
    "24.10767377": "Cisco Systems, Inc",
    "24.10230839": "AltoBeam (China) Inc.",
    "24.10756490": "Nortel Networks",
    "12.346856091648": "Audio Processing Technology Ltd",
    "24.11566609": "Texas Instruments",
    "24.12067866": "Sweda Informatica Ltda",
    "24.11570293": "Apple, Inc.",
    "12.123917678956544": "P4Q Electronics, S.L.",
    "24.6345288": "ARRIS Group, Inc.",
    "20.106480355246080": "Zhejiang Send Intelligent Technology,Ltd",
    "24.3358": "Control Techniques",
    "24.8182108": "Google, Inc.",
    "12.123917680939008": "Bunka Shutter Co., Ltd.",
    "24.3676129": "Technicolor CH USA Inc.",
    "24.14702750": "Laerdal Medical",
    "24.9968565": "ASSA ABLOY Korea Co., Ltd iRevo",
    "20.136823636492288": "K-Vision Technology (Shanghai), Ltd",
    "24.11855938": "Hong Kong Bouffalo Lab Limited",
    "24.14972779": "Aartesys Ag",
    "24.6851253": "EDP Servicos",
    "24.2669237": "Samsung Electronics Co.,Ltd",
    "12.123917689708544": "System West dba ICS Electronics",
    "24.8710807": "Shenzhen iComm Semiconductor CO.,LTD",
    "24.3965813": "Microsoft Corporation",
    "24.6556962": "LG Electronics (Mobile Communications)",
    "20.268735388581888": "TianJin KCHT Information Technology Co., Ltd.",
    "24.8710809": "Vieworks",
    "20.180394590732288": "Dspread Technology (Beijing) Inc.",
    "24.13948187": "Huawei Technologies Co.,Ltd",
    "24.9470842": "General Electric Water & Process Technologies",
    "24.14948312": "8Bitdo Technology Hk Limited",
    "24.2673665": "Samsung Electronics Co.,Ltd",
    "12.346855501824": "Pultronics",
    "24.9459346": "Autotalks Ltd.",
    "24.7363254": "Compal Information (Kunshan) Co., Ltd.",
    "12.123917683294208": "Mannkind Corporation",
    "12.346868113408": "Visitech AS",
    "24.7380111": "Nero AG",
    "24.16280801": "Taicang T&W Electronics",
    "12.346856009728": "taskit Rechnertechnik GmbH",
    "24.14964136": "Cisco Meraki",
    "24.16532610": "Intel Corporate",
    "24.13145275": "Huawei Technologies Co.,Ltd",
    "20.167594810146816": "Shenzhen Fise Technology Holding Co.,Ltd.",
    "24.5775537": "Hewlett Packard",
    "20.83999502368768": "Instant Byte, S.L.",
    "24.10529533": "EraThink Technologies Corp.",
    "24.9762450": "Geminico co.,Ltd.",
    "24.1080270": "Nokia",
    "24.839561": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.6351190": "Ayla Networks, Inc",
    "24.11048368": "Aker Subsea Ltd",
    "12.346860613632": "Selekron Microcontrol s.l.",
    "24.4762066": "Wistron Neweb Corporation",
    "20.175931705524224": "bitbee Inc",
    "24.7910029": "Beijing Tengling Technology CO.Ltd",
    "24.14972190": "Getinge Sterilization AB",
    "12.346856816640": "Cybectec inc.",
    "24.14165181": "Cisco Systems, Inc",
    "24.9466714": "Motorola Mobility LLC, a Lenovo Company",
    "12.346859454464": "Cryogenic Control Systems, Inc.",
    "12.123917681586176": "Kzta",
    "24.5246848": "Cisco Systems, Inc",
    "24.10546038": "AirCUVE Inc.",
    "24.12120362": "Apple, Inc.",
    "24.5904639": "FiRa Consortium",
    "12.123917687803904": "Vibrationmaster",
    "12.71297883525120": "Helmholtz Zentrum Dresden Rossendorf e.V.",
    "12.346866634752": "dilitronics GmbH",
    "24.12636178": "Apple, Inc.",
    "24.1084623": "Cisco Systems, Inc",
    "24.3448372": "Evervictory Electronic Co.Ltd",
    "12.71297885122560": "Nippon Techno Lab.,Inc,",
    "12.346856620032": "Advanced Modular Computers Ltd.",
    "12.123917690707968": "NextEra Energy Resources, LLC",
    "24.10262545": "Guangzhou Sunrise Electronics Development Co., Ltd",
    "24.7901673": "Raisecom Technology CO.,LTD",
    "24.6596457": "HTC Corporation",
    "12.346861887488": "MEGWARE Computer GmbH",
    "24.10114635": "Cubix Corporation",
    "24.12373317": "Voismart",
    "24.12645606": "Huawei Technologies Co.,Ltd",
    "24.1363981": "Apple, Inc.",
    "24.551889": "Shenzhen SuperElectron Technology Co.,Ltd.",
    "24.15237723": "ARRIS Group, Inc.",
    "12.123917676060672": "Oxford Monitoring Solutions Ltd",
    "24.4242571": "itelio GmbH",
    "24.6039075": "WaveLynx Technologies Corporation",
    "24.14208300": "Wuxi Neihua Network Technology Co., Ltd",
    "20.207147373690880": "EBlink",
    "24.4981059": "eero inc.",
    "12.123917678526464": "ACS electronics srl",
    "24.3935779": "Cisco Systems, Inc",
    "20.84197041504256": "Wuhan Maiwe communication Co.,Ltd",
    "20.168210321113088": "Broadtech Technologies Co., Ltd.",
    "24.7396776": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.11586152": "Apple, Inc.",
    "24.3978557": "Hunan Goke Microelectronics Co.,Ltd",
    "12.123917677355008": "Tornado Modular Systems",
    "24.529475": "Unibrain S.A.",
    "24.9501606": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917675528192": "Abb",
    "24.13928511": "Hdpro Co., Ltd.",
    "24.13681438": "JUNGJIN Electronics Co.,Ltd",
    "12.346858151936": "Kontron (BeiJing) Technology Co.,Ltd",
    "20.220958183915520": "Valeo Interior Controls (Shenzhen) Co.,Ltd",
    "12.346856628224": "Pico Computing, Inc.",
    "24.9737254": "Apple, Inc.",
    "12.346852769792": "Analytical Spectral Devices, Inc.",
    "12.346856706048": "Gogo BA",
    "12.123917675331584": "Compusign Systems Pty Ltd",
    "24.6243": "Veritech Electronics Limited",
    "12.123917680656384": "christmann informationstechnik + medien GmbH & Co. KG",
    "24.11296396": "Utillink",
    "24.9220244": "Airtech I&C Co., Ltd",
    "12.123917682769920": "Kazan Networks Corporation",
    "24.9738402": "Shanghai LISTEN TECH.LTD",
    "24.5812478": "Team EPS GmbH",
    "12.123917679165440": "The Fire Horn, Inc.",
    "12.123917684940800": "Orion Corporation",
    "24.15206335": "Shenzhen Boomtech Industry Co.,Ltd",
    "24.3413995": "Dell Inc.",
    "24.6240": "SIM Technology Group Shanghai Simcom Ltd.,",
    "24.12624071": "Fairfield Industries",
    "24.15215172": "Liteon Technology Corporation",
    "24.1108307": "Huawei Device Co., Ltd.",
    "24.6245": "Siemens Healthcare Diagnostics Manufacturing Ltd",
    "20.4471480385536": "FORT Robotics Inc.",
    "12.123917676859392": "Elsag",
    "12.123917688762368": "Embsec AB",
    "24.9208488": "Insigma Technology Co.,Ltd",
    "24.1855925": "Techaya LTD",
    "24.12060438": "vivo Mobile Communication Co., Ltd.",
    "12.346866655232": "Electronic Applications, Inc.",
    "24.14203150": "Triple Domain Vision Co.,Ltd.",
    "24.3177650": "Anov France",
    "24.10803762": "Texas Instruments",
    "24.5011309": "Cisco Systems, Inc",
    "12.346860810240": "Phytec Messtechnik GmbH",
    "24.5011302": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "24.3979686": "Belden Deutschland GmbH",
    "24.13426906": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "12.123917684228096": "Piranha EMS Inc.",
    "24.2892427": "Ctr Srl",
    "24.16509": "Extension Technology Corp.",
    "24.16502": "Sun Conversion Technologies",
    "24.16503": "Maxton Technology Corporation",
    "24.16500": "Cable And Wireless",
    "24.16501": "Tattile SRL",
    "24.16506": "Societe D'Exploitation Du Cnit",
    "24.16507": "Scientific Atlanta",
    "24.16504": "Wearnes Automation Pte Ltd",
    "24.16505": "Juko Manufacture Company, Ltd.",
    "12.346852888576": "Beeline Networks, Inc.",
    "24.7080383": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.44256": "ARRIS Group, Inc.",
    "24.6614711": "Netcore Technology Inc",
    "24.8171582": "OSRAM GmbH",
    "12.346854064128": "Yazaki North America, Inc.",
    "24.1312736": "Abrantix AG",
    "24.10264884": "Nike, Inc.",
    "24.7868672": "Quantumsolution",
    "20.30931572228096": "Wuhan TieChi Detection Technology Co., Ltd.",
    "12.71297883512832": "Dan Electronics System (P) Limited",
    "24.12848534": "Huawei Technologies Co.,Ltd",
    "24.8933883": "Hewlett Packard",
    "20.31346174984192": "ESTec Corporation",
    "24.7907092": "Amphenol",
    "24.13918325": "Baidu Online Network Technology (Beijing) Co., Ltd",
    "24.5259": "Globo Electronic GmbH & Co. KG",
    "24.5258": "Elin Ebg Traction Gmbh",
    "20.224641221132288": "Shanghai Zenchant Electornics Co.,LTD",
    "24.5251": "eXS Inc.",
    "24.5250": "Aurora Networks",
    "24.5253": "Giga-Byte",
    "24.5252": "Cermate Technologies Inc.",
    "24.5255": "American Technology Integrators",
    "24.5254": "Echo Digital Audio Corporation",
    "24.5257": "B15402100 - Jandei, S.L.",
    "24.5256": "Akorri",
    "24.11552508": "MitraStar Technology Corp.",
    "24.5828866": "BLU Products Inc.",
    "24.3710354": "Seiko Epson Corporation",
    "12.346859814912": "Saia-Burgess Controls AG",
    "12.346859569152": "Wharton Electronics Ltd",
    "12.123917678809088": "Reckeen HDP Media sp. z o.o. sp. k.",
    "24.4241127": "Huawei Device Co., Ltd.",
    "12.346861551616": "Critical Link LLC",
    "12.346859540480": "RO.VE.R. Laboratories S.p.A",
    "24.3692213": "Peerbridge Health Inc",
    "24.8145074": "Vida Resources Lte Ltd",
    "24.13415416": "Fujitsu Isotec Limited",
    "24.9198044": "ASR Microelectronics (Shanghai) Co., Ltd.",
    "12.123917691383808": "M.A.C. Solutions (UK) Ltd",
    "24.9044144": "Eoptolink Technology Inc. Ltd,",
    "24.1103792": "Sagemcom Broadband SAS",
    "12.123917684277248": "VNG Corporation",
    "24.10246061": "Cisco Systems, Inc",
    "24.4748398": "Apple, Inc.",
    "24.12533": "Wild Lab. Ltd.",
    "24.12532": "Stardot Technologies",
    "24.12531": "At Work Computers",
    "24.12530": "Cisco Systems, Inc",
    "24.12537": "Sollae Systems Co., Ltd.",
    "24.1573421": "Huawei Technologies Co.,Ltd",
    "24.12535": "Ramix Inc.",
    "24.12534": "Securelogix Corporation",
    "12.346860384256": "Newtec A/S",
    "12.346860220416": "C2 Microsystems",
    "24.12538": "Telica, Inc.",
    "12.123917690617856": "O-Net Automation Technology (Shenzhen)Limited",
    "12.123917688905728": "Dsp Design",
    "12.123917688176640": "Diretta",
    "12.123917682790400": "Trimble",
    "24.12895747": "Cisco Systems, Inc",
    "24.5551468": "Xi'an NovaStar Tech Co., Ltd",
    "24.7367002": "Cisco Systems, Inc",
    "24.1327430": "Honeywell (China) Co., LTD",
    "20.66426707640320": "Qingdao iGuan Technology Co., Ltd.",
    "24.1360403": "Apple, Inc.",
    "12.119269396480": "2 France Marine",
    "24.12351054": "Rackspace US, Inc.",
    "20.132811189125120": "Elanview Technology Co.,Ltd",
    "24.5810299": "Fitbit, Inc.",
    "24.16519357": "Control Sistematizado S.A.",
    "12.123917685522432": "BioBusiness",
    "12.346860036096": "Phytec Messtechnik GmbH",
    "20.277958200655872": "Shenzhen Smartbyte Technology Co., Ltd.",
    "24.14723803": "Nanjing PANENG Technology Development Co.,Ltd",
    "24.3940499": "GD Midea Air-Conditioning Equipment Co.,Ltd.",
    "24.9476083": "Samsung Electronics Co.,Ltd",
    "24.1322485": "Amphitech",
    "24.12381023": "Fujian Beifeng Telecom Technology Co., Ltd.",
    "12.346858323968": "Phytec Messtechnik GmbH",
    "24.14957696": "University of Oklahoma",
    "20.66129352458240": "Siemens AG - Siemens Deutschland Mobility",
    "12.123917688356864": "Procon Electronics Pty Ltd",
    "24.4612": "u10 Networks, Inc.",
    "24.4613": "Terrasat Communications, Inc.",
    "24.4610": "Decrane Aerospace - Audio International Inc.",
    "24.4611": "ActivNetworks",
    "24.4616": "Gantner Instruments GmbH",
    "24.4617": "Fastrax Ltd",
    "24.4614": "iQuest (NZ) Ltd",
    "24.4615": "Head Strong International Limited",
    "24.4618": "Emerson Climate Technologies GmbH",
    "24.4619": "Chinasys Technologies Limited",
    "24.12383772": "Geomodeling Wuxi Technology Co. Ltd.",
    "24.15474047": "zte corporation",
    "24.927": "Cisco Systems, Inc",
    "24.13673233": "Neuberger Gebäudeautomation GmbH",
    "24.10768681": "Xag",
    "24.3440951": "Unicard Sa",
    "24.1318776": "Samsung Electronics Co.,Ltd",
    "24.5011898": "Intel Corporate",
    "24.3969": "PAL Pacific Inc.",
    "24.3188955": "Sony Mobile Communications Inc",
    "24.3965": "Xirrus",
    "24.3964": "ACTi Corporation",
    "24.3967": "UBSTORAGE Co.,Ltd.",
    "24.3966": "Ablerex Electronics Co., LTD",
    "24.3961": "Bluetooth Interest Group Inc.",
    "24.3960": "Datacap Systems Inc",
    "24.3963": "Arce Sistemas, S.A.",
    "24.3962": "BeiJing NuQX Technology CO.,LTD",
    "24.57556": "Excellent Computer",
    "24.57557": "Emulex Corporation",
    "24.57554": "Versanet Communications, Inc.",
    "24.57555": "DATENTECHNIK GmbH",
    "24.57552": "Netspeed, Inc.",
    "24.57553": "Telsis Limited",
    "24.57550": "Arn",
    "24.57551": "Integrated Device",
    "24.4486102": "Logitech",
    "12.346865700864": "Private",
    "24.10256959": "Huawei Device Co., Ltd.",
    "12.346852483072": "Industrial Electronic Engineers, Inc.",
    "24.57559": "Sunshine Electronics, Inc.",
    "24.4470841": "Cumulus Networks, inc",
    "24.16556934": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.6076983": "Microsoft Corporation",
    "24.1364382": "Apple, Inc.",
    "24.707983": "Cisco Systems Norway",
    "24.14192950": "Dover Fueling Solutions",
    "12.123917675216896": "Digital Systems Engineering",
    "12.346858557440": "Linkbit, Inc.",
    "12.346868723712": "Propagation Systems Limited",
    "24.11584391": "GOEFER, Inc.",
    "20.36069851529216": "Swarovski Optik KG",
    "12.123917691277312": "Solystic",
    "24.15789008": "Shanghai Feixun Communication Co.,Ltd.",
    "12.346857058304": "JVF Communications Ltd",
    "12.123917680525312": "Silicon Safe Ltd",
    "24.14163678": "B&W Group Ltd",
    "24.5259361": "Nokia",
    "12.346859581440": "3DX-Ray Limited",
    "24.3341": "ITSupported, LLC",
    "24.3340": "MDI Security Systems",
    "24.3343": "Finlux Ltd",
    "24.3342": "Inqnet Systems, Inc.",
    "24.3345": "DENTSPLY - Gendex",
    "24.3344": "Embedtronics Oy",
    "24.3347": "Wilhelm Rutenbeck GmbH&Co.KG",
    "24.3346": "AXELL Corporation",
    "24.3349": "Voipac s.r.o.",
    "24.3348": "Vtech Innovation LP dba Advanced American Telephones",
    "12.346855190528": "MG Industrieelektronik GmbH",
    "24.8441521": "Theissen Training Systems GmbH",
    "20.149935431352320": "DAIDONG Industrial System Co., Ltd.",
    "12.346853396480": "Megisto Systems, Inc.",
    "24.11571326": "Cisco Systems, Inc",
    "24.8878": "Mattel Inc.",
    "24.8879": "Safety Vision, LLC",
    "24.8872": "Ouman Oy",
    "24.3440220": "Sonos, Inc.",
    "24.8870": "Sony Computer Entertainment America",
    "24.8871": "Tyco Electronics AMP GmbH",
    "24.8876": "Hangzhou Siyuan Tech. Co., Ltd",
    "20.31372476416000": "Shenzhenfreelink Electronic Co.,Ltd",
    "24.8874": "Nintendo Co., Ltd.",
    "24.8875": "Shenzhen Turbosight Technology Ltd",
    "24.12075428": "Apple, Inc.",
    "24.14433873": "Solberg & Andersen AS",
    "24.3945814": "Cognitec Systems GmbH",
    "24.1330730": "Ciena Corporation",
    "24.14717267": "XAVi Technologies Corp.",
    "12.346868469760": "Guangdong East Power Co.,",
    "24.1051755": "Netgear",
    "24.4476442": "Samsung Electronics Co.,Ltd",
    "12.123917683159040": "Sicon srl",
    "20.14223577972736": "Premium Sa",
    "24.12299": "mPHASE Technologies, Inc.",
    "24.12296": "Avio Digital, Inc.",
    "24.12297": "Tachion Networks, Inc.",
    "24.12294": "Superpower Computer",
    "24.12295": "Opti, Inc.",
    "24.12292": "Leadtek Research Inc.",
    "24.12293": "Fujitsu Siemens Computers",
    "24.12290": "Expand Networks",
    "24.12291": "Phasys Ltd.",
    "24.6593042": "P2 Mobile Technologies Limited",
    "24.795497": "Honeywell SPS",
    "24.6596035": "Apple, Inc.",
    "24.2686669": "Lemobile Information Technology (Beijing) Co., Ltd.",
    "12.346866827264": "I-Evo Ltd",
    "24.9445761": "Shanghai Huaqin Telecom Technology Co.,Ltd",
    "24.6449": "Balluff GmbH",
    "24.6448": "Cisco Systems, Inc",
    "24.6445": "Nokia Corporation",
    "24.6444": "ARRIS Group, Inc.",
    "24.6447": "Cisco Systems, Inc",
    "24.6446": "Spectral Instruments, Inc.",
    "24.6441": "2M2B Montadora de Maquinas Bahia Brasil LTDA",
    "24.6440": "Siemens AG, Transportation Systems",
    "24.6443": "Aclara RF Systems Inc.",
    "20.62070511894528": "Annapurna labs",
    "24.1596943": "Intel Corporate",
    "24.1079942": "U&U Engineering Inc.",
    "24.1867345": "3bumen.com",
    "24.7036": "A & R Cambridge",
    "24.15998914": "Cisco Systems, Inc",
    "24.15226292": "SAI Technology Inc.",
    "24.7391821": "Shenzhen lnovance Technology Co.,Ltd.",
    "20.159182270496768": "Lachmann & Rink GmbH",
    "12.123917688614912": "Screen Innovations",
    "24.1628559": "Huawei Device Co., Ltd.",
    "24.15992425": "Intel Corporate",
    "24.3708054": "Samsung Electronics Co.,Ltd",
    "20.233258806673408": "bvk technology",
    "24.7370461": "OxyGuard Internation A/S",
    "24.4488132": "DualShine Technology(SZ)Co.,Ltd",
    "20.171977810640896": "Elesta GmbH",
    "24.3150922": "Wistron Neweb Corporation",
    "24.9981934": "Intel Corporate",
    "24.4479299": "zte corporation",
    "24.8656661": "GP Electronics (HK) Ltd.",
    "24.11303510": "Cisco Systems, Inc",
    "12.123917675094016": "Guardian Controls International Ltd",
    "24.2630645": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "24.15759382": "Cisco Systems, Inc",
    "24.13934016": "Samsung Electronics Co.,Ltd",
    "12.123917688262656": "PTS Technologies Pte Ltd",
    "24.540604": "zte corporation",
    "24.1050403": "IEEE Registration Authority",
    "20.83999493980160": "WELT Corporation",
    "24.2922082": "Invenit BV",
    "24.2922": "Asiarock Technology Limited",
    "24.2923": "Wistron Neweb Corporation",
    "24.2920": "Addvalue Communications Pte Ltd",
    "24.2921": "Franke Finland Oy",
    "24.2926": "Neff Instrument Corp.",
    "24.2927": "Media Streaming Networks Inc",
    "24.2924": "Sychip Inc.",
    "24.2925": "Solectron Japan Nakaniida",
    "24.2928": "Load Technology, Inc.",
    "24.2929": "Litchfield Communications Inc.",
    "12.123917677035520": "Harmonic Design GmbH",
    "24.9040": "RDC, Inc. dba LynTec",
    "24.9041": "2Wire Inc",
    "24.9042": "DATASENSOR S.p.A.",
    "12.119269412864": "DDTRONIK Dariusz Dowgiert",
    "24.9044": "ASUSTek COMPUTER INC.",
    "24.9045": "Kinco Automation(Shanghai) Ltd.",
    "24.9046": "Packet Forensics LLC",
    "24.9047": "Pitronot Technologies and Engineering P.T.E. Ltd.",
    "24.9048": "Systel Sa",
    "24.9049": "Benchmark Electronics ( Thailand ) Public Company Limited",
    "24.387": "Anite Telecoms",
    "24.386": "Dica Technologies Ag",
    "24.381": "ThermoQuest",
    "24.380": "AG-E GmbH",
    "24.383": "Experience Music Project",
    "24.382": "ADTEK System Science Co., Ltd.",
    "24.14181237": "Xiaomi Communications Co Ltd",
    "24.5938": "Science-Technical Center \"RISSA\"",
    "24.5939": "Sfr",
    "12.346854236160": "SP Controls, Inc",
    "24.5934": "FXC Inc.",
    "24.5935": "NeuLion Incorporated",
    "24.5936": "Automation Electronics",
    "24.5937": "ASUSTek COMPUTER INC.",
    "24.5930": "Proware Technology Corp.(By Unifosa)",
    "24.5931": "Global Technologies Inc.",
    "24.5932": "Taejin Infotech",
    "24.5933": "Axcen Photonics Corporation",
    "20.145211813527552": "C Tech Bilisim Teknolojileri San. Ve Tic. A.S.",
    "24.6847924": "Cisco Systems, Inc",
    "24.7132932": "Huawei Technologies Co.,Ltd",
    "20.167594812243968": "blossom communications corp.",
    "20.255861685485568": "Unicacces Groupe",
    "24.14999826": "Texas Instruments",
    "24.6595072": "Xiaomi Communications Co Ltd",
    "12.123917675253760": "Quercus Technologies, S.L.",
    "12.346855346176": "Rackmaster Systems, Inc.",
    "24.626": "CC&C Technologies, Inc.",
    "24.9200035": "ARRIS Group, Inc.",
    "12.346860249088": "Alstom (Schweiz) AG",
    "24.15530684": "Espressif Inc.",
    "24.4488123": "Shenzhen YOUHUA Technology Co., Ltd",
    "12.119269695488": "Ohio Semitronics, Inc.",
    "12.346853928960": "PESA Switching Systems, Inc.",
    "20.246679829741568": "Contemporary Amperex Technology Co., Limited",
    "24.623": "Senao International Co., Ltd.",
    "24.16566068": "Samsung Electronics Co.,Ltd",
    "24.8915205": "MTMCommunications",
    "24.8915207": "MKT Systemtechnik GmbH & Co. KG",
    "24.2743": "Cisco Systems, Inc",
    "12.123917687132160": "Dynetics, Inc.",
    "24.621": "Adept Telecom",
    "12.123917683154944": "Utrend Technology (Shanghai) Co., Ltd",
    "20.184991251169280": "Shenzhen BIO I/E Co.,Ltd",
    "12.119269584896": "EREE Electronique",
    "20.31614748852224": "Yun Yang Fire Safety Equipment Co.,Ltd.",
    "24.11298738": "Espressif Inc.",
    "12.346858348544": "Musatel",
    "20.127937505394688": "Haikou Frun Flash&Mcu Microcontrol Technology Development Co.,Ltd",
    "20.52821124710400": "Realtime biometrics India pvt ltd",
    "12.123917676306432": "Stara S/A Indústria de Implementos Agrícolas",
    "12.123917682434048": "Lode BV",
    "24.8319": "Kyoei Sangyo Co., Ltd.",
    "24.8318": "Finecom Co., Ltd.",
    "24.8317": "Advanced Computer Applications",
    "24.8316": "Autec Gmbh",
    "24.8315": "Intel Corporation",
    "24.8314": "WiSE Communications, Inc.",
    "24.8313": "Mikron Gmbh",
    "24.8312": "Runtop, Inc.",
    "24.8311": "Kardios Systems Corp.",
    "24.8310": "Reudo Corporation",
    "24.16309128": "Dell Inc.",
    "24.16561382": "Askey Computer Corp",
    "24.10792376": "Vitec Group, Camera Dynamics Ltd",
    "24.12887042": "Huawei Technologies Co.,Ltd",
    "24.11337221": "Itel Mobile Limited",
    "24.2121316": "Huawei Device Co., Ltd.",
    "12.123917678604288": "BuddyGuard GmbH",
    "12.346868531200": "R&D Kometeh",
    "12.123917689081856": "ACD Elekronik GmbH",
    "12.346865692672": "Echola Systems",
    "12.346855342080": "FiveCo",
    "12.123917689102336": "OnYield Inc Ltd",
    "24.3162484": "Altec Lansing Llc",
    "24.563911": "zte corporation",
    "12.123917677092864": "MobiPromo",
    "24.8710534": "Huawei Device Co., Ltd.",
    "12.346865418240": "Vox Technologies",
    "12.123917678022656": "hiSky S.C.S LTD",
    "12.123917687222272": "Digital Yacht Ltd",
    "12.346863558656": "Neoptix Inc.",
    "24.14993414": "Ceiec Electric Technology Inc.",
    "24.1594639": "Cisco SPVTG",
    "24.5280441": "Samsung Electronics Co.,Ltd",
    "24.2424146": "Liteon Technology Corporation",
    "12.123917684494336": "Modtronix Engineering",
    "24.269924": "Apple, Inc.",
    "24.2415597": "Samsung Electronics Co.,Ltd",
    "12.346853253120": "Quinx AG",
    "12.346866008064": "GAON TECH corp.",
    "24.7344420": "Apple, Inc.",
    "12.123917678845952": "Rcatsone",
    "20.159182262108160": "Innovative Electronic Technology",
    "12.123917691527168": "Hunter Liberty Corporation",
    "24.569046": "Samsung Electronics Co.,Ltd",
    "12.123917684359168": "Rudy Tellert",
    "12.346857992192": "XoIP Systems Pty Ltd",
    "24.12853704": "Kyocera Corporation",
    "12.123917675233280": "DONG IL VISION Co., Ltd.",
    "24.3996371": "IEEE Registration Authority",
    "24.8684276": "Beijing Huasun Unicreate Technology Co., Ltd",
    "24.6353537": "Tranwo Technology Co., Ltd.",
    "24.41016": "Email Electronics",
    "12.123917677580288": "Internet Protocolo Logica Sl",
    "20.123598314405888": "Foxconn Brasil Industria e Comercio Ltda",
    "12.346858995712": "REASON Tecnologia S.A.",
    "24.15473241": "Belkin International Inc.",
    "24.13908740": "Intel Corporate",
    "20.67047992066048": "Shenzhen zhong ju Fiber optical Co.Ltd",
    "24.1349070": "Apple, Inc.",
    "24.538429": "Samsung Electronics Co.,Ltd",
    "12.123917680496640": "Accel Corp",
    "24.5306349": "Huawei Device Co., Ltd.",
    "24.8953711": "Samsung Electronics Co.,Ltd",
    "24.8940406": "China Hopeful Group Hopeful Electric Co.,Ltd",
    "12.346858192896": "Pyott-Boone Electronics",
    "24.8931864": "Opulinks",
    "12.123917682487296": "LG Electronics",
    "12.123917677748224": "B2cloud lda",
    "24.5006913": "Samsung Electro-Mechanics(Thailand)",
    "24.5274525": "LCFC(HeFei) Electronics Technology co., ltd",
    "24.9862": "Quantenna Communcations, Inc.",
    "24.9863": "corega K.K",
    "24.9860": "Kisan System",
    "24.9861": "Digital Innovation",
    "24.9866": "Terrier SC Ltd",
    "24.9867": "Guangzhou Escene Computer Technology Limited",
    "24.9864": "Juniper Networks",
    "24.9865": "General Dynamics Robotic Systems",
    "24.9868": "StarLeaf Ltd.",
    "24.9869": "CellTel S.p.A.",
    "12.346866413568": "ERS electronic GmbH",
    "24.6346": "Viprinet GmbH",
    "20.123731214073856": "Bitfinder Inc",
    "20.39919500132352": "Shenzhen AWT science & technology limited",
    "12.123917685293056": "RJ45 Technologies",
    "24.8403195": "Intel Corporate",
    "24.10231445": "Actiontec Electronics, Inc",
    "12.123917686951936": "Rotem Industry LTD",
    "12.346861740032": "Metrum Sweden AB",
    "24.8403197": "LeapFrog Enterprises, Inc.",
    "24.10754165": "Cisco Systems, Inc",
    "24.11573407": "Lg Cns",
    "20.180680394801152": "SOS LAB Co., Ltd.",
    "24.9079": "Isotek Electronics Ltd",
    "24.9078": "HTC Corporation",
    "20.114975477596160": "solvimus GmbH",
    "24.12354471": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.5269383": "Planet Networks",
    "12.346865012736": "COMPRION GmbH",
    "12.123917688508416": "EA Elektroautomatik GmbH & Co. KG",
    "12.346859147264": "Armida Technologies Corporation",
    "24.117": "Bell Northern Research (BNR)",
    "24.37088": "Systran Corp.",
    "24.9076": "ARRIS Group, Inc.",
    "24.6856766": "Huawei Technologies Co.,Ltd",
    "24.7351761": "Newings Communication CO., LTD.",
    "24.1599691": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.2666": "SVM Microwaves s.r.o.",
    "12.346863063040": "Mark-O-Print GmbH",
    "12.346865098752": "Spirent Communications",
    "24.15006445": "Samsung Electronics Co.,Ltd",
    "20.255553893826560": "ASSA ABLOY(GuangZhou) Smart Technology Co., Ltd",
    "24.3975210": "Hewlett Packard",
    "12.346863554560": "GD Mission Systems",
    "24.14161119": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "24.5909": "Qstik",
    "20.272805453103104": "Xperio Labs Limited",
    "24.7397885": "Flextronics",
    "24.2661": "GentechMedia.co.,ltd.",
    "12.346867458048": "Proyecson S.A.",
    "12.346863370240": "FRAKO Kondensatoren- und Anlagenbau GmbH",
    "12.123917679923200": "Lace Llc.",
    "24.8662567": "Samsung Electronics Co.,Ltd",
    "12.123917686411264": "DimoCore Corporation",
    "24.20734": "PCTVnet ASA",
    "24.20735": "Hakko Electronics Co., Ltd.",
    "24.2660": "Eracom Technologies",
    "12.346852880384": "Latus Lightworks, Inc.",
    "12.346856214528": "ELMIC GmbH",
    "24.1682": "Intruvert Networks, Inc.",
    "12.119269871616": "Navitar Inc",
    "24.2157255": "Shenzhen Riopine Electronics Co., Ltd",
    "24.7312": "Empacket Corporation",
    "24.11280018": "Samsung Electronics Co.,LTD",
    "24.1680": "Euracom Communication GmbH",
    "24.27503": "Huawei Technologies Co.,Ltd",
    "24.32944": "Advanced Information",
    "24.32945": "Softcom A/S",
    "24.32946": "NET (Network Equipment Technologies)",
    "24.32947": "Aval Data Corporation",
    "24.32940": "Imlogix, Division Of Genesys",
    "24.32941": "Cnet Technology, Inc.",
    "24.32942": "Hughes Network Systems",
    "24.32943": "Allumer Co., Ltd.",
    "24.1686": "Advent Networks",
    "24.7653795": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.346863214592": "Audio Engineering Ltd.",
    "24.32948": "Sophia Systems",
    "24.32949": "United Networks Inc.",
    "24.1687": "R & D Center",
    "24.5907": "Tiger NetCom",
    "24.3179945": "ASUSTek COMPUTER INC.",
    "24.2682005": "Apple, Inc.",
    "24.1685": "Ensure Technologies, Inc.",
    "24.5799863": "Sonar Industrial Co., Ltd.",
    "24.4194528": "Derek(Shaoguan)Limited",
    "12.119269924864": "Senceive Ltd",
    "24.5825167": "Apple, Inc.",
    "24.14975364": "Nokia",
    "24.1885997": "Shenzhen Huapu Digital CO.,Ltd",
    "12.123917676630016": "Foro Tel",
    "24.560584": "Ryowa Electronics Co.,Ltd",
    "24.2099725": "IEEE Registration Authority",
    "24.15485331": "Qihan Technology Co., Ltd",
    "24.15771492": "GST Co., Ltd.",
    "24.3160149": "Nokia Corporation",
    "12.346861391872": "Cambridge Sound Management, LLC",
    "24.15257094": "SkyHawke Technologies, LLC.",
    "20.273458017599488": "LifePrint Products, Inc.",
    "12.346863751168": "Japan Electronics System, Inc",
    "24.14165689": "Guangdong Coagent Electronics S&amp;T Co.,Ltd.",
    "12.123917686784000": "Dorlet Sau",
    "12.346854109184": "Applied Systems Engineering, Inc.",
    "24.14734686": "Giga-Byte Technology Co.,Ltd.",
    "24.6610704": "JingLue Semiconductor(SH) Ltd.",
    "12.119269834752": "reelyActive",
    "24.2640979": "Intune Networks",
    "12.123917677228032": "Chronomedia",
    "24.14981334": "Apple, Inc.",
    "24.7366229": "AzureWave Technology Inc.",
    "12.71297885138944": "Chengdu Meihuan Technology Co., Ltd",
    "20.233630192369664": "Tendzone International Pte Ltd",
    "12.123917684731904": "Checkbill Co,Ltd.",
    "24.12855581": "Klemsan Elektrik Elektronik San.Ve Tic.As.",
    "24.16572889": "Stable Imaging Solutions LLC",
    "12.123917679099904": "COSMOS web Co., Ltd.",
    "24.9962400": "ABB n.v. Power Quality Products",
    "24.8449573": "Shenzhen Skyworth Digital Technology CO., Ltd",
    "24.9709614": "New H3C Technologies Co., Ltd",
    "24.2371972": "SHARP Corporation",
    "24.3730445": "Apple, Inc.",
    "24.9210362": "Zhejiang Hechuan Technology Co., Ltd.",
    "24.10261476": "Private",
    "24.3730449": "Novatek Microelectronics Corp.",
    "12.123917679198208": "Vtron Pty Ltd",
    "12.123917688995840": "Idyllic Engineering Pte Ltd",
    "12.123917686530048": "Mesotechnic",
    "24.7864478": "Samsung Electronics Co.,Ltd",
    "24.8413535": "Beijing Sinead Technology Co., Ltd.",
    "12.123917675147264": "Cambridge Pixel",
    "24.16526258": "Actiontec Electronics, Inc",
    "24.2111191": "Shanghai Reacheng Communication Technology Co.,Ltd",
    "24.15522085": "Rami",
    "12.123917675986944": "Avionica",
    "24.4990104": "Texas Instruments",
    "20.194363557675008": "Private",
    "24.12122152": "Changshu Gaoshida Optoelectronic Technology Co. Ltd.",
    "12.346858815488": "OpenPKG GmbH",
    "24.4237775": "Amazon Technologies Inc.",
    "12.346866683904": "Metraware",
    "12.346858565632": "NCI Technologies Inc.",
    "24.5259160": "Cmitech",
    "12.346859884544": "Fluid Analytics, Inc.",
    "12.346864676864": "Sociedad Iberica De Construcciones Electricas, S.A. (Sice)",
    "12.346856890368": "Priamus System Technologies Ag",
    "24.14217930": "Intel Corporate",
    "24.5258694": "Xiaomi Communications Co Ltd",
    "20.203238156533760": "IPM Sales and service Co.,Ltd.",
    "12.346856312832": "Phaedrus Limited",
    "24.3417018": "Samsung Electro-Mechanics(Thailand)",
    "12.123917685104640": "O-Net Communications(Shenzhen)Limited",
    "24.2912597": "HiveMotion",
    "12.346868449280": "RTC Manufacturing Inc.",
    "12.123917681270784": "Tex Computer Srl",
    "12.346855747584": "Minerva Technology Inc",
    "24.4506163": "Guangzhou Comet Technology Development Co.Ltd",
    "12.346868645888": "Techno Co.,Ltd.",
    "24.9966387": "zte corporation",
    "20.31346173935616": "Shanghai Xiaoyan Technology Co., Ltd.",
    "24.8391296": "IEEE Registration Authority",
    "20.79215079718912": "Popit Oy",
    "24.4213130": "Synapticon GmbH",
    "24.41179": "Fisher & Paykel Production",
    "24.41178": "INTEGRATED SYSTEMS Technology, Inc.",
    "24.41177": "Convex Computer Corporation",
    "24.41176": "Spectra - Tek",
    "24.41175": "Kasten Chase Applied Research",
    "24.41174": "SBE, Inc.",
    "24.41173": "Sierra Wireless",
    "24.41172": "Radiolan, Inc.",
    "24.41171": "Instem Computer Systems, Ltd.",
    "24.41170": "Allied Telesyn",
    "24.13418073": "Calix Inc.",
    "24.16042725": "Freebox Sas",
    "12.346860675072": "Channelot Ltd.",
    "24.8974385": "Cisco Systems, Inc",
    "24.12356259": "NHN Entertainment",
    "20.149935435546624": "Hi-Profile Achievement (M) Sdn Bhd",
    "24.584873": "Samsung Electronics Co.,Ltd",
    "12.346867175424": "Motec Pty Ltd",
    "24.16276425": "M-Cube Spa",
    "24.9207475": "Lytro, Inc.",
    "20.224641232666624": "Buttons (Beijing) Technology Limited",
    "24.4461520": "zte corporation",
    "12.123917676412928": "EAX Labs s.r.o.",
    "12.346860146688": "MiraTrek",
    "24.15266824": "Intel Corporate",
    "20.149935443935232": "Unitac Technology Limited",
    "24.13158289": "Zero1.tv GmbH",
    "12.123917676748800": "Echo Ridge, LLC",
    "12.123917690593280": "Agrident GmbH",
    "24.12632512": "Cisco-Linksys, LLC",
    "24.5506399": "Alcatel Lucent",
    "24.2110715": "Xiaomi Communications Co Ltd",
    "24.10501169": "Shenzhen Belon Technology CO.,LTD",
    "24.16046534": "Unionman Technology Co.,Ltd",
    "12.346855116800": "Miritek, Inc.",
    "12.71297884184576": "Private",
    "24.11309048": "Nokia",
    "12.346865053696": "R.V.R. elettronica s.p.a.",
    "24.45081": "Utc Ccs",
    "24.45086": "Rantic Labs, Inc.",
    "20.58071053238272": "Vtrek Group International Ltd.",
    "24.45084": "Westport Technologies",
    "24.10004400": "Mcst Zao",
    "24.11544527": "Extreme Networks, Inc.",
    "24.2944550": "Petroleum Geo-Services",
    "12.123917679153152": "Dolby Laboratories, Inc.",
    "24.3711628": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "20.48476182282240": "Shenzhen Haiying Wire Tech Co., Ltd.",
    "24.13925173": "NEO Monitors AS",
    "24.2678828": "Apple, Inc.",
    "24.14695515": "Shenzhen Jiaxinjie Electron Co.,Ltd",
    "12.346860138496": "Schniewindt GmbH & Co. KG",
    "24.45281": "Cisco Systems, Inc",
    "12.123917678870528": "Millennial Net, Inc.",
    "24.9984747": "Apple, Inc.",
    "20.44768539181056": "Ray Pte Ltd",
    "20.22126504443904": "Wisnetworks Technologies Co., Ltd.",
    "24.7914667": "Shenzhen Runsil Technology Co.,Ltd",
    "12.123917681328128": "Eiit Sa",
    "24.287712": "Nome Oy",
    "20.13692010758144": "Dinkle Enterprise Co., Ltd.",
    "24.8140973": "Samsung Electronics Co.,Ltd",
    "20.194284242337792": "Innotas Elektronik GmbH",
    "24.13955252": "Napco Security Technologies",
    "12.123917677985792": "Unisight Digital Products",
    "24.9498472": "Zyxel Communications Corporation",
    "24.12611593": "Huawei Technologies Co.,Ltd",
    "24.16538406": "MikroBits",
    "20.145383546159104": "Emotiq s.r.l.",
    "24.8141088": "Ssoma Security",
    "12.123917683900416": "Nautel Limited",
    "12.346863710208": "Spider Tecnologia Ind. e Com. Ltda.",
    "24.6939": "Siemens AG,",
    "24.6938": "e-trees Japan, Inc.",
    "24.6931": "Icron Technologies Corporation",
    "24.6930": "Apprion",
    "24.6933": "Voxtel, Inc.",
    "24.6932": "Carex Lighting Equipment Factory",
    "24.6935": "Palo Alto Networks",
    "24.6934": "Celtro Ltd.",
    "24.6937": "IEEE I&M Society TC9",
    "24.6936": "Tsuken Electric Ind. Co.,Ltd",
    "20.159304574304256": "Huddly AS",
    "12.123917682958336": "GAMPT mbH",
    "24.12886886": "zte corporation",
    "12.123917690449920": "Beijing Wing ICT Technology Co., Ltd.",
    "12.123917682991104": "CW2. Gmbh & Co. KG",
    "20.229206570041344": "TAPKO Technologies GmbH",
    "12.346854150144": "Dorlet Sau",
    "12.346859204608": "System LSI CO.Ltd.",
    "12.346854645760": "MITE Hradec Kralove, s.r.o.",
    "24.5023309": "Cisco Systems, Inc",
    "24.11010432": "IMAGO Technologies GmbH",
    "24.5036484": "Magneti Marelli Automotive Electronics (Guangzhou) Co. Ltd",
    "24.13942": "ARRIS Group, Inc.",
    "24.14187741": "Sanix Incorporated",
    "12.123917680406528": "Procon Electronics Pty Ltd",
    "12.346864230400": "TANABIKI Inc.",
    "24.4194758": "3Com Europe Ltd",
    "24.15759712": "Apple, Inc.",
    "12.123917688836096": "Solace Systems Inc.",
    "24.4817": "Texas Instruments",
    "12.123917686759424": "Cetitec GmbH",
    "24.2410606": "Chaney Instrument Co.",
    "12.123917682122752": "Lineage Power Pvt Ltd.,",
    "24.6596": "Infocrypt Inc.",
    "24.9437818": "Shenzhen Sworix Techonlogy Co., Ltd",
    "24.7910350": "Elo touch solutions",
    "12.346865127424": "SweMet AB",
    "24.10015719": "Netafim L",
    "24.10797921": "Apple, Inc.",
    "24.1595699": "Cisco SPVTG",
    "20.132869108269056": "Kverneland Group Mechatronics",
    "12.346857664512": "Ellips B.V.",
    "12.123917688684544": "Connor Winfield LTD",
    "24.3180868": "DEVA Broadcast Ltd.",
    "24.8305": "Ibr Gmbh",
    "24.14734106": "EQUES Technology Co., Limited",
    "24.11800379": "Tenda Technology Co.,Ltd.Dongguan branch",
    "24.5772": "DSL Partner AS",
    "24.5773": "KORWIN CO., Ltd.",
    "24.5770": "id-Confirm Inc",
    "24.5771": "Paralan Corporation",
    "24.5776": "J-Tek Incorporation",
    "24.5777": "Moser-Baer AG",
    "24.5774": "Vimicro corporation",
    "24.5775": "GN Netcom A/S",
    "12.346863984640": "Mobileye",
    "24.5778": "Scientific-Atlanta, Inc.",
    "24.5779": "PowerLink Technology Inc.",
    "24.13941039": "Technicolor CH USA Inc.",
    "20.84197038358528": "Advantics",
    "12.123917691592704": "Dalian Levear Electric Co., Ltd",
    "24.5804950": "Ruckus Wireless",
    "24.5769466": "Fiber Optic & telecommunication INC.",
    "24.16041031": "Coagent International Enterprise Limited",
    "12.123917684498432": "Maytronics Ltd.",
    "24.13670139": "DASAN Network Solutions",
    "12.346852851712": "Kaveri Networks",
    "24.1098330": "Technicolor CH USA Inc.",
    "24.6586330": "TEKTELIC Communications Inc.",
    "12.123917679476736": "AMS Controls, Inc.",
    "24.11063525": "Juniper Networks",
    "12.123917675573248": "Peter Huber Kaeltemaschinenbau AG",
    "24.320840": "Cohda Wireless Pty Ltd",
    "24.4797": "Avantec Manufacturing Limited",
    "24.4796": "Echolab LLC",
    "24.4795": "Telecommunications Industry Association TR-41 Committee",
    "24.4794": "FSI Systems, Inc.",
    "12.346856607744": "Ann Arbor Sensor Systems LLC",
    "24.4792": "G2 Microsystems",
    "24.4791": "PTW Freiburg",
    "24.4790": "Santa Barbara Infrared, Inc.",
    "24.7908192": "ABB Australia",
    "24.4799": "Arcadyan Technology Corporation",
    "24.4798": "Astek Corporation",
    "24.5021412": "Hanyang Navicom Co.,Ltd.",
    "12.123917678157824": "DogWatch Inc",
    "12.123917680889856": "servicios de consultoria independiente S.L.",
    "12.123917686566912": "Arrowvale Electronics",
    "20.31477531148288": "Wisnu and Supak Co.,Ltd.",
    "24.2367687": "Sagemcom Broadband SAS",
    "24.9979086": "Huawei Technologies Co.,Ltd",
    "12.346853605376": "TTI - Telecom International Ltd.",
    "24.4421": "ValuePoint Networks",
    "24.4420": "Assurance Technology Corp",
    "24.4423": "Secom-Industry co.LTD.",
    "24.4422": "Telecard-Pribor Ltd",
    "24.4425": "Proliphix Inc.",
    "24.4424": "Prolon Control Systems",
    "24.4427": "Francotyp-Postalia GmbH",
    "24.4426": "KAYABA INDUSTRY Co,.Ltd.",
    "24.4429": "Atsumi Electric Co.,LTD.",
    "24.4428": "caffeina applied research ltd.",
    "24.295342": "Clack Corporation",
    "24.16569817": "Shenzhen SDMC Technology Co., Ltd.",
    "24.3690480": "AzureWave Technology Inc.",
    "24.2409519": "Philips Lifeline",
    "24.998": "Entone, Inc.",
    "24.999": "Logostek Co. Ltd.",
    "24.990": "OTC Wireless",
    "24.991": "Desana Systems",
    "24.992": "ARRIS Group, Inc.",
    "24.993": "Winmate Communication, Inc.",
    "24.994": "Comspace Corporation",
    "24.995": "Cisco Systems, Inc",
    "24.996": "Cisco Systems, Inc",
    "24.997": "Hermstedt SG",
    "24.12378269": "Eoslink",
    "12.346852466688": "Telecom Analysis Systems, LP",
    "24.6833521": "Huawei Device Co., Ltd.",
    "24.9219803": "Nag Llc",
    "24.7213": "FlexRadio Systems",
    "24.7212": "Synapse",
    "24.7211": "Alertme.com Limited",
    "24.7210": "Envisacor Technologies Inc.",
    "24.7217": "Mobile XP Technology Co., LTD",
    "24.7216": "Mode Lighting (UK ) Ltd.",
    "24.7215": "Pfister GmbH",
    "24.7214": "HPN Supply Chain",
    "24.7219": "Sutron",
    "24.7218": "Telian Corporation",
    "12.346865795072": "Bodensee Gravitymeter Geosystem GmbH",
    "12.123917683863552": "Vishay Nobel AB",
    "24.820172": "Alpha Networks Inc.",
    "12.123917680013312": "SOLVERIS sp. z o.o.",
    "20.150399882362880": "Richbeam (Beijing) Technology Co., Ltd.",
    "24.4856091": "ZigBee Alliance",
    "12.123917682663424": "PushCorp, Inc.",
    "24.11975": "Huawei Technologies Co.,Ltd",
    "24.12870651": "Neatframe AS",
    "12.346855845888": "ICS Electronics",
    "20.238070235398144": "ComNav Technology Ltd.",
    "24.49308": "Hioki E.E. Corporation",
    "24.49309": "Distributed Systems Int'L, Inc",
    "24.49304": "Chuntex Electronic Co., Ltd.",
    "24.49305": "Yoshiki Industrial Co.,Ltd.",
    "24.49306": "Photonics Corporation",
    "24.49307": "Reliance Comm/Tec, R-Tec Systems Inc",
    "24.49300": "Vmx Inc.",
    "24.49301": "Znyx (Network Appliance); Jupiter Systems (MX-700); Apple (G3)  all seen",
    "24.49302": "Tamura Corporation",
    "24.49303": "Archipel Sa",
    "12.71297883979776": "Inspired Systems",
    "24.8939": "Data Respons A/S",
    "24.8938": "Rustelcom Inc.",
    "24.8937": "ProVision Communications",
    "24.8936": "Applition Co., Ltd.",
    "24.8935": "WPS Parking Systems",
    "24.8934": "Intelligent Data",
    "24.8933": "Fisher-Rosemount Systems Inc.",
    "24.8932": "Apass Technology Co., Ltd.",
    "24.8931": "Amerigon",
    "24.8930": "WABTEC Transit Division",
    "24.12910524": "IEEE Registration Authority",
    "20.79602630262784": "SmartDisplayer Technology Co., Ltd.",
    "24.13921190": "Huawei Technologies Co.,Ltd",
    "12.123917680021504": "Elsist Srl",
    "24.1836471": "Chongqing Trantor Technology Co., Ltd.",
    "24.15475103": "VidaBox LLC",
    "24.15735660": "vivo Mobile Communication Co., Ltd.",
    "24.2902069": "IEEE Registration Authority",
    "24.16291667": "Huawei Device Co., Ltd.",
    "12.346861219840": "RTW GmbH & Co.KG",
    "24.12326549": "zte corporation",
    "20.31368105951232": "Claber Spa",
    "12.346863050752": "Grossenbacher Systeme AG",
    "12.123917686222848": "Micas Ag",
    "20.66256279437312": "Grandway Technology (Shenzhen) Limited",
    "24.8689": "Tutus Data AB",
    "24.8688": "EW3 Technologies LLC",
    "12.346853773312": "CC Systems AB",
    "24.8681": "Apple, Inc.",
    "24.8680": "Murata Manufacturing Co., Ltd.",
    "24.8683": "Esp Systems, Llc",
    "24.8682": "Bystronic Laser AG",
    "24.8685": "Telegesis",
    "24.8684": "Solutronic GmbH",
    "24.8687": "Kapsys",
    "24.8686": "Full Spectrum Inc.",
    "24.2063": "Proximion Fiber Optics AB",
    "24.2062": "ARRIS Group, Inc.",
    "24.2061": "Toshiba",
    "24.2060": "VDA Group S.p.a.",
    "24.2067": "Diskbank, Inc.",
    "24.2066": "GM-2 Corporation",
    "24.2065": "VOIX Corporation",
    "24.2064": "Key Technology, Inc.",
    "24.2069": "CATS Co., Ltd.",
    "24.2068": "TIL Technologies",
    "24.4222985": "ARRIS Group, Inc.",
    "12.123917690245120": "JSC Kaluga Astral",
    "12.123917687087104": "Hilo",
    "24.2927246": "Wyze Labs Inc",
    "12.346860855296": "Keystone Electronic Solutions",
    "12.346866888704": "Seartech",
    "24.7997": "Qbit GmbH",
    "20.61822326538240": "Colooc AB",
    "24.15258848": "Beijing BenyWave Technology Co., Ltd.",
    "24.2686033": "The Electric Controller and Manufacturing Co., LLC",
    "20.75685692440576": "Shenzhen Nation RFID Technology Co.,Ltd.",
    "24.16551550": "ARRIS Group, Inc.",
    "12.71297883959296": "Shenzhen SETEC Power Co.,Ltd",
    "24.3721879": "Contour Design LLC",
    "24.3972038": "Huawei Device Co., Ltd.",
    "12.346856181760": "Fluke Biomedical, Radiation Management Services",
    "20.176726070001664": "Filo SRL",
    "24.16523200": "Eurecam",
    "24.16001174": "Specto Paineis Eletronicos Ltda",
    "24.11826006": "Hangzhou Treebear Networking Co., Ltd.",
    "12.123917681524736": "Parkalot Denmark ApS",
    "12.123917678501888": "BRS Sistemas Eletrônicos",
    "24.7923894": "zte corporation",
    "24.5537774": "Cisco Systems, Inc",
    "20.176772030136320": "KNS Group LLC (YADRO Company)",
    "24.6344873": "Samsung Electronics Co.,Ltd",
    "24.1374355": "Wuhan FiberHome Digital Technology Co.,Ltd.",
    "24.14426440": "Intel Corporate",
    "12.346857930752": "Buyang Electronics Industrial co.,Ltd.",
    "24.2687": "Teradon Industries, Inc",
    "24.2686": "The Advantage Group",
    "24.2685": "Valo, Inc.",
    "24.2684": "Tecton Ltd",
    "24.2683": "Cornelius Consult",
    "24.2682": "Kyoritsu Electric Co., Ltd.",
    "24.2681": "corega K.K",
    "24.2680": "Olitec",
    "12.123917684355072": "Procon Electronics Pty Ltd",
    "24.1591101": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.2689": "TEIMA Audiotex S.L.",
    "24.2688": "Telkonet Inc.",
    "24.460": "Japan Total Design Communication Co., Ltd.",
    "24.461": "ARtem",
    "24.462": "Custom Micro Products, Ltd.",
    "24.463": "Alpha Data Parallel Systems, Ltd.",
    "24.464": "VitalPoint, Inc.",
    "24.6199": "Universal ABIT Co., Ltd.",
    "24.466": "inXtron, Inc.",
    "24.467": "PAXCOMM, Inc.",
    "24.468": "Leisure Time, Inc.",
    "24.469": "Haedong Info & Comm Co., Ltd",
    "24.24737": "VPNet, Inc.",
    "12.123917690933248": "The-Box Development",
    "12.123917682413568": "X-Laser LLC",
    "24.6197": "Thoratec / ITC",
    "24.6196": "Texas Instruments",
    "24.14439485": "Itel Mobile Limited",
    "12.123917679783936": "Seiwa Giken",
    "24.9762518": "Apple, Inc.",
    "24.7929802": "Tecno Mobile Limited",
    "24.1610284": "Virtual Instruments",
    "24.39629": "Huawei Technologies Co.,Ltd",
    "24.16281566": "Shenzhen Goodix Technology Co.,Ltd.",
    "20.251018417799168": "Muzik Inc",
    "12.123917690662912": "Tattile Srl",
    "12.123917690454016": "Refine Technology, LLC",
    "24.14220136": "Cloud Corner Ltd.",
    "24.4200095": "Tp-Link Technologies Co.,Ltd.",
    "24.1626742": "Huawei Technologies Co.,Ltd",
    "24.6877372": "Shenzhen LiWiFi Technology Co., Ltd",
    "12.346857586688": "Vitec Multimedia",
    "24.6657": "Smiths Medical",
    "24.2376674": "DASAN Network Solutions",
    "12.346862514176": "CODEC Co., Ltd.",
    "24.7909907": "DWnet Technologies(Suzhou) Corporation",
    "24.1088": "cyberPIXIE, Inc.",
    "24.1089": "Half Dome Systems, Inc.",
    "24.1084": "SONOS Co., Ltd.",
    "24.1085": "Indel Ag",
    "24.1086": "Telencomm",
    "24.1087": "ESTeem Wireless Modems, Inc",
    "24.1080": "Nortel Networks",
    "24.1081": "Rosco Entertainment Technology, Inc.",
    "24.1082": "Intelligent Telecommunications, Inc.",
    "24.1083": "Lava Computer Mfg., Inc.",
    "24.2685562": "Zhejiang Tmall Technology Co., Ltd.",
    "20.35758078427136": "Mastodon Design",
    "24.7100281": "Aclima, Inc.",
    "12.346855178240": "Baxter International Inc",
    "20.189549219151872": "Digibird Technology Co., Ltd.",
    "12.123917677981696": "Private",
    "24.10025179": "Marini Impianti Industriali s.r.l.",
    "20.145211807236096": "Hitachi,Ltd.",
    "24.2924268": "Jabil Circuit Penang",
    "24.1335424": "zte corporation",
    "24.8913897": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.14707002": "Innophase Inc.",
    "24.3938463": "Nokia Corporation",
    "24.16043574": "Nordic Semiconductor ASA",
    "24.3938464": "Luxshare Precision Industry Company Limited",
    "24.14719073": "Motorola Mobility LLC, a Lenovo Company",
    "24.9450862": "Vodafone Omnitel N.V.",
    "12.123917678788608": "GD Mission Systems",
    "24.10803093": "Wingtech Mobile Communications Co.,Ltd",
    "12.346866782208": "Saia-Burgess Controls AG",
    "20.198237806919680": "Shenzhen Tita Interactive Technology Co.,Ltd",
    "24.7347652": "NorthStar Battery Company, LLC",
    "24.2670908": "Hon Hai Precision Ind. Co., Ltd.",
    "12.123917684994048": "Axis Corporation",
    "24.11577454": "Juniper Networks",
    "24.6652": "PT. Ufoakses Sukses Luarbiasa",
    "24.9746457": "Nokia",
    "24.10787416": "Samsung Electronics Co.,Ltd",
    "24.5251179": "Emerson Climate Technologies Transportation Solutions",
    "24.1882199": "Becon Technologies Co,.Ltd.",
    "12.346861748224": "Link Electric & Safety Control Co.",
    "24.9480226": "Amazon Technologies Inc.",
    "24.8108": "Goodmill Systems Ltd",
    "24.8109": "Brown Innovations, Inc",
    "24.8106": "Taseon, Inc.",
    "24.8107": "I.S High Tech.Inc",
    "24.8104": "Smart Energy Instruments Inc.",
    "24.8105": "Atlanta DTH, Inc.",
    "24.8102": "Stilo srl",
    "24.8103": "Sony Interactive Entertainment Inc.",
    "24.8100": "Shenzhen Gongjin Electronics Co.,Lt",
    "24.8101": "Blue-White Industries",
    "12.346855460864": "Federal Aviation Administration",
    "24.12906352": "Home Skinovations",
    "12.123917686136832": "Argon ST",
    "24.13396400": "Global Traffic Technologies, LLC",
    "20.264036219355136": "Ubiant",
    "24.8448547": "Wistron Neweb Corporation",
    "12.346864467968": "ImpactSystems",
    "24.1883577": "D-Link International",
    "12.123917681700864": "Intrinsic Group Limited",
    "24.3960921": "MakerBot Industries",
    "20.84372889796608": "Quake Global Inc",
    "12.123917684166656": "Mahle Electronics, Slu",
    "24.1840200": "ARRIS Group, Inc.",
    "24.6580668": "Hytera Communications Co.,ltd",
    "24.6585294": "Samsung Electronics Co.,Ltd",
    "12.123917683265536": "Applied Materials",
    "12.71297885044736": "Sensata Technologies",
    "20.102196699987968": "Hangzhou Signwei Electronics Technology Co., Ltd",
    "12.346852597760": "Unimark",
    "24.4231069": "Juniper Networks",
    "12.123917683007488": "Alpha ESS Co., Ltd.",
    "24.9226484": "IEEE Registration Authority",
    "20.189549211811840": "Wittmann Kunststoffgeräte GmbH",
    "24.10281430": "Junger Audio-Studiotechnik GmbH",
    "24.2151851": "Korea Infocom Co.,Ltd.",
    "24.4753610": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "24.12893162": "Pakedge Device and Software Inc",
    "24.4478647": "Spawn Labs, Inc",
    "24.6353506": "Intel Corporate",
    "24.9963066": "Home Control Singapore Pte Ltd",
    "12.123917679460352": "Discover Video",
    "24.3426651": "Sagemcom Broadband SAS",
    "12.346858434560": "All-Systems Electronics Pty Ltd",
    "12.123917679976448": "Gy-Fx Sas",
    "24.2899663": "Cisco Systems, Inc",
    "12.346855907328": "Phytec Messtechnik GmbH",
    "20.277783358996480": "Innovative Advantage",
    "12.71297885134848": "Vidisys GmbH",
    "12.123917684588544": "Meridian Technologies Inc",
    "20.62636961038336": "Macherey-Nagel GmbH & Co. KG",
    "12.346862129152": "Novotronik GmbH",
    "24.6605661": "SIA \"TeleSet\"",
    "24.12080044": "Apple, Inc.",
    "24.11809900": "Matsunichi Digital Technology (Hong Kong) Limited",
    "24.2401324": "KOUKAAM a.s.",
    "24.2936038": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "12.346854912000": "Baudisch Electronic GmbH",
    "12.123917687078912": "OOO Research and Production Center \"Computer Technologies\"",
    "24.4779099": "Nokia Solutions and Networks GmbH & Co. KG",
    "20.62070499311616": "KingWay Information Co.,Ltd.",
    "24.4210572": "Tp-Link Technologies Co.,Ltd.",
    "12.346862469120": "First",
    "24.530854": "Huawei Technologies Co.,Ltd",
    "24.4472097": "Nuvolt",
    "24.6337415": "Synergics Technologies GmbH",
    "24.1601325": "Shenzhen Sundray Technologies Company Limited",
    "24.5560804": "Brilliantts Co., Ltd",
    "24.2643815": "Dish Technologies Corp",
    "24.308": "Selectron Systems AG",
    "24.3155113": "HP Inc.",
    "24.37037": "Aspect Electronics, Inc.",
    "24.9227752": "Nintendo Co., Ltd.",
    "24.1614": "Broad Net Technology Inc.",
    "24.1617": "Aspen Networks Inc.",
    "24.1616": "Tiburon Networks, Inc.",
    "24.5008360": "Apple, Inc.",
    "24.1611": "Alexon Co., Ltd.",
    "20.22126497103872": "Shenzhen Mining Technology Co.,Ltd.",
    "24.1610": "Honeywell Co., Ltd. (KOREA)",
    "24.5316": "Vitelcom Mobile Technology",
    "24.14166654": "Nokia Corporation",
    "24.11011584": "Samsung Electronics Co.,Ltd",
    "24.1612": "Invicta Networks, Inc.",
    "24.826770": "ASUSTek COMPUTER INC.",
    "12.123917683585024": "SIANA Systems",
    "24.11019755": "Cambridge Industries(Group) Co.,Ltd.",
    "20.49277262888960": "PIN SHANG LED Co., LTD.",
    "24.9750880": "Zhongshan B&T technology.co.,ltd",
    "24.14705074": "Hangzhou Zenointel Technology Co., Ltd",
    "24.3974955": "MRV Communications (Networks) LTD",
    "12.123917675376640": "PAL Inc.",
    "20.36069849432064": "Rugged Monitoring",
    "12.346866233344": "Nemec Automation",
    "12.346867937280": "Treehaven Technologies, Inc.",
    "24.15495319": "ARRIS Group, Inc.",
    "24.10786652": "VTech Telecommunications Ltd.",
    "24.12848905": "Hermes electronic GmbH",
    "24.48917": "Genetec Inc.",
    "24.9475300": "D-Link International",
    "24.2390511": "Greenpacket Berhad, Taiwan",
    "12.346860257280": "Design 2000 Pty Ltd",
    "12.346857103360": "General Resources Co., LTD.",
    "24.1606672": "CoreTrust Inc.",
    "12.346856853504": "Unlimited Bandwidth LLC",
    "20.150399885508608": "Shenzhen Viewsmart Technology Co.,Ltd",
    "24.4992925": "ICM Controls",
    "12.123917687537664": "CreevX",
    "12.123917685174272": "K&J Schmittschneider AG",
    "24.7381": "ZeeVee, Inc.",
    "24.7081441": "Dongguan Cannice Precision Manufacturing Co., Ltd.",
    "20.106480345808896": "SKS Automaatio oy",
    "12.71297884119040": "Kyoritsu Electric Corp.",
    "20.127937508540416": "ComNot",
    "24.14726495": "Apple, Inc.",
    "24.9991108": "Sage Electronic Engineering LLC",
    "12.71297883209728": "Kaori Industria Eletronica Ltda",
    "12.123917687627776": "Infosocket Co., Ltd.",
    "24.4774005": "China TransInfo Technology Co., Ltd",
    "24.289881": "Intel Corporate",
    "24.12334080": "IEEE Registration Authority",
    "24.4735868": "Apple, Inc.",
    "20.84197043601408": "Celltron",
    "12.346857951232": "Wireless Cables Inc.",
    "24.2100175": "Nintendo Co.,Ltd",
    "12.346858414080": "Cambridge Consultants Ltd",
    "20.66256294117376": "Xiaoniu network technology (Shanghai) Co., Ltd.",
    "12.346864558080": "Percello Ltd.",
    "24.58982": "ARIMA Communications Corp.",
    "24.11049308": "aizo ag",
    "24.5255735": "Apple, Inc.",
    "24.8959527": "Gembird Europe BV",
    "24.14192938": "Zyxel Communications Corporation",
    "24.16294280": "LED Roadway Lighting",
    "24.12592390": "AVM GmbH",
    "20.127654888996864": "Starcor Beijing Co.,Limited",
    "12.123917683486720": "Aplex Technology Inc.",
    "24.7922": "Micro Motion Inc",
    "20.123731221413888": "Cable Matters Inc.",
    "12.123917691371520": "SureFlap Ltd",
    "20.48476174942208": "Saft AB",
    "12.123917690179584": "Acrodea, Inc.",
    "12.123917688033280": "Cyviz AS",
    "24.7890903": "Apple, Inc.",
    "24.10751420": "Xiamen Intretech Inc.",
    "20.45069249806336": "Apollo Digital (Taiwan) Ltd.",
    "24.8129087": "Samsung Electronics Co.,Ltd",
    "24.10025746": "Grand Electronics (HK) Ltd",
    "24.13171014": "LOCOSYS Technology Inc.",
    "24.10533694": "IEEE Registration Authority",
    "24.12608361": "Binxin Technology(Zhejiang) Ltd.",
    "20.84197028921344": "Vestaboard, Inc.",
    "24.14979325": "Apple, Inc.",
    "24.3412729": "Huawei Technologies Co.,Ltd",
    "24.9372": "Bimeng Comunication System Co. Ltd",
    "24.2908": "Newtech Co.,Ltd",
    "20.79602633408512": "EnBW Energie Baden-Württemberg AG",
    "24.13655992": "Texas Instruments",
    "24.7115455": "Inspur Electronic Information Industry Co.,Ltd.",
    "24.10803924": "Yamato Denki Co.,Ltd.",
    "20.228784610476032": "Realwave Inc.",
    "12.346863955968": "M-Tronic Design and Technology GmbH",
    "12.346859081728": "Bulletendpoints Enterprises Inc",
    "12.346864803840": "AVItronic GmbH",
    "24.2909": "Fujitsu Limited",
    "24.3672020": "Zeppelin Systems GmbH",
    "24.9206261": "Samsung Electronics Co.,Ltd",
    "12.346860490752": "STUHL Regelsysteme GmbH",
    "24.4990328": "Nokia Corporation",
    "24.3538": "Simrad Optronics ASA",
    "24.3539": "SAMWOO Telecommunication Co.,Ltd.",
    "24.7609383": "Itel Mobile Limited",
    "24.2367915": "Sony Mobile Communications Inc",
    "20.22333154656256": "D&S Cable Industries (HK) Limited",
    "24.3532": "NEOSMART Corp.",
    "24.3533": "Groupe Txcom",
    "24.3530": "Tait Electronics",
    "24.3531": "Petcomkorea Co., Ltd.",
    "24.3536": "TetraTec Instruments GmbH",
    "24.3537": "Stryker Corporation",
    "24.3534": "Dynavac Technology Pte Ltd",
    "24.3535": "Cidra Corp.",
    "12.123917678714880": "KSE GmbH",
    "24.10494879": "Cisco Systems, Inc",
    "12.123917680152576": "Adetec Sas",
    "24.12901719": "Espressif Inc.",
    "12.346857279488": "SS Telecoms CC",
    "12.346867757056": "Monitor Business Machines",
    "24.4981294": "Cmr Korea Co., Ltd",
    "24.5023563": "Alcatel Lucent",
    "24.8425770": "Lih Rong electronic Enterprise Co., Ltd.",
    "24.7136462": "Villiger Security Solutions AG",
    "24.8425779": "AzureWave Technology Inc.",
    "24.11845622": "Samsung Electronics Co.,Ltd",
    "24.11027111": "Jiangsu Huitong Group Co.,Ltd.",
    "12.346859298816": "EMITALL Surveillance S.A,",
    "12.123917676863488": "Bita-International Co., Ltd.",
    "24.14185779": "Nokia Corporation",
    "20.167137995915264": "Private",
    "24.15507921": "Huawei Technologies Co.,Ltd",
    "12.346861694976": "Cirtem",
    "24.11141124": "DEC",
    "24.9962116": "Theobroma Systems GmbH",
    "12.123917688426496": "Suzhou PowerCore Technology Co.,Ltd.",
    "24.7364830": "LaVision GmbH",
    "24.14473646": "Nokia Shanghai Bell Co., Ltd.",
    "24.14700836": "NXP Semiconductors",
    "12.123917685071872": "KEVIC. inc,",
    "24.14468642": "Buheung System",
    "24.11141121": "DEC",
    "24.15738693": "F-Secure Corporation",
    "12.346854785024": "Agecodagis Sarl",
    "12.346856165376": "Braintronics BV",
    "24.12850146": "Extreme Networks, Inc.",
    "24.5287281": "Tessolve Semiconductor Private Limited",
    "24.12617819": "SnD Tech Co., Ltd.",
    "24.4223050": "Power Idea Technology Limited",
    "24.3719866": "Cctv Neotech",
    "20.62335522701312": "System Q Ltd",
    "24.5268523": "XIAOMI Electronics,CO.,LTD",
    "24.4775735": "Hewlett Packard Enterprise",
    "24.3977563": "Avaya Inc",
    "12.123917689643008": "Tiab Limited",
    "24.9459118": "Ericsson, EAB/RWI/K",
    "24.7916126": "Elno",
    "12.346862821376": "Astronics Custom Control Concepts",
    "24.2415155": "Huawei Technologies Co.,Ltd",
    "24.7949": "L3 Communications - Telemetry West",
    "24.7948": "Intelligent Digital Services GmbH",
    "24.7947": "Federal State Unitary Enterprise Industrial Union\"Electropribor\"",
    "24.7946": "Nortel Networks",
    "24.7945": "Jastec",
    "24.7944": "Risco Ltd",
    "24.7943": "AZTEQ Mobile",
    "24.7942": "Integrated Dispatch Solutions",
    "24.7941": "iTAS Technology Corp.",
    "24.7940": "Granch Ltd.",
    "24.53480": "Mac System Co., Ltd.",
    "12.346865827840": "Reimesch Kommunikationssysteme GmbH",
    "24.7652": "Visioneered Image Systems",
    "20.44213338112000": "Chargelib",
    "12.123917675016192": "Anyroam",
    "24.53482": "Nextone Communications, Inc.",
    "20.136823632297984": "Shanghai magcomm communication technology co ltd",
    "24.6018": "LoBenn Inc.",
    "24.6019": "Texas Instruments",
    "24.13157477": "Huawei Technologies Co.,Ltd",
    "12.123917685698560": "Tiama",
    "24.6012": "Smartlink Network Systems Limited",
    "24.6013": "IDT Technology Limited",
    "24.6010": "Assa Abloy Ab",
    "24.6011": "Azalea Networks inc",
    "24.6016": "Applied Biosystems B.V.",
    "24.6017": "Greystone Data System, Inc.",
    "24.6014": "Meshcom Technologies Inc.",
    "24.7656": "Nikko Denki Tsushin Corporation(NDTC)",
    "24.833810": "JM-DATA GmbH",
    "24.2136280": "Shenzhen Yingdakang Technology CO., LTD",
    "24.53486": "Dictaphone Corporation",
    "24.53487": "Igt",
    "12.71297883238400": "Digital Dynamics, Inc.",
    "24.21978": "IEEE Registration Authority",
    "24.12108371": "Juniper Networks",
    "12.346860695552": "Robot Visual Systems GmbH",
    "20.167152471506944": "Petersime",
    "24.1619345": "I-Storm",
    "24.7398020": "Wistron Infocomm (Zhongshan) Corporation",
    "24.3978935": "vivo Mobile Communication Co., Ltd.",
    "20.27478326247424": "Kwang Yang Motor Co.,Ltd",
    "24.5027693": "Novi Security",
    "24.8673921": "ffly4u",
    "24.9209318": "Cleondris GmbH",
    "24.61099": "Cisco Systems, Inc",
    "24.2667197": "Intel Corporate",
    "12.123917676163072": "HoseoTelnet Inc...",
    "24.3691079": "Huawei Device Co., Ltd.",
    "20.70559711887360": "Annapurna labs",
    "24.6306079": "ABB Ltd",
    "24.11814750": "Avaya Inc",
    "24.12587379": "Samsung Electronics Co.,Ltd",
    "24.2906": "HyperEdge",
    "24.1248": "Procket Networks",
    "24.1249": "Infinior Microsystems",
    "24.1242": "Relax Technology, Inc.",
    "24.1243": "Tellus Group Corp.",
    "24.1240": "IPWireless, Inc.",
    "24.1241": "Titan Electronics, Inc.",
    "24.1246": "Cisco Systems, Inc",
    "24.1247": "Teracom Telematica S.A",
    "24.1244": "Nortel Networks",
    "24.1245": "Cisco Systems, Inc",
    "24.6607267": "Sitecom Europe BV",
    "24.8925851": "Samsung Electro-Mechanics(Thailand)",
    "12.346858696704": "Powis Corporation",
    "12.123917680652288": "Zhejiang Hao Teng Electronic Technology Co., Ltd.",
    "20.145727907954688": "Aerionics Inc.",
    "24.10521878": "Huawei Technologies Co.,Ltd",
    "20.22126510735360": "AJIS(DALIAN)co.,LTD",
    "24.13682163": "Apple, Inc.",
    "24.5093": "Tenosys, Inc.",
    "24.5092": "Yangjae Systems Corp.",
    "24.5091": "CoVi Technologies, Inc.",
    "24.5090": "GeoVision Inc.",
    "24.5097": "VeriWave, Inc.",
    "24.5096": "Intel Corporate",
    "24.5095": "Halcro",
    "24.5094": "Technolution",
    "24.10793409": "Intel Corporate",
    "24.5099": "Sysmaster Corporation",
    "24.5098": "Kamstrup A/S",
    "12.346866343936": "Ecomm Era",
    "24.12621607": "Samsung Electro-Mechanics(Thailand)",
    "12.123917691469824": "Array Technologies Inc.",
    "24.7120735": "zte corporation",
    "20.127654893191168": "Princip a.s.",
    "24.13417781": "Steinel GmbH",
    "12.346857648128": "Teleks Co. Ltd.",
    "24.9082029": "Nexgen A/S",
    "24.5803649": "Silicon Laboratories",
    "24.53376": "Exabyte Corporation",
    "24.53377": "RTD Embedded Technologies, Inc.",
    "24.53374": "Keycorp Ltd.",
    "24.53375": "Strategy & Technology, Limited",
    "24.1586257": "Apple, Inc.",
    "24.53373": "Cosine Communications",
    "24.53370": "Amaquest Computer Corp.",
    "24.53371": "Comcam International Inc",
    "24.2367276": "Calsys, Inc.",
    "24.53378": "Iowave Inc.",
    "24.53379": "Invertex, Inc.",
    "12.123917685112832": "Nordmann International GmbH",
    "12.346860847104": "DASA ROBOT Co., Ltd.",
    "24.27986": "Apple, Inc.",
    "12.123917677273088": "Shishido Electrostatic, Ltd.",
    "24.6085332": "Murata Manufacturing Co., Ltd.",
    "20.22739788234752": "Kayamatics Limited",
    "24.10015698": "MEKRA Lang GmbH & Co. KG",
    "12.346852896768": "SMX Corporation",
    "24.6871958": "OPWILL Technologies CO.,LTD",
    "20.97755438514176": "McWong International Inc",
    "24.6617936": "RoomReady/Zdi, Inc.",
    "24.10253338": "Huawei Technologies Co.,Ltd",
    "20.57194436362240": "Connected IO",
    "24.1329528": "Digital Control Incorporated",
    "24.5637": "Yorkville Sound Inc.",
    "24.5636": "Sigpro",
    "24.5635": "COOLKSKY Co., LTD",
    "24.5634": "Ceyon Technology Co.,Ltd.",
    "24.5633": "Buffalo.Inc",
    "24.5632": "CelleBrite Mobile Synchronization",
    "24.5631": "Novatel Wireless Solutions, Inc.",
    "24.5630": "Schilling Robotics Llc",
    "24.1854393": "Smg Enterprise, Llc",
    "24.15244679": "Toshiba",
    "24.15994651": "Cisco Systems, Inc",
    "24.5639": "Curves International Inc.",
    "24.5638": "Ideal Industries",
    "20.75685690343424": "Valeo Interior Controls (Shenzhen) Co.,Ltd",
    "12.123917682900992": "Slan",
    "12.346857250816": "MEDIORNET GmbH",
    "24.4322": "ArrayComm, Inc.",
    "24.4323": "Hewlett Packard",
    "24.4320": "Oracle Corporation",
    "24.4321": "S.I. Tech, Inc.",
    "24.4326": "Applied Intelligent Systems, Inc.",
    "24.4327": "Breezecom, Ltd.",
    "24.4324": "Nsi Corporation",
    "24.4325": "Solectron Texas",
    "12.123917682917376": "Survalent Technology Corporation",
    "24.4328": "Telocity, Incorporated",
    "24.4329": "Raidtec Ltd.",
    "24.15268136": "Rftech Srl",
    "24.3671": "Iworld Networking, Inc.",
    "24.3670": "4G Systems GmbH & Co. KG",
    "24.3673": "Sagemcom Broadband SAS",
    "24.3672": "Sonos, Inc.",
    "24.3675": "ParkerVision - Direct2Data",
    "24.3674": "TELEFIELD inc.",
    "24.3677": "Triple Play Technologies A/S",
    "24.3676": "ARRIS Group, Inc.",
    "24.3679": "activ-net GmbH & Co. KG",
    "24.3678": "Raisecom Technology",
    "24.16289543": "Samsung Electronics Co.,Ltd",
    "24.9971242": "Applied Research Associates, Inc",
    "24.10512964": "Vizio, Inc",
    "24.12874782": "zte corporation",
    "24.8408920": "ThinkEco, Inc.",
    "12.123917680996352": "on-systems limited",
    "24.15218199": "Apple, Inc.",
    "20.132869110366208": "Shanghai Espacetime Technology Co.,Ltd.",
    "24.1327586": "Monaco Enterprises, Inc.",
    "12.119269621760": "Enstech",
    "20.13592313200640": "Yamasei",
    "24.15241993": "Samsung Electronics Co.,Ltd",
    "12.346858119168": "Cps Europe B.V.",
    "24.6829193": "LEA Professional",
    "12.123917678993408": "Aditec GmbH",
    "24.7078515": "Shenzhen Jin Yun Video Equipment Co., Ltd.",
    "12.346854612992": "Information Technology Corp.",
    "24.9985": "INCOstartec GmbH",
    "24.9984": "Shenzhen Siglent Technology Co., Ltd.",
    "24.9987": "Testech Electronics Pte Ltd",
    "24.9986": "SolarEdge Technologies",
    "24.9981": "Interactive Intelligence",
    "24.9980": "AcSiP Technology Corp.",
    "24.9983": "BlackBerry RTS",
    "24.9982": "MKD Technology Inc.",
    "24.9989": "Sectronic",
    "24.9988": "Accelerated Concepts, Inc",
    "24.3689551": "Huawei Technologies Co.,Ltd",
    "24.12107508": "devolo AG",
    "24.5514730": "Huawei Technologies Co.,Ltd",
    "24.7373454": "OleumTech Corporation",
    "12.123917675343872": "Yamakatsu Electronics Industry Co., Ltd.",
    "24.1591498": "Murata Manufacturing Co., Ltd.",
    "24.13955591": "DIAODIAO(Beijing)Technology CO.,Ltd",
    "24.9964709": "Motorola (Wuhan) Mobility Technologies Communication Co., Ltd.",
    "12.346859909120": "XtendWave",
    "12.346865799168": "Aanderaa Data Instruments",
    "24.6087203": "Delphin Technology AG",
    "24.1349757": "Technicolor CH USA Inc.",
    "24.1349751": "Apple, Inc.",
    "24.10287416": "AREVA NP GmbH",
    "24.12854824": "Airo Wireless",
    "24.5042590": "Groupe Atlantic",
    "24.16521625": "Xiaomi Communications Co Ltd",
    "24.7130692": "1More",
    "12.346866638848": "Mecsel Oy",
    "12.346866921472": "Automation Assist Japan Company",
    "12.123917690531840": "Ipitek",
    "24.12882816": "Speedytel technology",
    "24.1809": "Acterna",
    "24.2188": "Quanta Network Systems Inc.",
    "24.2189": "Sigma-Links Inc.",
    "24.1847304": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.2180": "Index Braille AB",
    "24.2181": "EMS Dr. Thomas Wünsche",
    "24.2182": "Hansung Teliann, Inc.",
    "24.2183": "Maschinenfabrik Reinhausen GmbH",
    "24.2184": "OULLIM Information Technology Inc,.",
    "24.2185": "Dish Technologies Corp",
    "24.2186": "Minds@Work",
    "24.2187": "Tropic Networks Inc.",
    "24.20648": "OpenCon Systems, Inc.",
    "24.5383": "Renaissance Learning Inc",
    "24.5046034": "Fuze Entertainment Co., ltd",
    "20.71414662037504": "Fast Precision Technologies Co. Ltd.",
    "24.5382": "Neo Photonics",
    "24.20640": "Delta Computer Systems, Inc.",
    "24.20641": "Carlo Gavazzi, Inc.",
    "24.20642": "Cisco Systems, Inc",
    "24.20643": "TransMedia Communications, Inc.",
    "24.20644": "Io Tech, Inc.",
    "24.5381": "Actiontec Electronics, Inc",
    "24.20646": "Optronics",
    "24.20647": "Cisco Systems, Inc",
    "24.7002": "Apollo Imaging Technologies, Inc.",
    "24.7003": "2Wire Inc",
    "20.45069244563456": "Xiaocong Network Limited",
    "24.1807": "Fujant, Inc.",
    "24.7006": "BPL Limited",
    "24.7007": "Alien Technology",
    "24.7004": "Azuretec Co., Ltd.",
    "24.7005": "Vololink Pty Ltd",
    "24.2632882": "Tp-Link Technologies Co.,Ltd.",
    "24.7008": "Navigon Ag",
    "24.7009": "Digital Acoustics, LLC",
    "12.346854514688": "Kallastra Inc.",
    "24.5386": "Sonoa Systems, Inc",
    "24.8660287": "silex technology, Inc.",
    "24.5385": "Plus Technology Co., Ltd",
    "12.346852708352": "Crossport Systems",
    "12.123917679112192": "Grossenbacher Systeme AG",
    "12.123917687611392": "XotonicsMED GmbH",
    "24.5384": "Global Target Enterprise Inc",
    "20.225207127113728": "Cujo Llc",
    "24.5001198": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.12910370": "Huawei Device Co., Ltd.",
    "20.14223587409920": "Infinisource Inc.",
    "24.13431037": "China Mobile (Hangzhou) Information Technology Co., Ltd.",
    "24.6319037": "Samsung Electronics Co.,Ltd",
    "20.203238165970944": "Docobo Limited",
    "12.71297883918336": "Data Sharing Consulting",
    "24.8768": "Universal Telecom S/A",
    "24.8769": "Apple, Inc.",
    "12.346853515264": "Dycec, S.A.",
    "24.8767": "Netgear",
    "24.8764": "RATIO Entwicklungen GmbH",
    "24.8765": "JumpGen Systems, LLC",
    "24.8762": "Cisco SPVTG",
    "24.8763": "Communication Networks, LLC",
    "24.8760": "Logiplus",
    "24.8761": "Indiana Life Sciences Incorporated",
    "24.18516": "Digital SemiConductor",
    "24.10281791": "Apple, Inc.",
    "24.14704453": "Apple, Inc.",
    "20.220092037070848": "Kunshan SVL Electric Co.,Ltd",
    "24.6963301": "Open Connectivity Foundation",
    "12.119269625856": "Scientific-Technical Center \"Epsilon\" Limited company",
    "12.71297883439104": "GHL Systems Berhad",
    "20.158667117690880": "Teleepoch Ltd",
    "20.225207118725120": "NTmore.Co.,Ltd",
    "12.346854940672": "Brannstroms Elektronik AB",
    "20.206931535855616": "Faro Technologies, Inc.",
    "20.233258811916288": "E-MetroTel",
    "24.9703898": "ITF Fröschl GmbH",
    "24.13115111": "Apple, Inc.",
    "12.123917676765184": "EcoG",
    "24.12114253": "Sagemcom Broadband SAS",
    "24.2682450": "Tp-Link Technologies Co.,Ltd.",
    "24.6599197": "Chengdu Phycom Tech Co., Ltd.",
    "20.150262244179968": "Twk-Elektronik",
    "24.6751": "KitWorks.fi Ltd.",
    "24.6750": "Thincom Technology Co.,Ltd",
    "24.6753": "PacStar Corp.",
    "24.6752": "Wave Electronics Co.,Ltd.",
    "24.6755": "Elster Solutions, LLC,",
    "24.6754": "Data Robotics, Incorporated",
    "24.6757": "Seluxit",
    "24.6756": "IBM Corp",
    "24.6759": "Infinite QL Sdn Bhd",
    "24.6758": "ARRIS Group, Inc.",
    "24.2238": "XENPAK MSA Group",
    "24.2239": "Aptus Elektronik AB",
    "24.1761": "Techno Trade s.a",
    "24.1760": "MAT Co., Ltd.",
    "24.1763": "Quantitative Imaging Corporation",
    "24.1762": "Ceemax Technology Co., Ltd.",
    "24.1765": "Fujian Newland Computer Ltd. Co.",
    "24.1764": "Citel Technologies Ltd.",
    "24.1767": "Bit Blitz Communications Inc.",
    "24.1766": "DongYang Telecom Co., Ltd.",
    "24.1769": "Intime Corp.",
    "24.1768": "Optical Network Testing, Inc.",
    "24.13636625": "IEEE Registration Authority",
    "24.1099142": "Bio Sound Lab Co., Ltd.",
    "24.10532450": "Acutvista Innovation Co., Ltd.",
    "24.5029044": "ABB SpA - DIN Rail",
    "24.10024107": "Espressif Inc.",
    "12.123917681623040": "GIFAS-ELECTRIC GmbH",
    "12.123917682700288": "Gabo",
    "12.123917691097088": "Korea Spectral Products",
    "24.6599440": "Samsung Electronics Co.,Ltd",
    "24.7100980": "Shenzhen Haitianxiong Electronic Co., Ltd.",
    "24.6594291": "Cisco Systems, Inc",
    "20.39919494889472": "Jiangsu Xuanbo Electronic Technologies Co.,Ltd",
    "20.137149123919872": "Vantage Integrated Security Solutions Pvt Ltd",
    "12.346867195904": "Rfengine Co., Ltd.",
    "12.346858569728": "TestPro Systems, Inc.",
    "24.12592476": "Cisco Systems, Inc",
    "12.123917682651136": "Code Blue Corporation",
    "24.4230492": "Giga-Byte Technology Co.,Ltd.",
    "24.5267958": "Universal Electronics, Inc.",
    "24.4481606": "Comat AG",
    "12.71297883807744": "CoherentPlus Sdn Bhd",
    "20.62663545585664": "Inspero Inc",
    "24.14958295": "Arcadyan Corporation",
    "20.189549209714688": "Shanghai Zte Technologies Co.,Ltd",
    "12.71297884692480": "Yawata Electric Industrial Co.,Ltd.",
    "12.123917691633664": "Tapdn",
    "24.11582829": "Echostreams Innovative Solutions",
    "24.9174": "Samsung Electronics Co.,Ltd",
    "24.9175": "Samsung Electronics Co.,Ltd",
    "24.9176": "Ball-It Oy",
    "24.9177": "Banner Engineering",
    "24.9170": "Inhand Electronics, Inc.",
    "12.119269474304": "Xiphos Systems Corp.",
    "24.9172": "Texas Instruments",
    "24.9173": "WAREMA electronic GmbH",
    "24.15211180": "ORFEO SOUNDWORKS Inc.",
    "24.289573": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.9178": "Industrial Computer Source (Deutschland)GmbH",
    "24.9179": "saxnet gmbh",
    "24.2898472": "Fagor Electrónica",
    "12.123917680857088": "PLR Prueftechnik Linke und Ruehe GmbH",
    "20.194600865103872": "Everynet Oy",
    "20.233630181883904": "Exafore Oy",
    "24.13638843": "Beijing Ctimes Digital Technology Co., Ltd.",
    "20.150399878168576": "Brabender Technologie GmbH & Co, KG",
    "12.123917684154368": "Tiama",
    "24.8923472": "Netmoon Technology Co., Ltd",
    "12.123917676199936": "Forschungs- und Transferzentrum Leipzig e.V.",
    "24.3191966": "Tp-Link Technologies Co.,Ltd.",
    "12.123917681369088": "Edge Power Solutions",
    "24.13680831": "Actions Microelectronics Co., Ltd",
    "20.79602629214208": "Venture Research Inc.",
    "24.8155876": "Unikey Technologies",
    "24.2111830": "HMD Global Oy",
    "24.13127438": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "20.211525435392000": "Mad Piece Llc.",
    "24.5783840": "Tp-Link Technologies Co.,Ltd.",
    "20.27478323101696": "Translite Global Llc",
    "12.346864099328": "Measurement Technology NW",
    "12.346858491904": "Toptech Systems, Inc.",
    "24.11305234": "Gemtek Technology Co., Ltd.",
    "12.123917677731840": "Profusion Limited",
    "24.4518966": "zte corporation",
    "12.123917688709120": "Fideltronik Poland Sp. Z O.O.",
    "24.2384218": "China Mobile Group Device Co.,Ltd.",
    "24.9759693": "BlackBerry RTS",
    "24.6302416": "Cisco SPVTG",
    "12.71297884868608": "EMAC, Inc.",
    "12.123917688094720": "Beijing Aumiwalker technology CO.,LTD",
    "12.346855010304": "RAFI GmbH & Co.KG",
    "24.5521581": "Samsung Electronics Co.,Ltd",
    "24.1081780": "Hunan Greatwall Galaxy Science and Technology Co.,Ltd.",
    "24.11048565": "Motorola Mobility LLC, a Lenovo Company",
    "24.10787603": "Digital Check",
    "24.1612893": "Cisco Systems, Inc",
    "24.7619885": "Huawei Device Co., Ltd.",
    "12.346852786176": "EPEL Industrial, S.A.",
    "12.123917684666368": "Tattile Srl",
    "24.9178256": "Ruckus Wireless",
    "24.7099880": "Technicolor CH USA Inc.",
    "24.16582": "Fibernet Research, Inc.",
    "24.16583": "Ruby Tech Corporation",
    "24.16580": "Kinkei System Corporation",
    "24.16581": "Micom Communications Corp.",
    "24.16586": "First Internat'L Computer, Inc",
    "24.16587": "Lanwan Technologies",
    "24.16584": "Milan Technology Corp.",
    "24.16585": "Ncube",
    "24.6586661": "Alphatron Marine BV",
    "24.14464814": "Xiaomi Communications Co Ltd",
    "12.346852798464": "Seffle Instrument AB",
    "24.16589": "Tera Microsystems, Inc.",
    "24.5563967": "ShenZhen LingKeWeiEr Technology Co., Ltd.",
    "24.3200713": "Samsung Electronics Co.,Ltd",
    "24.4506438": "Huawei Technologies Co.,Ltd",
    "12.346860056576": "Austco Marketing & Service (USA) ltd.",
    "24.1344091": "Hefei Radio Communication Technology Co., Ltd",
    "24.8935406": "Buffalo.Inc",
    "24.3177520": "Rim",
    "20.119143424589824": "Guangzhou Guangri Elevator Industry Co.,Ltd",
    "20.228784614670336": "CYLTek Limited",
    "20.264625135288320": "Med-Pat/Inn-Phone",
    "24.6834328": "Samsung Electronics Co.,Ltd",
    "24.7122709": "Webasto SE",
    "12.123917685313536": "Southern Tier Technologies",
    "24.15264931": "Dongguan Liesheng Electronic Co.Ltd",
    "24.8424887": "Hewlett Packard Enterprise",
    "20.89058724282368": "metraTec GmbH",
    "24.3995746": "Intel Corporate",
    "12.123917690785792": "MeQ Inc.",
    "12.123917675130880": "SPX Flow Technology BV",
    "24.4718717": "Dts Elektronik San. Tic. Ltd. Sti.",
    "24.7907307": "1More",
    "12.123917678477312": "Tickster AB",
    "12.346857189376": "Verint Systems Ltd",
    "24.3687944": "zte corporation",
    "24.11069309": "Texas Instruments",
    "24.6827879": "Apple, Inc.",
    "24.1072902": "Actiontec Electronics, Inc",
    "24.1316983": "Dell Inc.",
    "20.79215071330304": "shanghai Rinlink Intelligent Technology Co., Ltd.",
    "24.4466107": "Bamboo Entertainment Corporation",
    "12.346861867008": "Rolbit",
    "24.6066194": "Owl Cyber Defense Solutions, LLC",
    "12.346853683200": "DeMeTec GmbH",
    "24.16033181": "Huawei Device Co., Ltd.",
    "12.123917680820224": "Optsys",
    "12.123917691777024": "Private",
    "12.346868424704": "Sincair Systems International",
    "24.8443699": "QSR Automations, Inc.",
    "20.180394595975168": "Shenzhen Zhishi Technology Co., Ltd.",
    "24.10751171": "Cisco Systems, Inc",
    "24.8677438": "Intel Corporate",
    "12.346864627712": "Jiskoot Ltd",
    "12.123917685358592": "ES Industrial Systems Co., Ltd.",
    "12.346859319296": "DigiSensory technologies Pty Ltd",
    "24.9476374": "ELVEES NeoTek OJSC",
    "24.3705936": "LG Electronics",
    "24.4483630": "Huawei Technologies Co.,Ltd",
    "24.12634595": "Tp-Link Technologies Co.,Ltd.",
    "12.346867998720": "Aqua Management",
    "24.12588690": "China Mobile Group Device Co.,Ltd.",
    "12.123917684654080": "RBS Netkom GmbH",
    "12.346852245504": "Canal +",
    "24.15489907": "Advanced & Wise Technology Corp.",
    "24.6298895": "Midnite Solar",
    "24.13900488": "Nippon Printer Engineering",
    "24.526791": "Zhuhai Unitech Power Technology Co., Ltd.",
    "24.1102730": "Samsung Electronics Co.,Ltd",
    "24.16537674": "xiamenshi c-chip technology co., ltd",
    "24.3209547": "Universal Electronics, Inc.",
    "24.13410841": "Shenzhen Along Investment Co.,Ltd",
    "12.346858041344": "Sapec",
    "24.12066218": "LG Electronics (Mobile Communications)",
    "12.71297884721152": "Servo-Robot Inc.",
    "20.277958194364416": "Suzhou lehui display co.,ltd",
    "12.346861780992": "Exatrol Corporation",
    "24.11599589": "Huawei Device Co., Ltd.",
    "24.13147970": "VDII Innovation AB",
    "24.1095443": "Private",
    "12.123917688299520": "Thema",
    "24.3205670": "Sycada BV",
    "24.10801106": "Ubee Interactive Co., Limited",
    "20.242213043830784": "Astrogate Inc.",
    "24.3410483": "D-Link International",
    "24.3696787": "Intel Corporate",
    "24.7122571": "IBM Corporation",
    "20.194363548237824": "shanghai University Ding-Tech software Corp.,ltd",
    "24.4782335": "Chengdu Kt Electronic Hi-Tech Co.,Ltd",
    "24.10250551": "Qorvo International Pte. Ltd.",
    "20.277981927833600": "Link (Far-East) Corporation",
    "24.12862056": "Crestron Electronics, Inc.",
    "24.10257025": "vivo Mobile Communication Co., Ltd.",
    "24.13664": "Rosen Aviation",
    "24.2140983": "Apple, Inc.",
    "20.39672892882944": "Teknic, Inc.",
    "12.346867372032": "Adaptive Technologies, Inc.",
    "12.123917683986432": "neocontrol soluções em automação",
    "24.6075454": "Huawei Technologies Co.,Ltd",
    "24.6089550": "R. STAHL HMI Systems GmbH",
    "12.123917683437568": "Fischer Block, Inc.",
    "24.803073": "Ruichi Auto Technology (Guangzhou) Co., Ltd.",
    "24.4120": "Broadcom",
    "24.4121": "SIRONA DENTAL SYSTEMS GmbH & Co. KG",
    "24.4122": "PictureTel Corp.",
    "24.4123": "Cornet Technology, Inc.",
    "24.4124": "Ohm Technologies Intl, Llc",
    "24.4125": "Winbond Electronics Corp.",
    "24.4126": "Matsushita Electronic Instruments Corp.",
    "24.4127": "Cisco Systems",
    "24.4128": "Hand Held Products Inc",
    "24.4129": "Encanto Networks, Inc.",
    "20.84372893990912": "Humanplus Intelligent Robotics Technology Co.,Ltd.",
    "20.216145531502592": "KAT Mekatronik Urunleri AS",
    "24.3479": "ABB Inc./Tropos",
    "24.3478": "Vtera Technology Inc.",
    "24.3477": "Opti-cell, Inc.",
    "24.3476": "AFAR Communications,Inc",
    "24.3475": "Apple, Inc.",
    "24.3474": "ARIMA Communications Corp.",
    "24.3473": "Eclipse (HQ Espana) S.L.",
    "24.3472": "Factum Electronics AB",
    "24.3471": "King Tsushin Kogyo Co., LTD.",
    "24.3470": "Koden Electronics Co., Ltd.",
    "24.13370362": "Technicolor CH USA Inc.",
    "24.12084156": "ROBOTIS, Co, Ltd",
    "24.4765216": "ROLI Ltd.",
    "24.7379964": "Bryton Inc.",
    "24.15220476": "ARRIS Group, Inc.",
    "24.9198328": "Beijing Xiaomi Electronics Co., Ltd.",
    "12.123917683417088": "International Super Computer Co., Ltd.",
    "24.4741695": "Wisol",
    "24.4698": "Microsoft Corporation",
    "24.4699": "Kaimei Electroni",
    "24.4692": "Spectra Technologies Holdings Company Ltd",
    "24.4693": "NetEffect Incorporated",
    "24.4690": "Citronix, LLC",
    "24.4691": "AudioDev AB",
    "24.4696": "TechVoIP Sp z o.o.",
    "24.4697": "Thermo Electron Karlsruhe",
    "24.4694": "Lg Information & Comm.",
    "24.4695": "LeapComm Communication Technologies Inc.",
    "24.5263931": "Nokia",
    "12.123917687623680": "RCH Vietnam Limited Liability Company",
    "12.346861690880": "CHK GridSense P/L",
    "24.6864299": "Cisco Systems, Inc",
    "24.1849019": "Apple, Inc.",
    "24.6085782": "Arcadyan Technology Corporation",
    "12.346858307584": "RM Michaelides Software & Elektronik GmbH",
    "12.71297884860416": "Northern Star Technologies",
    "12.346854674432": "Belik S.P.R.L.",
    "24.10267163": "Microsoft Corporation",
    "20.180577509572608": "BOI Solutions",
    "24.15471579": "Juniper Networks",
    "12.346853564416": "CommServ Solutions Inc.",
    "12.123917681848320": "Volution Group UK",
    "12.346859720704": "Ant Lamp, Inc",
    "24.6324729": "Helium Systems, Inc",
    "12.123917685866496": "Molekule",
    "24.822730": "Samsung Electronics Co.,Ltd",
    "24.7358711": "Nintendo Co.,Ltd",
    "12.346867105792": "Ecrin Systems",
    "12.123917685379072": "Antek Technology",
    "12.346859913216": "Argus-Spectrum",
    "24.3182664": "Apple, Inc.",
    "20.194363552432128": "Astyx GmbH",
    "24.5254562": "Intel Corporate",
    "24.14712659": "Techboard Srl",
    "24.10764327": "zte corporation",
    "24.14981753": "Apple, Inc.",
    "12.123917683687424": "Aldridge Electrical Industries",
    "24.12889377": "MEDIAEDGE Corporation",
    "24.10759052": "Netgear",
    "24.9200651": "Beyond Devices d.o.o.",
    "12.71297883721728": "Peek Traffic Corporation",
    "24.8920213": "Itel Mobile Limited",
    "12.71297884827648": "Forza Silicon Corp.",
    "24.5506916": "Huawei Device Co., Ltd.",
    "24.9203063": "Huawei Technologies Co.,Ltd",
    "12.346866212864": "Uvax Concepts",
    "24.7920135": "Zhejiang Tmall Technology Co., Ltd.",
    "12.123917690003456": "Midfin Systems",
    "24.584949": "Cisco Systems, Inc",
    "24.11860964": "Ubiquiti Networks Inc.",
    "24.11049436": "i-TOP DESING TECHNOLOGY CO.,LTD",
    "24.15791684": "Whitesky Science & Technology Co.,Ltd.",
    "24.15745016": "R L Drake",
    "24.7355497": "Apple, Inc.",
    "24.4497145": "vivo Mobile Communication Co., Ltd.",
    "24.15998994": "Cuciniale GmbH",
    "24.11055240": "eero inc.",
    "24.7087670": "Huawei Technologies Co.,Ltd",
    "12.123917676380160": "YUYAMA MFG Co.,Ltd",
    "12.123917685870592": "Position Imaging",
    "12.346858889216": "ABB Transmission and Distribution Automation Equipment (Xiamen) Co., Ltd.",
    "24.16010979": "Intel Corporate",
    "20.17622843260928": "Beijing Sooall Information Technology Co.,Ltd",
    "24.10499743": "Intel Corporate",
    "20.452709056512": "Hangzhou Wise IOT Technology Co.,Ltd",
    "12.123917689307136": "Crde",
    "24.8433787": "zte corporation",
    "24.659": "Solid Data Systems",
    "24.658": "Logic Innovations, Inc.",
    "12.123917689278464": "Electroimpact, Inc.",
    "24.653": "Movita Technologies, Inc.",
    "24.652": "Micrel-Synergy Semiconductor",
    "24.651": "VDSL Systems OY",
    "24.650": "Ambit Microsystems Corporation",
    "24.657": "Open Network Co., Ltd.",
    "24.656": "Woorigisool, Inc.",
    "24.655": "Globetek, Inc.",
    "24.654": "Rapid 5 Networks, Inc.",
    "24.4469903": "Mxt Industrial Ltda",
    "24.278697": "New H3C Technologies Co., Ltd",
    "12.346865233920": "AIM Co.,Ltd",
    "12.346865164288": "KS Beschallungstechnik GmbH",
    "20.211670368518144": "Huaxin SM Optics Co. LTD.",
    "24.7578": "Godex International Co., Ltd",
    "24.7579": "Hokuyo Automatic Co., Ltd.",
    "24.12068460": "ANOV France",
    "24.7572": "Climax Technology Co., Ltd",
    "24.7573": "Flash, Inc.",
    "24.7570": "Micro-Star Int'L Co.,Ltd.",
    "24.7571": "Modacom",
    "24.7576": "Nokia Danmark A/S",
    "24.7577": "Cyan Optic, Inc.",
    "24.7574": "WatchGuard Video",
    "24.7575": "Alertus Technologies LLC",
    "12.346866642944": "MCS Engenharia ltda",
    "24.14691903": "Huawei Device Co., Ltd.",
    "24.14220943": "IDFone Co., Ltd.",
    "24.7088533": "Intel Corporate",
    "20.88914836586496": "Langogo Technology Co., Ltd.",
    "12.346864943104": "Wagner Group GmbH",
    "24.12594172": "iNES. applied informatics GmbH",
    "12.71297884446720": "Devriecom B.V.",
    "12.123917689057280": "ALPHA Corporation",
    "12.123917678211072": "Cardinal Scales Manufacturing Co",
    "24.13382976": "Technicolor CH USA Inc.",
    "20.167860152303616": "dots Inc.",
    "24.4490496": "Intel Corporate",
    "24.7762": "Apple, Inc.",
    "24.2430": "Imi Technology Co., Ltd",
    "24.2431": "Vsecure 2000 LTD.",
    "24.2432": "Power Zenith Inc.",
    "24.2433": "Newport Networks",
    "24.2434": "Loewe Opta GmbH",
    "24.2435": "GlobalTop Technology, Inc.",
    "24.2436": "MyCasa Network Inc.",
    "24.2437": "Auto Telecom Company",
    "24.2438": "Metalink LTD.",
    "24.2439": "Nishi Nippon Electric Wire & Cable Co.,Ltd.",
    "12.346852995072": "Studiel",
    "12.346858524672": "Brookhaven National Laboratory",
    "24.1168": "Optical Access",
    "24.1167": "TD Systems Corporation",
    "24.1166": "Ohm Tech Labs, Inc.",
    "24.1165": "Teo Technologies, Inc",
    "24.1164": "Nayna Networks, Inc.",
    "24.1163": "Poscon Corporation",
    "24.1162": "Temia Vertriebs GmbH",
    "24.2903417": "Weifang Goertek Electronics Co.,Ltd",
    "24.1160": "Eurotherm Controls",
    "24.9984329": "Luxottica Group S.P.A.",
    "24.1352495": "Huawei Device Co., Ltd.",
    "12.123917686456320": "Paratec Ltd.",
    "20.39919499083776": "UniMAT Automation Technology Co., Ltd.",
    "24.9984325": "Texas Instruments",
    "24.9748536": "Tallac Networks",
    "24.5782489": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.16007719": "S & S Research Inc.",
    "12.71297884876800": "Patria Aviation Oy",
    "24.2935876": "Sony Interactive Entertainment Inc.",
    "24.6229": "Aeromaritime Systembau GmbH",
    "24.6228": "Argard Co., Ltd",
    "12.123917689962496": "BrainboxAI Inc",
    "20.158667127128064": "Showtacle s.r.o.",
    "24.6221": "Netgear",
    "24.6220": "Bogen Communications",
    "24.6223": "8 Ways Technology Corp.",
    "24.6222": "Lianhe Technologies, Inc.",
    "24.6225": "SWsoft",
    "24.6224": "Secfone Kft",
    "24.6227": "Atera Networks LTD.",
    "24.6226": "StorLink Semiconductors, Inc.",
    "12.123917689229312": "FoodALYT GmbH",
    "24.14474466": "Samsung Electronics Co.,Ltd",
    "24.7643878": "Cisco Systems, Inc",
    "12.123917675413504": "Ab Precision (Poole) Ltd",
    "24.1373912": "Intel Corporate",
    "24.12351070": "Samsung Electronics Co.,Ltd",
    "24.13666713": "APCON, Inc.",
    "12.346856087552": "Pauly Steuer- und Regelanlagen GmbH & Co. KG",
    "12.123917682450432": "MB connect line GmbH Fernwartungssysteme",
    "24.1852441": "Tp-Link Technologies Co.,Ltd.",
    "24.11821624": "Exablox",
    "24.37626": "Shenzhen Wisky Technology Co.,Ltd",
    "24.13165157": "Intel Corporate",
    "24.9876": "Senscient Ltd",
    "24.2946245": "Avaya Inc",
    "12.123917687193600": "Season Electronics Ltd",
    "24.10757341": "Cambrionix Ltd",
    "24.9733837": "Seoul Electronics&Telecom",
    "24.1573747": "Dell Inc.",
    "20.145383535673344": "Hino Engineering, Inc",
    "20.242213050122240": "Happiest Baby Inc.",
    "24.9757549": "Texas Instruments",
    "24.15498513": "vivo Mobile Communication Co., Ltd.",
    "24.11848708": "Piranti",
    "24.11057262": "LG Electronics (Mobile Communications)",
    "12.123917683707904": "Quantum Design Inc.",
    "12.346861797376": "NETA Elektronik AS",
    "24.13909374": "Micro-Star Int'l Co, Ltd",
    "24.3734680": "Intel Corporate",
    "24.8656742": "Weifang Goertek Electronics Co.,Ltd",
    "12.123917676609536": "Birket Engineering",
    "24.13894861": "ARRIS Group, Inc.",
    "24.5277624": "Nuvyyo Inc",
    "24.12072664": "Videoswitch",
    "20.31346176032768": "KeyWest Networks, Inc",
    "12.346856206336": "Osmetech Inc.",
    "24.9454854": "Tokyo Boeki Medisys Inc.",
    "24.138": "Datahouse Information Systems",
    "24.139": "Infotron",
    "12.346862104576": "Smartquantum Sa",
    "24.137": "Cayman Systems",
    "24.134": "Megahertz Corporation",
    "24.135": "Hitachi, Ltd.",
    "24.132": "Supernet",
    "24.133": "Canon Inc.",
    "24.130": "Lectra Systemes Sa",
    "24.131": "Tadpole Technology  [had Optical Data Systems which is wrong according to both]",
    "24.8404950": "bytes at work AG",
    "24.5539036": "zte corporation",
    "24.10277406": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13164412": "Nokia Corporation",
    "24.14721964": "Huawei Technologies Co.,Ltd",
    "24.10266133": "Tp-Link Technologies Co.,Ltd.",
    "12.346866442240": "Vocality International Ltd",
    "20.145735986184192": "Chongqing Huayi Kangdao Technology Co.,Ltd.",
    "20.89058715893760": "Shenzhen Kingvt Electronics Co.,Ltd",
    "12.123917688713216": "IRT Technologies",
    "24.10768169": "IEEE Registration Authority",
    "24.13114204": "BCTech",
    "24.3708886": "Hangzhou H3C Technologies Co., Limited",
    "24.9490197": "Nokia Corporation",
    "12.346854686720": "Parabit Systems",
    "12.346852294656": "Icon Research Ltd.",
    "24.4313": "Ibm Japan, Fujisawa Mt+D",
    "12.346854780928": "Air Broadband Communications, Inc.",
    "24.16039848": "Juniper Networks",
    "12.346858950656": "Industrial Vacuum Systems",
    "24.4469138": "Hewlett Packard",
    "20.84197040455680": "Felten Electronics",
    "24.2670083": "TCT mobile ltd",
    "24.3674352": "Aruba, a Hewlett Packard Enterprise Company",
    "24.4317": "Enable Semiconductor, Inc.",
    "24.12607898": "IEEE Registration Authority",
    "20.88010610704384": "Ewin Technology Limited",
    "24.13894911": "Juniper Networks",
    "24.4512846": "Itel Mobile Limited",
    "24.2121533": "Cambridge Communication Systems Ltd",
    "24.11861644": "Centro Sicurezza Italia SpA",
    "24.7107303": "Apple, Inc.",
    "20.136823641735168": "dogtra",
    "12.71297884844032": "Berg Cloud Limited",
    "24.12598531": "Samsung Electronics Co.,Ltd",
    "12.123917682642944": "Granite River Labs Inc",
    "20.224420101619712": "Sichuan Dianjia network technology Co.Ltd.",
    "24.8439467": "Technicolor CH USA Inc.",
    "24.6603734": "Juniper Networks",
    "24.2635983": "Gen2wave",
    "24.10514585": "Beijing Huacan Electronics Co., Ltd",
    "24.2905921": "Cisco Systems, Inc",
    "20.48476181233664": "Shanghai Walktech Information Technology Co.,Ltd.",
    "20.180680384315392": "Ubisafe Smart Devices",
    "24.5541210": "Intel Corporate",
    "24.550734": "Hisense broadband multimedia technology Co.,Ltd",
    "24.16264216": "2Wire Inc",
    "24.3732270": "Skullcandy",
    "12.123917682118656": "Redstone Sunshine(Beijing)Technology Co.,Ltd.",
    "24.12636067": "Creative Electronics & Software, Inc.",
    "24.3437357": "Cisco Systems, Inc",
    "24.8182680": "Askey Computer Corp",
    "24.9229864": "Ikor Metering",
    "12.346855370752": "Digicontrole Lda",
    "24.14954326": "OptoMET GmbH",
    "24.1629911": "Huawei Technologies Co.,Ltd",
    "24.15227601": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917689040896": "Crde",
    "12.123917691215872": "Thermo Fisher Scientific",
    "24.9468082": "Xiaomi Communications Co Ltd",
    "12.346862907392": "LHA Systems CC",
    "20.176094524211200": "Imeshx Corporation Limited",
    "20.79602623971328": "CaptionCall",
    "12.346866176000": "Globalcom Engineering Srl",
    "24.1896": "Danfoss A/S",
    "12.123917684834304": "Horizon Telecom",
    "12.123917680619520": "Procon Electronics Pty Ltd",
    "24.12631505": "Samsung Electro-Mechanics(Thailand)",
    "12.123917676146688": "SysCom Automationstechnik GmbH",
    "24.24578": "Screen Subtitling Systems, Ltd",
    "24.24579": "Teraoka Weigh System Pte, Ltd.",
    "20.53091438166016": "IK Elektronik GmbH",
    "12.123917689458688": "Sensoterra BV",
    "24.1591512": "Fastback Networks",
    "24.24576": "Xycom Inc.",
    "24.24577": "InnoSys, Inc.",
    "12.123917681807360": "Acrodea, Inc.",
    "24.1890": "Group Sense Limited",
    "12.346859823104": "Tronico",
    "12.123917680967680": "Qxperts Italia S.r.l.",
    "24.14705204": "Texas Instruments",
    "24.6030137": "Huawei Technologies Co.,Ltd",
    "20.154055553843200": "Raychem RPG PVT. LTD.",
    "12.123917683167232": "Council Rock",
    "12.346866065408": "Hagenuk KMT GmbH",
    "12.346860314624": "Lyngdorf Audio Aps",
    "24.5262462": "The 41St Institute Of Cetc",
    "12.346865967104": "Millennium Electronics Pty.Ltd.",
    "24.6322033": "Texas Instruments",
    "24.6087438": "ARRIS Group, Inc.",
    "12.123917675024384": "CT Company",
    "12.346860437504": "Meta Vision Systems Ltd.",
    "24.20622": "Optimation, Inc.",
    "20.167152474652672": "Shanghai Jinnian information technology Co. Ltd",
    "24.20623": "Asita Technologies Int'L Ltd.",
    "24.9452723": "Edgecore Networks Corporation",
    "24.20620": "Rsi Systems",
    "24.2377": "Glyph Technologies Inc.",
    "24.2376": "Vista Control Systems, Corp.",
    "24.2375": "Aztek, Inc.",
    "12.346864287744": "NAONWORKS Co., Ltd",
    "24.2373": "Palmmicro Communications Inc",
    "24.2372": "Cisco Systems, Inc",
    "24.2371": "Cisco Systems, Inc",
    "24.2370": "Wireless Technologies, Inc",
    "12.123917680422912": "GE Aviation Cheltenham",
    "24.20626": "Rigaku Corporation Osaka Plant",
    "24.4517424": "Budelmann Elektronik GmbH",
    "24.2379": "FillFactory NV",
    "24.2378": "Homenet Communications",
    "24.2884021": "Cisco Systems, Inc",
    "24.20624": "Dctri",
    "20.75247155937280": "Sichuan subao network technology ltd.co.",
    "12.346854211584": "Covia, Inc",
    "24.20625": "Netaccess, Inc.",
    "24.5028296": "Conet Co., Ltd.",
    "24.5555221": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "12.123917679366144": "Redco Audio Inc",
    "24.2660939": "SteelSeries ApS",
    "24.9699657": "AutoHotBox",
    "12.346856534016": "Vlsip Technologies, Inc.",
    "24.7631": "ARRIS Group, Inc.",
    "24.7630": "ARRIS Group, Inc.",
    "24.7633": "ARRIS Group, Inc.",
    "24.7632": "ARRIS Group, Inc.",
    "24.7635": "ARRIS Group, Inc.",
    "24.7634": "ARRIS Group, Inc.",
    "24.7637": "ARRIS Group, Inc.",
    "24.7636": "ARRIS Group, Inc.",
    "24.7639": "Algolith",
    "24.7638": "ARRIS Group, Inc.",
    "24.13663717": "Forward Pay Systems, Inc.",
    "24.4194307": "Net Ware (?)",
    "24.8679719": "Silicon Laboratories",
    "24.2152994": "Huawei Technologies Co.,Ltd",
    "12.346868838400": "Erhardt+Leimer GmbH",
    "24.3450337": "Sensorist ApS",
    "24.3210132": "Shanghai Fangzhiwei Information Technology CO.,Ltd.",
    "24.7903590": "Musilab Electronics (DongGuan)Co.,Ltd.",
    "24.3409743": "AccelStor, Inc.",
    "12.346856865792": "AUCONET GmbH",
    "24.3165310": "Panasonic Electric Works Automation Controls Techno Co.,Ltd.",
    "24.2927088": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5260104": "ThingsMatrix Inc.",
    "24.10240380": "Hame Technology Co., Limited",
    "12.123917684535296": "KSJ Co.Ltd",
    "24.6075382": "Human, Incorporated",
    "12.119269826560": "Arnouse Digital Devices, Corp.",
    "24.16273325": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "12.346853875712": "ABB Switzerland Inc",
    "24.12633378": "ARRIS Group, Inc.",
    "24.10761507": "Guangdong Donyan Network Technologies Co.,Ltd.",
    "24.12633376": "Ruckus Wireless",
    "24.5252752": "Fujitsu Limited",
    "20.114869202321408": "PCTEL, Inc.",
    "12.346863034368": "Ansen Investment Holdings Ltd.",
    "24.8972798": "Apple, Inc.",
    "24.11538497": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13378072": "InnoDigital Co., Ltd.",
    "24.7896930": "GiK mbH",
    "12.123917686226944": "SYS TEC electronic GmbH",
    "12.123917686853632": "Avidbots Corporation",
    "24.2125468": "Samsung Electronics Co.,Ltd",
    "24.10540949": "Apple, Inc.",
    "24.13117490": "Beijing Xiaomi Electronics Co., Ltd.",
    "24.14457938": "Sapphire Technology Limited.",
    "12.346862366720": "OMICRON electronics GmbH",
    "20.128612174921728": "Enercon Technologies",
    "24.5795542": "Private",
    "12.123917684547584": "Fujian Landfone Information Technology Co.,Ltd",
    "12.123917687197696": "Cathwell AS",
    "24.9199847": "Mpgio Co.,Ltd",
    "24.7405430": "Texas Instruments",
    "24.6608832": "Huawei Device Co., Ltd.",
    "12.123917678272512": "Gillam-Fei S.A.",
    "20.264625144725504": "Simprints Technology Ltd",
    "24.3171820": "Wistron (ChongQing)",
    "24.1098667": "Huawei Technologies Co.,Ltd",
    "24.8705744": "Amazon Technologies Inc.",
    "24.7401040": "Netatmo",
    "12.346856927232": "ads-tec GmbH",
    "24.8186236": "Itel Mobile Limited",
    "20.132811190173696": "Hubei Boyuan Zhijia Network Media Co. Ltd.",
    "12.71297884102656": "Keocko Holding Kft.",
    "20.36069843140608": "Cuculus GmbH",
    "24.12095199": "Zencheer Communication Technology Co., Ltd.",
    "12.119269314560": "Corporate Systems Engineering",
    "24.12858118": "China Mobile Group Device Co.,Ltd.",
    "24.12338228": "Shenzhen TINNO Mobile Technology Corp.",
    "24.14953851": "China Mobile IOT Company Limited",
    "12.346858012672": "Pixel Velocity, Inc",
    "24.12120192": "Sps, Inc.",
    "20.228844758892544": "UAB &quot;SALDA&quot;",
    "24.32788": "Esprit Systems",
    "24.32789": "Seiko Systems, Inc.",
    "24.32786": "IMS Corp.",
    "24.32787": "Thomas Conrad Corp.",
    "24.32784": "Commodore",
    "24.32785": "Digital Systems Int'L. Inc.",
    "24.32782": "Atlantix Corporation",
    "24.32783": "Standard Microsystems",
    "24.32780": "Videcom Limited",
    "24.32781": "Vosswinkel FU",
    "24.2893509": "zte corporation",
    "20.141720413011968": "Akenori PTE LTD",
    "24.11296746": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.3416329": "Jensen Scandinavia AS",
    "12.123917690540032": "Przemyslowy Instytut Automatyki i Pomiarow",
    "12.346857119744": "Mistletoe Technologies",
    "12.346853101568": "Clearwater Networks",
    "12.346860253184": "Tattile srl",
    "24.6091772": "IBM Corp",
    "24.14742393": "Vaddio",
    "24.32250": "Volkswagen Group of America",
    "12.346865434624": "Systems With Intelligence Inc.",
    "24.9442266": "Athom B.V.",
    "24.14420762": "ADB Broadband Italia",
    "24.10754868": "China Mobile Group Device Co.,Ltd.",
    "12.123917676822528": "Flextronics International Kft",
    "24.7092868": "Shenzhen Aero-Startech. Co.Ltd",
    "24.3422383": "Inlab Software GmbH",
    "24.15235233": "Beoview Intercom Doo",
    "24.16049528": "LLC Proizvodstvennaya Kompania \"TransService\"",
    "24.11297141": "Huawei Technologies Co.,Ltd",
    "24.4992766": "Shenzhen Comnect Technology Co.,LTD",
    "24.7900663": "Hitron Technologies. Inc",
    "20.162752335183872": "Dongguan CXWE Technology Co.,Ltd.",
    "12.123917676503040": "Jemac Sweden AB",
    "24.3159987": "Huawei Technologies Co.,Ltd",
    "24.15487708": "WORLD MEDIA AND TECHNOLOGY Corp.",
    "12.123917679869952": "DEUTA-WERKE GmbH",
    "12.123917688623104": "iLOQ Oy",
    "24.14171241": "Shenzhen TINNO Mobile Technology Corp.",
    "12.123917685669888": "Tlv Co., Ltd.",
    "24.10803402": "Hong Kong Water World Technology Co. Limited",
    "24.12332790": "Huawei Device Co., Ltd.",
    "12.123917684969472": "Zamir Recognition Systems Ltd.",
    "24.4216332": "In Home Displays Ltd",
    "24.2117227": "Axxana(israel) ltd",
    "12.71297883226112": "Aplex Technology Inc.",
    "24.6612610": "Apple, Inc.",
    "12.346863476736": "Soft & Control Technology s.r.o.",
    "24.1317539": "Motorola Mobility LLC, a Lenovo Company",
    "24.14163677": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.4781181": "TechVision Holding Internation Limited",
    "24.5040962": "Intel Corporate",
    "24.9465769": "Nanjing Putian Telecommunications Technology Co.,Ltd.",
    "24.7606963": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.2167": "Liebert-Hiross Spa",
    "12.123917679890432": "Warecube,Inc",
    "20.137215123390464": "Easy Broadband Technology Co., Ltd.",
    "12.346859642880": "Centric TSolve BV",
    "24.33018": "Rwt Gmbh",
    "24.6820397": "Special Instrument Development Co., Ltd.",
    "24.16278335": "Intel Corporate",
    "12.123917687025664": "Motec Pty Ltd",
    "24.15776968": "MaxID (Pty) Ltd",
    "24.15776969": "Pfu Limited",
    "24.3183669": "vivo Mobile Communication Co., Ltd.",
    "24.15265267": "Cisco Systems, Inc",
    "20.198611825590272": "Annapurna labs",
    "24.2362725": "Shenzhen Vsun Communication Technology Co., Ltd.",
    "20.154238423400448": "Private",
    "24.16039130": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "24.10750411": "Alfred Kaercher GmbH &amp; Co KG",
    "24.10800178": "Inficomm Co., Ltd",
    "24.557556": "Juniper Networks",
    "12.346864537600": "Highlight Parking Systems Ltd",
    "24.4988310": "Intel Corporate",
    "12.346859606016": "ennovatis GmbH",
    "24.6831618": "SIEMENS AG, Digital Factory, Motion Control System",
    "20.31372473270272": "Airsmart System Co.,Ltd",
    "24.10260173": "Amcrest Technologies",
    "20.206931526418432": "LifeSmart",
    "20.247076404330496": "Shenzhen Civicom Technology Co.,Limited",
    "24.13632491": "Texas Instruments",
    "24.9197687": "Apple, Inc.",
    "12.346857283584": "Applied Research Laboratories: UT",
    "24.12476": "Optronic AG",
    "24.12477": "Belkin Components",
    "24.12474": "Ac&T System Co., Ltd.",
    "24.12475": "CacheFlow, Inc.",
    "12.346865401856": "Kiefer technic GmbH",
    "24.12473": "Ectel",
    "24.12470": "Cisco Systems, Inc",
    "24.12471": "Teletrol Systems, Inc.",
    "24.15783158": "Lars Thrane A/S",
    "20.70559716081664": "Guangzhou Qi'an Technology Co., Ltd.",
    "24.12478": "City-Net Technology, Inc.",
    "24.12479": "Multidata Gmbh",
    "12.346864619520": "Tattile srl",
    "12.123917675757568": "Lucas-Nülle GmbH",
    "24.59563": "Meggitt Training Systems, Inc.",
    "20.368738041856": "BroadSoft, Inc.",
    "24.13942573": "SE Controls Asia Pacific Ltd",
    "20.89058720088064": "Legend Winner Limited",
    "12.346855772160": "Digital Art System",
    "24.11553797": "Fresenius Medical Care Deutschland GmbH",
    "12.123917689729024": "ENTEC Electric & Electronic Co., LTD.",
    "12.123917690572800": "AADONA Communication Pvt Ltd",
    "24.7884699": "Juniper Networks",
    "24.16548649": "Trei technics",
    "24.7924096": "RainUs Co.,Ltd",
    "24.527561": "Mission Technology Group, dba Magma",
    "20.88914843926528": "Brainware Teraherta Information Technology Co.,Ltd.",
    "24.4470147": "Apple, Inc.",
    "24.13385706": "BAE Systems, Inc",
    "20.114869206515712": "McDonald's Corporation",
    "24.4258118": "Apple, Inc.",
    "20.184991246974976": "Shenzhen ITLONG Intelligent Technology Co.,Ltd",
    "24.16055346": "Texas Instruments",
    "12.123917685227520": "Schneider Displaytechnik GmbH",
    "24.3940598": "Samsung Electronics Co.,Ltd",
    "12.119269580800": "ThinKom Solutions, Inc",
    "24.13900673": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.346854924288": "Epelsa, SL",
    "24.16537754": "Cisco Systems, Inc",
    "12.123917690851328": "Critical Link LLC",
    "24.5440": "Nortel Networks",
    "20.14223573778432": "Goerlitz AG",
    "24.5442": "Microhard S.R.L.",
    "24.5443": "Aberdeen Test Center",
    "24.5444": "coM.s.a.t. AG",
    "24.5445": "SEECODE Co., Ltd.",
    "24.5446": "ITG Worldwide Sdn Bhd",
    "24.5447": "AiZen Solutions Inc.",
    "24.5448": "Cube Technologies",
    "24.5449": "Dixtal Biomedica Ind. Com. Ltda",
    "12.346859073536": "Monarch Instrument",
    "24.7913895": "zte corporation",
    "24.1112431": "LG Electronics (Mobile Communications)",
    "24.4719": "Rayson Technology Co., Ltd.",
    "12.346859999232": "Coral Telecom Ltd",
    "24.4717": "University of California, Berkeley",
    "24.4716": "Visonic Technologies 1993 Ltd.",
    "24.4715": "Ascalade Communications Limited",
    "24.4714": "OPTOELECTRONICS Co., Ltd.",
    "24.4713": "Value Electronics",
    "24.4712": "IPS d.o.o.",
    "24.4711": "Panasonic Corporation",
    "24.4710": "Swisscom Hospitality Services SA",
    "12.123917675139072": "HGL Dynamics Ltd",
    "12.346855837696": "Realtime Engineering AG",
    "24.13120331": "Apple, Inc.",
    "24.10779668": "ChargeStorm AB",
    "24.12386283": "Motorola Mobility LLC, a Lenovo Company",
    "24.3848": "Indagon Oy",
    "24.3849": "Private",
    "24.7885870": "D-Link International",
    "24.3842": "Digicube Technology Co., Ltd",
    "24.3843": "Com&C Co., Ltd",
    "24.3840": "Legra Systems, Inc.",
    "24.3841": "Digitalks Inc",
    "24.3846": "Nortel Networks",
    "24.3847": "Mangrove Systems, Inc.",
    "24.3844": "cim-usa inc",
    "24.3845": "3B System Inc.",
    "20.250817670021120": "IDvaco Private Limited",
    "20.212188451045376": "Minato Advanced Technologies inc",
    "24.558651": "Belkin International Inc.",
    "12.123917689774080": "China Entropy Co., Ltd.",
    "24.5531473": "Compal Broadband Networks, Inc.",
    "12.346860265472": "Infodev Electronic Designers Intl.",
    "24.6299025": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917678227456": "DIEHL Controls",
    "24.11464": "Cisco Systems, Inc",
    "24.1636203": "Zhejiang Winsight Technology CO.,LTD",
    "24.57349": "Technical Corp.",
    "24.57348": "Pmc-Sierra, Inc.",
    "24.57347": "Nokia Wireless Business Commun",
    "24.57346": "Crossroads Systems, Inc.",
    "24.57345": "Strand Lighting Limited",
    "24.57344": "Fujitsu Limited",
    "20.58144486064128": "Genius Pros",
    "12.346857746432": "ATH system",
    "20.168198565527552": "ShenZhen Chuangwei Electronic Appliance Co.,Ltd",
    "24.16037674": "Time Interconnect Ltd",
    "24.3696804": "Samsung Electronics Co.,LTD",
    "24.11857668": "DAIHAN Scientific Co., Ltd.",
    "20.268341267660800": "Private",
    "20.88668802908160": "XinLian'AnBao（Beijing）Technology Co.，LTD.",
    "12.346852278272": "Tadiran Scopus",
    "12.346854858752": "AMIRIX Systems",
    "24.10284008": "BizLink (Kunshan) Co.,Ltd",
    "24.8135426": "Samsung Electronics Co.,Ltd",
    "12.123917679382528": "Rch Italia Spa",
    "12.346861723648": "Automation Tec",
    "24.6074470": "ARRIS Group, Inc.",
    "20.44768538132480": "Enedo Finland Oy",
    "24.3961279": "Espressif Inc.",
    "24.1063191": "Samsung Electronics Co.,Ltd",
    "24.10550929": "AVAT Automation GmbH",
    "24.5785634": "Sony Mobile Communications Inc",
    "12.346857902080": "Gastager Systemtechnik GmbH",
    "24.14469365": "Shanghai UMEinfo CO.,LTD.",
    "12.346863788032": "Neothings, Inc.",
    "24.3158": "Megatel Computer (1986) Corp.",
    "24.3159": "MACKIE Engineering Services Belgium BVBA",
    "24.15762479": "ADB Broadband Italia",
    "24.571999": "Qingdao Hisense Electronics Co.,Ltd.",
    "24.14479306": "Murata Manufacturing Co., Ltd.",
    "24.2106834": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.3151": "UDTech Japan Corporation",
    "24.3152": "Seagate Technology",
    "20.5297552752640": "PacPort Corporation",
    "24.3154": "Roll Systems Inc.",
    "24.3155": "Private",
    "24.3156": "Pedestal Networks, Inc",
    "24.3157": "Microlink Communications Inc.",
    "24.547552": "Recovision Technology Co., Ltd.",
    "24.3418992": "Arris",
    "20.224447338381312": "Manufacturas Y Transformados Ab, S.L.",
    "24.10288113": "MESOMATIC GmbH & Co.KG",
    "24.7648638": "zte corporation",
    "24.4488853": "Alpha and Omega, Inc",
    "24.1573882": "IBT Interfaces",
    "24.12073057": "Cisco Systems, Inc",
    "24.14693604": "Feit Electric Company, Inc.",
    "24.23923": "Cisco Systems, Inc",
    "24.6528": "Gridpoint Systems",
    "24.6529": "Vivox Inc",
    "24.6522": "MAZeT GmbH",
    "24.6523": "Picotest Corp.",
    "24.6520": "Datum Systems, Inc.",
    "24.6521": "Nokia Danmark A/S",
    "24.6526": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.6527": "Plantronics, Inc.",
    "24.6524": "Riedel Communications GmbH",
    "24.6525": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.123917690630144": "Blue-Solutions Canada Inc.",
    "24.14169915": "Shenzhen Jingxun Software Telecommunication Technology Co.,Ltd",
    "20.154680184274944": "Precitec Optronik GmbH",
    "12.123917680087040": "Coda Octopus Products Limited",
    "24.6035637": "Talon Communications",
    "24.11275774": "Ekon GmbH - myGEKKO",
    "24.585197": "Zhejiang Dahua Technology Co., Ltd.",
    "24.665": "Apex, Inc.",
    "12.346865340416": "Media Technologies Ltd.",
    "24.7870766": "NASCENT Technology",
    "24.11811124": "Pen Generations, Inc.",
    "24.758": "Equipe Communications",
    "24.759": "Arm",
    "24.756": "PCTEL, Inc.",
    "24.757": "VIVE Synergies, Inc.",
    "24.754": "eDevice, Inc.",
    "24.755": "Media Serve Co., Ltd.",
    "24.752": "AME Optimedia Technology Co., Ltd.",
    "24.753": "Pinetron Co., Ltd.",
    "24.750": "Nokia Danmark A/S",
    "24.751": "CCC Network Systems Group Ltd.",
    "24.1839928": "Kimball Electronics Group, LLC",
    "24.8178961": "MS-Magnet",
    "24.20299": "Pine Technology Ltd.",
    "12.123917690806272": "Incoil Induktion AB",
    "20.180659910868992": "Halliburton",
    "24.12599585": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.20297": "Realtek",
    "24.4774550": "Eddy Smart Home Solutions Inc.",
    "24.3955908": "Amazon Technologies Inc.",
    "24.7439": "Tp-Link Technologies Co.,Ltd.",
    "24.7438": "Agapha Technology co., Ltd.",
    "24.7437": "Sony Interactive Entertainment Inc.",
    "24.7436": "MobileCompia",
    "12.346860634112": "Telvent",
    "24.7434": "Davis Instruments, Inc.",
    "24.7433": "Dell Inc.",
    "24.7432": "Jiangsu Yinhe Electronics Co.,Ltd.",
    "24.7431": "Shenzhen Sang Fei Consumer Communications Co.,Ltd",
    "12.346855972864": "Vegas Amusement",
    "24.13901169": "Palo Alto Networks",
    "12.123917678206976": "Private",
    "24.2609": "HCV Consulting",
    "24.2608": "Visteon Corporation",
    "24.6386": "Beijing Tianyu Communication Equipment Co., Ltd",
    "24.6387": "ASUSTek COMPUTER INC.",
    "24.6384": "JOYTOTO Co., Ltd.",
    "24.6385": "Chunichi Denshi Co.,LTD.",
    "24.6382": "Videology Imaging Solutions, Inc.",
    "24.6383": "Escape Communications, Inc.",
    "24.6380": "Welding Technology Corporation",
    "24.6381": "Accutech Ultrasystems Co., Ltd.",
    "24.16527168": "Calxeda, Inc.",
    "24.9729": "Cutera Inc",
    "24.9728": "TEAC Australia Pty Ltd.",
    "12.346858528768": "IHP-GmbH",
    "24.9725": "OBR Centrum Techniki Morskiej S.A.",
    "24.9724": "Enda Endustriyel Elektronik Ltd. Sti.",
    "24.9727": "CreNova Multimedia Co., Ltd",
    "24.9726": "Pilot Electronics Corporation",
    "24.9721": "GMK electronic design GmbH",
    "24.9723": "Tunstall Healthcare A/S",
    "24.9722": "J&M Analytik AG",
    "24.5879": "L-3 Communications, Aviation Recorders",
    "24.5878": "Video Products Group",
    "24.1558": "Tel Net Co., Ltd.",
    "24.1559": "Redswitch Inc.",
    "24.5871": "Koko Fitness, Inc.",
    "24.5870": "Royaldigital Inc.",
    "24.5873": "OmniSense, LLC",
    "24.5872": "Dell",
    "24.5875": "CAST Information Co., Ltd",
    "24.5874": "Dmobile System Co., Ltd.",
    "24.5877": "Dalian Golden Hualu Digital Technology Co.,Ltd",
    "24.5876": "Eidicom Co., Ltd.",
    "24.6593678": "Texas Instruments",
    "12.123917690949632": "carbonTRACK",
    "24.4482901": "Orbit Irrigation",
    "24.16517592": "ACTEON Group",
    "20.57194438459392": "Nanjing Mythware Information Technology Co., Ltd.",
    "12.123917678489600": "Digital Outfit",
    "20.180680391655424": "Dongguan HuaFuu industrial co., LTD",
    "20.268735387533312": "Hefei STAROT Technology Co.,Ltd",
    "24.16572806": "A3M Co., LTD",
    "24.2908762": "Finest Security Systems Co., Ltd",
    "12.123917686435840": "E-T-A Elektrotechnische Apparate GmbH",
    "20.260167868350464": "Shenzhen Compare Electronics Co., Ltd",
    "24.3471591": "Cisco Systems, Inc",
    "24.14719345": "Samsung Electronics Co.,Ltd",
    "24.11798521": "Samsung Electro Mechanics Co., Ltd.",
    "12.123917691359232": "Tex Computer Srl",
    "24.10283345": "Shenzhen Sang Fei Consumer Communications Ltd., Co.",
    "24.13411860": "Aifa Technology Corp.",
    "24.1352352": "Cisco Systems, Inc",
    "24.1590852": "Home Control Singapore Pte Ltd",
    "24.3466282": "Automatic Bar Controls Inc.",
    "24.11820368": "Aruba, a Hewlett Packard Enterprise Company",
    "12.123917690142720": "DISK Multimedia s.r.o.",
    "12.123917689864192": "Fourth Frontier Technologies Private Limited",
    "24.16517622": "Guangdong Tonze Electric Co.,Ltd",
    "12.119269281792": "Advanced Scientific Concepts, Inc.",
    "24.41198": "Nashoba Networks",
    "24.14718962": "Atomax Inc.",
    "24.564": "Imperial Technology, Inc.",
    "12.346864726016": "ECS Srl",
    "24.24624": "VillageTronic",
    "24.14986121": "MitraStar Technology Corp.",
    "24.24625": "Hrk Systems",
    "24.12059300": "Aeonsemi, Inc.",
    "24.24622": "Cyclades Corporation",
    "12.123917688799232": "Toshiba Electron Tubes & Devices Co., Ltd.",
    "24.560": "Intersoft Electronics",
    "12.123917687398400": "Burk Technology",
    "24.8458": "byd:sign Corporation",
    "24.8459": "Gemini Traze Rfid Pvt. Ltd.",
    "20.277981934125056": "Teamly Digital",
    "24.8454": "RIM Testing Services",
    "24.8455": "Seowonintech Co Ltd.",
    "24.8456": "Nokia Danmark A/S",
    "24.8457": "Nokia Danmark A/S",
    "24.8450": "UpdateLogic Inc.",
    "24.8451": "GHI Electronics, LLC",
    "24.8452": "Gigaset Communications GmbH",
    "24.8453": "Alcatel-Lucent IPD",
    "24.16264640": "ARRIS Group, Inc.",
    "24.5797573": "Digi I'S Ltd",
    "12.123917690322944": "ATG UV Technology",
    "24.11023994": "frogblue TECHNOLOGY GmbH",
    "24.5127": "Sperian Protection Instrumentation",
    "24.5126": "Go Networks",
    "24.5125": "OpenIB, Inc.",
    "24.5124": "ARRIS Group, Inc.",
    "24.5123": "Renasis, LLC",
    "24.5122": "kk-electronic a/s",
    "24.5121": "Rivertree Networks Corp.",
    "24.5120": "Minerva Korea Co., Ltd",
    "24.8187786": "Inhon International Ltd.",
    "24.5129": "MAGNETI MARELLI S.E. S.p.A.",
    "24.5128": "Eka Systems Inc.",
    "24.8186": "Coretree, Co, Ltd",
    "24.8187": "Green Packet Bhd",
    "24.8184": "Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems",
    "24.8185": "Advanced Knowledge Associates",
    "24.8182": "PS Audio International",
    "24.8183": "Nakajima All Precision Co., Ltd.",
    "24.8180": "Power Monitors, Inc.",
    "24.8181": "Kongsberg Defence & Aerospace",
    "12.123917677850624": "Matrix Orbital Corporation",
    "24.8188": "Riccius+Sohn GmbH",
    "24.8189": "Indigo Mobile Technologies Corp.",
    "24.4832": "Codan Limited",
    "24.4833": "Alliant Networks, Inc",
    "24.4830": "Radio Components Sweden AB",
    "24.4831": "Novomatic AG",
    "24.4836": "ZIEHL industrie-electronik GmbH + Co KG",
    "24.4837": "Time America, Inc.",
    "24.4834": "ALAXALA Networks Corporation",
    "24.4835": "Agat Soft LLC",
    "12.346860122112": "Tiefenbach Control Systems GmbH",
    "12.346861305856": "DSS Networks, Inc.",
    "24.4838": "Spectec Computer Co., Ltd.",
    "24.4839": "Projectek Networking Electronics Corp.",
    "12.346862202880": "Unetconvergence Co., Ltd",
    "24.13914234": "CEIVA Logic, Inc",
    "24.1594575": "Samsung Electronics Co.,Ltd",
    "24.9226154": "Radinet Communications Inc.",
    "12.123917690703872": "Critical Link LLC",
    "24.6343675": "AirScape Inc.",
    "24.11818723": "Apple, Inc.",
    "24.1593035": "Samsung Electronics Co.,Ltd",
    "12.346856943616": "DENGYOSHA co.,LTD.",
    "12.123917685628928": "eSys Solutions Sweden AB",
    "20.149945506070528": "Lisle Design Ltd",
    "12.123917678129152": "Ik Multimedia Production Srl",
    "12.123917690224640": "ENTEC Electric & Electronic Co., LTD.",
    "24.16053830": "Dediprog Technology Co. Ltd.",
    "24.7667532": "Skyworth Digital Technology(Shenzhen) Co.,Ltd",
    "24.15205852": "Verifone Inc.",
    "24.9452146": "Ruckus Wireless",
    "24.56577": "Ungermann-Bass Inc.",
    "24.56576": "Ungermann-Bass",
    "24.15490153": "Mitsubishi Heavy Industries Mechatronics Systems,Ltd.",
    "24.290114": "Bryston Ltd.",
    "24.10798671": "Huawei Technologies Co.,Ltd",
    "24.56579": "Ungermann-Bass Inc.",
    "24.56578": "Ungermann-Bass Inc.",
    "24.9403": "CENTRAL Corporation",
    "24.11301132": "China Mobile Group Device Co.,Ltd.",
    "24.6561804": "Apple, Inc.",
    "24.3825": "Ezquest Inc.",
    "20.27478319955968": "CISTECH Solutions",
    "12.123917686697984": "Naval Group",
    "20.97843035504640": "Timex Group USA Inc",
    "12.123917681098752": "InterTalk Critical Information Systems",
    "24.10008297": "Nintendo Co.,Ltd",
    "20.70559707693056": "Baumer Bourdon-Haenni",
    "24.1841887": "Microsoft Corporation",
    "24.6581426": "Samsung Electronics Co.,Ltd",
    "24.9196483": "Beijing Xiaomi Mobile Software Co., Ltd",
    "12.346853777408": "Basler Electric Company",
    "24.5287558": "Linkintec Co., Ltd",
    "20.97843037601792": "Shanghai Qianjin Electronic Equipment Co. Ltd",
    "20.229631766560768": "Appotronics Co., Ltd",
    "24.6055034": "Huawei Technologies Co.,Ltd",
    "24.6058007": "Apple, Inc.",
    "24.1118481": "Private",
    "24.9735663": "Oetiker Schweiz AG",
    "24.15753818": "Realme Chongqing Mobile Telecommunications Corp.,Ltd.",
    "24.15741466": "Mita-Teknik A/S",
    "24.13152750": "Qorvo",
    "24.14168666": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.2837": "Platypus Technology",
    "12.123917690314752": "Meta Computing Services, Corp",
    "24.15522694": "Api-K",
    "24.2831": "Bosch Rexroth",
    "12.123917689364480": "Micro Electroninc Products",
    "24.4779654": "UTStarcom (China) Co.,Ltd",
    "24.7908543": "Igneous Systems",
    "12.123917684420608": "MB connect line GmbH Fernwartungssysteme",
    "24.821520": "Samsung Electronics Co.,Ltd",
    "12.346866843648": "Aplex Technology Inc.",
    "12.71297883320320": "Shanghai o-solution electronics & Technology Co., Ltd.",
    "12.346867326976": "Embedded Solution Bank Co., Ltd.",
    "12.71297883648000": "Diamond Technologies, Inc",
    "24.7929570": "Shanghai Diveo Technology Co., Ltd",
    "24.3427528": "Echodyne Corp",
    "24.8441881": "Embed, Inc",
    "24.13926116": "China Mobile IOT Company Limited",
    "20.268735383339008": "Junchuang (Xiamen) Automation Technology Co.,Ltd",
    "24.14210803": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.12365713": "TE Connectivity Sensor Solutions",
    "12.123917686661120": "Fen Systems Ltd",
    "24.14966071": "Juniper Networks",
    "24.15224180": "Huawei Technologies Co.,Ltd",
    "24.6356957": "C.E. Electronics, Inc",
    "24.7397964": "SAE IT-systems GmbH & Co. KG",
    "24.9113": "Samsung Electronics Co.,Ltd",
    "12.123917676572672": "MB connect line GmbH Fernwartungssysteme",
    "24.2146375": "Verizon",
    "24.12859603": "Tagatec Co.,Ltd",
    "24.3159250": "Availink, Inc.",
    "12.346858741760": "Trutouch Technologies Inc",
    "12.123917675692032": "Symetrics Industries d.b.a. Extant Aerospace",
    "12.346859827200": "German Technologies",
    "24.9455266": "Tedipay UK Ltd",
    "24.48993": "Samsung Electronics Co.,Ltd",
    "24.1334241": "nyantec GmbH",
    "24.832601": "Marketech International Corp.",
    "12.346852642816": "ENTTEC Pty Ltd.",
    "20.176094527356928": "Kalray S.A.",
    "12.346863775744": "Measy Electronics Co., Ltd.",
    "12.123917677912064": "AM General Contractor",
    "24.7892372": "Palo Alto Networks",
    "20.52912297345024": "Smart Component Technologies LTD",
    "24.313745": "Phistek INC.",
    "20.181256788639744": "SolidPro Technology Corporation",
    "24.8184522": "Juniper Networks",
    "24.6305216": "Sfr",
    "24.32867": "Richard Hirschmann Gmbh & Co",
    "24.32866": "Interface Co.",
    "24.32865": "Litton Systems, Inc.",
    "24.32864": "Network Interface Corporation",
    "24.32863": "Hewlett Packard",
    "24.32862": "Lsi Logic Corporation",
    "24.32861": "Canstar",
    "24.32860": "Agilis",
    "24.8416283": "Motorola Mobility LLC, a Lenovo Company",
    "24.32869": "Cybergraphic Systems Pty Ltd.",
    "24.32868": "Wyse Technology Llc",
    "20.31368129019904": "Unjo AB",
    "20.149945510264832": "Unicom Global, Inc.",
    "12.346864439296": "Mroad Information System",
    "12.346865594368": "Task Sistemas De Computacao Ltda",
    "24.7102699": "Zhi Yuan Electronics Co., Limited",
    "24.12896537": "Energy Imports Ltd",
    "24.2099806": "Xiangshan Giant Eagle Technology Developing Co., Ltd.",
    "12.346861858816": "TSB Solutions Inc.",
    "24.6302292": "CardioTek B.V.",
    "24.16552535": "Renesas Electronics (Penang) Sdn. Bhd.",
    "20.228784616767488": "Guangdong Shiqi Manufacture Co., Ltd.",
    "12.119269445632": "RTLS Ltd.",
    "24.15376": "Cisco Systems, Inc",
    "24.2378979": "Amazon Technologies Inc.",
    "24.12638836": "Hangzhou Sunyard Technology Co., Ltd.",
    "24.13898427": "Quadrant Components Inc. Ltd",
    "12.71297885159424": "STEK Ltd",
    "12.346867003392": "CONTROL SYSTEMS Srl",
    "24.5295394": "Avaya Inc",
    "12.123917689376768": "Xiamen Point Circle Technologh Co,ltd",
    "12.123917684604928": "Done Design Inc",
    "24.6827395": "Shenzhen Dinghe Communication Company",
    "24.6850279": "Orbotix, Inc.",
    "24.5466": "Dainippon Pharmaceutical Co., Ltd.",
    "20.220958189158400": "Jabil",
    "12.346855890944": "Nordic Alarm AB",
    "20.163603973931008": "Shanxi Baixin Information Technology Co., Ltd.",
    "12.346864275456": "Cemsi",
    "12.123917682888704": "Density Inc.",
    "24.1593863": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.15516685": "Nintendo Co.,Ltd",
    "24.6336365": "Samsung Electronics Co.,Ltd",
    "24.2128112": "Apple, Inc.",
    "20.211670375858176": "Shenzhen Pay Device Technology Co., Ltd.",
    "24.11315616": "Audioengine, Ltd.",
    "12.123917675536384": "Sakura Seiki Co.,Ltd.",
    "24.11284893": "Shenzhen TG-NET Botone Technology Co.,Ltd.",
    "24.1866951": "Coriant GmbH",
    "12.346867613696": "Diehl AKO Stiftung & Co. KG",
    "24.7613706": "Norfolk Elektronik AG",
    "20.216039226867712": "HHCC Plant Technology Co.,Ltd.",
    "24.8149532": "Sophos Ltd",
    "24.3672317": "Silca Spa",
    "24.12852479": "Vizio, Inc",
    "12.123917684469760": "Ningbo Crrc Times Transducer Technology Co., Ltd",
    "12.346861215744": "UAVISION Engenharia de Sistemas",
    "20.83889198465024": "Beijing AutoAi Technology co. LTD",
    "24.12621945": "Acer Inc.",
    "12.123917691260928": "Littlemore Scientific",
    "24.10247417": "Sony Mobile Communications Inc",
    "20.180698123075584": "Changsha Tai Hui Network Technology Co.,Ltd",
    "24.311873": "Nokia",
    "24.7905159": "Siemens AG I IA PP PRM",
    "24.311870": "HTC Corporation",
    "24.10026001": "Cisco-Linksys, LLC",
    "24.5016953": "Apple, Inc.",
    "24.12885836": "Huawei Technologies Co.,Ltd",
    "24.524300": "Miklyn Development Co.",
    "24.7615662": "this is engineering Inc.",
    "24.5294449": "GN Netcom A/S",
    "24.5284031": "Alba Fiber Systems Inc.",
    "24.4237032": "Cisco Systems, Inc",
    "12.346864713728": "Crtiical Link",
    "12.123917677252608": "RCD Radiokomunikace",
    "24.8708930": "Tuya (HK) Limietd",
    "24.5523968": "Cisco Systems, Inc",
    "20.228784615718912": "Shen Zhen HaiHe Hi-Tech Co., Ltd",
    "20.31368136359936": "Jiangsu ETERN COMMUNICATION Co.,ltd",
    "24.13914175": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.13377279": "Metrological Group BV",
    "24.12868995": "Samsung Electronics Co.,Ltd",
    "12.346860503040": "Wireless Cables Inc.",
    "12.346861162496": "Marine Communications Limited",
    "12.346857693184": "Universidad de Chile Facultad de Medicina",
    "20.48543325749248": "Rexgen Inc.",
    "24.8187999": "Apple, Inc.",
    "24.9970510": "Micromedia AG",
    "12.346855546880": "United Telecoms Ltd",
    "12.123917686054912": "JSC VIST Group",
    "12.346866024448": "Kyowa Electronics Co.,Ltd.",
    "20.114869213855744": "Swisscom Broadcast Ltd",
    "24.16284225": "Cisco Systems, Inc",
    "24.41032": "Questech, Ltd.",
    "24.41033": "Digitech Industries, Inc.",
    "24.41030": "Scitex Corp. Ltd.",
    "24.41031": "Integrated Fitness Corp.",
    "24.41036": "Innovative Systems & Technologies, Inc.",
    "24.41037": "Eda Instruments, Inc.",
    "24.41034": "Nisshin Electric Co., Ltd.",
    "24.41035": "Tfl Lan Inc.",
    "12.346861056000": "UAVNavigation S.L.",
    "24.41038": "Voelker Technologies, Inc.",
    "24.41039": "Ameritec Corp.",
    "24.15204406": "Befs co,. ltd",
    "24.1351484": "Samsung Electronics Co.,Ltd",
    "24.5787766": "Linear Equipamentos Eletronicos SA",
    "24.539378": "Espressif Inc.",
    "24.11810473": "Fibocom Wireless Inc.",
    "12.123917690318848": "NUMATA R&D Co.,Ltd",
    "24.3715486": "IEEE Registration Authority",
    "12.123917691027456": "Heitec Ag",
    "24.16276549": "IC Nexus Co. Ltd.",
    "12.346865225728": "Rovsing A/S",
    "24.4206695": "Conlog (Pty) Ltd",
    "12.123917675618304": "Havelsan A.Ş.",
    "24.8697111": "Cisco Systems, Inc",
    "24.9993930": "Inventus Power Eletronica do Brasil LTDA",
    "24.6823896": "Siemens Industry, Inc.",
    "12.123917676273664": "Miguel Corporate Services Pte Ltd",
    "24.14689409": "Huawei Technologies Co.,Ltd",
    "24.8936848": "IEEE Registration Authority",
    "12.123917680250880": "Peek Traffic",
    "20.154314889756672": "Chongqing beimoting technology co.ltd",
    "12.346865876992": "Saet I.S.",
    "24.15774494": "Universal Electronics, Inc.",
    "24.14479232": "Huawei Technologies Co.,Ltd",
    "24.3728906": "Xiaomi Communications Co Ltd",
    "24.809497": "Axtion Sdn Bhd",
    "12.123917686038528": "Divigraph (Pty) LTD",
    "24.24626": "I-Cube, Inc.",
    "24.4476072": "Arista Networks",
    "24.2384504": "sysmocom - systems for mobile communications GmbH",
    "24.15506036": "Hewlett Packard",
    "20.110162582962176": "Product Development Associates, Inc.",
    "12.123917680246784": "Mistral Solutions Pvt. LTD",
    "12.123917686878208": "Revolution Retail Systems, LLC",
    "12.123917688627200": "Secturion Systems",
    "12.346864640000": "Docobo Ltd",
    "24.10790569": "Private",
    "24.14183657": "Sagemcom Broadband SAS",
    "24.1630879": "vivo Mobile Communication Co., Ltd.",
    "12.346866229248": "A&T Corporation, Electrics Group , LAS R&D Unit,",
    "20.194600867201024": "Eagle Acoustics Manufacturing, LLC",
    "12.123917683806208": "Insitu, Inc.",
    "12.71297884753920": "eROCCA",
    "12.346861420544": "Internet Protocolo Lógica SL",
    "24.10268527": "Itel Mobile Limited",
    "12.346863525888": "Unisensor A/S",
    "24.3974529": "vivo Mobile Communication Co., Ltd.",
    "24.3746": "McAfee, Inc",
    "24.45275": "Nextcell, Inc.",
    "12.346856108032": "4g Technologies, L.P.",
    "24.9090010": "HomePlug Powerline Alliance, Inc.",
    "24.45279": "Starboard Storage Systems",
    "24.13672662": "Chengdu TD Tech Ltd.",
    "24.5254651": "IGShare Co., Ltd.",
    "12.123917690765312": "Szabo Software & Engineering UK Ltd",
    "24.10011602": "Ningbo Sanxing Medical Electric Co.,Ltd",
    "24.1355483": "VTech Telecommunications Ltd.",
    "24.14478185": "Technicolor CH USA Inc.",
    "24.12344396": "Elecom Co.,Ltd.",
    "24.8917046": "Panodic(ShenZhen) Electronics Limted",
    "24.5784908": "Ericsson AB",
    "20.251018408361984": "ACS-Solutions GmbH",
    "24.3743": "TEMIC SDS GmbH",
    "24.8930858": "Telechips Inc.",
    "12.346861846528": "FractureCode Corporation",
    "24.1061915": "Spacelink",
    "24.1077231": "Ad-Sol Nissin Corp",
    "24.7122678": "eero inc.",
    "20.14286626750464": "SELECTRIC Nachrichten-Systeme GmbH",
    "24.16618": "PlainTree Systems Inc",
    "24.16619": "Martin Marietta Corporation",
    "24.16616": "Charles River Data Systems,Inc",
    "24.16617": "Accord Systems, Inc.",
    "12.346863194112": "Joysystem",
    "24.16615": "Arnos Instruments & Computer",
    "24.16612": "E-M Technology, Inc.",
    "24.16613": "Sybus Corporation",
    "24.16610": "Mesa Ridge Technologies, Inc.",
    "24.16611": "Quin Systems Ltd",
    "24.11543162": "HP Inc.",
    "12.346860003328": "Smith Meter, Inc",
    "12.123917682765824": "Franklin France",
    "24.3995218": "zte corporation",
    "12.123917689819136": "Llvision Technology Co.,Ltd",
    "24.10801618": "Apple, Inc.",
    "12.123917678116864": "Tunstall A/S",
    "12.123917684686848": "GSP Sprachtechnologie GmbH",
    "24.14202551": "Comtrend Corporation",
    "24.2667771": "Sprocomm Technologies CO.,LTD.",
    "20.255191451435008": "Digital Dynamics, Inc.",
    "12.119269724160": "HPI High Pressure Instrumentation GmbH",
    "24.566728": "Sunnovo International Limited",
    "24.4987450": "PRIMA Research And Production Enterprise Ltd.",
    "24.2647216": "Huawei Device Co., Ltd.",
    "12.346852286464": "CallTech International Limited",
    "12.123917679849472": "B.P.A. Srl",
    "12.346868572160": "Hilkom digital GmbH",
    "24.2655533": "Touch Networks",
    "24.16260914": "Motorola Mobility LLC, a Lenovo Company",
    "24.7360440": "ARRIS Group, Inc.",
    "24.1826": "The Nielsen Company",
    "12.346859610112": "Logus Broadband Wireless Solutions Inc.",
    "24.1824": "Trutzschler GmbH & Co. KG",
    "24.1825": "Formac Elektronik GmbH",
    "24.1822": "Tri-M Engineering / Nupak Dev. Corp.",
    "24.1823": "European Systems Integration",
    "24.1820": "At&T",
    "24.1821": "Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A.",
    "12.123917682626560": "Tmsi Llc",
    "24.1828": "Telemax Co., Ltd.",
    "24.1829": "Bematech International Corp.",
    "12.346868649984": "Coral Telecom Ltd",
    "20.119143416201216": "ShenZhen Hugsun Technology Co.,Ltd.",
    "12.346863116288": "Gamber-Johnson LLC.",
    "24.7132720": "Rootous System Co.,Ltd",
    "24.10010260": "Ibm",
    "24.1063401": "Juniper Networks",
    "24.6823074": "Rosewill Inc.",
    "24.7896392": "Itel Mobile Limited",
    "12.346856079360": "Tyzx, Inc.",
    "24.13371307": "Samsung Electronics Co.,Ltd",
    "12.346854379520": "ESG Elektroniksystem u. Logistik GmbH",
    "20.452701716480": "Ningbo Shen Link Communication Technology Co.,Ltd",
    "24.1621939": "Samsung Electronics Co., Ltd., Memory Division",
    "24.845267": "DH electronics GmbH",
    "24.4767817": "Broad Telecom SA",
    "12.123917675929600": "JESE Ltd",
    "24.1323130": "Avermetrics",
    "12.346861387776": "LyconSys GmbH & Co.KG",
    "24.10002019": "Apple, Inc.",
    "24.16041174": "Shenzhen Xinfa Electronic Co.,ltd",
    "24.5532686": "Texas Instruments",
    "24.5808163": "Intel Corporate",
    "24.10765521": "Wingtech Group (HongKong）Limited",
    "24.13136676": "Sow Cheng Technology Co. Ltd.",
    "20.5239432282112": "SLOC GmbH",
    "20.70682226458624": "Beijing C&W Electronics(Group)Co.,Ltd",
    "24.9463733": "Alcatel-Lucent",
    "24.2670473": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.268341251932160": "realphone technology co.,ltd",
    "12.123917676896256": "Clear Flow by Antiference",
    "24.9199736": "Swissbit AG",
    "24.14698213": "Gosuncn Technology Group Co., Ltd.",
    "24.4729170": "Sagemcom Broadband SAS",
    "24.10804937": "QLove Mobile Intelligence Information Technology (W.H.) Co. Ltd.",
    "24.851075": "Airoha Technology Corp.,",
    "24.15758254": "PEAK-System Technik",
    "24.548995": "zte corporation",
    "12.123917675188224": "NEXTtec srl",
    "12.346859622400": "MG s.r.l.",
    "24.14687751": "Anite Telecoms US. Inc",
    "24.23686": "Shenzhen Fast Technologies Co.,Ltd",
    "24.4777621": "Insigma Inc",
    "12.123917676019712": "Special Services Group, LLC",
    "12.346868445184": "Vaisala Oyj",
    "12.71297884717056": "REDER Domotic GmbH",
    "24.7086763": "Ainsworth Game Technology",
    "24.7927938": "LG Electronics (Mobile Communications)",
    "12.346868158464": "FairyDevices Inc.",
    "20.203238162825216": "Visual Productions BV",
    "24.10550881": "Vivint Wireless Inc.",
    "24.9909": "ICOMM Tele Ltd",
    "24.9908": "Ford Motor Company",
    "12.123917691502592": "Twoway Communications, Inc.",
    "24.9905": "Navis Auto Motive Systems, Inc.",
    "24.9904": "Apple, Inc.",
    "24.9907": "Thales Communications Inc",
    "24.9906": "Setrix GmbH",
    "24.9901": "Arada Systems, Inc.",
    "24.9900": "Shanghai LUSTER Teraband photonic Co., Ltd.",
    "24.9903": "Duelco A/S",
    "24.9902": "Wireless Measurement Ltd",
    "24.15993304": "Samsung Electro-Mechanics(Thailand)",
    "24.4508": "EP&T Energy",
    "24.4509": "Diginfo Technology Corporation",
    "24.4506": "Alkeria srl",
    "24.4507": "Telesynergy Research Inc.",
    "24.4504": "Prism Media Products Limited",
    "24.1848439": "Innovation Wireless",
    "24.4502": "Actuality Systems, Inc.",
    "24.4503": "Monitoring Technologies Limited",
    "24.4500": "Chi Mei Communication Systems, Inc.",
    "24.4501": "D-Link Corporation",
    "24.1336072": "Jabil Circuit (Shanghai) Ltd.",
    "20.105645769490432": "Jiangsu Sunfy Technologies Holding Co.,Ltd.",
    "12.71297884643328": "Anhui Jiante Network Technology Co., Ltd.",
    "24.5794853": "Intel Corporate",
    "24.10514615": "Huawei Technologies Co.,Ltd",
    "12.346867023872": "EN ElectronicNetwork Hamburg GmbH",
    "12.346858831872": "VTZ d.o.o.",
    "24.14720440": "Planex Communications Inc.",
    "12.346867273728": "Netmaker",
    "24.12882234": "Shenzhen Sanland Electronic Co., ltd.",
    "24.6833649": "Toshiba Client Solutions Co., Ltd.",
    "24.2153387": "Samsung Electronics Co., Ltd.",
    "24.15758876": "Compal Information (Kunshan) Co., Ltd.",
    "12.346857013248": "Advanced technologies & Engineering (pty) Ltd",
    "12.123917675646976": "PuS GmbH und Co. KG",
    "24.3679787": "UTran Technology",
    "12.346854051840": "INCAA Informatica Italia srl",
    "12.123917687123968": "Assembly Contracts Ltd",
    "24.9978874": "Intel Corporate",
    "24.15205474": "Cisco Systems, Inc",
    "24.8131702": "Damall Technologies SAS",
    "24.3166325": "GN Audio A/S",
    "24.15762881": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.290306": "OpenRTLS Group",
    "24.1362713": "HP Inc.",
    "24.16556001": "CONWIN.Tech. Ltd",
    "12.346852192256": "Clive Green & Co. Ltd.",
    "24.49229": "Mitec Ltd",
    "24.49228": "Department Of Foreign Affairs",
    "12.346868912128": "AVA Monitoring AB",
    "24.49226": "Group 2000 Ag",
    "24.49225": "US Robotics Total Control (tm) NETServer Card",
    "24.49224": "Bay Technical Associates",
    "24.49223": "Unimicro Systems, Inc.",
    "24.49222": "Blue Chip Technology Ltd",
    "24.49221": "Isolation Systems Inc",
    "24.325604": "Apple, Inc.",
    "24.7088": "Bharat Electronics Limited",
    "24.7089": "Wistron Neweb Corporation",
    "24.7082": "XenICs nv",
    "24.7083": "Telchemy, Incorporated",
    "24.7080": "UBI&MOBI,.Inc",
    "24.7081": "Brother industries, LTD.",
    "24.7086": "Micro Control Systems, Inc",
    "24.7087": "Nokia Danmark A/S",
    "24.7084": "Curtiss Wright Controls Embedded Computing",
    "24.7085": "iControl Incorporated",
    "12.123917686521856": "BAE Systems",
    "24.14680196": "Huawei Technologies Co.,Ltd",
    "24.5524364": "Juniper Networks",
    "12.123917684789248": "Dynacard Co., Ltd.",
    "12.346858848256": "Izisoft",
    "12.123917682946048": "Gastech Australia Pty Ltd",
    "24.4256800": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.346852519936": "Lite F GmBH",
    "12.123917688819712": "Evolute Systems Private Limited",
    "24.3470923": "Intel Corporate",
    "24.15792051": "Phorm",
    "24.4760690": "Intel Corporate",
    "20.277783363190784": "Shenzhen Nokelock Technology Co, Ltd.",
    "20.154794722328576": "Private",
    "12.123917680971776": "EPD Electronics",
    "24.3936193": "KBC Networks",
    "24.3965726": "CKD Corporation",
    "12.123917685620736": "Design For Life Systems",
    "24.13677375": "Shenzhen TINNO Mobile Technology Corp.",
    "24.1054878": "Elmo Motion Control",
    "12.123917676900352": "Dalcnet srl",
    "24.2927360": "Huawei Technologies Co.,Ltd",
    "12.346864648192": "Vehicle Testing Equipment, S.L.",
    "24.13921471": "Goodrich ISR",
    "24.6048121": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.9237273": "Siemens Industrial Automation Products Ltd., Chengdu",
    "24.6048125": "Samsung Electronics Co.,Ltd",
    "24.5265087": "Samsung Electronics Co.,Ltd",
    "12.123917686591488": "Alfi",
    "24.12896085": "Beijing HuaqinWorld Technology Co.,Ltd",
    "24.11821715": "Samsung Electronics Co.,Ltd",
    "24.8694212": "Huawei Technologies Co.,Ltd",
    "24.8423064": "Netronics Technologies Inc.",
    "24.4736705": "Fn-Link Technology Limited",
    "24.3165723": "Huawei Device Co., Ltd.",
    "24.1892111": "Cisco Systems, Inc",
    "24.14739281": "Nintendo Co., Ltd.",
    "24.24648": "Dell EMC",
    "24.24649": "Vina Technologies",
    "24.24644": "Litton/Poly-Scientific",
    "24.24645": "Pathlight Technologies",
    "24.24646": "Vmetro, Inc.",
    "24.24647": "Cisco Systems, Inc",
    "24.24640": "Netro Corp.",
    "24.24641": "Yokogawa Digital Computer Corporation",
    "24.24642": "Tks (Usa), Inc.",
    "24.24643": "iDirect, INC.",
    "24.9682": "InpegVision Co., Ltd",
    "24.9683": "AzureWave Technology Inc.",
    "24.9680": "Nokia Danmark A/S",
    "24.9681": "Eastern Asia Technology Limited",
    "24.9686": "The Kroger Co.",
    "24.9687": "Cedo",
    "24.9684": "General Dynamics Mission Systems",
    "24.548": "C-Cor",
    "24.547": "ClickTV",
    "24.546": "Chromisys, Inc.",
    "24.545": "DSP Application, Ltd.",
    "24.544": "Canon Finetech Inc.",
    "24.543": "Aculab PLC",
    "24.542": "Simtel S.R.L.",
    "24.541": "Data General Communication Ltd.",
    "24.540": "Network Elements, Inc.",
    "12.346863562752": "Altinex, Inc",
    "24.9455420": "China Mobile Group Device Co.,Ltd.",
    "12.346854014976": "Palm Solutions Group",
    "24.1459": "Asahi-Engineering Co., Ltd.",
    "24.1458": "Medison Co., Ltd.",
    "24.1457": "ASB Technology BV",
    "24.1456": "Korea Computer Technology Co., Ltd.",
    "12.346859528192": "DioDigiWorks. CO., LTD.",
    "24.1454": "Mediaport USA",
    "24.1453": "Topspin Communications, Inc.",
    "24.1452": "Northern Digital, Inc.",
    "24.1451": "Cyber Fone, Inc.",
    "24.1450": "Moore Industries International Inc.",
    "12.123917685387264": "Xiamen Maxincom Technologies Co., Ltd.",
    "12.123917678747648": "Contec Americas Inc.",
    "20.168210320064512": "Zerone Co., Ltd.",
    "24.2818": "Dallmeier electronic",
    "24.2819": "Taekwang Industrial Co., Ltd",
    "24.2816": "Fujian Start Computer Equipment Co.,Ltd",
    "24.2817": "Daiichi Electronics Co., Ltd.",
    "24.2814": "NovaPal Ltd",
    "24.2815": "Kilchherr Elektronik AG",
    "24.2812": "Core Tec Communications, LLC",
    "24.2813": "Kentec Electronics",
    "24.2810": "Traverse Technologies Australia",
    "24.2811": "Ambri Limited",
    "24.14426594": "Huawei Technologies Co.,Ltd",
    "24.8923742": "New H3C Technologies Co., Ltd",
    "24.1626661": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.2112239": "Sivantos GmbH",
    "24.5000707": "TCT mobile ltd",
    "24.4237631": "Pivotal Commware, Inc.",
    "24.14180706": "Wistron Neweb Corporation",
    "24.1331082": "Intel Corporate",
    "20.30931570130944": "Scientific-Production Enterprise Dynamics",
    "12.346860433408": "Progentech Limited",
    "24.13123366": "Microsoft Corporation",
    "24.6046674": "Hazemeyer",
    "12.346868551680": "N-Hands GmbH und Co KG",
    "24.13147416": "Huawei Device Co., Ltd.",
    "24.13637570": "ASUSTek COMPUTER INC.",
    "12.123917689847808": "iotec GmbH",
    "24.6305806": "Alpsalpine Co,.Ltd",
    "24.1852866": "Huizhou City Sunsin lntelligent Technology Co.,Ltd",
    "12.346852872192": "BMC Messsysteme GmbH",
    "24.10025639": "Innonet",
    "24.14683263": "Nintendo Co., Ltd.",
    "24.6600787": "Samsung Electronics Co.,Ltd",
    "24.8664215": "Hewlett Packard",
    "20.62070510845952": "LG Electronics",
    "24.9978469": "Sagemcom Broadband SAS",
    "24.2424037": "Samsung Electronics Co.,Ltd",
    "24.1591919": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.9702509": "Tp-Link Technologies Co.,Ltd.",
    "24.327295": "Cisco Systems, Inc",
    "24.3969367": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "24.8174474": "Nintendo Co., Ltd.",
    "20.79215082864640": "Beijing MFOX technology Co., Ltd.",
    "24.850067": "ARRIS Group, Inc.",
    "24.1101481": "Texas Instruments",
    "24.16571315": "Murata Manufacturing Co., Ltd.",
    "24.4977": "ARRIS Group, Inc.",
    "24.4976": "Nokia Danmark A/S",
    "24.8229": "Control Technology Inc (Industrial Controls and Network Interfaces)",
    "24.8228": "Pacific Communication Sciences",
    "24.4973": "Tentaculus AB",
    "24.4972": "TomTom",
    "24.4971": "E-Tec",
    "24.4970": "Hach Lange Sarl",
    "24.8223": "Best Power Technology, Inc.",
    "24.8222": "Netquest Corporation",
    "24.8221": "Katana Products",
    "24.8220": "Excel, Inc.",
    "24.8227": "T.C. Technologies Pty. Ltd",
    "24.8226": "NMS Communications",
    "24.4979": "BLwave Electronics Co., Ltd",
    "24.4978": "Dell Inc.",
    "24.36928": "Siemens Network Convergence LLC",
    "24.36929": "Applied Digital Access",
    "12.123917675622400": "Avant Technologies",
    "24.6579891": "Tp-Link Technologies Co.,Ltd.",
    "24.36922": "Nihon Media Tool Inc.",
    "24.36923": "TriEMS Research Lab, Inc.",
    "24.36920": "Fountain Technologies, Inc.",
    "24.36921": "Shasta Networks",
    "24.36926": "N.V. Philips Industrial Activities",
    "24.36927": "Aztec Radiomedia",
    "24.3149098": "Zhejiang Wellcom Technology Co.,Ltd.",
    "24.36925": "Biopac Systems, Inc.",
    "12.346861346816": "Access Control Systems JSC",
    "20.189244298493952": "Crde",
    "24.5774525": "Affinegy",
    "24.3172048": "Huawei Device Co., Ltd.",
    "20.452708007936": "Ambient-System sp. z o.o.",
    "24.11280554": "DMATEK Co., Ltd.",
    "24.14196997": "Syslane, Co., Ltd.",
    "24.2905233": "Microsoft Corporation",
    "12.123917682970624": "Vitec System Engineering Inc.",
    "12.71297883975680": "APG Cash Drawer, LLC",
    "24.6585396": "Ubee Interactive Co., Limited",
    "24.16302665": "Private",
    "24.5285072": "IEEE Registration Authority",
    "24.10536434": "IEEE Registration Authority",
    "12.346866851840": "Cleverscope",
    "12.346866745344": "Visual Productions",
    "24.7124135": "Landauer, Inc.",
    "20.255191443046400": "Wetek Electronics Limited",
    "20.268735392776192": "ShenZhenShi EVADA technology Co.,Ltd",
    "12.123917681467392": "Doga",
    "24.35478": "Cisco Systems, Inc",
    "24.11306301": "Huawei Technologies Co.,Ltd",
    "24.7635220": "Apple, Inc.",
    "24.2948513": "ASUSTek COMPUTER INC.",
    "24.275972": "Gyeyoung I&T",
    "12.346855489536": "Integrated Silicon Solution (Taiwan), Inc.",
    "24.9965284": "Private",
    "12.346862833664": "Inventure Inc.",
    "24.14966133": "Samsung Electronics Co.,Ltd",
    "12.346857488384": "Tadian Electronics Systems LTD",
    "24.9720125": "iSmart Alarm, Inc.",
    "24.8167773": "GN ReSound A/S",
    "24.12870251": "ZPT Vigantice",
    "12.123917682757632": "LG Electronics",
    "24.15499667": "Logitech, Inc",
    "20.127658780262400": "Revl Inc.",
    "24.269584": "Dream Ware Inc.",
    "12.71297883181056": "Vemotion",
    "12.123917683044352": "Microgate Srl",
    "12.346853810176": "Advanced Concepts, Inc.",
    "24.287447": "Alstom Hydro France",
    "12.123917684269056": "Beijing Xiansheng Technology Co., Ltd",
    "24.6577680": "Huawei Technologies Co.,Ltd",
    "24.1049269": "Intel Corporate",
    "24.4514970": "Omnitronics Pty Ltd",
    "24.2906765": "SYSTRONIK Elektronik u. Systemtechnik GmbH",
    "20.224447328944128": "Beijing Safesoft Greatmaker Co.,ltd",
    "12.346860212224": "Turun Turvatekniikka Oy",
    "20.264625140531200": "Telefonix Incorporated",
    "24.9738": "Cisco Systems, Inc",
    "24.4232453": "Acoinfo Technology Co.,Ltd",
    "24.8942001": "ARRIS Group, Inc.",
    "12.119269859328": "Shenzhen Guang Lian Zhi Tong Limited",
    "12.346862325760": "Bon Hora GmbH",
    "12.346862727168": "Talyst, Inc.",
    "24.308150": "Seamap (UK) Ltd",
    "12.123917688041472": "nyantec GmbH",
    "20.154314890805248": "Guralp Systems Limited",
    "24.5354": "S Digm Inc. (Safe Paradigm Inc.)",
    "24.14967881": "ARRIS Group, Inc.",
    "12.123917678743552": "Changshu Ruite Electric Co.,Ltd.",
    "24.2385169": "AVM GmbH",
    "24.3204238": "Vodafone UK",
    "24.5019181": "Fresh AB",
    "24.14174511": "Canon Inc.",
    "12.123917688991744": "TRIUMPH BOARD a.s.",
    "12.346862813184": "Kenda Electronic Systems Limited",
    "24.7630653": "Contec GmbH",
    "12.346852675584": "Caspian Networks",
    "12.346868293632": "EMAC, Inc.",
    "24.6858370": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.16284448": "Cisco Systems, Inc",
    "24.268671": "Grasphere Japan",
    "24.4765095": "Glory Horse Industries Ltd.",
    "20.255553898020864": "Shenzhen Rongda Computer Co.,Ltd",
    "12.346867871744": "Norbit ODM AS",
    "24.3670431": "Shenzhen Fast Technologies Co.,Ltd",
    "24.15517852": "Hangzhou Hikvision Digital Technology Co.,Ltd.",
    "24.525773": "DongGuang EnMai Electronic Product Co.Ltd.",
    "20.168210319015936": "Zymbit",
    "24.2386603": "IT-IS International",
    "24.14705626": "Ubiquiti Networks Inc.",
    "20.194284244434944": "Hangzhou Hikrobot Technology Co., Ltd.",
    "24.7619585": "Netgear",
    "12.346865561600": "TOPROOTTechnology Corp. Ltd.",
    "12.123917679742976": "Claresys Limited",
    "24.9204640": "Fiberhome Telecommunication Technologies Co.,LTD",
    "12.123917679788032": "Xiamen Beogold Technology Co. Ltd.",
    "12.123917676990464": "VendNovation LLC",
    "12.123917676998656": "DogWatch Inc",
    "24.12620973": "zte corporation",
    "12.123917691031552": "Zmbizi App Llc",
    "24.9991229": "Shenzhen Jun Kai Hengye Technology Co. Ltd",
    "24.4769890": "Simo Wireless,Inc.",
    "24.7611204": "Xiaomi Communications Co Ltd",
    "24.4248159": "Itel Mobile Limited",
    "20.277783353753600": "Ant Financial（Hang Zhou）Network Technology Co.,Ltd.",
    "20.14286634090496": "Bepal Technology Co.,Ltd.",
    "12.346868228096": "Rohde&Schwarz Topex SA",
    "20.264809157230592": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "12.123917689368576": "Klaxoon",
    "12.346853183488": "Raven Industries",
    "12.71297884319744": "RCS Energy Management Limited",
    "24.4785": "Dai Nippon Printing Co., Ltd",
    "24.3965877": "Advance Vision Electronics Co. Ltd.",
    "24.14733798": "Aliph dba Jawbone",
    "24.15268599": "Guangdong Uniteddata Holding Group Co., Ltd.",
    "24.13948841": "Shanghai Kaixiang Info Tech LTD",
    "12.123917686804480": "T&M Media Pty Ltd",
    "20.132777855942656": "Private",
    "24.2152282": "Posh Mobile Limited",
    "24.14192872": "Samsung Electronics Co.,Ltd",
    "24.9199533": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.11031914": "Pocketnet Technology Inc.",
    "24.14179823": "Busch-Jaeger Elektro GmbH",
    "24.16542769": "LXinstruments GmbH",
    "12.123917687881728": "Weltek Technologies Co. Ltd.",
    "12.346860453888": "Mistral Solutions Pvt. Ltd",
    "24.15471862": "BioControl AS",
    "24.3413173": "Texas Instruments",
    "24.5419": "Cisco Systems, Inc",
    "24.12098253": "Elite Optoelectronic(Asia)Co.,Ltd",
    "24.822227": "Itel Mobile Limited",
    "24.3712718": "BSkyB Ltd",
    "12.346857246720": "GE Security Kampro",
    "12.346853277696": "QuesCom",
    "24.7366171": "Sonova AG",
    "24.14975782": "Huawei Technologies Co.,Ltd",
    "24.269204": "Host Mobility AB",
    "20.220958187061248": "Evotor Llc",
    "24.10236280": "Foshan Huadian Intelligent Communications Teachnologies Co.,Ltd",
    "24.11598902": "CyberTAN Technology Inc.",
    "24.4240380": "Logital Co. Limited",
    "24.13661776": "IEEE Registration Authority",
    "24.15502023": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.15502020": "Amazon Technologies Inc.",
    "12.346856099840": "MicroTek Electronics, Inc.",
    "12.123917687431168": "5-D Systems, Inc.",
    "24.15768791": "Guangzhou Blue Cheetah Intelligent Technology Co., Ltd.",
    "12.123917680910336": "Wallner Automation GmbH",
    "24.14440230": "Power-One",
    "20.203238161776640": "Neuropace Inc.",
    "24.4217816": "Chongqing Fugui Electronics Co.,Ltd.",
    "24.5281905": "Inventum Technologies Private Limited",
    "20.229558876897280": "Beijing Inspiry Technology Co., Ltd.",
    "24.4200418": "Intai Technology Corp.",
    "12.346859343872": "Ajeco Oy",
    "20.198151822639104": "Lezyne INC USA",
    "24.2158278": "Elefirst Science & Tech Co., ltd",
    "24.9735": "Enabling Technology Pty Ltd",
    "24.15209315": "Comstock RD, Inc.",
    "20.132777849651200": "Ses",
    "12.123917678534656": "EIDOS s.p.a.",
    "20.264165333663744": "Shenzhen Umind Technology Co., Ltd.",
    "12.123917688168448": "Coral Telecom Limited",
    "24.8679374": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.5272141": "BEG Brueck Electronic GmbH",
    "12.123917685035008": "Javasparrow Inc.",
    "24.8706504": "Unipattern Co.,",
    "24.7922654": "Apple, Inc.",
    "24.12894382": "Midori Electronic Co., Ltd.",
    "24.2940309": "Taicang T&W Electronics",
    "12.346867884032": "PLR Information Systems Ltd.",
    "20.168198552944640": "Shenzhen Huntkey Electric Co., Ltd.",
    "20.180680389558272": "Beijing Lanke Science and Technology Co.,LTd.",
    "24.7890112": "Apple, Inc.",
    "24.6032571": "CELIZION Inc.",
    "12.346862927872": "TORC Technologies",
    "24.57387": "Extreme Networks, Inc.",
    "24.1073872": "Technicolor CH USA Inc.",
    "24.7378774": "Happyelectronics Co.,Ltd",
    "12.346852757504": "StorLogic, Inc.",
    "24.8393792": "Sunlit System Technology Corp",
    "24.3834": "Optoway Technology Incorporation",
    "24.6036665": "vivo Mobile Communication Co., Ltd.",
    "24.2894231": "Microsoft Corporation",
    "12.346862301184": "Electrodata Inc.",
    "20.123598307065856": "Hebei Baina Xinda Technology Co., Ltd.",
    "24.57386": "Tandberg Television As",
    "24.7364979": "Calantec GmbH",
    "24.5558898": "Nuki Home Solutions GmbH",
    "24.8926657": "Oi Electric Co.,Ltd",
    "24.1862266": "Cisco Systems, Inc",
    "12.346858553344": "Intergrated Security Mfg. Ltd",
    "24.15773573": "ARRIS Group, Inc.",
    "24.15506267": "Nokia Corporation",
    "12.346857787392": "ACD Elektronik GmbH",
    "24.7404870": "Samsung Electronics Co.,Ltd",
    "24.46445": "David Electronics Co., LTD.",
    "12.123917681614848": "BigStuff3, Inc.",
    "24.4216036": "Wearsafe Labs Inc",
    "24.770": "Charles Industries, Ltd.",
    "24.4213007": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13164": "SynapSense Corporation",
    "24.15789656": "PACE Telematics GmbH",
    "20.88383272517632": "Southern Ground Audio LLC",
    "12.346863026176": "JSC \"Component-ASU\"",
    "24.15475579": "zte corporation",
    "12.346865917952": "Tobila Systems, Inc.",
    "24.9077": "ARRIS Group, Inc.",
    "12.71297885118464": "Energy Technology and Control Ltd.",
    "24.4986620": "Juniper Networks",
    "24.52": "Network Resources Corporation",
    "24.53": "Spectragraphics Corporation",
    "24.50": "Marconi plc",
    "24.51": "Egan Machinery Company",
    "24.56": "Css Labs",
    "24.57": "Toshiba Corporation",
    "24.54": "Atari Corporation",
    "24.55": "Oxford Metrics Limited",
    "24.4224937": "New H3C Technologies Co., Ltd",
    "24.58": "Chyron Corporation",
    "24.59": "i Controls, Inc.",
    "20.84372892942336": "Gronic Systems GmbH",
    "24.283335": "Bose Corporation",
    "24.540172": "ARRIS Group, Inc.",
    "24.8924762": "storONE",
    "24.7868": "Wintech Automation Co.,Ltd.",
    "24.7869": "Cisco Systems, Inc",
    "24.7864": "Aloys, Inc",
    "24.7865": "Sing Fai Technology Limited",
    "24.7866": "High Density Devices AS",
    "24.7867": "Bluelight Technology Inc.",
    "24.7860": "Unifat Technology Ltd.",
    "24.7861": "Ever Sparkle Technologies Ltd",
    "24.7862": "TAG Heuer SA",
    "24.7863": "TBTech, Co., Ltd.",
    "12.346865704960": "Keith & Koep GmbH",
    "12.123917679931392": "Bacsoft",
    "24.7354894": "Aruba, a Hewlett Packard Enterprise Company",
    "24.14693728": "Challenger Supply Holdings, LLC",
    "24.5285653": "Aboundi, Inc.",
    "24.5010279": "Genius Bytes Software Solutions GmbH",
    "24.5771404": "Intelbras",
    "24.2363720": "Entropix, LLC",
    "24.8707360": "Texas Instruments",
    "12.123917691310080": "Flextronics Canafa Design Services",
    "12.346859008000": "Access Specialties, Inc",
    "24.1100234": "Ace Technology Corp.",
    "12.346860015616": "Lincoln Industrial",
    "24.14473807": "Getck Technology, Inc",
    "20.97843043893248": "Applied Device Technologies",
    "20.233630190272512": "Beijing Maystar Information Technology Co., Ltd.",
    "20.79215072378880": "Hangzhou Freely Communication Co., Ltd.",
    "12.346855051264": "Stage Tec Entwicklungsgesellschaft für professionelle Audio",
    "24.1629170": "Dell Inc.",
    "12.119269339136": "Commonwealth Scientific and Industrial Research Organisation",
    "20.229108126580736": "TytoCare LTD.",
    "12.123917690847232": "Bavaria Digital Technik GmbH",
    "24.14478854": "Huawei Technologies Co.,Ltd",
    "12.123917679489024": "Avid Controls Inc",
    "12.346865324032": "Emergency Message Controls LLC",
    "20.18540132302848": "Private",
    "12.123917684985856": "Burk Technology",
    "24.1621788": "Jiangsu Huitong Group Co.,Ltd.",
    "24.11542401": "IEEE Registration Authority",
    "24.986": "Takamisawa Cybernetics Co., Ltd.",
    "12.123917679566848": "Palarum LLC",
    "24.9452192": "Nokia",
    "12.123917691752448": "InOut Communication Systems",
    "24.10244334": "Cambridge Industries(Group) Co.,Ltd.",
    "12.123917683339264": "Shenzhen Virtual Clusters Information Technology Co.,Ltd.",
    "12.346861412352": "TTi Ltd",
    "24.5284146": "Shenzhen MiaoMing Intelligent Technology Co.,Ltd",
    "24.982": "RADVision, Ltd.",
    "24.4740328": "Winbond?",
    "24.9980180": "Wistron Neweb Corporation",
    "24.4520305": "Shenzhen Mercury Communication Technologies Co.,Ltd.",
    "12.123917683343360": "KST technology",
    "24.11067416": "Nokia Corporation",
    "24.6805": "Hisense Mobile Communications Technoligy Co.,Ltd.",
    "24.6804": "Votronic GmbH",
    "24.6807": "fitivision technology Inc.",
    "24.6806": "Ecler S.A.",
    "24.6801": "FusionDynamic Ltd.",
    "12.346853314560": "Global Opto Communication Tech. Corp",
    "24.6803": "ERCO Leuchten GmbH",
    "24.6802": "ASUSTek COMPUTER INC.",
    "24.1050801": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.6809": "Smarty (HZ) Information Electronics Co., Ltd",
    "24.6808": "Asotel Communication Limited Taiwan Branch",
    "24.4256489": "Ibm",
    "20.273458024939520": "CloudMinds (Shenzhen) Holdings Co., Ltd",
    "24.15221686": "QNAP Systems, Inc.",
    "24.5805616": "Technicolor CH USA Inc.",
    "12.346863689728": "Trolex Limited",
    "12.123917690212352": "Slat",
    "24.2918098": "Cisco Systems, Inc",
    "12.123917675417600": "Exatel",
    "20.159182273642496": "Shanghai HuRong Communication Technology Development Co., Ltd.",
    "24.14473494": "Huawei Technologies Co.,Ltd",
    "12.346863681536": "Odus Technologies SA",
    "24.5011607": "Arrowhead Alarm Products Ltd",
    "24.9484593": "Apple, Inc.",
    "24.9094958": "Roku, Inc.",
    "20.171812086349824": "R-S-I Elektrotechnik GmbH CO KG",
    "24.2623": "Data East Corporation",
    "24.6068897": "Huawei Device Co., Ltd.",
    "12.123917679206400": "AKIS technologies",
    "24.2622": "EADS Telecom",
    "24.12871321": "vivo Mobile Communication Co., Ltd.",
    "24.846989": "Motorola Mobility LLC, a Lenovo Company",
    "24.13924803": "Cameronet GmbH & Co. KG",
    "24.846980": "Shenzhen TINNO Mobile Technology Corp.",
    "24.1572909": "Sony Mobile Communications Inc",
    "24.7636420": "New H3C Technologies Co., Ltd",
    "24.293232": "Huawei Technologies Co.,Ltd",
    "24.293237": "Honor Device Co., Ltd.",
    "24.1328868": "Avistel",
    "24.15760583": "Juniper Networks",
    "12.346863611904": "UMLogics Corporation",
    "12.123917680611328": "Eagle Mountain Technology",
    "24.14981852": "Apple, Inc.",
    "12.123917676040192": "JSC \"Ural Factories\"",
    "20.255191448289280": "JDM Mobile Internet Solution(Shanghai) Co., Ltd.",
    "24.1323729": "Samsung Electronics Co.,Ltd",
    "24.15492556": "Panasonic Automotive Systems Company of America",
    "24.1365246": "ARRIS Group, Inc.",
    "12.123917691449344": "Radig Hard & Software",
    "24.10765773": "IoT Diagnostics",
    "12.123917683396608": "Glory Technology Service Inc.",
    "24.11558356": "ARRIS Group, Inc.",
    "12.71297884082176": "Embed Limited",
    "12.123917679022080": "oxynet Solutions",
    "12.123917678141440": "Yaham Optoelectronics Co., Ltd",
    "24.5574": "Cisco Systems, Inc",
    "24.5575": "Cisco Systems, Inc",
    "24.5576": "FlexiPanel Ltd",
    "24.5577": "Gumstix, Inc",
    "24.5570": "3M Germany",
    "24.5571": "Ruf Telematik AG",
    "24.5572": "Flovel Co., Ltd.",
    "24.5573": "Dell Inc.",
    "20.62366876172288": "CeeNex Inc",
    "24.5578": "TeraRecon, Inc.",
    "24.5579": "Surf Communication Solutions Ltd.",
    "20.52819397705728": "Maytronics Ltd.",
    "24.16006753": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.123917686071296": "Omesh City Group",
    "12.123917687603200": "Prolan Zrt.",
    "12.346859360256": "Zmicro Systems Inc",
    "12.123917678563328": "CircuitMeter Inc.",
    "20.31346166595584": "Diatrend Corporation",
    "12.346863939584": "Sportvision Inc.",
    "12.123917681123328": "BMT Messtechnik Gmbh",
    "20.31368096514048": "Weber Marking Systems GmbH",
    "24.4213879": "ARRIS Group, Inc.",
    "24.11325317": "Huawei Technologies Co.,Ltd",
    "20.225209246285824": "Continental Control Systems",
    "12.346867056640": "HGL Dynamics",
    "12.119269425152": "OctoGate IT Security Systems GmbH",
    "24.10793249": "Arantia 2010 S.L.",
    "24.15218554": "Zyxel Communications Corporation",
    "12.123917691547648": "Invert Robotics Ltd.",
    "24.4485080": "IEEE Registration Authority",
    "20.184991242780672": "Sercomm Corporation.",
    "24.12326645": "Cisco Systems, Inc",
    "24.5291158": "Apple, Inc.",
    "24.7137372": "Apple, Inc.",
    "24.7659455": "Liteon Technology Corporation",
    "12.123917679644672": "Guan Show Technologe Co., Ltd.",
    "12.346856763392": "Codem Systems, Inc.",
    "24.51339": "Cisco Systems, Inc",
    "24.10248481": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "12.123917675110400": "Cubro Acronet GesmbH",
    "12.123917690515456": "Re spa - Controlli Industriali - IT01782300154",
    "20.272805465686016": "Guangdong Enok Communication Co., Ltd.",
    "12.123917689151488": "Triax A/S",
    "20.216145528356864": "LLC \"TechnoEnergo\"",
    "12.123917690384384": "APEN GROUP SpA (VAT IT08767740155)",
    "12.71297884962816": "Inforce Computing Inc.",
    "12.346866724864": "Breakaway Systems LLC",
    "20.71070628446208": "PH Technical Labs",
    "24.16260719": "EBG compleo GmbH",
    "24.15257850": "MKS Instruments Deutschland GmbH",
    "20.45033654845440": "Valeo Siemens eAutomotive Norway",
    "12.346865041408": "Cytek Media Systems, INC.",
    "12.123917691572224": "Engage Technologies",
    "24.15248806": "Sagemcom Broadband SAS",
    "24.15257852": "Liteon Technology Corporation",
    "12.346865713152": "Erhardt+Leimer GmbH",
    "12.346857447424": "Monitor Business Machines Ltd",
    "12.346865790976": "Infrasafe, Inc.",
    "24.3685320": "2Wire Inc",
    "12.123917686304768": "vivaMOS",
    "20.242185283829760": "Rohde&Schwarz Topex SA",
    "24.14687547": "Cambridge Industries(Group) Co.,Ltd.",
    "24.14687544": "Beijing HuaqinWorld Technology Co.,Ltd",
    "24.11306612": "Open Mesh, Inc.",
    "12.123917685354496": "DOBE Computing",
    "20.260167875690496": "SKS Control Oy",
    "12.346865422336": "Yokowo Co.,Ltd",
    "12.346867265536": "Critical Link LLC",
    "24.3149525": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346867150848": "Rohde&Schwarz Topex SA",
    "24.576923": "samtec automotive electronics & software GmbH",
    "24.10001757": "Technicolor CH USA Inc.",
    "24.5778740": "Qingping Electronics (Suzhou) Co., Ltd",
    "12.71297884540928": "Somfy Sas",
    "12.123917690638336": "SC3 Automation",
    "24.10239439": "Netgear",
    "20.18540146982912": "Shenzhen Sunwoda intelligent hardware Co.,Ltd",
    "20.22333148364800": "i-SENS, Inc.",
    "12.123917689389056": "Sistem Sa",
    "24.52734": "Apple, Inc.",
    "20.9814042214400": "Hantas Co., Ltd.",
    "20.57353110028288": "Albert Handtmann Maschinenfabrik GmbH&Co.KG",
    "24.11042355": "Nokia Danmark A/S",
    "24.7079381": "Ethertronics Inc",
    "24.864": "PAC Interactive Technology, Inc.",
    "24.865": "Widcomm, Inc.",
    "24.866": "Vodtel Communications, Inc.",
    "12.119269605376": "Suretrak Global Pty Ltd",
    "24.860": "Saint Song Corp.",
    "24.861": "Bosung Hi-Net Co., Ltd.",
    "24.862": "Metropolitan Area Networks, Inc.",
    "24.863": "Prüftechnik Condition Monitoring GmbH & Co. KG",
    "24.13126964": "LIONS Taiwan Technology Inc.",
    "24.868": "Scenix Semiconductor, Inc.",
    "24.869": "Kira Information & Communications, Ltd.",
    "12.346863988736": "MCS MICRONIC Computer Systeme GmbH",
    "24.11799416": "Brusa Elektronik AG",
    "24.12593293": "Logitech, Inc",
    "24.6353978": "Edifier International",
    "24.7329": "Akamai Technologies, Inc.",
    "24.7328": "Production Resource Group, LLC",
    "24.7327": "Razorstream, LLC",
    "24.7326": "Dualtech IT AB",
    "24.7325": "Liecthi AG",
    "24.7324": "Nortel Networks",
    "24.3679595": "Microsoft Mobile Oy",
    "24.7322": "Nokia Danmark A/S",
    "24.7321": "Shunra Software Ltd.",
    "24.7320": "Lucky Technology (Hk) Company Limited",
    "20.88010614898688": "Shenzhen Xinfa Electronic Co.,ltd",
    "12.123917688492032": "Teneo IoT B.V.",
    "24.3166810": "NST Co., LTD",
    "24.14969178": "Dictum Health, Inc.",
    "12.123917678850048": "Camozzi Automation SpA",
    "12.346855858176": "Precision System Science Co.,Ltd",
    "24.11799418": "Brusa Elektronik AG",
    "12.123917690998784": "Duevi Srl",
    "12.123917685248000": "Lineage Power Pvt Ltd.,",
    "24.813798": "Stanley Security Solutions",
    "24.2649549": "Beijing Winner Microelectronics Co.,Ltd.",
    "24.5549607": "Apple, Inc.",
    "24.8929623": "Shenzhen Atsmart Technology Co.,Ltd.",
    "20.137215118147584": "SY Electronics Limited",
    "24.11272911": "RW Tecnologia Industria e Comercio Ltda",
    "24.7142841": "Proware Technologies Co Ltd.",
    "24.9202404": "Viogem Limited",
    "24.6351721": "Quanta Computer Inc.",
    "20.49277261840384": "Resus Industries",
    "24.8943772": "Game Technologies SA",
    "20.255553892777984": "Trapeze Switzerland GmbH",
    "24.14951913": "Color-Chip",
    "24.7619211": "Apple, Inc.",
    "24.14951911": "Apple, Inc.",
    "24.12071489": "Wuhan Tianyu Information Industry Co., Ltd.",
    "24.15509824": "ARRIS Group, Inc.",
    "24.1871452": "Huawei Technologies Co.,Ltd",
    "12.123917690396672": "Orlaco Products B.V.",
    "24.7999": "AVM GmbH",
    "12.123917678571520": "Living Water",
    "12.346852384768": "Arcturus Networks Inc.",
    "24.6498": "Commerciant, LP",
    "24.6499": "Sony Mobile Communications Inc",
    "24.6496": "DoCoMo Systems, Inc.",
    "24.6497": "Blaupunkt Embedded Systems GmbH",
    "24.6494": "ARRIS Group, Inc.",
    "24.6495": "Valemount Networks Corporation",
    "24.6492": "Innotech Corporation",
    "24.6493": "ShenZhen XinHuaTong Opto Electronics Co.,Ltd",
    "24.6490": "Jenaer Antriebstechnik GmbH",
    "24.6491": "D-Link Corporation",
    "24.317349": "New H3C Technologies Co., Ltd",
    "12.346859180032": "Burk Technology",
    "20.233258801430528": "Nordi Telekommunikatsiooni Oü",
    "24.1648": "Upponetti Oy",
    "24.1649": "Softing AG",
    "24.1646": "Delta Electronics, Inc.",
    "24.1647": "Korea Data Systems",
    "24.1644": "Robinson Corporation",
    "24.1645": "Compuprint S.P.A.",
    "24.1642": "InfiniCon Systems, Inc.",
    "24.1643": "Sysmex Corporation",
    "24.1640": "Vicon Industries Inc.",
    "24.1641": "Datasound Laboratories Ltd",
    "12.346858631168": "First Control Systems AB",
    "12.346864091136": "Ooo Npf Atis",
    "24.15770458": "Apple, Inc.",
    "24.11035050": "Cloudview Limited",
    "12.123917679513600": "Vessel Technology Ltd",
    "24.10273138": "Huawei Technologies Co.,Ltd",
    "12.123917686452224": "Association Romandix",
    "24.10781166": "Nokia Corporation",
    "24.6148": "E-Tek Digital Technology Limited",
    "24.6149": "Beijing InHand Networking Technology Co.,Ltd.",
    "20.194284245483520": "Wuuk Labs Corp.",
    "24.6140": "Suprema Inc.",
    "24.6141": "Amulet Hotkey",
    "24.6142": "Talos System Inc.",
    "24.6143": "PLAYLINE Co.,Ltd.",
    "24.1060935": "Samsung Electronics Co.,Ltd",
    "24.6145": "Actiontec Electronics, Inc",
    "24.6146": "Alpha Networks Inc.",
    "24.6147": "ArcSoft Shanghai Co. LTD",
    "24.2379549": "iRule LLC",
    "24.4723582": "Samsung Electronics Co.,Ltd",
    "24.1099358": "Adapt-IP",
    "24.4510775": "Inteno Broadband Technology AB",
    "24.8924": "Vigil Health Solutions Inc.",
    "12.123917688893440": "Globalcom Engineering Spa",
    "24.3160762": "Guangzhou BaoLun Electronics Co., Ltd",
    "24.8926": "OPPO Digital, Inc.",
    "24.11292981": "Mpi Tech",
    "24.8927": "TAMUZ Monitors",
    "24.4517207": "Longicorn, inc.",
    "20.57353102688256": "Highlite International B.V.",
    "12.123917677277184": "Vitec",
    "12.346862067712": "Switch Science (Panini Keikaku)",
    "24.4505054": "BHTC GmbH",
    "24.37048": "Rohde & Schwarz Gmbh & Co. Kg",
    "24.37049": "Beran Instruments Ltd.",
    "24.37046": "Fibex Systems",
    "24.37047": "Digital Lightwave, Inc.",
    "24.37044": "Willowbrook Technologies",
    "24.37045": "Nikon Corporation",
    "24.37042": "Avici Systems Inc.",
    "24.37043": "Agranat Systems",
    "24.37040": "Vadem",
    "24.37041": "Cisco Systems, Inc",
    "20.35758075281408": "Eilersen Electric A/S",
    "24.12118102": "Apple, Inc.",
    "24.3989560": "Plumeria Networks, Inc.",
    "12.123917677584384": "TELL Software Hungaria Kft.",
    "12.346855268352": "Clifford Chance LLP",
    "20.31477066629120": "Intertecno SRL \"NISUTA\"",
    "12.346857869312": "Oberon Service srl",
    "20.5239438573568": "Teleepoch Ltd",
    "24.11597971": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "20.62636962086912": "Apt Mobile Satcom Limited",
    "24.11290669": "Ralinwi Nanjing Electronic Technology Co., Ltd.",
    "24.3969756": "Octopod Technology Co. Ltd.",
    "24.9349": "ConteXtream Ltd",
    "24.9348": "Bang and Olufsen Medicom a/s",
    "24.9343": "Nortel Networks",
    "24.9342": "Universal Global Scientific Industrial Co., Ltd.",
    "24.9341": "Nokia Danmark A/S",
    "24.9340": "Nokia Danmark A/S",
    "24.9347": "LG Electronics (Mobile Communications)",
    "24.9346": "Ruckus Wireless",
    "24.9345": "Hewlett Packard",
    "24.9344": "Meteocontrol GmbH",
    "24.10753416": "ARRIS Group, Inc.",
    "12.123917690478592": "Lightside Instruments AS",
    "24.5826375": "Deltanet AG",
    "24.1610623": "AMPAK Technology, Inc.",
    "24.15991331": "PAX Computer Technology(Shenzhen) Ltd.",
    "12.119269777408": "CAST Group of Companies Inc.",
    "20.171977805398016": "Suzhou Fitcan Technology Co.,LTD",
    "24.13402410": "vivo Mobile Communication Co., Ltd.",
    "24.34660": "Cisco Systems, Inc",
    "20.154238421303296": "Square Inc.",
    "24.6576644": "Chicony Electronics Co., Ltd.",
    "12.123917687357440": "Loma Systems s.r.o.",
    "24.8168091": "D.SignT GmbH & Co. KG",
    "24.6856298": "Huawei Device Co., Ltd.",
    "20.242876975939584": "Giant Power Technology Biomedical Corporation",
    "20.39672881348608": "Safetrust Inc",
    "24.3935304": "Servergy, Inc.",
    "12.346855768064": "Europe Technologies",
    "20.268735393824768": "Konntek Inc",
    "24.3932655": "Sony Mobile Communications Inc",
    "12.346859732992": "Miracom Technology Co., Ltd.",
    "12.346863415296": "Volantic AB",
    "24.5250224": "Cisco Systems, Inc",
    "24.524349": "Cadnetix Corporations",
    "24.524348": "Schlumberger Well Services",
    "24.524347": "Torus Systems Limited",
    "24.524346": "Orcatech Inc.",
    "24.524345": "Spider Systems Limited",
    "24.524344": "Bull S.A.S.",
    "24.524343": "Fuji-Xerox Co. Ltd.",
    "12.346855571456": "Virtu",
    "24.524341": "Microfive Corporation",
    "24.524340": "Filenet Corporation",
    "24.10013479": "Galore Networks Pvt. Ltd.",
    "24.2115399": "Dell Inc.",
    "24.1616283": "Dell Inc.",
    "20.75685685100544": "Beam Communications Pty Ltd",
    "12.346853154816": "KMS Systems, Inc.",
    "12.123917683372032": "db Broadcast Products Ltd",
    "20.48835644620800": "Tinyco",
    "12.123917691674624": "Shizuki Electric Co.,Inc",
    "24.9441953": "We Corporation Inc.",
    "12.123917686325248": "Schneider Electric Motion USA",
    "12.71297883459584": "Genadsystem",
    "24.6066144": "ARRIS Group, Inc.",
    "24.16271149": "X2gen Digital Corp. Ltd",
    "24.1598578": "Aruba, a Hewlett Packard Enterprise Company",
    "24.16465": "Garbee and Garbee",
    "24.16464": "Ironics, Incorporated",
    "24.16467": "Ampro Computers",
    "24.16466": "Star Technologies, Inc.",
    "24.16461": "Telecomm Techniques",
    "24.16460": "Hypertec Pty Ltd.",
    "24.16463": "Space & Naval Warfare Systems",
    "24.16462": "Fluent, Inc.",
    "12.346868891648": "Burk Technology",
    "12.123917685145600": "Suzhou Sapa Automotive Technology Co.,Ltd",
    "24.16469": "Metronix Gmbh",
    "24.16468": "Thinking Machines Corporation",
    "20.198151836270592": "ZXY Sport Tracking",
    "24.8418393": "K's Co.,Ltd.",
    "24.3705308": "Opticon Sensors Europe B.V.",
    "24.10023321": "NEC Platforms, Ltd.",
    "12.123917691650048": "Blueprint Lab",
    "12.346857017344": "nVent, Schroff GmbH",
    "24.14977749": "Rf Window Co., Ltd.",
    "12.123917688557568": "Advanced Products Corporation Pte Ltd",
    "24.13893719": "MC Technologies GmbH",
    "20.194363553480704": "D&T Inc.",
    "12.346867191808": "TTi Ltd",
    "12.346863439872": "DAISHIN-DENSHI Co., Ltd",
    "20.4471485628416": "Ceita Communication Technology Co.,Ltd",
    "24.825916": "Intel Corporate",
    "12.123917678669824": "IDS Innomic GmbH",
    "24.13632479": "Samsung Electronics Co.,Ltd",
    "24.13954219": "vivo Mobile Communication Co., Ltd.",
    "20.27478314713088": "Staclar, Inc.",
    "12.71297883389952": "Dacom West GmbH",
    "24.11013219": "Huawei Technologies Co.,Ltd",
    "12.123917687943168": "eBZ GmbH",
    "24.11317177": "Samsung Electronics Co.,Ltd",
    "24.3734": "Cubic Defense Applications, Inc.",
    "24.3735": "Ultracker Technology CO., Inc",
    "24.3736": "HME Clear-Com LTD.",
    "12.123917682073600": "Vantage Integrated Security Solutions Pvt Ltd",
    "24.3185581": "Bbk Educational Electronics Corp.,Ltd.",
    "24.3737": "Spectrum Digital, Inc",
    "24.8174854": "AE REFUsol",
    "24.3730": "Open Telecom",
    "24.3731": "Milénio 3 Sistemas Electrónicos, Lda.",
    "20.206931540049920": "Private",
    "12.346859700224": "haber & koenig electronics gmbh",
    "24.4245265": "Apple, Inc.",
    "24.586484": "Net One Partners Co.,Ltd.",
    "24.3733": "Fujiya Denki Seisakusho Co.,Ltd.",
    "20.123598313357312": "Faurecia Clarion Electronics (Dongguan) Co., Ltd",
    "12.346853650432": "Masterclock, Inc.",
    "24.11038813": "ARRIS Group, Inc.",
    "24.7657418": "Panasonic Corporation Automotive",
    "24.10487734": "Advanced Technical Support, Inc.",
    "24.11812411": "Viableware, Inc",
    "12.346868813824": "dotOcean",
    "24.16294735": "zte corporation",
    "24.15749610": "Fitbit, Inc.",
    "12.123917687586816": "BESO sp. z o.o.",
    "12.123917682499584": "Atgs",
    "24.2651608": "Fujian Star-Net Communication Co.,Ltd",
    "20.93020686909440": "Shenzhen Excera Technology Co.,Ltd.",
    "24.1605902": "Apple, Inc.",
    "20.84531191218176": "jvi",
    "24.1347361": "Garmin International",
    "24.2076": "@pos.com",
    "24.16305262": "DEPO Electronics Ltd",
    "12.123917677342720": "Fink Telecom Services",
    "24.2077": "Ipsil, Incorporated",
    "12.346864971776": "B.E.A.R. Solutions (Australasia) Pty, Ltd",
    "24.2078": "Repeatit AB",
    "24.14172898": "Canary Connect, Inc.",
    "24.2079": "Pou Yuen Tech Corp. Ltd.",
    "24.11858316": "Huawei Device Co., Ltd.",
    "24.6555": "Diversified Technical Systems, Inc.",
    "24.8126855": "Curtis Instruments, Inc.",
    "12.346855100416": "DataWind Research",
    "24.7369675": "Apple, Inc.",
    "24.12852942": "HMI Sources Ltd.",
    "12.123917686644736": "Aaronia AG",
    "24.9214438": "TCT mobile ltd",
    "24.13649857": "Elma Electronic AG",
    "12.123917677735936": "DermaLumics S.L.",
    "20.106480350003200": "Phase One A/S",
    "12.346858663936": "Rice University",
    "12.123917679857664": "Herholdt Controls srl",
    "24.273844": "First Property (Beijing) Co., Ltd Modern MOMA Branch",
    "24.11043313": "Bmeye B.V.",
    "24.3944460": "Dewar Electronics Pty Ltd",
    "24.7889494": "Huawei Technologies Co.,Ltd",
    "24.14700355": "Universal Global Scientific Industrial Co., Ltd.",
    "12.346866704384": "DRS Imaging and Targeting Solutions",
    "24.6556076": "TCT mobile ltd",
    "12.119269494784": "DesignA Electronics Limited",
    "12.123917679886336": "Acrodea, Inc.",
    "12.346863910912": "Meisol co.,ltd",
    "12.123917685231616": "Zeroplus Technology Co.,Ltd.",
    "24.1897674": "Private",
    "24.12634438": "Mitsuya Laboratories Inc.",
    "24.10257343": "PRO-VISION, Inc.",
    "20.154680191614976": "Shenzhen C & D Electronics Co., Ltd.",
    "12.123917685342208": "Shanghai Daorech Industry Developmnet Co.,Ltd",
    "12.346854486016": "Tandata Systems Ltd",
    "12.123917686620160": "Pantherun Technologies Pvt Ltd",
    "24.7106829": "Cisco Systems, Inc",
    "24.5523411": "Tsat As",
    "12.123917685219328": "Izome",
    "24.16547047": "Fci Usa Llc",
    "24.3423684": "Buffalo.Inc",
    "24.7884611": "Apple, Inc.",
    "12.123917676670976": "Lattech Systems Pty Ltd",
    "24.33020": "Avatar Corporation",
    "24.33021": "Exsceed Corpration",
    "24.33022": "Azure Technologies, Inc.",
    "24.33023": "Soc. De Teleinformatique Rtc",
    "24.3700426": "Shenzhen Smart Intelligent Technology Co.Ltd",
    "24.11552048": "BSkyB Ltd",
    "12.123917681307648": "Egistech Co.,Ltd.",
    "12.346853498880": "APCON, Inc.",
    "24.11555214": "Holl technology CO.Ltd.",
    "24.11587096": "Zhejiang shenghui lighting co.,Ltd",
    "24.7869112": "Orantek Limited",
    "24.10497779": "United Integrated Services Co., Led.",
    "24.9456122": "Intel Corporate",
    "24.5811953": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.5276373": "Apple, Inc.",
    "20.264809145696256": "Beijing Serviatech lnformation Tech Co.,Ltd",
    "24.309814": "Li Seng Technology Ltd",
    "20.207357799825408": "Alpha ESS Co., Ltd.",
    "12.123917679915008": "Agramkow Fluid Systems A/S",
    "24.9753705": "Cisco Systems, Inc",
    "24.3443323": "Samsung Electronics Co.,Ltd",
    "24.8705150": "Aruba, a Hewlett Packard Enterprise Company",
    "24.13945453": "Routerboard.com",
    "24.13945454": "u-blox AG",
    "20.212188452093952": "Haguenet",
    "12.346860421120": "C&D Technologies, Inc",
    "24.1843156": "LifeBEAM Technologies LTD",
    "12.123917679484928": "Santa Barbara Imaging Systems",
    "24.13375102": "Cisco Systems, Inc",
    "24.14723734": "Samsung Electronics Co.,Ltd",
    "12.123917676384256": "Virsae Group Ltd",
    "24.8414697": "BenQ Corporation",
    "24.11576913": "Cisco Systems, Inc",
    "12.346864291840": "Finnet-Service Ltd.",
    "24.8424183": "Nanoleaf",
    "20.184991239634944": "Medcaptain Medical Technology Co., Ltd.",
    "24.6869501": "Shenzhen Trimax Technology Co.,Ltd",
    "24.2678373": "Aruba, a Hewlett Packard Enterprise Company",
    "12.119269306368": "Zamir Recognition Systems Ltd.",
    "20.84531192266752": "Shanghai Ehong Technology Co.,Ltd",
    "12.123917686484992": "New Japan Radio Co., Ltd",
    "24.3168142": "eero inc.",
    "12.123917686902784": "FutureTechnologyLaboratories INC.",
    "24.11304766": "Apple, Inc.",
    "24.15772008": "Antailiye Technology Co.,Ltd",
    "24.2933767": "machineQ",
    "24.14298077": "AT&T, Inc.",
    "24.13160579": "Apple, Inc.",
    "24.2940119": "AzureWave Technology Inc.",
    "24.14173968": "Sichuan Changhong Electric Ltd.",
    "24.15474546": "Samsung Electro-Mechanics(Thailand)",
    "24.1622429": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "12.123917690810368": "MEDIAM Sp. z o.o.",
    "24.12108735": "Henan Chengshi NetWork Technology Co.，Ltd",
    "12.123917686001664": "Prior Scientific Instruments Ltd",
    "24.4476766": "Pan Studios Co.,Ltd.",
    "24.16557520": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.13644048": "Ingenic Semiconductor Co.,Ltd",
    "12.346856841216": "Ascon S.p.a.",
    "20.106480352100352": "LongSung Technology (Shanghai) Co.,Ltd.",
    "12.346868535296": "JSC \"Kaluga Teletypes Manufacturing Plant\"",
    "24.3226": "Hitech Electronics Corp.",
    "24.3227": "EE Solutions, Inc",
    "24.2665575": "Mach Power, Rappresentanze Internazionali s.r.l.",
    "24.3225": "HITEL LINK Co.,Ltd",
    "24.3222": "OQO, Inc.",
    "24.3223": "NV ADB TTV Technologies SA",
    "24.3220": "United Electronic Industries, Inc. (EUI)",
    "24.3221": "PrimeNet",
    "24.14720810": "Iton Technology Corp.",
    "24.3228": "Chongho information & communications",
    "24.3229": "UbeeAirWalk, Inc.",
    "24.1862010": "EliteGroup Computer Systems Co., LTD",
    "24.7926325": "Sichuan AI-Link Technology Co., Ltd.",
    "24.3449791": "Terewave. Inc.",
    "24.585117": "Tecno Mobile Limited",
    "24.10026420": "Primax Electronics Ltd.",
    "24.7916942": "SMC Networks Inc",
    "24.841666": "Axium Technologies, Inc.",
    "24.4196288": "Railtec Systems GmbH",
    "24.8410866": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.1338268": "Shenzhen Bilian Electronic Co.，Ltd",
    "12.123917685424128": "FUJICOM Co.,Ltd.",
    "24.6863235": "b-plus GmbH",
    "24.1338266": "zte corporation",
    "20.127654886899712": "Essential Trading Systems Corp",
    "12.346863714304": "Ubiquiti Networks Inc.",
    "12.346859794432": "CDSA Dam Neck",
    "24.7641792": "Cachengo, Inc.",
    "24.41188": "Optiquest",
    "24.41189": "Nhc Communications",
    "24.41186": "Keisokugiken Corporation",
    "24.41187": "Xkl Systems Corp.",
    "24.41184": "Tennyson Technologies Pty Ltd",
    "24.41185": "Westport Research Associates, Inc.",
    "24.41182": "Yamaha Corporation",
    "24.41183": "Sts Technologies, Inc.",
    "24.41180": "O.N. Electronic Co., Ltd.",
    "24.41181": "Azonix Corporation",
    "24.9961895": "Apple, Inc.",
    "20.146101702230016": "MedicusTek Inc.",
    "24.1621421": "L-Tech Corporation",
    "12.123917680603136": "Heitec Ag",
    "20.233630183981056": "YunDing Network Technology (Beijing) Co., Ltd",
    "12.346866790400": "INSEVIS GmbH",
    "24.11830869": "Huawei Technologies Co.,Ltd",
    "20.268341266612224": "Elektronika Naglic d.o.o.",
    "24.13434768": "Huawei Device Co., Ltd.",
    "24.9759116": "Nokia",
    "24.14994946": "WyreStorm Technologies Ltd",
    "12.123917680631808": "BRS Sistemas Eletrônicos",
    "24.3985243": "Icomm Hk Limited",
    "24.5043291": "Blue Clover Devices",
    "12.346857701376": "I.T.W. Betaprint",
    "24.9212486": "Oerlikon Textile Gmbh&Co.KG",
    "24.10488301": "Avaya Inc",
    "24.13122155": "Roku, Inc",
    "20.272805463588864": "Shenzhuoyue Technology.,Ltd",
    "20.159304577449984": "Stanley Security",
    "24.12106531": "Guangdong Nufront CSC Co., Ltd",
    "20.181338942472192": "Netas Telekomunikasyon A.S.",
    "20.212188461531136": "Navitas Digital Safety Ltd",
    "24.7646303": "Qiku Internet Network Scientific (Shenzhen) Co., Ltd.",
    "12.123917686554624": "Herrmann Datensysteme GmbH",
    "24.3466202": "Hoval Aktiengesellschaft",
    "20.194284248629248": "Ledger",
    "24.9980746": "ARRIS Group, Inc.",
    "24.13125877": "MEDISON Xray Co., Ltd",
    "24.6306": "XIP Technology AB",
    "24.6307": "Zippy Technology Corp.",
    "24.6304": "Cierma Ascenseurs",
    "24.6305": "Tiqit Computers, Inc.",
    "24.6302": "OMNIKEY GmbH.",
    "12.346857168896": "Rubin Ltd.",
    "24.6300": "Weldex Corporation",
    "24.6301": "Navcast Inc.",
    "24.6308": "ARRIS Group, Inc.",
    "24.6309": "ADigit Technologies Corp.",
    "24.10265530": "SAKURA Internet Inc.",
    "24.13377676": "Huawei Technologies Co.,Ltd",
    "20.45069241417728": "Zhixiang Technology Co., Ltd.",
    "12.123917676699648": "J-Factor Embedded Technologies",
    "12.346857439232": "Putercom Enterprise Co., Ltd.",
    "24.2363717": "Xiaomi Communications Co Ltd",
    "24.8172955": "Tp-Link Technologies Co.,Ltd.",
    "12.123917689217024": "Axatel SrL",
    "12.346852298752": "DRS Technologies Canada Co.",
    "24.8138461": "Samsung Electronics Co.,Ltd",
    "24.2694": "Lenze",
    "24.16007582": "Huawei Device Co., Ltd.",
    "24.10248112": "Samsung Electronics Co.,Ltd",
    "24.4222095": "Apple, Inc.",
    "24.6297646": "ShenZhen Protruly Electronic Ltd co.",
    "24.2697": "Creval Systems, Inc.",
    "24.13931681": "Texas Instruments",
    "24.3995434": "Nokia Corporation",
    "24.8684313": "Hangzhou Zero Zero Technology Co., Ltd.",
    "20.58144490258432": "APG Cash Drawer, LLC",
    "24.3419830": "Huawei Technologies Co.,Ltd",
    "24.3419831": "Intel Corporate",
    "20.167295460573184": "CATS Power design",
    "20.97755443757056": "Chengdu Vision-Zenith Technology Co.,Ltd",
    "20.119709245636608": "Chongqing Baoli Yota Technologies Limited",
    "12.346858684416": "HUNGAROCOM Telecommunication Ltd.",
    "24.6580236": "Comtrend Corporation",
    "24.13107800": "ITW GSE ApS",
    "20.181338952957952": "Toec Technology Co.，Ltd.",
    "24.12104668": "Derek (Shaoguan) Limited",
    "24.4719146": "B-Link Electronic Limited",
    "24.2698": "Cisco Systems, Inc",
    "12.123917689548800": "AeroVision Avionics, Inc.",
    "24.16029265": "NAKAYO Inc",
    "24.1336": "Merilus, Inc.",
    "24.1337": "A Brand New World in Sweden AB",
    "24.1334": "Danam Communications, Inc.",
    "24.1335": "Nets Technology Co., Ltd.",
    "24.1332": "Northstar Engineering Ltd.",
    "24.1333": "Chip PC Ltd.",
    "24.1330": "Cisco Systems, Inc",
    "24.1331": "Brocade Communications Systems LLC",
    "24.9179011": "Angie Hospitality LLC",
    "24.1338": "Willowglen Services Pte Ltd",
    "24.1339": "Harbour Networks Ltd., Co. Beijing",
    "24.15492327": "MOCACARE Corporation",
    "12.123917688590336": "Petring Energietechnik GmbH",
    "24.9997229": "Corintech Ltd.",
    "24.851549": "IEEE Registration Authority",
    "24.1360009": "Dune Hd Ltd",
    "12.123917681729536": "Private",
    "20.141720404623360": "BroadMedia Co., Ltd.",
    "12.346854084608": "Redpoint Controls",
    "24.6093112": "Apple, Inc.",
    "24.279": "Canal +",
    "24.278": "Netspect Technologies, Inc.",
    "24.275": "Olympus Corporation",
    "24.274": "Shark Multimedia Inc.",
    "24.277": "Extratech Corporation",
    "24.276": "Kanda Tsushin Kogyo Co., Ltd.",
    "24.271": "Brocade Communications Systems LLC",
    "24.270": "Bri-Link Technologies Co., Ltd",
    "24.273": "iDigm Inc.",
    "24.272": "Gotham Networks",
    "24.7136182": "Nera Telecommunications Ltd.",
    "20.97755437465600": "Coala Life AB",
    "12.123917691379712": "International Center for Elementary Particle Physics, The University of Tokyo",
    "24.8962626": "Poynt Co.",
    "24.3733256": "National Resource Management, Inc.",
    "24.14721807": "Pevco",
    "20.106196525645824": "Robot S.A.",
    "24.4218": "Ambicom (was Tandy?)",
    "24.4219": "Cisco Systems, Inc",
    "24.4216": "Nuera Communications, Inc.",
    "24.4217": "Cisco",
    "24.4214": "EUREM GmbH",
    "24.4215": "Saf Drive Systems, Ltd.",
    "12.346868625408": "CT Company",
    "24.4213": "Segate Technology LLC",
    "24.4210": "Gvn Technologies, Inc.",
    "24.4211": "Technobox, Inc.",
    "24.12601595": "Shenzhen JingHanDa Electronics Co.Ltd",
    "24.13671851": "TCT mobile ltd",
    "24.14959270": "Intel Corporate",
    "24.13150753": "ARRIS Group, Inc.",
    "20.149945505021952": "Sowee",
    "12.123917679394816": "Algodue Elettronica Srl",
    "12.123917679190016": "Mentor Graphics",
    "24.5208": "HS Automatic ApS",
    "24.8962995": "Sovico",
    "24.10782382": "Quectel Wireless Solutions Co., Ltd.",
    "12.346868178944": "Reason Tecnologia S.A.",
    "12.123917682180096": "Shanghai YuanAn Environmental Protection Technology Co.,Ltd",
    "24.321122": "Acroname Inc.",
    "24.11543978": "Private",
    "12.346853224448": "Green Hills Software, Inc.",
    "24.7347816": "Hanna Instruments, Inc.",
    "12.346864492544": "Xslent Energy Technologies. LLC",
    "12.123917685899264": "Newshine",
    "24.10768751": "Microsoft Mobile Oy",
    "24.7649203": "Shenzhen YOUHUA Technology Co., Ltd",
    "20.111031368024064": "Pricer AB",
    "20.207357812408320": "LISTEC GmbH",
    "24.5201": "Apple, Inc.",
    "24.3731427": "Ingenico Terminals Sas",
    "12.346854125568": "Lulea University of Technology",
    "20.132777843359744": "Ningbo Sanxing Electric Co., Ltd.",
    "24.11280907": "ASUSTek COMPUTER INC.",
    "12.123917684678656": "Braemar Manufacturing, LLC",
    "12.119269969920": "Exibea AB",
    "24.6818530": "Silicon Laboratories",
    "24.11280901": "Compal Broadband Networks, Inc.",
    "24.10753382": "Weifang Goertek Electronics Co.,Ltd",
    "24.5508696": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346862587904": "IDEAS s.r.l.",
    "24.319195": "Rockport Networks Inc",
    "24.2665630": "Cisco Systems, Inc",
    "24.13641914": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "24.10793454": "H. ZANDER GmbH & Co. KG",
    "12.123917679210496": "Mighty Cube Co., Ltd.",
    "24.15496510": "Huawei Technologies Co.,Ltd",
    "24.9989212": "Jiangxi Gosun Guard Security Co.,Ltd",
    "20.39986079465472": "ARTICONA - Bechtle Logistik & Service GmbH",
    "24.11059346": "Huawei Device Co., Ltd.",
    "20.198611824541696": "Softel SA de CV",
    "24.8940303": "Bluegiga Technologies OY",
    "24.12596062": "Microsoft",
    "24.12861726": "Belkin International Inc.",
    "12.346867298304": "Leyden Engineering",
    "24.9746814": "Espressif Inc.",
    "20.106196530888704": "Trophy SAS",
    "24.827704": "Blinq Wireless Inc.",
    "24.2097487": "Linea Research Ltd",
    "20.167295464767488": "KRISTECH Krzysztof Kajstura",
    "24.10245148": "Shenzhen My-power Technology Co.,Ltd",
    "24.10026499": "Ericsson - North America",
    "12.123917684129792": "Scenario Automation",
    "12.123917684662272": "Faro Technologies, Inc.",
    "20.154314891853824": "Spectranetix",
    "20.180698129367040": "Eye Io, Llc",
    "24.11325219": "Hi-flying electronics technology Co.,Ltd",
    "24.15759705": "ASUSTek COMPUTER INC.",
    "12.123917675741184": "Aver",
    "24.2668412": "IEEE Registration Authority",
    "24.1884385": "IEEE Registration Authority",
    "24.3451703": "Apple, Inc.",
    "24.15776104": "Itel Mobile Limited",
    "20.224420102668288": "MDT technologies GmbH",
    "20.88383271469056": "Roda industrial development Co.,Ltd.",
    "24.1591385": "Castlenet Technology Inc.",
    "20.171812092641280": "SuZhou Jinruiyang Information Technology CO.,LTD",
    "20.229206571089920": "Private",
    "12.123917691445248": "Silixa Ltd",
    "12.346866151424": "Hijikata Denki Corp.",
    "24.9568": "Ibridge Networks & Communications Ltd.",
    "24.10269416": "Huawei Technologies Co.,Ltd",
    "24.9563": "CoachComm, LLC",
    "24.9562": "Tantalus Systems Corp.",
    "24.9561": "Syphan Technologies Ltd",
    "24.9560": "Mpedia",
    "24.9567": "SenTec AG",
    "24.9566": "Shanghai Dare Technologies Co.,Ltd.",
    "24.9565": "Morningstar Corporation",
    "24.9564": "NEC Corporation",
    "12.346857549824": "AMRDEC Prototype Integration Facility",
    "20.207147363205120": "InSync Technology Ltd",
    "24.12892340": "Huawei Technologies Co.,Ltd",
    "24.3971703": "Technicolor CH USA Inc.",
    "24.2122591": "Abeeway",
    "24.16563835": "vivo Mobile Communication Co., Ltd.",
    "24.8651175": "Greyware Automation Products, Inc",
    "12.123917687525376": "Flashnet SRL",
    "20.45033655894016": "Umojo",
    "24.4761272": "Sonos, Inc.",
    "24.9188422": "Huawei Device Co., Ltd.",
    "24.11838889": "MODI GmbH",
    "12.123917681360896": "Polvision",
    "20.127654897385472": "Ansjer Electronics Co., Ltd.",
    "12.123917689712640": "Baader Planetarium GmbH",
    "20.211776059736064": "The Professional Monitor Company Ltd",
    "24.7178": "Shenzhen AEE Technology Co.,Ltd.",
    "24.7179": "SmartAnt Telecom",
    "24.7176": "Echo360, Inc.",
    "24.7177": "SAE Electronic Co.,Ltd.",
    "24.7174": "Siemens Numerical Control Ltd., Nanjing",
    "24.7175": "Cwlinux Limited",
    "24.7172": "Airgain, Inc.",
    "24.7173": "Nonin Medical Inc.",
    "24.7170": "Pano Logic",
    "24.7171": "Betty TV Technology AG",
    "24.2528": "Xemics S.A.",
    "24.2529": "Gemtek Technology Co., Ltd.",
    "24.2526": "Samjin Information & Communications Co., Ltd.",
    "24.2527": "Vestel Elektronik San ve Tic. A.Ş.",
    "24.2524": "Galaxis Technology AG",
    "24.2525": "Mavin Technology Inc.",
    "24.2522": "Control Module Inc.",
    "24.2523": "eSpace",
    "24.2520": "Fält Communications AB",
    "24.2521": "Neoscale Systems, Inc",
    "24.11849470": "James Czekaj",
    "24.11021237": "Edgecore Networks Corporation",
    "24.4782305": "Nokia",
    "20.97843032358912": "Custom Biogenic Systems",
    "24.8693448": "Intel Corporate",
    "24.12376384": "ASR Co,.Ltd.",
    "24.7660811": "Intel Corporate",
    "12.346865549312": "GD Mission Systems",
    "24.13685778": "ADB Broadband Italia",
    "24.12872375": "Xiaomi Communications Co Ltd",
    "24.16559668": "ARRIS Group, Inc.",
    "24.49384": "Plexcom, Inc.",
    "20.216145532551168": "Multicom, Inc",
    "24.49385": "Oak Solutions, Ltd.",
    "12.346861727744": "2Ncomm Design Srl",
    "24.49386": "Array Technology Ltd.",
    "24.6336901": "ATH system",
    "12.123917689597952": "Dorlet Sau",
    "24.49387": "Seh Computertechnik Gmbh",
    "20.194363550334976": "SunTech Medical, Inc.",
    "24.8060228": "Rosenberger Hochfrequenztechnik GmbH & Co. KG",
    "24.49380": "Siemens Building",
    "20.242213041733632": "Nuove Tecnologie srl",
    "24.49381": "Gespac, S.A.",
    "24.49382": "Verilink Corporation",
    "24.49383": "Fiberdata Ab",
    "24.4491260": "Elitegroup Computer Systems Co.,Ltd.",
    "12.346853208064": "IPAXS Corporation",
    "24.16314970": "G-Lab GmbH",
    "24.8959030": "Private",
    "24.4204564": "RFI Engineering",
    "24.11071270": "Tritonwave",
    "24.7905667": "Advidia",
    "24.14952070": "DWnet Technologies(Suzhou) Corporation",
    "12.346857897984": "Delphi Display Systems, Inc.",
    "12.123917681725440": "Panoramic Power",
    "12.346864779264": "juiceboss",
    "24.49388": "Dauphin Technology",
    "12.346858704896": "IDERs Inc",
    "24.49389": "US Army Electronic Proving Ground",
    "12.346867249152": "Prima sistemi",
    "24.10506507": "Intel Corporate",
    "24.6094494": "Wiwynn Corporation Tainan Branch",
    "24.5024764": "zte corporation",
    "12.346853261312": "Photonic Bridges, Inc.",
    "12.346866040832": "ASSYSTEM France",
    "12.346857795584": "Moog Broad Reach",
    "12.346868920320": "Sefram",
    "24.9995869": "Nokia Shanghai Bell Co., Ltd.",
    "12.346855612416": "NIE Corporation",
    "20.181256793882624": "Shen Zhen City YaKun Electronics Co., Ltd",
    "24.11029834": "Audiowise Technology Inc.",
    "20.242876964405248": "ShenZhen C&D Electronics CO.Ltd.",
    "12.123917682466816": "Motec Pty Ltd",
    "20.62366869880832": "barox Kommunikation GmbH",
    "24.11280244": "Apple, Inc.",
    "24.13404510": "Huawei Technologies Co.,Ltd",
    "12.346865004544": "ANXeBusiness Corporation",
    "24.3972993": "Shenzhen CATIC Bit Communications Technology Co.,Ltd",
    "24.10280223": "Intelligent Steward Co.,Ltd",
    "24.4724371": "Lear Corporation GmbH",
    "12.123917687111680": "ePOINT Embedded Computing Limited",
    "12.346859192320": "Somesca",
    "24.10008803": "Apple, Inc.",
    "24.11831568": "Coster T.E. S.P.A.",
    "24.4197648": "ARRIS Group, Inc.",
    "24.8390562": "Esson Technology Inc.",
    "24.2381952": "Vizeo",
    "20.84187713372160": "Alibaba (Beijing) Software Service Inc.",
    "24.807169": "Apple, Inc.",
    "12.123917689790464": "Bizerba Luceo",
    "20.251018420944896": "Telo Systems Limitd",
    "12.346858344448": "Stresstech OY",
    "12.346855485440": "Digital Multimedia Technologies Spa",
    "24.9997149": "Sagemcom Broadband SAS",
    "24.15219351": "Toshiba Corporation",
    "12.346868801536": "Peek Traffic",
    "12.123917691170816": "DEUTA-WERKE GmbH",
    "24.1316509": "Cisco Systems, Inc",
    "20.171812083204096": "Advanced Logic Technology (ALT) sa",
    "12.123917690404864": "Nautel Limted",
    "24.2407554": "Dali Wireless, Inc.",
    "20.93065830203392": "Bj Cotytech Technology Co.,Ltd",
    "20.44169490857984": "Shenzhen Domenor Technology Llc",
    "12.346867879936": "Sanmina",
    "24.839414": "Guizhou Fortuneship Technology Co., Ltd",
    "12.346864914432": "Newtel Engineering S.r.l.",
    "24.11286339": "HTC Corporation",
    "24.7097715": "Apple, Inc.",
    "20.96896786890752": "Suzhou Ruilisi Technology Ltd.",
    "12.346867290112": "Allerta Inc",
    "24.6323933": "Microsoft Mobile Oy",
    "20.71414666231808": "Digital Bros S.p.A.",
    "12.123917686636544": "Abbas, a.s.",
    "24.11050221": "Xiaomi Communications Co Ltd",
    "12.346858000384": "Grosvenor Technology Ltd",
    "24.11287732": "Nilan A/S",
    "24.3727021": "Intel Corporate",
    "12.123917684404224": "I.E. Sevko A.V.",
    "24.14431056": "TechSAT GmbH",
    "24.11835025": "Intel Corporate",
    "12.346857873408": "Chell Instruments Ltd",
    "20.9814038020096": "Fio Corporation",
    "24.5566462": "Fullpower Technologies, Inc.",
    "12.123917682794496": "Grupo Epelsa S.L.",
    "24.6609289": "Cisco Systems, Inc",
    "20.233630188175360": "ASDA ICT Co., Ltd.",
    "12.346859245568": "RADIUS Sweden AB",
    "12.123917684649984": "Sarl S@Tis",
    "12.123917678804992": "chiconypower",
    "12.123917684363264": "NanoSense",
    "24.2913551": "Shanghai Dongzhou-Lawton Communication Technology Co. Ltd.",
    "12.346866589696": "Tessera Technology Inc.",
    "24.1631164": "TCT mobile ltd",
    "24.11049744": "inMotion Ltd.",
    "20.52912293150720": "Triax A/S",
    "12.346862395392": "Team Enginers",
    "12.123917684912128": "Autonomic Controls, Inc.",
    "24.7354995": "Shenzhen Sundray Technologies Company Limited",
    "12.123917681815552": "IAAN Co., Ltd",
    "24.3420433": "Samsung Electronics Co.,Ltd",
    "24.9443272": "Huawei Technologies Co.,Ltd",
    "24.2890241": "Huawei Technologies Co.,Ltd",
    "24.11297209": "WAMA Technology Limited",
    "24.14443379": "Obara Korea",
    "24.8166043": "VSE valencia smart energy",
    "12.123917681618944": "Gunma Electronics Co Ltd",
    "24.5509471": "Atamo Pty Ltd",
    "24.11584828": "Cisco Systems, Inc",
    "24.6853172": "Ruckus Wireless",
    "24.1351656": "Lenovo Mobile Communication Technology Ltd.",
    "20.154039758094336": "Shenzhen Huanuo Internet Technology Co.,Ltd",
    "12.346861375488": "A+S Aktuatorik und Sensorik GmbH",
    "20.255861671854080": "Autocom Diagnostic Partner AB",
    "24.1872285": "Dongguan Liesheng Electronic Co., Ltd.",
    "24.10226911": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.288645": "scemtec Hard- und Software fuer Mess- und Steuerungstechnik GmbH",
    "12.346861355008": "Soldig Industria e Comercio de Equipamentos Eletronicos LTDA",
    "24.4498798": "Netgear",
    "24.41208": "Zebra Technologies Inc",
    "20.49277254500352": "Shanghai RW ELE&TEC CO.,LTD",
    "20.146101707472896": "Liaoning IK'SONYA Science and Technology Co., Ltd.",
    "20.145735980941312": "Twtg R&D B.V.",
    "24.10516046": "Espressif Inc.",
    "20.137215116050432": "Ningbo bird sales co.,LTD",
    "24.4480908": "Intercel Group Limited",
    "24.9477194": "Apple, Inc.",
    "24.3983774": "Huiyang Technology Co., Ltd",
    "12.346861559808": "K-MAC Corp.",
    "24.7106328": "Stereotaxis, Inc.",
    "24.10808643": "IEEE Registration Authority",
    "24.2641623": "OutSmart Power Systems, Inc.",
    "24.2912201": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.13147155": "Inspiremobile",
    "20.224641229520896": "Ontex BV",
    "24.2408308": "Private",
    "24.9238985": "MESADA Technology Co.,Ltd.",
    "24.14175376": "Apple, Inc.",
    "24.2926464": "True Technologies Inc.",
    "12.123917676408832": "Shanghai Jupper Technology Co.Ltd",
    "24.10543765": "DAT System Co.,Ltd",
    "12.123917680295936": "Hangzhou landesker digital technology co. LTD",
    "12.346864607232": "Evo-Teh d.o.o.",
    "24.12637826": "Arista Networks",
    "24.11560433": "WIO Manufacturing HK Limited",
    "12.123917684047872": "Dm Radiocom",
    "24.9755478": "Sony Home Entertainment&Sound Products Inc",
    "24.8969593": "Voltaire",
    "24.5010725": "Espressif Inc.",
    "12.346853752832": "H M Computing Limited",
    "24.10507599": "Cisco Systems, Inc",
    "24.818216": "Nokia Solutions and Networks GmbH & Co. KG",
    "12.346860969984": "Saia Burgess Controls AG",
    "24.8947360": "Simton Technologies, Ltd.",
    "12.346866159616": "Buanco System A/S",
    "24.7121158": "Telefield Ltd",
    "24.9762129": "Cadi Scientific Pte Ltd",
    "24.14189": "Murata Manufacturing Co., Ltd.",
    "24.12360275": "Huawei Device Co., Ltd.",
    "24.8669219": "WOXTER TECHNOLOGY Co. Ltd",
    "12.346865094656": "Watterott electronic",
    "12.346856001536": "Westronic Systems Inc.",
    "24.16548761": "Avaya Inc",
    "20.5297542266880": "s.d.i. s.p.a.",
    "12.123917685526528": "Smart Embedded Systems",
    "24.8194": "Seritech Enterprise Co., Ltd.",
    "24.2656796": "Juniper Networks",
    "20.127654899482624": "Private",
    "20.225209250480128": "Krontech",
    "12.346853883904": "Protech Optronics Co. Ltd.",
    "24.14957166": "Shenzhen Zeroone Technology CO.,LTD",
    "12.123917681356800": "Tanaka Information System, LLC.",
    "12.71297884905472": "RB-LINK Wireless",
    "24.40648": "Xiaomi Communications Co Ltd",
    "24.280925": "Huawei Device Co., Ltd.",
    "24.5329": "TRENDnet, Inc.",
    "24.5328": "BTI Systems Inc.",
    "12.123917687820288": "MeshWorks Wireless Oy",
    "24.5325": "DigitalZone Co., Ltd.",
    "24.5324": "Zetec, Inc.",
    "24.5327": "INVISIO Communications",
    "24.5326": "Nf Corporation",
    "24.5321": "Brocade Communications Systems LLC",
    "24.5320": "Contemporary Research Corp",
    "24.5323": "LifeSync Corporation",
    "24.5322": "Key Radio Systems Limited",
    "24.4058": "Yazaki Corporation",
    "24.4059": "Westell Technologies Inc.",
    "24.2359546": "China Mobile (Hangzhou) Information Technology Co., Ltd",
    "24.4055": "Harman Music Group",
    "24.4056": "Force, Inc.",
    "24.4057": "FlexDSL Telecommunications AG",
    "24.4050": "EWA Technologies, Inc.",
    "24.4051": "Digium",
    "24.4052": "Soundcraft",
    "24.4053": "Schwechat - RISE",
    "24.790789": "Akuvox (Xiamen) Networks Co., Ltd",
    "12.346867527680": "Saia-Burgess Controls AG",
    "24.8948015": "Huawei Technologies Co.,Ltd",
    "12.346861035520": "Privatquelle Gruber GmbH & CO KG",
    "24.4986609": "zte corporation",
    "24.14707790": "Shenzhen TINNO Mobile Technology Corp.",
    "24.4993581": "Teledata Networks",
    "12.123917689618432": "astozi consulting Tomasz Zieba",
    "24.7121669": "Cisco Systems, Inc",
    "24.13915478": "Fiber Mountain Inc.",
    "24.5565487": "Ruckus Wireless",
    "24.11274849": "Labor S.r.L.",
    "24.7873085": "Affirmed Networks",
    "12.346858475520": "Swistec GmbH",
    "24.14444556": "Hewlett Packard Enterprise",
    "24.12601865": "Zhejiang Everbright Communication Equip. Co,. Ltd",
    "24.7914176": "IEEE Registration Authority",
    "24.7881": "Dell Inc.",
    "12.123917687865344": "Franz Kessler GmbH",
    "24.2585": "Valere Power, Inc.",
    "12.123917685305344": "Communication Technology Ltd.",
    "24.7113947": "Otus Technologies Ltd",
    "24.3994924": "DSPECIALISTS GmbH",
    "24.7887": "Philips Electronics Uk Ltd",
    "24.57422": "Sanyo Denki Co., Ltd.",
    "24.57423": "Cisco Systems, Inc",
    "24.57420": "Realtek Semiconductor Corp.",
    "24.57421": "Internet Initiative Japan, Inc",
    "24.57426": "Brocade Communications Systems LLC",
    "24.57427": "Cellport Labs, Inc.",
    "24.57424": "Executone Information Systems, Inc.",
    "24.57425": "Talx Corporation",
    "24.57428": "Kodai Hitec Co., Ltd.",
    "24.57429": "Ingenieria Electronica Comercial Inelcom S.A.",
    "12.346863337472": "Nexans Cabling Solutions",
    "24.8179": "Apple, Inc.",
    "24.8700644": "Jiangsu Qinheng Co., Ltd.",
    "24.4844": "Movacolor b.v.",
    "24.797780": "Huawei Technologies Co.,Ltd",
    "24.4985279": "Zhejiang Dahua Technology Co., Ltd.",
    "20.242876963356672": "Remko GmbH & Co. KG",
    "12.123917678276608": "Elcoma",
    "24.11638": "TITECH GmbH",
    "24.3589": "Wireless Matrix Corp.",
    "24.3588": "CMA/Microdialysis AB",
    "24.16050791": "Cisco Systems, Inc",
    "24.4734953": "Huawei Technologies Co.,Ltd",
    "24.3583": "Chenming Mold Industry Corp.",
    "24.3582": "Hauppauge Computer Works, Inc.",
    "24.3581": "Huges Hi-Tech Inc.,",
    "24.3580": "ITFOR Inc.",
    "24.3587": "Emulex Corporation",
    "24.3586": "Advantech AMT Inc.",
    "24.3585": "ASIP Technologies Inc.",
    "24.3584": "Atrie",
    "24.6189": "Artec Design",
    "12.346861600768": "Magor Communications Corp",
    "24.5283924": "Actineon",
    "24.3203441": "Hewlett Packard",
    "24.4586": "IWICS Inc.",
    "24.4587": "Innovative Integration",
    "24.4584": "Tixi.Com",
    "24.4585": "Starnex Co., Ltd.",
    "12.119269273600": "Private",
    "24.4583": "WORLDSAT - Texas de France",
    "24.4580": "Danelec Electronics A/S",
    "24.4581": "KCodes Corporation",
    "24.8148391": "Kastle Systems",
    "24.4848": "Intel Corporate",
    "24.4588": "Avix Inc.",
    "24.4589": "802 Global",
    "24.3055": "Code Corporation",
    "24.3054": "inc.jet, Incorporated",
    "24.3057": "LAP Laser Applikations",
    "24.3056": "MoTEX Products Co., Ltd.",
    "24.3051": "Systegra AG",
    "24.3050": "Zultys Technologies",
    "24.3053": "ELM Inc.",
    "24.3052": "Nippon Electric Instrument, Inc.",
    "24.3059": "Bae Systems",
    "24.3058": "Chih-Kan Technology Co., Ltd.",
    "12.123917677109248": "DEUTA-WERKE GmbH",
    "24.13679332": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "24.13370743": "Huawei Technologies Co.,Ltd",
    "24.7654457": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.6654": "Shenzhen Seecomm Technology Co.,Ltd.",
    "12.346856751104": "Vitelnet",
    "24.11028901": "Lisantech Co., Ltd.",
    "20.31368104902656": "New Nordic Engineering",
    "24.10024361": "Ohsung",
    "24.6656": "Matrix Inc.",
    "12.346856771584": "PDTS GmbH",
    "24.20530": "Picazo Communications, Inc.",
    "24.20531": "Mayan Networks",
    "24.20536": "Dain Telecom Co., Ltd.",
    "24.20537": "Mariner Networks",
    "24.20534": "Netcam, Ltd.",
    "24.20535": "Koga Electronics Co.",
    "24.6650": "Cable Vision Electronics CO., LTD.",
    "24.20538": "Datong Electronics Ltd.",
    "24.20539": "Mediafire Corporation",
    "12.346853318656": "Teamaxess Ticketing GmbH",
    "24.6651": "BSkyB Ltd",
    "12.123917677383680": "Newtec A/S",
    "20.31614734172160": "Shenzhen Highsharp Electronics Ltd.",
    "24.8162229": "Samsung Electronics Co.,Ltd",
    "12.346858897408": "Desy",
    "20.203238164922368": "Kiwigrid GmbH",
    "12.346858737664": "The Software Group Limited",
    "24.6653": "Nintendo Co., Ltd.",
    "24.13166552": "Mercury Communication Technologies Co.,Ltd.",
    "24.7354385": "Siemens Mobility Limited",
    "24.2388306": "Dell Inc.",
    "24.2636674": "D-Link International",
    "12.346866610176": "AGT Holdings Limited",
    "20.264036218306560": "Aquametro AG",
    "20.124214330785792": "System Level Solutions (India) Pvt.",
    "12.346866618368": "KinotonGmbH",
    "24.6658": "Secure Care Products, Inc",
    "24.6659": "Angel Electronics Co., Ltd.",
    "24.14170046": "Shenzhen Gongjin Electronics Co.,Lt",
    "12.346852917248": "LAN Controls, Inc.",
    "20.171812090544128": "ST Access Control System Corp.",
    "24.51968": "Private",
    "24.6583994": "Apple, Inc.",
    "20.176726071050240": "Beijing Techshino Technology Co., Ltd.",
    "20.175931692941312": "Vitec Imaging Solutions Spa",
    "24.6849306": "Pandora Mobility Corporation",
    "24.15263190": "Samsung Electronics Co.,Ltd",
    "12.119269572608": "Private",
    "24.9725671": "WOM Engineering",
    "24.1862602": "Mitsunami Co., Ltd.",
    "24.1354498": "Huawei Technologies Co.,Ltd",
    "20.277981926785024": "Cg Power And Industrial Solutions Ltd",
    "12.123917678313472": "Hipodromo De Agua Caliente Sa Cv",
    "24.6073085": "Sonos, Inc.",
    "24.7778": "Siemon",
    "24.7779": "Vibro-Meter SA",
    "24.7774": "COmputime Ltd.",
    "24.7775": "KwikByte, LLC",
    "24.7776": "Digital Lighting Systems, Inc",
    "24.7777": "ITEC GmbH",
    "24.7770": "ARRIS Group, Inc.",
    "24.7771": "Unitron Company, Inc.",
    "24.844865": "Apple, Inc.",
    "24.7773": "Holosys d.o.o.",
    "24.14433326": "Manufacturing System Insights, Inc.",
    "24.7870593": "AzureWave Technology Inc.",
    "24.2728": "ePipe Pty. Ltd.",
    "24.2729": "Brooks Automation GmbH",
    "24.2724": "Shanghai Surveillance Technology Co,Ltd",
    "24.2725": "Maxlink Industries Limited",
    "24.2726": "Hochiki Corporation",
    "24.2727": "FEI Electron Optics",
    "24.2720": "Cedar Point Communications",
    "24.2721": "V V S Limited",
    "24.2722": "Systek Inc.",
    "24.2723": "Shimafuji Electric Co.,Ltd.",
    "24.9608": "Genie Industries, Inc.",
    "24.9609": "Hills Industries Limited",
    "24.3442269": "Espressif Inc.",
    "24.9602": "Maksat Technologies (P) Ltd",
    "24.1062943": "Sichuan AI-Link Technology Co., Ltd.",
    "24.9600": "Equipson S.A.",
    "24.9601": "x-star networks Inc.",
    "24.9606": "Tp-Link Technologies Co.,Ltd.",
    "24.9607": "Vitality, Inc.",
    "24.9604": "Cisco Systems, Inc",
    "24.9605": "KOKUYO S&T Co., Ltd.",
    "24.13162416": "Samsung Electronics Co.,Ltd",
    "24.12060887": "Huawei Technologies Co.,Ltd",
    "24.11815894": "IEEE Registration Authority",
    "24.11815890": "Apple, Inc.",
    "24.11548063": "Intel Corporate",
    "24.5831047": "Huawei Technologies Co.,Ltd",
    "24.12872176": "Huawei Technologies Co.,Ltd",
    "24.10778630": "Cisco Systems, Inc",
    "12.123917675671552": "Solace Systems Inc.",
    "24.14186725": "Kuhn Sa",
    "12.346866200576": "Dataregis S.A.",
    "24.16036609": "Intel Corporate",
    "24.11572844": "Lanbowan Technology Ltd.",
    "24.13434135": "TCT mobile ltd",
    "12.346853896192": "Alpha Beta Technologies, Inc.",
    "24.1328941": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.2627199": "Xiaomi Communications Co Ltd",
    "24.2900051": "Samsung Electronics Co.,Ltd",
    "24.2162486": "Iflytek Co.,Ltd.",
    "24.13389942": "Ocom Communications, Inc.",
    "24.13933789": "AMPAK Technology,Inc.",
    "24.7878348": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.12848075": "Xiaomi Communications Co Ltd",
    "24.579635": "Shenzhen RF Technology Co., Ltd",
    "24.6064696": "Hewlett Packard",
    "24.19063": "zte corporation",
    "12.123917675712512": "Vega-Absolute",
    "12.123917681090560": "IP Devices",
    "12.123917675716608": "Norsat International Inc.",
    "24.11303121": "Juniper Networks",
    "24.2152459": "Juniper Networks",
    "24.3471720": "ATEK Products, LLC",
    "24.6327519": "Private",
    "12.346861592576": "Cross Country Systems AB",
    "24.8558": "Function ATI (Huizhou) Telecommunications Co., Ltd.",
    "20.9814045360128": "Szok Energy and Communication Co., Ltd.",
    "24.8551": "HWA JIN T&I Corp.",
    "24.8550": "NovAtel Inc.",
    "24.8553": "Prologix, LLC.",
    "24.8552": "iVeia, LLC",
    "24.8555": "Intel Corporate",
    "24.8554": "Intel Corporate",
    "24.8557": "Soltech Co., Ltd.",
    "24.8556": "Odva",
    "24.2896": "Oxygnet",
    "24.2897": "Micetek International Inc.",
    "24.2894": "Communications & Power Industries",
    "24.2895": "Verifone",
    "24.2892": "Clarion (M) Sdn Bhd",
    "24.2893": "Emuzed",
    "24.2890": "Visimetrics (UK) Ltd",
    "24.2891": "Visiowave Sa",
    "24.11065482": "Wyconn",
    "24.2898": "Joymax Electronics Co. Ltd.",
    "24.2899": "INITIUM Co., Ltd.",
    "24.5044": "Appear TV",
    "24.5045": "Wavesat",
    "24.5046": "Sling Media, Inc.",
    "24.5047": "Scantech ID",
    "24.5040": "Jablotron",
    "24.5041": "Intelligent Control Systems (Asia) Pte Ltd",
    "24.5042": "Carallon Limited",
    "24.5043": "Ecom Communications Technology Co., Ltd.",
    "24.4256076": "ISE Europe SPRL",
    "24.5048": "RyCo Electronic Systems Limited",
    "24.5049": "Bm Spa",
    "12.123917691625472": "ACD Elektronik GmbH",
    "20.18540139642880": "Moya Commumication Technology (Shenzhen) Co.,Ltd.",
    "12.123917683671040": "SANSFIL Technologies",
    "24.9757244": "Intel Corporate",
    "20.102196714668032": "Daisen Electronic Industrial Co., Ltd.",
    "24.1059507": "Xiaomi Communications Co Ltd",
    "24.15243458": "ZETLAB Company",
    "24.12076206": "TCT mobile ltd",
    "24.11568798": "Chloride S.r.L",
    "12.123917688971264": "Sensoronic Co.,Ltd",
    "12.346857160704": "Black Diamond Video, Inc.",
    "24.10512318": "Sicon srl",
    "24.8404829": "Winstars Technology Ltd",
    "12.123917691006976": "Divelbiss Corporation",
    "20.220958188109824": "Pinmicro K K",
    "12.346860965888": "Persy Control Services B.v.",
    "24.11289128": "Parta Networks",
    "20.62366868832256": "1.A Connect GmbH",
    "24.13661055": "Mini-Circuits",
    "24.843996": "Samsung Electronics Co.,Ltd",
    "24.11273747": "Senselogix Ltd",
    "24.327217": "Samsung Electronics Co.,Ltd",
    "20.13692013903872": "Pi Innovo LLC",
    "24.14198525": "Ghost Locomotion",
    "24.7341332": "LG Electronics (Mobile Communications)",
    "12.123917675298816": "AvMap srlu",
    "24.5542409": "Huawei Technologies Co.,Ltd",
    "24.13682569": "Cisco Systems, Inc",
    "24.12116913": "Huawei Technologies Co.,Ltd",
    "24.10525629": "Total Aviation Solutions Pty Ltd",
    "24.2673996": "Individual Computers GmbH",
    "20.48688645799936": "Fast",
    "24.8654861": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917686497280": "S.C.E. srl",
    "12.119269511168": "Marvel Digital International Limited",
    "24.11333896": "LG Innotek",
    "24.7623463": "Commsen Co., Limited",
    "24.3731656": "Mevo Inc.",
    "24.14446344": "Bay Storage Technology",
    "24.7626496": "Samsung Semiconductor Inc.",
    "24.6303264": "Huawei Technologies Co.,Ltd",
    "12.123917685723136": "SPETSSTROY-SVYAZ Ltd",
    "12.119270031360": "Eldes",
    "24.57740": "Intel Corporate",
    "24.14190549": "Leadcore Technology CO.,LTD",
    "12.119269765120": "Diwel",
    "12.123917681082368": "Akse srl",
    "12.346855505920": "Sicon srl",
    "12.123917680840704": "Private",
    "12.123917675880448": "Tattile Srl",
    "24.14213345": "Samsung Electronics Co.,Ltd",
    "12.123917681491968": "Lge",
    "24.15498356": "Justone Technologies Co., Ltd.",
    "12.123917690150912": "KDT Corp.",
    "12.123917677236224": "LX Design House",
    "24.6054847": "Tp-Link Technologies Co.,Ltd.",
    "24.5534950": "Webtech Wireless",
    "12.123917681795072": "Micatu",
    "12.346858545152": "Honeywell",
    "20.277783351656448": "Broadcom Inc.",
    "24.6050177": "Samsung Electronics Co.,Ltd",
    "12.123917689044992": "Mettler Toledo",
    "24.9448988": "Intel Corporate",
    "12.123917679398912": "Power Electronics Espana, S.L.",
    "24.7359150": "Xstream Flow (Pty) Ltd",
    "24.7141138": "Nokia Solutions and Networks GmbH & Co. KG",
    "24.9746993": "Visiontec da Amazônia Ltda.",
    "24.1871754": "Phase Motion Control SpA",
    "24.7140840": "Mooredoll Inc.",
    "24.7612220": "ChangYang Technology (Nanjing) Co., LTD",
    "24.12084045": "Apple, Inc.",
    "12.123917690560512": "Metrasens Limited",
    "24.5002700": "Zentri Pty Ltd",
    "24.4719775": "Elecom Co., Ltd",
    "12.346863427584": "AP Labs",
    "12.123917687730176": "Sumitomo Heavy Industries, Ltd.",
    "20.31477065580544": "Dynamic Connect (Suzhou) Hi-Tech Electronic Co.,Ltd.",
    "24.3988077": "Tiandy Technologies CO.,LTD",
    "24.16052570": "Apple, Inc.",
    "24.7095704": "Antex Electronic Corp.",
    "24.63584": "PT. Panggung Electric Citrabuana",
    "24.1889292": "Kriwan Industrie-Elektronik GmbH",
    "12.346868047872": "WaveIP",
    "24.7637801": "Micobiomed",
    "24.11561329": "DEI Sales, Inc.",
    "24.4466675": "70mai Co.,Ltd.",
    "12.123917684391936": "ACQUA-SYSTEMS srls",
    "24.10243775": "BoxCast",
    "24.6309323": "ASUSTek COMPUTER INC.",
    "24.13640382": "Samsung Electro-Mechanics(Thailand)",
    "12.71297883889664": "GD Mission Systems",
    "12.346860142592": "Vital Systems Inc",
    "24.10789241": "Nanjing dianyan electric power automation co. LTD",
    "12.346861506560": "Unidata",
    "24.14703472": "Innovid, Co., Ltd.",
    "20.67047986823168": "LIPS Corporation",
    "24.5265767": "Intent Solutions Inc",
    "24.14962527": "EDS Elektronik Destek San.Tic.Ltd.Sti",
    "24.563096": "Compal Information (Kunshan) Co., Ltd.",
    "12.346865262592": "Buerkert Werke GmbH",
    "24.2895923": "Netgear",
    "24.39938": "Hewlett Packard",
    "24.13922824": "Bragi GmbH",
    "24.6315": "Beijing Lhwt Microelectronics Inc.",
    "24.10536101": "Sygn House Co.,Ltd",
    "24.5562418": "Juniper Networks",
    "24.16422129": "IEEE 802.1 Working Group",
    "24.7114085": "Wireless Glue Networks, Inc.",
    "12.123917675032576": "Sensoneo",
    "24.3971575": "Lansentechnology AB",
    "12.346856198144": "Bavaria Digital Technik GmbH",
    "12.346861572096": "Honeywell",
    "24.16051000": "Sagemcom Broadband SAS",
    "20.154238428643328": "Edge Networks Inc",
    "24.7404869": "Huawei Technologies Co.,Ltd",
    "20.58071054286848": "Skytech Creations Limited",
    "12.123917685780480": "Kanoa Inc",
    "24.12117881": "9Solutions Oy",
    "12.123917677408256": "Tosei Engineering Corp.",
    "12.346861166592": "Blankom",
    "24.8976327": "Technicolor CH USA Inc.",
    "20.84372896088064": "Shenzhen Shanling Digital Technology Development Co.,Ltd.",
    "24.13648329": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.2150752": "Private",
    "24.2402201": "Huawei Device Co., Ltd.",
    "12.346861920256": "Zynix Original Sdn. Bhd.",
    "12.346854907904": "SELEX Communications Ltd",
    "12.123917681143808": "Movicom LLC",
    "24.16570163": "Tp-Link Technologies Co.,Ltd.",
    "24.12868542": "Espressif Inc.",
    "12.119270039552": "Connode",
    "24.15261223": "Tp-Link Technologies Co.,Ltd.",
    "12.123917675638784": "Jacarta Ltd",
    "24.1502": "Gi Fone Korea, Inc.",
    "12.123917679443968": "Scenario Automation",
    "12.346863398912": "Twinfalls Technologies",
    "24.12864465": "Wallys Communications Teachnologies Co.,Ltd.",
    "24.14972707": "zte corporation",
    "24.12890681": "Cisco Systems, Inc",
    "24.8394919": "Samsung Electronics Co.,Ltd",
    "24.8714371": "Luminar Technologies",
    "24.8976498": "Huawei Technologies Co.,Ltd",
    "12.346864832512": "INO - Institut National d'Optique",
    "24.10526086": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "24.11842436": "ShenZhen Figigantic Electronic Co.,Ltd",
    "24.14210108": "Sony Corporation",
    "24.5525787": "Huawei Technologies Co.,Ltd",
    "20.145727912148992": "BYDA Co. Ltd.,",
    "24.11072116": "Samsung Electronics Co.,Ltd",
    "12.123917683277824": "C21 Systems Ltd",
    "24.1504": "Empirix Corp.",
    "24.9970415": "Liteon Technology Corporation",
    "24.6847851": "Samsung Electronics Co.,Ltd",
    "24.15785657": "ShangHai Y&Y Electronics Co., Ltd",
    "12.123917675548672": "Human Systems Integration",
    "24.12884088": "Shanghai Moaan Intelligent Technology Co.,Ltd",
    "24.11315481": "TrekStor GmbH",
    "24.9206768": "Xufeng Development Limited",
    "24.7129608": "ARRIS Group, Inc.",
    "24.10798228": "Intel Corporate",
    "20.119143429832704": "Syowatsusinkougyo Co.,Ltd.",
    "24.3457113": "Apple, Inc.",
    "20.154039763337216": "smartHome Partner GmbH",
    "12.346862460928": "Saia-Burgess Controls AG",
    "12.346857791488": "Hanzas Elektronika, Sia",
    "12.123917681717248": "CT Company",
    "20.31368183545856": "TASC Systems Inc.",
    "24.2645979": "Huawei Technologies Co.,Ltd",
    "24.13642528": "Apple, Inc.",
    "24.12064879": "Oriental Motor Co., Ltd.",
    "20.101728427966464": "Nautech Electronics Ltd",
    "24.7905361": "iiNet Labs Pty Ltd",
    "12.123917688520704": "Waves System",
    "12.71297884770304": "Standard Change Makers",
    "24.2449": "Intelligent Platforms, LLC.",
    "24.2448": "ACKSYS Communications & systems",
    "24.4489957": "Huawei Technologies Co.,Ltd",
    "24.8441665": "Lenovo Mobile Communication Technology Ltd.",
    "24.16047791": "Huawei Technologies Co.,Ltd",
    "24.2415286": "Sistemas de Gestión Energética S.A. de C.V",
    "12.123917685559296": "Trioptics",
    "24.13949209": "GoPro",
    "24.1606169": "Alibaba Cloud Computing Ltd.",
    "20.251018418847744": "SmallHD",
    "24.667": "Kreatel Communications AB",
    "24.3714446": "New H3C Technologies Co., Ltd",
    "12.123917688451072": "Controlrad",
    "20.220515721543680": "Comlab AG",
    "12.346855706624": "Contec",
    "12.346858217472": "Technische Alternative GmbH",
    "12.123917688938496": "OÜ ELIKO Tehnoloogia Arenduskeskus",
    "24.3933611": "Product Creation Studio",
    "12.346858790912": "Wideco Sweden AB",
    "24.13929653": "Cisco Systems, Inc",
    "24.2443": "Entropic Communications, Inc.",
    "24.4780071": "Chengdu newifi Co.,Ltd",
    "12.346853236736": "Matsuoka",
    "24.669": "Merix Corp.",
    "24.14209483": "Apple, Inc.",
    "12.123917691289600": "Odawarakiki Auto-Machine Mfg.Co.,Ltd",
    "24.57592": "Dicna Control Ab",
    "24.57593": "Cisco Systems, Inc",
    "24.9454562": "Cornami, Inc",
    "24.11069189": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.57590": "Decision Europe",
    "24.57591": "Cisco Systems, Inc",
    "24.16576230": "Formike Electronic Co., Ltd",
    "24.3925": "Datawire Communication Networks Inc.",
    "24.8971779": "Avotek corporation",
    "24.57597": "A-Trend Technology Co., Ltd.",
    "12.346866991104": "DiTEST Fahrzeugdiagnose GmbH",
    "24.57594": "Trl Technology, Ltd.",
    "24.11046920": "Beijing Yuecheng Technology Co. Ltd.",
    "24.3926": "Continuum Photonics Inc",
    "12.123917684576256": "QDevil",
    "24.3436525": "Enovation Controls",
    "24.1588574": "Huawei Technologies Co.,Ltd",
    "24.34029": "Private",
    "24.45130": "Cisco Systems, Inc",
    "24.5273947": "Interexport Telecomunicaciones S.A.",
    "24.6067630": "Apple, Inc.",
    "24.45138": "Atheros Communications",
    "24.1579478": "Samsung Electronics Co.,Ltd",
    "24.6557238": "Taztag",
    "12.123917685710848": "Beijing DamingWuzhou Science&Technology Co., Ltd.",
    "12.346867142656": "Ctf Technologies Do Brasil Ltda",
    "20.207357802971136": "Precision Galaxy Pvt. Ltd",
    "20.61822323392512": "Foresight Sports",
    "12.346859483136": "WaveIP",
    "12.346865246208": "Protonic Holland",
    "20.176189955112960": "KoCoS Messtechnik AG",
    "24.8449324": "Apple, Inc.",
    "24.13653695": "Amosense",
    "24.2123150": "Huawei Technologies Co.,Ltd",
    "12.346853806080": "Partner Voxstream A/S",
    "24.7541": "Radioscape PLC",
    "20.88668795568128": "Beijing ANTVR Technology Co., LTD",
    "24.12615989": "Ningbo Forfan technology Co., LTD",
    "24.11836852": "Axion Technologies Inc.",
    "24.11815995": "Zhejiang Dahua Technology Co., Ltd.",
    "12.71297898799104": "Narinet, Inc.",
    "24.4246440": "Huawei Technologies Co.,Ltd",
    "24.11585349": "Buffalo.Inc",
    "24.5024778": "zte corporation",
    "24.1863398": "Nhnetworks",
    "12.123917686046720": "Shangdong Bosure Automation Technology Ltd",
    "24.7120388": "ARRIS Group, Inc.",
    "20.119709251928064": "Hashtrend AG",
    "24.1853646": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.16551": "Itautec Philco S.A.",
    "24.2109724": "vivo Mobile Communication Co., Ltd.",
    "24.16553": "Datacom Inc.",
    "24.16552": "Imf International Ltd.",
    "24.16555": "Roland Dg Corporation",
    "24.16554": "Valmet Automation Inc",
    "24.16557": "Sma Regelsysteme Gmbh",
    "24.16556": "Super Workstation, Inc.",
    "24.16559": "Digital Products, Inc. (DPI).",
    "24.16558": "Delta Controls, Inc.",
    "12.346865786880": "Techleader",
    "24.12874857": "Bt9",
    "20.48543324700672": "Polara Engineering",
    "24.1070609": "ARRIS Group, Inc.",
    "20.206931539001344": "LLD Technology Ltd.",
    "12.346853638144": "ExtremeSpeed Inc.",
    "12.123917676015616": "Bespoon",
    "24.15530666": "The IMS Company",
    "24.13691806": "Seowoo Inc.",
    "20.14223572729856": "Kenmore",
    "20.137215121293312": "Hangzhou Haohaokaiche Technology Co.,Ltd.",
    "12.71297884958720": "Bettini Srl",
    "24.7905136": "Apple, Inc.",
    "12.119270019072": "EREE Electronique",
    "24.12908501": "Cisco Systems, Inc",
    "12.123917683318784": "Adveez",
    "24.1576055": "Westinghouse Electric Company, LLC",
    "12.123917676834816": "DEUTA-WERKE GmbH",
    "24.58368": "Sichuan Changhong Electric Ltd.",
    "24.10244047": "TCT mobile ltd",
    "24.12589136": "Quanta Computer Inc.",
    "24.9965092": "Phytium Technology Co.,Ltd.",
    "12.123917680312320": "Psl Elektroni̇k Sanayi̇ Ve Ti̇caret A.S.",
    "20.62335513264128": "Hdanywhere",
    "24.5282": "Core Micro Systems Inc.",
    "24.5283": "Vitelec BV",
    "24.5280": "Accsense, Inc.",
    "24.5281": "Synchronous Communication Corp",
    "24.5286": "Teranetics, Inc.",
    "24.5287": "Nokia Danmark A/S",
    "24.5284": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5285": "Gemtek Technology Co., Ltd.",
    "24.5288": "Cisco Systems, Inc",
    "24.5289": "Cisco Systems, Inc",
    "24.15220406": "Rim",
    "12.346857594880": "Dsp Design",
    "24.803604": "Silicon Laboratories",
    "24.15238342": "Huawei Technologies Co.,Ltd",
    "24.41727": "abatec group AG",
    "24.4741417": "LG Electronics (Mobile Communications)",
    "24.9672": "S-Access GmbH",
    "24.3201345": "Raydium Semiconductor Corp.",
    "12.346866532352": "ZAO NPC \"Kompjuternie Technologii\"",
    "24.4750174": "Smt Telecomm Hk",
    "20.84372902379520": "Wonder Workshop",
    "24.6889": "Nintendo Co., Ltd.",
    "24.6888": "Unify Software and Solutions GmbH & Co. KG",
    "24.6885": "Mvox Technologies Inc.",
    "24.6884": "Medicis Technologies Corporation",
    "24.6887": "Aztek Networks, Inc.",
    "24.6886": "Atlanta Advanced Communications Holdings Limited",
    "24.6881": "Edge Access Inc",
    "24.6880": "Mythology Tech Express Inc.",
    "24.6883": "Cisco Systems, Inc",
    "24.6882": "Cisco Systems, Inc",
    "24.6595531": "Apple, Inc.",
    "20.145211811430400": "Ei3 Corporation",
    "24.16567524": "Ascon Ltd.",
    "24.5266860": "Huawei Technologies Co.,Ltd",
    "20.93020691103744": "eTauro LLC",
    "12.346854998016": "Polatis Ltd",
    "24.6320651": "BLU Products Inc",
    "24.11800286": "Intel Corporate",
    "24.11800284": "LG-Ericsson Co.,Ltd.",
    "24.6332682": "Samsung Electronics Co.,Ltd",
    "24.14715293": "Cisco Systems, Inc",
    "24.3418511": "IEEE Registration Authority",
    "24.1329154": "THE YEOLRIM Co.,Ltd.",
    "24.1072604": "Huawei Device Co., Ltd.",
    "24.5530090": "Zaplox AB",
    "24.1060145": "Shenzhen Americas Trading Company LLC",
    "24.16016201": "Diffon Corporation",
    "24.13685266": "K2NET Co.,Ltd.",
    "24.526926": "Planet Bingo® — 3rd Rock Gaming®",
    "20.251018411507712": "Beijing Zhongchuangwei Nanjing Quantum Communication Technology Co., Ltd.",
    "12.123917684011008": "EFG CZ spol. s r.o.",
    "20.180680388509696": "Shenzhen Xunqi Interconnet Technology Co., Ltd",
    "12.123917681713152": "Aplex Technology Inc.",
    "24.11550146": "Wasp artificial intelligence(Shenzhen) Co.,ltd",
    "24.5547002": "BQT Solutions (Australia)Pty Ltd",
    "24.9476053": "Espressif Inc.",
    "12.346858647552": "Qes",
    "12.71297884266496": "Davis Derby Limited",
    "24.11544192": "Cisco Systems, Inc",
    "12.123917687513088": "Sicon srl",
    "24.9973873": "Daikyung Vascom",
    "20.88914846023680": "Penny & Giles Aerospace Ltd",
    "24.6830403": "ittim",
    "24.4629": "iStor Networks, Inc.",
    "24.4628": "Koenig & Bauer AG",
    "24.13423517": "IEEE Registration Authority",
    "24.4623": "Ieee 802.3",
    "24.4622": "AboCom",
    "24.4621": "Advanced Telecommunication Technologies, Inc.",
    "24.4620": "CE-Infosys Pte Ltd",
    "24.4627": "Metrohm AG",
    "24.4626": "PLUS Corporation",
    "24.4625": "Protechna Herbst GmbH & Co. KG",
    "24.4624": "WideRay Corp",
    "24.3936": "Lifetron Co.,Ltd",
    "24.1353515": "currentoptronics Pvt.Ltd",
    "24.3934": "Veo",
    "24.3935": "Nicety Technologies Inc. (NTS)",
    "24.3932": "Day One Digital Media Limited",
    "24.3933": "Genexis BV",
    "24.3930": "Peribit Networks",
    "24.3931": "Delta Information Systems, Inc.",
    "24.3938": "Alcatel Bell Space N.V.",
    "24.3939": "Obzerv Technologies",
    "24.57589": "Teles Ag",
    "24.57588": "INSIDE Technology A/S",
    "12.346859667456": "InterDigital Canada Ltd",
    "24.57585": "That Corporation",
    "24.57584": "Abler Technology, Inc.",
    "24.57587": "WebSprint Communications, Inc.",
    "24.57586": "Arlotto Comnet, Inc.",
    "24.57581": "Silicom, Ltd.",
    "24.57580": "Celestica Inc.",
    "24.57583": "Dionex",
    "24.57582": "Marel Hf",
    "24.12380054": "Wireless CCTV Ltd",
    "24.11813670": "Huawei Technologies Co.,Ltd",
    "12.123917689659392": "Newtouch Electronics (Shanghai) Co.,Ltd.",
    "24.14726922": "ARRIS Group, Inc.",
    "24.16024652": "Texas Instruments",
    "24.11549014": "Netgear",
    "20.22739790331904": "Qingdao iTechene Technologies Co., Ltd.",
    "24.16263193": "Liteon Technology Corporation",
    "24.13671173": "Emtronix",
    "12.123917689921536": "Cyanview",
    "24.8171714": "Intel Corporate",
    "24.14173276": "DEV Tecnologia Ind Com Man Eq LTDA",
    "20.242185271246848": "Tesla,Inc.",
    "24.27550": "Vizio, Inc",
    "24.27552": "Shenzhen Universal Intellisys Pte Ltd",
    "20.207357813456896": "Rollock Oy",
    "24.9444042": "ARRIS Group, Inc.",
    "20.57194440556544": "Seeiner Technology Co.,LTD",
    "20.31614744657920": "Secheron Sa",
    "20.84187702886400": "Chongqing Unisinsight Technology Co.,Ltd.",
    "12.123917677080576": "Crane-elec. Co., LTD.",
    "24.7927127": "Huawei Technologies Co.,Ltd",
    "20.53091437117440": "Advanced Microwave Engineering Srl",
    "12.71297883787264": "Heart Force Medical",
    "20.229558882140160": "shenzhen cnsun",
    "12.346852728832": "Rising Edge Technologies",
    "24.5044573": "Huawei Technologies Co.,Ltd",
    "24.8843": "Kensington Computer Products Group",
    "24.8842": "Teratronik elektronische systeme gmbh",
    "24.8841": "Vanderlande APC inc.",
    "24.8840": "Sagrad, Inc.",
    "24.8847": "Cnrs",
    "24.8846": "Tv-Numeric",
    "24.8845": "GBS Laboratories LLC",
    "24.8844": "Photon Europe GmbH",
    "24.4739681": "Soreel",
    "24.8849": "Cisco Systems, Inc",
    "24.8848": "Cisco Systems, Inc",
    "24.6578723": "Cellient Co., Ltd.",
    "12.346853335040": "Volvo Car Corporation",
    "24.53463": "B2C2, Inc.",
    "24.53460": "V-Bits, Inc.",
    "24.53461": "Grundig Ag",
    "24.53466": "Taicom Data Systems Co., Ltd.",
    "24.53467": "Mcquay International",
    "24.53464": "3Com",
    "24.53465": "Dedicated Microcomputers",
    "20.211776049250304": "Osprey Video, Inc",
    "24.53468": "Modular Mining Systems, Inc.",
    "24.53469": "Sunrise Telecom, Inc.",
    "24.8154012": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.346854080512": "Phase IV Engineering Inc.",
    "24.13166454": "PTCOM Technology",
    "24.12598745": "MitraStar Technology Corp.",
    "12.346853789696": "CCII Systems (Pty) Ltd",
    "24.6315599": "Huawei Device Co., Ltd.",
    "12.346866405376": "Honeywell",
    "24.11850037": "Cloudistics",
    "12.346867826688": "Homaetrix Ltd",
    "24.12374677": "Cisco Systems, Inc",
    "24.15755048": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917675446272": "AppTek",
    "24.11851342": "Caldero Limited",
    "24.4739535": "Intelbras",
    "24.8420835": "Chongqing Sichuan Instrument Microcircuit Co.LTD.",
    "24.2361002": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.6416": "Knick Elektronische Messgeraete GmbH & Co. KG",
    "24.6417": "Just In Mobile Information Technologies (Shanghai) Co., Ltd.",
    "24.6414": "Atech Technology Co., Ltd.",
    "24.6415": "Advansus Corp.",
    "24.6412": "Encore Electronics, Inc.",
    "24.6413": "IEEE 1394c",
    "24.6410": "Hasware Inc.",
    "24.6411": "Southern Vision Systems, Inc.",
    "12.346862575616": "Michael Stevens & Partners Ltd",
    "24.6418": "Welcat Inc",
    "24.6419": "Chuang-Yi Network Equipment Co.Ltd.",
    "24.9443227": "Nanomegas",
    "20.229206572138496": "Annapurna Labs",
    "24.14965410": "vivo Mobile Communication Co., Ltd.",
    "20.233630189223936": "Pongee Industries Co., Ltd.",
    "24.11019993": "HTC Corporation",
    "24.10772064": "Apple, Inc.",
    "24.15992461": "devolo AG",
    "24.7639071": "Ragile Networks Inc.",
    "12.123917680132096": "W.A. Benjamin Electric Co.",
    "24.13921645": "Intel Corporate",
    "24.5537724": "iodyne",
    "24.24769": "WaveSpan Corporation",
    "24.24768": "Nera Networks AS",
    "12.346863013888": "GSP Sprachtechnologie GmbH",
    "24.24766": "Webtronics",
    "24.24765": "Enginuity Communications",
    "24.24764": "KeunYoung Electronics & Communication Co., Ltd.",
    "24.24763": "Cabletron Systems, Inc.",
    "24.24762": "Sahara Networks, Inc.",
    "24.24761": "NEC Platforms, Ltd",
    "24.24760": "CORELIS Inc.",
    "24.14444090": "Apacer Technology Inc.",
    "24.8924499": "Baidu Online Network Technology (Beijing) Co., Ltd.",
    "20.127937509588992": "Celyss Sas",
    "12.123917683097600": "SerEnergy A/S",
    "24.1587566": "Sunsea Telecommunications Co.,Ltd.",
    "20.93065818669056": "ShenZhen Smart&Aspiration Co.,LTD",
    "12.123917679247360": "Tornado Modular Systems",
    "24.14434789": "Shenzhen Zhixin Data Service Co., Ltd.",
    "24.6296978": "IEEE Registration Authority",
    "24.9998486": "Dell Inc.",
    "24.374": "Orient Silver Enterprises",
    "24.375": "Edsl",
    "24.376": "MARGI Systems, Inc.",
    "24.377": "Wireless Technology, Inc.",
    "24.370": "TechnoLand Co., LTD.",
    "24.371": "Amcc",
    "24.372": "CyberOptics Corporation",
    "24.373": "Radiant Communications Corp.",
    "24.2913": "Friedrich Lütze GmbH & Co. KG",
    "24.2912": "Cisco Systems, Inc",
    "24.2911": "Cisco Systems, Inc",
    "20.52821118418944": "KAZUtechnica Co.,Ltd.",
    "24.378": "Chengdu Maipu Electric Industrial Co., Ltd.",
    "24.379": "Heidelberger Druckmaschinen AG",
    "24.2915": "Kaleidescape",
    "20.52912290004992": "NCM Supplies, Inc.",
    "24.9039": "Luminous Power Technologies Pvt. Ltd.",
    "24.9038": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9031": "ProCurve Networking by HP",
    "24.9030": "Vestac",
    "24.9033": "Helmholtz Centre Berlin for Material and Energy",
    "24.9032": "Sagemcom Broadband SAS",
    "24.9035": "Inyuan Technology Inc.",
    "24.9034": "Private",
    "24.9037": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9036": "Ktc Ab",
    "24.11854276": "Microsoft Mobile Oy",
    "12.346852765696": "Korealink",
    "24.12591373": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.11330741": "Apple, Inc.",
    "24.2422186": "Samsung Electronics Co.,Ltd",
    "24.4772431": "Sagemcom Broadband SAS",
    "24.2419557": "SAET I.S. S.r.l.",
    "24.10245069": "ENGICAM s.r.l.",
    "24.5547339": "NSC Communications Siberia Ltd",
    "24.294474": "moobox CO., Ltd.",
    "24.13410603": "TCL King Electrical Appliances (Huizhou) Co., Ltd",
    "24.16308962": "NDC Technologies",
    "12.346854408192": "PANNOCOM Ltd.",
    "24.6846738": "IEEE Registration Authority",
    "20.9814033825792": "D2SLink Systems",
    "12.123917682569216": "SD Biosensor",
    "24.14682920": "Inovonics",
    "24.6856217": "Texas Instruments",
    "24.4255791": "Liteon Technology Corporation",
    "24.7116024": "Apple, Inc.",
    "12.123917676544000": "Gamber Johnson-LLC",
    "20.273272608391168": "Excetop Technology (Beijing) Co., Ltd.",
    "24.1080132": "ASUSTek COMPUTER INC.",
    "24.3986149": "New H3C Technologies Co., Ltd",
    "24.7097232": "LiteON",
    "24.11292305": "Huawei Technologies Co.,Ltd",
    "20.70559708741632": "Guangzhou LANGO Electronics Technology Co., Ltd.",
    "24.12635384": "Apple, Inc.",
    "24.16022366": "Samsung Electronics Co.,Ltd",
    "24.2406072": "Friem Spa",
    "24.9443422": "Apex Tool Group GmbH & Co OHG",
    "24.1101230": "Avaya Inc",
    "24.8950237": "Racktivity",
    "24.10496051": "IEEE Registration Authority",
    "24.1101238": "Essential Products, Inc.",
    "24.2999": "Micro Systems Co.,Ltd.",
    "24.8179663": "Shanghai SEARI Intelligent System Co., Ltd",
    "12.123917689028608": "Flintab AB",
    "24.8072": "FMS Force Measuring Systems AG",
    "24.8073": "Signalion GmbH",
    "24.8070": "digEcor",
    "24.8071": "Skydigital Inc.",
    "24.8076": "CCS Inc.",
    "24.8077": "Ingenieurbuero Stark GmbH und Ko. KG",
    "24.8074": "Ellion Digital Inc.",
    "24.8075": "Cache IQ",
    "24.8078": "Metris USA Inc.",
    "24.8079": "Shanghai Bellmann Digital Source Co.,Ltd.",
    "24.4230134": "Shenzhen TCL New Technology Co., Ltd",
    "24.12636415": "China Mobile IOT Company Limited",
    "24.26187": "Huawei Technologies Co.,Ltd",
    "24.2382832": "Liteon, Inc.",
    "12.346861404160": "Saia-Burgess Controls AG",
    "20.277783356899328": "Syfer",
    "24.6054490": "Canon Inc.",
    "24.1613356": "Huawei Device Co., Ltd.",
    "24.12896985": "Hangzhou H3C Technologies Co., Limited",
    "20.159182258962432": "Shenzhen Lencotion Technology Co.,Ltd",
    "24.1883406": "Amplified Engineering Pty Ltd",
    "12.123917679239168": "Monroe Electronics, Inc.",
    "24.4729571": "Wistron InfoComm(Kunshan)Co.,Ltd.",
    "12.346862764032": "Microtechnica",
    "12.346862751744": "Icon Time Systems",
    "24.4478404": "Huawei Device Co., Ltd.",
    "20.18540133351424": "ABB Switzerland Ltd.",
    "20.176094523162624": "Hzhy Technology",
    "12.123917677531136": "Gilbarco Veeder-Root ‎",
    "24.3467537": "vivo Mobile Communication Co., Ltd.",
    "20.193649369415680": "Advanced & Wise Technology Corp.",
    "12.123917682294784": "Private",
    "12.346868465664": "UBSTechnology Co., Ltd",
    "24.4775325": "Itel Mobile Limited",
    "12.346855333888": "Noran Tel Communications Ltd.",
    "24.7341107": "California Things Inc.",
    "12.71297883369472": "Advatek Lighting Pty Ltd",
    "12.123917685948416": "Gateview Technologies",
    "24.6092833": "Texas Instruments",
    "24.15739489": "Waldo Networks, Inc.",
    "24.291155": "Sernet (Suzhou) Technologies Corporation",
    "24.10786611": "Askey Computer Corp",
    "12.71297884241920": "Imple Sistemas Eletronicos Embarcados Ltda",
    "24.2134489": "I'M Spa",
    "20.180659897237504": "Dermalog Identification Systems GmbH",
    "24.8930550": "Nokia Corporation",
    "24.13920913": "SnapAV",
    "24.325826": "Flaircomm Microelectronics, Inc.",
    "12.346855710720": "AZD Praha s.r.o.",
    "24.5513501": "Huawei Device Co., Ltd.",
    "12.346864664576": "DRS Laruel Technologies",
    "20.97400611930112": "Gmv sistemas SAU",
    "24.15250092": "Apple, Inc.",
    "12.123917691555840": "Shanghai EICT Global Service Co., Ltd",
    "24.4506991": "NGN Easy Satfinder (Tianjin) Electronic Co., Ltd",
    "24.14737660": "Huawei Device Co., Ltd.",
    "12.123917675020288": "Leidos",
    "24.9859": "Ajoho Enterprise Co., Ltd.",
    "24.9858": "Gemtek Technology Co., Ltd.",
    "24.9853": "A-Max Technology Macao Commercial Offshore Company Limited",
    "12.346866294784": "Transbit Sp. z o.o.",
    "24.9851": "GSI Helmholtzzentrum für Schwerionenforschung GmbH",
    "24.9850": "wuhan hongxin telecommunication technologies co.,ltd",
    "24.9857": "Interspiro AB",
    "24.9856": "SIL3 Pty.Ltd",
    "24.9855": "Zenterio AB",
    "24.9854": "Parrot Sa",
    "12.71297885093888": "Prisma Engineering srl",
    "12.123917683589120": "Nordson Corporation",
    "12.123917688926208": "YotaScope Technologies Co., Ltd.",
    "20.220092033925120": "Verifone Systems (China),lnc.",
    "24.1590675": "Taicang T&W Electronics",
    "24.5778395": "Pax AB",
    "24.8400353": "Solarbridge Technologies",
    "12.346863820800": "Abbott Medical Optics",
    "24.11058458": "Honey Bee (Hong Kong) Limited",
    "12.71297883463680": "Circuitlink Pty Ltd",
    "12.123917690155008": "Zeus Control Systems Ltd",
    "24.10801551": "Shenzhen Skyee Optical Fiber Communication Technology Ltd.",
    "24.12371091": "Cisco Systems, Inc",
    "12.123917688950784": "The Morey Corporation",
    "24.6305107": "Buwon Technology",
    "20.216602651918336": "Mobiletron Electronics Co., Ltd",
    "24.1069596": "Espressif Inc.",
    "24.12117385": "Payter BV",
    "24.824641": "Chipsea Technologies (Shenzhen) Corp.",
    "20.154039764385792": "Srett",
    "12.346863742976": "Honeywell",
    "12.123917682384896": "Pullnet Technology, Sa De Cv Ssc1012302S73",
    "24.4467717": "AMPAK Technology, Inc.",
    "24.12073562": "Aruba, a Hewlett Packard Enterprise Company",
    "24.10763857": "Anov France",
    "24.9139": "Lyyn AB",
    "20.45069239320576": "Millcode",
    "20.132869121900544": "Korea Micro Wireless Co.,Ltd.",
    "24.6564879": "vivo Mobile Communication Co., Ltd.",
    "20.84531185975296": "NewVastek",
    "24.16050644": "QLogic Corporation",
    "12.123917689131008": "GOMA ELETTRONICA SpA",
    "20.181338946666496": "Linseis Messgeraete GmbH",
    "24.11035653": "nimbus 9, Inc",
    "24.13635289": "Taicang T&W Electronics",
    "12.346858328064": "Ittiam Systems (P) Ltd",
    "24.1616586": "Sichuan tianyi kanghe communications co., LTD",
    "24.11841067": "Microsoft",
    "24.13147165": "Cisco Systems, Inc",
    "12.346863804416": "Elcus",
    "24.32919": "Centralp Automatismes",
    "24.32918": "Human Designed Systems, Inc.",
    "12.346856685568": "Enconair Ecological Chambers Inc.",
    "24.32913": "Tokyo Electric Co.,Ltd",
    "24.32912": "Microtek International, Inc.",
    "24.32911": "C. Itoh Electronics, Inc.",
    "24.32910": "Radstone Technology",
    "24.32917": "Basic Merton Handelsges.M.B.H.",
    "24.32916": "Alfa Laval Automation Ab",
    "24.32915": "Xyron Corporation",
    "24.32914": "Silex Technology, Inc.",
    "20.62070505603072": "PingGPS Inc",
    "20.101728424820736": "Zhejiang Jetron Ark Digital Technology Co., Ltd",
    "24.7401878": "Actiontec Electronics, Inc",
    "20.123598311260160": "Intelitech SIA",
    "24.1881065": "Shenzhen Zhongke United Communication Technology",
    "24.8959726": "Dish Technologies Corp",
    "24.6401": "F1Media",
    "20.83889189027840": "Remedee Labs",
    "24.9198753": "d-broad,INC",
    "24.6400": "Intelliverese - DBA Voicecom",
    "24.6403": "Bigfoot Networks Inc",
    "12.71297884237824": "Peek Traffic Corporation",
    "24.6402": "Cambridge Consultants Ltd",
    "20.264165322129408": "Shenzhen Wisewing Internet Technology Co.,Ltd",
    "24.6405": "SCHRACK Seconet AG",
    "24.2156660": "Apple, Inc.",
    "20.172584085749760": "Beijing Sifang Automation Co., Ltd.",
    "24.6404": "WB Electronics Sp. z o.o.",
    "24.13653883": "Intel Corporate",
    "24.6407": "Cisco Systems, Inc",
    "24.15501458": "Motorola Mobility LLC, a Lenovo Company",
    "20.66219401019392": "Marposs Spa",
    "12.346857512960": "SpeasTech, Inc.",
    "24.6406": "Cisco Systems, Inc",
    "24.10502567": "NL Ministry of Defense",
    "20.180577520058368": "Private",
    "24.2100885": "Iec – Tc9 Wg43",
    "12.346856894464": "Ses",
    "12.123917681864704": "Sadel S.p.A.",
    "24.7096965": "Apple, Inc.",
    "24.8668335": "Zhejiang Tmall Technology Co., Ltd.",
    "24.16952": "Intel Corporate",
    "24.2898941": "Netstor Technology Co., Ltd.",
    "24.11560751": "Bowers & Wilkins",
    "12.119269744640": "Bio Molecular System Pty Ltd",
    "24.16052705": "Motorola Mobility LLC, a Lenovo Company",
    "12.123917684436992": "IMST GmbH",
    "24.14981307": "Phyplus Microelectronics Limited",
    "24.7392908": "Samsung Electronics Co.,Ltd",
    "12.346856611840": "Coppercom",
    "24.11843190": "Intel Corporate",
    "24.14697837": "China Mobile Group Device Co.,Ltd.",
    "12.123917679706112": "Par-Tech, Inc.",
    "24.11808850": "Huawei Technologies Co.,Ltd",
    "24.9962456": "Apple, Inc.",
    "24.7349593": "CyberTAN Technology Inc.",
    "12.71297883652096": "Sphere Medical Ltd",
    "24.15264564": "Cisco Systems, Inc",
    "12.346863128576": "JTL Systems Ltd.",
    "12.123917677076480": "EBZ SysTec GmbH",
    "24.13145090": "Chongqing Fugui Electronics Co.,Ltd.",
    "12.123917682135040": "Northbound Networks Pty. Ltd.",
    "12.71297885089792": "Kaluga Teletypes Manufacturing Plant",
    "20.114869208612864": "Appotronics Co., Ltd",
    "20.110373782945792": "Protectli",
    "24.8156043": "Cocoon Alarm Ltd",
    "20.13592320540672": "Energybox Limited",
    "12.346866778112": "Koncar Electrical Engineering Institute",
    "12.123917684908032": "AixControl GmbH",
    "24.4212989": "IEEE Registration Authority",
    "24.9726583": "Technicolor CH USA Inc.",
    "24.11281204": "Infinix mobility limited",
    "24.8936069": "Wistron Neweb Corporation",
    "24.14176486": "ASUSTek COMPUTER INC.",
    "20.194284247580672": "Hanmecips Co.",
    "12.123917690552320": "Wilmore Electronics Company",
    "24.1844429": "Askey Computer Corp.",
    "12.346857689088": "SecuInfo Co., Ltd.",
    "24.5807356": "FreeBSD Foundation",
    "20.70682230652928": "Dynamic Engineering",
    "24.15761781": "Sagemcom Broadband SAS",
    "12.346864111616": "Peek Traffic Corporation",
    "12.71297883336704": "Grupo Epelsa S.L.",
    "24.1050033": "ARRIS Group, Inc.",
    "24.3452851": "Amazon Technologies Inc.",
    "12.346867146752": "GHL Systems Bhd",
    "24.14694335": "Wistron Neweb Corporation",
    "12.346861891584": "IMEXHIGHWAY cvba",
    "24.11015888": "Samsung Electronics Co.,Ltd",
    "12.346853134336": "Valley Products Corporation",
    "24.13402586": "Phicomm (Shanghai) Co., Ltd.",
    "24.13942489": "Dell Inc.",
    "24.12602131": "WonderSound Technology Co., Ltd",
    "24.322134": "Intel Corporate",
    "24.3981433": "Shenzhen One-Nine Intelligent Electronic Science and Technology Co., Ltd",
    "24.7883528": "f.robotics acquisitions ltd",
    "24.7387106": "Jiangsu Miter Technology Co.,Ltd.",
    "24.3980741": "Arcadyan Corporation",
    "20.44213342306304": "Shenzhen Zhi Hua Creative Technology Co., Ltd.",
    "20.260167865204736": "Waverly Labs Inc.",
    "24.41106": "H. Bollmann Manufacturers, Ltd",
    "24.41107": "B/E AEROSPACE, Inc.",
    "24.41104": "TimeStep Corporation",
    "24.41105": "Applicom International",
    "24.41102": "Check Point Software Technologies",
    "24.41103": "Desknet Systems, Inc.",
    "24.41100": "MultiMedia LANs, Inc.",
    "24.41101": "Jacomo Corporation",
    "24.11568086": "ARRIS Group, Inc.",
    "24.41108": "Comsat Corporation",
    "24.41109": "Acacia Networks, Inc.",
    "24.16047628": "ESPOD Ltd.",
    "24.1846811": "Suzhou Tremenet Communication Technology Co., Ltd.",
    "24.10550382": "Telegrafia a.s.",
    "20.224641231618048": "Beijing Neutron Technology CO.,LTD.",
    "24.9721055": "Yst Corp.",
    "12.123917683449856": "Utopi Ltd",
    "24.15484484": "Ttk Sas",
    "12.123917682163712": "Telerob Gesellschaft für Fernhantierungs",
    "12.346859417600": "Harrison Audio, LLC",
    "20.220092039168000": "UBITRON Co.,LTD",
    "24.14709599": "Nucom",
    "24.7083255": "Erhardt+Leimer GmbH",
    "24.7383730": "BlackBerry RTS",
    "12.346856292352": "megatec electronic GmbH",
    "24.12374303": "Samsung Electronics Co.,Ltd",
    "20.31368093368320": "Sigfox",
    "20.132811179687936": "Neofon GmbH",
    "20.22333156753408": "Edan Instruments, Inc.",
    "24.8968182": "ASUSTek COMPUTER INC.",
    "12.123917682323456": "Shachihata Inc.",
    "12.123917679685632": "digitrol limited",
    "20.74781681516544": "Shenzhen Ut-King Technology Co.,Ltd",
    "24.16551229": "Leapfive Tech. Ltd.",
    "12.71297884712960": "TRI Engineering co.,ltd.",
    "24.7398434": "Cisco Systems, Inc",
    "20.14286621507584": "Chengdu Ledong Information & Technology Co., Ltd.",
    "12.123917676003328": "Visual Robotics",
    "24.12847921": "Apple, Inc.",
    "24.9181621": "Arcadyan Corporation",
    "20.127654884802560": "Trend-tech Technology Co., Limited",
    "24.7138539": "Texas Instruments",
    "24.4983742": "Microsoft",
    "24.10551152": "Technicolor CH USA Inc.",
    "12.346862862336": "Casabyte Inc.",
    "12.346865807360": "Peekel Instruments B.V.",
    "12.123917679243264": "Quercus Technologies, S.L.",
    "24.1322279": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7027": "DTL Broadcast Ltd",
    "12.346855313408": "Delphi Display Systems, Inc.",
    "12.346859978752": "GeoSIG Ltd.",
    "12.346863968256": "ETM Electromatic Incorporated",
    "12.123917688229888": "APG Cash Drawer, LLC",
    "24.45301": "NetWorth Technologies, Inc.",
    "24.7394998": "Metrum Technologies",
    "12.119269588992": "optiMEAS GmbH",
    "24.7360687": "BSkyB Ltd",
    "24.14972711": "Huawei Technologies Co.,Ltd",
    "20.247076411670528": "funktel GmbH",
    "20.145383538819072": "ShenZhen Fudeyu Technology co.,Ltd",
    "12.346853175296": "Foss NIRSystems, Inc.",
    "12.123917682520064": "Medipense Inc.",
    "24.10251285": "Microsoft Corporation",
    "24.8140908": "Real Time Logic",
    "12.123917689163776": "PDD Group Ltd",
    "24.10519755": "SonicSensory,Inc.",
    "12.71297885147136": "Ad Qualite",
    "24.8140902": "Texas Instruments",
    "12.123917678759936": "ICsec S.A.",
    "24.267737": "Viwone",
    "12.346863173632": "ard sa",
    "24.4733964": "Huawei Technologies Co.,Ltd",
    "24.8934356": "Huawei Technologies Co.,Ltd",
    "20.9814035922944": "TES Touch Embedded Solutions Inc.",
    "24.9177227": "Remote Solution",
    "12.123917687738368": "speedsignal GmbH",
    "24.12109661": "Apple, Inc.",
    "24.4772238": "Metis Communication Co.,Ltd",
    "24.12101005": "Axe Group Holdings Limited",
    "24.2147853": "Shanghai annijie Information technology Co.,LTD",
    "24.11292819": "Magna Electronics Europe GmbH & Co. OHG",
    "12.123917690855424": "Atx Networks Ltd",
    "24.10261863": "Huawei Device Co., Ltd.",
    "24.10005789": "Huawei Device Co., Ltd.",
    "12.123917683175424": "KeyW Corporation",
    "24.5523612": "Cdoubles Electronics Co. Ltd.",
    "12.346855182336": "Access IS",
    "12.123917689683968": "Jeaway CCTV Security Ltd,.",
    "24.10026960": "Lenovo Mobile Communication Technology Ltd.",
    "12.123917683130368": "Torgovyy Dom Tehnologiy Llc",
    "24.12592617": "Tp-Link Technologies Co.,Ltd.",
    "24.1079411": "Shenzhen Jinkeyi Communication Co., Ltd.",
    "24.6846536": "Westunitis Co., Ltd.",
    "24.16032982": "Huawei Technologies Co.,Ltd",
    "12.346866053120": "Communication Systems Solutions",
    "24.5003210": "Apple, Inc.",
    "12.346862051328": "Peek Traffic Corporation",
    "24.570545": "Google, Inc.",
    "24.6348356": "Huawei Technologies Co.,Ltd",
    "20.140782589706240": "Llvision Technology Co.,Ltd",
    "20.83999503417344": "Shenzhen Lisaier Tronics Co.,Ltd",
    "12.346864889856": "Vidicon LLC",
    "12.123917683732480": "Hgh Systemes Infrarouges",
    "24.47233": "New platforms LLC",
    "24.50704": "Apple, Inc.",
    "24.2627837": "Aditya Infotech Ltd.",
    "12.346857947136": "Toyota Motorsport GmbH",
    "24.13380995": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "12.123917688004608": "Blue Access Inc",
    "12.346861883392": "Exi Flow Measurement Ltd",
    "24.53248": "Ferran Scientific, Inc.",
    "24.53249": "Vst Technologies, Inc.",
    "24.3464730": "Intel Corporate",
    "24.1912": "GERSTEL GmbH & Co. KG",
    "24.1913": "Sungil Telecom Co., Ltd.",
    "24.1910": "Federal APD",
    "24.1911": "Motah Ltd.",
    "24.1916": "Westermo Network Technologies AB",
    "24.1917": "Cisco Systems, Inc",
    "24.1914": "Infoware System Co., Ltd.",
    "24.1915": "Millimetrix Broadband Networks",
    "24.1918": "Elrest GmbH",
    "24.1919": "J Communications Co., Ltd.",
    "24.7870398": "Huawei Technologies Co.,Ltd",
    "24.2900762": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.327505": "Novamedia Innovision Sp. Z O.O.",
    "24.9228124": "BEFEGA GmbH",
    "12.346863235072": "Quantum3D, Inc.",
    "12.123917685084160": "Opsys-Tech",
    "24.10793367": "Apple, Inc.",
    "20.88010609655808": "Annapurna labs",
    "24.9229454": "Itel Mobile Limited",
    "24.5749": "ARRIS Group, Inc.",
    "12.346854404096": "visicontrol GmbH",
    "24.3997182": "Intel Corporate",
    "24.8187672": "Creative Product Design Pty. Ltd.",
    "24.5743": "Intel Corporate",
    "24.2104200": "Dongguan Liesheng Electronic Co., Ltd.",
    "24.5741": "Yulong Computer Telecommunication Scientific (Shenzhen) Co.,Ltd",
    "24.5740": "Samsung Electronics Co.,Ltd",
    "24.5747": "Bury GmbH & Co. KG",
    "24.5746": "Zenway enterprise ltd",
    "24.5745": "Symphox Information Co.",
    "24.5744": "SKNET Corporation",
    "24.4296": "Communications Electronics Security Group",
    "24.4297": "Mitsubishi Electronics Logistic Support Co.",
    "24.4294": "Universal Global Scientific Industrial Co., Ltd.",
    "24.4295": "Data Transmission Network",
    "24.4292": "Media Global Links Co., Ltd.",
    "24.4293": "Protocol Technologies, Inc.",
    "24.4290": "Willnet, Inc.",
    "24.4291": "Csi-Control Systems",
    "20.4884637155328": "uAvionix Corporation",
    "12.123917678821376": "Ascenix Corporation",
    "24.4298": "Telco Systems, Inc.",
    "24.4299": "Facit K.K.",
    "24.9004": "Senticare",
    "24.9005": "SandForce",
    "24.9006": "Kedah Electronics Engineering, LLC",
    "24.9007": "Advanced Card Systems Ltd.",
    "24.11857445": "Shenzhen Belon Technology CO.,LTD",
    "24.11576416": "Intel Corporate",
    "20.277958195412992": "Asesorias y Servicios Innovaxxion SPA",
    "24.9000": "Alcon Telecommunications Co., Ltd.",
    "24.9001": "DDRdrive LLC",
    "24.6072627": "Luma Home",
    "24.9002": "eonas IT-Beratung und -Entwicklung GmbH",
    "24.9003": "Ird A/S",
    "24.15481992": "MCNEX Co.,Ltd.",
    "24.7121206": "DisplayLink (UK) Ltd",
    "24.1851766": "China Mobile Group Device Co.,Ltd.",
    "12.123917689769984": "Benetel",
    "24.8657720": "Shenzhen Excelsecu Data Technology Co.,Ltd",
    "12.346862305280": "Questek Australia Pty Ltd",
    "12.346861518848": "Balfour Beatty Rail GmbH",
    "12.123917687201792": "MAMAC Systems, Inc.",
    "12.123917679505408": "Open System Solutions Limited",
    "24.5793254": "infomark",
    "24.12847178": "Nokia",
    "24.3733598": "HUMAX Co., Ltd.",
    "24.16023349": "Cisco Systems, Inc",
    "24.3697271": "Samsung Electronics Co.,Ltd",
    "20.264809144647680": "ID Tech Japan Co.,Ltd.",
    "24.15991884": "ValenceTech Limited",
    "20.212014904377344": "B9Creations",
    "24.9758455": "Intel Corporate",
    "20.31368179351552": "Visual Land Inc.",
    "12.346864521216": "Nextmove Technologies",
    "24.7658040": "Edimax Technology Co. Ltd.",
    "24.3181011": "Hongkong Ucloudlink Network Technology Limited",
    "12.71297883254784": "Grande Vitesse Systems",
    "24.3745": "Formosa Teletek Corporation",
    "24.3744": "NetKlass Technology Inc.",
    "24.3747": "CNCR-IT CO.,LTD,HangZhou P.R.CHINA",
    "24.940": "Fronius Schweissmaschinen",
    "24.3741": "Tiscali UK Ltd",
    "24.3740": "Benchmark Electronics",
    "24.945": "Hospira Inc.",
    "24.3742": "Topfield Co., Ltd",
    "24.22791": "LenovoEMC Products USA, LLC",
    "12.346862645248": "Humphrey Products",
    "24.948": "Macrotek International Corp.",
    "24.3749": "BLIP Systems",
    "24.3748": "Quantum Corp.",
    "24.6579648": "Nuvon, Inc",
    "24.15755365": "Taicang T&W Electronics",
    "24.16041491": "Alcatel-Lucent Shanghai Bell Co., Ltd",
    "12.346864656384": "Daeyoung inc.",
    "24.20488": "Tiva Microcomputer Corp. (Tmc)",
    "24.20489": "Philips Broadband Networks",
    "24.20486": "Tac Ab",
    "24.20487": "Siemens Telecommunication Systems Limited",
    "24.20484": "3com",
    "24.20482": "Omnisec Ag",
    "24.20483": "Xrite Inc",
    "24.20480": "Nexo Communications, Inc.",
    "24.20481": "Yamashita Systems Corp.",
    "24.7248": "TCL Technoly Electronics (Huizhou) Co., Ltd.",
    "24.7249": "Celeno Communications",
    "24.7244": "Petrotest Instruments",
    "24.7245": "Aplix IP Holdings Corporation",
    "24.820122": "Huawei Device Co., Ltd.",
    "24.7247": "Macab Ab",
    "24.7240": "WiDeFi, Inc.",
    "24.7241": "Zoltan Technology Inc.",
    "24.7242": "AVM GmbH",
    "24.7243": "Gener8, Inc.",
    "24.2921833": "ARRIS Group, Inc.",
    "24.10779436": "Nokia",
    "24.13667198": "Passif Semiconductor",
    "24.16257171": "Apple, Inc.",
    "24.3682732": "Weg",
    "24.49353": "Bailey Controls Co",
    "24.49352": "Micro Byte Pty. Ltd.",
    "24.49351": "Sparktrum Microsystems, Inc.",
    "12.346868342784": "Smith Meter, Inc.",
    "24.49357": "Comelta, S.A.",
    "24.49356": "Telesciences Co Systems, Inc.",
    "12.346862858240": "Fuji Firmware",
    "24.49354": "Alfa, Inc.",
    "24.49359": "Imatran Voima Oy",
    "24.49358": "Cei Systems & Engineering Pte",
    "24.6581868": "Radio Datacom LLC",
    "24.5831795": "Arria Live Media, Inc.",
    "24.3991588": "Huawei Technologies Co.,Ltd",
    "24.15005852": "Intel Corporate",
    "24.5019887": "Zeo",
    "24.7664659": "Maxwell Forest",
    "24.6581865": "Liteon Technology Corporation",
    "24.9196573": "Lge",
    "24.793625": "Longconn Electronics(Shenzhen) Co.,Ltd",
    "20.224420103716864": "MobiStor Technology Inc.",
    "20.22333155704832": "Shanghai B&A Technology Co., Ltd",
    "12.346853490688": "Phytec Messtechnik GmbH",
    "12.346862522368": "Enreduce Energy Control AB",
    "20.229206579478528": "Electro-Motive Diesel",
    "24.1899707": "Realfiction ApS",
    "24.16020616": "New H3C Technologies Co., Ltd",
    "24.8179004": "Guangzhou Juzing Technology Co., Ltd",
    "12.123917690183680": "Art Spa",
    "24.8652": "Flextronics International",
    "24.8653": "LiveTV",
    "24.8650": "ART System Co., Ltd.",
    "24.8651": "Sms Tecnologia Eletronica Ltda",
    "24.8656": "Global Display Solutions Spa",
    "24.8657": "Samsung Electronics Co.,Ltd",
    "24.8654": "NTC-Metrotek",
    "24.8655": "The Crypto Group",
    "20.110373792382976": "Signal Hound",
    "24.1086651": "Apple, Inc.",
    "24.8658": "Samsung Electronics Co.,Ltd",
    "24.8659": "Bocom Security(Asia Pacific) Limited",
    "12.123917687619584": "Talleres de Escoriaza SA",
    "24.33220": "Cisco Systems, Inc",
    "12.346857521152": "TOA Systems",
    "12.71297883607040": "Smartcom-Bulgaria AD",
    "12.346861137920": "Keynote SIGOS GmbH",
    "24.6599": "Cambridge Industries(Group) Co.,Ltd.",
    "24.6598": "zte corporation",
    "24.2038": "Qqest Software Systems",
    "24.2039": "Galtronics",
    "24.2034": "IOA Corporation",
    "24.2035": "Thinkengine Networks",
    "24.2036": "Eletex Co., Ltd.",
    "24.6590": "Altai Technologies Limited",
    "24.2030": "telco Informationssysteme GmbH",
    "24.2031": "Lockheed Martin Tactical Systems",
    "24.2032": "LogiSync LLC",
    "24.2033": "TeraBurst Networks Inc.",
    "24.3436434": "White Rodgers Division",
    "12.346858811392": "Monytel S.A.",
    "24.3436432": "Cisco Systems, Inc",
    "20.44213344403456": "ShangHai Canall Information Technology Co.,Ltd",
    "24.6625": "Nortel Networks",
    "24.6624": "Tp-Link Technologies Co.,Ltd.",
    "24.6627": "Apple, Inc.",
    "24.6626": "Juniper Networks",
    "24.6621": "FEI-Zyfer, Inc.",
    "24.6620": "ENENSYS Technologies",
    "24.6623": "Thomson Inc.",
    "24.6622": "Mobitek",
    "12.346866544640": "SOREL GmbH Mikroelektronik",
    "24.2924925": "ARRIS Group, Inc.",
    "24.2377111": "GEMUE Gebr. Mueller Apparatebau",
    "24.6628": "2Wire Inc",
    "12.346856525824": "Matthews Australasia",
    "12.346865508352": "Delta Tau Data Systems, Inc.",
    "12.346865500160": "Raith GmbH",
    "24.16522585": "I Smart Cities HK Ltd",
    "12.123917683892224": "Accolade Technology Inc",
    "24.13128289": "ARRIS Group, Inc.",
    "12.346858905600": "Linear Systems Ltd.",
    "24.3160405": "Shenzhen Jinhengjia Electronic Co., Ltd.",
    "12.346863308800": "Elrem Electronic Ag",
    "24.15777070": "H+S Polatis Ltd",
    "12.346855337984": "Uniondigital.,Co.Ltd",
    "24.798753": "Apple, Inc.",
    "20.27478312615936": "SOTHIS CIC TEC (Shanghai) Co., Ltd",
    "24.14731924": "Askey Computer Corp",
    "12.123917689831424": "Gogo Business Aviation",
    "20.31614736269312": "Abbott Medical Optics Inc.",
    "24.7089778": "B&B Exporting Limited",
    "24.3462803": "Huawei Device Co., Ltd.",
    "24.15234247": "Sentinhealth",
    "20.194600872443904": "Shenzhen FEIBIT Electronic Technology Co.,LTD",
    "12.123917689839616": "Feitian Co.,Ltd.",
    "12.346856263680": "MedAvant Healthcare",
    "20.70443725750272": "Table Trac Inc",
    "12.346858508288": "Sicon srl",
    "24.5555578": "Sunnovo International Limited",
    "24.6293559": "ivvi Scientific(Nanchang)Co.Ltd",
    "12.346861862912": "Phitek Systems Ltd.",
    "20.181256791785472": "Klashwerks Inc.",
    "24.6342118": "Skullcandy",
    "24.1617054": "Itel Mobile Limited",
    "24.2896657": "Cisco Systems, Inc",
    "24.6569823": "Exablaze",
    "20.158667118739456": "Wrtnode technology Inc.",
    "24.8191821": "AVM Audiovisuelles Marketing und Computersysteme GmbH",
    "24.37096": "Moxa Technologies Corp., Ltd.",
    "24.7094955": "Subscriber Networks, Inc.",
    "24.8661082": "Saffron Solutions Inc",
    "24.9268": "Lectrosonics, Inc.",
    "24.9269": "Wide Corporation",
    "24.1077": "InfiNet LLC",
    "24.1076": "Accelent Systems, Inc.",
    "24.1071": "International Communications Products, Inc.",
    "24.1070": "Netous Technologies, Ltd.",
    "24.1073": "GlobalStreams, Inc.",
    "24.1072": "Netgem",
    "24.9260": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.9262": "Datastrip Inc.",
    "24.9263": "Micron",
    "24.9264": "Ruby Tech Corp.",
    "24.9265": "Uni-v co.,ltd",
    "24.9266": "Neostar Technology Co.,LTD",
    "24.9267": "Alpsalpine Co,.Ltd",
    "12.123917684092928": "EMAC, Inc.",
    "24.10279371": "Lesira Manufacturing Pty Ltd",
    "12.346868588544": "Yjsystem",
    "12.346854596608": "ads-tec GmbH",
    "12.123917679001600": "ORtek Technology, Inc.",
    "24.10792192": "Ragsdale Technology",
    "20.220515720495104": "Linx Technologies",
    "20.207147367399424": "Argus-Spectrum",
    "24.8186751": "Dmet Products Corp.",
    "24.16516845": "M2Motive Technology Inc.",
    "12.346864586752": "AMS Controls, Inc.",
    "24.16574872": "Apple, Inc.",
    "20.277783366336512": "Hefei Feier Smart Science&Technology Co. Ltd",
    "24.7351305": "Nintendo Co.,Ltd",
    "12.123917677543424": "EA Elektroautomatik GmbH & Co. KG",
    "24.15006237": "PAD Peripheral Advanced Design Inc.",
    "20.211525431197696": "Lyand Acoustic Technology Co.,Ltd.",
    "20.246679826595840": "OMB Guitars LLC",
    "12.346857316352": "Pmc",
    "24.11839998": "GHIA Technology (Shenzhen) LTD",
    "24.11839996": "Quanta Computer Inc.",
    "20.225207114530816": "Rondo Burgdorf AG",
    "12.123917691056128": "Power Electronics Espana, S.L.",
    "12.71297883443200": "T.Q.M. Itaca Technology s.r.l.",
    "24.6822207": "Digital Airways",
    "24.7617263": "Kaonmedia CO., LTD.",
    "24.11278751": "Sungrow Power Supply Co.,Ltd.",
    "12.123917681995776": "Arrow (China) Electronics Trading Co., Ltd.",
    "24.2383356": "IEEE Registration Authority",
    "12.346857443328": "Hillcrest Laboratories, Inc.",
    "24.9443584": "Scs Sa",
    "20.62663540342784": "Waytone (Beiijng) Communications Co.,Ltd",
    "24.8137": "Cisco Systems, Inc",
    "24.1344560": "Mitac Computing Technology Corporation",
    "24.8135": "Casio Hitachi Mobile Communications Co., Ltd.",
    "24.8134": "ASUSTek COMPUTER INC.",
    "24.8133": "Nintendo Co., Ltd.",
    "12.346863042560": "Arrow Central Europe GmbH - Division Spoerle",
    "24.8131": "SmartSynch, Inc",
    "24.8130": "Jow Tong Technology Co Ltd",
    "24.4502317": "Tp-Link Technologies Co.,Ltd.",
    "20.110373787140096": "Pass & Seymour, Inc d/b/a Legrand",
    "24.3463863": "zte corporation",
    "24.538313": "Netgear",
    "24.8138": "Cisco Systems, Inc",
    "24.321406": "We Corporation Inc.",
    "24.8161729": "Palo Alto Networks",
    "20.106196528791552": "Xiamen Sigmastar Technology Ltd.",
    "24.16306365": "Samsung Electronics Co.,Ltd",
    "24.7095338": "Huawei Technologies Co.,Ltd",
    "24.11278280": "Cisco Meraki",
    "12.346864578560": "Slat",
    "24.2889902": "Trend Electronics Co., Ltd.",
    "24.3158397": "Hosiden Corporation",
    "24.11323032": "Kohzu Precision Co., Ltd.",
    "20.229631755026432": "Beijing Yiwangxuntong Technology",
    "20.140782587609088": "Sumitomo Wiring Systems, Ltd.",
    "24.1612709": "IEEE Registration Authority",
    "24.315173": "Manycolors, Inc.",
    "12.346867892224": "Syscom Instruments",
    "24.1088624": "Prusa Research s.r.o.",
    "24.7919215": "Aristocrat Technologies Australia Pty. Ltd.",
    "12.123917682307072": "Egicon Srl",
    "20.62636949504000": "RIPower Co.,Ltd",
    "24.1872200": "Apple, Inc.",
    "20.53091432923136": "Guangzhou Lian-med Technology Co.,Ltd.",
    "24.5246573": "TrafficCast International",
    "12.346855161856": "Wiener, Plein & Baus GmbH",
    "24.16017877": "IEEE Registration Authority",
    "24.14430364": "Heyrex Limited",
    "12.346868170752": "Steinbichler Optotechnik GmbH",
    "24.5263566": "Hangzhou Dianyixia Communication Technology Co. Ltd.",
    "12.123917682319360": "Dataforth Corporation",
    "24.14992170": "Tp-Link Technologies Co.,Ltd.",
    "24.7633786": "ETH Inc.",
    "24.13153709": "Hewlett Packard Enterprise",
    "24.15790082": "Hon Hai Precision Ind. Co.,Ltd.",
    "12.346861912064": "Temix",
    "24.7352215": "Aava Mobile Oy",
    "12.346860371968": "WR Systems, Ltd.",
    "20.251330454093824": "NationalchipKorea",
    "24.5044023": "SamJi Electronics Co., Ltd",
    "24.11040741": "Shenzhen Chuangwei-Rgb Electronics Co.,Ltd",
    "24.12857368": "MCS Logic Inc.",
    "12.346853588992": "Aoptix Technologies",
    "24.3987918": "Eclipse WiFi",
    "12.123917679816704": "Albiral Display Solutions Sl",
    "24.2891192": "ARRIS Group, Inc.",
    "24.8934190": "Intel Corporate",
    "20.35758081572864": "Regloplas AG",
    "24.1874982": "Zoovel Technologies",
    "12.123917680717824": "Gega Electronique",
    "12.346858991616": "MEGGITT Safety System",
    "24.14466682": "Zhejiang Nurotron Biotechnology Co.",
    "24.7081485": "Sony Mobile Communications Inc",
    "24.2646838": "Siemens AG",
    "20.216602654015488": "iMageTech CO.,LTD.",
    "20.84531189121024": "Nanjing Tongke Technology Development Co., LTD",
    "24.11315342": "SHARP Corporation",
    "24.10237419": "Apple, Inc.",
    "20.189244297445376": "ConectaIP Tecnologia S.L.",
    "24.14733578": "Katoudenkikougyousyo co ltd",
    "20.225135223111680": "Ebiologic Technology Co., Ltd.",
    "24.10520262": "Xiaomi Communications Co Ltd",
    "24.4517823": "China Dragon Technology Limited",
    "24.15730566": "Shandong Bittel Electronics Co., Ltd",
    "24.6592928": "AG Elektronik AB",
    "24.3971263": "Quest Controls, Inc.",
    "12.123917687758848": "Array Telepresence",
    "12.346853265408": "Implementa GmbH",
    "20.181256784445440": "Loriot Ag",
    "24.13688754": "Genie Networks Limited",
    "12.123917685862400": "Christ Elektronik GmbH",
    "24.9724881": "Samsung Electronics Co.,Ltd",
    "24.4226173": "Nokia",
    "24.6614370": "WOORI SYSTEMS Co., Ltd",
    "24.5304650": "Texas Instruments",
    "24.13669831": "Pantech Co., Ltd.",
    "12.119269986304": "Denki Kogyo Company, Limited",
    "12.346861015040": "R-S-I Elektrotechnik GmbH & Co. KG",
    "12.346861465600": "Dipl.-Ing. W. Nophut GmbH",
    "24.7128635": "Taicang T&W Electronics",
    "24.16850": "Cisco Systems, Inc",
    "24.5805234": "BrainCo",
    "12.123917681070080": "ICTK Holdings",
    "24.5805230": "Huawei Device Co., Ltd.",
    "24.2478": "Okano Electric Co.,Ltd",
    "12.123917690609664": "Montrade Spa",
    "24.11068729": "Moimstone Co.,Ltd",
    "20.150399873974272": "Cervoz Technology Co; Ltd.",
    "20.119709254025216": "YongTechs Electric Co. Ltd",
    "24.13420384": "Apple, Inc.",
    "12.346868473856": "Gpo",
    "12.346858512384": "Automation and Control Technology, Inc.",
    "20.48548678729728": "WAYCOM Technology Co.,Ltd",
    "24.3969126": "SmartRG, Inc.",
    "12.346855673856": "Kramer Electronics Ltd.",
    "12.346860392448": "Volumatic Limited",
    "24.14968995": "Huawei Technologies Co.,Ltd",
    "20.44213337063424": "Dspread Technology (Beijing) Inc.",
    "24.1604243": "AMICCOM Electronics Corporation",
    "24.9705216": "Xiaomi Communications Co Ltd",
    "24.6874143": "Unglaube Identech GmbH",
    "12.123917681025024": "Caswa",
    "24.9479013": "HMD Global Oy",
    "12.346866036736": "Gitsn Inc.",
    "24.1666": "Convedia",
    "24.4247076": "Cisco Systems, Inc",
    "12.346859888640": "Sky-Skan, Incorporated",
    "24.5783630": "Apple, Inc.",
    "24.13685382": "Beckman Coulter K.K.",
    "24.1077753": "Cloud Telecomputers, LLC",
    "24.1661": "Takasago Ltd.",
    "12.123917690601472": "Stecomp",
    "24.1662": "WinCom Systems, Inc.",
    "24.1847": "Soriya Co. Ltd.",
    "24.5347": "mm-lab GmbH",
    "24.1849": "Scotty Group Austria Gmbh",
    "20.229206573187072": "Picobrew LLC",
    "24.3200106": "Autosales Incorporated",
    "12.123917684699136": "Vision Sensing Co., Ltd.",
    "12.346862383104": "2N TELEKOMUNIKACE a.s.",
    "12.123917680521216": "wallbe GmbH",
    "20.35758077378560": "Aasset Security",
    "12.123917687779328": "ATBiS Co.,Ltd",
    "24.8921956": "Watchnet DVR Inc",
    "24.9989984": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "20.162752338329600": "Chengdu Zhongheng Network Co.,Ltd.",
    "12.119269404672": "Saturn South Pty Ltd",
    "24.9961578": "zte corporation",
    "12.123917685284864": "Terragene S.A",
    "24.12090610": "Extreme Networks, Inc.",
    "24.3415659": "Huawei Technologies Co.,Ltd",
    "24.2134344": "Texas Instruments",
    "12.123917687742464": "Triple Play Communications",
    "20.176772043767808": "Synapsys Solutions Ltd.",
    "24.11846250": "Garmin International",
    "24.5253621": "Guangdong Oppo Mobile Telecommunications Corp.,Ltd",
    "24.14212160": "Apple, Inc.",
    "24.3714494": "New H3C Technologies Co., Ltd",
    "12.123917689061376": "Tobi Tribe Inc",
    "24.797167": "Dell Inc.",
    "12.346864402432": "Viteco Vnpt Jsc",
    "12.346856775680": "LONAP Limited",
    "24.6611297": "DEP Corp.",
    "24.7959": "IDX Company, Ltd.",
    "24.14716476": "Aztech Electronics Pte Ltd",
    "12.123917688139776": "Siemens AG",
    "24.5535770": "Cisco Systems, Inc",
    "24.12359953": "Zyxel Communications Corporation",
    "24.3503": "Plexus Corp (UK) Ltd",
    "24.3502": "Samsung Heavy Industries Co., Ltd.",
    "24.3501": "Dataprobe, Inc.",
    "24.3500": "Japan CBM Corporation",
    "24.3507": "SDO Communication Corperation",
    "24.3506": "Ammasso, Inc.",
    "24.3505": "Japan Network Service Co., Ltd.",
    "24.3504": "Olym-tech Co.,Ltd.",
    "24.12115313": "zte corporation",
    "24.3509": "Globalsat Technology Corporation",
    "24.3508": "Stormshield",
    "12.346852806656": "Zelpos",
    "24.15737785": "IEEE Registration Authority",
    "24.47808": "Biometric Access Company",
    "12.123917686329344": "Crde",
    "24.12362": "Fraunhofer IPMS",
    "24.12363": "Orbacom Systems, Inc.",
    "24.12360": "Super Micro Computer, Inc.",
    "24.12361": "Bryant Technology, Ltd.",
    "24.12366": "Bustec Production Ltd.",
    "24.12367": "PLANET Technology Corporation",
    "24.12364": "Appian Communications, Inc.",
    "24.12365": "Esi",
    "24.12368": "Versa Technology",
    "24.12369": "Orbit Avionic & Communication",
    "12.346858921984": "DDS Elettronica srl",
    "24.275188": "Partron",
    "20.31477068726272": "Desarrollos y Soluciones Guinea I+D S.L.",
    "24.556222": "Quintic Corp.",
    "24.11033464": "Apple, Inc.",
    "12.346865709056": "Toyon Research Corporation",
    "24.9207611": "ARRIS Group, Inc.",
    "12.123917685768192": "Fsr, Inc.",
    "24.3449506": "Huawei Technologies Co.,Ltd",
    "20.229108119240704": "Hangzhou ToupTek Photonics Co., Ltd.",
    "20.251330446753792": "SMC Networks, Inc",
    "24.10270477": "Liteon Technology Corporation",
    "24.15765211": "VeloCloud Networks",
    "24.8420666": "Espressif Inc.",
    "24.3457283": "Hangzhou Huamu Technology Co.,Ltd.",
    "12.346866245632": "BC Illumination, Inc.",
    "12.71297884471296": "shanghai Anjian Information technology co. , ltd.",
    "12.123917688160256": "Rollogo Limited",
    "12.346868105216": "Sequip S+E GmbH",
    "12.346863628288": "National CineMedia",
    "24.6325307": "Soladigm, Inc.",
    "24.16010808": "Olibra LLC",
    "12.346859384832": "Miros AS",
    "24.5547612": "Technicolor CH USA Inc.",
    "20.110373794480128": "Kobol Innovations Pte. Ltd.",
    "24.24787": "At&T",
    "24.2384001": "razberi technologies",
    "24.9206119": "zte corporation",
    "24.24786": "Lucent Technologies Taiwan Telecommunications Co., Ltd.",
    "12.346864209920": "ELETECH Srl",
    "12.346857033728": "Teledyne Monitor Labs",
    "24.8675875": "Shenzhen Coship Electronics Co., Ltd.",
    "24.24789": "AMADA MIYACHI Co., Ltd",
    "24.6127": "IBM Corp",
    "24.9178991": "Pegatron Corporation",
    "12.123917691387904": "Tiama",
    "24.791865": "Apple, Inc.",
    "24.13166080": "Huawei Technologies Co.,Ltd",
    "24.13673649": "Sonifex Ltd.",
    "20.93065823911936": "Hannto Technology Co., Ltd",
    "24.7385228": "Shenou Communication Equipment Co.,Ltd",
    "24.5274309": "Apple, Inc.",
    "24.6872358": "Shanghai Focus Vision Security Technology Co.,Ltd",
    "24.1578016": "Teb Sas",
    "12.346861023232": "XRONET Corporation",
    "24.10511888": "Fujitsu Limited",
    "24.11826335": "Askey Computer Corp",
    "24.15220800": "Cisco Systems, Inc",
    "12.123917686722560": "Plexus",
    "12.123917676974080": "Tex Computer Srl",
    "24.7998": "RP-Technik e.K.",
    "24.302854": "Smobile Co., Ltd.",
    "24.8691138": "Shanghai MXCHIP Information Technology Co., Ltd.",
    "24.7990": "Bellwin Information Co. Ltd.,",
    "24.7991": "Genesis I&C",
    "24.7992": "Positron",
    "24.7993": "Construcciones y Auxiliar de Ferrocarriles, S.A.",
    "24.7994": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.7995": "Intel Corporate",
    "24.7996": "Intel Corporate",
    "24.2627118": "2Wire Inc",
    "24.6023": "Brother, Brother & Sons ApS",
    "24.6022": "wisembed",
    "24.6021": "Sparr Electronics Ltd",
    "24.6020": "ARRIS Group, Inc.",
    "24.6027": "Teledyne Technologies Incorporated",
    "24.6026": "Darts Technologies Corp.",
    "24.6025": "Zenitron Corporation",
    "24.6024": "Philips Lighting BV",
    "24.6029": "Checkpoint Systems, Inc.",
    "24.6028": "Independent Witness, Inc",
    "12.123917677576192": "Stercom Power Solutions GmbH",
    "24.1577773": "zte corporation",
    "12.346852380672": "Alliant Techsystems, Inc.",
    "24.6592553": "Integrated Device Technology (Malaysia) Sdn. Bhd.",
    "24.12106843": "Samsung Electronics Co.,Ltd",
    "24.8694566": "Tiinlab Corporation",
    "24.8136071": "chaowifi.com",
    "24.8136070": "Juniper Networks",
    "24.1097544": "ASUSTek COMPUTER INC.",
    "24.666163": "Digistor",
    "24.568485": "Benu Video, Inc.",
    "24.11803165": "Samsung Electronics Co.,Ltd",
    "24.12321703": "Mfp Michelin",
    "24.8961217": "Huawei Technologies Co.,Ltd",
    "12.346855055360": "Smarteye Corporation",
    "12.71297884098560": "STV Electronic GmbH",
    "24.8133736": "Samsung Electronics Co.,Ltd",
    "24.15239468": "ARRIS Group, Inc.",
    "12.123917684682752": "Changchun FAW Yanfeng Visteon Automotive Electronics.,Ltd.",
    "24.15504229": "Mapper.ai, Inc.",
    "24.13163302": "Logitech",
    "24.10251122": "Realme Chongqing MobileTelecommunications Corp Ltd",
    "24.10771766": "Hewlett Packard",
    "20.96896788987904": "Infodev Electronic Designers Intl.",
    "24.15232340": "Digit Mobile Inc",
    "24.791138": "zte corporation",
    "12.346865487872": "Private",
    "12.346862936064": "Softronics Ltd.",
    "12.123917688688640": "Logiwaste AB",
    "24.2393844": "Ctek, Inc.",
    "12.123917689925632": "ComNav Technology Ltd.",
    "24.4256645": "IEEE Registration Authority",
    "24.10799070": "Cambridge Industries(Group) Co.,Ltd.",
    "24.587270": "zte corporation",
    "20.246679833935872": "ShenZhen Arts Changhua Intelligent Technology Co., Ltd",
    "12.346864836608": "OMICRON electronics GmbH",
    "12.123917679370240": "Critical Link LLC",
    "24.8148796": "Aruba, a Hewlett Packard Enterprise Company",
    "24.9494907": "Apple, Inc.",
    "24.1865418": "Terasic Technologies Inc.",
    "20.250817671069696": "Avast Software s.r.o.",
    "24.1273": "Xtera Communications, Inc.",
    "24.1272": "QUALICABLE TV Industria E Com., Ltda",
    "24.1271": "Omega Band, Inc.",
    "24.1270": "Amphus",
    "24.1277": "Japan Control Engineering Co., Ltd.",
    "24.1276": "Stratus Technologies",
    "24.1275": "Commtech, Inc.",
    "24.1274": "NBS Technologies Inc.",
    "12.346858229760": "Shenzhen Quanlong Technique Co.Ltd",
    "24.1279": "Acronet Co., Ltd.",
    "24.1278": "Pelago Networks",
    "12.123917687648256": "Honeywell Safety Products USA, Inc",
    "12.123917677334528": "Applied Silver",
    "24.10241190": "SwiftTest, Inc.",
    "24.15228499": "Infratec Datentechnik GmbH",
    "12.346867052544": "Morita Technical Center Company",
    "24.10748601": "Intel Corporate",
    "20.97755436417024": "Beijing Perabytes IS Technology Co., Ltd",
    "24.12648360": "Huawei Technologies Co.,Ltd",
    "24.7897346": "Sercomm Corporation.",
    "20.233258815062016": "Combined Energy Technologies Pty Ltd",
    "24.4319": "Rise Computer Inc.",
    "24.4318": "International Datacasting Corporation",
    "12.346863091712": "Figment Design Laboratories",
    "24.4312": "Calista",
    "24.4311": "Argosy",
    "24.4310": "Exelis",
    "24.2359663": "Huawei Device Co., Ltd.",
    "24.4316": "Micro-Star International Co., Ltd.",
    "24.4315": "Juniper Networks",
    "24.4314": "Kollmorgen Corp",
    "12.123917681045504": "DIC Corporation",
    "24.2673919": "Corporacion Empresarial Altra SL",
    "12.346864873472": "Specialist Electronics Services Ltd",
    "20.176726059515904": "Link Labs",
    "24.1897": "Italiana Macchi SpA",
    "24.37068": "Planex Communications Inc.",
    "24.1895": "Yuxing Electronics Company Limited",
    "24.1894": "Chou Chin Industrial Co., Ltd.",
    "24.1893": "Jade Quantum Technologies, Inc.",
    "24.1892": "YoungWoo Telecom Co. Ltd.",
    "24.1891": "Sunniwell Cyber Tech. Co., Ltd.",
    "24.37069": "Ent-Empresa Nacional De Telecommunicacoes, S.A.",
    "24.3409924": "D-Link Corporation",
    "12.123917676658688": "Husty M.Styczen J.Hupert Sp.J.",
    "24.1899": "Stralfors AB",
    "24.1898": "NEXTEYE Co., Ltd.",
    "12.123917684932608": "Satixfy Israel Ltd.",
    "12.123917678817280": "CDS Institute of Management Strategy, Inc.",
    "24.1360166": "Nokia Corporation",
    "24.15232199": "IEEE Registration Authority",
    "24.12353163": "Samsung Electronics Co.,Ltd",
    "24.7665170": "ARRIS Group, Inc.",
    "24.12119566": "Sagemcom Broadband SAS",
    "12.123917682896896": "Den Automation",
    "12.346865860608": "Rosslare Enterprises Ltd.",
    "24.15994660": "Apple, Inc.",
    "12.123917689049088": "Crde",
    "12.346865287168": "Domis",
    "12.71297883770880": "Kdt",
    "24.37066": "Accord Video Telecommunications, Ltd.",
    "24.37067": "Wireless OnLine, Inc.",
    "24.12105597": "Guangdong Transtek Medical Electronics CO.,Ltd",
    "12.346855137280": "WIS Technologies",
    "24.12859250": "Virtuozzo International GmbH",
    "24.14731370": "SHENZHEN TW-SCIE Co., Ltd",
    "24.7902852": "ARRIS Group, Inc.",
    "20.75685691392000": "neocontrol soluções em automação",
    "24.7902850": "zte corporation",
    "24.49199": "Okuma Corp",
    "12.346868375552": "Rong Jie(FuZhou)Electronics Co.,Ltd",
    "24.15502498": "Ruckus Wireless",
    "24.11333810": "Becker Electronics Taiwan Ltd.",
    "24.569429": "Askey Computer Corp",
    "24.15752405": "Apple, Inc.",
    "12.123917676355584": "ExSens Technology (Pty) Ltd.",
    "24.13690754": "AzureWave Technology Inc.",
    "24.7882841": "Hewlett Packard",
    "24.5269488": "Zyxel Communications Corporation",
    "20.171977819029504": "\"Intellect module\" LLC",
    "12.123917677416448": "Info Creative (Hk) Ltd",
    "12.71297884708864": "Schneider Electric Motion, Inc. USA",
    "24.6067869": "Apple, Inc.",
    "20.111031366975488": "MOBILUS Inc.",
    "12.123917675311104": "Dream I System Co., Ltd",
    "24.16317231": "Santur Corporation",
    "12.71297884925952": "Rosslare Enterprises Limited",
    "24.49192": "Jasco Corporation",
    "24.5822933": "Tenda Technology Co.,Ltd.Dongguan branch",
    "12.123917689020416": "Barcelona Smart Technologies",
    "24.6313752": "Intel Corporate",
    "24.1360834": "K.A. Schmersal GmbH & Co. KG",
    "24.1598180": "Texas Instruments",
    "20.255553899069440": "Klab",
    "12.119269543936": "eiraku electric corp.",
    "12.123917691355136": "Metrum Sweden AB",
    "24.16551824": "Samsung Electronics Co.,Ltd",
    "24.4500761": "Xingfei （H.K）Limited",
    "12.123917689212928": "Pullnet Technology,S.L.",
    "24.4459773": "NetMan Co., Ltd.",
    "24.2625163": "Beijing Spirit Technology Development Co., Ltd.",
    "12.346852540416": "Nortel Networks PLC (UK)",
    "24.14996274": "Tp-Link Technologies Co.,Ltd.",
    "24.297987": "ThinPAD Technology (Shenzhen)CO.,LTD",
    "12.123917691113472": "Potter Electric Signal Co. LLC",
    "24.9408": "Nti Comodo Inc",
    "24.9409": "ARRIS Group, Inc.",
    "12.123917678006272": "Melissa Climate Jsc",
    "24.8946466": "Samsung Electronics Co.,Ltd",
    "24.9400": "free alliance sdn bhd",
    "24.9401": "Wuhan Higheasy Electronic Technology Development Co.Ltd",
    "24.9402": "Texas Instruments",
    "12.346861928448": "Tele and Radio Research Institute",
    "24.9404": "HuRob Co.,Ltd",
    "24.9405": "Hainzl Industriesysteme GmbH",
    "24.9406": "Sony Corporation",
    "24.9407": "Ciat",
    "12.346864762880": "SoGEME",
    "12.346862874624": "Software Systems Plus",
    "24.2652937": "zte corporation",
    "24.12482": "Comone",
    "24.5043616": "Scalable Network Technologies Inc",
    "12.123917691772928": "Private",
    "12.123917686964224": "S-Rain Control A/S",
    "20.272805454151680": "Shanghai Sun Telecommunication Co., Ltd.",
    "24.8397018": "Sagemcom Broadband SAS",
    "24.1593986": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.7621284": "zte corporation",
    "12.346858438656": "FilmLight Limited",
    "24.2630966": "Shenzhen Holatek Co.,Ltd",
    "24.3149214": "Ruijie Networks Co.,LTD",
    "24.10286717": "Ricardo Prague, s.r.o.",
    "24.7039": "TMN Technologies Telecomunicacoes Ltda",
    "24.7038": "Beckmann GmbH",
    "24.7033": "Faiveley Transport",
    "24.7032": "Hewlett Packard",
    "24.7031": "Intel Corporate",
    "24.7030": "Ripcode, Inc.",
    "24.7037": "CXR Anderson Jacobson",
    "24.820356": "Zenovia Electronics Inc.",
    "24.7035": "The Tintometer Ltd",
    "24.7034": "Nintendo Co., Ltd.",
    "24.2209": "CNet Technology Inc.",
    "24.2208": "Stotz Feinmesstechnik GmbH",
    "24.2203": "ICP Electronics Inc.",
    "12.119269842944": "Sanstreak Corp.",
    "24.2201": "Netbind, Inc.",
    "24.2200": "Gigabit Optics Corporation",
    "24.2207": "EFM Networks",
    "24.2206": "Beijing Enter-Net co.LTD",
    "24.2205": "UHD-Elektronik",
    "24.2204": "Elecs Industry Co., Ltd.",
    "12.123917691285504": "Shinwa Industries, Inc.",
    "24.8757": "Strukton Systems bv",
    "24.8756": "Corventis Inc.",
    "24.8755": "ADB Broadband Italia",
    "24.8754": "Design Design Technology Ltd",
    "24.8753": "SMT&C Co., Ltd.",
    "24.8752": "FutureLogic Inc.",
    "24.8751": "Open Grid Computing, Inc.",
    "24.8750": "maintech GmbH",
    "24.16293949": "Dinstar Technologies Co., Ltd.",
    "24.8759": "Shinhint Group",
    "24.8758": "Vector Sp. Z O.O.",
    "24.2672270": "Netgear",
    "24.6296195": "Tsb Real Time Location Systems S.L.",
    "24.9325": "Weinzierl Engineering GmbH",
    "24.13688898": "Cisco Systems, Inc",
    "24.9324": "Aruba, a Hewlett Packard Enterprise Company",
    "24.11547558": "Juniper Networks",
    "24.7635906": "Ubiquiti Networks Inc.",
    "24.5786862": "Digital One Technologies, Limited",
    "20.246679831838720": "ShenZhen Mornsun Smartlinker Limited Co., LTD",
    "12.123917683216384": "SICS Swedish ICT",
    "24.6849648": "eSSys Co.,Ltd",
    "24.5511624": "vivo Mobile Communication Co., Ltd.",
    "12.123917682216960": "Shanghai Holystar Information Technology Co.,Ltd",
    "12.123917676953600": "TEKVEL Ltd.",
    "20.233630185029632": "Omron Sentech Co., Ltd.",
    "20.228784603136000": "EkkoSense Ltd",
    "24.2938": "L-3 Linkabit",
    "24.3989278": "Cisco Systems, Inc",
    "12.346862964736": "LevelStar LLC.",
    "24.6566270": "Dexin Corporation",
    "24.6600266": "ViVOtech, Inc.",
    "24.3960810": "Panasonic India Pvt. Ltd.",
    "12.123917677367296": "Bolide Technology Group, Inc.",
    "24.3715031": "Fujitsu Limited",
    "24.8974102": "ARRIS Group, Inc.",
    "24.14732061": "Private",
    "12.346866376704": "LSD Science&Technology Co.,Ltd.",
    "24.2420282": "Chengdu Yingji Electronic Hi-tech Co Ltd",
    "24.24699": "Fore Systems, Inc.",
    "24.24698": "Dvs Gmbh",
    "24.13637300": "Huawei Technologies Co.,Ltd",
    "20.127658776068096": "Huano International Technology Limited",
    "24.12867778": "Shenzhen TATFOOK Technology Co., Ltd.",
    "24.24693": "Pentek, Inc.",
    "24.24692": "Qsc Llc",
    "24.24691": "Redcreek Communications, Inc.",
    "24.24690": "Vxl Instruments, Limited",
    "24.24697": "Mainstream Data, Inc.",
    "24.24696": "Power Measurement Ltd.",
    "24.24695": "Prisa Networks",
    "24.24694": "Schlumberger Technologies Retail Petroleum Systems",
    "12.346862628864": "Villbau Kft.",
    "24.6616733": "Cisco Systems, Inc",
    "24.5294309": "Samsung Electronics Co.,Ltd",
    "24.15510053": "Samsung Electronics Co.,Ltd",
    "24.16522774": "IPEVO corp",
    "24.11828313": "Jiangsu Hengxin Technology Co.,Ltd.",
    "24.6314932": "AzureWave Technology Inc.",
    "24.10024481": "Xiaomi Communications Co Ltd",
    "24.8701350": "Intel Corporate",
    "24.4569": "TiVo",
    "24.7618091": "Ruckus Wireless",
    "24.2382723": "Renesas Electronics (Penang) Sdn. Bhd.",
    "24.15793895": "Huawei Device Co., Ltd.",
    "12.123917690392576": "Cobo, Inc.",
    "12.123917691539456": "Eltwin A/S",
    "12.123917683798016": "Shenzhen Wesion Technology Co., Ltd",
    "24.4560": "Tandberg Data ASA",
    "24.4215148": "Grandex International Corporation",
    "24.4561": "Soft Imaging System GmbH",
    "12.346855436288": "Geotech Instruments, LLC",
    "20.212014908571648": "WiTagg, Inc",
    "24.7342849": "Sagemcom Broadband SAS",
    "24.15209319": "AIRSOUND Inc.",
    "24.10236966": "ARRIS Group, Inc.",
    "24.4242528": "Apple, Inc.",
    "24.12124015": "Shanghai Typrotech Technology Co.Ltd",
    "12.346858049536": "DSS Networks, Inc.",
    "24.16052845": "Tp-Link Technologies Co.,Ltd.",
    "12.346860355584": "Radar Tronic ltd.",
    "20.45033657991168": "Primetech Engineering Corp.",
    "12.119269355520": "Saturn Solutions Ltd",
    "24.5778167": "Huawei Technologies Co.,Ltd",
    "20.132869117706240": "SightLine Applications",
    "12.346855661568": "Baytech Cinema",
    "24.13431717": "Chi Mei Communication Systems, Inc",
    "12.346852335616": "VersaLogic Corp.",
    "12.346866728960": "EGS Technologies Ltd",
    "24.1866531": "Qingdao Hisense Communications Co.,Ltd.",
    "20.62366881415168": "Yellowbrick Data, Inc.",
    "24.12865196": "Shenzhen Shiningworth Technology Co., Ltd.",
    "24.7105801": "Kyowa Electronics Co.,Ltd.",
    "12.346867597312": "dspnor",
    "12.346861019136": "Oliotalo Oy",
    "24.10541063": "Huawei Device Co., Ltd.",
    "24.12894520": "OWLink Technology Inc",
    "24.8958818": "Nokia Shanghai Bell Co., Ltd.",
    "12.346859065344": "Purelink Technology, inc.",
    "24.2890696": "Hunan Topview Network System CO.,LTD",
    "12.346868084736": "Arcos Technologies LTD",
    "12.123917689065472": "Thermo Fisher Scientific",
    "24.15260548": "Espressif Inc.",
    "24.11849897": "China Mobile Group Device Co.,Ltd.",
    "12.346868318208": "Newtec A/S",
    "20.61822316052480": "Shanghai Greatwall Safety System Co.,Ltd",
    "12.123917684318208": "Zhiye Electronics Co., Ltd.",
    "24.14425763": "Accensus LLC",
    "24.8682120": "HTC Corporation",
    "24.1633897": "Apple, Inc.",
    "12.346866577408": "One-Nemoto Engineering Corporation",
    "12.123917676105728": "CoreEL Technologies Pvt Ltd",
    "24.3416291": "Ruckus Wireless",
    "24.42698": "Cisco Systems, Inc",
    "12.123917675163648": "Redcap Solutions s.r.o.",
    "12.123917676945408": "Promess Inc.",
    "12.346864631808": "Grupo Epelsa S.L.",
    "24.5824556": "Micro Technic A/S",
    "24.16030738": "Structab AB",
    "24.14694006": "Huawei Technologies Co.,Ltd",
    "24.1055572": "Huawei Technologies Co.,Ltd",
    "12.346864476160": "Onlinepizza Norden AB",
    "12.123917675610112": "Circuitlink Pty Ltd",
    "24.5563931": "Apple, Inc.",
    "24.7095100": "Tecno Mobile Limited",
    "20.229558874800128": "Mobicon",
    "24.12854318": "Galvanic Applied Sciences Inc",
    "20.70443727847424": "ShenZhen LanShuo Communication Equipment CO.,LTD.",
    "24.16310393": "Motorola Mobility LLC, a Lenovo Company",
    "24.4511864": "Hui Zhou Gaoshengda Technology Co.,LTD",
    "24.786983": "Technicolor CH USA Inc.",
    "24.8451475": "IRCO Sistemas de Telecomunicación S.A.",
    "24.4982410": "Basler Electric Company",
    "24.5248121": "iRobot Corporation",
    "24.1083407": "Daruma Telecomunicações e Informática S.A.",
    "12.346865393664": "Rohde&Schwarz Topex SA",
    "24.8145337": "Plexus Manufacturing Sdn Bhd",
    "20.176032225165312": "Osatec",
    "24.5305404": "Leeo Inc",
    "24.4753438": "EthoSwitch LLC",
    "12.346862239744": "CaTs3 Limited",
    "20.184991252217856": "Guangzhou Navigateworx Technologies Co., Limited",
    "24.12907620": "Spica international",
    "24.8934431": "Google, Inc.",
    "24.16519142": "Sony Interactive Entertainment Inc.",
    "24.6327054": "Apple, Inc.",
    "20.114975469207552": "Central Railway Manufacturing",
    "12.71297884368896": "ZAO NPC Kompjuternie Technologii",
    "24.11320161": "YSTen Technology Co.,Ltd",
    "24.7090058": "Samsung Electronics Co.,Ltd",
    "24.818556": "Kexiang Information Technology Co, Ltd.",
    "24.3714109": "Nephos Inc",
    "24.3458865": "Alpsalpine Co,.Ltd",
    "12.123917676601344": "Rohde&Schwarz Topex SA",
    "24.3185699": "Micro-Star INTL CO., LTD.",
    "12.346858057728": "ads-tec GmbH",
    "12.346867519488": "Aerodata AG",
    "24.9963983": "OnePlus Technology (Shenzhen) Co., Ltd",
    "24.16049325": "zte corporation",
    "24.12542": "DSA GmbH",
    "24.12543": "DataFab Systems Inc.",
    "24.12540": "Terawave Communications, Inc.",
    "24.12541": "Integrated Systems Design",
    "12.123917688303616": "Dave Srl",
    "20.268341258223616": "Alpha Design Technologies Pvt Ltd",
    "12.346865139712": "Integration Technologies Limited",
    "20.176772032233472": "Shenzhen Feima Robotics Technology Co.,Ltd",
    "24.3695606": "3Nod Multimedia(Shenzhen)Co.,Ltd",
    "12.346865131520": "CellPlus technologies, Inc.",
    "12.123917678833664": "Varikorea",
    "24.5256176": "Yichen (Shenzhen) Technology Co.Ltd",
    "24.303650": "Arcadyan Corporation",
    "12.346852491264": "Dataprobe, Inc.",
    "12.123917685272576": "Alligator Communications",
    "24.4456891": "Shenzhen Bilian Electronic Co.，Ltd",
    "24.13650163": "DATTUS Inc",
    "24.10540834": "Samsung Electronics Co.,Ltd",
    "24.1311468": "Hewlett Packard Enterprise",
    "24.4769686": "Samsung Electronics Co.,Ltd",
    "12.346861453312": "Frame Systems Limited",
    "24.306026": "Chun-il Co.,Ltd.",
    "24.5209": "Moram Co., Ltd.",
    "24.6331551": "Brocade Communications Systems LLC",
    "24.5202": "Calculex,Inc.",
    "24.5203": "Advantech Technologies Co.,Ltd",
    "24.5200": "Heim Systems GmbH",
    "24.524366": "3Com Europe Ltd",
    "24.5206": "Edge Products",
    "24.5207": "T-Vips As",
    "24.5204": "Symwave",
    "24.5205": "Coder Electronics Corporation",
    "24.4159": "Tollgrade Communications, Inc.",
    "24.4158": "Netschools Corporation",
    "12.123917681721344": "Öresundskraft AB",
    "20.97755450048512": "Baoruh Electronic Co., Ltd.",
    "24.4151": "CYQ've Technology Co., Ltd.",
    "24.4150": "Inter-Tel Integrated Systems",
    "24.4153": "Vectron Systems AG",
    "24.4152": "Micro Research Ltd.",
    "24.4155": "Hippi Networking Forum",
    "24.4154": "Diamond Network Tech",
    "24.4157": "Phasecom, Ltd.",
    "24.4156": "Ic Ensemble, Inc.",
    "24.15239075": "Loxley Public Company Limited",
    "12.119269449728": "Lomar Srl",
    "24.13132785": "Sony Interactive Entertainment Inc.",
    "24.7642789": "Ohsung",
    "24.9200852": "Hyeco Smart Tech Co.,Ltd",
    "12.346860347392": "Gogo BA",
    "20.88010615947264": "Panasonic Enterprise Solutions Company",
    "12.123917678661632": "The DX Shop Limited",
    "24.8926462": "Bosch Automotive Products (Suzhou) Co. Ltd",
    "12.346865836032": "Airpoint Co., Ltd.",
    "24.7612394": "Elitegroup Computer Systems Co.,Ltd.",
    "24.13898390": "Anobit Technologies Ltd.",
    "12.119269441536": "InterCEL Pty Ltd",
    "24.12645115": "Apple, Inc.",
    "20.137149120774144": "Opnt Bv",
    "12.346868596736": "Adetel Group",
    "24.16517761": "Extreme Networks, Inc.",
    "24.7898806": "Shenzhen YOUHUA Technology Co., Ltd",
    "24.13375611": "Manzanita Systems, Inc.",
    "12.346855366656": "Obvius",
    "24.3681144": "Eco Plugs Enterprise Co., Ltd",
    "24.15265673": "OPMEX Tech.",
    "12.123917688074240": "Yönnet Akıllı Bina ve Otomasyon Sistemleri",
    "24.1574125": "Texas Instruments",
    "20.88383275663360": "AED Distribution",
    "12.123917690818560": "Areca Embedded Systems Pvt Ltd",
    "20.268341257175040": "E-Song",
    "24.57505": "HIMA PAUL HILDEBRANDT GmbH Co. KG",
    "24.57504": "Wiltron Co.",
    "24.57507": "Cisco Systems",
    "24.57506": "Microslate Inc.",
    "24.57501": "Sarnoff Corporation",
    "24.57500": "Mii",
    "24.57503": "Pixel Vision",
    "24.57502": "Quantum Corporation",
    "24.12646124": "Huawei Technologies Co.,Ltd",
    "24.57509": "ComCore Semiconductor, Inc.",
    "24.57508": "ESAOTE S.p.A.",
    "24.16012700": "Huawei Technologies Co.,Ltd",
    "12.123917683527680": "HL2 group",
    "12.123917682585600": "S-I-C",
    "12.123917679689728": "Touchnet/OneCard",
    "12.346857361408": "Monroe Electronics, Inc.",
    "24.6555108": "Alpsalpine Co.,Ltd",
    "24.7922220": "Huawei Device Co., Ltd.",
    "12.346855481344": "Phytec Messtechnik GmbH",
    "24.278889": "GoPro",
    "24.10249355": "Apple, Inc.",
    "24.3439653": "Apple, Inc.",
    "20.83889193222144": "Tecnoplus Srl",
    "12.123917677031424": "Adtek",
    "12.346859544576": "Efficient Channel Coding",
    "24.9216765": "Starry, Inc.",
    "20.31368138457088": "Strone Technology",
    "24.3312": "M & N GmbH",
    "24.3313": "Intel Corporation",
    "24.3310": "jp-embedded",
    "24.3311": "Open Networks Engineering Ltd",
    "24.3316": "Akatsuki Electric Mfg.Co.,Ltd.",
    "24.3317": "InfoExpress",
    "24.3314": "GAMESA Eólica",
    "24.3315": "Call Image Sa",
    "24.5982": "Zed-3",
    "24.3318": "Sitecom Europe BV",
    "24.3319": "Nortel Networks",
    "24.21760": "Xerox Corporation",
    "24.4486740": "Huawei Technologies Co.,Ltd",
    "20.48476184379392": "Beijing CHJ Automotive Co., Ltd.",
    "12.123917676933120": "Private",
    "12.123917677858816": "BELECTRIC GmbH",
    "12.346857172992": "Transbit Sp.z o.o.",
    "24.6075285": "Huawei Technologies Co.,Ltd",
    "24.3726944": "Mohlenhoff GmbH",
    "12.123917683867648": "Northbound Networks Pty. Ltd.",
    "24.5806133": "Technicolor",
    "12.123917682913280": "Jingtu Printing Systems Co., Ltd",
    "12.123917678198784": "Sicon srl",
    "12.346860654592": "Cabinplant A/S",
    "12.123917682716672": "Netool LLC",
    "24.6826460": "Ficosa Electronics S.L.U.",
    "24.11843206": "Hewlett Packard",
    "12.346863083520": "Infocrossing",
    "12.123917677887488": "Sensative AB",
    "24.16540730": "Samsung Electronics Co.,Ltd",
    "20.22739792429056": "Centervue Spa",
    "24.10541714": "Nanjing Glarun Atten Technology Co. Ltd.",
    "24.11064311": "Arcadyan Technology Corporation",
    "12.346862432256": "NIS-time GmbH",
    "24.11857236": "Wohler Technologies",
    "12.123917680984064": "Systems Technologies",
    "24.3957368": "Shenzhen Jetnet Technology Co.,Ltd.",
    "12.346858876928": "Vocality International Ltd",
    "24.12074585": "Roku, Inc.",
    "12.123917690499072": "Neoptix Inc.",
    "12.123917676867584": "Hoshin Electronics Co., Ltd.",
    "24.8172411": "Qingdao Intelligent&Precise Electronics Co.,Ltd.",
    "24.3678248": "Huawei Technologies Co.,Ltd",
    "24.9445717": "HTC Corporation",
    "24.801530": "New H3C Technologies Co., Ltd",
    "12.346864865280": "Powerbase Energy Systems Inc.",
    "24.6849266": "grandcentrix GmbH",
    "24.6088939": "Samsung Electronics Co.,Ltd",
    "12.346853249024": "ITK Dr. Kassen GmbH",
    "12.123917688123392": "CertusNet Inc.",
    "24.5989": "Nortel Networks",
    "24.2654370": "Novatel Wireless Solutions, Inc.",
    "24.13417939": "Ultimaker B.V.",
    "24.7873319": "Samsung Electronics Co.,Ltd",
    "24.6093": "CEC Wireless R&D Ltd.",
    "24.6090": "Qisda Corporation",
    "24.6091": "Juniper Networks",
    "24.16441": "Optec Daiichi Denko Co., Ltd.",
    "24.4203245": "Digital Projection Ltd",
    "24.6096": "Opticom Communications, LLC",
    "12.123917689397248": "Kodenshi Corp.",
    "24.6097": "Nortel Networks",
    "24.662": "Lectron Co,. Ltd.",
    "24.663": "C-COR.net",
    "24.660": "Tokyo Sokushin Co., Ltd.",
    "24.661": "IP.Access Limited",
    "24.666": "Storage Apps",
    "24.2775": "Origin ELECTRIC CO.,LTD.",
    "24.664": "Broadframe Corporation",
    "24.3196846": "Samsung Electronics Co.,Ltd",
    "24.2441": "VividLogic Inc.",
    "24.2440": "Nudian Electron Co., Ltd.",
    "24.668": "3Com",
    "24.2774": "BeamReach Networks",
    "24.2445": "Velocity Semiconductor",
    "24.2444": "Option Wireless Sweden",
    "24.2447": "Cetacean Networks",
    "24.2446": "ipcas GmbH",
    "24.1638190": "Shenzhen Rui Ying Da Technology Co., Ltd",
    "24.2885436": "Devline Limited",
    "24.6098": "Thinlinx Pty Ltd",
    "24.7543": "NSGate",
    "24.7542": "Eyeheight Ltd.",
    "20.39986090999808": "Senix Corporation",
    "24.7540": "Tianjin China-Silicon Microelectronics Co., Ltd.",
    "24.7547": "Ice Energy, Inc.",
    "24.7546": "Wideband Semiconductor, Inc.",
    "24.7545": "Signamax Llc",
    "24.7544": "Invengo Information Technology Co.,Ltd",
    "24.7549": "Giga-Byte Technology Co.,Ltd.",
    "24.7548": "ABE Elettronica S.p.A.",
    "20.264625133191168": "Intenta GmbH",
    "20.168198557138944": "Promess GmbH",
    "24.8970292": "Shinwa industries(China) ltd.",
    "24.5627": "setex schermuly textile computer gmbh",
    "24.15766044": "Hewlett Packard",
    "24.11579667": "Texas Instruments",
    "24.10782059": "Q Electronics Ltd",
    "12.346854113280": "Imarda New Zealand Limited",
    "24.16538404": "Weibel Scientific A/S",
    "12.346861633536": "Codex Digital Ltd",
    "24.32908": "Netscout Systems (formerly Frontier Software Development)",
    "20.251330458288128": "iConservo Inc",
    "24.22905": "Networked Energy Services",
    "24.8677576": "Totolink Technology Int‘L Limited",
    "24.8139022": "Cisco Systems, Inc",
    "24.5299535": "Automation Components, Inc",
    "12.346865274880": "Odyssee Systemes SAS",
    "24.5949": "Neology",
    "24.5948": "Extreme Engineering Solutions",
    "24.5945": "Bright Headphone Electronics Company",
    "24.5944": "International Business Machines",
    "12.346863501312": "Geonautics Australia Pty Ltd",
    "24.5946": "Cloudastructure Inc",
    "24.5941": "Intel Wireless Network Group",
    "24.5940": "ADC Telecommunications",
    "24.5943": "Industrie Dial Face S.p.A.",
    "24.5942": "iiTron Inc.",
    "24.14426120": "ASD Electronics Ltd",
    "24.13376250": "zte corporation",
    "24.2922788": "ARRIS Group, Inc.",
    "24.7634212": "Phicomm (Shanghai) Co., Ltd.",
    "24.7878422": "Samsung Electronics Co.,Ltd",
    "20.228784605233152": "Evoco Labs CO., LTD",
    "24.15997410": "Volterra",
    "12.123917681487872": "BARCO, s.r.o.",
    "12.346855587840": "ads-tec GmbH",
    "24.8171895": "Satelco AG",
    "24.7377912": "Space Monkey, Inc.",
    "24.15530588": "Beijing Xiaomi Electronics Co., Ltd.",
    "24.4500913": "Sagemcom Broadband SAS",
    "24.14953268": "Apple, Inc.",
    "12.123917691211776": "Enlaps",
    "24.15476305": "Apple, Inc.",
    "12.123917688180736": "Grossenbacher Systeme AG",
    "24.8435465": "Viptela, Inc",
    "24.10759088": "Tp-Link Technologies Co.,Ltd.",
    "24.23059": "Huawei Technologies Co.,Ltd",
    "24.14444323": "barox Kommunikation GmbH",
    "24.11570714": "URadio Systems Co., Ltd",
    "24.1356575": "Dell Inc.",
    "24.2992": "Sysnet Telematica srl",
    "24.2991": "WOOJU COMMUNICATIONS Co,.Ltd",
    "24.2990": "Vitals System Inc.",
    "24.2997": "nStor Technologies, Inc.",
    "24.2996": "RDC Semiconductor Inc.,",
    "24.2995": "RiT technologies Ltd.",
    "24.2994": "Smallbig Technology",
    "20.39672891834368": "Unitronux(Shenzhen) Intelligence Technology Co.,Ltd",
    "24.2998": "Metalligence Technology Corp.",
    "24.4772701": "Private",
    "20.137149118676992": "Shenzhen Kuang-chi Space Technology Co., Ltd.",
    "24.9992167": "Kaonmedia CO., LTD.",
    "24.8527": "Alpsalpine Co,.Ltd",
    "24.4510666": "Cisco Systems, Inc",
    "24.8340": "Cubix Corporation",
    "24.8341": "Riva Electronics",
    "24.8342": "Invensys",
    "24.8343": "Applied Signal Technology",
    "24.8344": "Hectronic Ab",
    "24.8345": "Bon Electric Co., Ltd.",
    "24.8346": "The 3Do Company",
    "24.8347": "Ersat Electronic Gmbh",
    "24.8348": "Primary Access Corp.",
    "24.8349": "Lippert Automationstechnik",
    "12.123917687300096": "Samriddi Automations Pvt. Ltd.",
    "24.12111983": "Guangzhou Hkust Fok Ying Tung Research Institute",
    "20.45033659039744": "Phyn LLC",
    "24.1056474": "Ingenico Terminals Sas",
    "12.123917680906240": "BOE Technology Group Co., Ltd.",
    "24.14434655": "Apple, Inc.",
    "12.123917683073024": "Rail Power Systems GmbH",
    "24.12069490": "Dell Inc.",
    "24.11859873": "LG Electronics (Mobile Communications)",
    "12.346856931328": "Uniton Ag",
    "24.15243716": "HTC Corporation",
    "24.15767025": "Carl Zeiss AG",
    "24.11321949": "Fiberhome Telecommunication Technologies Co.,LTD",
    "20.216602661355520": "Kaga Electronics Co.,Ltd.",
    "20.264036225646592": "Transcend Building Automation control network corporation",
    "24.4758498": "SCI Innovations Ltd",
    "24.1587300": "Cap-Tech International Co., Ltd.",
    "24.5546063": "t-mac Technologies Ltd",
    "24.5546064": "ASUSTek COMPUTER INC.",
    "12.346862870528": "Shenzhen Shangji electronic Co.Ltd",
    "20.264165318983680": "Linpa Acoustic Technology Co.,Ltd",
    "24.1357629": "ARCHEAN Technologies",
    "12.346853425152": "DRS Photronics",
    "24.13430600": "Cisco Systems, Inc",
    "12.123917689581568": "Innominds Software Private Limited",
    "24.5768395": "Tianjin Huisun Technology Co.,Ltd.",
    "24.1583684": "Hirose Electronic System",
    "24.1596136": "Zenotech.Co.,Ltd",
    "12.346854342656": "Oshimi System Design Inc.",
    "24.5806859": "Shineway Technologies, Inc.",
    "24.36888": "Ito Electric Industry Co, Ltd.",
    "24.9993693": "China Mobile Group Device Co.,Ltd.",
    "24.10490827": "Fiberhome Telecommunication Technologies Co.,LTD",
    "24.8657032": "Juniper Networks",
    "12.346860740608": "LogiCom GmbH",
    "24.7373326": "as electronics GmbH",
    "20.106480353148928": "Wilderness Labs Inc.",
    "20.48835642523648": "NanChang LangJie Technology Co.,Ltd",
    "12.123917681451008": "Kronotech Srl",
    "24.8434201": "Elektron Technology Uk Limited",
    "12.71297883189248": "AlphaNavigation coltd",
    "24.36885": "Centigram Communications Corp.",
    "24.36884": "Rotork Instruments, Ltd.",
    "24.36887": "Zypcom, Inc",
    "24.36886": "Zac",
    "24.36881": "WAVTrace, Inc.",
    "24.36880": "Simulation Laboratories, Inc.",
    "24.36883": "Samsan Corp.",
    "24.36882": "Globespan Semiconductor, Inc.",
    "24.9456970": "Sagemcom Broadband SAS",
    "24.846976": "Texas Instruments",
    "24.36889": "Hermes Electronics Co., Ltd.",
    "12.346860154880": "Zao Npc",
    "12.346867232768": "Pantec Engineering AG",
    "12.346858008576": "Broadlight",
    "24.4764243": "Marketaxess Corporation",
    "24.1611775": "TechFaith Wireless Technology Limited",
    "24.8661948": "Modelleisenbahn GmbH",
    "24.11813028": "Apple, Inc.",
    "12.123917680435200": "Swissponic Sagl",
    "24.15218205": "Sense Labs, Inc.",
    "12.123917682012160": "Sensorberg GmbH",
    "12.346852319232": "Abatis Systems Corp.",
    "12.119269347328": "Rose + Herleth GbR",
    "12.123917691105280": "A.F.Mensah, Inc",
    "24.12903047": "Huawei Technologies Co.,Ltd",
    "20.132777846505472": "Sichuan Tianyi Comheart Telecom Co.,LTD",
    "24.5832126": "Shenzhen Taikaida Technology Co., Ltd",
    "12.123917687595008": "Garmo Instruments S.L.",
    "20.17622852698112": "First Chair Acoustics Co., Ltd.",
    "12.346861002752": "Hanyang Electric Cp., Ltd",
    "24.8419441": "Askey Computer Corp",
    "24.14468683": "Huawei Technologies Co.,Ltd",
    "24.10535646": "Costar Video Systems",
    "24.12361026": "Shenzhen Rf-Link Technology Co.,Ltd.",
    "24.6321084": "zte corporation",
    "24.12206425": "Anapass Inc.",
    "24.2632153": "GlobalBeiMing technology (Beijing)Co. Ltd",
    "12.346855600128": "Row Seven Ltd",
    "24.9451529": "Ericsson AB",
    "12.123917690974208": "Epdm Pty Ltd",
    "12.123917681266688": "MB connect line GmbH Fernwartungssysteme",
    "12.123917690019840": "Guangzhou Moblin Technology Co., Ltd.",
    "20.48548668243968": "Kunyi electronic technology (Shanghai) Co., Ltd.",
    "24.14991564": "China Mobile Group Device Co.,Ltd.",
    "24.15786972": "Tecon MT, LLC",
    "24.1585689": "EM Microelectronic",
    "20.211670369566720": "PCH Engineering A/S",
    "24.9204216": "Samsung Electronics Co.,Ltd",
    "24.6046042": "Amazon.com, LLC",
    "12.346861654016": "EVK DI Kerschhaggl GmbH",
    "24.11575226": "Apple, Inc.",
    "12.346853847040": "Wilcoxon Research, Inc.",
    "12.346859560960": "Audico Systems Oy",
    "20.250817675264000": "Tritium Pty Ltd",
    "24.1615857": "Qingdao Haier Technology Co.,Ltd",
    "24.12113163": "Sunitec Enterprise Co.,Ltd",
    "24.6304587": "Apple, Inc.",
    "24.13647299": "AQ Corporation",
    "24.12873201": "DataGravity",
    "24.8441930": "Technicolor CH USA Inc.",
    "24.8941708": "Lenovo Mobile Communication Technology Ltd.",
    "12.123917679833088": "Algodue Elettronica Srl",
    "24.13641608": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.5002629": "Hamilton Systems",
    "24.3973896": "Texas Instruments",
    "12.123917684113408": "IES S.r.l.",
    "20.101728429015040": "Guoyi Liangzi (Hefei) Technology Co., Ltd(CIQTEK)",
    "20.194600870346752": "Haltian Products Oy",
    "12.346853310464": "Microbit 2.0 AB",
    "24.8965882": "Huawei Technologies Co.,Ltd",
    "12.346855120896": "Airnet Communications Corp",
    "20.273272602099712": "Beijing Zhong Chuang Communication Technology Ltd.",
    "20.181338944569344": "Shanghai Facom Electronics Technology Co, ltd.",
    "12.346854944768": "Micro System Architecturing srl",
    "12.123917690191872": "Aplex Technology Inc.",
    "24.9468426": "Gebr. Bode GmbH & Co KG",
    "24.10536684": "ShenZhen ANYK Technology Co.,LTD",
    "24.3953735": "Huawei Technologies Co.,Ltd",
    "20.88383269371904": "Shenzhen Sinway South Technology Co., Ltd",
    "24.1372111": "Schreder",
    "24.3164935": "Motorola Mobility LLC, a Lenovo Company",
    "24.6092250": "Apple, Inc.",
    "12.346855813120": "Freescale Semiconductor",
    "24.9468304": "Benchmark Electronics Romania SRL",
    "12.123917691060224": "Vectology,Inc",
    "20.53091435020288": "Sercomm Corporation.",
    "24.7625887": "TCT mobile ltd",
    "24.1888736": "Cisco Systems, Inc",
    "24.5265487": "waytotec,Inc.",
    "12.346865143808": "HaiVision Systems Incorporated",
    "24.563": "Mantra Communications, Inc.",
    "12.346860830720": "Prodigy Electronics Limited",
    "24.3455569": "Se-Kure Controls, Inc.",
    "20.35758074232832": "Wave-In Communication",
    "24.49198": "Netwiz",
    "24.49191": "Cipher Systems, Inc.",
    "24.49190": "Lans Technology Co., Ltd.",
    "24.49193": "Nexans Deutschland GmbH - ANS",
    "24.562": "Avision, Inc.",
    "24.49195": "Gerloff Gesellschaft Fur",
    "24.49194": "Ohkura Electric Co",
    "24.49197": "Fuji Photo Film Co., Ltd.",
    "24.49196": "Centrum Communications, Inc.",
    "24.12610575": "Dobbs Stanford",
    "12.346859098112": "Dvation.co.,Ltd",
    "24.14211174": "Shenzhen Tozed Technologies Co.,Ltd.",
    "24.16274607": "ECI Telecom Ltd.",
    "20.154019631726592": "UrbanHello",
    "24.11046267": "SunDroid Global limited.",
    "24.15207148": "Jiangsu Hengtong Optic-Electric Co., LTD",
    "24.15737245": "Cal-Comp Electronics & Communications Company Ltd.",
    "24.13141819": "Net Optics",
    "24.32993": "STMicroelectronics SRL",
    "24.32992": "Xtp Systems, Inc.",
    "24.32991": "Adc Codenoll Technology Corp.",
    "24.32990": "Gipsi S.A.",
    "24.32997": "NetApp",
    "24.32996": "Northwest Digital Systems, Inc",
    "24.32995": "Coral",
    "24.32994": "T.D.I. Co., Ltd.",
    "24.32999": "Leonardo Tactical Systems.",
    "24.32998": "Peer Networks, Inc.",
    "24.3697819": "Ivy Biomedical",
    "12.123917683326976": "YDK Technologies Co.,Ltd",
    "20.70682239041536": "Ecotap B.V.",
    "20.220327876493312": "Telematix AG",
    "20.128510964269056": "Emerging Technology (Holdings) Ltd.",
    "20.62636955795456": "BlastWave Inc.",
    "24.11835458": "zte corporation",
    "24.11331710": "Bytemark Computer Consulting Ltd",
    "24.2386341": "Huawei Technologies Co.,Ltd",
    "12.346868154368": "iBWorld co.,ltd.",
    "24.9992": "Nordiag ASA",
    "12.123917677457408": "LG Electronics",
    "24.16256428": "HP Inc.",
    "24.6344338": "The Refined Industry Company Limited",
    "24.4758820": "BlackBerry RTS",
    "24.16023023": "Samsung Electronics Co.,Ltd",
    "20.52819400851456": "Shenzhen Tencent Computer System Co., Ltd.",
    "12.346862993408": "Diamond Point International (Europe) Ltd",
    "24.11538947": "Dynamics Hong Kong Limited",
    "20.31477063483392": "Savelec",
    "12.123917688778752": "COTT Electronics",
    "12.346857816064": "Last Mile Gear",
    "20.31368190885888": "ASSYSTEM France",
    "12.123917689188352": "Zigen Corp",
    "24.3932849": "Creation Technologies LP",
    "12.346852524032": "Vision Research, Inc.",
    "24.7362710": "InShow Technology",
    "24.9964903": "Zyxel Communications Corporation",
    "12.123917684244480": "Raft Technologies",
    "24.4217499": "Anovo",
    "24.11273345": "Jiangsu Huaxing Electronics Co., Ltd.",
    "24.5563382": "Alcatel-Lucent",
    "24.10258568": "Simac Techniek NV",
    "24.8663269": "SkyHawke Technologies, LLC",
    "24.4248921": "Micro S.E.R.I.",
    "20.31368189837312": "Shenzhen Innovaconn Systems Co.,Ltd",
    "24.4753252": "vivo Mobile Communication Co., Ltd.",
    "24.4502626": "Flextronics Tech.(Ind) Pvt Ltd",
    "20.194363547189248": "abode systems, inc.",
    "20.255191437803520": "DigiMagus Technology (Shenzhen) Co., Ltd",
    "24.5568007": "Huawei Device Co., Ltd.",
    "24.1334853": "Bamboo Systems Group",
    "24.10792465": "Hon Hai Precision Ind. Co., Ltd.",
    "24.10792466": "Hon Hai Precision Ind. Co., Ltd.",
    "12.123917682495488": "Swiss Timing LTD",
    "20.66256292020224": "Privacy Labs",
    "20.176069200052224": "Closed Joint-Stock Company \"NORSI-TRANS\"",
    "24.14687470": "Bravo Tech, Inc.",
    "24.10256257": "Shenzhen Fast Technologies Co.,Ltd",
    "24.10023976": "zte corporation",
    "24.7391566": "Liteon Technology Corporation",
    "20.176189951967232": "718th Research Institute of CSIC",
    "24.8415920": "Shenzhen TINNO Mobile Technology Corp.",
    "24.14170125": "SHENZHEN IP-COM Network Co.,Ltd",
    "12.346855247872": "Geometrics, Inc.",
    "24.16538425": "Cisco Systems, Inc",
    "24.14477368": "CK Telecom (Shenzhen) Limited",
    "24.2896762": "Hon Hai Precision Ind. Co.,Ltd.",
    "24.6855351": "Atelier Vision Corporation",
    "24.16260472": "IEEE Registration Authority",
    "20.264165325275136": "AED Engineering GmbH",
    "12.346866249728": "Imtron Messtechnik GmbH",
    "12.123917689106432": "ERMINE Corporation",
    "12.346862571520": "Picell B.V.",
    "24.10224875": "Apple, Inc.",
    "24.6088519": "Huawei Technologies Co.,Ltd",
    "12.123917676011520": "KSE GmbH",
    "12.123917687185408": "Polsystem Si Sp. Z O.O., S.K.A.",
    "20.158667128176640": "SKODA ELECTRIC a.s.",
    "12.123917679357952": "Key Chemical & Equipment Company",
    "24.16049650": "Huawei Technologies Co.,Ltd",
}


func ManufLookup(mac string) string {
    macHex := strings.Replace(mac, ":", "", -1)
    macInt := new(big.Int)

    if _, ok := macInt.SetString(macHex, 16); ok == false {
        return ""
    }

    for mask := uint(0); mask < 48; mask++ {
        shifted := new(big.Int).Rsh(macInt, mask)
        key := fmt.Sprintf("%d.%s", mask, shifted)
        if vendor, found := manuf[key]; found {
            return vendor
        }   
    }

	return ""
}
